@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/csh.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/csh.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -46,6 +46,7 @@ static char sccsid[] = "@(#)csh.c	8.2 (Berkeley) 10/12/93";
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <pwd.h>
@@ -114,10 +115,10 @@ main(argc, argv)
     int     argc;
     char  **argv;
 {
-    register Char *cp;
-    register char *tcp;
-    register int f;
-    register char **tempv;
+    Char *cp;
+    char *tcp;
+    int f;
+    char **tempv;
     struct sigvec osv;
 
     cshin = stdin;
@@ -186,7 +187,9 @@ main(argc, argv)
 	AsciiOnly = k > 0377;
     }
 #else
-    AsciiOnly = getenv("LANG") == NULL && getenv("LC_CTYPE") == NULL;
+    AsciiOnly = getenv("LANG") == NULL &&
+		getenv("LC_ALL") == NULL &&
+		getenv("LC_CTYPE") == NULL;
 #endif				/* NLS */
 
     /*
@@ -227,7 +230,7 @@ main(argc, argv)
      */
     set(STRstatus, Strsave(STR0));
 
-    if ((tcp = getenv("HOME")) != NULL)
+    if ((tcp = getenv("HOME")) != NULL && strlen(tcp) < MAXPATHLEN)
 	cp = SAVE(tcp);
     else
 	cp = NULL;
@@ -522,7 +525,7 @@ notty:
 	{
 	    int     osetintr = setintr;
 	    sig_t   oparintr = parintr;
-	    sigset_t omask = sigblock(sigmask(SIGINT));
+	    int omask = sigblock(sigmask(SIGINT));
 
 	    setintr = 0;
 	    parintr = SIG_IGN;	/* Disable onintr */
@@ -601,9 +604,9 @@ void
 importpath(cp)
     Char   *cp;
 {
-    register int i = 0;
-    register Char *dp;
-    register Char **pv;
+    int i = 0;
+    Char *dp;
+    Char **pv;
     int     c;
 
     for (dp = cp; *dp; dp++)
@@ -621,7 +624,7 @@ importpath(cp)
 	    if ((c = *dp) == ':' || c == 0) {
 		*dp = 0;
 		if (*cp != '/' && (euid == 0 || uid == 0) &&
-		    (intact || intty && isatty(SHOUT)))
+		    (intact || (intty && isatty(SHOUT))))
 		    (void) fprintf(csherr,
 	    "Warning: imported path contains relative components\n");
 		pv[i++] = Strsave(*cp ? cp : STRdot);
@@ -645,7 +648,7 @@ static int
 srccat(cp, dp)
     Char   *cp, *dp;
 {
-    register Char *ep = Strspl(cp, dp);
+    Char *ep = Strspl(cp, dp);
     char   *ptr = short2str(ep);
 
     xfree((ptr_t) ep);
@@ -660,7 +663,7 @@ srcfile(f, onlyown, flag)
     char   *f;
     bool    onlyown, flag;
 {
-    register int unit;
+    int unit;
 
     if ((unit = open(f, O_RDONLY)) == -1)
 	return 0;
@@ -678,7 +681,7 @@ srcfile(f, onlyown, flag)
 int     insource;
 static void
 srcunit(unit, onlyown, hflg)
-    register int unit;
+    int unit;
     bool    onlyown, hflg;
 {
     /* We have to push down a lot of state here */
@@ -693,7 +696,7 @@ srcunit(unit, onlyown, hflg)
     bool    otell = cantell;
 
     struct Bin saveB;
-    volatile sigset_t omask;
+    volatile int omask;
     jmp_buf oldexit;
 
     /* The (few) real local variables */
@@ -729,7 +732,7 @@ srcunit(unit, onlyown, hflg)
     if (setintr)
 	omask = sigblock(sigmask(SIGINT));
     /* Setup the new values of the state stuff saved above */
-    bcopy((char *) &B, (char *) &(saveB), sizeof(B));
+    memmove((char *) &(saveB), (char *) &B, sizeof(B));
     fbuf = NULL;
     fseekp = feobp = fblocks = 0;
     oSHIN = SHIN, SHIN = unit, arginp = 0, onelflg = 0;
@@ -754,7 +757,7 @@ srcunit(unit, onlyown, hflg)
     if (setintr)
 	(void) sigsetmask(omask);
     if (oSHIN >= 0) {
-	register int i;
+	int i;
 
 	/* We made it to the new state... free up its storage */
 	/* This code could get run twice but xfree doesn't care */
@@ -763,7 +766,7 @@ srcunit(unit, onlyown, hflg)
 	xfree((ptr_t) fbuf);
 
 	/* Reset input arena */
-	bcopy((char *) &(saveB), (char *) &B, sizeof(B));
+	memmove((char *) &B, (char *) &(saveB), sizeof(B));
 
 	(void) close(SHIN), SHIN = oSHIN;
 	arginp = oarginp, onelflg = oonelflg;
@@ -929,9 +932,9 @@ pintr1(wantnl)
     bool    wantnl;
 {
     Char **v;
-    sigset_t omask;
+    int omask;
 
-    omask = sigblock((sigset_t) 0);
+    omask = sigblock(0);
     if (setintr) {
 	(void) sigsetmask(omask & ~sigmask(SIGINT));
 	if (pjobs) {
@@ -1001,7 +1004,7 @@ process(catch)
 	 * Interruptible during interactive reads
 	 */
 	if (setintr)
-	    (void) sigsetmask(sigblock((sigset_t) 0) & ~sigmask(SIGINT));
+	    (void) sigsetmask(sigblock(0) & ~sigmask(SIGINT));
 
 	/*
 	 * For the sake of reset()
@@ -1118,7 +1121,7 @@ dosource(v, t)
     struct command *t;
 
 {
-    register Char *f;
+    Char *f;
     bool    hflg = 0;
     Char    buf[BUFSIZ];
 
@@ -1148,8 +1151,8 @@ dosource(v, t)
 static void
 mailchk()
 {
-    register struct varent *v;
-    register Char **vp;
+    struct varent *v;
+    Char **vp;
     time_t  t;
     int     intvl, cnt;
     struct stat stb;
@@ -1336,7 +1339,7 @@ defaultpath()
 void
 printprompt()
 {
-    register Char *cp;
+    Char *cp;
 
     if (!whyles) {
 	for (cp = value(STRprompt); *cp; cp++)
