@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/dir.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/dir.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -75,9 +75,9 @@ void
 dinit(hp)
     Char   *hp;
 {
-    register char *tcp;
-    register Char *cp;
-    register struct directory *dp;
+    char *tcp;
+    Char *cp;
+    struct directory *dp;
     char    path[MAXPATHLEN];
     static char *emsg = "csh: Trying to start from \"%s\"\n";
 
@@ -145,7 +145,7 @@ Char *dp;
      * Don't call set() directly cause if the directory contains ` or
      * other junk characters glob will fail.
      */
-    register Char **vec = (Char **) xmalloc((size_t) (2 * sizeof(Char **)));
+    Char **vec = (Char **) xmalloc((size_t) (2 * sizeof(Char **)));
 
     vec[0] = Strsave(dp);
     vec[1] = 0;
@@ -203,7 +203,7 @@ dodirs(v, t)
 static void
 printdirs()
 {
-    register struct directory *dp;
+    struct directory *dp;
     Char   *s, *hp = value(STRhome);
     int     idx, len, cur;
 
@@ -240,7 +240,7 @@ printdirs()
 
 void
 dtildepr(home, dir)
-    register Char *home, *dir;
+    Char *home, *dir;
 {
 
     if (!eq(home, STRslash) && prefix(home, dir))
@@ -340,8 +340,8 @@ dochngd(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *cp;
-    register struct directory *dp;
+    Char *cp;
+    struct directory *dp;
 
     skipargs(&v, " [<dir>]");
     printd = 0;
@@ -389,7 +389,7 @@ dgoto(cp)
     Char   *dp;
 
     if (*cp != '/') {
-	register Char *p, *q;
+	Char *p, *q;
 	int     cwdlen;
 
 	for (p = dcwd->di_name; *p++;)
@@ -423,9 +423,9 @@ dgoto(cp)
  */
 static Char *
 dfollow(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register Char *dp;
+    Char *dp;
     struct varent *c;
     char    ebuf[MAXPATHLEN];
     int serrno;
@@ -449,7 +449,7 @@ dfollow(cp)
     if (cp[0] != '/' && !prefix(STRdotsl, cp) && !prefix(STRdotdotsl, cp)
 	&& (c = adrof(STRcdpath))) {
 	Char  **cdp;
-	register Char *p;
+	Char *p;
 	Char    buf[MAXPATHLEN];
 
 	for (cdp = c->vec; *cdp; cdp++) {
@@ -491,7 +491,7 @@ dopushd(v, t)
     Char **v;
     struct command *t;
 {
-    register struct directory *dp;
+    struct directory *dp;
 
     skipargs(&v, " [<dir>|+<n>]");
     printd = 1;
@@ -523,7 +523,7 @@ dopushd(v, t)
 	    stderror(ERR_SYSTEM, tmp, strerror(errno));
     }
     else {
-	register Char *ccp;
+	Char *ccp;
 
 	ccp = dfollow(*v);
 	dp = (struct directory *) xcalloc(sizeof(struct directory), 1);
@@ -542,11 +542,11 @@ dopushd(v, t)
  */
 static struct directory *
 dfind(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register struct directory *dp;
-    register int i;
-    register Char *ep;
+    struct directory *dp;
+    int i;
+    Char *ep;
 
     if (*cp++ != '+')
 	return (0);
@@ -576,7 +576,7 @@ dopopd(v, t)
     Char **v;
     struct command *t;
 {
-    register struct directory *dp, *p = NULL;
+    struct directory *dp, *p = NULL;
 
     skipargs(&v, " [+<n>]");
     printd = 1;
@@ -614,7 +614,7 @@ dopopd(v, t)
  */
 void
 dfree(dp)
-    register struct directory *dp;
+    struct directory *dp;
 {
 
     if (dp->di_count != 0) {
@@ -633,10 +633,10 @@ dfree(dp)
  */
 Char   *
 dcanon(cp, p)
-    register Char *cp, *p;
+    Char *cp, *p;
 {
-    register Char *sp;
-    register Char *p1, *p2;	/* general purpose */
+    Char *sp;
+    Char *p1, *p2;	/* general purpose */
     bool    slash;
 
     Char    link[MAXPATHLEN];
@@ -702,8 +702,7 @@ dcanon(cp, p)
 	     */
 	    *--sp = 0;		/* form the pathname for readlink */
 	    if (sp != cp && !adrof(STRignore_symlinks) &&
-		(cc = readlink(short2str(cp), tlink,
-			       sizeof tlink)) >= 0) {
+		(cc = readlink(short2str(cp), tlink, sizeof(tlink) - 1)) >= 0) {
 		(void) Strcpy(link, str2short(tlink));
 		link[cc] = '\0';
 
@@ -787,8 +786,7 @@ dcanon(cp, p)
 
 	    if (sp != cp && adrof(STRchase_symlinks) &&
 		!adrof(STRignore_symlinks) &&
-		(cc = readlink(short2str(cp), tlink,
-			       sizeof tlink)) >= 0) {
+		(cc = readlink(short2str(cp), tlink, sizeof(tlink) - 1)) >= 0) {
 		(void) Strcpy(link, str2short(tlink));
 		link[cc] = '\0';
 
@@ -923,7 +921,7 @@ dcanon(cp, p)
  */
 static void
 dnewcwd(dp)
-    register struct directory *dp;
+    struct directory *dp;
 {
     dcwd = dp;
     dset(dcwd->di_name);
