@@ -30,13 +30,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/dol.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/dol.c 68369 2000-11-05 21:57:59Z asmodai $
  */
 
 #ifndef lint
 static char sccsid[] = "@(#)dol.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
+#include <sys/times.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -105,10 +106,10 @@ static void	 Dtestq __P((int));
  */
 void
 Dfix(t)
-    register struct command *t;
+    struct command *t;
 {
-    register Char **pp;
-    register Char *p;
+    Char **pp;
+    Char *p;
 
     if (noexec)
 	return;
@@ -130,7 +131,7 @@ Dfix(t)
  */
 Char   *
 Dfix1(cp)
-    register Char *cp;
+    Char *cp;
 {
     Char   *Dv[2];
 
@@ -173,8 +174,8 @@ static Char *
 Dpack(wbuf, wp)
     Char   *wbuf, *wp;
 {
-    register int c;
-    register int i = MAXWLEN - (wp - wbuf);
+    int c;
+    int i = MAXWLEN - (wp - wbuf);
 
     for (;;) {
 	c = DgetC(DODOL);
@@ -220,11 +221,11 @@ Dpack(wbuf, wp)
 static int
 Dword()
 {
-    register int c, c1;
+    int c, c1;
     Char    wbuf[BUFSIZ];
-    register Char *wp = wbuf;
-    register int i = MAXWLEN;
-    register bool dolflg;
+    Char *wp = wbuf;
+    int i = MAXWLEN;
+    bool dolflg;
     bool    sofar = 0, done = 0;
 
     while (!done) {
@@ -340,9 +341,9 @@ Dword()
  */
 static int
 DgetC(flag)
-    register int flag;
+    int flag;
 {
-    register int c;
+    int c;
 
 top:
     if ((c = Dpeekc) != '\0') {
@@ -401,8 +402,8 @@ dolerror(s)
 static void
 Dgetdol()
 {
-    register Char *np;
-    register struct varent *vp = NULL;
+    Char *np;
+    struct varent *vp = NULL;
     Char    name[4 * MAXVARLEN + 1];
     int     c, sc;
     int     subscr = 0, lwb = 1, upb = 0;
@@ -574,7 +575,7 @@ Dgetdol()
 	else if (*np != '-')
 	    stderror(ERR_MISSING, '-');
 	else {
-	    register int i = upb;
+	    int i = upb;
 
 	    np++;
 	    if (Isdigit(*np)) {
@@ -632,7 +633,7 @@ eatbrac:
 static void
 fixDolMod()
 {
-    register int c;
+    int c;
 
     c = DgetC(0);
     if (c == ':') {
@@ -691,9 +692,9 @@ fixDolMod()
 
 static void
 setDolp(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register Char *dp;
+    Char *dp;
     int i;
 
     if (dolnmod == 0 || dolmcnt == 0) {
@@ -804,7 +805,7 @@ unDredc(c)
 static int
 Dredc()
 {
-    register int c;
+    int c;
 
     if ((c = Dpeekrd) != '\0') {
 	Dpeekrd = 0;
@@ -822,7 +823,7 @@ Dredc()
 
 static void
 Dtestq(c)
-    register int c;
+    int c;
 {
 
     if (cmap(c, QUOTES))
@@ -839,21 +840,28 @@ void
 heredoc(term)
     Char *term;
 {
-    register int c;
+    int c;
     Char   *Dv[2];
     Char    obuf[BUFSIZ], lbuf[BUFSIZ], mbuf[BUFSIZ];
     int     ocnt, lcnt, mcnt;
-    register Char *lbp, *obp, *mbp;
+    Char *lbp, *obp, *mbp;
     Char  **vp;
     bool    quoted;
     char   *tmp;
+    struct timeval tv;
 
-    if (creat(tmp = short2str(shtemp), 0600) < 0)
-	stderror(ERR_SYSTEM, tmp, strerror(errno));
-    (void) close(0);
-    if (open(tmp, O_RDWR) < 0) {
+again:
+    tmp = short2str(shtemp);
+    if (open(tmp, O_RDWR|O_CREAT|O_TRUNC|O_EXCL, 0600) < 0) {
 	int     oerrno = errno;
-
+	if (errno == EEXIST) {
+	    if (unlink(tmp) == -1) {
+		(void) gettimeofday(&tv, NULL);
+		shtemp = Strspl(STRtmpsh, putn((((int)tv.tv_sec) ^
+		    ((int)tv.tv_usec) ^ ((int)getpid())) & 0x00ffffff));
+	    }
+	    goto again;
+	}
 	(void) unlink(tmp);
 	errno = oerrno;
 	stderror(ERR_SYSTEM, tmp, strerror(errno));
