@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/exec.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/exec.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -113,10 +113,10 @@ doexec(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *dp, **pv, **av, *sav;
-    register struct varent *pathv;
-    register bool slash;
-    register int hashval = 0, hashval1, i;
+    Char *dp, **pv, **av, *sav;
+    struct varent *pathv;
+    bool slash;
+    int hashval = 0, hashval1, i;
     Char   *blk[2];
 
     /*
@@ -193,7 +193,7 @@ doexec(v, t)
      * We must do this AFTER any possible forking (like `foo` in glob) so that
      * this shell can still do subprocesses.
      */
-    (void) sigsetmask((sigset_t) 0);
+    (void) sigsetmask(0);
     /*
      * If no path, no words in path, or a / in the filename then restrict the
      * command search.
@@ -265,12 +265,12 @@ pexerr()
 static void
 texec(sf, st)
     Char   *sf;
-    register Char **st;
+    Char **st;
 {
-    register char **t;
-    register char *f;
-    register struct varent *v;
-    register Char **vp;
+    char **t;
+    char *f;
+    struct varent *v;
+    Char **vp;
     Char   *lastsh[2];
     int     fd;
     unsigned char c;
@@ -361,7 +361,7 @@ texec(sf, st)
 void
 execash(t, kp)
     Char  **t;
-    register struct command *kp;
+    struct command *kp;
 {
     int     saveIN, saveOUT, saveDIAG, saveSTD;
     int     oSHIN;
@@ -446,8 +446,8 @@ dohash(v, t)
     struct command *t;
 {
     DIR    *dirp;
-    register struct dirent *dp;
-    register int cnt;
+    struct dirent *dp;
+    int cnt;
     int     i = 0;
     struct varent *pathv = adrof(STRpath);
     Char  **pv;
@@ -504,9 +504,9 @@ hashstat(v, t)
  */
 static int
 hashname(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register long h = 0;
+    long h = 0;
 
     while (*cp)
 	h = hash(h, *cp++);
@@ -517,11 +517,11 @@ static int
 iscommand(name)
     Char   *name;
 {
-    register Char **pv;
-    register Char *sav;
-    register struct varent *v;
-    register bool slash = any(short2str(name), '/');
-    register int hashval = 0, hashval1, i;
+    Char **pv;
+    Char *sav;
+    struct varent *v;
+    bool slash = any(short2str(name), '/');
+    int hashval = 0, hashval1, i;
 
     v = adrof(STRpath);
     if (v == 0 || v->vec[0] == 0 || slash)
@@ -618,7 +618,7 @@ executable(dir, name, dir_ok)
 /*ARGSUSED*/
 void
 dowhich(v, c)
-    register Char **v;
+    Char **v;
     struct command *c;
 {
     struct wordent lex[3];
@@ -652,9 +652,9 @@ static void
 tellmewhat(lex)
     struct wordent *lex;
 {
-    register int i;
-    register struct biltins *bptr;
-    register struct wordent *sp = lex->next;
+    int i;
+    struct biltins *bptr;
+    struct wordent *sp = lex->next;
     bool    aliased = 0;
     Char   *s0, *s1, *s2, *cmd;
     Char    qc;
@@ -706,8 +706,8 @@ tellmewhat(lex)
     sp->word = cmd = globone(sp->word, G_IGNORE);
 
     if ((i = iscommand(strip(sp->word))) != 0) {
-	register Char **pv;
-	register struct varent *v;
+	Char **pv;
+	struct varent *v;
 	bool    slash = any(short2str(sp->word), '/');
 
 	v = adrof(STRpath);
