@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/func.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/func.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -72,8 +72,8 @@ struct biltins *
 isbfunc(t)
     struct command *t;
 {
-    register Char *cp = t->t_dcom[0];
-    register struct biltins *bp, *bp1, *bp2;
+    Char *cp = t->t_dcom[0];
+    struct biltins *bp, *bp1, *bp2;
     static struct biltins label = {"", dozip, 0, 0};
     static struct biltins foregnd = {"%job", dofg1, 0, 0};
     static struct biltins backgnd = {"%job &", dobg1, 0, 0};
@@ -96,7 +96,7 @@ isbfunc(t)
      * one past the end.
      */
     for (bp1 = bfunc, bp2 = bfunc + nbfunc; bp1 < bp2;) {
-	register i;
+	int i;
 
 	bp = bp1 + ((bp2 - bp1) >> 1);
 	if ((i = *cp - *bp->bname) == 0 &&
@@ -112,8 +112,8 @@ isbfunc(t)
 
 void
 func(t, bp)
-    register struct command *t;
-    register struct biltins *bp;
+    struct command *t;
+    struct biltins *bp;
 {
     int     i;
 
@@ -133,8 +133,8 @@ doonintr(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *cp;
-    register Char *vv = v[1];
+    Char *cp;
+    Char *vv = v[1];
 
     if (parintr == SIG_IGN)
 	return;
@@ -194,8 +194,8 @@ doalias(v, t)
     Char **v;
     struct command *t;
 {
-    register struct varent *vp;
-    register Char *p;
+    struct varent *vp;
+    Char *p;
 
     v++;
     p = *v++;
@@ -265,8 +265,8 @@ doif(v, kp)
     Char  **v;
     struct command *kp;
 {
-    register int i;
-    register Char **vv;
+    int i;
+    Char **vv;
 
     v++;
     i = expr(&v);
@@ -302,7 +302,7 @@ doif(v, kp)
  */
 static void
 reexecute(kp)
-    register struct command *kp;
+    struct command *kp;
 {
     kp->t_dflg &= F_SAVE;
     kp->t_dflg |= F_REPEAT;
@@ -339,7 +339,7 @@ void
 gotolab(lab)
     Char *lab;
 {
-    register struct whyle *wp;
+    struct whyle *wp;
     /*
      * While we still can, locate any unknown ends of existing loops. This
      * obscure code is the WORST result of the fact that we don't really parse.
@@ -365,7 +365,7 @@ doswitch(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *cp, *lp;
+    Char *cp, *lp;
 
     v++;
     if (!*v || *(*v++) != '(')
@@ -419,8 +419,8 @@ doforeach(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *cp, *sp;
-    register struct whyle *nwp;
+    Char *cp, *sp;
+    struct whyle *nwp;
 
     v++;
     sp = cp = strip(*v);
@@ -463,8 +463,8 @@ dowhile(v, t)
     Char **v;
     struct command *t;
 {
-    register int status;
-    register bool again = whyles != 0 && SEEKEQ(&whyles->w_start, &lineloc) &&
+    int status;
+    bool again = whyles != 0 && SEEKEQ(&whyles->w_start, &lineloc) &&
     whyles->w_fename == 0;
 
     v++;
@@ -479,7 +479,7 @@ dowhile(v, t)
     if (*v)
 	stderror(ERR_NAME | ERR_EXPRESSION);
     if (!again) {
-	register struct whyle *nwp =
+	struct whyle *nwp =
 	(struct whyle *) xcalloc(1, sizeof(*nwp));
 
 	nwp->w_start = lineloc;
@@ -507,7 +507,7 @@ preread()
 {
     whyles->w_end.type = I_SEEK;
     if (setintr)
-	(void) sigsetmask(sigblock((sigset_t) 0) & ~sigmask(SIGINT));
+	(void) sigsetmask(sigblock(0) & ~sigmask(SIGINT));
 
     search(T_BREAK, 0, NULL);		/* read the expression in */
     if (setintr)
@@ -564,8 +564,8 @@ dorepeat(v, kp)
     Char  **v;
     struct command *kp;
 {
-    register int i;
-    register sigset_t omask = 0;
+    int i;
+    int omask = 0;
 
     i = getn(v[1]);
     if (setintr)
@@ -593,10 +593,10 @@ doswbrk(v, t)
 
 int
 srchx(cp)
-    register Char *cp;
+    Char *cp;
 {
-    register struct srch *sp, *sp1, *sp2;
-    register i;
+    struct srch *sp, *sp1, *sp2;
+    int i;
 
     /*
      * Binary search Sp1 is the beginning of the current search range. Sp2 is
@@ -622,12 +622,12 @@ static Char *Sgoal;
 static void
 search(type, level, goal)
     int     type;
-    register int level;
+    int level;
     Char   *goal;
 {
     Char    wordbuf[BUFSIZ];
-    register Char *aword = wordbuf;
-    register Char *cp;
+    Char *aword = wordbuf;
+    Char *cp;
 
     Stype = type;
     Sgoal = goal;
@@ -722,10 +722,10 @@ search(type, level, goal)
 
 static int
 getword(wp)
-    register Char *wp;
+    Char *wp;
 {
-    register int found = 0;
-    register int c, d;
+    int found = 0;
+    int c, d;
     int     kwd = 0;
     Char   *owp = wp;
 
@@ -851,7 +851,7 @@ wfree()
     btell(&o);
 
     for (; whyles; whyles = nwp) {
-	register struct whyle *wp = whyles;
+	struct whyle *wp = whyles;
 	nwp = wp->w_next;
 
 	/*
@@ -901,13 +901,13 @@ doglob(v, t)
 static void
 xecho(sep, v)
     int    sep;
-    register Char **v;
+    Char **v;
 {
-    register Char *cp;
+    Char *cp;
     int     nonl = 0;
 
     if (setintr)
-	(void) sigsetmask(sigblock((sigset_t) 0) & ~sigmask(SIGINT));
+	(void) sigsetmask(sigblock(0) & ~sigmask(SIGINT));
     v++;
     if (*v == 0)
 	return;
@@ -924,7 +924,7 @@ xecho(sep, v)
     if (sep == ' ' && *v && eq(*v, STRmn))
 	nonl++, v++;
     while ((cp = *v++) != NULL) {
-	register int c;
+	int c;
 
 	while ((c = *cp++) != '\0')
 	    (void) vis_fputc(c | QUOTE, cshout);
@@ -952,10 +952,10 @@ dosetenv(v, t)
 
     v++;
     if ((vp = *v++) == 0) {
-	register Char **ep;
+	Char **ep;
 
 	if (setintr)
-	    (void) sigsetmask(sigblock((sigset_t) 0) & ~sigmask(SIGINT));
+	    (void) sigsetmask(sigblock(0) & ~sigmask(SIGINT));
 	for (ep = STR_environ; *ep; ep++)
 	    (void) fprintf(cshout, "%s\n", vis_str(*ep));
 	return;
@@ -967,7 +967,8 @@ dosetenv(v, t)
 	importpath(lp);
 	dohash(NULL, NULL);
     }
-    else if (eq(vp, STRLANG) || eq(vp, STRLC_CTYPE)) {
+    else if (eq(vp, STRLANG) || eq(vp, STRLC_CTYPE) ||
+	     eq(vp, STRLC_ALL) || eq(vp, STRLC_COLLATE)) {
 #ifdef NLS
 	int     k;
 
@@ -1015,7 +1016,12 @@ dounsetenv(v, t)
 		if (!Gmatch(name, *v))
 		    continue;
 		maxi = 1;
-		if (eq(name, STRLANG) || eq(name, STRLC_CTYPE)) {
+		/*
+		 * Delete name, and start again cause the environment changes
+		 */
+		Unsetenv(name);
+		if (eq(name, STRLANG) || eq(name, STRLC_CTYPE) ||
+		    eq(name, STRLC_ALL) || eq(name, STRLC_COLLATE)) {
 #ifdef NLS
 		    int     k;
 
@@ -1025,13 +1031,10 @@ dounsetenv(v, t)
 		    AsciiOnly = k > 0377;
 #else
 		    AsciiOnly = getenv("LANG") == NULL &&
+			getenv("LC_ALL") == NULL &&
 			getenv("LC_CTYPE") == NULL;
 #endif				/* NLS */
 		}
-		/*
-		 * Delete name, and start again cause the environment changes
-		 */
-		Unsetenv(name);
 		break;
 	    }
     xfree((ptr_t) name);
@@ -1042,8 +1045,8 @@ void
 Setenv(name, val)
     Char   *name, *val;
 {
-    register Char **ep = STR_environ;
-    register Char *cp, *dp;
+    Char **ep = STR_environ;
+    Char *cp, *dp;
     Char   *blk[2];
     Char  **oep = ep;
 
@@ -1075,8 +1078,8 @@ static void
 Unsetenv(name)
     Char   *name;
 {
-    register Char **ep = STR_environ;
-    register Char *cp, *dp;
+    Char **ep = STR_environ;
+    Char *cp, *dp;
     Char  **oep = ep;
 
     for (; *ep; ep++) {
@@ -1101,8 +1104,8 @@ doumask(v, t)
     Char **v;
     struct command *t;
 {
-    register Char *cp = v[1];
-    register int i;
+    Char *cp = v[1];
+    int i;
 
     if (cp == 0) {
 	i = umask(0);
@@ -1148,7 +1151,7 @@ static struct limits *
 findlim(cp)
     Char   *cp;
 {
-    register struct limits *lp, *res;
+    struct limits *lp, *res;
 
     res = (struct limits *) NULL;
     for (lp = limits; lp->limconst >= 0; lp++)
@@ -1170,8 +1173,8 @@ dolimit(v, t)
     Char **v;
     struct command *t;
 {
-    register struct limits *lp;
-    register RLIM_TYPE limit;
+    struct limits *lp;
+    RLIM_TYPE limit;
     char    hard = 0;
 
     v++;
@@ -1196,11 +1199,10 @@ dolimit(v, t)
 
 static  RLIM_TYPE
 getval(lp, v)
-    register struct limits *lp;
+    struct limits *lp;
     Char  **v;
 {
-    register float f;
-    double  atof();
+    float f;
     Char   *cp = *v++;
 
     f = atof(short2str(cp));
@@ -1280,7 +1282,7 @@ limtail(cp, str)
 /*ARGSUSED*/
 static void
 plim(lp, hard)
-    register struct limits *lp;
+    struct limits *lp;
     Char    hard;
 {
     struct rlimit rlim;
@@ -1307,7 +1309,7 @@ dounlimit(v, t)
     Char **v;
     struct command *t;
 {
-    register struct limits *lp;
+    struct limits *lp;
     int     lerr = 0;
     Char    hard = 0;
 
@@ -1333,7 +1335,7 @@ dounlimit(v, t)
 
 static int
 setlim(lp, hard, limit)
-    register struct limits *lp;
+    struct limits *lp;
     Char    hard;
     RLIM_TYPE limit;
 {
@@ -1365,7 +1367,7 @@ dosuspend(v, t)
 {
     int     ctpgrp;
 
-    void    (*old) ();
+    void    (*old) __P((int));
 
     if (loginsh)
 	stderror(ERR_SUSPLOG);
