@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/proc.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/proc.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -82,10 +82,11 @@ void
 pchild(notused)
 	int notused;
 {
-    register struct process *pp;
-    register struct process *fp;
-    register int pid;
+    struct process *pp;
+    struct process *fp;
+    int pid;
     extern int insource;
+    int save_errno = errno;
     union wait w;
     int     jobflags;
     struct rusage ru;
@@ -101,6 +102,7 @@ loop:
 	    goto loop;
 	}
 	pnoprocesses = pid == -1;
+	errno = save_errno;
 	return;
     }
     for (pp = proclist.p_next; pp != NULL; pp = pp->p_next)
@@ -201,9 +203,9 @@ found:
 void
 pnote()
 {
-    register struct process *pp;
+    struct process *pp;
     int     flags;
-    sigset_t omask;
+    int	omask;
 
     neednote = 0;
     for (pp = proclist.p_next; pp != NULL; pp = pp->p_next) {
@@ -225,8 +227,8 @@ pnote()
 void
 pwait()
 {
-    register struct process *fp, *pp;
-    sigset_t omask;
+    struct process *fp, *pp;
+    int	omask;
 
     /*
      * Here's where dead procs get flushed.
@@ -253,11 +255,11 @@ pwait()
  */
 void
 pjwait(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *fp;
+    struct process *fp;
     int     jobflags, reason;
-    sigset_t omask;
+    int	omask;
 
     while (pp->p_pid != pp->p_jobid)
 	pp = pp->p_friends;
@@ -345,8 +347,8 @@ dowait(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
-    sigset_t omask;
+    struct process *pp;
+    int	omask;
 
     pjobs++;
     omask = sigblock(sigmask(SIGCHLD));
@@ -354,7 +356,7 @@ loop:
     for (pp = proclist.p_next; pp; pp = pp->p_next)
 	if (pp->p_pid &&	/* pp->p_pid == pp->p_jobid && */
 	    pp->p_flags & PRUNNING) {
-	    (void) sigpause((sigset_t) 0);
+	    (void) sigpause(0);
 	    goto loop;
 	}
     (void) sigsetmask(omask);
@@ -367,7 +369,7 @@ loop:
 static void
 pflushall()
 {
-    register struct process *pp;
+    struct process *pp;
 
     for (pp = proclist.p_next; pp != NULL; pp = pp->p_next)
 	if (pp->p_pid)
@@ -381,10 +383,10 @@ pflushall()
  */
 static void
 pflush(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *np;
-    register int idx;
+    struct process *np;
+    int idx;
 
     if (pp->p_pid == 0) {
 	(void) fprintf(csherr, "BUG: process flushed twice");
@@ -415,7 +417,7 @@ pflush(pp)
  */
 static void
 pclrcurr(pp)
-    register struct process *pp;
+    struct process *pp;
 {
 
     if (pp == pcurrent)
@@ -443,9 +445,9 @@ static Char *cmdp;
 void
 palloc(pid, t)
     int     pid;
-    register struct command *t;
+    struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
     int     i;
 
     pp = (struct process *) xcalloc(1, (size_t) sizeof(struct process));
@@ -509,7 +511,7 @@ palloc(pid, t)
 
 static void
 padd(t)
-    register struct command *t;
+    struct command *t;
 {
     Char  **argp;
 
@@ -570,7 +572,7 @@ static void
 pads(cp)
     Char   *cp;
 {
-    register int i;
+    int i;
 
     /*
      * Avoid the Quoted Space alias hack! Reported by:
@@ -626,7 +628,7 @@ prestjob()
 void
 pendjob()
 {
-    register struct process *pp, *tp;
+    struct process *pp, *tp;
 
     if (pcurrjob && (pcurrjob->p_flags & (PFOREGND | PSTOPPED)) == 0) {
 	pp = pcurrjob;
@@ -648,10 +650,10 @@ pendjob()
  */
 static int
 pprint(pp, flag)
-    register struct process *pp;
+    struct process *pp;
     bool    flag;
 {
-    register status, reason;
+    int status, reason;
     struct process *tp;
     int     jobflags, pstatus;
     bool hadnl = 1;	/* did we just have a newline */
@@ -814,13 +816,13 @@ prcomd:
 
 static void
 ptprint(tp)
-    register struct process *tp;
+    struct process *tp;
 {
     struct timeval tetime, diff;
     static struct timeval ztime;
     struct rusage ru;
     static struct rusage zru;
-    register struct process *pp = tp;
+    struct process *pp = tp;
 
     ru = zru;
     tetime = ztime;
@@ -842,8 +844,8 @@ dojobs(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
-    register int flag = NUMBER | NAME | REASON;
+    struct process *pp;
+    int flag = NUMBER | NAME | REASON;
     int     i;
 
     if (chkstop)
@@ -872,7 +874,7 @@ dofg(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
 
     okpcntl();
     ++v;
@@ -892,7 +894,7 @@ dofg1(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
 
     okpcntl();
     pp = pfind(v[0]);
@@ -909,7 +911,7 @@ dobg(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
 
     okpcntl();
     ++v;
@@ -928,7 +930,7 @@ dobg1(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
 
     pp = pfind(v[0]);
     pstart(pp, 0);
@@ -955,8 +957,8 @@ dokill(v, t)
     Char **v;
     struct command *t;
 {
-    register int signum = SIGTERM;
-    register char *name;
+    int signum = SIGTERM;
+    char *name;
 
     v++;
     if (v[0] && v[0][0] == '-') {
@@ -998,10 +1000,10 @@ pkill(v, signum)
     Char  **v;
     int     signum;
 {
-    register struct process *pp, *np;
-    register int jobflags = 0;
+    struct process *pp, *np;
+    int jobflags = 0;
     int     pid, err1 = 0;
-    sigset_t omask;
+    int omask;
     Char   *cp;
 
     omask = sigmask(SIGCHLD);
@@ -1081,11 +1083,11 @@ cont:
  */
 void
 pstart(pp, foregnd)
-    register struct process *pp;
+    struct process *pp;
     int     foregnd;
 {
-    register struct process *np;
-    sigset_t omask;
+    struct process *np;
+    int omask;
     long    jobflags = 0;
 
     omask = sigblock(sigmask(SIGCHLD));
@@ -1115,7 +1117,7 @@ void
 panystop(neednl)
     bool    neednl;
 {
-    register struct process *pp;
+    struct process *pp;
 
     chkstop = 2;
     for (pp = proclist.p_next; pp; pp = pp->p_next)
@@ -1127,7 +1129,7 @@ struct process *
 pfind(cp)
     Char   *cp;
 {
-    register struct process *pp, *np;
+    struct process *pp, *np;
 
     if (cp == 0 || cp[1] == 0 || eq(cp, STRcent2) || eq(cp, STRcentplus)) {
 	if (pcurrent == NULL)
@@ -1151,7 +1153,7 @@ pfind(cp)
     for (pp = proclist.p_next; pp; pp = pp->p_next)
 	if (pp->p_pid == pp->p_jobid) {
 	    if (cp[1] == '?') {
-		register Char *dp;
+		Char *dp;
 
 		for (dp = pp->p_command; *dp; dp++) {
 		    if (*dp != cp[2])
@@ -1169,7 +1171,7 @@ pfind(cp)
 	}
     if (np)
 	return (np);
-    stderror(ERR_NAME | cp[1] == '?' ? ERR_JOBPAT : ERR_NOSUCHJOB);
+    stderror(ERR_NAME | (cp[1] == '?' ? ERR_JOBPAT : ERR_NOSUCHJOB));
     /* NOTREACHED */
     return (0);
 }
@@ -1180,10 +1182,10 @@ pfind(cp)
  */
 static struct process *
 pgetcurr(pp)
-    register struct process *pp;
+    struct process *pp;
 {
-    register struct process *np;
-    register struct process *xp = NULL;
+    struct process *np;
+    struct process *xp = NULL;
 
     for (np = proclist.p_next; np; np = np->p_next)
 	if (np != pcurrent && np != pp && np->p_pid &&
@@ -1205,7 +1207,7 @@ donotify(v, t)
     Char **v;
     struct command *t;
 {
-    register struct process *pp;
+    struct process *pp;
 
     pp = pfind(*++v);
     pp->p_flags |= PNOTIFY;
@@ -1228,10 +1230,10 @@ pfork(t, wanttty)
     struct command *t;		/* command we are forking for */
     int     wanttty;
 {
-    register int pid;
+    int pid;
     bool    ignint = 0;
     int     pgrp;
-    sigset_t omask;
+    int	omask;
 
     /*
      * A child will be uninterruptible only under very special conditions.
@@ -1328,7 +1330,7 @@ void
 pgetty(wanttty, pgrp)
     int     wanttty, pgrp;
 {
-    sigset_t omask = 0;
+    int omask = 0;
 
     /*
      * christos: I am blocking the tty signals till I've set things
