@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/csh/sem.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/csh/sem.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -62,7 +62,7 @@ static void	 chkclob __P((char *));
 
 void
 execute(t, wanttty, pipein, pipeout)
-    register struct command *t;
+    struct command *t;
     int     wanttty, *pipein, *pipeout;
 {
     bool    forked = 0;
@@ -70,9 +70,9 @@ execute(t, wanttty, pipein, pipeout)
     int     pid = 0;
     int     pv[2];
 
-    static sigset_t csigmask;
+    static int csigmask;
 
-    static sigset_t ocsigmask;
+    static int ocsigmask;
     static int onosigchld = 0;
     static int nosigchld = 0;
 
@@ -226,7 +226,7 @@ execute(t, wanttty, pipein, pipeout)
 	    else {
 		int     ochild, osetintr, ohaderr, odidfds;
 		int     oSHIN, oSHOUT, oSHERR, oOLDSTD, otpgrp;
-		sigset_t omask;
+		int	omask;
 
 		/*
 		 * Prepare for the vfork by saving everything that the child
@@ -451,7 +451,7 @@ static void
 vffree(i)
 int i;
 {
-    register Char **v;
+    Char **v;
 
     if ((v = gargv) != NULL) {
 	gargv = 0;
@@ -481,7 +481,7 @@ int i;
  */
 static Char *
 splicepipe(t, cp)
-    register struct command *t;
+    struct command *t;
     Char *cp;	/* word after < or > */
 {
     Char *blk[2];
@@ -524,12 +524,12 @@ splicepipe(t, cp)
  */
 static void
 doio(t, pipein, pipeout)
-    register struct command *t;
+    struct command *t;
     int    *pipein, *pipeout;
 {
-    register int fd;
-    register Char *cp;
-    register int flags = t->t_dflg;
+    int fd;
+    Char *cp;
+    int flags = t->t_dflg;
 
     if (didfds || (flags & F_REPEAT))
 	return;
@@ -620,7 +620,7 @@ doio(t, pipein, pipeout)
 
 void
 mypipe(pv)
-    register int *pv;
+    int *pv;
 {
 
     if (pipe(pv) < 0)
@@ -635,7 +635,7 @@ oops:
 
 static void
 chkclob(cp)
-    register char *cp;
+    char *cp;
 {
     struct stat stb;
 
