@@ -34,11 +34,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/dd/conv_tab.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/dd/conv_tab.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)conv_tab.c	8.1 (Berkeley) 5/31/93";
+static char const sccsid[] = "@(#)conv_tab.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
 #include <sys/types.h>
