@@ -40,7 +40,7 @@ static char copyright[] =
 #ifndef lint
 /*static char sccsid[] = "From: @(#)hostname.c	8.1 (Berkeley) 5/31/93"; */
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/bin/domainname/domainname.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/bin/domainname/domainname.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -51,14 +51,26 @@ static const char rcsid[] =
 #include <string.h>
 #include <unistd.h>
 
+void usage __P((void));
+
 int
 main(argc,argv)
 	int argc;
 	char *argv[];
 {
+	int ch;
 	char domainname[MAXHOSTNAMELEN];
 
-	argc--, argv++;
+	while ((ch = getopt(argc, argv, "")) != -1)
+		switch (ch) {
+		default:
+			usage();
+		}
+	argc -= optind;
+	argv += optind;
+
+	if (argc > 1)
+		usage();
 
 	if (*argv) {
 		if (setdomainname(*argv, strlen(*argv)))
@@ -70,3 +82,10 @@ main(argc,argv)
 	}
 	exit(0);
 }
+
+void
+usage()
+{
+	(void)fprintf(stderr, "usage: domainname [ypdomain]\n");
+	exit(1);
+}
