@@ -25,7 +25,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/ed/main.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/ed/main.c 61267 2000-06-04 23:46:44Z kris $
  */
 
 #ifndef lint
@@ -57,8 +57,9 @@ static char *rcsid = "@(#)main.c,v 1.1 1994/02/01 00:34:42 alm Exp";
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <ctype.h>
-#include <setjmp.h>
+#include <locale.h>
 #include <pwd.h>
+#include <setjmp.h>
 
 #include "ed.h"
 
@@ -99,10 +100,6 @@ char *dps = "*";		/* default command-line prompt */
 
 char *usage = "usage: %s [-] [-sx] [-p string] [name]\n";
 
-extern char errmsg[];
-extern int optind;
-extern char *optarg;
-
 /* ed: line editor */
 int
 main(argc, argv)
@@ -111,10 +108,17 @@ main(argc, argv)
 {
 	int c, n;
 	long status = 0;
+#if __GNUC__
+	/* Avoid longjmp clobbering */
+	(void) &argc;
+	(void) &argv;
+#endif
+
+	(void)setlocale(LC_ALL, "");
 
 	red = (n = strlen(argv[0])) > 2 && argv[0][n - 3] == 'r';
 top:
-	while ((c = getopt(argc, argv, "p:sx")) != EOF)
+	while ((c = getopt(argc, argv, "p:sx")) != -1)
 		switch(c) {
 		case 'p':				/* set prompt */
 			prompt = optarg;
@@ -280,7 +284,7 @@ extract_addr_range()
 }
 
 
-#define	SKIP_BLANKS() while (isspace(*ibufp) && *ibufp != '\n') ibufp++
+#define SKIP_BLANKS() while (isspace((unsigned char)*ibufp) && *ibufp != '\n') ibufp++
 
 #define MUST_BE_FIRST() \
 	if (!first) { sprintf(errmsg, "invalid address"); return ERR; }
@@ -305,10 +309,10 @@ next_addr()
 		case '^':
 			ibufp++;
 			SKIP_BLANKS();
-			if (isdigit(*ibufp)) {
+			if (isdigit((unsigned char)*ibufp)) {
 				STRTOL(n, ibufp);
 				addr += (c == '-' || c == '^') ? -n : n;
-			} else if (!isspace(c))
+			} else if (!isspace((unsigned char)c))
 				addr += (c == '-' || c == '^') ? -1 : 1;
 			break;
 		case '0': case '1': case '2':
@@ -490,7 +494,7 @@ exec_command()
 		if (addr_cnt > 0) {
 			sprintf(errmsg, "unexpected address");
 			return ERR;
-		} else if (!isspace(*ibufp)) {
+		} else if (!isspace((unsigned char)*ibufp)) {
 			sprintf(errmsg, "unexpected command suffix");
 			return ERR;
 		} else if ((fnp = get_filename()) == NULL)
@@ -520,7 +524,7 @@ exec_command()
 		if (addr_cnt > 0) {
 			sprintf(errmsg, "unexpected address");
 			return ERR;
-		} else if (!isspace(*ibufp)) {
+		} else if (!isspace((unsigned char)*ibufp)) {
 			sprintf(errmsg, "unexpected command suffix");
 			return ERR;
 		} else if ((fnp = get_filename()) == NULL)
@@ -651,7 +655,7 @@ exec_command()
 		gflag =  (modified && !scripted && c == 'q') ? EMOD : EOF;
 		break;
 	case 'r':
-		if (!isspace(*ibufp)) {
+		if (!isspace((unsigned char)*ibufp)) {
 			sprintf(errmsg, "unexpected command suffix");
 			return ERR;
 		} else if (addr_cnt == 0)
@@ -784,7 +788,7 @@ exec_command()
 			gflag = EOF;
 			ibufp++;
 		}
-		if (!isspace(*ibufp)) {
+		if (!isspace((unsigned char)*ibufp)) {
 			sprintf(errmsg, "unexpected command suffix");
 			return ERR;
 		} else if ((fnp = get_filename()) == NULL)
@@ -1260,7 +1264,7 @@ mark_line_node(lp, n)
 	line_t *lp;
 	int n;
 {
-	if (!islower(n)) {
+	if (!islower((unsigned char)n)) {
 		sprintf(errmsg, "invalid mark character");
 		return ERR;
 	} else if (mark[n - 'a'] == NULL)
@@ -1275,7 +1279,7 @@ long
 get_marked_node_addr(n)
 	int n;
 {
-	if (!islower(n)) {
+	if (!islower((unsigned char)n)) {
 		sprintf(errmsg, "invalid mark character");
 		return ERR;
 	}
@@ -1351,7 +1355,8 @@ signal_hup(signo)
 {
 	if (mutex)
 		sigflags |= (1 << (signo - 1));
-	else	handle_hup(signo);
+	else
+		handle_hup(signo);
 }
 
 
@@ -1361,7 +1366,8 @@ signal_int(signo)
 {
 	if (mutex)
 		sigflags |= (1 << (signo - 1));
-	else	handle_int(signo);
+	else
+		handle_int(signo);
 }
 
 
@@ -1411,6 +1417,8 @@ void
 handle_winch(signo)
 	int signo;
 {
+	int save_errno = errno;
+
 	struct winsize ws;		/* window size structure */
 
 	sigflags &= ~(1 << (signo - 1));
@@ -1418,6 +1426,7 @@ handle_winch(signo)
 		if (ws.ws_row > 2) rows = ws.ws_row - 2;
 		if (ws.ws_col > 8) cols = ws.ws_col - 8;
 	}
+	errno = save_errno;
 }
 
 
