@@ -34,25 +34,22 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/ar_io.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/ar_io.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)ar_io.c	8.2 (Berkeley) 4/18/94";
+static char const sccsid[] = "@(#)ar_io.c	8.2 (Berkeley) 4/18/94";
 #endif /* not lint */
 
 #include <sys/types.h>
-#include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/mtio.h>
-#include <sys/param.h>
 #include <signal.h>
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
 #include "pax.h"
@@ -121,14 +118,14 @@ ar_open(name)
 			arfd = STDIN_FILENO;
 			arcname = STDN;
 		} else if ((arfd = open(name, EXT_MODE, DMOD)) < 0)
-			syswarn(0, errno, "Failed open to read on %s", name);
+			sys_warn(0, errno, "Failed open to read on %s", name);
 		break;
 	case ARCHIVE:
 		if (name == NULL) {
 			arfd = STDOUT_FILENO;
 			arcname = STDO;
 		} else if ((arfd = open(name, AR_MODE, DMOD)) < 0)
-			syswarn(0, errno, "Failed open to write on %s", name);
+			sys_warn(0, errno, "Failed open to write on %s", name);
 		else
 			can_unlnk = 1;
 		break;
@@ -137,7 +134,7 @@ ar_open(name)
 			arfd = STDOUT_FILENO;
 			arcname = STDO;
 		} else if ((arfd = open(name, APP_MODE, DMOD)) < 0)
-			syswarn(0, errno, "Failed open to read/write on %s",
+			sys_warn(0, errno, "Failed open to read/write on %s",
 				name);
 		break;
 	case COPY:
@@ -155,14 +152,14 @@ ar_open(name)
 	 * set up is based on device type
 	 */
 	if (fstat(arfd, &arsb) < 0) {
-		syswarn(0, errno, "Failed stat on %s", arcname);
+		sys_warn(0, errno, "Failed stat on %s", arcname);
 		(void)close(arfd);
 		arfd = -1;
 		can_unlnk = 0;
 		return(-1);
 	}
 	if (S_ISDIR(arsb.st_mode)) {
-		warn(0, "Cannot write an archive on top of a directory %s",
+		pax_warn(0, "Cannot write an archive on top of a directory %s",
 		    arcname);
 		(void)close(arfd);
 		arfd = -1;
@@ -471,7 +468,7 @@ ar_set_wr()
 	 */
 	if (((cpos = lseek(arfd, (off_t)0L, SEEK_CUR)) < 0) ||
 	    (ftruncate(arfd, cpos) < 0)) {
-		syswarn(1, errno, "Unable to truncate archive file");
+		sys_warn(1, errno, "Unable to truncate archive file");
 		return(-1);
 	}
 	return(0);
@@ -495,13 +492,13 @@ ar_app_ok()
 #endif
 {
 	if (artyp == ISPIPE) {
-		warn(1, "Cannot append to an archive obtained from a pipe.");
+		pax_warn(1, "Cannot append to an archive obtained from a pipe.");
 		return(-1);
 	}
 
 	if (!invld_rec)
 		return(0);
-	warn(1,"Cannot append, device record size %d does not support %s spec",
+	pax_warn(1,"Cannot append, device record size %d does not support %s spec",
 		rdblksz, argv0);
 	return(-1);
 }
@@ -588,9 +585,9 @@ ar_read(buf, cnt)
 	 */
 	lstrval = res;
 	if (res < 0)
-		syswarn(1, errno, "Failed read on archive volume %d", arvol);
+		sys_warn(1, errno, "Failed read on archive volume %d", arvol);
 	else
-		warn(0, "End of archive volume %d reached", arvol);
+		pax_warn(0, "End of archive volume %d reached", arvol);
 	return(res);
 }
 
@@ -669,7 +666,7 @@ ar_write(buf, bsz)
 		if (res >= 0)
 			break;
 		if (errno == EACCES) {
-			warn(0, "Write failed, archive is write protected.");
+			pax_warn(0, "Write failed, archive is write protected.");
 			res = lstrval = 0;
 			return(0);
 		}
@@ -707,18 +704,18 @@ ar_write(buf, bsz)
 	 * must quit right away.
 	 */
 	if (!wr_trail && (res <= 0)) {
-		warn(1,"Unable to append, trailer re-write failed. Quitting.");
+		pax_warn(1,"Unable to append, trailer re-write failed. Quitting.");
 		return(res);
 	}
 
 	if (res == 0)
-		warn(0, "End of archive volume %d reached", arvol);
+		pax_warn(0, "End of archive volume %d reached", arvol);
 	else if (res < 0)
-		syswarn(1, errno, "Failed write to archive volume: %d", arvol);
+		sys_warn(1, errno, "Failed write to archive volume: %d", arvol);
 	else if (!frmt->blkalgn || ((res % frmt->blkalgn) == 0))
-		warn(0,"WARNING: partial archive write. Archive MAY BE FLAWED");
+		pax_warn(0,"WARNING: partial archive write. Archive MAY BE FLAWED");
 	else
-		warn(1,"WARNING: partial archive write. Archive IS FLAWED");
+		pax_warn(1,"WARNING: partial archive write. Archive IS FLAWED");
 	return(res);
 }
 
@@ -752,7 +749,7 @@ ar_rdsync()
 		return(-1);
 
 	if ((act == APPND) || (act == ARCHIVE)) {
-		warn(1, "Cannot allow updates to an archive with flaws.");
+		pax_warn(1, "Cannot allow updates to an archive with flaws.");
 		return(-1);
 	}
 	if (io_ok)
@@ -804,10 +801,10 @@ ar_rdsync()
 		break;
 	}
 	if (lstrval <= 0) {
-		warn(1, "Unable to recover from an archive read failure.");
+		pax_warn(1, "Unable to recover from an archive read failure.");
 		return(-1);
 	}
-	warn(0, "Attempting to recover from an archive read failure.");
+	pax_warn(0, "Attempting to recover from an archive read failure.");
 	return(0);
 }
 
@@ -871,7 +868,7 @@ ar_fow(sksz, skipped)
 		if (lseek(arfd, mpos, SEEK_SET) >= 0)
 			return(0);
 	}
-	syswarn(1, errno, "Foward positioning operation on archive failed");
+	sys_warn(1, errno, "Foward positioning operation on archive failed");
 	lstrval = -1;
 	return(-1);
 }
@@ -913,7 +910,7 @@ ar_rev(sksz)
 		/*
 		 * cannot go backwards on these critters
 		 */
-		warn(1, "Reverse positioning on pipes is not supported.");
+		pax_warn(1, "Reverse positioning on pipes is not supported.");
 		lstrval = -1;
 		return(-1);
 	case ISREG:
@@ -931,7 +928,7 @@ ar_rev(sksz)
 		 * First we figure out where we are in the archive.
 		 */
 		if ((cpos = lseek(arfd, (off_t)0L, SEEK_CUR)) < 0) {
-			syswarn(1, errno,
+			sys_warn(1, errno,
 			   "Unable to obtain current archive byte offset");
 			lstrval = -1;
 			return(-1);
@@ -949,14 +946,14 @@ ar_rev(sksz)
 				/*
 				 * this should never happen
 				 */
-				warn(1,"Reverse position on previous volume.");
+				pax_warn(1,"Reverse position on previous volume.");
 				lstrval = -1;
 				return(-1);
 			}
 			cpos = (off_t)0L;
 		}
 		if (lseek(arfd, cpos, SEEK_SET) < 0) {
-			syswarn(1, errno, "Unable to seek archive backwards");
+			sys_warn(1, errno, "Unable to seek archive backwards");
 			lstrval = -1;
 			return(-1);
 		}
@@ -991,7 +988,7 @@ ar_rev(sksz)
 		 * ok we have to move. Make sure the tape drive can do it.
 		 */
 		if (sksz % phyblk) {
-			warn(1,
+			pax_warn(1,
 			    "Tape drive unable to backspace requested amount");
 			lstrval = -1;
 			return(-1);
@@ -1003,7 +1000,7 @@ ar_rev(sksz)
 		mb.mt_op = MTBSR;
 		mb.mt_count = sksz/phyblk;
 		if (ioctl(arfd, MTIOCTOP, &mb) < 0) {
-			syswarn(1,errno, "Unable to backspace tape %d blocks.",
+			sys_warn(1,errno, "Unable to backspace tape %d blocks.",
 			    mb.mt_count);
 			lstrval = -1;
 			return(-1);
@@ -1051,7 +1048,7 @@ get_phys()
 		while ((res = read(arfd, scbuf, sizeof(scbuf))) > 0)
 			padsz += res;
 		if (res < 0) {
-			syswarn(1, errno, "Unable to locate tape filemark.");
+			sys_warn(1, errno, "Unable to locate tape filemark.");
 			return(-1);
 		}
 	}
@@ -1063,7 +1060,7 @@ get_phys()
 	mb.mt_op = MTBSF;
 	mb.mt_count = 1;
 	if (ioctl(arfd, MTIOCTOP, &mb) < 0) {
-		syswarn(1, errno, "Unable to backspace over tape filemark.");
+		sys_warn(1, errno, "Unable to backspace over tape filemark.");
 		return(-1);
 	}
 
@@ -1074,11 +1071,11 @@ get_phys()
 	mb.mt_op = MTBSR;
 	mb.mt_count = 1;
 	if (ioctl(arfd, MTIOCTOP, &mb) < 0) {
-		syswarn(1, errno, "Unable to backspace over last tape block.");
+		sys_warn(1, errno, "Unable to backspace over last tape block.");
 		return(-1);
 	}
 	if ((phyblk = read(arfd, scbuf, sizeof(scbuf))) <= 0) {
-		syswarn(1, errno, "Cannot determine archive tape blocksize.");
+		sys_warn(1, errno, "Cannot determine archive tape blocksize.");
 		return(-1);
 	}
 
@@ -1089,13 +1086,13 @@ get_phys()
 	while ((res = read(arfd, scbuf, sizeof(scbuf))) > 0)
 		;
 	if (res < 0) {
-		syswarn(1, errno, "Unable to locate tape filemark.");
+		sys_warn(1, errno, "Unable to locate tape filemark.");
 		return(-1);
 	}
 	mb.mt_op = MTBSF;
 	mb.mt_count = 1;
 	if (ioctl(arfd, MTIOCTOP, &mb) < 0) {
-		syswarn(1, errno, "Unable to backspace over tape filemark.");
+		sys_warn(1, errno, "Unable to backspace over tape filemark.");
 		return(-1);
 	}
 
@@ -1115,7 +1112,7 @@ get_phys()
 	 * never fail).
 	 */
 	if (padsz % phyblk) {
-		warn(1, "Tape drive unable to backspace requested amount");
+		pax_warn(1, "Tape drive unable to backspace requested amount");
 		return(-1);
 	}
 
@@ -1126,7 +1123,7 @@ get_phys()
 	mb.mt_op = MTBSR;
 	mb.mt_count = padsz/phyblk;
 	if (ioctl(arfd, MTIOCTOP, &mb) < 0) {
-		syswarn(1,errno,"Unable to backspace tape over %d pad blocks",
+		sys_warn(1,errno,"Unable to backspace tape over %d pad blocks",
 		    mb.mt_count);
 		return(-1);
 	}
@@ -1161,10 +1158,10 @@ ar_next()
 	 * also be called via a signal handler, so we must prevent a race.
 	 */
 	if (sigprocmask(SIG_BLOCK, &s_mask, &o_mask) < 0)
-		syswarn(0, errno, "Unable to set signal mask");
+		sys_warn(0, errno, "Unable to set signal mask");
 	ar_close();
 	if (sigprocmask(SIG_SETMASK, &o_mask, (sigset_t *)NULL) < 0)
-		syswarn(0, errno, "Unable to restore signal mask");
+		sys_warn(0, errno, "Unable to restore signal mask");
 
 	if (done || !wr_trail)
 		return(-1);
@@ -1277,7 +1274,7 @@ ar_next()
 			if ((arcname = strdup(buf)) == NULL) {
 				done = 1;
 				lstrval = -1;
-				warn(0, "Cannot save archive name.");
+				pax_warn(0, "Cannot save archive name.");
 				return(-1);
 			}
 			freeit = 1;
