@@ -34,21 +34,19 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/ar_subs.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/ar_subs.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)ar_subs.c	8.2 (Berkeley) 4/18/94";
+static char const sccsid[] = "@(#)ar_subs.c	8.2 (Berkeley) 4/18/94";
 #endif /* not lint */
 
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <signal.h>
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
@@ -426,7 +424,7 @@ wr_archive(arcn, is_app)
 			 * the link table).
 			 */
 			if ((fd = open(arcn->org_name, O_RDONLY, 0)) < 0) {
-				syswarn(1,errno, "Unable to open %s to read",
+				sys_warn(1,errno, "Unable to open %s to read",
 					arcn->org_name);
 				purg_lnk(arcn);
 				continue;
@@ -573,7 +571,7 @@ append()
 	if (get_arc() < 0)
 		return;
 	if ((orgfrmt != NULL) && (orgfrmt != frmt)) {
-		warn(1, "Cannot mix current archive format %s with %s",
+		pax_warn(1, "Cannot mix current archive format %s with %s",
 		    frmt->name, orgfrmt->name);
 		return;
 	}
@@ -744,7 +742,7 @@ copy()
 	 * set up the destination dir path and make sure it is a directory. We
 	 * make sure we have a trailing / on the destination
 	 */
-	dlen = l_strncpy(dirbuf, dirptr, PAXPATHLEN);
+	dlen = l_strncpy(dirbuf, dirptr, sizeof(dirbuf) - 1);
 	dest_pt = dirbuf + dlen;
 	if (*(dest_pt-1) != '/') {
 		*dest_pt++ = '/';
@@ -754,12 +752,12 @@ copy()
 	drem = PAXPATHLEN - dlen;
 
 	if (stat(dirptr, &sb) < 0) {
-		syswarn(1, errno, "Cannot access destination directory %s",
+		sys_warn(1, errno, "Cannot access destination directory %s",
 			dirptr);
 		return;
 	}
 	if (!S_ISDIR(sb.st_mode)) {
-		warn(1, "Destination is not a directory %s", dirptr);
+		pax_warn(1, "Destination is not a directory %s", dirptr);
 		return;
 	}
 
@@ -813,7 +811,7 @@ copy()
 			else
 				res = 0;
 			if ((arcn->nlen - res) > drem) {
-				warn(1, "Destination pathname too long %s",
+				pax_warn(1, "Destination pathname too long %s",
 					arcn->name);
 				continue;
 			}
@@ -918,7 +916,7 @@ copy()
 		 * first open source file and then create the destination file
 		 */
 		if ((fdsrc = open(arcn->org_name, O_RDONLY, 0)) < 0) {
-			syswarn(1, errno, "Unable to open %s to read",
+			sys_warn(1, errno, "Unable to open %s to read",
 			    arcn->org_name);
 			purg_lnk(arcn);
 			continue;
@@ -1010,16 +1008,16 @@ next_head(arcn)
 			 * storage device, better give the user the bad news.
 			 */
 			if ((ret == 0) || (rd_sync() < 0)) {
-				warn(1,"Premature end of file on archive read");
+				pax_warn(1,"Premature end of file on archive read");
 				return(-1);
 			}
 			if (!in_resync) {
 				if (act == APPND) {
-					warn(1,
+					pax_warn(1,
 				          "Archive I/O error, cannot continue");
 					return(-1);
 				}
-				warn(1,"Archive I/O error. Trying to recover.");
+				pax_warn(1,"Archive I/O error. Trying to recover.");
 				++in_resync;
 			}
 
@@ -1080,10 +1078,10 @@ next_head(arcn)
 		 */
 		if (!in_resync) {
 			if (act == APPND) {
-				warn(1,"Unable to append, archive header flaw");
+				pax_warn(1,"Unable to append, archive header flaw");
 				return(-1);
 			}
-			warn(1,"Invalid header, starting valid header search.");
+			pax_warn(1,"Invalid header, starting valid header search.");
 			++in_resync;
 		}
 		bcopy(hdbuf+1, hdbuf, shftsz);
@@ -1175,7 +1173,7 @@ get_arc()
 			if (!notice) {
 				if (act == APPND)
 					return(-1);
-				warn(1,"Cannot identify format. Searching...");
+				pax_warn(1,"Cannot identify format. Searching...");
 				++notice;
 			}
 		}
@@ -1210,7 +1208,7 @@ get_arc()
 		if (!notice) {
 			if (act == APPND)
 				return(-1);
-			warn(1, "Cannot identify format. Searching...");
+			pax_warn(1, "Cannot identify format. Searching...");
 			++notice;
 		}
 
@@ -1235,6 +1233,6 @@ get_arc()
 	/*
 	 * we cannot find a header, bow, apologize and quit
 	 */
-	warn(1, "Sorry, unable to determine archive format.");
+	pax_warn(1, "Sorry, unable to determine archive format.");
 	return(-1);
 }
