@@ -34,19 +34,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/cpio.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/cpio.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)cpio.c	8.1 (Berkeley) 5/31/93";
+static char const sccsid[] = "@(#)cpio.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <string.h>
-#include <ctype.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -217,7 +215,7 @@ rd_nm(arcn, nsz)
 	 * do not even try bogus values
 	 */
 	if ((nsz == 0) || (nsz > sizeof(arcn->name))) {
-		warn(1, "Cpio file name length %d is out of range", nsz);
+		pax_warn(1, "Cpio file name length %d is out of range", nsz);
 		return(-1);
 	}
 
@@ -226,7 +224,7 @@ rd_nm(arcn, nsz)
 	 */
 	if ((rd_wrbuf(arcn->name,nsz) != nsz) || (arcn->name[nsz-1] != '\0') ||
 	    (arcn->name[0] == '\0')) {
-		warn(1, "Cpio file name in header is corrupted");
+		pax_warn(1, "Cpio file name in header is corrupted");
 		return(-1);
 	}
 	return(0);
@@ -255,10 +253,10 @@ rd_ln_nm(arcn)
 	if ((arcn->sb.st_size == 0) ||
 	    (arcn->sb.st_size >= sizeof(arcn->ln_name))) {
 #		ifdef NET2_STAT
-		warn(1, "Cpio link name length is invalid: %lu",
+		pax_warn(1, "Cpio link name length is invalid: %lu",
 		    arcn->sb.st_size);
 #		else
-		warn(1, "Cpio link name length is invalid: %qu",
+		pax_warn(1, "Cpio link name length is invalid: %qu",
 		    arcn->sb.st_size);
 #		endif
 		return(-1);
@@ -269,7 +267,7 @@ rd_ln_nm(arcn)
 	 */
 	if (rd_wrbuf(arcn->ln_name, (int)arcn->sb.st_size) !=
 	    (int)arcn->sb.st_size) {
-		warn(1, "Cpio link name read error");
+		pax_warn(1, "Cpio link name read error");
 		return(-1);
 	}
 	arcn->ln_nlen = arcn->sb.st_size;
@@ -279,7 +277,7 @@ rd_ln_nm(arcn)
 	 * watch out for those empty link names
 	 */
 	if (arcn->ln_name[0] == '\0') {
-		warn(1, "Cpio link name is corrupt");
+		pax_warn(1, "Cpio link name is corrupt");
 		return(-1);
 	}
 	return(0);
@@ -481,7 +479,7 @@ cpio_wr(arcn)
 		if (uqd_asc((u_quad_t)arcn->sb.st_size, hd->c_filesize,
 		    sizeof(hd->c_filesize), OCT)) {
 #		endif
-			warn(1,"File is too large for cpio format %s",
+			pax_warn(1,"File is too large for cpio format %s",
 			    arcn->org_name);
 			return(1);
 		}
@@ -532,7 +530,7 @@ cpio_wr(arcn)
 	 */
 	if ((wr_rdbuf(hdblk, (int)sizeof(HD_CPIO)) < 0) ||
 	    (wr_rdbuf(arcn->name, nsz) < 0)) {
-		warn(1, "Unable to write cpio header for %s", arcn->org_name);
+		pax_warn(1, "Unable to write cpio header for %s", arcn->org_name);
 		return(-1);
 	}
 
@@ -551,7 +549,7 @@ cpio_wr(arcn)
 	 * next file as we are done.
 	 */
 	if (wr_rdbuf(arcn->ln_name, arcn->ln_nlen) < 0) {
-		warn(1,"Unable to write cpio link name for %s",arcn->org_name);
+		pax_warn(1,"Unable to write cpio link name for %s",arcn->org_name);
 		return(-1);
 	}
 	return(1);
@@ -560,7 +558,7 @@ cpio_wr(arcn)
 	/*
 	 * header field is out of range
 	 */
-	warn(1, "Cpio header field is too small to store file %s",
+	pax_warn(1, "Cpio header field is too small to store file %s",
 	    arcn->org_name);
 	return(1);
 }
@@ -848,7 +846,7 @@ vcpio_wr(arcn)
 		if (uqd_asc((u_quad_t)arcn->sb.st_size, hd->c_filesize,
 		    sizeof(hd->c_filesize), HEX)) {
 #		endif
-			warn(1,"File is too large for sv4cpio format %s",
+			pax_warn(1,"File is too large for sv4cpio format %s",
 			    arcn->org_name);
 			return(1);
 		}
@@ -906,7 +904,7 @@ vcpio_wr(arcn)
 	if ((wr_rdbuf(hdblk, (int)sizeof(HD_VCPIO)) < 0) ||
 	    (wr_rdbuf(arcn->name, (int)nsz) < 0)  ||
 	    (wr_skip((off_t)(VCPIO_PAD(sizeof(HD_VCPIO) + nsz))) < 0)) {
-		warn(1,"Could not write sv4cpio header for %s",arcn->org_name);
+		pax_warn(1,"Could not write sv4cpio header for %s",arcn->org_name);
 		return(-1);
 	}
 
@@ -928,7 +926,7 @@ vcpio_wr(arcn)
 	 */
 	if ((wr_rdbuf(arcn->ln_name, arcn->ln_nlen) < 0) ||
 	    (wr_skip((off_t)(VCPIO_PAD(arcn->ln_nlen))) < 0)) {
-		warn(1,"Could not write sv4cpio link name for %s",
+		pax_warn(1,"Could not write sv4cpio link name for %s",
 		    arcn->org_name);
 		return(-1);
 	}
@@ -938,7 +936,7 @@ vcpio_wr(arcn)
 	/*
 	 * header field is out of range
 	 */
-	warn(1,"Sv4cpio header field is too small for file %s",arcn->org_name);
+	pax_warn(1,"Sv4cpio header field is too small for file %s",arcn->org_name);
 	return(1);
 }
 
@@ -1159,7 +1157,7 @@ bcpio_wr(arcn)
 		t_offt = (off_t)(SHRT_EXT(hd->h_filesize_1));
 		t_offt = (t_offt<<16) | ((off_t)(SHRT_EXT(hd->h_filesize_2)));
 		if (arcn->sb.st_size != t_offt) {
-			warn(1,"File is too large for bcpio format %s",
+			pax_warn(1,"File is too large for bcpio format %s",
 			    arcn->org_name);
 			return(1);
 		}
@@ -1244,7 +1242,7 @@ bcpio_wr(arcn)
 	if ((wr_rdbuf(hdblk, (int)sizeof(HD_BCPIO)) < 0) ||
 	    (wr_rdbuf(arcn->name, nsz) < 0) ||
 	    (wr_skip((off_t)(BCPIO_PAD(sizeof(HD_BCPIO) + nsz))) < 0)) {
-		warn(1, "Could not write bcpio header for %s", arcn->org_name);
+		pax_warn(1, "Could not write bcpio header for %s", arcn->org_name);
 		return(-1);
 	}
 
@@ -1266,7 +1264,7 @@ bcpio_wr(arcn)
 	 */
 	if ((wr_rdbuf(arcn->ln_name, arcn->ln_nlen) < 0) ||
 	    (wr_skip((off_t)(BCPIO_PAD(arcn->ln_nlen))) < 0)) {
-		warn(1,"Could not write bcpio link name for %s",arcn->org_name);
+		pax_warn(1,"Could not write bcpio link name for %s",arcn->org_name);
 		return(-1);
 	}
 	return(1);
@@ -1275,6 +1273,6 @@ bcpio_wr(arcn)
 	/*
 	 * header field is out of range
 	 */
-	warn(1,"Bcpio header field is too small for file %s", arcn->org_name);
+	pax_warn(1,"Bcpio header field is too small for file %s", arcn->org_name);
 	return(1);
 }
