@@ -34,19 +34,16 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/pat_rep.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/pat_rep.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)pat_rep.c	8.2 (Berkeley) 4/18/94";
+static char const sccsid[] = "@(#)pat_rep.c	8.2 (Berkeley) 4/18/94";
 #endif /* not lint */
 
 #include <sys/types.h>
-#include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -120,7 +117,7 @@ rep_add(str)
 	 * throw out the bad parameters
 	 */
 	if ((str == NULL) || (*str == '\0')) {
-		warn(1, "Empty replacement string");
+		pax_warn(1, "Empty replacement string");
 		return(-1);
 	}
 
@@ -129,7 +126,7 @@ rep_add(str)
 	 * this expression
 	 */
 	if ((pt1 = strchr(str+1, *str)) == NULL) {
-		warn(1, "Invalid replacement string %s", str);
+		pax_warn(1, "Invalid replacement string %s", str);
 		return(-1);
 	}
 
@@ -138,7 +135,7 @@ rep_add(str)
 	 * and split out the regular expression and try to compile it
 	 */
 	if ((rep = (REPLACE *)malloc(sizeof(REPLACE))) == NULL) {
-		warn(1, "Unable to allocate memory for replacement string");
+		pax_warn(1, "Unable to allocate memory for replacement string");
 		return(-1);
 	}
 
@@ -148,7 +145,7 @@ rep_add(str)
 #	else
 	if ((res = regcomp(&(rep->rcmp), str+1, 0)) != 0) {
 		regerror(res, &(rep->rcmp), rebuf, sizeof(rebuf));
-		warn(1, "%s while compiling regular expression %s", rebuf, str);
+		pax_warn(1, "%s while compiling regular expression %s", rebuf, str);
 #	endif
 		(void)free((char *)rep);
 		return(-1);
@@ -167,7 +164,7 @@ rep_add(str)
 		regfree(&(rep->rcmp));
 #		endif
 		(void)free((char *)rep);
-		warn(1, "Invalid replacement string %s", str);
+		pax_warn(1, "Invalid replacement string %s", str);
 		return(-1);
 	}
 
@@ -197,7 +194,7 @@ rep_add(str)
 #			endif
 			(void)free((char *)rep);
 			*pt1 = *str;
-			warn(1, "Invalid replacement string option %s", str);
+			pax_warn(1, "Invalid replacement string option %s", str);
 			return(-1);
 		}
 		++pt2;
@@ -242,7 +239,7 @@ pat_add(str)
 	 * throw out the junk
 	 */
 	if ((str == NULL) || (*str == '\0')) {
-		warn(1, "Empty pattern string");
+		pax_warn(1, "Empty pattern string");
 		return(-1);
 	}
 
@@ -252,7 +249,7 @@ pat_add(str)
 	 * node to the end of the pattern list
 	 */
 	if ((pt = (PATTERN *)malloc(sizeof(PATTERN))) == NULL) {
-		warn(1, "Unable to allocate memory for pattern string");
+		pax_warn(1, "Unable to allocate memory for pattern string");
 		return(-1);
 	}
 
@@ -295,7 +292,7 @@ pat_chk()
 		if (pt->flgs & MTCH)
 			continue;
 		if (!wban) {
-			warn(1, "WARNING! These patterns were not matched:");
+			pax_warn(1, "WARNING! These patterns were not matched:");
 			++wban;
 		}
 		(void)fprintf(stderr, "%s\n", pt->pstr);
@@ -374,7 +371,7 @@ pat_sel(arcn)
 			*pt->pend = '\0';
 
 		if ((pt->pstr = strdup(arcn->name)) == NULL) {
-			warn(1, "Pattern select out of memory");
+			pax_warn(1, "Pattern select out of memory");
 			if (pt->pend != NULL)
 				*pt->pend = '/';
 			pt->pend = NULL;
@@ -422,7 +419,7 @@ pat_sel(arcn)
 		/*
 		 * should never happen....
 		 */
-		warn(1, "Pattern list inconsistant");
+		pax_warn(1, "Pattern list inconsistant");
 		return(-1);
 	}
 	*ppt = pt->fow;
@@ -777,6 +774,7 @@ tty_rename(arcn)
 	tty_prnt("Processing continues, name changed to: %s\n", tmpname);
 	res = add_name(arcn->name, arcn->nlen, tmpname);
 	arcn->nlen = l_strncpy(arcn->name, tmpname, PAXPATHLEN+1);
+	arcn->name[PAXPATHLEN] = '\0';
 	if (res < 0)
 		return(-1);
 	return(0);
@@ -856,7 +854,7 @@ fix_path(or_name, or_len, dir_name, dir_len)
 		--dest;
 	}
 	if ((len = dest - or_name) > PAXPATHLEN) {
-		warn(1, "File name %s/%s, too long", dir_name, start);
+		pax_warn(1, "File name %s/%s, too long", dir_name, start);
 		return(-1);
 	}
 	*or_len = len;
@@ -983,7 +981,7 @@ rep_name(name, nlen, prnt)
 			    < 0) {
 #			endif
 				if (prnt)
-					warn(1, "Replacement name error %s",
+					pax_warn(1, "Replacement name error %s",
 					    name);
 				return(1);
 			}
@@ -1034,7 +1032,7 @@ rep_name(name, nlen, prnt)
 		*outpt = '\0';
 		if ((outpt == endpt) && (*inpt != '\0')) {
 			if (prnt)
-				warn(1,"Replacement name too long %s >> %s",
+				pax_warn(1,"Replacement name too long %s >> %s",
 				    name, nname);
 			return(1);
 		}
@@ -1057,6 +1055,7 @@ rep_name(name, nlen, prnt)
 		if (*nname == '\0')
 			return(1);
 		*nlen = l_strncpy(name, nname, PAXPATHLEN + 1);
+		name[PAXPATHLEN] = '\0';
 	}
 	return(0);
 }
