@@ -34,21 +34,19 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/sel_subs.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/sel_subs.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)sel_subs.c	8.1 (Berkeley) 5/31/93";
+static char const sccsid[] = "@(#)sel_subs.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <pwd.h>
 #include <grp.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <string.h>
 #include <strings.h>
 #include <unistd.h>
@@ -130,7 +128,7 @@ usr_add(str)
 		return(-1);
 	if ((usrtb == NULL) &&
  	    ((usrtb = (USRT **)calloc(USR_TB_SZ, sizeof(USRT *))) == NULL)) {
-                warn(1, "Unable to allocate memory for user selection table");
+                pax_warn(1, "Unable to allocate memory for user selection table");
                 return(-1);
 	}
 
@@ -144,7 +142,7 @@ usr_add(str)
 		if ((str[0] == '\\') && (str[1] == '#'))
 			++str;
 		if ((pw = getpwnam(str)) == NULL) {
-                	warn(1, "Unable to find uid for user: %s", str);
+                	pax_warn(1, "Unable to find uid for user: %s", str);
                 	return(-1);
 		}
 		uid = (uid_t)pw->pw_uid;
@@ -177,7 +175,7 @@ usr_add(str)
 		usrtb[indx] = pt;
 		return(0);
 	}
-        warn(1, "User selection table out of memory");
+        pax_warn(1, "User selection table out of memory");
         return(-1);
 }
 
@@ -243,7 +241,7 @@ grp_add(str)
 		return(-1);
 	if ((grptb == NULL) &&
  	    ((grptb = (GRPT **)calloc(GRP_TB_SZ, sizeof(GRPT *))) == NULL)) {
-                warn(1, "Unable to allocate memory fo group selection table");
+                pax_warn(1, "Unable to allocate memory fo group selection table");
                 return(-1);
 	}
 
@@ -257,7 +255,7 @@ grp_add(str)
 		if ((str[0] == '\\') && (str[1] == '#'))
 			++str;
 		if ((gr = getgrnam(str)) == NULL) {
-                	warn(1,"Cannot determine gid for group name: %s", str);
+                	pax_warn(1,"Cannot determine gid for group name: %s", str);
                 	return(-1);
 		}
 		gid = (gid_t)gr->gr_gid;
@@ -290,7 +288,7 @@ grp_add(str)
 		grptb[indx] = pt;
 		return(0);
 	}
-        warn(1, "Group selection table out of memory");
+        pax_warn(1, "Group selection table out of memory");
         return(-1);
 }
 
@@ -376,7 +374,7 @@ trng_add(str)
 	 * throw out the badly formed time ranges
 	 */
 	if ((str == NULL) || (*str == '\0')) {
-		warn(1, "Empty time range string");
+		pax_warn(1, "Empty time range string");
 		return(-1);
 	}
 
@@ -403,7 +401,7 @@ trng_add(str)
 			++dot;
 			continue;
 		}
-		warn(1, "Improperly specified time range: %s", str);
+		pax_warn(1, "Improperly specified time range: %s", str);
 		goto out;
 	}
 
@@ -411,7 +409,7 @@ trng_add(str)
 	 * allocate space for the time range and store the limits
 	 */
 	if ((pt = (TIME_RNG *)malloc(sizeof(TIME_RNG))) == NULL) {
-		warn(1, "Unable to allocate memory for time range");
+		pax_warn(1, "Unable to allocate memory for time range");
 		return(-1);
 	}
 
@@ -434,7 +432,7 @@ trng_add(str)
 				pt->flgs |= CMPCTME;
 				break;
 			default:
-				warn(1, "Bad option %c with time range %s",
+				pax_warn(1, "Bad option %c with time range %s",
 				    *flgpt, str);
 				goto out;
 			}
@@ -451,7 +449,7 @@ trng_add(str)
 		 * add lower limit
 		 */
 		if (str_sec(str, &(pt->low_time)) < 0) {
-			warn(1, "Illegal lower time range %s", str);
+			pax_warn(1, "Illegal lower time range %s", str);
 			(void)free((char *)pt);
 			goto out;
 		}
@@ -463,7 +461,7 @@ trng_add(str)
 		 * add upper limit
 		 */
 		if (str_sec(up_pt, &(pt->high_time)) < 0) {
-			warn(1, "Illegal upper time range %s", up_pt);
+			pax_warn(1, "Illegal upper time range %s", up_pt);
 			(void)free((char *)pt);
 			goto out;
 		}
@@ -474,7 +472,7 @@ trng_add(str)
 		 */
 		if (pt->flgs & HASLOW) {
 			if (pt->low_time > pt->high_time) {
-				warn(1, "Upper %s and lower %s time overlap",
+				pax_warn(1, "Upper %s and lower %s time overlap",
 					up_pt, str);
 				(void)free((char *)pt);
 				return(-1);
@@ -492,7 +490,7 @@ trng_add(str)
 	return(0);
 
     out:
-	warn(1, "Time range format is: [yy[mm[dd[hh]]]]mm[.ss][/[c][m]]");
+	pax_warn(1, "Time range format is: [yy[mm[dd[hh]]]]mm[.ss][/[c][m]]");
 	return(-1);
 }
 
