@@ -34,24 +34,22 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/pax/tables.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/pax/tables.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)tables.c	8.1 (Berkeley) 5/31/93";
+static char const sccsid[] = "@(#)tables.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <sys/fcntl.h>
+#include <errno.h>
 #include <stdio.h>
-#include <ctype.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
-#include <stdlib.h>
 #include "pax.h"
 #include "tables.h"
 #include "extern.h"
@@ -116,7 +114,7 @@ lnk_start()
 	if (ltab != NULL)
 		return(0);
  	if ((ltab = (HRDLNK **)calloc(L_TAB_SZ, sizeof(HRDLNK *))) == NULL) {
-                warn(1, "Cannot allocate memory for hard link table");
+                pax_warn(1, "Cannot allocate memory for hard link table");
                 return(-1);
         }
 	return(0);
@@ -181,6 +179,7 @@ chk_lnk(arcn)
 			 */
 			arcn->ln_nlen = l_strncpy(arcn->ln_name, pt->name,
 				PAXPATHLEN+1);
+			arcn->ln_name[PAXPATHLEN] = '\0';
 			if (arcn->type == PAX_REG)
 				arcn->type = PAX_HRG;
 			else
@@ -215,7 +214,7 @@ chk_lnk(arcn)
 		(void)free((char *)pt);
 	}
 
-	warn(1, "Hard link table out of memory");
+	pax_warn(1, "Hard link table out of memory");
 	return(-1);
 }
 
@@ -364,7 +363,7 @@ ftime_start()
 	if (ftab != NULL)
 		return(0);
  	if ((ftab = (FTM **)calloc(F_TAB_SZ, sizeof(FTM *))) == NULL) {
-                warn(1, "Cannot allocate memory for file time table");
+                pax_warn(1, "Cannot allocate memory for file time table");
                 return(-1);
         }
 
@@ -377,7 +376,7 @@ ftime_start()
 	(void)unlink(pt);
 
 	if ((ffd = open(pt, O_RDWR | O_CREAT,  S_IRWXU)) < 0) {
-		syswarn(1, errno, "Unable to open temporary file: %s", pt);
+		sys_warn(1, errno, "Unable to open temporary file: %s", pt);
 		return(-1);
 	}
 
@@ -435,12 +434,12 @@ chk_ftime(arcn)
 				 * from the scratch file.
 				 */
 				if (lseek(ffd,pt->seek,SEEK_SET) != pt->seek) {
-					syswarn(1, errno,
+					sys_warn(1, errno,
 					    "Failed ftime table seek");
 					return(-1);
 				}
 				if (read(ffd, ckname, namelen) != namelen) {
-					syswarn(1, errno,
+					sys_warn(1, errno,
 					    "Failed ftime table read");
 					return(-1);
 				}
@@ -492,11 +491,11 @@ chk_ftime(arcn)
 				ftab[indx] = pt;
 				return(0);
 			}
-			syswarn(1, errno, "Failed write to file time table");
+			sys_warn(1, errno, "Failed write to file time table");
 		} else
-			syswarn(1, errno, "Failed seek on file time table");
+			sys_warn(1, errno, "Failed seek on file time table");
 	} else
-		warn(1, "File time table ran out of memory");
+		pax_warn(1, "File time table ran out of memory");
 
 	if (pt != NULL)
 		(void)free((char *)pt);
@@ -533,7 +532,7 @@ name_start()
 	if (ntab != NULL)
 		return(0);
  	if ((ntab = (NAMT **)calloc(N_TAB_SZ, sizeof(NAMT *))) == NULL) {
-                warn(1, "Cannot allocate memory for interactive rename table");
+                pax_warn(1, "Cannot allocate memory for interactive rename table");
                 return(-1);
         }
 	return(0);
@@ -566,7 +565,7 @@ add_name(oname, onamelen, nname)
 		/*
 		 * should never happen
 		 */
-		warn(0, "No interactive rename table, links may fail\n");
+		pax_warn(0, "No interactive rename table, links may fail\n");
 		return(0);
 	}
 
@@ -592,7 +591,7 @@ add_name(oname, onamelen, nname)
 
 			(void)free((char *)pt->nname);
 			if ((pt->nname = strdup(nname)) == NULL) {
-				warn(1, "Cannot update rename table");
+				pax_warn(1, "Cannot update rename table");
 				return(-1);
 			}
 			return(0);
@@ -613,7 +612,7 @@ add_name(oname, onamelen, nname)
 		}
 		(void)free((char *)pt);
 	}
-	warn(1, "Interactive rename table out of memory");
+	pax_warn(1, "Interactive rename table out of memory");
 	return(-1);
 }
 
@@ -656,6 +655,7 @@ sub_name(oname, onamelen)
 			 * and return (we know that oname has enough space)
 			 */
 			*onamelen = l_strncpy(oname, pt->nname, PAXPATHLEN+1);
+			oname[PAXPATHLEN] = '\0';
 			return;
 		}
 		pt = pt->fow;
@@ -725,7 +725,7 @@ dev_start()
 	if (dtab != NULL)
 		return(0);
  	if ((dtab = (DEVT **)calloc(D_TAB_SZ, sizeof(DEVT *))) == NULL) {
-                warn(1, "Cannot allocate memory for device mapping table");
+                pax_warn(1, "Cannot allocate memory for device mapping table");
                 return(-1);
         }
 	return(0);
@@ -811,7 +811,7 @@ chk_dev(dev, add)
 	 * list must be NULL.
 	 */
 	if ((pt = (DEVT *)malloc(sizeof(DEVT))) == NULL) {
-		warn(1, "Device map table out of memory");
+		pax_warn(1, "Device map table out of memory");
 		return(NULL);
 	}
 	pt->dev = dev;
@@ -953,9 +953,9 @@ map_dev(arcn, dev_mask, ino_mask)
 	return(0);
 
     bad:
-	warn(1, "Unable to fix truncated inode/device field when storing %s",
+	pax_warn(1, "Unable to fix truncated inode/device field when storing %s",
 	    arcn->name);
-	warn(0, "Archive may create improper hard links when extracted");
+	pax_warn(0, "Archive may create improper hard links when extracted");
 	return(0);
 }
 
@@ -994,7 +994,7 @@ atdir_start()
 	if (atab != NULL)
 		return(0);
  	if ((atab = (ATDIR **)calloc(A_TAB_SZ, sizeof(ATDIR *))) == NULL) {
-                warn(1,"Cannot allocate space for directory access time table");
+                pax_warn(1,"Cannot allocate space for directory access time table");
                 return(-1);
         }
 	return(0);
@@ -1101,7 +1101,7 @@ add_atdir(fname, dev, ino, mtime, atime)
 		(void)free((char *)pt);
 	}
 
-	warn(1, "Directory access time reset table ran out of memory");
+	pax_warn(1, "Directory access time reset table ran out of memory");
 	return;
 }
 
@@ -1223,7 +1223,7 @@ dir_start()
 		(void)unlink(pt);
 		return(0);
 	}
-	warn(1, "Unable to create temporary file for directory times: %s", pt);
+	pax_warn(1, "Unable to create temporary file for directory times: %s", pt);
 	return(-1);
 }
 
@@ -1262,7 +1262,7 @@ add_dir(name, nlen, psb, frc_mode)
 	 * in the trailer
 	 */
 	if ((dblk.npos = lseek(dirfd, 0L, SEEK_CUR)) < 0) {
-		warn(1,"Unable to store mode and times for directory: %s",name);
+		pax_warn(1,"Unable to store mode and times for directory: %s",name);
 		return;
 	}
 
@@ -1280,7 +1280,7 @@ add_dir(name, nlen, psb, frc_mode)
 		return;
 	}
 
-	warn(1,"Unable to store mode and times for created directory: %s",name);
+	pax_warn(1,"Unable to store mode and times for created directory: %s",name);
 	return;
 }
 
@@ -1336,7 +1336,7 @@ proc_dir()
 	(void)close(dirfd);
 	dirfd = -1;
 	if (cnt != dircnt)
-		warn(1,"Unable to set mode and times for created directories");
+		pax_warn(1,"Unable to set mode and times for created directories");
 	return;
 }
 
