@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/ps/fmt.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/ps/fmt.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -41,10 +41,10 @@ static char sccsid[] = "@(#)fmt.c	8.4 (Berkeley) 4/15/94";
 #include <sys/syslimits.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <vis.h>
 #include "ps.h"
 
@@ -59,8 +59,16 @@ static char *
 shquote(argv)
 	char **argv;
 {
+	long arg_max;
 	char **p, *dst, *src;
-	static char buf[4*ARG_MAX];		/* XXX */
+	static char *buf = NULL;
+
+	if (buf == NULL) {
+		if ((arg_max = sysconf(_SC_ARG_MAX)) == -1)
+			errx(1, "sysconf _SC_ARG_MAX failed");
+		if ((buf = malloc((4 * arg_max)  +  1)) == NULL)
+			errx(1, "malloc failed");
+	}
 
 	if (*argv == 0) {
 		buf[0] = 0;
