@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/ps/keyword.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/ps/keyword.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -57,6 +57,7 @@ static char sccsid[] = "@(#)keyword.c	8.5 (Berkeley) 4/2/94";
 
 #ifdef NEWVM
 #include <sys/ucred.h>
+#include <sys/user.h>
 #include <sys/sysctl.h>
 #endif
 
@@ -134,11 +135,11 @@ VAR var[] = {
 	{"re", "RE", NULL, 0, pvar, 3, POFF(p_swtime), ULONG, "d"},
 	{"rgid", "RGID", NULL, 0, evar, UIDLEN, EOFF(e_pcred.p_rgid),
 		ULONG, UIDFMT},
-	{"rlink", "RLINK", NULL, 0, pvar, 8, POFF(p_back), KPTR, "x"},
+	{"rlink", "RLINK", NULL, 0, pvar, 8, POFF(p_procq.tqe_prev), KPTR, "x"},
 	{"rss", "RSS", NULL, 0, p_rssize, 4},
 	{"rssize", "", "rsz"},
 	{"rsz", "RSZ", NULL, 0, rssize, 4},
-	{"rtprio", "RTPRIO", NULL, 0, pvar, 7, POFF(p_rtprio), LONG, "d"},
+	{"rtprio", "RTPRIO", NULL, 0, rtprior, 7, POFF(p_rtprio)},
 	{"ruid", "RUID", NULL, 0, evar, UIDLEN, EOFF(e_pcred.p_ruid),
 		ULONG, UIDFMT},
 	{"ruser", "RUSER", NULL, LJUST, runame, USERLEN},
@@ -344,8 +345,7 @@ findvar(p)
 	if (!v) {
 		warnx("%s: keyword not found", p);
 		eval = 1;
-	}
-	if (hp)
+	} else if (hp)
 		v->header = hp;
 	return (v);
 }
