@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/ps/print.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/ps/print.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -49,6 +49,7 @@ static char sccsid[] = "@(#)print.c	8.6 (Berkeley) 4/16/94";
 
 #ifdef NEWVM
 #include <sys/ucred.h>
+#include <sys/user.h>
 #include <sys/sysctl.h>
 #include <vm/vm.h>
 #else
@@ -63,6 +64,7 @@ static char sccsid[] = "@(#)print.c	8.6 (Berkeley) 4/16/94";
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <vis.h>
 
@@ -98,6 +100,16 @@ command(k, ve)
 	int left;
 	char *cp, *vis_env, *vis_args;
 
+	v = ve->var;
+
+	if (cflag) {
+		if (ve->next == NULL)	/* last field, don't pad */
+			(void)printf("%s", KI_PROC(k)->p_comm);
+		else
+			(void)printf("%-*s", v->width, KI_PROC(k)->p_comm);
+		return;
+	}
+
 	if ((vis_args = malloc(strlen(k->ki_args) * 4 + 1)) == NULL)
 		err(1, NULL);
 	strvis(vis_args, k->ki_args, VIS_TAB | VIS_NL | VIS_NOSLASH);
@@ -108,7 +120,6 @@ command(k, ve)
 	} else
 		vis_env = NULL;
 
-	v = ve->var;
 	if (ve->next == NULL) {
 		/* last field */
 		if (termwidth == UNLIMITED) {
@@ -153,12 +164,13 @@ logname(k, ve)
 	VARENT *ve;
 {
 	VAR *v;
+	char *s;
 
 	v = ve->var;
 #ifndef NEWVM
 	(void)printf("%-*s", v->width, KI_PROC(k)->p_logname);
 #else
-	(void)printf("%-*s", v->width, KI_EPROC(k)->e_login);
+	(void)printf("%-*s", v->width, (s = KI_EPROC(k)->e_login, *s) ? s : "-");
 #endif
 }
 
@@ -394,7 +406,7 @@ lstarted(k, ve)
 		(void)printf("%-*s", v->width, "-");
 		return;
 	}
-	(void)strftime(buf, sizeof(buf) -1, "%C",
+	(void)strftime(buf, sizeof(buf) -1, "%c",
 	    localtime(&k->ki_u.u_start.tv_sec));
 	(void)printf("%-*s", v->width, buf);
 }
@@ -418,7 +430,7 @@ wchan(k, ve)
 		(void)printf("%-*s", v->width, "-");
 }
 
-#define pgtok(a)        (((a)*NBPG)/1024)
+#define pgtok(a)        (((a)*getpagesize())/1024)
 
 void
 vsize(k, ve)
@@ -530,8 +542,8 @@ getpcpu(k)
 	if (p->p_swtime == 0 || (p->p_flag & P_INMEM) == 0)
 		return (0.0);
 	if (rawcpu)
-		return (100.0 * fxtofl(p->p_pctcpu));
-	return (100.0 * fxtofl(p->p_pctcpu) /
+		return (10000.0 / sysconf(_SC_CLK_TCK) * fxtofl(p->p_pctcpu));
+	return (10000.0 / sysconf(_SC_CLK_TCK) * fxtofl(p->p_pctcpu) /
 		(1.0 - exp(p->p_swtime * log(fxtofl(ccpu)))));
 }
 
@@ -574,7 +586,7 @@ getpmem(k)
 	/* XXX want pmap ptpages, segtab, etc. (per architecture) */
 	szptudot = UPAGES;
 	/* XXX don't have info about shared */
-	fracmem = ((float)e->e_vm.vm_rssize + szptudot)/CLSIZE/mempages;
+	fracmem = ((float)e->e_vm.vm_rssize + szptudot)/mempages;
 #endif
 	return (100.0 * fracmem);
 }
@@ -611,7 +623,7 @@ maxrss(k, ve)
 
 	v = ve->var;
 #ifndef NEWVM	/* not yet */
-	if (KI_PROC(k)->p_maxrss != (RLIM_INFINITY/NBPG))
+	if (KI_PROC(k)->p_maxrss != (RLIM_INFINITY/PAGE_SIZE))
 		(void)printf("%*d", v->width, pgtok(KI_PROC(k)->p_maxrss));
 	else
 #endif
@@ -646,6 +658,37 @@ trss(k, ve)
 }
 #endif
 
+void
+rtprior(k, ve)
+	KINFO *k;
+	VARENT *ve;
+{
+	VAR *v;
+	struct rtprio *prtp;
+	char str[8];
+	unsigned type, prio;
+
+	v = ve->var;
+	prtp = (struct rtprio *) ((char *)KI_PROC(k) + v->off);
+	prio = prtp->prio;
+	switch (type = prtp->type) {
+	case RTP_PRIO_REALTIME:
+	    snprintf(str, sizeof(str), "real:%u", prio);
+	    break;
+	case RTP_PRIO_NORMAL:
+	    strncpy(str, "normal", sizeof(str));
+	    break;
+	case RTP_PRIO_IDLE:
+	    snprintf(str, sizeof(str), "idle:%u", prio);
+	    break;
+	default:
+	    snprintf(str, sizeof(str), "%u:%u", type, prio);
+	    break;
+	}
+	str[sizeof(str)-1] = '\0';
+	(void)printf("%*s", v->width, str);
+}
+
 /*
  * Generic output routines.  Print fields from various prototype
  * structures.
