@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/ps/ps.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/ps/ps.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -47,7 +47,6 @@ static char sccsid[] = "@(#)ps.c	8.4 (Berkeley) 4/2/94";
 #include <sys/user.h>
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <sys/proc.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/sysctl.h>
@@ -57,13 +56,15 @@ static char sccsid[] = "@(#)ps.c	8.4 (Berkeley) 4/2/94";
 #include <errno.h>
 #include <fcntl.h>
 #include <kvm.h>
+#include <limits.h>
 #include <nlist.h>
 #include <paths.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <pwd.h>
 #include <unistd.h>
+#include <locale.h>
+#include <pwd.h>
 
 #include "ps.h"
 
@@ -75,6 +76,7 @@ KINFO *kinfo;
 struct varent *vhead, *vtail;
 
 int	eval;			/* exit value */
+int	cflag;			/* -c */
 int	rawcpu;			/* -C */
 int	sumrusage;		/* -S */
 int	termwidth;		/* width of screen (0 == infinity) */
@@ -82,6 +84,12 @@ int	totwidth;		/* calculated width of requested variables */
 
 static int needuser, needcomm, needenv;
 
+#if defined(LAZY_PS)
+static int forceuread=0;
+#else
+static int forceuread=1;
+#endif
+
 enum sort { DEFAULT, SORTMEM, SORTCPU } sortby = DEFAULT;
 
 static char	*fmt __P((char **(*)(kvm_t *, const struct kinfo_proc *, int),
@@ -116,7 +124,9 @@ main(argc, argv)
 	uid_t uid;
 	int all, ch, flag, i, fmt, lineno, nentries;
 	int prtheader, wflag, what, xflg;
-	char *nlistf, *memf, *swapf, errbuf[256];
+	char *nlistf, *memf, *swapf, errbuf[_POSIX2_LINE_MAX];
+
+	(void) setlocale(LC_ALL, "");
 
 	if ((ioctl(STDOUT_FILENO, TIOCGWINSZ, (char *)&ws) == -1 &&
 	     ioctl(STDERR_FILENO, TIOCGWINSZ, (char *)&ws) == -1 &&
@@ -135,17 +145,30 @@ main(argc, argv)
 	ttydev = NODEV;
 	memf = nlistf = swapf = NULL;
 	while ((ch = getopt(argc, argv,
-	    "aCeghjLlM:mN:O:o:p:rSTt:U:uvW:wx")) != EOF)
+#if defined(LAZY_PS)
+	    "aCcefghjLlM:mN:O:o:p:rSTt:U:uvW:wx")) != EOF)
+#else
+	    "aCceghjLlM:mN:O:o:p:rSTt:U:uvW:wx")) != EOF)
+#endif
 		switch((char)ch) {
 		case 'a':
 			all = 1;
 			break;
-		case 'e':			/* XXX set ufmt */
-			needenv = 1;
-			break;
 		case 'C':
 			rawcpu = 1;
 			break;
+		case 'c':
+			cflag = 1;
+			break;
+		case 'e':			/* XXX set ufmt */
+			needenv = 1;
+			break;
+#if defined(LAZY_PS)
+		case 'f':
+			if (getuid() == 0 || getgid() == 0)
+				forceuread = 1;
+			break;
+#endif
 		case 'g':
 			break;			/* no-op */
 		case 'h':
@@ -386,6 +409,8 @@ fmt(fn, ki, comm, maxlen)
 	return (s);
 }
 
+#define UREADOK(ki)	(forceuread || (KI_PROC(ki)->p_flag & P_INMEM))
+
 static void
 saveuser(ki)
 	KINFO *ki;
@@ -395,7 +420,7 @@ saveuser(ki)
 	struct user *u_addr = (struct user *)USRSTACK;
 
 	usp = &ki->ki_u;
-	if (kvm_uread(kd, KI_PROC(ki), (unsigned long)&u_addr->u_stats,
+	if (UREADOK(ki) && kvm_uread(kd, KI_PROC(ki), (unsigned long)&u_addr->u_stats,
 	    (char *)&pstats, sizeof(pstats)) == sizeof(pstats)) {
 		/*
 		 * The u-area might be swapped out, and we can't get
@@ -412,15 +437,23 @@ saveuser(ki)
 	/*
 	 * save arguments if needed
 	 */
-	if (needcomm)
+	if (needcomm && UREADOK(ki)) {
 		ki->ki_args = fmt(kvm_getargv, ki, KI_PROC(ki)->p_comm,
 		    MAXCOMLEN);
-	else
+	} else if (needcomm) {
+		ki->ki_args = malloc(strlen(KI_PROC(ki)->p_comm) + 3);
+		sprintf(ki->ki_args, "(%s)", KI_PROC(ki)->p_comm);
+	} else {
 		ki->ki_args = NULL;
-	if (needenv)
+	}
+	if (needenv && UREADOK(ki)) {
 		ki->ki_env = fmt(kvm_getenvv, ki, (char *)NULL, 0);
-	else
+	} else if (needenv) {
+		ki->ki_env = malloc(3);
+		strcpy(ki->ki_env, "()");
+	} else {
 		ki->ki_env = NULL;
+	}
 }
 
 static int
@@ -510,10 +543,9 @@ static void
 usage()
 {
 
-	(void)fprintf(stderr,
-	    "usage:\t%s\n\t   %s\n\t%s\n",
-	    "ps [-aChjlmrSTuvwx] [-O|o fmt] [-p pid] [-t tty] [-U user]",
-	    "[-M core] [-N system] [-W swap]",
-	    "ps [-L]");
+	(void)fprintf(stderr, "%s\n%s\n%s\n",
+	    "usage: ps [-aChjlmrSTuvwx] [-O|o fmt] [-p pid] [-t tty] [-U user]",
+	    "          [-M core] [-N system] [-W swap]",
+	    "       ps [-L]");
 	exit(1);
 }
