@@ -31,10 +31,12 @@
  * SUCH DAMAGE.
  *
  *	@(#)pathnames.h	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/rcp/pathnames.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/rcp/pathnames.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <paths.h>
 
 #define	_PATH_CP	"/bin/cp"
+#define	_PATH_RCP	"/bin/rcp"
+#define	_PATH_RLOGIN	"/usr/bin/rlogin"
 #define	_PATH_RSH	"/usr/bin/rsh"
