@@ -32,14 +32,17 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/alias.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)alias.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)alias.c	8.3 (Berkeley) 5/4/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/alias.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <stdlib.h>
 #include "shell.h"
 #include "input.h"
 #include "output.h"
@@ -53,12 +56,15 @@ static char sccsid[] = "@(#)alias.c	8.1 (Berkeley) 5/31/93";
 
 struct alias *atab[ATABSIZE];
 
+STATIC void setalias __P((char *, char *));
+STATIC int unalias __P((char *));
 STATIC struct alias **hashalias __P((char *));
 
 STATIC
+void
 setalias(name, val)
 	char *name, *val;
-	{
+{
 	struct alias *ap, **app;
 
 	app = hashalias(name);
@@ -98,7 +104,7 @@ setalias(name, val)
 	{
 	int len = strlen(val);
 	ap->val = ckmalloc(len + 2);
-	bcopy(val, ap->val, len);
+	memcpy(ap->val, val, len);
 	ap->val[len] = ' ';	/* fluff */
 	ap->val[len+1] = '\0';
 	}
@@ -173,7 +179,8 @@ rmaliases() {
 struct alias *
 lookupalias(name, check)
 	char *name;
-	{
+	int check;
+{
 	struct alias *ap = *hashalias(name);
 
 	for (; ap; ap = ap->next) {
@@ -190,9 +197,11 @@ lookupalias(name, check)
 /*
  * TODO - sort output
  */
+int
 aliascmd(argc, argv)
+	int argc;
 	char **argv;
-	{
+{
 	char *n, *v;
 	int ret = 0;
 	struct alias *ap;
@@ -207,7 +216,7 @@ aliascmd(argc, argv)
 			}
 		return (0);
 	}
-	while (n = *++argv) {
+	while ((n = *++argv) != NULL) {
 		if ((v = strchr(n+1, '=')) == NULL) /* n+1: funny ksh stuff */
 			if ((ap = lookupalias(n, 0)) == NULL) {
 				outfmt(out2, "alias: %s not found\n", n);
@@ -223,9 +232,11 @@ aliascmd(argc, argv)
 	return (ret);
 }
 
+int
 unaliascmd(argc, argv)
-	char **argv;
-	{
+	int argc __unused;
+	char **argv __unused;
+{
 	int i;
 
 	while ((i = nextopt("a")) != '\0') {
@@ -242,7 +253,7 @@ unaliascmd(argc, argv)
 
 STATIC struct alias **
 hashalias(p)
-	register char *p;
+	char *p;
 	{
 	unsigned int hashval;
 
