@@ -33,8 +33,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)bltin.h	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/sh/bltin/bltin.h 50952 1999-09-05 11:54:50Z peter $
+ *	@(#)bltin.h	8.2 (Berkeley) 5/4/95
+ * $FreeBSD: stable/2.2/bin/sh/bltin/bltin.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 /*
@@ -60,11 +60,12 @@
 #define fputs outstr
 #define fflush flushout
 #define INITARGS(argv)
-#define warnx(a, b, c) {                                               \
-       char buf[64];                                                   \
-       (void)snprintf(buf, sizeof(buf), a, b, c);                       \
-       error(buf);                                                     \
+#define warnx(a, b, c) {				\
+	char buf[64];					\
+	(void)snprintf(buf, sizeof(buf), a, b, c);	\
+	error("%s", buf);				\
 }
+
 #else
 #undef NULL
 #include <stdio.h>
