@@ -33,27 +33,28 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)echo.c	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/sh/bltin/echo.c 50952 1999-09-05 11:54:50Z peter $
+ *	@(#)echo.c	8.2 (Berkeley) 5/4/95
+ * $FreeBSD: stable/2.2/bin/sh/bltin/echo.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 /*
  * Echo command.
  */
 
-#ifdef main
-#undef main
-#endif
 #define main echocmd
 
 #include "bltin.h"
 
 /* #define eflag 1 */
 
-main(argc, argv)  char **argv; {
-	register char **ap;
-	register char *p;
-	register char c;
+int
+main(argc, argv)
+	int argc;
+	char **argv;
+{
+	char **ap;
+	char *p;
+	char c;
 	int count;
 	int nflag = 0;
 #ifndef eflag
