@@ -32,14 +32,19 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/eval.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)eval.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)eval.c	8.9 (Berkeley) 6/8/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/eval.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <signal.h>
+#include <unistd.h>
+
 /*
  * Evaluate a command.
  */
@@ -61,9 +66,11 @@ static char sccsid[] = "@(#)eval.c	8.1 (Berkeley) 5/31/93";
 #include "var.h"
 #include "memalloc.h"
 #include "error.h"
+#include "show.h"
 #include "mystring.h"
+#ifndef NO_HISTORY
 #include "myhistedit.h"
-#include <signal.h>
+#endif
 
 
 /* flags in argument to evaltree */
@@ -71,12 +78,6 @@ static char sccsid[] = "@(#)eval.c	8.1 (Berkeley) 5/31/93";
 #define EV_TESTED 02		/* exit status is checked; ignore -e flag */
 #define EV_BACKCMD 04		/* command executing within back quotes */
 
-
-/* reasons for skipping commands (see comment on breakcmd routine) */
-#define SKIPBREAK 1
-#define SKIPCONT 2
-#define SKIPFUNC 3
-
 MKINIT int evalskip;		/* set if we are skipping commands */
 STATIC int skipcount;		/* number of levels to skip */
 MKINIT int loopnest;		/* current loop nesting level */
@@ -86,28 +87,17 @@ int funcnest;			/* depth of function calls */
 char *commandname;
 struct strlist *cmdenviron;
 int exitstatus;			/* exit status of last command */
+int oexitstatus;		/* saved exit status */
 
 
-#ifdef __STDC__
-STATIC void evalloop(union node *);
-STATIC void evalfor(union node *);
-STATIC void evalcase(union node *, int);
-STATIC void evalsubshell(union node *, int);
-STATIC void expredir(union node *);
-STATIC void evalpipe(union node *);
-STATIC void evalcommand(union node *, int, struct backcmd *);
-STATIC void prehash(union node *);
-#else
-STATIC void evalloop();
-STATIC void evalfor();
-STATIC void evalcase();
-STATIC void evalsubshell();
-STATIC void expredir();
-STATIC void evalpipe();
-STATIC void evalcommand();
-STATIC void prehash();
-#endif
-
+STATIC void evalloop __P((union node *));
+STATIC void evalfor __P((union node *));
+STATIC void evalcase __P((union node *, int));
+STATIC void evalsubshell __P((union node *, int));
+STATIC void expredir __P((union node *));
+STATIC void evalpipe __P((union node *));
+STATIC void evalcommand __P((union node *, int, struct backcmd *));
+STATIC void prehash __P((union node *));
 
 
 /*
@@ -134,7 +124,9 @@ SHELLPROC {
  * The eval commmand.
  */
 
+int
 evalcmd(argc, argv)
+	int argc;
 	char **argv;
 {
         char *p;
@@ -193,14 +185,17 @@ evalstring(s)
 void
 evaltree(n, flags)
 	union node *n;
-	{
+	int flags;
+{
 	if (n == NULL) {
 		TRACE(("evaltree(NULL) called\n"));
 		exitstatus = 0;
 		goto out;
 	}
+#ifndef NO_HISTORY
 	displayhist = 1;	/* show history substitutions done with fc */
-	TRACE(("evaltree(0x%x: %d) called\n", (int)n, n->type));
+#endif
+	TRACE(("evaltree(0x%lx: %d) called\n", (long)n, n->type));
 	switch (n->type) {
 	case NSEMI:
 		evaltree(n->nbinary.ch1, 0);
@@ -210,8 +205,10 @@ evaltree(n, flags)
 		break;
 	case NAND:
 		evaltree(n->nbinary.ch1, EV_TESTED);
-		if (evalskip || exitstatus != 0)
+		if (evalskip || exitstatus != 0) {
+			flags |= EV_TESTED;
 			goto out;
+		}
 		evaltree(n->nbinary.ch2, flags);
 		break;
 	case NOR:
@@ -233,19 +230,15 @@ evaltree(n, flags)
 		evalsubshell(n, flags);
 		break;
 	case NIF: {
-		int status = 0;
-
 		evaltree(n->nif.test, EV_TESTED);
 		if (evalskip)
 			goto out;
-		if (exitstatus == 0) {
+		if (exitstatus == 0)
 			evaltree(n->nif.ifpart, flags);
-			status = exitstatus;
-		} else if (n->nif.elsepart) {
+		else if (n->nif.elsepart)
 			evaltree(n->nif.elsepart, flags);
-			status = exitstatus;
-		}
-		exitstatus = status;
+		else
+			exitstatus = 0;
 		break;
 	}
 	case NWHILE:
@@ -289,7 +282,7 @@ out:
 STATIC void
 evalloop(n)
 	union node *n;
-	{
+{
 	int status;
 
 	loopnest++;
@@ -325,8 +318,8 @@ skipping:	  if (evalskip == SKIPCONT && --skipcount <= 0) {
 
 STATIC void
 evalfor(n)
-	union node *n;
-	{
+    union node *n;
+{
 	struct arglist arglist;
 	union node *argp;
 	struct strlist *sp;
@@ -335,6 +328,7 @@ evalfor(n)
 	setstackmark(&smark);
 	arglist.lastp = &arglist.list;
 	for (argp = n->nfor.args ; argp ; argp = argp->narg.next) {
+		oexitstatus = exitstatus;
 		expandarg(argp, &arglist, EXP_FULL | EXP_TILDE);
 		if (evalskip)
 			goto out;
@@ -366,7 +360,8 @@ out:
 STATIC void
 evalcase(n, flags)
 	union node *n;
-	{
+	int flags;
+{
 	union node *cp;
 	union node *patp;
 	struct arglist arglist;
@@ -374,6 +369,7 @@ evalcase(n, flags)
 
 	setstackmark(&smark);
 	arglist.lastp = &arglist.list;
+	oexitstatus = exitstatus;
 	expandarg(n->ncase.expr, &arglist, EXP_TILDE);
 	for (cp = n->ncase.cases ; cp && evalskip == 0 ; cp = cp->nclist.next) {
 		for (patp = cp->nclist.pattern ; patp ; patp = patp->narg.next) {
@@ -398,7 +394,8 @@ out:
 STATIC void
 evalsubshell(n, flags)
 	union node *n;
-	{
+	int flags;
+{
 	struct job *jp;
 	int backgnd = (n->type == NBACKGND);
 
@@ -426,17 +423,27 @@ evalsubshell(n, flags)
 STATIC void
 expredir(n)
 	union node *n;
-	{
-	register union node *redir;
+{
+	union node *redir;
 
 	for (redir = n ; redir ; redir = redir->nfile.next) {
-		if (redir->type == NFROM
-		 || redir->type == NTO
-		 || redir->type == NAPPEND) {
-			struct arglist fn;
-			fn.lastp = &fn.list;
+		struct arglist fn;
+		fn.lastp = &fn.list;
+		oexitstatus = exitstatus;
+		switch (redir->type) {
+		case NFROM:
+		case NTO:
+		case NAPPEND:
 			expandarg(redir->nfile.fname, &fn, EXP_TILDE | EXP_REDIR);
 			redir->nfile.expfname = fn.list->text;
+			break;
+		case NFROMFD:
+		case NTOFD:
+			if (redir->ndup.vname) {
+				expandarg(redir->ndup.vname, &fn, EXP_FULL | EXP_TILDE);
+				fixredir(redir, fn.list->text, 1);
+			}
+			break;
 		}
 	}
 }
@@ -453,14 +460,14 @@ expredir(n)
 STATIC void
 evalpipe(n)
 	union node *n;
-	{
+{
 	struct job *jp;
 	struct nodelist *lp;
 	int pipelen;
 	int prevfd;
 	int pip[2];
 
-	TRACE(("evalpipe(0x%x) called\n", (int)n));
+	TRACE(("evalpipe(0x%lx) called\n", (long)n));
 	pipelen = 0;
 	for (lp = n->npipe.cmdlist ; lp ; lp = lp->next)
 		pipelen++;
@@ -520,7 +527,7 @@ void
 evalbackcmd(n, result)
 	union node *n;
 	struct backcmd *result;
-	{
+{
 	int pip[2];
 	struct job *jp;
 	struct stackmark smark;		/* unnecessary */
@@ -530,12 +537,15 @@ evalbackcmd(n, result)
 	result->buf = NULL;
 	result->nleft = 0;
 	result->jp = NULL;
-	exitstatus = 0;
-	if (n == NULL)
+	if (n == NULL) {
+		exitstatus = 0;
 		goto out;
+	}
 	if (n->type == NCMD) {
+		exitstatus = oexitstatus;
 		evalcommand(n, EV_BACKCMD, result);
 	} else {
+		exitstatus = 0;
 		if (pipe(pip) < 0)
 			error("Pipe call failed");
 		jp = makejob(n, 1);
@@ -568,8 +578,9 @@ out:
 STATIC void
 evalcommand(cmd, flags, backcmd)
 	union node *cmd;
+	int flags;
 	struct backcmd *backcmd;
-	{
+{
 	struct stackmark smark;
 	union node *argp;
 	struct arglist arglist;
@@ -579,7 +590,6 @@ evalcommand(cmd, flags, backcmd)
 	char **envp;
 	int varflag;
 	struct strlist *sp;
-	register char *p;
 	int mode;
 	int pip[2];
 	struct cmdentry cmdentry;
@@ -591,15 +601,24 @@ evalcommand(cmd, flags, backcmd)
 	struct localvar *volatile savelocalvars;
 	volatile int e;
 	char *lastarg;
+#if __GNUC__
+	/* Avoid longjmp clobbering */
+	(void) &argv;
+	(void) &argc;
+	(void) &lastarg;
+	(void) &flags;
+#endif
 
 	/* First expand the arguments. */
-	TRACE(("evalcommand(0x%x, %d) called\n", (int)cmd, flags));
+	TRACE(("evalcommand(0x%lx, %d) called\n", (long)cmd, flags));
 	setstackmark(&smark);
 	arglist.lastp = &arglist.list;
 	varlist.lastp = &varlist.list;
 	varflag = 1;
+	oexitstatus = exitstatus;
+	exitstatus = 0;
 	for (argp = cmd->ncmd.args ; argp ; argp = argp->narg.next) {
-		p = argp->narg.text;
+		char *p = argp->narg.text;
 		if (varflag && is_name(*p)) {
 			do {
 				p++;
@@ -650,9 +669,20 @@ evalcommand(cmd, flags, backcmd)
 		cmdentry.cmdtype = CMDBUILTIN;
 		cmdentry.u.index = BLTINCMD;
 	} else {
-		find_command(argv[0], &cmdentry, 1);
+		static const char PATH[] = "PATH=";
+		char *path = pathval();
+
+		/*
+		 * Modify the command lookup path, if a PATH= assignment
+		 * is present
+		 */
+		for (sp = varlist.list ; sp ; sp = sp->next)
+			if (strncmp(sp->text, PATH, sizeof(PATH) - 1) == 0)
+				path = sp->text + sizeof(PATH) - 1;
+
+		find_command(argv[0], &cmdentry, 1, path);
 		if (cmdentry.cmdtype == CMDUNKNOWN) {	/* command not found */
-			exitstatus = 2;
+			exitstatus = 127;
 			flushout(&errout);
 			return;
 		}
@@ -664,7 +694,7 @@ evalcommand(cmd, flags, backcmd)
 					break;
 				if ((cmdentry.u.index = find_builtin(*argv)) < 0) {
 					outfmt(&errout, "%s: not found\n", *argv);
-					exitstatus = 2;
+					exitstatus = 127;
 					flushout(&errout);
 					return;
 				}
@@ -676,11 +706,11 @@ evalcommand(cmd, flags, backcmd)
 
 	/* Fork off a child process if necessary. */
 	if (cmd->ncmd.backgnd
-	 || cmdentry.cmdtype == CMDNORMAL && (flags & EV_EXIT) == 0
-	 || (flags & EV_BACKCMD) != 0
+	 || (cmdentry.cmdtype == CMDNORMAL && (flags & EV_EXIT) == 0)
+	 || ((flags & EV_BACKCMD) != 0
 	    && (cmdentry.cmdtype != CMDBUILTIN
 		 || cmdentry.u.index == DOTCMD
-		 || cmdentry.u.index == EVALCMD)) {
+		 || cmdentry.u.index == EVALCMD))) {
 		jp = makejob(cmd, 1);
 		mode = cmd->ncmd.backgnd;
 		if (flags & EV_BACKCMD) {
@@ -705,10 +735,13 @@ evalcommand(cmd, flags, backcmd)
 	/* This is the child process if a fork occurred. */
 	/* Execute the command. */
 	if (cmdentry.cmdtype == CMDFUNCTION) {
+#ifdef DEBUG
 		trputs("Shell function:  ");  trargs(argv);
+#endif
 		redirect(cmd->ncmd.redirect, REDIR_PUSH);
 		saveparam = shellparam;
 		shellparam.malloc = 0;
+		shellparam.reset = 1;
 		shellparam.nparam = argc - 1;
 		shellparam.p = argv + 1;
 		shellparam.optnext = NULL;
@@ -733,7 +766,10 @@ evalcommand(cmd, flags, backcmd)
 		for (sp = varlist.list ; sp ; sp = sp->next)
 			mklocal(sp->text);
 		funcnest++;
-		evaltree(cmdentry.u.func, 0);
+		if (flags & EV_TESTED)
+			evaltree(cmdentry.u.func, EV_TESTED);
+		else
+			evaltree(cmdentry.u.func, 0);
 		funcnest--;
 		INTOFF;
 		poplocalvars();
@@ -750,7 +786,9 @@ evalcommand(cmd, flags, backcmd)
 		if (flags & EV_EXIT)
 			exitshell(exitstatus);
 	} else if (cmdentry.cmdtype == CMDBUILTIN) {
+#ifdef DEBUG
 		trputs("builtin command:  ");  trargs(argv);
+#endif
 		mode = (cmdentry.u.index == EXECCMD)? 0 : REDIR_PUSH;
 		if (flags == EV_BACKCMD) {
 			memout.nleft = 0;
@@ -786,11 +824,14 @@ cmddone:
 		}
 		handler = savehandler;
 		if (e != -1) {
-			if (e != EXERROR || cmdentry.u.index == BLTINCMD
-					       || cmdentry.u.index == DOTCMD
-					       || cmdentry.u.index == EVALCMD
-					       || cmdentry.u.index == HISTCMD
-					       || cmdentry.u.index == EXECCMD)
+			if ((e != EXERROR && e != EXEXEC)
+			   || cmdentry.u.index == BLTINCMD
+			   || cmdentry.u.index == DOTCMD
+			   || cmdentry.u.index == EVALCMD
+#ifndef NO_HISTORY
+			   || cmdentry.u.index == HISTCMD
+#endif
+			   || cmdentry.u.index == EXECCMD)
 				exraise(e);
 			FORCEINTON;
 		}
@@ -802,19 +843,15 @@ cmddone:
 			memout.buf = NULL;
 		}
 	} else {
+#ifdef DEBUG
 		trputs("normal command:  ");  trargs(argv);
+#endif
 		clearredir();
 		redirect(cmd->ncmd.redirect, 0);
-		if (varlist.list) {
-			p = stalloc(strlen(pathval()) + 1);
-			scopy(pathval(), p);
-		} else {
-			p = pathval();
-		}
 		for (sp = varlist.list ; sp ; sp = sp->next)
 			setvareq(sp->text, VEXPORT|VSTACK);
 		envp = environment();
-		shellexec(argv, envp, p, cmdentry.u.index);
+		shellexec(argv, envp, pathval(), cmdentry.u.index);
 		/*NOTREACHED*/
 	}
 	goto out;
@@ -848,12 +885,13 @@ out:
 STATIC void
 prehash(n)
 	union node *n;
-	{
+{
 	struct cmdentry entry;
 
-	if (n->type == NCMD && n->ncmd.args &&
-	    goodname(n->ncmd.args->narg.text))
-		find_command(n->ncmd.args->narg.text, &entry, 0);
+	if (n->type == NCMD && n->ncmd.args)
+		if (goodname(n->ncmd.args->narg.text))
+			find_command(n->ncmd.args->narg.text, &entry, 0,
+				     pathval());
 }
 
 
@@ -868,8 +906,16 @@ prehash(n)
  * specified variables.
  */
 
-bltincmd(argc, argv)  char **argv; {
+int
+bltincmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	listsetvar(cmdenviron);
+	/*
+	 * Preserve exitstatus of a previous possible redirection
+	 * as POSIX mandates
+	 */
 	return exitstatus;
 }
 
@@ -885,12 +931,13 @@ bltincmd(argc, argv)  char **argv; {
  * in the standard shell so we don't make it one here.
  */
 
-breakcmd(argc, argv)  char **argv; {
-	int n;
+int
+breakcmd(argc, argv)
+	int argc;
+	char **argv;
+{
+	int n = argc > 1 ? number(argv[1]) : 1;
 
-	n = 1;
-	if (argc > 1)
-		n = number(argv[1]);
 	if (n > loopnest)
 		n = loopnest;
 	if (n > 0) {
@@ -905,30 +952,56 @@ breakcmd(argc, argv)  char **argv; {
  * The return command.
  */
 
-returncmd(argc, argv)  char **argv; {
-	int ret;
+int
+returncmd(argc, argv)
+	int argc;
+	char **argv;
+{
+	int ret = argc > 1 ? number(argv[1]) : oexitstatus;
 
-	ret = exitstatus;
-	if (argc > 1)
-		ret = number(argv[1]);
 	if (funcnest) {
 		evalskip = SKIPFUNC;
 		skipcount = 1;
+	} else {
+		/* skip the rest of the file */
+		evalskip = SKIPFILE;
+		skipcount = 1;
 	}
 	return ret;
 }
 
 
-truecmd(argc, argv)  char **argv; {
+int
+falsecmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
+	return 1;
+}
+
+
+int
+truecmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	return 0;
 }
 
 
-execcmd(argc, argv)  char **argv; {
+int
+execcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	if (argc > 1) {
+		struct strlist *sp;
+
 		iflag = 0;		/* exit on error */
 		mflag = 0;
 		optschanged();
+		for (sp = cmdenviron; sp ; sp = sp->next)
+			setvareq(sp->text, VEXPORT|VSTACK);
 		shellexec(argv + 1, environment(), pathval(), 0);
 
 	}
