@@ -32,14 +32,27 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/expand.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)expand.c	8.2 (Berkeley) 10/22/93";
+#if 0
+static char sccsid[] = "@(#)expand.c	8.5 (Berkeley) 5/15/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/expand.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <sys/types.h>
+#include <sys/time.h>
+#include <sys/stat.h>
+#include <errno.h>
+#include <dirent.h>
+#include <unistd.h>
+#include <pwd.h>
+#include <stdlib.h>
+#include <limits.h>
+#include <stdio.h>
+
 /*
  * Routines to expand arguments to commands.  We have to deal with
  * backquotes, shell variables, and file metacharacters.
@@ -60,12 +73,8 @@ static char sccsid[] = "@(#)expand.c	8.2 (Berkeley) 10/22/93";
 #include "memalloc.h"
 #include "error.h"
 #include "mystring.h"
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <dirent.h>
-#include <pwd.h>
+#include "arith.h"
+#include "show.h"
 
 /*
  * Structure specifying which parts of the string should be searched
@@ -86,39 +95,41 @@ struct ifsregion ifsfirst;	/* first struct in list of ifs regions */
 struct ifsregion *ifslastp;	/* last struct in list */
 struct arglist exparg;		/* holds expanded arg list */
 
-#ifdef __STDC__
-STATIC void argstr(char *, int);
-STATIC void expbackq(union node *, int, int);
-STATIC char *evalvar(char *, int);
-STATIC int varisset(int);
-STATIC void varvalue(int, int, int);
-STATIC void recordregion(int, int, int);
-STATIC void ifsbreakup(char *, struct arglist *);
-STATIC void expandmeta(struct strlist *, int);
-STATIC void expmeta(char *, char *);
-STATIC void expari(int);
-STATIC void addfname(char *);
-STATIC struct strlist *expsort(struct strlist *);
-STATIC struct strlist *msort(struct strlist *, int);
-STATIC int pmatch(char *, char *);
-STATIC char *exptilde(char *, int);
-#else
-STATIC void argstr();
-STATIC void expbackq();
-STATIC char *evalvar();
-STATIC int varisset();
-STATIC void varvalue();
-STATIC void recordregion();
-STATIC void ifsbreakup();
-STATIC void expandmeta();
-STATIC void expmeta();
-STATIC void expari();
-STATIC void addfname();
-STATIC struct strlist *expsort();
-STATIC struct strlist *msort();
-STATIC int pmatch();
-STATIC char *exptilde();
-#endif
+STATIC void argstr __P((char *, int));
+STATIC char *exptilde __P((char *, int));
+STATIC void expbackq __P((union node *, int, int));
+STATIC int subevalvar __P((char *, char *, int, int, int, int));
+STATIC char *evalvar __P((char *, int));
+STATIC int varisset __P((char *, int));
+STATIC void varvalue __P((char *, int, int));
+STATIC void recordregion __P((int, int, int));
+STATIC void removerecordregions __P((int)); 
+STATIC void ifsbreakup __P((char *, struct arglist *));
+STATIC void expandmeta __P((struct strlist *, int));
+STATIC void expmeta __P((char *, char *));
+STATIC void addfname __P((char *));
+STATIC struct strlist *expsort __P((struct strlist *));
+STATIC struct strlist *msort __P((struct strlist *, int));
+STATIC int pmatch __P((char *, char *));
+STATIC char *cvtnum __P((int, char *));
+STATIC int collate_range_cmp __P((int, int));
+
+STATIC int collate_range_cmp (c1, c2)
+	int c1, c2;
+{
+	static char s1[2], s2[2];
+	int ret;
+
+	c1 &= UCHAR_MAX;
+	c2 &= UCHAR_MAX;
+	if (c1 == c2)
+		return (0);
+	s1[0] = c1;
+	s2[0] = c2;
+	if ((ret = strcoll(s1, s2)) != 0)
+		return (ret);
+	return (c1 - c2);
+}
 
 /*
  * Expand shell variables and backquotes inside a here document.
@@ -146,7 +157,8 @@ void
 expandarg(arg, arglist, flag)
 	union node *arg;
 	struct arglist *arglist;
-	{
+	int flag;
+{
 	struct strlist *sp;
 	char *p;
 
@@ -202,9 +214,10 @@ expandarg(arg, arglist, flag)
 
 STATIC void
 argstr(p, flag)
-	register char *p;
-	{
-	register char c;
+	char *p;
+	int flag;
+{
+	char c;
 	int quotes = flag & (EXP_FULL | EXP_CASE);	/* do CTLESC */
 	int firsteq = 1;
 
@@ -215,6 +228,13 @@ argstr(p, flag)
 		case '\0':
 		case CTLENDVAR: /* ??? */
 			goto breakloop;
+		case CTLQUOTEMARK:
+			/* "$@" syntax adherence hack */
+			if (p[0] == CTLVAR && p[2] == '@' && p[3] == '=')
+				break;
+			if ((flag & EXP_FULL) != 0)
+				STPUTC(c, expdest);
+			break;
 		case CTLESC:
 			if (quotes)
 				STPUTC(c, expdest);
@@ -259,16 +279,19 @@ breakloop:;
 STATIC char *
 exptilde(p, flag)
 	char *p;
-	{
+	int flag;
+{
 	char c, *startp = p;
 	struct passwd *pw;
 	char *home;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
 
-	while (c = *p) {
+	while ((c = *p) != '\0') {
 		switch(c) {
 		case CTLESC:
 			return (startp);
+		case CTLQUOTEMARK:
+			return (startp);
 		case ':':
 			if (flag & EXP_VARTILDE)
 				goto done;
@@ -291,7 +314,7 @@ done:
 	if (*home == '\0')
 		goto lose;
 	*p = c;
-	while (c = *home++) {
+	while ((c = *home++) != '\0') {
 		if (quotes && SQSYNTAX[c] == CCTL)
 			STPUTC(CTLESC, expdest);
 		STPUTC(c, expdest);
@@ -303,16 +326,60 @@ lose:
 }
 
 
+STATIC void 
+removerecordregions(endoff)
+	int endoff;
+{
+	if (ifslastp == NULL)
+		return;
+
+	if (ifsfirst.endoff > endoff) {
+		while (ifsfirst.next != NULL) {
+			struct ifsregion *ifsp;
+			INTOFF;
+			ifsp = ifsfirst.next->next;
+			ckfree(ifsfirst.next);
+			ifsfirst.next = ifsp;
+			INTON;
+		}
+		if (ifsfirst.begoff > endoff)
+			ifslastp = NULL;
+		else {
+			ifslastp = &ifsfirst;
+			ifsfirst.endoff = endoff;
+		}
+		return;
+	}
+	
+	ifslastp = &ifsfirst;
+	while (ifslastp->next && ifslastp->next->begoff < endoff)
+		ifslastp=ifslastp->next;
+	while (ifslastp->next != NULL) {
+		struct ifsregion *ifsp;
+		INTOFF;
+		ifsp = ifslastp->next->next;
+		ckfree(ifslastp->next);
+		ifslastp->next = ifsp;
+		INTON;
+	}
+	if (ifslastp->endoff > endoff)
+		ifslastp->endoff = endoff;
+}
+
 /*
  * Expand arithmetic expression.  Backup to start of expression,
  * evaluate, place result in (backed up) result, adjust string position.
  */
 void
 expari(flag)
-	{
+	int flag;
+{
 	char *p, *start;
 	int result;
+	int begoff;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
+	int quoted;
+
 
 	/*
 	 * This routine is slightly over-compilcated for
@@ -324,7 +391,10 @@ expari(flag)
 	 * have to rescan starting from the beginning since CTLESC
 	 * characters have to be processed left to right.
 	 */
-	CHECKSTRSPACE(8, expdest);
+#if INT_MAX / 1000000000 >= 10 || INT_MIN / 1000000000 <= -10
+#error "integers with more than 10 digits are not supported"
+#endif
+	CHECKSTRSPACE(12 - 2, expdest);
 	USTPUTC('\0', expdest);
 	start = stackblock();
 	p = expdest;
@@ -336,12 +406,21 @@ expari(flag)
 		for (p = start; *p != CTLARI; p++)
 			if (*p == CTLESC)
 				p++;
+
+	if (p[1] == '"')
+		quoted=1;
+	else
+		quoted=0;
+	begoff = p - start;
+	removerecordregions(begoff);
 	if (quotes)
-		rmescapes(p+1);
-	result = arith(p+1);
-	fmtstr(p, 10, "%d", result);
+		rmescapes(p+2);
+	result = arith(p+2);
+	fmtstr(p, 12, "%d", result);
 	while (*p++)
 		;
+	if (quoted == 0)
+		recordregion(begoff, p - 1 - start, 0);
 	result = expdest - p + 1;
 	STADJUST(-result, expdest);
 }
@@ -354,7 +433,9 @@ expari(flag)
 STATIC void
 expbackq(cmd, quoted, flag)
 	union node *cmd;
-	{
+	int quoted;
+	int flag;
+{
 	struct backcmd in;
 	int i;
 	char buf[128];
@@ -402,11 +483,11 @@ expbackq(cmd, quoted, flag)
 			STPUTC(lastc, dest);
 		}
 	}
-	p--;
-	while (lastc == '\n') {
+
+	/* Eat all trailing newlines */
+	for (p--; lastc == '\n'; lastc = *--p)
 		STUNPUTC(dest);
-		lastc = *--p;
-	}
+
 	if (in.fd >= 0)
 		close(in.fd);
 	if (in.buf)
@@ -425,6 +506,109 @@ expbackq(cmd, quoted, flag)
 
 
 
+STATIC int
+subevalvar(p, str, strloc, subtype, startloc, varflags)
+	char *p;
+	char *str;
+	int strloc;
+	int subtype;
+	int startloc;
+	int varflags;
+{
+	char *startp;
+	char *loc = NULL;
+	int c = 0;
+	int saveherefd = herefd;
+	struct nodelist *saveargbackq = argbackq;
+	int amount;
+
+	herefd = -1;
+	argstr(p, 0);
+	STACKSTRNUL(expdest);
+	herefd = saveherefd;
+	argbackq = saveargbackq;
+	startp = stackblock() + startloc;
+	if (str == NULL)
+	    str = stackblock() + strloc;
+
+	switch (subtype) {
+	case VSASSIGN:
+		setvar(str, startp, 0);
+		amount = startp - expdest;
+		STADJUST(amount, expdest);
+		varflags &= ~VSNUL;
+		if (c != 0)
+			*loc = c;
+		return 1;
+
+	case VSQUESTION:
+		if (*p != CTLENDVAR) {
+			outfmt(&errout, "%s\n", startp);
+			error((char *)NULL);
+		}
+		error("%.*s: parameter %snot set", p - str - 1,
+		      str, (varflags & VSNUL) ? "null or "
+					      : nullstr);
+		return 0;
+
+	case VSTRIMLEFT:
+		for (loc = startp; loc < str; loc++) {
+			c = *loc;
+			*loc = '\0';
+			if (patmatch(str, startp)) {
+				*loc = c;
+				goto recordleft;
+			}
+			*loc = c;
+		}
+		return 0;
+
+	case VSTRIMLEFTMAX:
+		for (loc = str - 1; loc >= startp; loc--) {
+			c = *loc;
+			*loc = '\0';
+			if (patmatch(str, startp)) {
+				*loc = c;
+				goto recordleft;
+			}
+			*loc = c;
+		}
+		return 0;
+
+	case VSTRIMRIGHT:
+		for (loc = str - 1; loc >= startp; loc--) {
+			if (patmatch(str, loc)) {
+				amount = loc - expdest;
+				STADJUST(amount, expdest);
+				return 1;
+			}
+		}
+		return 0;
+
+	case VSTRIMRIGHTMAX:
+		for (loc = startp; loc < str - 1; loc++) {
+			if (patmatch(str, loc)) {
+				amount = loc - expdest;
+				STADJUST(amount, expdest);
+				return 1;
+			}
+		}
+		return 0;
+
+
+	default:
+		abort();
+	}
+
+recordleft:
+	amount = ((str - 1) - (loc - startp)) - expdest;
+	STADJUST(amount, expdest);
+	while (loc != str - 1)
+		*startp++ = *loc++;
+	return 1;
+}
+
+
 /*
  * Expand a variable, and return a pointer to the next character in the
  * input string.
@@ -433,15 +617,19 @@ expbackq(cmd, quoted, flag)
 STATIC char *
 evalvar(p, flag)
 	char *p;
-	{
+	int flag;
+{
 	int subtype;
 	int varflags;
 	char *var;
 	char *val;
+	char *pat;
 	int c;
 	int set;
 	int special;
 	int startloc;
+	int varlen;
+	int easy;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
 
 	varflags = *p++;
@@ -453,64 +641,118 @@ evalvar(p, flag)
 	p = strchr(p, '=') + 1;
 again: /* jump here after setting a variable with ${var=text} */
 	if (special) {
-		set = varisset(*var);
+		set = varisset(var, varflags & VSNUL);
 		val = NULL;
 	} else {
 		val = lookupvar(var);
-		if (val == NULL || (varflags & VSNUL) && val[0] == '\0') {
+		if (val == NULL || ((varflags & VSNUL) && val[0] == '\0')) {
 			val = NULL;
 			set = 0;
 		} else
 			set = 1;
 	}
+	varlen = 0;
 	startloc = expdest - stackblock();
 	if (set && subtype != VSPLUS) {
 		/* insert the value of the variable */
 		if (special) {
-			varvalue(*var, varflags & VSQUOTE, flag & EXP_FULL);
+			varvalue(var, varflags & VSQUOTE, flag & EXP_FULL);
+			if (subtype == VSLENGTH) {
+				varlen = expdest - stackblock() - startloc;
+				STADJUST(-varlen, expdest);
+			}
 		} else {
-			char const *syntax = (varflags & VSQUOTE)? DQSYNTAX : BASESYNTAX;
+			char const *syntax = (varflags & VSQUOTE) ? DQSYNTAX
+								  : BASESYNTAX;
+
+			if (subtype == VSLENGTH) {
+				for (;*val; val++)
+					varlen++;
+			}
+			else {
+				while (*val) {
+					if (quotes && syntax[*val] == CCTL)
+						STPUTC(CTLESC, expdest);
+					STPUTC(*val++, expdest);
+				}
 
-			while (*val) {
-				if (quotes && syntax[*val] == CCTL)
-					STPUTC(CTLESC, expdest);
-				STPUTC(*val++, expdest);
 			}
 		}
 	}
+
 	if (subtype == VSPLUS)
 		set = ! set;
-	if (((varflags & VSQUOTE) == 0 || (*var == '@' && shellparam.nparam != 1))
-	 && (set || subtype == VSNORMAL))
-		recordregion(startloc, expdest - stackblock(), varflags & VSQUOTE);
-	if (! set && subtype != VSNORMAL) {
-		if (subtype == VSPLUS || subtype == VSMINUS) {
+
+	easy = ((varflags & VSQUOTE) == 0 ||
+		(*var == '@' && shellparam.nparam != 1));
+
+
+	switch (subtype) {
+	case VSLENGTH:
+		expdest = cvtnum(varlen, expdest);
+		goto record;
+
+	case VSNORMAL:
+		if (!easy)
+			break;
+record:
+		recordregion(startloc, expdest - stackblock(),
+			     varflags & VSQUOTE);
+		break;
+
+	case VSPLUS:
+	case VSMINUS:
+		if (!set) {
 			argstr(p, flag);
-		} else {
-			char *startp;
-			int saveherefd = herefd;
-			struct nodelist *saveargbackq = argbackq;
-			herefd = -1;
-			argstr(p, 0);
-			STACKSTRNUL(expdest);
-			herefd = saveherefd;
-			argbackq = saveargbackq;
-			startp = stackblock() + startloc;
-			if (subtype == VSASSIGN) {
-				setvar(var, startp, 0);
-				STADJUST(startp - expdest, expdest);
-				varflags &=~ VSNUL;
+			break;
+		}
+		if (easy)
+			goto record;
+		break;
+
+	case VSTRIMLEFT:
+	case VSTRIMLEFTMAX:
+	case VSTRIMRIGHT:
+	case VSTRIMRIGHTMAX:
+		if (!set)
+			break;
+		/*
+		 * Terminate the string and start recording the pattern
+		 * right after it
+		 */
+		STPUTC('\0', expdest);
+		pat = expdest;
+		if (subevalvar(p, NULL, expdest - stackblock(), subtype,
+			       startloc, varflags) == 0) {
+			int amount = (expdest - pat) + 1;
+			STADJUST(-amount, expdest);
+		}
+		/* Remove any recorded regions beyond start of variable */
+		removerecordregions(startloc);
+		goto record;
+
+	case VSASSIGN:
+	case VSQUESTION:
+		if (!set) {
+			if (subevalvar(p, var, 0, subtype, startloc, varflags)) {
+				varflags &= ~VSNUL;
+				/* 
+				 * Remove any recorded regions beyond 
+				 * start of variable 
+				 */
+				removerecordregions(startloc);
 				goto again;
 			}
-			/* subtype == VSQUESTION */
-			if (*p != CTLENDVAR) {
-				outfmt(&errout, "%s\n", startp);
-				error((char *)NULL);
-			}
-			error("%.*s: parameter %snot set", p - var - 1,
-				var, (varflags & VSNUL)? "null or " : nullstr);
+			break;
 		}
+		if (easy)
+			goto record;
+		break;
+
+	default:
+		abort();
 	}
+
 	if (subtype != VSNORMAL) {	/* skip to end of alternative */
 		int nesting = 1;
 		for (;;) {
@@ -538,23 +780,39 @@ again: /* jump here after setting a variable with ${var=text} */
  */
 
 STATIC int
-varisset(name)
-	char name;
-	{
-	char **ap;
+varisset(name, nulok)
+	char *name;
+	int nulok;
+{
 
-	if (name == '!') {
-		if (backgndpid == -1)
-			return 0;
-	} else if (name == '@' || name == '*') {
+	if (*name == '!')
+		return backgndpid != -1;
+	else if (*name == '@' || *name == '*') {
 		if (*shellparam.p == NULL)
 			return 0;
-	} else if ((unsigned)(name -= '1') <= '9' - '1') {
-		ap = shellparam.p;
-		do {
-			if (*ap++ == NULL)
-				return 0;
-		} while (--name >= 0);
+
+		if (nulok) {
+			char **av;
+
+			for (av = shellparam.p; *av; av++)
+				if (**av != '\0')
+					return 1;
+			return 0;
+		}
+	} else if (is_digit(*name)) {
+		char *ap;
+		int num = atoi(name);
+
+		if (num > shellparam.nparam)
+			return 0;
+
+		if (num == 0)
+			ap = arg0;
+		else
+			ap = shellparam.p[num - 1];
+
+		if (nulok && (ap == NULL || *ap == '\0'))
+			return 0;
 	}
 	return 1;
 }
@@ -567,13 +825,14 @@ varisset(name)
 
 STATIC void
 varvalue(name, quoted, allow_split)
-	char name;
-	{
+	char *name;
+	int quoted;
+	int allow_split;
+{
 	int num;
-	char temp[32];
 	char *p;
 	int i;
-	extern int exitstatus;
+	extern int oexitstatus;
 	char sep;
 	char **ap;
 	char const *syntax;
@@ -593,12 +852,12 @@ varvalue(name, quoted, allow_split)
 	} while (0)
 
 
-	switch (name) {
+	switch (*name) {
 	case '$':
 		num = rootpid;
 		goto numvar;
 	case '?':
-		num = exitstatus;
+		num = oexitstatus;
 		goto numvar;
 	case '#':
 		num = shellparam.nparam;
@@ -606,13 +865,7 @@ varvalue(name, quoted, allow_split)
 	case '!':
 		num = backgndpid;
 numvar:
-		p = temp + 31;
-		temp[31] = '\0';
-		do {
-			*--p = num % 10 + '0';
-		} while ((num /= 10) != 0);
-		while (*p)
-			STPUTC(*p++, expdest);
+		expdest = cvtnum(num, expdest);
 		break;
 	case '-':
 		for (i = 0 ; i < NOPTS ; i++) {
@@ -621,17 +874,23 @@ numvar:
 		}
 		break;
 	case '@':
-		if (allow_split) {
-			sep = '\0';
-			goto allargs;
+		if (allow_split && quoted) {
+			for (ap = shellparam.p ; (p = *ap++) != NULL ; ) {
+				STRTODEST(p);
+				if (*ap)
+					STPUTC('\0', expdest);
+			}
+			break;
 		}
 		/* fall through */
 	case '*':
-		sep = ' ';
-allargs:
+		if (ifsset() != 0)
+			sep = ifsval()[0];
+		else
+			sep = ' ';
 		for (ap = shellparam.p ; (p = *ap++) != NULL ; ) {
 			STRTODEST(p);
-			if (*ap)
+			if (*ap && sep)
 				STPUTC(sep, expdest);
 		}
 		break;
@@ -640,9 +899,12 @@ allargs:
 		STRTODEST(p);
 		break;
 	default:
-		if ((unsigned)(name -= '1') <= '9' - '1') {
-			p = shellparam.p[name];
-			STRTODEST(p);
+		if (is_digit(*name)) {
+			num = atoi(name);
+			if (num > 0 && num <= shellparam.nparam) {
+				p = shellparam.p[num - 1];
+				STRTODEST(p);
+			}
 		}
 		break;
 	}
@@ -656,8 +918,12 @@ allargs:
  */
 
 STATIC void
-recordregion(start, end, nulonly) {
-	register struct ifsregion *ifsp;
+recordregion(start, end, nulonly)
+	int start;
+	int end;
+	int nulonly;
+{
+	struct ifsregion *ifsp;
 
 	if (ifslastp == NULL) {
 		ifsp = &ifsfirst;
@@ -679,7 +945,6 @@ recordregion(start, end, nulonly) {
  * strings to the argument list.  The regions of the string to be
  * searched for IFS characters have been stored by recordregion.
  */
-
 STATIC void
 ifsbreakup(string, arglist)
 	char *string;
@@ -688,46 +953,73 @@ ifsbreakup(string, arglist)
 	struct ifsregion *ifsp;
 	struct strlist *sp;
 	char *start;
-	register char *p;
+	char *p;
 	char *q;
 	char *ifs;
+	int ifsspc;
+	int nulonly;
+
 
 	start = string;
+	ifsspc = 0;
+	nulonly = 0;
 	if (ifslastp != NULL) {
 		ifsp = &ifsfirst;
 		do {
 			p = string + ifsp->begoff;
-			ifs = ifsp->nulonly? nullstr : ifsval();
+			nulonly = ifsp->nulonly;
+			ifs = nulonly ? nullstr : 
+				( ifsset() ? ifsval() : " \t\n" );
+			ifsspc = 0;
 			while (p < string + ifsp->endoff) {
 				q = p;
 				if (*p == CTLESC)
 					p++;
-				if (strchr(ifs, *p++)) {
-					if (q > start || *ifs != ' ') {
-						*q = '\0';
-						sp = (struct strlist *)stalloc(sizeof *sp);
-						sp->text = start;
-						*arglist->lastp = sp;
-						arglist->lastp = &sp->next;
+				if (strchr(ifs, *p)) {
+					if (!nulonly)
+						ifsspc = (strchr(" \t\n", *p) != NULL);
+					/* Ignore IFS whitespace at start */
+					if (q == start && ifsspc) {
+						p++;
+						start = p;
+						continue;
 					}
-					if (*ifs == ' ') {
+					*q = '\0';
+					sp = (struct strlist *)stalloc(sizeof *sp);
+					sp->text = start;
+					*arglist->lastp = sp;
+					arglist->lastp = &sp->next;
+					p++;
+					if (!nulonly) {
 						for (;;) {
-							if (p >= string + ifsp->endoff)
+							if (p >= string + ifsp->endoff) {
 								break;
+							}
 							q = p;
 							if (*p == CTLESC)
 								p++;
-							if (strchr(ifs, *p++) == NULL) {
+							if (strchr(ifs, *p) == NULL ) {
 								p = q;
 								break;
-							}
+							} else if (strchr(" \t\n",*p) == NULL) {
+								if (ifsspc) {
+									p++;
+									ifsspc = 0;
+								} else {
+									p = q;
+									break;
+								}
+							} else
+								p++;
 						}
 					}
 					start = p;
-				}
+				} else
+					p++;
 			}
 		} while ((ifsp = ifsp->next) != NULL);
-		if (*start || (*ifs != ' ' && start > string)) {
+		if (*start || (!ifsspc && start > string && 
+			(nulonly || 1))) {
 			sp = (struct strlist *)stalloc(sizeof *sp);
 			sp->text = start;
 			*arglist->lastp = sp;
@@ -754,7 +1046,8 @@ char *expdir;
 STATIC void
 expandmeta(str, flag)
 	struct strlist *str;
-	{
+	int flag __unused;
+{
 	char *p;
 	struct strlist **savelastp;
 	struct strlist *sp;
@@ -811,7 +1104,7 @@ expmeta(enddir, name)
 	char *enddir;
 	char *name;
 	{
-	register char *p;
+	char *p;
 	char *q;
 	char *start;
 	char *endname;
@@ -829,9 +1122,11 @@ expmeta(enddir, name)
 			metaflag = 1;
 		else if (*p == '[') {
 			q = p + 1;
-			if (*q == '!')
+			if (*q == '!' || *q == '^')
 				q++;
 			for (;;) {
+				while (*q == CTLQUOTEMARK)
+					q++;
 				if (*q == CTLESC)
 					q++;
 				if (*q == '/' || *q == '\0')
@@ -845,6 +1140,8 @@ expmeta(enddir, name)
 			metaflag = 1;
 		} else if (*p == '\0')
 			break;
+		else if (*p == CTLQUOTEMARK)
+			continue;
 		else if (*p == CTLESC)
 			p++;
 		if (*p == '/') {
@@ -857,6 +1154,8 @@ expmeta(enddir, name)
 		if (enddir != expdir)
 			metaflag++;
 		for (p = name ; ; p++) {
+			if (*p == CTLQUOTEMARK)
+				continue;
 			if (*p == CTLESC)
 				p++;
 			*enddir++ = *p;
@@ -871,6 +1170,8 @@ expmeta(enddir, name)
 	if (start != name) {
 		p = name;
 		while (p < start) {
+			while (*p == CTLQUOTEMARK)
+				p++;
 			if (*p == CTLESC)
 				p++;
 			*enddir++ = *p++;
@@ -895,7 +1196,12 @@ expmeta(enddir, name)
 		*endname++ = '\0';
 	}
 	matchdot = 0;
-	if (start[0] == '.' || start[0] == CTLESC && start[1] == '.')
+	p = start;
+	while (*p == CTLQUOTEMARK)
+		p++;
+	if (*p == CTLESC)
+		p++;
+	if (*p == '.')
 		matchdot++;
 	while (! int_pending() && (dp = readdir(dirp)) != NULL) {
 		if (dp->d_name[0] == '.' && ! matchdot)
@@ -906,7 +1212,9 @@ expmeta(enddir, name)
 				addfname(expdir);
 			} else {
 				char *q;
-				for (p = enddir, q = dp->d_name ; *p++ = *q++ ;);
+				for (p = enddir, q = dp->d_name;
+				     (*p++ = *q++) != '\0';)
+					continue;
 				p[-1] = '/';
 				expmeta(p, endname);
 			}
@@ -961,8 +1269,9 @@ expsort(str)
 STATIC struct strlist *
 msort(list, len)
 	struct strlist *list;
-	{
-	struct strlist *p, *q;
+	int len;
+{
+	struct strlist *p, *q = NULL;
 	struct strlist **lpp;
 	int half;
 	int n;
@@ -1024,8 +1333,8 @@ pmatch(pattern, string)
 	char *pattern;
 	char *string;
 	{
-	register char *p, *q;
-	register char c;
+	char *p, *q;
+	char c;
 
 	p = pattern;
 	q = string;
@@ -1037,13 +1346,18 @@ pmatch(pattern, string)
 			if (*q++ != *p++)
 				return 0;
 			break;
+		case CTLQUOTEMARK:
+			continue;
 		case '?':
 			if (*q++ == '\0')
 				return 0;
 			break;
 		case '*':
 			c = *p;
-			if (c != CTLESC && c != '?' && c != '*' && c != '[') {
+			while (c == CTLQUOTEMARK || c == '*')
+				c = *++p;
+			if (c != CTLESC &&  c != CTLQUOTEMARK &&
+			    c != '?' && c != '*' && c != '[') {
 				while (*q != c) {
 					if (*q == '\0')
 						return 0;
@@ -1061,9 +1375,11 @@ pmatch(pattern, string)
 			char chr;
 
 			endp = p;
-			if (*endp == '!')
+			if (*endp == '!' || *endp == '^')
 				endp++;
 			for (;;) {
+				while (*endp == CTLQUOTEMARK)
+					endp++;
 				if (*endp == '\0')
 					goto dft;		/* no matching ] */
 				if (*endp == CTLESC)
@@ -1072,21 +1388,29 @@ pmatch(pattern, string)
 					break;
 			}
 			invert = 0;
-			if (*p == '!') {
+			if (*p == '!' || *p == '^') {
 				invert++;
 				p++;
 			}
 			found = 0;
 			chr = *q++;
+			if (chr == '\0')
+				return 0;
 			c = *p++;
 			do {
+				if (c == CTLQUOTEMARK)
+					continue;
 				if (c == CTLESC)
 					c = *p++;
 				if (*p == '-' && p[1] != ']') {
 					p++;
+					while (*p == CTLQUOTEMARK)
+						p++;
 					if (*p == CTLESC)
 						p++;
-					if (chr >= c && chr <= *p)
+					if (   collate_range_cmp(chr, c) >= 0
+					    && collate_range_cmp(chr, *p) <= 0
+					   )
 						found = 1;
 					p++;
 				} else {
@@ -1119,16 +1443,20 @@ breakloop:
 void
 rmescapes(str)
 	char *str;
-	{
-	register char *p, *q;
+{
+	char *p, *q;
 
 	p = str;
-	while (*p != CTLESC) {
+	while (*p != CTLESC && *p != CTLQUOTEMARK) {
 		if (*p++ == '\0')
 			return;
 	}
 	q = p;
 	while (*p) {
+		if (*p == CTLQUOTEMARK) {
+			p++;
+			continue;
+		}
 		if (*p == CTLESC)
 			p++;
 		*q++ = *p++;
@@ -1162,3 +1490,30 @@ casematch(pattern, val)
 	popstackmark(&smark);
 	return result;
 }
+
+/*
+ * Our own itoa().
+ */
+
+STATIC char *
+cvtnum(num, buf)
+	int num;
+	char *buf;
+	{
+	char temp[32];
+	int neg = num < 0;
+	char *p = temp + 31;
+
+	temp[31] = '\0';
+
+	do {
+		*--p = num % 10 + '0';
+	} while ((num /= 10) != 0);
+
+	if (neg)
+		*--p = '-';
+
+	while (*p)
+		STPUTC(*p++, buf);
+	return buf;
+}
