@@ -32,68 +32,77 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/jobs.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)jobs.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)jobs.c	8.5 (Berkeley) 5/4/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/jobs.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <fcntl.h>
+#include <signal.h>
+#include <errno.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <sys/param.h>
+#ifdef BSD
+#include <sys/wait.h>
+#include <sys/time.h>
+#include <sys/resource.h>
+#endif
+#include <sys/ioctl.h>
+
 #include "shell.h"
 #if JOBS
+#if OLD_TTY_DRIVER
 #include "sgtty.h"
+#else
+#include <termios.h>
+#endif
 #undef CEOF			/* syntax.h redefines this */
 #endif
+#include "redir.h"
+#include "show.h"
 #include "main.h"
 #include "parser.h"
 #include "nodes.h"
 #include "jobs.h"
 #include "options.h"
 #include "trap.h"
-#include "signames.h"
 #include "syntax.h"
 #include "input.h"
 #include "output.h"
 #include "memalloc.h"
 #include "error.h"
 #include "mystring.h"
-#include <fcntl.h>
-#include <signal.h>
-#include <errno.h>
-#ifdef BSD
-#include <sys/types.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <sys/resource.h>
-#endif
-
 
 
 struct job *jobtab;		/* array of jobs */
 int njobs;			/* size of array */
-MKINIT short backgndpid = -1;	/* pid of last background process */
+MKINIT pid_t backgndpid = -1;	/* pid of last background process */
 #if JOBS
 int initialpgrp;		/* pgrp of shell on invocation */
-short curjob;			/* current job */
+int curjob;			/* current job */
 #endif
+int in_waitcmd = 0;		/* are we in waitcmd()? */
+int in_dowait = 0;		/* are we in dowait()? */
+volatile sig_atomic_t breakwaitcmd = 0;	/* should wait be terminated? */
 
-#ifdef __STDC__
-STATIC void restartjob(struct job *);
-STATIC struct job *getjob(char *);
-STATIC void freejob(struct job *);
-STATIC int procrunning(int);
-STATIC int dowait(int, struct job *);
-STATIC int waitproc(int, int *);
-#else
-STATIC void restartjob();
-STATIC struct job *getjob();
-STATIC void freejob();
-STATIC int procrunning();
-STATIC int dowait();
-STATIC int waitproc();
+#if JOBS
+STATIC void restartjob __P((struct job *));
 #endif
-
+STATIC void freejob __P((struct job *));
+STATIC struct job *getjob __P((char *));
+STATIC int dowait __P((int, struct job *));
+#if SYSV
+STATIC int onsigchild __P((void));
+#endif
+STATIC int waitproc __P((int, int *));
+STATIC void cmdtxt __P((union node *));
+STATIC void cmdputs __P((char *));
 
 
 /*
@@ -106,8 +115,11 @@ STATIC int waitproc();
 
 MKINIT int jobctl;
 
+#if JOBS
 void
-setjobctl(on) {
+setjobctl(on)
+	int on;
+{
 #ifdef OLD_TTY_DRIVER
 	int ldisc;
 #endif
@@ -116,14 +128,19 @@ setjobctl(on) {
 		return;
 	if (on) {
 		do { /* while we are in the background */
+#ifdef OLD_TTY_DRIVER
 			if (ioctl(2, TIOCGPGRP, (char *)&initialpgrp) < 0) {
+#else
+			initialpgrp = tcgetpgrp(2);
+			if (initialpgrp < 0) {
+#endif
 				out2str("sh: can't access tty; job control turned off\n");
 				mflag = 0;
 				return;
 			}
 			if (initialpgrp == -1)
-				initialpgrp = getpgrp(0);
-			else if (initialpgrp != getpgrp(0)) {
+				initialpgrp = getpgrp();
+			else if (initialpgrp != getpgrp()) {
 				killpg(initialpgrp, SIGTTIN);
 				continue;
 			}
@@ -138,20 +155,31 @@ setjobctl(on) {
 		setsignal(SIGTSTP);
 		setsignal(SIGTTOU);
 		setsignal(SIGTTIN);
-		setpgrp(0, rootpid);
+		setpgid(0, rootpid);
+#ifdef OLD_TTY_DRIVER
 		ioctl(2, TIOCSPGRP, (char *)&rootpid);
+#else
+		tcsetpgrp(2, rootpid);
+#endif
 	} else { /* turning job control off */
-		setpgrp(0, initialpgrp);
+		setpgid(0, initialpgrp);
+#ifdef OLD_TTY_DRIVER
 		ioctl(2, TIOCSPGRP, (char *)&initialpgrp);
+#else
+		tcsetpgrp(2, initialpgrp);
+#endif
 		setsignal(SIGTSTP);
 		setsignal(SIGTTOU);
 		setsignal(SIGTTIN);
 	}
 	jobctl = on;
 }
+#endif
 
 
 #ifdef mkinit
+INCLUDE <sys/types.h>
+INCLUDE <stdlib.h>
 
 SHELLPROC {
 	backgndpid = -1;
@@ -165,7 +193,11 @@ SHELLPROC {
 
 
 #if JOBS
-fgcmd(argc, argv)  char **argv; {
+int
+fgcmd(argc, argv)
+	int argc __unused;
+	char **argv;
+{
 	struct job *jp;
 	int pgrp;
 	int status;
@@ -174,7 +206,11 @@ fgcmd(argc, argv)  char **argv; {
 	if (jp->jobctl == 0)
 		error("job not created under job control");
 	pgrp = jp->ps[0].pid;
+#ifdef OLD_TTY_DRIVER
 	ioctl(2, TIOCSPGRP, (char *)&pgrp);
+#else
+	tcsetpgrp(2, pgrp);
+#endif
 	restartjob(jp);
 	INTOFF;
 	status = waitforjob(jp);
@@ -183,7 +219,11 @@ fgcmd(argc, argv)  char **argv; {
 }
 
 
-bgcmd(argc, argv)  char **argv; {
+int
+bgcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	struct job *jp;
 
 	do {
@@ -199,7 +239,7 @@ bgcmd(argc, argv)  char **argv; {
 STATIC void
 restartjob(jp)
 	struct job *jp;
-	{
+{
 	struct procstat *ps;
 	int i;
 
@@ -208,7 +248,7 @@ restartjob(jp)
 	INTOFF;
 	killpg(jp->ps[0].pid, SIGCONT);
 	for (ps = jp->ps, i = jp->nprocs ; --i >= 0 ; ps++) {
-		if ((ps->status & 0377) == 0177) {
+		if (WIFSTOPPED(ps->status)) {
 			ps->status = -1;
 			jp->state = 0;
 		}
@@ -219,7 +259,10 @@ restartjob(jp)
 
 
 int
-jobscmd(argc, argv)  char **argv; {
+jobscmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	showjobs(0);
 	return 0;
 }
@@ -235,7 +278,9 @@ jobscmd(argc, argv)  char **argv; {
  */
 
 void
-showjobs(change) {
+showjobs(change)
+	int change;
+{
 	int jobno;
 	int procno;
 	int i;
@@ -266,19 +311,20 @@ showjobs(change) {
 			s[0] = '\0';
 			if (ps->status == -1) {
 				/* don't print anything */
-			} else if ((ps->status & 0xFF) == 0) {
-				fmtstr(s, 64, "Exit %d", ps->status >> 8);
+			} else if (WIFEXITED(ps->status)) {
+				fmtstr(s, 64, "Exit %d", WEXITSTATUS(ps->status));
 			} else {
-				i = ps->status;
 #if JOBS
-				if ((i & 0xFF) == 0177)
-					i >>= 8;
+				if (WIFSTOPPED(ps->status)) 
+					i = WSTOPSIG(ps->status);
+				else
 #endif
-				if ((i & 0x7F) <= MAXSIG && sigmesg[i & 0x7F])
-					scopy(sigmesg[i & 0x7F], s);
+					i = WTERMSIG(ps->status);
+				if ((i & 0x7F) < NSIG && sys_siglist[i & 0x7F])
+					scopy(sys_siglist[i & 0x7F], s);
 				else
 					fmtstr(s, 64, "Signal %d", i & 0x7F);
-				if (i & 0x80)
+				if (WCOREDUMP(ps->status))
 					strcat(s, " (core dumped)");
 			}
 			out1str(s);
@@ -329,9 +375,12 @@ freejob(jp)
 
 
 int
-waitcmd(argc, argv)  char **argv; {
+waitcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	struct job *job;
-	int status;
+	int status, retval;
 	struct job *jp;
 
 	if (argc > 1) {
@@ -339,38 +388,53 @@ waitcmd(argc, argv)  char **argv; {
 	} else {
 		job = NULL;
 	}
-	for (;;) {	/* loop until process terminated or stopped */
+
+	/*
+	 * Loop until a process is terminated or stopped, or a SIGINT is
+	 * received.
+	 */
+
+	in_waitcmd++;
+	do {
 		if (job != NULL) {
 			if (job->state) {
 				status = job->ps[job->nprocs - 1].status;
-				if ((status & 0xFF) == 0)
-					status = status >> 8 & 0xFF;
+				if (WIFEXITED(status))
+					retval = WEXITSTATUS(status);
 #if JOBS
-				else if ((status & 0xFF) == 0177)
-					status = (status >> 8 & 0x7F) + 128;
+				else if (WIFSTOPPED(status))
+					retval = WSTOPSIG(status) + 128;
 #endif
 				else
-					status = (status & 0x7F) + 128;
+					retval = WTERMSIG(status) + 128;
 				if (! iflag)
 					freejob(job);
-				return status;
+				in_waitcmd--;
+				return retval;
 			}
 		} else {
 			for (jp = jobtab ; ; jp++) {
 				if (jp >= jobtab + njobs) {	/* no running procs */
+					in_waitcmd--;
 					return 0;
 				}
 				if (jp->used && jp->state == 0)
 					break;
 			}
 		}
-		dowait(1, (struct job *)NULL);
-	}
+	} while (dowait(1, (struct job *)NULL) != -1);
+	in_waitcmd--;
+
+	return 0;
 }
 
 
 
-jobidcmd(argc, argv)  char **argv; {
+int
+jobidcmd(argc, argv)
+	int argc __unused;
+	char **argv;
+{
 	struct job *jp;
 	int i;
 
@@ -393,7 +457,7 @@ getjob(name)
 	char *name;
 	{
 	int jobno;
-	register struct job *jp;
+	struct job *jp;
 	int pid;
 	int i;
 
@@ -417,7 +481,7 @@ currentjob:
 			goto currentjob;
 #endif
 		} else {
-			register struct job *found = NULL;
+			struct job *found = NULL;
 			for (jp = jobtab, i = njobs ; --i >= 0 ; jp++) {
 				if (jp->used && jp->nprocs > 0
 				 && prefix(name + 1, jp->ps[0].cmd)) {
@@ -438,6 +502,8 @@ currentjob:
 		}
 	}
 	error("No such job: %s", name);
+	/*NOTREACHED*/
+	return NULL;
 }
 
 
@@ -448,8 +514,9 @@ currentjob:
 
 struct job *
 makejob(node, nprocs)
-	union node *node;
-	{
+	union node *node __unused;
+	int nprocs;
+{
 	int i;
 	struct job *jp;
 
@@ -459,15 +526,12 @@ makejob(node, nprocs)
 			if (njobs == 0) {
 				jobtab = ckmalloc(4 * sizeof jobtab[0]);
 			} else {
-				struct job *ojp;
-
 				jp = ckmalloc((njobs + 4) * sizeof jobtab[0]);
-				for (i = njobs, ojp = jobtab; --i >= 0;
-				     jp++, ojp++)
-					if (ojp->ps == &ojp->ps0)
-						ojp->ps = &jp->ps0;
-				jp -= njobs;
-				bcopy(jobtab, jp, njobs * sizeof jp[0]);
+				memcpy(jp, jobtab, njobs * sizeof jp[0]);
+				/* Relocate `ps' pointers */
+				for (i = 0; i < njobs; i++)
+					if (jp[i].ps == &jobtab[i].ps0)
+						jp[i].ps = &jp[i].ps0;
 				ckfree(jobtab);
 				jobtab = jp;
 			}
@@ -493,7 +557,8 @@ makejob(node, nprocs)
 		jp->ps = &jp->ps0;
 	}
 	INTON;
-	TRACE(("makejob(0x%x, %d) returns %%%d\n", (int)node, nprocs, jp - jobtab + 1));
+	TRACE(("makejob(0x%lx, %d) returns %%%d\n", (long)node, nprocs,
+	    jp - jobtab + 1));
 	return jp;
 }
 
@@ -517,11 +582,13 @@ int
 forkshell(jp, n, mode)
 	union node *n;
 	struct job *jp;
-	{
+	int mode;
+{
 	int pid;
 	int pgrp;
 
-	TRACE(("forkshell(%%%d, 0x%x, %d) called\n", jp - jobtab, (int)n, mode));
+	TRACE(("forkshell(%%%d, 0x%lx, %d) called\n", jp - jobtab, (long)n,
+	    mode));
 	INTOFF;
 	pid = fork();
 	if (pid == -1) {
@@ -550,11 +617,15 @@ forkshell(jp, n, mode)
 				pgrp = getpid();
 			else
 				pgrp = jp->ps[0].pid;
-			setpgrp(0, pgrp);
-			if (mode == FORK_FG) {
+			if (setpgid(0, pgrp) == 0 && mode == FORK_FG) {
 				/*** this causes superfluous TIOCSPGRPS ***/
+#ifdef OLD_TTY_DRIVER
 				if (ioctl(2, TIOCSPGRP, (char *)&pgrp) < 0)
-					error("TIOCSPGRP failed, errno=%d\n", errno);
+					error("TIOCSPGRP failed, errno=%d", errno);
+#else
+				if (tcsetpgrp(2, pgrp) < 0)
+					error("tcsetpgrp failed, errno=%d", errno);
+#endif
 			}
 			setsignal(SIGTSTP);
 			setsignal(SIGTTOU);
@@ -592,7 +663,7 @@ forkshell(jp, n, mode)
 			pgrp = pid;
 		else
 			pgrp = jp->ps[0].pid;
-		setpgrp(pid, pgrp);
+		setpgid(pid, pgrp);
 	}
 	if (mode == FORK_BG)
 		backgndpid = pid;		/* set $! */
@@ -632,42 +703,50 @@ forkshell(jp, n, mode)
 
 int
 waitforjob(jp)
-	register struct job *jp;
+	struct job *jp;
 	{
 #if JOBS
-	int mypgrp = getpgrp(0);
+	int mypgrp = getpgrp();
 #endif
 	int status;
 	int st;
 
 	INTOFF;
 	TRACE(("waitforjob(%%%d) called\n", jp - jobtab + 1));
-	while (jp->state == 0) {
-		dowait(1, jp);
-	}
+	while (jp->state == 0)
+		if (dowait(1, jp) == -1)
+			dotrap();
 #if JOBS
 	if (jp->jobctl) {
+#ifdef OLD_TTY_DRIVER
 		if (ioctl(2, TIOCSPGRP, (char *)&mypgrp) < 0)
 			error("TIOCSPGRP failed, errno=%d\n", errno);
+#else
+		if (tcsetpgrp(2, mypgrp) < 0)
+			error("tcsetpgrp failed, errno=%d\n", errno);
+#endif
 	}
 	if (jp->state == JOBSTOPPED)
 		curjob = jp - jobtab + 1;
 #endif
 	status = jp->ps[jp->nprocs - 1].status;
 	/* convert to 8 bits */
-	if ((status & 0xFF) == 0)
-		st = status >> 8 & 0xFF;
+	if (WIFEXITED(status))
+		st = WEXITSTATUS(status);
 #if JOBS
-	else if ((status & 0xFF) == 0177)
-		st = (status >> 8 & 0x7F) + 128;
+	else if (WIFSTOPPED(status))
+		st = WSTOPSIG(status) + 128;
 #endif
 	else
-		st = (status & 0x7F) + 128;
+		st = WTERMSIG(status) + 128;
 	if (! JOBS || jp->state == JOBDONE)
 		freejob(jp);
-	CLEAR_PENDING_INT;
-	if ((status & 0x7F) == SIGINT)
-		kill(getpid(), SIGINT);
+	if (int_pending()) {
+		if (WIFSIGNALED(status) && WTERMSIG(status) == SIGINT)
+			kill(getpid(), SIGINT);
+		else
+			CLEAR_PENDING_INT;
+	}
 	INTON;
 	return st;
 }
@@ -680,8 +759,9 @@ waitforjob(jp)
 
 STATIC int
 dowait(block, job)
+	int block;
 	struct job *job;
-	{
+{
 	int pid;
 	int status;
 	struct procstat *sp;
@@ -690,12 +770,19 @@ dowait(block, job)
 	int done;
 	int stopped;
 	int core;
+	int sig;
 
+	in_dowait++;
 	TRACE(("dowait(%d) called\n", block));
 	do {
 		pid = waitproc(block, &status);
 		TRACE(("wait returns %d, status=%d\n", pid, status));
-	} while (pid == -1 && errno == EINTR);
+	} while (pid == -1 && errno == EINTR && breakwaitcmd == 0);
+	in_dowait--;
+	if (breakwaitcmd != 0) {
+		breakwaitcmd = 0;
+		return -1;
+	}
 	if (pid <= 0)
 		return pid;
 	INTOFF;
@@ -708,13 +795,14 @@ dowait(block, job)
 				if (sp->pid == -1)
 					continue;
 				if (sp->pid == pid) {
-					TRACE(("Changin status of proc %d from 0x%x to 0x%x\n", pid, sp->status, status));
+					TRACE(("Changing status of proc %d from 0x%x to 0x%x\n",
+						   pid, sp->status, status));
 					sp->status = status;
 					thisjob = jp;
 				}
 				if (sp->status == -1)
 					stopped = 0;
-				else if ((sp->status & 0377) == 0177)
+				else if (WIFSTOPPED(sp->status))
 					done = 0;
 			}
 			if (stopped) {		/* stopped or done */
@@ -732,29 +820,35 @@ dowait(block, job)
 	}
 	INTON;
 	if (! rootshell || ! iflag || (job && thisjob == job)) {
+		core = WCOREDUMP(status);
 #if JOBS
-		if ((status & 0xFF) == 0177)
-			status >>= 8;
+		if (WIFSTOPPED(status))
+			sig = WSTOPSIG(status);
+		else
 #endif
-		core = status & 0x80;
-		status &= 0x7F;
-		if (status != 0 && status != SIGINT && status != SIGPIPE) {
+			if (WIFEXITED(status))
+				sig = 0;
+			else
+				sig = WTERMSIG(status);
+
+		if (sig != 0 && sig != SIGINT && sig != SIGPIPE) {
 			if (thisjob != job)
 				outfmt(out2, "%d: ", pid);
 #if JOBS
-			if (status == SIGTSTP && rootshell && iflag)
+			if (sig == SIGTSTP && rootshell && iflag)
 				outfmt(out2, "%%%d ", job - jobtab + 1);
 #endif
-			if (status <= MAXSIG && sigmesg[status])
-				out2str(sigmesg[status]);
+			if (sig < NSIG && sys_siglist[sig])
+				out2str(sys_siglist[sig]);
 			else
-				outfmt(out2, "Signal %d", status);
+				outfmt(out2, "Signal %d", sig);
 			if (core)
 				out2str(" - core dumped");
 			out2c('\n');
 			flushout(&errout);
 		} else {
-			TRACE(("Not printing status: status=%d\n", status));
+			TRACE(("Not printing status: status=%d, sig=%d\n", 
+				   status, sig));
 		}
 	} else {
 		TRACE(("Not printing status, rootshell=%d, job=0x%x\n", rootshell, job));
@@ -796,7 +890,7 @@ dowait(block, job)
  */
 
 #ifdef SYSV
-STATIC int gotsigchild;
+STATIC sig_atomic_t gotsigchild;
 
 STATIC int onsigchild() {
 	gotsigchild = 1;
@@ -806,8 +900,9 @@ STATIC int onsigchild() {
 
 STATIC int
 waitproc(block, status)
+	int block;
 	int *status;
-	{
+{
 #ifdef BSD
 	int flags;
 
@@ -846,8 +941,8 @@ int job_warning = 0;
 int
 stoppedjobs()
 {
-	register int jobno;
-	register struct job *jp;
+	int jobno;
+	struct job *jp;
 
 	if (job_warning)
 		return (0);
@@ -1019,8 +1114,8 @@ STATIC void
 cmdputs(s)
 	char *s;
 	{
-	register char *p, *q;
-	register char c;
+	char *p, *q;
+	char c;
 	int subtype = 0;
 
 	if (cmdnleft <= 0)
@@ -1040,7 +1135,7 @@ cmdputs(s)
 			subtype = 0;
 		} else if (c == CTLENDVAR) {
 			*q++ = '}';
-		} else if (c == CTLBACKQ | c == CTLBACKQ+CTLQUOTE)
+		} else if (c == CTLBACKQ || c == CTLBACKQ+CTLQUOTE)
 			cmdnleft++;		/* ignore it */
 		else
 			*q++ = c;
