@@ -33,8 +33,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)machdep.h	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/sh/machdep.h 50952 1999-09-05 11:54:50Z peter $
+ *	@(#)machdep.h	8.2 (Berkeley) 5/4/95
+ * $FreeBSD: stable/2.2/bin/sh/machdep.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 /*
@@ -48,5 +48,5 @@ union align {
 	char *cp;
 };
 
-#define ALIGN(nbytes)	((nbytes) + sizeof(union align) - 1 &~ (sizeof(union align) - 1))
+#define ALIGN(nbytes)	(((nbytes) + sizeof(union align) - 1) & ~(sizeof(union align) - 1))
 #endif
