@@ -32,12 +32,14 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/memalloc.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)memalloc.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)memalloc.c	8.3 (Berkeley) 5/4/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/memalloc.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include "shell.h"
@@ -46,15 +48,19 @@ static char sccsid[] = "@(#)memalloc.c	8.1 (Berkeley) 5/31/93";
 #include "error.h"
 #include "machdep.h"
 #include "mystring.h"
+#include "expand.h"
+#include <stdlib.h>
+#include <unistd.h>
 
 /*
  * Like malloc, but returns an error when out of space.
  */
 
 pointer
-ckmalloc(nbytes) {
-	register pointer p;
-	pointer malloc();
+ckmalloc(nbytes)
+	int nbytes;
+{
+	pointer p;
 
 	if ((p = malloc(nbytes)) == NULL)
 		error("Out of space");
@@ -68,9 +74,9 @@ ckmalloc(nbytes) {
 
 pointer
 ckrealloc(p, nbytes)
-	register pointer p;
-	{
-	pointer realloc();
+	pointer p;
+	int nbytes;
+{
 
 	if ((p = realloc(p, nbytes)) == NULL)
 		error("Out of space");
@@ -86,7 +92,7 @@ char *
 savestr(s)
 	char *s;
 	{
-	register char *p;
+	char *p;
 
 	p = ckmalloc(strlen(s) + 1);
 	scopy(s, p);
@@ -121,8 +127,10 @@ int herefd = -1;
 
 
 pointer
-stalloc(nbytes) {
-	register char *p;
+stalloc(nbytes)
+	int nbytes;
+{
+	char *p;
 
 	nbytes = ALIGN(nbytes);
 	if (nbytes > stacknleft) {
@@ -202,7 +210,7 @@ popstackmark(mark)
 void
 growstackblock() {
 	char *p;
-	int newlen = stacknleft * 2 + 100;
+	int newlen = ALIGN(stacknleft * 2 + 100);
 	char *oldspace = stacknxt;
 	int oldlen = stacknleft;
 	struct stack_block *sp;
@@ -219,7 +227,7 @@ growstackblock() {
 		INTON;
 	} else {
 		p = stalloc(newlen);
-		bcopy(oldspace, p, oldlen);
+		memcpy(p, oldspace, oldlen);
 		stacknxt = p;			/* free the space */
 		stacknleft += newlen;		/* we just allocated */
 	}
@@ -228,7 +236,9 @@ growstackblock() {
 
 
 void
-grabstackblock(len) {
+grabstackblock(len)
+	int len;
+{
 	len = ALIGN(len);
 	stacknxt += len;
 	stacknleft -= len;
