@@ -30,8 +30,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)myhistedit.h	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/sh/myhistedit.h 50952 1999-09-05 11:54:50Z peter $
+ *	@(#)myhistedit.h	8.2 (Berkeley) 5/4/95
+ * $FreeBSD: stable/2.2/bin/sh/myhistedit.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <histedit.h>
@@ -39,3 +39,10 @@
 extern History *hist;
 extern EditLine *el;
 extern int displayhist;
+
+void histedit __P((void));
+void sethistsize __P((const char *));
+int histcmd __P((int, char **));
+int not_fcnumber __P((char *));
+int str_to_event __P((char *, int));
+
