@@ -33,23 +33,16 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)mystring.h	8.1 (Berkeley) 5/31/93
- * $FreeBSD: stable/2.1/bin/sh/mystring.h 50952 1999-09-05 11:54:50Z peter $
+ *	@(#)mystring.h	8.2 (Berkeley) 5/4/95
+ * $FreeBSD: stable/2.2/bin/sh/mystring.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <string.h>
 
-#ifdef __STDC__
-void scopyn(const char *, char *, int);
-int prefix(const char *, const char *);
-int number(const char *);
-int is_number(const char *);
-#else
-void scopyn();
-int prefix();
-int number();
-int is_number();
-#endif
+void scopyn __P((const char *, char *, int));
+int prefix __P((const char *, const char *));
+int number __P((const char *));
+int is_number __P((const char *));
 
 #define equal(s1, s2)	(strcmp(s1, s2) == 0)
 #define scopy(s1, s2)	((void)strcpy(s2, s1))
