@@ -32,14 +32,18 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/parser.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)parser.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)parser.c	8.7 (Berkeley) 5/16/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/parser.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <stdlib.h>
+
 #include "shell.h"
 #include "parser.h"
 #include "nodes.h"
@@ -54,8 +58,10 @@ static char sccsid[] = "@(#)parser.c	8.1 (Berkeley) 5/31/93";
 #include "memalloc.h"
 #include "mystring.h"
 #include "alias.h"
+#include "show.h"
+#ifndef NO_HISTORY
 #include "myhistedit.h"
-
+#endif
 
 /*
  * Shell command parser.
@@ -64,7 +70,7 @@ static char sccsid[] = "@(#)parser.c	8.1 (Berkeley) 5/31/93";
 #define EOFMARKLEN 79
 
 /* values returned by readtoken */
-#include "token.def"
+#include "token.h"
 
 
 
@@ -91,6 +97,8 @@ struct heredoc *heredoc;
 int quoteflag;			/* set if (part of) last token was quoted */
 int startlinno;			/* line # where last token started */
 
+/* XXX When 'noaliases' is set to one, no alias expansion takes place. */
+static int noaliases = 0;
 
 #define GDB_HACK 1 /* avoid local declarations which gdb can't handle */
 #ifdef GDB_HACK
@@ -104,23 +112,28 @@ STATIC union node *andor __P((void));
 STATIC union node *pipeline __P((void));
 STATIC union node *command __P((void));
 STATIC union node *simplecmd __P((union node **, union node *));
+STATIC union node *makename __P((void));
 STATIC void parsefname __P((void));
 STATIC void parseheredoc __P((void));
+STATIC int peektoken __P((void));
 STATIC int readtoken __P((void));
+STATIC int xxreadtoken __P((void));
 STATIC int readtoken1 __P((int, char const *, char *, int));
-STATIC void attyline __P((void));
 STATIC int noexpand __P((char *));
 STATIC void synexpect __P((int));
 STATIC void synerror __P((char *));
 STATIC void setprompt __P((int));
 
+
 /*
  * Read and parse a command.  Returns NEOF on end of file.  (NULL is a
  * valid parse tree indicating a blank line.)
  */
 
 union node *
-parsecmd(interact) {
+parsecmd(interact)
+	int interact;
+{
 	int t;
 
 	doprompt = interact;
@@ -140,31 +153,58 @@ parsecmd(interact) {
 
 
 STATIC union node *
-list(nlflag) {
+list(nlflag)
+	int nlflag;
+{
 	union node *n1, *n2, *n3;
+	int tok;
 
 	checkkwd = 2;
 	if (nlflag == 0 && tokendlist[peektoken()])
 		return NULL;
-	n1 = andor();
+	n1 = NULL;
 	for (;;) {
-		switch (readtoken()) {
-		case TBACKGND:
-		case TNL:
-			parseheredoc();
-			if (nlflag)
-				return n1;
-			/* fall through */
-		case TSEMI:
-			checkkwd = 2;
-			if (tokendlist[peektoken()])
-				return n1;
-			n2 = andor();
+		n2 = andor();
+		tok = readtoken();
+		if (tok == TBACKGND) {
+			if (n2->type == NCMD || n2->type == NPIPE) {
+				n2->ncmd.backgnd = 1;
+			} else if (n2->type == NREDIR) {
+				n2->type = NBACKGND;
+			} else {
+				n3 = (union node *)stalloc(sizeof (struct nredir));
+				n3->type = NBACKGND;
+				n3->nredir.n = n2;
+				n3->nredir.redirect = NULL;
+				n2 = n3;
+			}
+		}
+		if (n1 == NULL) {
+			n1 = n2;
+		}
+		else {
 			n3 = (union node *)stalloc(sizeof (struct nbinary));
 			n3->type = NSEMI;
 			n3->nbinary.ch1 = n1;
 			n3->nbinary.ch2 = n2;
 			n1 = n3;
+		}
+		switch (tok) {
+		case TBACKGND:
+		case TSEMI:
+			tok = readtoken();
+			/* fall through */
+		case TNL:
+			if (tok == TNL) {
+				parseheredoc();
+				if (nlflag)
+					return n1;
+			} else {
+				tokpushback++;
+			}
+			checkkwd = 2;
+			if (tokendlist[peektoken()])
+				return n1;
 			break;
 		case TEOF:
 			if (heredoclist)
@@ -195,19 +235,6 @@ andor() {
 		} else if (t == TOR) {
 			t = NOR;
 		} else {
-			if (t == TBACKGND) {
-				if (n1->type == NCMD || n1->type == NPIPE) {
-					n1->ncmd.backgnd = 1;
-				} else if (n1->type == NREDIR) {
-					n1->type = NBACKGND;
-				} else {
-					n3 = (union node *)stalloc(sizeof (struct nredir));
-					n3->type = NBACKGND;
-					n3->nredir.n = n1;
-					n3->nredir.redirect = NULL;
-					n1 = n3;
-				}
-			}
 			tokpushback++;
 			return n1;
 		}
@@ -253,7 +280,7 @@ pipeline() {
 	}
 	tokpushback++;
 	if (negate) {
-		notnode = (union node *)stalloc(sizeof (struct nnot));
+		notnode = (union node *)stalloc(sizeof(struct nnot));
 		notnode->type = NNOT;
 		notnode->nnot.com = n1;
 		n1 = notnode;
@@ -272,8 +299,10 @@ command() {
 	int t;
 
 	checkkwd = 2;
-	redir = 0;
+	redir = NULL;
+	n1 = NULL;
 	rpp = &redir;
+
 	/* Check for redirection which may precede command */
 	while (readtoken() == TREDIR) {
 		*rpp = n2 = redirnode;
@@ -390,6 +419,7 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 		if (lasttoken != TWORD || ! equal(wordtext, "in"))
 			synerror("expecting \"in\"");
 		cpp = &n1->ncase.cases;
+		noaliases = 1;	/* turn off alias expansion */
 		checkkwd = 2, readtoken();
 		do {
 			*cpp = cp = (union node *)stalloc(sizeof (struct nclist));
@@ -407,18 +437,19 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 			}
 			ap->narg.next = NULL;
 			if (lasttoken != TRP)
-				synexpect(TRP);
+				noaliases = 0, synexpect(TRP);
 			cp->nclist.body = list(0);
 
 			checkkwd = 2;
 			if ((t = readtoken()) != TESAC) {
 				if (t != TENDCASE)
-					synexpect(TENDCASE);
+					noaliases = 0, synexpect(TENDCASE);
 				else
 					checkkwd = 2, readtoken();
 			}
 			cpp = &cp->nclist.next;
 		} while(lasttoken != TESAC);
+		noaliases = 0;	/* reset alias expansion */
 		*cpp = NULL;
 		checkkwd = 1;
 		break;
@@ -438,13 +469,19 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 		checkkwd = 1;
 		break;
 	/* Handle an empty command like other simple commands.  */
+	case TSEMI:
+		/*
+		 * An empty command before a ; doesn't make much sense, and
+		 * should certainly be disallowed in the case of `if ;'.
+		 */
+		if (!redir)
+			synexpect(-1);
 	case TAND:
 	case TOR:
 	case TNL:
-	case TSEMI:
-	/* Handle EOF as an empty command, too */
 	case TEOF:
 	case TWORD:
+	case TRP:
 		tokpushback++;
 		return simplecmd(rpp, redir);
 	default:
@@ -478,7 +515,7 @@ simplecmd(rpp, redir)
 	{
 	union node *args, **app;
 	union node **orig_rpp = rpp;
-	union node *n;
+	union node *n = NULL;
 
 	/* If we don't have any redirections already, then we must reset */
 	/* rpp to be the address of the local redir variable.  */
@@ -533,6 +570,40 @@ simplecmd(rpp, redir)
 	return n;
 }
 
+STATIC union node *
+makename() {
+	union node *n;
+
+	n = (union node *)stalloc(sizeof (struct narg));
+	n->type = NARG;
+	n->narg.next = NULL;
+	n->narg.text = wordtext;
+	n->narg.backquote = backquotelist;
+	return n;
+}
+
+void fixredir(n, text, err)
+	union node *n;
+	const char *text;
+	int err;
+	{
+	TRACE(("Fix redir %s %d\n", text, err));
+	if (!err)
+		n->ndup.vname = NULL;
+
+	if (is_digit(text[0]) && text[1] == '\0')
+		n->ndup.dupfd = digit_val(text[0]);
+	else if (text[0] == '-' && text[1] == '\0')
+		n->ndup.dupfd = -1;
+	else {
+
+		if (err)
+			synerror("Bad fd number");
+		else
+			n->ndup.vname = makename();
+	}
+}
+
 
 STATIC void
 parsefname() {
@@ -564,23 +635,9 @@ parsefname() {
 			p->next = here;
 		}
 	} else if (n->type == NTOFD || n->type == NFROMFD) {
-		if (is_digit(wordtext[0]))
-			n->ndup.dupfd = digit_val(wordtext[0]);
-		else if (wordtext[0] == '-')
-			n->ndup.dupfd = -1;
-		else
-			goto bad;
-		if (wordtext[1] != '\0') {
-bad:
-			synerror("Bad fd number");
-		}
+		fixredir(n, wordtext, 0);
 	} else {
-		n->nfile.fname = (union node *)stalloc(sizeof (struct narg));
-		n = n->nfile.fname;
-		n->type = NARG;
-		n->narg.next = NULL;
-		n->narg.text = wordtext;
-		n->narg.backquote = backquotelist;
+		n->nfile.fname = makename();
 	}
 }
 
@@ -650,17 +707,20 @@ readtoken() {
 		/*
 		 * check for keywords and aliases
 		 */
-		if (t == TWORD && !quoteflag) {
-			register char * const *pp, *s;
+		if (t == TWORD && !quoteflag)
+		{
+			char * const *pp;
 
 			for (pp = (char **)parsekwd; *pp; pp++) {
-				if (**pp == *wordtext && equal(*pp, wordtext)) {
+				if (**pp == *wordtext && equal(*pp, wordtext))
+				{
 					lasttoken = t = pp - parsekwd + KWDOFFSET;
 					TRACE(("keyword %s recognized\n", tokname[t]));
 					goto out;
 				}
 			}
-			if (ap = lookupalias(wordtext, 1)) {
+			if (noaliases == 0 &&
+			    (ap = lookupalias(wordtext, 1)) != NULL) {
 				pushstring(ap->val, strlen(ap->val), ap);
 				checkkwd = savecheckkwd;
 				goto top;
@@ -701,7 +761,7 @@ out:
 
 STATIC int
 xxreadtoken() {
-	register c;
+	int c;
 
 	if (tokpushback) {
 		tokpushback = 0;
@@ -796,8 +856,8 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 	char *eofmark;
 	int striptabs;
 	{
-	register c = firstc;
-	register char *out;
+	int c = firstc;
+	char *out;
 	int len;
 	char line[EOFMARKLEN + 1];
 	struct nodelist *bqlist;
@@ -808,6 +868,18 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 	int parenlevel;	/* levels of parens in arithmetic */
 	int oldstyle;
 	char const *prevsyntax;	/* syntax before arithmetic */
+#if __GNUC__
+	/* Avoid longjmp clobbering */
+	(void) &out;
+	(void) &quotef;
+	(void) &dblquote;
+	(void) &varnest;
+	(void) &arinest;
+	(void) &parenlevel;
+	(void) &oldstyle;
+	(void) &prevsyntax;
+	(void) &syntax;
+#endif
 
 	startlinno = plinno;
 	dblquote = 0;
@@ -834,13 +906,6 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 		CHECKEND();	/* set c to PEOF if at end of here document */
 		for (;;) {	/* until end of line or end of word */
 			CHECKSTRSPACE(3, out);	/* permit 3 calls to USTPUTC */
-			if (parsebackquote && c == '\\') {
-				c = pgetc();	/* XXX - compat with old /bin/sh */
-				if (/*c != '\\' && */c != '`' && c != '$') {
-					pungetc();
-					c = '\\';
-				}
-			}
 			switch(syntax[c]) {
 			case CNL:	/* '\n' */
 				if (syntax == BASESYNTAX)
@@ -877,27 +942,36 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 						USTPUTC('\\', out);
 					if (SQSYNTAX[c] == CCTL)
 						USTPUTC(CTLESC, out);
+					else if (eofmark == NULL)
+						USTPUTC(CTLQUOTEMARK, out);
 					USTPUTC(c, out);
 					quotef++;
 				}
 				break;
 			case CSQUOTE:
+				if (eofmark == NULL)
+					USTPUTC(CTLQUOTEMARK, out);
 				syntax = SQSYNTAX;
 				break;
 			case CDQUOTE:
+				if (eofmark == NULL)
+					USTPUTC(CTLQUOTEMARK, out);
 				syntax = DQSYNTAX;
 				dblquote = 1;
 				break;
 			case CENDQUOTE:
-				if (eofmark) {
+				if (eofmark != NULL && arinest == 0 &&
+				    varnest == 0) {
 					USTPUTC(c, out);
 				} else {
-					if (arinest)
+					if (arinest) {
 						syntax = ARISYNTAX;
-					else
+						dblquote = 0;
+					} else if (eofmark == NULL) {
 						syntax = BASESYNTAX;
+						dblquote = 0;
+					}
 					quotef++;
-					dblquote = 0;
 				}
 				break;
 			case CVAR:	/* '$' */
@@ -924,6 +998,10 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 						if (--arinest == 0) {
 							USTPUTC(CTLENDARI, out);
 							syntax = prevsyntax;
+							if (syntax == DQSYNTAX)
+								dblquote = 1;
+							else
+								dblquote = 0;
 						} else
 							USTPUTC(')', out);
 					} else {
@@ -995,7 +1073,7 @@ checkend: {
 		}
 		if (c == *eofmark) {
 			if (pfgets(line, sizeof line) != NULL) {
-				register char *p, *q;
+				char *p, *q;
 
 				p = line;
 				for (q = eofmark + 1 ; *q && *p == *q ; p++, q++);
@@ -1079,6 +1157,7 @@ parsesub: {
 #ifndef GDB_HACK
 	static const char types[] = "}-+?=";
 #endif
+       int bracketed_name = 0; /* used to handle ${[0-9]*} variables */
 
 	c = pgetc();
 	if (c != '(' && c != '{' && !is_name(c) && !is_special(c)) {
@@ -1097,14 +1176,32 @@ parsesub: {
 		USTPUTC(VSNORMAL, out);
 		subtype = VSNORMAL;
 		if (c == '{') {
+			bracketed_name = 1;
 			c = pgetc();
-			subtype = 0;
+			if (c == '#') {
+				if ((c = pgetc()) == '}')
+					c = '#';
+				else
+					subtype = VSLENGTH;
+			}
+			else
+				subtype = 0;
 		}
 		if (is_name(c)) {
 			do {
 				STPUTC(c, out);
 				c = pgetc();
 			} while (is_in_name(c));
+		} else if (is_digit(c)) {
+			if (bracketed_name) {
+				do {
+					STPUTC(c, out);
+					c = pgetc();
+				} while (is_digit(c));
+			} else {
+				STPUTC(c, out);
+				c = pgetc();
+			}
 		} else {
 			if (! is_special(c))
 badsub:				synerror("Bad substitution");
@@ -1114,14 +1211,31 @@ badsub:				synerror("Bad substitution");
 		STPUTC('=', out);
 		flags = 0;
 		if (subtype == 0) {
-			if (c == ':') {
+			switch (c) {
+			case ':':
 				flags = VSNUL;
 				c = pgetc();
+				/*FALLTHROUGH*/
+			default:
+				p = strchr(types, c);
+				if (p == NULL)
+					goto badsub;
+				subtype = p - types + VSNORMAL;
+				break;
+			case '%':
+			case '#':
+				{
+					int cc = c;
+					subtype = c == '#' ? VSTRIMLEFT :
+							     VSTRIMRIGHT;
+					c = pgetc();
+					if (c == cc)
+						subtype++;
+					else
+						pungetc();
+					break;
+				}
 			}
-			p = strchr(types, c);
-			if (p == NULL)
-				goto badsub;
-			subtype = p - types + VSNORMAL;
 		} else {
 			pungetc();
 		}
@@ -1150,6 +1264,11 @@ parsebackq: {
 	struct jmploc jmploc;
 	struct jmploc *volatile savehandler;
 	int savelen;
+	int saveprompt;
+#if __GNUC__
+	/* Avoid longjmp clobbering */
+	(void) &saveprompt;
+#endif
 
 	savepbq = parsebackquote;
 	if (setjmp(jmploc.loc)) {
@@ -1164,7 +1283,7 @@ parsebackq: {
 	savelen = out - stackblock();
 	if (savelen > 0) {
 		str = ckmalloc(savelen);
-		bcopy(stackblock(), str, savelen);
+		memcpy(str, stackblock(), savelen);
 	}
 	savehandler = handler;
 	handler = &jmploc;
@@ -1173,32 +1292,65 @@ parsebackq: {
                 /* We must read until the closing backquote, giving special
                    treatment to some slashes, and then push the string and
                    reread it as input, interpreting it normally.  */
-                register char *out;
-                register c;
+                char *out;
+                int c;
                 int savelen;
                 char *str;
 
+
                 STARTSTACKSTR(out);
-                while ((c = pgetc ()) != '`') {
-                       if (c == PEOF) {
-                                startlinno = plinno;
-                                synerror("EOF in backquote substitution");
-                       }
-                       if (c == '\\') {
-                                c = pgetc ();
-                                if ( c != '\\' && c != '`' && c != '$'
+		for (;;) {
+			if (needprompt) {
+				setprompt(2);
+				needprompt = 0;
+			}
+			switch (c = pgetc()) {
+			case '`':
+				goto done;
+
+			case '\\':
+                                if ((c = pgetc()) == '\n') {
+					plinno++;
+					if (doprompt)
+						setprompt(2);
+					else
+						setprompt(0);
+					/*
+					 * If eating a newline, avoid putting
+					 * the newline into the new character
+					 * stream (via the STPUTC after the
+					 * switch).
+					 */
+					continue;
+				}
+                                if (c != '\\' && c != '`' && c != '$'
                                     && (!dblquote || c != '"'))
                                         STPUTC('\\', out);
-                       }
-                       STPUTC(c, out);
+				break;
+
+			case '\n':
+				plinno++;
+				needprompt = doprompt;
+				break;
+
+			case PEOF:
+			        startlinno = plinno;
+				synerror("EOF in backquote substitution");
+ 				break;
+
+			default:
+				break;
+			}
+			STPUTC(c, out);
                 }
+done:
                 STPUTC('\0', out);
                 savelen = out - stackblock();
                 if (savelen > 0) {
                         str = ckmalloc(savelen);
-                        bcopy(stackblock(), str, savelen);
+                        memcpy(str, stackblock(), savelen);
+			setinputstring(str, 1);
                 }
-                setinputstring(str, 1);
         }
 	nlpp = &bqlist;
 	while (*nlpp)
@@ -1206,18 +1358,35 @@ parsebackq: {
 	*nlpp = (struct nodelist *)stalloc(sizeof (struct nodelist));
 	(*nlpp)->next = NULL;
 	parsebackquote = oldstyle;
+
+	if (oldstyle) {
+		saveprompt = doprompt;
+		doprompt = 0;
+	}
+
 	n = list(0);
-        if (!oldstyle && (readtoken() != TRP))
-                synexpect(TRP);
+
+	if (oldstyle)
+		doprompt = saveprompt;
+	else {
+		if (readtoken() != TRP)
+			synexpect(TRP);
+	}
+
 	(*nlpp)->n = n;
-        /* Start reading from old file again.  */
-        if (oldstyle)
+        if (oldstyle) {
+		/*
+		 * Start reading from old file again, ignoring any pushed back
+		 * tokens left from the backquote parsing
+		 */
                 popfile();
+		tokpushback = 0;
+	}
 	while (stackblocksize() <= savelen)
 		growstackblock();
 	STARTSTACKSTR(out);
 	if (str) {
-		bcopy(str, out, savelen);
+		memcpy(out, str, savelen);
 		STADJUST(savelen, out);
 		INTOFF;
 		ckfree(str);
@@ -1245,6 +1414,10 @@ parsearith: {
 		prevsyntax = syntax;
 		syntax = ARISYNTAX;
 		USTPUTC(CTLARI, out);
+		if (dblquote)
+			USTPUTC('"',out);
+		else
+			USTPUTC(' ',out);
 	} else {
 		/*
 		 * we collapse embedded arithmetic expansion to
@@ -1275,11 +1448,13 @@ STATIC int
 noexpand(text)
 	char *text;
 	{
-	register char *p;
-	register char c;
+	char *p;
+	char c;
 
 	p = text;
 	while ((c = *p++) != '\0') {
+		if ( c == CTLQUOTEMARK)
+			continue;
 		if (c == CTLESC)
 			p++;
 		else if (BASESYNTAX[c] == CCTL)
@@ -1298,7 +1473,7 @@ int
 goodname(name)
 	char *name;
 	{
-	register char *p;
+	char *p;
 
 	p = name;
 	if (! is_name(*p))
@@ -1318,7 +1493,9 @@ goodname(name)
  */
 
 STATIC void
-synexpect(token) {
+synexpect(token)
+	int token;
+{
 	char msg[64];
 
 	if (token >= 0) {
@@ -1347,7 +1524,9 @@ setprompt(which)
 	{
 	whichprompt = which;
 
+#ifndef NO_HISTORY
 	if (!el)
+#endif
 		out2str(getprompt(NULL));
 }
 
@@ -1357,8 +1536,8 @@ setprompt(which)
  */
 char *
 getprompt(unused)
-	void *unused;
-	{
+	void *unused __unused;
+{
 	switch (whichprompt) {
 	case 0:
 		return "";
