@@ -32,42 +32,60 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/show.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)show.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)show.c	8.3 (Berkeley) 5/4/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/show.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <stdio.h>
+#ifdef __STDC__
+#include <stdarg.h>
+#else
+#include <varargs.h>
+#endif
+
 #include "shell.h"
 #include "parser.h"
 #include "nodes.h"
 #include "mystring.h"
+#include "show.h"
 
 
 #ifdef DEBUG
-static shtree(), shcmd(), sharg(), indent();
+static void shtree __P((union node *, int, char *, FILE*));
+static void shcmd __P((union node *, FILE *));
+static void sharg __P((union node *, FILE *));
+static void indent __P((int, char *, FILE *));
+static void trstring __P((char *));
 
 
+void
 showtree(n)
 	union node *n;
-	{
+{
 	trputs("showtree called\n");
 	shtree(n, 1, NULL, stdout);
 }
 
 
-static
+static void
 shtree(n, ind, pfx, fp)
 	union node *n;
+	int ind;
 	char *pfx;
 	FILE *fp;
-	{
+{
 	struct nodelist *lp;
 	char *s;
 
+	if (n == NULL)
+		return;
+
 	indent(ind, pfx, fp);
 	switch(n->type) {
 	case NSEMI:
@@ -110,11 +128,11 @@ binop:
 
 
 
-static
+static void
 shcmd(cmd, fp)
 	union node *cmd;
 	FILE *fp;
-	{
+{
 	union node *np;
 	int first;
 	char *s;
@@ -136,6 +154,7 @@ shcmd(cmd, fp)
 			case NTOFD:	s = ">&"; dftfd = 1; break;
 			case NFROM:	s = "<";  dftfd = 0; break;
 			case NFROMFD:	s = "<&"; dftfd = 0; break;
+			default:  	s = "*error*"; dftfd = 0; break;
 		}
 		if (np->nfile.fd != dftfd)
 			fprintf(fp, "%d", np->nfile.fd);
@@ -151,7 +170,7 @@ shcmd(cmd, fp)
 
 
 
-static
+static void
 sharg(arg, fp)
 	union node *arg;
 	FILE *fp;
@@ -175,10 +194,15 @@ sharg(arg, fp)
 			putc('$', fp);
 			putc('{', fp);
 			subtype = *++p;
+			if (subtype == VSLENGTH)
+				putc('#', fp);
+
 			while (*p != '=')
 				putc(*p++, fp);
+
 			if (subtype & VSNUL)
 				putc(':', fp);
+
 			switch (subtype & VSTYPE) {
 			case VSNORMAL:
 				putc('}', fp);
@@ -195,6 +219,22 @@ sharg(arg, fp)
 			case VSASSIGN:
 				putc('=', fp);
 				break;
+			case VSTRIMLEFT:
+				putc('#', fp);
+				break;
+			case VSTRIMLEFTMAX:
+				putc('#', fp);
+				putc('#', fp);
+				break;
+			case VSTRIMRIGHT:
+				putc('%', fp);
+				break;
+			case VSTRIMRIGHTMAX:
+				putc('%', fp);
+				putc('%', fp);
+				break;
+			case VSLENGTH:
+				break;
 			default:
 				printf("<subtype %d>", subtype);
 			}
@@ -217,11 +257,12 @@ sharg(arg, fp)
 }
 
 
-static
+static void
 indent(amount, pfx, fp)
+	int amount;
 	char *pfx;
 	FILE *fp;
-	{
+{
 	int i;
 
 	for (i = 0 ; i < amount ; i++) {
@@ -230,8 +271,6 @@ indent(amount, pfx, fp)
 		putc('\t', fp);
 	}
 }
-#endif
-
 
 
 /*
@@ -248,50 +287,62 @@ int debug = 0;
 #endif
 
 
-trputc(c) {
-#ifdef DEBUG
+void
+trputc(c)
+	int c;
+{
 	if (tracefile == NULL)
 		return;
 	putc(c, tracefile);
 	if (c == '\n')
 		fflush(tracefile);
-#endif
 }
 
 
-trace(fmt, a1, a2, a3, a4, a5, a6, a7, a8)
+void
+#ifdef __STDC__
+trace(const char *fmt, ...)
+#else
+trace(va_alist)
+	va_dcl
+#endif
+{
+	va_list va;
+#ifdef __STDC__
+	va_start(va, fmt);
+#else
 	char *fmt;
-	{
-#ifdef DEBUG
-	if (tracefile == NULL)
-		return;
-	fprintf(tracefile, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
-	if (strchr(fmt, '\n'))
-		fflush(tracefile);
+	va_start(va);
+	fmt = va_arg(va, char *);
 #endif
+	if (tracefile != NULL) {
+		(void) vfprintf(tracefile, fmt, va);
+		if (strchr(fmt, '\n'))
+			(void) fflush(tracefile);
+	}
+	va_end(va);
 }
 
 
+void
 trputs(s)
 	char *s;
-	{
-#ifdef DEBUG
+{
 	if (tracefile == NULL)
 		return;
 	fputs(s, tracefile);
 	if (strchr(s, '\n'))
 		fflush(tracefile);
-#endif
 }
 
 
+static void
 trstring(s)
 	char *s;
-	{
-	register char *p;
+{
+	char *p;
 	char c;
 
-#ifdef DEBUG
 	if (tracefile == NULL)
 		return;
 	putc('"', tracefile);
@@ -323,14 +374,13 @@ backslash:	  putc('\\', tracefile);
 		}
 	}
 	putc('"', tracefile);
-#endif
 }
 
 
+void
 trargs(ap)
 	char **ap;
-	{
-#ifdef DEBUG
+{
 	if (tracefile == NULL)
 		return;
 	while (*ap) {
@@ -341,28 +391,31 @@ trargs(ap)
 			putc('\n', tracefile);
 	}
 	fflush(tracefile);
-#endif
 }
 
 
+void
 opentrace() {
 	char s[100];
-	char *p;
 	char *getenv();
+#ifdef O_APPEND
 	int flags;
+#endif
 
-#ifdef DEBUG
 	if (!debug)
 		return;
 #ifdef not_this_way
-	if ((p = getenv("HOME")) == NULL) {
-		if (geteuid() == 0)
-			p = "/";
-		else
-			p = "/tmp";
+	{
+		char *p;
+		if ((p = getenv("HOME")) == NULL) {
+			if (geteuid() == 0)
+				p = "/";
+			else
+				p = "/tmp";
+		}
+		scopy(p, s);
+		strcat(s, "/trace");
 	}
-	scopy(p, s);
-	strcat(s, "/trace");
 #else
 	scopy("./trace", s);
 #endif /* not_this_way */
@@ -376,5 +429,5 @@ opentrace() {
 #endif
 	fputs("\nTracing started.\n", tracefile);
 	fflush(tracefile);
-#endif /* DEBUG */
 }
+#endif /* DEBUG */
