@@ -32,28 +32,33 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/trap.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)trap.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)trap.c	8.5 (Berkeley) 6/5/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/trap.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <signal.h>
+#include <unistd.h>
+#include <stdlib.h>
+
 #include "shell.h"
 #include "main.h"
 #include "nodes.h"	/* for other headers */
 #include "eval.h"
 #include "jobs.h"
+#include "show.h"
 #include "options.h"
 #include "syntax.h"
-#include "signames.h"
 #include "output.h"
 #include "memalloc.h"
 #include "error.h"
 #include "trap.h"
 #include "mystring.h"
-#include <signal.h>
 
 
 /*
@@ -69,37 +74,101 @@ static char sccsid[] = "@(#)trap.c	8.1 (Berkeley) 5/31/93";
 #define S_RESET 5		/* temporary - to reset a hard ignored sig */
 
 
-extern char nullstr[1];		/* null string */
+MKINIT char sigmode[NSIG];	/* current value of signal */
+int pendingsigs;		/* indicates some signal received */
+int in_dotrap;			/* do we execute in a trap handler? */
+static char *volatile trap[NSIG];	/* trap handler commands */
+static volatile sig_atomic_t gotsig[NSIG]; 
+				/* indicates specified signal received */
+static int ignore_sigchld;	/* Used while handling SIGCHLD traps. */
+
+static int getsigaction __P((int, sig_t *));
+
+
+/*
+ * Map a string to a signal number.
+ */
+static int
+sigstring_to_signum(sig)
+	char *sig;
+{
+
+	if (is_number(sig)) {
+		int signo;
+
+		signo = atoi(sig);
+		return ((signo >= 0 && signo < NSIG) ? signo : (-1));
+	} else if (strcasecmp(sig, "exit") == 0) {
+		return (0);
+	} else {
+		int n;
+
+		if (strncasecmp(sig, "sig", 3) == 0)
+			sig += 3;
+		for (n = 1; n < NSIG; n++)
+			if (strcasecmp(sys_signame[n], sig) == 0)
+				return (n);
+	}
+	return (-1);
+}
+
+
+/*
+ * Print a list of valid signal names.
+ */
+static void
+printsignals()
+{
+	int n;
+
+	for (n = 1; n < NSIG; n++) {
+		out1fmt("%s", sys_signame[n]);
+		if (n == (NSIG / 2) || n == (NSIG - 1))
+			out1str("\n");
+		else
+			out1c(' ');
+	}
+}
 
-char *trap[MAXSIG+1];		/* trap handler commands */
-MKINIT char sigmode[MAXSIG];	/* current value of signal */
-char gotsig[MAXSIG];		/* indicates specified signal received */
-int pendingsigs;			/* indicates some signal received */
 
 /*
  * The trap builtin.
  */
-
-trapcmd(argc, argv)  char **argv; {
+int
+trapcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	char *action;
-	char **ap;
 	int signo;
 
 	if (argc <= 1) {
-		for (signo = 0 ; signo <= MAXSIG ; signo++) {
+		for (signo = 0 ; signo < NSIG ; signo++) {
 			if (trap[signo] != NULL)
-				out1fmt("%d: %s\n", signo, trap[signo]);
+				out1fmt("trap -- '%s' %s\n", trap[signo],
+					(signo) ? sys_signame[signo] : "exit");
 		}
 		return 0;
 	}
-	ap = argv + 1;
-	if (is_number(*ap))
-		action = NULL;
-	else
-		action = *ap++;
-	while (*ap) {
-		if ((signo = number(*ap)) < 0 || signo > MAXSIG)
-			error("%s: bad trap", *ap);
+	action = NULL;
+	if (*++argv && strcmp(*argv, "--") == 0)
+		argv++;
+	if (*argv && sigstring_to_signum(*argv) == -1) {
+		if ((*argv)[0] != '-') {
+			action = *argv;
+			argv++;
+		} else if ((*argv)[1] == '\0') {
+			argv++;
+		} else if ((*argv)[1] == 'l' && (*argv)[2] == '\0') {
+			printsignals();
+			return 0;
+		} else {
+			error("bad option %s", *argv);
+		}
+	}
+	while (*argv) {
+		if ((signo = sigstring_to_signum(*argv)) == -1)
+			error("bad signal %s", *argv);
 		INTOFF;
 		if (action)
 			action = savestr(action);
@@ -109,22 +178,21 @@ trapcmd(argc, argv)  char **argv; {
 		if (signo != 0)
 			setsignal(signo);
 		INTON;
-		ap++;
+		argv++;
 	}
 	return 0;
 }
 
 
-
 /*
  * Clear traps on a fork.
  */
-
 void
-clear_traps() {
-	char **tp;
+clear_traps()
+{
+	char *volatile *tp;
 
-	for (tp = trap ; tp <= &trap[MAXSIG] ; tp++) {
+	for (tp = trap ; tp <= &trap[NSIG - 1] ; tp++) {
 		if (*tp && **tp) {	/* trap not NULL or SIG_IGN */
 			INTOFF;
 			ckfree(*tp);
@@ -137,19 +205,17 @@ clear_traps() {
 }
 
 
-
 /*
  * Set the signal handler for the specified signal.  The routine figures
  * out what it should be set to.
  */
-
-int
-setsignal(signo) {
+void
+setsignal(signo)
+	int signo;
+{
 	int action;
-	sig_t sigact;
+	sig_t sig, sigact = SIG_DFL;
 	char *t;
-	extern void onsig();
-	extern sig_t getsigaction();
 
 	if ((t = trap[signo]) == NULL)
 		action = S_DFL;
@@ -157,11 +223,10 @@ setsignal(signo) {
 		action = S_CATCH;
 	else
 		action = S_IGN;
-	if (rootshell && action == S_DFL) {
+	if (action == S_DFL) {
 		switch (signo) {
 		case SIGINT:
-			if (iflag)
-				action = S_CATCH;
+			action = S_CATCH;
 			break;
 		case SIGQUIT:
 #ifdef DEBUG
@@ -172,26 +237,35 @@ setsignal(signo) {
 				break;
 			}
 #endif
-			/* FALLTHROUGH */
+			action = S_CATCH;
+			break;
 		case SIGTERM:
-			if (iflag)
+			if (rootshell && iflag)
 				action = S_IGN;
 			break;
 #if JOBS
 		case SIGTSTP:
 		case SIGTTOU:
-			if (mflag)
+			if (rootshell && mflag)
 				action = S_IGN;
 			break;
 #endif
 		}
 	}
-	t = &sigmode[signo - 1];
+
+	t = &sigmode[signo];
 	if (*t == 0) {
 		/*
 		 * current setting unknown
 		 */
-		sigact = getsigaction(signo);
+		if (!getsigaction(signo, &sigact)) {
+			/*
+			 * Pretend it worked; maybe we should give a warning
+			 * here, but other shells don't. We don't alter
+			 * sigmode, so that we retry every time.
+			 */
+			return;
+		}
 		if (sigact == SIG_IGN) {
 			if (mflag && (signo == SIGTSTP ||
 			     signo == SIGTTIN || signo == SIGTTOU)) {
@@ -203,51 +277,62 @@ setsignal(signo) {
 		}
 	}
 	if (*t == S_HARD_IGN || *t == action)
-		return 0;
+		return;
 	switch (action) {
 		case S_DFL:	sigact = SIG_DFL;	break;
 		case S_CATCH:  	sigact = onsig;		break;
 		case S_IGN:	sigact = SIG_IGN;	break;
 	}
 	*t = action;
-	return (int)signal(signo, sigact);
+	sig = signal(signo, sigact);
+#ifdef BSD
+	if (sig != SIG_ERR && action == S_CATCH)
+		siginterrupt(signo, 1);
+#endif
 }
 
+
 /*
  * Return the current setting for sig w/o changing it.
  */
-sig_t
-getsigaction(signo) {
+static int
+getsigaction(signo, sigact)
+	int signo;
+	sig_t *sigact;
+{
 	struct sigaction sa;
 
 	if (sigaction(signo, (struct sigaction *)0, &sa) == -1)
-		error("Sigaction system call failed");
-
-	return sa.sa_handler;
+		return 0;
+	*sigact = (sig_t) sa.sa_handler;
+	return 1;
 }
 
+
 /*
  * Ignore a signal.
  */
-
 void
-ignoresig(signo) {
-	if (sigmode[signo - 1] != S_IGN && sigmode[signo - 1] != S_HARD_IGN) {
+ignoresig(signo)
+	int signo;
+{
+
+	if (sigmode[signo] != S_IGN && sigmode[signo] != S_HARD_IGN) {
 		signal(signo, SIG_IGN);
 	}
-	sigmode[signo - 1] = S_HARD_IGN;
+	sigmode[signo] = S_HARD_IGN;
 }
 
 
 #ifdef mkinit
-INCLUDE "signames.h"
+INCLUDE <signal.h>
 INCLUDE "trap.h"
 
 SHELLPROC {
 	char *sm;
 
 	clear_traps();
-	for (sm = sigmode ; sm < sigmode + MAXSIG ; sm++) {
+	for (sm = sigmode ; sm < sigmode + NSIG ; sm++) {
 		if (*sm == S_IGN)
 			*sm = S_HARD_IGN;
 	}
@@ -255,60 +340,87 @@ SHELLPROC {
 #endif
 
 
-
 /*
  * Signal handler.
  */
-
 void
-onsig(signo) {
+onsig(signo)
+	int signo;
+{
+
+#ifndef BSD
 	signal(signo, onsig);
+#endif
 	if (signo == SIGINT && trap[SIGINT] == NULL) {
 		onint();
 		return;
 	}
-	gotsig[signo - 1] = 1;
-	pendingsigs++;
-}
 
+	if (signo != SIGCHLD || !ignore_sigchld)
+		gotsig[signo] = 1;
+	pendingsigs++;
+
+	/* If we are currently in a wait builtin, prepare to break it */
+	if ((signo == SIGINT || signo == SIGQUIT) && in_waitcmd != 0)
+		breakwaitcmd = 1;
+	/* 
+	 * If a trap is set, not ignored and not the null command, we need 
+	 * to make sure traps are executed even when a child blocks signals.
+	 */
+	if (trap[signo] != NULL && 
+	    ! trap[signo][0] == '\0' &&
+	    ! (trap[signo][0] == ':' && trap[signo][1] == '\0'))
+		breakwaitcmd = 1;
+}
 
 
 /*
  * Called to execute a trap.  Perhaps we should avoid entering new trap
  * handlers while we are executing a trap handler.
  */
-
 void
-dotrap() {
+dotrap()
+{
 	int i;
 	int savestatus;
 
+	in_dotrap++;
 	for (;;) {
-		for (i = 1 ; ; i++) {
-			if (gotsig[i - 1])
+		for (i = 1; i < NSIG; i++) {
+			if (gotsig[i]) {
+				gotsig[i] = 0;
+				if (trap[i]) {
+					/*
+					 * Ignore SIGCHLD to avoid infinite recursion
+					 * if the trap action does a fork.
+					 */
+					if (i == SIGCHLD)
+						ignore_sigchld++;
+					savestatus = exitstatus;
+					evalstring(trap[i]);
+					exitstatus = savestatus;
+					if (i == SIGCHLD)
+						ignore_sigchld--;
+				}
 				break;
-			if (i >= MAXSIG)
-				goto done;
+			}
 		}
-		gotsig[i - 1] = 0;
-		savestatus=exitstatus;
-		evalstring(trap[i]);
-		exitstatus=savestatus;
+		if (i >= NSIG)
+			break;
 	}
-done:
+	in_dotrap--;
 	pendingsigs = 0;
 }
 
 
-
 /*
  * Controls whether the shell is interactive or not.
  */
-
-
 void
-setinteractive(on) {
-	static int is_interactive;
+setinteractive(on)
+	int on;
+{
+	static int is_interactive = -1;
 
 	if (on == is_interactive)
 		return;
@@ -319,13 +431,13 @@ setinteractive(on) {
 }
 
 
-
 /*
  * Called to exit the shell.
  */
-
 void
-exitshell(status) {
+exitshell(status)
+	int status;
+{
 	struct jmploc loc1, loc2;
 	char *p;
 
