@@ -32,18 +32,25 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/bin/sh/var.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)var.c	8.1 (Berkeley) 5/31/93";
+#if 0
+static char sccsid[] = "@(#)var.c	8.3 (Berkeley) 5/4/95";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/bin/sh/var.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
+#include <unistd.h>
+#include <stdlib.h>
+
 /*
  * Shell variables.
  */
 
+#include <locale.h>
+
 #include "shell.h"
 #include "output.h"
 #include "expand.h"
@@ -57,6 +64,10 @@ static char sccsid[] = "@(#)var.c	8.1 (Berkeley) 5/31/93";
 #include "memalloc.h"
 #include "error.h"
 #include "mystring.h"
+#include "parser.h"
+#ifndef NO_HISTORY
+#include "myhistedit.h"
+#endif
 
 
 #define VTABSIZE 39
@@ -66,13 +77,16 @@ struct varinit {
 	struct var *var;
 	int flags;
 	char *text;
+	void (*func) __P((const char *));
 };
 
 
 #if ATTY
 struct var vatty;
 #endif
+#ifndef NO_HISTORY
 struct var vhistsize;
+#endif
 struct var vifs;
 struct var vmail;
 struct var vmpath;
@@ -83,31 +97,45 @@ struct var vvers;
 #if ATTY
 struct var vterm;
 #endif
+struct var voptind;
 
 const struct varinit varinit[] = {
 #if ATTY
-	{&vatty,	VSTRFIXED|VTEXTFIXED|VUNSET,	"ATTY="},
+	{ &vatty,	VSTRFIXED|VTEXTFIXED|VUNSET,	"ATTY=",
+	  NULL },
 #endif
-	{&vhistsize,	VSTRFIXED|VTEXTFIXED|VUNSET,	"HISTSIZE="},
-	{&vifs,	VSTRFIXED|VTEXTFIXED,		"IFS= \t\n"},
-	{&vmail,	VSTRFIXED|VTEXTFIXED|VUNSET,	"MAIL="},
-	{&vmpath,	VSTRFIXED|VTEXTFIXED|VUNSET,	"MAILPATH="},
-	{&vpath,	VSTRFIXED|VTEXTFIXED,		"PATH=:/bin:/usr/bin"},
+#ifndef NO_HISTORY
+	{ &vhistsize,	VSTRFIXED|VTEXTFIXED|VUNSET,	"HISTSIZE=",
+	  sethistsize },
+#endif
+	{ &vifs,	VSTRFIXED|VTEXTFIXED,		"IFS= \t\n",
+	  NULL },
+	{ &vmail,	VSTRFIXED|VTEXTFIXED|VUNSET,	"MAIL=",
+	  NULL },
+	{ &vmpath,	VSTRFIXED|VTEXTFIXED|VUNSET,	"MAILPATH=",
+	  NULL },
+	{ &vpath,	VSTRFIXED|VTEXTFIXED,		"PATH=/bin:/usr/bin",
+	  changepath },
 	/*
 	 * vps1 depends on uid
 	 */
-	{&vps2,	VSTRFIXED|VTEXTFIXED,		"PS2=> "},
+	{ &vps2,	VSTRFIXED|VTEXTFIXED,		"PS2=> ",
+	  NULL },
 #if ATTY
-	{&vterm,	VSTRFIXED|VTEXTFIXED|VUNSET,	"TERM="},
+	{ &vterm,	VSTRFIXED|VTEXTFIXED|VUNSET,	"TERM=",
+	  NULL },
 #endif
-	{NULL,	0,				NULL}
+	{ &voptind,	VSTRFIXED|VTEXTFIXED,		"OPTIND=1",
+	  getoptsreset },
+	{ NULL,	0,				NULL,
+	  NULL }
 };
 
 struct var *vartab[VTABSIZE];
 
-STATIC int unsetvar __P((char *));
 STATIC struct var **hashvar __P((char *));
 STATIC int varequal __P((char *, char *));
+STATIC int localevar __P((char *));
 
 /*
  * Initialize the varable symbol tables and import the environment
@@ -147,6 +175,7 @@ initvar() {
 			*vpp = vp;
 			vp->text = ip->text;
 			vp->flags = ip->flags;
+			vp->func = ip->func;
 		}
 	}
 	/*
@@ -161,6 +190,33 @@ initvar() {
 	}
 }
 
+/*
+ * Safe version of setvar, returns 1 on success 0 on failure.
+ */
+
+int
+setvarsafe(name, val, flags)
+	char *name, *val;
+	int flags;
+{
+	struct jmploc jmploc;
+	struct jmploc *volatile savehandler = handler;
+	int err = 0;
+#if __GNUC__
+	/* Avoid longjmp clobbering */
+	(void) &err;
+#endif
+
+	if (setjmp(jmploc.loc))
+		err = 1;
+	else {
+		handler = &jmploc;
+		setvar(name, val, flags);
+	}
+	handler = savehandler;
+	return err;
+}
+
 /*
  * Set the value of a variable.  The flags argument is ored with the
  * flags of the variable.  If val is NULL, the variable is unset.
@@ -169,7 +225,8 @@ initvar() {
 void
 setvar(name, val, flags)
 	char *name, *val;
-	{
+	int flags;
+{
 	char *p, *q;
 	int len;
 	int namelen;
@@ -178,8 +235,9 @@ setvar(name, val, flags)
 
 	isbad = 0;
 	p = name;
-	if (! is_name(*p++))
+	if (! is_name(*p))
 		isbad = 1;
+	p++;
 	for (;;) {
 		if (! is_in_name(*p)) {
 			if (*p == '\0' || *p == '=')
@@ -208,7 +266,27 @@ setvar(name, val, flags)
 	setvareq(nameeq, flags);
 }
 
+STATIC int
+localevar(s)
+	char *s;
+	{
+	static char *lnames[7] = {
+		"ALL", "COLLATE", "CTYPE", "MONETARY",
+		"NUMERIC", "TIME", NULL
+	};
+	char **ss;
 
+	if (*s != 'L')
+		return 0;
+	if (varequal(s + 1, "ANG"))
+		return 1;
+	if (strncmp(s + 1, "C_", 2) != 0)
+		return 0;
+	for (ss = lnames; *ss ; ss++)
+		if (varequal(s + 3, *ss))
+			return 1;
+	return 0;
+}
 
 /*
  * Same as setvar except that the variable and value are passed in
@@ -220,28 +298,39 @@ setvar(name, val, flags)
 void
 setvareq(s, flags)
 	char *s;
-	{
+	int flags;
+{
 	struct var *vp, **vpp;
 
 	vpp = hashvar(s);
 	for (vp = *vpp ; vp ; vp = vp->next) {
 		if (varequal(s, vp->text)) {
 			if (vp->flags & VREADONLY) {
-				int len = strchr(s, '=') - s;
+				size_t len = strchr(s, '=') - s;
 				error("%.*s: is read only", len, s);
 			}
 			INTOFF;
-			if (vp == &vpath)
-				changepath(s + 5);	/* 5 = strlen("PATH=") */
+
+			if (vp->func && (flags & VNOFUNC) == 0)
+				(*vp->func)(strchr(s, '=') + 1);
+
 			if ((vp->flags & (VTEXTFIXED|VSTACK)) == 0)
 				ckfree(vp->text);
-			vp->flags &=~ (VTEXTFIXED|VSTACK|VUNSET);
+
+			vp->flags &= ~(VTEXTFIXED|VSTACK|VUNSET);
 			vp->flags |= flags;
 			vp->text = s;
+
+			/*
+			 * We could roll this to a function, to handle it as
+			 * a regular variable function callback, but why bother?
+			 */
 			if (vp == &vmpath || (vp == &vmail && ! mpathset()))
 				chkmail(1);
-			if (vp == &vhistsize)
-				sethistsize();
+			if ((vp->flags & VEXPORT) && localevar(s)) {
+				putenv(s);
+				(void) setlocale(LC_ALL, "");
+			}
 			INTON;
 			return;
 		}
@@ -251,7 +340,14 @@ setvareq(s, flags)
 	vp->flags = flags;
 	vp->text = s;
 	vp->next = *vpp;
+	vp->func = NULL;
+	INTOFF;
 	*vpp = vp;
+	if ((vp->flags & VEXPORT) && localevar(s)) {
+		putenv(s);
+		(void) setlocale(LC_ALL, "");
+	}
+	INTON;
 }
 
 
@@ -306,7 +402,8 @@ lookupvar(name)
 char *
 bltinlookup(name, doall)
 	char *name;
-	{
+	int doall;
+{
 	struct strlist *sp;
 	struct var *v;
 
@@ -316,8 +413,8 @@ bltinlookup(name, doall)
 	}
 	for (v = *hashvar(name) ; v ; v = v->next) {
 		if (varequal(v->text, name)) {
-			if (v->flags & VUNSET
-			 || ! doall && (v->flags & VEXPORT) == 0)
+			if ((v->flags & VUNSET)
+			 || (!doall && (v->flags & VEXPORT) == 0))
 				return NULL;
 			return strchr(v->text, '=') + 1;
 		}
@@ -404,7 +501,10 @@ shprocvar() {
  */
 
 int
-showvarscmd(argc, argv)  char **argv; {
+showvarscmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	struct var **vpp;
 	struct var *vp;
 
@@ -424,7 +524,10 @@ showvarscmd(argc, argv)  char **argv; {
  */
 
 int
-exportcmd(argc, argv)  char **argv; {
+exportcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	struct var **vpp;
 	struct var *vp;
 	char *name;
@@ -441,6 +544,10 @@ exportcmd(argc, argv)  char **argv; {
 				for (vp = *vpp ; vp ; vp = vp->next) {
 					if (varequal(vp->text, name)) {
 						vp->flags |= flag;
+						if ((vp->flags & VEXPORT) && localevar(vp->text)) {
+							putenv(vp->text);
+							(void) setlocale(LC_ALL, "");
+						}
 						goto found;
 					}
 				}
@@ -467,7 +574,11 @@ found:;
  * The "local" command.
  */
 
-localcmd(argc, argv)  char **argv; {
+int
+localcmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	char *name;
 
 	if (! in_function())
@@ -498,7 +609,7 @@ mklocal(name)
 	lvp = ckmalloc(sizeof (struct localvar));
 	if (name[0] == '-' && name[1] == '\0') {
 		lvp->text = ckmalloc(sizeof optlist);
-		bcopy(optlist, lvp->text, sizeof optlist);
+		memcpy(lvp->text, optlist, sizeof optlist);
 		vp = NULL;
 	} else {
 		vpp = hashvar(name);
@@ -539,7 +650,7 @@ poplocalvars() {
 		localvars = lvp->next;
 		vp = lvp->vp;
 		if (vp == NULL) {	/* $- saved */
-			bcopy(lvp->text, optlist, sizeof optlist);
+			memcpy(optlist, lvp->text, sizeof optlist);
 			ckfree(lvp->text);
 		} else if ((lvp->flags & (VUNSET|VSTRFIXED)) == VUNSET) {
 			(void)unsetvar(vp->text);
@@ -554,7 +665,11 @@ poplocalvars() {
 }
 
 
-setvarcmd(argc, argv)  char **argv; {
+int
+setvarcmd(argc, argv)
+	int argc;
+	char **argv;
+{
 	if (argc <= 2)
 		return unsetcmd(argc, argv);
 	else if (argc == 3)
@@ -571,7 +686,11 @@ setvarcmd(argc, argv)  char **argv; {
  * with the same name.
  */
 
-unsetcmd(argc, argv)  char **argv; {
+int
+unsetcmd(argc, argv)
+	int argc __unused;
+	char **argv __unused;
+{
 	char **ap;
 	int i;
 	int flg_func = 0;
@@ -601,7 +720,7 @@ unsetcmd(argc, argv)  char **argv; {
  * Unset the specified variable.
  */
 
-STATIC int
+int
 unsetvar(s)
 	char *s;
 	{
@@ -616,7 +735,11 @@ unsetvar(s)
 			INTOFF;
 			if (*(strchr(vp->text, '=') + 1) != '\0')
 				setvar(s, nullstr, 0);
-			vp->flags &=~ VEXPORT;
+			if ((vp->flags & VEXPORT) && localevar(vp->text)) {
+				unsetenv(s);
+				setlocale(LC_ALL, "");
+			}
+			vp->flags &= ~VEXPORT;
 			vp->flags |= VUNSET;
 			if ((vp->flags & VSTRFIXED) == 0) {
 				if ((vp->flags & VTEXTFIXED) == 0)
@@ -640,13 +763,13 @@ unsetvar(s)
 
 STATIC struct var **
 hashvar(p)
-	register char *p;
+	char *p;
 	{
 	unsigned int hashval;
 
-	hashval = *p << 4;
+	hashval = ((unsigned char) *p) << 4;
 	while (*p && *p != '=')
-		hashval += *p++;
+		hashval += (unsigned char) *p++;
 	return &vartab[hashval % VTABSIZE];
 }
 
@@ -660,7 +783,7 @@ hashvar(p)
 
 STATIC int
 varequal(p, q)
-	register char *p, *q;
+	char *p, *q;
 	{
 	while (*p == *q++) {
 		if (*p++ == '=')
