@@ -30,11 +30,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/stty/cchar.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/stty/cchar.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)cchar.c	8.5 (Berkeley) 4/2/94";
+static char const sccsid[] = "@(#)cchar.c	8.5 (Berkeley) 4/2/94";
 #endif /* not lint */
 
 #include <sys/types.h>
