@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/bin/stty/key.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/bin/stty/key.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -122,11 +122,11 @@ ksearch(argvp, ip)
 	    sizeof(keys)/sizeof(struct key), sizeof(struct key), c_key)))
 		return (0);
 	if (!(kp->flags & F_OFFOK) && ip->off) {
-		errx(1, "illegal option -- %s", name);
+		warnx("illegal option -- -%s", name);
 		usage();
 	}
 	if (kp->flags & F_NEEDARG && !(ip->arg = *++*argvp)) {
-		errx(1, "option requires an argument -- %s", name);
+		warnx("option requires an argument -- %s", name);
 		usage();
 	}
 	kp->f(ip);
@@ -192,11 +192,11 @@ f_extproc(ip)
 	struct info *ip;
 {
 
-	if (ip->set) {
-		int tmp = 1;
+	if (ip->off) {
+		int tmp = 0;
 		(void)ioctl(ip->fd, TIOCEXT, &tmp);
 	} else {
-		int tmp = 0;
+		int tmp = 1;
 		(void)ioctl(ip->fd, TIOCEXT, &tmp);
 	}
 }
@@ -286,7 +286,7 @@ f_speed(ip)
 	struct info *ip;
 {
 
-	(void)printf("%ld\n", cfgetospeed(&ip->t));
+	(void)printf("%lu\n", (u_long)cfgetospeed(&ip->t));
 }
 
 void
