@@ -42,6 +42,7 @@ static char sccsid[] = "@(#)save.c	8.1 (Berkeley) 5/31/93";
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include "hdr.h"
 
 struct savestruct
@@ -107,7 +108,7 @@ struct savestruct save_array[] =
 	dseen,          sizeof(dseen),
 	fixed,          sizeof(fixed),
 	hinted,         sizeof(hinted),
-	link,           sizeof(link),
+	linkx,           sizeof(linkx),
 	odloc,          sizeof(odloc),
 	place,          sizeof(place),
 	prop,           sizeof(prop),
@@ -137,6 +138,7 @@ char *outfile;  /* to output the data using checksum to start random #s */
 		outfile);
 	    return 1;
 	}
+
 	fwrite(&sum, sizeof(sum), 1, out);      /* Here's the random() key */
 	for (p = save_array; p->address != NULL; p++)
 	{
@@ -164,6 +166,7 @@ char *infile;
 		infile);
 	    return 1;
 	}
+
 	fread(&sum, sizeof(sum), 1, in);        /* Get the seed */
 	srandom((int) sum);
 	for (p = save_array; p->address != NULL; p++)
