@@ -36,6 +36,8 @@ static char sccsid[] = "@(#)subs.c	8.1 (Berkeley) 5/31/93";
 #endif /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include "back.h"
 
 int	buffnum;
@@ -46,15 +48,15 @@ static char	plwhite[] = "Player is white, computer is red.";
 static char	nocomp[] = "(No computer play.)";
 
 char  *descr[] = {
-	"Usage:  backgammon [-] [n r w b pr pw pb t3a]\n",
-	"\t-\tgets this list\n\tn\tdon't ask for rules or instructions",
-	"\tr\tplayer is red (implies n)\n\tw\tplayer is white (implies n)",
-	"\tb\ttwo players, red and white (implies n)",
-	"\tpr\tprint the board before red's turn",
-	"\tpw\tprint the board before white's turn",
-	"\tpb\tprint the board before both player's turn",
-	"\tterm\tterminal is a term",
-	"\tsfile\trecover saved game from file",
+	"Usage:  backgammon [-h n r w b pr pw pb tterm sfile]\n",
+	"\t-h\tgets this list\n\t-n\tdon't ask for rules or instructions",
+	"\t-r\tplayer is red (implies -n)\n\t-w\tplayer is white (implies -n)",
+	"\t-b\ttwo players, red and white (implies -n)",
+	"\t-pr\tprint the board before red's turn",
+	"\t-pw\tprint the board before white's turn",
+	"\t-pb\tprint the board before both player's turn",
+	"\t-tterm\tterminal is a term",
+	"\t-sfile\trecover saved game from file",
 	0
 };
 
@@ -298,25 +300,28 @@ nexturn ()  {
 	colorptr += c;
 }
 
-getarg (arg)
-register char	***arg;
+getarg (argc, argv)
+register int    argc;
+register char	**argv;
 
 {
-	register char	**s;
+	register char	ch;
+	extern int optind;
+	extern char *optarg;
+	int i;
 
 	/* process arguments here.  dashes are ignored, nbrw are ignored
 	   if the game is being recovered */
 
-	s = *arg;
-	while (s[0][0] == '-') {
-		switch (s[0][1])  {
+	while ((ch = getopt (argc, argv, "nbrwp:t:s:h")) != -1) {
+		switch (ch)  {
 
 		/* don't ask if rules or instructions needed */
 		case 'n':
 			if (rflag)
 				break;
 			aflag = 0;
-			args[acnt++] = 'n';
+			args[acnt++] = strdup ("-n");
 			break;
 
 		/* player is both read and white */
@@ -325,7 +330,7 @@ register char	***arg;
 				break;
 			pnum = 0;
 			aflag = 0;
-			args[acnt++] = 'b';
+			args[acnt++] = strdup ("-b");
 			break;
 
 		/* player is red */
@@ -334,7 +339,7 @@ register char	***arg;
 				break;
 			pnum = -1;
 			aflag = 0;
-			args[acnt++] = 'r';
+			args[acnt++] = strdup ("-r");
 			break;
 
 		/* player is white */
@@ -343,41 +348,41 @@ register char	***arg;
 				break;
 			pnum = 1;
 			aflag = 0;
-			args[acnt++] = 'w';
+			args[acnt++] = strdup ("-w");
 			break;
 
 		/* print board after move according to following character */
 		case 'p':
-			if (s[0][2] != 'r' && s[0][2] != 'w' && s[0][2] != 'b')
+			if (optarg[0] != 'r' && optarg[0] != 'w' && optarg[0] != 'b')
 				break;
-			args[acnt++] = 'p';
-			args[acnt++] = s[0][2];
-			if (s[0][2] == 'r')
+			args[acnt] = strdup ("-p ");
+			args[acnt++][2] = optarg[0];
+			if (optarg[0] == 'r')
 				bflag = 1;
-			if (s[0][2] == 'w')
+			if (optarg[0] == 'w')
 				bflag = -1;
-			if (s[0][2] == 'b')
+			if (optarg[0] == 'b')
 				bflag = 0;
 			break;
 
 		case 't':
-			if (s[0][2] == '\0') {	/* get terminal caps */
-				s++;
-				tflag = getcaps (*s);
-			} else
-				tflag = getcaps (&s[0][2]);
+		        tflag = getcaps (optarg);
 			break;
 
 		case 's':
-			s++;
 			/* recover file */
-			recover (s[0]);
+			recover (optarg);
 			break;
+		case 'h':
+			for (i = 0; descr[i] != 0; i++)
+				puts (descr[i]);
+			getout();
 		}
-		s++;
 	}
-	if (s[0] != 0)
-		recover(s[0]);
+	argc -= optind;
+	argv += optind;
+	if ( argc && argv[0][0] != '\0' )
+		recover(argv[0]);
 }
 
 init ()  {
@@ -428,7 +433,7 @@ getout ()  {
 
 	/* fix terminal status */
 	fixtty (old);
-	exit();
+	exit(0);
 }
 roll ()  {
 	register char	c;
