@@ -58,6 +58,8 @@ static char sccsid[] = "@(#)canfield.c	8.1 (Berkeley) 5/31/93";
 #include <ctype.h>
 #include <signal.h>
 #include <termios.h>
+#include <unistd.h>
+#include <stdlib.h>
 
 #include "pathnames.h"
 
@@ -1567,15 +1569,13 @@ initall()
 {
 	int i;
 
-	srandom(getpid());
+	if (dbfd < 0)
+		return;
+	srandomdev();
 	time(&acctstart);
 	initdeck(deck);
 	uid = getuid();
-	if (uid < 0)
-		uid = 0;
-	dbfd = open(_PATH_SCORE, 2);
-	if (dbfd < 0)
-		return;
+
 	i = lseek(dbfd, uid * sizeof(struct betinfo), 0);
 	if (i < 0) {
 		close(dbfd);
@@ -1675,6 +1675,11 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
+	dbfd = open(_PATH_SCORE, 2);
+
+	/* revoke */
+	setgid(getgid());
+
 #ifdef MAXLOAD
 	double vec[3];
 
@@ -1691,6 +1696,7 @@ main(argc, argv)
 	raw();
 	noecho();
 	initall();
+
 	instruct();
 	makeboard();
 	for (;;) {
