@@ -41,7 +41,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)factor.c	8.3 (Berkeley) 3/30/94";
+static char sccsid[] = "@(#)factor.c	8.4 (Berkeley) 5/4/95";
 #endif /* not lint */
 
 /*
@@ -69,6 +69,7 @@ static char sccsid[] = "@(#)factor.c	8.3 (Berkeley) 3/30/94";
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include "primes.h"
 
@@ -93,7 +94,7 @@ main(argc, argv)
 	int ch;
 	char *p, buf[100];		/* > max number of digits. */
 
-	while ((ch = getopt(argc, argv, "")) != EOF)
+	while ((ch = getopt(argc, argv, "")) != -1)
 		switch (ch) {
 		case '?':
 		default:
