@@ -46,8 +46,8 @@ static const char sccsid[] = "@(#)fortune.c   8.1 (Berkeley) 5/31/93";
 
 # include	<sys/param.h>
 # include	<sys/stat.h>
-# include	<sys/dir.h>
 
+# include	<dirent.h>
 # include	<fcntl.h>
 # include	<assert.h>
 # include	<unistd.h>
@@ -189,7 +189,7 @@ char	*av[];
 	int	fd;
 #endif	/* OK_TO_WRITE_DISK */
 
-	(void) setlocale(LC_CTYPE, "");
+	(void) setlocale(LC_ALL, "");
 
 	getargs(ac, av);
 
@@ -199,7 +199,7 @@ char	*av[];
 #endif
 
 	init_prob();
-	srandom((int)(time((time_t *) NULL) ^ getpid()));
+	srandomdev();
 	do {
 		get_fort();
 	} while ((Short_only && fortlen() > SLEN) ||
@@ -306,9 +306,9 @@ register char	**argv;
 	pat = NULL;
 
 # ifdef DEBUG
-	while ((ch = getopt(argc, argv, "aDefilm:osw")) != EOF)
+	while ((ch = getopt(argc, argv, "aDefilm:osw")) != -1)
 #else
-	while ((ch = getopt(argc, argv, "aefilm:osw")) != EOF)
+	while ((ch = getopt(argc, argv, "aefilm:osw")) != -1)
 #endif /* DEBUG */
 		switch(ch) {
 		case 'a':		/* any fortune */
@@ -693,11 +693,7 @@ add_dir(fp)
 register FILEDESC	*fp;
 {
 	register DIR		*dir;
-#ifdef SYSV
-	register struct dirent	*dirent;	/* NIH, of course! */
-#else
-	register struct direct	*dirent;
-#endif
+	register struct dirent	*dirent;
 	auto FILEDESC		*tailp;
 	auto char		*name;
 
