@@ -38,16 +38,17 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)number.c	8.2 (Berkeley) 3/31/94";
+static char sccsid[] = "@(#)number.c	8.3 (Berkeley) 5/4/95";
 #endif /* not lint */
 
 #include <sys/types.h>
 
 #include <ctype.h>
+#include <err.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <err.h>
+#include <unistd.h>
 
 #define	MAXNUM		65		/* Biggest number we handle. */
 
@@ -91,7 +92,7 @@ main(argc, argv)
 	char line[256];
 
 	lflag = 0;
-	while ((ch = getopt(argc, argv, "l")) != EOF)
+	while ((ch = getopt(argc, argv, "l")) !=  -1)
 		switch (ch) {
 		case 'l':
 			lflag = 1;
@@ -164,6 +165,7 @@ badnum:			errx(1, "illegal number: %s", line);
 	if (*line == '-') {
 		(void)printf("minus%s", lflag ? " " : "\n");
 		++line;
+		--len;
 	}
 
 	rval = len > 0 ? unit(len, line) : 0;
