@@ -1,7 +1,7 @@
 /*
  * piano.c - a piano emulator
  */
-static char rcsid[] = "$FreeBSD: stable/2.1/games/piano/piano.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/games/piano/piano.c 50952 1999-09-05 11:54:50Z peter $";
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -139,7 +139,7 @@ main(int argc, char *argv[])
 	extern int optind, opterr;
 	int ch, ex, show_usage = 0;
 	myname = argv[0];
-	while ((ch = getopt(argc, argv, "-vi:")) != EOF) {
+	while ((ch = getopt(argc, argv, "-vi:")) !=  -1) {
 		switch (ch) {
 		default:
 		case 'V':
