@@ -54,6 +54,7 @@ static char sccsid[] = "@(#)wump.c	8.1 (Berkeley) 5/31/93";
 #include <sys/types.h>
 #include <sys/file.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include "pathnames.h"
 
 /* some defines to spec out what our wumpus cave should look like */
@@ -111,10 +112,13 @@ main(argc, argv)
 	extern char *optarg;
 	int c;
 
+	/* revoke */
+	setgid(getgid());
+
 #ifdef DEBUG
-	while ((c = getopt(argc, argv, "a:b:hp:r:t:d")) != EOF)
+	while ((c = getopt(argc, argv, "a:b:hp:r:t:d")) != -1)
 #else
-	while ((c = getopt(argc, argv, "a:b:hp:r:t:")) != EOF)
+	while ((c = getopt(argc, argv, "a:b:hp:r:t:")) != -1)
 #endif
 		switch (c) {
 		case 'a':
@@ -478,7 +482,6 @@ cave_init()
 {
 	register int i, j, k, link;
 	int delta, int_compare();
-	time_t time();
 
 	/*
 	 * This does most of the interesting work in this program actually!
@@ -489,7 +492,7 @@ cave_init()
 	 * than three links, regardless of the quality of the random number
 	 * generator that we're using.
 	 */
-	srandom((int)time((time_t *)0));
+	srandomdev();
 
 	/* initialize the cave first off. */
 	for (i = 1; i <= room_num; ++i)
