@@ -26,7 +26,7 @@
 #include "uucp.h"
 
 #if USE_RCS_ID
-const char serial_rcsid[] = "$FreeBSD: stable/2.1/gnu/libexec/uucp/libunix/serial.c 50952 1999-09-05 11:54:50Z peter $";
+const char serial_rcsid[] = "$FreeBSD: stable/2.2/gnu/libexec/uucp/libunix/serial.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "uudefs.h"
@@ -919,6 +919,9 @@ static struct sbaud_table
 #endif
 #ifdef B115200
   { B115200, 115200 },
+#endif
+#ifdef B230400
+  { B230400, 230400 },
 #endif
   { B0, 0 }
 };
