@@ -18,7 +18,7 @@
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/as/config/atof-ieee.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/as/config/atof-ieee.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "as.h"
@@ -128,12 +128,13 @@ static void
 LITTLENUM_TYPE *words;
 {
 	as_bad("cannot create floating-point number");
-	words[0] = ((unsigned) -1) >> 1; /* Zero the leftmost bit */
-	words[1] = -1;
-	words[2] = -1;
-	words[3] = -1;
-	words[4] = -1;
-	words[5] = -1;
+	/* Zero the leftmost bit */
+	words[0] = (LITTLENUM_TYPE) ((unsigned) -1) >> 1;
+	words[1] = (LITTLENUM_TYPE) -1;
+	words[2] = (LITTLENUM_TYPE) -1;
+	words[3] = (LITTLENUM_TYPE) -1;
+	words[4] = (LITTLENUM_TYPE) -1;
+	words[5] = (LITTLENUM_TYPE) -1;
 }
 
 /***********************************************************************\
@@ -173,7 +174,7 @@ LITTLENUM_TYPE *words;	/* Build the binary here. */
 	generic_floating_point_number.low	= bits + MAX_PRECISION;
 	generic_floating_point_number.high	= NULL;
 	generic_floating_point_number.leader	= NULL;
-	generic_floating_point_number.exponent	= NULL;
+	generic_floating_point_number.exponent	= 0;
 	generic_floating_point_number.sign	= '\0';
 
 	/* Use more LittleNums than seems */
