@@ -86,12 +86,12 @@ int md_short_jump_size = 4;
 int md_long_jump_size = 4;
 #if defined(BFD_HEADERS)
 #ifdef RELSZ
-int md_reloc_size = RELSZ;	/* Coff headers */
+const int md_reloc_size = RELSZ;	/* Coff headers */
 #else
-int md_reloc_size = 12;		/* something else headers */
+const int md_reloc_size = 12;		/* something else headers */
 #endif
 #else
-int md_reloc_size = 12;		/* Not bfdized*/
+const int md_reloc_size = 12;		/* Not bfdized*/
 #endif
 
 /* This array holds the chars that always start a comment.  If the
