@@ -25,7 +25,7 @@
   */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/as/config/tc-i386.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/as/config/tc-i386.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "as.h"
@@ -1606,7 +1606,7 @@ char *operand_string;
 			 * into a temporary buffer...
 			 */
 			register char *cp;
-			if (flagseen['k'] &&
+			if (picmode &&
 				(cp = strchr(input_line_pointer,'@'))) {
 				char tmpbuf[BUFSIZ];
 
@@ -1759,7 +1759,7 @@ register segT	segment;
 /* XXX - oops, the JMP_TBL relocation info should have percolated through
  * here, define a field in frag to this?
  */
-	(flagseen['k'] && S_GET_SEGMENT(fragP->fr_symbol) == SEG_UNKNOWN)?
+	(picmode && S_GET_SEGMENT(fragP->fr_symbol) == SEG_UNKNOWN)?
 			RELOC_JMP_TBL :
 #endif
 				 NO_RELOC, (symbolS *)0);
@@ -1776,7 +1776,7 @@ register segT	segment;
 				 (symbolS *) 0,
 				 fragP->fr_offset, 1,
 #ifdef PIC
-/*XXX*/	(flagseen['k'] && S_GET_SEGMENT(fragP->fr_symbol) == SEG_UNKNOWN)?
+/*XXX*/	(picmode && S_GET_SEGMENT(fragP->fr_symbol) == SEG_UNKNOWN)?
 			RELOC_JMP_TBL :
 #endif
 				NO_RELOC, (symbolS *)0);
@@ -1871,7 +1871,7 @@ register fragS *	fragP;
 
 int md_short_jump_size = 2;	/* size of byte displacement jmp */
 int md_long_jump_size  = 5;	/* size of dword displacement jmp */
-int md_reloc_size = 8;		/* Size of relocation record */
+const int md_reloc_size = 8;		/* Size of relocation record */
 
 void md_create_short_jump(ptr, from_addr, to_addr, frag, to_symbol)
 char	*ptr;
@@ -1913,8 +1913,10 @@ char **argP;
 int *cntP;
 char ***vecP;
 {
+	switch (**argP) {
 #ifdef PIC
-	if (argP && *argP && **argP == 'k') {
+	case 'k':
+	case 'K':
 #if 00
 		char *tmp = xmalloc(3+1+strlen(operand_special_chars));
 		strcpy(tmp, operand_special_chars);
@@ -1931,8 +1933,12 @@ char ***vecP;
 
 		/* Predefine GOT symbol */
 		GOT_symbol = symbol_find_or_make("__GLOBAL_OFFSET_TABLE_");
-	}
+		break;
 #endif
+
+	default:
+		return 0;
+	}
 	return 1;
 }
 
@@ -2065,7 +2071,7 @@ relax_addressT segment_address_in_file;
 	case NO_RELOC:
 		break;
 	case RELOC_32:
-		if (!flagseen['k'] || !S_IS_EXTERNAL(fixP->fx_addsy))
+		if (!picmode || !S_IS_EXTERNAL(fixP->fx_addsy))
 			break;
 		r_symbolnum = fixP->fx_addsy->sy_number;
 		extrn_bit = 1;
@@ -2076,6 +2082,7 @@ relax_addressT segment_address_in_file;
 		if (!extrn_bit && !S_IS_EXTERNAL(fixP->fx_addsy))
 			as_warn("GOT relocation burb: `%s' should be global",
 					S_GET_NAME(fixP->fx_addsy));
+		S_SET_EXTERNAL(fixP->fx_addsy);
 		extrn_bit = 1;
 		break;
 	case RELOC_GOTOFF:
