@@ -25,7 +25,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/as/expr.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/as/expr.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include <ctype.h>
@@ -126,7 +126,7 @@ register expressionS *	expressionP;
 		/* likewise for the b's.  xoxorich. */
 		if ((c == 'f' || c == 'b' || c == 'B')
 		    && (!*input_line_pointer ||
-			(!strchr("+-.0123456789",*input_line_pointer) &&
+			(!strchr("+-.0123456789iInN",*input_line_pointer) &&
 			 !strchr(EXP_CHARS,*input_line_pointer)))) {
 		    maxdig = radix = 10;
 		    too_many_digits = 11;
