@@ -17,7 +17,7 @@
    along with GAS; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /*
- * $FreeBSD: stable/2.1/gnu/usr.bin/as/struc-symbol.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/as/struc-symbol.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 
@@ -50,9 +50,13 @@ struct symbol			/* our version of an nlist node */
 	void 	*sy_sizexp;	/* (expressionS *) */
 
 	/* Auxiliary type information as given by the .type directive */
-	int	sy_aux;
+	unsigned char	sy_aux;
 #define AUX_OBJECT	1
 #define AUX_FUNC	2
+	unsigned char	sy_bind;
+#define BIND_LOCAL	0	/* currently not used */
+#define BIND_GLOBAL	1	/* currently not used */
+#define BIND_WEAK	2
 };
 
 typedef struct symbol symbolS;
