@@ -19,7 +19,7 @@
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/as/symbols.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/as/symbols.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "as.h"
@@ -316,10 +316,6 @@ register char *  sym_name; /* symbol name, as a cannonical string */
 				 * on the sparc also depends on it.
 				 */
 				/*			  char New_Type = SEGMENT_TO_SYMBOL_TYPE((int) now_seg); */
-#ifdef MANY_SEGMENTS
-#define SEG_BSS SEG_E2
-#define SEG_DATA SEG_E1
-#endif
 
 				if (((!S_IS_DEBUG(symbolP) && !S_IS_DEFINED(symbolP) && S_IS_EXTERNAL(symbolP))
 				     || (S_GET_SEGMENT(symbolP) == SEG_BSS))
