@@ -21,7 +21,7 @@
 /* This thing should be set up to do byteordering correctly.  But... */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/as/write.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/as/write.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "as.h"
@@ -726,11 +726,13 @@ segT		segment; /* SEG_DATA or SEG_TEXT */
 						offset = lie->add->sy_frag->fr_address+ S_GET_VALUE(lie->add) + lie->addnum -
 						    (lie->sub->sy_frag->fr_address+ S_GET_VALUE(lie->sub));
 						if (offset <= -32768 || offset >= 32767) {
+#if 0
 							if (flagseen['K'])
 							    as_warn(".word %s-%s+%ld didn't fit",
 								    S_GET_NAME(lie->add),
 								    S_GET_NAME(lie->sub),
 								    lie->addnum);
+#endif
 							lie->added=1;
 							if (fragP->fr_subtype == 0) {
 								fragP->fr_subtype++;
@@ -1005,7 +1007,7 @@ segT this_segment_type; /* N_TYPE bits for segment. */
 				}
 #endif	/* TC_I960 */
 #ifdef PIC
-				if (flagseen['k'] &&
+				if (picmode &&
 						S_IS_EXTERNAL(add_symbolP)) {
 					as_bad("Can't reduce difference of external symbols in PIC code");
 				}
@@ -1080,7 +1082,7 @@ segT this_segment_type; /* N_TYPE bits for segment. */
 					 * Do not fixup refs to global data
 					 * even if defined here.
 					 */
-					if (!flagseen['k'] ||
+					if (!picmode ||
 #ifdef TC_NS32K
 					   fixP->fx_pcrel ||
 #endif
