@@ -45,7 +45,7 @@ static char  *license_msg[] = {
  */
 
 #ifdef RCSID
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/gzip/gzip.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/gzip/gzip.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include <ctype.h>
@@ -1006,6 +1006,13 @@ local int get_istat(iname, sbuf)
     char *dot; /* pointer to ifname extension, or NULL */
 #endif
 
+    if (strlen(iname) >= sizeof(ifname) - 3) {
+	errno = ENAMETOOLONG;
+	perror(iname);
+	exit_code = ERROR;
+	return ERROR;
+    }
+
     strcpy(ifname, iname);
 
     /* If input file exists, return OK. */
@@ -1573,7 +1580,6 @@ local int check_ofname()
 	    return ERROR;
 	}
     }
-    (void) chmod(ofname, 0777);
     if (unlink(ofname)) {
 	fprintf(stderr, "%s: ", progname);
 	perror(ofname);
@@ -1633,7 +1639,6 @@ local void copy_stat(ifstat)
 #endif
     remove_ofname = 0;
     /* It's now safe to remove the input file: */
-    (void) chmod(ifname, 0777);
     if (unlink(ifname)) {
 	WARN((stderr, "%s: ", progname));
 	if (!quiet) perror(ifname);
