@@ -97,7 +97,7 @@
  */
 
 #ifdef RCSID
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/gzip/inflate.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/gzip/inflate.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include <sys/types.h>
@@ -767,6 +767,8 @@ int inflate_dynamic()
     return i;                   /* incomplete code set */
   }
 
+  if (tl == NULL) /* Grrrhhh */
+	return 2;
 
   /* read in literal and distance code lengths */
   n = nl + nd;
