@@ -59,7 +59,7 @@
 #include "gzip.h"
 
 #ifdef RCSID
-static char rcsid[] = "$FreeBSD: stable/2.1/gnu/usr.bin/gzip/trees.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/gnu/usr.bin/gzip/trees.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* ===========================================================================
@@ -1070,6 +1070,6 @@ local void set_file_type()
     while (n < LITERALS) bin_freq += dyn_ltree[n++].Freq;
     *file_type = bin_freq > (ascii_freq >> 2) ? BINARY : ASCII;
     if (*file_type == BINARY && translate_eol) {
-        warn("-l used on binary file", "");
+        WARN((stderr, "-l used on binary file", ""));
     }
 }
