@@ -27,10 +27,11 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/i386/md.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/i386/md.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -40,7 +41,13 @@
 #include <stab.h>
 #include <string.h>
 
-#include "ld.h"
+#include "dynamic.h"
+
+#if defined(RTLD) && defined(SUN_COMPAT)
+#define REL_SIZE(r) (2)		/* !!!!! Sun BUG compatible */
+#else
+#define REL_SIZE(r) ((r)->r_length)
+#endif
 
 /*
  * Get relocation addend corresponding to relocation record RP
@@ -51,7 +58,7 @@ md_get_addend(rp, addr)
 struct relocation_info	*rp;
 unsigned char		*addr;
 {
-	switch (RELOC_TARGET_SIZE(rp)) {
+	switch (REL_SIZE(rp)) {
 	case 0:
 		return get_byte(addr);
 	case 1:
@@ -60,7 +67,7 @@ unsigned char		*addr;
 		return get_long(addr);
 	default:
 		errx(1, "Unsupported relocation size: %x",
-		    RELOC_TARGET_SIZE(rp));
+		    REL_SIZE(rp));
 	}
 }
 
@@ -74,7 +81,7 @@ long			relocation;
 unsigned char		*addr;
 int			relocatable_output;
 {
-	switch (RELOC_TARGET_SIZE(rp)) {
+	switch (REL_SIZE(rp)) {
 	case 0:
 		put_byte(addr, relocation);
 		break;
@@ -86,7 +93,7 @@ int			relocatable_output;
 		break;
 	default:
 		errx(1, "Unsupported relocation size: %x",
-		    RELOC_TARGET_SIZE(rp));
+		    REL_SIZE(rp));
 	}
 }
 
@@ -102,7 +109,7 @@ int			type;
 	/* Relocation size */
 	r->r_length = rp->r_length;
 
-	if (RELOC_PCREL_P(rp))
+	if (rp->r_pcrel)
 		r->r_pcrel = 1;
 
 	if (type & RELTYPE_RELATIVE)
@@ -165,6 +172,23 @@ u_long		addr;
 	sp->reloc_index = 0;
 }
 
+/*
+ * Bind a jmpslot to its target address.  TARGET is where the jmpslot
+ * should jump to, and WHERE is a pointer to the jmpslot's address field.
+ * This is called by the dynamic linker when LD_BIND_NOW is set in the
+ * environment.
+ */
+void
+md_bind_jmpslot(target, where)
+u_long target;
+caddr_t where;
+{
+	jmpslot_t	*sp =
+		(jmpslot_t *) (where - offsetof(jmpslot_t, addr[0]));
+
+	md_fix_jmpslot(sp, (long) sp, target);
+}
+
 /*
  * Update the relocation record for a RRS jmpslot.
  */
