@@ -27,9 +27,11 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/i386/md.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/i386/md.h 50952 1999-09-05 11:54:50Z peter $
  */
 
+#ifndef __MD_H__
+#define __MD_H__
 
 #if defined(CROSS_LINKER) && defined(XHOST) && XHOST==sparc
 #define NEED_SWAP
@@ -93,6 +95,8 @@
 
 #define md_get_rt_segment_addend(r,a)	md_get_addend(r,a)
 
+#define RELOC_INIT_SEGMENT_RELOC(r)	((r)->r_length = 2)
+
 /* Width of a Global Offset Table entry */
 #define GOT_ENTRY_SIZE	4
 typedef long	got_t;
@@ -224,3 +228,18 @@ void	md_swapout_jmpslot __P((jmpslot_t *, int));
 #define put_long(where,what)		(*(long *)(where) = (what))
 
 #endif /* CROSS_LINKER */
+
+void	md_init_header __P((struct exec *, int, int));
+long	md_get_addend __P((struct relocation_info *, unsigned char *));
+void	md_relocate __P((struct relocation_info *, long, unsigned char *, int));
+void	md_make_jmpslot __P((jmpslot_t *, long, long));
+void	md_fix_jmpslot __P((jmpslot_t *, long, u_long));
+void	md_bind_jmpslot __P((u_long, caddr_t));
+int	md_make_reloc __P((struct relocation_info *, struct relocation_info *, int));
+void	md_make_jmpreloc __P((struct relocation_info *, struct relocation_info *, int));
+void	md_make_gotreloc __P((struct relocation_info *, struct relocation_info *, int));
+void	md_make_copyreloc __P((struct relocation_info *, struct relocation_info *));
+void	md_set_breakpoint __P((long, long *));
+
+
+#endif /* __MD_H__ */
