@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/rrs.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/rrs.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
@@ -36,6 +36,7 @@
 #include <sys/file.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -48,6 +49,7 @@
 #include <string.h>
 
 #include "ld.h"
+#include "dynamic.h"
 
 static struct _dynamic		rrs_dyn;		/* defined in link.h */
 static struct so_debug		rrs_so_debug;		/* defined in link.h */
@@ -59,6 +61,8 @@ static struct nzlist		*rrs_symbols;		/* RRS symbol table */
 static char			*rrs_strtab;		/* RRS strings */
 static struct rrs_hash		*rrs_hashtab;		/* RT hash table */
 static struct shobj		*rrs_shobjs;
+char				*rrs_search_paths;	/* `-L' RT search */
+static int			rrs_search_paths_size;
 
 static int	reserved_rrs_relocs;
 static int	claimed_rrs_relocs;
@@ -73,6 +77,8 @@ static int	rrs_symbol_size;
 
 static int	current_jmpslot_offset;
 static int	current_got_offset;
+static int	max_got_offset;
+static int	min_got_offset;
 static int	got_origin;
 static int	current_reloc_offset;
 static int	current_hash_index;
@@ -168,7 +174,7 @@ alloc_rrs_reloc(entry, sp)
 	symbol			*sp;
 {
 #ifdef DEBUG
-printf("alloc_rrs_reloc: %s in %s\n", sp->name, get_file_name(entry));
+printf("alloc_rrs_reloc: %s in %s\n", demangle(sp->name), get_file_name(entry));
 #endif
 	reserved_rrs_relocs++;
 }
@@ -254,7 +260,7 @@ alloc_rrs_cpy_reloc(entry, sp)
 	if (sp->flags & GS_CPYRELOCRESERVED)
 		return;
 #ifdef DEBUG
-printf("alloc_rrs_copy: %s in %s\n", sp->name, get_file_name(entry));
+printf("alloc_rrs_copy: %s in %s\n", demangle(sp->name), get_file_name(entry));
 #endif
 	sp->flags |= GS_CPYRELOCRESERVED;
 	reserved_rrs_relocs++;
@@ -289,12 +295,13 @@ claim_rrs_reloc(entry, rp, sp, relocation)
 {
 	struct relocation_info	*r = rrs_next_reloc();
 
-	if (rp->r_address < text_start + text_size)
+	if (rp->r_address < text_start + text_size
+	    && (link_mode & WARNRRSTEXT))
 		warnx("%s: RRS text relocation at %#x for \"%s\"",
-			get_file_name(entry), rp->r_address, sp->name);
+			get_file_name(entry), rp->r_address, demangle(sp->name));
 
 #ifdef DEBUG
-printf("claim_rrs_reloc: %s in %s\n", sp->name, get_file_name(entry));
+printf("claim_rrs_reloc: %s in %s\n", demangle(sp->name), get_file_name(entry));
 #endif
 	r->r_address = rp->r_address;
 	r->r_symbolnum = sp->rrs_symbolnum;
@@ -302,7 +309,7 @@ printf("claim_rrs_reloc: %s in %s\n", sp->name, get_file_name(entry));
 	if (link_mode & SYMBOLIC) {
 		if (!sp->defined)
 			warnx("Cannot reduce symbol \"%s\" in %s",
-				sp->name, get_file_name(entry));
+				demangle(sp->name), get_file_name(entry));
 		RELOC_EXTERN_P(r) = 0;
 		*relocation += sp->value;
 		(void) md_make_reloc(rp, r, RELTYPE_RELATIVE);
@@ -329,7 +336,7 @@ claim_rrs_jmpslot(entry, rp, sp, addend)
 		errx(1, "internal error: "
 			"%s: claim_rrs_jmpslot: %s: no reservation",
 			get_file_name(entry),
-			sp->name);
+			demangle(sp->name));
 
 	if (sp->jmpslot_offset != -1)
 		return rrs_sdt.sdt_plt + sp->jmpslot_offset;
@@ -340,13 +347,13 @@ claim_rrs_jmpslot(entry, rp, sp, addend)
 #ifdef DEBUG
 printf("claim_rrs_jmpslot: %s: %s(%d) -> offset %x\n",
 	get_file_name(entry),
-	sp->name, sp->rrs_symbolnum, sp->jmpslot_offset);
+	demangle(sp->name), sp->rrs_symbolnum, sp->jmpslot_offset);
 #endif
 
 	if ((link_mode & SYMBOLIC) || rrs_section_type == RRS_PARTIAL) {
 		if (!sp->defined)
 			warnx("Cannot reduce symbol \"%s\" in %s",
-				sp->name, get_file_name(entry));
+				demangle(sp->name), get_file_name(entry));
 
 		md_fix_jmpslot( rrs_plt + sp->jmpslot_offset/sizeof(jmpslot_t),
 				rrs_sdt.sdt_plt + sp->jmpslot_offset,
@@ -408,7 +415,7 @@ claim_rrs_gotslot(entry, rp, lsp, addend)
 	if (!(sp->flags & GS_HASGOTSLOT))
 		errx(1, "internal error: "
 			"%s: claim_rrs_gotslot: %s: no reservation",
-			get_file_name(entry), sp->name);
+			get_file_name(entry), demangle(sp->name));
 
 	if (sp->gotslot_offset != -1) {
 #ifdef DIAGNOSTIC
@@ -417,7 +424,7 @@ claim_rrs_gotslot(entry, rp, lsp, addend)
 		       ? sp->value : 0))
 			errx(1, "%s: %s: gotslot at %#x is multiple valued, "
 				"*got = %#x, addend = %#x, sp->value = %#x",
-				get_file_name(entry), sp->name,
+				get_file_name(entry), demangle(sp->name),
 				sp->gotslot_offset,
 				*GOTP(sp->gotslot_offset), addend, sp->value);
 #endif
@@ -429,16 +436,16 @@ claim_rrs_gotslot(entry, rp, lsp, addend)
 		/* GOT offset 0 is reserved */
 		current_got_offset += sizeof(got_t);
 
-	if (current_got_offset > MAX_GOTOFF)
+	if (current_got_offset > max_got_offset)
 		errx(1, "%s: GOT overflow on symbol `%s' at %#x",
-		      get_file_name(entry), sp->name, RELOC_ADDRESS(rp));
+		      get_file_name(entry), demangle(sp->name), RELOC_ADDRESS(rp));
 
 	sp->gotslot_offset = current_got_offset;
 	current_got_offset += sizeof(got_t);
 
 #ifdef DEBUG
 printf("claim_rrs_gotslot: %s(%d,%#x) slot offset %#x, addend %#x\n",
-	 sp->name, sp->rrs_symbolnum, sp->value, sp->gotslot_offset, addend);
+	 demangle(sp->name), sp->rrs_symbolnum, sp->value, sp->gotslot_offset, addend);
 #endif
 
 	if (sp->defined &&
@@ -458,7 +465,7 @@ printf("claim_rrs_gotslot: %s(%d,%#x) slot offset %#x, addend %#x\n",
 		 * so again all symbols must be known.
 		 */
 		warnx("Cannot reduce symbol \"%s\" in %s",
-		      sp->name, get_file_name(entry));
+		      demangle(sp->name), get_file_name(entry));
 
 	} else {
 
@@ -477,7 +484,7 @@ printf("claim_rrs_gotslot: %s(%d,%#x) slot offset %#x, addend %#x\n",
 		 */
 		if (!sp->defined)
 			warnx("Cannot reduce symbol \"%s\" in %s",
-			      sp->name, get_file_name(entry));
+			      demangle(sp->name), get_file_name(entry));
 		discarded_rrs_relocs++;
 		return sp->gotslot_offset;
 	}
@@ -536,7 +543,7 @@ claim_rrs_internal_gotslot(entry, rp, lsp, addend)
 		/* GOT offset 0 is reserved */
 		current_got_offset += sizeof(got_t);
 
-	if (current_got_offset > MAX_GOTOFF)
+	if (current_got_offset > max_got_offset)
 		errx(1, "%s: GOT overflow for relocation at %#x",
 		      get_file_name(entry), RELOC_ADDRESS(rp));
 
@@ -579,11 +586,11 @@ claim_rrs_cpy_reloc(entry, rp, sp)
 	if (!(sp->flags & GS_CPYRELOCRESERVED))
 		errx(1, "internal error: "
 			"%s: claim_cpy_reloc: %s: no reservation",
-			get_file_name(entry), sp->name);
+			get_file_name(entry), demangle(sp->name));
 
 #ifdef DEBUG
 printf("claim_rrs_copy: %s: %s -> %x\n",
-	get_file_name(entry), sp->name, sp->so_defined);
+	get_file_name(entry), demangle(sp->name), sp->so_defined);
 #endif
 
 	r = rrs_next_reloc();
@@ -662,9 +669,10 @@ void
 consider_rrs_section_lengths()
 {
 	int		n;
-	struct shobj	*shp, **shpp;
+	struct shobj	*shp;
 
 #ifdef notyet
+	struct shobj    **shpp;
 /* We run into trouble with this as long as shared object symbols
    are not checked for definitions */
 	/*
@@ -686,7 +694,7 @@ consider_rrs_section_lengths()
 		rrs_section_type = RRS_NONE;
 	else if (link_mode & SHAREABLE)
 		rrs_section_type = RRS_FULL;
-	else if (number_of_shobjs == 0 /*&& !(link_mode & DYNAMIC)*/) {
+	else if (number_of_shobjs == 0 && !(link_mode & FORCEDYNAMIC)) {
 		/*
 		 * First slots in both tables are reserved
 		 * hence the "> 1" condition
@@ -816,6 +824,11 @@ consider_rrs_section_lengths()
 	rrs_text_size = reserved_rrs_relocs * sizeof(struct relocation_info);
 	rrs_text_size += number_of_rrs_hash_entries * sizeof(struct rrs_hash);
 	rrs_text_size += number_of_rrs_symbols * rrs_symbol_size;
+	rrs_search_paths_size = rrs_search_paths
+					? strlen(rrs_search_paths) + 1
+					: 0;
+	rrs_search_paths_size = MALIGN(rrs_search_paths_size);
+	rrs_text_size += rrs_search_paths_size;
 
 	/* Align strings size */
 	rrs_strtab_size = MALIGN(rrs_strtab_size);
@@ -839,6 +852,7 @@ consider_rrs_section_lengths()
 void
 relocate_rrs_addresses()
 {
+	int gotsize;
 
 	dynamic_symbol->value = 0;
 
@@ -849,16 +863,16 @@ relocate_rrs_addresses()
 	 */
 	current_jmpslot_offset = sizeof(jmpslot_t);
 	current_got_offset = 0;
+	max_got_offset = MAX_GOTOFF(pic_type);
+	min_got_offset = MIN_GOTOFF(pic_type);
+	gotsize = number_of_gotslots * sizeof(got_t);
 
-	if (1 /* Not "-fPIC" seen */) {
-		int gotsize = number_of_gotslots * sizeof(got_t);
+	if (gotsize + min_got_offset - (int)sizeof(got_t) > max_got_offset)
+		warnx("Global Offset Table overflow (use `-fPIC')");
 
-		if (gotsize + MIN_GOTOFF - (int)sizeof(got_t) > MAX_GOTOFF)
-			warnx("Global Offset Table overflow");
-		if (gotsize > MAX_GOTOFF)
-			/* Position at "two-complements" origin */
-			current_got_offset += MIN_GOTOFF;
-	}
+	if (gotsize > max_got_offset)
+		/* Position at "two-complements" origin */
+		current_got_offset += min_got_offset;
 
 	got_origin = -current_got_offset;
 
@@ -905,14 +919,18 @@ relocate_rrs_addresses()
 			number_of_rrs_hash_entries * sizeof(struct rrs_hash);
 	rrs_sdt.sdt_strings = rrs_sdt.sdt_nzlist +
 			number_of_rrs_symbols * rrs_symbol_size;
+	rrs_sdt.sdt_paths = rrs_search_paths
+				? rrs_sdt.sdt_strings + rrs_strtab_size
+				: 0;
+	rrs_sdt.sdt_sods = rrs_shobjs
+				? rrs_sdt.sdt_strings + rrs_strtab_size +
+				  rrs_search_paths_size
+				: 0;
+	rrs_sdt.sdt_filler2 = 0;
 	rrs_sdt.sdt_str_sz = rrs_strtab_size;
 	rrs_sdt.sdt_text_sz = text_size;
 	rrs_sdt.sdt_plt_sz = number_of_jmpslots * sizeof(jmpslot_t);
 
-	rrs_sdt.sdt_sods = rrs_shobjs ? rrs_sdt.sdt_strings+rrs_strtab_size : 0;
-	rrs_sdt.sdt_filler1 = 0;
-	rrs_sdt.sdt_filler2 = 0;
-
 	/*
 	 * Assign addresses to _GLOBAL_OFFSET_TABLE_ and __DYNAMIC.
 	 * The value `&__DYNAMIC' is in the GOT table at offset 0.
@@ -930,7 +948,7 @@ write_rrs_data()
 	if (rrs_section_type == RRS_NONE)
 		return;
 
-	pos = rrs_data_start + (N_DATOFF(outheader) - DATA_START(outheader));
+	pos = rrs_data_start + N_TXTOFF(outheader) - text_start;
 	if (fseek(outstream, pos, SEEK_SET) != 0)
 		err(1, "write_rrs_data: fseek");
 
@@ -972,6 +990,7 @@ write_rrs_text()
 	int			symsize;
 	struct nzlist		*nlp;
 	int			offset = 0;
+	int			aligned_offset;
 	struct shobj		*shp;
 	struct sod		*sodp;
 	int			bind;
@@ -979,7 +998,7 @@ write_rrs_text()
 	if (rrs_section_type == RRS_PARTIAL)
 		return;
 
-	pos = rrs_text_start + (N_TXTOFF(outheader) - TEXT_START(outheader));
+	pos = rrs_text_start + N_TXTOFF(outheader) - text_start;
 	if (fseek(outstream, pos, SEEK_SET) != 0)
 		err(1, "write_rrs_text: fseek");
 
@@ -1112,14 +1131,14 @@ write_rrs_text()
 				 */
 				if (sp->aux != AUX_FUNC)
 					errx(1, "%s: non-function jmpslot",
-						sp->name);
+						demangle(sp->name));
 				nlp->nz_other = N_OTHER(bind, sp->aux);
 				nlp->nz_value =
 					rrs_sdt.sdt_plt + sp->jmpslot_offset;
 			}
 		} else
 			errx(1, "internal error: %s defined in mysterious way",
-			     sp->name);
+			     demangle(sp->name));
 
 		/* Set symbol's name */
 		nlp->nz_strx = offset;
@@ -1148,7 +1167,10 @@ write_rrs_text()
 
 	} END_EACH_SYMBOL;
 
-	if (MALIGN(offset) != rrs_strtab_size)
+	aligned_offset = MALIGN(offset);
+	while (offset < aligned_offset)		/* Pad deterministically */
+		rrs_strtab[offset++] = '\0';
+	if (offset != rrs_strtab_size)
 		errx(1, "internal error: "
 			"inconsistent RRS string table length: %d, expected %d",
 			offset, rrs_strtab_size);
@@ -1163,6 +1185,9 @@ write_rrs_text()
 	/* Write the strings */
 	mywrite(rrs_strtab, rrs_strtab_size, 1, outstream);
 
+	/* Write RT search path */
+	mywrite(rrs_search_paths, rrs_search_paths_size, 1, outstream);
+
 	/*
 	 * Write the names of the shared objects needed at run-time
 	 */
