@@ -33,7 +33,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)malloc.c	5.11 (Berkeley) 2/23/91";*/
-static char *rcsid = "$FreeBSD: stable/2.1/gnu/usr.bin/ld/rtld/malloc.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/gnu/usr.bin/ld/rtld/malloc.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
 /*
@@ -66,6 +66,7 @@ static char *rcsid = "$FreeBSD: stable/2.1/gnu/usr.bin/ld/rtld/malloc.c 50952 19
 
 #define	NULL 0
 
+extern void xprintf __P((char *, ...));
 static void morecore();
 static int findbucket();
 
@@ -390,7 +391,7 @@ realloc(cp, nbytes)
  * header starts at ``freep''.  If srchlen is -1 search the whole list.
  * Return bucket number, or -1 if not found.
  */
-static
+static int
 findbucket(freep, srchlen)
 	union overhead *freep;
 	int srchlen;
@@ -479,3 +480,16 @@ int	n;
 #endif
 	return n;
 }
+
+void *
+calloc(num, size)
+	size_t num;
+	register size_t size;
+{
+	register void *p;
+
+	size *= num;
+	if ( (p = malloc(size)) )
+		bzero(p, size);
+	return(p);
+}
