@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/rtld/rtld.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/rtld/rtld.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
@@ -41,6 +41,7 @@
 #ifndef MAP_COPY
 #define MAP_COPY	MAP_PRIVATE
 #endif
+#include <dlfcn.h>
 #include <err.h>
 #include <fcntl.h>
 #include <a.out.h>
@@ -55,7 +56,12 @@
 #include <varargs.h>
 #endif
 
-#include "ld.h"
+#include <link.h>
+
+#include "md.h"
+#include "shlib.h"
+#include "support.h"
+#include "dynamic.h"
 
 #ifndef MAP_ANON
 #define MAP_ANON	0
@@ -88,13 +94,14 @@ struct somap_private {
 	struct so_map	*spd_parent;
 	struct so_list	*spd_children;
 	struct so_map	*spd_prev;
+	dev_t		spd_dev;
+	ino_t		spd_ino;
 	int		spd_refcount;
 	int		spd_flags;
 #define RTLD_MAIN	0x01
 #define RTLD_RTLD	0x02
 #define RTLD_DL		0x04
 #define RTLD_INIT	0x08
-#define RTLD_TRACED	0x10
 	unsigned long	a_text;    /* text size, if known     */
 	unsigned long	a_data;    /* initialized data size   */
 	unsigned long	a_bss;     /* uninitialized data size */
@@ -136,6 +143,10 @@ struct somap_private {
 #define LM_STRINGS(smp)	((char *) \
 	((smp)->som_addr + LM_OFFSET(smp) + LD_STRINGS((smp)->som_dynamic)))
 
+/* Start of search paths */
+#define LM_PATHS(smp)	((char *) \
+	((smp)->som_addr + LM_OFFSET(smp) + LD_PATHS((smp)->som_dynamic)))
+
 /* End of text */
 #define LM_ETEXT(smp)	((char *) \
 	((smp)->som_addr + LM_TXTADDR(smp) + LD_TEXTSZ((smp)->som_dynamic)))
@@ -151,6 +162,22 @@ struct somap_private {
 /* Parent of link map */
 #define LM_PARENT(smp)	(LM_PRIVATE(smp)->spd_parent)
 
+#ifndef RELOC_EXTERN_P
+#define RELOC_EXTERN_P(s) ((s)->r_extern)
+#endif
+
+#ifndef RELOC_SYMBOL
+#define RELOC_SYMBOL(s) ((s)->r_symbolnum)
+#endif
+
+#ifndef RELOC_PCREL_P
+#define RELOC_PCREL_P(s) ((s)->r_pcrel)
+#endif
+
+static char		__main_progname[] = "main";
+static char		*main_progname = __main_progname;
+static char		us[] = "/usr/libexec/ld.so";
+
 char			**environ;
 char			*__progname;
 int			errno;
@@ -158,44 +185,47 @@ int			errno;
 static uid_t		uid, euid;
 static gid_t		gid, egid;
 static int		careful;
-static char		__main_progname[] = "main";
-static char		*main_progname = __main_progname;
-static char		us[] = "/usr/libexec/ld.so";
 static int		anon_fd = -1;
+
+static char		*ld_bind_now;
+static char		*ld_ignore_missing_objects;
 static char		*ld_library_path;
 static char		*ld_preload;
-static int		tracing;
+static char		*ld_tracing;
+static char		*ld_suppress_warnings;
+static char		*ld_warn_non_pure_code;
 
 struct so_map		*link_map_head;
 struct so_map		*link_map_tail;
 struct rt_symbol	*rt_symbol_head;
 
-static void		*__dlopen __P((char *, int));
+static void		*__dlopen __P((const char *, int));
 static int		__dlclose __P((void *));
-static void		*__dlsym __P((void *, char *));
-static char		*__dlerror __P((void));
+static void		*__dlsym __P((void *, const char *));
+static const char	*__dlerror __P((void));
 static void		__dlexit __P((void));
+static void		*__dlsym3 __P((void *, const char *, void *));
 
 static struct ld_entry	ld_entry = {
-	__dlopen, __dlclose, __dlsym, __dlerror, __dlexit
+	__dlopen, __dlclose, __dlsym, __dlerror, __dlexit, __dlsym3
 };
 
        void		xprintf __P((char *, ...));
-static struct so_map	*map_object __P((	char *,
+static struct so_map	*map_object __P((	const char *,
 						struct sod *,
 						struct so_map *));
 static int		map_preload __P((void));
 static int		map_sods __P((struct so_map *));
-static int		reloc_and_init __P((struct so_map *));
+static int		reloc_and_init __P((struct so_map *, int));
 static void		unmap_object __P((struct so_map	*, int));
-static struct so_map	*alloc_link_map __P((	char *, struct sod *,
+static struct so_map	*alloc_link_map __P((	const char *, struct sod *,
 						struct so_map *, caddr_t,
 						struct _dynamic *));
 static void		free_link_map __P((struct so_map *));
 static inline int	check_text_reloc __P((	struct relocation_info *,
 						struct so_map *,
 						caddr_t));
-static int		reloc_map __P((struct so_map *));
+static int		reloc_map __P((struct so_map *, int));
 static void		reloc_copy __P((struct so_map *));
 static void		init_object __P((struct so_map *));
 static void		init_sods __P((struct so_list *));
@@ -209,9 +239,14 @@ static struct nzlist	*lookup __P((char *, struct so_map **, int));
 static inline struct rt_symbol	*lookup_rts __P((char *));
 static struct rt_symbol	*enter_rts __P((char *, long, int, caddr_t,
 						long, struct so_map *));
+static void		die __P((void));
 static void		generror __P((char *, ...));
 static int		maphints __P((void));
 static void		unmaphints __P((void));
+static void		ld_trace __P((struct so_map *));
+static void		rt_readenv __P((void));
+static int		hinthash __P((char *, int));
+int			rtld __P((int, struct crt_ldso *, struct _dynamic *));
 
 static inline int
 strcmp (register const char *s1, register const char *s2)
@@ -239,10 +274,12 @@ struct _dynamic		*dp;
 	struct so_debug		*ddp;
 	struct so_map		*main_map;
 	struct so_map		*smp;
+	char			*add_paths;
 
 	/* Check version */
 	if (version != CRT_VERSION_BSD_2 &&
 	    version != CRT_VERSION_BSD_3 &&
+	    version != CRT_VERSION_BSD_4 &&
 	    version != CRT_VERSION_SUN)
 		return -1;
 
@@ -267,10 +304,21 @@ struct _dynamic		*dp;
 		++reloc;
 	}
 
-	__progname = "ld.so";
+	if (version >= CRT_VERSION_BSD_4)
+		__progname = crtp->crt_ldso;
 	if (version >= CRT_VERSION_BSD_3)
 		main_progname = crtp->crt_prog;
 
+	/* Some buggy versions of crt0.o have crt_ldso filled in as NULL. */
+	if (__progname == NULL)
+		__progname = us;
+
+	/* Fill in some fields in _DYNAMIC or crt structure */
+	if (version >= CRT_VERSION_BSD_4)
+		crtp->crt_ldentry = &ld_entry;		/* crt */
+	else
+		crtp->crt_dp->d_entry = &ld_entry;	/* _DYNAMIC */
+
 	/* Setup out (private) environ variable */
 	environ = crtp->crt_ep;
 
@@ -280,22 +328,7 @@ struct _dynamic		*dp;
 
 	careful = (uid != euid) || (gid != egid);
 
-	if (careful) {
-		unsetenv("LD_LIBRARY_PATH");
-		unsetenv("LD_PRELOAD");
-	} else {
-		ld_library_path = getenv("LD_LIBRARY_PATH");
-	        ld_preload = getenv("LD_PRELOAD");
-	}
-
-	tracing = getenv("LD_TRACE_LOADED_OBJECTS") != NULL;
-
-	/*
-	 * Setup the directory search list for findshlib.  We use only
-	 * the standard search path.  Any extra directories from
-	 * LD_LIBRARY_PATH are searched explicitly, in rtfindlib.
-	 */
-	std_search_path();
+	rt_readenv();
 
 	anon_open();
 
@@ -313,27 +346,43 @@ struct _dynamic		*dp;
 	LM_PRIVATE(smp)->spd_refcount++;
 	LM_PRIVATE(smp)->spd_flags |= RTLD_RTLD;
 
-	/* Fill in some fields in main's __DYNAMIC structure */
-	crtp->crt_dp->d_entry = &ld_entry;
-	crtp->crt_dp->d_un.d_sdt->sdt_loaded = link_map_head->som_next;
+	/*
+	 * Setup the executable's run path
+	 */
+	if (version >= CRT_VERSION_BSD_4) {
+		add_paths = LM_PATHS(main_map);
+		if (add_paths)
+			add_search_path(add_paths);
+	}
+
+	/*
+	 * Setup the directory search list for findshlib.  We use only
+	 * the standard search path.  Any extra directories from
+	 * LD_LIBRARY_PATH are searched explicitly, in rtfindlib.
+	 */
+	std_search_path();
 
 	/* Map in LD_PRELOADs before the main program's shared objects so we
 	   can intercept those calls */
-	if (ld_preload != NULL && *ld_preload != '\0') {
+	if (ld_preload != NULL) {
 	        if(map_preload() == -1)			/* Failed */
-			return -1;
+			die();
 	}
 
 	/* Map all the shared objects that the main program depends upon */
 	if(map_sods(main_map) == -1)
-		return -1;
+		die();
 
-	if(tracing)	/* We're done */
+	if(ld_tracing) {	/* We're done */
+		ld_trace(link_map_head);
 		exit(0);
+	}
+
+	crtp->crt_dp->d_un.d_sdt->sdt_loaded = link_map_head->som_next;
 
 	/* Relocate and initialize all mapped objects */
-	if(reloc_and_init(main_map) == -1)		/* Failed */
-		return -1;
+	if(reloc_and_init(main_map, ld_bind_now != NULL) == -1)	/* Failed */
+		die();
 
 	ddp = crtp->crt_dp->d_debug;
 	ddp->dd_cc = rt_symbol_head;
@@ -363,9 +412,93 @@ struct _dynamic		*dp;
 	(void)close(crtp->crt_ldfd);
 	anon_close();
 
-	return LDSO_VERSION_HAS_DLEXIT;
+	return LDSO_VERSION_HAS_DLSYM3;
 }
 
+void
+ld_trace(smp)
+	struct so_map *smp;
+{
+	char	*fmt1, *fmt2, *fmt, *main_local;
+	int	c;
+
+	if ((main_local = getenv("LD_TRACE_LOADED_OBJECTS_PROGNAME")) == NULL)
+		main_local = "";
+
+	if ((fmt1 = getenv("LD_TRACE_LOADED_OBJECTS_FMT1")) == NULL)
+		fmt1 = "\t-l%o.%m => %p (%x)\n";
+
+	if ((fmt2 = getenv("LD_TRACE_LOADED_OBJECTS_FMT2")) == NULL)
+		fmt2 = "\t%o (%x)\n";
+
+	for (; smp; smp = smp->som_next) {
+		struct sod	*sodp;
+		char		*name, *path;
+
+		if ((sodp = smp->som_sod) == NULL)
+			continue;
+
+		name = (char *)sodp->sod_name;
+		if (LM_PARENT(smp))
+			name += (long)LM_LDBASE(LM_PARENT(smp));
+
+		if ((path = smp->som_path) == NULL)
+			path = "not found";
+
+		fmt = sodp->sod_library ? fmt1 : fmt2;
+		while ((c = *fmt++) != '\0') {
+			switch (c) {
+			default:
+				putchar(c);
+				continue;
+			case '\\':
+				switch (c = *fmt) {
+				case '\0':
+					continue;
+				case 'n':
+					putchar('\n');
+					break;
+				case 't':
+					putchar('\t');
+					break;
+				}
+				break;
+			case '%':
+				switch (c = *fmt) {
+				case '\0':
+					continue;
+				case '%':
+				default:
+					putchar(c);
+					break;
+				case 'A':
+					printf("%s", main_local);
+					break;
+				case 'a':
+					printf("%s", main_progname);
+					break;
+				case 'o':
+					printf("%s", name);
+					break;
+				case 'm':
+					printf("%d", sodp->sod_major);
+					break;
+				case 'n':
+					printf("%d", sodp->sod_minor);
+					break;
+				case 'p':
+					printf("%s", path);
+					break;
+				case 'x':
+					printf("%p", smp->som_addr);
+					break;
+				}
+				break;
+			}
+			++fmt;
+		}
+	}
+}
 
 /*
  * Allocate a new link map and return a pointer to it.
@@ -382,7 +515,7 @@ struct _dynamic		*dp;
  */
 	static struct so_map *
 alloc_link_map(path, sodp, parent, addr, dp)
-	char		*path;
+	const char	*path;
 	struct sod	*sodp;
 	struct so_map	*parent;
 	caddr_t		addr;
@@ -418,8 +551,9 @@ alloc_link_map(path, sodp, parent, addr, dp)
 	}
 
 	smp->som_addr = addr;
-	smp->som_path = strdup(path);
+	smp->som_path = path ? strdup(path) : NULL;
 	smp->som_sod = sodp;
+	smp->som_write = 0;
 	smp->som_dynamic = dp;
 	smp->som_spd = (caddr_t)smpp;
 
@@ -486,22 +620,46 @@ free_link_map(smp)
  */
 	static struct so_map *
 map_object(path, sodp, parent)
-	char		*path;
+	const char	*path;
 	struct sod	*sodp;
 	struct so_map	*parent;
 {
 	struct so_map	*smp;
+	struct stat	statbuf;
 
 	if(path == NULL)	/* Special case for the main program itself */
 		smp = link_map_head;
 	else {
-		/* Check whether the shared object is already mapped */
+		/*
+		 * Check whether the shared object is already mapped.
+		 * We check first for an exact match by pathname.  That
+		 * will detect the usual case.  If no match is found by
+		 * pathname, then stat the file, and check for a match by
+		 * device and inode.  That will detect the less common case
+		 * involving multiple links to the same library.
+		 */
 		for(smp = link_map_head;  smp != NULL;  smp = smp->som_next) {
 			if(!(LM_PRIVATE(smp)->spd_flags & (RTLD_MAIN|RTLD_RTLD))
 			&& smp->som_path != NULL
 			&& strcmp(smp->som_path, path) == 0)
 				break;
 		}
+		if(smp == NULL) {  /* Check for a match by device and inode */
+			if (stat(path, &statbuf) == -1) {
+				generror ("cannot stat \"%s\" : %s",
+					path, strerror(errno));
+				return NULL;
+			}
+			for (smp = link_map_head;  smp != NULL;
+			     smp = smp->som_next) {
+				struct somap_private *smpp = LM_PRIVATE(smp);
+
+				if (!(smpp->spd_flags & (RTLD_MAIN | RTLD_RTLD))
+				&& smpp->spd_ino == statbuf.st_ino
+				&& smpp->spd_dev == statbuf.st_dev)
+					break;
+			}
+		}
 	}
 
 	if (smp == NULL) {	/* We must map the object */
@@ -589,6 +747,14 @@ map_object(path, sodp, parent)
 		smpp->a_text = hdr.a_text;
 		smpp->a_data = hdr.a_data;
 		smpp->a_bss = hdr.a_bss;
+
+		/*
+		 * Save the device and inode, so we can detect multiple links
+		 * to the same library.  Note, if we reach this point, then
+		 * statbuf is guaranteed to have been filled in.
+		 */
+		smpp->spd_dev = statbuf.st_dev;
+		smpp->spd_ino = statbuf.st_ino;
 	}
 
 	LM_PRIVATE(smp)->spd_refcount++;
@@ -675,7 +841,7 @@ map_sods(parent)
 		if(sodp->sod_library) {
 			path = rtfindlib(name, sodp->sod_major,
 					 sodp->sod_minor);
-			if(path == NULL) {
+			if(path == NULL && !ld_tracing) {
 				generror ("Can't find shared library"
 					  " \"lib%s.so.%d.%d\"", name,
 					  sodp->sod_major, sodp->sod_minor);
@@ -688,28 +854,10 @@ map_sods(parent)
 				path = strdup(name);
 		}
 
-		if(path != NULL)
+		if(path != NULL) {
 			smp = map_object(path, sodp, parent);
-
-		if(tracing &&
-		(smp == NULL || !(LM_PRIVATE(smp)->spd_flags & RTLD_TRACED))) {
-			if(sodp->sod_library)
-				printf("\t-l%s.%d => ", name, sodp->sod_major);
-			else
-				printf("\t%s => ", name);
-
-			if(path == NULL)
-				printf("not found\n");
-			else if(smp == NULL)
-				printf("can't load\n");
-			else {
-				printf("%s (%p)\n", path, smp->som_addr);
-				LM_PRIVATE(smp)->spd_flags |= RTLD_TRACED;
-			}
-		}
-
-		if(path != NULL)
 			free(path);
+		}
 
 		if(smp != NULL) {
 			struct so_list	*solp = (struct so_list *)
@@ -718,7 +866,22 @@ map_sods(parent)
 			solp->sol_next = NULL;
 			*soltail = solp;
 			soltail = &solp->sol_next;
-		} else if(!tracing)
+		} else if(ld_tracing) {
+			/*
+			 * Allocate a dummy map entry so that we will get the
+			 * "not found" message.
+			 */
+			(void)alloc_link_map(NULL, sodp, parent, 0, 0);
+		} else if (ld_ignore_missing_objects) {
+			const char *msg;
+			/*
+			 * Call __dlerror() even it we're not going to use
+			 * the message, in order to clear the saved message.
+			 */
+			msg = __dlerror();  /* Should never be NULL */
+			if (!ld_suppress_warnings)
+				warnx("warning: %s", msg);
+		} else  /* Give up */
 			break;
 
 		next = sodp->sod_next;
@@ -758,8 +921,9 @@ map_sods(parent)
  * an error message can be retrieved via dlerror().
  */
 	static int
-reloc_and_init(root)
+reloc_and_init(root, bind_now)
 	struct so_map	*root;
+	int		bind_now;
 {
 	struct so_map	*smp;
 
@@ -780,7 +944,7 @@ reloc_and_init(root)
 	 */
 	for(smp = root;  smp != NULL;  smp = smp->som_next) {
 		if(!(LM_PRIVATE(smp)->spd_flags & RTLD_RTLD)) {
-			if(reloc_map(smp) < 0)
+			if(reloc_map(smp, bind_now) < 0)
 				return -1;
 		}
 	}
@@ -903,8 +1067,7 @@ caddr_t			addr;
 	else
 		sym = "";
 
-	if (getenv("LD_SUPPRESS_WARNINGS") == NULL &&
-	    getenv("LD_WARN_NON_PURE_CODE") != NULL)
+	if (!ld_suppress_warnings && ld_warn_non_pure_code)
 		warnx("warning: non pure code in %s at %x (%s)",
 				smp->som_path, r->r_address, sym);
 
@@ -922,8 +1085,9 @@ caddr_t			addr;
 }
 
 static int
-reloc_map(smp)
+reloc_map(smp, bind_now)
 	struct so_map		*smp;
+	int			bind_now;
 {
 	/*
 	 * Caching structure for reducing the number of calls to
@@ -996,7 +1160,7 @@ reloc_map(smp)
 			struct nzlist	*p, *np;
 			long	relocation;
 
-			if (RELOC_LAZY_P(r))
+			if (RELOC_JMPTAB_P(r) && !bind_now)
 				continue;
 
 			p = (struct nzlist *)
@@ -1015,7 +1179,7 @@ reloc_map(smp)
 			if(np != NULL)	/* Symbol already cached */
 				src_map = symcache[RELOC_SYMBOL(r)].src_map;
 			else {	/* Symbol not cached yet */
-				np = lookup(sym, &src_map, 0/*XXX-jumpslots!*/);
+				np = lookup(sym, &src_map, RELOC_JMPTAB_P(r));
 				/*
 				 * Record the needed information about
 				 * the symbol in the caching vector,
@@ -1040,11 +1204,17 @@ reloc_map(smp)
 			 * Otherwise it's a run-time allocated common
 			 * whose value is already up-to-date.
 			 */
-			relocation = md_get_addend(r, addr);
-			relocation += np->nz_value;
+			relocation = np->nz_value;
 			if (src_map)
 				relocation += (long)src_map->som_addr;
 
+			if (RELOC_JMPTAB_P(r)) {
+				md_bind_jmpslot(relocation, addr);
+				continue;
+			}
+
+			relocation += md_get_addend(r, addr);
+
 			if (RELOC_PCREL_P(r))
 				relocation -= (long)smp->som_addr;
 
@@ -1056,8 +1226,8 @@ reloc_map(smp)
 					np->nz_size, src_map);
 				continue;
 			}
-			md_relocate(r, relocation, addr, 0);
 
+			md_relocate(r, relocation, addr, 0);
 		} else {
 			md_relocate(r,
 #ifdef SUN_COMPAT
@@ -1435,6 +1605,7 @@ static char			*hstrtab;
 maphints __P((void))
 {
 	static int		hints_bad;	/* TRUE if hints are unusable */
+	static int		paths_added;
 	int			hfd;
 	struct hints_header	hdr;
 	caddr_t			addr;
@@ -1454,7 +1625,8 @@ maphints __P((void))
 
 	if (read(hfd, &hdr, sizeof hdr) != sizeof hdr ||
 	    HH_BADMAG(hdr) ||
-	    hdr.hh_version != LD_HINTS_VERSION_1) {
+	    (hdr.hh_version != LD_HINTS_VERSION_1 &&
+	     hdr.hh_version != LD_HINTS_VERSION_2)) {
 		close(hfd);
 		hints_bad = 1;
 		return -1;
@@ -1474,6 +1646,11 @@ maphints __P((void))
 	hheader = (struct hints_header *)addr;
 	hbuckets = (struct hints_bucket *)(addr + hheader->hh_hashtab);
 	hstrtab = (char *)(addr + hheader->hh_strtab);
+	/* pluck out the system ldconfig path */
+	if (hheader->hh_version >= LD_HINTS_VERSION_2 && !paths_added) {
+		add_search_path(hstrtab + hheader->hh_dirlist);
+		paths_added = 1;
+	}
 
 	return 0;
 }
@@ -1546,6 +1723,10 @@ findhint(name, major, minorp)
 		 */
 		if (strcmp(name, hstrtab + bp->hi_namex) == 0 &&
 		    bp->hi_major == major) {
+			struct stat s;
+
+			if (stat(hstrtab + bp->hi_pathx, &s) == -1)
+				return NULL;  /* Doesn't actually exist */
 			*minorp = bp->hi_ndewey >= 2 ? bp->hi_minor : -1;
 			return strdup(hstrtab + bp->hi_pathx);
 		}
@@ -1596,9 +1777,7 @@ rtfindlib(name, major, minor)
 	if (path == NULL)	/* Search the standard directories */
 		path = findshlib(name, &major, &realminor, 0);
 
-	if (path != NULL &&
-	    realminor < minor &&
-	    getenv("LD_SUPPRESS_WARNINGS") == NULL) {
+	if (path != NULL && realminor < minor && !ld_suppress_warnings) {
 		warnx("warning: %s: minor version %d"
 		      " older than expected %d, using it anyway",
 		      path, realminor, minor);
@@ -1661,13 +1840,18 @@ static char  dlerror_buf [DLERROR_BUF_SIZE];
 static char *dlerror_msg = NULL;
 
 
-	static void *
+static void *
 __dlopen(path, mode)
-	char	*path;
-	int	mode;
+	const char	*path;
+	int		 mode;
 {
 	struct so_map	*old_tail = link_map_tail;
 	struct so_map	*smp;
+	int		bind_now = mode == RTLD_NOW;
+
+	/*
+	 * path == NULL is handled by map_object()
+	 */
 
 	anon_open();
 
@@ -1679,7 +1863,7 @@ __dlopen(path, mode)
 
 	/* Relocate and initialize all newly-mapped objects */
 	if(link_map_tail != old_tail) {  /* We have mapped some new objects */
-		if(reloc_and_init(smp) == -1)	/* Failed */
+		if(reloc_and_init(smp, bind_now) == -1)	/* Failed */
 			return NULL;
 	}
 
@@ -1689,7 +1873,7 @@ __dlopen(path, mode)
 	return smp;
 }
 
-	static int
+static int
 __dlclose(fd)
 	void	*fd;
 {
@@ -1714,20 +1898,76 @@ __dlclose(fd)
 	return 0;
 }
 
-	static void *
+/*
+ * This form of dlsym is obsolete.  Current versions of crt0 don't call
+ * it.  It can still be called by old executables that were linked with
+ * old versions of crt0.
+ */
+static void *
 __dlsym(fd, sym)
+	void		*fd;
+	const char	*sym;
+{
+	if (fd == RTLD_NEXT) {
+		generror("RTLD_NEXT not supported by this version of"
+		    " /usr/lib/crt0.o");
+		return NULL;
+	}
+	return __dlsym3(fd, sym, NULL);
+}
+
+	static void *
+resolvesym(fd, sym, retaddr)
 	void	*fd;
 	char	*sym;
+	void	*retaddr;
 {
-	struct so_map	*src_map = (struct so_map *)fd;
+	struct so_map	*smp;
+	struct so_map	*src_map;
 	struct nzlist	*np;
 	long		addr;
 
-	np = lookup(sym, &src_map, 1);
+	if (fd == RTLD_NEXT) {
+		/* Find the shared object that contains the caller. */
+		for (smp = link_map_head;  smp != NULL;  smp = smp->som_next) {
+			void *textbase = smp->som_addr + LM_TXTADDR(smp);
+			void *textlimit = LM_ETEXT(smp);
+
+			if (textbase <= retaddr && retaddr < textlimit)
+				break;
+		}
+		if (smp == NULL) {
+			generror("Cannot determine caller's shared object");
+			return NULL;
+		}
+		smp = smp->som_next;
+		if (smp != NULL && LM_PRIVATE(smp)->spd_flags & RTLD_RTLD)
+			smp = smp->som_next;
+		if (smp == NULL) {
+			generror("No next shared object for RTLD_NEXT");
+			return NULL;
+		}
+		do {
+			src_map = smp;
+			np = lookup(sym, &src_map, 1);
+		} while (np == NULL && (smp = smp->som_next) != NULL);
+	} else {
+		smp = (struct so_map *)fd;
+		src_map = NULL;
+
+		/*
+		 * Restrict search to passed map if dlopen()ed.
+		 */
+		if (smp != NULL && LM_PRIVATE(smp)->spd_flags & RTLD_DL)
+			src_map = smp;
+
+		np = lookup(sym, &src_map, 1);
+	}
+
 	if (np == NULL) {
-		generror ("Symbol \"%s\" not found", sym);
+		generror("Undefined symbol");
 		return NULL;
-        }
+	}
 
 	addr = np->nz_value;
 	if (src_map)
@@ -1736,10 +1976,38 @@ __dlsym(fd, sym)
 	return (void *)addr;
 }
 
-	static char *
+static void *
+__dlsym3(fd, sym, retaddr)
+	void		*fd;
+	const char	*sym;
+	void		*retaddr;
+{
+	void *result;
+
+	result = resolvesym(fd, sym, retaddr);
+	/*
+	 * XXX - Ugly, but it makes the least impact on the run-time loader
+	 * sources.  We assume that most of the time the error is a
+	 * undefined symbol error from above, so we try again.  If it's
+	 * not an undefined symbol we end up getting the same error twice,
+	 * but that's acceptable.
+	 */
+	if (result == NULL) {
+		/* Prepend an underscore and try again */
+		char *newsym = malloc(strlen(sym) + 2);
+
+		newsym[0] = '_';
+		strcpy(&newsym[1], sym);
+		result = resolvesym(fd, newsym, retaddr);
+		free(newsym);
+	}
+	return result;
+}
+
+static const char *
 __dlerror __P((void))
 {
-        char *err;
+        const char	*err;
 
         err = dlerror_msg;
         dlerror_msg = NULL;  /* Next call will return NULL */
@@ -1747,7 +2015,7 @@ __dlerror __P((void))
         return err;
 }
 
-	static void
+static void
 __dlexit __P((void))
 {
 #ifdef DEBUG
@@ -1757,6 +2025,22 @@ xprintf("__dlexit called\n");
 	unmap_object(link_map_head, 1);
 }
 
+/*
+ * Print the current error message and exit with failure status.
+ */
+static void
+die __P((void))
+{
+	const char	*msg;
+
+	fprintf(stderr, "ld.so failed");
+	if ((msg = __dlerror()) != NULL)
+		fprintf(stderr, ": %s", msg);
+	putc('\n', stderr);
+	_exit(1);
+}
+
+
 /*
  * Generate an error message that can be later be retrieved via dlerror.
  */
@@ -1796,7 +2080,70 @@ char	*fmt;
 	va_start(ap);
 #endif
 
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	(void)write(1, buf, strlen(buf));
 	va_end(ap);
 }
+
+/*
+ * rt_readenv() etc.
+ *
+ * Do a sweep over the environment once only, pick up what
+ * looks interesting.
+ *
+ * This is pretty obscure, but is relatively simple.  Simply
+ * look at each environment variable, if it starts with "LD_" then
+ * look closer at it.  If it's in our table, set the variable
+ * listed.  effectively, this is like:
+ *    ld_preload = careful ? NULL : getenv("LD_PRELOAD");
+ * except that the environment is scanned once only to pick up all
+ * known variables, rather than scanned multiple times for each
+ * variable.
+ *
+ * If an environment variable of interest is set to the empty string, we
+ * treat it as if it were unset.
+ */
+
+#define L(n, u, v) { n, sizeof(n) - 1, u, v },
+struct env_scan_tab {
+	char	*name;
+	int	len;
+	int	unsafe;
+	char	**value;
+} scan_tab[] = {
+	L("LD_LIBRARY_PATH=",		1, &ld_library_path)
+	L("LD_PRELOAD=",		1, &ld_preload)
+	L("LD_IGNORE_MISSING_OBJECTS=",	1, &ld_ignore_missing_objects)
+	L("LD_TRACE_LOADED_OBJECTS=",	0, &ld_tracing)
+	L("LD_BIND_NOW=",		0, &ld_bind_now)
+	L("LD_SUPPRESS_WARNINGS=",	0, &ld_suppress_warnings)
+	L("LD_WARN_NON_PURE_CODE=",	0, &ld_warn_non_pure_code)
+	{ NULL, 0, NULL }
+};
+#undef L
+
+static void
+rt_readenv()
+{
+	char **p = environ;
+	char *v;
+	struct env_scan_tab *t;
+
+	/* for each string in the environment... */
+	while ((v = *p++)) {
+
+		/* check for LD_xxx */
+		if (v[0] != 'L' || v[1] != 'D' || v[2] != '_')
+			continue;
+
+		for (t = scan_tab; t->name; t++) {
+			if (careful && t->unsafe)
+				continue;	/* skip for set[ug]id */
+			if (strncmp(t->name, v, t->len) == 0) {
+				if (*(v + t->len) != '\0')	/* Not empty */
+					*t->value = v + t->len;
+				break;
+			}
+		}
+	}
+}
