@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/sparc/md.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/sparc/md.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
@@ -40,7 +40,7 @@
 #include <stab.h>
 #include <string.h>
 
-#include "ld.h"
+#include "dynamic.h"
 
 /*
  * Relocation masks and sizes for the Sparc architecture.
