@@ -1,5 +1,36 @@
+/*-
+ * This code is derived from software copyrighted by the Free Software
+ * Foundation.
+ *
+ * Modified 1991 by Donn Seeley at UUNET Technologies, Inc.
+ *
+ * Modified 1993 by Paul Kranenburg, Erasmus University
+ */
+
+/* Derived from ld.c: "@(#)ld.c 6.10 (Berkeley) 5/22/91"; */
+
+/* Linker `ld' for GNU
+   Copyright (C) 1988 Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 1, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+
+/* Written by Richard Stallman with some help from Eric Albert.
+   Set, indirect, and warning symbol features added by Randy Smith. */
+
 /*
- * $FreeBSD: stable/2.1/gnu/usr.bin/ld/warnings.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/gnu/usr.bin/ld/warnings.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
@@ -24,9 +55,30 @@
 #endif
 
 #include "ld.h"
+#include "dynamic.h"
 
 static int reported_undefineds;
 
+#ifdef DEMANGLE_CPLUSPLUS
+#include "demangle.h"
+
+char *demangle(name)
+     char *name;
+{
+  static char* saved_result = NULL;
+
+  if (saved_result)
+    free (saved_result);
+
+  saved_result = cplus_demangle (name[0] == '_' ? name + 1 : name, DMGL_PARAMS | DMGL_ANSI);
+
+  if (saved_result)
+    return saved_result;
+  else
+    return name;
+}
+#endif
+
 /*
  * Print the filename of ENTRY on OUTFILE (a stdio stream),
  * and then a newline.
@@ -103,7 +155,7 @@ print_symbols(outfile)
 
 	fprintf(outfile, "\nGlobal symbols:\n\n");
 	FOR_EACH_SYMBOL(i, sp) {
-		fprintf(outfile, "  %s: ", sp->name);
+		fprintf(outfile, "  %s: ", demangle(sp->name));
 		if (!(sp->flags & GS_REFERENCED))
 			fprintf(outfile, "unreferenced");
 		else if (sp->so_defined)
@@ -113,10 +165,10 @@ print_symbols(outfile)
 		else if (sp->defined == (N_UNDF|N_EXT))
 			fprintf(outfile, "common: size %#x", sp->common_size);
 		else
-			fprintf(outfile, "type %d, value %#x, size %#x",
+			fprintf(outfile, "type %d, value %#lx, size %#x",
 				sp->defined, sp->value, sp->size);
 		if (sp->alias)
-			fprintf(outfile, ", aliased to %s", sp->alias->name);
+			fprintf(outfile, ", aliased to %s", demangle(sp->alias->name));
 		fprintf(outfile, "\n");
 	} END_EACH_SYMBOL;
 
@@ -133,7 +185,7 @@ describe_file_sections(entry, outfile)
 	if (entry->flags & (E_JUST_SYMS | E_DYNAMIC))
 		fprintf(outfile, " symbols only\n");
 	else
-		fprintf(outfile, " text %x(%x), data %x(%x), bss %x(%x) hex\n",
+		fprintf(outfile, " text %x(%lx), data %x(%lx), bss %x(%lx) hex\n",
 			entry->text_start_address, entry->header.a_text,
 			entry->data_start_address, entry->header.a_data,
 			entry->bss_start_address, entry->header.a_bss);
@@ -161,7 +213,7 @@ list_file_locals (entry, outfile)
 		 * update it if necessary by this file's start address.
 		 */
 		if (!(p->n_type & (N_STAB | N_EXT)))
-			fprintf(outfile, "  %s: 0x%x\n",
+			fprintf(outfile, "  %s: 0x%lx\n",
 				entry->strings + p->n_un.n_strx, p->n_value);
 	}
 
@@ -458,7 +510,7 @@ do_relocation_warnings(entry, data_segment, outfile, nlist_bitvector)
 
 		if (!(lsp->nzlist.nz_type & N_EXT) &&
 		    !SET_ELEMENT_P(lsp->nzlist.nz_type)) {
-			warnx("internal error: `%s' N_EXT not set", g->name);
+			warnx("internal error: `%s' N_EXT not set", demangle(g->name));
 			continue;
 		}
 
@@ -502,14 +554,10 @@ do_relocation_warnings(entry, data_segment, outfile, nlist_bitvector)
 
 		/* If errfmt == 0, errmsg has already been defined.  */
 		if (errfmt != 0) {
-			char           *nm;
-
-			nm = g->name;
+			char *nm = demangle(g->name);
 			errmsg = (char *)
 				xmalloc(strlen(errfmt) + strlen(nm) + 1);
 			sprintf(errmsg, errfmt, nm, data_segment?"data":"text");
-			if (nm != g->name)
-				free(nm);
 		}
 		address_to_line(RELOC_ADDRESS(rp) + start_of_segment,
 				state_pointer);
@@ -582,7 +630,7 @@ do_file_warnings (entry, outfile)
 			continue;
 
 		if (!(np->n_type & N_EXT) && !SET_ELEMENT_P(np->n_type)) {
-			warnx("internal error: `%s' N_EXT not set", g->name);
+			warnx("internal error: `%s' N_EXT not set", demangle(g->name));
 			continue;
 		}
 
@@ -602,7 +650,7 @@ do_file_warnings (entry, outfile)
 			if (type == (N_UNDF | N_EXT)) {
 				fprintf(stderr,
 					"Undefined symbol %s referenced from %s\n",
-					g->name,
+					demangle(g->name),
 					get_file_name(entry));
 			}
 #endif
@@ -658,7 +706,7 @@ do_file_warnings (entry, outfile)
 				warnx("%s: unexpected multiple definitions "
 				      "of symbol `%s', type %#x",
 				      get_file_name(entry),
-				      g->name, np->n_type);
+				      demangle(g->name), np->n_type);
 				break;
 			}
 
@@ -682,7 +730,7 @@ do_file_warnings (entry, outfile)
 			fprintf(outfile,
 			"%s: Undefined symbol `%s' referenced (use %s ?)\n",
 				get_file_name(entry),
-				g->name,
+				demangle(g->name),
 				g->def_lsp->entry->local_sym_name);
 			continue;
 		} else if (g->warning) {
@@ -710,7 +758,7 @@ do_file_warnings (entry, outfile)
 		if (dont_allow_symbol_name)
 			fprintf(outfile, "%s", errfmt);
 		else
-			fprintf(outfile, errfmt, g->name);
+			fprintf(outfile, errfmt, demangle(g->name));
 
 		fputc('\n', outfile);
 	}
@@ -738,7 +786,7 @@ do_warnings(outfile)
 
 	if (entry_symbol && !entry_symbol->defined)
 		fprintf(outfile, "Undefined entry symbol `%s'\n",
-			entry_symbol->name);
+			demangle(entry_symbol->name));
 
 	each_file(do_file_warnings, (void *)outfile);
 
@@ -755,4 +803,3 @@ do_warnings(outfile)
 
 	return 1;
 }
-
