@@ -468,7 +468,7 @@ is_section (name)
   register char **vs;
 
   for (vs = section_list; *vs != NULL; vs++)
-    if ((strcmp (*vs, name) == NULL)
+    if ((strcmp (*vs, name) == 0)
 	|| (isdigit (name[0]) && strlen(name) == 1))
       return strdup (name);
 
@@ -557,7 +557,7 @@ convert_name (name, to_cat)
   t1 = strrchr (to_name, '/');
   if (t1 != NULL)
     {
-      *t1 = NULL;
+      *t1 = '\0';
       t2 = strrchr (to_name, '/');
       *t1 = '/';
     }
@@ -1072,9 +1072,9 @@ make_cat_file (path, man_file, cat_file, manid)
   char *roff_command;
   char command[FILENAME_MAX];
 
-      roff_command = make_roff_command (man_file);
-      if (roff_command == NULL)
-	return 0;
+  roff_command = make_roff_command (man_file);
+  if (roff_command == NULL)
+      return 0;
 
   snprintf(temp, sizeof(temp), "%s.tmpXXXXXX", cat_file);
   if ((f = mkstemp(temp)) >= 0 && (fp = fdopen(f, "w")) != NULL)
@@ -1112,7 +1112,7 @@ make_cat_file (path, man_file, cat_file, manid)
 #endif
 	if ((pp = popen(command, "r")) == NULL) {
 	  s = errno;
-	fprintf(stderr, "Failed.\n");
+	  fprintf(stderr, "Failed.\n");
 	  errno = s;
 	  perror("popen");
 #ifdef SETREUID
@@ -1121,11 +1121,11 @@ make_cat_file (path, man_file, cat_file, manid)
 	    setregid(-1, egid);
 	  }
 #endif
-	unlink(temp);
+	  unlink(temp);
 	  restore_sigs();
 	  fclose(fp);
 	  return 0;
-      }
+	}
 #ifdef SETREUID
 	if (manid) {
 	  setreuid(-1, euid);
@@ -1161,16 +1161,16 @@ make_cat_file (path, man_file, cat_file, manid)
 	unlink(temp);
       else if (rename(temp, cat_file) == -1) {
 	s = errno;
-			fprintf(stderr,
-				"\nHmm!  Can't seem to rename %s to %s, check permissions on man dir!\n",
-				temp, cat_file);
+	fprintf(stderr,
+		 "\nHmm!  Can't seem to rename %s to %s, check permissions on man dir!\n",
+		 temp, cat_file);
 	errno = s;
 	perror("rename");
-		unlink(temp);
+	unlink(temp);
 	restore_sigs();
 	fclose(fp);
-		return 0;
-	}
+	return 0;
+      }
       restore_sigs();
 
       if (fclose(fp)) {
@@ -1186,7 +1186,7 @@ make_cat_file (path, man_file, cat_file, manid)
       if (debug) {
 	fprintf(stderr, "No output, debug mode.\n");
 	return 0;
-	}
+      }
 
       fprintf(stderr, "Done.\n");
 
@@ -1251,7 +1251,7 @@ format_and_display (path, man_file, cat_file)
     {
       status = is_newer (man_file, cat_file);
       if (debug)
-	fprintf (stderr, "status from is_newer() = %d\n");
+	fprintf (stderr, "status from is_newer() = %d\n", status);
 
       if (status == 1 || status == -2)
 	{
