@@ -1,5 +1,5 @@
 char rcsid[] =
-  "$FreeBSD: stable/2.1/gnu/usr.bin/patch/patch.c 50952 1999-09-05 11:54:50Z peter $";
+	"$Header: /home/ncvs/src/gnu/usr.bin/patch/patch.c,v 1.6.6.3 1998/01/22 07:45:54 ache Exp $";
 
 /* patch - a program to apply diffs to original files
  *
@@ -9,7 +9,48 @@ char rcsid[] =
  * money off of it, or pretend that you wrote it.
  *
  * $Log: patch.c,v $
- * Revision 1.6  1995/05/30  05:02:34  rgrimes
+ * Revision 1.6.6.3  1998/01/22 07:45:54  ache
+ * MFC: PATCH_INDEX_FIRST environment variable
+ *
+ * Revision 1.14  1998/01/22 07:44:04  ache
+ * Add PATCH_INDEX_FIRST environment variable to do the same as -I option
+ *
+ * Revision 1.13  1998/01/21 15:15:39  ache
+ * Update usage line with new option
+ *
+ * Revision 1.12  1998/01/21 15:10:13  ache
+ * Add -I or --index-first option to take Index: precedence over context diff,
+ * as it was in hacked FreeBSD version
+ *
+ * Revision 1.11  1998/01/21 14:37:22  ache
+ * Resurrect patch 2.1 without FreeBSD Index: hack
+ *
+ * Revision 1.9  1997/10/23 02:44:22  ache
+ * Add (unsigned char) cast to ctype macros
+ *
+ * Revision 1.8  1997/03/17 01:44:42  jmg
+ * fix compilation warnings in patch... (with slight modification)
+ *
+ * also remove -Wall that I acidentally committed last time I was here...
+ *
+ * Submitted-by: Philippe Charnier
+ *
+ * Closes PR#2998
+ *
+ * Revision 1.7  1997/02/13 21:10:41  jmg
+ * Fix a problem with patch in that is will always default, even when the
+ * controlling terminal is closed.  Now the function ask() will return 1 when th
+ * input is known to come from a file or terminal, or it will return 0 when ther
+ * was a read error.
+ *
+ * Modified the question "Skip patch?" so that on an error from ask it will skip
+ * the patch instead of looping.
+ *
+ * Closes PR#777
+ *
+ * 2.2 candidate
+ *
+ * Revision 1.6  1995/05/30 05:02:34  rgrimes
  * Remove trailing whitespace.
  *
  * Revision 1.5  1995/01/12  22:09:40  hsu
@@ -147,6 +188,10 @@ static int reverse_flag_specified = FALSE;
 /* TRUE if -C was specified on command line.  */
 int check_patch = FALSE;
 
+/* TRUE if -I was specified on command line  */
+/* or PATCH_INDEX_FIRST env. variable is set */
+int index_first;
+
 /* Apply a set of diffs as appropriate. */
 
 int
@@ -170,6 +215,8 @@ char **argv;
 
     myuid = getuid();
 
+    index_first = getenv ("PATCH_INDEX_FIRST") != 0;
+
     /* Cons up the names of the temporary files.  */
     {
       /* Directory for temporary files.  */
@@ -299,12 +346,12 @@ char **argv;
 				reverse ? "Assuming" : "Ignoring");
 			}
 			else {
-			    ask3(
+			    (void) ask3(
 "%seversed (or previously applied) patch detected!  %s -R? [y] ",
 				reverse ? "R" : "Unr",
 				reverse ? "Assume" : "Ignore");
 			    if (*buf == 'n') {
-				ask1("Apply anyway? [n] ");
+				(void) ask1("Apply anyway? [n] ");
 				if (*buf == 'y')
 				    rev_okayed = TRUE;
 				else
@@ -417,6 +464,7 @@ char **argv;
 	set_signals(1);
     }
     my_exit(failtotal);
+    return (failtotal);
 }
 
 /* Prepare to find the next patch to do in the patch file. */
@@ -459,7 +507,7 @@ reinitialize_almost_everything()
 	fatal1("you may not change to a different patch file\n");
 }
 
-static char *shortopts = "-b:B:cCd:D:eEfF:lnNo:p::r:RsStuvV:x:";
+static char *shortopts = "-b:B:cCd:D:eEfF:IlnNo:p::r:RsStuvV:x:";
 static struct option longopts[] =
 {
   {"suffix", 1, NULL, 'b'},
@@ -472,6 +520,7 @@ static struct option longopts[] =
   {"remove-empty-files", 0, NULL, 'E'},
   {"force", 0, NULL, 'f'},
   {"fuzz", 1, NULL, 'F'},
+  {"index-first", 0, NULL, 'I'},
   {"ignore-whitespace", 0, NULL, 'l'},
   {"normal", 0, NULL, 'n'},
   {"forward", 0, NULL, 'N'},
@@ -530,7 +579,7 @@ get_some_switches()
 		break;
 	    case 'D':
 	    	do_defines = TRUE;
-		if (!isalpha(*optarg) && '_' != *optarg)
+		if (!isalpha((unsigned char)*optarg) && '_' != *optarg)
 		    fatal1("argument to -D is not an identifier\n");
 		Sprintf(if_defined, "#ifdef %s\n", optarg);
 		Sprintf(not_defined, "#ifndef %s\n", optarg);
@@ -548,6 +597,9 @@ get_some_switches()
 	    case 'F':
 		maxfuzz = atoi(optarg);
 		break;
+	    case 'I':
+		index_first = TRUE;
+		break;
 	    case 'l':
 		canonicalize = TRUE;
 		break;
@@ -613,7 +665,7 @@ Options:\n\
 		fprintf(stderr, "\
        [--strip[=strip-count]] [--normal] [--reject-file=rej-name] [--skip]\n\
        [--remove-empty-files] [--quiet] [--silent] [--unified] [--version]\n\
-       [--version-control={numbered,existing,simple}]\n");
+       [--version-control={numbered,existing,simple}] [--index-first]\n");
 		my_exit(1);
 	    }
 	}
@@ -940,12 +992,12 @@ Reg2 char *b;
 Reg3 int len;
 {
     while (len) {
-	if (isspace(*b)) {		/* whitespace (or \n) to match? */
-	    if (!isspace(*a))		/* no corresponding whitespace? */
+	if (isspace((unsigned char)*b)) {              /* whitespace (or \n) to match? */
+	    if (!isspace((unsigned char)*a))           /* no corresponding whitespace? */
 		return FALSE;
-	    while (len && isspace(*b) && *b != '\n')
+	    while (len && isspace((unsigned char)*b) && *b != '\n')
 		b++,len--;		/* skip pattern whitespace */
-	    while (isspace(*a) && *a != '\n')
+	    while (isspace((unsigned char)*a) && *a != '\n')
 		a++;			/* skip target whitespace */
 	    if (*a == '\n' || *b == '\n')
 		return (*a == *b);	/* should end in sync */
