@@ -1,13 +1,98 @@
-/* $FreeBSD: stable/2.1/gnu/usr.bin/patch/pch.c 50952 1999-09-05 11:54:50Z peter $
+/* $Header: /home/ncvs/src/gnu/usr.bin/patch/pch.c,v 1.8.2.3 1998/01/21 19:01:54 ache Exp $
  *
  * $Log: pch.c,v $
- * Revision 1.5.4.2  1996/04/11  10:33:45  markm
- * Merge with HEAD an improved Index: priority fix.
+ * Revision 1.8.2.3  1998/01/21 19:01:54  ache
+ * MFC: -I, --index-first option to emulate FreeBSD hack
  *
- * Revision 1.5.4.1  1995/09/17 06:39:59  davidg
- * Brought in changes from rev 1.6: Give 'Index:' priority over other paths.
+ * Revision 1.14  1998/01/21 15:10:14  ache
+ * Add -I or --index-first option to take Index: precedence over context diff,
+ * as it was in hacked FreeBSD version
  *
- * Revision 1.6  1995/09/14  04:33:35  gibbs
+ * Revision 1.13  1998/01/21 14:37:23  ache
+ * Resurrect patch 2.1 without FreeBSD Index: hack
+ *
+ * Revision 1.8.2.2  1998/01/03 23:52:05  ache
+ * MFC: ctype and Index over +++/--- precedence
+ *
+ * Revision 1.11  1998/01/03 23:42:56  ache
+ * Back out Index over +++/--- precedence.
+ * It maybe right, if patch was FreeBSD-own program, but it break compatibility
+ * with pre-existent patches in other systems.
+ * The example is big ncurses patch which don't apply on FreeBSD
+ * due to "fixed" precedence.
+ *
+ * Revision 1.10  1997/10/23 02:44:22  ache
+ * Add (unsigned char) cast to ctype macros
+ *
+ * Revision 1.9  1997/02/13 21:10:43  jmg
+ * Fix a problem with patch in that is will always default, even when the
+ * controlling terminal is closed.  Now the function ask() will return 1 when th
+ * input is known to come from a file or terminal, or it will return 0 when ther
+ * was a read error.
+ *
+ * Modified the question "Skip patch?" so that on an error from ask it will skip
+ * the patch instead of looping.
+ *
+ * Closes PR#777
+ *
+ * 2.2 candidate
+ *
+ * Revision 1.8  1996/04/12 11:37:32  markm
+ * Attempt to break a $Log: pch.c,v $
+ * Attempt to break a Revision 1.8.2.3  1998/01/21 19:01:54  ache
+ * Attempt to break a MFC: -I, --index-first option to emulate FreeBSD hack
+ * Attempt to break a
+ * Attempt to break a Revision 1.14  1998/01/21 15:10:14  ache
+ * Attempt to break a Add -I or --index-first option to take Index: precedence over context diff,
+ * Attempt to break a as it was in hacked FreeBSD version
+ * Attempt to break a
+ * Attempt to break a Revision 1.13  1998/01/21 14:37:23  ache
+ * Attempt to break a Resurrect patch 2.1 without FreeBSD Index: hack
+ * Attempt to break a
+ * Attempt to break a Revision 1.8.2.2  1998/01/03 23:52:05  ache
+ * Attempt to break a MFC: ctype and Index over +++/--- precedence
+ * Attempt to break a
+ * Attempt to break a Revision 1.11  1998/01/03 23:42:56  ache
+ * Attempt to break a Back out Index over +++/--- precedence.
+ * Attempt to break a It maybe right, if patch was FreeBSD-own program, but it break compatibility
+ * Attempt to break a with pre-existent patches in other systems.
+ * Attempt to break a The example is big ncurses patch which don't apply on FreeBSD
+ * Attempt to break a due to "fixed" precedence.
+ * Attempt to break a
+ * Attempt to break a Revision 1.10  1997/10/23 02:44:22  ache
+ * Attempt to break a Add (unsigned char) cast to ctype macros
+ * Attempt to break a
+ * Attempt to break a Revision 1.9  1997/02/13 21:10:43  jmg
+ * Attempt to break a Fix a problem with patch in that is will always default, even when the
+ * Attempt to break a controlling terminal is closed.  Now the function ask() will return 1 when th
+ * Attempt to break a input is known to come from a file or terminal, or it will return 0 when ther
+ * Attempt to break a was a read error.
+ * Attempt to break a
+ * Attempt to break a Modified the question "Skip patch?" so that on an error from ask it will skip
+ * Attempt to break a the patch instead of looping.
+ * Attempt to break a
+ * Attempt to break a Closes PR#777
+ * Attempt to break a
+ * Attempt to break a 2.2 candidate
+ * Attempt to break a snafu where a *** /--- (minus space)
+ * was fouling up a comment in the checked-out code.
+ *
+ * Revision 1.7  1996/04/11 10:13:40  markm
+ * Priorities were broken. If there was an Index: line and *** /--- lines
+ * with valid names, the *** /---names were taken first.
+ * this broke eg:
+ * Index: foo/Makefile
+ * ==========
+ * RCS <blah>
+ * Retrieving <blah>
+ * diff <blah>
+ * *** Makefile <blah>
+ * --- Makefile <blah>
+ *
+ * By trying to patch the Makefile in the _curent_ directory, rather than
+ * the one in the foo/ directory.
+ *
+ * Revision 1.6  1995/09/14 04:33:35  gibbs
  * Give "Index" specified filenames preference over other filenames specified
  * in the diff.  This makes it so that diffs containing files in different
  * subdirectories that have the same name not patch the same file.  For example
@@ -210,7 +295,7 @@ there_is_another_patch()
 	    skip_rest_of_patch = TRUE;
 	    return TRUE;
 	}
-	ask1("File to patch: ");
+	(void) ask1("File to patch: ");
 	if (*buf != '\n') {
 	    if (bestguess)
 		free(bestguess);
@@ -218,9 +303,10 @@ there_is_another_patch()
 	    filearg[0] = fetchname(buf, 0, FALSE);
 	}
 	if (filearg[0] == Nullch) {
-	    ask1("No file found--skip this patch? [n] ");
-	    if (*buf != 'y') {
-		continue;
+	    if (ask1("No file found--skip this patch? [n] ")) {
+	    	if (*buf != 'y') {
+		    continue;
+	    	}
 	    }
 	    if (verbose)
 		say1("Skipping patch...\n");
@@ -256,6 +342,7 @@ intuit_diff_type()
     char *newname = Nullch;
     Reg11 int retval;
     bool no_filearg = (filearg[0] == Nullch);
+    extern int index_first;
 
     ok_to_create_file = FALSE;
     Fseek(pfp, p_base, 0);
@@ -288,8 +375,8 @@ intuit_diff_type()
 	    else
 		indent++;
 	}
-	for (t=s; isdigit(*t) || *t == ','; t++) ;
-	this_is_a_command = (isdigit(*s) &&
+	for (t=s; isdigit((unsigned char)*t) || *t == ','; t++) ;
+	this_is_a_command = (isdigit((unsigned char)*s) &&
 	  (*t == 'd' || *t == 'c' || *t == 'a') );
 	if (first_command_line < 0L && this_is_a_command) {
 	    first_command_line = this_line;
@@ -305,9 +392,9 @@ intuit_diff_type()
 	else if (strnEQ(s, "Index:", 6))
 	    indtmp = savestr(s+6);
 	else if (strnEQ(s, "Prereq:", 7)) {
-	    for (t=s+7; isspace(*t); t++) ;
+	    for (t=s+7; isspace((unsigned char)*t); t++) ;
 	    revision = savestr(t);
-	    for (t=revision; *t && !isspace(*t); t++) ;
+	    for (t=revision; *t && !isspace((unsigned char)*t); t++) ;
 	    *t = '\0';
 	    if (!*revision) {
 		free(revision);
@@ -365,15 +452,14 @@ intuit_diff_type()
 	    oldname = fetchname(oldtmp, strippath, ok_to_create_file);
 	if (newtmp != Nullch)
 	    newname = fetchname(newtmp, strippath, ok_to_create_file);
-	if (indname)
+	if (index_first && indname)
 	    filearg[0] = savestr(indname);
 	else if (oldname && newname) {
 	    if (strlen(oldname) < strlen(newname))
 		filearg[0] = savestr(oldname);
 	    else
 		filearg[0] = savestr(newname);
-	}
-	else if (indname)
+	} else if (indname)
 	    filearg[0] = savestr(indname);
 	else if (oldname)
 	    filearg[0] = savestr(oldname);
@@ -555,15 +641,15 @@ another_hunk()
 		    p_end--;
 		    return FALSE;
 		}
-		for (s=buf; *s && !isdigit(*s); s++) ;
+		for (s=buf; *s && !isdigit((unsigned char)*s); s++) ;
 		if (!*s)
 		    malformed ();
 		if (strnEQ(s,"0,0",3))
 		    strcpy(s,s+2);
 		p_first = (LINENUM) atol(s);
-		while (isdigit(*s)) s++;
+		while (isdigit((unsigned char)*s)) s++;
 		if (*s == ',') {
-		    for (; *s && !isdigit(*s); s++) ;
+		    for (; *s && !isdigit((unsigned char)*s); s++) ;
 		    if (!*s)
 			malformed ();
 		    p_ptrn_lines = ((LINENUM)atol(s)) - p_first + 1;
@@ -621,13 +707,13 @@ another_hunk()
 			return FALSE;
 		    }
 		    p_Char[p_end] = '=';
-		    for (s=buf; *s && !isdigit(*s); s++) ;
+		    for (s=buf; *s && !isdigit((unsigned char)*s); s++) ;
 		    if (!*s)
 			malformed ();
 		    p_newfirst = (LINENUM) atol(s);
-		    while (isdigit(*s)) s++;
+		    while (isdigit((unsigned char)*s)) s++;
 		    if (*s == ',') {
-			for (; *s && !isdigit(*s); s++) ;
+			for (; *s && !isdigit((unsigned char)*s); s++) ;
 			if (!*s)
 			    malformed ();
 			p_repl_lines = ((LINENUM)atol(s)) - p_newfirst + 1;
@@ -655,7 +741,7 @@ another_hunk()
 	      change_line:
 		if (buf[1] == '\n' && canonicalize)
 		    strcpy(buf+1," \n");
-		if (!isspace(buf[1]) && buf[1] != '>' && buf[1] != '<' &&
+		if (!isspace((unsigned char)buf[1]) && buf[1] != '>' && buf[1] != '<' &&
 		  repl_beginning && repl_could_be_missing) {
 		    repl_missing = TRUE;
 		    goto hunk_done;
@@ -691,7 +777,7 @@ another_hunk()
 		}
 		break;
 	    case ' ':
-		if (!isspace(buf[1]) &&
+		if (!isspace((unsigned char)buf[1]) &&
 		  repl_beginning && repl_could_be_missing) {
 		    repl_missing = TRUE;
 		    goto hunk_done;
@@ -815,20 +901,20 @@ another_hunk()
 	if (!*s)
 	    malformed ();
 	p_first = (LINENUM) atol(s);
-	while (isdigit(*s)) s++;
+	while (isdigit((unsigned char)*s)) s++;
 	if (*s == ',') {
 	    p_ptrn_lines = (LINENUM) atol(++s);
-	    while (isdigit(*s)) s++;
+	    while (isdigit((unsigned char)*s)) s++;
 	} else
 	    p_ptrn_lines = 1;
 	if (*s == ' ') s++;
 	if (*s != '+' || !*++s)
 	    malformed ();
 	p_newfirst = (LINENUM) atol(s);
-	while (isdigit(*s)) s++;
+	while (isdigit((unsigned char)*s)) s++;
 	if (*s == ',') {
 	    p_repl_lines = (LINENUM) atol(++s);
-	    while (isdigit(*s)) s++;
+	    while (isdigit((unsigned char)*s)) s++;
 	} else
 	    p_repl_lines = 1;
 	if (*s == ' ') s++;
@@ -950,15 +1036,15 @@ another_hunk()
 	p_context = 0;
 	ret = pgets(buf, sizeof buf, pfp);
 	p_input_line++;
-	if (ret == Nullch || !isdigit(*buf)) {
+	if (ret == Nullch || !isdigit((unsigned char)*buf)) {
 	    next_intuit_at(line_beginning,p_input_line);
 	    return FALSE;
 	}
 	p_first = (LINENUM)atol(buf);
-	for (s=buf; isdigit(*s); s++) ;
+	for (s=buf; isdigit((unsigned char)*s); s++) ;
 	if (*s == ',') {
 	    p_ptrn_lines = (LINENUM)atol(++s) - p_first + 1;
-	    while (isdigit(*s)) s++;
+	    while (isdigit((unsigned char)*s)) s++;
 	}
 	else
 	    p_ptrn_lines = (*s != 'a');
@@ -966,7 +1052,7 @@ another_hunk()
 	if (hunk_type == 'a')
 	    p_first++;			/* do append rather than insert */
 	min = (LINENUM)atol(++s);
-	for (; isdigit(*s); s++) ;
+	for (; isdigit((unsigned char)*s); s++) ;
 	if (*s == ',')
 	    max = (LINENUM)atol(++s);
 	else
@@ -1300,8 +1386,8 @@ do_ed_script()
 	    break;
 	}
 	p_input_line++;
-	for (t=buf; isdigit(*t) || *t == ','; t++) ;
-	this_line_is_command = (isdigit(*buf) &&
+	for (t=buf; isdigit((unsigned char)*t) || *t == ','; t++) ;
+	this_line_is_command = (isdigit((unsigned char)*buf) &&
 	  (*t == 'd' || *t == 'c' || *t == 'a') );
 	if (this_line_is_command) {
 	    if (!skip_rest_of_patch)
