@@ -1,4 +1,4 @@
-char rcsid[] = "$RCSfile: perl.c,v $$Revision: 1.4.4.1 $$Date: 1996/06/08 20:18:56 $\nPatch level: ###\n";
+char rcsid[] = "$RCSfile: perl.c,v $$Revision: 1.7.2.2 $$Date: 1998/03/12 19:27:47 $\nPatch level: ###\n";
 /*
  *    Copyright (c) 1991, Larry Wall
  *
@@ -6,8 +6,19 @@ char rcsid[] = "$RCSfile: perl.c,v $$Revision: 1.4.4.1 $$Date: 1996/06/08 20:18:
  *    License or the Artistic License, as specified in the README file.
  *
  * $Log: perl.c,v $
- * Revision 1.4.4.1  1996/06/08 20:18:56  pst
- * Bring in changes from head
+ * Revision 1.7.2.2  1998/03/12 19:27:47  guido
+ * MFC: mktemp -> mkstemp
+ *
+ * Revision 1.7.2.1  1998/02/15 16:30:09  jkh
+ * MFC: security tweak, support for -T and -B flags.
+ *
+ * Revision 1.7  1996/06/30 09:47:56  joerg
+ * Back out Nate's changes from rev. 1.6; our Perl has not been
+ * vulnerable since it used setreuid() as opposed to Posix saved IDs.
+ * The change broke setuid scripts.
+ *
+ * Revision 1.5  1996/06/02 19:59:24  gpalmer
+ * Use setreuid instead of seteuid for permissions management
  *
  * Revision 1.4  1995/05/30 05:03:10  rgrimes
  * Remove trailing whitespace.
@@ -202,13 +213,17 @@ setuid perl scripts securely.\n");
 		fatal("No -e allowed in setuid scripts");
 #endif
 	    if (!e_fp) {
+		int fd;
+
 	        e_tmpname = savestr(TMPPATH);
-		(void)mktemp(e_tmpname);
-		if (!*e_tmpname)
-		    fatal("Can't mktemp()");
-		e_fp = fopen(e_tmpname,"w");
-		if (!e_fp)
+		fd = mkstemp(e_tmpname);
+		if (fd == -1)
+		    fatal("Can't mkstemp()");
+		e_fp = fdopen(fd,"w");
+		if (!e_fp) {
+		    close(fd);
 		    fatal("Cannot open temporary file");
+		}
 	    }
 	    if (argv[1]) {
 		fputs(argv[1],e_fp);
@@ -496,6 +511,17 @@ sed %s -e \"/^[^#]/b\" \
 #ifdef DOSUID
     if (fstat(fileno(rsfp),&statbuf) < 0)	/* normal stat is insecure */
 	fatal("Can't stat script \"%s\"",origfilename);
+#ifdef IAMSUID
+    {
+	struct statfs stfs;
+
+	if (fstatfs(fileno(rsfp),&stfs) < 0)
+	    fatal("Can't statfs filesystem of script \"%s\"",origfilename);
+
+	if (stfs.f_flags & MNT_NOSUID)
+	    fatal("Permission denied");
+    }
+#endif /* IAMSUID */
     if (statbuf.st_mode & (S_ISUID|S_ISGID)) {
 	int len;
 
