@@ -1,4 +1,4 @@
-/* $RCSfile: perl.h,v $$Revision: 1.1.1.1 $$Date: 1994/09/10 06:27:35 $
+/* $RCSfile: perl.h,v $$Revision: 1.2.6.1 $$Date: 1998/02/15 16:30:09 $
  *
  *    Copyright (c) 1991, Larry Wall
  *
@@ -6,6 +6,12 @@
  *    License or the Artistic License, as specified in the README file.
  *
  * $Log: perl.h,v $
+ * Revision 1.2.6.1  1998/02/15 16:30:09  jkh
+ * MFC: security tweak, support for -T and -B flags.
+ *
+ * Revision 1.2  1995/05/30 05:03:11  rgrimes
+ * Remove trailing whitespace.
+ *
  * Revision 1.1.1.1  1994/09/10  06:27:35  gclarkii
  * Initial import of Perl 4.046 bmaked
  *
@@ -251,6 +257,8 @@ typedef unsigned int MEM_SIZE;
 #endif
 #endif
 
+#include <sys/mount.h>
+
 #ifdef I_TIME
 #   include <time.h>
 #endif
