@@ -1,4 +1,4 @@
-/* $RCSfile: tdoio.c,v $$Revision: 1.1.1.1 $$Date: 1994/09/10 06:27:36 $
+/* $RCSfile: tdoio.c,v $$Revision: 1.2.6.1 $$Date: 1998/02/15 16:30:09 $
  *
  *    Copyright (c) 1991, Larry Wall
  *
@@ -6,6 +6,12 @@
  *    License or the Artistic License, as specified in the README file.
  *
  * $Log: tdoio.c,v $
+ * Revision 1.2.6.1  1998/02/15 16:30:09  jkh
+ * MFC: security tweak, support for -T and -B flags.
+ *
+ * Revision 1.2  1995/05/30 05:03:23  rgrimes
+ * Remove trailing whitespace.
+ *
  * Revision 1.1.1.1  1994/09/10  06:27:36  gclarkii
  * Initial import of Perl 4.046 bmaked
  *
@@ -1213,8 +1219,26 @@ STR *str;
 		return &str_yes;
 	    len = stio->ifp->_cnt + (stio->ifp->_ptr - stio->ifp->_base);
 	    s = stio->ifp->_base;
+#else
+#if (defined(BSD) && (BSD >= 199103))
+	    s = stio->ifp->_base;
+	    fstat(fileno(stio->ifp),&statcache);
+	    if (S_ISDIR(statcache.st_mode))     /* handle NFS glitch */
+		return arg->arg_type == O_FTTEXT ? &str_no : &str_yes;
+
+	    if (stio->ifp->_bf._size  <=  0) {
+		i = getc(stio->ifp);
+		if (i != EOF)
+		    (void)ungetc(i,stio->ifp);
+	    }
+
+	    if (stio->ifp->_bf._size  <=  0)
+		return &str_yes;
+	    len = stio->ifp->_bf._size+(stio->ifp->_p - stio->ifp->_bf._base);
+	    s = stio->ifp->_bf._base;
 #else
 	    fatal("-T and -B not implemented on filehandles");
+#endif
 #endif
 	}
 	else {
