@@ -1,4 +1,4 @@
-/* $RCSfile: util.h,v $$Revision: 1.1.1.1 $$Date: 1994/09/10 06:27:55 $
+/* $RCSfile: util.h,v $$Revision: 1.2.6.1 $$Date: 1997/02/23 11:46:32 $
  *
  *    Copyright (c) 1991, Larry Wall
  *
@@ -6,6 +6,12 @@
  *    License or the Artistic License, as specified in the README file.
  *
  * $Log: util.h,v $
+ * Revision 1.2.6.1  1997/02/23 11:46:32  joerg
+ * YAMFC (rev 1.3: remove bogus redeclaration of getenv())
+ *
+ * Revision 1.2  1995/05/30 05:03:46  rgrimes
+ * Remove trailing whitespace.
+ *
  * Revision 1.1.1.1  1994/09/10  06:27:55  gclarkii
  * Initial import of Perl 4.046 bmaked
  *
@@ -48,7 +54,6 @@ void	prexit();
 char	*get_a_line();
 char	*savestr();
 int	makedir();
-void	setenv();
 int	envix();
 void	notincl();
 char	*getval();
