@@ -178,7 +178,7 @@ static void cleanup P((void));
 
 static char const quietarg[] = "-q";
 
-static char const *expandarg, *suffixarg, *versionarg, *zonearg, *incexcarg;
+static char const *expandarg, *suffixarg, *versionarg, *zonearg;
 static char const **joinlist; /* revisions to be joined */
 static int joinlength;
 static FILE *neworkptr;
@@ -191,7 +191,7 @@ static struct hshentries *gendeltas;	/* deltas to be generated	*/
 static struct hshentry *targetdelta;	/* final delta to be generated	*/
 static struct stat workstat;
 
-mainProg(coId, "co", "$FreeBSD: stable/2.1/gnu/usr.bin/rcs/co/co.c 50952 1999-09-05 11:54:50Z peter $")
+mainProg(coId, "co", "$FreeBSD: stable/2.2/gnu/usr.bin/rcs/co/co.c 50952 1999-09-05 11:54:50Z peter $")
 {
 	static char const cmdusage[] =
 		"\nco usage: co -{fIlMpqru}[rev] -ddate -jjoins -ksubst -sstate -T -w[who] -Vn -xsuff -zzone file ...";
@@ -314,11 +314,6 @@ mainProg(coId, "co", "$FreeBSD: stable/2.1/gnu/usr.bin/rcs/co/co.c 50952 1999-09
 			zone_set(a);
 			break;
 
-		case 'K':    /*  set keyword inclusions/exclusions  */
-			incexcarg = *argv;
-			setIncExc(incexcarg);
-			break;
-
 		case 'k':    /*  set keyword expand mode  */
 			expandarg = *argv;
 			if (0 <= expmode) redefined('k');
