@@ -186,7 +186,7 @@ Report problems and direct all questions to:
 
 #include "rcsbase.h"
 
-libId(utilId, "$FreeBSD: stable/2.1/gnu/usr.bin/rcs/lib/rcsutil.c 50952 1999-09-05 11:54:50Z peter $")
+libId(utilId, "$FreeBSD: stable/2.2/gnu/usr.bin/rcs/lib/rcsutil.c 50952 1999-09-05 11:54:50Z peter $")
 
 #if !has_memcmp
 	int
@@ -1230,8 +1230,15 @@ getRCSINIT(argc, argv, newargv)
 	char **argv, ***newargv;
 {
 	register char *p, *q, **pp;
+	char const *ev;
 	size_t n;
 
+	if ((ev = cgetenv("RCSLOCALID")))
+		setRCSLocalId(ev);
+
+	if ((ev = cgetenv("RCSINCEXC")))
+		setIncExc(ev);
+
 	if (!(q = cgetenv("RCSINIT")))
 		*newargv = argv;
 	else {
