@@ -2,7 +2,7 @@
 
 # rcsfreeze - assign a symbolic revision number to a configuration of RCS files
 
-# $FreeBSD: stable/2.1/gnu/usr.bin/rcs/rcsfreeze/rcsfreeze.sh 50952 1999-09-05 11:54:50Z peter $
+# $FreeBSD: stable/2.2/gnu/usr.bin/rcs/rcsfreeze/rcsfreeze.sh 50952 1999-09-05 11:54:50Z peter $
 
 #       The idea is to run rcsfreeze each time a new version is checked
 #       in. A unique symbolic revision number (C_[number], where number
@@ -25,10 +25,10 @@
 #       {RCS/}.rcsfreeze.ver	version number
 #       {RCS/}.rscfreeze.log	log messages, most recent first
 
-PATH=/usr/local/bin:/bin:/usr/bin:/usr/ucb:$PATH
+PATH=/bin:/usr/bin:$PATH
 export PATH
 
-DATE=`date` || exit
+DATE=`LC_TIME=C date` || exit
 # Check whether we have an RCS subdirectory, so we can have the right
 # prefix for our paths.
 if test -d RCS
