@@ -158,7 +158,10 @@ extract_archive ()
   userec (head);		/* And go past it in the archive */
   decode_header (head, &hstat, &head_standard, 1);	/* Snarf fields */
 
-  if (f_confirm && !confirm ("extract", current_file_name))
+  if ((f_confirm && !confirm ("extract", current_file_name)) ||
+      (f_exstdout && head->header.linkflag != LF_OLDNORMAL &&
+       head->header.linkflag != LF_NORMAL &&
+       head->header.linkflag != LF_CONTIG))
     {
       if (head->header.isextended)
 	skip_extended_headers ();
@@ -550,7 +553,7 @@ extract_archive ()
 	  if (chmod (skipcrud + current_file_name,
 		     notumask & (int) hstat.st_mode) < 0)
 	    {
-	      msg_perror ("cannot change mode of file %s to %ld",
+	      msg_perror ("cannot change mode of file %s to 0%o",
 			  skipcrud + current_file_name,
 			  notumask & (int) hstat.st_mode);
 	    }
@@ -755,7 +758,7 @@ extract_archive ()
 	      if (chmod (skipcrud + current_file_name,
 			 notumask & (int) hstat.st_mode) < 0)
 		{
-		  msg_perror ("cannot change mode of file %s to %ld",
+		  msg_perror ("cannot change mode of file %s to 0%o",
 			      skipcrud + current_file_name,
 			      notumask & (int) hstat.st_mode);
 		}
@@ -933,7 +936,7 @@ restore_saved_dir_info ()
 	  if (chmod (saved_dir_info_head->path,
 		     notumask & saved_dir_info_head->mode) < 0)
 	    {
-	      msg_perror ("cannot change mode of file %s to %ld",
+	      msg_perror ("cannot change mode of file %s to 0%o",
 			  saved_dir_info_head->path,
 			  notumask & saved_dir_info_head->mode);
 	    }
