@@ -1,4 +1,4 @@
-/* $FreeBSD: stable/2.1/include/nl_types.h 50952 1999-09-05 11:54:50Z peter $ */
+/* $FreeBSD: stable/2.2/include/nl_types.h 50952 1999-09-05 11:54:50Z peter $ */
 
 /***********************************************************
 Copyright 1990, by Alfalfa Software Incorporated, Cambridge, Massachusetts.
@@ -36,13 +36,14 @@ up-to-date.  Many thanks.
 #define _NL_TYPES_H_
 #include <sys/cdefs.h>
 
-#define	NL_SETD	0
+#define	NL_SETD		0
+#define	NL_CAT_LOCALE	1
 
-typedef	long	nl_catd;
-typedef long	nl_item;
+typedef	int	nl_item;
+typedef	void	*nl_catd;
 
 extern nl_catd 	catopen __P((__const char *, int));
-extern char    *catgets __P((nl_catd, int, int, char *));
+extern char    *catgets	__P((nl_catd, int, int,	__const	char *));
 extern int	catclose __P((nl_catd));
 
 #endif	/* _NL_TYPES_H_ */
