@@ -34,6 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)stdio.h	8.4 (Berkeley) 1/4/94
+ * $FreeBSD: stable/2.2/include/stdio.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef	_STDIO_H_
@@ -67,7 +68,7 @@ typedef	_BSD_SIZE_T_	size_t;
  * boundaries.  THIS IS A CROCK, but for now there is no way around it.
  */
 #if !defined(_ANSI_SOURCE) && !defined(__STRICT_ANSI__)
-typedef off_t fpos_t;
+typedef	_BSD_OFF_T_	fpos_t;
 #else
 typedef struct __sfpos {
 	char	_pos[8];
@@ -162,8 +163,8 @@ __END_DECLS
 #define	__SMBF	0x0080		/* _buf is from malloc */
 #define	__SAPP	0x0100		/* fdopen()ed in append mode */
 #define	__SSTR	0x0200		/* this is an sprintf/snprintf string */
-#define	__SOPT	0x0400		/* do fseek() optimisation */
-#define	__SNPT	0x0800		/* do not do fseek() optimisation */
+#define	__SOPT	0x0400		/* do fseek() optimization */
+#define	__SNPT	0x0800		/* do not do fseek() optimization */
 #define	__SOFF	0x1000		/* set iff _offset is in fact correct */
 #define	__SMOD	0x2000		/* true => fgetln modified _p text */
 
@@ -224,7 +225,7 @@ int	 ferror __P((FILE *));
 int	 fflush __P((FILE *));
 int	 fgetc __P((FILE *));
 int	 fgetpos __P((FILE *, fpos_t *));
-char	*fgets __P((char *, size_t, FILE *));
+char	*fgets __P((char *, int, FILE *));
 FILE	*fopen __P((const char *, const char *));
 int	 fprintf __P((FILE *, const char *, ...));
 int	 fputc __P((int, FILE *));
@@ -234,7 +235,7 @@ FILE	*freopen __P((const char *, const char *, FILE *));
 int	 fscanf __P((FILE *, const char *, ...));
 int	 fseek __P((FILE *, long, int));
 int	 fsetpos __P((FILE *, const fpos_t *));
-long	 ftell __P((const FILE *));
+long	 ftell __P((FILE *));
 size_t	 fwrite __P((const void *, size_t, size_t, FILE *));
 int	 getc __P((FILE *));
 int	 getchar __P((void));
@@ -268,16 +269,45 @@ __END_DECLS
  * Functions defined in POSIX 1003.1.
  */
 #ifndef _ANSI_SOURCE
-#define	L_cuserid	9	/* size for cuserid(); UT_NAMESIZE + 1 */
-#define	L_ctermid	1024	/* size for ctermid(); PATH_MAX */
+/* size for cuserid(3); UT_NAMESIZE + 1, see <utmp.h> */
+#define	L_cuserid	9
+
+#define	L_ctermid	1024	/* size for ctermid(3); PATH_MAX */
 
 __BEGIN_DECLS
 char	*ctermid __P((char *));
 FILE	*fdopen __P((int, const char *));
 int	 fileno __P((FILE *));
+int	 ftrylockfile __P((FILE *));
+void	 flockfile __P((FILE *));
+void	 funlockfile __P((FILE *));
 __END_DECLS
 #endif /* not ANSI */
 
+/*
+ * Portability hacks.  See <sys/types.h>.
+ */
+#if !defined (_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
+__BEGIN_DECLS
+#ifndef _FTRUNCATE_DECLARED
+#define	_FTRUNCATE_DECLARED
+int	 ftruncate __P((int, _BSD_OFF_T_));
+#endif
+#ifndef _LSEEK_DECLARED
+#define	_LSEEK_DECLARED
+_BSD_OFF_T_ lseek __P((int, _BSD_OFF_T_, int));
+#endif
+#ifndef _MMAP_DECLARED
+#define	_MMAP_DECLARED
+char	*mmap __P((char *, size_t, int, int, int, _BSD_OFF_T_));
+#endif
+#ifndef _TRUNCATE_DECLARED
+#define	_TRUNCATE_DECLARED
+int	 truncate __P((const char *, _BSD_OFF_T_));
+#endif
+__END_DECLS
+#endif /* !_ANSI_SOURCE && !_POSIX_SOURCE */
+
 /*
  * Routines that are purely local.
  */
@@ -298,7 +328,6 @@ int	 vasprintf __P((char **, const char *, _BSD_VA_LIST_));
 int	 vsnprintf __P((char *, size_t, const char *, _BSD_VA_LIST_));
 int	 vscanf __P((const char *, _BSD_VA_LIST_));
 int	 vsscanf __P((const char *, const char *, _BSD_VA_LIST_));
-FILE	*zopen __P((const char *, const char *, int));
 __END_DECLS
 
 /*
@@ -362,19 +391,73 @@ static __inline int __sputc(int _c, FILE *_p) {
 #define	__sclearerr(p)	((void)((p)->_flags &= ~(__SERR|__SEOF)))
 #define	__sfileno(p)	((p)->_file)
 
-#define	feof(p)		__sfeof(p)
-#define	ferror(p)	__sferror(p)
-#define	clearerr(p)	__sclearerr(p)
+/*
+ * See ISO/IEC 9945-1 ANSI/IEEE Std 1003.1 Second Edition 1996-07-12
+ * B.8.2.7 for the rationale behind the *_unlocked() macros.
+ */
+#define	feof_unlocked(p)	__sfeof(p)
+#define	ferror_unlocked(p)	__sferror(p)
+#define	clearerr_unlocked(p)	__sclearerr(p)
 
 #ifndef _ANSI_SOURCE
-#define	fileno(p)	__sfileno(p)
+#define	fileno_unlocked(p)	__sfileno(p)
+#endif
+
+#ifndef  _THREAD_SAFE
+#define	feof(p)		feof_unlocked(p)
+#define	ferror(p)	ferror_unlocked(p)
+#define	clearerr(p)	clearerr_unlocked(p)
+
+#ifndef _ANSI_SOURCE
+#define	fileno(p)	fileno_unlocked(p)
+#endif
 #endif
 
 #ifndef lint
-#define	getc(fp)	__sgetc(fp)
-#define putc(x, fp)	__sputc(x, fp)
+#define	getc_unlocked(fp)	__sgetc(fp)
+#define putc_unlocked(x, fp)	__sputc(x, fp)
+#ifdef	_THREAD_SAFE
+void	_flockfile_debug __P((FILE *, char *, int));
+#ifdef	_FLOCK_DEBUG
+#define _FLOCKFILE(x)	_flockfile_debug(x, __FILE__, __LINE__)
+#else
+#define _FLOCKFILE(x)	flockfile(x)
+#endif
+static __inline int			\
+__getc_locked(FILE *_fp)		\
+{					\
+	extern int __isthreaded;	\
+	int _ret;			\
+	if (__isthreaded)		\
+		_FLOCKFILE(_fp);	\
+	_ret = getc_unlocked(_fp);	\
+	if (__isthreaded)		\
+		funlockfile(_fp);	\
+	return (_ret);			\
+}
+static __inline int			\
+__putc_locked(int _x, FILE *_fp)	\
+{					\
+	extern int __isthreaded;	\
+	int _ret;			\
+	if (__isthreaded)		\
+		_FLOCKFILE(_fp);	\
+	_ret = putc_unlocked(_x, _fp);	\
+	if (__isthreaded)		\
+		funlockfile(_fp);	\
+	return (_ret);			\
+}
+#define	getc(fp)	__getc_locked(fp)
+#define	putc(x, fp)	__putc_locked(x, fp)
+#else
+#define	getc(fp)	getc_unlocked(fp)
+#define putc(x, fp)	putc_unlocked(x, fp)
+#endif
 #endif /* lint */
 
-#define	getchar()	getc(stdin)
-#define	putchar(x)	putc(x, stdout)
-#endif /* _STDIO_H_ */
+#define	getchar()		getc(stdin)
+#define	getchar_unlocked()	getc_unlocked(stdin)
+#define	putchar(x)		putc(x, stdout)
+#define	putchar_unlocked(x)	putc_unlocked(x, stdout)
+
+#endif /* !_STDIO_H_ */
