@@ -47,7 +47,7 @@ static char sccsid[] = "@(#)getcwd.c	8.1 (Berkeley) 6/4/93";
 
 #define	ISDOT(dp) \
 	(dp->d_name[0] == '.' && (dp->d_name[1] == '\0' || \
-	    dp->d_name[1] == '.' && dp->d_name[2] == '\0'))
+	    (dp->d_name[1] == '.' && dp->d_name[2] == '\0')))
 
 char *
 getcwd(pt, size)
@@ -55,7 +55,7 @@ getcwd(pt, size)
 	size_t size;
 {
 	register struct dirent *dp;
-	register DIR *dir;
+	register DIR *dir = NULL;
 	register dev_t dev;
 	register ino_t ino;
 	register int first;
@@ -94,7 +94,7 @@ getcwd(pt, size)
 	/*
 	 * Allocate bytes (1024 - malloc space) for the string of "../"'s.
 	 * Should always be enough (it's 340 levels).  If it's not, allocate
-	 * as necessary.  Special * case the first stat, it's ".", not "..".
+	 * as necessary.  Special case the first stat, it's ".", not "..".
 	 */
 	if ((up = malloc(upsize = 1024 - 4)) == NULL)
 		goto err;
@@ -154,6 +154,7 @@ getcwd(pt, size)
 
 		/* Add trailing slash for next directory. */
 		*bup++ = '/';
+		*bup = '\0';
 
 		/*
 		 * If it's a mount point, have to stat each element because
@@ -211,7 +212,8 @@ getcwd(pt, size)
 			*--bpt = '/';
 		bpt -= dp->d_namlen;
 		bcopy(dp->d_name, bpt, dp->d_namlen);
-		(void)closedir(dir);
+		(void) closedir(dir);
+		dir = NULL;
 
 		/* Truncate any file name. */
 		*bup = '\0';
@@ -227,8 +229,14 @@ notfound:
 		errno = save_errno ? save_errno : ENOENT;
 	/* FALLTHROUGH */
 err:
+	save_errno = errno;
+
 	if (ptsize)
 		free(pt);
+	if (dir)
+		(void) closedir(dir);
 	free(up);
+
+	errno = save_errno;
 	return (NULL);
 }
