@@ -36,13 +36,15 @@
 static char sccsid[] = "From: @(#)gethostname.c	8.1 (Berkeley) 6/4/93";
 */
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/lib/libc/gen/getdomainname.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/lib/libc/gen/getdomainname.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
 #include <sys/sysctl.h>
 
-long
+#include <unistd.h>
+
+int
 getdomainname(name, namelen)
 	char *name;
 	int namelen;
@@ -51,7 +53,7 @@ getdomainname(name, namelen)
 	size_t size;
 
 	mib[0] = CTL_KERN;
-	mib[1] = KERN_DOMAINNAME;
+	mib[1] = KERN_NISDOMAINNAME;
 	size = namelen;
 	if (sysctl(mib, 2, name, &size, NULL, 0) == -1)
 		return (-1);
