@@ -40,6 +40,7 @@ static char sccsid[] = "@(#)getgrent.c	8.2 (Berkeley) 3/21/94";
 #include <stdlib.h>
 #include <string.h>
 #include <grp.h>
+#include <syslog.h>
 
 static FILE *_gr_fp;
 static struct group _gr_group;
@@ -60,6 +61,8 @@ static char *members[MAXGRP];
 #define	MAXLINELENGTH	1024
 static char line[MAXLINELENGTH];
 
+#define GROUP_IGNORE_COMMENTS	1	/* allow comments in /etc/group */
+
 struct group *
 getgrent()
 {
@@ -163,7 +166,7 @@ start_gr()
 		char *line;
 		size_t linelen;
 		_gr_yp_enabled = 0;
-		while(line = fgetln(_gr_fp, &linelen)) {
+		while((line = fgetln(_gr_fp, &linelen)) != NULL) {
 			if(line[0] == '+') {
 				if(line[1] && line[1] != ':' && !_gr_yp_enabled) {
 					_gr_yp_enabled = 1;
@@ -235,8 +238,20 @@ grscan(search, gid, name)
 				;
 			continue;
 		}
-		if ((_gr_group.gr_name = strsep(&bp, ":\n")) == NULL)
-			break;
+
+#ifdef GROUP_IGNORE_COMMENTS
+		for (cp = line; *cp != '\0'; cp++)
+			if (*cp != ' ' && *cp != '\t')
+				break;
+		if (*cp == '#' || *cp == '\0')
+			continue;
+#endif
+
+		if ((_gr_group.gr_name = strsep(&bp, ":\n")) == NULL) {
+			/* this may be benign - eg. empty line at end of file */
+		/*	syslog(LOG_ALERT, "/etc/group is corrupt: group field is empty"); */
+			continue;
+		}
 #ifdef YP
 		/*
 		 * XXX   We need to be careful to avoid proceeding
@@ -281,18 +296,24 @@ grscan(search, gid, name)
 		if ((cp = strsep(&bp, ":\n")) == NULL)
 			if (_ypfound)
 				return(1);
-			else
-				break;
+			else {
+				syslog(LOG_ALERT, "/etc/group corrupt: invalid field");
+				continue;
+			}
 		if (strlen(cp) || !_ypfound)
 			_gr_group.gr_passwd = cp;
 #else
-		if ((_gr_group.gr_passwd = strsep(&bp, ":\n")) == NULL)
-			break;
+		if ((_gr_group.gr_passwd = strsep(&bp, ":\n")) == NULL) {
+			syslog(LOG_ALERT, "/etc/group corrupt: invalid field");
+			continue;
+		}
 #endif
 		if (!(cp = strsep(&bp, ":\n")))
+#ifdef YP
 			if (_ypfound)
 				return(1);
 			else
+#endif
 				continue;
 #ifdef YP
 		/*
@@ -305,11 +326,12 @@ grscan(search, gid, name)
 		if (search && name == NULL && _gr_group.gr_gid != gid)
 			continue;
 		cp = NULL;
-		if (bp == NULL) /* !!! Must check for this! */
-			break;
+		if (bp == NULL) {  /* !!! Must check for this! */
+			continue;
+		}
 #ifdef YP
 		if ((cp = strsep(&bp, ":\n")) == NULL)
-			break;
+			continue;
 
 		if (!strlen(cp) && _ypfound)
 			return(1);
@@ -340,6 +362,7 @@ grscan(search, gid, name)
 		return(1);
 	}
 	/* NOTREACHED */
+	return (0);
 }
 
 #ifdef YP
@@ -398,7 +421,7 @@ static int
 _getypgroup(struct group *gr, const char *name, char *map)
 {
 	char *result, *s;
-	static char resultbuf[1024];
+	static char resultbuf[YPMAXRECORD + 2];
 	int resultlen;
 
 	if(!_gr_yp_domain) {
@@ -415,6 +438,7 @@ _getypgroup(struct group *gr, const char *name, char *map)
 
 	if(resultlen >= sizeof resultbuf) return 0;
 	strncpy(resultbuf, result, resultlen);
+	resultbuf[resultlen] = '\0';
 	free(result);
 	return(_gr_breakout_yp(gr, resultbuf));
 
@@ -427,7 +451,7 @@ _nextypgroup(struct group *gr)
 	static char *key;
 	static int keylen;
 	char *lastkey, *result;
-	static char resultbuf[1024];
+	static char resultbuf[YPMAXRECORD + 2];
 	int resultlen;
 	int rv;
 
@@ -462,9 +486,11 @@ unpack:
 			goto tryagain;
 		}
 
-		strcpy(resultbuf, result);
+		strncpy(resultbuf, result, resultlen);
+		resultbuf[resultlen] = '\0';
 		free(result);
-		if(result = strchr(resultbuf, '\n')) *result = '\0';
+		if((result = strchr(resultbuf, '\n')) != NULL)
+			*result = '\0';
 		if (_gr_breakout_yp(gr, resultbuf))
 			return(1);
 		else
