@@ -49,6 +49,10 @@ static char sccsid[] = "@(#)getpwent.c	8.1 (Berkeley) 6/4/93";
 #include <limits.h>
 #include <grp.h>
 
+extern void setnetgrent __P(( char * ));
+extern int getnetgrent __P(( char **, char **, char ** ));
+extern int innetgr __P(( const char *, const char *, const char *, const char * ));
+
 static struct passwd _pw_passwd;	/* password structure */
 static DB *_pw_db;			/* password database */
 static int _pw_keynum;			/* key counter */
@@ -134,7 +138,9 @@ getpwnam(name)
 
 	bf[0] = _PW_KEYBYNAME;
 	len = strlen(name);
-	bcopy(name, bf + 1, MIN(len, UT_NAMESIZE));
+	if (len > UT_NAMESIZE)
+		return(NULL);
+	bcopy(name, bf + 1, len);
 	key.data = (u_char *)bf;
 	key.size = len + 1;
 	rval = __hashpw(&key);
@@ -237,7 +243,7 @@ endpwent()
 #endif
 }
 
-static
+static int
 __initdb()
 {
 	static int warned;
@@ -273,7 +279,7 @@ __initdb()
 	return(0);
 }
 
-static
+static int
 __hashpw(key)
 	DBT *key;
 {
@@ -289,7 +295,7 @@ __hashpw(key)
 		return(0);
 
 	t = line;
-#define	EXPAND(e)	e = t; while (*t++ = *p++);
+#define	EXPAND(e)	e = t; while ( (*t++ = *p++) );
 	EXPAND(_pw_passwd.pw_name);
 	EXPAND(_pw_passwd.pw_passwd);
 	bcopy(p, (char *)&_pw_passwd.pw_uid, sizeof(int));
@@ -548,7 +554,7 @@ again:
 }
 	
 static int
-_pw_breakout_yp(struct passwd *pw, char *res, int master)
+_pw_breakout_yp(struct passwd *pw, char *res, int resultlen, int master)
 {
 	char *s, *result;
 	static char resbuf[YPMAXRECORD+2];
@@ -568,7 +574,8 @@ _pw_breakout_yp(struct passwd *pw, char *res, int master)
 	 * a static buffer here since the memory pointed to by
 	 * res will be free()ed when this function returns.
 	 */
-	strncpy((char *)&resbuf, res, YPMAXRECORD);
+	strncpy((char *)&resbuf, res, resultlen);
+	resbuf[resultlen] = '\0';
 	result = (char *)&resbuf;
 
 	/*
@@ -700,7 +707,7 @@ _getyppass(struct passwd *pw, const char *name, const char *map)
 		*s = ':'; /* Put back the colon we previously replaced with a NUL. */
 	}
 
-	rv = _pw_breakout_yp(pw, result, _gotmaster);
+	rv = _pw_breakout_yp(pw, result, resultlen, _gotmaster);
 	free(result);
 	return(rv);
 }
@@ -760,7 +767,7 @@ unpack:
 		}
 
 		*s = ':'; /* Put back the colon we previously replaced with a NUL. */
-		if (_pw_breakout_yp(pw, result, _gotmaster)) {
+		if (_pw_breakout_yp(pw, result, resultlen, _gotmaster)) {
 			free(result);
 			return(1);
 		} else {
