@@ -35,7 +35,7 @@
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
-static char sccsid[] = "@(#)popen.c	8.1 (Berkeley) 6/4/93";
+static char sccsid[] = "@(#)popen.c	8.3 (Berkeley) 5/3/95";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
@@ -49,6 +49,8 @@ static char sccsid[] = "@(#)popen.c	8.1 (Berkeley) 6/4/93";
 #include <string.h>
 #include <paths.h>
 
+extern char **environ;
+
 static struct pid {
 	struct pid *next;
 	FILE *fp;
@@ -56,25 +58,41 @@ static struct pid {
 } *pidlist;
 
 FILE *
-popen(program, type)
-	const char *program;
-	const char *type;
+popen(command, type)
+	const char *command, *type;
 {
 	struct pid *cur;
 	FILE *iop;
-	int pdes[2], pid;
+	int pdes[2], pid, twoway;
+	char *argv[4];
+	struct pid *p;
 
-	if (*type != 'r' && *type != 'w' || type[1])
+	/*
+	 * Lite2 introduced two-way popen() pipes using socketpair().
+	 * FreeBSD's pipe() is bidirectional, so we use that.
+	 */
+	if (strchr(type, '+')) {
+		twoway = 1;
+		type = "r+";
+	} else  {
+		twoway = 0;
+		if ((*type != 'r' && *type != 'w') || type[1])
+			return (NULL);
+	}
+	if (pipe(pdes) < 0)
 		return (NULL);
 
-	if ((cur = malloc(sizeof(struct pid))) == NULL)
-		return (NULL);
-
-	if (pipe(pdes) < 0) {
-		free(cur);
+	if ((cur = malloc(sizeof(struct pid))) == NULL) {
+		(void)close(pdes[0]);
+		(void)close(pdes[1]);
 		return (NULL);
 	}
 
+	argv[0] = "sh";
+	argv[1] = "-c";
+	argv[2] = (char *) command;
+	argv[3] = NULL;
+
 	switch (pid = vfork()) {
 	case -1:			/* Error. */
 		(void)close(pdes[0]);
@@ -84,11 +102,22 @@ popen(program, type)
 		/* NOTREACHED */
 	case 0:				/* Child. */
 		if (*type == 'r') {
+			/*
+			 * The dup2() to STDIN_FILENO is repeated to avoid
+			 * writing to pdes[1], which might corrupt the
+			 * parent's copy.  This isn't good enough in
+			 * general, since the _exit() is no return, so
+			 * the compiler is free to corrupt all the local
+			 * variables.
+			 */
+			(void) close(pdes[0]);
 			if (pdes[1] != STDOUT_FILENO) {
 				(void)dup2(pdes[1], STDOUT_FILENO);
 				(void)close(pdes[1]);
-			}
-			(void) close(pdes[0]);
+				if (twoway)
+					(void)dup2(STDOUT_FILENO, STDIN_FILENO);
+			} else if (twoway && (pdes[1] != STDIN_FILENO))
+				(void)dup2(pdes[1], STDIN_FILENO);
 		} else {
 			if (pdes[0] != STDIN_FILENO) {
 				(void)dup2(pdes[0], STDIN_FILENO);
@@ -96,7 +125,10 @@ popen(program, type)
 			}
 			(void)close(pdes[1]);
 		}
-		execl(_PATH_BSHELL, "sh", "-c", program, NULL);
+		for (p = pidlist; p; p = p->next) {
+			(void)close(fileno(p->fp));
+		}
+		execve(_PATH_BSHELL, argv, environ);
 		_exit(127);
 		/* NOTREACHED */
 	}
@@ -130,11 +162,9 @@ pclose(iop)
 {
 	register struct pid *cur, *last;
 	int omask;
-	union wait pstat;
+	int pstat;
 	pid_t pid;
 
-	(void)fclose(iop);
-
 	/* Find the appropriate file pointer. */
 	for (last = NULL, cur = pidlist; cur; last = cur, cur = cur->next)
 		if (cur->fp == iop)
@@ -142,12 +172,11 @@ pclose(iop)
 	if (cur == NULL)
 		return (-1);
 
-	/* Get the status of the process. */
-	omask = sigblock(sigmask(SIGINT)|sigmask(SIGQUIT)|sigmask(SIGHUP));
+	(void)fclose(iop);
+
 	do {
-		pid = waitpid(cur->pid, (int *) &pstat, 0);
+		pid = waitpid(cur->pid, &pstat, 0);
 	} while (pid == -1 && errno == EINTR);
-	(void)sigsetmask(omask);
 
 	/* Remove the entry from the linked list. */
 	if (last == NULL)
@@ -156,5 +185,5 @@ pclose(iop)
 		last->next = cur->next;
 	free(cur);
 
-	return (pid == -1 ? -1 : pstat.w_status);
+	return (pid == -1 ? -1 : pstat);
 }
