@@ -36,26 +36,21 @@
 static char sccsid[] = "From: @(#)sethostname.c	8.1 (Berkeley) 6/4/93";
 */
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/lib/libc/gen/setdomainname.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/lib/libc/gen/setdomainname.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
 #include <sys/sysctl.h>
 
-#if __STDC__
-long
+#include <unistd.h>
+
+int
 setdomainname(const char *name, int namelen)
-#else
-long
-setdomainname(name, namelen)
-	char *name;
-	int namelen;
-#endif
 {
 	int mib[2];
 
 	mib[0] = CTL_KERN;
-	mib[1] = KERN_DOMAINNAME;
+	mib[1] = KERN_NISDOMAINNAME;
 	if (sysctl(mib, 2, NULL, NULL, (void *)name, namelen) == -1)
 		return (-1);
 	return (0);
