@@ -32,13 +32,18 @@
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
-static char sccsid[] = "@(#)syslog.c	8.4 (Berkeley) 3/18/94";
+/*
+static char sccsid[] = "@(#)syslog.c	8.5 (Berkeley) 4/29/95";
+*/
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/lib/libc/gen/syslog.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/syslog.h>
 #include <sys/uio.h>
+#include <sys/un.h>
 #include <netdb.h>
 
 #include <errno.h>
@@ -57,12 +62,16 @@ static char sccsid[] = "@(#)syslog.c	8.4 (Berkeley) 3/18/94";
 
 static int	LogFile = -1;		/* fd for log */
 static int	connected;		/* have done connect */
+static int	opened;			/* have done openlog() */
 static int	LogStat = 0;		/* status bits, set by openlog() */
 static const char *LogTag = NULL;	/* string to tag the entry with */
 static int	LogFacility = LOG_USER;	/* default facility code */
 static int	LogMask = 0xff;		/* mask of priorities to be logged */
 extern char	*__progname;		/* Program name, from crt0. */
 
+static void	disconnectlog __P((void)); /* disconnect from syslogd */
+static void	connectlog __P((void));	/* (re)connect to syslogd */
+
 /*
  * Format of the magic cookie passed through the stdio hook
  */
@@ -76,7 +85,8 @@ struct bufcookie {
  * XXX: Maybe one day, dynamically allocate it so that the line length
  *      is `unlimited'.
  */
-static writehook(cookie, buf, len)
+static
+int writehook(cookie, buf, len)
 	void	*cookie;	/* really [struct bufcookie *] */
 	char	*buf;		/* characters to copy */
 	int	len;		/* length to copy */
@@ -128,7 +138,7 @@ vsyslog(pri, fmt, ap)
 	va_list ap;
 {
 	register int cnt;
-	register char ch, *p, *t;
+	register char ch, *p;
 	time_t now;
 	int fd, saved_errno;
 	char *stdp, tbuf[2048], fmt_cpy[1024];
@@ -193,7 +203,7 @@ vsyslog(pri, fmt, ap)
 		}
 
 		/* Substitute error message for %m. */
-		for ( ; ch = *fmt; ++fmt)
+		for ( ; (ch = *fmt); ++fmt)
 			if (ch == '%' && fmt[1] == 'm') {
 				++fmt;
 				fputs(strerror(saved_errno), fmt_fp);
@@ -229,8 +239,18 @@ vsyslog(pri, fmt, ap)
 	}
 
 	/* Get connected, output the message to the local logger. */
-	if (!connected)
+	if (!opened)
 		openlog(LogTag, LogStat | LOG_NDELAY, 0);
+	connectlog();
+	if (send(LogFile, tbuf, cnt, 0) >= 0)
+		return;
+
+	/*
+	 * If the send() failed, the odds are syslogd was restarted.
+	 * Make one (only) attempt to reconnect to /dev/log.
+	 */
+	disconnectlog();
+	connectlog();
 	if (send(LogFile, tbuf, cnt, 0) >= 0)
 		return;
 
@@ -254,8 +274,57 @@ vsyslog(pri, fmt, ap)
 		(void)close(fd);
 	}
 }
+static void
+disconnectlog()
+{
+	/*
+	 * If the user closed the FD and opened another in the same slot,
+	 * that's their problem.  They should close it before calling on
+	 * system services.
+	 */
+	if (LogFile != -1) {
+		close(LogFile);
+		LogFile = -1;
+	}
+	connected = 0;			/* retry connect */
+}
 
-static struct sockaddr SyslogAddr;	/* AF_UNIX address of local logger */
+static void
+connectlog()
+{
+	struct sockaddr_un SyslogAddr;	/* AF_UNIX address of local logger */
+
+	if (LogFile == -1) {
+		if ((LogFile = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1)
+			return;
+		(void)fcntl(LogFile, F_SETFD, 1);
+	}
+	if (LogFile != -1 && !connected) {
+		SyslogAddr.sun_len = sizeof(SyslogAddr);
+		SyslogAddr.sun_family = AF_UNIX;
+		(void)strncpy(SyslogAddr.sun_path, _PATH_LOG,
+		    sizeof SyslogAddr.sun_path);
+		connected = connect(LogFile, (struct sockaddr *)&SyslogAddr,
+			sizeof(SyslogAddr)) != -1;
+
+		if (!connected) {
+			/*
+			 * Try the old "/dev/log" path, for backward
+			 * compatibility.
+			 */
+			(void)strncpy(SyslogAddr.sun_path, _PATH_OLDLOG,
+			    sizeof SyslogAddr.sun_path);
+			connected = connect(LogFile,
+				(struct sockaddr *)&SyslogAddr,
+				sizeof(SyslogAddr)) != -1;
+		}
+
+		if (!connected) {
+			(void)close(LogFile);
+			LogFile = -1;
+		}
+	}
+}
 
 void
 openlog(ident, logstat, logfac)
@@ -268,22 +337,10 @@ openlog(ident, logstat, logfac)
 	if (logfac != 0 && (logfac &~ LOG_FACMASK) == 0)
 		LogFacility = logfac;
 
-	if (LogFile == -1) {
-		SyslogAddr.sa_family = AF_UNIX;
-		(void)strncpy(SyslogAddr.sa_data, _PATH_LOG,
-		    sizeof(SyslogAddr.sa_data));
-		if (LogStat & LOG_NDELAY) {
-			if ((LogFile = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1)
-				return;
-			(void)fcntl(LogFile, F_SETFD, 1);
-		}
-	}
-	if (LogFile != -1 && !connected)
-		if (connect(LogFile, &SyslogAddr, sizeof(SyslogAddr)) == -1) {
-			(void)close(LogFile);
-			LogFile = -1;
-		} else
-			connected = 1;
+	if (LogStat & LOG_NDELAY)	/* open immediately */
+		connectlog();
+
+	opened = 1;	/* ident and facility has been set */
 }
 
 void
