@@ -35,39 +35,52 @@
  *
  *	from: @(#)SYS.h	5.5 (Berkeley) 5/7/91
  *
- * $FreeBSD: stable/2.1/lib/libc/i386/SYS.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/lib/libc/i386/SYS.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/syscall.h>
 #include "DEFS.h"
 
-#ifdef PIC
-#define PIC_PROLOGUE    \
-        pushl   %ebx;   \
-        call    1f;     \
-1:                      \
-        popl    %ebx;   \
-        addl    $_GLOBAL_OFFSET_TABLE_+[.-1b], %ebx
-#define PIC_EPILOGUE    \
-        popl    %ebx
-#define PIC_PLT(x)      x@PLT
-#define PIC_GOT(x)      x@GOT(%ebx)
-#define PIC_GOTOFF(x)   x@GOTOFF(%ebx)
-#else
-#define PIC_PROLOGUE
-#define PIC_EPILOGUE
-#define PIC_PLT(x)      x
-#define PIC_GOT(x)      x
-#define PIC_GOTOFF(x)   x
-#endif
-
-#define	SYSCALL(x)	2: jmp cerror; ENTRY(x); lea SYS_/**/x,%eax; LCALL(7,0); jb 2b
+#define	SYSCALL(x)	2: PIC_PROLOGUE; jmp PIC_PLT(HIDENAME(cerror)); \
+			ENTRY(x); lea __CONCAT(SYS_,x),%eax; KERNCALL; jb 2b
 #define	RSYSCALL(x)	SYSCALL(x); ret
-#define	PSEUDO(x,y)	ENTRY(x); lea SYS_/**/y, %eax; ; LCALL(7,0); ret
-#define	CALL(x,y)	call _/**/y; addl $4*x,%esp
-/* gas fucks up offset -- although we don't currently need it, do for BCS */
+
+#define	PSEUDO(x,y)	ENTRY(x); lea __CONCAT(SYS_,y), %eax; KERNCALL; ret
+/* gas messes up offset -- although we don't currently need it, do for BCS */
 #define	LCALL(x,y)	.byte 0x9a ; .long y; .word x
 
-#define	ASMSTR		.asciz
+/*
+ * Design note:
+ *
+ * The macros PSYSCALL() and PRSYSCALL() are intended for use where a
+ * syscall needs to be renamed in the threaded library. When building
+ * a normal library, they default to the traditional SYSCALL() and
+ * RSYSCALL(). This avoids the need to #ifdef _THREAD_SAFE everywhere
+ * that the renamed function needs to be called.
+ */
+#ifdef _THREAD_SAFE
+/*
+ * For the thread_safe versions, we prepend _thread_sys_ to the function
+ * name so that the 'C' wrapper can go around the real name.
+ */
+#define	PSYSCALL(x)	2: PIC_PROLOGUE; jmp PIC_PLT(HIDENAME(cerror)); \
+			ENTRY(__CONCAT(_thread_sys_,x)); \
+			lea __CONCAT(SYS_,x),%eax; KERNCALL; jb 2b
+#define	PRSYSCALL(x)	PSYSCALL(x); ret
+#define	PPSEUDO(x,y)	ENTRY(__CONCAT(_thread_sys_,x)); \
+			lea __CONCAT(SYS_,y), %eax; KERNCALL; ret
+#else
+/*
+ * The non-threaded library defaults to traditional syscalls where
+ * the function name matches the syscall name.
+ */
+#define	PSYSCALL(x)	SYSCALL(x)
+#define	PRSYSCALL(x)	RSYSCALL(x)
+#define	PPSEUDO(x,y)	PSEUDO(x,y)
+#endif
 
-	.globl	cerror
+#ifdef __ELF__
+#define KERNCALL	int $0x80	/* Faster */
+#else
+#define KERNCALL	LCALL(7,0)	/* The old way */
+#endif
