@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/lib/libc/locale/collate.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/lib/libc/locale/collate.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef COLLATE_H_INCLUDED
@@ -36,24 +36,20 @@
 
 #define STR_LEN 10
 #define TABLE_SIZE 100
+#define COLLATE_VERSION "1.0\n"
 
 struct __collate_st_char_pri {
 	int prim, sec;
 };
-struct __collate_st_name_pri {
-	u_char str[STR_LEN];
-	int prim, sec;
-};
 struct __collate_st_chain_pri {
 	u_char str[STR_LEN];
 	int prim, sec;
 };
 
 extern int __collate_load_error;
-extern u_char __collate_charmap_table[UCHAR_MAX + 1][STR_LEN];
+extern char __collate_version[STR_LEN];
 extern u_char __collate_substitute_table[UCHAR_MAX + 1][STR_LEN];
 extern struct __collate_st_char_pri __collate_char_pri_table[UCHAR_MAX + 1];
-extern struct __collate_st_name_pri __collate_name_pri_table[TABLE_SIZE];
 extern struct __collate_st_chain_pri __collate_chain_pri_table[TABLE_SIZE];
 
 __BEGIN_DECLS
@@ -61,6 +57,7 @@ u_char *__collate_strdup __P((u_char *));
 u_char *__collate_substitute __P((const u_char *));
 int __collate_load_tables __P((char *));
 void __collate_lookup __P((u_char *, int *, int *, int *));
+int  __collate_range_cmp __P((int, int));
 #ifdef COLLATE_DEBUG
 void __collate_print_tables __P((void));
 #endif
