@@ -31,14 +31,27 @@
  * SUCH DAMAGE.
  */
 
+/* Portions Copyright (c) 1993 Carlos Leandro and Rui Salgueiro
+ *	Dep. Matematica Universidade de Coimbra, Portugal, Europe
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * from getnetent.c	1.1 (Coimbra) 93/06/02
+ */
+
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)getnetent.c	8.1 (Berkeley) 6/4/93";
+static char orig_rcsid[] = "From: Id: getnetent.c,v 8.4 1997/06/01 20:34:37 vixie Exp";
+static chat rcsid[] = "$FreeBSD: stable/2.2/lib/libc/net/getnetbyht.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <arpa/nameser.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <string.h>
@@ -55,6 +68,7 @@ void
 _setnethtent(f)
 	int f;
 {
+
 	if (netf == NULL)
 		netf = fopen(_PATH_NETWORKS, "r" );
 	else
@@ -65,6 +79,7 @@ _setnethtent(f)
 void
 _endnethtent()
 {
+
 	if (netf) {
 		fclose(netf);
 		netf = NULL;
@@ -81,7 +96,7 @@ getnetent()
 	if (netf == NULL && (netf = fopen(_PATH_NETWORKS, "r" )) == NULL)
 		return (NULL);
 again:
-	p = fgets(line, BUFSIZ, netf);
+	p = fgets(line, sizeof line, netf);
 	if (p == NULL)
 		return (NULL);
 	if (*p == '#')
@@ -103,7 +118,7 @@ again:
 	net.n_net = inet_network(cp);
 	net.n_addrtype = AF_INET;
 	q = net.n_aliases = net_aliases;
-	if (p != NULL)
+	if (p != NULL) 
 		cp = p;
 	while (cp && *cp) {
 		if (*cp == ' ' || *cp == '\t') {
@@ -128,7 +143,7 @@ _getnetbyhtname(name)
 	register char **cp;
 
 	setnetent(_net_stayopen);
-	while ((p = getnetent())) {
+	while ( (p = getnetent()) ) {
 		if (strcasecmp(p->n_name, name) == 0)
 			break;
 		for (cp = p->n_aliases; *cp != 0; cp++)
@@ -141,16 +156,15 @@ found:
 	return (p);
 }
 
-
 struct netent *
 _getnetbyhtaddr(net, type)
-	register long net;
+	register unsigned long net;
 	register int type;
 {
 	register struct netent *p;
 
 	setnetent(_net_stayopen);
-	while ((p = getnetent()))
+	while ( (p = getnetent()) )
 		if (p->n_addrtype == type && p->n_net == net)
 			break;
 	if (!_net_stayopen)
