@@ -1,4 +1,4 @@
-/* $FreeBSD: stable/2.1/lib/libc/nls/catgets.c 50952 1999-09-05 11:54:50Z peter $ */
+/* $FreeBSD: stable/2.2/lib/libc/nls/catgets.c 50952 1999-09-05 11:54:50Z peter $ */
 
 /*
  * Written by J.T. Conklin, 10/05/94
@@ -13,14 +13,14 @@ __indr_reference(_catgets,catgets);
 
 #include <nl_types.h>
 
-extern char * _catgets __P((nl_catd, int, int, char *));
+extern char * _catgets __P((nl_catd, int, int, __const char *));
 
 char *
 catgets(catd, set_id, msg_id, s)
 	nl_catd catd;
 	int set_id;
 	int msg_id;
-	char *s;
+	__const char *s;
 {
 	return _catgets(catd, set_id, msg_id, s);
 }
