@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)auth_none.c 1.19 87/08/11 Copyr 1984 Sun Micro";*/
 /*static char *sccsid = "from: @(#)auth_none.c	2.1 88/07/29 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/auth_none.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/rpc/auth_none.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -41,11 +41,14 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/auth_none.c 50952 1999-0
  * Copyright (C) 1984, Sun Microsystems, Inc.
  */
 
+#include <stdlib.h>
 #include <rpc/types.h>
 #include <rpc/xdr.h>
 #include <rpc/auth.h>
 #define MAX_MARSHEL_SIZE 20
 
+bool_t xdr_opaque_auth(XDR *xdrs, struct opaque_auth *ap);
+
 /*
  * Authenticator operations routines
  */
