@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)clnt_generic.c 1.4 87/08/11 (C) 1987 SMI";*/
 /*static char *sccsid = "from: @(#)clnt_generic.c	2.2 88/08/01 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/clnt_generic.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/rpc/clnt_generic.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -40,6 +40,7 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/clnt_generic.c 50952 199
 #include <sys/socket.h>
 #include <sys/errno.h>
 #include <netdb.h>
+#include <string.h>
 
 /*
  * Generic client creation: takes (hostname, program-number, protocol) and
@@ -73,6 +74,8 @@ clnt_create(hostname, prog, vers, proto)
 		rpc_createerr.cf_error.re_errno = EAFNOSUPPORT;
 		return (NULL);
 	}
+	memset(&sin, 0, sizeof(sin));
+	sin.sin_len = sizeof(struct sockaddr_in);
 	sin.sin_family = h->h_addrtype;
 	sin.sin_port = 0;
 	bzero(sin.sin_zero, sizeof(sin.sin_zero));
