@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)clnt_udp.c 1.39 87/08/11 Copyr 1984 Sun Micro";*/
 /*static char *sccsid = "from: @(#)clnt_udp.c	2.2 88/08/01 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/clnt_udp.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/rpc/clnt_udp.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -40,6 +40,9 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/clnt_udp.c 50952 1999-09
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <rpc/rpc.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -47,7 +50,9 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/clnt_udp.c 50952 1999-09
 #include <errno.h>
 #include <rpc/pmap_clnt.h>
 
-extern int errno;
+int bindresvport(int sd, struct sockaddr_in *);
+int _rpc_dtablesize(void);
+bool_t xdr_opaque_auth(XDR *, struct opaque_auth *);
 
 /*
  * UDP bases client side rpc operations
@@ -288,8 +293,8 @@ send_again:
 #endif /* def FD_SETSIZE */
 	for (;;) {
 		readfds = mask;
-		switch (select(_rpc_dtablesize(), &readfds, (int *)NULL,
-			       (int *)NULL, &(cu->cu_wait))) {
+		switch (select(_rpc_dtablesize(), &readfds, (fd_set *)NULL,
+			       (fd_set *)NULL, &(cu->cu_wait))) {
 
 		case 0:
 			time_waited.tv_sec += cu->cu_wait.tv_sec;
