@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)pmap_getmaps.c 1.10 87/08/11 Copyr 1984 Sun Micro";*/
 /*static char *sccsid = "from: @(#)pmap_getmaps.c	2.2 88/08/01 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/pmap_getmaps.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/rpc/pmap_getmaps.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -47,14 +47,13 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/pmap_getmaps.c 50952 199
 #include <sys/socket.h>
 #include <netdb.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <errno.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
 #define NAMELEN 255
 #define MAX_BROADCAST_SIZE 1400
 
-extern int errno;
-
 /*
  * Get a copy of the current port maps.
  * Calls the pmap service remotely to do get the maps.
