@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)rpc_callmsg.c 1.4 87/08/11 Copyr 1984 Sun Micro";*/
 /*static char *sccsid = "from: @(#)rpc_callmsg.c	2.1 88/07/29 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/rpc_callmsg.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/rpc/rpc_callmsg.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -41,9 +41,12 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/rpc/rpc_callmsg.c 50952 1999
  */
 
 #include <sys/param.h>
-
+#include <stdlib.h>
+#include <string.h>
 #include <rpc/rpc.h>
 
+bool_t xdr_opaque_auth(XDR *xdrs, struct opaque_auth *ap);
+
 /*
  * XDR a call message
  */
