@@ -32,13 +32,17 @@
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
+#if 0
 static char sccsid[] = "@(#)qsort.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/lib/libc/stdlib/qsort.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* LIBC_SCCS and not lint */
 
-#include <sys/types.h>
 #include <stdlib.h>
 
-static inline char	*med3 __P((char *, char *, char *, int (*)()));
+typedef int		 cmp_t __P((const void *, const void *));
+static inline char	*med3 __P((char *, char *, char *, cmp_t *));
 static inline void	 swapfunc __P((char *, char *, int, int));
 
 #define min(a, b)	(a) < (b) ? a : b
@@ -84,7 +88,7 @@ swapfunc(a, b, n, swaptype)
 static inline char *
 med3(a, b, c, cmp)
 	char *a, *b, *c;
-	int (*cmp)();
+	cmp_t *cmp;
 {
 	return cmp(a, b) < 0 ?
 	       (cmp(b, c) < 0 ? b : (cmp(a, c) < 0 ? c : a ))
@@ -95,7 +99,7 @@ void
 qsort(a, n, es, cmp)
 	void *a;
 	size_t n, es;
-	int (*cmp)();
+	cmp_t *cmp;
 {
 	char *pa, *pb, *pc, *pd, *pl, *pm, *pn;
 	int d, r, swaptype, swap_cnt;
@@ -103,16 +107,16 @@ qsort(a, n, es, cmp)
 loop:	SWAPINIT(a, es);
 	swap_cnt = 0;
 	if (n < 7) {
-		for (pm = a + es; pm < (char *) a + n * es; pm += es)
-			for (pl = pm; pl > (char *) a && cmp(pl - es, pl) > 0;
+		for (pm = (char *)a + es; pm < (char *)a + n * es; pm += es)
+			for (pl = pm; pl > (char *)a && cmp(pl - es, pl) > 0;
 			     pl -= es)
 				swap(pl, pl - es);
 		return;
 	}
-	pm = a + (n / 2) * es;
+	pm = (char *)a + (n / 2) * es;
 	if (n > 7) {
 		pl = a;
-		pn = a + (n - 1) * es;
+		pn = (char *)a + (n - 1) * es;
 		if (n > 40) {
 			d = (n / 8) * es;
 			pl = med3(pl, pl + d, pl + 2 * d, cmp);
@@ -122,9 +126,9 @@ loop:	SWAPINIT(a, es);
 		pm = med3(pl, pm, pn, cmp);
 	}
 	swap(a, pm);
-	pa = pb = a + es;
+	pa = pb = (char *)a + es;
 
-	pc = pd = a + (n - 1) * es;
+	pc = pd = (char *)a + (n - 1) * es;
 	for (;;) {
 		while (pb <= pc && (r = cmp(pb, a)) <= 0) {
 			if (r == 0) {
@@ -150,14 +154,14 @@ loop:	SWAPINIT(a, es);
 		pc -= es;
 	}
 	if (swap_cnt == 0) {  /* Switch to insertion sort */
-		for (pm = a + es; pm < (char *) a + n * es; pm += es)
-			for (pl = pm; pl > (char *) a && cmp(pl - es, pl) > 0;
+		for (pm = (char *)a + es; pm < (char *)a + n * es; pm += es)
+			for (pl = pm; pl > (char *)a && cmp(pl - es, pl) > 0;
 			     pl -= es)
 				swap(pl, pl - es);
 		return;
 	}
 
-	pn = a + n * es;
+	pn = (char *)a + n * es;
 	r = min(pa - (char *)a, pb - pa);
 	vecswap(a, pb - r, r);
 	r = min(pd - pc, pn - pd - es);
