@@ -1,8 +1,11 @@
-
+/*
+** This file is in the public domain, so clarified as of
+** June 5, 1996 by Arthur David Olson (arthur_david_olson@nih.gov).
+*/
 
 #ifndef lint
 #ifndef NOID
-static char	elsieid[] = "@(#)localtime.c	7.44";
+static char	elsieid[] = "@(#)localtime.c	7.57";
 #endif /* !defined NOID */
 #endif /* !defined lint */
 
@@ -14,9 +17,16 @@ static char	elsieid[] = "@(#)localtime.c	7.44";
 
 /*LINTLIBRARY*/
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "private.h"
 #include "tzfile.h"
 #include "fcntl.h"
+#ifdef	_THREAD_SAFE
+#include <pthread.h>
+#include "pthread_private.h"
+#endif
 
 /*
 ** SunOS 4.1.1 headers lack O_BINARY.
@@ -160,6 +170,12 @@ static struct state	gmtmem;
 static char		lcl_TZname[TZ_STRLEN_MAX + 1];
 static int		lcl_is_set;
 static int		gmt_is_set;
+#ifdef	_THREAD_SAFE
+static struct pthread_mutex	_lcl_mutexd = PTHREAD_MUTEX_STATIC_INITIALIZER;
+static struct pthread_mutex	_gmt_mutexd = PTHREAD_MUTEX_STATIC_INITIALIZER;
+static pthread_mutex_t		lcl_mutex   = &_lcl_mutexd;
+static pthread_mutex_t		gmt_mutex   = &_gmt_mutexd;
+#endif
 
 char *			tzname[2] = {
 	wildabbr,
@@ -192,7 +208,7 @@ const char * const	codep;
 	register long	result;
 	register int	i;
 
- 	result = (codep[0] & 0x80) ? ~0L : 0L;
+	result = (codep[0] & 0x80) ? ~0L : 0L;
 	for (i = 0; i < 4; ++i)
 		result = (result << 8) | (codep[i] & 0xff);
 	return result;
@@ -201,8 +217,8 @@ const char * const	codep;
 static void
 settzname P((void))
 {
-	register struct state * const		sp = lclptr;
-	register int				i;
+	register struct state * const	sp = lclptr;
+	register int			i;
 
 	tzname[0] = wildabbr;
 	tzname[1] = wildabbr;
@@ -257,14 +273,20 @@ register struct state * const	sp;
 	register int		i;
 	register int		fid;
 
+	/* XXX The following is from OpenBSD, and I'm not sure it is correct */
+	if (name != NULL && issetugid() != 0)
+		if ((name[0] == ':' && name[1] == '/') || 
+		    name[0] == '/' || strchr(name, '.'))
+			name = NULL;
 	if (name == NULL && (name = TZDEFAULT) == NULL)
 		return -1;
 	{
 		register int	doaccess;
+		struct stat	stab;
 		/*
 		** Section 4.9.1 of the C standard says that
 		** "FILENAME_MAX expands to an integral constant expression
-		** that is the sie needed for an array of char large enough
+		** that is the size needed for an array of char large enough
 		** to hold the longest file name string that the implementation
 		** guarantees can be opened."
 		*/
@@ -276,7 +298,7 @@ register struct state * const	sp;
 		if (!doaccess) {
 			if ((p = TZDIR) == NULL)
 				return -1;
-			if ((strlen(p) + strlen(name) + 1) >= sizeof fullname)
+			if ((strlen(p) + 1 + strlen(name) + 1) >= sizeof fullname)
 				return -1;
 			(void) strcpy(fullname, p);
 			(void) strcat(fullname, "/");
@@ -289,9 +311,11 @@ register struct state * const	sp;
 			name = fullname;
 		}
 		if (doaccess && access(name, R_OK) != 0)
-			return -1;
+		     	return -1;
 		if ((fid = open(name, OPEN_MODE)) == -1)
 			return -1;
+		if ((fstat(fid, &stab) < 0) || !S_ISREG(stab.st_mode))
+			return -1;
 	}
 	{
 		struct tzhead *	tzhp;
@@ -417,7 +441,7 @@ register const char *	strp;
 {
 	register char	c;
 
-	while ((c = *strp) != '\0' && !isdigit(c) && c != ',' && c != '-' &&
+	while ((c = *strp) != '\0' && !is_digit(c) && c != ',' && c != '-' &&
 		c != '+')
 			++strp;
 	return strp;
@@ -440,15 +464,15 @@ const int		max;
 	register char	c;
 	register int	num;
 
-	if (strp == NULL || !isdigit(*strp))
+	if (strp == NULL || !is_digit(c = *strp))
 		return NULL;
 	num = 0;
-	while ((c = *strp) != '\0' && isdigit(c)) {
+	do {
 		num = num * 10 + (c - '0');
 		if (num > max)
 			return NULL;	/* illegal value */
-		++strp;
-	}
+		c = *++strp;
+	} while (is_digit(c));
 	if (num < min)
 		return NULL;		/* illegal value */
 	*nump = num;
@@ -510,14 +534,13 @@ getoffset(strp, offsetp)
 register const char *	strp;
 long * const		offsetp;
 {
-	register int	neg;
+	register int	neg = 0;
 
 	if (*strp == '-') {
 		neg = 1;
 		++strp;
-	} else if (isdigit(*strp) || *strp++ == '+')
-		neg = 0;
-	else	return NULL;		/* illegal offset */
+	} else if (*strp == '+')
+		++strp;
 	strp = getsecs(strp, offsetp);
 	if (strp == NULL)
 		return NULL;		/* illegal time */
@@ -562,7 +585,7 @@ register struct rule * const	rulep;
 		if (*strp++ != '.')
 			return NULL;
 		strp = getnum(strp, &rulep->r_day, 0, DAYSPERWEEK - 1);
-	} else if (isdigit(*strp)) {
+	} else if (is_digit(*strp)) {
 		/*
 		** Day of year.
 		*/
@@ -706,18 +729,19 @@ const int			lastditch;
 		name += stdlen;
 		if (stdlen >= sizeof sp->chars)
 			stdlen = (sizeof sp->chars) - 1;
+		stdoffset = 0;
 	} else {
 		name = getzname(name);
 		stdlen = name - stdname;
 		if (stdlen < 3)
 			return -1;
-	}
-	if (*name == '\0')
-		return -1;	/* was "stdoffset = 0;" */
-	else {
-		name = getoffset(name, &stdoffset);
-		if (name == NULL)
-			return -1;
+		if (*name == '\0')
+			return -1;	/* was "stdoffset = 0;" */
+		else {
+			name = getoffset(name, &stdoffset);
+			if (name == NULL)
+				return -1;
+		}
 	}
 	load_result = tzload(TZDEFRULES, sp);
 	if (load_result != 0)
@@ -804,7 +828,8 @@ const int			lastditch;
 			for (i = 0; i < sp->timecnt; ++i) {
 				j = sp->types[i];
 				if (!sp->ttis[j].tt_isdst) {
-					theirstdoffset = -sp->ttis[j].tt_gmtoff;
+					theirstdoffset =
+						-sp->ttis[j].tt_gmtoff;
 					break;
 				}
 			}
@@ -812,7 +837,8 @@ const int			lastditch;
 			for (i = 0; i < sp->timecnt; ++i) {
 				j = sp->types[i];
 				if (sp->ttis[j].tt_isdst) {
-					theirdstoffset = -sp->ttis[j].tt_gmtoff;
+					theirdstoffset =
+						-sp->ttis[j].tt_gmtoff;
 					break;
 				}
 			}
@@ -908,8 +934,13 @@ struct state * const	sp;
 */
 static
 #endif /* !defined STD_INSPIRED */
+#ifdef	_THREAD_SAFE
+void
+tzsetwall_basic P((void))
+#else
 void
 tzsetwall P((void))
+#endif
 {
 	if (lcl_is_set < 0)
 		return;
@@ -929,8 +960,23 @@ tzsetwall P((void))
 	settzname();
 }
 
+#ifdef	_THREAD_SAFE
+void
+tzsetwall P((void))
+{
+	pthread_mutex_lock(&lcl_mutex);
+	tzsetwall_basic();
+	pthread_mutex_unlock(&lcl_mutex);
+}
+#endif
+
+#ifdef	_THREAD_SAFE
+static void
+tzset_basic P((void))
+#else
 void
 tzset P((void))
+#endif
 {
 	register const char *	name;
 
@@ -970,6 +1016,16 @@ tzset P((void))
 	settzname();
 }
 
+#ifdef	_THREAD_SAFE
+void
+tzset P((void))
+{
+	pthread_mutex_lock(&lcl_mutex);
+	tzset_basic();
+	pthread_mutex_unlock(&lcl_mutex);
+}
+#endif
+
 /*
 ** The easy way to behave "as if no library function calls" localtime
 ** is to not call it--so we drop its guts into "localsub", which can be
@@ -1026,13 +1082,54 @@ struct tm * const	tmp;
 #endif /* defined TM_ZONE */
 }
 
+#ifdef	_THREAD_SAFE
+struct tm *
+localtime_r(timep, p_tm)
+const time_t * const	timep;
+struct tm *p_tm;
+{
+	pthread_mutex_lock(&lcl_mutex);
+	tzset();
+	localsub(timep, 0L, p_tm);
+	pthread_mutex_unlock(&lcl_mutex);
+	return(p_tm);
+}
+#endif
+
 struct tm *
 localtime(timep)
 const time_t * const	timep;
 {
+#ifdef	_THREAD_SAFE
+	static struct pthread_mutex _localtime_mutex = PTHREAD_MUTEX_STATIC_INITIALIZER;
+	static pthread_mutex_t localtime_mutex = &_localtime_mutex;
+	static pthread_key_t localtime_key = -1;
+	struct tm *p_tm;
+
+	pthread_mutex_lock(&localtime_mutex);
+	if (localtime_key < 0) {
+		if (pthread_key_create(&localtime_key, free) < 0) {
+			pthread_mutex_unlock(&localtime_mutex);
+			return(NULL);
+		}
+	}
+	pthread_mutex_unlock(&localtime_mutex);
+	p_tm = pthread_getspecific(localtime_key);
+	if (p_tm == NULL) {
+		if ((p_tm = (struct tm *)malloc(sizeof(struct tm))) == NULL)
+			return(NULL);
+		pthread_setspecific(localtime_key, p_tm);
+	}
+	pthread_mutex_lock(&lcl_mutex);
+	tzset();
+	localsub(timep, 0L, p_tm);
+	pthread_mutex_unlock(&lcl_mutex);
+	return p_tm;
+#else
 	tzset();
 	localsub(timep, 0L, &tm);
 	return &tm;
+#endif
 }
 
 /*
@@ -1045,6 +1142,9 @@ const time_t * const	timep;
 const long		offset;
 struct tm * const	tmp;
 {
+#ifdef	_THREAD_SAFE
+	pthread_mutex_lock(&gmt_mutex);
+#endif
 	if (!gmt_is_set) {
 		gmt_is_set = TRUE;
 #ifdef ALL_STATE
@@ -1053,6 +1153,9 @@ struct tm * const	tmp;
 #endif /* defined ALL_STATE */
 			gmtload(gmtptr);
 	}
+#ifdef	_THREAD_SAFE
+	pthread_mutex_unlock(&gmt_mutex);
+#endif
 	timesub(timep, offset, gmtptr, tmp);
 #ifdef TM_ZONE
 	/*
@@ -1079,10 +1182,47 @@ struct tm *
 gmtime(timep)
 const time_t * const	timep;
 {
+#ifdef	_THREAD_SAFE
+	static struct pthread_mutex _gmtime_mutex = PTHREAD_MUTEX_STATIC_INITIALIZER;
+	static pthread_mutex_t gmtime_mutex = &_gmtime_mutex;
+	static pthread_key_t gmtime_key = -1;
+	struct tm *p_tm;
+
+	pthread_mutex_lock(&gmtime_mutex);
+	if (gmtime_key < 0) {
+		if (pthread_key_create(&gmtime_key, free) < 0) {
+			pthread_mutex_unlock(&gmtime_mutex);
+			return(NULL);
+		}
+	}
+	pthread_mutex_unlock(&gmtime_mutex);
+	/*
+	 * Changed to follow draft 4 pthreads standard, which
+	 * is what BSD currently has.
+	 */
+	if ((p_tm = pthread_getspecific(gmtime_key)) == NULL) {
+		if ((p_tm = (struct tm *)malloc(sizeof(struct tm))) == NULL) {
+			return(NULL);
+		}
+		pthread_setspecific(gmtime_key, p_tm);
+	}
+	gmtsub(timep, 0L, p_tm);
+	return(p_tm);
+#else
 	gmtsub(timep, 0L, &tm);
 	return &tm;
+#endif
 }
 
+#ifdef	_THREAD_SAFE
+struct tm *
+gmtime_r(const time_t * timep, struct tm * tm)
+{
+	gmtsub(timep, 0L, tm);
+	return(tm);
+}
+#endif
+
 #ifdef STD_INSPIRED
 
 struct tm *
@@ -1164,30 +1304,27 @@ register struct tm * const		tmp;
 	tmp->tm_hour = (int) (rem / SECSPERHOUR);
 	rem = rem % SECSPERHOUR;
 	tmp->tm_min = (int) (rem / SECSPERMIN);
-	tmp->tm_sec = (int) (rem % SECSPERMIN);
-	if (hit)
-		/*
-		** A positive leap second requires a special
-		** representation.  This uses "... ??:59:60" et seq.
-		*/
-		tmp->tm_sec += hit;
+	/*
+	** A positive leap second requires a special
+	** representation.  This uses "... ??:59:60" et seq.
+	*/
+	tmp->tm_sec = (int) (rem % SECSPERMIN) + hit;
 	tmp->tm_wday = (int) ((EPOCH_WDAY + days) % DAYSPERWEEK);
 	if (tmp->tm_wday < 0)
 		tmp->tm_wday += DAYSPERWEEK;
 	y = EPOCH_YEAR;
-	if (days >= 0)
-		for ( ; ; ) {
-			yleap = isleap(y);
-			if (days < (long) year_lengths[yleap])
-				break;
-			++y;
-			days = days - (long) year_lengths[yleap];
-		}
-	else do {
-		--y;
-		yleap = isleap(y);
-		days = days + (long) year_lengths[yleap];
-	} while (days < 0);
+#define LEAPS_THRU_END_OF(y)	((y) / 4 - (y) / 100 + (y) / 400)
+	while (days < 0 || days >= (long) year_lengths[yleap = isleap(y)]) {
+		register int	newy;
+
+		newy = y + days / DAYSPERNYEAR;
+		if (days < 0)
+			--newy;
+		days -= (newy - y) * DAYSPERNYEAR +
+			LEAPS_THRU_END_OF(newy - 1) -
+			LEAPS_THRU_END_OF(y - 1);
+		y = newy;
+	}
 	tmp->tm_year = y - TM_YEAR_BASE;
 	tmp->tm_yday = (int) days;
 	ip = mon_lengths[yleap];
@@ -1213,10 +1350,22 @@ const time_t * const	timep;
 	return asctime(localtime(timep));
 }
 
+#ifdef _THREAD_SAFE
+char *
+ctime_r(timep, buf)
+const time_t * const	timep;
+char *buf;
+{
+        struct tm tm;
+	return asctime_r(localtime_r(timep, &tm), buf);
+}
+#endif
+
 /*
 ** Adapted from code provided by Robert Elz, who writes:
 **	The "best" way to do mktime I think is based on an idea of Bob
-**	Kridle's (so its said...) from a long time ago. (mtxinu!kridle now).
+**	Kridle's (so its said...) from a long time ago.
+**	[kridle@xinet.com as of 1996-01-16.]
 **	It does a binary search of the time_t space.  Since time_t's are
 **	just 32 bits, its a max of 32 iterations (even at 64 bits it
 **	would still be very reasonable).
@@ -1306,10 +1455,12 @@ int * const		okayp;
 	while (yourtm.tm_mday <= 0) {
 		if (increment_overflow(&yourtm.tm_year, -1))
 			return WRONG;
-		yourtm.tm_mday += year_lengths[isleap(yourtm.tm_year)];
+		i = yourtm.tm_year + (1 < yourtm.tm_mon);
+		yourtm.tm_mday += year_lengths[isleap(i)];
 	}
 	while (yourtm.tm_mday > DAYSPERLYEAR) {
-		yourtm.tm_mday -= year_lengths[isleap(yourtm.tm_year)];
+		i = yourtm.tm_year + (1 < yourtm.tm_mon);
+		yourtm.tm_mday -= year_lengths[isleap(i)];
 		if (increment_overflow(&yourtm.tm_year, 1))
 			return WRONG;
 	}
@@ -1344,17 +1495,16 @@ int * const		okayp;
 		yourtm.tm_sec = 0;
 	}
 	/*
-	** Calculate the number of magnitude bits in a time_t
-	** (this works regardless of whether time_t is
-	** signed or unsigned, though lint complains if unsigned).
+	** Divide the search space in half
+	** (this works whether time_t is signed or unsigned).
 	*/
-	for (bits = 0, t = 1; t > 0; ++bits, t <<= 1)
-		continue;
+	bits = TYPE_BIT(time_t) - 1;
 	/*
-	** If time_t is signed, then 0 is the median value,
-	** if time_t is unsigned, then 1 << bits is median.
+	** If time_t is signed, then 0 is just above the median,
+	** assuming two's complement arithmetic.
+	** If time_t is unsigned, then (1 << bits) is just above the median.
 	*/
-	t = (t < 0) ? 0 : ((time_t) 1 << bits);
+	t = TYPE_SIGNED(time_t) ? 0 : (((time_t) 1) << bits);
 	for ( ; ; ) {
 		(*funcp)(&t, offset, &mytm);
 		dir = tmcomp(&mytm, &yourtm);
@@ -1362,10 +1512,10 @@ int * const		okayp;
 			if (bits-- < 0)
 				return WRONG;
 			if (bits < 0)
-				--t;
+				--t; /* may be needed if new t is minimal */
 			else if (dir > 0)
-				t -= (time_t) 1 << bits;
-			else	t += (time_t) 1 << bits;
+				t -= ((time_t) 1) << bits;
+			else	t += ((time_t) 1) << bits;
 			continue;
 		}
 		if (yourtm.tm_isdst < 0 || mytm.tm_isdst == yourtm.tm_isdst)
@@ -1386,10 +1536,10 @@ int * const		okayp;
 		if (sp == NULL)
 			return WRONG;
 #endif /* defined ALL_STATE */
-		for (i = 0; i < sp->typecnt; ++i) {
+		for (i = sp->typecnt - 1; i >= 0; --i) {
 			if (sp->ttis[i].tt_isdst != yourtm.tm_isdst)
 				continue;
-			for (j = 0; j < sp->typecnt; ++j) {
+			for (j = sp->typecnt - 1; j >= 0; --j) {
 				if (sp->ttis[j].tt_isdst == yourtm.tm_isdst)
 					continue;
 				newt = t + sp->ttis[j].tt_gmtoff -
@@ -1421,7 +1571,7 @@ label:
 static time_t
 time1(tmp, funcp, offset)
 struct tm * const	tmp;
-void (* const		funcp) P((const time_t*, long, struct tm*));
+void (* const		funcp) P((const time_t *, long, struct tm *));
 const long		offset;
 {
 	register time_t			t;
@@ -1460,10 +1610,10 @@ const long		offset;
 	if (sp == NULL)
 		return WRONG;
 #endif /* defined ALL_STATE */
-	for (samei = 0; samei < sp->typecnt; ++samei) {
+	for (samei = sp->typecnt - 1; samei >= 0; --samei) {
 		if (sp->ttis[samei].tt_isdst != tmp->tm_isdst)
 			continue;
-		for (otheri = 0; otheri < sp->typecnt; ++otheri) {
+		for (otheri = sp->typecnt - 1; otheri >= 0; --otheri) {
 			if (sp->ttis[otheri].tt_isdst == tmp->tm_isdst)
 				continue;
 			tmp->tm_sec += sp->ttis[otheri].tt_gmtoff -
@@ -1484,8 +1634,16 @@ time_t
 mktime(tmp)
 struct tm * const	tmp;
 {
+	time_t mktime_return_value;
+#ifdef	_THREAD_SAFE
+	pthread_mutex_lock(&lcl_mutex);
+#endif
 	tzset();
-	return time1(tmp, localsub, 0L);
+	mktime_return_value = time1(tmp, localsub, 0L);
+#ifdef	_THREAD_SAFE
+	pthread_mutex_unlock(&lcl_mutex);
+#endif
+	return(mktime_return_value);
 }
 
 #ifdef STD_INSPIRED
