@@ -17,7 +17,7 @@
 
 #ifdef LIBC_RCS
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/lib/libc/stdtime/strftime.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/lib/libc/stdtime/strftime.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #ifndef lint
@@ -40,75 +40,8 @@ static const char	sccsid[] = "@(#)strftime.c	5.4 (Berkeley) 3/14/89";
 
 #include "tzfile.h"
 #include <fcntl.h>
-#include <locale.h>
-#include <rune.h>		/* for _PATH_LOCALE */
 #include <sys/stat.h>
-
-#define LOCALE_HOME _PATH_LOCALE
-
-struct lc_time_T {
-	const char *	mon[12];
-	const char *	month[12];
-	const char *	wday[7];
-	const char *	weekday[7];
-	const char *	X_fmt;
-	const char *	x_fmt;
-	const char *	c_fmt;
-	const char *	am;
-	const char *	pm;
-	const char *	date_fmt;
-};
-
-static struct lc_time_T		localebuf;
-static struct lc_time_T *	_loc P((void));
-static int using_locale;
-
-#define Locale	(using_locale ? &localebuf : &C_time_locale)
-
-static const struct lc_time_T	C_time_locale = {
-	{
-		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
-	}, {
-		"January", "February", "March", "April", "May", "June",
-		"July", "August", "September", "October", "November", "December"
-	}, {
-		"Sun", "Mon", "Tue", "Wed",
-		"Thu", "Fri", "Sat"
-	}, {
-		"Sunday", "Monday", "Tuesday", "Wednesday",
-		"Thursday", "Friday", "Saturday"
-	},
-
-	/* X_fmt */
-	"%H:%M:%S",
-
-	/*
-	** x_fmt
-	** Since the C language standard calls for
-	** "date, using locale's date format," anything goes.
-	** Using just numbers (as here) makes Quakers happier;
-	** it's also compatible with SVR4.
-	*/
-	"%m/%d/%y",
-
-	/*
-	** c_fmt (ctime-compatible)
-	** Note that
-	**	"%a %b %d %H:%M:%S %Y"
-	** is used by Solaris 2.3.
-	*/
-	"%a %b %e %X %Y",
-
-	/* am */
-	"AM",
-
-	/* pm */
-	"PM",
-
-	/* date_fmt */
-	"%a %b %e %X %Z %Y"
-};
+#include "timelocal.h"
 
 static char *	_add P((const char *, char *, const char *));
 static char *	_conv P((int, const char *, char *, const char *));
@@ -301,67 +234,89 @@ label:
 				pt = _conv((t->tm_wday == 0) ? 7 : t->tm_wday,
 					"%d", pt, ptlim);
 				continue;
-			case 'V':
-				/*
-				** From Arnold Robbins' strftime version 3.0:
-				** "the week number of the year (the first
-				** Monday as the first day of week 1) as a
-				** decimal number (01-53).  The method for
-				** determining the week number is as specified
-				** by ISO 8601 (to wit: if the week containing
-				** January 1 has four or more days in the new
-				** year, then it is week 1, otherwise it is
-				** week 53 of the previous year and the next
-				** week is week 1)."
-				** (ado, 5/24/93)
-				*/
-				/*
-				** XXX--If January 1 falls on a Friday,
-				** January 1-3 are part of week 53 of the
-				** previous year.  By analogy, if January
-				** 1 falls on a Thursday, are December 29-31
-				** of the PREVIOUS year part of week 1???
-				** (ado 5/24/93)
-				*/
-				/*
-				** You are understood not to expect this.
-				*/
+			case 'V':	/* ISO 8601 week number */
+			case 'G':	/* ISO 8601 year (four digits) */
+			case 'g':	/* ISO 8601 year (two digits) */
+/*
+** From Arnold Robbins' strftime version 3.0:  "the week number of the
+** year (the first Monday as the first day of week 1) as a decimal number
+** (01-53)."
+** (ado, 1993-05-24)
+**
+** From "http://www.ft.uni-erlangen.de/~mskuhn/iso-time.html" by Markus Kuhn:
+** "Week 01 of a year is per definition the first week which has the
+** Thursday in this year, which is equivalent to the week which contains
+** the fourth day of January. In other words, the first week of a new year
+** is the week which has the majority of its days in the new year. Week 01
+** might also contain days from the previous year and the week before week
+** 01 of a year is the last week (52 or 53) of the previous year even if
+** it contains days from the new year. A week starts with Monday (day 1)
+** and ends with Sunday (day 7).  For example, the first week of the year
+** 1997 lasts from 1996-12-30 to 1997-01-05..."
+** (ado, 1996-01-02)
+*/
 				{
-					int	i;
+					int	year;
+					int	yday;
+					int	wday;
+					int	w;
 
-					i = (t->tm_yday + 10 - (t->tm_wday ?
-						(t->tm_wday - 1) : 6)) / 7;
-					if (i == 0) {
+					year = t->tm_year + TM_YEAR_BASE;
+					yday = t->tm_yday;
+					wday = t->tm_wday;
+					for ( ; ; ) {
+						int	len;
+						int	bot;
+						int	top;
+
+						len = isleap(year) ?
+							DAYSPERLYEAR :
+							DAYSPERNYEAR;
 						/*
-						** What day of the week does
-						** January 1 fall on?
+						** What yday (-3 ... 3) does
+						** the ISO year begin on?
 						*/
-						i = t->tm_wday -
-							(t->tm_yday - 1);
+						bot = ((yday + 11 - wday) %
+							DAYSPERWEEK) - 3;
 						/*
-						** Fri Jan 1: 53
-						** Sun Jan 1: 52
-						** Sat Jan 1: 53 if previous
-						**		 year a leap
-						**		 year, else 52
+						** What yday does the NEXT
+						** ISO year begin on?
 						*/
-						if (i == TM_FRIDAY)
-							i = 53;
-						else if (i == TM_SUNDAY)
-							i = 52;
-						else	i = isleap(t->tm_year +
-								TM_YEAR_BASE) ?
-								53 : 52;
-#ifdef XPG4_1994_04_09
-						/*
-						** As of 4/9/94, though,
-						** XPG4 calls for 53
-						** unconditionally.
-						*/
-						i = 53;
-#endif /* defined XPG4_1994_04_09 */
+						top = bot -
+							(len % DAYSPERWEEK);
+						if (top < -3)
+							top += DAYSPERWEEK;
+						top += len;
+						if (yday >= top) {
+							++year;
+							w = 1;
+							break;
+						}
+						if (yday >= bot) {
+							w = 1 + ((yday - bot) /
+								DAYSPERWEEK);
+							break;
+						}
+						--year;
+						yday += isleap(year) ?
+							DAYSPERLYEAR :
+							DAYSPERNYEAR;
 					}
-					pt = _conv(i, "%02d", pt, ptlim);
+#ifdef XPG4_1994_04_09
+					if ((w == 52
+					     && t->tm_mon == TM_JANUARY)
+					    || (w == 1
+						&& t->tm_mon == TM_DECEMBER))
+						w = 53;
+#endif /* defined XPG4_1994_04_09 */
+					if (*format == 'V')
+						pt = _conv(w, "%02d",
+							pt, ptlim);
+					else if (*format == 'g') {
+						pt = _conv(year % 100, "%02d",
+							pt, ptlim);
+					} else	pt = _conv(year, "%04d",
+							pt, ptlim);
 				}
 				continue;
 			case 'v':
@@ -464,109 +419,3 @@ _add(str, pt, ptlim)
 		++pt;
 	return pt;
 }
-
-extern char *_PathLocale;
-
-int
-__time_load_locale(const char *name)
-{
-	static const char	lc_time[] = "LC_TIME";
-	static char *		locale_buf;
-	static char		locale_buf_C[] = "C";
-
-	int			fd;
-	char *			lbuf;
-	char *			p;
-	const char **		ap;
-	const char *		plim;
-	char			filename[FILENAME_MAX];
-	struct stat		st;
-	size_t			namesize;
-	size_t			bufsize;
-
-	using_locale = 0;
-
-	if (!strcmp(name, "C") || !strcmp(name, "POSIX"))
-		return 0;
-
-	if (name == NULL || *name == '\0') {
-		goto no_locale;
-	}
-	/*
-	** If the locale name is the same as our cache, use the cache.
-	*/
-	lbuf = locale_buf;
-	if (lbuf != NULL && strcmp(name, lbuf) == 0) {
-		p = lbuf;
-		for (ap = (const char **) &localebuf;
-			ap < (const char **) (&localebuf + 1);
-				++ap)
-					*ap = p += strlen(p) + 1;
-		using_locale = 1;
-		return 0;
-	}
-	/*
-	** Slurp the locale file into the cache.
-	*/
-	namesize = strlen(name) + 1;
-
-	snprintf(filename, sizeof filename, 
-		 "%s/%s/%s",
-		 _PathLocale, name, lc_time);
-	fd = open(filename, O_RDONLY);
-	if (fd < 0) {
-		goto no_locale;
-	}
-	if (fstat(fd, &st) != 0)
-		goto bad_locale;
-	if (st.st_size <= 0)
-		goto bad_locale;
-	bufsize = namesize + st.st_size;
-	locale_buf = NULL;
-	lbuf = (lbuf == NULL || lbuf == locale_buf_C) ?
-		malloc(bufsize) : realloc(lbuf, bufsize);
-	if (lbuf == NULL)
-		goto bad_locale;
-	(void) strcpy(lbuf, name);
-	p = lbuf + namesize;
-	plim = p + st.st_size;
-	if (read(fd, p, (size_t) st.st_size) != st.st_size)
-		goto bad_lbuf;
-	if (close(fd) != 0)
-		goto bad_lbuf;
-	/*
-	** Parse the locale file into localebuf.
-	*/
-	if (plim[-1] != '\n')
-		goto bad_lbuf;
-	for (ap = (const char **) &localebuf;
-		ap < (const char **) (&localebuf + 1);
-			++ap) {
-				if (p == plim)
-					goto bad_lbuf;
-				*ap = p;
-				while (*p != '\n')
-					++p;
-				*p++ = '\0';
-	}
-	/*
-	** Record the successful parse in the cache.
-	*/
-	locale_buf = lbuf;
-
-	using_locale = 1;
-	return 0;
-
-bad_lbuf:
-	free(lbuf);
-bad_locale:
-	(void) close(fd);
-no_locale:
-	/*
-	 * XXX - This may not be the correct thing to do in this case.
-	 * setlocale() assumes that we left the old locale alone.
-	 */
-	locale_buf = locale_buf_C;
-	localebuf = C_time_locale;
-	return -1;
-}
