@@ -30,7 +30,7 @@
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)xdr_reference.c 1.11 87/08/11 SMI";*/
 /*static char *sccsid = "from: @(#)xdr_reference.c	2.1 88/07/29 4.0 RPCSRC";*/
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/xdr/xdr_reference.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/xdr/xdr_reference.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -43,6 +43,8 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/xdr/xdr_reference.c 50952 19
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <rpc/types.h>
 #include <rpc/xdr.h>
 
