@@ -28,7 +28,7 @@
  */
 
 #ifndef LINT
-static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/yp/yplib.c 50952 1999-09-05 11:54:50Z peter $";
+static char *rcsid = "$FreeBSD: stable/2.2/lib/libc/yp/yplib.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include <sys/param.h>
@@ -43,7 +43,25 @@ static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/yp/yplib.c 50952 1999-09-05
 #include <unistd.h>
 #include <rpc/rpc.h>
 #include <rpc/xdr.h>
-#include <rpcsvc/yp_prot.h>
+#include <rpcsvc/yp.h>
+
+
+/*
+ * We have to define these here due to clashes between yp_prot.h and
+ * yp.h.
+ */
+
+struct dom_binding {
+	struct dom_binding *dom_pnext;
+	char dom_domain[YPMAXDOMAIN + 1];
+	struct sockaddr_in dom_server_addr;
+	u_short dom_server_port;
+	int dom_socket;
+	CLIENT *dom_client;
+	u_short dom_local_port;	/* now I finally know what this is for. */
+	long dom_vers;
+};
+
 #include <rpcsvc/ypclnt.h>
 
 #ifndef BINDINGDIR
@@ -61,6 +79,7 @@ extern bool_t xdr_ypresp_master();
 int (*ypresp_allfn)();
 void *ypresp_data;
 
+static void _yp_unbind __P(( struct dom_binding * ));
 struct dom_binding *_ypbindlist;
 static char _yp_domain[MAXHOSTNAMELEN];
 int _yplib_timeout = 10;
@@ -77,11 +96,11 @@ static struct ypmatch_ent {
 
 static void
 ypmatch_add(map, key, keylen, val, vallen)
-char *map;
-char *key;
-int keylen;
-char *val;
-int vallen;
+	char *map;
+	char *key;
+	int keylen;
+	char *val;
+	int vallen;
 {
 	struct ypmatch_ent *ep;
 	time_t t;
@@ -137,11 +156,11 @@ int vallen;
 
 static bool_t
 ypmatch_find(map, key, keylen, val, vallen)
-char *map;
-char *key;
-int keylen;
-char **val;
-int *vallen;
+	char *map;
+	char *key;
+	int keylen;
+	char **val;
+	int *vallen;
 {
 	struct ypmatch_ent *ep;
 	time_t t;
@@ -171,7 +190,7 @@ int *vallen;
 
 char *
 ypbinderr_string(incode)
-int incode;
+	int incode;
 {
 	static char err[80];
 	switch(incode) {
@@ -190,19 +209,22 @@ int incode;
 
 int
 _yp_dobind(dom, ypdb)
-char *dom;
-struct dom_binding **ypdb;
+	char *dom;
+	struct dom_binding **ypdb;
 {
-	static int pid = -1;
+	static pid_t pid = -1;
 	char path[MAXPATHLEN];
 	struct dom_binding *ysd, *ysd2;
 	struct ypbind_resp ypbr;
 	struct timeval tv;
 	struct sockaddr_in clnt_sin;
-	int clnt_sock, fd, gpid;
+	int clnt_sock, fd;
+	pid_t gpid;
 	CLIENT *client;
 	int new = 0, r;
 	int retries = 0;
+	struct sockaddr_in check;
+	int checklen = sizeof(struct sockaddr_in);
 
 	/* Not allowed; bad doggie. Bad. */
 	if (strchr(dom, '/') != NULL)
@@ -212,8 +234,8 @@ struct dom_binding **ypdb;
 	if( !(pid==-1 || pid==gpid) ) {
 		ysd = _ypbindlist;
 		while(ysd) {
-			if(ysd->dom_client)
-				clnt_destroy(ysd->dom_client);
+			if(ysd->dom_client != NULL)
+				_yp_unbind(ysd);
 			ysd2 = ysd->dom_pnext;
 			free(ysd);
 			ysd = ysd2;
@@ -231,13 +253,33 @@ struct dom_binding **ypdb;
 	for(ysd = _ypbindlist; ysd; ysd = ysd->dom_pnext)
 		if( strcmp(dom, ysd->dom_domain) == 0)
 			break;
+
+
 	if(ysd==NULL) {
 		ysd = (struct dom_binding *)malloc(sizeof *ysd);
 		bzero((char *)ysd, sizeof *ysd);
 		ysd->dom_socket = -1;
 		ysd->dom_vers = 0;
 		new = 1;
+	} else {
+	/* Check the socket -- may have been hosed by the caller. */
+		if (getsockname(ysd->dom_socket, (struct sockaddr *)&check,
+		    &checklen) == -1 || check.sin_family != AF_INET ||
+		    check.sin_port != ysd->dom_local_port) {
+		/* Socket became bogus somehow... need to rebind. */
+			int save, sock;
+
+			sock = ysd->dom_socket;
+			save = dup(ysd->dom_socket);
+			if (ysd->dom_client != NULL)
+				clnt_destroy(ysd->dom_client);
+			ysd->dom_vers = 0;
+			ysd->dom_client = NULL;
+			sock = dup2(save, sock);
+			close(save);
+		}
 	}
+
 again:
 	retries++;
 	if (retries > MAX_RETRIES) {
@@ -247,6 +289,15 @@ again:
 	}
 #ifdef BINDINGDIR
 	if(ysd->dom_vers==0) {
+		/*
+		 * We're trying to make a new binding: zorch the
+		 * existing handle now (if any).
+		 */
+		if(ysd->dom_client != NULL) {
+			clnt_destroy(ysd->dom_client);
+			ysd->dom_client = NULL;
+			ysd->dom_socket = -1;
+		}
 		sprintf(path, "%s/%s.%d", BINDINGDIR, dom, 2);
 		if( (fd=open(path, O_RDONLY)) == -1) {
 			/* no binding file, YP is dead. */
@@ -274,10 +325,10 @@ again:
 			bzero(&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
 			ysd->dom_server_addr.sin_family = AF_INET;
 			ysd->dom_server_addr.sin_len = sizeof(struct sockaddr_in);
-			ysd->dom_server_addr.sin_addr =
-			    ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr;
+			ysd->dom_server_addr.sin_addr.s_addr =
+			    *(u_long *)&ybr.ypbind_resp_u.ypbind_bindinfo.ypbind_binding_addr;
 			ysd->dom_server_addr.sin_port =
-			    ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+			    *(u_short *)&ybr.ypbind_resp_u.ypbind_bindinfo.ypbind_binding_port;
 
 			ysd->dom_server_port = ysd->dom_server_addr.sin_port;
 			close(fd);
@@ -292,6 +343,15 @@ again:
 skipit:
 #endif
 	if(ysd->dom_vers==-1 || ysd->dom_vers==0) {
+		/*
+		 * We're trying to make a new binding: zorch the
+		 * existing handle now (if any).
+		 */
+		if(ysd->dom_client != NULL) {
+			clnt_destroy(ysd->dom_client);
+			ysd->dom_client = NULL;
+			ysd->dom_socket = -1;
+		}
 		bzero((char *)&clnt_sin, sizeof clnt_sin);
 		clnt_sin.sin_family = AF_INET;
 		clnt_sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -315,15 +375,32 @@ skipit:
 			return (YPERR_YPBIND);
 		}
 
+		/*
+		 * Check the port number -- should be < IPPORT_RESERVED.
+		 * If not, it's possible someone has registered a bogus
+		 * ypbind with the portmapper and is trying to trick us.
+		 */
+		if (ntohs(clnt_sin.sin_port) >= IPPORT_RESERVED) {
+			if (client != NULL)
+				clnt_destroy(client);
+			if (new)
+				free(ysd);
+			return(YPERR_YPBIND);
+		}
 		tv.tv_sec = _yplib_timeout/2;
 		tv.tv_usec = 0;
 		r = clnt_call(client, YPBINDPROC_DOMAIN,
-			xdr_domainname, dom, xdr_ypbind_resp, &ypbr, tv);
+			xdr_domainname, (char *)&dom, xdr_ypbind_resp, &ypbr, tv);
 		if(r != RPC_SUCCESS) {
-			fprintf(stderr,
-			"YP: server for domain %s not responding, retrying\n", dom);
 			clnt_destroy(client);
 			ysd->dom_vers = -1;
+			if (r == RPC_PROGUNAVAIL || r == RPC_PROCUNAVAIL) {
+				if (new)
+					free(ysd);
+				return(YPERR_YPBIND);
+			}
+			fprintf(stderr,
+			"YP: server for domain %s not responding, retrying\n", dom);
 			goto again;
 		} else {
 			if (ypbr.ypbind_status != YPBIND_SUCC_VAL) {
@@ -338,30 +415,55 @@ skipit:
 		bzero((char *)&ysd->dom_server_addr, sizeof ysd->dom_server_addr);
 		ysd->dom_server_addr.sin_family = AF_INET;
 		ysd->dom_server_addr.sin_port =
-			ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+			*(u_short *)&ypbr.ypbind_resp_u.ypbind_bindinfo.ypbind_binding_port;
 		ysd->dom_server_addr.sin_addr.s_addr =
-			ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr.s_addr;
+			*(u_long *)&ypbr.ypbind_resp_u.ypbind_bindinfo.ypbind_binding_addr;
+
+		/*
+		 * We could do a reserved port check here too, but this
+		 * could pose compatibility problems. The local ypbind is
+		 * supposed to decide whether or not to trust yp servers
+		 * on insecure ports. For now, we trust its judgement.
+		 */
 		ysd->dom_server_port =
-			ypbr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port;
+			*(u_short *)&ypbr.ypbind_resp_u.ypbind_bindinfo.ypbind_binding_port;
 gotit:
 		ysd->dom_vers = YPVERS;
 		strcpy(ysd->dom_domain, dom);
 	}
 
-	tv.tv_sec = _yplib_timeout/2;
-	tv.tv_usec = 0;
-	if(ysd->dom_client)
-		clnt_destroy(ysd->dom_client);
-	ysd->dom_socket = RPC_ANYSOCK;
-	ysd->dom_client = clntudp_create(&ysd->dom_server_addr,
-		YPPROG, YPVERS, tv, &ysd->dom_socket);
-	if(ysd->dom_client==NULL) {
-		clnt_pcreateerror("clntudp_create");
-		ysd->dom_vers = -1;
-		goto again;
+	/* Don't rebuild the connection to the server unless we have to. */
+	if (ysd->dom_client == NULL) {
+		tv.tv_sec = _yplib_timeout/2;
+		tv.tv_usec = 0;
+		ysd->dom_socket = RPC_ANYSOCK;
+		ysd->dom_client = clntudp_create(&ysd->dom_server_addr,
+			YPPROG, YPVERS, tv, &ysd->dom_socket);
+		if(ysd->dom_client==NULL) {
+			clnt_pcreateerror("clntudp_create");
+			ysd->dom_vers = -1;
+			goto again;
+		}
+		if( fcntl(ysd->dom_socket, F_SETFD, 1) == -1)
+			perror("fcntl: F_SETFD");
+		/*
+		 * We want a port number associated with this socket
+		 * so that we can check its authenticity later.
+		 */
+		checklen = sizeof(struct sockaddr_in);
+		bzero((char *)&check, checklen);
+		bind(ysd->dom_socket, (struct sockaddr *)&check, checklen);
+		check.sin_family = AF_INET;
+		if (!getsockname(ysd->dom_socket,
+		    (struct sockaddr *)&check, &checklen)) {
+			ysd->dom_local_port = check.sin_port;
+		} else {
+			clnt_destroy(ysd->dom_client);
+			if (new)
+				free(ysd);
+			return(YPERR_YPBIND);
+		}
 	}
-	if( fcntl(ysd->dom_socket, F_SETFD, 1) == -1)
-		perror("fcntl: F_SETFD");
 
 	if(new) {
 		ysd->dom_pnext = _ypbindlist;
@@ -375,31 +477,49 @@ gotit:
 
 static void
 _yp_unbind(ypb)
-struct dom_binding *ypb;
+	struct dom_binding *ypb;
 {
-	if (ypb->dom_client != NULL)
-		clnt_destroy(ypb->dom_client);
+	struct sockaddr_in check;
+	int checklen = sizeof(struct sockaddr_in);
+
+	if (ypb->dom_client != NULL) {
+		/* Check the socket -- may have been hosed by the caller. */
+		if (getsockname(ypb->dom_socket, (struct sockaddr *)&check,
+	    	&checklen) == -1 || check.sin_family != AF_INET ||
+	    	check.sin_port != ypb->dom_local_port) {
+			int save, sock;
+
+			sock = ypb->dom_socket;
+			save = dup(ypb->dom_socket);
+			clnt_destroy(ypb->dom_client);
+			sock = dup2(save, sock);
+			close(save);
+		} else
+			clnt_destroy(ypb->dom_client);
+	}
+
 	ypb->dom_client = NULL;
 	ypb->dom_socket = -1;
+	ypb->dom_vers = -1;
 }
 
 int
 yp_bind(dom)
-char *dom;
+	char *dom;
 {
 	return _yp_dobind(dom, NULL);
 }
 
 void
 yp_unbind(dom)
-char *dom;
+	char *dom;
 {
 	struct dom_binding *ypb, *ypbp;
 
 	ypbp = NULL;
 	for(ypb=_ypbindlist; ypb; ypb=ypb->dom_pnext) {
 		if( strcmp(dom, ypb->dom_domain) == 0) {
-			clnt_destroy(ypb->dom_client);
+			_yp_unbind(ypb);
 			if(ypbp)
 				ypbp->dom_pnext = ypb->dom_pnext;
 			else
@@ -414,12 +534,12 @@ char *dom;
 
 int
 yp_match(indomain, inmap, inkey, inkeylen, outval, outvallen)
-char *indomain;
-char *inmap;
-const char *inkey;
-int inkeylen;
-char **outval;
-int *outvallen;
+	char *indomain;
+	char *inmap;
+	const char *inkey;
+	int inkeylen;
+	char **outval;
+	int *outvallen;
 {
 	struct dom_binding *ysd;
 	struct ypresp_val yprv;
@@ -439,10 +559,10 @@ int *outvallen;
 
 #ifdef YPMATCHCACHE
 	if( !strcmp(_yp_domain, indomain) && ypmatch_find(inmap, inkey,
-	    inkeylen, &yprv.valdat.dptr, &yprv.valdat.dsize)) {
-		*outvallen = yprv.valdat.dsize;
+	    inkeylen, &yprv.val.valdat_val, &yprv.val.valdat_len)) {
+		*outvallen = yprv.val.valdat_len;
 		*outval = (char *)malloc(*outvallen+1);
-		bcopy(yprv.valdat.dptr, *outval, *outvallen);
+		bcopy(yprv.val.valdat_val, *outval, *outvallen);
 		(*outval)[*outvallen] = '\0';
 		return 0;
 	}
@@ -457,8 +577,8 @@ again:
 
 	yprk.domain = indomain;
 	yprk.map = inmap;
-	yprk.keydat.dptr = (char *)inkey;
-	yprk.keydat.dsize = inkeylen;
+	yprk.key.keydat_val = (char *)inkey;
+	yprk.key.keydat_len = inkeylen;
 
 	bzero((char *)&yprv, sizeof yprv);
 
@@ -466,21 +586,22 @@ again:
 		xdr_ypreq_key, &yprk, xdr_ypresp_val, &yprv, tv);
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_match: clnt_call");
-		ysd->dom_vers = -1;
+		_yp_unbind(ysd);
 		goto again;
 	}
-	if( !(r=ypprot_err(yprv.status)) ) {
-		*outvallen = yprv.valdat.dsize;
+
+	if( !(r=ypprot_err(yprv.stat)) ) {
+		*outvallen = yprv.val.valdat_len;
 		*outval = (char *)malloc(*outvallen+1);
-		bcopy(yprv.valdat.dptr, *outval, *outvallen);
+		bcopy(yprv.val.valdat_val, *outval, *outvallen);
 		(*outval)[*outvallen] = '\0';
 #ifdef YPMATCHCACHE
 		if( strcmp(_yp_domain, indomain)==0 )
 			 ypmatch_add(inmap, inkey, inkeylen, *outval, *outvallen);
 #endif
 	}
+
 	xdr_free(xdr_ypresp_val, (char *)&yprv);
-	_yp_unbind(ysd);
 	return r;
 }
 
@@ -498,12 +619,12 @@ char **domp;
 
 int
 yp_first(indomain, inmap, outkey, outkeylen, outval, outvallen)
-char *indomain;
-char *inmap;
-char **outkey;
-int *outkeylen;
-char **outval;
-int *outvallen;
+	char *indomain;
+	char *inmap;
+	char **outkey;
+	int *outkeylen;
+	char **outval;
+	int *outvallen;
 {
 	struct ypresp_key_val yprkv;
 	struct ypreq_nokey yprnk;
@@ -535,34 +656,34 @@ again:
 		xdr_ypreq_nokey, &yprnk, xdr_ypresp_key_val, &yprkv, tv);
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_first: clnt_call");
-		ysd->dom_vers = 0;
+		_yp_unbind(ysd);
 		goto again;
 	}
-	if( !(r=ypprot_err(yprkv.status)) ) {
-		*outkeylen = yprkv.keydat.dsize;
+	if( !(r=ypprot_err(yprkv.stat)) ) {
+		*outkeylen = yprkv.key.keydat_len;
 		*outkey = (char *)malloc(*outkeylen+1);
-		bcopy(yprkv.keydat.dptr, *outkey, *outkeylen);
+		bcopy(yprkv.key.keydat_val, *outkey, *outkeylen);
 		(*outkey)[*outkeylen] = '\0';
-		*outvallen = yprkv.valdat.dsize;
+		*outvallen = yprkv.val.valdat_len;
 		*outval = (char *)malloc(*outvallen+1);
-		bcopy(yprkv.valdat.dptr, *outval, *outvallen);
+		bcopy(yprkv.val.valdat_val, *outval, *outvallen);
 		(*outval)[*outvallen] = '\0';
 	}
+
 	xdr_free(xdr_ypresp_key_val, (char *)&yprkv);
-	_yp_unbind(ysd);
 	return r;
 }
 
 int
 yp_next(indomain, inmap, inkey, inkeylen, outkey, outkeylen, outval, outvallen)
-char *indomain;
-char *inmap;
-char *inkey;
-int inkeylen;
-char **outkey;
-int *outkeylen;
-char **outval;
-int *outvallen;
+	char *indomain;
+	char *inmap;
+	char *inkey;
+	int inkeylen;
+	char **outkey;
+	int *outkeylen;
+	char **outval;
+	int *outvallen;
 {
 	struct ypresp_key_val yprkv;
 	struct ypreq_key yprk;
@@ -589,44 +710,44 @@ again:
 
 	yprk.domain = indomain;
 	yprk.map = inmap;
-	yprk.keydat.dptr = inkey;
-	yprk.keydat.dsize = inkeylen;
+	yprk.key.keydat_val = inkey;
+	yprk.key.keydat_len = inkeylen;
 	bzero((char *)&yprkv, sizeof yprkv);
 
 	r = clnt_call(ysd->dom_client, YPPROC_NEXT,
 		xdr_ypreq_key, &yprk, xdr_ypresp_key_val, &yprkv, tv);
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_next: clnt_call");
-		ysd->dom_vers = -1;
+		_yp_unbind(ysd);
 		goto again;
 	}
-	if( !(r=ypprot_err(yprkv.status)) ) {
-		*outkeylen = yprkv.keydat.dsize;
+	if( !(r=ypprot_err(yprkv.stat)) ) {
+		*outkeylen = yprkv.key.keydat_len;
 		*outkey = (char *)malloc(*outkeylen+1);
-		bcopy(yprkv.keydat.dptr, *outkey, *outkeylen);
+		bcopy(yprkv.key.keydat_val, *outkey, *outkeylen);
 		(*outkey)[*outkeylen] = '\0';
-		*outvallen = yprkv.valdat.dsize;
+		*outvallen = yprkv.val.valdat_len;
 		*outval = (char *)malloc(*outvallen+1);
-		bcopy(yprkv.valdat.dptr, *outval, *outvallen);
+		bcopy(yprkv.val.valdat_val, *outval, *outvallen);
 		(*outval)[*outvallen] = '\0';
 	}
+
 	xdr_free(xdr_ypresp_key_val, (char *)&yprkv);
-	_yp_unbind(ysd);
 	return r;
 }
 
 int
 yp_all(indomain, inmap, incallback)
-char *indomain;
-char *inmap;
-struct ypall_callback *incallback;
+	char *indomain;
+	char *inmap;
+	struct ypall_callback *incallback;
 {
 	struct ypreq_nokey yprnk;
 	struct dom_binding *ysd;
 	struct timeval tv;
 	struct sockaddr_in clnt_sin;
 	CLIENT *clnt;
-	u_long status;
+	u_long status, savstat;
 	int clnt_sock;
 
 	/* Sanity check */
@@ -635,11 +756,16 @@ struct ypall_callback *incallback;
 	    inmap == NULL || !strlen(inmap))
 		return YPERR_BADARGS;
 
+again:
+
 	if( _yp_dobind(indomain, &ysd) != 0)
 		return YPERR_DOMAIN;
 
 	tv.tv_sec = _yplib_timeout;
 	tv.tv_usec = 0;
+
+	/* YPPROC_ALL manufactures its own channel to ypserv using TCP */
+
 	clnt_sock = RPC_ANYSOCK;
 	clnt_sin = ysd->dom_server_addr;
 	clnt_sin.sin_port = 0;
@@ -654,22 +780,28 @@ struct ypall_callback *incallback;
 	ypresp_allfn = incallback->foreach;
 	ypresp_data = (void *)incallback->data;
 
-	(void) clnt_call(clnt, YPPROC_ALL,
-		xdr_ypreq_nokey, &yprnk, xdr_ypresp_all_seq, &status, tv);
-	clnt_destroy(clnt);
-	xdr_free(xdr_ypresp_all_seq, (char *)&status);	/* not really needed... */
-	_yp_unbind(ysd);
+	if (clnt_call(clnt, YPPROC_ALL,
+		xdr_ypreq_nokey, &yprnk,
+		xdr_ypresp_all_seq, &status, tv) != RPC_SUCCESS) {
+			clnt_perror(ysd->dom_client, "yp_next: clnt_call");
+			clnt_destroy(clnt);
+			_yp_unbind(ysd);
+			goto again;
+	}
 
-	if(status != YP_FALSE)
-		return ypprot_err(status);
+	clnt_destroy(clnt);
+	savstat = status;
+	xdr_free(xdr_ypresp_all_seq, (char *)&status);	/* not really needed... */
+	if(savstat != YP_NOMORE)
+		return ypprot_err(savstat);
 	return 0;
 }
 
 int
 yp_order(indomain, inmap, outorder)
-char *indomain;
-char *inmap;
-int *outorder;
+	char *indomain;
+	char *inmap;
+	int *outorder;
 {
  	struct dom_binding *ysd;
 	struct ypresp_order ypro;
@@ -702,31 +834,29 @@ again:
 	 * NIS+ in YP compat mode doesn't support the YPPROC_ORDER
 	 * procedure.
 	 */
-
 	if (r == RPC_PROCUNAVAIL) {
 		return(YPERR_YPERR);
-		goto bail;
 	}
-
+		
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_order: clnt_call");
-		ysd->dom_vers = -1;
+		_yp_unbind(ysd);
 		goto again;
 	}
 
-	*outorder = ypro.ordernum;
-	xdr_free(xdr_ypresp_order, (char *)&ypro);
+	if( !(r=ypprot_err(ypro.stat)) ) {
+		*outorder = ypro.ordernum;
+	}
 
-bail:
-	_yp_unbind(ysd);
-	return ypprot_err(ypro.status);
+	xdr_free(xdr_ypresp_order, (char *)&ypro);
+	return (r);
 }
 
 int
 yp_master(indomain, inmap, outname)
-char *indomain;
-char *inmap;
-char **outname;
+	char *indomain;
+	char *inmap;
+	char **outname;
 {
 	struct dom_binding *ysd;
 	struct ypresp_master yprm;
@@ -755,20 +885,21 @@ again:
 		xdr_ypreq_nokey, &yprnk, xdr_ypresp_master, &yprm, tv);
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_master: clnt_call");
-		ysd->dom_vers = -1;
+		_yp_unbind(ysd);
 		goto again;
 	}
-	if( !(r=ypprot_err(yprm.status)) ) {
-		*outname = (char *)strdup(yprm.master);
+
+	if( !(r=ypprot_err(yprm.stat)) ) {
+		*outname = (char *)strdup(yprm.peer);
 	}
+
 	xdr_free(xdr_ypresp_master, (char *)&yprm);
-	_yp_unbind(ysd);
-	return r;
+	return (r);
 }
 int
 yp_maplist(indomain, outmaplist)
-char *indomain;
-struct ypmaplist **outmaplist;
+	char *indomain;
+	struct ypmaplist **outmaplist;
 {
 	struct dom_binding *ysd;
 	struct ypresp_maplist ypml;
@@ -790,21 +921,23 @@ again:
 	bzero((char *)&ypml, sizeof ypml);
 
 	r = clnt_call(ysd->dom_client, YPPROC_MAPLIST,
-		xdr_domainname, indomain, xdr_ypresp_maplist, &ypml, tv);
+		xdr_domainname,(char *)&indomain,xdr_ypresp_maplist,&ypml,tv);
 	if (r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_maplist: clnt_call");
-		ysd->dom_vers = -1;
+		_yp_unbind(ysd);
 		goto again;
 	}
-	*outmaplist = ypml.list;
+	if( !(r=ypprot_err(ypml.stat)) ) {
+		*outmaplist = ypml.maps;
+	}
+
 	/* NO: xdr_free(xdr_ypresp_maplist, &ypml);*/
-	_yp_unbind(ysd);
-	return ypprot_err(ypml.status);
+	return (r);
 }
 
 char *
 yperr_string(incode)
-int incode;
+	int incode;
 {
 	static char err[80];
 
@@ -850,7 +983,7 @@ int incode;
 
 int
 ypprot_err(incode)
-unsigned int incode;
+	unsigned int incode;
 {
 	switch(incode) {
 	case YP_TRUE:
@@ -881,7 +1014,7 @@ unsigned int incode;
 
 int
 _yp_check(dom)
-char **dom;
+	char **dom;
 {
 	char *unused;
 
@@ -892,7 +1025,9 @@ char **dom;
 	if(dom)
 		*dom = _yp_domain;
 
-	if( yp_bind(_yp_domain)==0 )
+	if( yp_bind(_yp_domain)==0 ) {
+		yp_unbind(_yp_domain);
 		return 1;
+	}
 	return 0;
 }
