@@ -1,6 +1,6 @@
 /*
- * $FreeBSD: stable/2.1/lib/libcom_err/init_et.c 50952 1999-09-05 11:54:50Z peter $
- * $Source: /pub/FreeBSD/FreeBSD-CVS/src/lib/libcom_err/init_et.c,v $
+ * $Header: /home/ncvs/src/lib/libcom_err/init_et.c,v 1.1.1.1 1995/01/14 22:23:41 wollman Exp $
+ * $Source: /home/ncvs/src/lib/libcom_err/init_et.c,v $
  * $Locker:  $
  *
  * Copyright 1986, 1987, 1988 by MIT Information Systems and
@@ -19,7 +19,7 @@
 
 #ifndef	lint
 static const char rcsid_init_et_c[] =
-  "$FreeBSD: stable/2.1/lib/libcom_err/init_et.c 50952 1999-09-05 11:54:50Z peter $";
+    "$Header: /home/ncvs/src/lib/libcom_err/init_et.c,v 1.1.1.1 1995/01/14 22:23:41 wollman Exp $";
 #endif
 
 extern char *malloc(), *realloc();
