@@ -32,6 +32,7 @@
  * precedence is structured in regular expressions.  Serious changes in
  * regular-expression syntax might require a total rethink.
  */
+#include <limits.h>
 #include <regexp.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -180,10 +181,28 @@ STATIC void regc();
 STATIC void reginsert();
 STATIC void regtail();
 STATIC void regoptail();
+STATIC int  collate_range_cmp();
 #ifdef STRCSPN
 STATIC int strcspn();
 #endif
 
+static int collate_range_cmp (c1, c2)
+	int c1, c2;
+{
+	static char s1[2], s2[2];
+	int ret;
+
+	c1 &= UCHAR_MAX;
+	c2 &= UCHAR_MAX;
+	if (c1 == c2)
+		return (0);
+	s1[0] = c1;
+	s2[0] = c2;
+	if ((ret = strcoll(s1, s2)) != 0)
+		return (ret);
+	return (c1 - c2);
+}
+
 /*
  - regcomp - compile a regular expression into internal code
  *
@@ -489,6 +508,7 @@ int *flagp;
 	case '[': {
 			register int class;
 			register int classend;
+			int i;
 
 			if (*regparse == '^') {	/* Complement of range. */
 				ret = regnode(ANYBUT);
@@ -503,12 +523,16 @@ int *flagp;
 					if (*regparse == ']' || *regparse == '\0')
 						regc('-');
 					else {
-						class = UCHARAT(regparse-2)+1;
+						class = UCHARAT(regparse-2);
 						classend = UCHARAT(regparse);
-						if (class > classend+1)
+						if (collate_range_cmp(class, classend) > 0)
 							FAIL("invalid [] range");
-						for (; class <= classend; class++)
-							regc(class);
+						for (i = 0; i <= UCHAR_MAX; i++)
+							if (   i != class
+							    && collate_range_cmp(class, i) <= 0
+							    && collate_range_cmp(i, classend) <= 0
+							   )
+								regc(i);
 						regparse++;
 					}
 				} else
@@ -888,7 +912,6 @@ char *prog;
 {
 	register char *scan;	/* Current node. */
 	char *next;		/* Next node. */
-	extern char *strchr();
 
 	scan = prog;
 #ifdef DEBUG
@@ -913,16 +936,16 @@ char *prog;
 			break;
 		case WORDA:
 			/* Must be looking at a letter, digit, or _ */
-			if ((!isalnum(*reginput)) && *reginput != '_')
+			if ((!isalnum((unsigned char)*reginput)) && *reginput != '_')
 				return(0);
 			/* Prev must be BOL or nonword */
 			if (reginput > regbol &&
-			    (isalnum(reginput[-1]) || reginput[-1] == '_'))
+			    (isalnum((unsigned char)reginput[-1]) || reginput[-1] == '_'))
 				return(0);
 			break;
 		case WORDZ:
 			/* Must be looking at non letter, digit, or _ */
-			if (isalnum(*reginput) || *reginput == '_')
+			if (isalnum((unsigned char)*reginput) || *reginput == '_')
 				return(0);
 			/* We don't care what the previous char was */
 			break;
