@@ -86,13 +86,17 @@ read__fixio(fd, e)
 
 #ifdef EWOULDBLOCK
     case EWOULDBLOCK:
-# define TRY_AGAIN
+# ifndef TRY_AGAIN
+#  define TRY_AGAIN
+# endif
 #endif /* EWOULDBLOCK */
 
 #if defined(POSIX) && defined(EAGAIN)
 # if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
     case EAGAIN:
-#  define TRY_AGAIN
+#  ifndef TRY_AGAIN
+#   define TRY_AGAIN
+#  endif
 # endif /* EWOULDBLOCK && EWOULDBLOCK != EAGAIN */
 #endif /* POSIX && EAGAIN */
 
@@ -192,10 +196,10 @@ read_getcmd(el, cmdnum, ch)
     el_action_t *cmdnum;
     char *ch;
 {
-    el_action_t  cmd = 0;
+    el_action_t  cmd = ED_UNASSIGNED;
     int     num;
 
-    while (cmd == 0 || cmd == ED_SEQUENCE_LEAD_IN) {
+    while (cmd == ED_UNASSIGNED || cmd == ED_SEQUENCE_LEAD_IN) {
 	if ((num = el_getc(el, ch)) != 1)	/* if EOF or error */
 	    return num;
 
@@ -379,6 +383,11 @@ el_gets(el, nread)
 	    re_refresh_cursor(el);
 	    break;
 
+	case CC_REDISPLAY:
+	    re_clear_lines(el);
+	    re_clear_display(el);
+	    /* FALLTHROUGH */
+
 	case CC_REFRESH:
 	    el->el_state.argument = 1;
 	    el->el_state.doingarg = 0;
