@@ -47,7 +47,7 @@ static char sccsid[] = "@(#)tty.c	8.1 (Berkeley) 6/4/93";
 
 typedef struct ttymodes_t {
     char *m_name;
-    int   m_value;
+    u_int m_value;
     int   m_type;
 } ttymodes_t;
 
@@ -77,7 +77,7 @@ private ttyperm_t ttyperm = {
 		     C_SH(C_PGOFF)|C_SH(C_PAGE)|C_SH(C_STATUS)), 0 }
     },
     {
-	{ "iflag:", 0, IXON | IXOFF },
+	{ "iflag:", 0, IXON | IXOFF | INLCR | ICRNL },
 	{ "oflag:", 0, 0 },
 	{ "cflag:", 0, 0 },
 	{ "lflag:", 0, ISIG | IEXTEN },
@@ -793,7 +793,7 @@ protected int
 tty_rawmode(el)
     EditLine *el;
 {
-    if (el->el_tty.t_mode == ED_IO)
+    if (el->el_tty.t_mode == ED_IO || el->el_tty.t_mode == QU_IO)
 	return (0);
 
     if (tty_getty(el, &el->el_tty.t_ts) == -1) {
