@@ -14,7 +14,7 @@
  * Turned inside out. Now returns xfers as new file ids, not as a special
  * `state' of FTP_t
  *
- * $FreeBSD: stable/2.1/lib/libftpio/ftpio.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/lib/libftpio/ftpio.c 50952 1999-09-05 11:54:50Z peter $
  *
  */
 
@@ -103,7 +103,7 @@ networkInit()
 static int
 check_code(FTP_t ftp, int var, int preferred)
 {
-    ftp->errno = 0;
+    ftp->error = 0;
     while (1) {
 	if (var == preferred)
 	    return 0;
@@ -114,7 +114,7 @@ check_code(FTP_t ftp, int var, int preferred)
 	else if (var == FTP_GENERALLY_HAPPY)	/* general success code */
 	    var = get_a_number(ftp, NULL);
 	else {
-	    ftp->errno = var;
+	    ftp->error = var;
 	    return 1;
 	}
     }
@@ -172,18 +172,21 @@ int
 ftpErrno(FILE *fp)
 {
     FTP_t ftp = fcookie(fp);
-    return ftp->errno;
+    return ftp->error;
 }
 
 const char *
-ftpErrString(int errno)
+ftpErrString(int error)
 {
     int	k;
 
-    if (errno == -1)
+    if (error == -1)
 	return("connection in wrong state");
+    if (error < 100)
+	/* XXX soon UNIX errnos will catch up with FTP protocol errnos */
+	return strerror(error);
     for (k = 0; k < ftpErrListLength; k++)
-      if (ftpErrList[k].num == errno)
+      if (ftpErrList[k].num == error)
 	return(ftpErrList[k].string);
     return("Unknown error");
 }
@@ -282,7 +285,7 @@ ftpLogin(char *host, char *user, char *passwd, int port, int verbose, int *retco
 	if (!n)
 	    *retcode = (FtpTimedOut ? FTP_TIMED_OUT : -1);
 	/* Poor attempt at mapping real errnos to FTP error codes */
-	else switch(n->errno) {
+	else switch(n->error) {
 	    case EADDRNOTAVAIL:
 		*retcode = FTP_TIMED_OUT;	/* Actually no such host, but we have no way of saying that. :-( */
 		break;
@@ -292,7 +295,7 @@ ftpLogin(char *host, char *user, char *passwd, int port, int verbose, int *retco
 		break;
 
 	    default:
-		*retcode = n->errno;
+		*retcode = n->error;
 		break;
 	}
     }
@@ -323,6 +326,8 @@ ftpPassive(FILE *fp, int st)
     i = cmd(ftp, "PASV");
     if (i < 0)
         return i;
+    if (i != FTP_PASSIVE_HAPPY)
+        return FAILURE;
     ftp->is_passive = !ftp->is_passive;
     return SUCCESS;
 }
@@ -417,7 +422,7 @@ get_url_info(char *url_in, char *host_ret, int *port_ret, char *name_ret)
 
     name = host = NULL;
     /* XXX add http:// here or somewhere reasonable at some point XXX */
-    if (strncmp("ftp://", url_in, 6) != NULL)
+    if (strncmp("ftp://", url_in, 6) != 0)
 	return FAILURE;
     /* We like to stomp a lot on the URL string in dissecting it, so copy it first */
     strncpy(url, url_in, BUFSIZ);
@@ -454,7 +459,7 @@ ftp_new(void)
     ftp->is_binary = FALSE;
     ftp->is_passive = FALSE;
     ftp->is_verbose = FALSE;
-    ftp->errno = 0;
+    ftp->error = 0;
     return ftp;
 }
 
@@ -607,26 +612,24 @@ get_a_number(FTP_t ftp, char **q)
 static int
 ftp_close(FTP_t ftp)
 {
-    int i;
+    int i, rcode;
 
+    rcode = FAILURE;
     if (ftp->con_state == isopen) {
 	ftp->con_state = quit;
 	/* If last operation timed out, don't try to quit - just close */
-	if (ftp->errno != FTP_TIMED_OUT)
+	if (ftp->error != FTP_TIMED_OUT)
 	    i = cmd(ftp, "QUIT");
 	else
 	    i = FTP_QUIT_HAPPY;
+	if (!check_code(ftp, i, FTP_QUIT_HAPPY))
+	    rcode = SUCCESS;
 	close(ftp->fd_ctrl);
 	ftp->fd_ctrl = -1;
-	if (check_code(ftp, i, FTP_QUIT_HAPPY)) {
-	    ftp->errno = i;
-	    return FAILURE;
-	}
-	/* Debug("ftp_pkg: ftp_close() - proper shutdown"); */
-	return SUCCESS;
     }
-    /* Debug("ftp_pkg: ftp_close() - improper shutdown"); */
-    return FAILURE;
+    else if (ftp->con_state == quit)
+	rcode = SUCCESS;
+    return rcode;
 }
 
 static int
@@ -676,7 +679,7 @@ ftp_login_session(FTP_t ftp, char *host, char *user, char *passwd, int port, int
 
     if (ftp->con_state != init) {
 	ftp_close(ftp);
-	ftp->errno = -1;
+	ftp->error = -1;
 	return FAILURE;
     }
 
@@ -697,7 +700,7 @@ ftp_login_session(FTP_t ftp, char *host, char *user, char *passwd, int port, int
     else {
 	he = gethostbyname(host);
 	if (!he) {
-	    ftp->errno = 0;
+	    ftp->error = 0;
 	    return FAILURE;
 	}
 	ftp->addrtype = sin.sin_family = he->h_addrtype;
@@ -707,13 +710,13 @@ ftp_login_session(FTP_t ftp, char *host, char *user, char *passwd, int port, int
     sin.sin_port = htons(port);
 
     if ((s = socket(ftp->addrtype, SOCK_STREAM, 0)) < 0) {
-	ftp->errno = -1;
+	ftp->error = -1;
 	return FAILURE;
     }
 
     if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
 	(void)close(s);
-	ftp->errno = errno;
+	ftp->error = errno;
 	return FAILURE;
     }
 
@@ -727,7 +730,7 @@ ftp_login_session(FTP_t ftp, char *host, char *user, char *passwd, int port, int
     if (i >= 299 || i < 0) {
 	ftp_close(ftp);
 	if (i > 0)
-	    ftp->errno = i;
+	    ftp->error = i;
 	return FAILURE;
     }
     return SUCCESS;
@@ -750,7 +753,7 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	return botch("ftp_file_op", "open");
 
     if ((s = socket(ftp->addrtype, SOCK_STREAM, 0)) < 0) {
-	ftp->errno = errno;
+	ftp->error = errno;
 	return FAILURE;
     }
 
@@ -760,7 +763,7 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	if (writes(ftp->fd_ctrl, "PASV\r\n")) {
 	    ftp_close(ftp);
 	    if (FtpTimedOut)
-		ftp->errno = FTP_TIMED_OUT;
+		ftp->error = FTP_TIMED_OUT;
 	    return FTP_TIMED_OUT;
 	}
 	i = get_a_number(ftp, &q);
@@ -792,7 +795,7 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	    i = cmd(ftp, "REST %d", *seekto);
 	    if (i < 0 || FTP_TIMEOUT(i)) {
 		close(s);
-		ftp->errno = i;
+		ftp->error = i;
 		*seekto = (off_t)0;
 		return i;
 	    }
@@ -800,7 +803,7 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	i = cmd(ftp, "%s %s", operation, file);
 	if (i < 0 || i > 299) {
 	    close(s);
-	    ftp->errno = i;
+	    ftp->error = i;
 	    return i;
 	}
 	*fp = fdopen(s, mode);
@@ -837,7 +840,7 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	    i = cmd(ftp, "REST %d", *seekto);
 	    if (i < 0 || FTP_TIMEOUT(i)) {
 		close(s);
-		ftp->errno = i;
+		ftp->error = i;
 		return i;
 	    }
 	    else if (i != 350)
@@ -846,13 +849,13 @@ ftp_file_op(FTP_t ftp, char *operation, char *file, FILE **fp, char *mode, off_t
 	i = cmd(ftp, "%s %s", operation, file);
 	if (i < 0 || i > 299) {
 	    close(s);
-	    ftp->errno = i;
+	    ftp->error = i;
 	    return FAILURE;
 	}
 	fd = accept(s, 0, 0);
 	if (fd < 0) {
 	    close(s);
-	    ftp->errno = 401;
+	    ftp->error = 401;
 	    return FAILURE;
 	}
 	close(s);
