@@ -54,6 +54,8 @@ static char sccsid[] = "@(#)kvm_proc.c	8.3 (Berkeley) 9/23/93";
 #include <sys/ioctl.h>
 #include <sys/tty.h>
 #include <sys/file.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <nlist.h>
 #include <kvm.h>
@@ -71,6 +73,7 @@ static char sccsid[] = "@(#)kvm_proc.c	8.3 (Berkeley) 9/23/93";
 
 #include "kvm_private.h"
 
+#if used
 static char *
 kvm_readswap(kd, p, va, cnt)
 	kvm_t *kd;
@@ -84,6 +87,7 @@ kvm_readswap(kd, p, va, cnt)
 	return(0);
 #endif	/* __FreeBSD__ */
 }
+#endif
 
 #define KREAD(kd, addr, obj) \
 	(kvm_read(kd, addr, (char *)(obj), sizeof(*obj)) != sizeof(*obj))
@@ -107,14 +111,14 @@ kvm_proclist(kd, what, arg, p, bp, maxcnt)
 	struct tty tty;
 	struct proc proc;
 
-	for (; cnt < maxcnt && p != NULL; p = proc.p_next) {
+	for (; cnt < maxcnt && p != NULL; p = proc.p_list.le_next) {
 		if (KREAD(kd, (u_long)p, &proc)) {
 			_kvm_err(kd, kd->program, "can't read proc at %x", p);
 			return (-1);
 		}
 		if (KREAD(kd, (u_long)proc.p_cred, &eproc.e_pcred) == 0)
-			KREAD(kd, (u_long)eproc.e_pcred.pc_ucred,
-			      &eproc.e_ucred);
+			(void)(KREAD(kd, (u_long)eproc.e_pcred.pc_ucred,
+			             &eproc.e_ucred));
 
 		switch(what) {
 
@@ -264,7 +268,8 @@ kvm_getprocs(kd, op, arg, cnt)
 	int op, arg;
 	int *cnt;
 {
-	int mib[4], size, st, nprocs;
+	int mib[4], st, nprocs;
+	size_t size;
 
 	if (kd->procbase != 0) {
 		free((void *)kd->procbase);
@@ -280,15 +285,21 @@ kvm_getprocs(kd, op, arg, cnt)
 		mib[1] = KERN_PROC;
 		mib[2] = op;
 		mib[3] = arg;
-		st = sysctl(mib, 4, NULL, &size, NULL, 0);
+		st = sysctl(mib, op == KERN_PROC_ALL ? 3 : 4, NULL, &size, NULL, 0);
 		if (st == -1) {
 			_kvm_syserr(kd, kd->program, "kvm_getprocs");
 			return (0);
 		}
-		kd->procbase = (struct kinfo_proc *)_kvm_malloc(kd, size);
-		if (kd->procbase == 0)
-			return (0);
-		st = sysctl(mib, 4, kd->procbase, &size, NULL, 0);
+		kd->procbase = 0;
+		do {
+			size += size / 10;
+			kd->procbase = (struct kinfo_proc *)
+				_kvm_realloc(kd, kd->procbase, size);
+			if (kd->procbase == 0)
+				return (0);
+			st = sysctl(mib, op == KERN_PROC_ALL ? 3 : 4,
+				    kd->procbase, &size, NULL, 0);
+		} while (st == -1 && errno == ENOMEM);
 		if (st == -1) {
 			_kvm_syserr(kd, kd->program, "kvm_getprocs");
 			return (0);
@@ -413,17 +424,17 @@ kvm_argv(kd, p, addr, narg, maxcnt)
 	 *              arrays are left pointing to the collected strings.
 	 */
 	if (kd->argspc == 0) {
-		kd->argspc = (char *)_kvm_malloc(kd, NBPG);
+		kd->argspc = (char *)_kvm_malloc(kd, PAGE_SIZE);
 		if (kd->argspc == 0)
 			return (0);
-		kd->arglen = NBPG;
+		kd->arglen = PAGE_SIZE;
 	}
 	/*
 	 * kd->argbuf : used to pull in pages from the target process.
 	 *              the strings are copied out of here.
 	 */
 	if (kd->argbuf == 0) {
-		kd->argbuf = (char *)_kvm_malloc(kd, NBPG);
+		kd->argbuf = (char *)_kvm_malloc(kd, PAGE_SIZE);
 		if (kd->argbuf == 0)
 			return (0);
 	}
@@ -454,22 +465,22 @@ kvm_argv(kd, p, addr, narg, maxcnt)
 	while (argv < kd->argv + narg && *argv != 0) {
 
 		/* get the address that the current argv string is on */
-		addr = (u_long)*argv & ~(NBPG - 1);
+		addr = (u_long)*argv & ~(PAGE_SIZE - 1);
 
 		/* is it the same page as the last one? */
 		if (addr != oaddr) {
-			if (kvm_uread(kd, p, addr, kd->argbuf, NBPG) !=
-			    NBPG)
+			if (kvm_uread(kd, p, addr, kd->argbuf, PAGE_SIZE) !=
+			    PAGE_SIZE)
 				return (0);
 			oaddr = addr;
 		}
 
 		/* offset within the page... kd->argbuf */
-		addr = (u_long)*argv & (NBPG - 1);
+		addr = (u_long)*argv & (PAGE_SIZE - 1);
 
 		/* cp = start of string, cc = count of chars in this chunk */
 		cp = kd->argbuf + addr;
-		cc = NBPG - addr;
+		cc = PAGE_SIZE - addr;
 
 		/* dont get more than asked for by user process */
 		if (maxcnt > 0 && cc > maxcnt - len)
@@ -599,13 +610,23 @@ kvm_doargv(kd, kp, nchr, info)
 	register char **ap;
 	u_long addr;
 	int cnt;
-	struct ps_strings arginfo;
+	struct ps_strings arginfo, *ps_strings;
+	int mib[2];
+	size_t len;
+
+	ps_strings = NULL;
+	mib[0] = CTL_KERN;
+	mib[1] = KERN_PS_STRINGS;
+	len = sizeof(ps_strings);
+	if (sysctl(mib, 2, &ps_strings, &len, NULL, 0) < 0 ||
+	    ps_strings == NULL)
+		ps_strings = PS_STRINGS;
 
 	/*
 	 * Pointers are stored at the top of the user stack.
 	 */
 	if (p->p_stat == SZOMB ||
-	    kvm_uread(kd, p, USRSTACK - sizeof(arginfo), (char *)&arginfo,
+	    kvm_uread(kd, p, ps_strings, (char *)&arginfo,
 		      sizeof(arginfo)) != sizeof(arginfo))
 		return (0);
 
@@ -649,7 +670,7 @@ kvm_getenvv(kd, kp, nchr)
 ssize_t
 kvm_uread(kd, p, uva, buf, len)
 	kvm_t *kd;
-	register struct proc *p;
+	register const struct proc *p;
 	register u_long uva;
 	register char *buf;
 	register size_t len;
@@ -659,6 +680,11 @@ kvm_uread(kd, p, uva, buf, len)
 	ssize_t amount;
 	int fd;
 
+	if (!ISALIVE(kd)) {
+		_kvm_err(kd, kd->program, "cannot read user space from dead kernel");
+		return(0);
+	}
+
 	cp = buf;
 
 	sprintf(procfile, "/proc/%d/mem", p->p_pid);
