@@ -1,5 +1,5 @@
 /* MD2C.C - RSA Data Security, Inc., MD2 message-digest algorithm
- * $FreeBSD: stable/2.1/lib/libmd/md2c.c 6684 1995-02-24 08:51:34Z phk $
+ * $FreeBSD: stable/2.2/lib/libmd/md2c.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 /* Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. All
@@ -22,11 +22,12 @@
 
 #include "md2.h"
 #include <string.h>
+#include <sys/types.h>
 
 
 typedef unsigned char *POINTER;
-typedef unsigned short int UINT2;
-typedef unsigned long int UINT4;
+typedef u_int16_t UINT2;
+typedef u_int32_t UINT4;
 
 #define PROTO_LIST(list) list
 
