@@ -1,5 +1,5 @@
 /* MD4.H - header file for MD4C.C
- * $FreeBSD: stable/2.1/lib/libmd/md4.h 12981 1995-12-22 22:09:31Z peter $
+ * $FreeBSD: stable/2.2/lib/libmd/md4.h 50952 1999-09-05 11:54:50Z peter $
  */
 
 /* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
@@ -27,16 +27,20 @@
 #define _MD4_H_
 /* MD4 context. */
 typedef struct MD4Context {
-  unsigned long state[4];	/* state (ABCD) */
-  unsigned long count[2];	/* number of bits, modulo 2^64 (lsb first) */
+  u_int32_t state[4];	/* state (ABCD) */
+  u_int32_t count[2];	/* number of bits, modulo 2^64 (lsb first) */
   unsigned char buffer[64];	/* input buffer */
 } MD4_CTX;
 
+#include <sys/cdefs.h>
+
+__BEGIN_DECLS
 void   MD4Init(MD4_CTX *);
 void   MD4Update(MD4_CTX *, const unsigned char *, unsigned int);
 void   MD4Final(unsigned char [16], MD4_CTX *);
 char * MD4End(MD4_CTX *, char *);
-char * MD4File(char *, char *);
+char * MD4File(const char *, char *);
 char * MD4Data(const unsigned char *, unsigned int, char *);
+__END_DECLS
 
 #endif /* _MD4_H_ */
