@@ -1,5 +1,5 @@
 /* MD4C.C - RSA Data Security, Inc., MD4 message-digest algorithm
- * $FreeBSD: stable/2.1/lib/libmd/md4c.c 6684 1995-02-24 08:51:34Z phk $
+ * $FreeBSD: stable/2.2/lib/libmd/md4c.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 /* Copyright (C) 1990-2, RSA Data Security, Inc. All rights reserved.
@@ -23,12 +23,13 @@
    documentation and/or software.
  */
 
-#include "md4.h"
+#include <sys/types.h>
 #include <string.h>
+#include "md4.h"
 
 typedef unsigned char *POINTER;
-typedef unsigned short int UINT2;
-typedef unsigned long int UINT4;
+typedef u_int16_t UINT2;
+typedef u_int32_t UINT4;
 
 #define PROTO_LIST(list) list
 
