@@ -11,8 +11,6 @@
  *
  */
 
-#include "version.h"
-
 #ifndef __GNUC__
 #define inline
 #endif
@@ -32,8 +30,21 @@ typedef struct sigaction sigaction_t;
 #define FG(n)	((n) & 0x0f)
 #define BG(n)	(((n) & 0xf0) >> 4)
 
+#define TextOf(c)    ((c) & (chtype)A_CHARTEXT)
+#define AttrOf(c)    ((c) & (chtype)A_ATTRIBUTES)
+
+#define BLANK        (' '|A_NORMAL)
+
 #define CHANGED     -1
 
+#define ALL_BUT_COLOR ((chtype)~(A_COLOR))
+
+/* Macro to put together character and attribute info and return it.
+   If colors are in the attribute, they have precedence. */
+#define ch_or_attr(ch,at) \
+    ((PAIR_NUMBER(at) > 0) ? \
+     ((((chtype)ch) & ALL_BUT_COLOR) | (at)) : ((((chtype)ch) | (at))))
+
 extern WINDOW	*newscr;
 
 #ifdef TRACE
@@ -51,6 +62,10 @@ extern void init_acs(void);
 extern void tstp(int);
 extern WINDOW *makenew(int, int, int, int);
 extern int timed_wait(int fd, int wait, int *timeleft);
+extern chtype _nc_background(WINDOW *);
+extern chtype _nc_render(WINDOW *, chtype);
+extern int _nc_keypad(bool);
+extern void _nc_scroll_window(WINDOW *, int, short const, short const, chtype);
 
 struct try {
         struct try      *child;     /* ptr to child.  NULL if none          */
