@@ -10,7 +10,7 @@ static void insert __P((char *s, int x, int start, int length));
 static int wsrch __P((char *w,int low,int high));
 
 /* Dictionary for integer-word translations */
-char Wp[2048][4] = {
+static char Wp[2048][4] = {
 "A",
 "ABE",
 "ACE",
@@ -2107,7 +2107,8 @@ char *out;
 char *e;
 {
 	char *word, *cp;
-	int i, p, v,l, low,high;
+	int i, v,l, low,high;
+	unsigned int p;
 	char b[9];
 	char input[36];
 
