@@ -4,24 +4,23 @@
 
 /* skey_getpass - read regular or s/key password */
 
-char   *skey_getpass(prompt, pwd, pwok)
-char   *prompt;
+char *skey_getpass(prompt, pwd, pwok)
+const char *prompt;
 struct passwd *pwd;
 int     pwok;
 {
     static char buf[128];
     struct skey skey;
     char   *pass;
-    char   *username = pwd ? pwd->pw_name : "nope";
     int     sflag;
 
     /* Attempt an s/key challenge. */
-    sflag = skeyinfo(&skey, username, buf);
-    if (!sflag)
+    sflag = (pwd == NULL || skeyinfo(&skey, pwd->pw_name, buf));
+    if (!sflag) {
 	printf("%s\n", buf);
-
-    if (!pwok)
-	printf("(s/key required)\n");
+	if (!pwok)
+	    printf("(s/key required)\n");
+    }
 
     pass = getpass(prompt);
 
