@@ -72,6 +72,9 @@ static struct in_addr *lookup_internet_addr();
 #ifndef CONSOLE
 #define CONSOLE		"console"
 #endif
+#ifndef VTY_PREFIX
+#define VTY_PREFIX      "ttyv"
+#endif
 
 struct login_info {
     char   *host_name;			/* host name */
@@ -80,6 +83,8 @@ struct login_info {
     char   *port;			/* login port */
 };
 
+static int _skeyaccess __P(( FILE *, struct login_info * ));
+
 /* skeyaccess - find out if UNIX passwords are permitted */
 
 int     skeyaccess(user, port, host, addr)
@@ -157,17 +162,21 @@ char   *addr;
 
 /* _skeyaccess - find out if UNIX passwords are permitted */
 
-int     _skeyaccess(fp, login_info)
+static int _skeyaccess(fp, login_info)
 FILE   *fp;
 struct login_info *login_info;
 {
     char    buf[BUFSIZ];
     char   *tok;
     int     match;
-    int     permission;
+    int     permission=DENY;
 
 #ifdef PERMIT_CONSOLE
-    if (login_info->port != 0 && strcasecmp(login_info->port, CONSOLE) == 0)
+    if (login_info->port != 0 &&
+	(strcmp(login_info->port, CONSOLE) == 0 ||
+	 strncmp(login_info->port, VTY_PREFIX, sizeof(VTY_PREFIX) - 1) == 0
+	)
+       )
 	return (1);
 #endif
 
@@ -220,9 +229,9 @@ struct login_info *login_info;
 static int match_internet_addr(login_info)
 struct login_info *login_info;
 {
-    char   *tok;
-    long    pattern;
-    long    mask;
+    char *     tok;
+    u_int32_t  pattern;
+    u_int32_t  mask;
     struct in_addr *addrp;
 
     if (login_info->internet_addr == 0)
@@ -240,7 +249,7 @@ struct login_info *login_info;
      */
 
     for (addrp = login_info->internet_addr; addrp->s_addr; addrp++)
-	if (addrp->s_addr != -1 && (addrp->s_addr & mask) == pattern)
+	if (addrp->s_addr != INADDR_NONE && (addrp->s_addr & mask) == pattern)
 	    return (1);
     return (0);
 }
@@ -313,7 +322,7 @@ static char *get_token()
 {
     char   *cp;
 
-    if (cp = prev_token) {
+    if ( (cp = prev_token) ) {
 	prev_token = 0;
     } else {
 	while ((cp = strsep(&line_pointer, " \t")) != NULL && *cp == '\0')
