@@ -12,6 +12,7 @@
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -21,11 +22,15 @@
 #include "skey.h"
 #include "pathnames.h"
 
-char *skipspace();
-int skeylookup __P((struct skey *mp,char *name));
+static char *skipspace();
 
 #define setpriority(x,y,z)	/* nothing */
 
+static char *month[12] = {
+	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+};
+
 /* Look up skey info for user 'name'. If successful, fill in the caller's
  * skey structure and return 0. If unsuccessful (e.g., if name is unknown)
  * return -1. If an optional challenge string buffer is given, update it.
@@ -36,7 +41,7 @@ int skeylookup __P((struct skey *mp,char *name));
 int
 skeyinfo(mp,name,ss)
 struct skey *mp;
-char *name;
+const char *name;
 char *ss;
 {
 	int rval;
@@ -68,7 +73,7 @@ char *ss;
 int
 skeychallenge(mp,name, ss)
 struct skey *mp;
-char *name;
+const char *name;
 char *ss;
 {
 	int rval;
@@ -96,18 +101,20 @@ char *ss;
 int
 skeylookup(mp,name)
 struct skey *mp;
-char *name;
+const char *name;
 {
 	int found;
-	int len;
-	long recstart;
+	size_t len;
+	long recstart = 0;
 	char *cp, *p;
 	struct stat statbuf;
+	mode_t oldmask;
 
 	/* See if the _PATH_SKEYFILE exists, and create it if not */
 	if(stat(_PATH_SKEYFILE,&statbuf) == -1 && errno == ENOENT){
+		oldmask = umask(S_IRWXG|S_IRWXO);
 		mp->keyfile = fopen(_PATH_SKEYFILE,"w+");
-		(void) chmod(_PATH_SKEYFILE, 0644);
+		(void)umask(oldmask);
 	} else {
 		/* Otherwise open normally for update */
 		mp->keyfile = fopen(_PATH_SKEYFILE,"r+");
@@ -172,21 +179,19 @@ skeyverify(mp,response)
 struct skey *mp;
 char *response;
 {
- struct timeval startval;
- struct timeval endval;
 	char key[8];
 	char fkey[8];
 	char filekey[8];
 	time_t now;
 	struct tm *tm;
-	char tbuf[27],buf[60];
-	char me[80];
-	int rval;
+	char tbuf[27], fbuf[20];
 	char *cp, *p;
 
 	time(&now);
 	tm = localtime(&now);
-	strftime(tbuf, sizeof(tbuf), " %b %d,%Y %T", tm);
+/* can't use %b here, because it can be in national form */
+	strftime(fbuf, sizeof(fbuf), "%d,%Y %T", tm);
+	snprintf(tbuf, sizeof(tbuf), " %s %s", month[tm->tm_mon], fbuf);
 
 	if(response == NULL){
 		fclose(mp->keyfile);
@@ -264,6 +269,7 @@ char *response;
 /* Convert 8-byte hex-ascii string to binary array
  * Returns 0 on success, -1 on error
  */
+int
 atob8(out,in)
 register char *out,*in;
 {
@@ -289,6 +295,7 @@ register char *out,*in;
 	return 0;
 }
 
+static
 char *
 skipspace(cp)
 register char *cp;
