@@ -45,11 +45,12 @@ static char sccsid[] = "@(#)termcap.c	8.1 (Berkeley) 6/4/93";
 #include <string.h>
 #include <unistd.h>
 #include <termios.h>
+#include <signal.h>
+#include <errno.h>
+#include <sys/syscall.h>
 #include "termcap.h"
 #include "pathnames.h"
 
-extern void __set_ospeed(speed_t speed);
-
 /*
  * termcap - routines for dealing with the terminal capability data base
  *
@@ -66,6 +67,8 @@ extern void __set_ospeed(speed_t speed);
 
 static	char *tbuf;	/* termcap buffer */
 
+static	int use_issetugid = 1;	/* 0 = not present, 1 = try it, 2 = exists */
+
 /*
  * Get an entry for terminal name in buffer bp from the termcap file.
  */
@@ -101,12 +104,13 @@ tgetent(char *bp, const char *name)
 	 * becomes "$HOME/.termcap /etc/termcap" if no TERMPATH exists.
 	 */
 	if (!cp || *cp != '/') {	/* no TERMCAP or it holds an entry */
-		if (termpath = getenv("TERMPATH"))
+		if ( (termpath = getenv("TERMPATH")) )
 			strncpy(pathbuf, termpath, PBUFSIZ);
 		else {
-			if (home = getenv("HOME")) {	/* set up default */
-				p += strlen(home);	/* path, looking in */
-				strcpy(pathbuf, home);	/* $HOME first */
+			if ( (home = getenv("HOME")) ) {/* set up default */
+				strncpy(pathbuf, home, PBUFSIZ - 1); /* $HOME first */
+				pathbuf[PBUFSIZ - 2] = '\0'; /* -2 because we add a slash */
+				p += strlen(pathbuf);	/* path, looking in */
 				*p++ = '/';
 			}	/* if no $HOME look in current directory */
 			strncpy(p, _PATH_DEF, PBUFSIZ - (p - pathbuf));
@@ -114,6 +118,44 @@ tgetent(char *bp, const char *name)
 	}
 	else				/* user-defined name in TERMCAP */
 		strncpy(pathbuf, cp, PBUFSIZ);	/* still can be tokenized */
+	pathbuf[PBUFSIZ - 1] = '\0';
+
+#ifdef SYS_issetugid
+	/*
+	 * This stuff is to try and detect the presence of the issetugid()
+	 * syscall without breaking dynamic linking during compiles.. Sigh.
+	 */
+othersyscall:
+	if (use_issetugid) {
+		struct sigaction sa, osa;
+		int sigsys_installed = 0;
+		int unsafe;
+
+		if (use_issetugid == 1) {
+			bzero(&sa, sizeof(sa));
+			sa.sa_handler = SIG_IGN;
+			if (sigaction(SIGSYS, &sa, &osa) >= 0)
+				sigsys_installed = 1;
+		}
+		errno = 0;
+		unsafe = syscall(SYS_issetugid);
+		if (sigsys_installed == 1) {
+			int oerrno = errno;
+			sigaction(SIGSYS, &osa, NULL);
+			errno = oerrno;
+		}
+		if (errno == ENOSYS || errno == EINVAL) {
+			use_issetugid = 0;
+			goto othersyscall;
+		} else if (sigsys_installed == 1)
+			sigsys_installed = 2;
+		if (unsafe)
+			strcpy(pathbuf, _PATH_DEF_SEC);
+	} else
+#endif	/* SYS_issetugid */
+		/* issetugid() not in kernel or undefined - try second best */	
+		if (getuid() != geteuid() || getgid() != getegid())
+			strcpy(pathbuf, _PATH_DEF_SEC);
 
 	*fname++ = pathbuf;	/* tokenize path into vector of names */
 	while (*++p)
