@@ -32,7 +32,12 @@
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
+#if 0
 static char sccsid[] = "@(#)pty.c	8.3 (Berkeley) 5/16/94";
+#else
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/lib/libutil/pty.c 50952 1999-09-05 11:54:50Z peter $";
+#endif
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
@@ -47,7 +52,9 @@ static char sccsid[] = "@(#)pty.c	8.3 (Berkeley) 5/16/94";
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
+#include <libutil.h>
 
+int
 openpty(amaster, aslave, name, termp, winp)
 	int *amaster, *aslave;
 	char *name;
@@ -98,6 +105,7 @@ openpty(amaster, aslave, name, termp, winp)
 	return (-1);
 }
 
+int
 forkpty(amaster, name, termp, winp)
 	int *amaster;
 	char *name;
