@@ -1,9 +1,9 @@
 /* adler32.c -- compute the Adler-32 checksum of a data stream
- * Copyright (C) 1995-1996 Mark Adler
+ * Copyright (C) 1995-1998 Mark Adler
  * For conditions of distribution and use, see copyright notice in zlib.h 
  */
 
-/* $FreeBSD: stable/2.1/lib/libz/adler32.c 50952 1999-09-05 11:54:50Z peter $ */
+/* @(#) $FreeBSD: stable/2.2/lib/libz/adler32.c 52082 1999-10-10 07:53:09Z peter $ */
 
 #include "zlib.h"
 
@@ -18,7 +18,7 @@
 #define DO16(buf)   DO8(buf,0); DO8(buf,8);
 
 /* ========================================================================= */
-uLong adler32(adler, buf, len)
+uLong ZEXPORT adler32(adler, buf, len)
     uLong adler;
     const Bytef *buf;
     uInt len;
