@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_acos.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_acos.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* __ieee754_acos(x)
@@ -35,7 +35,7 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_acos.c 50952 1999-09-
  *	if x is NaN, return x itself;
  *	if |x|>1, return NaN with invalid signal.
  *
- * Function needed: sqrt
+ * Function needed: __ieee754_sqrt
  */
 
 #include "math.h"
@@ -62,9 +62,9 @@ qS3 = -6.88283971605453293030e-01, /* 0xBFE6066C, 0x1B8D0159 */
 qS4 =  7.70381505559019352791e-02; /* 0x3FB3B8C5, 0xB12E9282 */
 
 #ifdef __STDC__
-	double __ieee754_acos(double x)
+	double __generic___ieee754_acos(double x)
 #else
-	double __ieee754_acos(x)
+	double __generic___ieee754_acos(x)
 	double x;
 #endif
 {
@@ -92,13 +92,13 @@ qS4 =  7.70381505559019352791e-02; /* 0x3FB3B8C5, 0xB12E9282 */
 	    z = (one+x)*0.5;
 	    p = z*(pS0+z*(pS1+z*(pS2+z*(pS3+z*(pS4+z*pS5)))));
 	    q = one+z*(qS1+z*(qS2+z*(qS3+z*qS4)));
-	    s = sqrt(z);
+	    s = __ieee754_sqrt(z);
 	    r = p/q;
 	    w = r*s-pio2_lo;
 	    return pi - 2.0*(s+w);
 	} else {			/* x > 0.5 */
 	    z = (one-x)*0.5;
-	    s = sqrt(z);
+	    s = __ieee754_sqrt(z);
 	    df = s;
 	    SET_LOW_WORD(df,0);
 	    c  = (z-df*df)/(s+df);
