@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_acosf.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_acosf.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "math.h"
@@ -68,14 +68,14 @@ qS4 =  7.7038154006e-02; /* 0x3d9dc62e */
 	    z = (one+x)*(float)0.5;
 	    p = z*(pS0+z*(pS1+z*(pS2+z*(pS3+z*(pS4+z*pS5)))));
 	    q = one+z*(qS1+z*(qS2+z*(qS3+z*qS4)));
-	    s = sqrtf(z);
+	    s = __ieee754_sqrtf(z);
 	    r = p/q;
 	    w = r*s-pio2_lo;
 	    return pi - (float)2.0*(s+w);
 	} else {			/* x > 0.5 */
 	    int32_t idf;
 	    z = (one-x)*(float)0.5;
-	    s = sqrtf(z);
+	    s = __ieee754_sqrtf(z);
 	    df = s;
 	    GET_FLOAT_WORD(idf,df);
 	    SET_FLOAT_WORD(df,idf&0xfffff000);
