@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_acoshf.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_acoshf.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "math.h"
@@ -49,9 +49,9 @@ ln2	= 6.9314718246e-01;  /* 0x3f317218 */
 	    return 0.0;			/* acosh(1) = 0 */
 	} else if (hx > 0x40000000) {	/* 2**28 > x > 2 */
 	    t=x*x;
-	    return __ieee754_logf((float)2.0*x-one/(x+sqrtf(t-one)));
+	    return __ieee754_logf((float)2.0*x-one/(x+__ieee754_sqrtf(t-one)));
 	} else {			/* 1<x<2 */
 	    t = x-one;
-	    return log1pf(t+sqrtf((float)2.0*t+t*t));
+	    return log1pf(t+__ieee754_sqrtf((float)2.0*t+t*t));
 	}
 }
