@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_asinf.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_asinf.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "math.h"
@@ -49,7 +49,7 @@ qS4 =  7.7038154006e-02; /* 0x3d9dc62e */
 	float x;
 #endif
 {
-	float t,w,p,q,c,r,s;
+	float t=0.0,w,p,q,c,r,s;
 	int32_t hx,ix;
 	GET_FLOAT_WORD(hx,x);
 	ix = hx&0x7fffffff;
@@ -73,7 +73,7 @@ qS4 =  7.7038154006e-02; /* 0x3d9dc62e */
 	t = w*(float)0.5;
 	p = t*(pS0+t*(pS1+t*(pS2+t*(pS3+t*(pS4+t*pS5)))));
 	q = one+t*(qS1+t*(qS2+t*(qS3+t*qS4)));
-	s = sqrtf(t);
+	s = __ieee754_sqrtf(t);
 	if(ix>=0x3F79999A) { 	/* if |x| > 0.975 */
 	    w = p/q;
 	    t = pio2_hi-((float)2.0*(s+s*w)-pio2_lo);
