@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_exp.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_exp.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* __ieee754_exp(x)
@@ -104,14 +104,14 @@ P5   =  4.13813679705723846039e-08; /* 0x3E663769, 0x72BEA4D0 */
 
 
 #ifdef __STDC__
-	double __ieee754_exp(double x)	/* default IEEE double exp */
+	double __generic___ieee754_exp(double x)	/* default IEEE double exp */
 #else
-	double __ieee754_exp(x)	/* default IEEE double exp */
+	double __generic___ieee754_exp(x)	/* default IEEE double exp */
 	double x;
 #endif
 {
-	double y,hi,lo,c,t;
-	int32_t k,xsb;
+	double y,hi=0.0,lo=0.0,c,t;
+	int32_t k=0,xsb;
 	u_int32_t hx;
 
 	GET_HIGH_WORD(hx,x);
