@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_expf.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_expf.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include "math.h"
@@ -49,8 +49,8 @@ P5   =  4.1381369442e-08; /* 0x3331bb4c */
 	float x;
 #endif
 {
-	float y,hi,lo,c,t;
-	int32_t k,xsb;
+	float y,hi=0.0,lo=0.0,c,t;
+	int32_t k=0,xsb;
 	u_int32_t hx;
 
 	GET_FLOAT_WORD(hx,x);
