@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_scalb.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_scalb.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -25,16 +25,16 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_scalb.c 50952 1999-09
 
 #ifdef _SCALB_INT
 #ifdef __STDC__
-	double __ieee754_scalb(double x, int fn)
+	double __generic___ieee754_scalb(double x, int fn)
 #else
-	double __ieee754_scalb(x,fn)
+	double __generic___ieee754_scalb(x,fn)
 	double x; int fn;
 #endif
 #else
 #ifdef __STDC__
-	double __ieee754_scalb(double x, double fn)
+	double __generic___ieee754_scalb(double x, double fn)
 #else
-	double __ieee754_scalb(x,fn)
+	double __generic___ieee754_scalb(x,fn)
 	double x, fn;
 #endif
 #endif
