@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/e_sqrt.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/e_sqrt.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* __ieee754_sqrt(x)
@@ -94,9 +94,9 @@ static	double	one	= 1.0, tiny=1.0e-300;
 #endif
 
 #ifdef __STDC__
-	double __ieee754_sqrt(double x)
+	double __generic___ieee754_sqrt(double x)
 #else
-	double __ieee754_sqrt(x)
+	double __generic___ieee754_sqrt(x)
 	double x;
 #endif
 {
