@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_atan.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_atan.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* atan(x)
@@ -86,9 +86,9 @@ one   = 1.0,
 huge   = 1.0e300;
 
 #ifdef __STDC__
-	double atan(double x)
+	double __generic_atan(double x)
 #else
-	double atan(x)
+	double __generic_atan(x)
 	double x;
 #endif
 {
