@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_ceil.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_ceil.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -33,9 +33,9 @@ static double huge = 1.0e300;
 #endif
 
 #ifdef __STDC__
-	double ceil(double x)
+	double __generic_ceil(double x)
 #else
-	double ceil(x)
+	double __generic_ceil(x)
 	double x;
 #endif
 {
