@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_cos.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_cos.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* cos(x)
@@ -49,9 +49,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_cos.c 50952 1999-09-0
 #include "math_private.h"
 
 #ifdef __STDC__
-	double cos(double x)
+	double __generic_cos(double x)
 #else
-	double cos(x)
+	double __generic_cos(x)
 	double x;
 #endif
 {
