@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_finite.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_finite.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -23,9 +23,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_finite.c 50952 1999-0
 #include "math_private.h"
 
 #ifdef __STDC__
-	int finite(double x)
+	int __generic_finite(double x)
 #else
-	int finite(x)
+	int __generic_finite(x)
 	double x;
 #endif
 {
