@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_ilogb.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_ilogb.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* ilogb(double x)
@@ -24,9 +24,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_ilogb.c 50952 1999-09
 #include "math_private.h"
 
 #ifdef __STDC__
-	int ilogb(double x)
+	int __generic_ilogb(double x)
 #else
-	int ilogb(x)
+	int __generic_ilogb(x)
 	double x;
 #endif
 {
