@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_logb.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_logb.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -24,9 +24,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_logb.c 50952 1999-09-
 #include "math_private.h"
 
 #ifdef __STDC__
-	double logb(double x)
+	double __generic_logb(double x)
 #else
-	double logb(x)
+	double __generic_logb(x)
 	double x;
 #endif
 {
