@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_rint.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_rint.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -45,9 +45,9 @@ TWO52[2]={
 };
 
 #ifdef __STDC__
-	double rint(double x)
+	double __generic_rint(double x)
 #else
-	double rint(x)
+	double __generic_rint(x)
 	double x;
 #endif
 {
