@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_sin.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_sin.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* sin(x)
@@ -49,9 +49,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_sin.c 50952 1999-09-0
 #include "math_private.h"
 
 #ifdef __STDC__
-	double sin(double x)
+	double __generic_sin(double x)
 #else
-	double sin(x)
+	double __generic_sin(x)
 	double x;
 #endif
 {
