@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_tan.c 50952 1999-09-05 11:54:50Z peter $";
+static char rcsid[] = "$FreeBSD: stable/2.2/lib/msun/src/s_tan.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /* tan(x)
@@ -48,9 +48,9 @@ static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/s_tan.c 50952 1999-09-0
 #include "math_private.h"
 
 #ifdef __STDC__
-	double tan(double x)
+	double __generic_tan(double x)
 #else
-	double tan(x)
+	double __generic_tan(x)
 	double x;
 #endif
 {
