@@ -18,13 +18,11 @@ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
+
+ $FreeBSD: stable/2.2/libexec/bootpd/hash.c 50952 1999-09-05 11:54:50Z peter $
+
 ************************************************************************/
 
-#ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/libexec/bootpd/hash.c 50952 1999-09-05 11:54:50Z peter $";
-#endif
-
-
 /*
  * Generalized hash table ADT
  *
