@@ -18,13 +18,11 @@ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
+
+ $FreeBSD: stable/2.2/libexec/bootpd/tools/bootpef/bootpef.c 50952 1999-09-05 11:54:50Z peter $
+
 ************************************************************************/
 
-#ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/libexec/bootpd/tools/bootpef/bootpef.c 50952 1999-09-05 11:54:50Z peter $";
-#endif
-
-
 /*
  * bootpef - BOOTP Extension File generator
  *	Makes an "Extension File" for each host entry that
@@ -97,7 +95,6 @@ static char rcsid[] = "$FreeBSD: stable/2.1/libexec/bootpd/tools/bootpef/bootpef
 #define P(args) ()
 #endif
 
-static void dovend_rfc1048 P((struct bootp *, struct host *, int32));
 static void mktagfile P((struct host *));
 static void usage P((void));
 
@@ -291,7 +288,6 @@ mktagfile(hp)
 	FILE *fp;
 	int bytesleft, len;
 	byte *vp;
-	char *tmpstr;
 
 	if (!hp->flags.exten_file)
 		return;
@@ -332,7 +328,7 @@ mktagfile(hp)
 	}
 	fclose(fp);
 
-} /* dovend_rfc1048 */
+} /* mktagfile */
 
 /*
  * Local Variables:
