@@ -22,19 +22,20 @@
  *
  * This file was copied from tcpdump-2.1.1 and modified.
  * There is an e-mail list for tcpdump: <tcpdump@ee.lbl.gov>
+ *
+ * $FreeBSD: stable/2.2/libexec/bootpd/tools/bootptest/print-bootp.c 50952 1999-09-05 11:54:50Z peter $
  */
-#ifndef lint
-static char rcsid[] = "$FreeBSD: stable/2.1/libexec/bootpd/tools/bootptest/print-bootp.c 50952 1999-09-05 11:54:50Z peter $";
-/* 93/10/10 <gwr@mc.com> New data-driven option print routine. */
-#endif
 
 #include <stdio.h>
 
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+
+#include <sys/time.h>	/* for struct timeval in net/if.h */
 #include <net/if.h>
 #include <netinet/in.h>
+
 #include <string.h>
 #include <ctype.h>
 
@@ -42,6 +43,7 @@ static char rcsid[] = "$FreeBSD: stable/2.1/libexec/bootpd/tools/bootptest/print
 #include "bootptest.h"
 
 /* These decode the vendor data. */
+extern int printfn();
 static void rfc1048_print();
 static void cmu_print();
 static void other_print();
@@ -271,8 +273,6 @@ rfc1048_opts[] = {
 };
 #define	KNOWN_OPTIONS (sizeof(rfc1048_opts) / sizeof(rfc1048_opts[0]))
 
-static void print_string();
-
 static void
 rfc1048_print(bp, length)
 	register u_char *bp;
@@ -280,7 +280,7 @@ rfc1048_print(bp, length)
 {
 	u_char tag;
 	u_char *ep;
-	register int len, j;
+	register int len;
 	u_int32 ul;
 	u_short us;
 	struct in_addr ia;
@@ -433,7 +433,6 @@ other_print(bp, length)
 {
 	u_char *ep;					/* end pointer */
 	u_char *zp;					/* points one past last non-zero byte */
-	register int i, j;
 
 	/* Setup end pointer */
 	ep = bp + length;
