@@ -1,5 +1,7 @@
 /*
  * trygetea.c - test program for getether.c
+ *
+ * $FreeBSD: stable/2.2/libexec/bootpd/trygetea.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/types.h>
@@ -9,6 +11,9 @@
 #include <sys/sockio.h>
 #endif
 
+#ifdef _AIX32
+#include <sys/time.h>	/* for struct timeval in net/if.h */
+#endif
 #include <net/if.h>				/* for struct ifreq */
 #include <netinet/in.h>
 #include <arpa/inet.h>			/* inet_ntoa */
@@ -18,10 +23,14 @@
 #include <ctype.h>
 #include <errno.h>
 
+#include "getether.h"
+
 int debug = 0;
 char *progname;
 
+void
 main(argc, argv)
+	int argc;
 	char **argv;
 {
 	u_char ea[16];				/* Ethernet address */
@@ -33,7 +42,7 @@ main(argc, argv)
 		printf("need interface name\n");
 		exit(1);
 	}
-	if ((i = getether(argv[1], ea)) < 0) {
+	if ((i = getether(argv[1], (char*)ea)) < 0) {
 		printf("Could not get Ethernet address (rc=%d)\n", i);
 		exit(1);
 	}
