@@ -1,5 +1,7 @@
 /*
  * trygetif.c - test program for getif.c
+ *
+ * $FreeBSD: stable/2.2/libexec/bootpd/trygetif.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/types.h>
@@ -9,6 +11,9 @@
 #include <sys/sockio.h>
 #endif
 
+#ifdef _AIX32
+#include <sys/time.h>	/* for struct timeval in net/if.h */
+#endif
 #include <net/if.h>				/* for struct ifreq */
 #include <netinet/in.h>
 #include <arpa/inet.h>			/* inet_ntoa */
@@ -23,16 +28,17 @@
 int debug = 0;
 char *progname;
 
+void
 main(argc, argv)
+	int argc;
 	char **argv;
 {
 	struct hostent *hep;
-	struct sockaddr ea;			/* Ethernet address */
 	struct sockaddr_in *sip;	/* Interface address */
 	struct ifreq *ifr;
 	struct in_addr dst_addr;
 	struct in_addr *dap;
-	int i, s;
+	int s;
 
 	progname = argv[0];			/* for report */
 
