@@ -29,18 +29,22 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/libexec/ftpd/ftpd.c 50952 1999-09-05 11:54:50Z peter $
  */
 
+#if 0
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright (c) 1985, 1988, 1990, 1992, 1993, 1994\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
+#endif
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)ftpd.c	8.4 (Berkeley) 4/16/94";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/ftpd/ftpd.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
@@ -72,6 +76,7 @@ static char sccsid[] = "@(#)ftpd.c	8.4 (Berkeley) 4/16/94";
 #include <limits.h>
 #include <netdb.h>
 #include <pwd.h>
+#include <grp.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <stdio.h>
@@ -80,6 +85,10 @@ static char sccsid[] = "@(#)ftpd.c	8.4 (Berkeley) 4/16/94";
 #include <syslog.h>
 #include <time.h>
 #include <unistd.h>
+#include <libutil.h>
+#ifdef	LOGIN_CAP
+#include <login_cap.h>
+#endif
 
 #ifdef	SKEY
 #include <skey.h>
@@ -94,17 +103,24 @@ static char sccsid[] = "@(#)ftpd.c	8.4 (Berkeley) 4/16/94";
 #include <varargs.h>
 #endif
 
+#ifdef	INTERNAL_LS
+static char version[] = "Version 6.00LS";
+#undef main
+#else
 static char version[] = "Version 6.00";
+#endif
 
 extern	off_t restart_point;
 extern	char cbuf[];
 
+struct	sockaddr_in server_addr;
 struct	sockaddr_in ctrl_addr;
 struct	sockaddr_in data_source;
 struct	sockaddr_in data_dest;
 struct	sockaddr_in his_addr;
 struct	sockaddr_in pasv_addr;
 
+int	daemon_mode;
 int	data;
 jmp_buf	errcatch, urgcatch;
 int	logged_in;
@@ -114,12 +130,12 @@ int	timeout = 900;    /* timeout after 15 minutes of inactivity */
 int	maxtimeout = 7200;/* don't allow idle time to be set beyond 2 hours */
 int	logging;
 int	restricted_data_ports = 1;
-int	paranoid = 1;
+int	paranoid = 1;	  /* be extra careful about security */
+int	anon_only = 0;    /* Only anonymous ftp allowed */
 int	guest;
-#ifdef STATS
+int	dochroot;
 int	stats;
 int	statfd = -1;
-#endif
 int	type;
 int	form;
 int	stru;			/* avoid C keyword */
@@ -135,10 +151,38 @@ off_t	byte_count;
 #endif
 int	defumask = CMASK;		/* default umask value */
 char	tmpline[7];
-char	hostname[MAXHOSTNAMELEN];
+char	*hostname;
+#ifdef VIRTUAL_HOSTING
+char	*ftpuser;
+
+static struct ftphost {
+	struct ftphost	*next;
+	struct in_addr	hostaddr;
+	char		*hostname;
+	char		*anonuser;
+	char		*statfile;
+	char		*welcome;
+	char		*loginmsg;
+} *thishost, *firsthost;
+
+#endif
 char	remotehost[MAXHOSTNAMELEN];
-#ifdef STATS
 char	*ident = NULL;
+
+static char ttyline[20];
+char	*tty = ttyline;		/* for klogin */
+
+#ifdef KERBEROS
+int	 klogin __P((struct passwd *, char *, char *, char *));
+#endif
+
+struct	in_addr bind_address;
+char	*pid_file = NULL;
+
+#if defined(KERBEROS)
+int	notickets = 1;
+int	noticketsdontcomplain = 1;
+char	*krbtkfile_env = NULL;
 #endif
 
 /*
@@ -153,8 +197,10 @@ int	swaitmax = SWAITMAX;
 int	swaitint = SWAITINT;
 
 #ifdef SETPROCTITLE
+#ifdef OLD_SETPROCTITLE
 char	**Argv = NULL;		/* pointer to argument vector */
 char	*LastArgv = NULL;	/* end of argv */
+#endif /* OLD_SETPROCTITLE */
 char	proctitle[LINE_MAX];	/* initial part of title */
 #endif /* SETPROCTITLE */
 
@@ -182,9 +228,13 @@ char	addr_string[20];	/* XXX */
 			cmd, (*(file) == '/') ? "" : curdir(), file, cnt); \
 	}
 
+#ifdef VIRTUAL_HOSTING
+static void	 inithosts __P((void));
+static void	selecthost __P((struct in_addr *));
+#endif
 static void	 ack __P((char *));
 static void	 myoob __P((int));
-static int	 checkuser __P((char *));
+static int	 checkuser __P((char *, char *, int));
 static FILE	*dataconn __P((char *, off_t, char *));
 static void	 dolog __P((struct sockaddr_in *));
 static char	*curdir __P((void));
@@ -197,6 +247,8 @@ static void	 send_data __P((FILE *, FILE *, off_t, off_t, int));
 static struct passwd *
 		 sgetpwnam __P((char *));
 static char	*sgetsave __P((char *));
+static void	 reapchild __P((int));
+static void      logxfer __P((char *, long, long));
 
 static char *
 curdir()
@@ -223,32 +275,7 @@ main(argc, argv, envp)
 
 	tzset();		/* in case no timezone database in ~ftp */
 
-	/*
-	 * LOG_NDELAY sets up the logging connection immediately,
-	 * necessary for anonymous ftp's that chroot and can't do it later.
-	 */
-	openlog("ftpd", LOG_PID | LOG_NDELAY, LOG_FTP);
-	addrlen = sizeof(his_addr);
-	if (getpeername(0, (struct sockaddr *)&his_addr, &addrlen) < 0) {
-		syslog(LOG_ERR, "getpeername (%s): %m",argv[0]);
-		exit(1);
-	}
-#ifdef SKEY
-	strcpy(addr_string, inet_ntoa(his_addr.sin_addr));
-#endif
-	addrlen = sizeof(ctrl_addr);
-	if (getsockname(0, (struct sockaddr *)&ctrl_addr, &addrlen) < 0) {
-		syslog(LOG_ERR, "getsockname (%s): %m",argv[0]);
-		exit(1);
-	}
-#ifdef IP_TOS
-	tos = IPTOS_LOWDELAY;
-	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
-		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
-#endif
-	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
-	debug = 0;
-#ifdef SETPROCTITLE
+#ifdef OLD_SETPROCTITLE
 	/*
 	 *  Save start and extent of argv for setproctitle.
 	 */
@@ -256,17 +283,18 @@ main(argc, argv, envp)
 	while (*envp)
 		envp++;
 	LastArgv = envp[-1] + strlen(envp[-1]);
-#endif /* SETPROCTITLE */
+#endif /* OLD_SETPROCTITLE */
 
 
-#ifdef STATS
-	while ((ch = getopt(argc, argv, "RdlSUt:T:u:v")) != EOF) {
-#else
-	while ((ch = getopt(argc, argv, "RdlUt:T:u:v")) != EOF) {
-#endif
+	bind_address.s_addr = htonl(INADDR_ANY);
+	while ((ch = getopt(argc, argv, "AdlDSURt:T:u:va:p:")) != -1) {
 		switch (ch) {
+		case 'D':
+			daemon_mode++;
+			break;
+
 		case 'd':
-			debug = 1;
+			debug++;
 			break;
 
 		case 'l':
@@ -277,8 +305,14 @@ main(argc, argv, envp)
 			paranoid = 0;
 			break;
 
-		case 'U':
-			restricted_data_ports = 0;
+		case 'S':
+			stats++;
+			break;
+
+		case 'T':
+			maxtimeout = atoi(optarg);
+			if (timeout > maxtimeout)
+				timeout = maxtimeout;
 			break;
 
 		case 't':
@@ -286,15 +320,18 @@ main(argc, argv, envp)
 			if (maxtimeout < timeout)
 				maxtimeout = timeout;
 			break;
-#ifdef STATS
-		case 'S':
-			stats =  1;
+
+		case 'U':
+			restricted_data_ports = 0;
 			break;
-#endif
-		case 'T':
-			maxtimeout = atoi(optarg);
-			if (timeout > maxtimeout)
-				timeout = maxtimeout;
+
+		case 'a':
+			if (!inet_aton(optarg, &bind_address))
+				errx(1, "invalid address for -a");
+			break;
+
+		case 'p':
+			pid_file = optarg;
 			break;
 
 		case 'u':
@@ -308,6 +345,9 @@ main(argc, argv, envp)
 				defumask = val;
 			break;
 		    }
+		case 'A':
+			anon_only = 1;
+			break;
 
 		case 'v':
 			debug = 1;
@@ -318,12 +358,140 @@ main(argc, argv, envp)
 			break;
 		}
 	}
+
+#ifdef VIRTUAL_HOSTING
+	inithosts();
+#endif
 	(void) freopen(_PATH_DEVNULL, "w", stderr);
-	(void) signal(SIGPIPE, lostconn);
+
+	/*
+	 * LOG_NDELAY sets up the logging connection immediately,
+	 * necessary for anonymous ftp's that chroot and can't do it later.
+	 */
+	openlog("ftpd", LOG_PID | LOG_NDELAY, LOG_FTP);
+
+	if (daemon_mode) {
+		int ctl_sock, fd;
+		struct servent *sv;
+
+		/*
+		 * Detach from parent.
+		 */
+		if (daemon(1, 1) < 0) {
+			syslog(LOG_ERR, "failed to become a daemon");
+			exit(1);
+		}
+		(void) signal(SIGCHLD, reapchild);
+		/*
+		 * Get port number for ftp/tcp.
+		 */
+		sv = getservbyname("ftp", "tcp");
+		if (sv == NULL) {
+			syslog(LOG_ERR, "getservbyname for ftp failed");
+			exit(1);
+		}
+		/*
+		 * Open a socket, bind it to the FTP port, and start
+		 * listening.
+		 */
+		ctl_sock = socket(AF_INET, SOCK_STREAM, 0);
+		if (ctl_sock < 0) {
+			syslog(LOG_ERR, "control socket: %m");
+			exit(1);
+		}
+		if (setsockopt(ctl_sock, SOL_SOCKET, SO_REUSEADDR,
+		    (char *)&on, sizeof(on)) < 0)
+			syslog(LOG_ERR, "control setsockopt: %m");;
+		server_addr.sin_family = AF_INET;
+		server_addr.sin_addr = bind_address;
+		server_addr.sin_port = sv->s_port;
+		if (bind(ctl_sock, (struct sockaddr *)&server_addr, sizeof(server_addr))) {
+			syslog(LOG_ERR, "control bind: %m");
+			exit(1);
+		}
+		if (listen(ctl_sock, 32) < 0) {
+			syslog(LOG_ERR, "control listen: %m");
+			exit(1);
+		}
+		/*
+		 * Atomically write process ID
+		 */
+		if (pid_file)
+		{   
+			int fd;
+			char buf[20];
+
+			fd = open(pid_file, O_CREAT | O_WRONLY | O_TRUNC
+				| O_NONBLOCK | O_EXLOCK, 0644);
+			if (fd < 0)
+				if (errno == EAGAIN)
+					errx(1, "%s: file locked", pid_file);
+				else
+					err(1, "%s", pid_file);
+			snprintf(buf, sizeof(buf),
+				"%lu\n", (unsigned long) getpid());
+			if (write(fd, buf, strlen(buf)) < 0)
+				err(1, "%s: write", pid_file);
+			/* Leave the pid file open and locked */
+		}
+		/*
+		 * Loop forever accepting connection requests and forking off
+		 * children to handle them.
+		 */
+		while (1) {
+			addrlen = sizeof(his_addr);
+			fd = accept(ctl_sock, (struct sockaddr *)&his_addr, &addrlen);
+			if (fork() == 0) {
+				/* child */
+				(void) dup2(fd, 0);
+				(void) dup2(fd, 1);
+				close(ctl_sock);
+				break;
+			}
+			close(fd);
+		}
+	} else {
+		addrlen = sizeof(his_addr);
+		if (getpeername(0, (struct sockaddr *)&his_addr, &addrlen) < 0) {
+			syslog(LOG_ERR, "getpeername (%s): %m",argv[0]);
+			exit(1);
+		}
+	}
+
 	(void) signal(SIGCHLD, SIG_IGN);
-	if ((int)signal(SIGURG, myoob) < 0)
+	(void) signal(SIGPIPE, lostconn);
+	if (signal(SIGURG, myoob) == SIG_ERR)
 		syslog(LOG_ERR, "signal: %m");
 
+#ifdef SKEY
+	strncpy(addr_string, inet_ntoa(his_addr.sin_addr), sizeof(addr_string));
+#endif
+	addrlen = sizeof(ctrl_addr);
+	if (getsockname(0, (struct sockaddr *)&ctrl_addr, &addrlen) < 0) {
+		syslog(LOG_ERR, "getsockname (%s): %m",argv[0]);
+		exit(1);
+	}
+#ifdef VIRTUAL_HOSTING
+	/* select our identity from virtual host table */
+	selecthost(&ctrl_addr.sin_addr);
+#endif
+#ifdef IP_TOS
+	tos = IPTOS_LOWDELAY;
+	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
+		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
+#endif
+	/*
+	 * Disable Nagle on the control channel so that we don't have to wait
+	 * for peer's ACK before issuing our next reply.
+	 */
+	if (setsockopt(0, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on)) < 0)
+		syslog(LOG_WARNING, "control setsockopt TCP_NODELAY: %m");
+
+	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
+
+	/* set this here so klogin can use it... */
+	(void)snprintf(ttyline, sizeof(ttyline), "ftp%d", getpid());
+
 	/* Try to handle urgent data inline */
 #ifdef SO_OOBINLINE
 	if (setsockopt(0, SOL_SOCKET, SO_OOBINLINE, (char *)&on, sizeof(on)) < 0)
@@ -357,7 +525,11 @@ main(argc, argv, envp)
 		reply(530, "System not available.");
 		exit(0);
 	}
+#ifdef VIRTUAL_HOSTING
+	if ((fd = fopen(thishost->welcome, "r")) != NULL) {
+#else
 	if ((fd = fopen(_PATH_FTPWELCOME, "r")) != NULL) {
+#endif
 		while (fgets(line, sizeof(line), fd) != NULL) {
 			if ((cp = strchr(line, '\n')) != NULL)
 				*cp = '\0';
@@ -367,7 +539,11 @@ main(argc, argv, envp)
 		(void) fclose(fd);
 		/* reply(220,) must follow */
 	}
-	(void) gethostname(hostname, sizeof(hostname));
+#ifndef VIRTUAL_HOSTING
+	if ((hostname = malloc(MAXHOSTNAMELEN)) == NULL)
+		fatal("Ran out of memory.");
+	(void) gethostname(hostname, MAXHOSTNAMELEN);
+#endif
 	reply(220, "%s FTP server (%s) ready.", hostname, version);
 	(void) setjmp(errcatch);
 	for (;;)
@@ -382,10 +558,149 @@ lostconn(signo)
 
 	if (debug)
 		syslog(LOG_DEBUG, "lost connection");
-	dologout(-1);
+	dologout(1);
 }
 
-static char ttyline[20];
+#ifdef VIRTUAL_HOSTING
+/*
+ * read in virtual host tables (if they exist)
+ */
+
+static void
+inithosts()
+{
+	FILE *fp;
+	char *cp;
+	struct hostent *hp;
+	struct ftphost *hrp, *lhrp;
+	char line[1024];
+
+	/*
+	 * Fill in the default host information
+	 */
+	if (gethostname(line, sizeof(line)) < 0)
+		line[0] = '\0';
+	if ((hrp = malloc(sizeof(struct ftphost))) == NULL ||
+	    (hrp->hostname = strdup(line)) == NULL)
+		fatal("Ran out of memory.");
+	memset(&hrp->hostaddr, 0, sizeof hrp->hostaddr);
+	if ((hp = gethostbyname(hrp->hostname)) != NULL)
+		(void) memcpy(&hrp->hostaddr,
+			      hp->h_addr_list[0],
+			      sizeof(hrp->hostaddr));
+	hrp->statfile = _PATH_FTPDSTATFILE;
+	hrp->welcome  = _PATH_FTPWELCOME;
+	hrp->loginmsg = _PATH_FTPLOGINMESG;
+	hrp->anonuser = "ftp";
+	hrp->next = NULL;
+	thishost = firsthost = lhrp = hrp;
+	if ((fp = fopen(_PATH_FTPHOSTS, "r")) != NULL) {
+		while (fgets(line, sizeof(line), fp) != NULL) {
+			int	i;
+
+			if ((cp = strchr(line, '\n')) == NULL) {
+				/* ignore long lines */
+				while (fgets(line, sizeof(line), fp) != NULL &&
+					strchr(line, '\n') == NULL)
+					;
+				continue;
+			}
+			*cp = '\0';
+			cp = strtok(line, " \t");
+			/* skip comments and empty lines */
+			if (cp == NULL || line[0] == '#')
+				continue;
+			/* first, try a standard gethostbyname() */
+			if ((hp = gethostbyname(cp)) == NULL)
+				continue;
+			for (hrp = firsthost; hrp != NULL; hrp = hrp->next) {
+				if (memcmp(&hrp->hostaddr,
+					   hp->h_addr_list[0],
+					   sizeof(hrp->hostaddr)) == 0)
+					break;
+			}
+			if (hrp == NULL) {
+				if ((hrp = malloc(sizeof(struct ftphost))) == NULL)
+					continue;
+				/* defaults */
+				hrp->statfile = _PATH_FTPDSTATFILE;
+				hrp->welcome  = _PATH_FTPWELCOME;
+				hrp->loginmsg = _PATH_FTPLOGINMESG;
+				hrp->anonuser = "ftp";
+				hrp->next     = NULL;
+				lhrp->next = hrp;
+				lhrp = hrp;
+			}
+			(void) memcpy(&hrp->hostaddr,
+				      hp->h_addr_list[0],
+				      sizeof(hrp->hostaddr));
+			/*
+			 * determine hostname to use.
+			 * force defined name if it is a valid alias
+			 * otherwise fallback to primary hostname
+			 */
+			if ((hp = gethostbyaddr((char*)&hrp->hostaddr,
+						sizeof(hrp->hostaddr),
+						AF_INET)) != NULL) {
+				if (strcmp(cp, hp->h_name) != 0) {
+					if (hp->h_aliases == NULL)
+						cp = hp->h_name;
+					else {
+						i = 0;
+						while (hp->h_aliases[i] &&
+						       strcmp(cp, hp->h_aliases[i]) != 0)
+							++i;
+						if (hp->h_aliases[i] == NULL)
+							cp = hp->h_name;
+					}
+				}
+			}
+			hrp->hostname = strdup(cp);
+			/* ok, now we now peel off the rest */
+			i = 0;
+			while (i < 4 && (cp = strtok(NULL, " \t")) != NULL) {
+				if (*cp != '-' && (cp = strdup(cp)) != NULL) {
+					switch (i) {
+					case 0:	/* anon user permissions */
+						hrp->anonuser = cp;
+						break;
+					case 1: /* statistics file */
+						hrp->statfile = cp;
+						break;
+					case 2: /* welcome message */
+						hrp->welcome  = cp;
+						break;
+					case 3: /* login message */
+						hrp->loginmsg = cp;
+						break;
+					}
+				}
+				++i;
+			}
+		}
+		(void) fclose(fp);
+	}
+}
+
+static void
+selecthost(a)
+	struct in_addr *a;
+{
+	struct ftphost	*hrp;
+
+	hrp = thishost = firsthost;	/* default */
+	while (hrp != NULL) {
+		if (memcmp(a, &hrp->hostaddr, sizeof(hrp->hostaddr)) == 0) {
+			thishost = hrp;
+			break;
+		}
+		hrp = hrp->next;
+	}
+	/* setup static variables as appropriate */
+	hostname = thishost->hostname;
+	ftpuser = thishost->anonuser;
+}
+#endif
 
 /*
  * Helper function for sgetpwnam().
@@ -460,15 +775,23 @@ user(name)
 		if (guest) {
 			reply(530, "Can't change user from guest login.");
 			return;
+		} else if (dochroot) {
+			reply(530, "Can't change user from chroot user.");
+			return;
 		}
 		end_login();
 	}
 
 	guest = 0;
 	if (strcmp(name, "ftp") == 0 || strcmp(name, "anonymous") == 0) {
-		if (checkuser("ftp") || checkuser("anonymous"))
+		if (checkuser(_PATH_FTPUSERS, "ftp", 0) ||
+		    checkuser(_PATH_FTPUSERS, "anonymous", 0))
 			reply(530, "User %s access denied.", name);
+#ifdef VIRTUAL_HOSTING
+		else if ((pw = sgetpwnam(thishost->anonuser)) != NULL) {
+#else
 		else if ((pw = sgetpwnam("ftp")) != NULL) {
+#endif
 			guest = 1;
 			askpasswd = 1;
 			reply(331,
@@ -480,7 +803,12 @@ user(name)
 			    "ANONYMOUS FTP LOGIN REFUSED FROM %s", remotehost);
 		return;
 	}
-	if (pw = sgetpwnam(name)) {
+	if (anon_only != 0) {
+		reply(530, "Sorry, only anonymous ftp allowed.");
+		return;
+	}
+		
+	if ((pw = sgetpwnam(name))) {
 		if ((shell = pw->pw_shell) == NULL || *shell == 0)
 			shell = _PATH_BSHELL;
 		while ((cp = getusershell()) != NULL)
@@ -488,7 +816,7 @@ user(name)
 				break;
 		endusershell();
 
-		if (cp == NULL || checkuser(name)) {
+		if (cp == NULL || checkuser(_PATH_FTPUSERS, name, 1)) {
 			reply(530, "User %s access denied.", name);
 			if (logging)
 				syslog(LOG_NOTICE,
@@ -516,26 +844,51 @@ user(name)
 }
 
 /*
- * Check if a user is in the file _PATH_FTPUSERS
+ * Check if a user is in the file "fname"
  */
 static int
-checkuser(name)
+checkuser(fname, name, pwset)
+	char *fname;
 	char *name;
+	int pwset;
 {
 	FILE *fd;
 	int found = 0;
 	char *p, line[BUFSIZ];
 
-	if ((fd = fopen(_PATH_FTPUSERS, "r")) != NULL) {
-		while (fgets(line, sizeof(line), fd) != NULL)
+	if ((fd = fopen(fname, "r")) != NULL) {
+		while (!found && fgets(line, sizeof(line), fd) != NULL)
 			if ((p = strchr(line, '\n')) != NULL) {
 				*p = '\0';
 				if (line[0] == '#')
 					continue;
-				if (strcmp(line, name) == 0) {
-					found = 1;
-					break;
+				/*
+				 * if first chr is '@', check group membership
+				 */
+				if (line[0] == '@') {
+					int i = 0;
+					struct group *grp;
+
+					if ((grp = getgrnam(line+1)) == NULL)
+						continue;
+					/*
+					 * Check user's default group
+					 */
+					if (pwset && grp->gr_gid == pw->pw_gid)
+						found = 1;
+					/*
+					 * Check supplementary groups
+					 */
+					while (!found && grp->gr_mem[i])
+						found = strcmp(name,
+							grp->gr_mem[i++])
+							== 0;
 				}
+				/*
+				 * Otherwise, just check for username match
+				 */
+				else
+					found = strcmp(line, name) == 0;
 			}
 		(void) fclose(fd);
 	}
@@ -552,18 +905,26 @@ end_login()
 
 	(void) seteuid((uid_t)0);
 	if (logged_in)
-		logwtmp(ttyline, "", "");
+		ftpd_logwtmp(ttyline, "", "");
 	pw = NULL;
+#ifdef	LOGIN_CAP
+	setusercontext(NULL, getpwuid(0), (uid_t)0,
+		       LOGIN_SETPRIORITY|LOGIN_SETRESOURCES|LOGIN_SETUMASK);
+#endif
 	logged_in = 0;
 	guest = 0;
+	dochroot = 0;
 }
 
 void
 pass(passwd)
 	char *passwd;
 {
-	char *salt, *xpasswd;
+	int rval;
 	FILE *fd;
+#ifdef	LOGIN_CAP
+	login_cap_t *lc = NULL;
+#endif
 	static char homedir[MAXPATHLEN];
 
 	if (logged_in || askpasswd == 0) {
@@ -572,20 +933,33 @@ pass(passwd)
 	}
 	askpasswd = 0;
 	if (!guest) {		/* "ftp" is only account allowed no password */
-		if (pw == NULL)
-			salt = "xx";
-		else
-			salt = pw->pw_passwd;
+		if (pw == NULL) {
+			rval = 1;	/* failure below */
+			goto skip;
+		}
+#if defined(KERBEROS)
+		rval = klogin(pw, "", hostname, passwd);
+		if (rval == 0)
+			goto skip;
+#endif
 #ifdef SKEY
-		xpasswd = skey_crypt(passwd, salt, pw, pwok);
+		rval = strcmp(skey_crypt(passwd, pw->pw_passwd, pw, pwok),
+			      pw->pw_passwd);
 		pwok = 0;
 #else
-		xpasswd = crypt(passwd, salt);
+		rval = strcmp(crypt(passwd, pw->pw_passwd), pw->pw_passwd);
 #endif
 		/* The strcmp does not catch null passwords! */
-		if (pw == NULL || *pw->pw_passwd == '\0' ||
-		    (pw->pw_expire && time(NULL) >= pw->pw_expire) ||
-		    strcmp(xpasswd, pw->pw_passwd)) {
+		if (*pw->pw_passwd == '\0' ||
+		    (pw->pw_expire && time(NULL) >= pw->pw_expire))
+			rval = 1;	/* failure */
+skip:
+		/*
+		 * If rval == 1, the user failed the authentication check
+		 * above.  If rval == 0, either Kerberos or local authentication
+		 * succeeded.
+		 */
+		if (rval) {
 			reply(530, "Login incorrect.");
 			if (logging)
 				syslog(LOG_NOTICE,
@@ -606,19 +980,52 @@ pass(passwd)
 		reply(550, "Can't set gid.");
 		return;
 	}
+	/* May be overridden by login.conf */
+	(void) umask(defumask);
+#ifdef	LOGIN_CAP
+	if ((lc = login_getpwclass(pw)) != NULL) {
+		char	remote_ip[MAXHOSTNAMELEN];
+
+		strncpy(remote_ip, inet_ntoa(his_addr.sin_addr),
+			sizeof(remote_ip) - 1);
+		remote_ip[sizeof(remote_ip) - 1] = 0;
+		if (!auth_hostok(lc, remotehost, remote_ip)) {
+			syslog(LOG_INFO|LOG_AUTH,
+			    "FTP LOGIN FAILED (HOST) as %s: permission denied.",
+			    pw->pw_name);
+			reply(530, "Permission denied.\n");
+			pw = NULL;
+			return;
+		}
+		if (!auth_timeok(lc, time(NULL))) {
+			reply(530, "Login not available right now.\n");
+			pw = NULL;
+			return;
+		}
+	}
+	setusercontext(lc, pw, (uid_t)0,
+	LOGIN_SETGROUP|LOGIN_SETPRIORITY|LOGIN_SETRESOURCES|LOGIN_SETUMASK);
+#else
 	(void) initgroups(pw->pw_name, pw->pw_gid);
-
-	/* open wtmp before chroot */
-	(void)sprintf(ttyline, "ftp%d", getpid());
-	logwtmp(ttyline, pw->pw_name, remotehost);
-	logged_in = 1;
-
-#ifdef STATS
-	if (guest && stats == 1 && statfd < 0)
-		if ((statfd = open(_PATH_FTPDSTATFILE, O_WRONLY|O_APPEND)) < 0)
-			stats = 0;
 #endif
 
+	/* open wtmp before chroot */
+	ftpd_logwtmp(ttyline, pw->pw_name, remotehost);
+	logged_in = 1;
+
+	if (guest && stats && statfd < 0)
+#ifdef VIRTUAL_HOSTING
+		if ((statfd = open(thishost->statfile, O_WRONLY|O_APPEND)) < 0)
+#else
+		if ((statfd = open(_PATH_FTPDSTATFILE, O_WRONLY|O_APPEND)) < 0)
+#endif
+			stats = 0;
+
+	dochroot =
+#ifdef	LOGIN_CAP	/* Allow login.conf configuration as well */
+		login_getcapbool(lc, "ftp-chroot", 0) ||
+#endif
+		checkuser(_PATH_FTPCHROOT, pw->pw_name, 1);
 	if (guest) {
 		/*
 		 * We MUST do a chdir() after the chroot. Otherwise
@@ -629,6 +1036,11 @@ pass(passwd)
 			reply(550, "Can't set guest privileges.");
 			goto bad;
 		}
+	} else if (dochroot) {
+		if (chroot(pw->pw_dir) < 0 || chdir("/") < 0) {
+			reply(550, "Can't change root.");
+			goto bad;
+		}
 	} else if (chdir(pw->pw_dir) < 0) {
 		if (chdir("/") < 0) {
 			reply(530, "User %s: can't change directory to %s.",
@@ -652,7 +1064,11 @@ pass(passwd)
 	 * Display a login message, if it exists.
 	 * N.B. reply(230,) must follow the message.
 	 */
+#ifdef VIRTUAL_HOSTING
+	if ((fd = fopen(thishost->loginmsg, "r")) != NULL) {
+#else
 	if ((fd = fopen(_PATH_FTPLOGINMESG, "r")) != NULL) {
+#endif
 		char *cp, line[LINE_MAX];
 
 		while (fgets(line, sizeof(line), fd) != NULL) {
@@ -664,39 +1080,56 @@ pass(passwd)
 		(void) fclose(fd);
 	}
 	if (guest) {
-#ifdef STATS
-		char * copy();
-
 		if (ident != NULL)
 			free(ident);
-		ident = (char *) copy(passwd);
-#endif
+		ident = strdup(passwd);
+		if (ident == NULL)
+			fatal("Ran out of memory.");
+
 		reply(230, "Guest login ok, access restrictions apply.");
 #ifdef SETPROCTITLE
-		snprintf(proctitle, sizeof(proctitle),
-		    "%s: anonymous/%.*s", remotehost,
-		    sizeof(proctitle) - sizeof(remotehost) -
-		    sizeof(": anonymous/"), passwd);
-		setproctitle(proctitle);
+#ifdef VIRTUAL_HOSTING
+		if (thishost != firsthost)
+			snprintf(proctitle, sizeof(proctitle),
+				 "%s: anonymous(%s)/%.*s", remotehost, hostname,
+				 sizeof(proctitle) - sizeof(remotehost) -
+				 sizeof(": anonymous/"), passwd);
+		else
+#endif
+			snprintf(proctitle, sizeof(proctitle),
+				 "%s: anonymous/%.*s", remotehost,
+				 sizeof(proctitle) - sizeof(remotehost) -
+				 sizeof(": anonymous/"), passwd);
+		setproctitle("%s", proctitle);
 #endif /* SETPROCTITLE */
 		if (logging)
 			syslog(LOG_INFO, "ANONYMOUS FTP LOGIN FROM %s, %s",
 			    remotehost, passwd);
 	} else {
+	    if (dochroot)
+		reply(230, "User %s logged in, access restrictions apply.", 
+			pw->pw_name);
+	    else
 		reply(230, "User %s logged in.", pw->pw_name);
+
 #ifdef SETPROCTITLE
 		snprintf(proctitle, sizeof(proctitle),
-		    "%s: %s", remotehost, pw->pw_name);
-		setproctitle(proctitle);
+			 "%s: %s", remotehost, pw->pw_name);
+		setproctitle("%s", proctitle);
 #endif /* SETPROCTITLE */
 		if (logging)
 			syslog(LOG_INFO, "FTP LOGIN FROM %s as %s",
 			    remotehost, pw->pw_name);
 	}
-	(void) umask(defumask);
+#ifdef	LOGIN_CAP
+	login_close(lc);
+#endif
 	return;
 bad:
 	/* Forget all about it... */
+#ifdef	LOGIN_CAP
+	login_close(lc);
+#endif
 	end_login();
 }
 
@@ -707,9 +1140,7 @@ retrieve(cmd, name)
 	FILE *fin, *dout;
 	struct stat st;
 	int (*closefunc) __P((FILE *));
-#ifdef STATS
-	long start;
-#endif
+	time_t start;
 
 	if (cmd == 0) {
 		fin = fopen(name, "r"), closefunc = fclose;
@@ -717,7 +1148,7 @@ retrieve(cmd, name)
 	} else {
 		char line[BUFSIZ];
 
-		(void) sprintf(line, cmd, name), name = line;
+		(void) snprintf(line, sizeof(line), cmd, name), name = line;
 		fin = ftpd_popen(line, "r"), closefunc = ftpd_pclose;
 		st.st_size = -1;
 		st.st_blksize = BUFSIZ;
@@ -759,15 +1190,11 @@ retrieve(cmd, name)
 	dout = dataconn(name, st.st_size, "w");
 	if (dout == NULL)
 		goto done;
-#ifdef STATS
 	time(&start);
-#endif
 	send_data(fin, dout, st.st_blksize, st.st_size,
 		  restart_point == 0 && cmd == 0 && S_ISREG(st.st_mode));
-#ifdef STATS
 	if (cmd == 0 && guest && stats)
-		logxfer( name, st.st_size, start);
-#endif
+		logxfer(name, st.st_size, start);
 	(void) fclose(dout);
 	data = -1;
 	pdata = -1;
@@ -786,7 +1213,7 @@ store(name, mode, unique)
 	struct stat st;
 	int (*closefunc) __P((FILE *));
 
-	if (unique && stat(name, &st) == 0 &&
+	if ((unique || guest) && stat(name, &st) == 0 &&
 	    (name = gunique(name)) == NULL) {
 		LOGCMD(*mode == 'w' ? "put" : "append", name);
 		return;
@@ -865,6 +1292,7 @@ getdatasock(mode)
 	    (char *) &on, sizeof(on)) < 0)
 		goto bad;
 	/* anchor socket to avoid multi-homing problems */
+	data_source.sin_len = sizeof(struct sockaddr_in);
 	data_source.sin_family = AF_INET;
 	data_source.sin_addr = ctrl_addr.sin_addr;
 	for (tries = 1; ; tries++) {
@@ -921,15 +1349,23 @@ dataconn(name, size, mode)
 	file_size = size;
 	byte_count = 0;
 	if (size != (off_t) -1)
-		(void) sprintf(sizebuf, " (%qd bytes)", size);
+		(void) snprintf(sizebuf, sizeof(sizebuf), " (%qd bytes)", size);
 	else
-		(void) strcpy(sizebuf, "");
+		*sizebuf = '\0';
 	if (pdata >= 0) {
 		struct sockaddr_in from;
 		int s, fromlen = sizeof(from);
+		struct timeval timeout;
+		fd_set set;
 
-		s = accept(pdata, (struct sockaddr *)&from, &fromlen);
-		if (s < 0) {
+		FD_ZERO(&set);
+		FD_SET(pdata, &set);
+
+		timeout.tv_usec = 0;
+		timeout.tv_sec = 120;
+
+		if (select(pdata+1, &set, (fd_set *) 0, (fd_set *) 0, &timeout) == 0 ||
+		    (s = accept(pdata, (struct sockaddr *) &from, &fromlen)) < 0) {
 			reply(425, "Can't open data connection.");
 			(void) close(pdata);
 			pdata = -1;
@@ -938,7 +1374,7 @@ dataconn(name, size, mode)
 		(void) close(pdata);
 		pdata = s;
 #ifdef IP_TOS
-		tos = IPTOS_LOWDELAY;
+		tos = IPTOS_THROUGHPUT;
 		(void) setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
 		    sizeof(int));
 #endif
@@ -1035,7 +1471,7 @@ send_data(instr, outstr, blksize, filesize, isreg)
 		if (isreg && filesize < (off_t)16 * 1024 * 1024) {
 			buf = mmap(0, filesize, PROT_READ, MAP_SHARED, filefd,
 				   (off_t)0);
-			if (!buf) {
+			if (buf == MAP_FAILED) {
 				syslog(LOG_WARNING, "mmap(%lu): %m",
 				       (unsigned long)filesize);
 				goto oldway;
@@ -1103,7 +1539,7 @@ receive_data(instr, outstr)
 	FILE *instr, *outstr;
 {
 	int c;
-	int cnt, bare_lfs = 0;
+	int cnt, bare_lfs;
 	char buf[BUFSIZ];
 
 	transflag++;
@@ -1111,6 +1547,9 @@ receive_data(instr, outstr)
 		transflag = 0;
 		return (-1);
 	}
+
+	bare_lfs = 0;
+
 	switch (type) {
 
 	case TYPE_I:
@@ -1185,7 +1624,7 @@ statfilecmd(filename)
 	int c;
 	char line[LINE_MAX];
 
-	(void)snprintf(line, sizeof(line), "/bin/ls -lgA %s", filename);
+	(void)snprintf(line, sizeof(line), _PATH_LS " -lgA %s", filename);
 	fin = ftpd_popen(line, "r");
 	lreply(211, "status of %s:", filename);
 	while ((c = getc(fin)) != EOF) {
@@ -1348,7 +1787,7 @@ yyerror(s)
 {
 	char *cp;
 
-	if (cp = strchr(cbuf,'\n'))
+	if ((cp = strchr(cbuf,'\n')))
 		*cp = '\0';
 	reply(500, "'%s': command not understood.", cbuf);
 }
@@ -1443,8 +1882,15 @@ void
 renamecmd(from, to)
 	char *from, *to;
 {
+	struct stat st;
 
 	LOGCMD2("rename", from, to);
+
+	if (guest && (stat(to, &st) == 0)) {
+		reply(550, "%s: permission denied", to);
+		return;
+	}
+
 	if (rename(from, to) < 0)
 		perror_reply(550, "rename");
 	else
@@ -1464,12 +1910,27 @@ dolog(sin)
 		(void) strncpy(remotehost, inet_ntoa(sin->sin_addr),
 		    sizeof(remotehost));
 #ifdef SETPROCTITLE
-	snprintf(proctitle, sizeof(proctitle), "%s: connected", remotehost);
-	setproctitle(proctitle);
+#ifdef VIRTUAL_HOSTING
+	if (thishost != firsthost)
+		snprintf(proctitle, sizeof(proctitle), "%s: connected (to %s)",
+			 remotehost, hostname);
+	else
+#endif
+		snprintf(proctitle, sizeof(proctitle), "%s: connected",
+			 remotehost);
+	setproctitle("%s", proctitle);
 #endif /* SETPROCTITLE */
 
-	if (logging)
-		syslog(LOG_INFO, "connection from %s", remotehost);
+	if (logging) {
+#ifdef VIRTUAL_HOSTING
+		if (thishost != firsthost)
+			syslog(LOG_INFO, "connection from %s (to %s)",
+			       remotehost, hostname);
+		else
+#endif
+			syslog(LOG_INFO, "connection from %s (%s)", remotehost,
+				inet_ntoa(sin->sin_addr));
+	}
 }
 
 /*
@@ -1480,10 +1941,19 @@ void
 dologout(status)
 	int status;
 {
+	/*
+	 * Prevent reception of SIGURG from resulting in a resumption
+	 * back to the main program loop.
+	 */
+	transflag = 0;
 
 	if (logged_in) {
 		(void) seteuid((uid_t)0);
-		logwtmp(ttyline, "", "");
+		ftpd_logwtmp(ttyline, "", "");
+#if defined(KERBEROS)
+		if (!notickets && krbtkfile_env)
+			unlink(krbtkfile_env);
+#endif
 	}
 	/* beware of flushing buffers after a SIGPIPE */
 	_exit(status);
@@ -1529,48 +1999,37 @@ void
 passive()
 {
 	int len;
-	u_short port;
 	char *p, *a;
 
-	if (pw == NULL) {
-		reply(530, "Please login with USER and PASS");
-		return;
-	}
+	if (pdata >= 0)		/* close old port if one set */
+		close(pdata);
+
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 	if (pdata < 0) {
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
 
-	if (restricted_data_ports) {
-		for (port = FTP_DATA_BOTTOM; port <= FTP_DATA_TOP; port++) {
-			pasv_addr = ctrl_addr;
-			pasv_addr.sin_port = htons(port);
-			(void) seteuid((uid_t)0);
-			if (bind(pdata, (struct sockaddr *)&pasv_addr,
-				 sizeof(pasv_addr)) < 0) {
-				(void) seteuid((uid_t)pw->pw_uid);
-				if (errno == EADDRINUSE)
-					continue;
-				else
-					goto pasv_error;
-			}
-			(void) seteuid((uid_t)pw->pw_uid);
-			break;
-		}
-		if (port > FTP_DATA_TOP)
-			goto pasv_error;
-	} else {
-		pasv_addr = ctrl_addr;
-		pasv_addr.sin_port = 0;
-		(void) seteuid((uid_t)0);
-		if (bind(pdata, (struct sockaddr *)&pasv_addr,
-			 sizeof(pasv_addr)) < 0) {
-			(void) seteuid((uid_t)pw->pw_uid);
-			goto pasv_error;
-		}
-		(void) seteuid((uid_t)pw->pw_uid);
+	(void) seteuid((uid_t)0);
+
+#ifdef IP_PORTRANGE
+	{
+	    int on = restricted_data_ports ? IP_PORTRANGE_HIGH
+					   : IP_PORTRANGE_DEFAULT;
+
+	    if (setsockopt(pdata, IPPROTO_IP, IP_PORTRANGE,
+			    (char *)&on, sizeof(on)) < 0)
+		    goto pasv_error;
 	}
+#endif
+
+	pasv_addr = ctrl_addr;
+	pasv_addr.sin_port = 0;
+	if (bind(pdata, (struct sockaddr *)&pasv_addr,
+		 sizeof(pasv_addr)) < 0)
+		goto pasv_error;
+
+	(void) seteuid((uid_t)pw->pw_uid);
 
 	len = sizeof(pasv_addr);
 	if (getsockname(pdata, (struct sockaddr *) &pasv_addr, &len) < 0)
@@ -1587,6 +2046,7 @@ passive()
 	return;
 
 pasv_error:
+	(void) seteuid((uid_t)pw->pw_uid);
 	(void) close(pdata);
 	pdata = -1;
 	perror_reply(425, "Can't open passive connection");
@@ -1616,7 +2076,8 @@ gunique(local)
 	}
 	if (cp)
 		*cp = '/';
-	(void) strcpy(new, local);
+	/* -4 is for the .nn<null> we put on the end below */
+	(void) snprintf(new, sizeof(new) - 4, "%s", local);
 	cp = new + strlen(new);
 	*cp++ = '.';
 	for (count = 1; count < 100; count++) {
@@ -1682,7 +2143,7 @@ send_file_list(whichf)
 		transflag = 0;
 		goto out;
 	}
-	while (dirname = *dirlist++) {
+	while ((dirname = *dirlist++)) {
 		if (stat(dirname, &st) < 0) {
 			/*
 			 * If user typed "ls -l", etc, and the client
@@ -1690,7 +2151,7 @@ send_file_list(whichf)
 			 */
 			if (dirname[0] == '-' && *dirlist == NULL &&
 			    transflag == 0) {
-				retrieve("/bin/ls %s", dirname);
+				retrieve(_PATH_LS " %s", dirname);
 				goto out;
 			}
 			perror_reply(550, whichf);
@@ -1729,7 +2190,8 @@ send_file_list(whichf)
 			    dir->d_namlen == 2)
 				continue;
 
-			sprintf(nbuf, "%s/%s", dirname, dir->d_name);
+			snprintf(nbuf, sizeof(nbuf), 
+				"%s/%s", dirname, dir->d_name);
 
 			/*
 			 * We have to do a stat to insure it's
@@ -1775,7 +2237,14 @@ out:
 	}
 }
 
-#ifdef SETPROCTITLE
+void
+reapchild(signo)
+	int signo;
+{
+	while (wait3(NULL, WNOHANG, NULL) > 0);
+}
+
+#ifdef OLD_SETPROCTITLE
 /*
  * Clobber argv so ps will show what we're doing.  (Stolen from sendmail.)
  * Warning, since this is usually started from inetd.conf, it often doesn't
@@ -1818,9 +2287,9 @@ setproctitle(fmt, va_alist)
 	while (p < LastArgv)
 		*p++ = ' ';
 }
-#endif /* SETPROCTITLE */
+#endif /* OLD_SETPROCTITLE */
 
-#ifdef STATS
+static void
 logxfer(name, size, start)
 	char *name;
 	long size;
@@ -1828,27 +2297,13 @@ logxfer(name, size, start)
 {
 	char buf[1024];
 	char path[MAXPATHLEN + 1];
-	long now;
+	time_t now;
 
 	if (statfd >= 0 && getwd(path) != NULL) {
 		time(&now);
-		sprintf(buf, "%.20s!%s!%s!%s/%s!%ld!%ld\n",
+		snprintf(buf, sizeof(buf), "%.20s!%s!%s!%s/%s!%ld!%ld\n",
 			ctime(&now)+4, ident, remotehost,
 			path, name, size, now - start + (now == start));
 		write(statfd, buf, strlen(buf));
 	}
 }
-
-char *
-copy(s)
-	char *s;
-{
-	char *p;
-
-	p = malloc((unsigned) strlen(s) + 1);
-	if (p == NULL)
-		fatal("Ran out of memory.");
-	(void) strcpy(p, s);
-	return (p);
-}
-#endif
