@@ -38,19 +38,24 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)bpf.c	8.1 (Berkeley) 6/4/93
+ *	from: @(#)bpf.c	8.1 (Berkeley) 6/4/93
  *
- * Utah $Hdr: bpf.c 3.1 92/07/06$
+ * From: Utah Hdr: bpf.c 3.1 92/07/06
  * Author: Jeff Forys, University of Utah CSS
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)bpf.c	8.1 (Berkeley) 6/4/93";
+#if 0
+static const char sccsid[] = "@(#)bpf.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/rbootd/bpf.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/time.h>
 
 #include <net/if.h>
 #include <net/bpf.h>
@@ -68,7 +73,7 @@ static char sccsid[] = "@(#)bpf.c	8.1 (Berkeley) 6/4/93";
 
 static int BpfFd = -1;
 static unsigned BpfLen = 0;
-static u_char *BpfPkt = NULL;
+static u_int8_t *BpfPkt = NULL;
 
 /*
 **  BpfOpen -- Open and initialize a BPF device.
@@ -144,15 +149,10 @@ BpfOpen()
 	ifr.ifr_addr.sa_len = RMP_ADDRLEN + 2;
 #endif
 	ifr.ifr_addr.sa_family = AF_UNSPEC;
-	bcopy(&RmpMcastAddr[0], (char *)&ifr.ifr_addr.sa_data[0], RMP_ADDRLEN);
-	if (ioctl(BpfFd, SIOCADDMULTI, (caddr_t)&ifr) < 0) {
-		syslog(LOG_WARNING,
-		    "bpf: can't add mcast addr (%m), setting promiscuous mode");
-
-		if (ioctl(BpfFd, BIOCPROMISC, (caddr_t)0) < 0) {
-			syslog(LOG_ERR, "bpf: can't set promiscuous mode: %m");
-			Exit(0);
-		}
+	memmove((char *)&ifr.ifr_addr.sa_data[0], &RmpMcastAddr[0], RMP_ADDRLEN);
+	if (ioctl(BpfFd, BIOCPROMISC, (caddr_t)0) < 0) {
+		syslog(LOG_ERR, "bpf: can't set promiscuous mode: %m");
+		Exit(0);
 	}
 
 	/*
@@ -163,7 +163,7 @@ BpfOpen()
 		Exit(0);
 	}
 	if (BpfPkt == NULL)
-		BpfPkt = (u_char *)malloc(BpfLen);
+		BpfPkt = (u_int8_t *)malloc(BpfLen);
 
 	if (BpfPkt == NULL) {
 		syslog(LOG_ERR, "bpf: out of memory (%u bytes for bpfpkt)",
@@ -311,8 +311,8 @@ BpfRead(rconn, doread)
 	RMPCONN *rconn;
 	int doread;
 {
-	register int datlen, caplen, hdrlen;
-	static u_char *bp = NULL, *ep = NULL;
+	int datlen, caplen, hdrlen;
+	static u_int8_t *bp = NULL, *ep = NULL;
 	int cc;
 
 	/*
@@ -348,9 +348,9 @@ BpfRead(rconn, doread)
 			       caplen);
 		else {
 			rconn->rmplen = caplen;
-			bcopy((char *)&bhp->bh_tstamp, (char *)&rconn->tstamp,
+			memmove((char *)&rconn->tstamp, (char *)&bhp->bh_tstamp,
 			      sizeof(struct timeval));
-			bcopy((char *)bp + hdrlen, (char *)&rconn->rmp, caplen);
+			memmove((char *)&rconn->rmp, (char *)bp + hdrlen, caplen);
 		}
 		bp += BPF_WORDALIGN(caplen + hdrlen);
 		return(1);
@@ -413,7 +413,7 @@ BpfClose()
 	ifr.ifr_addr.sa_len = RMP_ADDRLEN + 2;
 #endif
 	ifr.ifr_addr.sa_family = AF_UNSPEC;
-	bcopy(&RmpMcastAddr[0], (char *)&ifr.ifr_addr.sa_data[0], RMP_ADDRLEN);
+	memmove((char *)&ifr.ifr_addr.sa_data[0], &RmpMcastAddr[0], RMP_ADDRLEN);
 	if (ioctl(BpfFd, SIOCDELMULTI, (caddr_t)&ifr) < 0)
 		(void) ioctl(BpfFd, BIOCPROMISC, (caddr_t)0);
 
