@@ -32,13 +32,17 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1983, 1988, 1989, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)rlogind.c	8.1 (Berkeley) 6/4/93";
+#if 0
+static const char sccsid[] = "@(#)rlogind.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/rlogind/rlogind.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
@@ -51,6 +55,7 @@ static char sccsid[] = "@(#)rlogind.c	8.1 (Berkeley) 6/4/93";
  */
 
 #define	FD_SETSIZE	16		/* don't need many bits for select */
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -61,16 +66,18 @@ static char sccsid[] = "@(#)rlogind.c	8.1 (Berkeley) 6/4/93";
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
+#include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#include <errno.h>
+#include <libutil.h>
 #include <pwd.h>
 #include <syslog.h>
-#include <errno.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include "pathnames.h"
 
 #ifndef TIOCPKT_WINDOW
@@ -79,7 +86,7 @@ static char sccsid[] = "@(#)rlogind.c	8.1 (Berkeley) 6/4/93";
 
 #ifdef	KERBEROS
 #include <des.h>
-#include <kerberosIV/krb.h>
+#include <krb.h>
 #define	SECURE_MESSAGE "This rlogin session is using DES encryption for all transmissions.\r\n"
 
 AUTH_DAT	*kdata;
@@ -89,9 +96,9 @@ u_char		tick_buf[sizeof(KTEXT_ST)];
 Key_schedule	schedule;
 int		doencrypt, retval, use_kerberos, vacuous;
 
-#define		ARGSTR			"alnkvx"
+#define		ARGSTR			"Dalnkvx"
 #else
-#define		ARGSTR			"aln"
+#define		ARGSTR			"Daln"
 #endif	/* KERBEROS */
 
 char	*env[2];
@@ -101,6 +108,7 @@ static	char term[64] = "TERM=";
 #define	ENVSIZE	(sizeof("TERM=")-1)	/* skip null for concatenation */
 int	keepalive = 1;
 int	check_all = 0;
+int	no_delay;
 
 struct	passwd *pwd;
 
@@ -129,8 +137,11 @@ main(argc, argv)
 	openlog("rlogind", LOG_PID | LOG_CONS, LOG_AUTH);
 
 	opterr = 0;
-	while ((ch = getopt(argc, argv, ARGSTR)) != EOF)
+	while ((ch = getopt(argc, argv, ARGSTR)) != -1)
 		switch (ch) {
+		case 'D':
+			no_delay = 1;
+			break;
 		case 'a':
 			check_all = 1;
 			break;
@@ -176,10 +187,15 @@ main(argc, argv)
 	if (keepalive &&
 	    setsockopt(0, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof (on)) < 0)
 		syslog(LOG_WARNING, "setsockopt (SO_KEEPALIVE): %m");
+	if (no_delay &&
+	    setsockopt(0, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on)) < 0)
+		syslog(LOG_WARNING, "setsockopt (TCP_NODELAY): %m");
 	on = IPTOS_LOWDELAY;
 	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
+
 	doit(0, &from);
+	return 0;
 }
 
 int	child;
@@ -215,10 +231,12 @@ doit(f, fromp)
 	fromp->sin_port = ntohs((u_short)fromp->sin_port);
 	hp = gethostbyaddr((char *)&fromp->sin_addr, sizeof(struct in_addr),
 	    fromp->sin_family);
-	if (hp)
-		(void)strcpy(hostname, hp->h_name);
-	else
-		(void)strcpy(hostname, inet_ntoa(fromp->sin_addr));
+	if (hp) {
+		(void)strncpy(hostname, hp->h_name, sizeof(hostname));
+	} else {
+		(void)strncpy(hostname, inet_ntoa(fromp->sin_addr), sizeof(hostname));
+	}
+	hostname[sizeof(hostname) - 1] = '\0';
 
 #ifdef	KERBEROS
 	if (use_kerberos) {
@@ -277,7 +295,10 @@ doit(f, fromp)
 #ifdef	KERBEROS
 #ifdef	CRYPT
 	if (doencrypt)
-		(void) des_write(f, SECURE_MESSAGE, sizeof(SECURE_MESSAGE) - 1);
+		(void) des_enc_write(f,
+				     SECURE_MESSAGE,
+				     strlen(SECURE_MESSAGE),
+				     schedule, &kdata->session);
 #endif
 #endif
 	netf = f;
@@ -293,7 +314,7 @@ doit(f, fromp)
 		if (f > 2)	/* f should always be 0, but... */
 			(void) close(f);
 		setup_term(0);
-		if (strchr(lusername, '-')) {
+		 if (*lusername=='-') {
 			syslog(LOG_ERR, "tried to pass user \"%s\" to login",
 			       lusername);
 			fatal(STDERR_FILENO, "invalid user", 0);
@@ -369,7 +390,7 @@ void
 protocol(f, p)
 	register int f, p;
 {
-	char pibuf[1024+1], fibuf[1024], *pbp, *fbp;
+	char pibuf[1024+1], fibuf[1024], *pbp = NULL, *fbp = NULL;
 	register pcc = 0, fcc = 0;
 	int cc, nfd, n;
 	char cntl;
@@ -434,7 +455,8 @@ protocol(f, p)
 #ifdef	CRYPT
 #ifdef	KERBEROS
 			if (doencrypt)
-				fcc = des_read(f, fibuf, sizeof(fibuf));
+				fcc = des_enc_read(f, fibuf, sizeof(fibuf),
+					schedule, &kdata->session);
 			else
 #endif
 #endif
@@ -502,7 +524,8 @@ protocol(f, p)
 #ifdef	CRYPT
 #ifdef	KERBEROS
 			if (doencrypt)
-				cc = des_write(f, pbp, pcc);
+				cc = des_enc_write(f, pbp, pcc,
+					schedule, &kdata->session);
 			else
 #endif
 #endif
@@ -578,10 +601,9 @@ do_rlogin(dest)
 	pwd = getpwnam(lusername);
 	if (pwd == NULL)
 		return (-1);
-	if (pwd->pw_uid == 0)
-		return (-1);
 	/* XXX why don't we syslog() failure? */
-	return (iruserok(dest->sin_addr.s_addr, 0, rusername, lusername));
+	return (iruserok(dest->sin_addr.s_addr, pwd->pw_uid == 0,
+		rusername, lusername));
 }
 
 void
@@ -681,7 +703,7 @@ do_krb_login(dest)
 			ticket, "rcmd",
 			instance, dest, &faddr,
 			kdata, "", schedule, version);
-		 des_set_key(kdata->session, schedule);
+		 des_set_key(&kdata->session, schedule);
 
 	} else
 #endif
@@ -689,7 +711,7 @@ do_krb_login(dest)
 			authopts, 0,
 			ticket, "rcmd",
 			instance, dest, (struct sockaddr_in *) 0,
-			kdata, "", (bit_64 *) 0, version);
+			kdata, "", NULL, version);
 
 	if (rc != KSUCCESS)
 		return (rc);
@@ -715,9 +737,9 @@ void
 usage()
 {
 #ifdef KERBEROS
-	syslog(LOG_ERR, "usage: rlogind [-aln] [-k | -v]");
+	syslog(LOG_ERR, "usage: rlogind [-Daln] [-k | -v]");
 #else
-	syslog(LOG_ERR, "usage: rlogind [-aln]");
+	syslog(LOG_ERR, "usage: rlogind [-Daln]");
 #endif
 }
 
