@@ -32,13 +32,17 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1988, 1989, 1992, 1993, 1994\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)rshd.c	8.2 (Berkeley) 4/6/94";
+#if 0
+static const char sccsid[] = "@(#)rshd.c	8.2 (Berkeley) 4/6/94";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/rshd/rshd.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
@@ -70,6 +74,9 @@ static char sccsid[] = "@(#)rshd.c	8.2 (Berkeley) 4/6/94";
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#ifdef	LOGIN_CAP
+#include <login_cap.h>
+#endif
 
 int	keepalive = 1;
 int	check_all;
@@ -85,7 +92,7 @@ void	 usage __P((void));
 
 #ifdef	KERBEROS
 #include <des.h>
-#include <kerberosIV/krb.h>
+#include <krb.h>
 #define	VERSION_SIZE	9
 #define SECURE_MESSAGE  "This rsh session is using DES encryption for all transmissions.\r\n"
 #define	OPTIONS		"alnkvxL"
@@ -110,7 +117,7 @@ main(argc, argv)
 	openlog("rshd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
 
 	opterr = 0;
-	while ((ch = getopt(argc, argv, OPTIONS)) != EOF)
+	while ((ch = getopt(argc, argv, OPTIONS)) != -1)
 		switch (ch) {
 		case 'a':
 			check_all = 1;
@@ -164,7 +171,7 @@ main(argc, argv)
 	fromlen = sizeof (from);
 	if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
 		syslog(LOG_ERR, "getpeername: %m");
-		_exit(1);
+		exit(1);
 	}
 	if (keepalive &&
 	    setsockopt(0, SOL_SOCKET, SO_KEEPALIVE, (char *)&on,
@@ -177,6 +184,7 @@ main(argc, argv)
 		syslog(LOG_WARNING, "setsockopt (SO_LINGER): %m");
 	doit(&from);
 	/* NOTREACHED */
+	return(0);
 }
 
 char	username[20] = "USER=";
@@ -196,13 +204,16 @@ doit(fromp)
 	struct passwd *pwd;
 	u_short port;
 	fd_set ready, readfrom;
-	int cc, nfd, pv[2], pid, s;
+	int cc, nfd, pv[2], pid, s = 0;
 	int one = 1;
-	char *hostname, *errorstr, *errorhost;
+	char *hostname, *errorstr, *errorhost = NULL;
 	char *cp, sig, buf[BUFSIZ];
 	char cmdbuf[NCARGS+1], locuser[16], remuser[16];
 	char remotehost[2 * MAXHOSTNAMELEN + 1];
 	char fromhost[2 * MAXHOSTNAMELEN + 1];
+#ifdef	LOGIN_CAP
+	login_cap_t *lc;
+#endif
 
 #ifdef	KERBEROS
 	AUTH_DAT	*kdata = (AUTH_DAT *) NULL;
@@ -230,7 +241,7 @@ doit(fromp)
 #endif
 	fromp->sin_port = ntohs((u_short)fromp->sin_port);
 	if (fromp->sin_family != AF_INET) {
-		syslog(LOG_ERR, "malformed \"from\" address (af %d)\n",
+		syslog(LOG_ERR, "malformed \"from\" address (af %d)",
 		    fromp->sin_family);
 		exit(1);
 	}
@@ -250,7 +261,7 @@ doit(fromp)
 			u_char c = optbuf[i];
 			if (c == IPOPT_LSRR || c == IPOPT_SSRR) {
 				syslog(LOG_NOTICE,
-					"Connection refused from %s with IP option %s",
+					"connection refused from %s with IP option %s",
 					inet_ntoa(fromp->sin_addr),
 					c == IPOPT_LSRR ? "LSRR" : "SSRR");
 				exit(1);
@@ -269,7 +280,7 @@ doit(fromp)
 		if (fromp->sin_port >= IPPORT_RESERVED ||
 		    fromp->sin_port < IPPORT_RESERVED/2) {
 			syslog(LOG_NOTICE|LOG_AUTH,
-			    "Connection from %s on illegal port %u",
+			    "connection from %s on illegal port %u",
 			    inet_ntoa(fromp->sin_addr),
 			    fromp->sin_port);
 			exit(1);
@@ -301,8 +312,12 @@ doit(fromp)
 #ifdef	KERBEROS
 		if (!use_kerberos)
 #endif
-			if (port >= IPPORT_RESERVED) {
-				syslog(LOG_ERR, "2nd port not reserved\n");
+			if (port >= IPPORT_RESERVED ||
+			    port < IPPORT_RESERVED/2) {
+				syslog(LOG_NOTICE|LOG_AUTH,
+				    "2nd socket from %s on unreserved port %u",
+				    inet_ntoa(fromp->sin_addr),
+				    port);
 				exit(1);
 			}
 		fromp->sin_port = htons(port);
@@ -348,7 +363,7 @@ doit(fromp)
 			hp = gethostbyname(remotehost);
 			if (hp == NULL) {
 				syslog(LOG_INFO,
-				    "Couldn't look up address for %s",
+				    "couldn't look up address for %s",
 				    remotehost);
 				errorstr =
 				"Couldn't look up address for your host (%s)\n";
@@ -359,7 +374,7 @@ doit(fromp)
 			} else for (; ; hp->h_addr_list++) {
 				if (hp->h_addr_list[0] == NULL) {
 					syslog(LOG_NOTICE,
-					  "Host addr %s not listed for host %s",
+					  "host addr %s not listed for host %s",
 					    inet_ntoa(fromp->sin_addr),
 					    hp->h_name);
 					errorstr =
@@ -373,7 +388,7 @@ doit(fromp)
 				if (!bcmp(hp->h_addr_list[0],
 				    (caddr_t)&fromp->sin_addr,
 				    sizeof(fromp->sin_addr))) {
-					hostname = hp->h_name;
+					hostname = remotehost;
 					break;
 				}
 			}
@@ -407,13 +422,13 @@ doit(fromp)
 				"rcmd", instance, &fromaddr,
 				&local_addr, kdata, "", schedule,
 				version);
-			des_set_key(kdata->session, schedule);
+			des_set_key(&kdata->session, schedule);
 		} else
 #endif
 			rc = krb_recvauth(authopts, 0, ticket, "rcmd",
 				instance, &fromaddr,
 				(struct sockaddr_in *) 0,
-				kdata, "", (bit_64 *) 0, version);
+				kdata, "", NULL, version);
 		if (rc != KSUCCESS) {
 			error("Kerberos authentication failure: %s\n",
 				  krb_err_txt[rc]);
@@ -435,7 +450,20 @@ doit(fromp)
 			errorstr = "Login incorrect.\n";
 		goto fail;
 	}
+#ifdef	LOGIN_CAP
+	lc = login_getpwclass(pwd);
+#endif
 	if (chdir(pwd->pw_dir) < 0) {
+#ifdef	LOGIN_CAP
+		if (chdir("/") < 0 ||
+		    login_getcapbool(lc, "requirehome", !!pwd->pw_uid)) {
+			syslog(LOG_INFO|LOG_AUTH,
+			"%s@%s as %s: no home directory. cmd='%.80s'",
+			remuser, hostname, locuser, cmdbuf);
+			error("No remote home directory.\n");
+			exit(0);
+		}
+#else
 		(void) chdir("/");
 #ifdef notdef
 		syslog(LOG_INFO|LOG_AUTH,
@@ -444,6 +472,8 @@ doit(fromp)
 		error("No remote directory.\n");
 		exit(1);
 #endif
+#endif
+		pwd->pw_dir = "/";
 	}
 
 #ifdef	KERBEROS
@@ -453,7 +483,7 @@ doit(fromp)
 				syslog(LOG_INFO|LOG_AUTH,
 				    "Kerberos rsh denied to %s.%s@%s",
 				    kdata->pname, kdata->pinst, kdata->prealm);
-				error("Permission denied.\n");
+				error("Login incorrect.\n");
 				exit(1);
 			}
 		}
@@ -476,7 +506,7 @@ doit(fromp)
 				    remuser, hostname, locuser, cmdbuf);
 fail:
 			if (errorstr == NULL)
-				errorstr = "Permission denied.\n";
+				errorstr = "Login incorrect.\n";
 			error(errorstr, errorhost);
 			exit(1);
 		}
@@ -485,6 +515,32 @@ fail:
 		error("Logins currently disabled.\n");
 		exit(1);
 	}
+#ifdef	LOGIN_CAP
+	if (lc != NULL) {
+		char	remote_ip[MAXHOSTNAMELEN];
+
+		strncpy(remote_ip, inet_ntoa(fromp->sin_addr),
+			sizeof(remote_ip) - 1);
+		remote_ip[sizeof(remote_ip) - 1] = 0;
+		if (!auth_hostok(lc, fromhost, remote_ip)) {
+			syslog(LOG_INFO|LOG_AUTH,
+			    "%s@%s as %s: permission denied (%s). cmd='%.80s'",
+			    remuser, hostname, locuser, __rcmd_errstr,
+			    cmdbuf);
+			error("Login incorrect.\n");
+			exit(1);
+		}
+		if (!auth_timeok(lc, time(NULL))) {
+			error("Logins not available right now\n");
+			exit(1);
+		}
+	}
+#endif	/* !LOGIN_CAP */
+#if	BSD > 43
+	/* before fork, while we're session leader */
+	if (setlogin(pwd->pw_name) < 0)
+		syslog(LOG_ERR, "setlogin() failed: %m");
+#endif
 
 	(void) write(STDERR_FILENO, "\0", 1);
 	sent_null = 1;
@@ -520,7 +576,8 @@ fail:
 				static char msg[] = SECURE_MESSAGE;
 				(void) close(pv1[1]);
 				(void) close(pv2[1]);
-				des_write(s, msg, sizeof(msg) - 1);
+				des_enc_write(s, msg, sizeof(msg) - 1, 
+					schedule, &kdata->session);
 
 			} else
 #endif
@@ -576,7 +633,8 @@ fail:
 #ifdef CRYPT
 #ifdef KERBEROS
 					if (doencrypt)
-						ret = des_read(s, &sig, 1);
+						ret = des_enc_read(s, &sig, 1,
+						schedule, &kdata->session);
 					else
 #endif
 #endif
@@ -597,7 +655,8 @@ fail:
 #ifdef KERBEROS
 						if (doencrypt)
 							(void)
-							  des_write(s, buf, cc);
+							  des_enc_write(s, buf, cc,
+								schedule, &kdata->session);
 						else
 #endif
 #endif
@@ -614,14 +673,16 @@ fail:
 						shutdown(pv1[0], 1+1);
 						FD_CLR(pv1[0], &readfrom);
 					} else
-						(void) des_write(STDOUT_FILENO,
-						    buf, cc);
+						(void) des_enc_write(STDOUT_FILENO,
+						    buf, cc,
+							schedule, &kdata->session);
 				}
 
 				if (doencrypt && FD_ISSET(pv2[0], &wready)) {
 					errno = 0;
-					cc = des_read(STDIN_FILENO,
-					    buf, sizeof(buf));
+					cc = des_enc_read(STDIN_FILENO,
+					    buf, sizeof(buf),
+						schedule, &kdata->session);
 					if (cc <= 0) {
 						shutdown(pv2[0], 1+1);
 						FD_CLR(pv2[0], &writeto);
@@ -659,13 +720,6 @@ fail:
 	}
 	if (*pwd->pw_shell == '\0')
 		pwd->pw_shell = _PATH_BSHELL;
-#if	BSD > 43
-	if (setlogin(pwd->pw_name) < 0)
-		syslog(LOG_ERR, "setlogin() failed: %m");
-#endif
-	(void) setgid((gid_t)pwd->pw_gid);
-	initgroups(pwd->pw_name, pwd->pw_gid);
-	(void) setuid((uid_t)pwd->pw_uid);
 	environ = envinit;
 	strncat(homedir, pwd->pw_dir, sizeof(homedir)-6);
 	strcat(path, _PATH_DEFPATH);
@@ -676,6 +730,17 @@ fail:
 		cp++;
 	else
 		cp = pwd->pw_shell;
+#ifdef	LOGIN_CAP
+	if (setusercontext(lc, pwd, pwd->pw_uid, LOGIN_SETALL) != 0) {
+                syslog(LOG_ERR, "setusercontext: %m");
+		exit(1);
+	}
+	login_close(lc);
+#else
+	(void) setgid((gid_t)pwd->pw_gid);
+	initgroups(pwd->pw_name, pwd->pw_gid);
+	(void) setuid((uid_t)pwd->pw_uid);
+#endif
 	endpwent();
 	if (log_success || pwd->pw_uid == 0) {
 #ifdef	KERBEROS
