@@ -32,7 +32,11 @@
  */
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)print.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/talkd/print.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /* debug print routines */
@@ -40,8 +44,8 @@ static char sccsid[] = "@(#)print.c	8.1 (Berkeley) 6/4/93";
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <protocols/talkd.h>
-#include <syslog.h>
 #include <stdio.h>
+#include <syslog.h>
 
 static	char *types[] =
     { "leave_invite", "look_up", "delete", "announce" };
@@ -51,6 +55,7 @@ static	char *answers[] =
       "unknown_request", "badversion", "badaddr", "badctladdr" };
 #define	NANSWERS	(sizeof (answers) / sizeof (answers[0]))
 
+void
 print_request(cp, mp)
 	char *cp;
 	register CTL_MSG *mp;
@@ -58,7 +63,7 @@ print_request(cp, mp)
 	char tbuf[80], *tp;
 
 	if (mp->type > NTYPES) {
-		(void)sprintf(tbuf, "type %d", mp->type);
+		(void)snprintf(tbuf, sizeof(tbuf), "type %d", mp->type);
 		tp = tbuf;
 	} else
 		tp = types[mp->type];
@@ -66,6 +71,7 @@ print_request(cp, mp)
 	    cp, tp, mp->id_num, mp->l_name, mp->r_name, mp->r_tty);
 }
 
+void
 print_response(cp, rp)
 	char *cp;
 	register CTL_RESPONSE *rp;
@@ -73,12 +79,12 @@ print_response(cp, rp)
 	char tbuf[80], *tp, abuf[80], *ap;
 
 	if (rp->type > NTYPES) {
-		(void)sprintf(tbuf, "type %d", rp->type);
+		(void)snprintf(tbuf, sizeof(tbuf), "type %d", rp->type);
 		tp = tbuf;
 	} else
 		tp = types[rp->type];
 	if (rp->answer > NANSWERS) {
-		(void)sprintf(abuf, "answer %d", rp->answer);
+		(void)snprintf(abuf, sizeof(abuf), "answer %d", rp->answer);
 		ap = abuf;
 	} else
 		ap = answers[rp->answer];
