@@ -32,7 +32,11 @@
  */
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)sys_term.c	8.2 (Berkeley) 12/15/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/telnetd/sys_term.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include "telnetd.h"
@@ -185,6 +189,12 @@ int ttyfd = -1;
 # endif
 #endif	/* USE_TERMIO */
 
+#include <sys/types.h>
+#include <libutil.h>
+
+int cleanopen __P((char *));
+void scrub_env __P((void));
+
 /*
  * init_termbuf()
  * copy_termbuf(cp)
@@ -507,11 +517,11 @@ int *ptynum;
 #endif
 
 #ifndef	__hpux
-	(void) sprintf(line, "/dev/ptyXX");
+	(void) strcpy(line, "/dev/ptyXX");
 	p1 = &line[8];
 	p2 = &line[9];
 #else
-	(void) sprintf(line, "/dev/ptym/ptyXX");
+	(void) strcpy(line, "/dev/ptym/ptyXX");
 	p1 = &line[13];
 	p2 = &line[14];
 #endif
@@ -1044,10 +1054,9 @@ extern void utmp_sig_notify P((int));
  * getptyslave()
  *
  * Open the slave side of the pty, and do any initialization
- * that is necessary.  The return value is a file descriptor
- * for the slave side.
+ * that is necessary.
  */
-	int
+	void
 getptyslave()
 {
 	register int t = -1;
@@ -1367,7 +1376,6 @@ startslave(host, autologin, autoname)
 {
 	register int i;
 	long time();
-	char name[256];
 #ifdef	NEWINIT
 	extern char *ptyip;
 	struct init_request request;
@@ -1442,7 +1450,7 @@ startslave(host, autologin, autoname)
 	 */
 	if ((i = open(INIT_FIFO, O_WRONLY)) < 0) {
 		char tbuf[128];
-		(void) sprintf(tbuf, "Can't open %s\n", INIT_FIFO);
+		(void) snprintf(tbuf, sizeof(tbuf), "Can't open %s\n", INIT_FIFO);
 		fatalperror(net, tbuf);
 	}
 	memset((char *)&request, 0, sizeof(request));
@@ -1465,7 +1473,7 @@ startslave(host, autologin, autoname)
 #endif /* BFTPDAEMON */
 	if (write(i, (char *)&request, sizeof(request)) < 0) {
 		char tbuf[128];
-		(void) sprintf(tbuf, "Can't write to %s\n", INIT_FIFO);
+		(void) snprintf(tbuf, sizeof(tbuf), "Can't write to %s\n", INIT_FIFO);
 		fatalperror(net, tbuf);
 	}
 	(void) close(i);
@@ -1477,7 +1485,7 @@ startslave(host, autologin, autoname)
 		if (i == 3 || n >= 0 || !gotalarm)
 			break;
 		gotalarm = 0;
-		sprintf(tbuf, "telnetd: waiting for /etc/init to start login process on %s\r\n", line);
+		snprintf(tbuf, sizeof(tbuf), "telnetd: waiting for /etc/init to start login process on %s\r\n", line);
 		(void) write(net, tbuf, strlen(tbuf));
 	}
 	if (n < 0 && gotalarm)
@@ -1500,7 +1508,7 @@ init_env()
 	char **envp;
 
 	envp = envinit;
-	if (*envp = getenv("TZ"))
+	if ((*envp = getenv("TZ")))
 		*envp++ -= 3;
 #if	defined(CRAY) || defined(__hpux)
 	else
@@ -1525,9 +1533,8 @@ start_login(host, autologin, name)
 	int autologin;
 	char *name;
 {
-	register char *cp;
 	register char **argv;
-	char **addarg(), *user;
+	char **addarg();
 	extern char *getenv();
 #ifdef	UTMPX
 	register int pid = getpid();
@@ -1673,7 +1680,8 @@ start_login(host, autologin, name)
 			len = strlen(name)+1;
 			write(xpty, name, len);
 			write(xpty, name, len);
-			sprintf(speed, "%s/%d", (cp = getenv("TERM")) ? cp : "",
+			snprintf(speed, sizeof(speed),
+				"%s/%d", (cp = getenv("TERM")) ? cp : "",
 				(def_rspeed > 0) ? def_rspeed : 9600);
 			len = strlen(speed)+1;
 			write(xpty, speed, len);
@@ -1734,7 +1742,7 @@ start_login(host, autologin, name)
 	}
 	execv(altlogin, argv);
 
-	syslog(LOG_ERR, "%s: %m\n", altlogin);
+	syslog(LOG_ERR, "%s: %m", altlogin);
 	fatalperror(net, altlogin);
 	/*NOTREACHED*/
 }
@@ -1779,6 +1787,7 @@ addarg(argv, val)
  * Remove a few things from the environment that
  * don't need to be there.
  */
+	void
 scrub_env()
 {
 	register char **cpp, **cpp2;
@@ -1786,7 +1795,6 @@ scrub_env()
 	for (cpp2 = cpp = environ; *cpp; cpp++) {
 #ifdef __FreeBSD__
 		if (strncmp(*cpp, "LD_LIBRARY_PATH=", 16) &&
-		    strncmp(*cpp, "LD_NOSTD_PATH=", 14) &&
 		    strncmp(*cpp, "LD_PRELOAD=", 11) &&
 #else
 		if (strncmp(*cpp, "LD_", 3) &&
@@ -1999,7 +2007,7 @@ cleantmp(wtp)
 
 	utp = getutid(wtp);
 	if (utp == 0) {
-		syslog(LOG_ERR, "Can't get /etc/utmp entry to clean TMPDIR");
+		syslog(LOG_ERR, "can't get /etc/utmp entry to clean TMPDIR");
 		return(-1);
 	}
 	/*
@@ -2051,12 +2059,12 @@ cleantmpdir(jid, tpath, user)
 {
 	switch(fork()) {
 	case -1:
-		syslog(LOG_ERR, "TMPDIR cleanup(%s): fork() failed: %m\n",
+		syslog(LOG_ERR, "TMPDIR cleanup(%s): fork() failed: %m",
 							tpath);
 		break;
 	case 0:
 		execl(CLEANTMPCMD, CLEANTMPCMD, user, tpath, 0);
-		syslog(LOG_ERR, "TMPDIR cleanup(%s): execl(%s) failed: %m\n",
+		syslog(LOG_ERR, "TMPDIR cleanup(%s): execl(%s) failed: %m",
 							tpath, CLEANTMPCMD);
 		exit(1);
 	default:
