@@ -32,13 +32,17 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1989, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)telnetd.c	8.2 (Berkeley) 12/15/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/telnetd/telnetd.c 80208 2001-07-23 14:13:45Z ru $";
 #endif /* not lint */
 
 #include "telnetd.h"
@@ -53,6 +57,11 @@ static char sccsid[] = "@(#)telnetd.c	8.2 (Berkeley) 12/15/93";
 # undef _SC_CRAY_SECURE_SYS
 #endif
 
+#include <err.h>
+#include <arpa/inet.h>
+
+#include <sys/mman.h>
+
 #if	defined(_SC_CRAY_SECURE_SYS)
 #include <sys/sysv.h>
 #include <sys/secdev.h>
@@ -123,9 +132,11 @@ int	lowpty = 0, highpty;	/* low, high pty numbers */
 
 int debug = 0;
 int keepalive = 1;
-char *progname;
 char *altlogin;
 
+void doit __P((struct sockaddr_in *));
+int terminaltypeok __P((char *));
+void startslave __P((char *, int, char *));
 extern void usage P((void));
 
 /*
@@ -159,14 +170,13 @@ char valid_opts[] = {
 	'\0'
 };
 
+	int
 main(argc, argv)
 	char *argv[];
 {
 	struct sockaddr_in from;
 	int on = 1, fromlen;
 	register int ch;
-	extern char *optarg;
-	extern int optind;
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 	int tos = -1;
 #endif
@@ -175,7 +185,21 @@ main(argc, argv)
 	netip = netibuf;
 	nfrontp = nbackp = netobuf;
 
-	progname = *argv;
+	/*
+	 * This initialization causes linemode to default to a configuration
+	 * that works on all telnet clients, including the FreeBSD client.
+	 * This is not quite the same as the telnet client issuing a "mode
+	 * character" command, but has most of the same benefits, and is
+	 * preferable since some clients (like usofts) don't have the
+	 * mode character command anyway and linemode breaks things.
+	 * The most notable symptom of fix is that csh "set filec" operations
+	 * like <ESC> (filename completion) and ^D (choices) keys now work
+	 * in telnet sessions and can be used more than once on the same line.
+	 * CR/LF handling is also corrected in some termio modes.  This 
+	 * change resolves problem reports bin/771 and bin/1037.
+	 */
+
+	linemode=1;	/*Default to mode that works on bulk of clients*/
 
 #ifdef CRAY
 	/*
@@ -185,7 +209,7 @@ main(argc, argv)
 	highpty = getnpty();
 #endif /* CRAY */
 
-	while ((ch = getopt(argc, argv, valid_opts)) != EOF) {
+	while ((ch = getopt(argc, argv, valid_opts)) != -1) {
 		switch(ch) {
 
 #ifdef	AUTHENTICATION
@@ -210,8 +234,7 @@ main(argc, argv)
 				 */
 				auth_level = -1;
 			} else {
-				fprintf(stderr,
-			    "telnetd: unknown authorization level for -a\n");
+				warnx("unknown authorization level for -a");
 			}
 			break;
 #endif	/* AUTHENTICATION */
@@ -326,12 +349,11 @@ main(argc, argv)
 		case 'S':
 #ifdef	HAS_GETTOS
 			if ((tos = parsetos(optarg, "tcp")) < 0)
-				fprintf(stderr, "%s%s%s\n",
-					"telnetd: Bad TOS argument '", optarg,
+				warnx("%s%s%s",
+					"bad TOS argument '", optarg,
 					"'; will try to use default TOS");
 #else
-			fprintf(stderr, "%s%s\n", "TOS option unavailable; ",
-						"-S flag not supported\n");
+			warnx("TOS option unavailable; -S flag not supported");
 #endif
 			break;
 
@@ -353,7 +375,7 @@ main(argc, argv)
 #endif	/* AUTHENTICATION */
 
 		default:
-			fprintf(stderr, "telnetd: %c: unknown option\n", ch);
+			warnx("%c: unknown option", ch);
 			/* FALLTHROUGH */
 		case '?':
 			usage();
@@ -373,12 +395,12 @@ main(argc, argv)
 		usage();
 		/* NOT REACHED */
 	    } else if (argc == 1) {
-		    if (sp = getservbyname(*argv, "tcp")) {
+		    if ((sp = getservbyname(*argv, "tcp"))) {
 			sin.sin_port = sp->s_port;
 		    } else {
 			sin.sin_port = atoi(*argv);
 			if ((int)sin.sin_port <= 0) {
-			    fprintf(stderr, "telnetd: %s: bad port #\n", *argv);
+			    warnx("%s: bad port #", *argv);
 			    usage();
 			    /* NOT REACHED */
 			}
@@ -386,34 +408,24 @@ main(argc, argv)
 		   }
 	    } else {
 		sp = getservbyname("telnet", "tcp");
-		if (sp == 0) {
-		    fprintf(stderr, "telnetd: tcp/telnet: unknown service\n");
-		    exit(1);
-		}
+		if (sp == 0)
+		    errx(1, "tcp/telnet: unknown service");
 		sin.sin_port = sp->s_port;
 	    }
 
 	    s = socket(AF_INET, SOCK_STREAM, 0);
-	    if (s < 0) {
-		    perror("telnetd: socket");;
-		    exit(1);
-	    }
+	    if (s < 0)
+		    err(1, "socket");
 	    (void) setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 				(char *)&on, sizeof(on));
-	    if (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0) {
-		perror("bind");
-		exit(1);
-	    }
-	    if (listen(s, 1) < 0) {
-		perror("listen");
-		exit(1);
-	    }
+	    if (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0)
+		err(1, "bind");
+	    if (listen(s, 1) < 0)
+		err(1, "listen");
 	    foo = sizeof sin;
 	    ns = accept(s, (struct sockaddr *)&sin, &foo);
-	    if (ns < 0) {
-		perror("accept");
-		exit(1);
-	    }
+	    if (ns < 0)
+		err(1, "accept");
 	    (void) dup2(ns, 0);
 	    (void) close(ns);
 	    (void) close(s);
@@ -441,10 +453,8 @@ main(argc, argv)
 
 		bzero((char *)&dv, sizeof(dv));
 
-		if (getsysv(&sysv, sizeof(struct sysv)) != 0) {
-			perror("getsysv");
-			exit(1);
-		}
+		if (getsysv(&sysv, sizeof(struct sysv)) != 0)
+			err(1, "getsysv");
 
 		/*
 		 *	Get socket security label and set device values
@@ -455,8 +465,7 @@ main(argc, argv)
 			       (char *)&ss, &szss) < 0) ||
 		    (getsockopt(0, SOL_SOCKET, SO_SEC_MULTI,
 				(char *)&sock_multi, &szi) < 0)) {
-			perror("getsockopt");
-			exit(1);
+			err(1, "getsockopt");
 		} else {
 			dv.dv_actlvl = ss.ss_actlabel.lt_level;
 			dv.dv_actcmp = ss.ss_actlabel.lt_compart;
@@ -486,8 +495,7 @@ main(argc, argv)
 	openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
 	fromlen = sizeof (from);
 	if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
-		fprintf(stderr, "%s: ", progname);
-		perror("getpeername");
+		warn("getpeername");
 		_exit(1);
 	}
 	if (keepalive &&
@@ -515,12 +523,13 @@ main(argc, argv)
 	net = 0;
 	doit(&from);
 	/* NOTREACHED */
+	return(0);
 }  /* end of main */
 
 	void
 usage()
 {
-	fprintf(stderr, "Usage: telnetd");
+	fprintf(stderr, "usage: telnetd");
 #ifdef	AUTHENTICATION
 	fprintf(stderr, " [-a (debug|other|user|valid|off|none)]\n\t");
 #endif
@@ -610,34 +619,29 @@ getterminaltype(name)
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
 
-	bcopy(sb, nfrontp, sizeof sb);
-	nfrontp += sizeof sb;
+	output_datalen(sb, sizeof sb);
     }
     if (his_state_is_will(TELOPT_XDISPLOC)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE };
 
-	bcopy(sb, nfrontp, sizeof sb);
-	nfrontp += sizeof sb;
+	output_datalen(sb, sizeof sb);
     }
     if (his_state_is_will(TELOPT_NEW_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_NEW_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	bcopy(sb, nfrontp, sizeof sb);
-	nfrontp += sizeof sb;
+	output_datalen(sb, sizeof sb);
     }
     else if (his_state_is_will(TELOPT_OLD_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_OLD_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	bcopy(sb, nfrontp, sizeof sb);
-	nfrontp += sizeof sb;
+	output_datalen(sb, sizeof sb);
     }
     if (his_state_is_will(TELOPT_TTYPE)) {
 
-	bcopy(ttytype_sbbuf, nfrontp, sizeof ttytype_sbbuf);
-	nfrontp += sizeof ttytype_sbbuf;
+	output_datalen(ttytype_sbbuf, sizeof ttytype_sbbuf);
     }
     if (his_state_is_will(TELOPT_TSPEED)) {
 	while (sequenceIs(tspeedsubopt, baseline))
@@ -714,8 +718,7 @@ _gettermname()
     if (his_state_is_wont(TELOPT_TTYPE))
 	return;
     settimer(baseline);
-    bcopy(ttytype_sbbuf, nfrontp, sizeof ttytype_sbbuf);
-    nfrontp += sizeof ttytype_sbbuf;
+    output_datalen(ttytype_sbbuf, sizeof ttytype_sbbuf);
     while (sequenceIs(ttypesubopt, baseline))
 	ttloop();
 }
@@ -756,11 +759,11 @@ char user_name[256];
 /*
  * Get a pty, scan input lines.
  */
+	void
 doit(who)
 	struct sockaddr_in *who;
 {
-	char *host, *inet_ntoa();
-	int t;
+	char *host = NULL;
 	struct hostent *hp;
 	int ptynum;
 
@@ -885,6 +888,11 @@ telnet(f, p, host)
 	char *HE;
 	char *HN;
 	char *IM;
+	char *IF;
+	char *if_buf;
+	int if_fd;
+	struct stat statbuf;
+
 	void netflush();
 
 	/*
@@ -970,9 +978,7 @@ telnet(f, p, host)
 	 * mode, which we do not want.
 	 */
 	if (his_want_state_is_will(TELOPT_ECHO)) {
-		DIAG(TD_OPTIONS,
-			{sprintf(nfrontp, "td: simulating recv\r\n");
-			 nfrontp += strlen(nfrontp);});
+		DIAG(TD_OPTIONS, output_data("td: simulating recv\r\n"));
 		willoption(TELOPT_ECHO);
 	}
 
@@ -1038,9 +1044,9 @@ telnet(f, p, host)
 	 * side.  Set up signal handler now.
 	 */
 	if ((int)signal(SIGUSR1, termstat) < 0)
-		perror("signal");
+		warn("signal");
 	else if (ioctl(p, TCSIGME, (char *)SIGUSR1) < 0)
-		perror("ioctl:TCSIGME");
+		warn("ioctl:TCSIGME");
 	/*
 	 * Make processing loop check terminal characteristics early on.
 	 */
@@ -1083,8 +1089,11 @@ telnet(f, p, host)
 		HE = getstr("he", &cp);
 		HN = getstr("hn", &cp);
 		IM = getstr("im", &cp);
+		IF = getstr("if", &cp);
 		if (HN && *HN)
 			(void) strcpy(host_name, HN);
+		if (IF && (if_fd = open(IF, O_RDONLY, 000)) != -1)
+			IM = 0;
 		if (IM == 0)
 			IM = "";
 	} else {
@@ -1094,6 +1103,14 @@ telnet(f, p, host)
 	edithost(HE, host_name);
 	if (hostinfo && *IM)
 		putf(IM, ptyibuf2);
+	else if (IF && if_fd != -1) {
+		fstat (if_fd, &statbuf);
+		if_buf = (char *) mmap (0, statbuf.st_size, PROT_READ, 
+		    0, if_fd, 0);
+		putf(if_buf, ptyibuf2);
+		munmap (if_buf, statbuf.st_size);
+		close (if_fd);
+	}
 
 	if (pcc)
 		(void) strncat(ptyibuf2, ptyip, pcc+1);
@@ -1107,9 +1124,7 @@ telnet(f, p, host)
 	localstat();
 #endif	/* LINEMODE */
 
-	DIAG(TD_REPORT,
-		{sprintf(nfrontp, "td: Entering processing loop\r\n");
-		 nfrontp += strlen(nfrontp);});
+	DIAG(TD_REPORT, output_data("td: Entering processing loop\r\n"));
 
 	/*
 	 * Startup the login process on the slave side of the terminal
@@ -1237,8 +1252,7 @@ telnet(f, p, host)
 			netip = netibuf;
 		    }
 		    DIAG((TD_REPORT | TD_NETDATA),
-			    {sprintf(nfrontp, "td: netread %d chars\r\n", ncc);
-			     nfrontp += strlen(nfrontp);});
+			output_data("td: netread %d chars\r\n", ncc));
 		    DIAG(TD_NETDATA, printdata("nd", netip, ncc));
 		}
 
@@ -1285,8 +1299,7 @@ telnet(f, p, host)
 					 * royally if we send them urgent
 					 * mode data.
 					 */
-					*nfrontp++ = IAC;
-					*nfrontp++ = DM;
+					output_data("%c%c", IAC, DM);
 					neturg = nfrontp-1; /* off by one XXX */
 #endif
 				}
@@ -1297,13 +1310,11 @@ telnet(f, p, host)
 					    ptyibuf[0] & TIOCPKT_DOSTOP ? 1 : 0;
 					if (newflow != flowmode) {
 						flowmode = newflow;
-						(void) sprintf(nfrontp,
-							"%c%c%c%c%c%c",
+						output_data("%c%c%c%c%c%c",
 							IAC, SB, TELOPT_LFLOW,
 							flowmode ? LFLOW_ON
 								 : LFLOW_OFF,
 							IAC, SE);
-						nfrontp += 6;
 					}
 				}
 				pcc--;
@@ -1326,19 +1337,19 @@ telnet(f, p, host)
 				break;
 			c = *ptyip++ & 0377, pcc--;
 			if (c == IAC)
-				*nfrontp++ = c;
+				output_data("%c", c);
 #if	defined(CRAY2) && defined(UNICOS5)
 			else if (c == '\n' &&
 				     my_state_is_wont(TELOPT_BINARY) && newmap)
-				*nfrontp++ = '\r';
+				output_data("\r");
 #endif	/* defined(CRAY2) && defined(UNICOS5) */
-			*nfrontp++ = c;
+			output_data("%c", c);
 			if ((c == '\r') && (my_state_is_wont(TELOPT_BINARY))) {
 				if (pcc > 0 && ((*ptyip & 0377) == '\n')) {
-					*nfrontp++ = *ptyip++ & 0377;
+					output_data("%c", *ptyip++ & 0377);
 					pcc--;
 				} else
-					*nfrontp++ = '\0';
+					output_data("%c", '\0');
 			}
 		}
 #if	defined(CRAY2) && defined(UNICOS5)
@@ -1523,8 +1534,7 @@ recv_ayt()
 		return;
 	}
 #endif
-	(void) strcpy(nfrontp, "\r\n[Yes]\r\n");
-	nfrontp += 9;
+	output_data("\r\n[Yes]\r\n");
 }
 
 	void
