@@ -35,13 +35,17 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1985, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)uucpd.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/uucpd/uucpd.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
@@ -53,22 +57,30 @@ static char sccsid[] = "@(#)uucpd.c	8.1 (Berkeley) 6/4/93";
 #include <sys/wait.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/param.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <time.h>
-#include <pwd.h>
-#include <unistd.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <netdb.h>
+#include <pwd.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <utmp.h>
 #include <syslog.h>
+#include <time.h>
+#include <unistd.h>
+#include <utmp.h>
+#include <sys/param.h>
 #include "pathnames.h"
 
+#if (MAXLOGNAME-1) > UT_NAMESIZE
+#define LOGNAMESIZE UT_NAMESIZE
+#else
+#define LOGNAMESIZE (MAXLOGNAME-1)
+#endif
+
 #define	SCPYN(a, b)	strncpy(a, b, sizeof (a))
 
 struct	sockaddr_in hisctladdr;
@@ -108,7 +120,7 @@ void main(int argc, char **argv)
 
 void badlogin(char *name, struct sockaddr_in *sin)
 {
-	char remotehost[32];
+	char remotehost[MAXHOSTNAMELEN];
 	struct hostent *hp = gethostbyaddr((char *)&sin->sin_addr,
 		sizeof (struct in_addr), AF_INET);
 
@@ -119,6 +131,8 @@ void badlogin(char *name, struct sockaddr_in *sin)
 		strncpy(remotehost, inet_ntoa(sin->sin_addr),
 		    sizeof (remotehost));
 
+	remotehost[sizeof remotehost - 1] = '\0';
+
 	syslog(LOG_NOTICE, "LOGIN FAILURE FROM %s", remotehost);
 	syslog(LOG_AUTHPRIV|LOG_NOTICE,
 	    "LOGIN FAILURE FROM %s, %s", remotehost, name);
@@ -127,24 +141,13 @@ void badlogin(char *name, struct sockaddr_in *sin)
 	exit(1);
 }
 
-void login_incorrect(char *name, struct sockaddr_in *sinp)
-{
-	char passwd[64];
-
-	printf("Password: "); fflush(stdout);
-	if (readline(passwd, sizeof passwd, 1) < 0) {
-		syslog(LOG_WARNING, "passwd read: %m");
-		_exit(1);
-	}
-	badlogin(name, sinp);
-}
-
 void doit(struct sockaddr_in *sinp)
 {
 	char user[64], passwd[64];
 	char *xpasswd, *crypt();
 	struct passwd *pw;
 	pid_t s;
+	int pwdok =0;
 
 	alarm(60);
 	printf("login: "); fflush(stdout);
@@ -152,23 +155,32 @@ void doit(struct sockaddr_in *sinp)
 		syslog(LOG_WARNING, "login read: %m");
 		_exit(1);
 	}
-	/* truncate username to 8 characters */
-	user[8] = '\0';
+	/* truncate username to LOGNAMESIZE characters */
+	user[LOGNAMESIZE] = '\0';
 	pw = getpwnam(user);
-	if (pw == NULL)
-		login_incorrect(user, sinp);
-	if (strcmp(pw->pw_shell, _PATH_UUCICO))
-		login_incorrect(user, sinp);
-	if (pw->pw_expire && time(NULL) >= pw->pw_expire)
-		login_incorrect(user, sinp);
-	if (pw->pw_passwd && *pw->pw_passwd != '\0') {
+	/*
+	 * Fail after password if:
+	 * 1. Invalid user
+	 * 2. Shell is not uucico
+	 * 3. Account has expired
+	 * 4. Password is incorrect
+	 */
+	if (pw != NULL && strcmp(pw->pw_shell, _PATH_UUCICO) == 0 &&
+	    (!pw->pw_expire || time(NULL) >= pw->pw_expire))
+		pwdok = 1;
+	/* always ask for passwords to deter account guessing */
+	if (!pwdok || (pw->pw_passwd && *pw->pw_passwd != '\0')) {
 		printf("Password: "); fflush(stdout);
 		if (readline(passwd, sizeof passwd, 1) < 0) {
 			syslog(LOG_WARNING, "passwd read: %m");
 			_exit(1);
 		}
-		xpasswd = crypt(passwd, pw->pw_passwd);
-		if (strcmp(xpasswd, pw->pw_passwd))
+		if (pwdok) {
+			xpasswd = crypt(passwd, pw->pw_passwd);
+			if (strcmp(xpasswd, pw->pw_passwd))
+				pwdok = 0;
+		}
+		if (!pwdok)
 			badlogin(user, sinp);
 	}
 	alarm(0);
@@ -224,7 +236,7 @@ void dologout(void)
 	char line[32];
 
 	while ((pid=wait((int *)&status)) > 0) {
-		sprintf(line, "uucp%ld", pid);
+		sprintf(line, "uucp%ld", (long)pid);
 		logwtmp(line, "", "");
 	}
 }
@@ -235,7 +247,7 @@ void dologout(void)
 void dologin(struct passwd *pw, struct sockaddr_in *sin)
 {
 	char line[32];
-	char remotehost[32];
+	char remotehost[MAXHOSTNAMELEN];
 	int f;
 	time_t cur_time;
 	struct hostent *hp = gethostbyaddr((char *)&sin->sin_addr,
@@ -247,8 +259,9 @@ void dologin(struct passwd *pw, struct sockaddr_in *sin)
 	} else
 		strncpy(remotehost, inet_ntoa(sin->sin_addr),
 		    sizeof (remotehost));
+	remotehost[sizeof remotehost - 1] = '\0';
 	/* hack, but must be unique and no tty line */
-	sprintf(line, "uucp%ld", getpid());
+	sprintf(line, "uucp%ld", (long)getpid());
 	time(&cur_time);
 	if ((f = open(_PATH_LASTLOG, O_RDWR)) >= 0) {
 		struct lastlog ll;
