@@ -29,6 +29,11 @@
  * SUCH DAMAGE.
  */
 
+#ifndef lint
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/xtend/packet.c 50952 1999-09-05 11:54:50Z peter $";
+#endif /* not lint */
+
 #include <stdio.h>
 #include <sys/time.h>
 #include "xtend.h"
@@ -53,6 +58,7 @@ char *X10cmdnames[] = {
  * Log a packet and update device status accordingly
  */
 
+void
 logpacket(p)
 unsigned char *p;
 {
@@ -70,6 +76,7 @@ unsigned char *p;
  * in core and on disk.
  */
 
+void
 processpacket(p)
 unsigned char *p;
 {
