@@ -29,6 +29,11 @@
  * SUCH DAMAGE.
  */
 
+#ifndef lint
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/libexec/xtend/status.c 50952 1999-09-05 11:54:50Z peter $";
+#endif /* not lint */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/param.h>
@@ -37,14 +42,15 @@
 #include "xten.h"
 #include "paths.h"
 
+void printstatus __P((FILE *, STATUS *));
+
 /*
  * Initialize the status table from the status files
  */
 
+void
 initstatus()
 {
-  int h, i;
-
   if(lseek(status, 0, SEEK_SET) != 0) {
     fprintf(Log, "%s:  Seek error on status file\n", thedate());
     return;
@@ -60,6 +66,7 @@ initstatus()
  * and notify anyone monitoring those devices.
  */
 
+void
 checkpoint_status()
 {
   int h, i, k, offset;
