@@ -25,22 +25,20 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/lkm/syscons/green/green_saver.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/lkm/syscons/green/green_saver.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/conf.h>
 #include <sys/exec.h>
 #include <sys/sysent.h>
 #include <sys/lkm.h>
-#include <sys/errno.h>
+
+#include <i386/isa/isa.h>
+
 #include <saver.h>
 
-MOD_MISC("green_saver")
-
-void (*current_saver)();
-void (*old_saver)();
+MOD_MISC(green_saver);
 
 static void
 green_saver(int blank)
@@ -48,38 +46,82 @@ green_saver(int blank)
 	u_char val;
 	if (blank) {
 		scrn_blanked = 1;
-		outb(TSIDX, 0x01); val = inb(TSREG);
-		outb(TSIDX, 0x01); outb(TSREG, val | 0x20);
-		outb(crtc_addr, 0x17); val = inb(crtc_addr + 1);
-		outb(crtc_addr + 1, val & ~0x80);
+		switch (crtc_type) {
+		case KD_VGA:
+			outb(TSIDX, 0x01); val = inb(TSREG);
+			outb(TSIDX, 0x01); outb(TSREG, val | 0x20);
+			outb(crtc_addr, 0x17); val = inb(crtc_addr + 1);
+			outb(crtc_addr + 1, val & ~0x80);
+			break;
+		case KD_EGA:
+			/* not yet done XXX */
+			break;
+		case KD_CGA:
+			outb(crtc_addr + 4, 0x25);
+			break;
+		case KD_MONO:
+		case KD_HERCULES:
+			outb(crtc_addr + 4, 0x21);
+			break;
+		default:
+			break;
+		}
 	}
 	else {
-		outb(TSIDX, 0x01); val = inb(TSREG);
-		outb(TSIDX, 0x01); outb(TSREG, val & 0xDF);
-		outb(crtc_addr, 0x17); val = inb(crtc_addr + 1);
-		outb(crtc_addr + 1, val | 0x80);
+		switch (crtc_type) {
+		case KD_VGA:
+			outb(TSIDX, 0x01); val = inb(TSREG);
+			outb(TSIDX, 0x01); outb(TSREG, val & 0xDF);
+			outb(crtc_addr, 0x17); val = inb(crtc_addr + 1);
+			outb(crtc_addr + 1, val | 0x80);
+			break;
+		case KD_EGA:
+			/* not yet done XXX */
+			break;
+		case KD_CGA:
+			outb(crtc_addr + 4, 0x2d);
+			break;
+		case KD_MONO:
+		case KD_HERCULES:
+			outb(crtc_addr + 4, 0x29);
+			break;
+		default:
+			break;
+		}
 		scrn_blanked = 0;
 	}
 }
 
-saver_load(struct lkm_table *lkmtp, int cmd)
+static int
+green_saver_load(struct lkm_table *lkmtp, int cmd)
 {
-	(*current_saver)(0);
-	old_saver = current_saver;
-	current_saver = green_saver;
-	uprintf("green screen saver installed\n");
-	return 0;
+	switch (crtc_type) {
+	case KD_MONO:
+	case KD_HERCULES:
+	case KD_CGA:
+		/* 
+		 * `green' saver is not fully implemented for MDA and CGA.
+		 * It simply blanks the display instead.
+		 */
+	case KD_VGA:
+		break;
+	case KD_EGA:
+		/* EGA is yet to be supported */
+	default:
+		return ENODEV;
+	}
+	return add_scrn_saver(green_saver);
 }
 
-saver_unload(struct lkm_table *lkmtp, int cmd)
+static int
+green_saver_unload(struct lkm_table *lkmtp, int cmd)
 {
-	(*current_saver)(0);
-	current_saver = old_saver;
-	uprintf("green screen saver removed\n");
-	return 0;
+	return remove_scrn_saver(green_saver);
 }
 
-saver_init(struct lkm_table *lkmtp, int cmd, int ver)
+int
+green_saver_mod(struct lkm_table *lkmtp, int cmd, int ver)
 {
-	DISPATCH(lkmtp, cmd, ver, saver_load, saver_unload, nosys);
+	MOD_DISPATCH(green_saver, lkmtp, cmd, ver,
+		green_saver_load, green_saver_unload, lkm_nullcmd);
 }
