@@ -1,7 +1,9 @@
 :
 #set -ex
 
-VNDEVICE=vn0
+if [ "x$VNDEVICE" = "x" ] ; then
+	VNDEVICE=vn0
+fi
 export BLOCKSIZE=512
 
 RD=$1 ; shift
@@ -45,9 +47,9 @@ do
 
 	( set -e && cd ${FSPROTO} && find . -print | cpio -dump ${MNT} )
 
-	df -ki /mnt
+	df -ki ${MNT}
 
-	set `df -ki /mnt | tail -1`
+	set `df -ki ${MNT} | tail -1`
 
 	umount ${MNT}
 
