@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/attr.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/attr.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -60,7 +60,7 @@ attr_parse(Attribs *attr, FILE *fp)
 {
     char hold_n[MAX_NAME+1];
     char hold_v[MAX_VALUE+1];
-    char buf[BUFSIZ];
+    char buf[BUFSIZ * 4];
     int bp, n, v, max;
     enum { LOOK, COMMENT, NAME, VALUE, MVALUE, COMMIT, FILL, STOP } state;
     int num_attribs;
@@ -98,7 +98,7 @@ attr_parse(Attribs *attr, FILE *fp)
 		state = COMMENT;
 		continue;
 	    }
-	    else if (isalpha(ch) || ch == '_') {
+	    else if (isalnum(ch) || ch == '_') {
 		if (n >= MAX_NAME) {
 		    msgDebug("Attribute name overflow at character %d, ignoring entry..\n", n);
 		    n = 0;
