@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/config.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/config.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -104,15 +104,7 @@ check_rootdev(Chunk **list, int n)
 static char *
 name_of(Chunk *c1)
 {
-    static char rootname[32];
-
-    /* Our boot blocks can't deal with root partitions on slices - need the compatbility name */
-    if (c1->type == part && c1->flags & CHUNK_IS_ROOT) {
-        sprintf(rootname, "%sa", c1->disk->name);
-        return rootname;
-    }
-    else
-        return c1->name;
+    return c1->name;
 }
 
 static char *
@@ -167,14 +159,16 @@ seq_num(Chunk *c1)
     if (c1->type == part && c1->subtype != FS_SWAP) {
 	if (rootdev_is_od == 0 && strncmp(c1->name, "od", 2) == 0)
 	    return 0;
-	else
+	else if (c1->flags & CHUNK_IS_ROOT)
 	    return 1;
+	else
+	    return 2;
     }
     return 0;
 }
 
 int
-configFstab(void)
+configFstab(dialogMenuItem *self)
 {
     Device **devs;
     Disk *disk;
@@ -219,49 +213,42 @@ configFstab(void)
     }
     chunk_list[nchunks] = 0;
     chunk_sort();
-
+    
     fstab = fopen("/etc/fstab", "w");
     if (!fstab) {
 	msgConfirm("Unable to create a new /etc/fstab file!  Manual intervention\n"
 		   "will be required.");
 	return DITEM_FAILURE;
     }
-
+    
     check_rootdev(chunk_list, nchunks);
-
+    
     /* Go for the burn */
     msgDebug("Generating /etc/fstab file\n");
     fprintf(fstab, "# Device\t\tMountpoint\tFStype\tOptions\t\tDump\tPass#\n");
     for (i = 0; i < nchunks; i++)
 	fprintf(fstab, "/dev/%s\t\t%s\t\t%s\t%s\t\t%d\t%d\n", name_of(chunk_list[i]), mount_point(chunk_list[i]),
 		fstype(chunk_list[i]), fstype_short(chunk_list[i]), seq_num(chunk_list[i]), seq_num(chunk_list[i]));
-    Mkdir("/proc");
-    fprintf(fstab, "proc\t\t\t/proc\t\tprocfs\trw\t\t0\t0\n");
-
+    
     /* Now look for the CDROMs */
     devs = deviceFind(NULL, DEVICE_TYPE_CDROM);
     cnt = deviceCount(devs);
-
-    /* Write the first one out as /cdrom */
-    if (cnt) {
-	if (Mkdir("/cdrom")) {
-	    msgConfirm("Unable to make mount point for: /cdrom");
-	}
-	else
-	    fprintf(fstab, "/dev/%s\t\t/cdrom\t\tcd9660\tro,noauto\t0\t0\n", devs[0]->name);
-    }
-
-    /* Write the others out as /cdrom<n> */
-    for (i = 1; i < cnt; i++) {
+    
+    /* Write out the CDROM entries */
+    for (i = 0; i < cnt; i++) {
 	char cdname[10];
-
-	sprintf(cdname, "/cdrom%d", i);
-	if (Mkdir(cdname)) {
+	
+	sprintf(cdname, "/cdrom%s", i ? itoa(i) : "");
+	if (Mkdir(cdname))
 	    msgConfirm("Unable to make mount point for: %s", cdname);
-	}
 	else
 	    fprintf(fstab, "/dev/%s\t\t%s\tcd9660\tro,noauto\t0\t0\n", devs[i]->name, cdname);
     }
+    
+    /* And finally, a /proc. */
+    fprintf(fstab, "proc\t\t\t/proc\t\tprocfs\trw\t\t0\t0\n");
+    Mkdir("/proc");
+
     fclose(fstab);
     if (isDebug())
 	msgDebug("Wrote out /etc/fstab file\n");
@@ -270,7 +257,7 @@ configFstab(void)
 
 /* Do the work of sucking in a config file.
  * config is the filename to read in.
- * lines is a fixed (max) sized array of char *.
+ * lines is a fixed (max) sized array of char*
  * returns number of lines read.  line contents
  * are malloc'd and must be freed by the caller.
  */
@@ -300,12 +287,12 @@ readConfig(char *config, char **lines, int max)
 
 #define MAX_LINES  2000 /* Some big number we're not likely to ever reach - I'm being really lazy here, I know */
 
-/* Load the environment from a sysconfig file */
+/* Load the environment from an rc.conf file */
 void
-configEnvironmentSysconfig(char *config)
+configEnvironmentRC_conf(char *config)
 {
     char *lines[MAX_LINES], *cp, *cp2;
-    int i, j, nlines;
+    int i, nlines;
 
     nlines = readConfig(config, lines, MAX_LINES);
     if (nlines == -1)
@@ -318,19 +305,22 @@ configEnvironmentSysconfig(char *config)
 	    continue;
 	}
 	*cp++ = '\0';
-	(void)string_prune(lines[i]);
-	cp = string_skipwhite(string_prune(cp));
-	if ((cp2 = index(cp, '"')))	/* Eliminate leading quote if it's quoted */
+	/* Find quotes */
+	if ((cp2 = index(cp, '"')) || (cp2 = index(cp, '\047'))) {
 	    cp = cp2 + 1;
-	j = strlen(cp) - 1;
-	if (cp[j] == '"') /* And trailing one */
-	    cp[j] = '\0';
-	if (strlen(cp))
-	    variable_set2(lines[i], cp);
+	    cp2 = index(cp, *cp2);
+	}
+	/* If valid quotes, use it */
+	if (cp2) {
+	    *cp2 = '\0';
+ 	    /* If we have a legit value and it's not already set, set it */
+	    if (strlen(cp) && !variable_get(lines[i]))
+		variable_set2(lines[i], cp);
+	}
 	free(lines[i]);
     }
 }
-    
+
 /* Load the environment from a resolv.conf file */
 void
 configEnvironmentResolv(char *config)
@@ -342,30 +332,37 @@ configEnvironmentResolv(char *config)
     if (nlines == -1)
 	return;
     for (i = 0; i < nlines; i++) {
-	Boolean name_set = FALSE;
+	Boolean name_set = (Boolean)variable_get(VAR_NAMESERVER);
 
-	if (!strncmp(lines[i], "domain", 6))
-	    variable_set2(VAR_DOMAINNAME, string_skipwhite(lines[i] + 6));
-	else if (!strncmp(lines[i], "nameserver", 10) && !name_set) {
+	if (!strncmp(lines[i], "domain", 6) && !variable_get(VAR_DOMAINNAME))
+	    variable_set2(VAR_DOMAINNAME, string_skipwhite(string_prune(lines[i] + 6)));
+	else if (!name_set && !strncmp(lines[i], "nameserver", 10)) {
 	    /* Only take the first nameserver setting - we're lame */
-	    variable_set2(VAR_NAMESERVER, string_skipwhite(lines[i] + 10));
-	    name_set = TRUE;
+	    variable_set2(VAR_NAMESERVER, string_skipwhite(string_prune(lines[i] + 10)));
 	}
 	free(lines[i]);
     }
 }
 
+/* Version of below for dispatch routines */
+int
+configRC(dialogMenuItem *unused)
+{
+    configRC_conf("/etc/rc.conf");
+    return DITEM_SUCCESS;
+}
+
 /*
- * This sucks in /etc/sysconfig, substitutes anything needing substitution, then
+ * This sucks in /etc/rc.conf, substitutes anything needing substitution, then
  * writes it all back out.  It's pretty gross and needs re-writing at some point.
  */
 void
-configSysconfig(char *config)
+configRC_conf(char *config)
 {
     FILE *fp;
     char *lines[MAX_LINES], *cp;
     Variable *v;
-    int i, nlines;
+    int i, nlines, len;
 
     nlines = readConfig(config, lines, MAX_LINES);
     if (nlines == -1)
@@ -373,17 +370,30 @@ configSysconfig(char *config)
 
     /* Now do variable substitutions */
     for (v = VarHead; v; v = v->next) {
-	char line[512];
-
 	for (i = 0; i < nlines; i++) {
 	    /* Skip the comments & non-variable settings */
 	    if (lines[i][0] == '#' || !(cp = index(lines[i], '=')))
 		continue;
-	    sstrncpy(line, lines[i], cp - lines[i]);
-	    if (!strcmp(line, v->name)) {
+	    len = strlen(v->name);
+	    if (!strncmp(lines[i], v->name, cp - lines[i]) && (cp - lines[i]) == len) {
+		char *cp2, *comment = NULL;
+
+		/* If trailing comment, try and preserve it */
+		if ((index(lines[i], '#')) != NULL) {
+		    /* Find quotes */
+		    if ((cp2 = index(cp, '"')) || (cp2 = index(cp, '\047')))
+			cp2 = index(cp2 + 1, *cp2);
+		    if (cp2 && strlen(cp2 + 1)) {
+			comment = alloca(strlen(cp2));
+			strcpy(comment, cp2 + 1);
+		    }
+		}
 		free(lines[i]);
-		lines[i] = (char *)malloc(strlen(v->name) + strlen(v->value) + 5);
-		sprintf(lines[i], "%s=\"%s\"\n", v->name, v->value);
+		lines[i] = (char *)malloc(strlen(v->name) + strlen(v->value) + (comment ? strlen(comment) : 0) + 10);
+		if (comment)
+		    sprintf(lines[i], "%s=\"%s\"%s", v->name, v->value, comment);
+		else
+		    sprintf(lines[i], "%s=\"%s\"\n", v->name, v->value);
 	    }
 	}
     }
@@ -444,7 +454,8 @@ configSaver(dialogMenuItem *self)
 int
 configSaverTimeout(dialogMenuItem *self)
 {
-    return (variable_get_value(VAR_BLANKTIME, "Enter time-out period in seconds for screen saver") ?
+    return (variable_get_value(VAR_BLANKTIME,
+	    "Enter time-out period in seconds for screen saver") ?
 	    DITEM_SUCCESS : DITEM_FAILURE) | DITEM_RESTORE;
 }
 
@@ -459,12 +470,16 @@ configNTP(dialogMenuItem *self)
 {
     int status;
 
-    status = variable_get_value(VAR_NTPDATE, "Enter the name of an NTP server") ? DITEM_SUCCESS : DITEM_FAILURE;
+    status = variable_get_value(VAR_NTPDATE_FLAGS,
+				"Enter the name of an NTP server")
+	     ? DITEM_SUCCESS : DITEM_FAILURE;
     if (status == DITEM_SUCCESS) {
 	static char tmp[255];
 
-	snprintf(tmp, 255, "%s=%s", VAR_NTPDATE, variable_get(VAR_NTPDATE));
-	self->aux = (int)tmp;
+	snprintf(tmp, sizeof(tmp), "ntpdate_enable=YES,ntpdate_flags=%s",
+		 variable_get(VAR_NTPDATE_FLAGS));
+	self->data = tmp;
+	dmenuSetVariables(self);
     }
     return status | DITEM_RESTORE;
 }
@@ -479,83 +494,58 @@ configUsers(dialogMenuItem *self)
 }
 
 int
-configXFree86(dialogMenuItem *self)
+configXEnvironment(dialogMenuItem *self)
 {
-#ifndef USE_XIG_ENVIRONMENT
     char *config, *execfile;
+    char *moused;
 
+tryagain:
     dialog_clear_norefresh();
     if (!dmenuOpenSimple(&MenuXF86Config, FALSE))
 	return DITEM_FAILURE | DITEM_RESTORE;
-#endif
-    systemExecute("/sbin/ldconfig /usr/lib /usr/X11R6/lib /usr/local/lib /usr/lib/compat");
-#ifdef USE_XIG_ENVIRONMENT
-    if (!file_readable("/usr/X11R6/lib/X11/AcceleratedX/bin/Xinstall") ||
-	!file_readable("/usr/X11R6/lib/X11/AcceleratedX/bin/Xsetup")) {
-	dialog_clear_norefresh();
-	msgConfirm("Hmmm!  It looks like you elected not to install the AccelleratedX\n"
-		   "server package (or the installation failed somehow).  If this was\n"
-		   "an omission rather than an error, please go to the Distributions\n"
-		   "menu, select the Custom distribution options and then choose your X11\n"
-		   "distribution components from the XFree86 menu.  AccelX will be selected\n"
-		   "as the default server automatically.");
-	return DITEM_FAILURE | DITEM_RESTORE;
-    }
-    else {
-	int i;
-
-	if (directory_exists("/dist/CDE") && !msgYesNo("Would you like to install the CDE desktop package now?")) {
-	    dialog_clear_norefresh();
-	    msgNotify("Running CDE installation - please wait (this may take awhile!).");
-	    dialog_clear();
-	    i = systemExecute("(cd /dist/CDE; sh Install)");
-	    dialog_clear();
-	    if (i) {
-		msgConfirm("/dist/CDE/dtinstall script returned an error status!\n\n"
-			   "To try again, you should run this command manually after the system\n"
-			   "is up (and if your CDROM is mounted in the standard location, the path\n"
-			   "to it will actually be /cdrom/CDE/dtinstall when you run it later).\n");
-	    }
-	}
-
-	dialog_clear_norefresh();
-	msgNotify("Running AcceleratedX 3.1 installation procedure, please wait.");
-	if ((i = vsystem("/usr/X11R6/lib/X11/AcceleratedX/bin/Xinstall"))) {
-	    msgConfirm("Installation procedure failed, error code %d!  Please report\n"
-		       "error to Walnut Creek CDROM tech support (either send email\n"
-		       "to support@cdrom.com or call +1 510 603 1234).  Thank you!", i);
-	    return DITEM_FAILURE | DITEM_RESTORE;
-	}
-	dialog_clear();
-	systemExecute("/usr/X11R6/lib/X11/AcceleratedX/bin/Xsetup");
-	return DITEM_SUCCESS | DITEM_RESTORE;
-    }
-
-#else	/* !USE_XIG_ENVIRONMENT */
-
+    if (file_readable("/var/run/ld.so.hints"))
+	systemExecute("/sbin/ldconfig -m /usr/lib /usr/X11R6/lib /usr/local/lib /usr/lib/compat");
+    else
+	systemExecute("/sbin/ldconfig /usr/lib /usr/X11R6/lib /usr/local/lib /usr/lib/compat");
     config = variable_get(VAR_XF86_CONFIG);
     if (!config)
 	return DITEM_FAILURE | DITEM_RESTORE;
     execfile = string_concat("/usr/X11R6/bin/", config);
     if (file_executable(execfile)) {
 	dialog_clear_norefresh();
-	if (!file_readable("/dev/mouse") && !msgYesNo("Does this system have a mouse attached to it?"))
+	moused = variable_get(VAR_MOUSED);
+	while (!moused || strcmp(moused, "YES")) {
+	    if (msgYesNo("The X server may access the mouse in two ways: direct access\n"
+			 "or indirect access via the mouse daemon.  You have not\n"
+			 "configured the mouse daemon.  Would you like to configure it\n"
+			 "now?  If you intend to let the X server access the mouse\n"
+			 "directly, choose \"No\" at this time."))
+		break;
 	    dmenuOpenSimple(&MenuMouse, FALSE); 
+	    dialog_clear();
+	    moused = variable_get(VAR_MOUSED);
+	}
+	if (moused && !strcmp(moused, "YES"))
+	    msgConfirm("You have configured and are now running the mouse daemon.\n"
+	   	       "Choose \"/dev/sysmouse\" as the mouse port and \"SysMouse\" or\n"
+		       "\"MouseSystems\" as the mouse protocol in the X configuration\n"
+		       "utility.");
 	dialog_clear();
 	systemExecute(execfile);
+	if (!file_readable("/etc/XF86Config") && !msgYesNo("The XFree86 configuration process seems to have\nfailed.  Would you like to try again?"))
+	    goto tryagain;
 	return DITEM_SUCCESS | DITEM_RESTORE;
     }
     else {
 	dialog_clear_norefresh();
-	msgConfirm("XFree86 does not appear to be installed!  Please install\n"
-		   "The XFree86 distribution before attempting to configure it.");
+	msgConfirm("The XFree86 setup utility you chose does not appear to be installed!\n"
+		   "Please install this before attempting to configure XFree86.");
 	return DITEM_FAILURE | DITEM_RESTORE;
     }
-#endif	/* USE_XIG_ENVIRONMENT */
 }
 
-void
-configResolv(void)
+int
+configResolv(dialogMenuItem *ditem)
 {
     FILE *fp;
     char *cp, *dp, *hp;
@@ -566,7 +556,7 @@ configResolv(void)
     Mkdir("/etc");
     fp = fopen("/etc/resolv.conf", "w");
     if (!fp)
-	return;
+	return DITEM_FAILURE;
     if (variable_get(VAR_DOMAINNAME))
 	fprintf(fp, "domain\t%s\n", variable_get(VAR_DOMAINNAME));
     fprintf(fp, "nameserver\t%s\n", cp);
@@ -581,9 +571,12 @@ skip:
     /* Tack ourselves into /etc/hosts */
     fp = fopen("/etc/hosts", "w");
     if (!fp)
-	return;
+	return DITEM_FAILURE;
     /* Add an entry for localhost */
-    fprintf(fp, "127.0.0.1\t\tlocalhost.%s localhost\n", dp ? dp : "my.domain");
+    if (dp)
+	fprintf(fp, "127.0.0.1\t\tlocalhost.%s localhost\n", dp);
+    else
+	fprintf(fp, "127.0.0.1\t\tlocalhost\n");
     /* Now the host entries, if applicable */
     if (cp && cp[0] != '0' && hp) {
 	char cp2[255];
@@ -600,6 +593,7 @@ skip:
     fclose(fp);
     if (isDebug())
 	msgDebug("Wrote out /etc/hosts\n");
+    return DITEM_SUCCESS;
 }
 
 int
@@ -614,29 +608,39 @@ configRouter(dialogMenuItem *self)
 			     "will attempt to load if you select gated.  Any other\n"
 			     "choice of routing daemon will be assumed to be something\n"
 			     "the user intends to install themselves before rebooting\n"
-			     "the system.  If you don't want any routing daemon, choose NO") ?
-	DITEM_SUCCESS : DITEM_FAILURE;
-
+			     "the system.  If you don't want any routing daemon, choose NO")
+      ? DITEM_SUCCESS : DITEM_FAILURE;
+  
     if (ret == DITEM_SUCCESS) {
-	char *cp;
-
-	cp = variable_get(VAR_ROUTER);
-	if (strcmp(cp, "NO")) {
+	char *cp = variable_get(VAR_ROUTER);
+    
+	if (cp && strcmp(cp, "NO")) {
+	    variable_set2(VAR_ROUTER_ENABLE, "YES");
 	    if (!strcmp(cp, "gated")) {
 		if (package_add(variable_get(VAR_GATED_PKG)) != DITEM_SUCCESS) {
 		    msgConfirm("Unable to load gated package.  Falling back to no router.");
-		    variable_set2(VAR_ROUTER, "NO");
+		    variable_unset(VAR_ROUTER);
+		    variable_unset(VAR_ROUTERFLAGS);
+		    variable_set2(VAR_ROUTER_ENABLE, "NO");
+		    cp = NULL;
 		}
 	    }
-	    /* Now get the flags, if they chose a router */
-	    ret = variable_get_value(VAR_ROUTERFLAGS, 
-				     "Please Specify the routing daemon flags; if you're running routed\n"
-				     "then -q is the right choice for nodes and -s for gateway hosts.\n") ? DITEM_SUCCESS : DITEM_FAILURE;
-	    if (ret != DITEM_SUCCESS) {
-		variable_unset(VAR_ROUTER);
-		variable_unset(VAR_ROUTERFLAGS);
+	    if (cp) {
+		/* Now get the flags, if they chose a router */
+		ret = variable_get_value(VAR_ROUTERFLAGS, 
+					 "Please Specify the routing daemon flags; if you're running routed\n"
+					 "then -q is the right choice for nodes and -s for gateway hosts.\n")
+		  ? DITEM_SUCCESS : DITEM_FAILURE;
+		if (ret != DITEM_SUCCESS)
+		    variable_unset(VAR_ROUTERFLAGS);
 	    }
 	}
+	else {
+	    /* No router case */
+	    variable_set2(VAR_ROUTER_ENABLE, "NO");
+	    variable_unset(VAR_ROUTERFLAGS);
+	    variable_unset(VAR_ROUTER);
+	}
     }
     return ret | DITEM_RESTORE;
 }
@@ -717,28 +721,6 @@ configPackages(dialogMenuItem *self)
     return DITEM_SUCCESS | DITEM_RESTORE;
 }
 
-#ifdef NETCON_EXTENTIONS
-/* Load novell client/server package */
-int
-configNovell(dialogMenuItem *self)
-{
-    int ret = DITEM_SUCCESS;
-
-    if (!RunningAsInit) {
-	msgConfirm("This package can only be installed in multi-user mode.");
-	return ret;
-    }
-    if (variable_get(VAR_NOVELL))
-	variable_unset(VAR_NOVELL);
-    else {
-	ret = package_add(PACKAGE_NETCON);
-	if (DITEM_STATUS(ret) == DITEM_SUCCESS)
-	    variable_set2(VAR_NOVELL, "YES");
-    }
-    return ret | DITEM_RESTORE;
-}
-#endif
-
 /* Load pcnfsd package */
 int
 configPCNFSD(dialogMenuItem *self)
@@ -751,7 +733,7 @@ configPCNFSD(dialogMenuItem *self)
 	ret = package_add(variable_get(VAR_PCNFSD_PKG));
 	if (DITEM_STATUS(ret) == DITEM_SUCCESS) {
 	    variable_set2(VAR_PCNFSD, "YES");
-	    variable_set2("weak_mountd_authentication", "YES");
+	    variable_set2("mountd_flags", "-n");
 	}
     }
     return ret;
@@ -766,28 +748,32 @@ configNFSServer(dialogMenuItem *self)
     if (!file_readable("/etc/exports")) {
 	WINDOW *w = savescr();
 
-	dialog_clear_norefresh();
-	msgConfirm("Operating as an NFS server means that you must first configure\n"
-		   "an /etc/exports file to indicate which hosts are allowed certain\n"
-		   "kinds of access to your local file systems.\n"
-		   "Press [ENTER] now to invoke an editor on /etc/exports\n");
-	vsystem("echo '#The following examples export /usr to 3 machines named after ducks,' > /etc/exports");
-	vsystem("echo '#/home and all directories under it to machines named after dead rock stars' >> /etc/exports");
-	vsystem("echo '#and, finally, /a to 2 privileged machines allowed to write on it as root.' >> /etc/exports");
-	vsystem("echo '#/usr                huey louie dewie' >> /etc/exports");
-	vsystem("echo '#/home   -alldirs    janice jimmy frank' >> /etc/exports");
-	vsystem("echo '#/a      -maproot=0  bill albert' >> /etc/exports");
-	vsystem("echo '#' >> /etc/exports");
-	vsystem("echo '# You should replace these lines with your actual exported filesystems.' >> /etc/exports");
-	vsystem("echo >> /etc/exports");
-	sprintf(cmd, "%s /etc/exports", variable_get(VAR_EDITOR));
-	dialog_clear();
-	systemExecute(cmd);
-	restorescr(w);
+	if (file_readable("/etc/exports.disabled"))
+	    vsystem("mv /etc/exports.disabled /etc/exports");
+	else {
+	    dialog_clear_norefresh();
+	    msgConfirm("Operating as an NFS server means that you must first configure\n"
+		       "an /etc/exports file to indicate which hosts are allowed certain\n"
+		       "kinds of access to your local file systems.\n"
+		       "Press [ENTER] now to invoke an editor on /etc/exports\n");
+	    vsystem("echo '#The following examples export /usr to 3 machines named after ducks,' > /etc/exports");
+	    vsystem("echo '#/home and all directories under it to machines named after dead rock stars' >> /etc/exports");
+	    vsystem("echo '#and, finally, /a to 2 privileged machines allowed to write on it as root.' >> /etc/exports");
+	    vsystem("echo '#/usr                huey louie dewie' >> /etc/exports");
+	    vsystem("echo '#/home   -alldirs    janice jimmy frank' >> /etc/exports");
+	    vsystem("echo '#/a      -maproot=0  bill albert' >> /etc/exports");
+	    vsystem("echo '#' >> /etc/exports");
+	    vsystem("echo '# You should replace these lines with your actual exported filesystems.' >> /etc/exports");
+	    vsystem("echo >> /etc/exports");
+	    sprintf(cmd, "%s /etc/exports", variable_get(VAR_EDITOR));
+	    dialog_clear();
+	    systemExecute(cmd);
+	    restorescr(w);
+	}
 	variable_set2(VAR_NFS_SERVER, "YES");
     }
     else if (variable_get(VAR_NFS_SERVER)) { /* We want to turn it off again? */
-	unlink("/etc/exports");
+	vsystem("mv -f /etc/exports /etc/exports.disabled");
 	variable_unset(VAR_NFS_SERVER);
     }
     return DITEM_SUCCESS;
