@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/disks.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/disks.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -45,6 +45,8 @@
 static struct chunk *chunk_info[16];
 static int current_chunk;
 
+static void	diskPartitionNonInteractive(Device *dev);
+
 static void
 record_chunks(Disk *d)
 {
@@ -85,8 +87,8 @@ print_chunks(Disk *d)
 		   "the Hardware Guide in the Documentation submenu or use the\n"
 		   "(G)eometry command to change it now.\n\n"
 		   "Remember: you need to enter whatever your BIOS thinks the\n"
-		   "geometry were!  For IDE, that's what you have told in the BIOS\n"
-		   "setup, for SCSI, it's the translation mode your controller is\n"
+		   "geometry is!  For IDE, it's what you were told in the BIOS\n"
+		   "setup. For SCSI, it's the translation mode your controller is\n"
 		   "using.  Do NOT use a ``physical geometry''.",
 	  d->bios_cyl, d->bios_hd, d->bios_sect, d->name);
 	Sanitize_Bios_Geom(d);
@@ -100,13 +102,13 @@ print_chunks(Disk *d)
 	     "DISK Geometry:\t%lu cyls/%lu heads/%lu sectors = %lu sectors",
 	     d->bios_cyl, d->bios_hd, d->bios_sect,
 	     d->bios_cyl * d->bios_hd * d->bios_sect);
-    mvprintw(3, 1, "%10s %10s %10s %8s %8s %8s %8s %8s",
+    mvprintw(3, 0, "%10s %10s %10s %8s %6s %10s %8s %8s",
 	     "Offset", "Size", "End", "Name", "PType", "Desc",
 	     "Subtype", "Flags");
     for (i = 0, row = CHUNK_START_ROW; chunk_info[i]; i++, row++) {
 	if (i == current_chunk)
 	    attrset(ATTR_SELECTED);
-	mvprintw(row, 2, "%10ld %10lu %10lu %8s %8d %8s %8d\t%-6s",
+	mvprintw(row, 0, "%10ld %10lu %10lu %8s %6d %10s %8d\t%-6s",
 		 chunk_info[i]->offset, chunk_info[i]->size,
 		 chunk_info[i]->end, chunk_info[i]->name,
 		 chunk_info[i]->type, 
@@ -123,7 +125,7 @@ print_command_summary()
     mvprintw(14, 0, "The following commands are supported (in upper or lower case):");
     mvprintw(16, 0, "A = Use Entire Disk    B = Bad Block Scan       C = Create Slice");
     mvprintw(17, 0, "D = Delete Slice       G = Set Drive Geometry   S = Set Bootable");
-    mvprintw(18, 0, "U = Undo All Changes   Q = Finish");
+    mvprintw(18, 0, "T = Change Type        U = Undo All Changes     Q = Finish");
     if (!RunningAsInit)
 	mvprintw(18, 48, "W = Write Changes");
     mvprintw(21, 0, "Use F1 or ? to get more help, arrow keys to select.");
@@ -169,8 +171,27 @@ getBootMgr(char *dname)
     return NULL;
 }
 
+int
+diskGetSelectCount(Device ***devs)
+{
+    int i, cnt, enabled;
+    char *cp;
+    Device **dp;
+
+    cp = variable_get(VAR_DISK);
+    dp = *devs = deviceFind(cp, DEVICE_TYPE_DISK);
+    cnt = deviceCount(dp);
+    if (!cnt)
+	return -1;
+    for (i = 0, enabled = 0; i < cnt; i++) {
+	if (dp[i]->enabled)
+	    ++enabled;
+    }
+    return enabled;
+}
+
 void
-diskPartition(Device *dev, Disk *d)
+diskPartition(Device *dev)
 {
     char *cp, *p;
     int rv, key = 0;
@@ -178,6 +199,7 @@ diskPartition(Device *dev, Disk *d)
     char *msg = NULL;
     u_char *mbrContents;
     WINDOW *w = savescr();
+    Disk *d = (Disk *)dev->private;
 
     chunking = TRUE;
     keypad(stdscr, TRUE);
@@ -247,27 +269,18 @@ diskPartition(Device *dev, Disk *d)
 	    break;
 
 	case 'A':
-	    rv = msgYesNo("Do you want to do this with a true partition entry\n"
-			  "so as to remain cooperative with any future possible\n"
-			  "operating systems on the drive(s)?");
-	    if (rv != 0) {
-		rv = !msgYesNo("This is dangerous in that it will make the drive totally\n"
-			       "uncooperative with other potential operating systems on the\n"
-			       "same disk.  It will lead instead to a totally dedicated disk,\n"
-			       "starting at the very first sector, bypassing all BIOS geometry\n"
-			       "considerations.  This precludes the existance of any boot\n"
-			       "manager or other stuff in sector 0, since the BSD bootstrap\n"
-			       "will live there.\n"
-			       "You will run into serious trouble with ST-506 and ESDI drives\n"
-			       "and possibly some IDE drives (e.g. drives running under the\n"
-			       "control of sort of disk manager).  SCSI drives are considerably\n"
-			       "less at risk.\n\n"
-			       "If, on the other hand, your goal is a dedicated FreeBSD machine\n"
-			       "and nothing else, this option is for you.\n\n"
-			       "Do you insist on dedicating the entire disk this way?");
+	    cp = variable_get(VAR_DEDICATE_DISK);
+	    if (cp && !strcasecmp(cp, "always"))
+		rv = 1;
+	    else {
+		rv = msgYesNo("Do you want to do this with a true partition entry\n"
+			      "so as to remain cooperative with any future possible\n"
+			      "operating systems on the drive(s)?\n"
+			      "(See also the section about ``dangerously dedicated''\n"
+			      "disks in the FreeBSD FAQ.)");
+		if (rv == -1)
+		    rv = 0;
 	    }
-	    if (rv == -1)
-		rv = 0;
 	    All_FreeBSD(d, rv);
 	    variable_set2(DISK_PARTITIONED, "yes");
 	    record_chunks(d);
@@ -296,7 +309,7 @@ diskPartition(Device *dev, Disk *d)
 		int size, subtype;
 		chunk_e partitiontype;
 		
-		snprintf(tmp, 20, "%d", chunk_info[current_chunk]->size);
+		snprintf(tmp, 20, "%lu", chunk_info[current_chunk]->size);
 		val = msgGetInput(tmp, "Please specify the size for new FreeBSD slice in blocks\n"
 				  "or append a trailing `M' for megabytes (e.g. 20M).");
 		if (val && (size = strtol(val, &cp, 0)) > 0) {
@@ -312,12 +325,12 @@ diskPartition(Device *dev, Disk *d)
 				      "for you to use another tool, such as DOS FORMAT, to later format\n"
 				      "and use the partition.");
 		    if (val && (subtype = strtol(val, NULL, 0)) > 0) {
-			if (subtype==165)
-			    partitiontype=freebsd;
-			else if (subtype==6)
-			    partitiontype=fat;
+			if (subtype == 165)
+			    partitiontype = freebsd;
+			else if (subtype == 6)
+			    partitiontype = fat;
 			else
-			    partitiontype=unknown;
+			    partitiontype = unknown;
 			Create_Chunk(d, chunk_info[current_chunk]->offset, size, partitiontype, subtype,
 				     (chunk_info[current_chunk]->flags & CHUNK_ALIGN));
 			variable_set2(DISK_PARTITIONED, "yes");
@@ -339,6 +352,39 @@ diskPartition(Device *dev, Disk *d)
 	    }
 	    break;
 	    
+	case 'T':
+	    if (chunk_info[current_chunk]->type == unused)
+		msg = "Slice is currently unused (use create instead)";
+	    else {
+		char *val, tmp[20];
+		int subtype;
+		chunk_e partitiontype;
+		WINDOW *save = savescr();
+
+		strcpy(tmp, "165");
+		val = msgGetInput(tmp, "New partition type:\n\n"
+				  "Pressing Enter will choose the default, a native FreeBSD\n"
+				  "slice (type 165).  Other popular values are 6 for\n"
+				  "DOS FAT partition, 131 for a Linux ext2fs partition or\n"
+				  "130 for a Linux swap partition.\n\n"
+				  "Note:  If you choose a non-FreeBSD partition type, it will not\n"
+				  "be formatted or otherwise prepared, it will simply reserve space\n"
+				  "for you to use another tool, such as DOS format, to later format\n"
+				  "and actually use the partition.");
+		if (val && (subtype = strtol(val, NULL, 0)) > 0) {
+		    if (subtype == 165)
+			partitiontype = freebsd;
+		    else if (subtype == 6)
+			partitiontype = fat;
+		    else
+			partitiontype = unknown;
+		    chunk_info[current_chunk]->type = partitiontype;
+		    chunk_info[current_chunk]->subtype = subtype;
+		}
+		restorescr(save);
+	    }
+	    break;
+	    
 	case 'G':
 	    snprintf(geometry, 80, "%lu/%lu/%lu", d->bios_cyl, d->bios_hd, d->bios_sect);
 	    val = msgGetInput(geometry, "Please specify the new geometry in cyl/hd/sect format.\n"
@@ -382,15 +428,12 @@ diskPartition(Device *dev, Disk *d)
 	    break;
 
 	case 'W':
-	    if ((cp = variable_get(DISK_LABELLED)) && !strcmp(cp, "written")) {
-		msgConfirm("You've already written this information out - if\n"
-			   "you wish to overwrite it, you'll have to restart.");
-	    }
-	    else if (!msgYesNo("WARNING:  This should only be used when modifying an EXISTING\n"
+	    if (!msgYesNo("WARNING:  This should only be used when modifying an EXISTING\n"
 			       "installation.  If you are installing FreeBSD for the first time\n"
 			       "then you should simply type Q when you're finished here and your\n"
 			       "changes will be committed in one batch automatically at the end of\n"
-			       "these questions.\n\n"
+			       "these questions.  If you're adding a disk, you should NOT write\n"
+			       "from this screen, you should do it from the label editor.\n\n"
 			       "Are you absolutely sure you want to do this now?")) {
 		variable_set2(DISK_PARTITIONED, "yes");
 		
@@ -398,8 +441,19 @@ diskPartition(Device *dev, Disk *d)
 		 * disk (i.e., the disklabel starts at sector 0), even in cases where the user has requested
 		 * booteasy or a "standard" MBR -- both would be fatal in this case.
 		 */
-		if (!(d->chunks->part->flags & CHUNK_FORCE_ALL) && (mbrContents = getBootMgr(d->name)) != NULL)
+#if 0
+		if ((d->chunks->part->flags & CHUNK_FORCE_ALL) != CHUNK_FORCE_ALL
+		    && (mbrContents = getBootMgr(d->name)) != NULL)
 		    Set_Boot_Mgr(d, mbrContents);
+#else
+		/*
+		 * Don't offer to update the MBR on this disk if the first "real" chunk looks like
+		 * a FreeBSD "all disk" partition, or the disk is entirely FreeBSD.
+		 */
+		if (((d->chunks->part->type != freebsd) || (d->chunks->part->offset > 1)) &&
+		    (mbrContents = getBootMgr(d->name)) != NULL)
+		    Set_Boot_Mgr(d, mbrContents);
+#endif
 		
 		if (DITEM_STATUS(diskPartitionWrite(NULL)) != DITEM_SUCCESS)
 		    msgConfirm("Disk partition write returned an error status!");
@@ -430,9 +484,19 @@ diskPartition(Device *dev, Disk *d)
 	     * disk (i.e., the disklabel starts at sector 0), even in cases where the user has requested
 	     * booteasy or a "standard" MBR -- both would be fatal in this case.
 	     */
+#if 0
 	    if ((d->chunks->part->flags & CHUNK_FORCE_ALL) != CHUNK_FORCE_ALL
 		&& (mbrContents = getBootMgr(d->name)) != NULL)
 		Set_Boot_Mgr(d, mbrContents);
+#else
+	    /*
+	     * Don't offer to update the MBR on this disk if the first "real" chunk looks like
+	     * a FreeBSD "all disk" partition, or the disk is entirely FreeBSD. 
+	     */
+	    if (((d->chunks->part->type != freebsd) || (d->chunks->part->offset > 1)) &&
+		(mbrContents = getBootMgr(d->name)) != NULL)
+		Set_Boot_Mgr(d, mbrContents);
+#endif
 	    break;
 	    
 	default:
@@ -448,7 +512,8 @@ diskPartition(Device *dev, Disk *d)
 	dialog_clear_norefresh();
         use_helpline("Press F1 to read more about disk slices.");
 	use_helpfile(systemHelpFile("partition", buf));
-	dialog_mesgbox("Disk slicing warning:", p, -1, -1);
+	if (!variable_get(VAR_NO_WARN))
+	    dialog_mesgbox("Disk slicing warning:", p, -1, -1);
 	free(p);
     }
     restorescr(w);
@@ -467,11 +532,11 @@ partitionHook(dialogMenuItem *selected)
     /* Toggle enabled status? */
     if (!devs[0]->enabled) {
 	devs[0]->enabled = TRUE;
-	diskPartition(devs[0], (Disk *)devs[0]->private);
+	diskPartition(devs[0]);
     }
     else
 	devs[0]->enabled = FALSE;
-    return DITEM_SUCCESS | DITEM_REDRAW;
+    return DITEM_SUCCESS | DITEM_RESTORE;
 }
 
 static int
@@ -490,55 +555,63 @@ diskPartitionEditor(dialogMenuItem *self)
 {
     DMenu *menu;
     Device **devs;
-    int i, cnt;
-    char *cp;
+    int i, cnt, devcnt;
 
-    cp = variable_get(VAR_DISK);
-    devs = deviceFind(cp, DEVICE_TYPE_DISK);
-    cnt = deviceCount(devs);
-    if (!cnt) {
+    cnt = diskGetSelectCount(&devs);
+    devcnt = deviceCount(devs);
+    if (cnt == -1) {
 	msgConfirm("No disks found!  Please verify that your disk controller is being\n"
 		   "properly probed at boot time.  See the Hardware Guide on the\n"
 		   "Documentation menu for clues on diagnosing this type of problem.");
-	i = DITEM_FAILURE;
+	return DITEM_FAILURE;
     }
-    else if (cnt == 1) {
-	devs[0]->enabled = TRUE;
-	diskPartition(devs[0], (Disk *)devs[0]->private);
-	i = DITEM_SUCCESS;
+    else if (cnt) {
+	/* Some are already selected */
+	for (i = 0; i < devcnt; i++) {
+	    if (devs[i]->enabled) {
+		if (variable_get(VAR_NONINTERACTIVE))
+		    diskPartitionNonInteractive(devs[i]);
+		else
+		    diskPartition(devs[i]);
+	    }
+	}
     }
     else {
-	menu = deviceCreateMenu(&MenuDiskDevices, DEVICE_TYPE_DISK, partitionHook, partitionCheck);
-	if (!menu) {
-	    msgConfirm("No devices suitable for installation found!\n\n"
-		       "Please verify that your disk controller (and attached drives)\n"
-		       "were detected properly.  This can be done by pressing the\n"
-		       "[Scroll Lock] key and using the Arrow keys to move back to\n"
-		       "the boot messages.  Press [Scroll Lock] again to return.");
-	    i = DITEM_FAILURE;
+	/* No disks are selected, fall-back case now */
+	if (devcnt == 1) {
+	    devs[0]->enabled = TRUE;
+	    if (variable_get(VAR_NONINTERACTIVE))
+		diskPartitionNonInteractive(devs[0]);
+	    else
+		diskPartition(devs[0]);
+	    return DITEM_SUCCESS;
 	}
 	else {
-	    i = dmenuOpenSimple(menu, FALSE) ? DITEM_SUCCESS : DITEM_FAILURE;
-	    free(menu);
+	    menu = deviceCreateMenu(&MenuDiskDevices, DEVICE_TYPE_DISK, partitionHook, partitionCheck);
+	    if (!menu) {
+		msgConfirm("No devices suitable for installation found!\n\n"
+			   "Please verify that your disk controller (and attached drives)\n"
+			   "were detected properly.  This can be done by pressing the\n"
+			   "[Scroll Lock] key and using the Arrow keys to move back to\n"
+			   "the boot messages.  Press [Scroll Lock] again to return.");
+		return DITEM_FAILURE;
+	    }
+	    else {
+		i = dmenuOpenSimple(menu, FALSE) ? DITEM_SUCCESS : DITEM_FAILURE;
+		free(menu);
+	    }
+	    return i | DITEM_RESTORE;
 	}
-	i = i | DITEM_RESTORE;
     }
-    return i;
+    return DITEM_SUCCESS;
 }
 
 int
 diskPartitionWrite(dialogMenuItem *self)
 {
     Device **devs;
-    char *cp;
     int i;
-
-    if ((cp = variable_get(DISK_PARTITIONED)) && strcmp(cp, "yes"))
-	return DITEM_SUCCESS;
-    else if (!cp) {
-	msgConfirm("You must partition the disk(s) before this option can be used.");
-	return DITEM_FAILURE;
-    }
+    char *cp;
 
     devs = deviceFind(NULL, DEVICE_TYPE_DISK);
     if (!devs) {
@@ -547,6 +620,9 @@ diskPartitionWrite(dialogMenuItem *self)
     }
     if (isDebug())
 	msgDebug("diskPartitionWrite: Examining %d devices\n", deviceCount(devs));
+    cp = variable_get(DISK_PARTITIONED);
+    if (cp && !strcmp(cp, "written"))
+	return DITEM_SUCCESS;
 
     for (i = 0; devs[i]; i++) {
 	Chunk *c1;
@@ -561,6 +637,11 @@ diskPartitionWrite(dialogMenuItem *self)
 	    msgConfirm("ERROR: Unable to write data to disk %s!", d->name);
 	    return DITEM_FAILURE;
 	}
+
+	/* If we've been through here before, we don't need to do the rest */
+	if (cp && !strcmp(cp, "written"))
+	    return DITEM_SUCCESS;
+
 	/* Now scan for bad blocks, if necessary */
 	for (c1 = d->chunks->part; c1; c1 = c1->next) {
 	    if (c1->flags & CHUNK_BAD144) {
@@ -582,3 +663,95 @@ diskPartitionWrite(dialogMenuItem *self)
     variable_set2(DISK_PARTITIONED, "written");
     return DITEM_SUCCESS;
 }
+
+/* Partition a disk based wholly on which variables are set */
+static void
+diskPartitionNonInteractive(Device *dev)
+{
+    char *cp;
+    int i, sz, all_disk = 0;
+    u_char *mbrContents;
+    Disk *d = (Disk *)dev->private;
+
+    record_chunks(d);
+    cp = variable_get(VAR_GEOMETRY);
+    if (cp) {
+	msgDebug("Setting geometry from script to: %s\n", cp);
+	d->bios_cyl = strtol(cp, &cp, 0);
+	d->bios_hd = strtol(cp + 1, &cp, 0);
+	d->bios_sect = strtol(cp + 1, 0, 0);
+    }
+
+    cp = variable_get(VAR_PARTITION);
+    if (cp) {
+	if (!strcmp(cp, "free")) {
+	    /* Do free disk space case */
+	    for (i = 0; chunk_info[i]; i++) {
+		/* If a chunk is at least 10MB in size, use it. */
+		if (chunk_info[i]->type == unused && chunk_info[i]->size > (10 * ONE_MEG)) {
+		    Create_Chunk(d, chunk_info[i]->offset, chunk_info[i]->size, freebsd, 3,
+				 (chunk_info[i]->flags & CHUNK_ALIGN));
+		    variable_set2(DISK_PARTITIONED, "yes");
+		    break;
+		}
+	    }
+	    if (!chunk_info[i]) {
+		dialog_clear();
+		msgConfirm("Unable to find any free space on this disk!");
+		return;
+	    }
+	}
+	else if (!strcmp(cp, "all")) {
+	    /* Do all disk space case */
+	    msgDebug("Warning:  Devoting all of disk %s to FreeBSD.\n", d->name);
+
+	    All_FreeBSD(d, FALSE);
+	}
+	else if (!strcmp(cp, "exclusive")) {
+	    /* Do really-all-the-disk-space case */
+	    msgDebug("Warning:  Devoting all of disk %s to FreeBSD.\n", d->name);
+
+	    All_FreeBSD(d, all_disk = TRUE);
+	}
+	else if ((sz = strtol(cp, &cp, 0))) {
+	    /* Look for sz bytes free */
+	    if (*cp && toupper(*cp) == 'M')
+		sz *= ONE_MEG;
+	    for (i = 0; chunk_info[i]; i++) {
+		/* If a chunk is at least sz MB, use it. */
+		if (chunk_info[i]->type == unused && chunk_info[i]->size >= sz) {
+		    Create_Chunk(d, chunk_info[i]->offset, sz, freebsd, 3, (chunk_info[i]->flags & CHUNK_ALIGN));
+		    variable_set2(DISK_PARTITIONED, "yes");
+		    break;
+		}
+	    }
+	    if (!chunk_info[i]) {
+		dialog_clear();
+		msgConfirm("Unable to find %d free blocks on this disk!", sz);
+		return;
+	    }
+	}
+	else if (!strcmp(cp, "existing")) {
+	    /* Do existing FreeBSD case */
+	    for (i = 0; chunk_info[i]; i++) {
+		if (chunk_info[i]->type == freebsd)
+		    break;
+	    }
+	    if (!chunk_info[i]) {
+		dialog_clear();
+		msgConfirm("Unable to find any existing FreeBSD partitions on this disk!");
+		return;
+	    }
+	}
+	else {
+	    dialog_clear();
+	    msgConfirm("`%s' is an invalid value for %s - is config file valid?", cp, VAR_PARTITION);
+	    return;
+	}
+	if (!all_disk) {
+	    mbrContents = getBootMgr(d->name);
+	    Set_Boot_Mgr(d, mbrContents);
+	}
+	variable_set2(DISK_PARTITIONED, "yes");
+    }
+}
