@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/dist.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/dist.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -41,11 +41,9 @@
 unsigned int Dists;
 unsigned int DESDists;
 unsigned int SrcDists;
-#ifndef USE_XIG_ENVIRONMENT
 unsigned int XF86Dists;
 unsigned int XF86ServerDists;
 unsigned int XF86FontDists;
-#endif
 
 typedef struct _dist {
     char *my_name;
@@ -58,11 +56,9 @@ typedef struct _dist {
 extern Distribution DistTable[];
 extern Distribution DESDistTable[];
 extern Distribution SrcDistTable[];
-#ifndef USE_XIG_ENVIRONMENT
 extern Distribution XF86DistTable[];
 extern Distribution XF86FontDistTable[];
 extern Distribution XF86ServerDistTable[];
-#endif
 
 /* The top-level distribution categories */
 static Distribution DistTable[] = {
@@ -80,11 +76,7 @@ static Distribution DistTable[] = {
 { "compat20",	"/",			&Dists,		DIST_COMPAT20,		NULL		},
 { "compat21",	"/",			&Dists,		DIST_COMPAT21,		NULL		},
 { "ports",	"/usr",			&Dists,		DIST_PORTS,		NULL		},
-#ifdef USE_XIG_ENVIRONMENT
-{ "accelx",	"/usr/X11R6/lib/X11",	&Dists,		DIST_XIG_SERVER,	NULL		},
-#else
-{ "XF8632",	"/usr",			&Dists,		DIST_XF86,		XF86DistTable	},
-#endif
+{ "XF86333",	"/usr",			&Dists,		DIST_XF86,		XF86DistTable	},
 { NULL },
 };
 
@@ -93,7 +85,8 @@ static Distribution DESDistTable[] = {
 { "des",        "/",                    &DESDists,	DIST_DES_DES,		NULL		},
 { "krb",	"/",			&DESDists,	DIST_DES_KERBEROS,	NULL		},
 { "ssecure",	"/usr/src",		&DESDists,	DIST_DES_SSECURE,	NULL		},
-{ "sebones",	"/usr/src",		&DESDists,	DIST_DES_SEBONES,	NULL		},
+{ "scrypto",	"/usr/src",		&DESDists,	DIST_DES_SCRYPTO,	NULL		},
+{ "skerbero",	"/usr/src",		&DESDists,	DIST_DES_SKERBEROS,	NULL		},
 { NULL },
 };
 
@@ -119,143 +112,204 @@ static Distribution SrcDistTable[] = {
 { NULL },
 };
 
-#ifndef USE_XIG_ENVIRONMENT
 /* The XFree86 distribution */
 static Distribution XF86DistTable[] = {
-{ "XF8632",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_FONTS,	XF86FontDistTable },
-{ "XF8632",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_SERVER,	XF86ServerDistTable },
-{ "XF86-xc",	"/usr/X11R6/src",	&XF86Dists,	DIST_XF86_SRC,		NULL		},
-{ "XF86-co",	"/usr/X11R6/src",	&XF86Dists,	DIST_XF86_CSRC,		NULL		},
-{ "X32bin",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_BIN,		NULL		},
-{ "X32cfg",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_CFG,		NULL		},
-{ "X32doc",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_DOC,		NULL		},
-{ "X32html",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_HTML,		NULL		},
-{ "X32lib",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LIB,		NULL		},
-{ "X32lk98",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LKIT98,	NULL		},
-{ "X32lkit",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LKIT,		NULL		},
-{ "X32man",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_MAN,		NULL		},
-{ "X32prog",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_PROG,		NULL		},
-{ "X32ps",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_PS,		NULL		},
-{ "X32set",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_SET,		NULL		},
+{ "XF86333",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_FONTS,	XF86FontDistTable },
+{ "XF86333",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_SERVER,	XF86ServerDistTable },
+{ "Xsrc1",	"/usr/X11R6/src",	&XF86Dists,	DIST_XF86_SRC,		NULL		},
+{ "Xsrcctrb",	"/usr/X11R6/src",	&XF86Dists,	DIST_XF86_CSRC,		NULL		},
+{ "Xbin",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_BIN,		NULL		},
+{ "Xcfg",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_CFG,		NULL		},
+{ "Xdoc",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_DOC,		NULL		},
+{ "Xhtml",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_HTML,		NULL		},
+{ "Xlib",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LIB,		NULL		},
+{ "Xlk98",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LKIT98,	NULL		},
+{ "Xlkit",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_LKIT,		NULL		},
+{ "Xman",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_MAN,		NULL		},
+{ "Xprog",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_PROG,		NULL		},
+{ "Xps",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_PS,		NULL		},
+{ "Xset",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_SET,		NULL		},
+{ "X9set",	"/usr/X11R6",		&XF86Dists,	DIST_XF86_9SET,		NULL		},
 { NULL },
 };
 
 /* The XFree86 server distribution */
 static Distribution XF86ServerDistTable[] = {
-{ "X328514",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_8514,	NULL		},
-{ "X329480",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9480,	NULL		},
-{ "X329EGC",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9EGC,	NULL		},
-{ "X329GA9",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9GA9,	NULL		},
-{ "X329GAN",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9GAN,	NULL		},
-{ "X329LPW",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9LPW,	NULL		},
-{ "X329NKV",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9NKV,	NULL		},
-{ "X329NS3",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9NS3,	NULL		},
-{ "X329SPW",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9SPW,	NULL		},
-{ "X329TGU",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9TGU,	NULL		},
-{ "X329WEP",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WEP,	NULL		},
-{ "X329WS",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WS,	NULL		},
-{ "X329WSN",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WSN,	NULL		},
-{ "X32AGX",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_AGX,	NULL		},
-{ "X32I128",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_I128,	NULL		},
-{ "X32Ma8",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH8,	NULL		},
-{ "X32Ma32",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH32,NULL		},
-{ "X32Ma64",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH64,NULL		},
-{ "X32Mono",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MONO,	NULL		},
-{ "X32P9K",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_P9000,	NULL		},
-{ "X32S3",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_S3,	NULL		},
-{ "X32S3V",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_S3V,	NULL		},
-{ "X32SVGA",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_SVGA,	NULL		},
-{ "X32VG16",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_VGA16,	NULL		},
-{ "X32W32",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_W32,	NULL		},
-{ "X32nest",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_NEST,	NULL		},
-{ "X32vfb",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_VFB,	NULL		},
+{ "PC98-Servers/X9480",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9480,	NULL		},
+{ "PC98-Servers/X9EGC",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9EGC,	NULL		},
+{ "PC98-Servers/X9GA9",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9GA9,	NULL		},
+{ "PC98-Servers/X9GAN",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9GAN,	NULL		},
+{ "PC98-Servers/X9LPW",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9LPW,	NULL		},
+{ "PC98-Servers/X9MGA",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9MGA,	NULL		},
+{ "PC98-Servers/X9NKV",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9NKV,	NULL		},
+{ "PC98-Servers/X9NS3",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9NS3,	NULL		},
+{ "PC98-Servers/X9SPW",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9SPW,	NULL		},
+{ "PC98-Servers/X9SVG",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9SVG,	NULL		},
+{ "PC98-Servers/X9TGU",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9TGU,	NULL		},
+{ "PC98-Servers/X9WEP",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WEP,	NULL		},
+{ "PC98-Servers/X9WS",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WS,	NULL		},
+{ "PC98-Servers/X9WSN",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_9WSN,	NULL		},
+{ "Servers/X8514",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_8514,	NULL		},
+{ "Servers/XAGX",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_AGX,	NULL		},
+{ "Servers/XI128",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_I128,	NULL		},
+{ "Servers/XMa8",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH8,	NULL		},
+{ "Servers/XMa32",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH32,NULL		},
+{ "Servers/XMa64",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MACH64,NULL		},
+{ "Servers/XMono",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_MONO,	NULL		},
+{ "Servers/XP9K",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_P9000,	NULL		},
+{ "Servers/XS3",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_S3,	NULL		},
+{ "Servers/XS3V",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_S3V,	NULL		},
+{ "Servers/XSVGA",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_SVGA,	NULL		},
+{ "Servers/XVG16",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_VGA16,	NULL		},
+{ "Servers/XW32",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_W32,	NULL		},
+{ "Xnest",	"/usr/X11R6",		&XF86ServerDists,	DIST_XF86_SERVER_NEST,	NULL		},
 { NULL },
 };
 
 /* The XFree86 font distribution */
 static Distribution XF86FontDistTable[] = {
-{ "X32fnts",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_MISC,	NULL		},
-{ "X32f100",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_100,	NULL		},
-{ "X32fcyr",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_CYR,	NULL		},
-{ "X32fscl",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_SCALE,	NULL		},
-{ "X32fnon",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_NON,	NULL		},
-{ "X32fsrv",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_SERVER,	NULL		},
+{ "Xfnts",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_MISC,	NULL		},
+{ "Xf100",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_100,	NULL		},
+{ "Xfcyr",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_CYR,	NULL		},
+{ "Xfscl",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_SCALE,	NULL		},
+{ "Xfnon",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_NON,	NULL		},
+{ "Xfsrv",	"/usr/X11R6",		&XF86FontDists,		DIST_XF86_FONTS_SERVER,	NULL		},
 { NULL },
 };
-#endif	/* !USE_XIG_ENVIRONMENT */
 
 static int	distMaybeSetDES(dialogMenuItem *self);
 static int	distMaybeSetPorts(dialogMenuItem *self);
 
+
+static void
+distVerifyFlags(void)
+{
+    if (SrcDists)
+	Dists |= DIST_SRC;
+    if (DESDists) {
+	if (DESDists & DIST_DES_KERBEROS)
+	    DESDists |= DIST_DES_DES;
+	Dists |= DIST_DES;
+    }
+    if (XF86Dists & DIST_XF86_SET)
+	XF86ServerDists |= DIST_XF86_SERVER_VGA16;
+    if (XF86ServerDists)
+	XF86Dists |= DIST_XF86_SERVER;
+    if (XF86FontDists)
+	XF86Dists |= DIST_XF86_FONTS;
+    if (XF86Dists || XF86ServerDists || XF86FontDists)
+	Dists |= DIST_XF86;
+    if (isDebug())
+	msgDebug("Dist Masks: Dists: %0x, DES: %0x, Srcs: %0x\nXServer: %0x, XFonts: %0x, XDists: %0x\n",
+		 Dists, DESDists, SrcDists, XF86ServerDists, XF86FontDists, XF86Dists);
+}
+
 int
 distReset(dialogMenuItem *self)
 {
     Dists = 0;
     DESDists = 0;
     SrcDists = 0;
-#ifndef USE_XIG_ENVIRONMENT
     XF86Dists = 0;
     XF86ServerDists = 0;
     XF86FontDists = 0;
-#endif
+    return DITEM_SUCCESS | DITEM_REDRAW;
+}
+
+int
+distConfig(dialogMenuItem *self)
+{
+    char *cp;
+
+    distReset(NULL);
+
+    if ((cp = variable_get(VAR_DIST_MAIN)) != NULL)
+	Dists = atoi(cp);
+
+    if ((cp = variable_get(VAR_DIST_DES)) != NULL)
+	DESDists = atoi(cp);
+
+    if ((cp = variable_get(VAR_DIST_SRC)) != NULL)
+	SrcDists = atoi(cp);
+
+    if ((cp = variable_get(VAR_DIST_X11)) != NULL)
+	XF86Dists = atoi(cp);
+
+    if ((cp = variable_get(VAR_DIST_XSERVER)) != NULL)
+	XF86ServerDists = atoi(cp);
+
+    if ((cp = variable_get(VAR_DIST_XFONTS)) != NULL)
+	XF86FontDists = atoi(cp);
+    distVerifyFlags();
     return DITEM_SUCCESS | DITEM_REDRAW;
 }
 
 int
 distSetDeveloper(dialogMenuItem *self)
 {
+    int i;
+
     distReset(NULL);
     Dists = _DIST_DEVELOPER;
     SrcDists = DIST_SRC_ALL;
-    return distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i = distMaybeSetDES(self) | distMaybeSetPorts(self);
+    distVerifyFlags();
+    return i;
 }
 
 int
 distSetXDeveloper(dialogMenuItem *self)
 {
-    distReset(NULL);
-    Dists = _DIST_DEVELOPER;
-    SrcDists = DIST_SRC_ALL;
-#ifdef USE_XIG_ENVIRONMENT
-    Dists |= (DIST_XIG_SERVER | DIST_COMPAT21);
-#else
-    XF86Dists = DIST_XF86_BIN | DIST_COMPAT21 | DIST_XF86_SET | DIST_XF86_CFG | DIST_XF86_LIB | DIST_XF86_PROG | DIST_XF86_MAN | DIST_XF86_SERVER | DIST_XF86_FONTS;
+    int i;
+
+    i = distSetDeveloper(self);
+    Dists |= DIST_XF86;
+    XF86Dists = DIST_XF86_BIN | DIST_XF86_SET | DIST_XF86_CFG | DIST_XF86_LIB | DIST_XF86_PROG | DIST_XF86_MAN | DIST_XF86_SERVER | DIST_XF86_FONTS;
     XF86ServerDists = DIST_XF86_SERVER_SVGA | DIST_XF86_SERVER_VGA16;
     XF86FontDists = DIST_XF86_FONTS_MISC;
-#endif
-    return distSetXF86(NULL) | distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i |= distSetXF86(NULL);
+    distVerifyFlags();
+    return i;
 }
 
 int
 distSetKernDeveloper(dialogMenuItem *self)
 {
+    int i;
+
     distReset(NULL);
     Dists = _DIST_DEVELOPER;
     SrcDists = DIST_SRC_SYS;
-    return distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i = distMaybeSetDES(self) | distMaybeSetPorts(self);
+    distVerifyFlags();
+    return i;
 }
 
 int
 distSetUser(dialogMenuItem *self)
 {
+    int i;
+
     distReset(NULL);
     Dists = _DIST_USER;
-    return distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i = distMaybeSetDES(self) | distMaybeSetPorts(self);
+    distVerifyFlags();
+    return i;
 }
 
 int
 distSetXUser(dialogMenuItem *self)
 {
-    distReset(NULL);
-    Dists = _DIST_USER;
-#ifdef USE_XIG_ENVIRONMENT
-    Dists |= (DIST_XIG_SERVER | DIST_COMPAT21);
-#else
+    int i;
+
+    i = distSetUser(self);
+    Dists |= DIST_XF86;
     XF86ServerDists = DIST_XF86_SERVER_SVGA | DIST_XF86_SERVER_VGA16;
-    XF86Dists = DIST_XF86_BIN | DIST_COMPAT21 | DIST_XF86_SET | DIST_XF86_CFG | DIST_XF86_LIB | DIST_XF86_MAN | DIST_XF86_SERVER | DIST_XF86_FONTS;
+    XF86Dists = DIST_XF86_BIN | DIST_XF86_SET | DIST_XF86_CFG | DIST_XF86_LIB | DIST_XF86_MAN | DIST_XF86_SERVER | DIST_XF86_FONTS;
     XF86FontDists = DIST_XF86_FONTS_MISC;
-#endif
-    return distSetXF86(NULL) | distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i |= distSetXF86(NULL);
+    distVerifyFlags();
+    return i;
 }
 
 int
@@ -269,34 +323,29 @@ distSetMinimum(dialogMenuItem *self)
 int
 distSetEverything(dialogMenuItem *self)
 {
-    Dists = DIST_ALL;
+    int i;
+
+    Dists = DIST_ALL | DIST_XF86;
     SrcDists = DIST_SRC_ALL;
-#ifdef USE_XIG_ENVIRONMENT
-    Dists |= (DIST_XIG_SERVER | DIST_COMPAT21);
-#else
     XF86Dists = DIST_XF86_ALL;
     XF86ServerDists = DIST_XF86_SERVER_ALL;
     XF86FontDists = DIST_XF86_FONTS_ALL;
-#endif
-    return distMaybeSetDES(self) | distMaybeSetPorts(self);
+    i = distMaybeSetDES(self) | distMaybeSetPorts(self);
+    distVerifyFlags();
+    return i;
 }
 
 int
 distSetDES(dialogMenuItem *self)
 {
-    int i = DITEM_SUCCESS;
+    int i;
  
-    if (dmenuOpenSimple(&MenuDESDistributions, FALSE)) {
-	if (DESDists) {
-	    if (DESDists & DIST_DES_KERBEROS)
-		DESDists |= DIST_DES_DES;
-	    Dists |= DIST_DES;
-	    msgDebug("SetDES Masks: DES: %0x, Dists: %0x\n", DESDists, Dists);
-	}
-    }
-    else
+    if (!dmenuOpenSimple(&MenuDESDistributions, FALSE))
 	i = DITEM_FAILURE;
-    return i | DITEM_RECREATE;
+    else
+	i = DITEM_SUCCESS;
+    distVerifyFlags();
+    return i | DITEM_RESTORE;
 }
 
 static int
@@ -316,18 +365,11 @@ distMaybeSetDES(dialogMenuItem *self)
 		  "DES distribution from the U.S. server then switch your media type to\n"
 		  "point to an international FTP server, using the Custom installation\n"
 		  "option to select and extract the DES distribution in a second pass.")) {
-	if (dmenuOpenSimple(&MenuDESDistributions, FALSE)) {
-	    if (DESDists) {
-		if (DESDists & DIST_DES_KERBEROS)
-		    DESDists |= DIST_DES_DES;
-		Dists |= DIST_DES;
-		msgDebug("SetDES Masks: DES: %0x, Dists: %0x\n", DESDists, Dists);
-	    }
-	}
-	else
+	if (!dmenuOpenSimple(&MenuDESDistributions, FALSE))
 	    i = DITEM_FAILURE;
     }
-    return i | DITEM_RECREATE;
+    distVerifyFlags();
+    return i | DITEM_RESTORE;
 }
 
 static int
@@ -335,10 +377,10 @@ distMaybeSetPorts(dialogMenuItem *self)
 {
     dialog_clear_norefresh();
     if (!msgYesNo("Would you like to install the FreeBSD ports collection?\n\n"
-		  "This will give you ready access to over 800 ported software packages,\n"
+		  "This will give you ready access to over 1000 ported software packages,\n"
 		  "though at a cost of around 35MB of disk space when \"clean\" and possibly\n"
 		  "much more than that if a lot of the distribution tarballs are loaded\n"
-		  "(unless you have the 2nd CD from a FreeBSD CDROM distribution available\n"
+		  "(unless you have the 4th CD from a FreeBSD CDROM distribution available\n"
 		  "and can mount it on /cdrom, in which case this is far less of a problem).\n\n"
 		  "The ports collection is a very valuable resource and, if you have at least\n"
 		  "100MB to spare in your /usr partition, well worth having around.\n\n"
@@ -350,20 +392,67 @@ distMaybeSetPorts(dialogMenuItem *self)
     return DITEM_SUCCESS | DITEM_RESTORE;
 }
 
+static Boolean
+distSetByName(Distribution *dist, char *name)
+{
+    int i, status = FALSE;
+    
+    /* Loop through current set */
+    for (i = 0; dist[i].my_name; i++) {
+	/* This is shorthand for "dist currently disabled" */
+	if (!dist[i].my_dir)
+	    continue;
+	if (!strcmp(dist[i].my_name, name)) {
+	    *(dist[i].my_mask) |= dist[i].my_bit;
+	    status = TRUE;
+	}
+	if (dist[i].my_dist) {
+	    if (distSetByName(dist[i].my_dist, name)) {
+		status = TRUE;
+	    }
+	}
+    }
+    distVerifyFlags();
+    return status;
+}
+
+/* Just for the dispatch stuff */
+int
+distSetCustom(dialogMenuItem *self)
+{
+    char *cp, *cp2, *tmp;
+
+    if (!(tmp = variable_get(VAR_DISTS))) {
+	msgDebug("distSetCustom() called without %s variable set.\n", VAR_DISTS);
+	return DITEM_FAILURE;
+    }
+
+    cp = alloca(strlen(tmp) + 1);
+    if (!cp)
+	msgFatal("Couldn't alloca() %d bytes!\n", strlen(tmp) + 1);
+    strcpy(cp, tmp);
+    while (cp) {
+	if ((cp2 = index(cp, ' ')) != NULL)
+	    *(cp2++) = '\0';
+	if (!distSetByName(DistTable, cp))
+	    msgDebug("distSetCustom: Warning, no such release \"%s\"\n", cp);
+	cp = cp2;
+    }
+    distVerifyFlags();
+    return DITEM_SUCCESS;
+}
+    
 int
 distSetSrc(dialogMenuItem *self)
 {
-    int i = DITEM_SUCCESS;
+    int i;
 
-    if (dmenuOpenSimple(&MenuSrcDistributions, FALSE)) {
-	if (SrcDists) {
-	    Dists |= DIST_SRC;
-	    msgDebug("SetSrc Masks: Srcs: %0x, Dists: %0x\n", SrcDists, Dists);
-	}
-    }
-    else
+    if (!dmenuOpenSimple(&MenuSrcDistributions, FALSE))
 	i = DITEM_FAILURE;
-    return i | DITEM_RECREATE;
+    else
+	i = DITEM_SUCCESS;
+    distVerifyFlags();
+    return i | DITEM_RESTORE;
 }
 
 int
@@ -371,24 +460,10 @@ distSetXF86(dialogMenuItem *self)
 {
     int i = DITEM_SUCCESS;
 
-#ifdef USE_XIG_ENVIRONMENT
-    Dists |= (DIST_XIG_SERVER | DIST_COMPAT21);
-#else
-    if (dmenuOpenSimple(&MenuXF86Select, FALSE)) {
-	if (XF86ServerDists)
-	    XF86Dists |= DIST_XF86_SERVER;
-	if (XF86FontDists)
-	    XF86Dists |= DIST_XF86_FONTS;
-	if (XF86Dists)
-	    Dists |= (DIST_XF86 | DIST_COMPAT21);
-	if (isDebug())
-	    msgDebug("SetXF86 Masks: Server: %0x, Fonts: %0x, XDists: %0x, Dists: %0x\n",
-		     XF86ServerDists, XF86FontDists, XF86Dists, Dists);
-    }
-    else
+    if (!dmenuOpenSimple(&MenuXF86Select, FALSE))
 	i = DITEM_FAILURE;
-#endif
-    return i | DITEM_RECREATE;
+    distVerifyFlags();
+    return i | DITEM_RESTORE;
 }
 
 static Boolean got_intr = FALSE;
@@ -688,24 +763,34 @@ printSelected(char *buf, int selected, Distribution *me, int *col)
 int
 distExtractAll(dialogMenuItem *self)
 {
-    int retries = 0;
+    int old_dists, retries = 0, status = DITEM_SUCCESS;
     char buf[512];
 
     /* paranoia */
     if (!Dists) {
-	if (!dmenuOpenSimple(&MenuDistributions, FALSE) || !Dists)
+	if (!dmenuOpenSimple(&MenuSubDistributions, FALSE) || !Dists)
 	    return DITEM_FAILURE | DITEM_RESTORE;
     }
 
     if (!mediaVerify() || !mediaDevice->init(mediaDevice))
 	return DITEM_FAILURE;
 
+    old_dists = Dists;
+    distVerifyFlags();
+
     dialog_clear_norefresh();
     msgNotify("Attempting to install all selected distributions..");
+
     /* Try for 3 times around the loop, then give up. */
     while (Dists && ++retries < 3)
 	distExtract(NULL, DistTable);
 
+    /* Only do bin fixup if bin dist was successfully extracted */
+    if ((old_dists & DIST_BIN) && !(Dists & DIST_BIN))
+	status |= installFixupBin(self);
+    if (old_dists & DIST_XF86)
+	status |= installFixupXFree(self);
+
     if (Dists) {
 	int col = 0;
 
@@ -715,7 +800,7 @@ distExtractAll(dialogMenuItem *self)
 	msgConfirm("Couldn't extract the following distributions.  This may\n"
 		   "be because they were not available on the installation\n"
 		   "media you've chosen:\n\n\t%s", buf);
-	return DITEM_SUCCESS | DITEM_RESTORE;
+	status |= DITEM_RESTORE;
     }
-    return DITEM_SUCCESS;
+    return status;
 }
