@@ -17,11 +17,7 @@
 #define DIST_DES		0x1000
 #define DIST_CATPAGES		0x2000
 #define DIST_PORTS		0x4000
-#ifdef USE_XIG_ENVIRONMENT
-#define DIST_XIG_SERVER		0x8000
-#else
 #define DIST_USR1		0x8000
-#endif
 #define DIST_ALL		0x7FFF
 
 /* Canned distribution sets */
@@ -33,9 +29,10 @@
 
 /* Subtypes for DES distribution */
 #define DIST_DES_DES		0x0001
-#define DIST_DES_SEBONES	0x0002
+#define DIST_DES_SCRYPTO	0x0002
 #define DIST_DES_SSECURE	0x0004
 #define DIST_DES_KERBEROS	0x0008
+#define DIST_DES_SKERBEROS	0x0010
 
 /* Subtypes for SRC distribution */
 #define DIST_SRC_BASE		0x00001
@@ -57,7 +54,6 @@
 #define DIST_SRC_SMAILCF	0x10000
 #define DIST_SRC_ALL		0xFFFF	/* no SMAILCF, it's part of USBIN */
 
-#ifndef USE_XIG_ENVIRONMENT
 /* Subtypes for XFree86 distribution */
 #define DIST_XF86_BIN		0x0001
 #define DIST_XF86_CFG		0x0002
@@ -70,11 +66,12 @@
 #define DIST_XF86_PROG		0x0100
 #define DIST_XF86_PS		0x0200
 #define DIST_XF86_SET		0x0400
-#define DIST_XF86_SRC		0x0800
-#define DIST_XF86_CSRC		0x1000
-#define DIST_XF86_MISC_ALL	0x1FFF
+#define DIST_XF86_9SET		0x0800
+#define DIST_XF86_SRC		0x1000
+#define DIST_XF86_CSRC		0x2000
+#define DIST_XF86_MISC_ALL	0x3FFF
 #define DIST_XF86_SERVER	0x8000
-#define DIST_XF86_SERVER_8514		0x0000001
+#define DIST_XF86_SERVER_9MGA		0x0000001
 #define DIST_XF86_SERVER_9480		0x0000002
 #define DIST_XF86_SERVER_9EGC		0x0000004
 #define DIST_XF86_SERVER_9GA9		0x0000008
@@ -82,26 +79,27 @@
 #define DIST_XF86_SERVER_9LPW		0x0000020
 #define DIST_XF86_SERVER_9NKV		0x0000040
 #define DIST_XF86_SERVER_9NS3		0x0000080
-#define DIST_XF86_SERVER_9SPW		0x0000100
-#define DIST_XF86_SERVER_9TGU		0x0000200
-#define DIST_XF86_SERVER_9WEP		0x0000400
-#define DIST_XF86_SERVER_9WS		0x0000800
-#define DIST_XF86_SERVER_9WSN		0x0001000
-#define DIST_XF86_SERVER_AGX		0x0002000
-#define DIST_XF86_SERVER_I128		0x0004000
-#define DIST_XF86_SERVER_MACH8		0x0008000
-#define DIST_XF86_SERVER_MACH32		0x0010000
-#define DIST_XF86_SERVER_MACH64		0x0020000
-#define DIST_XF86_SERVER_MONO		0x0040000
-#define DIST_XF86_SERVER_P9000		0x0080000
-#define DIST_XF86_SERVER_S3		0x0100000
-#define DIST_XF86_SERVER_S3V		0x0200000
-#define DIST_XF86_SERVER_SVGA		0x0400000
-#define DIST_XF86_SERVER_VGA16		0x0800000
-#define DIST_XF86_SERVER_W32		0x1000000
-#define DIST_XF86_SERVER_NEST		0x2000000
-#define DIST_XF86_SERVER_VFB		0x4000000
-#define DIST_XF86_SERVER_ALL		0x7FFFFFF
+#define DIST_XF86_SERVER_9SVG		0x0000100
+#define DIST_XF86_SERVER_9SPW		0x0000200
+#define DIST_XF86_SERVER_9TGU		0x0000400
+#define DIST_XF86_SERVER_9WEP		0x0000800
+#define DIST_XF86_SERVER_9WS		0x0001000
+#define DIST_XF86_SERVER_9WSN		0x0002000
+#define DIST_XF86_SERVER_8514		0x0004000
+#define DIST_XF86_SERVER_AGX		0x0008000
+#define DIST_XF86_SERVER_I128		0x0010000
+#define DIST_XF86_SERVER_MACH8		0x0020000
+#define DIST_XF86_SERVER_MACH32		0x0040000
+#define DIST_XF86_SERVER_MACH64		0x0080000
+#define DIST_XF86_SERVER_MONO		0x0100000
+#define DIST_XF86_SERVER_P9000		0x0200000
+#define DIST_XF86_SERVER_S3		0x0400000
+#define DIST_XF86_SERVER_S3V		0x0800000
+#define DIST_XF86_SERVER_SVGA		0x1000000
+#define DIST_XF86_SERVER_VGA16		0x2000000
+#define DIST_XF86_SERVER_W32		0x4000000
+#define DIST_XF86_SERVER_NEST		0x8000000
+#define DIST_XF86_SERVER_ALL		0xFFFFFFF
 #define DIST_XF86_FONTS		0x10000
 #define DIST_XF86_FONTS_MISC		0x0001
 #define DIST_XF86_FONTS_100		0x0002
@@ -111,6 +109,5 @@
 #define DIST_XF86_FONTS_SERVER		0x0020
 #define DIST_XF86_FONTS_ALL		0x003F
 #define DIST_XF86_ALL		0x1FFFF
-#endif	/* !USE_XIG_ENVIRONMENT */
 
 #endif	/* _DIST_H_INCLUDE */
