@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated for what's essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/dmenu.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/dmenu.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -115,7 +115,7 @@ dmenuSetKmapVariable(dialogMenuItem *tmp)
     int err;
 
     variable_set((char *)tmp->data);
-    lang = variable_get("keymap");
+    lang = variable_get(VAR_KEYMAP);
     if (lang != NULL)
     {
 	err = loadKeymap(lang);
@@ -130,10 +130,38 @@ dmenuSetKmapVariable(dialogMenuItem *tmp)
 int
 dmenuToggleVariable(dialogMenuItem *tmp)
 {
-    if (!variable_get((char *)tmp->data))
-	variable_set((char *)tmp->data);
+    char *var;
+
+    if (!(var = (char *)tmp->data)) {
+	msgConfirm("Incorrect data field for `%s'!", tmp->title);
+	return DITEM_FAILURE;
+    }
+    if (!variable_check(var))
+	variable_set(var);
     else
-	variable_unset((char *)tmp->data);
+	variable_unset(var);
+    return DITEM_SUCCESS;
+}
+
+int
+dmenuISetVariable(dialogMenuItem *tmp)
+{
+    char *ans, *var;
+    WINDOW *w = NULL;	/* Keep lint happy */
+
+    if (!(var = (char *)tmp->data)) {
+	msgConfirm("Incorrect data field for `%s'!", tmp->title);
+	return DITEM_FAILURE;
+    }
+    w = savescr();
+    ans = msgGetInput(variable_get(var), tmp->title);
+    restorescr(w);
+    if (!ans)
+	return DITEM_FAILURE;
+    else if (!*ans)
+	variable_unset(var);
+    else
+	variable_set2(var, ans);
     return DITEM_SUCCESS;
 }
 
@@ -174,26 +202,40 @@ dmenuFlagCheck(dialogMenuItem *item)
 int
 dmenuVarCheck(dialogMenuItem *item)
 {
-    char *w, *cp, *cp2, *cp3, tmp[256];
+    char *w;
+
+    w = (char *)item->aux;
+    if (!w)
+	w = (char *)item->data;
+    return variable_check(w);
+}
+
+int
+dmenuVarsCheck(dialogMenuItem *item)
+{
+    int res, init;
+    char *w, *cp1, *cp2;
+    char *copy;
 
     w = (char *)item->aux;
     if (!w)
 	w = (char *)item->data;
     if (!w)
 	return FALSE;
-    SAFE_STRCPY(tmp, w);
-    if ((cp = index(tmp, '=')) != NULL) {
-        *(cp++) = '\0';
-	if ((cp3 = index(cp, ',')) != NULL)
-	    *cp3 = '\0';
-        cp2 = getenv(tmp);
-        if (cp2)
-            return !strcmp(cp, cp2);
-        else
-            return FALSE;
+    
+    copy = strdup(w);
+    res = TRUE;
+    init = FALSE;
+    for (cp1 = copy; cp1 != NULL;) {
+        init = TRUE;
+	cp2 = index(cp1, ',');
+	if (cp2 != NULL)
+	    *cp2++ = '\0';
+	res = res && variable_check(cp1);
+	cp1 = cp2;
     }
-    else
-        return (int)getenv(tmp);
+    free(copy);
+    return res && init;
 }
 
 int
