@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/dos.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/dos.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -44,52 +44,56 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <grp.h>
-#include "dosio.h"
+#define MSDOSFS
+#include <sys/mount.h>
+#undef MSDOSFS
 
-static DOS_FS DOSFS;
 static Boolean DOSMounted;
 
 Boolean
 mediaInitDOS(Device *dev)
 {
+    struct msdosfs_args args;
+
     if (!RunningAsInit || DOSMounted)
 	return TRUE;
+     
+    if (DITEM_STATUS(Mkdir("/dist")) != DITEM_SUCCESS)
+	return FALSE;
 
-    if (dos_mount(&DOSFS, dev->devname)) {
-	msgConfirm("Error mounting DOS partition %s : %s (%u)", dev->devname, strerror(errno), errno);
+    memset(&args, 0, sizeof(args));
+    args.fspec = dev->devname;
+    args.uid = args.gid = 0;
+    args.mask = 0777;
+
+    if (mount(MOUNT_MSDOS, "/dist", MNT_RDONLY, (caddr_t)&args) == -1) {
+	msgConfirm("Error mounting %s on /dist: %s (%u)", args.fspec, strerror(errno), errno);
 	return FALSE;
     }
+    else
+	msgDebug("Mounted DOS device (%s) on /dist.\n", args.fspec);
     DOSMounted = TRUE;
-    msgDebug("DOS partition %s mounted\n", dev->devname);
     return TRUE;
 }
 
 FILE *
 mediaGetDOS(Device *dev, char *file, Boolean probe)
 {
-    char buf[PATH_MAX];
-    FILE *fp;
-
-    if (!DOSMounted) {
-	msgDebug("Can't get DOS file %s - DOSFS currently unmounted!\n", file);
-	return NULL;
-    }
+    char	buf[PATH_MAX];
 
     if (isDebug())
 	msgDebug("Request for %s from DOS\n", file);
-    snprintf(buf, PATH_MAX, "/freebsd/%s", file);
-    if ((fp = dos_open(&DOSFS, buf)))
-	return fp;
-    snprintf(buf, PATH_MAX, "/freebsd/dists/%s", file);
-    if ((fp = dos_open(&DOSFS, buf)))
-	return fp;
-    snprintf(buf, PATH_MAX, "/%s", file);
-    if ((fp = dos_open(&DOSFS, buf)))
-	return fp;
-    snprintf(buf, PATH_MAX, "/dists/%s", file);
-    if ((fp = dos_open(&DOSFS, buf)))
-	return fp;
-    return NULL;
+    snprintf(buf, PATH_MAX, "/dist/%s", file);
+    if (file_readable(buf))
+	return fopen(buf, "r");
+    snprintf(buf, PATH_MAX, "/dist/FREEBSD/%s", file);
+    if (file_readable(buf))
+	return fopen(buf, "r");
+    snprintf(buf, PATH_MAX, "/dist/%s/%s", variable_get(VAR_RELNAME), file);
+    if (file_readable(buf))
+	return fopen(buf, "r");
+    snprintf(buf, PATH_MAX, "/dist/%s/dists/%s", variable_get(VAR_RELNAME), file);
+    return fopen(buf, "r");
 }
 
 void
@@ -97,10 +101,11 @@ mediaShutdownDOS(Device *dev)
 {
     if (!RunningAsInit || !DOSMounted)
 	return;
-    if (dos_unmount(&DOSFS))
-	msgConfirm("Could not unmount DOS partition %s : %s", dev->devname, strerror(errno));
-    else if (isDebug())
-	msgDebug("Unmount of DOS partition on %s successful\n", dev->devname);
+    msgDebug("Unmounting %s from /dist\n", dev->name);
+    if (unmount("/dist", MNT_FORCE) != 0)
+	msgConfirm("Could not unmount the DOS partition: %s", strerror(errno));
+    if (isDebug())
+	msgDebug("Unmount successful\n");
     DOSMounted = FALSE;
     return;
 }
