@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/floppy.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/floppy.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -51,87 +51,35 @@
 #include <sys/mount.h>
 #undef MSDOSFS
 
-static Device *floppyDev;
 static Boolean floppyMounted;
 
 char *distWanted;
 
-/* For finding floppies */
-static int
-floppyChoiceHook(dialogMenuItem *self)
-{
-    Device **devs;
-
-    devs = deviceFind(self->prompt, DEVICE_TYPE_FLOPPY);
-    if (devs)
-	floppyDev = devs[0];
-    return devs ? DITEM_LEAVE_MENU : DITEM_FAILURE;
-}
-
-/* Our last-ditch routine for getting ROOT from a floppy */
-int
-getRootFloppy(void)
-{
-    int fd = -1;
-
-    if (mediaDevice->type == DEVICE_TYPE_FLOPPY)
-	floppyDev = mediaDevice;
-    else {
-	Device **devs;
-	int cnt;
-
-	devs = deviceFind(NULL, DEVICE_TYPE_FLOPPY);
-	cnt = deviceCount(devs);
-	if (!cnt) {
-	    msgConfirm("No floppy devices found!  Something is seriously wrong!");
-	    return -1;
-	}
-	else if (cnt == 1)
-	    floppyDev = devs[0];
-	else {
-	    DMenu *menu;
-	    int ret;
-	    WINDOW *save = savescr();
-
-	    menu = deviceCreateMenu(&MenuMediaFloppy, DEVICE_TYPE_FLOPPY, floppyChoiceHook, NULL);
-	    menu->title = "Please insert the ROOT floppy";
-	    ret = dmenuOpenSimple(menu, FALSE);
-	    restorescr(save);
-	    if (!ret)
-		return -1;
-	}
-    }
-    while (fd == -1) {
-	msgConfirm("Please insert the ROOT floppy in %s and press [ENTER]", floppyDev->description);
-	fd = open(floppyDev->devname, O_RDONLY);
-	if (isDebug())
-	    msgDebug("getRootFloppy on %s yields fd of %d\n", floppyDev->devname, fd);
-	if (fd == -1 && msgYesNo("Couldn't open the floppy - do you want to try again?") != 0)
-	    break;
-    }
-    return fd;
-}
-
 Boolean
 mediaInitFloppy(Device *dev)
 {
     struct msdosfs_args dosargs;
     struct ufs_args u_args;
-    char *mountpoint = "/dist";
+    char *mountpoint;
 
     if (floppyMounted)
 	return TRUE;
 
+    mountpoint = (char *)dev->private;
     if (Mkdir(mountpoint)) {
 	msgConfirm("Unable to make %s directory mountpoint for %s!", mountpoint, dev->devname);
 	return FALSE;
     }
 
     msgDebug("Init floppy called for %s distribution.\n", distWanted ? distWanted : "some");
-    if (!distWanted)
-    	msgConfirm("Please insert floppy in %s", dev->description);
-    else
-	msgConfirm("Please insert floppy containing %s in %s", distWanted, dev->description);
+
+    if (!variable_get(VAR_NONINTERACTIVE)) {
+	if (!distWanted)
+	    msgConfirm("Please insert floppy in %s", dev->description);
+	else
+	    msgConfirm("Please insert floppy containing %s in %s",
+			distWanted, dev->description);
+    }
 
     memset(&dosargs, 0, sizeof dosargs);
     dosargs.fspec = dev->devname;
@@ -161,10 +109,10 @@ mediaGetFloppy(Device *dev, char *file, Boolean probe)
     FILE	*fp;
     int		nretries = 5;
 
-    snprintf(buf, PATH_MAX, "/dist/%s", file);
+    snprintf(buf, PATH_MAX, "%s/%s", (char *)dev->private, file);
 
     if (isDebug())
-	msgDebug("Request for %s from floppy on /dist, probe is %d.\n", buf, probe);
+	msgDebug("Request for %s from floppy on %s, probe is %d.\n", buf, (char *)dev->private, probe);
     if (!file_readable(buf)) {
 	if (probe)
 	    return NULL;
@@ -188,7 +136,7 @@ mediaGetFloppy(Device *dev, char *file, Boolean probe)
 void
 mediaShutdownFloppy(Device *dev)
 {
-    char *mountpoint = "/dist";
+    char *mountpoint = (char *)dev->private;
 
     if (floppyMounted) {
 	if (unmount(mountpoint, MNT_FORCE) != 0)
@@ -196,7 +144,9 @@ mediaShutdownFloppy(Device *dev)
 	else {
 	    floppyMounted = FALSE;
 	    msgDebug("Floppy unmounted successfully.\n");
-	    msgConfirm("You may remove the floppy from %s", dev->description);
+	    if (!variable_get(VAR_NONINTERACTIVE))
+		msgConfirm("You may remove the floppy from %s",
+			   dev->description);
 	}
     }
 }
