@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/install.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/install.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -47,6 +47,7 @@
 #include <sys/mount.h>
 #undef MSDOSFS
 #include <sys/stat.h>
+#include <sys/sysctl.h>
 #include <unistd.h>
 
 static void	create_termcap(void);
@@ -68,6 +69,10 @@ checkLabels(Boolean whinge, Chunk **rdev, Chunk **sdev, Chunk **udev, Chunk **vd
     Chunk *c1, *c2, *rootdev, *swapdev, *usrdev, *vardev;
     int i;
 
+    /* Don't allow whinging if noWarn is set */
+    if (variable_get(VAR_NO_WARN))
+	whinge = FALSE;
+
     status = TRUE;
     *rdev = *sdev = *udev = *vdev = rootdev = swapdev = usrdev = vardev = NULL;
 
@@ -88,7 +93,7 @@ checkLabels(Boolean whinge, Chunk **rdev, Chunk **sdev, Chunk **udev, Chunk **vd
 	    if (c1->type == freebsd) {
 		for (c2 = c1->part; c2; c2 = c2->next) {
 		    if (c2->type == part && c2->subtype != FS_SWAP && c2->private_data) {
-			if (c2->flags & CHUNK_IS_ROOT) {
+			if (!strcmp(((PartInfo *)c2->private_data)->mountpoint, "/")) {
 			    if (rootdev) {
 				if (whinge)
 				    msgConfirm("WARNING:  You have more than one root device set?!\n"
@@ -171,13 +176,13 @@ checkLabels(Boolean whinge, Chunk **rdev, Chunk **sdev, Chunk **udev, Chunk **vd
 		   "swap partition.");
 	status = FALSE;
     }
-    if (!usrdev && whinge) {
+    if (!usrdev && whinge && !variable_get(VAR_NO_USR)) {
 	msgConfirm("WARNING:  No /usr filesystem found.  This is not technically\n"
 		   "an error if your root filesystem is big enough (or you later\n"
 		   "intend to mount your /usr filesystem over NFS), but it may otherwise\n"
 		   "cause you trouble if you're not exactly sure what you are doing!");
     }
-    if (!vardev && whinge) {
+    if (!vardev && whinge && variable_cmp(SYSTEM_STATE, "upgrade")) {
 	msgConfirm("WARNING:  No /var filesystem found.  This is not technically\n"
 		   "an error if your root filesystem is big enough (or you later\n"
 		   "intend to link /var to someplace else), but it may otherwise\n"
@@ -191,6 +196,7 @@ static int
 installInitial(void)
 {
     static Boolean alreadyDone = FALSE;
+    int status = DITEM_SUCCESS;
 
     if (alreadyDone)
 	return DITEM_SUCCESS;
@@ -206,40 +212,46 @@ installInitial(void)
 
     /* If we refuse to proceed, bail. */
     dialog_clear_norefresh();
-    if (msgYesNo("Last Chance!  Are you SURE you want continue the installation?\n\n"
-		 "If you're running this on a disk with data you wish to save\n"
-		 "then WE STRONGLY ENCOURAGE YOU TO MAKE PROPER BACKUPS before\n"
-		 "proceeding!\n\n"
-		 "We can take no responsibility for lost disk contents!") != 0)
+    if (!variable_get(VAR_NO_WARN))
+	if (msgYesNo(
+	    "Last Chance!  Are you SURE you want continue the installation?\n\n"
+	     "If you're running this on a disk with data you wish to save\n"
+	     "then WE STRONGLY ENCOURAGE YOU TO MAKE PROPER BACKUPS before\n"
+	     "proceeding!\n\n"
+	     "We can take no responsibility for lost disk contents!") != 0)
 	return DITEM_FAILURE | DITEM_RESTORE;
 
     if (DITEM_STATUS(diskLabelCommit(NULL)) != DITEM_SUCCESS) {
 	msgConfirm("Couldn't make filesystems properly.  Aborting.");
 	return DITEM_FAILURE;
     }
-    else if (isDebug())
-	msgDebug("installInitial: Scribbled successfully on the disk(s)\n");
 
     if (!copySelf()) {
-	msgConfirm("Couldn't clone the boot floppy onto the root file system.\n"
-		   "Aborting.");
+	msgConfirm("installInitial: Couldn't clone the boot floppy onto the\n"
+		   "root file system.  Aborting!");
 	return DITEM_FAILURE;
     }
 
     if (chroot("/mnt") == -1) {
-	msgConfirm("Unable to chroot to %s - this is bad!", "/mnt");
+	msgConfirm("installInitial: Unable to chroot to %s - this is bad!",
+		   "/mnt");
 	return DITEM_FAILURE;
     }
 
     chdir("/");
     variable_set2(RUNNING_ON_ROOT, "yes");
-    configResolv();
+
+    /* Configure various files in /etc */
+    if (DITEM_STATUS(configResolv(NULL)) == DITEM_FAILURE)
+	status = DITEM_FAILURE;
+    if (DITEM_STATUS(configFstab(NULL)) == DITEM_FAILURE)
+	status = DITEM_FAILURE;
 
     /* stick a helpful shell over on the 4th VTY */
     systemCreateHoloshell();
 
     alreadyDone = TRUE;
-    return DITEM_SUCCESS;
+    return status;
 }
 
 int
@@ -332,28 +344,34 @@ installFixitFloppy(dialogMenuItem *self)
 	return DITEM_SUCCESS;
 
     variable_set2(SYSTEM_STATE, "fixit");
-    memset(&args, 0, sizeof(args));
-    args.fspec = "/dev/fd0";
     Mkdir("/mnt2");
 
+    /* Try to open the floppy drive */
+    if (DITEM_STATUS(mediaSetFloppy(NULL)) == DITEM_FAILURE) {
+	msgConfirm("Unable to set media device to floppy.");
+	mediaClose();
+	return DITEM_FAILURE;
+    }
+
+    memset(&args, 0, sizeof(args));
+    args.fspec = mediaDevice->devname;
+
     while (1) {
 	msgConfirm("Please insert a writable fixit floppy and press return");
-	if (mount(MOUNT_UFS, "/mnt2", 0, (caddr_t)&args) != -1)
+	mediaDevice->private = "/mnt2";
+	if (!mediaDevice->init(mediaDevice)) {
+	    if (msgYesNo("The attempt to mount the fixit floppy failed, bad floppy\n"
+			 "or unclean filesystem.  Do you want to try again?"))
+		return DITEM_FAILURE;
+	}
+	else
 	    break;
-	msgConfirm("An attempt to mount the fixit floppy failed, maybe the filesystem\n"
-		   "is unclean.  Trying a forcible mount as a last resort...");
-	if (mount(MOUNT_UFS, "/mnt2", MNT_FORCE, (caddr_t)&args) != -1)
-	    break;
-	if (msgYesNo("Unable to mount the fixit floppy - do you want to try again?") != 0)
-	    return DITEM_FAILURE;
     }
-
     if (!directory_exists("/tmp"))
 	(void)symlink("/mnt2/tmp", "/tmp");
-
     fixit_common();
-
-    unmount("/mnt2", MNT_FORCE);
+    mediaDevice->shutdown(mediaDevice);
+    mediaDevice = NULL;
     msgConfirm("Please remove the fixit floppy now.");
     return DITEM_SUCCESS;
 }
@@ -441,6 +459,7 @@ installExpress(dialogMenuItem *self)
     if (DITEM_STATUS((i = diskLabelEditor(self))) == DITEM_FAILURE)
 	return i;
 
+    dialog_clear_norefresh();
     if (DITEM_STATUS((i = installCommit(self))) == DITEM_SUCCESS) {
 	i |= DITEM_LEAVE_MENU;
 	/* Give user the option of one last configuration spree */
@@ -453,20 +472,29 @@ installExpress(dialogMenuItem *self)
 int
 installNovice(dialogMenuItem *self)
 {
-    int i;
+    int i, tries = 0;
+    Device **devs;
 
     variable_set2(SYSTEM_STATE, "novice");
     dialog_clear_norefresh();
     msgConfirm("In the next menu, you will need to set up a DOS-style (\"fdisk\") partitioning\n"
 	       "scheme for your hard disk.  If you simply wish to devote all disk space\n"
-	       "to FreeBSD (overwritting anything else that might be on the disk(s) selected)\n"
+	       "to FreeBSD (overwriting anything else that might be on the disk(s) selected)\n"
 	       "then use the (A)ll command to select the default partitioning scheme followed\n"
 	       "by a (Q)uit.  If you wish to allocate only free space to FreeBSD, move to a\n"
 	       "partition marked \"unused\" and use the (C)reate command.");
 
+nodisks:
     if (DITEM_STATUS(diskPartitionEditor(self)) == DITEM_FAILURE)
 	return DITEM_FAILURE;
-    
+
+    if (diskGetSelectCount(&devs) <= 0 && tries < 3) {
+	msgConfirm("You need to select some disks to operate on!  Be sure to use SPACE\n"
+		   "instead of RETURN in the disk selection menu when selecting a disk.");
+	++tries;
+	goto nodisks;
+    }
+
     dialog_clear_norefresh();
     msgConfirm("Next, you need to create BSD partitions inside of the fdisk partition(s)\n"
 	       "just created.  If you have a reasonable amount of disk space (200MB or more)\n"
@@ -478,6 +506,7 @@ installNovice(dialogMenuItem *self)
     if (DITEM_STATUS(diskLabelEditor(self)) == DITEM_FAILURE)
 	return DITEM_FAILURE;
 
+    dialog_clear_norefresh();
     if (DITEM_STATUS((i = installCommit(self))) == DITEM_FAILURE) {
 	dialog_clear_norefresh();
 	msgConfirm("Installation completed with some errors.  You may wish to\n"
@@ -510,16 +539,10 @@ installNovice(dialogMenuItem *self)
 	}
     }
 
-    dialog_clear_norefresh();
-    if (!msgYesNo("Would you like to configure Samba for connecting NETBUI clients to this\n"
-		  "machine?  Windows 95, Windows NT and Windows for Workgroups\n"
-		  "machines can use NETBUI transport for disk and printer sharing."))
-	configSamba(self);
-
     dialog_clear_norefresh();
     if (!msgYesNo("Will this machine be an IP gateway (e.g. will it forward packets\n"
 		  "between interfaces)?"))
-	variable_set2("gateway", "YES");
+	variable_set2("gateway_enable", "YES");
 
     dialog_clear_norefresh();
     if (!msgYesNo("Do you want to allow anonymous FTP connections to this machine?"))
@@ -531,11 +554,7 @@ installNovice(dialogMenuItem *self)
 
     dialog_clear_norefresh();
     if (!msgYesNo("Do you want to configure this machine as an NFS client?"))
-	variable_set2("nfs_client", "YES");
-
-    dialog_clear_norefresh();
-    if (!msgYesNo("Do you want to configure this machine as a WEB server?"))
-	configApache(self);
+	variable_set2("nfs_client_enable", "YES");
 
     dialog_clear_norefresh();
     if (!msgYesNo("Would you like to customize your system console settings?")) {
@@ -563,13 +582,13 @@ installNovice(dialogMenuItem *self)
     }
 
     /* Now would be a good time to checkpoint the configuration data */
-    configSysconfig("/etc/sysconfig");
+    configRC_conf("/etc/rc.conf");
     sync();
 
     if (directory_exists("/usr/X11R6")) {
 	dialog_clear_norefresh();
 	if (!msgYesNo("Would you like to configure your X server at this time?"))
-	    configXFree86(self);
+	    configXEnvironment(self);
     }
 
     dialog_clear_norefresh();
@@ -604,14 +623,17 @@ installNovice(dialogMenuItem *self)
 		  "This is where your registration can really help us, and you can also\n"
 		  "sign up for the new FreeBSD newsletter (its free!) at the same time.\n"))
 	configRegister(NULL);
-    else
+    else {
+	dialog_clear_norefresh();
 	msgConfirm("OK, but if you should change your mind then you always can register\n"
 		   "later by typing ``/stand/sysinstall register'' or by simply visiting our\n"
 		   "web site at http://www.freebsd.org/register.html");
 
+    }
     /* XXX Put whatever other nice configuration questions you'd like to ask the user here XXX */
 
     /* Give user the option of one last configuration spree */
+    dialog_clear_norefresh();
     installConfigure();
 
     return DITEM_LEAVE_MENU | DITEM_RESTORE;
@@ -623,6 +645,7 @@ installCustomCommit(dialogMenuItem *self)
 {
     int i;
 
+    dialog_clear_norefresh();
     i = installCommit(self);
     if (DITEM_STATUS(i) == DITEM_SUCCESS) {
 	/* Give user the option of one last configuration spree */
@@ -649,12 +672,13 @@ installCommit(dialogMenuItem *self)
 {
     int i;
     char *str;
-    Boolean need_bin;
 
-    if (!Dists) {
+    if (!Dists)
+	distConfig(NULL);
+
+    if (!Dists)
 	if (!dmenuOpenSimple(&MenuDistributions, FALSE) && !Dists)
 	    return DITEM_FAILURE | DITEM_RESTORE;
-    }
 
     if (!mediaVerify())
 	return DITEM_FAILURE | DITEM_RESTORE;
@@ -663,13 +687,9 @@ installCommit(dialogMenuItem *self)
     if (isDebug())
 	msgDebug("installCommit: System state is `%s'\n", str);
 
-    if (RunningAsInit) {
-	/* Do things we wouldn't do to a multi-user system */
-	if (DITEM_STATUS((i = installInitial())) == DITEM_FAILURE)
-	    return i;
-	if (DITEM_STATUS((i = configFstab())) == DITEM_FAILURE)
-	    return i;
-    }
+    /* Installation stuff we wouldn't do to a running system */
+    if (RunningAsInit && DITEM_STATUS((i = installInitial())) == DITEM_FAILURE)
+	return i;
 
 try_media:
     if (!mediaDevice->init(mediaDevice)) {
@@ -685,13 +705,14 @@ try_media:
 	    return DITEM_FAILURE | DITEM_RESTORE;
     }
 
-    need_bin = Dists & DIST_BIN;
+    /* Now go get it all */
     i = distExtractAll(self);
-    if (DITEM_STATUS(i) == DITEM_SUCCESS) {
-	if (need_bin && !(Dists & DIST_BIN))
-	    i = installFixup(self);
-    }
+
+    /* When running as init, *now* it's safe to grab the rc.foo vars */
+    installEnvironment();
+
     variable_set2(SYSTEM_STATE, DITEM_STATUS(i) == DITEM_FAILURE ? "error-install" : "full-install");
+
     return i | DITEM_RESTORE;
 }
 
@@ -707,36 +728,39 @@ installConfigure(void)
 	dmenuOpenSimple(&MenuConfigure, FALSE);
 	restorescr(w);
     }
+    configRC_conf("/etc/rc.conf");
+    sync();
 }
 
 int
-installFixup(dialogMenuItem *self)
+installFixupBin(dialogMenuItem *self)
 {
     Device **devs;
     int i;
 
-    if (!file_readable("/kernel")) {
-	if (file_readable("/kernel.GENERIC")) {
+    /* All of this is done only as init, just to be safe */
+    if (RunningAsInit) {
+	/* Fix up kernel first */
+	if (!file_readable("/kernel")) {
+	    if (file_readable("/kernel.GENERIC")) {
+		if (vsystem("cp -p /kernel.GENERIC /kernel")) {
+		    msgConfirm("Unable to copy /kernel into place!");
+		    return DITEM_FAILURE;
+		}
 #ifdef SAVE_USERCONFIG
-	    /* Snapshot any boot -c changes back to the GENERIC kernel */
-	    if (!strcmp(variable_get(VAR_RELNAME), RELEASE_NAME))
-		save_userconfig_to_kernel("/kernel.GENERIC");
+		/* Snapshot any boot -c changes back to the new kernel */
+		save_userconfig_to_kernel("/kernel");
 #endif
-	    if (vsystem("cp -p /kernel.GENERIC /kernel")) {
-		msgConfirm("Unable to link /kernel into place!");
+	    }
+	    else {
+		msgConfirm("Can't find a kernel image to link to on the root file system!\n"
+			   "You're going to have a hard time getting this system to\n"
+			   "boot from the hard disk, I'm afraid!");
 		return DITEM_FAILURE;
 	    }
 	}
-	else {
-	    msgConfirm("Can't find a kernel image to link to on the root file system!\n"
-		       "You're going to have a hard time getting this system to\n"
-		       "boot from the hard disk, I'm afraid!");
-	    return DITEM_FAILURE;
-	}
-    }
-
-    /* Resurrect /dev after bin distribution screws it up */
-    if (RunningAsInit) {
+	
+	/* BOGON #1: Resurrect /dev after bin distribution screws it up */
 	msgNotify("Remaking all devices.. Please wait!");
 	if (vsystem("cd /dev; sh MAKEDEV all")) {
 	    msgConfirm("MAKEDEV returned non-zero status");
@@ -752,7 +776,7 @@ installFixup(dialogMenuItem *self)
 	for (i = 0; devs[i]; i++) {
 	    Disk *disk = (Disk *)devs[i]->private;
 	    Chunk *c1;
-
+	    
 	    if (!devs[i]->enabled)
 		continue;
 	    if (!disk->chunks)
@@ -767,25 +791,52 @@ installFixup(dialogMenuItem *self)
 		}
 	    }
 	}
-	/* XXX Do all the last ugly work-arounds here which we'll try and excise someday right?? XXX */
-
-	msgNotify("Fixing permissions..");
-	/* BOGON #1:  XFree86 extracting /usr/X11R6 with root-only perms */
-	if (directory_exists("/usr/X11R6")) {
-	    vsystem("chmod -R a+r /usr/X11R6");
-	    vsystem("find /usr/X11R6 -type d | xargs chmod a+x");
-	}
+	
 	/* BOGON #2: We leave /etc in a bad state */
 	chmod("/etc", 0755);
-
+	
 	/* BOGON #3: No /var/db/mountdtab complains */
 	Mkdir("/var/db");
 	creat("/var/db/mountdtab", 0644);
+	
+	/* BOGON #4: /compat created by default in root fs */
+	Mkdir("/usr/compat");
+	vsystem("ln -s /usr/compat /compat");
+	
+	/* BOGON #5: aliases database not build for bin */
+	vsystem("newaliases");
+
+	/* BOGON #6: deal with new boot files */
+	vsystem("touch /kernel.config");
+	vsystem("touch /boot.config");
+	if (file_readable("/stand/boot.help") && !file_readable("/boot.help"))
+	    vsystem("mv /stand/boot.help /");
 
 	/* Now run all the mtree stuff to fix things up */
         vsystem("mtree -deU -f /etc/mtree/BSD.root.dist -p /");
         vsystem("mtree -deU -f /etc/mtree/BSD.var.dist -p /var");
         vsystem("mtree -deU -f /etc/mtree/BSD.usr.dist -p /usr");
+
+	/* Do all the last ugly work-arounds here */
+    }
+    return DITEM_SUCCESS;
+}
+
+/* Fix side-effects from the the XFree86 installation */
+int
+installFixupXFree(dialogMenuItem *self)
+{
+    /* BOGON #1:  XFree86 requires various specialized fixups */
+    if (directory_exists("/usr/X11R6")) {
+	msgNotify("Fixing permissions in XFree86 tree..");
+	vsystem("chmod -R a+r /usr/X11R6");
+	vsystem("find /usr/X11R6 -type d | xargs chmod a+x");
+
+	/* Also do bogus minimal package registration so ports don't whine */
+	if (file_readable("/usr/X11R6/lib/X11/pkgreg.tar.gz")) {
+	    msgNotify("Installing package metainfo..");
+	    vsystem("tar xpzf /usr/X11R6/lib/X11/pkgreg.tar.gz -C / && rm /usr/X11R6/lib/X11/pkgreg.tar.gz");
+	}
     }
     return DITEM_SUCCESS;
 }
@@ -799,16 +850,15 @@ installFilesystems(dialogMenuItem *self)
     Chunk *c1, *c2, *rootdev, *swapdev, *usrdev, *vardev;
     Device **devs;
     PartInfo *root;
-    char dname[80], *str;
+    char dname[80];
     extern int MakeDevChunk(Chunk *c, char *n);
     Boolean upgrade = FALSE;
 
     /* If we've already done this, bail out */
-    if ((str = variable_get(DISK_LABELLED)) && !strcmp(str, "written"))
+    if (!variable_cmp(DISK_LABELLED, "written"))
 	return DITEM_SUCCESS;
 
-    str = variable_get(SYSTEM_STATE);
-
+    upgrade = !variable_cmp(SYSTEM_STATE, "upgrade");
     if (!checkLabels(TRUE, &rootdev, &swapdev, &usrdev, &vardev))
 	return DITEM_FAILURE;
 
@@ -818,8 +868,6 @@ installFilesystems(dialogMenuItem *self)
 	root = NULL;
 
     command_clear();
-    upgrade = str && !strcmp(str, "upgrade");
-
     if (swapdev && RunningAsInit) {
 	/* As the very first thing, try to get ourselves some swap space */
 	sprintf(dname, "/dev/%s", swapdev->name);
@@ -841,7 +889,7 @@ installFilesystems(dialogMenuItem *self)
 
     if (rootdev && RunningAsInit) {
 	/* Next, create and/or mount the root device */
-	sprintf(dname, "/dev/r%sa", rootdev->disk->name);
+	sprintf(dname, "/dev/r%s", rootdev->name);
 	if (!Fake && (!MakeDevChunk(rootdev, "/dev") || !file_readable(dname))) {
 	    msgConfirm("Unable to make device node for %s in /dev!\n"
 		       "The creation of filesystems will be aborted.", dname);
@@ -850,7 +898,7 @@ installFilesystems(dialogMenuItem *self)
 	if (strcmp(root->mountpoint, "/"))
 	    msgConfirm("Warning: %s is marked as a root partition but is mounted on %s", rootdev->name, root->mountpoint);
 
-	if (root->newfs) {
+	if (root->newfs && (!upgrade || !msgYesNo("You are upgrading - are you SURE you want to newfs the root partition?"))) {
 	    int i;
 
 	    msgNotify("Making a new root filesystem on %s", dname);
@@ -874,7 +922,7 @@ installFilesystems(dialogMenuItem *self)
 	}
 
 	/* Switch to block device */
-	sprintf(dname, "/dev/%sa", rootdev->disk->name);
+	sprintf(dname, "/dev/%s", rootdev->name);
 	if (Mount("/mnt", dname)) {
 	    msgConfirm("Unable to mount the root file system on %s!  Giving up.", dname);
 	    return DITEM_FAILURE;
@@ -910,7 +958,7 @@ installFilesystems(dialogMenuItem *self)
 			if (c2 == rootdev)
 			    continue;
 
-			if (tmp->newfs)
+			if (tmp->newfs && (!upgrade || !msgYesNo("You are upgrading - are you SURE you want to newfs /dev/%s?", c2->name)))
 			    command_shell_add(tmp->mountpoint, "%s %s/dev/r%s", tmp->newfs_cmd, RunningAsInit ? "/mnt" : "", c2->name);
 			else
 			    command_shell_add(tmp->mountpoint, "fsck -y %s/dev/r%s", RunningAsInit ? "/mnt" : "", c2->name);
@@ -954,6 +1002,20 @@ installFilesystems(dialogMenuItem *self)
     return DITEM_SUCCESS;
 }
 
+static char *
+getRelname(void)
+{
+    static char buf[64];
+    int sz = (sizeof buf) - 1;
+
+    if (sysctlbyname("kern.osrelease", buf, &sz, NULL, 0) != -1) {
+	buf[sz] = '\0';
+	return buf;
+    }
+    else
+	return "<unknown>";
+}
+
 /* Initialize various user-settable values to their defaults */
 int
 installVarDefaults(dialogMenuItem *self)
@@ -961,25 +1023,23 @@ installVarDefaults(dialogMenuItem *self)
     char *cp;
 
     /* Set default startup options */
-    variable_set2(VAR_ROUTER,			"NO");
-    variable_set2(VAR_RELNAME,			RELEASE_NAME);
+    variable_set2(VAR_RELNAME,			getRelname());
     variable_set2(VAR_CPIO_VERBOSITY,		"high");
     variable_set2(VAR_TAPE_BLOCKSIZE,		DEFAULT_TAPE_BLOCKSIZE);
     variable_set2(VAR_INSTALL_ROOT,		"/");
+    variable_set2(VAR_INSTALL_CFG,		"install.cfg");
     cp = getenv("EDITOR");
     if (!cp)
 	cp = "/usr/bin/ee";
     variable_set2(VAR_EDITOR,			cp);
     variable_set2(VAR_FTP_USER,			"ftp");
-    variable_set2(VAR_BROWSER_PACKAGE,		PACKAGE_LYNX);
+    variable_set2(VAR_BROWSER_PACKAGE,		"lynx");
     variable_set2(VAR_BROWSER_BINARY,		"/usr/local/bin/lynx");
     variable_set2(VAR_FTP_STATE,		"passive");
     variable_set2(VAR_NFS_SECURE,		"YES");
     variable_set2(VAR_PKG_TMPDIR,		"/usr/tmp");
-    variable_set2(VAR_APACHE_PKG,		PACKAGE_APACHE);
-    variable_set2(VAR_SAMBA_PKG,		PACKAGE_SAMBA);
-    variable_set2(VAR_GATED_PKG,		PACKAGE_GATED);
-    variable_set2(VAR_PCNFSD_PKG,		PACKAGE_PCNFSD);
+    variable_set2(VAR_GATED_PKG,		"gated");
+    variable_set2(VAR_PCNFSD_PKG,		"pcnfsd");
     variable_set2(VAR_MEDIA_TIMEOUT,		itoa(MEDIA_TIMEOUT));
     if (getpid() != 1)
 	variable_set2(SYSTEM_STATE,		"update");
@@ -992,8 +1052,8 @@ installVarDefaults(dialogMenuItem *self)
 void
 installEnvironment(void)
 {
-    if (file_readable("/etc/sysconfig"))
-	configEnvironmentSysconfig("/etc/sysconfig");
+    if (file_readable("/etc/rc.conf"))
+	configEnvironmentRC_conf("/etc/rc.conf");
     if (file_readable("/etc/resolv.conf"))
 	configEnvironmentResolv("/etc/resolv.conf");
 }
@@ -1004,6 +1064,8 @@ copySelf(void)
 {
     int i;
 
+    if (file_readable("/boot.help"))
+	vsystem("cp /boot.help /mnt");
     msgWeHaveOutput("Copying the boot floppy to /stand on root filesystem");
     i = vsystem("find -x /stand | cpio %s -pdum /mnt", cpioVerbosity());
     if (i) {
