@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated for what's essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/main.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/main.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -67,7 +67,9 @@ main(int argc, char **argv)
 
     /* Set default flag and variable values */
     installVarDefaults(NULL);
-    installEnvironment();
+    /* only when multi-user is it reasonable to do this here */
+    if (!RunningAsInit)
+	installEnvironment();
 
     if (argc > 1 && !strcmp(argv[1], "-fake")) {
 	variable_set2(VAR_DEBUG, "YES");
@@ -104,30 +106,15 @@ main(int argc, char **argv)
 	if (argc > start_arg)
 	    systemShutdown(0);
     }
-    else {
-	FILE *fp;
-	char buf[BUFSIZ];
-
-	fp = fopen("install.cfg", "r");
-	if (fp) {
-	    msgNotify("Loading pre-configuration file");
-	    while (fgets(buf, sizeof buf, fp)) {
-		if (DITEM_STATUS(dispatchCommand(buf)) != DITEM_SUCCESS) {
-		    msgDebug("Command `%s' failed - rest of script aborted.\n", buf);
-		    break;
-		}
-	    }
-	    fclose(fp);
-	}
-    }
+    else
+	dispatch_load_file_int(TRUE);
 
     status = setjmp(BailOut);
     if (status) {
 	msgConfirm("A signal %d was caught - I'm saving what I can and shutting\n"
-		   "this thing down!  Please report this unfortunate incident\n"
-		   "to jkh@FreeBSD.org.  If you can reproduce the problem, please\n"
-		   "also turn Debug on in the Options menu for the extra information\n"
-		   "it provides in debugging problems like this.  Thanks!", status);
+		   "If you can reproduce the problem, please turn Debug on in\n"
+		   "the Options menu for the extra information it provides in\n"
+		   "debugging problems like this.", status);
 	systemShutdown(status);
     }
 
