@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/media.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/media.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -46,6 +46,7 @@
 #include <sys/wait.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <resolv.h>
 
 static Boolean got_intr = FALSE;
 
@@ -84,6 +85,28 @@ cdromHook(dialogMenuItem *self)
     return genericHook(self, DEVICE_TYPE_CDROM);
 }
 
+static void
+kickstart_dns(void)
+{
+    static Boolean initted = FALSE;
+    int time;
+    char *cp;
+
+    cp = variable_get(VAR_MEDIA_TIMEOUT);
+    if (!cp)
+	time = MEDIA_TIMEOUT;
+    else
+	time = atoi(cp);
+    if (!time)
+	time = 100;
+    if (!initted) {
+	res_init();
+	_res.retry = 2;	/* 2 times seems a reasonable number to me */
+	_res.retrans = time / 2; /* so spend half our alloted time on each try */
+	initted = TRUE;
+    }
+}
+
 char *
 cpioVerbosity()
 {
@@ -181,6 +204,8 @@ mediaSetFloppy(dialogMenuItem *self)
     }
     else
 	mediaDevice = devs[0];
+    if (mediaDevice)
+	mediaDevice->private = "/dist";
     return (mediaDevice ? DITEM_LEAVE_MENU : DITEM_FAILURE) | DITEM_RESTORE;
 }
 
@@ -295,8 +320,9 @@ mediaSetFTP(dialogMenuItem *self)
     mediaClose();
     cp = variable_get(VAR_FTP_PATH);
     /* If we've been through here before ... */
-    if (networkDev && cp && msgYesNo("Re-use old FTP site selection values?"))
-	cp = NULL;
+    if (!variable_get(VAR_NONINTERACTIVE))
+	if (networkDev && cp && msgYesNo("Re-use old FTP site selection values?"))
+	    cp = NULL;
 
     if (!cp) {
 	dialog_clear_norefresh();
@@ -361,17 +387,28 @@ mediaSetFTP(dialogMenuItem *self)
 	msgDebug("port # = `%d'\n", FtpPort);
     }
     if (variable_get(VAR_NAMESERVER)) {
-	if ((inet_addr(hostname) == INADDR_NONE) && (gethostbyname(hostname) == NULL)) {
-	    msgConfirm("Cannot resolve hostname `%s'!  Are you sure that your\n"
-		       "name server, gateway and network interface are correctly configured?", hostname);
-	    if (networkDev)
-		networkDev->shutdown(networkDev);
-	    networkDev = NULL;
-	    variable_unset(VAR_FTP_PATH);
-	    return DITEM_FAILURE | what;
+	msgNotify("Looking up host %s.", hostname);
+    	if (isDebug())
+	    msgDebug("Starting DNS.\n");
+	kickstart_dns();
+    	if (isDebug())
+	    msgDebug("Looking up hostname, %s, using inet_addr().\n", hostname);
+	if (inet_addr(hostname) == INADDR_NONE) {
+    	    if (isDebug())
+		msgDebug("Looking up hostname, %s, using gethostbyname().\n",
+			hostname);
+	    if (gethostbyname(hostname) == NULL) {
+		msgConfirm("Cannot resolve hostname `%s'!  Are you sure that"
+			" your\nname server, gateway and network interface are"
+			" correctly configured?", hostname);
+		if (networkDev)
+		    networkDev->shutdown(networkDev);
+		networkDev = NULL;
+		variable_unset(VAR_FTP_PATH);
+		return DITEM_FAILURE | what;
+	    }
 	}
-	else
-	    msgDebug("Found DNS entry for %s successfully.\n", hostname);
+	msgDebug("Found DNS entry for %s successfully..\n", hostname);
     }
     variable_set2(VAR_FTP_HOST, hostname);
     variable_set2(VAR_FTP_DIR, dir ? dir : "/");
@@ -455,6 +492,7 @@ mediaSetNFS(dialogMenuItem *self)
 	    msgDebug("mediaSetNFS: Net device init failed\n");
     }
     if (variable_get(VAR_NAMESERVER)) {
+	kickstart_dns();
 	if ((inet_addr(hostname) == INADDR_NONE) && (gethostbyname(hostname) == NULL)) {
 	    msgConfirm("Cannot resolve hostname `%s'!  Are you sure that your\n"
 		       "name server, gateway and network interface are correctly configured?", hostname);
@@ -465,7 +503,7 @@ mediaSetNFS(dialogMenuItem *self)
 	    return DITEM_FAILURE;
 	}
 	else
-	    msgDebug("Found DNS entry for %s successfully.\n", hostname);
+	    msgDebug("Found DNS entry for %s successfully..", hostname);
     }
     variable_set2(VAR_NFS_HOST, hostname);
     nfsDevice.type = DEVICE_TYPE_NFS;
