@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/network.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/network.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -58,9 +58,15 @@ mediaInitNetwork(Device *dev)
     if (!RunningAsInit || networkInitialized)
 	return TRUE;
 
-    msgDebug("Init routine called for network device %s.\n", dev->name);
-    if (!file_readable("/etc/resolv.conf"))
-	configResolv();
+    if (isDebug())
+	msgDebug("Init routine called for network device %s.\n", dev->name);
+
+    if (!file_readable("/etc/resolv.conf")) {
+	if (DITEM_STATUS(configResolv(NULL)) == DITEM_FAILURE) {
+	    msgConfirm("Can't seem to write out /etc/resolv.conf.  Net cannot be used.");
+	    return FALSE;
+	}
+    }
 
     /* Old PPP process lying around? */
     if (pppPID) {
@@ -132,7 +138,7 @@ mediaInitNetwork(Device *dev)
     }
     else {
 	msgNotify("Adding default route to %s.", rp);
-	vsystem("route add default %s", rp);
+	vsystem("route -n add default %s", rp);
     }
     if (isDebug())
 	msgDebug("Network initialized successfully.\n");
@@ -165,7 +171,7 @@ mediaShutdownNetwork(Device *dev)
 	cp = variable_get(VAR_GATEWAY);
 	if (cp) {
 	    msgNotify("Deleting default route.");
-	    vsystem("route delete default");
+	    vsystem("route -n delete default");
 	}
     }
     else if (pppPID) {
@@ -188,6 +194,7 @@ startPPP(Device *devp)
 
     /* These are needed to make ppp work */
     Mkdir("/var/log");
+    Mkdir("/var/run");
     Mkdir("/var/spool/lock");
     Mkdir("/etc/ppp");
 
@@ -247,6 +254,8 @@ startPPP(Device *devp)
     fprintf(fp, " set device %s\n", devp->devname);
     fprintf(fp, " set ifaddr %s %s\n", myaddr, provider);
     fprintf(fp, " set timeout 0\n");
+    fprintf(fp, " enable dns\n");
+    fprintf(fp, " set log local phase\n");
     fclose(fp);
 
     if (!Fake && !file_readable("/dev/tun0") && mknod("/dev/tun0", 0600 | S_IFCHR, makedev(52, 0))) {
