@@ -4,7 +4,7 @@
  * This is probably the last attempt in the `sysinstall' line, the next
  * generation being slated to essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/sysinstall.h 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/sysinstall.h 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -39,6 +39,7 @@
 
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <errno.h>
 #include <setjmp.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,18 +51,9 @@
 #include "colors.h"
 #include "libdisk.h"
 #include "dist.h"
-#include "version.h"
 
 /*** Defines ***/
 
-/* Different packages we depend on - update this when package version change! */
-#define PACKAGE_GATED	"gated-3.5b3"
-#define PACKAGE_APACHE	"apache-1.2b8"
-#define PACKAGE_NETCON	"commerce/netcon/bsd61"
-#define PACKAGE_PCNFSD	"pcnfsd-93.02.16"
-#define PACKAGE_SAMBA	"samba-1.9.16p11"
-#define PACKAGE_LYNX	"lynx-2.7.1"
-
 /* device limits */
 #define DEV_NAME_MAX		64	/* The maximum length of a device name	*/
 #define DEV_MAX			100	/* The maximum number of devices we'll deal with */
@@ -90,7 +82,6 @@
 #define TCP_CONFIGURED			"_tcpConfigured"
 
 /* Ones that can be tweaked from config files */
-#define VAR_APACHE_PKG			"apache_pkg"
 #define VAR_BLANKTIME			"blanktime"
 #define VAR_BOOTMGR			"bootManager"
 #define VAR_BROWSER_BINARY		"browserBinary"
@@ -98,9 +89,19 @@
 #define VAR_CPIO_VERBOSITY		"cpioVerbose"
 #define VAR_DEBUG			"debug"
 #define VAR_DISK			"disk"
+#define VAR_DISTS			"dists"
+#define VAR_DIST_MAIN			"distMain"
+#define VAR_DIST_DES			"distDES"
+#define VAR_DIST_SRC			"distSRC"
+#define VAR_DIST_X11			"distX11"
+#define VAR_DIST_XSERVER		"distXserver"
+#define VAR_DIST_XFONTS			"distXfonts"
+#define VAR_DEDICATE_DISK		"dedicateDisk"
 #define VAR_DOMAINNAME			"domainname"
 #define VAR_EDITOR			"editor"
 #define VAR_EXTRAS			"ifconfig_"
+#define VAR_COMMAND			"command"
+#define VAR_CONFIG_FILE			"configFile"
 #define VAR_FTP_DIR			"ftpDirectory"
 #define VAR_FTP_PASS			"ftpPass"
 #define VAR_FTP_PATH			"ftp"
@@ -117,19 +118,31 @@
 #define VAR_INSTALL_CFG			"installConfig"
 #define VAR_INSTALL_ROOT		"installRoot"
 #define VAR_IPADDR			"ipaddr"
+#define VAR_KEYMAP			"keymap"
 #define VAR_LABEL			"label"
 #define VAR_LABEL_COUNT			"labelCount"
 #define VAR_MEDIA_TYPE			"mediaType"
 #define VAR_MEDIA_TIMEOUT		"MEDIA_TIMEOUT"
+#define VAR_MOUSED			"moused_enable"
+#define VAR_MOUSED_PORT			"moused_port"
+#define VAR_MOUSED_TYPE			"moused_type"
 #define VAR_NAMESERVER			"nameserver"
+#define VAR_NETINTERACTIVE		"netInteractive"
 #define VAR_NETMASK			"netmask"
+#define VAR_NETWORK_DEVICE		"netDev"
 #define VAR_NFS_PATH			"nfs"
 #define VAR_NFS_HOST			"nfsHost"
 #define VAR_NFS_SECURE			"nfsSecure"
-#define VAR_NFS_SERVER			"nfs_server"
+#define VAR_NFS_SERVER			"nfs_server_enable"
 #define VAR_NO_CONFIRM			"noConfirm"
+#define VAR_NO_ERROR			"noError"
+#define VAR_NO_WARN			"noWarn"
+#define VAR_NO_USR			"noUsr"
+#define VAR_NONINTERACTIVE		"nonInteractive"
 #define VAR_NOVELL			"novell"
-#define VAR_NTPDATE			"ntpdate"
+#define VAR_NTPDATE_FLAGS		"ntpdate_flags"
+#define VAR_PACKAGE			"package"
+#define VAR_PARTITION			"partition"
 #define VAR_PCNFSD			"pcnfsd"
 #define VAR_PCNFSD_PKG			"pcnfsd_pkg"
 #define VAR_PKG_TMPDIR			"PKG_TMPDIR"
@@ -137,8 +150,8 @@
 #define VAR_RELNAME			"releaseName"
 #define VAR_ROOT_SIZE			"rootSize"
 #define VAR_ROUTER			"router"
+#define VAR_ROUTER_ENABLE		"router_enable"
 #define VAR_ROUTERFLAGS			"routerflags"
-#define VAR_SAMBA_PKG			"samba_pkg"
 #define VAR_SERIAL_SPEED		"serialSpeed"
 #define VAR_SLOW_ETHER			"slowEthernetCard"
 #define VAR_SWAP_SIZE			"swapSize"
@@ -180,7 +193,7 @@ typedef struct _dmenu {
     dialogMenuItem items[0];		/* Array of menu items		*/
 } DMenu;
 
-/* A sysconfig variable */
+/* An rc.conf variable */
 typedef struct _variable {
     struct _variable *next;
     char *name;
@@ -351,6 +364,7 @@ extern DMenu		MenuMediaFTP;		/* FTP media menu				*/
 extern DMenu		MenuMediaTape;		/* Tape media menu				*/
 extern DMenu		MenuNetworkDevice;	/* Network device menu				*/
 extern DMenu		MenuNTP;		/* NTP time server menu				*/
+extern DMenu		MenuStartup;		/* Startup services menu			*/
 extern DMenu		MenuSyscons;		/* System console configuration menu		*/
 extern DMenu		MenuSysconsFont;	/* System console font configuration menu	*/
 extern DMenu		MenuSysconsKeymap;	/* System console keymap configuration menu	*/
@@ -381,9 +395,6 @@ extern void display_helpline(WINDOW *w, int y, int width);
 
 /*** Prototypes ***/
 
-/* apache.c */
-extern int	configApache(dialogMenuItem *self);
-
 /* anonFTP.c */
 extern int	configAnonFTP(dialogMenuItem *self);
 
@@ -405,26 +416,23 @@ extern void	command_shell_add(char *key, char *fmt, ...);
 extern void	command_func_add(char *key, commandFunc func, void *data);
 
 /* config.c */
-extern int	configFstab(void);
-extern void	configEnvironmentSysconfig(char *config);
+extern void	configEnvironmentRC_conf(char *config);
 extern void	configEnvironmentResolv(char *config);
-extern void	configSysconfig(char *config);
+extern void	configRC_conf(char *config);
+extern int	configFstab(dialogMenuItem *self);
+extern int	configRC(dialogMenuItem *self);
 extern int	configRegister(dialogMenuItem *self);
-extern void	configResolv(void);
+extern int	configResolv(dialogMenuItem *self);
 extern int	configPackages(dialogMenuItem *self);
 extern int	configSaver(dialogMenuItem *self);
 extern int	configSaverTimeout(dialogMenuItem *self);
 extern int	configNTP(dialogMenuItem *self);
 extern int	configUsers(dialogMenuItem *self);
-extern int	configXFree86(dialogMenuItem *self);
+extern int	configXEnvironment(dialogMenuItem *self);
 extern int	configRouter(dialogMenuItem *self);
-extern int	configSamba(dialogMenuItem *self);
 extern int	configPCNFSD(dialogMenuItem *self);
 extern int	configNFSServer(dialogMenuItem *self);
-extern int	configWriteSysconfig(dialogMenuItem *self);
-#ifdef NETCON_EXTENTIONS
-extern int	configNovell(dialogMenuItem *self);
-#endif
+extern int	configWriteRC_conf(dialogMenuItem *self);
 
 /* crc.c */
 extern int	crc(int, unsigned long *, unsigned long *);
@@ -449,14 +457,20 @@ extern void	dummyShutdown(Device *dev);
 /* disks.c */
 extern int	diskPartitionEditor(dialogMenuItem *self);
 extern int	diskPartitionWrite(dialogMenuItem *self);
-extern void	diskPartition(Device *dev, Disk *d);
+extern int	diskGetSelectCount(Device ***devs);
+extern void	diskPartition(Device *dev);
 
 /* dispatch.c */
 extern int	dispatchCommand(char *command);
+extern int	dispatch_load_floppy(dialogMenuItem *self);
+extern int	dispatch_load_file_int(int);
+extern int	dispatch_load_file(dialogMenuItem *self);
+
 
 /* dist.c */
 extern int	distReset(dialogMenuItem *self);
-extern int	distSetCustom(char *str);
+extern int	distConfig(dialogMenuItem *self);
+extern int	distSetCustom(dialogMenuItem *self);
 extern int	distSetDeveloper(dialogMenuItem *self);
 extern int	distSetXDeveloper(dialogMenuItem *self);
 extern int	distSetKernDeveloper(dialogMenuItem *self);
@@ -475,6 +489,7 @@ extern int	dmenuSubmenu(dialogMenuItem *tmp);
 extern int	dmenuSystemCommand(dialogMenuItem *tmp);
 extern int	dmenuSystemCommandBox(dialogMenuItem *tmp);
 extern int	dmenuExit(dialogMenuItem *tmp);
+extern int	dmenuISetVariable(dialogMenuItem *tmp);
 extern int	dmenuSetVariable(dialogMenuItem *tmp);
 extern int	dmenuSetKmapVariable(dialogMenuItem *tmp);
 extern int	dmenuSetVariables(dialogMenuItem *tmp);
@@ -484,6 +499,7 @@ extern int	dmenuSetValue(dialogMenuItem *tmp);
 extern Boolean	dmenuOpen(DMenu *menu, int *choice, int *scroll, int *curr, int *max, Boolean buttons);
 extern Boolean	dmenuOpenSimple(DMenu *menu, Boolean buttons);
 extern int	dmenuVarCheck(dialogMenuItem *item);
+extern int	dmenuVarsCheck(dialogMenuItem *item);
 extern int	dmenuFlagCheck(dialogMenuItem *item);
 extern int	dmenuRadioCheck(dialogMenuItem *item);
 
@@ -530,7 +546,8 @@ extern int	installNovice(dialogMenuItem *self);
 extern int	installFixitHoloShell(dialogMenuItem *self);
 extern int	installFixitCDROM(dialogMenuItem *self);
 extern int	installFixitFloppy(dialogMenuItem *self);
-extern int	installFixup(dialogMenuItem *self);
+extern int	installFixupBin(dialogMenuItem *self);
+extern int	installFixupXFree(dialogMenuItem *self);
 extern int	installUpgrade(dialogMenuItem *self);
 extern int	installFilesystems(dialogMenuItem *self);
 extern int	installVarDefaults(dialogMenuItem *self);
@@ -548,6 +565,7 @@ extern int	diskLabelCommit(dialogMenuItem *self);
 extern int	lndir(char *from, char *to);
 
 /* makedevs.c (auto-generated) */
+extern const char	termcap_ansi[];
 extern const char	termcap_vt100[];
 extern const char	termcap_cons25[];
 extern const char	termcap_cons25_m[];
@@ -614,6 +632,10 @@ extern WINDOW	*savescr(void);
 extern void	restorescr(WINDOW *w);
 extern char	*sstrncpy(char *dst, const char *src, int size);
 
+/* mouse.c */
+extern int	mousedTest(dialogMenuItem *self);
+extern int	mousedDisable(dialogMenuItem *self);
+
 /* msg.c */
 extern Boolean	isDebug(void);
 extern void	msgInfo(char *fmt, ...);
@@ -643,6 +665,7 @@ extern void	mediaShutdownNFS(Device *dev);
 extern int	optionsEditor(dialogMenuItem *self);
 
 /* package.c */
+extern int	packageAdd(dialogMenuItem *self);
 extern int	package_add(char *name);
 extern int	package_extract(Device *dev, char *name, Boolean depended);
 extern Boolean	package_exists(char *name);
@@ -691,11 +714,11 @@ extern int	userAddUser(dialogMenuItem *self);
 extern void	variable_set(char *var);
 extern void	variable_set2(char *name, char *value);
 extern char 	*variable_get(char *var);
+extern int 	variable_cmp(char *var, char *value);
 extern void	variable_unset(char *var);
 extern char	*variable_get_value(char *var, char *prompt);
-
-/* variable_load.c */
-extern int	variableLoad(dialogMenuItem *self);
+extern int 	variable_check(char *data);
+extern int	dump_variables(dialogMenuItem *self);
 
 /* wizard.c */
 extern void	slice_wizard(Disk *d);
