@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/system.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/system.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Jordan Hubbard
  *
@@ -51,9 +51,11 @@ handle_intr(int sig)
 static char *
 expand(char *fname)
 {
+    char *gunzip = RunningAsInit ? "/stand/gunzip" : "/usr/bin/gunzip";
+
     Mkdir(DOC_TMP_DIR);
     unlink(DOC_TMP_FILE);
-    if (!file_readable(fname) || vsystem("gzip -c -d %s > %s", fname, DOC_TMP_FILE))
+    if (!file_readable(fname) || vsystem("%s < %s > %s", gunzip, fname, DOC_TMP_FILE))
 	return NULL;
     return DOC_TMP_FILE;
 }
@@ -69,16 +71,37 @@ systemInitialize(int argc, char **argv)
 
     /* Are we running as init? */
     if (getpid() == 1) {
+	int fd, type;
+
+	RunningAsInit = 1;
 	setsid();
 	close(0);
-	if (open("/dev/ttyv0", O_RDWR) < 0)
-	    open("/dev/console", O_RDWR);
+	fd = open("/dev/ttyv0", O_RDWR);
+	if (fd == -1)
+	    fd = open("/dev/console", O_RDWR);	/* fallback */
 	else
 	    OnVTY = TRUE;
+	/*
+	 * To make _sure_ we're on a VTY and don't have /dev/console switched
+	 * away to a serial port or something, attempt to set the cursor appearance.
+	 */
+	type = 0;	/* normal */
+	if (OnVTY) {
+	    int fd2;
+
+	    if ((fd2 = open("/dev/console", O_RDWR)) != -1) {
+		if (ioctl(fd2, CONS_CURSORTYPE, &type) == -1) {
+		    OnVTY = FALSE;
+		    close(fd); close(fd2);
+		    open("/dev/console", O_RDWR);
+		}
+		else
+		    close(fd2);
+	    }
+	}
 	close(1); dup(0);
 	close(2); dup(0);
-	printf("%s running as init\n", argv[0]);
-	RunningAsInit = 1;
+	printf("%s running as init on %s\n", argv[0], OnVTY ? "vty0" : "serial console");
 	i = ioctl(0, TIOCSCTTY, (char *)NULL);
 	setlogin("root");
 	setenv("PATH", "/stand:/bin:/sbin:/usr/sbin:/usr/bin:/mnt/bin:/mnt/sbin:/mnt/usr/sbin:/mnt/usr/bin:/usr/X11R6/bin", 1);
@@ -118,9 +141,8 @@ systemShutdown(int status)
     if (status >=0 && mediaDevice)
 	mediaDevice->shutdown(mediaDevice);
 
-    /* write out any changes to sysconfig .. */
-    if (!status)
-	configSysconfig("/etc/sysconfig");
+    /* write out any changes to rc.conf .. */
+    configRC_conf("/etc/rc.conf");
 
     /* Shut down the dialog library */
     if (DialogActive) {
@@ -199,9 +221,15 @@ systemHelpFile(char *file, char *buf)
 	return NULL;
 
     snprintf(buf, FILENAME_MAX, "/stand/help/%s.hlp.gz", file);
+    if (file_readable(buf)) 
+	return expand(buf);
+    snprintf(buf, FILENAME_MAX, "/stand/help/%s.TXT.gz", file);
     if (file_readable(buf)) 
 	return expand(buf);
     snprintf(buf, FILENAME_MAX, "/usr/src/release/sysinstall/help/%s.hlp", file);
+    if (file_readable(buf))
+	return buf;
+    snprintf(buf, FILENAME_MAX, "/usr/src/release/sysinstall/help/%s.TXT", file);
     if (file_readable(buf))
 	return buf;
     return NULL;
