@@ -1,5 +1,5 @@
 /*
- * $FreeBSD: stable/2.1/release/sysinstall/tcpip.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/tcpip.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *      Gary J Palmer. All rights reserved.
@@ -132,7 +132,7 @@ verifySettings(void)
 {
     if (!hostname[0])
 	feepout("Must specify a host name of some sort!");
-    else if (gateway[0] && !verifyIP(gateway))
+    else if (gateway[0] && strcmp(gateway, "NO") && !verifyIP(gateway))
 	feepout("Invalid gateway IP address specified");
     else if (nameserver[0] && !verifyIP(nameserver))
 	feepout("Invalid name server IP address specified");
@@ -149,36 +149,13 @@ verifySettings(void)
 int
 tcpOpenDialog(Device *devp)
 {
-    WINDOW              *ds_win, *save;
+    WINDOW              *ds_win, *save = NULL;
     ComposeObj          *obj = NULL;
     int                 n = 0, cancel = FALSE;
-    int			max, ret;
+    int			max, ret = DITEM_SUCCESS;
     char                *tmp;
     char		title[80];
 
-    if (!RunningAsInit) {
-	if (!msgYesNo("Running multi-user, assume that the network is already configured?"))
-	    return DITEM_SUCCESS;
-    }
-    save = savescr();
-    dialog_clear_norefresh();
-
-    /* We need a curses window */
-    if (!(ds_win = openLayoutDialog(TCP_HELPFILE, " Network Configuration ",
-				    TCP_DIALOG_X, TCP_DIALOG_Y, TCP_DIALOG_WIDTH, TCP_DIALOG_HEIGHT))) {
-	beep();
-	msgConfirm("Cannot open TCP/IP dialog window!!");
-	restorescr(save);
-	return DITEM_FAILURE;
-    }
-
-    /* Draw interface configuration box */
-    draw_box(ds_win, TCP_DIALOG_Y + 9, TCP_DIALOG_X + 8, TCP_DIALOG_HEIGHT - 13, TCP_DIALOG_WIDTH - 17,
-	     dialog_attr, border_attr);
-    wattrset(ds_win, dialog_attr);
-    sprintf(title, " Configuration for Interface %s ", devp->name);
-    mvwaddstr(ds_win, TCP_DIALOG_Y + 9, TCP_DIALOG_X + 14, title);
-
     /* Initialise vars from previous device values */
     if (devp->private) {
 	DevInfo *di = (DevInfo *)devp->private;
@@ -232,6 +209,37 @@ tcpOpenDialog(Device *devp)
     else
 	bzero(nameserver, sizeof(nameserver));
 
+    save = savescr();
+    /* If non-interactive, jump straight over the dialog crap and into config section */
+    if (variable_get(VAR_NONINTERACTIVE) &&
+	!variable_get(VAR_NETINTERACTIVE)) {
+	if (!hostname[0])
+	    msgConfirm("WARNING: hostname variable not set and is a non-optional\n"
+		       "parameter.  Please add this to your installation script\n"
+		       "or set the netInteractive variable (see sysinstall man page)");
+	else
+	    goto netconfig;
+    }
+
+    /* Now do all the screen I/O */
+    dialog_clear_norefresh();
+
+    /* We need a curses window */
+    if (!(ds_win = openLayoutDialog(TCP_HELPFILE, " Network Configuration ",
+				    TCP_DIALOG_X, TCP_DIALOG_Y, TCP_DIALOG_WIDTH, TCP_DIALOG_HEIGHT))) {
+	beep();
+	msgConfirm("Cannot open TCP/IP dialog window!!");
+	restorescr(save);
+	return DITEM_FAILURE;
+    }
+
+    /* Draw interface configuration box */
+    draw_box(ds_win, TCP_DIALOG_Y + 9, TCP_DIALOG_X + 8, TCP_DIALOG_HEIGHT - 13, TCP_DIALOG_WIDTH - 17,
+	     dialog_attr, border_attr);
+    wattrset(ds_win, dialog_attr);
+    sprintf(title, " Configuration for Interface %s ", devp->name);
+    mvwaddstr(ds_win, TCP_DIALOG_Y + 9, TCP_DIALOG_X + 14, title);
+
     /* Some more initialisation before we go into the main input loop */
     obj = initLayoutDialog(ds_win, layout, TCP_DIALOG_X, TCP_DIALOG_Y, &max);
 
@@ -268,6 +276,7 @@ reenter:
        data now if the user hasn't selected cancel.  Save the stuff
        out to the environment via the variable_set() mechanism */
 
+netconfig:
     if (!cancel) {
 	DevInfo *di;
 	char temp[512], ifn[255];
@@ -302,7 +311,7 @@ reenter:
 	}
 	if (ipaddr[0])
 	    variable_set2(VAR_IPADDR, ipaddr);
-	configResolv();	/* XXX this will do it on the MFS copy XXX */
+	configResolv(NULL);	/* XXX this will do it on the MFS copy XXX */
 	ret = DITEM_SUCCESS;
     }
     else
@@ -338,14 +347,27 @@ tcpDeviceSelect(void)
 
     devs = deviceFind(NULL, DEVICE_TYPE_NETWORK);
     cnt = deviceCount(devs);
+    rval = NULL;
+
     if (!cnt) {
 	msgConfirm("No network devices available!");
-	rval = NULL;
+	return NULL;
     }
-    else if (cnt == 1) {
-	if (DITEM_STATUS(tcpOpenDialog(devs[0]) == DITEM_FAILURE))
-	    return NULL;
-	rval = devs[0];
+    else if (!RunningAsInit) {
+	if (!msgYesNo("Running multi-user, assume that the network is already configured?"))
+	    return devs[0];
+    }
+    if (cnt == 1) {
+	if (DITEM_STATUS(tcpOpenDialog(devs[0]) == DITEM_SUCCESS))
+	    rval = devs[0];
+    }
+    else if (variable_get(VAR_NONINTERACTIVE) && variable_get(VAR_NETWORK_DEVICE)) {
+	devs = deviceFind(variable_get(VAR_NETWORK_DEVICE), DEVICE_TYPE_NETWORK);
+	cnt = deviceCount(devs);
+	if (cnt) {
+	    if (DITEM_STATUS(tcpOpenDialog(devs[0]) == DITEM_SUCCESS))
+		rval = devs[0];
+	}
     }
     else {
 	int status;
@@ -355,9 +377,7 @@ tcpDeviceSelect(void)
 	    msgFatal("Unable to create network device menu!  Argh!");
 	status = dmenuOpenSimple(menu, FALSE);
 	free(menu);
-	if (!status)
-	    rval = NULL;
-	else
+	if (status)
 	    rval = NetDev;
     }
     return rval;
