@@ -4,7 +4,7 @@
  * This is probably the last program in the `sysinstall' line - the next
  * generation being essentially a complete rewrite.
  *
- * $FreeBSD: stable/2.1/release/sysinstall/variable.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/release/sysinstall/variable.c 50952 1999-09-05 11:54:50Z peter $
  *
  * Copyright (c) 1995
  *	Jordan Hubbard.  All rights reserved.
@@ -105,22 +105,31 @@ variable_get(char *var)
     return getenv(var);
 }
 
+int
+variable_cmp(char *var, char *value)
+{
+    char *val;
+
+    if ((val = variable_get(var)))
+	return strcmp(val, value);
+    return -1;
+}
+
 void
 variable_unset(char *var)
 {
     Variable *vp;
     char name[512], *cp;
 
-    unsetenv(var);
-    if ((cp = index(var, '=')) != NULL) {
-	sstrncpy(name, cp, cp - var);
-	var = string_skipwhite(string_prune(name));
-    }
-
+    if ((cp = index(var, '=')) != NULL)
+	sstrncpy(name, var, cp - var);
+    else
+	SAFE_STRCPY(name, var);
+    unsetenv(name);
     /* Now search to see if it's in our list, if we have one.. */
     if (!VarHead)
 	return;
-    else if (!VarHead->next && !strcmp(VarHead->name, var)) {
+    else if (!VarHead->next && !strcmp(VarHead->name, name)) {
 	safe_free(VarHead->name);
 	safe_free(VarHead->value);
 	free(VarHead);
@@ -128,12 +137,12 @@ variable_unset(char *var)
     }
     else {
 	for (vp = VarHead; vp; vp = vp->next) {
-	    if (!strcmp(vp->name, var)) {
+	    if (!strcmp(vp->name, name)) {
 		Variable *save = vp->next;
 
+		safe_free(vp->name);
+		safe_free(vp->value);
 		*vp = *save;
-		safe_free(save->name);
-		safe_free(save->value);
 		safe_free(save);
 		break;
 	    }
@@ -146,10 +155,70 @@ char *
 variable_get_value(char *var, char *prompt)
 {
     char *cp;
-    
-    if ((cp = msgGetInput(variable_get(var), prompt)) != NULL)
+
+    cp = variable_get(var);
+    if (cp && variable_get(VAR_NONINTERACTIVE))
+	return cp;
+    else if ((cp = msgGetInput(cp, prompt)) != NULL)
 	variable_set2(var, cp);
     else
 	cp = NULL;
     return cp;
 }
+
+/* Check if value passed in data (in the form "variable=value") is equal to value of 
+   variable stored in env */
+int
+variable_check(char *data)
+{
+    char *cp, *cp2, *cp3, tmp[256];
+
+    if (!data)
+	return FALSE;
+    SAFE_STRCPY(tmp, data);
+    if ((cp = index(tmp, '=')) != NULL) {
+        *(cp++) = '\0';
+	if (*cp == '"') {	/* smash quotes if present */
+	    ++cp;
+	    if ((cp3 = index(cp, '"')) != NULL)
+		*cp3 = '\0';
+	}
+	else if ((cp3 = index(cp, ',')) != NULL)
+	    *cp3 = '\0';
+        cp2 = getenv(tmp);
+        if (cp2) {
+	    if (!*cp)
+		return TRUE;
+	    else
+        	return !strcmp(cp, cp2);
+	}
+        else
+            return FALSE;
+    }
+    else
+        return getenv(tmp) ? TRUE : FALSE;
+} 
+
+int
+dump_variables(dialogMenuItem *unused)
+{
+    FILE *fp;
+    Variable *vp;
+
+    if (isDebug())
+	msgDebug("Writing sysinstall variables to file..");
+
+    fp = fopen("/etc/sysinstall.vars", "w");
+    if (!fp) {
+	msgConfirm("Unable to write to /etc/sysinstall.vars: %s",
+		   strerror(errno));
+	return DITEM_FAILURE;
+    }
+
+    for (vp = VarHead; vp; vp = vp->next)
+	fprintf(fp, "%s=\"%s\"\n", vp->name, vp->value);
+
+    fclose(fp);
+
+    return DITEM_SUCCESS;
+}
