@@ -32,7 +32,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)tape.c	8.2 (Berkeley) 3/17/94";
+static char sccsid[] = "@(#)tape.c	8.4 (Berkeley) 5/1/95";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -45,8 +45,8 @@ static char sccsid[] = "@(#)tape.c	8.2 (Berkeley) 3/17/94";
 #include <ufs/fs.h>
 #include <ufs/inode.h>
 #else
-#include <ufs/ffs/fs.h>
 #include <ufs/ufs/dinode.h>
+#include <ufs/ffs/fs.h>
 #endif
 
 #include <protocols/dumprestore.h>
@@ -138,7 +138,7 @@ alloctape()
 	 * repositioning after stopping, i.e, streaming mode, where the gap is
 	 * variable, 0.30" to 0.45".  The gap is maximal when the tape stops.
 	 */
-	if (blocksperfile == 0)
+	if (blocksperfile == 0 && !unlimited)
 		tenths = writesize / density +
 		    (cartridge ? 16 : density == 625 ? 5 : 8);
 	/*
@@ -302,7 +302,7 @@ flushtape()
 	asize += tenths;
 	blockswritten += ntrec;
 	blocksthisvol += ntrec;
-	if (!pipeout && (blocksperfile ?
+	if (!pipeout && !unlimited && (blocksperfile ?
 	    (blocksthisvol >= blocksperfile) : (asize > tsize))) {
 		close_rewind();
 		startnewtape(0);
@@ -584,10 +584,10 @@ restore_check_point:
 		 * the remaining names for subsequent volumes.
 		 */
 		tapeno++;               /* current tape sequence */
-		if (nexttape || index(tape, ',')) {
+		if (nexttape || strchr(tape, ',')) {
 			if (nexttape && *nexttape)
 				tape = nexttape;
-			if ((p = index(tape, ',')) != NULL) {
+			if ((p = strchr(tape, ',')) != NULL) {
 				*p = '\0';
 				nexttape = p + 1;
 			} else
@@ -648,7 +648,7 @@ dumpabort(signo)
 	Exit(X_ABORT);
 }
 
-__dead void
+void
 Exit(status)
 	int status;
 {
