@@ -40,7 +40,7 @@ static char copyright[] =
 #ifndef lint
 /*static char sccsid[] = "From: @(#)swapon.c	8.1 (Berkeley) 6/5/93";*/
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/sbin/dumpon/dumpon.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/sbin/dumpon/dumpon.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <errno.h>
@@ -68,7 +68,7 @@ main(int argc, char **argv)
 
 	verbose = rv = 0;
 	whoami = argv[0];
-	while ((ch = getopt(argc, argv, "v")) != EOF)
+	while ((ch = getopt(argc, argv, "v")) != -1)
 		switch((char)ch) {
 		case 'v':
 			verbose = 1;
@@ -107,10 +107,10 @@ main(int argc, char **argv)
 
 	if (verbose) {
 		if (stab.st_rdev == NODEV) {
-			printf("%s: crash dumps disabled\n", whoami);
+			printf("dumpon: crash dumps disabled\n");
 		} else {
-			printf("%s: crash dumps to %s (%lu, %lu)\n",
-			       whoami, argv[0],
+			printf("dumpon: crash dumps to %s (%lu, %lu)\n",
+			       argv[0],
 			       (unsigned long)major(stab.st_rdev),
 			       (unsigned long)minor(stab.st_rdev));
 		}
@@ -123,7 +123,7 @@ void
 usage()
 {
 	fprintf(stderr,
-		"usage: %s [-v] special_file\n"
-		"       %s [-v] off\n", whoami, whoami);
+		"usage: dumpon [-v] special_file\n"
+		"       dumpon [-v] off\n");
 	exit(EX_USAGE);
 }
