@@ -26,6 +26,8 @@
  * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $FreeBSD: stable/2.2/sbin/fsdb/fsdb.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -47,6 +49,7 @@ static char rcsid[] = "$NetBSD: fsdb.c,v 1.2 1995/10/08 23:18:10 thorpej Exp $";
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <err.h>
 
 #include <ufs/ufs/dinode.h>
 #include <ufs/ufs/dir.h>
@@ -55,15 +58,14 @@ static char rcsid[] = "$NetBSD: fsdb.c,v 1.2 1995/10/08 23:18:10 thorpej Exp $";
 #include "fsdb.h"
 #include "fsck.h"
 
-extern char *__progname;	/* from crt0.o */
-
-void usage __P((void));
+static void usage __P((void));
 int cmdloop __P((void));
 
-void 
+static void 
 usage()
 {
-	errx(1, "usage: %s [-d] -f <fsname>", __progname);
+	fprintf(stderr, "usage: fsdb [-d] [-f] fsname\n");
+	exit(1);
 }
 
 int returntosingle = 0;
@@ -81,12 +83,13 @@ main(argc, argv)
 {
 	int ch, rval;
 	char *fsys = NULL;
-	struct stat stb;
 
-	while (-1 != (ch = getopt(argc, argv, "f:d"))) {
+	while (-1 != (ch = getopt(argc, argv, "fd"))) {
 		switch (ch) {
 		case 'f':
-			fsys = optarg;
+				/* The -f option is left for historical
+				 * reasons and has no meaning.
+				 */
 			break;
 		case 'd':
 			debug++;
@@ -95,8 +98,13 @@ main(argc, argv)
 			usage();
 		}
 	}
-	if (fsys == NULL)
+	argc -= optind;
+	argv += optind;
+	if (argc != 1)
 		usage();
+	else
+		fsys = argv[0];
+
 	if (!setup(fsys))
 		errx(1, "cannot set up file system `%s'", fsys);
 	printf("Editing file system `%s'\nLast Mounted on %s\n", fsys,
@@ -130,6 +138,7 @@ CMDFUNC(rm);				/* remove name */
 CMDFUNC(ln);				/* add name */
 CMDFUNC(newtype);			/* change type */
 CMDFUNC(chmode);			/* change mode */
+CMDFUNC(chlen);				/* change length */
 CMDFUNC(chaflags);			/* change flags */
 CMDFUNC(chgen);				/* change generation */
 CMDFUNC(chowner);			/* change owner */
@@ -162,6 +171,7 @@ struct cmdtable cmds[] = {
 	{ "chname", "Change dir entry number INDEX to NAME", 3, 3, chname },
 	{ "chtype", "Change type of current inode to TYPE", 2, 2, newtype },
 	{ "chmod", "Change mode of current inode to MODE", 2, 2, chmode },
+	{ "chlen", "Change length of current inode to LENGTH", 2, 2, chlen },
 	{ "chown", "Change owner of current inode to OWNER", 2, 2, chowner },
 	{ "chgrp", "Change group of current inode to GROUP", 2, 2, chgroup },
 	{ "chflags", "Change flags of current inode to FLAGS", 2, 2, chaflags },
@@ -220,7 +230,7 @@ cmdloop()
     hist = history_init();
     history(hist, H_EVENT, 100);	/* 100 elt history buffer */
 
-    elptr = el_init(__progname, stdin, stdout);
+    elptr = el_init("fsdb", stdin, stdout);
     el_set(elptr, EL_EDITOR, "emacs");
     el_set(elptr, EL_PROMPT, prompt);
     el_set(elptr, EL_HIST, history, hist);
@@ -228,7 +238,7 @@ cmdloop()
 
     while ((elline = el_gets(elptr, &scratch)) != NULL && scratch != 0) {
 	if (debug)
-	    printf("command `%s'\n", line);
+	    printf("command `%s'\n", elline);
 
 	history(hist, H_ENTER, elline);
 
@@ -455,7 +465,6 @@ CMDFUNCSTART(focusname)
 CMDFUNCSTART(ln)
 {
     ino_t inum;
-    struct dinode *dp;
     int rval;
     char *cp;
 
@@ -505,7 +514,6 @@ chinumfunc(idesc)
 
 CMDFUNCSTART(chinum)
 {
-    int rval;
     char *cp;
     ino_t inum;
     struct inodesc idesc;
@@ -559,7 +567,6 @@ CMDFUNCSTART(chname)
 {
     int rval;
     char *cp;
-    ino_t inum;
     struct inodesc idesc;
     
     slotcount = 0;
@@ -602,7 +609,6 @@ struct typemap {
 
 CMDFUNCSTART(newtype)
 {
-    int rval = 1;
     int type;
     struct typemap *tp;
 
@@ -630,6 +636,27 @@ CMDFUNCSTART(newtype)
     return 0;
 }
 
+CMDFUNCSTART(chlen)
+{
+    int rval = 1;
+    long len;
+    char *cp;
+
+    if (!checkactive())
+	return 1;
+
+    len = strtol(argv[1], &cp, 0);
+    if (cp == argv[1] || *cp != '\0' || len < 0) { 
+	warnx("bad length `%s'", argv[1]);
+	return 1;
+    }
+    
+    curinode->di_size = len;
+    inodirty();
+    printactive();
+    return rval;
+}
+
 CMDFUNCSTART(chmode)
 {
     int rval = 1;
@@ -832,8 +859,8 @@ badformat:
 	warnx("date/time out of range");
 	return 1;
     }
-    rts->ts_sec = sec;
-    rts->ts_nsec = nsec;
+    rts->tv_sec = sec;
+    rts->tv_nsec = nsec;
     return 0;
 }
 
