@@ -24,13 +24,21 @@
  * the rights to redistribute these changes.
  */
 
-#include <sys/types.h>
+#ifndef lint
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/sbin/i386/fdisk/fdisk.c 50952 1999-09-05 11:54:50Z peter $";
+#endif /* not lint */
+
 #include <sys/disklabel.h>
-#include <stdio.h>
-#include <errno.h>
 #include <sys/stat.h>
-#include <sys/ioctl.h>
+#include <ctype.h>
 #include <fcntl.h>
+#include <err.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 int iotest;
 
@@ -52,7 +60,9 @@ static char lbuf[LBUF];
 
 #define RoundCyl(x) ((((x) + cylsecs - 1) / cylsecs) * cylsecs)
 
-#define SECSIZE 512
+#define MAX_SEC_SIZE 2048	/* maximum section size that is supported */
+#define MIN_SEC_SIZE 512	/* the sector size to start sensing at */
+int secsize = 0;		/* the sensed sector size */
 
 const char *disk;
 const char *disks[] =
@@ -60,18 +70,18 @@ const char *disks[] =
   "/dev/rwd0", "/dev/rsd0", "/dev/rod0", 0
 };
 
-char *name;
-
 struct disklabel disklabel;		/* disk parameters */
 
 int cyls, sectors, heads, cylsecs, disksecs;
 
 struct mboot
 {
-	unsigned char padding[2]; /* force the longs to be long alligned */
+	unsigned char padding[2]; /* force the longs to be long aligned */
 	unsigned char bootinst[DOSPARTOFF];
 	struct	dos_partition parts[4];
 	unsigned short int	signature;
+	/* room to read in MBRs that are bigger then DEV_BSIZE */
+	unsigned char large_sector_overflow[MAX_SEC_SIZE-MIN_SEC_SIZE];
 };
 struct mboot mboot;
 
@@ -85,14 +95,34 @@ int dos_cylsecs;
 
 #define DOSSECT(s,c) ((s & 0x3f) | ((c >> 2) & 0xc0))
 #define DOSCYL(c)	(c & 0xff)
-static int dos();
-char *get_type();
 static int partition = -1;
 
 
+#define MAX_ARGS	10
+
+static int	current_line_number;
+
+static int	geom_processed = 0;
+static int	part_processed = 0;
+static int	active_processed = 0;
+
+
+typedef struct cmd {
+    char		cmd;
+    int			n_args;
+    struct arg {
+	char	argtype;
+	int	arg_val;
+    }			args[MAX_ARGS];
+} CMD;
+
+
 static int a_flag  = 0;		/* set active partition */
 static int i_flag  = 0;		/* replace partition data */
 static int u_flag  = 0;		/* update partition data */
+static int t_flag  = 0;		/* test only, if f_flag is given */
+static char *f_flag = NULL;	/* Read config info from file */
+static int v_flag  = 0;		/* Be verbose */
 
 static unsigned char bootcode[] = {
 0x33, 0xc0, 0xfa, 0x8e, 0xd0, 0xbc, 0x00, 0x7c, 0x8e, 0xc0, 0x8e, 0xd8, 0xfb, 0x8b, 0xf4, 0xbf,
@@ -138,13 +168,17 @@ struct part_type
 	,{0x01, "Primary DOS with 12 bit FAT"}
 	,{0x02, "XENIX / filesystem"}
 	,{0x03, "XENIX /usr filesystem"}
-	,{0x04, "Primary DOS with 16 bit FAT"}
+	,{0x04, "Primary DOS with 16 bit FAT (<= 32MB)"}
 	,{0x05, "Extended DOS"}
 	,{0x06, "Primary 'big' DOS (> 32MB)"}
-	,{0x07, "OS/2 HPFS, QNX or Advanced UNIX"}
+	,{0x07, "OS/2 HPFS, NTFS, QNX or Advanced UNIX"}
 	,{0x08, "AIX filesystem"}
 	,{0x09, "AIX boot partition or Coherent"}
 	,{0x0A, "OS/2 Boot Manager or OPUS"}
+	,{0x0B, "DOS or Windows 95 with 32 bit FAT"}
+	,{0x0C, "DOS or Windows 95 with 32 bit FAT, LBA"}
+	,{0x0E, "Primary 'big' DOS (> 32MB, LBA)"}
+	,{0x0F, "Extended DOS, LBA"}
 	,{0x10, "OPUS"}
 	,{0x40, "VENIX 286"}
 	,{0x50, "DM"}
@@ -163,6 +197,7 @@ struct part_type
 	,{0x93, "Amoeba filesystem"}
 	,{0x94, "Amoeba bad block table"}
 	,{0xA5, "FreeBSD/NetBSD/386BSD"}
+	,{0xA6, "OpenBSD"}
 	,{0xA7, "NEXTSTEP"}
 	,{0xB7, "BSDI BSD/386 filesystem"}
 	,{0xB8, "BSDI BSD/386 swap"}
@@ -176,26 +211,45 @@ struct part_type
 	,{0xFF, "BBT (Bad Blocks Table)"}
 };
 
+static void print_s0(int which);
+static void print_part(int i);
+static void init_sector0(unsigned long start);
+static void init_boot(void);
+static void change_part(int i);
+static void print_params();
+static void change_active(int which);
+static void get_params_to_use();
+static void dos(int sec, int size, unsigned char *c, unsigned char *s,
+		unsigned char *h);
+static int open_disk(int u_flag);
+static ssize_t read_disk(off_t sector, void *buf);
+static ssize_t write_disk(off_t sector, void *buf);
+static int get_params();
+static int read_s0();
+static int write_s0();
+static int ok(char *str);
+static int decimal(char *str, int *num, int deflt);
+static char *get_type(int type);
+static int read_config(char *config_file);
+static void reset_boot(void);
+static void usage(void);
+#if 0
+static int hex(char *str, int *num, int deflt);
+static int string(char *str, char **ans);
+#endif
 
-main(argc, argv)
-char **argv;
+
+int
+main(int argc, char *argv[])
 {
 	int	i;
 
-	name = *argv;
-	{register char *cp = name;
-		while (*cp) if (*cp++ == '/') name = cp;
-	}
-
 	for ( argv++ ; --argc ; argv++ ) { register char *token = *argv;
 		if (*token++ != '-' || !*token)
 			break;
 		else { register int flag;
-			for ( ; flag = *token++ ; ) {
+			for ( ; (flag = *token++) ; ) {
 				switch (flag) {
-				case '0':
-					partition = 0;
-					break;
 				case '1':
 					partition = 1;
 					break;
@@ -205,16 +259,43 @@ char **argv;
 				case '3':
 					partition = 3;
 					break;
+				case '4':
+					partition = 4;
+					break;
 				case 'a':
 					a_flag = 1;
 					break;
+				case 'f':
+					if (*token)
+					{
+					    f_flag = token;
+					    token = "";
+					}
+					else
+					{
+					    if (argc == 1)
+						usage();
+					    --argc;
+					    f_flag = *++argv;
+					}
+					/*
+					 * u_flag is needed, because we're
+					 * writing to the disk.
+					 */
+					u_flag = 1;
+					break;
 				case 'i':
 					i_flag = 1;
 				case 'u':
 					u_flag = 1;
 					break;
+				case 't':
+					t_flag = 1;
+				case 'v':
+					v_flag = 1;
+					break;
 				default:
-					goto usage;
+					usage();
 				}
 			}
 		}
@@ -233,15 +314,11 @@ char **argv;
 		}
 		
 		if (open_disk(u_flag) < 0)
-		{
-			fprintf(stderr, "Cannot open disk %s (%s)\n",
-				disk, sys_errlist[errno]);
-			exit(1);
-		}
+			err(1, "cannot open disk %s", disk);
 	}
 	else
 	{
-		int i, rv;
+		int i, rv = 0;
 
 		for(i = 0; disks[i]; i++)
 		{
@@ -250,59 +327,96 @@ char **argv;
 			if(rv != -2) break;
 		}
 		if(rv < 0)
-		{
-			fprintf(stderr, "Cannot open any disk (%s)\n",
-				sys_errlist[errno]);
-			exit(1);
-		}
+			err(1, "cannot open any disk");
 	}
 
 	printf("******* Working on device %s *******\n",disk);
-	if(u_flag)
+
+	if (f_flag)
 	{
-		get_params_to_use();
+	    if (read_s0() || i_flag)
+	    {
+		reset_boot();
+	    }
+
+	    if (!read_config(f_flag))
+	    {
+		exit(1);
+	    }
+	    if (v_flag)
+	    {
+		print_s0(-1);
+	    }
+	    if (!t_flag)
+	    {
+		write_s0();
+	    }
 	}
 	else
 	{
+	    if(u_flag)
+	    {
+		get_params_to_use();
+	    }
+	    else
+	    {
 		print_params();
-	}
+	    }
 
-	if (read_s0())
+	    if (read_s0())
 		init_sector0(1);
 
-	printf("Warning: BIOS sector numbering starts with sector 1\n");
-	printf("Information from DOS bootblock is:\n");
-	if (partition == -1)
-		for (i = 0; i < NDOSPART; i++)
-			change_part(i);
-	else
+	    printf("Media sector size is %d\n", secsize);
+	    printf("Warning: BIOS sector numbering starts with sector 1\n");
+	    printf("Information from DOS bootblock is:\n");
+	    if (partition == -1)
+		for (i = 1; i <= NDOSPART; i++)
+		    change_part(i);
+	    else
 		change_part(partition);
 
-	if (u_flag || a_flag)
+	    if (u_flag || a_flag)
 		change_active(partition);
 
-	if (u_flag || a_flag) {
-		printf("\nWe haven't changed the partition table yet.  ");
-		printf("This is your last chance.\n");
+	    if (u_flag || a_flag) {
+		if (!t_flag)
+		{
+		    printf("\nWe haven't changed the partition table yet.  ");
+		    printf("This is your last chance.\n");
+		}
 		print_s0(-1);
-		if (ok("Should we write new partition table?"))
+		if (!t_flag)
+		{
+		    if (ok("Should we write new partition table?"))
 			write_s0();
+		}
+		else
+		{
+		    printf("\n-t flag specified -- partition table not written.\n");
+		}
+	    }
 	}
 
 	exit(0);
-
-usage:
-	printf("fdisk {-a|-i|-u} [-{0,1,2,3}] [disk]\n");
 }
 
-print_s0(which)
+static void
+usage()
+{
+	fprintf(stderr,
+ "usage: fdisk {-a|-i|-u} [-f <config file> [-t] [-v]] [-{1,2,3,4}] [disk]\n");
+        exit(1);
+}
+
+static void
+print_s0(int which)
 {
 int	i;
 
 	print_params();
 	printf("Information from DOS bootblock is:\n");
 	if (which == -1)
-		for (i = 0; i < NDOSPART; i++)
+		for (i = 1; i <= NDOSPART; i++)
 			printf("%d: ", i), print_part(i);
 	else
 		print_part(which);
@@ -310,20 +424,31 @@ int	i;
 
 static struct dos_partition mtpart = { 0 };
 
-print_part(i)
+static void
+print_part(int i)
 {
-struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
+	struct	  dos_partition *partp;
+	u_int64_t part_mb;
 
+	partp = ((struct dos_partition *) &mboot.parts) + i - 1;
 
 	if (!bcmp(partp, &mtpart, sizeof (struct dos_partition))) {
 		printf("<UNUSED>\n");
 		return;
 	}
+	/*
+	 * Be careful not to overflow.
+	 */
+	part_mb = partp->dp_size;
+	part_mb *= secsize;
+	part_mb /= (1024 * 1024);
 	printf("sysid %d,(%s)\n", partp->dp_typ, get_type(partp->dp_typ));
-	printf("    start %d, size %d (%d Meg), flag %x\n",
-		partp->dp_start,
-		partp->dp_size, partp->dp_size * 512 / (1024 * 1024),
-		partp->dp_flag);
+	printf("    start %lu, size %lu (%qd Meg), flag %x%s\n",
+		(u_long)partp->dp_start,
+		(u_long)partp->dp_size, 
+		part_mb,
+		partp->dp_flag,
+		partp->dp_flag == ACTIVE ? " (active)" : "");
 	printf("\tbeg: cyl %d/ sector %d/ head %d;\n\tend: cyl %d/ sector %d/ head %d\n"
 		,DPCYL(partp->dp_scyl, partp->dp_ssect)
 		,DPSECT(partp->dp_ssect)
@@ -333,27 +458,38 @@ struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
 		,partp->dp_ehd);
 }
 
-init_sector0(start)
-{
-struct dos_partition *partp = (struct dos_partition *) (&mboot.parts[3]);
-int size = disksecs - start;
-int rest;
 
+static void
+init_boot(void)
+{
 	memcpy(mboot.bootinst, bootcode, sizeof(bootcode));
 	mboot.signature = BOOT_MAGIC;
+}
+
+
+static void
+init_sector0(unsigned long start)
+{
+struct dos_partition *partp = (struct dos_partition *) (&mboot.parts[3]);
+unsigned long size = disksecs - start;
+
+	init_boot();
 
 	partp->dp_typ = DOSPTYP_386BSD;
 	partp->dp_flag = ACTIVE;
 	partp->dp_start = start;
 	partp->dp_size = size;
 
-	dos(partp->dp_start, &partp->dp_scyl, &partp->dp_ssect, &partp->dp_shd);
-	dos(partp->dp_start+partp->dp_size, &partp->dp_ecyl, &partp->dp_esect, &partp->dp_ehd);
+	dos(partp->dp_start, partp->dp_size, 
+	    &partp->dp_scyl, &partp->dp_ssect, &partp->dp_shd);
+	dos(partp->dp_start + partp->dp_size - 1, partp->dp_size,
+	    &partp->dp_ecyl, &partp->dp_esect, &partp->dp_ehd);
 }
 
-change_part(i)
+static void
+change_part(int i)
 {
-struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
+struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i - 1;
 
     printf("The data for partition %d is:\n", i);
     print_part(i);
@@ -363,19 +499,19 @@ struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
 
 	if (i_flag) {
 		bzero((char *)partp, sizeof (struct dos_partition));
-		if (i == 3) {
+		if (i == 4) {
 			init_sector0(1);
-			printf("\nThe static data for the DOS partition 3 has been reinitialized to:\n");
+			printf("\nThe static data for the DOS partition 4 has been reinitialized to:\n");
 			print_part(i);
 		}
 	}
 
 	do {
-		Decimal("sysid", partp->dp_typ, tmp);
+		Decimal("sysid (165=FreeBSD)", partp->dp_typ, tmp);
 		Decimal("start", partp->dp_start, tmp);
 		Decimal("size", partp->dp_size, tmp);
 
-		if (ok("Explicitly specifiy beg/end address ?"))
+		if (ok("Explicitly specify beg/end address ?"))
 		{
 			int	tsec,tcyl,thd;
 			tcyl = DPCYL(partp->dp_scyl,partp->dp_ssect);
@@ -398,10 +534,10 @@ struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
 			partp->dp_esect = DOSSECT(tsec,tcyl);
 			partp->dp_ehd = thd;
 		} else {
-			dos(partp->dp_start,
-				&partp->dp_scyl, &partp->dp_ssect, &partp->dp_shd);
-			dos(partp->dp_start+partp->dp_size - 1,
-				&partp->dp_ecyl, &partp->dp_esect, &partp->dp_ehd);
+			dos(partp->dp_start, partp->dp_size,
+			    &partp->dp_scyl, &partp->dp_ssect, &partp->dp_shd);
+			dos(partp->dp_start + partp->dp_size - 1, partp->dp_size,
+			    &partp->dp_ecyl, &partp->dp_esect, &partp->dp_ehd);
 		}
 
 		print_part(i);
@@ -409,37 +545,47 @@ struct dos_partition *partp = ((struct dos_partition *) &mboot.parts) + i;
     }
 }
 
+static void
 print_params()
 {
 	printf("parameters extracted from in-core disklabel are:\n");
 	printf("cylinders=%d heads=%d sectors/track=%d (%d blks/cyl)\n\n"
 			,cyls,heads,sectors,cylsecs);
 	if((dos_sectors > 63) || (dos_cyls > 1023) || (dos_heads > 255))
-		printf(" Figures below won't work with BIOS for partitions not in cyl 1\n");
+		printf("Figures below won't work with BIOS for partitions not in cyl 1\n");
 	printf("parameters to be used for BIOS calculations are:\n");
 	printf("cylinders=%d heads=%d sectors/track=%d (%d blks/cyl)\n\n"
 		,dos_cyls,dos_heads,dos_sectors,dos_cylsecs);
 }
 
-change_active(which)
+static void
+change_active(int which)
 {
 int i;
-int active = 3, tmp;
+int active = 4, tmp;
 struct dos_partition *partp = ((struct dos_partition *) &mboot.parts);
 
 	if (a_flag && which != -1)
 		active = which;
 	if (!ok("Do you want to change the active partition?"))
 		return;
-	do
+setactive:
+	active = 4;
+	do {
 		Decimal("active partition", active, tmp);
-	while (!ok("Are you happy with this choice"));
+		if (active < 1 || 4 < active) {
+			printf("Active partition number must be in range 1-4."
+					"  Try again.\n");
+			goto setactive;
+		}
+	} while (!ok("Are you happy with this choice"));
 	for (i = 0; i < NDOSPART; i++)
 		partp[i].dp_flag = 0;
-	if (active >= 0 && active < NDOSPART)
-		partp[active].dp_flag = ACTIVE;
+	if (active > 0 && active <= NDOSPART)
+		partp[active-1].dp_flag = ACTIVE;
 }
 
+void
 get_params_to_use()
 {
 	int	tmp;
@@ -458,18 +604,19 @@ get_params_to_use()
 	}
 }
 
+
 /***********************************************\
 * Change real numbers into strange dos numbers	*
 \***********************************************/
-static
-dos(sec, c, s, h)
-int sec;
+static void
+dos(sec, size, c, s, h)
+int sec, size;
 unsigned char *c, *s, *h;
 {
 int cy;
 int hd;
 
-	if (sec == 0) {
+	if (sec == 0 && size == 0) {
 		*s = *c = *h = 0;
 		return;
 	}
@@ -489,52 +636,64 @@ int fd;
 
 	/* Getting device status */
 
-open_disk(u_flag)
+static int
+open_disk(int u_flag)
 {
 struct stat 	st;
 
 	if (stat(disk, &st) == -1) {
-		fprintf(stderr, "%s: Can't get file status of %s\n",
-			name, disk);
+		warnx("can't get file status of %s", disk);
 		return -1;
 	}
 	if ( !(st.st_mode & S_IFCHR) )
-		fprintf(stderr,"%s: Device %s is not character special\n",
-			name, disk);
+		warnx("device %s is not character special", disk);
 	if ((fd = open(disk, a_flag || u_flag ? O_RDWR : O_RDONLY)) == -1) {
 		if(errno == ENXIO)
 			return -2;
-		fprintf(stderr,"%s: Can't open device %s\n", name, disk);
+		warnx("can't open device %s", disk);
 		return -1;
 	}
 	if (get_params(0) == -1) {
-		fprintf(stderr, "%s: Can't get disk parameters on %s\n",
-			name, disk);
+		warnx("can't get disk parameters on %s", disk);
 		return -1;
 	}
 	return fd;
 }
 
-
-read_disk(sector, buf)
+static ssize_t
+read_disk(off_t sector, void *buf)
 {
 	lseek(fd,(sector * 512), 0);
-	return read(fd, buf, 512);
+	if( secsize == 0 )
+		for( secsize = MIN_SEC_SIZE; secsize <= MAX_SEC_SIZE; secsize *= 2 )
+			{
+			/* try the read */
+			int size = read(fd, buf, secsize);
+			if( size == secsize )
+				/* it worked so return */
+				return secsize;
+			}
+	else
+		return read( fd, buf, secsize );
+
+	/* we failed to read at any of the sizes */
+	return -1;
 }
 
-write_disk(sector, buf)
+static ssize_t
+write_disk(off_t sector, void *buf)
 {
 	lseek(fd,(sector * 512), 0);
-	return write(fd, buf, 512);
+	/* write out in the size that the read_disk found worked */
+	return write(fd, buf, secsize);
 }
 
-get_params(verbose)
+static int
+get_params()
 {
 
     if (ioctl(fd, DIOCGDINFO, &disklabel) == -1) {
-	fprintf(stderr,
-		"%s: Can't get disk parameters on %s; supplying dummy ones\n",
-		name, disk);
+	warnx("can't get disk parameters on %s; supplying dummy ones", disk);
 	dos_cyls = cyls = 1;
 	dos_heads = heads = 1;
 	dos_sectors = sectors = 1;
@@ -553,21 +712,22 @@ get_params(verbose)
 }
 
 
+static int
 read_s0()
 {
 	if (read_disk(0, (char *) mboot.bootinst) == -1) {
-		fprintf(stderr, "%s: Can't read fdisk partition table\n", name);
+		warnx("can't read fdisk partition table");
 		return -1;
 	}
 	if (mboot.signature != BOOT_MAGIC) {
-		fprintf(stderr, "%s: Invalid fdisk partition table found\n",
-			name);
+		warnx("invalid fdisk partition table found");
 		/* So should we initialize things */
 		return -1;
 	}
 	return 0;
 }
 
+static int
 write_s0()
 {
 	int	flag;
@@ -584,21 +744,21 @@ write_s0()
 	flag = 1;
 #ifdef NOT_NOW
 	if (ioctl(fd, DIOCWLABEL, &flag) < 0)
-		perror("ioctl DIOCWLABEL");
+		warn("ioctl DIOCWLABEL");
 #endif
 	if (write_disk(0, (char *) mboot.bootinst) == -1) {
-		fprintf(stderr, "%s: Can't write fdisk partition table\n",
-			name);
+		warnx("can't write fdisk partition table");
 		return -1;
 	flag = 0;
 #ifdef NOT_NOW
 	(void) ioctl(fd, DIOCWLABEL, &flag);
 #endif
 	}
+	return(0);
 }
 
 
-
+static int
 ok(str)
 char *str;
 {
@@ -614,9 +774,8 @@ char *str;
 		return 0;
 }
 
-decimal(str, num, deflt)
-char *str;
-int *num;
+static int
+decimal(char *str, int *num, int deflt)
 {
 int acc = 0, c;
 char *cp;
@@ -633,7 +792,7 @@ char *cp;
 		while ((c = *cp) && (c == ' ' || c == '\t')) cp++;
 		if (!c)
 			return 0;
-		while (c = *cp++) {
+		while ((c = *cp++)) {
 			if (c <= '9' && c >= '0')
 				acc = acc * 10 + c - '0';
 			else
@@ -645,15 +804,15 @@ char *cp;
 			*num = acc;
 			return 1;
 		} else
-			printf("%s is an invalid decimal number.  Try again\n",
+			printf("%s is an invalid decimal number.  Try again.\n",
 				lbuf);
 	}
 
 }
 
-hex(str, num, deflt)
-char *str;
-int *num;
+#if 0
+static int
+hex(char *str, int *num, int deflt)
 {
 int acc = 0, c;
 char *cp;
@@ -670,7 +829,7 @@ char *cp;
 		while ((c = *cp) && (c == ' ' || c == '\t')) cp++;
 		if (!c)
 			return 0;
-		while (c = *cp++) {
+		while ((c = *cp++)) {
 			if (c <= '9' && c >= '0')
 				acc = (acc << 4) + c - '0';
 			else if (c <= 'f' && c >= 'a')
@@ -686,15 +845,14 @@ char *cp;
 			*num = acc;
 			return 1;
 		} else
-			printf("%s is an invalid hex number.  Try again\n",
+			printf("%s is an invalid hex number.  Try again.\n",
 				lbuf);
 	}
 
 }
 
-string(str, ans)
-char *str;
-char **ans;
+static int
+string(char *str, char **ans)
 {
 int c;
 char *cp = lbuf;
@@ -722,9 +880,10 @@ char *cp = lbuf;
 		return 1;
 	}
 }
+#endif
 
-char *get_type(type)
-int	type;
+static char *
+get_type(int type)
 {
 	int	numentries = (sizeof(part_types)/sizeof(struct part_type));
 	int	counter = 0;
@@ -742,3 +901,417 @@ int	type;
 	}
 	return("unknown");
 }
+
+
+static void
+parse_config_line(line, command)
+    char	*line;
+    CMD		*command;
+{
+    char	*cp, *end;
+
+    cp = line;
+    while (1)	/* dirty trick used to insure one exit point for this
+		   function */
+    {
+	memset(command, 0, sizeof(*command));
+
+	while (isspace(*cp)) ++cp;
+	if (*cp == '\0' || *cp == '#')
+	{
+	    break;
+	}
+	command->cmd = *cp++;
+
+	/*
+	 * Parse args
+	 */
+	while (1)
+	{
+	    while (isspace(*cp)) ++cp;
+	    if (*cp == '#')
+	    {
+		break;		/* found comment */
+	    }
+	    if (isalpha(*cp))
+	    {
+		command->args[command->n_args].argtype = *cp++;
+	    }
+	    if (!isdigit(*cp))
+	    {
+		break;		/* assume end of line */
+	    }
+	    end = NULL;
+	    command->args[command->n_args].arg_val = strtol(cp, &end, 0);
+	    if (cp == end)
+	    {
+		break;		/* couldn't parse number */
+	    }
+	    cp = end;
+	    command->n_args++;
+	}
+	break;
+    }
+}
+
+
+static int
+process_geometry(command)
+    CMD		*command;
+{
+    int		status = 1, i;
+
+    while (1)
+    {
+	geom_processed = 1;
+	if (part_processed)
+	{
+	    warnx(
+	"ERROR line %d: the geometry specification line must occur before\n\
+    all partition specifications",
+		    current_line_number);
+	    status = 0;
+	    break;
+	}
+	if (command->n_args != 3)
+	{
+	    warnx("ERROR line %d: incorrect number of geometry args",
+		    current_line_number);
+	    status = 0;
+	    break;
+	}
+	dos_cyls = -1;
+	dos_heads = -1;
+	dos_sectors = -1;
+	for (i = 0; i < 3; ++i)
+	{
+	    switch (command->args[i].argtype)
+	    {
+	    case 'c':
+		dos_cyls = command->args[i].arg_val;
+		break;
+	    case 'h':
+		dos_heads = command->args[i].arg_val;
+		break;
+	    case 's':
+		dos_sectors = command->args[i].arg_val;
+		break;
+	    default:
+		warnx(
+		"ERROR line %d: unknown geometry arg type: '%c' (0x%02x)",
+			current_line_number, command->args[i].argtype,
+			command->args[i].argtype);
+		status = 0;
+		break;
+	    }
+	}
+	if (status == 0)
+	{
+	    break;
+	}
+
+	dos_cylsecs = dos_heads * dos_sectors;
+
+	/*
+	 * Do sanity checks on parameter values
+	 */
+	if (dos_cyls < 0)
+	{
+	    warnx("ERROR line %d: number of cylinders not specified",
+		    current_line_number);
+	    status = 0;
+	}
+	if (dos_cyls == 0 || dos_cyls > 1024)
+	{
+	    warnx(
+	"WARNING line %d: number of cylinders (%d) may be out-of-range\n\
+    (must be within 1-1024 for normal BIOS operation, unless the entire disk\n\
+    is dedicated to FreeBSD)",
+		    current_line_number, dos_cyls);
+	}
+
+	if (dos_heads < 0)
+	{
+	    warnx("ERROR line %d: number of heads not specified",
+		    current_line_number);
+	    status = 0;
+	}
+	else if (dos_heads < 1 || dos_heads > 256)
+	{
+	    warnx("ERROR line %d: number of heads must be within (1-256)",
+		    current_line_number);
+	    status = 0;
+	}
+
+	if (dos_sectors < 0)
+	{
+	    warnx("ERROR line %d: number of sectors not specified",
+		    current_line_number);
+	    status = 0;
+	}
+	else if (dos_sectors < 1 || dos_sectors > 63)
+	{
+	    warnx("ERROR line %d: number of sectors must be within (1-63)",
+		    current_line_number);
+	    status = 0;
+	}
+
+	break;
+    }
+    return (status);
+}
+
+
+static int
+process_partition(command)
+    CMD		*command;
+{
+    int				status = 0, partition;
+    unsigned long		chunks, adj_size, max_end;
+    struct dos_partition	*partp;
+
+    while (1)
+    {
+	part_processed = 1;
+	if (command->n_args != 4)
+	{
+	    warnx("ERROR line %d: incorrect number of partition args",
+		    current_line_number);
+	    break;
+	}
+	partition = command->args[0].arg_val;
+	if (partition < 1 || partition > 4)
+	{
+	    warnx("ERROR line %d: invalid partition number %d",
+		    current_line_number, partition);
+	    break;
+	}
+	partp = ((struct dos_partition *) &mboot.parts) + partition - 1;
+	bzero((char *)partp, sizeof (struct dos_partition));
+	partp->dp_typ = command->args[1].arg_val;
+	partp->dp_start = command->args[2].arg_val;
+	partp->dp_size = command->args[3].arg_val;
+	max_end = partp->dp_start + partp->dp_size;
+
+	if (partp->dp_typ == 0)
+	{
+	    /*
+	     * Get out, the partition is marked as unused.
+	     */
+	    /*
+	     * Insure that it's unused.
+	     */
+	    bzero((char *)partp, sizeof (struct dos_partition));
+	    status = 1;
+	    break;
+	}
+
+	/*
+	 * Adjust start upwards, if necessary, to fall on an head boundary.
+	 */
+	if (partp->dp_start % dos_sectors != 0)
+	{
+	    adj_size =
+		(partp->dp_start / dos_sectors + 1) * dos_sectors;
+	    if (adj_size > max_end)
+	    {
+		/*
+		 * Can't go past end of partition
+		 */
+		warnx(
+	"ERROR line %d: unable to adjust start of partition %d to fall on\n\
+    a cylinder boundary",
+			current_line_number, partition);
+		break;
+	    }
+	    warnx(
+	"WARNING: adjusting start offset of partition '%d' from %lu\n\
+    to %lu, to round to an head boundary",
+		    partition, (u_long)partp->dp_start, adj_size);
+	    partp->dp_start = adj_size;
+	}
+
+	/*
+	 * Adjust size downwards, if necessary, to fall on a cylinder
+	 * boundary.
+	 */
+	chunks =
+	    ((partp->dp_start + partp->dp_size) / dos_cylsecs) * dos_cylsecs;
+	adj_size = chunks - partp->dp_start;
+	if (adj_size != partp->dp_size)
+	{
+	    warnx(
+	"WARNING: adjusting size of partition '%d' from %lu to %lu,\n\
+    to round to a cylinder boundary",
+		    partition, (u_long)partp->dp_size, adj_size);
+	    if (chunks > 0)
+	    {
+		partp->dp_size = adj_size;
+	    }
+	    else
+	    {
+		partp->dp_size = 0;
+	    }
+	}
+	if (partp->dp_size < 1)
+	{
+	    warnx("ERROR line %d: size for partition '%d' is zero",
+		    current_line_number, partition);
+	    break;
+	}
+
+	dos(partp->dp_start, partp->dp_size,
+	    &partp->dp_scyl, &partp->dp_ssect, &partp->dp_shd);
+	dos(partp->dp_start+partp->dp_size - 1, partp->dp_size,
+	    &partp->dp_ecyl, &partp->dp_esect, &partp->dp_ehd);
+	status = 1;
+	break;
+    }
+    return (status);
+}
+
+
+static int
+process_active(command)
+    CMD		*command;
+{
+    int				status = 0, partition, i;
+    struct dos_partition	*partp;
+
+    while (1)
+    {
+	active_processed = 1;
+	if (command->n_args != 1)
+	{
+	    warnx("ERROR line %d: incorrect number of active args",
+		    current_line_number);
+	    status = 0;
+	    break;
+	}
+	partition = command->args[0].arg_val;
+	if (partition < 1 || partition > 4)
+	{
+	    warnx("ERROR line %d: invalid partition number %d",
+		    current_line_number, partition);
+	    break;
+	}
+	/*
+	 * Reset active partition
+	 */
+	partp = ((struct dos_partition *) &mboot.parts);
+	for (i = 0; i < NDOSPART; i++)
+	    partp[i].dp_flag = 0;
+	partp[partition-1].dp_flag = ACTIVE;
+
+	status = 1;
+	break;
+    }
+    return (status);
+}
+
+
+static int
+process_line(line)
+    char	*line;
+{
+    CMD		command;
+    int		status = 1;
+
+    while (1)
+    {
+	parse_config_line(line, &command);
+	switch (command.cmd)
+	{
+	case 0:
+	    /*
+	     * Comment or blank line
+	     */
+	    break;
+	case 'g':
+	    /*
+	     * Set geometry
+	     */
+	    status = process_geometry(&command);
+	    break;
+	case 'p':
+	    status = process_partition(&command);
+	    break;
+	case 'a':
+	    status = process_active(&command);
+	    break;
+	default:
+	    status = 0;
+	    break;
+	}
+	break;
+    }
+    return (status);
+}
+
+
+static int
+read_config(config_file)
+    char *config_file;
+{
+    FILE	*fp = NULL;
+    int		status = 1;
+    char	buf[1010];
+
+    while (1)	/* dirty trick used to insure one exit point for this
+		   function */
+    {
+	if (strcmp(config_file, "-") != 0)
+	{
+	    /*
+	     * We're not reading from stdin
+	     */
+	    if ((fp = fopen(config_file, "r")) == NULL)
+	    {
+		status = 0;
+		break;
+	    }
+	}
+	else
+	{
+	    fp = stdin;
+	}
+	current_line_number = 0;
+	while (!feof(fp))
+	{
+	    if (fgets(buf, sizeof(buf), fp) == NULL)
+	    {
+		break;
+	    }
+	    ++current_line_number;
+	    status = process_line(buf);
+	    if (status == 0)
+	    {
+		break;
+	    }
+	}
+	break;
+    }
+    if (fp)
+    {
+	/*
+	 * It doesn't matter if we're reading from stdin, as we've reached EOF
+	 */
+	fclose(fp);
+    }
+    return (status);
+}
+
+
+static void
+reset_boot(void)
+{
+    int				i;
+    struct dos_partition	*partp;
+
+    init_boot();
+    for (i = 0; i < 4; ++i)
+    {
+	partp = ((struct dos_partition *) &mboot.parts) + i;
+	bzero((char *)partp, sizeof (struct dos_partition));
+    }
+}
