@@ -35,14 +35,15 @@
 
 struct mntopt {
 	const char *m_option;	/* option name */
-	int m_inverse;		/* if a negative option, eg "dev" */
-	int m_flag;		/* bit to set, eg. MNT_RDONLY */
+	int m_inverse;		/* if a negative option, e.g. "dev" */
+	int m_flag;		/* bit to set, e.g. MNT_RDONLY */
 	int m_altloc;		/* zero if this is a real mount flag */
 };
 
 /* User-visible MNT_ flags. */
 #define MOPT_ASYNC		{ "async",	0, MNT_ASYNC, 0 }
-#define	MOPT_NOAUTO		{ "auto",	1, 0, 0 }
+#define MOPT_NOATIME		{ "atime",	1, MNT_NOATIME, 0 }
+#define MOPT_NOAUTO		{ "auto",	1, 0, 0 }
 #define MOPT_NODEV		{ "dev",	1, MNT_NODEV, 0 }
 #define MOPT_NOEXEC		{ "exec",	1, MNT_NOEXEC, 0 }
 #define MOPT_NOSUID		{ "suid",	1, MNT_NOSUID, 0 }
@@ -69,6 +70,7 @@ struct mntopt {
 /* Standard options which all mounts can understand. */
 #define MOPT_STDOPTS							\
 	MOPT_FSTAB_COMPAT,						\
+	MOPT_NOATIME,							\
 	MOPT_NOAUTO,							\
 	MOPT_NODEV,							\
 	MOPT_NOEXEC,							\
