@@ -76,10 +76,14 @@ mount_ufs(argc, argv)
 	int ch, mntflags;
 	char *fs_name;
 	struct vfsconf *vfc;
+#ifdef ROOTSLICE_HUNT
+	int unit, slice, result;
+	char part, devbuf[MAXPATHLEN], devpfx[MAXPATHLEN];
+#endif
 
 	mntflags = 0;
 	optind = optreset = 1;		/* Reset for parse of new argv. */
-	while ((ch = getopt(argc, argv, "o:")) != EOF)
+	while ((ch = getopt(argc, argv, "o:")) !=  -1)
 		switch (ch) {
 		case 'o':
 			getmntopts(optarg, mopts, &mntflags, 0);
@@ -117,8 +121,37 @@ mount_ufs(argc, argv)
 			/*warnx("ufs: filesystem not found");*/
 		}
 	}
-
+#ifdef ROOTSLICE_HUNT
+	result = -1;
+	/*
+	 * If we are mounting root, and we have a mount of something that
+	 * might be the compatibility slice, try mounting other slices
+	 * first.  If the kernel has done the right thing and mounted
+	 * the slice because the disk is really sliced, this will find
+	 * the real root filesystem.  If not, we'll try what was supplied.
+	 */
+	if (!strcmp(fs_name, "/") &&
+	    (sscanf(args.fspec, "%[^0-9]%d%c", devpfx, &unit, &part) == 3) &&
+	    (part >= 'a') &&
+	    (part <= 'h')) {
+		for (slice = 1; (slice < 32) && (result < 0); slice++) {
+			sprintf(devbuf, "%s%ds%d%c",
+				devpfx, unit, slice, part);
+			args.fspec = devbuf;
+			result = mount(vfc ? vfc->vfc_index : MOUNT_UFS,
+					fs_name, mntflags, &args);
+		}
+		args.fspec = argv[0];
+	}
+	if (result == 0)
+		warnx("*** update /etc/fstab entry for %s to use %s ***",
+			fs_name, devbuf);
+	/* Try the mount as originally planned */
+	if ((result < 0) &&
+		(mount(vfc ? vfc->vfc_index : MOUNT_UFS, fs_name, mntflags, &args) < 0)) {
+#else
 	if (mount(vfc ? vfc->vfc_index : MOUNT_UFS, fs_name, mntflags, &args) < 0) {
+#endif
 		(void)fprintf(stderr, "%s on %s: ", args.fspec, fs_name);
 		switch (errno) {
 		case EMFILE:
