@@ -35,19 +35,23 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1989, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
+#if 0
 static char sccsid[] = "@(#)ping.c	8.1 (Berkeley) 6/5/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/sbin/ping/ping.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
  *			P I N G . C
  *
- * Using the InterNet Control Message Protocol (ICMP) "ECHO" facility,
+ * Using the Internet Control Message Protocol (ICMP) "ECHO" facility,
  * measure round-trip-delays and packet loss across network paths.
  *
  * Author -
@@ -62,26 +66,36 @@ static char sccsid[] = "@(#)ping.c	8.1 (Berkeley) 6/5/93";
  *	This program has to run SUID to ROOT to access the ICMP socket.
  */
 
-#include <sys/param.h>
+#include <sys/param.h>		/* NB: we rely on this for <sys/types.h> */
+
+#include <ctype.h>
+#include <err.h>
+#include <errno.h>
+#include <math.h>
+#include <netdb.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sysexits.h>
+#include <termios.h>
+#include <unistd.h>
+
 #include <sys/socket.h>
-#include <sys/file.h>
 #include <sys/time.h>
-#include <sys/signal.h>
-#include <termios.h>
+#include <sys/uio.h>
 
-#include <netinet/in_systm.h>
 #include <netinet/in.h>
+#include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
 #include <netinet/ip_var.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <ctype.h>
-#include <errno.h>
-#include <string.h>
+#include <arpa/inet.h>
 
-#define	DEFDATALEN	(64 - 8)	/* default data length */
+#define	PHDR_LEN	sizeof(struct timeval)
+#define	DEFDATALEN	(64 - PHDR_LEN)	/* default data length */
+#define	FLOOD_BACKOFF	20000		/* usecs to back off if F_FLOOD mode */
+					/* runs out of buffer space */
 #define	MAXIPLEN	60
 #define	MAXICMPLEN	76
 #define	MAXPACKET	(65536 - 60 - 8)/* max packet size */
@@ -96,15 +110,20 @@ static char sccsid[] = "@(#)ping.c	8.1 (Berkeley) 6/5/93";
 
 /* various options */
 int options;
-#define	F_FLOOD		0x001
-#define	F_INTERVAL	0x002
-#define	F_NUMERIC	0x004
-#define	F_PINGFILLED	0x008
-#define	F_QUIET		0x010
-#define	F_RROUTE	0x020
-#define	F_SO_DEBUG	0x040
-#define	F_SO_DONTROUTE	0x080
-#define	F_VERBOSE	0x100
+#define	F_FLOOD		0x0001
+#define	F_INTERVAL	0x0002
+#define	F_NUMERIC	0x0004
+#define	F_PINGFILLED	0x0008
+#define	F_QUIET		0x0010
+#define	F_RROUTE	0x0020
+#define	F_SO_DEBUG	0x0040
+#define	F_SO_DONTROUTE	0x0080
+#define	F_VERBOSE	0x0100
+#define	F_QUIET2	0x0200
+#define	F_NOLOOP	0x0400
+#define	F_MTTL		0x0800
+#define	F_MIF		0x1000
+#define	F_AUDIBLE	0x2000
 
 /*
  * MAX_DUP_CHK is the number of bits in received table, i.e. the maximum
@@ -123,6 +142,7 @@ char BSPACE = '\b';		/* characters written for flood */
 char DOT = '.';
 char *hostname;
 int ident;			/* process id to identify our packets */
+int uid;			/* cached uid for micro-optimization */
 
 /* counters */
 long npackets;			/* max packets to transmit */
@@ -136,77 +156,121 @@ int timing;			/* flag to do timing */
 double tmin = 999999999.0;	/* minimum round trip time */
 double tmax = 0.0;		/* maximum round trip time */
 double tsum = 0.0;		/* sum of all times, for doing average */
+double tsumsq = 0.0;		/* sum of all times squared, for std. dev. */
 
+volatile sig_atomic_t finish_up;  /* nonzero if we've been told to finish up */
 int reset_kerninfo;
+volatile sig_atomic_t siginfo_p;
 
-char *pr_addr();
-void catcher(), finish(), status();
+static void fill(char *, char *);
+static u_short in_cksum(u_short *, int);
+static void check_status(void);
+static void finish(void) __dead2;
+static void pinger(void);
+static char *pr_addr(struct in_addr);
+static void pr_icmph(struct icmp *);
+static void pr_iph(struct ip *);
+static void pr_pack(char *, int, struct sockaddr_in *, struct timeval *);
+static void pr_retip(struct ip *);
+static void status(int);
+static void stopit(int);
+static void tvsub(struct timeval *, struct timeval *);
+static void usage(void) __dead2;
 
+int
 main(argc, argv)
 	int argc;
-	char **argv;
+	char *const *argv;
 {
-	extern int errno, optind;
-	extern char *optarg;
-	struct timeval timeout;
+	struct timeval last, intvl;
 	struct hostent *hp;
 	struct sockaddr_in *to;
-	struct protoent *proto;
 	struct termios ts;
 	register int i;
-	int ch, fdmask, hold, packlen, preload;
+	int ch, hold, packlen, preload, sockerrno, almost_done = 0;
+	struct in_addr ifaddr;
+	unsigned char ttl, loop;
 	u_char *datap, *packet;
-	char *target, hnamebuf[MAXHOSTNAMELEN], *malloc();
+	char *target, hnamebuf[MAXHOSTNAMELEN];
+	char *ep;
+	u_long ultmp;
 #ifdef IP_OPTIONS
 	char rspace[3 + 4 * NROUTES + 1];	/* record route space */
 #endif
+	struct sigaction si_sa;
+	struct iovec iov;
+	struct msghdr msg;
+	struct sockaddr_in from;
+	char ctrl[sizeof(struct cmsghdr) + sizeof(struct timeval)];
+
+	/*
+	 * Do the stuff that we need root priv's for *first*, and
+	 * then drop our setuid bit.  Save error reporting for
+	 * after arg parsing.
+	 */
+	s = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
+	sockerrno = errno;
+
+	setuid(getuid());
+	uid = getuid();
 
 	preload = 0;
-	if (tcgetattr (0, &ts) != -1) {
-		reset_kerninfo = !(ts.c_lflag & NOKERNINFO);
-		ts.c_lflag |= NOKERNINFO;
-		tcsetattr (0, TCSANOW, &ts);
-	}
 
-	datap = &outpack[8 + sizeof(struct timeval)];
-	while ((ch = getopt(argc, argv, "Rc:dfh:i:l:np:qrs:v")) != EOF)
+	datap = &outpack[8 + PHDR_LEN];
+	while ((ch = getopt(argc, argv, "I:LQRT:c:adfi:l:np:qrs:v")) != -1) {
 		switch(ch) {
+		case 'a':
+			options |= F_AUDIBLE;
+			break;
 		case 'c':
-			npackets = atoi(optarg);
-			if (npackets <= 0) {
-				(void)fprintf(stderr,
-				    "ping: bad number of packets to transmit.\n");
-				exit(1);
-			}
+			ultmp = strtoul(optarg, &ep, 0);
+			if (*ep || ep == optarg || ultmp > LONG_MAX || !ultmp)
+				errx(EX_USAGE,
+				    "invalid count of packets to transmit: `%s'",
+				    optarg);
+			npackets = ultmp;
 			break;
 		case 'd':
 			options |= F_SO_DEBUG;
 			break;
 		case 'f':
 			if (getuid()) {
-				(void)fprintf(stderr,
-				    "ping: %s\n", strerror(EPERM));
-				exit(1);
+				errno = EPERM;
+				err(EX_NOPERM, "-f flag");
 			}
 			options |= F_FLOOD;
 			setbuf(stdout, (char *)NULL);
 			break;
 		case 'i':		/* wait between sending packets */
-			interval = atoi(optarg);
-			if (interval <= 0) {
-				(void)fprintf(stderr,
-				    "ping: bad timing interval.\n");
-				exit(1);
-			}
+			ultmp = strtoul(optarg, &ep, 0);
+			if (*ep || ep == optarg || ultmp > INT_MAX)
+				errx(EX_USAGE,
+				     "invalid timing interval: `%s'", optarg);
 			options |= F_INTERVAL;
+			interval = ultmp;
+			break;
+		case 'I':		/* multicast interface */
+			if (inet_aton(optarg, &ifaddr) == 0)
+				errx(EX_USAGE, 
+				     "invalid multicast interface: `%s'",
+				     optarg);
+			options |= F_MIF;
 			break;
 		case 'l':
-			preload = atoi(optarg);
-			if (preload < 0) {
-				(void)fprintf(stderr,
-				    "ping: bad preload value.\n");
-				exit(1);
+			ultmp = strtoul(optarg, &ep, 0);
+			if (*ep || ep == optarg || ultmp > INT_MAX)
+				errx(EX_USAGE, 
+				     "invalid preload value: `%s'", optarg);
+			if (getuid()) {
+				errno = EPERM;
+				err(EX_NOPERM, "-l flag");
 			}
+			options |= F_FLOOD;
+			preload = ultmp;
+			break;
+		case 'L':
+			options |= F_NOLOOP;
+			loop = 0;
 			break;
 		case 'n':
 			options |= F_NUMERIC;
@@ -215,6 +279,9 @@ main(argc, argv)
 			options |= F_PINGFILLED;
 			fill((char *)datap, optarg);
 				break;
+		case 'Q':
+			options |= F_QUIET2;
+			break;
 		case 'q':
 			options |= F_QUIET;
 			break;
@@ -225,17 +292,22 @@ main(argc, argv)
 			options |= F_SO_DONTROUTE;
 			break;
 		case 's':		/* size of packet to send */
-			datalen = atoi(optarg);
-			if (datalen > MAXPACKET) {
-				(void)fprintf(stderr,
-				    "ping: packet size too large.\n");
-				exit(1);
-			}
-			if (datalen <= 0) {
-				(void)fprintf(stderr,
-				    "ping: illegal packet size.\n");
-				exit(1);
-			}
+			ultmp = strtoul(optarg, &ep, 0);
+			if (ultmp > MAXPACKET)
+				errx(EX_USAGE, "packet size too large: %lu",
+				     ultmp);
+			if (*ep || ep == optarg || !ultmp)
+				errx(EX_USAGE, "invalid packet size: `%s'", 
+				     optarg);
+			datalen = ultmp;
+			break;
+		case 'T':		/* multicast TTL */
+			ultmp = strtoul(optarg, &ep, 0);
+			if (*ep || ep == optarg || ultmp > 255)
+				errx(EX_USAGE, "invalid multicast TTL: `%s'",
+				     optarg);
+			ttl = ultmp;
+			options |= F_MTTL;
 			break;
 		case 'v':
 			options |= F_VERBOSE;
@@ -243,58 +315,58 @@ main(argc, argv)
 		default:
 			usage();
 		}
-	argc -= optind;
-	argv += optind;
+	}
 
-	if (argc != 1)
+	if (argc - optind != 1)
 		usage();
-	target = *argv;
+	target = argv[optind];
 
 	bzero((char *)&whereto, sizeof(struct sockaddr));
 	to = (struct sockaddr_in *)&whereto;
 	to->sin_family = AF_INET;
-	to->sin_addr.s_addr = inet_addr(target);
-	if (to->sin_addr.s_addr != (u_int)-1)
+	if (inet_aton(target, &to->sin_addr) != 0) {
 		hostname = target;
-	else {
-		hp = gethostbyname(target);
-		if (!hp) {
-			(void)fprintf(stderr,
-			    "ping: unknown host %s\n", target);
-			exit(1);
-		}
-		to->sin_family = hp->h_addrtype;
-		bcopy(hp->h_addr, (caddr_t)&to->sin_addr, hp->h_length);
+	} else {
+		hp = gethostbyname2(target, AF_INET);
+		if (!hp)
+			errx(EX_NOHOST, "cannot resolve %s: %s",
+			     target, hstrerror(h_errno));
+
+		to->sin_len = sizeof *to;
+		if (hp->h_length > sizeof(to->sin_addr))
+			errx(1,"gethostbyname2 returned an illegal address");
+		memcpy(&to->sin_addr, hp->h_addr_list[0], sizeof to->sin_addr);
 		(void)strncpy(hnamebuf, hp->h_name, sizeof(hnamebuf) - 1);
+		hnamebuf[sizeof(hnamebuf) - 1] = '\0';
 		hostname = hnamebuf;
 	}
 
-	if (options & F_FLOOD && options & F_INTERVAL) {
-		(void)fprintf(stderr,
-		    "ping: -f and -i incompatible options.\n");
-		exit(1);
-	}
+	if (options & F_FLOOD && options & F_INTERVAL)
+		errx(EX_USAGE, "-f and -i: incompatible options");
 
-	if (datalen >= sizeof(struct timeval))	/* can we time transfer */
+	if (options & F_FLOOD && IN_MULTICAST(ntohl(to->sin_addr.s_addr)))
+		errx(EX_USAGE, 
+		     "-f flag cannot be used with multicast destination");
+	if (options & (F_MIF | F_NOLOOP | F_MTTL)
+	    && !IN_MULTICAST(ntohl(to->sin_addr.s_addr)))
+		errx(EX_USAGE, 
+		     "-I, -L, -T flags cannot be used with unicast destination");
+
+	if (datalen >= PHDR_LEN)	/* can we time transfer */
 		timing = 1;
 	packlen = datalen + MAXIPLEN + MAXICMPLEN;
-	if (!(packet = (u_char *)malloc((u_int)packlen))) {
-		(void)fprintf(stderr, "ping: out of memory.\n");
-		exit(1);
-	}
+	if (!(packet = (u_char *)malloc((size_t)packlen)))
+		err(EX_UNAVAILABLE, "malloc");
+
 	if (!(options & F_PINGFILLED))
-		for (i = 8; i < datalen; ++i)
+		for (i = PHDR_LEN; i < datalen; ++i)
 			*datap++ = i;
 
 	ident = getpid() & 0xFFFF;
 
-	if (!(proto = getprotobyname("icmp"))) {
-		(void)fprintf(stderr, "ping: unknown protocol icmp.\n");
-		exit(1);
-	}
-	if ((s = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
-		perror("ping: socket");
-		exit(1);
+	if (s < 0) {
+		errno = sockerrno;
+		err(EX_OSERR, "socket");
 	}
 	hold = 1;
 	if (options & F_SO_DEBUG)
@@ -307,26 +379,51 @@ main(argc, argv)
 	/* record route option */
 	if (options & F_RROUTE) {
 #ifdef IP_OPTIONS
+		bzero(rspace, sizeof(rspace));
 		rspace[IPOPT_OPTVAL] = IPOPT_RR;
-		rspace[IPOPT_OLEN] = sizeof(rspace)-1;
+		rspace[IPOPT_OLEN] = sizeof(rspace) - 1;
 		rspace[IPOPT_OFFSET] = IPOPT_MINOFF;
+		rspace[sizeof(rspace) - 1] = IPOPT_EOL;
 		if (setsockopt(s, IPPROTO_IP, IP_OPTIONS, rspace,
-		    sizeof(rspace)) < 0) {
-			perror("ping: record route");
-			exit(1);
-		}
+		    sizeof(rspace)) < 0)
+			err(EX_OSERR, "setsockopt IP_OPTIONS");
 #else
-		(void)fprintf(stderr,
-		  "ping: record route not available in this implementation.\n");
-		exit(1);
+		errx(EX_UNAVAILABLE,
+		  "record route not available in this implementation");
 #endif /* IP_OPTIONS */
 	}
 
+	if (options & F_NOLOOP) {
+		if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_LOOP, &loop,
+		    sizeof(loop)) < 0) {
+			err(EX_OSERR, "setsockopt IP_MULTICAST_LOOP");
+		}
+	}
+	if (options & F_MTTL) {
+		if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &ttl,
+		    sizeof(ttl)) < 0) {
+			err(EX_OSERR, "setsockopt IP_MULTICAST_TTL");
+		}
+	}
+	if (options & F_MIF) {
+		if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, &ifaddr,
+		    sizeof(ifaddr)) < 0) {
+			err(EX_OSERR, "setsockopt IP_MULTICAST_IF");
+		}
+	}
+#ifdef SO_TIMESTAMP
+	{ int on = 1;
+	if (setsockopt(s, SOL_SOCKET, SO_TIMESTAMP, &on, sizeof(on)) < 0)
+		err(EX_OSERR, "setsockopt SO_TIMESTAMP");
+	}
+#endif
+
 	/*
 	 * When pinging the broadcast address, you can get a lot of answers.
 	 * Doing something so evil is useful if you are trying to stress the
 	 * ethernet, or just want to fill the arp cache to get some stuff for
-	 * /etc/ethers.
+	 * /etc/ethers.  But beware: RFC 1122 allows hosts to ignore broadcast
+	 * or multicast pings if they wish.
 	 */
 	hold = 48 * 1024;
 	(void)setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *)&hold,
@@ -334,80 +431,146 @@ main(argc, argv)
 
 	if (to->sin_family == AF_INET)
 		(void)printf("PING %s (%s): %d data bytes\n", hostname,
-		    inet_ntoa(*(struct in_addr *)&to->sin_addr.s_addr),
+		    inet_ntoa(to->sin_addr),
 		    datalen);
 	else
 		(void)printf("PING %s: %d data bytes\n", hostname, datalen);
 
-	(void)signal(SIGINT, finish);
-	(void)signal(SIGALRM, catcher);
-	(void)signal(SIGINFO, status);
+	/*
+	 * Use sigaction() instead of signal() to get unambiguous semantics,
+	 * in particular with SA_RESTART not set.
+	 */
+
+	sigemptyset(&si_sa.sa_mask);
+	si_sa.sa_flags = 0;
+
+	si_sa.sa_handler = stopit;
+	if (sigaction(SIGINT, &si_sa, 0) == -1) {
+		err(EX_OSERR, "sigaction SIGINT");
+	}
+
+	si_sa.sa_handler = status;
+	if (sigaction(SIGINFO, &si_sa, 0) == -1) {
+		err(EX_OSERR, "sigaction");
+	}
+
+	bzero(&msg, sizeof(msg));
+	msg.msg_name = (caddr_t)&from;
+	msg.msg_iov = &iov;
+	msg.msg_iovlen = 1;
+#ifdef SO_TIMESTAMP
+	msg.msg_control = (caddr_t)ctrl;
+#endif
+	iov.iov_base = packet;
+	iov.iov_len = packlen;
+
+	if (tcgetattr(STDOUT_FILENO, &ts) != -1) {
+		reset_kerninfo = !(ts.c_lflag & NOKERNINFO);
+		ts.c_lflag |= NOKERNINFO;
+		tcsetattr(STDOUT_FILENO, TCSANOW, &ts);
+	}
 
 	while (preload--)		/* fire off them quickies */
 		pinger();
 
-	if ((options & F_FLOOD) == 0)
-		catcher();		/* start things going */
+	if (options & F_FLOOD) {
+		intvl.tv_sec = 0;
+		intvl.tv_usec = 10000;
+	} else {
+		intvl.tv_sec = interval;
+		intvl.tv_usec = 0;
+	}
 
-	for (;;) {
-		struct sockaddr_in from;
+	pinger();			/* send the first ping */
+	(void)gettimeofday(&last, NULL);
+
+	while (!finish_up) {
 		register int cc;
-		int fromlen;
+		int n;
+		struct timeval timeout, now;
+		fd_set rfds;
 
-		if (options & F_FLOOD) {
-			pinger();
-			timeout.tv_sec = 0;
-			timeout.tv_usec = 10000;
-			fdmask = 1 << s;
-			if (select(s + 1, (fd_set *)&fdmask, (fd_set *)NULL,
-			    (fd_set *)NULL, &timeout) < 1)
-				continue;
+		check_status();
+		FD_ZERO(&rfds);
+		FD_SET(s, &rfds);
+		(void)gettimeofday(&now, NULL);
+		timeout.tv_sec = last.tv_sec + intvl.tv_sec - now.tv_sec;
+		timeout.tv_usec = last.tv_usec + intvl.tv_usec - now.tv_usec;
+		while (timeout.tv_usec < 0) {
+			timeout.tv_usec += 1000000;
+			timeout.tv_sec--;
 		}
-		fromlen = sizeof(from);
-		if ((cc = recvfrom(s, (char *)packet, packlen, 0,
-		    (struct sockaddr *)&from, &fromlen)) < 0) {
-			if (errno == EINTR)
+		while (timeout.tv_usec >= 1000000) {
+			timeout.tv_usec -= 1000000;
+			timeout.tv_sec++;
+		}
+		if (timeout.tv_sec < 0)
+			timeout.tv_sec = timeout.tv_usec = 0;
+		n = select(s + 1, &rfds, NULL, NULL, &timeout);
+		if (n == 1) {
+			struct timeval *t = 0;
+#ifdef SO_TIMESTAMP
+			struct cmsghdr *cmsg = (struct cmsghdr *)&ctrl;
+
+			msg.msg_controllen = sizeof(ctrl);
+#endif
+			msg.msg_namelen = sizeof(from);
+			if ((cc = recvmsg(s, &msg, 0)) < 0) {
+				if (errno == EINTR)
+					continue;
+				warn("recvmsg");
 				continue;
-			perror("ping: recvfrom");
-			continue;
+			}
+#ifdef SO_TIMESTAMP
+			if (cmsg->cmsg_level == SOL_SOCKET &&
+			    cmsg->cmsg_type == SCM_TIMESTAMP &&
+			    cmsg->cmsg_len == (sizeof *cmsg + sizeof *t)) {
+				/* Copy to avoid alignment problems: */
+				memcpy(&now,CMSG_DATA(cmsg),sizeof(now));
+				t = &now;
+			}
+#endif
+			if (t == 0) {
+				(void)gettimeofday(&now, NULL);
+				t = &now;
+			}
+			pr_pack((char *)packet, cc, &from, t);
+			if (npackets && nreceived >= npackets)
+				break;
+		}
+		if (n == 0) {
+			if (!npackets || ntransmitted < npackets)
+				pinger();
+			else {
+				if (almost_done)
+					break;
+				almost_done = 1;
+				if (nreceived) {
+					intvl.tv_sec = 2 * tmax / 1000;
+					if (!intvl.tv_sec)
+						intvl.tv_sec = 1;
+				} else
+					intvl.tv_sec = MAXWAIT;
+			}
+			(void)gettimeofday(&last, NULL);
 		}
-		pr_pack((char *)packet, cc, &from);
-		if (npackets && nreceived >= npackets)
-			break;
 	}
 	finish();
 	/* NOTREACHED */
+	exit(0);	/* Make the compiler happy */
 }
 
 /*
- * catcher --
- *	This routine causes another PING to be transmitted, and then
- * schedules another SIGALRM for 1 second from now.
- *
- * bug --
- *	Our sense of time will slowly skew (i.e., packets will not be
- * launched exactly at 1-second intervals).  This does not affect the
- * quality of the delay and loss statistics.
+ * stopit --
+ *	Set the global bit that causes the main loop to quit.
+ * Do NOT call finish() from here, since finish() does far too much
+ * to be called from a signal handler.
  */
 void
-catcher()
+stopit(sig)
+	int sig;
 {
-	int waittime;
-
-	pinger();
-	(void)signal(SIGALRM, catcher);
-	if (!npackets || ntransmitted < npackets)
-		alarm((u_int)interval);
-	else {
-		if (nreceived) {
-			waittime = 2 * tmax / 1000;
-			if (!waittime)
-				waittime = 1;
-		} else
-			waittime = MAXWAIT;
-		(void)signal(SIGALRM, finish);
-		(void)alarm((u_int)waittime);
-	}
+	finish_up = 1;
 }
 
 /*
@@ -415,10 +578,11 @@ catcher()
  *	Compose and transmit an ICMP ECHO REQUEST packet.  The IP packet
  * will be added on by the kernel.  The ID field is our UNIX process ID,
  * and the sequence number is an ascending integer.  The first 8 bytes
- * of the data portion are used to hold a UNIX "timeval" struct in VAX
+ * of the data portion are used to hold a UNIX "timeval" struct in host
  * byte-order, to compute the round-trip time.
  */
-pinger()
+static void
+pinger(void)
 {
 	register struct icmp *icp;
 	register int cc;
@@ -428,7 +592,7 @@ pinger()
 	icp->icmp_type = ICMP_ECHO;
 	icp->icmp_code = 0;
 	icp->icmp_cksum = 0;
-	icp->icmp_seq = ntransmitted++;
+	icp->icmp_seq = ntransmitted;
 	icp->icmp_id = ident;			/* ID */
 
 	CLR(icp->icmp_seq % mx_dup_ck);
@@ -437,7 +601,7 @@ pinger()
 		(void)gettimeofday((struct timeval *)&outpack[8],
 		    (struct timezone *)NULL);
 
-	cc = datalen + 8;			/* skips ICMP portion */
+	cc = datalen + PHDR_LEN;		/* skips ICMP portion */
 
 	/* compute ICMP checksum here */
 	icp->icmp_cksum = in_cksum((u_short *)icp, cc);
@@ -446,11 +610,18 @@ pinger()
 	    sizeof(struct sockaddr));
 
 	if (i < 0 || i != cc)  {
-		if (i < 0)
-			perror("ping: sendto");
-		(void)printf("ping: wrote %s %d chars, ret=%d\n",
-		    hostname, cc, i);
+		if (i < 0) {
+			if (options & F_FLOOD && errno == ENOBUFS) {
+				usleep(FLOOD_BACKOFF);
+				return;
+			}
+			warn("sendto");
+		} else {
+			warn("%s: partial write: %d of %d bytes",
+			     hostname, i, cc);
+		}
 	}
+	ntransmitted++;
 	if (!(options & F_QUIET) && options & F_FLOOD)
 		(void)write(STDOUT_FILENO, &DOT, 1);
 }
@@ -462,10 +633,12 @@ pinger()
  * which arrive ('tis only fair).  This permits multiple copies of this
  * program to be run without having intermingled output (or statistics!).
  */
-pr_pack(buf, cc, from)
+static void
+pr_pack(buf, cc, from, tv)
 	char *buf;
 	int cc;
 	struct sockaddr_in *from;
+	struct timeval *tv;
 {
 	register struct icmp *icp;
 	register u_long l;
@@ -474,20 +647,17 @@ pr_pack(buf, cc, from)
 	static int old_rrlen;
 	static char old_rr[MAX_IPOPTLEN];
 	struct ip *ip;
-	struct timeval tv, *tp;
+	struct timeval *tp;
 	double triptime;
 	int hlen, dupflag;
 
-	(void)gettimeofday(&tv, (struct timezone *)NULL);
-
 	/* Check the IP header */
 	ip = (struct ip *)buf;
 	hlen = ip->ip_hl << 2;
 	if (cc < hlen + ICMP_MINLEN) {
 		if (options & F_VERBOSE)
-			(void)fprintf(stderr,
-			  "ping: packet too short (%d bytes) from %s\n", cc,
-			  inet_ntoa(*(struct in_addr *)&from->sin_addr.s_addr));
+			warn("packet too short (%d bytes) from %s", cc,
+			     inet_ntoa(from->sin_addr));
 		return;
 	}
 
@@ -498,16 +668,21 @@ pr_pack(buf, cc, from)
 		if (icp->icmp_id != ident)
 			return;			/* 'Twas not our ECHO */
 		++nreceived;
+		triptime = 0.0;
 		if (timing) {
+			struct timeval tv1;
 #ifndef icmp_data
 			tp = (struct timeval *)&icp->icmp_ip;
 #else
 			tp = (struct timeval *)icp->icmp_data;
 #endif
-			tvsub(&tv, tp);
-			triptime = ((double)tv.tv_sec) * 1000.0 +
-			    ((double)tv.tv_usec) / 1000.0;
+			/* Avoid unaligned data: */
+			memcpy(&tv1,tp,sizeof(tv1));
+			tvsub(tv, &tv1);
+ 			triptime = ((double)tv->tv_sec) * 1000.0 +
+ 			    ((double)tv->tv_usec) / 1000.0;
 			tsum += triptime;
+			tsumsq += triptime * triptime;
 			if (triptime < tmin)
 				tmin = triptime;
 			if (triptime > tmax)
@@ -537,15 +712,25 @@ pr_pack(buf, cc, from)
 				(void)printf(" time=%.3f ms", triptime);
 			if (dupflag)
 				(void)printf(" (DUP!)");
+			if (options & F_AUDIBLE)
+				(void)printf("\a");
 			/* check the data */
-			cp = (u_char*)&icp->icmp_data[8];
-			dp = &outpack[8 + sizeof(struct timeval)];
-			for (i = 8; i < datalen; ++i, ++cp, ++dp) {
+			cp = (u_char*)&icp->icmp_data[PHDR_LEN];
+			dp = &outpack[8 + PHDR_LEN];
+			for (i = PHDR_LEN; i < datalen; ++i, ++cp, ++dp) {
 				if (*cp != *dp) {
 	(void)printf("\nwrong data byte #%d should be 0x%x but was 0x%x",
 	    i, *dp, *cp);
+					printf("\ncp:");
 					cp = (u_char*)&icp->icmp_data[0];
-					for (i = 8; i < datalen; ++i, ++cp) {
+					for (i = 0; i < datalen; ++i, ++cp) {
+						if ((i % 32) == 8)
+							(void)printf("\n\t");
+						(void)printf("%x ", *cp);
+					}
+					printf("\ndp:");
+					cp = &outpack[8];
+					for (i = 0; i < datalen; ++i, ++cp) {
 						if ((i % 32) == 8)
 							(void)printf("\n\t");
 						(void)printf("%x ", *cp);
@@ -555,12 +740,35 @@ pr_pack(buf, cc, from)
 			}
 		}
 	} else {
-		/* We've got something other than an ECHOREPLY */
-		if (!(options & F_VERBOSE))
-			return;
-		(void)printf("%d bytes from %s: ", cc,
-		    pr_addr(from->sin_addr.s_addr));
-		pr_icmph(icp);
+		/*
+		 * We've got something other than an ECHOREPLY.
+		 * See if it's a reply to something that we sent.
+		 * We can compare IP destination, protocol,
+		 * and ICMP type and ID.
+		 *
+		 * Only print all the error messages if we are running
+		 * as root to avoid leaking information not normally 
+		 * available to those not running as root.
+		 */
+#ifndef icmp_data
+		struct ip *oip = &icp->icmp_ip;
+#else
+		struct ip *oip = (struct ip *)icp->icmp_data;
+#endif
+		struct icmp *oicmp = (struct icmp *)(oip + 1);
+
+		if (((options & F_VERBOSE) && uid == 0) ||
+		    (!(options & F_QUIET2) &&
+		     (oip->ip_dst.s_addr ==
+			 ((struct sockaddr_in *)&whereto)->sin_addr.s_addr) &&
+		     (oip->ip_p == IPPROTO_ICMP) &&
+		     (oicmp->icmp_type == ICMP_ECHO) &&
+		     (oicmp->icmp_id == ident))) {
+		    (void)printf("%d bytes from %s: ", cc,
+			pr_addr(from->sin_addr));
+		    pr_icmph(icp);
+		} else
+		    return;
 	}
 
 	/* Display any IP options */
@@ -582,10 +790,13 @@ pr_pack(buf, cc, from)
 					l = (l<<8) + *++cp;
 					l = (l<<8) + *++cp;
 					l = (l<<8) + *++cp;
-					if (l == 0)
-						(void)printf("\t0.0.0.0");
-				else
-					(void)printf("\t%s", pr_addr(ntohl(l)));
+					if (l == 0) {
+						printf("\t0.0.0.0");
+					} else {
+						struct in_addr ina;
+						ina.s_addr = ntohl(l);
+						printf("\t%s", pr_addr(ina));
+					}
 				hlen -= 4;
 				j -= 4;
 				if (j <= IPOPT_MINOFF)
@@ -612,22 +823,35 @@ pr_pack(buf, cc, from)
 				cp += i;
 				break;
 			}
-			old_rrlen = i;
-			bcopy((char *)cp, old_rr, i);
+			if (i < MAX_IPOPTLEN) {
+				old_rrlen = i;
+				bcopy((char *)cp, old_rr, i);
+			} else
+				old_rrlen = 0;
+
 			(void)printf("\nRR: ");
+			j = 0;
 			for (;;) {
 				l = *++cp;
 				l = (l<<8) + *++cp;
 				l = (l<<8) + *++cp;
 				l = (l<<8) + *++cp;
-				if (l == 0)
-					(void)printf("\t0.0.0.0");
-				else
-					(void)printf("\t%s", pr_addr(ntohl(l)));
+				if (l == 0) {
+					printf("\t0.0.0.0");
+				} else {
+					struct in_addr ina;
+					ina.s_addr = ntohl(l);
+					printf("\t%s", pr_addr(ina));
+				}
 				hlen -= 4;
 				i -= 4;
+				j += 4;
 				if (i <= 0)
 					break;
+				if (j >= MAX_IPOPTLEN) {
+					(void) printf("\t(truncated route)");
+					break;
+				}
 				(void)putchar('\n');
 			}
 			break;
@@ -648,6 +872,7 @@ pr_pack(buf, cc, from)
  * in_cksum --
  *	Checksum routine for Internet Protocol family headers (C Version)
  */
+u_short
 in_cksum(addr, len)
 	u_short *addr;
 	int len;
@@ -685,6 +910,7 @@ in_cksum(addr, len)
  *	Subtract 2 timeval structs:  out = out - in.  Out is assumed to
  * be >= in.
  */
+static void
 tvsub(out, in)
 	register struct timeval *out, *in;
 {
@@ -700,35 +926,39 @@ tvsub(out, in)
  *	Print out statistics when SIGINFO is received.
  */
 
-void
-status()
+static void
+status(sig)
+	int sig;
 {
-	double temp_min = nreceived ? tmin : 0;
-	(void)fprintf(stderr, "%ld/%ld packets received (%ld%%) "
-		      "%.3f min / %.3f avg / %.3f max\n",
-		      nreceived, ntransmitted,
-		      (ntransmitted ?
-		       100 - (int) (((ntransmitted - nreceived) * 100)
-				    / ntransmitted)
-		       : 0),
-		      temp_min,
-		      ((nreceived + nrepeats) ?
-		       (tsum / (nreceived + nrepeats))/1000.0
-		       : tsum),
-		      tmax/ 1000.0);
+	siginfo_p = 1;
+}
+
+static void
+check_status()
+{
+	if (siginfo_p) {
+		siginfo_p = 0;
+		(void)fprintf(stderr,
+	"\r%ld/%ld packets received (%.0f%%) %.3f min / %.3f avg / %.3f max\n",
+		    nreceived, ntransmitted,
+		    ntransmitted ? nreceived * 100.0 / ntransmitted : 0.0,
+		    nreceived ? tmin : 0.0,
+		    nreceived + nrepeats ? tsum / (nreceived + nrepeats) : tsum,
+		    tmax);
+	}
 }
 
 /*
  * finish --
  *	Print out statistics, and give up.
  */
-void
+static void
 finish()
 {
-	register int i;
 	struct termios ts;
 
 	(void)signal(SIGINT, SIG_IGN);
+	(void)signal(SIGALRM, SIG_IGN);
 	(void)putchar('\n');
 	(void)fflush(stdout);
 	(void)printf("--- %s ping statistics ---\n", hostname);
@@ -745,14 +975,16 @@ finish()
 			    ntransmitted));
 	(void)putchar('\n');
 	if (nreceived && timing) {
-		/* Only display average to microseconds */
-		i = 1000.0 * tsum / (nreceived + nrepeats);
-		(void)printf("round-trip min/avg/max = %.3f/%.3f/%.3f ms\n",
-		    tmin, ((double)i) / 1000.0, tmax);
+		double n = nreceived + nrepeats;
+		double avg = tsum / n;
+		double vari = tsumsq / n - avg * avg;
+		printf("round-trip min/avg/max/stddev = "
+		       "%.3f/%.3f/%.3f/%.3f ms\n",
+		    tmin, avg, tmax, sqrt(vari));
 	}
-	if (reset_kerninfo && tcgetattr (0, &ts) != -1) {
+	if (reset_kerninfo && tcgetattr(STDOUT_FILENO, &ts) != -1) {
 		ts.c_lflag &= ~NOKERNINFO;
-		tcsetattr (0, TCSANOW, &ts);
+		tcsetattr(STDOUT_FILENO, TCSANOW, &ts);
 	}
 
 	if (nreceived)
@@ -781,6 +1013,7 @@ static char *ttab[] = {
  * pr_icmph --
  *	Print a descriptive string about an ICMP header.
  */
+static void
 pr_icmph(icp)
 	struct icmp *icp;
 {
@@ -804,11 +1037,15 @@ pr_icmph(icp)
 			(void)printf("Destination Port Unreachable\n");
 			break;
 		case ICMP_UNREACH_NEEDFRAG:
-			(void)printf("frag needed and DF set\n");
+			(void)printf("frag needed and DF set (MTU %d)\n",
+					ntohs(icp->icmp_nextmtu));
 			break;
 		case ICMP_UNREACH_SRCFAIL:
 			(void)printf("Source Route Failed\n");
 			break;
+		case ICMP_UNREACH_FILTER_PROHIB:
+			(void)printf("Communication prohibited by filter\n");
+			break;
 		default:
 			(void)printf("Dest Unreachable, Bad Code: %d\n",
 			    icp->icmp_code);
@@ -847,7 +1084,7 @@ pr_icmph(icp)
 			(void)printf("Redirect, Bad Code: %d", icp->icmp_code);
 			break;
 		}
-		(void)printf("(New addr: 0x%08lx)\n", icp->icmp_gwaddr.s_addr);
+		(void)printf("(New addr: %s)\n", inet_ntoa(icp->icmp_gwaddr));
 #ifndef icmp_data
 		pr_retip(&icp->icmp_ip);
 #else
@@ -902,16 +1139,18 @@ pr_icmph(icp)
 		(void)printf("Information Reply\n");
 		/* XXX ID + Seq */
 		break;
-#ifdef ICMP_MASKREQ
 	case ICMP_MASKREQ:
 		(void)printf("Address Mask Request\n");
 		break;
-#endif
-#ifdef ICMP_MASKREPLY
 	case ICMP_MASKREPLY:
 		(void)printf("Address Mask Reply\n");
 		break;
-#endif
+	case ICMP_ROUTERADVERT:
+		(void)printf("Router Advertisement\n");
+		break;
+	case ICMP_ROUTERSOLICIT:
+		(void)printf("Router Solicitation\n");
+		break;
 	default:
 		(void)printf("Bad ICMP type: %d\n", icp->icmp_type);
 	}
@@ -921,6 +1160,7 @@ pr_icmph(icp)
  * pr_iph --
  *	Print an IP header with options.
  */
+static void
 pr_iph(ip)
 	struct ip *ip;
 {
@@ -930,15 +1170,18 @@ pr_iph(ip)
 	hlen = ip->ip_hl << 2;
 	cp = (u_char *)ip + 20;		/* point to options */
 
-	(void)printf("Vr HL TOS  Len   ID Flg  off TTL Pro  cks      Src      Dst Data\n");
+	(void)printf("Vr HL TOS  Len   ID Flg  off TTL Pro  cks      Src      Dst\n");
 	(void)printf(" %1x  %1x  %02x %04x %04x",
-	    ip->ip_v, ip->ip_hl, ip->ip_tos, ip->ip_len, ip->ip_id);
-	(void)printf("   %1x %04x", ((ip->ip_off) & 0xe000) >> 13,
-	    (ip->ip_off) & 0x1fff);
-	(void)printf("  %02x  %02x %04x", ip->ip_ttl, ip->ip_p, ip->ip_sum);
+	    ip->ip_v, ip->ip_hl, ip->ip_tos, ntohs(ip->ip_len),
+	    ntohs(ip->ip_id));
+	(void)printf("   %1lx %04lx",
+	    (u_long) (ntohl(ip->ip_off) & 0xe000) >> 13,
+	    (u_long) ntohl(ip->ip_off) & 0x1fff);
+	(void)printf("  %02x  %02x %04x", ip->ip_ttl, ip->ip_p,
+							    ntohs(ip->ip_sum));
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_src.s_addr));
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_dst.s_addr));
-	/* dump and option bytes */
+	/* dump any option bytes */
 	while (hlen-- > 20) {
 		(void)printf("%02x", *cp++);
 	}
@@ -950,19 +1193,19 @@ pr_iph(ip)
  *	Return an ascii host address as a dotted quad and optionally with
  * a hostname.
  */
-char *
-pr_addr(l)
-	u_long l;
+static char *
+pr_addr(ina)
+	struct in_addr ina;
 {
 	struct hostent *hp;
-	static char buf[80];
+	static char buf[16 + 3 + MAXHOSTNAMELEN];
 
 	if ((options & F_NUMERIC) ||
-	    !(hp = gethostbyaddr((char *)&l, 4, AF_INET)))
-		(void)sprintf(buf, "%s", inet_ntoa(*(struct in_addr *)&l));
+	    !(hp = gethostbyaddr((char *)&ina, 4, AF_INET)))
+		return inet_ntoa(ina);
 	else
-		(void)sprintf(buf, "%s (%s)", hp->h_name,
-		    inet_ntoa(*(struct in_addr *)&l));
+		(void)snprintf(buf, sizeof(buf), "%s (%s)", hp->h_name,
+		    inet_ntoa(ina));
 	return(buf);
 }
 
@@ -970,6 +1213,7 @@ pr_addr(l)
  * pr_retip --
  *	Dump some info on a returned (via ICMP) IP packet.
  */
+static void
 pr_retip(ip)
 	struct ip *ip;
 {
@@ -988,6 +1232,7 @@ pr_retip(ip)
 			(*cp * 256 + *(cp + 1)), (*(cp + 2) * 256 + *(cp + 3)));
 }
 
+static void
 fill(bp, patp)
 	char *bp, *patp;
 {
@@ -995,12 +1240,12 @@ fill(bp, patp)
 	int pat[16];
 	char *cp;
 
-	for (cp = patp; *cp; cp++)
-		if (!isxdigit(*cp)) {
-			(void)fprintf(stderr,
-			    "ping: patterns must be specified as hex digits.\n");
-			exit(1);
-		}
+	for (cp = patp; *cp; cp++) {
+		if (!isxdigit(*cp))
+			errx(EX_USAGE, 
+			     "patterns must be specified as hex digits");
+			
+	}
 	ii = sscanf(patp,
 	    "%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x",
 	    &pat[0], &pat[1], &pat[2], &pat[3], &pat[4], &pat[5], &pat[6],
@@ -1009,7 +1254,7 @@ fill(bp, patp)
 
 	if (ii > 0)
 		for (kk = 0;
-		    kk <= MAXPACKET - (8 + sizeof(struct timeval) + ii);
+		    kk <= MAXPACKET - (8 + PHDR_LEN + ii);
 		    kk += ii)
 			for (jj = 0; jj < ii; ++jj)
 				bp[jj + kk] = pat[jj];
@@ -1021,9 +1266,11 @@ fill(bp, patp)
 	}
 }
 
+static void
 usage()
 {
-	(void)fprintf(stderr,
-	    "usage: ping [-Rdfnqrv] [-c count] [-i wait] [-l preload]\n\t[-p pattern] [-s packetsize] host\n");
-	exit(1);
+	fprintf(stderr, "%s\n%s\n",
+"usage: ping [-QRadfnqrv] [-c count] [-i wait] [-l preload] [-p pattern]",
+"            [-s packetsize] [host | [-L] [-I iface] [-T ttl] mcast-group]");
+	exit(EX_USAGE);
 }
