@@ -306,7 +306,7 @@ pathsearch(pathname)
 	while (*path == '/')
 		path++;
 	dp = NULL;
-	while ((name = strsep(&path, "/")) != NULL && *name != NULL) {
+	while ((name = strsep(&path, "/")) != NULL && *name != '\0') {
 		if ((dp = searchdir(ino, name)) == NULL)
 			return (NULL);
 		ino = dp->d_ino;
@@ -728,10 +728,10 @@ allocinotab(ino, dip, seekpt)
 	if (mf == NULL)
 		return (itp);
 	node.ino = ino;
-	node.timep[0].tv_sec = dip->di_atime.ts_sec;
-	node.timep[0].tv_usec = dip->di_atime.ts_nsec / 1000;
-	node.timep[1].tv_sec = dip->di_mtime.ts_sec;
-	node.timep[1].tv_usec = dip->di_mtime.ts_nsec / 1000;
+	node.timep[0].tv_sec = dip->di_atime.tv_sec;
+	node.timep[0].tv_usec = dip->di_atime.tv_nsec / 1000;
+	node.timep[1].tv_sec = dip->di_mtime.tv_sec;
+	node.timep[1].tv_usec = dip->di_mtime.tv_nsec / 1000;
 	node.mode = dip->di_mode;
 	node.uid = dip->di_uid;
 	node.gid = dip->di_gid;
@@ -757,7 +757,7 @@ inotablookup(ino)
 /*
  * Clean up and exit
  */
-__dead void
+void
 done(exitcode)
 	int exitcode;
 {
