@@ -231,7 +231,9 @@ setup()
 		fprintf(stderr, "cannot stat .: %s\n", strerror(errno));
 		done(1);
 	}
-	if (stbuf.st_blksize > 0 && stbuf.st_blksize <= MAXBSIZE)
+	if (stbuf.st_blksize > 0 && stbuf.st_blksize < TP_BSIZE )
+		fssize = TP_BSIZE;
+	if (stbuf.st_blksize >= TP_BSIZE && stbuf.st_blksize <= MAXBSIZE)
 		fssize = stbuf.st_blksize;
 	if (((fssize - 1) & fssize) != 0) {
 		fprintf(stderr, "bad block size %d\n", fssize);
@@ -274,7 +276,7 @@ setup()
  * Prompt user to load a new dump volume.
  * "Nextvol" is the next suggested volume to use.
  * This suggested volume is enforced when doing full
- * or incremental restores, but can be overrridden by
+ * or incremental restores, but can be overridden by
  * the user when only extracting a subset of the files.
  */
 void
@@ -315,7 +317,7 @@ again:
 			    "Unless you know which volume your",
 			    " file(s) are on you should start\n",
 			    "with the last volume and work",
-			    " towards towards the first.\n");
+			    " towards the first.\n");
 		} else {
 			fprintf(stderr, "You have read volumes");
 			strcpy(buf, ": ");
@@ -510,10 +512,10 @@ extractfile(name)
 
 	curfile.name = name;
 	curfile.action = USING;
-	timep[0].tv_sec = curfile.dip->di_atime.ts_sec;
-	timep[0].tv_usec = curfile.dip->di_atime.ts_nsec / 1000;
-	timep[1].tv_sec = curfile.dip->di_mtime.ts_sec;
-	timep[1].tv_usec = curfile.dip->di_mtime.ts_nsec / 1000;
+	timep[0].tv_sec = curfile.dip->di_atime.tv_sec;
+	timep[0].tv_usec = curfile.dip->di_atime.tv_nsec / 1000;
+	timep[1].tv_sec = curfile.dip->di_mtime.tv_sec;
+	timep[1].tv_usec = curfile.dip->di_mtime.tv_nsec / 1000;
 	mode = curfile.dip->di_mode;
 	switch (mode & IFMT) {
 
@@ -550,6 +552,8 @@ extractfile(name)
 		return (linkit(lnkbuf, name, SYMLINK));
 
 	case IFIFO:
+		if (uflag && !Nflag)
+			(void)unlink(name);
 		if (mkfifo(name, mode) < 0) {
 			fprintf(stderr, "%s: cannot create FIFO: %s\n",
 				name, strerror(errno));
@@ -569,6 +573,8 @@ extractfile(name)
 			skipfile();
 			return (GOOD);
 		}
+		if (uflag)
+			(void)unlink(name);
 		if (mknod(name, mode, (int)curfile.dip->di_rdev) < 0) {
 			fprintf(stderr, "%s: cannot create special file: %s\n",
 			    name, strerror(errno));
@@ -587,6 +593,8 @@ extractfile(name)
 			skipfile();
 			return (GOOD);
 		}
+		if (uflag)
+			(void)unlink(name);
 		if ((ofile = open(name, O_WRONLY | O_CREAT | O_TRUNC,
 		    0666)) < 0) {
 			fprintf(stderr, "%s: cannot create file: %s\n",
@@ -1022,8 +1030,11 @@ gethead(buf)
 		}
 		if (checksum((int *)buf) == FAIL)
 			return (FAIL);
-		if (Bcvt)
+		if (Bcvt) {
 			swabst((u_char *)"8l4s31l", (u_char *)buf);
+			swabst((u_char *)"l",(u_char *) &buf->c_level);
+			swabst((u_char *)"2l",(u_char *) &buf->c_flags);
+		}
 		goto good;
 	}
 	readtape((char *)(&u_ospcl.s_ospcl));
@@ -1042,9 +1053,9 @@ gethead(buf)
 	buf->c_dinode.di_gid = u_ospcl.s_ospcl.c_dinode.odi_gid;
 	buf->c_dinode.di_size = u_ospcl.s_ospcl.c_dinode.odi_size;
 	buf->c_dinode.di_rdev = u_ospcl.s_ospcl.c_dinode.odi_rdev;
-	buf->c_dinode.di_atime.ts_sec = u_ospcl.s_ospcl.c_dinode.odi_atime;
-	buf->c_dinode.di_mtime.ts_sec = u_ospcl.s_ospcl.c_dinode.odi_mtime;
-	buf->c_dinode.di_ctime.ts_sec = u_ospcl.s_ospcl.c_dinode.odi_ctime;
+	buf->c_dinode.di_atime.tv_sec = u_ospcl.s_ospcl.c_dinode.odi_atime;
+	buf->c_dinode.di_mtime.tv_sec = u_ospcl.s_ospcl.c_dinode.odi_mtime;
+	buf->c_dinode.di_ctime.tv_sec = u_ospcl.s_ospcl.c_dinode.odi_ctime;
 	buf->c_count = u_ospcl.s_ospcl.c_count;
 	bcopy(u_ospcl.s_ospcl.c_addr, buf->c_addr, (long)256);
 	if (u_ospcl.s_ospcl.c_magic != OFS_MAGIC ||
