@@ -145,7 +145,7 @@ newnode(np)
 	if (np->e_type != NODE)
 		badentry(np, "newnode: not a node");
 	cp = myname(np);
-	if (!Nflag && mkdir(cp, 0777) < 0) {
+	if (!Nflag && mkdir(cp, 0777) < 0 && !uflag) {
 		np->e_flags |= EXISTED;
 		fprintf(stderr, "warning: %s: %s\n", cp, strerror(errno));
 		return;
@@ -206,6 +206,10 @@ linkit(existing, new, type)
 	int type;
 {
 
+	/* if we want to unlink first, do it now so *link() won't fail */
+	if (uflag && !Nflag)
+		(void)unlink(new);
+
 	if (type == SYMLINK) {
 		if (!Nflag && symlink(existing, new) < 0) {
 			fprintf(stderr,
@@ -214,11 +218,26 @@ linkit(existing, new, type)
 			return (FAIL);
 		}
 	} else if (type == HARDLINK) {
-		if (!Nflag && link(existing, new) < 0) {
-			fprintf(stderr,
-			    "warning: cannot create hard link %s->%s: %s\n",
-			    new, existing, strerror(errno));
-			return (FAIL);
+		int ret;
+
+		if (!Nflag && (ret = link(existing, new)) < 0) {
+			struct stat s;
+
+			/*
+			 * Most likely, the schg flag is set.  Clear the
+			 * flags and try again.
+			 */
+			if (stat(existing, &s) == 0 && s.st_flags != 0 &&
+			    chflags(existing, 0) == 0) {
+				ret = link(existing, new);
+				chflags(existing, s.st_flags);
+			}
+			if (ret < 0) {
+				fprintf(stderr, "warning: cannot create "
+				    "hard link %s->%s: %s\n",
+				    new, existing, strerror(errno));
+				return (FAIL);
+			}
 		}
 	} else {
 		panic("linkit: unknown type %d\n", type);
