@@ -33,7 +33,7 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char copyright[] =
 "@(#) Copyright (c) 1983, 1989, 1991, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
 #endif /* not lint */
@@ -43,23 +43,23 @@ static char copyright[] =
 static char sccsid[] = "@(#)route.c	8.3 (Berkeley) 3/19/94";
 */
 static const char rcsid[] =
-  "$FreeBSD: stable/2.1/sbin/route/route.c 50952 1999-09-05 11:54:50Z peter $";
+  "$FreeBSD: stable/2.2/sbin/route/route.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <sys/mbuf.h>
 #include <sys/sysctl.h>
 
 #include <net/if.h>
 #include <net/route.h>
 #include <net/if_dl.h>
 #include <netinet/in.h>
+#include <netatalk/at.h>
+#ifdef NS
 #include <netns/ns.h>
-#include <netiso/iso.h>
-#include <netccitt/x25.h>
+#endif
 #include <arpa/inet.h>
 #include <netdb.h>
 
@@ -70,6 +70,8 @@ static const char rcsid[] =
 #include <stdlib.h>
 #include <string.h>
 #include <paths.h>
+#include <err.h>
+#include <sysexits.h>
 
 struct keytab {
 	char	*kt_cp;
@@ -83,10 +85,11 @@ struct	ortentry route;
 union	sockunion {
 	struct	sockaddr sa;
 	struct	sockaddr_in sin;
+	struct	sockaddr_at sat;
+#ifdef NS
 	struct	sockaddr_ns sns;
-	struct	sockaddr_iso siso;
+#endif
 	struct	sockaddr_dl sdl;
-	struct	sockaddr_x25 sx25;
 } so_dst, so_gate, so_mask, so_genmask, so_ifa, so_ifp;
 
 typedef union sockunion *sup;
@@ -98,35 +101,25 @@ int	locking, lockrest, debugonly;
 struct	rt_metrics rt_metrics;
 u_long  rtm_inits;
 struct	in_addr inet_makeaddr();
+int	atalk_aton __P((const char *, struct at_addr *));
+char	*atalk_ntoa __P((struct at_addr));
 char	*routename(), *netname();
 void	flushroutes(), newroute(), monitor(), sockaddr(), sodump(), bprintf();
 void	print_getmsg(), print_rtmsg(), pmsg_common(), pmsg_addrs(), mask_addr();
 int	getaddr(), rtmsg(), x25_makemask();
 extern	char *inet_ntoa(), *iso_ntoa(), *link_ntoa();
 
-__dead void
-usage(cp)
-	char *cp;
-{
-	if (cp)
-		(void) fprintf(stderr, "route: botched keyword: %s\n", cp);
-	(void) fprintf(stderr,
-	    "usage: route [ -nqv ] cmd [[ -<qualifers> ] args ]\n");
-	exit(1);
-	/* NOTREACHED */
-}
+void usage __P((const char *)) __dead2;
 
 void
-quit(s)
-	char *s;
+usage(cp)
+	const char *cp;
 {
-	int sverrno = errno;
-
-	(void) fprintf(stderr, "route: ");
-	if (s)
-		(void) fprintf(stderr, "%s: ", s);
-	(void) fprintf(stderr, "%s\n", strerror(sverrno));
-	exit(1);
+	if (cp)
+		warnx("bad keyword: %s", cp);
+	(void) fprintf(stderr,
+	    "usage: route [ -dnqtv ] cmd [[ -<qualifers> ] args ]\n");
+	exit(EX_USAGE);
 	/* NOTREACHED */
 }
 
@@ -145,7 +138,7 @@ main(argc, argv)
 	if (argc < 2)
 		usage((char *)NULL);
 
-	while ((ch = getopt(argc, argv, "nqdtv")) != EOF)
+	while ((ch = getopt(argc, argv, "nqdtv")) !=  -1)
 		switch(ch) {
 		case 'n':
 			nflag = 1;
@@ -176,7 +169,7 @@ main(argc, argv)
 	else
 		s = socket(PF_ROUTE, SOCK_RAW, 0);
 	if (s < 0)
-		quit("socket");
+		err(EX_OSERR, "socket");
 	setuid(uid);
 	if (*argv)
 		switch (keyword(*argv)) {
@@ -219,8 +212,7 @@ flushroutes(argc, argv)
 	register struct rt_msghdr *rtm;
 
 	if (uid) {
-		errno = EACCES;
-		quit("must be root to alter routing table");
+		errx(EX_NOPERM, "must be root to alter routing table");
 	}
 	shutdown(s, 0); /* Don't want to read back our messages */
 	if (argc > 1) {
@@ -230,18 +222,17 @@ flushroutes(argc, argv)
 			case K_INET:
 				af = AF_INET;
 				break;
+			case K_ATALK:
+				af = AF_APPLETALK;
+				break;
+#ifdef NS
 			case K_XNS:
 				af = AF_NS;
 				break;
+#endif
 			case K_LINK:
 				af = AF_LINK;
 				break;
-			case K_ISO:
-			case K_OSI:
-				af = AF_ISO;
-				break;
-			case K_X25:
-				af = AF_CCITT;
 			default:
 				goto bad;
 		} else
@@ -254,11 +245,11 @@ bad:			usage(*argv);
 	mib[4] = NET_RT_DUMP;
 	mib[5] = 0;		/* no flags */
 	if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
-		quit("route-sysctl-estimate");
+		err(EX_OSERR, "route-sysctl-estimate");
 	if ((buf = malloc(needed)) == NULL)
-		quit("malloc");
+		errx(EX_OSERR, "malloc failed");
 	if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0)
-		quit("actual retrieval of routing table");
+		err(EX_OSERR, "route-sysctl-get");
 	lim = buf + needed;
 	if (verbose)
 		(void) printf("Examining routing table from sysctl\n");
@@ -281,9 +272,7 @@ bad:			usage(*argv);
 		rtm->rtm_seq = seqno;
 		rlen = write(s, next, rtm->rtm_msglen);
 		if (rlen < (int)rtm->rtm_msglen) {
-			(void) fprintf(stderr,
-			    "route: write to routing socket: %s\n",
-			    strerror(errno));
+			warn("write to routing socket");
 			(void) printf("got only %d for rlen\n", rlen);
 			break;
 		}
@@ -312,7 +301,9 @@ routename(sa)
 	struct hostent *hp;
 	static char domain[MAXHOSTNAMELEN + 1];
 	static int first = 1;
+#ifdef NS
 	char *ns_print();
+#endif
 
 	if (first) {
 		first = 0;
@@ -348,7 +339,8 @@ routename(sa)
 		if (cp)
 			strncpy(line, cp, sizeof line);
 		else {
-#define C(x)	((x) & 0xff)
+			/* XXX - why not inet_ntoa()? */
+#define C(x)	(unsigned)((x) & 0xff)
 			in.s_addr = ntohl(in.s_addr);
 			(void) sprintf(line, "%u.%u.%u.%u", C(in.s_addr >> 24),
 			   C(in.s_addr >> 16), C(in.s_addr >> 8), C(in.s_addr));
@@ -356,17 +348,19 @@ routename(sa)
 		break;
 	    }
 
+	case AF_APPLETALK:
+		(void) snprintf(line, sizeof(line), "atalk %s",
+			atalk_ntoa(((struct sockaddr_at *)sa)->sat_addr));
+		break;
+
+#ifdef NS
 	case AF_NS:
 		return (ns_print((struct sockaddr_ns *)sa));
+#endif
 
 	case AF_LINK:
 		return (link_ntoa((struct sockaddr_dl *)sa));
 
-	case AF_ISO:
-		(void) snprintf(line, sizeof line, "iso %s",
-		    iso_ntoa(&((struct sockaddr_iso *)sa)->siso_addr));
-		break;
-
 	default:
 	    {	u_short *s = (u_short *)sa;
 		u_short *slim = s + ((sa->sa_len + 1) >> 1);
@@ -395,7 +389,9 @@ netname(sa)
 	u_long net, mask;
 	register u_long i;
 	int subnetshift;
+#ifdef NS
 	char *ns_print();
+#endif
 
 	switch (sa->sa_family) {
 
@@ -449,17 +445,20 @@ netname(sa)
 		break;
 	    }
 
+	case AF_APPLETALK:
+		(void) snprintf(line, sizeof(line), "atalk %s",
+			atalk_ntoa(((struct sockaddr_at *)sa)->sat_addr));
+		break;
+
+#ifdef NS
 	case AF_NS:
 		return (ns_print((struct sockaddr_ns *)sa));
 		break;
+#endif
 
 	case AF_LINK:
 		return (link_ntoa((struct sockaddr_dl *)sa));
 
-	case AF_ISO:
-		(void) snprintf(line, sizeof(line), "iso %s",
-		    iso_ntoa(&((struct sockaddr_iso *)sa)->siso_addr));
-		break;
 
 	default:
 	    {	u_short *s = (u_short *)sa->sa_data;
@@ -513,8 +512,7 @@ newroute(argc, argv)
 	struct hostent *hp = 0;
 
 	if (uid) {
-		errno = EACCES;
-		quit("must be root to alter routing table");
+		errx(EX_NOPERM, "must be root to alter routing table");
 	}
 	cmd = argv[0];
 	if (*cmd != 'g')
@@ -526,27 +524,24 @@ newroute(argc, argv)
 				af = AF_LINK;
 				aflen = sizeof(struct sockaddr_dl);
 				break;
-			case K_OSI:
-			case K_ISO:
-				af = AF_ISO;
-				aflen = sizeof(struct sockaddr_iso);
-				break;
 			case K_INET:
 				af = AF_INET;
 				aflen = sizeof(struct sockaddr_in);
 				break;
-			case K_X25:
-				af = AF_CCITT;
-				aflen = sizeof(struct sockaddr_x25);
+			case K_ATALK:
+				af = AF_APPLETALK;
+				aflen = sizeof(struct sockaddr_at);
 				break;
 			case K_SA:
 				af = PF_ROUTE;
 				aflen = sizeof(union sockunion);
 				break;
+#ifdef NS
 			case K_XNS:
 				af = AF_NS;
 				aflen = sizeof(struct sockaddr_ns);
 				break;
+#endif
 			case K_IFACE:
 			case K_INTERFACE:
 				iflag++;
@@ -554,6 +549,9 @@ newroute(argc, argv)
 			case K_NOSTATIC:
 				flags &= ~RTF_STATIC;
 				break;
+			case K_LLINFO:
+				flags |= RTF_LLINFO;
+				break;
 			case K_LOCK:
 				locking = 1;
 				break;
@@ -634,23 +632,6 @@ newroute(argc, argv)
 				gateway = *argv;
 				(void) getaddr(RTA_GATEWAY, *argv, &hp);
 			} else {
-				int ret = atoi(*argv);
-
-				if (ret == 0) {
-				    if (strcmp(*argv, "0") == 0)
-				        printf("%s,%s",
-					    "old usage of trailing 0",
-					    "assuming route to if\n");
-				    else
-					usage((char *)NULL);
-				    iflag = 1;
-				    continue;
-				} else if (ret > 0 && ret < 10) {
-				    printf("old usage of trailing digit, ");
-				    printf("assuming route via gateway\n");
-				    iflag = 0;
-				    continue;
-				}
 				(void) getaddr(RTA_NETMASK, *argv, 0);
 			}
 		}
@@ -709,8 +690,8 @@ newroute(argc, argv)
 }
 
 void
-inet_makenetandmask(net, sin)
-	u_long net;
+inet_makenetandmask(net, sin, bits)
+	u_long net, bits;
 	register struct sockaddr_in *sin;
 {
 	u_long addr, mask = 0;
@@ -719,7 +700,10 @@ inet_makenetandmask(net, sin)
 	rtm_addrs |= RTA_NETMASK;
 	if (net == 0)
 		mask = addr = 0;
-	else if (net < 128) {
+	else if (bits) {
+		addr = net;
+		mask = 0xffffffff << (32 - bits);
+	} else if (net < 128) {
 		addr = net << IN_CLASSA_NSHIFT;
 		mask = IN_CLASSA_NET;
 	} else if (net < 65536) {
@@ -761,11 +745,13 @@ getaddr(which, s, hpp)
 	struct hostent **hpp;
 {
 	register sup su;
+#ifdef NS
 	struct ns_addr ns_addr();
-	struct iso_addr *iso_addr();
+#endif
 	struct hostent *hp;
 	struct netent *np;
 	u_long val;
+	char *q,qs;
 
 	if (af == 0) {
 		af = AF_INET;
@@ -775,11 +761,51 @@ getaddr(which, s, hpp)
 	switch (which) {
 	case RTA_DST:
 		su = &so_dst;
-		su->sa.sa_family = af;
 		break;
 	case RTA_GATEWAY:
 		su = &so_gate;
-		su->sa.sa_family = af;
+		if (iflag) {
+			#define MAX_IFACES	400
+			int			sock;
+			struct ifreq		iflist[MAX_IFACES];
+			struct ifconf		ifconf;
+			struct ifreq		*ifr, *ifr_end;
+			struct sockaddr_dl	*dl, *sdl = NULL;
+
+			/* Get socket */
+			if ((sock = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
+				err(1, "socket");
+
+			/* Get interface list */
+			ifconf.ifc_req = iflist;
+			ifconf.ifc_len = sizeof(iflist);
+			if (ioctl(sock, SIOCGIFCONF, &ifconf) < 0)
+				err(1, "ioctl(SIOCGIFCONF)");
+			close(sock);
+
+			/* Look for this interface in the list */
+			for (ifr = ifconf.ifc_req,
+			    ifr_end = (struct ifreq *)
+				(ifconf.ifc_buf + ifconf.ifc_len);
+			    ifr < ifr_end;
+			    ifr = (struct ifreq *) ((char *) &ifr->ifr_addr
+						    + ifr->ifr_addr.sa_len)) {
+				dl = (struct sockaddr_dl *)&ifr->ifr_addr;
+				if (ifr->ifr_addr.sa_family == AF_LINK
+				    && (ifr->ifr_flags & IFF_POINTOPOINT)
+				    && !strncmp(s, dl->sdl_data, dl->sdl_nlen)
+				    && s[dl->sdl_nlen] == 0) {
+					sdl = dl;
+					break;
+				}
+			}
+
+			/* If we found it, then use it */
+			if (sdl) {
+				su->sdl = *sdl;
+				return(1);
+			}
+		}
 		break;
 	case RTA_NETMASK:
 		su = &so_mask;
@@ -789,30 +815,34 @@ getaddr(which, s, hpp)
 		break;
 	case RTA_IFP:
 		su = &so_ifp;
-		su->sa.sa_family = af;
 		break;
 	case RTA_IFA:
 		su = &so_ifa;
-		su->sa.sa_family = af;
 		break;
 	default:
-		usage("Internal Error");
+		usage("internal Error");
 		/*NOTREACHED*/
 	}
 	su->sa.sa_len = aflen;
+	su->sa.sa_family = af; /* cases that don't want it have left already */
 	if (strcmp(s, "default") == 0) {
+		/*
+		 * Default is net 0.0.0.0/0 
+		 */
 		switch (which) {
 		case RTA_DST:
 			forcenet++;
+			/* bzero(su, sizeof(*su)); *//* for readability */
 			(void) getaddr(RTA_NETMASK, s, 0);
 			break;
 		case RTA_NETMASK:
 		case RTA_GENMASK:
-			su->sa.sa_len = 0;
+			/* bzero(su, sizeof(*su)); *//* for readability */
 		}
 		return (0);
 	}
 	switch (af) {
+#ifdef NS
 	case AF_NS:
 		if (which == RTA_DST) {
 			extern short ns_bh[3];
@@ -825,24 +855,19 @@ getaddr(which, s, hpp)
 		}
 		su->sns.sns_addr = ns_addr(s);
 		return (!ns_nullhost(su->sns.sns_addr));
+#endif
 
-	case AF_OSI:
-		su->siso.siso_addr = *iso_addr(s);
-		if (which == RTA_NETMASK || which == RTA_GENMASK) {
-			register char *cp = (char *)TSEL(&su->siso);
-			su->siso.siso_nlen = 0;
-			do {--cp ;} while ((cp > (char *)su) && (*cp == 0));
-			su->siso.siso_len = 1 + cp - (char *)su;
-		}
-		return (1);
+
+	case AF_APPLETALK:
+		if (!atalk_aton(s, &su->sat.sat_addr))
+			errx(EX_NOHOST, "bad address: %s", s);
+		rtm_addrs |= RTA_NETMASK;
+		return(forcehost || su->sat.sat_addr.s_node != 0);
 
 	case AF_LINK:
 		link_addr(s, &su->sdl);
 		return (1);
 
-	case AF_CCITT:
-		ccitt_addr(s, &su->sx25);
-		return (which == RTA_DST ? x25_makemask() : 1);
 
 	case PF_ROUTE:
 		su->sa.sa_len = sizeof(*su);
@@ -857,7 +882,19 @@ getaddr(which, s, hpp)
 	if (hpp == NULL)
 		hpp = &hp;
 	*hpp = NULL;
-	if (((val = inet_addr(s)) != -1) &&
+
+	q = strchr(s,'/');
+	if (q && which == RTA_DST) {
+		qs = *q;
+		*q = '\0';
+		if (((val = inet_addr(s)) != INADDR_NONE)) {
+			inet_makenetandmask(
+				htonl(val), &su->sin, strtoul(q+1, 0, 0));
+			return (0);
+		}
+		*q =qs;
+	}
+	if (((val = inet_addr(s)) != INADDR_NONE) &&
 	    (which != RTA_DST || forcenet == 0)) {
 		su->sin.sin_addr.s_addr = val;
 		if (inet_lnaof(su->sin.sin_addr) != INADDR_ANY)
@@ -867,12 +904,12 @@ getaddr(which, s, hpp)
 			goto netdone;
 		}
 	}
-	if ((val = inet_network(s)) != -1 ||
+	if ((val = inet_network(s)) != INADDR_NONE ||
 	    (forcehost == 0 && (np = getnetbyname(s)) != NULL &&
 		    (val = np->n_net) != 0)) {
 netdone:
 		if (which == RTA_DST)
-			inet_makenetandmask(val, &su->sin);
+			inet_makenetandmask(val, &su->sin, 0);
 		return (0);
 	}
 	hp = gethostbyname(s);
@@ -882,25 +919,11 @@ netdone:
 		bcopy(hp->h_addr, (char *)&su->sin.sin_addr, hp->h_length);
 		return (1);
 	}
-	(void) fprintf(stderr, "%s: bad value\n", s);
-	exit(1);
+	errx(EX_NOHOST, "bad address: %s", s);
 }
 
-int
-x25_makemask()
-{
-	register char *cp;
-
-	if ((rtm_addrs & RTA_NETMASK) == 0) {
-		rtm_addrs |= RTA_NETMASK;
-		for (cp = (char *)&so_mask.sx25.x25_net;
-		     cp < &so_mask.sx25.x25_opts.op_flags; cp++)
-			*cp = -1;
-		so_mask.sx25.x25_len = (u_char)&(((sup)0)->sx25.x25_opts);
-	}
-	return 0;
-}
 
+#ifdef NS
 short ns_nullh[] = {0,0,0};
 short ns_bh[] = {-1,-1,-1};
 
@@ -944,10 +967,12 @@ ns_print(sns)
 	else
 		*cport = 0;
 
-	(void) snprintf(mybuf, sizeof mybuf, "%XH.%s%s",
-			ntohl(net.long_e), host, cport);
+	(void) snprintf(mybuf, sizeof(mybuf), "%lxH.%s%s", 
+                    (unsigned long)ntohl(net.long_e),
+		       host, cport);
 	return (mybuf);
 }
+#endif
 
 void
 interfaces()
@@ -964,11 +989,11 @@ interfaces()
 	mib[4] = NET_RT_IFLIST;
 	mib[5] = 0;		/* no flags */
 	if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
-		quit("route-sysctl-estimate");
+		err(EX_OSERR, "route-sysctl-estimate");
 	if ((buf = malloc(needed)) == NULL)
-		quit("malloc");
+		errx(EX_OSERR, "malloc failed");
 	if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0)
-		quit("actual retrieval of interface table");
+		err(EX_OSERR, "actual retrieval of interface table");
 	lim = buf + needed;
 	for (next = buf; next < lim; next += rtm->rtm_msglen) {
 		rtm = (struct rt_msghdr *)next;
@@ -1023,8 +1048,8 @@ rtmsg(cmd, flags)
 	else if (cmd == 'g') {
 		cmd = RTM_GET;
 		if (so_ifp.sa.sa_family == 0) {
-			so_ifp.sa.sa_family == AF_LINK;
-			so_ifp.sa.sa_len == sizeof(struct sockaddr_dl);
+			so_ifp.sa.sa_family = AF_LINK;
+			so_ifp.sa.sa_len = sizeof(struct sockaddr_dl);
 			rtm_addrs |= RTA_IFP;
 		}
 	} else
@@ -1060,9 +1085,7 @@ rtmsg(cmd, flags)
 			l = read(s, (char *)&m_rtmsg, sizeof(m_rtmsg));
 		} while (l > 0 && (rtm.rtm_seq != seq || rtm.rtm_pid != pid));
 		if (l < 0)
-			(void) fprintf(stderr,
-			    "route: read from routing socket: %s\n",
-			    strerror(errno));
+			warn("read from routing socket");
 		else
 			print_getmsg(&rtm, l);
 	}
@@ -1084,15 +1107,13 @@ mask_addr()
 	if ((rtm_addrs & RTA_DST) == 0)
 		return;
 	switch (so_dst.sa.sa_family) {
+#ifdef NS
 	case AF_NS:
+#endif
 	case AF_INET:
-	case AF_CCITT:
+	case AF_APPLETALK:
 	case 0:
 		return;
-	case AF_ISO:
-		olen = MIN(so_dst.siso.siso_nlen,
-			   MAX(so_mask.sa.sa_len - 6, 0));
-		break;
 	}
 	cp1 = so_mask.sa.sa_len + 1 + (char *)&so_dst;
 	cp2 = so_dst.sa.sa_len + 1 + (char *)&so_dst;
@@ -1101,11 +1122,6 @@ mask_addr()
 	cp2 = so_mask.sa.sa_len + 1 + (char *)&so_mask;
 	while (cp1 > so_dst.sa.sa_data)
 		*--cp1 &= *--cp2;
-	switch (so_dst.sa.sa_family) {
-	case AF_ISO:
-		so_dst.siso.siso_nlen = olen;
-		break;
-	}
 }
 
 char *msgtypes[] = {
@@ -1128,11 +1144,17 @@ char *msgtypes[] = {
 };
 
 char metricnames[] =
-"\011pksent\010rttvar\7rtt\6ssthresh\5sendpipe\4recvpipe\3expire\2hopcount\1mtu";
+"\011pksent\010rttvar\7rtt\6ssthresh\5sendpipe\4recvpipe\3expire\2hopcount"
+"\1mtu";
 char routeflags[] =
-"\1UP\2GATEWAY\3HOST\4REJECT\5DYNAMIC\6MODIFIED\7DONE\010MASK_PRESENT\011CLONING\012XRESOLVE\013LLINFO\014STATIC\017PROTO2\020PROTO1";
+"\1UP\2GATEWAY\3HOST\4REJECT\5DYNAMIC\6MODIFIED\7DONE\010MASK_PRESENT"
+"\011CLONING\012XRESOLVE\013LLINFO\014STATIC\015BLACKHOLE\016b016"
+"\017PROTO2\020PROTO1\021PRCLONING\022WASCLONED\023PROTO3\024CHAINDELETE"
+"\025PINNED\026LOCAL\027BROADCAST\030MULTICAST";
 char ifnetflags[] =
-"\1UP\2BROADCAST\3DEBUG\4LOOPBACK\5PTP\6NOTRAILERS\7RUNNING\010NOARP\011PPROMISC\012ALLMULTI\013OACTIVE\014SIMPLEX\015LINK0\016LINK1\017LINK2\020MULTICAST";
+"\1UP\2BROADCAST\3DEBUG\4LOOPBACK\5PTP\6b6\7RUNNING\010NOARP"
+"\011PPROMISC\012ALLMULTI\013OACTIVE\014SIMPLEX\015LINK0\016LINK1"
+"\017LINK2\020MULTICAST";
 char addrnames[] =
 "\1DST\2GATEWAY\3NETMASK\4GENMASK\5IFP\6IFA\7AUTHOR\010BRD";
 
@@ -1167,8 +1189,8 @@ print_rtmsg(rtm, msglen)
 		pmsg_addrs((char *)(ifam + 1), ifam->ifam_addrs);
 		break;
 	default:
-		(void) printf("pid: %d, seq %d, errno %d, flags:",
-			rtm->rtm_pid, rtm->rtm_seq, rtm->rtm_errno);
+		(void) printf("pid: %ld, seq %d, errno %d, flags:",
+			(long)rtm->rtm_pid, rtm->rtm_seq, rtm->rtm_errno);
 		bprintf(stdout, rtm->rtm_flags, routeflags);
 		pmsg_common(rtm);
 	}
@@ -1187,19 +1209,17 @@ print_getmsg(rtm, msglen)
 
 	(void) printf("   route to: %s\n", routename(&so_dst));
 	if (rtm->rtm_version != RTM_VERSION) {
-		(void)fprintf(stderr,
-		    "routing message version %d not understood\n",
-		    rtm->rtm_version);
+		warnx("routing message version %d not understood",
+		     rtm->rtm_version);
 		return;
 	}
 	if (rtm->rtm_msglen > msglen) {
-		(void)fprintf(stderr,
-		    "message length mismatch, in packet %d, returned %d\n",
-		    rtm->rtm_msglen, msglen);
+		warnx("message length mismatch, in packet %d, returned %d",
+		      rtm->rtm_msglen, msglen);
 	}
 	if (rtm->rtm_errno)  {
-		(void) fprintf(stderr, "RTM_GET: %s (errno %d)\n",
-		    strerror(rtm->rtm_errno), rtm->rtm_errno);
+		errno = rtm->rtm_errno;
+		warn("message indicates error %d", errno);
 		return;
 	}
 	cp = ((char *)(rtm + 1));
@@ -1249,16 +1269,16 @@ print_getmsg(rtm, msglen)
 
 	(void) printf("\n%s\n", "\
  recvpipe  sendpipe  ssthresh  rtt,msec    rttvar  hopcount      mtu     expire");
-	printf("%8d%c ", rtm->rtm_rmx.rmx_recvpipe, lock(RPIPE));
-	printf("%8d%c ", rtm->rtm_rmx.rmx_sendpipe, lock(SPIPE));
-	printf("%8d%c ", rtm->rtm_rmx.rmx_ssthresh, lock(SSTHRESH));
-	printf("%8d%c ", msec(rtm->rtm_rmx.rmx_rtt), lock(RTT));
-	printf("%8d%c ", msec(rtm->rtm_rmx.rmx_rttvar), lock(RTTVAR));
-	printf("%8d%c ", rtm->rtm_rmx.rmx_hopcount, lock(HOPCOUNT));
-	printf("%8d%c ", rtm->rtm_rmx.rmx_mtu, lock(MTU));
+	printf("%8ld%c ", rtm->rtm_rmx.rmx_recvpipe, lock(RPIPE));
+	printf("%8ld%c ", rtm->rtm_rmx.rmx_sendpipe, lock(SPIPE));
+	printf("%8ld%c ", rtm->rtm_rmx.rmx_ssthresh, lock(SSTHRESH));
+	printf("%8ld%c ", msec(rtm->rtm_rmx.rmx_rtt), lock(RTT));
+	printf("%8ld%c ", msec(rtm->rtm_rmx.rmx_rttvar), lock(RTTVAR));
+	printf("%8ld%c ", rtm->rtm_rmx.rmx_hopcount, lock(HOPCOUNT));
+	printf("%8ld%c ", rtm->rtm_rmx.rmx_mtu, lock(MTU));
 	if (rtm->rtm_rmx.rmx_expire)
 		rtm->rtm_rmx.rmx_expire -= time(0);
-	printf("%8d%c\n", rtm->rtm_rmx.rmx_expire, lock(EXPIRE));
+	printf("%8ld%c\n", rtm->rtm_rmx.rmx_expire, lock(EXPIRE));
 #undef lock
 #undef msec
 #define	RTA_IGN	(RTA_DST|RTA_GATEWAY|RTA_NETMASK|RTA_IFP|RTA_IFA|RTA_BRD)
@@ -1317,7 +1337,7 @@ bprintf(fp, b, s)
 
 	if (b == 0)
 		return;
-	while (i = *s++) {
+	while ((i = *s++) != 0) {
 		if (b & (1 << (i-1))) {
 			if (gotsome == 0)
 				i = '<';
@@ -1356,18 +1376,20 @@ sodump(su, which)
 		(void) printf("%s: link %s; ",
 		    which, link_ntoa(&su->sdl));
 		break;
-	case AF_ISO:
-		(void) printf("%s: iso %s; ",
-		    which, iso_ntoa(&su->siso.siso_addr));
-		break;
 	case AF_INET:
 		(void) printf("%s: inet %s; ",
 		    which, inet_ntoa(su->sin.sin_addr));
 		break;
+	case AF_APPLETALK:
+		(void) printf("%s: atalk %s; ",
+		    which, atalk_ntoa(su->sat.sat_addr));
+		break;
+#ifdef NS
 	case AF_NS:
 		(void) printf("%s: xns %s; ",
 		    which, ns_ntoa(su->sns.sns_addr));
 		break;
+#endif
 	}
 	(void) fflush(stdout);
 }
@@ -1389,7 +1411,7 @@ sockaddr(addr, sa)
 	register char *cp = (char *)sa;
 	int size = sa->sa_len;
 	char *cplim = cp + size;
-	register int byte = 0, state = VIRGIN, new;
+	register int byte = 0, state = VIRGIN, new = 0 /* foil gcc */;
 
 	bzero(cp, size);
 	cp++;
@@ -1424,3 +1446,25 @@ sockaddr(addr, sa)
 	} while (cp < cplim);
 	sa->sa_len = cp - (char *)sa;
 }
+
+int
+atalk_aton(const char *text, struct at_addr *addr)
+{
+	u_int net, node;
+
+	if (sscanf(text, "%u.%u", &net, &node) != 2
+	    || net > 0xffff || node > 0xff)
+		return(0);
+	addr->s_net = htons(net);
+	addr->s_node = node;
+	return(1);
+}
+
+char *
+atalk_ntoa(struct at_addr at)
+{
+	static char buf[20];
+
+	(void) snprintf(buf, sizeof(buf), "%u.%u", ntohs(at.s_net), at.s_node);
+	return(buf);
+}
