@@ -39,7 +39,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sbin/scsi/scsi.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/sbin/scsi/scsi.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <stdio.h>
@@ -52,6 +52,7 @@
 #include <scsi.h>
 #include <ctype.h>
 #include <signal.h>
+#include <err.h>
 
 int	fd;
 int	debuglevel;
@@ -73,26 +74,15 @@ int seconds = 2;
 
 void usage(void)
 {
-	printf(
-
-"Usage:\n"
-"\n"
-"  scsi -f device -d debug_level                    # To set debug level\n"
-"  scsi -f device [-v] -z seconds                   # To freeze bus\n"
-"  scsi -f device -m page [-P pc]                   # To read mode pages\n"
-"  scsi -f device -p [-b bus] [-l lun]              # To probe all devices\n"
-"  scsi -f device -r [-b bus] [-t targ] [-l lun]    # To reprobe a device\n"
-"  scsi -f device [-v] [-s seconds] -c cmd_fmt [arg0 ... argn] # A command...\n"
-"                 -o count out_fmt [arg0 ... argn]  #   EITHER (data out)\n"
-"                 -i count in_fmt                   #   OR     (data in)\n"
-"\n"
-"\"out_fmt\" can be \"-\" to read output data from stdin;\n"
-"\"in_fmt\" can be \"-\" to write input data to stdout;\n"
-"\n"
-"If debugging is not compiled in the kernel, \"-d\" will have no effect\n"
-
-);
-
+	fprintf(stderr, "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
+	"usage: scsi -f device -d debug_level",
+	"       scsi -f device [-v] -z seconds",
+	"       scsi -f device -m page [-P pc] [-e]",
+	"       scsi -f device -p [-b bus] [-l lun]",
+	"       scsi -f device -r [-b bus] [-t targ] [-l lun]",
+	"       scsi -f device [-v] [-s seconds] -c cmd_fmt [arg0 ... argn]",
+	"                      -o count out_fmt [arg0 ... argn]",
+	"                      -i count in_fmt");
 	exit (1);
 }
 
@@ -108,7 +98,7 @@ void procargs(int *argc_p, char ***argv_p)
 	fflag = 0;
 	commandflag = 0;
 	debugflag = 0;
-	while ((ch = getopt(argc, argv, "ceprvf:d:b:t:l:z:m:P:s:")) != EOF) {
+	while ((ch = getopt(argc, argv, "ceprvf:d:b:t:l:z:m:P:s:")) != -1) {
 		switch (ch) {
 		case 'p':
 			probe_all = 1;
@@ -126,12 +116,8 @@ void procargs(int *argc_p, char ***argv_p)
 			editflag = 1;
 			break;
 		case 'f':
-			if ((fd = scsi_open(optarg, O_RDWR)) < 0) {
-				(void) fprintf(stderr,
-					  "%s: unable to open device %s: %s\n",
-					       argv[0], optarg, strerror(errno));
-				exit(errno);
-			}
+			if ((fd = scsi_open(optarg, O_RDWR)) < 0)
+				err(errno, "unable to open device %s", optarg);
 			fflag = 1;
 			break;
 		case 'd':
@@ -240,6 +226,8 @@ void arg_put(void *hook, int letter, void *arg, int count, char *name)
 		case 'z':
 		{
 			char *p = malloc(count + 1);
+			if (p == NULL)
+				errx(1, "malloc failed");
 			p[count] = 0;
 			strncpy(p, (char *)arg, count);
 			if (letter == 'z')
@@ -330,6 +318,8 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 			data_fmt = cget(&h, 0);
 
 			scsireq->databuf = malloc(count);
+			if (scsireq->databuf == NULL)
+				errx(1, "malloc failed");
 
 			if (data_phase == out)
 			{
@@ -343,8 +333,7 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 					}
 					if (amount == -1)
 					{
-						perror("read");
-						exit(errno);
+						err(1, "read");
 					}
 					else if (amount == 0)
 					{
@@ -389,13 +378,13 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 			}
 			if (amount < 0)
 			{
-				perror("write");
-				exit(errno);
+				err(1, "write");
 			}
 			else if (amount == 0)
-				fprintf(stderr, "Warning: wrote only %d bytes out of %d.\n",
-					scsireq->datalen - count,
-					scsireq->datalen);
+				fprintf(stderr,
+			    "Warning: wrote only %lu bytes out of %lu.\n",
+				    (u_long)scsireq->datalen - count,
+				    (u_long)scsireq->datalen);
 			
 		}
 		else
@@ -410,13 +399,11 @@ static void
 freeze_ioctl(int fd, int op, void *data)
 {
 	if (ioctl(fd, SCIOCFREEZE, 0) == -1) {
-		if (errno == ENODEV) {
-			fprintf(stderr,
-			"Your kernel must be configured with option SCSI_FREEZE.\n");
-		}
+		if (errno == ENODEV)
+			errx(1,
+		"your kernel must be configured with option SCSI_FREEZE");
 		else
-			perror("SCIOCFREEZE");
-		exit(errno);
+			err(1, "ioctl [SCIOCFREEZE]");
 	}
 }
 
@@ -471,7 +458,7 @@ void mode_sense(int fd, u_char *data, int len, int pc, int page)
 	if (SCSIREQ_ERROR(scsireq))
 	{
 		scsi_debug(stderr, 0, scsireq);
-		exit(-1);
+		exit(1);
 	}
 
 	free(scsireq);
@@ -494,7 +481,7 @@ void mode_select(int fd, u_char *data, int len, int perm)
 	if (SCSIREQ_ERROR(scsireq))
 	{
 		scsi_debug(stderr, 0, scsireq);
-		exit(-1);
+		exit(1);
 	}
 
 	free(scsireq);
@@ -556,16 +543,14 @@ static char *mode_lookup(int page)
 			found = 1;
 
 		skipwhite(modes);
-		if (getc(modes) != START_ENTRY) {
-			fprintf(stderr, "Expected %c.\n", START_ENTRY);
-			exit(-1);
-		}
+		if (getc(modes) != START_ENTRY)
+			errx(1, "expected %c", START_ENTRY);
 
 		match = 1;
 		while (match != 0) {
 			c = getc(modes);
 			if (c == EOF) {
-				fprintf(stderr, "Expected %c.\n", END_ENTRY);
+				warnx("expected %c", END_ENTRY);
 			}
 
 			if (c == START_ENTRY) {
@@ -577,10 +562,8 @@ static char *mode_lookup(int page)
 					break;
 			}
 			if (found && c != '\n') {
-				if (next >= sizeof(fmt)) {
-					fprintf(stderr, "Stupid program: Buffer overflow.\n");
-					exit(ENOMEM);
-				}
+				if (next >= sizeof(fmt))
+					errx(1, "buffer overflow");
 
 				fmt[next++] = (u_char)c;
 			}
@@ -628,9 +611,9 @@ edit_done(void)
 	if (opened)
 	{
 		if (fclose(edit_file))
-			perror(edit_name);
+			warn("%s", edit_name);
 		if (unlink(edit_name))
-			perror(edit_name);
+			warn("%s", edit_name);
 	}
 }
 
@@ -638,14 +621,10 @@ static void
 edit_init(void)
 {
 	edit_rewind();
-	if (tmpnam(edit_name) == 0) {
-		perror("tmpnam failed");
-		exit(errno);
-	}
-	if ( (edit_file = fopen(edit_name, "w")) == 0) {
-		perror(edit_name);
-		exit(errno);
-	}
+	if (tmpnam(edit_name) == 0)
+		errx(1, "tmpnam failed");
+	if ((edit_file = fopen(edit_name, "w")) == 0)
+		err(1, "%s", edit_name);
 	edit_opened = 1;
 
 	atexit(edit_done);
@@ -654,15 +633,12 @@ edit_init(void)
 static void
 edit_check(void *hook, int letter, void *arg, int count, char *name)
 {
-	if (letter != 'i' && letter != 'b') {
-		fprintf(stderr, "Can't edit format %c.\n", letter);
-		exit(-1);
-	}
+	if (letter != 'i' && letter != 'b')
+		errx(1, "can't edit format %c", letter);
+
+	if (editind >= sizeof(editinfo) / sizeof(editinfo[0]))
+		errx(1, "edit table overflow");
 
-	if (editind >= sizeof(editinfo) / sizeof(editinfo[0])) {
-		fprintf(stderr, "edit table overflow\n");
-		exit(ENOMEM);
-	}
 	editinfo[editind].can_edit = ((int)arg != 0);
 	editind++;
 }
@@ -670,10 +646,8 @@ edit_check(void *hook, int letter, void *arg, int count, char *name)
 static void
 edit_defaults(void *hook, int letter, void *arg, int count, char *name)
 {
-	if (letter != 'i' && letter != 'b') {
-		fprintf(stderr, "Can't edit format %c.\n", letter);
-		exit(-1);
-	}
+	if (letter != 'i' && letter != 'b')
+		errx(1, "can't edit format %c", letter);
 
 	editinfo[editind].default_value = ((int)arg);
 	editind++;
@@ -683,10 +657,8 @@ static void
 edit_report(void *hook, int letter, void *arg, int count, char *name)
 {
 	if (editinfo[editind].can_edit) {
-		if (letter != 'i' && letter != 'b') {
-			fprintf(stderr, "Can't report format %c.\n", letter);
-			exit(-1);
-		}
+		if (letter != 'i' && letter != 'b')
+			errx(1, "can't report format %c", letter);
 
 		fprintf(edit_file, "%s:  %d\n", name, (int)arg);
 	}
@@ -697,23 +669,17 @@ edit_report(void *hook, int letter, void *arg, int count, char *name)
 static int
 edit_get(void *hook, char *name)
 {
-	struct get_hook *h = (struct get_hook *)hook;
 	int arg = editinfo[editind].default_value;
 
 	if (editinfo[editind].can_edit) {
 		char line[80];
-		if (fgets(line, sizeof(line), edit_file) == 0) {
-			perror("fgets");
-			exit(errno);
-		}
+		if (fgets(line, sizeof(line), edit_file) == 0)
+			err(1, "fgets");
 
 		line[strlen(line) - 1] = 0;
 
-		if (strncmp(name, line, strlen(name)) != 0) {
-			fprintf(stderr, "Expected \"%s\" and read \"%s\"\n",
-			name, line);
-			exit(-1);
-		}
+		if (strncmp(name, line, strlen(name)) != 0)
+			errx(1, "expected \"%s\" and read \"%s\"", name, line);
 
 		arg = strtoul(line + strlen(name) + 2, 0, 0);
 	}
@@ -733,14 +699,14 @@ edit_edit(void)
 	fclose(edit_file);
 
 	system_line = malloc(strlen(editor) + strlen(edit_name) + 6);
+	if (system_line == NULL)
+		errx(1, "malloc failed");
 	sprintf(system_line, "%s %s", editor, edit_name);
 	system(system_line);
 	free(system_line);
 
-	if ( (edit_file = fopen(edit_name, "r")) == 0) {
-		perror(edit_name);
-		exit(errno);
-	}
+	if ((edit_file = fopen(edit_name, "r")) == 0)
+		err(1, "%s", edit_name);
 }
 
 static void
@@ -774,16 +740,12 @@ mode_edit(int fd, int page, int edit, int argc, char *argv[])
 	}
 
 	if (edit) {
-		if (!fmt) {
-			fprintf(stderr, "Sorry: can't edit without a format.\n");
-			exit(-1);
-		}
+		if (!fmt)
+			errx(1, "can't edit without a format");
 
-		if (pagectl != 0 && pagectl != 3) {
-			fprintf(stderr,
-"It only makes sense to edit page 0 (current) or page 3 (saved values)\n");
-			exit(-1);
-		}
+		if (pagectl != 0 && pagectl != 3)
+			errx(1,
+"it only makes sense to edit page 0 (current) or page 3 (saved values)");
 
 		verbose = 1;
 
@@ -820,7 +782,7 @@ mode_edit(int fd, int page, int edit, int argc, char *argv[])
 		bcopy((char *)mph, ((char *)mh) + sizeof(*mh),
 		sizeof(*mph) + mph->page_length);
 
-		mh->bdl = 0;
+		mh->bdl = mh->dev_spec_par = 0;
 		mph = (struct mode_page_header *) (((char *)mh) + sizeof(*mh));
 		mode_pars = ((char *)mph) + 2;
 
@@ -878,9 +840,12 @@ void do_probe_all(void)
 
 	char vendor_id[8 + 1], product_id[16 + 1], revision[4 + 1];
 	int id;
-	u_char *inq_buf = malloc(96);
+	u_char *inq_buf;
 	struct scsi_addr addr;
 
+	inq_buf = malloc(96);
+	if (inq_buf == NULL)
+		errx(1, "malloc failed");
 	scsireq = scsireq_build(scsireq_new(),
 	96, inq_buf, SCCMD_READ,
 	"12 0 0 0 v 0", 96);
@@ -944,19 +909,15 @@ void main(int argc, char **argv)
 		scaddr.lun = lun;
 
 		if (ioctl(fd,SCIOCREPROBE,&scaddr) == -1)
-			perror("ioctl");
+			warn("ioctl [SCIOCREPROBE]");
 	} else if(debugflag) {
 		if (ioctl(fd,SCIOCDEBUG,&debuglevel) == -1)
-		{
-			perror("ioctl [SCIODEBUG]");
-			exit(1);
-		}
+			err(1, "ioctl [SCIODEBUG]");
 	} else if (commandflag) {
-		int i;
 		char *fmt;
 
 		if (argc < 1) {
-			fprintf(stderr, "Need the command format string.\n");
+			warnx("need the command format string");
 			usage();
 		}
 
