@@ -42,26 +42,32 @@ static char copyright[] =
 
 #ifndef lint
 /*static char sccsid[] = "from: @(#)slattach.c	4.6 (Berkeley) 6/1/90";*/
-static char rcsid[] = "$Id";
+static char rcsid[] = "$FreeBSD: stable/2.2/sbin/slattach/slattach.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <sys/param.h>
+#include <sys/types.h>
 #include <sys/ioctl.h>
-#include <termios.h>
 #include <sys/socket.h>
+#include <sys/time.h>
+
+#include <fcntl.h>
+#include <netdb.h>
+#include <paths.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <strings.h>
+#include <syslog.h>
+#include <termios.h>
+#include <unistd.h>
+#include <libutil.h>
+#include <err.h>
+
 #include <netinet/in.h>
 #include <net/if.h>
 #include <net/slip.h>
 #include <net/if_slvar.h>
-#include <netdb.h>
-#include <fcntl.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <paths.h>
-#include <syslog.h>
-#include <signal.h>
-#include <strings.h>
 
 #define DEFAULT_BAUD	9600
 
@@ -73,7 +79,8 @@ void	exit_handler(int ret);	/* run exit_cmd iff specified upon exit. */
 void	setup_line(int cflag);	/* configure terminal settings */
 void	slip_discipline();	/* switch to slip line discipline */
 void	configure_network();	/* configure slip interface */
-void	acquire_line();		/* get tty device as controling terminal */
+void	acquire_line();		/* get tty device as controlling terminal */
+static void usage __P((void));
 
 int	fd = -1;
 char	*dev = (char *)0;	/* path name of the tty (e.g. /dev/tty01) */
@@ -91,7 +98,7 @@ int     keepal = 0;             /* keepalive timeout */
 int     outfill = 0;            /* outfill timeout */
 int     sl_unit = -1;           /* unit number */
 int     uucp_lock = 0;          /* do uucp locking */
-int	exiting = 0;		/* allready running exit_handler */
+int	exiting = 0;		/* already running exit_handler */
 
 struct	termios tty;		/* tty configuration/state */
 
@@ -101,33 +108,23 @@ char	*redial_cmd = 0;	/* command to exec upon shutdown. */
 char	*config_cmd = 0;	/* command to exec if slip unit changes. */
 char	*exit_cmd = 0;		/* command to exec before exiting. */
 
-static char usage_str[] = "\
-usage: %s [-acfhlnz] [-e command] [-r command] [-s speed] [-u command] \\\n\
-	  [-L] [-K timeout] [-O timeout] [-S unit] device\n\
-  -a      -- autoenable VJ compression\n\
-  -c      -- enable VJ compression\n\
-  -e ECMD -- run ECMD before exiting\n\
-  -f      -- run in foreground (don't detach from controlling tty)\n\
-  -h      -- turn on cts/rts style flow control\n\
-  -l      -- disable modem control (CLOCAL) and ignore carrier detect\n\
-  -n      -- throw out ICMP packets\n\
-  -r RCMD -- run RCMD upon loss of carrier\n\
-  -s #    -- set baud rate (default 9600)\n\
-  -u UCMD -- run 'UCMD <old sl#> <new sl#>' before switch to slip discipline\n\
-  -z      -- run RCMD upon startup irrespective of carrier\n\
-  -L      -- do uucp-style device locking\n\
-  -K #    -- set SLIP \"keep alive\" timeout (default 0)\n\
-  -O #    -- set SLIP \"out fill\" timeout (default 0)\n\
-  -S #    -- set SLIP unit number (default is dynamic)\n\
-";
+static void
+usage()
+{
+	fprintf(stderr, "%s\n%s\n",
+"usage: slattach [-acfhlnz] [-e command] [-r command] [-s speed] [-u command]",
+"                [-L] [-K timeout] [-O timeout] [-S unit] device");
+	/* do not exit here */
+}
 
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 {
 	int option;
 	extern char *optarg;
 	extern int optind;
 
-	while ((option = getopt(argc, argv, "ace:fhlnr:s:u:zLK:O:S:")) != EOF) {
+	while ((option = getopt(argc, argv, "ace:fhlnr:s:u:zLK:O:S:")) != -1) {
 		switch (option) {
 		case 'a':
 			slflags |= IFF_LINK2;
@@ -176,11 +173,9 @@ int main(int argc, char **argv)
 		case 'S':
 			sl_unit = atoi(optarg);
 			break;
-		default:
-			fprintf(stderr, "%s: Invalid option -- '%c'\n",
-				argv[0], option);
 		case '?':
-			fprintf(stderr, usage_str, argv[0]);
+		default:
+			usage();
 			exit_handler(1);
 		}
 	}
@@ -188,15 +183,12 @@ int main(int argc, char **argv)
 	if (optind == argc - 1)
 		dev = argv[optind];
 
-	if (optind < (argc - 1)) {
-	    fprintf(stderr, "%s: Too many args, first='%s'\n",
-	      argv[0], argv[optind]);
-	}
-	if (optind > (argc - 1)) {
-	    fprintf(stderr, "%s: Not enough args\n", argv[0]);
-	}
+	if (optind < (argc - 1))
+	    warnx("too many args, first='%s'", argv[optind]);
+	if (optind > (argc - 1))
+	    warnx("not enough args");
 	if (dev == (char *)0) {
-		fprintf(stderr, usage_str, argv[0]);
+		usage();
 		exit_handler(2);
 	}
 	if (strncmp(_PATH_DEV, dev, sizeof(_PATH_DEV) - 1)) {
@@ -215,18 +207,18 @@ int main(int argc, char **argv)
 	/* daemon() closed stderr, so log errors from here on. */
 	openlog("slattach",LOG_CONS|LOG_PID,LOG_DAEMON);
 
-	acquire_line();		/* get tty device as controling terminal */
+	acquire_line();		/* get tty device as controlling terminal */
 	setup_line(0);		/* configure for slip line discipline */
 	slip_discipline();	/* switch to slip line discipline */
 
 	/* upon INT log a timestamp and exit.  */
-	if ((int)signal(SIGINT,sigint_handler) < 0)
+	if (signal(SIGINT,sigint_handler) == SIG_ERR)
 		syslog(LOG_NOTICE,"cannot install SIGINT handler: %m");
 	/* upon TERM log a timestamp and exit.  */
-	if ((int)signal(SIGTERM,sigterm_handler) < 0)
+	if (signal(SIGTERM,sigterm_handler) == SIG_ERR)
 		syslog(LOG_NOTICE,"cannot install SIGTERM handler: %m");
 	/* upon HUP redial and reconnect.  */
-	if ((int)signal(SIGHUP,sighup_handler) < 0)
+	if (signal(SIGHUP,sighup_handler) == SIG_ERR)
 		syslog(LOG_NOTICE,"cannot install SIGHUP handler: %m");
 
 	if (redial_on_startup)
@@ -239,8 +231,7 @@ int main(int argc, char **argv)
 			kill (getpid(), SIGHUP);
 		} else
 			configure_network();
-	}
-	else
+	} else
 		configure_network(); /* configure the network if needed. */
 
 	for (;;) {
@@ -281,16 +272,19 @@ void acquire_line()
 
 	/* create PID file */
 	if((pidfile = fopen(pidfilename, "w"))) {
-		fprintf(pidfile, "%ld\n", getpid());
+		fprintf(pidfile, "%ld\n", (long)getpid());
 		fclose(pidfile);
 	}
 
-	if ((int)signal(SIGHUP,sighup_handler) < 0) /* Re-enable HUP signal */
+	if (signal(SIGHUP,sighup_handler) == SIG_ERR) /* Re-enable HUP signal */
 		syslog(LOG_NOTICE,"cannot install SIGHUP handler: %m");
 
 	if (uucp_lock) {
 		/* unlock not needed here, always re-lock with new pid */
-		if (uu_lock(dvname)) {
+		int res;
+		if ((res = uu_lock(dvname)) != UU_LOCK_OK) {
+			if (res != UU_LOCK_INUSE)
+				syslog(LOG_ERR, "uu_lock: %s", uu_lockerr(res));
 			syslog(LOG_ERR, "can't lock %s", dev);
 			exit_handler(1);
 		}
@@ -317,7 +311,7 @@ void acquire_line()
 		(void)close (fd);
 	fd = STDIN_FILENO;
 
-	/* acquire the serial line as a controling terminal. */
+	/* acquire the serial line as a controlling terminal. */
 	if (ioctl(fd, TIOCSCTTY, 0) < 0) {
 		syslog(LOG_ERR,"ioctl(TIOCSCTTY): %m");
 		exit_handler(1);
@@ -414,7 +408,7 @@ void slip_discipline()
 	close(s);
 }
 
-/* configure the interface, eg. by passing the unit number to a script. */
+/* configure the interface, e.g. by passing the unit number to a script. */
 void configure_network()
 {
 	int new_unit;
@@ -424,13 +418,14 @@ void configure_network()
                 syslog(LOG_ERR, "ioctl(SLIOCGUNIT): %m");
                 exit_handler(1);
         }
-	/* don't compare unit numbers if this is the first time to attach. */
-	if (unit < 0)
-		unit = new_unit;
 	/* iff the unit number changes either invoke config_cmd or punt. */
 	if (config_cmd) {
 		char *s;
 		s = (char*) malloc(strlen(config_cmd) + 32);
+		if (s == NULL) {
+			syslog(LOG_ERR, "malloc failed");
+			exit(1);
+		}
 		sprintf (s, "%s %d %d", config_cmd, unit, new_unit);
 		syslog(LOG_NOTICE, "Configuring %s (sl%d):", dev, unit);
 		syslog(LOG_NOTICE, "  '%s'", s);
@@ -438,6 +433,9 @@ void configure_network()
 		free (s);
 		unit = new_unit;
 	} else {
+		/* don't compare unit numbers if this is the first time to attach. */
+		if (unit < 0)
+			unit = new_unit;
 		if (new_unit != unit) {
 			syslog(LOG_ERR, "slip unit changed from sl%d to sl%d, but no -u CMD was specified!");
 			exit_handler(1);
@@ -449,16 +447,19 @@ void configure_network()
 /* sighup_handler() is invoked when carrier drops, eg. before redial. */
 void sighup_handler()
 {
-	int ttydisc = TTYDISC;
-
 	if(exiting) return;
+
+	if (redial_cmd == NULL) {
+		syslog(LOG_NOTICE,"SIGHUP on %s (sl%d); exiting", dev, unit);
+		exit_handler(1);
+	}
 again:
 	/* invoke a shell for redial_cmd or punt. */
-	if (redial_cmd) {
-		acquire_line();
+	if (*redial_cmd) {
+		syslog(LOG_NOTICE,"SIGHUP on %s (sl%d); running '%s'",
+		       dev, unit, redial_cmd);
+		acquire_line(); /* reopen dead line */
 		setup_line(CLOCAL);
-		syslog(LOG_NOTICE,"SIGHUP on %s (sl%d); running %s",
-		       dev,unit,redial_cmd);
 		if (locked) {
 			if (uucp_lock)
 				uu_unlock(dvname);      /* for redial */
@@ -467,7 +468,10 @@ again:
 		if (system(redial_cmd))
 			goto again;
 		if (uucp_lock) {
-			if (uu_lock(dvname)) {
+			int res;
+			if ((res = uu_lock(dvname)) != UU_LOCK_OK) {
+				if (res != UU_LOCK_INUSE)
+					syslog(LOG_ERR, "uu_lock: %s", uu_lockerr(res));
 				syslog(LOG_ERR, "can't relock %s after %s, aborting",
 					dev, redial_cmd);
 				exit_handler(1);
@@ -488,24 +492,16 @@ again:
 			}
 		} else
 			setup_line(0);
-	} else {
-#if 0
-		/*
-		 * XXX should do this except we are called from main() via
-		 * kill(getpid(), SIGHUP).  Ick.
-		 */
-		syslog(LOG_NOTICE, "SIGHUP on %s (sl%d); exiting", dev, unit);
-		exit_handler(0);
-#endif
-		if (ioctl(fd, TIOCSETD, &ttydisc) < 0) {
-			syslog(LOG_ERR, "ioctl(TIOCSETD): %m");
-			exit_handler(1);
-		}
+	} else {        /* Empty redial command */
+		syslog(LOG_NOTICE,"SIGHUP on %s (sl%d); reestablish connection",
+			dev, unit);
+		acquire_line(); /* reopen dead line */
 		setup_line(0);  /* restore ospeed from hangup (B0) */
 		/* If modem control, just wait for carrier before attaching.
 		   If no modem control, just fall through immediately. */
 		if (!(modem_control & CLOCAL)) {
 			int carrier = 0;
+
 			syslog(LOG_NOTICE, "Waiting for carrier on %s (sl%d)",
 			       dev, unit);
 			/* Now wait for carrier before attaching line. */
@@ -579,6 +575,10 @@ void exit_handler(int ret)
 	if (config_cmd) {
 		char *s;
 		s = (char*) malloc(strlen(config_cmd) + 32);
+		if (s == NULL) {
+			syslog(LOG_ERR, "malloc failed");
+			exit(1);
+		}
 		sprintf (s, "%s %d -1", config_cmd, unit);
 		syslog(LOG_NOTICE, "Deconfiguring %s (sl%d):", dev, unit);
 		syslog(LOG_NOTICE, "  '%s'", s);
