@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sbin/startslip/startslip.c 50952 1999-09-05 11:54:50Z peter $
+ * $FreeBSD: stable/2.2/sbin/startslip/startslip.c 60174 2000-05-07 18:24:08Z joe $
  */
 
 #ifndef lint
@@ -43,25 +43,30 @@ static char copyright[] =
 static char sccsid[] = "@(#)startslip.c	8.1 (Berkeley) 6/5/93";
 #endif /* not lint */
 
+#include <sys/types.h>
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+
+#include <errno.h>
+#include <fcntl.h>
+#include <paths.h>
+#include <netdb.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <syslog.h>
 #include <termios.h>
 #include <time.h>
-#include <sys/ioctl.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <syslog.h>
+#include <unistd.h>
+#include <libutil.h>
+#include <err.h>
+
 #include <netinet/in.h>
 #include <net/if.h>
 #include <net/if_slvar.h>
 #include <net/slip.h>
-#include <netdb.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <signal.h>
-#include <string.h>
-#include <unistd.h>
-#include <paths.h>
 
 #define DEFAULT_BAUD    B9600
 int     speed = DEFAULT_BAUD;
@@ -104,6 +109,12 @@ int	debug = 0;
 #endif
 #define	printd	if (debug) printf
 
+int carrier __P((void));
+void down __P((int));
+int getline __P((char *, int, int, time_t));
+static void usage __P((void));
+
+int
 main(argc, argv)
 	int argc;
 	char **argv;
@@ -124,7 +135,7 @@ main(argc, argv)
 	pid_t pid;
 	struct termios t;
 
-	while ((ch = getopt(argc, argv, "dhlb:s:t:w:A:U:D:W:K:O:S:L")) != EOF)
+	while ((ch = getopt(argc, argv, "dhlb:s:t:w:A:U:D:W:K:O:S:L")) != -1)
 		switch (ch) {
 		case 'd':
 			debug = 1;
@@ -133,11 +144,8 @@ main(argc, argv)
 			speed = atoi(optarg);
 			break;
 		case 's':
-			if (diali >= MAXDIALS) {
-				(void)fprintf(stderr,
-					"max dial strings number (%d) exceeded\n", MAXDIALS);
-				exit(1);
-			}
+			if (diali >= MAXDIALS)
+				errx(1, "max dial strings number (%d) exceeded", MAXDIALS);
 			dials[diali++] = strdup(optarg);
 			break;
 		case 't':
@@ -212,7 +220,9 @@ main(argc, argv)
 		dvname = devicename;
 	else
 		dvname++;
-	sprintf(pidfile, PIDFILE, _PATH_VARRUN, dvname);
+	if (snprintf(pidfile, sizeof(pidfile), PIDFILE, _PATH_VARRUN, dvname) >= sizeof(pidfile))
+		usage();
+
 	if ((pfd = fopen(pidfile, "r")) != NULL) {
 		pid = 0;
 		fscanf(pfd, "%ld\n", &pid);
@@ -285,7 +295,10 @@ restart:
 	}
 	printd("open");
 	if (uucp_lock) {
-		if (uu_lock(dvname)) {
+		int res;
+		if ((res = uu_lock(dvname)) != UU_LOCK_OK) {
+			if (res != UU_LOCK_INUSE)
+				syslog(LOG_ERR, "uu_lock: %s", uu_lockerr(res));
 			syslog(LOG_ERR, "%s: can't lock %s", username, devicename);
 			goto restart;
 		}
@@ -371,7 +384,7 @@ restart:
 			syslog(LOG_ERR, "%s: tcsetattr(%s): %m", username, devicename);
 			down(2);
 		}
-		/* Only now we able to receive HUP on carier drop! */
+		/* Only now we able to receive HUP on carrier drop! */
 	}
 
 	/*
@@ -465,6 +478,7 @@ restart:
 		printd("sigpause return\n");
 	}
 	goto restart;
+	return(0); /* not reached */
 }
 
 void
@@ -497,6 +511,7 @@ sigterm()
 	terminate = 1;
 }
 
+int
 getline(buf, size, fd, fintimeout)
 	char *buf;
 	int size, fd;
@@ -553,6 +568,7 @@ getline(buf, size, fd, fintimeout)
 	return (0);
 }
 
+int
 carrier()
 {
 	int comstate;
@@ -565,6 +581,7 @@ carrier()
 	return !!(comstate & TIOCM_CD);
 }
 
+void
 down(code)
 {
 	if (fd > -1)
@@ -576,12 +593,13 @@ down(code)
 	exit(code);
 }
 
+static void
 usage()
 {
-	(void)fprintf(stderr, "\
-usage: startslip [-d] [-b speed] [-s string1 [-s string2 [...]]] [-A annexname] \\\n\
-	[-h] [-l] [-U upscript] [-D downscript] [-t script_timeout] [-L]\\\n\
-	[-w retry_pause] [-W maxtries] [-K keepalive] [-O outfill] [-S unit] \\\n\
-	device user passwd\n");
+	(void)fprintf(stderr, "%s\n%s\n%s\n%s\n",  
+"usage: startslip [-d] [-b speed] [-s string1 [-s string2 [...]]] [-h] [-l]",
+"                 [-L] [-A annexname] [-U upscript] [-D downscript]",
+"                 [-t script_timeout] [-W maxtries] [-w retry_pause]",
+"                 [-K keepalive] [-O outfill] [-S unit] device user passwd");
 	exit(1);
 }
