@@ -87,7 +87,7 @@ main(argc, argv)
 	sync();
 
 	all = 0;
-	while ((ch = getopt(argc, argv, "aFfh:t:v")) != EOF)
+	while ((ch = getopt(argc, argv, "aFfh:t:v")) !=  -1)
 		switch (ch) {
 		case 'a':
 			all = 1;
@@ -219,12 +219,12 @@ umountfs(name)
 	if (!selected(type))
 		return (0);
 
-	if ((delimp = strchr(name, '@')) != NULL) {
+	if ((delimp = strchr(name, '@')) != NULL && type == MOUNT_NFS) {
 		hostp = delimp + 1;
 		*delimp = '\0';
 		hp = gethostbyname(hostp);
 		*delimp = '@';
-	} else if ((delimp = strchr(name, ':')) != NULL) {
+	} else if ((delimp = strchr(name, ':')) != NULL && type == MOUNT_NFS) {
 		*delimp = '\0';
 		hostp = name;
 		hp = gethostbyname(hostp);
@@ -416,9 +416,8 @@ xdr_dir(xdrsp, dirp)
 void
 usage()
 {
-	(void)fprintf(stderr,
-	    "usage: %s\n       %s\n",
-	    "umount [-fv] [-t fstypelist] special | node",
-	    "umount -a[fv] [-h host] [-t fstypelist]");
+	(void)fprintf(stderr, "%s\n%s\n",
+	    "usage: umount [-fv] special | node",
+	    "       umount -a [-fv] [-h host] [-t ufs | lfs | external_type]");
 	exit(1);
 }
