@@ -12,7 +12,7 @@
 if ! pkg_info -e team-3.1; then
 	echo "$0: You do not appear to have the team package installed."
 	echo
-	echo "Please see /usr/ports/misc/team-3.1 if you have the ports"
+	echo "Please see /usr/ports/misc/team if you have the ports"
 	echo "collection on your machine, or install the team package from"
 	echo "your CD or the net.  To install team from the net right now,"
 	echo "simply type:"
@@ -25,17 +25,15 @@ fi
 
 if [ $# -lt 1 ]; then
 	echo "usage: $0 input-file"
-elif [ ! -f $1 ]; then
-	echo "$0: $1 is not a valid file."
 elif [ "`id -u`" != "0" ]; then
 	echo "Sorry, this must be done as root."
 else
 	echo -n "Place CD in the worm drive now and press return: "
 	read junk
+	fname=$1; shift
 	scsi -f /dev/rworm0.ctl -c "0 0 0 0 0 0" >/dev/null 2>&1
-	wormcontrol select HP 4020i
-	wormcontrol prepdisk double
+	wormcontrol prepdisk double $*
 	wormcontrol track data
-	rtprio 5 team -v 1m 5 < $1 | dd of=/dev/rworm0 obs=20k
+	rtprio 5 team -v 1m 5 < $fname | rtprio 5 dd of=/dev/rworm0 obs=20k
 	wormcontrol fixate 1
 fi
