@@ -32,10 +32,10 @@
 # SUCH DAMAGE.
 #
 #	@(#)newvers.sh	8.1 (Berkeley) 4/20/94
-# $FreeBSD: stable/2.1/sys/conf/newvers.sh 50950 1999-09-05 08:39:22Z peter $
+# $FreeBSD: stable/2.2/sys/conf/newvers.sh 50950 1999-09-05 08:39:22Z peter $
 
 TYPE="FreeBSD"
-REVISION="2.1"
+REVISION="2.2.8"
 BRANCH="STABLE"
 RELEASE="${REVISION}-${BRANCH}"
 SNAPDATE=""
@@ -44,8 +44,38 @@ if [ "X${SNAPDATE}" != "X" ]; then
 fi
 VERSION="${TYPE} ${RELEASE}"
 
-RELDATE="199612"
+RELDATE="228001"
 
+b=share/examples/etc/bsd-style-copyright
+year=`date '+%Y'`
+# look for copyright template
+for bsd_copyright in ../$b ../../$b ../../../$b /usr/src/$b /usr/$b
+do
+	if [ -r "$bsd_copyright" ]; then
+		COPYRIGHT=`sed \
+		    -e "s/\[year\]/$year/" \
+		    -e 's/\[your name here\]\.* /FreeBSD Inc./' \
+		    -e 's/\[your name\]\.*/FreeBSD Inc./' \
+		    -e '/\[id for your version control system, if any\]/d' \
+		    $bsd_copyright` 
+		break
+	fi
+done
+
+# no copyright found, use a dummy
+if [ X"$COPYRIGHT" = X ]; then
+	COPYRIGHT="/*
+ * Copyright (c) $year
+ *	FreeBSD Inc. All rights reserved.
+ *
+ */"
+fi
+
+# add newline
+COPYRIGHT="$COPYRIGHT
+"
+
+LC_TIME=C; export LC_TIME
 if [ ! -r version ]
 then
 	echo 0 > version
@@ -53,7 +83,9 @@ fi
 
 touch version
 v=`cat version` u=${USER-root} d=`pwd` h=`hostname` t=`date`
-echo "char ostype[] = \"${TYPE}\";" > vers.c
+cat /dev/null > vers.c
+echo "$COPYRIGHT" >> vers.c
+echo "char ostype[] = \"${TYPE}\";" >> vers.c
 echo "char osrelease[] = \"${RELEASE}\";" >> vers.c
 echo "int osreldate = ${RELDATE};" >> vers.c
 echo "char sccs[4] = { '@', '(', '#', ')' };" >>vers.c
