@@ -23,11 +23,11 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_access.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_access.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _DDB_DB_ACCESS_H_
-#define _DDB_DB_ACCESS_H_ 1
+#define	_DDB_DB_ACCESS_H_
 
 /*
  *	Author: David B. Golub, Carnegie Mellon University
@@ -36,12 +36,8 @@
 /*
  * Data access functions for debugger.
  */
-#include <machine/db_machdep.h>		/* expression types */
+db_expr_t	db_get_value __P((db_addr_t addr, int size,
+				  boolean_t is_signed));
+void		db_put_value __P((db_addr_t addr, int size, db_expr_t value));
 
-extern db_expr_t db_get_value(/* db_addr_t addr,
-				 int size,
-				 boolean_t is_signed */);
-extern void	 db_put_value(/* db_addr_t addr,
-				 int size,
-				 db_expr_t value */);
-#endif /* _DDB_DB_ACCESS_H_ */
+#endif /* !_DDB_DB_ACCESS_H_ */
