@@ -23,19 +23,16 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_break.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_break.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
  *	Author: David B. Golub, Carnegie Mellon University
  *	Date:	7/90
  */
-#ifndef	_DDB_DB_BREAK_H_
+#ifndef _DDB_DB_BREAK_H_
 #define	_DDB_DB_BREAK_H_
 
-#include <vm/vm_map.h>
-#include <machine/db_machdep.h>
-
 /*
  * Breakpoint.
  */
@@ -53,12 +50,8 @@ struct db_breakpoint {
 };
 typedef struct db_breakpoint *db_breakpoint_t;
 
-extern db_breakpoint_t	db_find_breakpoint();
-extern db_breakpoint_t	db_find_breakpoint_here();
-extern void		db_set_breakpoints();
-extern void		db_clear_breakpoints();
+void		db_clear_breakpoints __P((void));
+db_breakpoint_t	db_find_breakpoint_here __P((db_addr_t addr));
+void		db_set_breakpoints __P((void));
 
-extern db_breakpoint_t	db_set_temp_breakpoint(/* db_addr_t addr */);
-extern void		db_delete_temp_breakpoint(/* db_breakpoint_t bkpt */);
-
-#endif	_DDB_DB_BREAK_H_
+#endif /* !_DDB_DB_BREAK_H_ */
