@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_expr.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_expr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -32,13 +32,19 @@
  */
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/proc.h>
+
 #include <ddb/ddb.h>
 #include <ddb/db_lex.h>
 #include <ddb/db_access.h>
 #include <ddb/db_command.h>
 
-boolean_t
+static boolean_t	db_add_expr __P((db_expr_t *valuep));
+static boolean_t	db_mult_expr __P((db_expr_t *valuep));
+static boolean_t	db_shift_expr __P((db_expr_t *valuep));
+static boolean_t	db_term __P((db_expr_t *valuep));
+static boolean_t	db_unary __P((db_expr_t *valuep));
+
+static boolean_t
 db_term(valuep)
 	db_expr_t *valuep;
 {
@@ -93,7 +99,7 @@ db_term(valuep)
 	return (FALSE);
 }
 
-boolean_t
+static boolean_t
 db_unary(valuep)
 	db_expr_t *valuep;
 {
@@ -121,7 +127,7 @@ db_unary(valuep)
 	return (db_term(valuep));
 }
 
-boolean_t
+static boolean_t
 db_mult_expr(valuep)
 	db_expr_t *valuep;
 {
@@ -158,7 +164,7 @@ db_mult_expr(valuep)
 	return (TRUE);
 }
 
-boolean_t
+static boolean_t
 db_add_expr(valuep)
 	db_expr_t *valuep;
 {
@@ -185,7 +191,7 @@ db_add_expr(valuep)
 	return (TRUE);
 }
 
-boolean_t
+static boolean_t
 db_shift_expr(valuep)
 	db_expr_t *valuep;
 {
