@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_input.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_input.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -32,11 +32,13 @@
  */
 
 #include <sys/param.h>
+#include <sys/malloc.h>
 #include <sys/systm.h>
-#include <sys/proc.h>
+
+#include <machine/cons.h>
+
 #include <ddb/ddb.h>
 #include <ddb/db_output.h>
-#include <machine/cons.h>
 
 /*
  * Character input and editing.
@@ -47,16 +49,30 @@
  * since input always ends with a new-line.  We just
  * reset the line position at the end.
  */
-char *	db_lbuf_start;	/* start of input line buffer */
-char *	db_lbuf_end;	/* end of input line buffer */
-char *	db_lc;		/* current character */
-char *	db_le;		/* one past last character */
+static char *	db_lbuf_start;	/* start of input line buffer */
+static char *	db_lbuf_end;	/* end of input line buffer */
+static char *	db_lc;		/* current character */
+static char *	db_le;		/* one past last character */
+
+/*
+ * Simple input line history support.
+ */
+static char *	db_lhistory;
+static char	db_lhistory_buffer[2048];
+static int	db_lhistlsize, db_lhistidx, db_lhistcur;
+static int	db_lhist_nlines;
 
 #define	CTRL(c)		((c) & 0x1f)
 #define	isspace(c)	((c) == ' ' || (c) == '\t')
 #define	BLANK		' '
 #define	BACKUP		'\b'
 
+static int	cnmaygetc __P((void));
+static void	db_delete __P((int n, int bwd));
+static int	db_inputchar __P((int c));
+static void	db_putnchars __P((int c, int count));
+static void	db_putstring __P((char *s, int count));
+
 void
 db_putstring(s, count)
 	char	*s;
@@ -105,7 +121,47 @@ int
 db_inputchar(c)
 	int	c;
 {
+	static int escstate;
+
+	if (escstate == 1) {
+		/* ESC seen, look for [ or O */
+		if (c == '[' || c == 'O')
+			escstate++;
+		else
+			escstate = 0; /* re-init state machine */
+		return (0);
+	} else if (escstate == 2) {
+		escstate = 0;
+		/*
+		 * If a valid cursor key has been found, translate
+		 * into an emacs-style control key, and fall through.
+		 * Otherwise, drop off.
+		 */
+		switch (c) {
+		case 'A':	/* up */
+			c = CTRL('p');
+			break;
+		case 'B':	/* down */
+			c = CTRL('n');
+			break;
+		case 'C':	/* right */
+			c = CTRL('f');
+			break;
+		case 'D':	/* left */
+			c = CTRL('b');
+			break;
+		default:
+			return (0);
+		}
+	}
+
 	switch (c) {
+	    case CTRL('['):
+		escstate = 1;
+		break;
+#if __i386__ && __FreeBSD__
+	    case 591:		/* syscons's idea of an arrow key... */
+#endif
 	    case CTRL('b'):
 		/* back up one character */
 		if (db_lc > db_lbuf_start) {
@@ -113,6 +169,9 @@ db_inputchar(c)
 		    db_lc--;
 		}
 		break;
+#if __i386__ && __FreeBSD__
+	    case 593:		/* syscons's idea of an arrow key... */
+#endif
 	    case CTRL('f'):
 		/* forward one character */
 		if (db_lc < db_le) {
@@ -164,11 +223,59 @@ db_inputchar(c)
 		break;
 	    case CTRL('r'):
 		db_putstring("^R\n", 3);
+	    redraw:
 		if (db_le > db_lbuf_start) {
 		    db_putstring(db_lbuf_start, db_le - db_lbuf_start);
 		    db_putnchars(BACKUP, db_le - db_lc);
 		}
 		break;
+#if __i386__ && __FreeBSD__
+	    case 588:		/* syscons's idea of an arrow key... */
+#endif
+	    case CTRL('p'):
+		/* Make previous history line the active one. */
+		if (db_lhistcur >= 0) {
+		    bcopy(db_lhistory + db_lhistcur * db_lhistlsize,
+			  db_lbuf_start, db_lhistlsize);
+		    db_lhistcur--;
+		    goto hist_redraw;
+		}
+		break;
+#if __i386__ && __FreeBSD__
+	    case 596:		/* syscons's idea of an arrow key... */
+#endif
+	    case CTRL('n'):
+		/* Make next history line the active one. */
+		if (db_lhistcur < db_lhistidx - 1) {
+		    db_lhistcur += 2;
+		    bcopy(db_lhistory + db_lhistcur * db_lhistlsize,
+			  db_lbuf_start, db_lhistlsize);
+		} else {
+		    /*
+		     * ^N through tail of history, reset the
+		     * buffer to zero length.
+		     */
+		    *db_lbuf_start = '\0';
+		    db_lhistcur = db_lhistidx;
+		}
+
+	    hist_redraw:
+		db_putnchars(BACKUP, db_le - db_lbuf_start);
+		db_putnchars(BLANK, db_le - db_lbuf_start);
+		db_putnchars(BACKUP, db_le - db_lbuf_start);
+		db_le = index(db_lbuf_start, '\0');
+		if (db_le[-1] == '\r' || db_le[-1] == '\n')
+		    *--db_le = '\0';
+		db_lc = db_le;
+		goto redraw;
+
+	    case -1:
+		/*
+		 * eek! the console returned eof.
+		 * probably that means we HAVE no console.. we should try bail
+		 * XXX
+		 */
+		c = '\r';
 	    case '\n':
 	    case '\r':
 		*db_le++ = c;
@@ -194,7 +301,7 @@ db_inputchar(c)
 }
 
 int
-cnmaygetc (void)
+cnmaygetc()
 {
 	return (-1);
 }
@@ -204,6 +311,19 @@ db_readline(lstart, lsize)
 	char *	lstart;
 	int	lsize;
 {
+	if (db_lhistory && lsize != db_lhistlsize) {
+		/* Should not happen, but to be sane, throw history away. */
+		db_lhistory = NULL;
+	}
+	if (db_lhistory == NULL) {
+		/* Initialize input line history. */
+		db_lhistory = db_lhistory_buffer;
+		db_lhist_nlines = (sizeof db_lhistory_buffer) / lsize;
+		db_lhistlsize = lsize;
+		db_lhistidx = -1;
+	}
+	db_lhistcur = db_lhistidx;
+
 	db_force_whitespace();	/* synch output position */
 
 	db_lbuf_start = lstart;
@@ -214,9 +334,21 @@ db_readline(lstart, lsize)
 	while (!db_inputchar(cngetc()))
 	    continue;
 
-	db_putchar('\n');	/* synch output position */
-
+	db_printf("\n");	/* synch output position */
 	*db_le = 0;
+
+	if (db_lhistory && (db_le - db_lbuf_start > 1)) {
+	    /* Maintain input line history for non-empty lines. */
+	    if (++db_lhistidx == db_lhist_nlines) {
+		/* Rotate history. */
+		ovbcopy(db_lhistory + db_lhistlsize, db_lhistory,
+			db_lhistlsize * (db_lhist_nlines - 1));
+		db_lhistidx--;
+	    }
+	    bcopy(lstart, db_lhistory + (db_lhistidx * db_lhistlsize),
+		  db_lhistlsize);
+	}
+
 	return (db_le - db_lbuf_start);
 }
 
