@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_lex.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_lex.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -35,13 +35,17 @@
  */
 #include <sys/param.h>
 #include <sys/systm.h>
+
 #include <ddb/ddb.h>
 #include <ddb/db_lex.h>
 
-char	db_line[120];
-char *	db_lp, *db_endlp;
+static char	db_line[120];
+static char *	db_lp, *db_endlp;
 
-static int db_lex(void);
+static int	db_lex __P((void));
+static void 	db_flush_line __P((void));
+static int 	db_read_char __P((void));
+static void 	db_unread_char __P((int));
 
 int
 db_read_line()
@@ -56,16 +60,16 @@ db_read_line()
 	return (i);
 }
 
-void
+static void
 db_flush_line()
 {
 	db_lp = db_line;
 	db_endlp = db_line;
 }
 
-int	db_look_char = 0;
+static int	db_look_char = 0;
 
-int
+static int
 db_read_char()
 {
 	int	c;
@@ -81,14 +85,14 @@ db_read_char()
 	return (c);
 }
 
-void
+static void
 db_unread_char(c)
 	int c;
 {
 	db_look_char = c;
 }
 
-int	db_look_token = 0;
+static int	db_look_token = 0;
 
 void
 db_unread_token(t)
