@@ -23,11 +23,11 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_lex.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_lex.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _DDB_DB_LEX_H_
-#define _DDB_DB_LEX_H_ 1
+#define	_DDB_DB_LEX_H_
 
 /*
  *	Author: David B. Golub, Carnegie Mellon University
@@ -36,13 +36,10 @@
 /*
  * Lexical analyzer.
  */
-extern int	db_read_line();
-extern void	db_flush_line();
-extern int	db_read_char();
-extern void	db_unread_char(/* char c */);
-extern int	db_read_token();
-extern void	db_unread_token(/* int t */);
-extern void	db_flush_lex();
+void	db_flush_lex __P((void));
+int	db_read_line __P((void));
+int	db_read_token __P((void));
+void	db_unread_token __P((int t));
 
 extern int	db_tok_number;
 #define	TOK_STRING_SIZE		120
@@ -70,7 +67,4 @@ extern char	db_tok_string[TOK_STRING_SIZE];
 #define	tSHIFT_R	19
 #define	tDOTDOT		20
 
-
-
-
-#endif /* _DDB_DB_LEX_H_ */
+#endif /* !_DDB_DB_LEX_H_ */
