@@ -23,11 +23,11 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_output.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_output.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _DDB_DB_OUTPUT_H_
-#define _DDB_DB_OUTPUT_H_ 1
+#define	_DDB_DB_OUTPUT_H_
 
 /*
  * 	Author: David B. Golub, Carnegie Mellon University
@@ -38,7 +38,8 @@
  * Printing routines for kernel debugger.
  */
 
-extern void	db_force_whitespace();
-extern int	db_print_position();
-extern void	db_end_line();
-#endif /* _DDB_DB_OUTPUT_H_ */
+void	db_end_line __P((void));
+void	db_force_whitespace __P((void));
+int	db_print_position __P((void));
+
+#endif /* !_DDB_DB_OUTPUT_H_ */
