@@ -30,14 +30,16 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_ps.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_ps.c 50950 1999-09-05 08:39:22Z peter $
  */
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
-#include <ddb/ddb.h>
+
 #include <machine/cons.h>
 
+#include <ddb/ddb.h>
+
 void
 db_ps(dummy1, dummy2, dummy3, dummy4)
 	db_expr_t	dummy1;
@@ -51,8 +53,8 @@ db_ps(dummy1, dummy2, dummy3, dummy4)
 
 	np = nprocs;
 
-	if (allproc != NULL)
-		p = allproc;
+	if (allproc.lh_first != NULL)
+		p = allproc.lh_first;
 	else
 		p = &proc0;
 
@@ -82,6 +84,10 @@ db_ps(dummy1, dummy2, dummy3, dummy4)
 				return;
 			}
 		}
+		if (p == NULL) {
+			printf("oops, ran out of processes early!\n");
+			break;
+		}
 		pp = p->p_pptr;
 		if (pp == NULL)
 			pp = p;
@@ -96,8 +102,8 @@ db_ps(dummy1, dummy2, dummy3, dummy4)
 		}
 		db_printf(" %s\n", p->p_comm ? p->p_comm : "");
 
-		p = p->p_next;
+		p = p->p_list.le_next;
 		if (p == NULL && np > 0)
-			p = zombproc;
+			p = zombproc.lh_first;
     	}
 }
