@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_sym.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_sym.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -32,18 +32,10 @@
  */
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/proc.h>
+
 #include <ddb/ddb.h>
 #include <ddb/db_sym.h>
 
-/*
- * We import from the symbol-table dependent routines:
- */
-extern db_sym_t	X_db_lookup();
-extern db_sym_t	X_db_search_symbol();
-extern boolean_t X_db_line_at_pc();
-extern void	X_db_symbol_values();
-
 /*
  * Multiple symbol tables
  */
@@ -51,12 +43,16 @@ extern void	X_db_symbol_values();
 #define	MAXNOSYMTABS	3	/* mach, ux, emulator */
 #endif
 
-db_symtab_t	db_symtabs[MAXNOSYMTABS] = {{0,},};
-int db_nsymtab = 0;
+static db_symtab_t	db_symtabs[MAXNOSYMTABS] = {{0,},};
+static int db_nsymtab = 0;
 
-db_symtab_t	*db_last_symtab;
+static db_symtab_t	*db_last_symtab;
 
-db_sym_t	db_lookup();	/* forward */
+static db_sym_t		db_lookup __P(( char *symstr));
+static char		*db_qualify __P((db_sym_t sym, char *symtabname));
+static boolean_t	db_symbol_is_ambiguous __P((db_sym_t sym));
+static boolean_t	db_line_at_pc __P((db_sym_t, char **, int *, 
+				db_expr_t));
 
 /*
  * Add symbol table, with given name, to list of symbol tables.
@@ -136,7 +132,7 @@ db_value_of_name(name, valuep)
  * then only the specified symbol table will be searched;
  * otherwise, all symbol tables will be searched.
  */
-db_sym_t
+static db_sym_t
 db_lookup(symstr)
 	char *symstr;
 {
@@ -185,9 +181,9 @@ db_lookup(symstr)
  * Does this symbol name appear in more than one symbol table?
  * Used by db_symbol_values to decide whether to qualify a symbol.
  */
-boolean_t db_qualify_ambiguous_names = FALSE;
+static boolean_t db_qualify_ambiguous_names = FALSE;
 
-boolean_t
+static boolean_t
 db_symbol_is_ambiguous(sym)
 	db_sym_t	sym;
 {
@@ -278,7 +274,7 @@ db_symbol_values(sym, namep, valuep)
  * then we just print the value in hex.  Small values might get
  * bogus symbol associations, e.g. 3 might get some absolute
  * value like _INCLUDE_VERSION or something, therefore we do
- * not accept symbols whose value is zero (and use plain hex).
+ * not accept symbols whose value is "small" (and use plain hex).
  */
 
 unsigned int	db_maxoff = 0x10000;
@@ -297,20 +293,26 @@ db_printsym(off, strategy)
 
 	cursym = db_search_symbol(off, strategy, &d);
 	db_symbol_values(cursym, &name, &value);
-	if (name == 0 || d >= db_maxoff || value == 0) {
+	if (name == 0)
+		value = off;
+	if (value >= DB_SMALL_VALUE_MIN && value <= DB_SMALL_VALUE_MAX) {
+		db_printf("%+#n", off);
+		return;
+	}
+	if (name == 0 || d >= db_maxoff) {
 		db_printf("%#n", off);
 		return;
 	}
 	db_printf("%s", name);
 	if (d)
-		db_printf("+%#r", d);
+		db_printf("+%+#n", d);
 	if (strategy == DB_STGY_PROC) {
 		if (db_line_at_pc(cursym, &filename, &linenum, off))
 			db_printf(" [%s:%d]", filename, linenum);
 	}
 }
 
-boolean_t
+static boolean_t
 db_line_at_pc( sym, filename, linenum, pc)
 	db_sym_t	sym;
 	char		**filename;
