@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/ddb/db_trap.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/ddb/db_trap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -36,12 +36,13 @@
  */
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/proc.h>
+
+#include <vm/vm.h>
+
 #include <ddb/ddb.h>
 #include <ddb/db_command.h>
 #include <ddb/db_break.h>
 
-
 void
 db_trap(type, code)
 	int	type, code;
