@@ -1,4 +1,4 @@
-/* $FreeBSD: stable/2.1/sys/dev/ccd/ccd.c 50950 1999-09-05 08:39:22Z peter $ */
+/* $FreeBSD: stable/2.2/sys/dev/ccd/ccd.c 50950 1999-09-05 08:39:22Z peter $ */
 
 /*	$NetBSD: ccd.c,v 1.22 1995/12/08 19:13:26 thorpej Exp $	*/
 
@@ -93,6 +93,9 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 #include <sys/proc.h>
 #include <sys/errno.h>
 #include <sys/dkstat.h>
@@ -104,7 +107,6 @@
 #include <sys/ioctl.h>
 #include <sys/disklabel.h>
 #include <ufs/ffs/fs.h> 
-#include <sys/devconf.h>
 #include <sys/device.h>
 #undef KERNEL			/* XXX */
 #include <sys/disk.h>
@@ -176,6 +178,16 @@ d_psize_t ccdsize;
 d_read_t ccdread;
 d_write_t ccdwrite;
 
+#define CDEV_MAJOR 74
+#define BDEV_MAJOR 21
+
+static struct cdevsw ccd_cdevsw;
+static struct bdevsw ccd_bdevsw = {
+  ccdopen, ccdclose, ccdstrategy, ccdioctl,
+  ccddump, ccdsize, 0,
+  "ccd", &ccd_cdevsw, -1
+};
+
 /* Called by main() during pseudo-device attachment */
 static void	ccdattach __P((void *));
 PSEUDO_SET(ccdattach, ccd);
@@ -195,8 +207,6 @@ static	void ccdmakedisklabel __P((struct ccd_softc *));
 static	int ccdlock __P((struct ccd_softc *));
 static	void ccdunlock __P((struct ccd_softc *));
 
-static	void loopdelay __P((void));
-
 #ifdef DEBUG
 static	void printiinfo __P((struct ccdiinfo *));
 #endif
@@ -206,6 +216,8 @@ struct	ccd_softc *ccd_softc;
 struct	ccddevice *ccddevs;
 int	numccd = 0;
 
+static int ccd_devsw_installed = 0;
+
 /*
  * Number of blocks to untouched in front of a component partition.
  * This is to avoid violating its disklabel area when it starts at the
@@ -217,7 +229,8 @@ int	numccd = 0;
 
 /*
  * Called by main() during pseudo-device attachment.  All we need
- * to do is allocate enough space for devices to be configured later.
+ * to do is allocate enough space for devices to be configured later, and
+ * add devsw entries.
  */
 void
 ccdattach(dummy)
@@ -225,7 +238,6 @@ ccdattach(dummy)
 {
 	int i;
 	int num = NCCD;
-	dev_t dev;
 
 	if (num > 1)
 		printf("ccd0-%d: Concatenated disk drivers\n", num-1);
@@ -251,6 +263,14 @@ ccdattach(dummy)
 	/* XXX: is this necessary? */
 	for (i = 0; i < numccd; ++i)
 		ccddevs[i].ccd_dk = -1;
+
+	if( ! ccd_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &ccd_bdevsw);
+		ccd_devsw_installed = 1;
+    	}
+	else {
+		printf("huh?\n");
+	}
 }
 
 static int
@@ -310,6 +330,10 @@ ccdinit(ccd, cpaths, p)
 				printf("ccd%d: can't copy path, error = %d\n",
 				    ccd->ccd_unit, error);
 #endif
+			while (ci > cs->sc_cinfo) {
+				ci--;
+				free(ci->ci_path, M_DEVBUF);
+			}
 			free(cs->sc_cinfo, M_DEVBUF);
 			return (error);
 		}
@@ -326,7 +350,10 @@ ccdinit(ccd, cpaths, p)
 				    ccd->ccd_unit, ci->ci_path,
 				    "error", error);
 #endif
-			free(ci->ci_path, M_DEVBUF);
+			while (ci >= cs->sc_cinfo) {
+				free(ci->ci_path, M_DEVBUF);
+				ci--;
+			}
 			free(cs->sc_cinfo, M_DEVBUF);
 			return (error);
 		}
@@ -342,7 +369,10 @@ ccdinit(ccd, cpaths, p)
 				 printf("ccd%d: %s: ioctl failed, error = %d\n",
 				     ccd->ccd_unit, ci->ci_path, error);
 #endif
-			free(ci->ci_path, M_DEVBUF);
+			while (ci >= cs->sc_cinfo) {
+				free(ci->ci_path, M_DEVBUF);
+				ci--;
+			}
 			free(cs->sc_cinfo, M_DEVBUF);
 			return (error);
 		}
@@ -357,7 +387,10 @@ ccdinit(ccd, cpaths, p)
 				printf("ccd%d: %s: incorrect partition type\n",
 				    ccd->ccd_unit, ci->ci_path);
 #endif
-			free(ci->ci_path, M_DEVBUF);
+			while (ci >= cs->sc_cinfo) {
+				free(ci->ci_path, M_DEVBUF);
+				ci--;
+			}
 			free(cs->sc_cinfo, M_DEVBUF);
 			return (EFTYPE);
 		}
@@ -380,7 +413,10 @@ ccdinit(ccd, cpaths, p)
 				printf("ccd%d: %s: size == 0\n",
 				    ccd->ccd_unit, ci->ci_path);
 #endif
-			free(ci->ci_path, M_DEVBUF);
+			while (ci >= cs->sc_cinfo) {
+				free(ci->ci_path, M_DEVBUF);
+				ci--;
+			}
 			free(cs->sc_cinfo, M_DEVBUF);
 			return (ENODEV);
 		}
@@ -402,7 +438,10 @@ ccdinit(ccd, cpaths, p)
 			printf("ccd%d: interleave must be at least %d\n",
 			    ccd->ccd_unit, (maxsecsize / DEV_BSIZE));
 #endif
-		free(ci->ci_path, M_DEVBUF);
+		while (ci >= cs->sc_cinfo) {
+			free(ci->ci_path, M_DEVBUF);
+			ci--;
+		}
 		free(cs->sc_cinfo, M_DEVBUF);
 		return (EINVAL);
 	}
@@ -416,10 +455,18 @@ ccdinit(ccd, cpaths, p)
 		     ci < &cs->sc_cinfo[cs->sc_nccdisks]; ci++)
 			ci->ci_size = minsize;
 		if (ccd->ccd_flags & CCDF_MIRROR) {
+			/*
+			 * Check to see if an even number of components
+			 * have been specified.
+			 */
 			if (cs->sc_nccdisks % 2) {
-				cs->sc_nccdisks--;
-				printf("ccd%d: mirroring requires even number of disks; using %d\n",
-				       ccd->ccd_unit, cs->sc_nccdisks);
+				printf("ccd%d: mirroring requires an even number of disks\n", ccd->ccd_unit );
+				while (ci > cs->sc_cinfo) {
+					ci--;
+					free(ci->ci_path, M_DEVBUF);
+				}
+				free(cs->sc_cinfo, M_DEVBUF);
+				return (EINVAL);
 			}
 			cs->sc_size = (cs->sc_nccdisks/2) * minsize;
 		}
@@ -595,7 +642,7 @@ ccdopen(dev, flags, fmt, p)
 		ccdgetdisklabel(dev);
 
 	/* Check that the partition exists. */
-	if (part != RAW_PART && ((part > lp->d_npartitions) ||
+	if (part != RAW_PART && ((part >= lp->d_npartitions) ||
 	    (lp->d_partitions[part].p_fstype == FS_UNUSED))) {
 		error = ENXIO;
 		goto done;
@@ -667,8 +714,7 @@ ccdstrategy(bp)
 {
 	register int unit = ccdunit(bp->b_dev);
 	register struct ccd_softc *cs = &ccd_softc[unit];
-	register daddr_t bn;
-	register int sz, s;
+	register int s;
 	int wlabel;
 	struct disklabel *lp;
 
@@ -850,6 +896,7 @@ ccdbuffer(cb, cs, bp, bn, addr, bcount)
 	 * Fill in the component buf structure.
 	 */
 	cbp = getccdbuf();
+	bzero(cbp, sizeof (struct ccdbuf));
 	cbp->cb_buf.b_flags = bp->b_flags | B_CALL;
 	cbp->cb_buf.b_iodone = (void (*)(struct buf *))ccdiodone;
 	cbp->cb_buf.b_proc = bp->b_proc;
@@ -884,6 +931,7 @@ ccdbuffer(cb, cs, bp, bn, addr, bcount)
 	    (cbp->cb_buf.b_flags & B_READ) == 0) {
 		/* mirror, start one more write */
 		cbp = getccdbuf();
+		bzero(cbp, sizeof (struct ccdbuf));
 		*cbp = *cb[0];
 		cbp->cb_buf.b_dev = ci2->ci_dev;
 		cbp->cb_buf.b_vp = ci2->ci_vp;
@@ -902,7 +950,6 @@ ccdintr(cs, bp)
 	register struct ccd_softc *cs;
 	register struct buf *bp;
 {
-
 #ifdef DEBUG
 	if (ccddebug & CCDB_FOLLOW)
 		printf("ccdintr(%x, %x)\n", cs, bp);
@@ -986,76 +1033,6 @@ ccdiodone(cbp)
 	splx(s);
 }
 
-/* ARGSUSED */
-int
-ccdread(dev, uio, flags)
-	dev_t dev;
-	struct uio *uio;
-	int flags;
-{
-	int unit = ccdunit(dev);
-	struct ccd_softc *cs;
-
-#ifdef DEBUG
-	if (ccddebug & CCDB_FOLLOW)
-		printf("ccdread(%x, %x)\n", dev, uio);
-#endif
-	if (unit >= numccd)
-		return (ENXIO);
-	cs = &ccd_softc[unit];
-
-	if ((cs->sc_flags & CCDF_INITED) == 0)
-		return (ENXIO);
-
-	/*
-	 * XXX: It's not clear that using minphys() is completely safe,
-	 * in particular, for raw I/O.  Underlying devices might have some
-	 * non-obvious limits, because of the copy to user-space.
-	 */
-	return (physio(ccdstrategy, NULL, dev, B_READ, minphys, uio));
-}
-
-/* ARGSUSED */
-int
-ccdwrite(dev, uio, flags)
-	dev_t dev;
-	struct uio *uio;
-	int flags;
-{
-	int unit = ccdunit(dev);
-	struct ccd_softc *cs;
-
-#ifdef DEBUG
-	if (ccddebug & CCDB_FOLLOW)
-		printf("ccdwrite(%x, %x)\n", dev, uio);
-#endif
-	if (unit >= numccd)
-		return (ENXIO);
-	cs = &ccd_softc[unit];
-
-	if ((cs->sc_flags & CCDF_INITED) == 0)
-		return (ENXIO);
-
-	/*
-	 * XXX: It's not clear that using minphys() is completely safe,
-	 * in particular, for raw I/O.  Underlying devices might have some
-	 * non-obvious limits, because of the copy to user-space.
-	 */
-	return (physio(ccdstrategy, NULL, dev, B_WRITE, minphys, uio));
-}
-
-static void
-loopdelay()
-{
-	int i, j, k, l;
-	printf("I'm now gonna wait for fifteen seconds\n");
-	printf("Press Ctl-Alt-Esc NOW!\n");
-	for (i = 0; i < 1000; i++)
-		for (j = 0; j < 1000; j++)
-			for (k = 0; k < 100; k++)
-				l = i * j * k;
-}
-
 int
 ccdioctl(dev, cmd, data, flag, p)
 	dev_t dev;
@@ -1308,7 +1285,7 @@ ccdioctl(dev, cmd, data, flag, p)
 				/*, &cs->sc_dkdev.dk_cpulabel); */
 		if (error == 0) {
 			if (cmd == DIOCWDINFO)
-				error = correct_writedisklabel(CCDLABELDEV(dev),
+				error = writedisklabel(CCDLABELDEV(dev),
 				    ccdstrategy, &cs->sc_dkdev.dk_label);
 				/*
 				    &cs->sc_dkdev.dk_cpulabel); */
@@ -1448,14 +1425,9 @@ ccdgetdisklabel(dev)
 	struct ccd_softc *cs = &ccd_softc[unit];
 	char *errstring;
 	struct disklabel *lp = &cs->sc_dkdev.dk_label;
-/*	struct cpu_disklabel *clp = &cs->sc_dkdev.dk_cpulabel; */
 	struct ccdgeom *ccg = &cs->sc_geom;
 
-	struct dos_partition dos_partdummy;
-	struct dkbad dkbaddummy;
-
 	bzero(lp, sizeof(*lp));
-/*	bzero(clp, sizeof(*clp)); */
 
 	lp->d_secperunit = cs->sc_size;
 	lp->d_secsize = ccg->ccg_secsize;
@@ -1486,9 +1458,8 @@ ccdgetdisklabel(dev)
 	/*
 	 * Call the generic disklabel extraction routine.
 	 */
-	if (errstring = correct_readdisklabel(CCDLABELDEV(dev), ccdstrategy,
-	    &cs->sc_dkdev.dk_label/*, &dos_partdummy, &dkbaddummy*/)) 
-		/*, &cs->sc_dkdev.dk_cpulabel)) */
+	if (errstring = readdisklabel(CCDLABELDEV(dev), ccdstrategy,
+	    &cs->sc_dkdev.dk_label))
 		ccdmakedisklabel(cs);
 
 #ifdef DEBUG
