@@ -37,7 +37,8 @@
  *
  * from: Utah Hdr: vn.c 1.13 94/04/02
  *
- *	@(#)vn.c	8.6 (Berkeley) 4/1/94
+ *	from: @(#)vn.c	8.6 (Berkeley) 4/1/94
+ * $FreeBSD: stable/2.2/sys/dev/vn/vn.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -83,11 +84,30 @@
 #include <sys/disklabel.h>
 #include <sys/diskslice.h>
 #include <sys/stat.h>
+#include <sys/conf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <miscfs/specfs/specdev.h>
 
 #include <sys/vnioctl.h>
 
+static	d_open_t	vnopen;
+static	d_close_t	vnclose;
+static	d_ioctl_t	vnioctl;
+static	d_dump_t	vndump;
+static	d_psize_t	vnsize;
+static	d_strategy_t	vnstrategy;
+
+#define CDEV_MAJOR 43
+#define BDEV_MAJOR 15
+static struct cdevsw vn_cdevsw;
+static struct bdevsw vn_bdevsw = 
+	{ vnopen,	vnclose,	vnstrategy,	vnioctl,	/*15*/
+	  vndump,	vnsize,		0,	"vn",	&vn_cdevsw,	-1 };
+
+
 #ifdef DEBUG
 int dovncluster = 1;
 int vndebug = 0x00;
@@ -107,6 +127,10 @@ int vndebug = 0x00;
 struct vn_softc {
 	int		 sc_flags;	/* flags */
 	size_t		 sc_size;	/* size of vn */
+#if defined(DEVFS) && defined(notyet)
+	void		*sc_bdev;	/* devfs token for whole disk */
+	void		*sc_cdev;	/* devfs token for raw whole disk */
+#endif
 	struct vnode	*sc_vp;		/* vnode */
 	struct ucred	*sc_cred;	/* credentials */
 	int		 sc_maxactive;	/* max # of active requests */
@@ -118,26 +142,17 @@ struct vn_softc {
 /* sc_flags */
 #define VNF_INITED	0x01
 
-struct vn_softc *vn_softc[NVN];
-u_long	vn_options;
+static struct vn_softc *vn_softc[NVN];
+static u_long	vn_options;
 
 #define IFOPT(vn,opt) if (((vn)->sc_options|vn_options) & (opt))
 
-/*
- * XXX these decls should be static (without __P(())) or elsewhere.
- */
-int	vnclose __P((dev_t dev, int flags, int mode, struct proc *p));
-int	vnopen __P((dev_t dev, int flags, int mode, struct proc *p));
-void	vnstrategy __P((struct buf *bp));
-int	vnioctl __P((dev_t dev, u_long cmd, caddr_t data, int flag, struct proc *p));
-void	vniodone __P((struct buf *bp));
-int	vnsetcred __P((struct vn_softc *vn, struct ucred *cred));
-void	vnshutdown __P((void));
-void	vnclear __P((struct vn_softc *vn));
-size_t	vnsize __P((dev_t dev));
-int	vndump __P((dev_t dev));
+static void	vniodone (struct buf *bp);
+static int	vnsetcred (struct vn_softc *vn, struct ucred *cred);
+static void	vnshutdown (int, void *);
+static void	vnclear (struct vn_softc *vn);
 
-int
+static	int
 vnclose(dev_t dev, int flags, int mode, struct proc *p)
 {
 	struct vn_softc *vn = vn_softc[vnunit(dev)];
@@ -148,7 +163,7 @@ vnclose(dev_t dev, int flags, int mode, struct proc *p)
 	return (0);
 }
 
-int
+static	int
 vnopen(dev_t dev, int flags, int mode, struct proc *p)
 {
 	int unit = vnunit(dev);
@@ -189,7 +204,8 @@ vnopen(dev_t dev, int flags, int mode, struct proc *p)
 					vn->sc_size;
 
 			return (dsopen("vn", dev, mode, &vn->sc_slices, &label,
-				       vnstrategy, (ds_setgeom_t *)NULL));
+				       vnstrategy, (ds_setgeom_t *)NULL,
+				       &vn_bdevsw, &vn_cdevsw));
 		}
 		if (dkslice(dev) != WHOLE_DISK_SLICE ||
 		    dkpart(dev) != RAW_PART ||
@@ -208,14 +224,15 @@ vnopen(dev_t dev, int flags, int mode, struct proc *p)
  * and the pageout daemon gets really unhappy (and so does the rest of the
  * system) when it runs out of memory.
  */
-void
+static	void
 vnstrategy(struct buf *bp)
 {
 	int unit = vnunit(bp->b_dev);
 	register struct vn_softc *vn = vn_softc[unit];
 	register daddr_t bn;
 	int error;
-	int sz;
+	int isvplocked = 0;
+	long sz;
 	struct uio auio;
 	struct iovec aiov;
 
@@ -255,7 +272,7 @@ vnstrategy(struct buf *bp)
 		aiov.iov_len = bp->b_bcount;
 		auio.uio_iov = &aiov;
 		auio.uio_iovcnt = 1;
-		auio.uio_offset = bn*DEV_BSIZE;
+		auio.uio_offset = dbtob(bn);
 		auio.uio_segflg = UIO_SYSSPACE;
 		if( bp->b_flags & B_READ)
 			auio.uio_rw = UIO_READ;
@@ -263,10 +280,18 @@ vnstrategy(struct buf *bp)
 			auio.uio_rw = UIO_WRITE;
 		auio.uio_resid = bp->b_bcount;
 		auio.uio_procp = curproc;
+		if (!VOP_ISLOCKED(vn->sc_vp)) {
+			isvplocked = 1;
+			VOP_LOCK(vn->sc_vp);
+		}
 		if( bp->b_flags & B_READ)
 			error = VOP_READ(vn->sc_vp, &auio, 0, vn->sc_cred);
 		else
 			error = VOP_WRITE(vn->sc_vp, &auio, 0, vn->sc_cred);
+		if (isvplocked) {
+			VOP_UNLOCK(vn->sc_vp);
+			isvplocked = 0;
+		}
 
 		bp->b_resid = auio.uio_resid;
 
@@ -274,15 +299,14 @@ vnstrategy(struct buf *bp)
 			bp->b_flags |= B_ERROR;
 		biodone(bp);
 	} else {
-		daddr_t bsize;
-		int flags, resid;
+		long bsize, resid;
+		off_t byten;
+		int flags;
 		caddr_t addr;
-
 		struct buf *nbp;
 
 		nbp = getvnbuf();
-
-		bn = dbtob(bn);
+		byten = dbtob(bn);
 		bsize = vn->sc_vp->v_mount->mnt_stat.f_iosize;
 		addr = bp->b_data;
 		flags = bp->b_flags | B_CALL;
@@ -292,14 +316,23 @@ vnstrategy(struct buf *bp)
 			int off, s, nra;
 
 			nra = 0;
-			error = VOP_BMAP(vn->sc_vp, bn / bsize, &vp, &nbn, &nra);
-			if (error == 0 && (long)nbn == -1)
+			if (!VOP_ISLOCKED(vn->sc_vp)) {
+				isvplocked = 1;
+				VOP_LOCK(vn->sc_vp);
+			}
+			error = VOP_BMAP(vn->sc_vp, (daddr_t)(byten / bsize),
+					 &vp, &nbn, &nra, NULL);
+			if (isvplocked) {
+				VOP_UNLOCK(vn->sc_vp);
+				isvplocked = 0;
+			}
+			if (error == 0 && nbn == -1)
 				error = EIO;
 
 			IFOPT(vn, VN_DONTCLUSTER)
 				nra = 0;
 
-			off = bn % bsize;
+			off = byten % bsize;
 			if (off)
 				sz = bsize - off;
 			else
@@ -316,8 +349,11 @@ vnstrategy(struct buf *bp)
 			}
 
 			IFOPT(vn,VN_IO)
-				printf("vnstrategy: vp %p/%p bn 0x%lx/0x%lx sz 0x%x\n",
-				       vn->sc_vp, vp, bn, nbn, sz);
+				printf(
+			/* XXX no %qx in kernel.  Synthesize it. */
+			"vnstrategy: vp %p/%p bn 0x%lx%08lx/0x%lx sz 0x%x\n",
+				       vn->sc_vp, vp, (long)(byten >> 32),
+				       (u_long)byten, nbn, sz);
 
 			nbp->b_flags = flags;
 			nbp->b_bcount = sz;
@@ -332,7 +368,6 @@ vnstrategy(struct buf *bp)
 			nbp->b_proc = bp->b_proc;
 			nbp->b_iodone = vniodone;
 			nbp->b_vp = vp;
-			nbp->b_pfcent = (int) bp;	/* XXX */
 			nbp->b_rcred = vn->sc_cred;	/* XXX crdup? */
 			nbp->b_wcred = vn->sc_cred;	/* XXX crdup? */
 			nbp->b_dirtyoff = bp->b_dirtyoff;
@@ -360,7 +395,7 @@ vnstrategy(struct buf *bp)
 				return;
 			}
 
-			bn += sz;
+			byten += sz;
 			addr += sz;
 			resid -= sz;
 		}
@@ -376,8 +411,8 @@ vniodone( struct buf *bp) {
 }
 
 /* ARGSUSED */
-int
-vnioctl(dev_t dev, u_long cmd, caddr_t data, int flag, struct proc *p)
+static	int
+vnioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 {
 	struct vn_softc *vn = vn_softc[vnunit(dev)];
 	struct vn_ioctl *vio;
@@ -388,7 +423,7 @@ vnioctl(dev_t dev, u_long cmd, caddr_t data, int flag, struct proc *p)
 
 
 	IFOPT(vn,VN_FOLLOW)
-		printf("vnioctl(0x%lx, 0x%lx, %p, 0x%x, %p): unit %d\n",
+		printf("vnioctl(0x%lx, 0x%x, %p, 0x%x, %p): unit %d\n",
 		       dev, cmd, data, flag, p, vnunit(dev));
 
 	switch (cmd) {
@@ -538,14 +573,16 @@ vnsetcred(struct vn_softc *vn, struct ucred *cred)
 	auio.uio_rw = UIO_READ;
 	auio.uio_segflg = UIO_SYSSPACE;
 	auio.uio_resid = aiov.iov_len;
+	VOP_LOCK(vn->sc_vp);
 	error = VOP_READ(vn->sc_vp, &auio, 0, vn->sc_cred);
+	VOP_UNLOCK(vn->sc_vp);
 
 	free(tmpbuf, M_TEMP);
 	return (error);
 }
 
 void
-vnshutdown()
+vnshutdown(int howto, void *ignored)
 {
 	int i;
 
@@ -554,8 +591,6 @@ vnshutdown()
 			vnclear(vn_softc[i]);
 }
 
-TEXT_SET(cleanup_set, vnshutdown);
-
 void
 vnclear(struct vn_softc *vn)
 {
@@ -576,7 +611,7 @@ vnclear(struct vn_softc *vn)
 		dsgone(&vn->sc_slices);
 }
 
-size_t
+static	int
 vnsize(dev_t dev)
 {
 	int unit = vnunit(dev);
@@ -587,9 +622,49 @@ vnsize(dev_t dev)
 	return(vn_softc[unit]->sc_size);
 }
 
-int
+static	int
 vndump(dev_t dev)
 {
 	return (ENODEV);
 }
+static vn_devsw_installed = 0;
+
+static void 
+vn_drvinit(void *unused)
+{
+#ifdef DEVFS
+	int mynor;
+	int unit;
+	struct vn_softc *vn;
+#endif
+
+	if( ! vn_devsw_installed ) {
+		if (at_shutdown(&vnshutdown, NULL, SHUTDOWN_POST_SYNC)) {
+			printf("vn: could not install shutdown hook\n");
+			return;
+		}
+		bdevsw_add_generic(BDEV_MAJOR, CDEV_MAJOR, &vn_bdevsw);
+#ifdef DEVFS
+		for (unit = 0; unit < NVN; unit++) {
+			vn = vn_softc[unit];
+			mynor = dkmakeminor(unit, WHOLE_DISK_SLICE, RAW_PART);
+			/*
+			 * XXX not saving tokens yet.  The vn devices don't
+			 * exist until after they have been opened :-).
+			 */
+			devfs_add_devswf(&vn_bdevsw, mynor, DV_BLK,
+					 UID_ROOT, GID_OPERATOR, 0640,
+					 "vn%d", unit);
+			devfs_add_devswf(&vn_cdevsw, mynor, DV_CHR,
+					 UID_ROOT, GID_OPERATOR, 0640,
+					 "rvn%d", unit);
+		}
+#endif
+		vn_devsw_installed = 1;
+	}
+}
+
+SYSINIT(vndev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,vn_drvinit,NULL)
+
+
 #endif
