@@ -54,14 +54,14 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/fpu_asm.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/fpu_asm.h 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
 #ifndef _FPU_ASM_H_
 #define _FPU_ASM_H_
 
-#include "fpu_emu.h"
+#include <gnu/i386/fpemul/fpu_emu.h>
 
 #define	EXCEPTION	_exception
 
