@@ -55,7 +55,7 @@
  *
  * W. Metzenthen   June 1994.
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/fpu_entry.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/fpu_entry.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -71,17 +71,23 @@
  +---------------------------------------------------------------------------*/
 
 
-#include "param.h"
-#include "systm.h"
-#include "proc.h"
-#include "machine/cpu.h"
-#include "machine/pcb.h"
+#include <sys/param.h>
+#include <sys/systm.h>
+#include <sys/proc.h>
+#include <sys/kernel.h>
 
-#include "fpu_emu.h"
-#include "fpu_system.h"
-#include "exception.h"
-#include "control_w.h"
-#include "status_w.h"
+#ifdef LKM
+#include <sys/lkm.h>
+#endif
+
+#include <machine/cpu.h>
+#include <machine/pcb.h>
+
+#include <gnu/i386/fpemul/fpu_emu.h>
+#include <gnu/i386/fpemul/fpu_system.h>
+#include <gnu/i386/fpemul/exception.h>
+#include <gnu/i386/fpemul/control_w.h>
+#include <gnu/i386/fpemul/status_w.h>
 
 
 #define __BAD__ Un_impl		/* Not implemented */
@@ -192,7 +198,7 @@ char    emulating = 0;
 #define math_abort(signo) \
     FPU_EIP = FPU_ORIG_EIP;REENTRANT_CHECK(OFF);return(signo);
 
-int
+static int
 math_emulate(struct trapframe * tframe)
 {
 
@@ -481,3 +487,47 @@ if (--lookahead_limit)
 	REENTRANT_CHECK(OFF);
 	return (0);		/* --pink-- */
 }
+
+#ifdef LKM
+MOD_MISC(gnufpu);
+static int
+gnufpu_load(struct lkm_table *lkmtp, int cmd)
+{
+	if (pmath_emulate) {
+		printf("Math emulator already present\n");
+		return EBUSY;
+	}
+	pmath_emulate = math_emulate;
+	return 0;
+}
+
+static int
+gnufpu_unload(struct lkm_table *lkmtp, int cmd)
+{
+	if (pmath_emulate != math_emulate) {
+		printf("Cannot unload another math emulator\n");
+		return EACCES;
+	}
+	pmath_emulate = 0;
+	return 0;
+}
+
+int
+gnufpu(struct lkm_table *lkmtp, int cmd, int ver)
+{
+	DISPATCH(lkmtp, cmd, ver, gnufpu_load, gnufpu_unload, lkm_nullcmd);
+}
+#else /* !LKM */
+
+static void
+gnufpu_init(void *unused)
+{
+	if (pmath_emulate)
+		printf("Another Math emulator already present\n");
+	else
+		pmath_emulate = math_emulate;
+}
+
+SYSINIT(gnufpu, SI_SUB_CPU, SI_ORDER_ANY, gnufpu_init, NULL);
+
+#endif /* LKM */
