@@ -56,20 +56,20 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/fpu_etc.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/fpu_etc.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
-#include "param.h"
-#include "proc.h"
-#include "machine/cpu.h"
-#include "machine/pcb.h"
+#include <sys/param.h>
+#include <sys/proc.h>
+#include <machine/cpu.h>
+#include <machine/pcb.h>
 
-#include "fpu_emu.h"
-#include "fpu_system.h"
-#include "exception.h"
-#include "status_w.h"
-#include "reg_constant.h"
+#include <gnu/i386/fpemul/fpu_emu.h>
+#include <gnu/i386/fpemul/fpu_system.h>
+#include <gnu/i386/fpemul/exception.h>
+#include <gnu/i386/fpemul/status_w.h>
+#include <gnu/i386/fpemul/reg_constant.h>
 
 
 static void
