@@ -1,6 +1,6 @@
 /*
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/fpu_proto.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/fpu_proto.h 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -41,7 +41,6 @@ extern void fsubp_(void);
 extern void fdivrp(void);
 extern void fdivp_(void);
 /* fpu_aux.c */
-extern void fclex(void);
 extern void finit(void);
 extern void finit_(void);
 extern void fstsw_(void);
@@ -59,7 +58,6 @@ extern int math_emulate(struct trapframe * info);
 /* fpu_etc.c */
 extern void fp_etc(void);
 /* fpu_trig.c */
-extern void convert_l2reg(long *arg, FPU_REG * dest);
 extern void trig_a(void);
 extern void trig_b(void);
 /* get_address.c */
@@ -70,7 +68,6 @@ extern void load_store_instr(char type);
 extern int poly_2xm1(FPU_REG * arg, FPU_REG * result);
 /* poly_atan.c */
 extern void poly_atan(FPU_REG * arg);
-extern void poly_add_1(FPU_REG * src);
 /* poly_l2.c */
 extern void poly_l2(FPU_REG * arg, FPU_REG * result);
 extern int poly_l2p1(FPU_REG * arg, FPU_REG * result);
