@@ -56,22 +56,24 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/fpu_trig.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/fpu_trig.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
 
-#include "param.h"
-#include "proc.h"
-#include "machine/cpu.h"
-#include "machine/pcb.h"
+#include <sys/param.h>
+#include <sys/proc.h>
+#include <machine/cpu.h>
+#include <machine/pcb.h>
 
-#include "fpu_emu.h"
-#include "fpu_system.h"
-#include "exception.h"
-#include "status_w.h"
-#include "reg_constant.h"
-#include "control_w.h"
+#include <gnu/i386/fpemul/fpu_emu.h>
+#include <gnu/i386/fpemul/fpu_system.h>
+#include <gnu/i386/fpemul/exception.h>
+#include <gnu/i386/fpemul/status_w.h>
+#include <gnu/i386/fpemul/reg_constant.h>
+#include <gnu/i386/fpemul/control_w.h>
+
+static void convert_l2reg(long *arg, FPU_REG * dest);
 
 static int
 trig_arg(FPU_REG * X)
@@ -104,7 +106,7 @@ trig_arg(FPU_REG * X)
 
 
 /* Convert a long to register */
-void
+static void
 convert_l2reg(long *arg, FPU_REG * dest)
 {
 	long    num = *arg;
