@@ -56,7 +56,7 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/get_address.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/get_address.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -67,16 +67,16 @@
  |    other processes using the emulator while swapping is in progress.      |
  +---------------------------------------------------------------------------*/
 
-#include "param.h"
-#include "proc.h"
-#include "systm.h"
-#include "machine/cpu.h"
-#include "machine/pcb.h"
-#include "machine/reg.h"
+#include <sys/param.h>
+#include <sys/proc.h>
+#include <sys/systm.h>
+#include <machine/cpu.h>
+#include <machine/pcb.h>
+#include <machine/reg.h>
 
-#include "fpu_emu.h"
-#include "fpu_system.h"
-#include "exception.h"
+#include <gnu/i386/fpemul/fpu_emu.h>
+#include <gnu/i386/fpemul/fpu_system.h>
+#include <gnu/i386/fpemul/exception.h>
 
 static int reg_offset[] = {
 tEAX, tECX, tEDX, tEBX, tESP, tEBP, tESI, tEDI};
