@@ -56,13 +56,13 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/poly_2xm1.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/poly_2xm1.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
-#include "exception.h"
-#include "reg_constant.h"
-#include "fpu_emu.h"
+#include <gnu/i386/fpemul/exception.h>
+#include <gnu/i386/fpemul/reg_constant.h>
+#include <gnu/i386/fpemul/fpu_emu.h>
 
 
 
