@@ -56,14 +56,14 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/poly_atan.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/poly_atan.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
-#include "exception.h"
-#include "reg_constant.h"
-#include "fpu_emu.h"
-#include "control_w.h"
+#include <gnu/i386/fpemul/exception.h>
+#include <gnu/i386/fpemul/reg_constant.h>
+#include <gnu/i386/fpemul/fpu_emu.h>
+#include <gnu/i386/fpemul/control_w.h>
 
 
 #define	HIPOWERon	6	/* odd poly, negative terms */
@@ -92,6 +92,7 @@ static unsigned denomterm[2] =
 {0xfc4bd208, 0xea2e6612};
 
 
+static void poly_add_1(FPU_REG * src);
 
 /*--- poly_atan() -----------------------------------------------------------+
  |                                                                           |
@@ -227,7 +228,7 @@ poly_atan(FPU_REG * arg)
    i.e. have an exponent (not checked) of EXP_BIAS-1 but need not
    be normalized.
    This function adds 1.0 to the (assumed positive) argument. */
-void
+static void
 poly_add_1(FPU_REG * src)
 {
 /* Rounding in a consistent direction produces better results
