@@ -54,14 +54,14 @@
  * W. Metzenthen   June 1994.
  *
  *
- * $FreeBSD: stable/2.1/sys/gnu/i386/fpemul/reg_constant.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/gnu/i386/fpemul/reg_constant.h 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
 #ifndef _REG_CONSTANT_H_
 #define _REG_CONSTANT_H_
 
-#include "fpu_emu.h"
+#include <gnu/i386/fpemul/fpu_emu.h>
 
 extern FPU_REG CONST_1;
 extern FPU_REG CONST_2;
