@@ -24,7 +24,7 @@
  * the rights to redistribute these changes.
  *
  *	from: Mach, Revision 2.2  92/04/04  11:35:49  rpd
- * $FreeBSD: stable/2.1/sys/i386/boot/biosboot/disk.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/boot/biosboot/disk.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -52,12 +52,12 @@
 #define	HEADS(di)	((((di)>>8)&0xff)+1)
 
 #ifdef DO_BAD144
-struct dkbad dkb;
-int do_bad144;
+static struct dkbad dkb;
+static int do_bad144;
 #endif DO_BAD144
-int bsize;
+static int bsize;
 
-int spt, spc;
+static int spt, spc;
 
 struct fs *fs;
 struct inode inode;
@@ -76,6 +76,7 @@ static int ra_end;
 static int ra_first;
 
 static int badsect(int sector);
+static char *Bread(int dosdev, int sector);
 
 int
 devopen(void)
@@ -114,7 +115,7 @@ devopen(void)
 		disklabel = *dl;	/* structure copy (maybe useful later)*/
 #endif	EMBEDDED_DISKLABEL
 		if (dl->d_magic != DISKMAGIC) {
-			printf("bad disklabel");
+			printf("bad disklabel\n");
 			return 1;
 		}
 		if( (maj == 4) || (maj == 0) || (maj == 1))
@@ -132,8 +133,12 @@ devopen(void)
 		boff = dl->d_partitions[part].p_offset -
 			dl->d_partitions[2].p_offset + sector;
 
-		/* This is a good idea for all disks */
 		bsize = dl->d_partitions[part].p_size;
+		if (bsize == 0) {
+			printf("empty partition\n");
+			return 1;
+		}
+
 #ifdef DO_BAD144
 		do_bad144 = 0;
 		if (dl->d_flags & D_BADSECT) {
@@ -207,7 +212,7 @@ devread(char *iodest, int sector, int cnt)
 }
 
 
-char *
+static char *
 Bread(int dosdev, int sector)
 {
 	if (dosdev != ra_dev || sector < ra_first || sector >= ra_end)
