@@ -12,11 +12,12 @@ extern int hostnamelen;
 extern unsigned long netmask;
 extern eth_reset();
 extern short aui;
+extern int howto;
 
 int cmd_ip(), cmd_server(), cmd_kernel(), cmd_help(), exit();
 int cmd_rootfs(), cmd_swapfs(), cmd_interface(), cmd_hostname();
 int cmd_netmask(), cmd_swapsize(), cmd_swapopts(), cmd_rootopts();
-int cmd_aui(), cmd_gateway();
+int cmd_aui(), cmd_gateway(), cmd_flags();
 
 struct bootcmds_t {
 	char *name;
@@ -39,6 +40,7 @@ struct bootcmds_t {
 	{"diskboot",	exit,		"          boot from disk"},
 	{"autoboot",	NULL,		"          continue"},
         {"trans",       cmd_aui,        "<on|off>     turn transceiver on|off"},
+	{"flags",       cmd_flags,      "[bcdghsv] set boot flags"},
 	{NULL,		NULL,		NULL}
 };
 
@@ -201,45 +203,55 @@ cmd_hostname(p)
 			(char*)&nfsdiskless.my_hostnam) + 3) & ~3;
 	else	printf("Hostname is: %s\r\n",nfsdiskless.my_hostnam);
 }
+
+static void mountopts(prefix,args,p)
+	char *prefix;    
+	struct nfs_args *args;
+	char *p;
+{
+        char *tmp;
+
+	if (*p) {
+		args->flags = NFSMNT_RSIZE | NFSMNT_WSIZE | NFSMNT_RESVPORT;
+		args->sotype = SOCK_DGRAM;
+		if ((tmp = (char *)substr(p,"rsize=")))
+			args->rsize=getdec(&tmp);
+		if ((tmp = (char *)substr(p,"wsize=")))
+			args->wsize=getdec(&tmp);
+		if ((tmp = (char *)substr(p,"intr")))
+			args->flags |= NFSMNT_INT;
+		if ((tmp = (char *)substr(p,"soft")))
+			args->flags |= NFSMNT_SOFT;
+		if ((tmp = (char *)substr(p,"noconn")))
+			args->flags |= NFSMNT_NOCONN;
+		if ((tmp = (char *)substr(p, "tcp")))
+		    args->sotype = SOCK_STREAM;
+	} else {
+		printf("%s mount options: rsize=%d,wsize=%d,resvport",
+		       prefix,
+		       args->rsize,
+		       args->wsize);
+		if (args->flags & NFSMNT_INT)
+			printf (",intr");
+		if (args->flags & NFSMNT_SOFT)
+			printf (",soft");
+		if (args->flags & NFSMNT_NOCONN)
+			printf (",noconn");
+		if (args->sotype == SOCK_STREAM)
+			printf (",tcp");
+		else
+			printf (",udp");
+		printf ("\r\n");
+	}
+}
+
 /**************************************************************************
 CMD_ROOTOPTS - Set root mount options
 **************************************************************************/
 cmd_rootopts(p)
         char *p;
 {
-        char *tmp;
-
-        if (*p) {
-                nfsdiskless.root_args.flags = NFSMNT_RSIZE | NFSMNT_WSIZE;
-                nfsdiskless.root_args.sotype = SOCK_DGRAM;
-                if ((tmp = (char *)substr(p,"rsize=")))
-                        nfsdiskless.root_args.rsize=getdec(&tmp);
-                if ((tmp = (char *)substr(p,"wsize=")))
-                        nfsdiskless.root_args.wsize=getdec(&tmp);
-                if ((tmp = (char *)substr(p,"resvport")))
-                        nfsdiskless.root_args.flags |= NFSMNT_RESVPORT;
-                if ((tmp = (char *)substr(p,"intr")))
-                        nfsdiskless.root_args.flags |= NFSMNT_INT;
-                if ((tmp = (char *)substr(p,"soft")))
-                        nfsdiskless.root_args.flags |= NFSMNT_SOFT;
-                if ((tmp = (char *)substr(p, "tcp")))
-                         nfsdiskless.root_args.sotype = SOCK_STREAM;
-        } else {
-                printf("Rootfs mount options: rsize=%d,wsize=%d",
-                nfsdiskless.root_args.rsize,
-                nfsdiskless.root_args.wsize);
-                if (nfsdiskless.root_args.flags & NFSMNT_RESVPORT)
-                        printf (",resvport");
-                if (nfsdiskless.root_args.flags & NFSMNT_SOFT)
-                        printf (",soft");
-                if (nfsdiskless.root_args.flags & NFSMNT_INT)
-                        printf (",intr");
-                if (nfsdiskless.root_args.sotype == SOCK_STREAM)
-                        printf (",tcp");
-                else
-                        printf (",udp");
-                printf ("\r\n");
-        }
+	mountopts("Rootfs",&nfsdiskless.root_args,p);
 }
 
 /**************************************************************************
@@ -248,39 +260,34 @@ CMD_SWAPOPTS - Set swap mount options
 cmd_swapopts(p)
         char *p;
 {
-	char *tmp;
+	mountopts("Swapfs",&nfsdiskless.swap_args,p);
+}
 
-	if (*p) {
-                nfsdiskless.swap_args.flags = NFSMNT_RSIZE | NFSMNT_WSIZE;
-                nfsdiskless.swap_args.sotype = SOCK_DGRAM;
-		if ((tmp = (char *)substr(p,"rsize=")))
-			nfsdiskless.swap_args.rsize=getdec(&tmp);
-		if ((tmp = (char *)substr(p,"wsize=")))
-			nfsdiskless.swap_args.wsize=getdec(&tmp);
-		if ((tmp = (char *)substr(p,"resvport")))
-			nfsdiskless.swap_args.flags |= NFSMNT_RESVPORT;
-		if ((tmp = (char *)substr(p,"intr")))
-			nfsdiskless.swap_args.flags |= NFSMNT_INT;
-		if ((tmp = (char *)substr(p,"soft")))
-			nfsdiskless.swap_args.flags |= NFSMNT_SOFT;
-		if ((tmp = (char *)substr(p, "tcp")))
-			 nfsdiskless.swap_args.sotype = SOCK_STREAM;
-        } else {
-		printf("Swapfs mount options: rsize=%d,wsize=%d",
-		nfsdiskless.swap_args.rsize,
-		nfsdiskless.swap_args.wsize);
-		if (nfsdiskless.swap_args.flags & NFSMNT_RESVPORT)
-			printf (",resvport");
-		if (nfsdiskless.swap_args.flags & NFSMNT_SOFT)
-			printf (",soft");
-		if (nfsdiskless.swap_args.flags & NFSMNT_INT)
-			printf (",intr");
-		if (nfsdiskless.swap_args.sotype == SOCK_STREAM)
-			printf (",tcp");
-		else
-			printf (",udp");
-		printf ("\r\n");
-        }
+/**************************************************************************
+CMD_FLAGS - Set boot flags
+**************************************************************************/
+cmd_flags(buf)
+	char *buf;
+{
+	char p;
+	int flags = 0;
+
+	while ((p = *buf++))
+	switch (p) {
+	case 'b':	flags |= RB_HALT; break;
+	case 'c':	flags |= RB_CONFIG; break;
+	case 'd':	flags |= RB_KDB; break;
+	case 'g':	flags |= RB_GDB; break;
+	case 'h':	flags ^= RB_SERIAL; break;
+	case 's':	flags |= RB_SINGLE; break;
+	case 'v':	flags |= RB_VERBOSE; break;
+	case ' ':
+	case '\t':	break;
+	default:	printf("Unknown boot flag: %c\n", p);
+	}
+
+	howto = flags;
+	return(0);
 }
 
 /**************************************************************************
@@ -302,7 +309,7 @@ execute(buf)
 		if ((!(*q)) && ((*p == ' ') || (*p == '\t') || (!(*p)))) {
 			if (!cmd->func) 
 				return(1);
-			while (*p == ' ')
+			while (*p == ' ' || *p == '\t')
 				p++;
 			(cmd->func)(p);
 			return(0);
