@@ -19,6 +19,11 @@ SMC8416 support added by Bill Paul (wpaul@ctr.columbia.edu) on 12/25/94
 
 **************************************************************************/
 
+DELAY(int x)
+{ volatile long a, b, l;
+   for (x; x>0; x--) b=a;
+}  
+
 #include "netboot.h"
 #include "ns8390.h"
 
@@ -43,9 +48,18 @@ unsigned char  *eth_node_addr;
 The following two variables are used externally
 **************************************************************************/
 char eth_driver[] = "ed0";
-char packet[ETH_MAX_PACKET];
+char packet[ETHER_MAX_LEN];
 int  packetlen;
 
+#ifdef	INCLUDE_NE
+static unsigned short ne_base_list[]= {
+#ifdef	NE_BASE
+	NE_BASE,
+#endif
+	0xff80, 0xff40, 0xff00, 0xfec0,
+	0x280, 0x300, 0
+};
+#endif
 /**************************************************************************
 ETH_PROBE - Look for an adapter
 **************************************************************************/
@@ -66,6 +80,10 @@ eth_probe()
 	for (eth_asic_base = WD_LOW_BASE; eth_asic_base <= WD_HIGH_BASE;
 		eth_asic_base += 0x20) {
 		chksum = 0;
+		/* Check for WD/SMC card by checking ethernet address */
+		if (inb(eth_asic_base+8) != 0) continue;
+		if (inb(eth_asic_base+9) != 0) continue;
+		if (inb(eth_asic_base+10) != 0xC0) continue;
 		for (i=8; i<16; i++)
 			chksum += inb(i+eth_asic_base);
 		if ((chksum & 0x00FF) == 0x00FF)
@@ -133,8 +151,7 @@ eth_probe()
 					WD_LAAR_M16EN | WD_LAAR_L16EN | 1));
 			}
 		}
-		printf("\r\n");
-
+		goto found_board;
 	}
 #endif
 #ifdef INCLUDE_3COM
@@ -256,7 +273,7 @@ eth_probe()
                 outb(eth_asic_base + _3COM_PSTR, eth_tx_start);
                 outb(eth_asic_base + _3COM_PSPR, eth_memsize);
 
-                printf ("\r\n");
+		goto found_board;
 
         }
 #endif
@@ -267,15 +284,24 @@ eth_probe()
 	if (eth_vendor == VENDOR_NONE) {
 		char romdata[16], testbuf[32];
 		char test[] = "NE1000/2000 memory";
+		unsigned short *tent_base=ne_base_list;
 		eth_bmem = (char *)0;		/* No shared memory */
-		eth_asic_base = NE_BASE + NE_ASIC_OFFSET;
-		eth_nic_base = NE_BASE;
+ne_again:
+		eth_asic_base = *tent_base + NE_ASIC_OFFSET;
+		eth_nic_base = *tent_base;
+		printf("Looking for NE1000/NE2000 at 0x%x\n", eth_nic_base);
+
 		eth_vendor = VENDOR_NOVELL;
 		eth_flags = FLAG_PIO;
 		eth_memsize = MEM_16384;
 		eth_tx_start = 32;
+#ifdef GWETHER
+		outb(eth_asic_base + NE_RESET, 0);
+		DELAY(200);
+#endif
 		c = inb(eth_asic_base + NE_RESET);
 		outb(eth_asic_base + NE_RESET, c);
+		DELAY(5000);
 	        inb(0x84);
 		outb(eth_nic_base + D8390_P0_COMMAND, D8390_COMMAND_STP |
 			D8390_COMMAND_RD2);
@@ -295,18 +321,25 @@ eth_probe()
 			outb(eth_nic_base + D8390_P0_PSTOP, MEM_32768);
 			eth_pio_write(test, 16384, sizeof(test));
 			eth_pio_read(16384, testbuf, sizeof(test));
-			if (!bcompare(testbuf, test, sizeof(test))) return (0);
+			if (!bcompare(testbuf, test, sizeof(test)))
+				if (*++tent_base)
+					goto ne_again;
+				else
+					return (0);
 		}
 		eth_pio_read(0, romdata, 16);
-		printf("\r\nNE1000/NE2000 base 0x%x, addr ", eth_nic_base);
+		printf("\nNE1000/NE2000 (%d bit) base 0x%x, addr ",
+			eth_flags & FLAG_16BIT ? 16:8, eth_nic_base);
 		for (i=0; i<6; i++) {
 			printf("%b",(int)(arptable[ARP_CLIENT].node[i] = romdata[i
 				+ ((eth_flags & FLAG_16BIT) ? i : 0)]));
 			if (i < 5) printf (":");
 		}
-		printf("\r\n");
+		goto found_board;
 	}
 #endif
+found_board:
+	printf("\r\n");
 	if (eth_vendor == VENDOR_NONE) return(0);
 
         if (eth_vendor != VENDOR_3COM) eth_rmem = eth_bmem;
@@ -392,12 +425,12 @@ eth_transmit(d,t,s,p)
 #ifdef INCLUDE_3COM
         if (eth_vendor == VENDOR_3COM) {
                 bcopy(d, eth_bmem, 6);                             /* dst */
-                bcopy(eth_node_addr, eth_bmem+6, ETHER_ADDR_SIZE); /* src */
+                bcopy(eth_node_addr, eth_bmem+6, ETHER_ADDR_LEN);  /* src */
                 *(eth_bmem+12) = t>>8;                             /* type */
                 *(eth_bmem+13) = t;
                 bcopy(p, eth_bmem+14, s);
                 s += 14;
-                while (s < ETH_MIN_PACKET) *(eth_bmem+(s++)) = 0;
+                while (s < ETHER_MIN_LEN) *(eth_bmem+(s++)) = 0;
         }
 #endif
 #ifdef INCLUDE_WD
@@ -412,12 +445,12 @@ eth_transmit(d,t,s,p)
 		}
 		inb(0x84);
 		bcopy(d, eth_bmem, 6);				   /* dst */
-		bcopy(eth_node_addr, eth_bmem+6, ETHER_ADDR_SIZE); /* src */
+		bcopy(eth_node_addr, eth_bmem+6, ETHER_ADDR_LEN);  /* src */
 		*(eth_bmem+12) = t>>8;				   /* type */
 		*(eth_bmem+13) = t;
 		bcopy(p, eth_bmem+14, s);
 		s += 14;
-		while (s < ETH_MIN_PACKET) *(eth_bmem+(s++)) = 0;
+		while (s < ETHER_MIN_LEN) *(eth_bmem+(s++)) = 0;
 		if (eth_flags & FLAG_790) {
 			outb(eth_asic_base + WD_MSR, 0);
 			inb(0x84);
@@ -437,7 +470,7 @@ eth_transmit(d,t,s,p)
 		eth_pio_write(&type, (eth_tx_start<<8)+12, 2);
 		eth_pio_write(p, (eth_tx_start<<8)+14, s);
 		s += 14;
-		if (s < ETH_MIN_PACKET) s = ETH_MIN_PACKET;
+		if (s < ETHER_MIN_LEN) s = ETHER_MIN_LEN;
 	}
 #endif
 	twiddle();
@@ -468,7 +501,7 @@ eth_poll()
 	int ret = 0;
 	unsigned short type = 0;
 	unsigned char bound,curr,rstat;
-	unsigned short len;
+	unsigned short len, copylen;
 	unsigned short pktoff;
 	unsigned char *p;
 	struct ringbuffer pkthdr;
@@ -507,7 +540,7 @@ eth_poll()
 	bound = pkthdr.bound;		/* New bound ptr */
 	if ( (pkthdr.status & D8390_RSTAT_PRX) && (len > 14) && (len < 1518)) {
 		p = packet;
-		packetlen = len;
+		packetlen = copylen = len;
 		len = (eth_memsize << 8) - pktoff;
 		if (packetlen > len) {		/* We have a wrap-around */
 			if (eth_flags & FLAG_PIO)
@@ -516,12 +549,12 @@ eth_poll()
 				bcopy(eth_rmem + pktoff, p, len);
 			pktoff = (eth_tx_start + D8390_TXBUF_SIZE) << 8;
 			p += len;
-			packetlen -= len;
+			copylen -= len;
 		}
 		if (eth_flags & FLAG_PIO)
-			eth_pio_read(pktoff, p, packetlen);
+			eth_pio_read(pktoff, p, copylen);
 		else
-			bcopy(eth_rmem + pktoff, p, packetlen);
+			bcopy(eth_rmem + pktoff, p, copylen);
 
 		type = (packet[12]<<8) | packet[13];
 		ret = 1;
@@ -544,7 +577,7 @@ eth_poll()
 	if (ret && (type == ARP)) {
 		struct arprequest *arpreq;
 		unsigned long reqip;
-		arpreq = (struct arprequest *)&packet[ETHER_HDR_SIZE];
+		arpreq = (struct arprequest *)&packet[ETHER_HDR_LEN];
 		convert_ipaddr(&reqip, arpreq->tipaddr);
 		if ((ntohs(arpreq->opcode) == ARP_REQUEST) &&
 			(reqip == arptable[ARP_CLIENT].ipaddr)) {
@@ -565,11 +598,10 @@ eth_poll()
 /**************************************************************************
 ETH_PIO_READ - Read a frame via Programmed I/O
 **************************************************************************/
-eth_pio_read(src, dst, cnt, init)
+eth_pio_read(src, dst, cnt)
 	unsigned short src;
 	unsigned char *dst;
 	unsigned short cnt;
-	int init;
 {
 	if (cnt & 1) cnt++;
 	outb(eth_nic_base + D8390_P0_COMMAND, D8390_COMMAND_RD2 |
@@ -596,11 +628,10 @@ eth_pio_read(src, dst, cnt, init)
 /**************************************************************************
 ETH_PIO_WRITE - Write a frame via Programmed I/O
 **************************************************************************/
-eth_pio_write(src, dst, cnt, init)
+eth_pio_write(src, dst, cnt)
 	unsigned char *src;
 	unsigned short dst;
 	unsigned short cnt;
-	int init;
 {
 	outb(eth_nic_base + D8390_P0_COMMAND, D8390_COMMAND_RD2 |
 		D8390_COMMAND_STA);
@@ -623,8 +654,9 @@ eth_pio_write(src, dst, cnt, init)
 		while (cnt--)
 			outb(eth_asic_base + NE_DATA, *(src++));
 	}
+	cnt = 200;
 	while((inb(eth_nic_base + D8390_P0_ISR) & D8390_ISR_RDC)
-		!= D8390_ISR_RDC);
+		!= D8390_ISR_RDC && --cnt);
 }
 #else
 /**************************************************************************
