@@ -14,17 +14,18 @@
  *
  * commenced: Sun Sep 27 18:14:01 PDT 1992
  *
- * $FreeBSD: stable/2.1/sys/i386/eisa/aha1742.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/eisa/aha1742.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/types.h>
 
 #ifdef	KERNEL			/* don't laugh, it compiles as a program too.. look */
+#include "opt_ddb.h"
 #include "ahb.h"
 #include <sys/param.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
 #include <sys/systm.h>
-#include <sys/devconf.h>
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/proc.h>
@@ -273,7 +274,7 @@ static struct ecb *
 		ahb_ecb_phys_kv __P((struct ahb_data *ahb, physaddr ecb_phys));
 static int	ahb_init __P((struct ahb_data *ahb));
 static void	ahbintr __P((void *arg));
-static char	*ahbmatch __P((eisa_id_t type));
+static const char *ahbmatch __P((eisa_id_t type));
 static void	ahbminphys __P((struct buf *bp));
 static int	ahb_poll __P((struct ahb_data *ahb, int wait));
 #ifdef AHBDEBUG
@@ -294,6 +295,7 @@ static	struct	ecb *cheat;
 
 static  u_long		ahb_unit = 0;
 static 	int		ahb_debug = 0;
+SYSCTL_INT(_debug, OID_AUTO, ahb_debug, CTLFLAG_RW, &ahb_debug, 0, "");
 
 #define AHB_SHOWECBS 0x01
 #define AHB_SHOWINTS 0x02
@@ -338,18 +340,6 @@ static struct scsi_device ahb_dev =
     { 0, 0 }
 };
 
-static struct kern_devconf kdc_ahb = {
-        0, 0, 0,                /* filled in by dev_attach */
-        "ahb", 0, { MDDT_EISA, 0, "bio" },
-        eisa_generic_externalize, 0, 0, EISA_EXTERNALLEN,
-        &kdc_eisa0,             /* parent */
-        0,                      /* parentdata */
-        DC_UNCONFIGURED,        /* always start out here */
-        NULL,
-        DC_CLS_MISC             /* host adapters aren't special */
-};
-
-
 #endif /*KERNEL */
 
 #ifndef	KERNEL
@@ -445,7 +435,7 @@ ahb_send_immed(struct ahb_data *ahb, int target, u_long cmd)
 	splx(s);
 }
 
-static	char *
+static const char *
 ahbmatch(type)     
 	eisa_id_t type;
 {                         
@@ -501,7 +491,7 @@ ahbprobe(void)
                                 continue;
 		}               
 		eisa_add_intr(e_dev, irq);
-		eisa_registerdev(e_dev, &ahb_eisa_driver, &kdc_ahb);
+		eisa_registerdev(e_dev, &ahb_eisa_driver);
 		count++;        
 	}               
 	return count;   
@@ -593,7 +583,12 @@ ahb_attach(e_dev)
 	int	unit = e_dev->unit;
 	struct	ahb_data *ahb;
 	resvaddr_t *iospace;
-	int	irq = ffs(e_dev->ioconf.irq) - 1;
+	int	irq;
+
+	if (TAILQ_FIRST(&e_dev->ioconf.irqs) == NULL)
+		return (-1);
+
+	irq = TAILQ_FIRST(&e_dev->ioconf.irqs)->irq_no;
 
 	iospace = e_dev->ioconf.ioaddrs.lh_first;
 
@@ -637,7 +632,6 @@ ahb_attach(e_dev)
 		eisa_release_intr(e_dev, irq, ahbintr);
 		return -1;
         }
-	e_dev->kdc->kdc_state = DC_BUSY; /* host adapters always busy */
 
 	/* Attach sub-devices - always succeeds */
 	ahb_bus_attach(ahb);
@@ -706,7 +700,6 @@ ahbintr(arg)
 	int     target;
 	long int mboxval;
 	int	port;
-	int	unit;
 
 	ahb = (struct ahb_data *)arg;
 	port = ahb->baseport;
