@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1995, 1996 Justin T. Gibbs.
  * All rights reserved.
- *
+ * 
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,18 +13,31 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Absolutely no warranty of function or purpose is made by the author
- *    Justin T. Gibbs.
- * 4. Modifications may be freely made to this file if the above conditions
- *    are met.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND  
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/eisa/eisaconf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/eisa/eisaconf.c 50950 1999-09-05 08:39:22Z peter $
  */
+
+#include "opt_eisa.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/devconf.h>
 #include <sys/kernel.h>
-#include <sys/conf.h>		/* For kdc_isa */
+#include <sys/sysctl.h>
+#include <sys/conf.h>
 #include <sys/malloc.h>
 
 #include <i386/eisa/eisaconf.h>
@@ -36,20 +49,6 @@ struct eisa_device_node{
 	struct	eisa_device_node *next;
 };
 
-extern struct kern_devconf kdc_cpu0;
-extern int bootverbose;
- 
-struct kern_devconf kdc_eisa0 = {
-	0, 0, 0,                /* filled in by dev_attach */ 
-	"eisa", 0, { MDDT_BUS, 0 },
-	0, 0, 0, BUS_EXTERNALLEN,
-	&kdc_cpu0,              /* parent is the CPU */
-	0,                      /* no parentdata */
-	DC_BUSY,                /* busses are always busy */
-	NULL,        
-	DC_CLS_BUS              /* class */
-};
-
 /* 
  * This should probably be a list of "struct device" once it exists.
  * A struct device will incorperate ioconf and driver entry point data
@@ -99,6 +98,12 @@ static struct {
 	int	column;		/* How much we have output so far. */
 #define	MAX_COL 80
 } reg_state;
+
+/* Global variable, so UserConfig can change it. */
+#ifndef EISA_SLOTS
+#define EISA_SLOTS 10   /* PCI clashes with higher ones.. fix later */
+#endif
+int num_eisa_slots = EISA_SLOTS;
 		
 /*
 ** probe for EISA devices
@@ -107,7 +112,6 @@ void
 eisa_configure()
 {
 	int i,slot;
-	char *id_string;
 	struct eisa_device_node *dev_node;
 	struct eisa_driver **e_drvp;
 	struct eisa_driver *e_drv;
@@ -117,7 +121,7 @@ eisa_configure()
 
 	e_drvp = (struct eisa_driver**)eisadriver_set.ls_items;
 
-	for (slot = 0; slot < EISA_SLOTS; eisaBase+=0x1000, slot++) {
+	for (slot = 0; slot < num_eisa_slots; eisaBase+=0x1000, slot++) {
 		int id_size = sizeof(eisa_id);
 		eisa_id = 0;
     		for( i = 0; i < id_size; i++ ) {
@@ -138,30 +142,15 @@ eisa_configure()
 		e_dev = &(dev_node->dev);
 
 		e_dev->id = eisa_id;
-		/*
-		 * Add an EISA ID based descriptive name incase we don't
-		 * have a driver for it.  We do this now instead of after all
-		 * probes because in the future, the eisa module will only
-		 * be responsible for creating the list of devices in the system
-		 * for the configuration manager to use.
-		 */
-		e_dev->full_name = (char *)malloc(10*sizeof(char),
-						  M_DEVBUF, M_NOWAIT);
-		if (!e_dev->full_name) {
-			panic("Eisa probe unable to malloc");
-		}
-		sprintf(e_dev->full_name, "%c%c%c%x%x",
-			EISA_MFCTR_CHAR0(e_dev->id),
-			EISA_MFCTR_CHAR1(e_dev->id),
-			EISA_MFCTR_CHAR2(e_dev->id),
-			EISA_PRODUCT_ID(e_dev->id),
-			EISA_REVISION_ID(e_dev->id));
+
+		e_dev->full_name = "Unattached Device";
 
 		e_dev->ioconf.slot = slot; 
 
 		/* Initialize our lists of reserved addresses */
 		LIST_INIT(&(e_dev->ioconf.ioaddrs));
 		LIST_INIT(&(e_dev->ioconf.maddrs));
+		TAILQ_INIT(&(e_dev->ioconf.irqs));
 
 		*eisa_dev_list_tail = dev_node;
 		eisa_dev_list_tail = &dev_node->next;
@@ -175,18 +164,23 @@ eisa_configure()
 
 	/* The first will be the motherboard in a true EISA system */
 	if (dev_node && (dev_node->dev.ioconf.slot == 0)) {
+		char *idstring;
+
 		e_dev = &dev_node->dev;
 		e_dev->driver = &mainboard_drv;
 		e_dev->unit = (*e_dev->driver->unit)++; 
-		id_string = e_dev->full_name;
-		e_dev->full_name = (char *)malloc(strlen(e_dev->full_name)
-						  + sizeof(" (System Board)")
-						  + 1, M_DEVBUF, M_NOWAIT);
-		if (!e_dev->full_name) {
+		idstring = (char *)malloc(8 + sizeof(" (System Board)") + 1,
+					  M_DEVBUF, M_NOWAIT);
+		if (idstring == NULL) {
 			panic("Eisa probe unable to malloc");
 		}
-		sprintf(e_dev->full_name, "%s (System Board)", id_string);
-		free(id_string, M_DEVBUF);
+		sprintf(idstring, "%c%c%c%x%x (System Board)",
+			EISA_MFCTR_CHAR0(e_dev->id),
+			EISA_MFCTR_CHAR1(e_dev->id),
+			EISA_MFCTR_CHAR2(e_dev->id),
+			EISA_PRODUCT_ID(e_dev->id),
+			EISA_REVISION_ID(e_dev->id));
+		e_dev->full_name = idstring;
 
 		printf("%s%ld: <%s>\n",
 		       e_dev->driver->name,
@@ -194,16 +188,6 @@ eisa_configure()
 		       e_dev->full_name);
 
 		/* Should set the iosize, but I don't have a spec handy */
-		kdc_eisa0.kdc_description = 
-			(char *)malloc(strlen(e_dev->full_name)
-				       + sizeof("EISA bus <>")
-				       + 1, M_DEVBUF, M_NOWAIT);
-		if (!kdc_eisa0.kdc_description) {
-			panic("Eisa probe unable to malloc");
-		}
-		sprintf((char *)kdc_eisa0.kdc_description, "EISA bus <%s>",
-			e_dev->full_name);
-		dev_attach(&kdc_eisa0);
 		printf("Probing for devices on the EISA bus\n");
 		dev_node = dev_node->next;
 	}
@@ -253,15 +237,18 @@ eisa_configure()
 			}
 			/* Ensure registration has ended */
 			reg_state.in_registration = 0;
-			e_dev->kdc->kdc_unit = e_dev->unit;
 		}
 		else {
 			/* Announce unattached device */
-			printf("%s0:%d <%s=0x%x> unknown device\n",
-				mainboard_drv.name,
-				e_dev->ioconf.slot,
-				e_dev->full_name,
-				e_dev->id);
+! 			printf("%s0:%d <%c%c%c%x%x=0x%x> unknown device\n",
+  				mainboard_drv.name,
+  				e_dev->ioconf.slot,
+				EISA_MFCTR_CHAR0(e_dev->id),
+				EISA_MFCTR_CHAR1(e_dev->id), 
+				EISA_MFCTR_CHAR2(e_dev->id), 
+				EISA_PRODUCT_ID(e_dev->id),
+				EISA_REVISION_ID(e_dev->id),
+  				e_dev->id);
 		}
 	}
 }
@@ -269,7 +256,7 @@ eisa_configure()
 struct eisa_device *
 eisa_match_dev(e_dev, match_func)
 	struct eisa_device *e_dev;
-	char* (*match_func)(eisa_id_t);
+	const char* (*match_func)(eisa_id_t);
 {
 	struct eisa_device_node *e_node = eisa_dev_list;
 
@@ -279,14 +266,13 @@ eisa_match_dev(e_dev, match_func)
 	}
 
 	for(; e_node; e_node = e_node->next) {
-		char *result;
+		const char *result;
 		if (e_node->dev.driver) {
 			/* Already claimed */
 			continue;
 		}
 		result = (*match_func)(e_node->dev.id);
 		if (result) {
-			free(e_node->dev.full_name, M_DEVBUF);
 			e_node->dev.full_name = result;
 			return (&(e_node->dev));
 		}
@@ -332,17 +318,16 @@ eisa_reg_print(e_dev, string, separator)
 	char *string;
 	char *separator;
 {
-#if PRINTF_RETURNS_INT
 	int len = strlen(string);
 
-	if( separator )
+	if(separator)
 		len++;
 
 	if(reg_state.column + len > MAX_COL) {
 		printf("\n");
 		reg_state.column = 0;
 	}
-	else if( separator ) {
+	else if(separator) {
 		printf("%c", *separator);
 		reg_state.column++;
 	}
@@ -354,17 +339,6 @@ eisa_reg_print(e_dev, string, separator)
 					   string);
 	else
 		reg_state.column += printf("%s", string);
-#else
-	if(reg_state.column == 0) {
-		printf("%s%ld:%s",
-		        e_dev->driver->name,
-			e_dev->unit,
-			string);
-		reg_state.column++;
-	}
-	else
-		printf("%s", string);
-#endif
 }
 
 /* Interrupt and I/O space registration facitlities */
@@ -374,21 +348,11 @@ eisa_reg_end(e_dev)
 {
 	if( reg_state.in_registration )
 	{
-		/*
-		 * The device should have called eisa_registerdev()
-		 * during its probe.  So hopefully we can use the kdc
-		 * to weed out ISA/VL devices that use EISA id registers.
-		 */
 		char string[25];
 
-		if (e_dev->kdc && (e_dev->kdc->kdc_parent == &kdc_isa0)) {
-			sprintf(string, " on isa");
-		}
-		else {
-			sprintf(string, " on %s0 slot %d",
-				mainboard_drv.name,
-				e_dev->ioconf.slot);
-		}
+		sprintf(string, " on %s0 slot %d",
+			mainboard_drv.name,
+			e_dev->ioconf.slot);
 		eisa_reg_print(e_dev, string, NULL);
 		printf("\n");
 		reg_state.in_registration = 0;
@@ -403,7 +367,16 @@ eisa_add_intr(e_dev, irq)
 	struct eisa_device *e_dev;
 	int irq;
 {
-	e_dev->ioconf.irq |= 1ul << irq;
+	struct	irq_node *irq_info;
+ 
+	irq_info = (struct irq_node *)malloc(sizeof(*irq_info), M_DEVBUF,
+					     M_NOWAIT);
+	if (irq_info == NULL)
+		return (1);
+
+	irq_info->irq_no = irq;
+	irq_info->idesc = NULL;
+	TAILQ_INSERT_TAIL(&e_dev->ioconf.irqs, irq_info, links);
 	return 0;
 }
 
@@ -411,13 +384,11 @@ int
 eisa_reg_intr(e_dev, irq, func, arg, maskptr, shared)
 	struct eisa_device *e_dev;
 	int   irq;
-	void  (*func)(void *);
+	void (*func)(void *);
 	void  *arg;
 	u_int *maskptr;
 	int   shared;
 {
-	int result;
-	int s;
 	char string[25];
 	char separator = ',';
 
@@ -432,30 +403,44 @@ eisa_reg_intr(e_dev, irq, func, arg, maskptr, shared)
         	return 1;
 #endif
 	if (reg_state.in_registration) {
-		s = splhigh();
 		/*
-		 * This should really go to a routine that can optionally
-		 * handle shared interrupts.
+		 * Find the first instance of this irq that has a
+		 * NULL idesc.
 		 */
-		result = register_intr(irq,		   /* isa irq	   */
-				       0,		   /* deviced??    */
-				       0,		   /* flags?       */
-				       (inthand2_t*) func, /* handler      */
-				       maskptr,		   /* mask pointer */
-				       (int)arg);	   /* handler arg  */
+		struct irq_node *cur_irq;
 
-		if (result) {
-			printf ("\neisa_reg_int: result=%d\n", result);
-			splx(s);
-			return (result);
-		};
-		update_intr_masks();
-		splx(s);
+		cur_irq = TAILQ_FIRST(&e_dev->ioconf.irqs);
+		while (cur_irq != NULL) {
+			if (cur_irq->irq_no == irq
+			 && cur_irq->idesc == NULL) {
+				int s;
+				int result;
+
+				s = splhigh();
+				result = register_intr(irq, 0, 0,
+						       (inthand2_t*)func,
+						       maskptr,
+						       (int)arg);
+
+				if (result) {
+					printf("\neisa_reg_int: result=%d\n",
+					       result);
+					splx(s);
+					return (result);
+				}
+				update_intr_masks();
+				cur_irq->idesc = (void *)1;
+				splx(s);
+				break;
+			}
+			cur_irq = TAILQ_NEXT(cur_irq, links);
+		}
+		if (cur_irq == NULL || cur_irq->idesc == NULL)
+			return (-1);
+	} else {
+  		return EPERM;
 	}
-	else
-		return EPERM;
 
-	e_dev->ioconf.irq |= 1ul << irq;
 	sprintf(string, " irq %d", irq);
 	eisa_reg_print(e_dev, string, reg_state.num_interrupts ? 
 				      &separator : NULL);
@@ -469,27 +454,40 @@ eisa_release_intr(e_dev, irq, func)
 	int   irq;
 	void  (*func)(void *);
 {
-	int result;
-	int s;
-       	
-	if (!(e_dev->ioconf.irq & (1ul << irq))) {
+	int	result;
+	struct	irq_node *cur_irq;
+
+	result = -1;
+	cur_irq = TAILQ_FIRST(&e_dev->ioconf.irqs);
+	while (cur_irq != NULL) {
+		if (cur_irq->irq_no == irq) {
+			struct	irq_node *next_irq;
+
+			next_irq = TAILQ_NEXT(cur_irq, links);
+			if (cur_irq->idesc != NULL) {
+				int s;
+
+				s = splhigh();
+				INTRDIS ((1ul<<irq));
+				unregister_intr(irq, (inthand2_t*)func);
+				update_intr_masks();
+				cur_irq->idesc = NULL;
+				splx(s);
+			}
+			TAILQ_REMOVE(&e_dev->ioconf.irqs, cur_irq, links);
+			free(cur_irq, M_DEVBUF);
+			cur_irq = next_irq;
+			result = 0;
+		} else {
+			cur_irq = TAILQ_NEXT(cur_irq, links);
+		}
+	}
+	if (result != 0) {
 		printf("%s%ld: Attempted to release an interrupt (%d) "
 		       "it doesn't own\n", e_dev->driver->name,
 			e_dev->unit, irq);
-		return (-1);
 	}
 
-	s = splhigh();
-	INTRDIS ((1ul<<irq));
-
-	result = unregister_intr (irq, (inthand2_t*)func);
-        
-	if (result)
-		printf ("eisa_release_intr: result=%d\n", result);
-  
-	update_intr_masks();
-
-	splx(s);
 	return (result);
 }
 
@@ -498,18 +496,24 @@ eisa_enable_intr(e_dev, irq)
 	struct eisa_device *e_dev;
 	int irq;
 {
-	int s;
+	struct	irq_node *cur_irq;
+	int	result;
+  
+	result = -1;
+	cur_irq = TAILQ_FIRST(&e_dev->ioconf.irqs);
+       	while (cur_irq != NULL) {
+		if (cur_irq->irq_no == irq
+		 && cur_irq->idesc != NULL) {
+			int s;
 
-	if (!(e_dev->ioconf.irq & (1ul << irq))) {
-		printf("%s%ld: Attempted to enable an interrupt (%d) "
-		       "it doesn't own\n", e_dev->driver->name,
-			e_dev->unit, irq);
-		return (-1);
+			s = splhigh();
+			INTREN((1ul << irq));
+			result = 0;
+			splx(s);
+		}
+		cur_irq = TAILQ_NEXT(cur_irq, links);
 	}
-	s = splhigh();
-	INTREN((1ul << irq));
-	splx(s);
-	return 0;
+	return (result);
 }
 
 static int
@@ -622,7 +626,6 @@ eisa_reg_resvaddr(e_dev, head, resvaddr, reg_count)
 				eisa_reg_print(e_dev, buf,
 						*reg_count ? &separator : NULL);
 				(*reg_count)++;
-				e_dev->kdc->kdc_datalen += sizeof(resvaddr_t);
 				return (0);
 			}
 		}
@@ -670,104 +673,11 @@ eisa_reg_iospace(e_dev, resvaddr)
 }
 
 int
-eisa_registerdev(e_dev, driver, kdc_template)
+eisa_registerdev(e_dev, driver)
 	struct eisa_device *e_dev;
 	struct eisa_driver *driver;
-	struct kern_devconf *kdc_template;
 {
-	resvaddr_t *node;
-
 	e_dev->driver = driver;	/* Driver now owns this device */
-	e_dev->kdc = (struct kern_devconf *)malloc(sizeof(struct kern_devconf),
-						   M_DEVBUF, M_NOWAIT);
-	if (!e_dev->kdc) {
-		printf("WARNING: eisa_registerdev unable to malloc! "
-		       "Device kdc will not be registerd\n");
-		return 1;
-	}
-	bcopy(kdc_template, e_dev->kdc, sizeof(*kdc_template));
-	e_dev->kdc->kdc_description = e_dev->full_name;
-	e_dev->kdc->kdc_parentdata = e_dev;
-	dev_attach(e_dev->kdc);
 	return (0);
 }
 
-int
-eisa_generic_externalize(p, kdc, userp, l)
-        struct proc *p;
-        struct kern_devconf *kdc;
-        void *userp;
-        size_t l;
-{
-	struct eisa_device *e_dev;
-	resvaddr_t *node;
-	void *buf;	/* Temporary externalizing buffer */
-	void *bufp;	/* Current offset in the buffer */
-	void *offset;	/* Offset relative to target address space */
-	void *ioa_prev; /* Prev Node entries relative to target address space */
-	void *ma_prev;	/* Prev Node entries relative to target address space */
-	int retval;
-
-	if(l < kdc->kdc_datalen)
-		return ENOMEM;
-
-	offset = userp;
-	buf = malloc(kdc->kdc_datalen, M_TEMP, M_NOWAIT);
-	if (!buf)
-		return 0;
-
-	bufp = buf;
-	bcopy(kdc->kdc_eisa, bufp, sizeof(struct eisa_device));
-	e_dev = bufp;
-
-	/* Calculate initial prev nodes */
-	ioa_prev = offset + ((void *)&(e_dev->ioconf.ioaddrs.lh_first)
-			  - (void *)e_dev);
-	ma_prev = offset + ((void *)&(e_dev->ioconf.maddrs.lh_first)
-			 - (void *)e_dev);
-
-	offset += sizeof(*e_dev);
-	bufp += sizeof(*e_dev);
-
-	if (e_dev->ioconf.ioaddrs.lh_first) {
-		node = e_dev->ioconf.ioaddrs.lh_first;
-		e_dev->ioconf.ioaddrs.lh_first = offset;
-		for(;node;node = node->links.le_next) {
-			resvaddr_t *out_node;
-
-			bcopy(node, bufp, sizeof(resvaddr_t));
-			out_node = (resvaddr_t *)bufp;
-			bufp += sizeof(resvaddr_t);
-			offset += sizeof(resvaddr_t);
-
-			out_node->links.le_prev = ioa_prev;
-			ioa_prev += sizeof(resvaddr_t);
-
-			if (out_node->links.le_next)
-				out_node->links.le_next = offset;
-		}
-	}
-	if (e_dev->ioconf.maddrs.lh_first) {
-		node = e_dev->ioconf.maddrs.lh_first;
-		e_dev->ioconf.maddrs.lh_first = offset;
-		for(;node;node = node->links.le_next) {
-			resvaddr_t *out_node;
-
-			bcopy(node, bufp, sizeof(resvaddr_t));
-			out_node = (resvaddr_t *)bufp;
-			bufp += sizeof(resvaddr_t);
-			offset += sizeof(resvaddr_t);
-
-			out_node->links.le_prev = ma_prev;
-			ma_prev += sizeof(resvaddr_t);
-
-			if (out_node->links.le_next)
-				out_node->links.le_next = offset;
-		}
-	}
-	
-	l -= kdc->kdc_datalen;
-	retval =  copyout(buf, userp, kdc->kdc_datalen);
-	free(buf, M_TEMP);
-	return retval;
-}
