@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)autoconf.c	7.1 (Berkeley) 5/9/91
- * $FreeBSD: stable/2.1/sys/i386/i386/autoconf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/autoconf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -45,65 +45,93 @@
  * devices are determined (from possibilities mentioned in ioconf.c),
  * and the drivers are initialized.
  */
+#include "opt_cd9660.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/buf.h>
-#include <sys/dkstat.h>
 #include <sys/conf.h>
+#include <sys/disklabel.h>
+#include <sys/diskslice.h> /* for BASE_SLICE, MAX_SLICES */
 #include <sys/dmap.h>
 #include <sys/reboot.h>
 #include <sys/kernel.h>
+#include <sys/mount.h>
+#include <sys/sysctl.h>
 
-#include <machine/cpu.h>
+#include <machine/bootinfo.h>
+#include <machine/cons.h>
 #include <machine/md_var.h>
-#include <machine/pte.h>
 #include <i386/isa/icu.h> /* For interrupts */
 
-static void setroot(void);
-
-/*
- * The following several variables are related to
- * the configuration process, and are used in initializing
- * the machine.
- */
-int	dkn;		/* number of iostat dk numbers assigned so far */
-
-extern int (*mountroot) __P((void));
-#ifdef FFS
-int ffs_mountroot __P((void));
-#endif
-#ifdef NFS
-int nfs_mountroot __P((void));
-#endif
-#ifdef CD9660
-int cd9660_mountroot __P((void));
-#endif
-#ifdef MSDOSFS
-int msdosfs_mountroot __P((void));
-#endif
-#ifdef MFS_ROOT
-int mfs_initminiroot __P((u_char *));
-u_char mfs_root[MFS_ROOT*1024] = "MFS Filesystem goes here";
-u_char end_mfs_root[] = "MFS Filesystem had better STOP here";
+#include "isa.h"
+#if NISA > 0
+#include <i386/isa/isa_device.h>
 #endif
 
+#include "pnp.h"
+#if NPNP > 0
+#include <i386/isa/pnp.h>
+#endif
+ 
 #include "eisa.h"
 #if NEISA > 0
-      #include <i386/eisa/eisaconf.h>
+#include <i386/eisa/eisaconf.h>
 #endif
 
 #include "pci.h"
 #if NPCI > 0
-      #include <pci/pcivar.h>
+#include <pci/pcivar.h>
 #endif
 
-#include "isa.h"
-#if NISA > 0
-      #include <i386/isa/isa_device.h>
+#include "card.h"
+#if NCARD > 0
+#include <pccard/driver.h>
 #endif
 
+#include "scbus.h"
+#if NSCBUS > 0
+#include <scsi/scsiconf.h>
+#endif
+
+static void	configure __P((void *));
+SYSINIT(configure, SI_SUB_CONFIGURE, SI_ORDER_FIRST, configure, NULL)
+
+#ifdef MFS_ROOT
+extern struct vfsops	mfs_vfsops;
+#endif
+#ifdef FFS
+extern struct vfsops	ufs_vfsops;
+#endif
+#ifdef LFS
+extern struct vfsops	lfs_vfsops;
+#endif
+#ifdef NFS
+extern int	nfs_mountroot __P((void *));
+#endif
+#ifdef MSDOSFS
+extern int	msdosfs_mountroot __P((void *));
+#endif
+
+static void	configure_finish __P((void));
+static void	configure_start __P((void));
+static int	setdumpdev __P((dev_t dev));
+static void	setroot __P((void));
+
 #ifdef CD9660
-/* We need to try out all our potential CDROM drives, so we need a table. */
+
+#include <sys/fcntl.h>
+#include <sys/proc.h>
+#include <sys/stat.h>
+#include <sys/vnode.h>
+#include <machine/clock.h>
+#include <isofs/cd9660/iso.h>
+
+/*
+ * XXX All this CD-ROM root stuff is fairly messy.  Ick.
+ *
+ * We need to try out all our potential CDROM drives, so we need a table.
+ */
 static struct {
 	char *name;
 	int major;
@@ -112,32 +140,49 @@ static struct {
 	{ "mcd", 7 },
 	{ "scd", 16 },
 	{ "matcd", 17 },
+	{ "wcd", 19 },
 	{ 0, 0}
 };
 
-int
+static int	find_cdrom_root __P((void));
+
+static int
 find_cdrom_root()
 {
-	int i,j,k;
+	int i, j, error;
+	struct bdevsw *bd;
+	dev_t orootdev;
 
-	for (j = 0 ; j < 2; j++)
-		for (k = 0 ; try_cdrom[k].name ; k++) {
-			rootdev = makedev(try_cdrom[k].major,j*8);
-			printf("trying rootdev=0x%lx (%s%d)\n",
-				rootdev, try_cdrom[k].name,j);
-			i = (*cd9660_mountroot)();
-			if (!i) return i;
+#if CD9660_ROOTDELAY > 0
+	DELAY(CD9660_ROOTDELAY * 1000000);
+#endif
+	orootdev = rootdev;
+	for (i = 0 ; i < 2; i++)
+		for (j = 0 ; try_cdrom[j].name ; j++) {
+			if (try_cdrom[j].major >= nblkdev)
+				continue;
+			rootdev = makedev(try_cdrom[j].major, i * 8);
+			bd = bdevsw[major(rootdev)];
+			if (bd == NULL || bd->d_open == NULL)
+				continue;
+			if (bootverbose)
+				printf("trying %s%d as rootdev (0x%x)\n",
+				       try_cdrom[j].name, i, rootdev);
+			error = (bd->d_open)(rootdev, FREAD, S_IFBLK, curproc);
+			if (error == 0) {
+				if (bd->d_close != NULL)
+					(bd->d_close)(rootdev, FREAD, S_IFBLK,
+						      curproc);
+				return 0;
+			}
 		}
+
+	rootdev = orootdev;
 	return EINVAL;
 }
 #endif /* CD9660 */
 
-#include "scbus.h"
-#if NSCBUS > 0
-      #include <scsi/scsiconf.h>
-#endif
-
-void
+static void
 configure_start()
 {
 #if NSCBUS > 0
@@ -145,7 +190,7 @@ configure_start()
 #endif
 }
 
-void
+static void
 configure_finish()
 {
 #if NSCBUS > 0
@@ -156,15 +201,29 @@ configure_finish()
 /*
  * Determine i/o configuration for a machine.
  */
-void
-configure()
+static void
+configure(dummy)
+	void *dummy;
 {
+	int i;
 
 	configure_start();
-
 	/* Allow all routines to decide for themselves if they want intrs */
-	enable_intr();
-	INTREN(IRQ_SLAVE);
+	/*
+	 * XXX Since this cannot be achieved on all architectures, we should
+	 * XXX go back to disabling all interrupts until configuration is
+	 * XXX completed and switch any devices that rely on the current
+	 * XXX behavior to no longer rely on interrupts or to register an
+	 * XXX interrupt_driven_config_hook for the task.
+	 */
+	/*
+	 * XXX The above is wrong, because we're implicitly at splhigh(),
+	 * XXX and should stay there, so enabling interrupts in the CPU
+	 * XXX and the ICU at most gives pending interrupts which just get
+	 * XXX in the way.
+	 */
+        enable_intr();
+        INTREN(IRQ_SLAVE);
 
 #if NEISA > 0
 	eisa_configure();
@@ -174,24 +233,94 @@ configure()
 	pci_configure();
 #endif
 
+#if NPNP > 0
+	pnp_configure();
+#endif
+
 #if NISA > 0
 	isa_configure();
 #endif
 
+#if NCARD > 0
+	/* After everyone else has a chance at grabbing resources */
+	pccard_configure();
+#endif
+
 	configure_finish();
 
-#ifdef MFS_ROOT
-	mfs_initminiroot(mfs_root);
-#endif /* MFS_ROOT */
+	cninit_finish();
+
+	if (bootverbose) {
+		/*
+		 * Print out the BIOS's idea of the disk geometries.
+		 */
+		printf("BIOS Geometries:\n");
+		for (i = 0; i < N_BIOS_GEOM; i++) {
+			unsigned long bios_geom;
+			int max_cylinder, max_head, max_sector;
+
+			bios_geom = bootinfo.bi_bios_geom[i];
+
+			/*
+			 * XXX the bootstrap punts a 1200K floppy geometry
+			 * when the get-disk-geometry interrupt fails.  Skip
+			 * drives that have this geometry.
+			 */
+			if (bios_geom == 0x4f010f)
+				continue;
+
+			printf(" %x:%08lx ", i, bios_geom);
+			max_cylinder = bios_geom >> 16;
+			max_head = (bios_geom >> 8) & 0xff;
+			max_sector = bios_geom & 0xff;
+			printf(
+		"0..%d=%d cylinders, 0..%d=%d heads, 1..%d=%d sectors\n",
+			       max_cylinder, max_cylinder + 1,
+			       max_head, max_head + 1,
+			       max_sector, max_sector);
+		}
+		printf(" %d accounted for\n", bootinfo.bi_n_bios_used);
 
-	if (bootverbose)
 		printf("Device configuration finished.\n");
+	}
+	cold = 0;
+}
 
+void
+cpu_rootconf()
+{
+	/*
+	 * XXX NetBSD has a much cleaner approach to finding root.
+	 * XXX We should adopt their code.
+	 */
 #ifdef CD9660
-	if ((boothowto & RB_CDROM) && !mountroot) {
+	if ((boothowto & RB_CDROM) != 0) {
 		if (bootverbose)
 			printf("Considering CD-ROM root f/s.\n");
-		mountroot = find_cdrom_root;
+		/* NB: find_cdrom_root() sets rootdev if successful. */
+		if (find_cdrom_root() == 0)
+			mountroot = cd9660_mountroot;	/* XXX goes away*/
+		else if (bootverbose)
+			printf("No CD-ROM available as root f/s.\n");
+	}
+#endif
+
+#ifdef MFS_ROOT
+	if (!mountroot) {
+		if (bootverbose)
+			printf("Considering MFS root f/s.\n");
+		mountroot = vfs_mountroot;	/* XXX goes away*/
+		mountrootvfsops = &mfs_vfsops;
+		/*
+		 * Ignore the -a flag if this kernel isn't compiled
+		 * with a generic root/swap configuration: if we skip
+		 * setroot() and we aren't a generic kernel, chaos
+		 * will ensue because setconf() will be a no-op.
+		 * (rootdev is always initialized to NODEV in a
+		 * generic configuration, so we test for that.)
+		 */
+		if ((boothowto & RB_ASKNAME) == 0 || rootdev != NODEV)
+			setroot();
 	}
 #endif
 
@@ -215,7 +344,8 @@ configure()
 	if (!mountroot) {
 		if (bootverbose)
 			printf("Considering FFS root f/s.\n");
-		mountroot = ffs_mountroot;
+		mountroot = vfs_mountroot;	/* XXX goes away*/
+		mountrootvfsops = &ufs_vfsops;
 		/*
 		 * Ignore the -a flag if this kernel isn't compiled
 		 * with a generic root/swap configuration: if we skip
@@ -228,22 +358,39 @@ configure()
 			setroot();
 	}
 #endif
+#ifdef LFS
+	if (!mountroot) {
+		if (bootverbose)
+			printf("Considering LFS root f/s.\n");
+		mountroot = vfs_mountroot;	/* XXX goes away*/
+		mountrootvfsops = &lfs_vfsops;
+		/*
+		 * Ignore the -a flag if this kernel isn't compiled
+		 * with a generic root/swap configuration: if we skip
+		 * setroot() and we aren't a generic kernel, chaos
+		 * will ensue because setconf() will be a no-op.
+		 * (rootdev is always initialized to NODEV in a
+		 * generic configuration, so we test for that.)
+		 */
+		if ((boothowto & RB_ASKNAME) == 0 || rootdev != NODEV)
+			setroot();
+	}
+#endif
+
 	if (!mountroot) {
 		panic("Nobody wants to mount my root for me");
 	}
-	/*
-	 * Configure swap area and related system
-	 * parameter based on device(s) used.
-	 */
-	if (bootverbose)
-		printf("Configuring root and swap devs.\n");
 	setconf();
-	cold = 0;
-	if (bootverbose)
-		printf("configure() finished.\n");
 }
 
-int
+void
+cpu_dumpconf()
+{
+	if (setdumpdev(dumpdev) != 0)
+		dumpdev = NODEV;
+}
+
+static int
 setdumpdev(dev)
 	dev_t dev;
 {
@@ -252,18 +399,26 @@ setdumpdev(dev)
 
 	if (dev == NODEV) {
 		dumpdev = dev;
-		dumplo = 0;
 		return (0);
 	}
 	maj = major(dev);
 	if (maj >= nblkdev)
 		return (ENXIO);
-	if (bdevsw[maj].d_psize == NULL)
-		return (ENXIO);		/* XXX should sometimes be ENODEV */
-	psize = bdevsw[maj].d_psize(dev);
+	if (bdevsw[maj] == NULL)
+		return (ENXIO);		/* XXX is this right? */
+	if (bdevsw[maj]->d_psize == NULL)
+		return (ENXIO);		/* XXX should be ENODEV ? */
+	psize = bdevsw[maj]->d_psize(dev);
 	if (psize == -1)
-		return (ENXIO);		/* XXX should sometimes be ENODEV */
-	newdumplo = psize - Maxmem * NBPG / DEV_BSIZE;
+		return (ENXIO);		/* XXX should be ENODEV ? */
+	/*
+	 * XXX should clean up checking in dumpsys() to be more like this,
+	 * and nuke dodump sysctl (too many knobs), and move this to
+	 * kern_shutdown.c...
+	 */
+	if (dkpart(dev) != SWAP_PART)
+		return (ENODEV);
+	newdumplo = psize - Maxmem * PAGE_SIZE / DEV_BSIZE;
 	if (newdumplo < 0)
 		return (ENOSPC);
 	dumpdev = dev;
@@ -273,13 +428,14 @@ setdumpdev(dev)
 
 u_long	bootdev = 0;		/* not a dev_t - encoding is different */
 
-static	char devname[][2] = {
-      {'w','d'},      /* 0 = wd */
-      {'s','w'},      /* 1 = sw */
+/* Name lookup for bootable majors */
+static char *devname[] = {
+	"wd",
+	"wfd",
 #define FDMAJOR 2
-      {'f','d'},      /* 2 = fd */
-      {'w','t'},      /* 3 = wt */
-      {'s','d'},      /* 4 = sd -- new SCSI system */
+	"fd",
+	"wt",
+	"sd",
 };
 
 #define	PARTITIONMASK	0x7
@@ -295,37 +451,55 @@ static	char devname[][2] = {
 static void
 setroot()
 {
-	int  majdev, mindev, unit, part, adaptor;
-	dev_t temp = 0, orootdev;
-	struct swdevt *swp;
+	int  majdev, mindev, unit, part, adaptor, slice;
+	dev_t orootdev;
+	char *sname, partname[2];
 
 /*printf("howto %x bootdev %x ", boothowto, bootdev);*/
 	if (boothowto & RB_DFLTROOT ||
 	    (bootdev & B_MAGICMASK) != (u_long)B_DEVMAGIC)
 		return;
-	majdev = (bootdev >> B_TYPESHIFT) & B_TYPEMASK;
+	majdev  = B_TYPE(bootdev);
+	adaptor = B_ADAPTOR(bootdev);
+	unit    = B_UNIT(bootdev);
+	slice   = B_SLICE(bootdev);
+	if ((slice < BASE_SLICE) || (slice >= MAX_SLICES))
+		slice = COMPATIBILITY_SLICE;
 	if (majdev > sizeof(devname) / sizeof(devname[0]))
 		return;
-	adaptor = (bootdev >> B_ADAPTORSHIFT) & B_ADAPTORMASK;
-	unit = (bootdev >> B_UNITSHIFT) & B_UNITMASK;
 	if (majdev == FDMAJOR) {
 		part = RAW_PART;
 		mindev = unit << FDUNITSHIFT;
 	}
 	else {
 		part = (bootdev >> B_PARTITIONSHIFT) & B_PARTITIONMASK;
-		mindev = (unit << PARTITIONSHIFT) + part;
+		mindev = dkmakeminor(unit, slice, part);
 	}
 	orootdev = rootdev;
 	rootdev = makedev(majdev, mindev);
 	/*
 	 * If the original rootdev is the same as the one
-	 * just calculated, don't need to adjust the swap configuration.
+	 * just calculated modulo the slice number, don't print an otherwise
+	 * confusing diagnostic.
 	 */
-	if (rootdev == orootdev)
+	if ((rootdev & ~0xff0000) == (orootdev & ~0xff0000))
 		return;
-	printf("changing root device to %c%c%d%c\n",
-		devname[majdev][0], devname[majdev][1],
-		mindev >> (majdev == FDMAJOR ? FDUNITSHIFT : PARTITIONSHIFT),
-		part + 'a');
+	sname = dsname(devname[majdev], unit, slice, part, partname);
+	printf("changing root device to %s%s\n", sname, partname);
 }
+
+static int
+sysctl_kern_dumpdev SYSCTL_HANDLER_ARGS
+{
+	int error;
+	dev_t ndumpdev;
+
+	ndumpdev = dumpdev;
+	error = sysctl_handle_opaque(oidp, &ndumpdev, sizeof ndumpdev, req);
+	if (error == 0 && req->newptr != NULL)
+		error = setdumpdev(ndumpdev);
+	return (error);
+}
+
+SYSCTL_PROC(_kern, KERN_DUMPDEV, dumpdev, CTLTYPE_OPAQUE|CTLFLAG_RW,
+	0, sizeof dumpdev, sysctl_kern_dumpdev, "T,dev_t", "");
