@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)cons.h	7.2 (Berkeley) 5/9/91
- * $FreeBSD: stable/2.1/sys/i386/i386/cons.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/cons.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _MACHINE_CONS_H_
@@ -60,6 +60,12 @@ cn_getc_t	pccngetc;
 cn_checkc_t	pccncheckc;
 cn_putc_t	pccnputc;
 
+cn_probe_t	sccnprobe;
+cn_init_t	sccninit;
+cn_getc_t	sccngetc;
+cn_checkc_t	sccncheckc;
+cn_putc_t	sccnputc;
+
 cn_probe_t	siocnprobe;
 cn_init_t	siocninit;
 cn_getc_t	siocngetc;
@@ -95,21 +101,13 @@ struct consdev {
 #ifdef KERNEL
 extern	struct consdev constab[];
 extern	struct consdev *cn_tab;
-extern	struct tty *cn_tty;
 extern	int cons_unavail;
 
 struct proc; struct uio;
 
-/* cdevsw[] entries */
-extern int cnopen(dev_t, int, int, struct proc *);
-extern int cnclose(dev_t, int, int, struct proc *);
-extern int cnread(dev_t, struct uio *, int);
-extern int cnwrite(dev_t, struct uio *, int);
-extern int cnioctl(dev_t, int, caddr_t, int, struct proc *);
-extern int cnselect(dev_t, int, struct proc *);
-
 /* other kernel entry points */
 extern void cninit(void);
+extern void cninit_finish(void);
 extern int cngetc(void);
 extern int cncheckc(void);
 extern void cnputc(int);
