@@ -23,7 +23,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $FreeBSD: stable/2.1/sys/i386/i386/db_disasm.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/db_disasm.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -32,6 +32,8 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
+#include <vm/vm.h>
+#include <vm/vm_param.h>
 #include <ddb/ddb.h>
 
 #include <ddb/db_access.h>
@@ -76,7 +78,6 @@
 #define	Ib	21			/* byte immediate, unsigned */
 #define	Ibs	22			/* byte immediate, signed */
 #define	Iw	23			/* word immediate, unsigned */
-#define	Il	24			/* long immediate */
 #define	O	25			/* direct address */
 #define	Db	26			/* byte displacement from EIP */
 #define	Dl	27			/* long displacement from EIP */
@@ -87,13 +88,16 @@
 #define	STI	32			/* FP stack */
 #define	X	33			/* extended FP op */
 #define	XA	34			/* for 'fstcw %ax' */
+#define	El	35			/* address, long size */
+#define	Ril	36			/* long register in instruction */
+#define	Iba	37			/* byte immediate, don't print if 0xa */
 
 struct inst {
-	char *	i_name;			/* name */
+	const char *	i_name;		/* name */
 	short	i_has_modrm;		/* has regmodrm byte */
 	short	i_size;			/* operand size */
 	int	i_mode;			/* addressing modes */
-	char *	i_extra;		/* pointer to extra opcode table */
+	const void *	i_extra;	/* pointer to extra opcode table */
 };
 
 #define	op1(x)		(x)
@@ -101,14 +105,14 @@ struct inst {
 #define	op3(x,y,z)	((x)|((y)<<8)|((z)<<16))
 
 struct finst {
-	char *	f_name;			/* name for memory instruction */
+	const char *	f_name;		/* name for memory instruction */
 	int	f_size;			/* size for memory instruction */
 	int	f_rrmode;		/* mode for rr instruction */
-	char *	f_rrname;		/* name for rr instruction
+	const void *	f_rrname;	/* name for rr instruction
 					   (or pointer to table) */
 };
 
-char *	db_Grp6[] = {
+static const char * const db_Grp6[] = {
 	"sldt",
 	"str",
 	"lldt",
@@ -119,7 +123,7 @@ char *	db_Grp6[] = {
 	""
 };
 
-char *	db_Grp7[] = {
+static const char * const db_Grp7[] = {
 	"sgdt",
 	"sidt",
 	"lgdt",
@@ -130,7 +134,7 @@ char *	db_Grp7[] = {
 	"invlpg"
 };
 
-char *	db_Grp8[] = {
+static const char * const db_Grp8[] = {
 	"",
 	"",
 	"",
@@ -141,9 +145,20 @@ char *	db_Grp8[] = {
 	"btc"
 };
 
-struct inst db_inst_0f0x[] = {
-/*00*/	{ "",	   TRUE,  NONE,  op1(Ew),     (char *)db_Grp6 },
-/*01*/	{ "",	   TRUE,  NONE,  op1(Ew),     (char *)db_Grp7 },
+static const char * const db_Grp9[] = {
+	"",
+	"cmpxchg8b",
+	"",
+	"",
+	"",
+	"",
+	"",
+	""
+};
+
+static const struct inst db_inst_0f0x[] = {
+/*00*/	{ "",	   TRUE,  NONE,  op1(Ew),     db_Grp6 },
+/*01*/	{ "",	   TRUE,  NONE,  op1(Ew),     db_Grp7 },
 /*02*/	{ "lar",   TRUE,  LONG,  op2(E,R),    0 },
 /*03*/	{ "lsl",   TRUE,  LONG,  op2(E,R),    0 },
 /*04*/	{ "",      FALSE, NONE,  0,	      0 },
@@ -161,14 +176,14 @@ struct inst db_inst_0f0x[] = {
 /*0f*/	{ "",      FALSE, NONE,  0,	      0 },
 };
 
-struct inst	db_inst_0f2x[] = {
-/*20*/	{ "mov",   TRUE,  LONG,  op2(CR,E),   0 }, /* use E for reg */
-/*21*/	{ "mov",   TRUE,  LONG,  op2(DR,E),   0 }, /* since mod == 11 */
-/*22*/	{ "mov",   TRUE,  LONG,  op2(E,CR),   0 },
-/*23*/	{ "mov",   TRUE,  LONG,  op2(E,DR),   0 },
-/*24*/	{ "mov",   TRUE,  LONG,  op2(TR,E),   0 },
+static const struct inst db_inst_0f2x[] = {
+/*20*/	{ "mov",   TRUE,  LONG,  op2(CR,El),  0 },
+/*21*/	{ "mov",   TRUE,  LONG,  op2(DR,El),  0 },
+/*22*/	{ "mov",   TRUE,  LONG,  op2(El,CR),  0 },
+/*23*/	{ "mov",   TRUE,  LONG,  op2(El,DR),  0 },
+/*24*/	{ "mov",   TRUE,  LONG,  op2(TR,El),  0 },
 /*25*/	{ "",      FALSE, NONE,  0,	      0 },
-/*26*/	{ "mov",   TRUE,  LONG,  op2(E,TR),   0 },
+/*26*/	{ "mov",   TRUE,  LONG,  op2(El,TR),  0 },
 /*27*/	{ "",      FALSE, NONE,  0,	      0 },
 
 /*28*/	{ "",      FALSE, NONE,  0,	      0 },
@@ -181,7 +196,27 @@ struct inst	db_inst_0f2x[] = {
 /*2f*/	{ "",      FALSE, NONE,  0,	      0 },
 };
 
-struct inst	db_inst_0f8x[] = {
+static const struct inst db_inst_0f3x[] = {
+/*30*/	{ "wrmsr", FALSE, NONE,  0,	      0 },
+/*31*/	{ "rdtsc", FALSE, NONE,  0,	      0 },
+/*32*/	{ "rdmsr", FALSE, NONE,  0,	      0 },
+/*33*/	{ "rdpmc", FALSE, NONE,  0,	      0 },
+/*34*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*35*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*36*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*37*/	{ "",	   FALSE, NONE,  0,	      0 },
+
+/*38*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*39*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3a*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3b*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3c*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3d*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3e*/	{ "",	   FALSE, NONE,  0,	      0 },
+/*3f*/	{ "",	   FALSE, NONE,  0,	      0 },
+};
+
+static const struct inst db_inst_0f8x[] = {
 /*80*/	{ "jo",    FALSE, NONE,  op1(Dl),     0 },
 /*81*/	{ "jno",   FALSE, NONE,  op1(Dl),     0 },
 /*82*/	{ "jb",    FALSE, NONE,  op1(Dl),     0 },
@@ -201,7 +236,7 @@ struct inst	db_inst_0f8x[] = {
 /*8f*/	{ "jnle",  FALSE, NONE,  op1(Dl),     0 },
 };
 
-struct inst	db_inst_0f9x[] = {
+static const struct inst db_inst_0f9x[] = {
 /*90*/	{ "seto",  TRUE,  NONE,  op1(Eb),     0 },
 /*91*/	{ "setno", TRUE,  NONE,  op1(Eb),     0 },
 /*92*/	{ "setb",  TRUE,  NONE,  op1(Eb),     0 },
@@ -221,47 +256,47 @@ struct inst	db_inst_0f9x[] = {
 /*9f*/	{ "setnle",TRUE,  NONE,  op1(Eb),     0 },
 };
 
-struct inst	db_inst_0fax[] = {
+static const struct inst db_inst_0fax[] = {
 /*a0*/	{ "push",  FALSE, NONE,  op1(Si),     0 },
 /*a1*/	{ "pop",   FALSE, NONE,  op1(Si),     0 },
-/*a2*/	{ "",      FALSE, NONE,  0,	      0 },
-/*a3*/	{ "bt",    TRUE,  LONG,  op2(E,R),    0 },
-/*a4*/	{ "shld",  TRUE,  LONG,  op3(Ib,E,R), 0 },
-/*a5*/	{ "shld",  TRUE,  LONG,  op3(CL,E,R), 0 },
+/*a2*/	{ "cpuid", FALSE, NONE,  0,	      0 },
+/*a3*/	{ "bt",    TRUE,  LONG,  op2(R,E),    0 },
+/*a4*/	{ "shld",  TRUE,  LONG,  op3(Ib,R,E), 0 },
+/*a5*/	{ "shld",  TRUE,  LONG,  op3(CL,R,E), 0 },
 /*a6*/	{ "",      FALSE, NONE,  0,	      0 },
 /*a7*/	{ "",      FALSE, NONE,  0,	      0 },
 
 /*a8*/	{ "push",  FALSE, NONE,  op1(Si),     0 },
 /*a9*/	{ "pop",   FALSE, NONE,  op1(Si),     0 },
-/*aa*/	{ "",      FALSE, NONE,  0,	      0 },
-/*ab*/	{ "bts",   TRUE,  LONG,  op2(E,R),    0 },
-/*ac*/	{ "shrd",  TRUE,  LONG,  op3(Ib,E,R), 0 },
-/*ad*/	{ "shrd",  TRUE,  LONG,  op3(CL,E,R), 0 },
+/*aa*/	{ "rsm",   FALSE, NONE,  0,	      0 },
+/*ab*/	{ "bts",   TRUE,  LONG,  op2(R,E),    0 },
+/*ac*/	{ "shrd",  TRUE,  LONG,  op3(Ib,R,E), 0 },
+/*ad*/	{ "shrd",  TRUE,  LONG,  op3(CL,R,E), 0 },
 /*a6*/	{ "",      FALSE, NONE,  0,	      0 },
 /*a7*/	{ "imul",  TRUE,  LONG,  op2(E,R),    0 },
 };
 
-struct inst	db_inst_0fbx[] = {
-/*b0*/	{ "",      FALSE, NONE,  0,	      0 },
-/*b1*/	{ "",      FALSE, NONE,  0,	      0 },
+static const struct inst db_inst_0fbx[] = {
+/*b0*/	{ "cmpxchg",TRUE, BYTE,	 op2(R, E),   0 },
+/*b0*/	{ "cmpxchg",TRUE, LONG,	 op2(R, E),   0 },
 /*b2*/	{ "lss",   TRUE,  LONG,  op2(E, R),   0 },
-/*b3*/	{ "bts",   TRUE,  LONG,  op2(R, E),   0 },
+/*b3*/	{ "btr",   TRUE,  LONG,  op2(R, E),   0 },
 /*b4*/	{ "lfs",   TRUE,  LONG,  op2(E, R),   0 },
 /*b5*/	{ "lgs",   TRUE,  LONG,  op2(E, R),   0 },
-/*b6*/	{ "movzb", TRUE,  LONG,  op2(E, R),   0 },
-/*b7*/	{ "movzw", TRUE,  LONG,  op2(E, R),   0 },
+/*b6*/	{ "movzb", TRUE,  LONG,  op2(Eb, R),  0 },
+/*b7*/	{ "movzw", TRUE,  LONG,  op2(Ew, R),  0 },
 
 /*b8*/	{ "",      FALSE, NONE,  0,	      0 },
 /*b9*/	{ "",      FALSE, NONE,  0,	      0 },
-/*ba*/	{ "",      TRUE,  LONG,  op2(Ib, E),  (char *)db_Grp8 },
+/*ba*/	{ "",      TRUE,  LONG,  op2(Ib, E),  db_Grp8 },
 /*bb*/	{ "btc",   TRUE,  LONG,  op2(R, E),   0 },
 /*bc*/	{ "bsf",   TRUE,  LONG,  op2(E, R),   0 },
 /*bd*/	{ "bsr",   TRUE,  LONG,  op2(E, R),   0 },
-/*be*/	{ "movsb", TRUE,  LONG,  op2(E, R),   0 },
-/*bf*/	{ "movsw", TRUE,  LONG,  op2(E, R),   0 },
+/*be*/	{ "movsb", TRUE,  LONG,  op2(Eb, R),  0 },
+/*bf*/	{ "movsw", TRUE,  LONG,  op2(Ew, R),  0 },
 };
 
-struct inst	db_inst_0fcx[] = {
+static const struct inst db_inst_0fcx[] = {
 /*c0*/	{ "xadd",  TRUE,  BYTE,	 op2(R, E),   0 },
 /*c1*/	{ "xadd",  TRUE,  LONG,	 op2(R, E),   0 },
 /*c2*/	{ "",	   FALSE, NONE,	 0,	      0 },
@@ -269,41 +304,22 @@ struct inst	db_inst_0fcx[] = {
 /*c4*/	{ "",	   FALSE, NONE,	 0,	      0 },
 /*c5*/	{ "",	   FALSE, NONE,	 0,	      0 },
 /*c6*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c7*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c8*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*c9*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*ca*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*cb*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*cc*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*cd*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*ce*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
-/*cf*/	{ "bswap", FALSE, LONG,  op1(Ri),     0 },
+/*c7*/	{ "",	   TRUE,  NONE,  op1(E),      db_Grp9 },
+/*c8*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*c9*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*ca*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*cb*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*cc*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*cd*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*ce*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
+/*cf*/	{ "bswap", FALSE, LONG,  op1(Ril),    0 },
 };
 
-struct inst	db_inst_0fdx[] = {
-/*c0*/	{ "cmpxchg",TRUE, BYTE,	 op2(R, E),   0 },
-/*c1*/	{ "cmpxchg",TRUE, LONG,	 op2(R, E),   0 },
-/*c2*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c3*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c4*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c5*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c6*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c7*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c8*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*c9*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*ca*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*cb*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*cc*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*cd*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*ce*/	{ "",	   FALSE, NONE,	 0,	      0 },
-/*cf*/	{ "",	   FALSE, NONE,	 0,	      0 },
-};
-
-struct inst *db_inst_0f[] = {
+static const struct inst * const db_inst_0f[] = {
 	db_inst_0f0x,
 	0,
 	db_inst_0f2x,
-	0,
+	db_inst_0f3x,
 	0,
 	0,
 	0,
@@ -313,48 +329,45 @@ struct inst *db_inst_0f[] = {
 	db_inst_0fax,
 	db_inst_0fbx,
 	db_inst_0fcx,
-	db_inst_0fdx,
+	0,
 	0,
 	0
 };
 
-char *	db_Esc92[] = {
+static const char * const db_Esc92[] = {
 	"fnop",	"",	"",	"",	"",	"",	"",	""
 };
-char *	db_Esc93[] = {
-	"",	"",	"",	"",	"",	"",	"",	""
-};
-char *	db_Esc94[] = {
+static const char * const db_Esc94[] = {
 	"fchs",	"fabs",	"",	"",	"ftst",	"fxam",	"",	""
 };
-char *	db_Esc95[] = {
+static const char * const db_Esc95[] = {
 	"fld1",	"fldl2t","fldl2e","fldpi","fldlg2","fldln2","fldz",""
 };
-char *	db_Esc96[] = {
+static const char * const db_Esc96[] = {
 	"f2xm1","fyl2x","fptan","fpatan","fxtract","fprem1","fdecstp",
 	"fincstp"
 };
-char *	db_Esc97[] = {
+static const char * const db_Esc97[] = {
 	"fprem","fyl2xp1","fsqrt","fsincos","frndint","fscale","fsin","fcos"
 };
 
-char *	db_Esca4[] = {
+static const char * const db_Esca5[] = {
 	"",	"fucompp","",	"",	"",	"",	"",	""
 };
 
-char *	db_Escb4[] = {
-	"",	"",	"fnclex","fninit","",	"",	"",	""
+static const char * const db_Escb4[] = {
+	"fneni","fndisi",	"fnclex","fninit","fsetpm",	"",	"",	""
 };
 
-char *	db_Esce3[] = {
+static const char * const db_Esce3[] = {
 	"",	"fcompp","",	"",	"",	"",	"",	""
 };
 
-char *	db_Escf4[] = {
+static const char * const db_Escf4[] = {
 	"fnstsw","",	"",	"",	"",	"",	"",	""
 };
 
-struct finst db_Esc8[] = {
+static const struct finst db_Esc8[] = {
 /*0*/	{ "fadd",   SNGL,  op2(STI,ST),	0 },
 /*1*/	{ "fmul",   SNGL,  op2(STI,ST),	0 },
 /*2*/	{ "fcom",   SNGL,  op2(STI,ST),	0 },
@@ -365,51 +378,51 @@ struct finst db_Esc8[] = {
 /*7*/	{ "fdivr",  SNGL,  op2(STI,ST),	0 },
 };
 
-struct finst db_Esc9[] = {
+static const struct finst db_Esc9[] = {
 /*0*/	{ "fld",    SNGL,  op1(STI),	0 },
 /*1*/	{ "",       NONE,  op1(STI),	"fxch" },
-/*2*/	{ "fst",    SNGL,  op1(X),	(char *)db_Esc92 },
-/*3*/	{ "fstp",   SNGL,  op1(X),	(char *)db_Esc93 },
-/*4*/	{ "fldenv", NONE,  op1(X),	(char *)db_Esc94 },
-/*5*/	{ "fldcw",  NONE,  op1(X),	(char *)db_Esc95 },
-/*6*/	{ "fnstenv",NONE,  op1(X),	(char *)db_Esc96 },
-/*7*/	{ "fnstcw", NONE,  op1(X),	(char *)db_Esc97 },
+/*2*/	{ "fst",    SNGL,  op1(X),	db_Esc92 },
+/*3*/	{ "fstp",   SNGL,  0,		0 },
+/*4*/	{ "fldenv", NONE,  op1(X),	db_Esc94 },
+/*5*/	{ "fldcw",  NONE,  op1(X),	db_Esc95 },
+/*6*/	{ "fnstenv",NONE,  op1(X),	db_Esc96 },
+/*7*/	{ "fnstcw", NONE,  op1(X),	db_Esc97 },
 };
 
-struct finst db_Esca[] = {
-/*0*/	{ "fiadd",  WORD,  0,		0 },
-/*1*/	{ "fimul",  WORD,  0,		0 },
-/*2*/	{ "ficom",  WORD,  0,		0 },
-/*3*/	{ "ficomp", WORD,  0,		0 },
-/*4*/	{ "fisub",  WORD,  op1(X),	(char *)db_Esca4 },
-/*5*/	{ "fisubr", WORD,  0,		0 },
-/*6*/	{ "fidiv",  WORD,  0,		0 },
-/*7*/	{ "fidivr", WORD,  0,		0 }
+static const struct finst db_Esca[] = {
+/*0*/	{ "fiadd",  LONG,  0,		0 },
+/*1*/	{ "fimul",  LONG,  0,		0 },
+/*2*/	{ "ficom",  LONG,  0,		0 },
+/*3*/	{ "ficomp", LONG,  0,		0 },
+/*4*/	{ "fisub",  LONG,  0,		0 },
+/*5*/	{ "fisubr", LONG,  op1(X),	db_Esca5 },
+/*6*/	{ "fidiv",  LONG,  0,		0 },
+/*7*/	{ "fidivr", LONG,  0,		0 }
 };
 
-struct finst db_Escb[] = {
-/*0*/	{ "fild",   WORD,  0,		0 },
+static const struct finst db_Escb[] = {
+/*0*/	{ "fild",   LONG,  0,		0 },
 /*1*/	{ "",       NONE,  0,		0 },
-/*2*/	{ "fist",   WORD,  0,		0 },
-/*3*/	{ "fistp",  WORD,  0,		0 },
-/*4*/	{ "",       WORD,  op1(X),	(char *)db_Escb4 },
+/*2*/	{ "fist",   LONG,  0,		0 },
+/*3*/	{ "fistp",  LONG,  0,		0 },
+/*4*/	{ "",       WORD,  op1(X),	db_Escb4 },
 /*5*/	{ "fld",    EXTR,  0,		0 },
 /*6*/	{ "",       WORD,  0,		0 },
 /*7*/	{ "fstp",   EXTR,  0,		0 },
 };
 
-struct finst db_Escc[] = {
+static const struct finst db_Escc[] = {
 /*0*/	{ "fadd",   DBLR,  op2(ST,STI),	0 },
 /*1*/	{ "fmul",   DBLR,  op2(ST,STI),	0 },
-/*2*/	{ "fcom",   DBLR,  op2(ST,STI),	0 },
-/*3*/	{ "fcomp",  DBLR,  op2(ST,STI),	0 },
+/*2*/	{ "fcom",   DBLR,  0,		0 },
+/*3*/	{ "fcomp",  DBLR,  0,		0 },
 /*4*/	{ "fsub",   DBLR,  op2(ST,STI),	"fsubr" },
 /*5*/	{ "fsubr",  DBLR,  op2(ST,STI),	"fsub" },
 /*6*/	{ "fdiv",   DBLR,  op2(ST,STI),	"fdivr" },
 /*7*/	{ "fdivr",  DBLR,  op2(ST,STI),	"fdiv" },
 };
 
-struct finst db_Escd[] = {
+static const struct finst db_Escd[] = {
 /*0*/	{ "fld",    DBLR,  op1(STI),	"ffree" },
 /*1*/	{ "",       NONE,  0,		0 },
 /*2*/	{ "fst",    DBLR,  op1(STI),	0 },
@@ -420,34 +433,34 @@ struct finst db_Escd[] = {
 /*7*/	{ "fnstsw", NONE,  0,		0 },
 };
 
-struct finst db_Esce[] = {
-/*0*/	{ "fiadd",  LONG,  op2(ST,STI),	"faddp" },
-/*1*/	{ "fimul",  LONG,  op2(ST,STI),	"fmulp" },
-/*2*/	{ "ficom",  LONG,  0,		0 },
-/*3*/	{ "ficomp", LONG,  op1(X),	(char *)db_Esce3 },
-/*4*/	{ "fisub",  LONG,  op2(ST,STI),	"fsubrp" },
-/*5*/	{ "fisubr", LONG,  op2(ST,STI),	"fsubp" },
-/*6*/	{ "fidiv",  LONG,  op2(ST,STI),	"fdivrp" },
-/*7*/	{ "fidivr", LONG,  op2(ST,STI),	"fdivp" },
+static const struct finst db_Esce[] = {
+/*0*/	{ "fiadd",  WORD,  op2(ST,STI),	"faddp" },
+/*1*/	{ "fimul",  WORD,  op2(ST,STI),	"fmulp" },
+/*2*/	{ "ficom",  WORD,  0,		0 },
+/*3*/	{ "ficomp", WORD,  op1(X),	db_Esce3 },
+/*4*/	{ "fisub",  WORD,  op2(ST,STI),	"fsubrp" },
+/*5*/	{ "fisubr", WORD,  op2(ST,STI),	"fsubp" },
+/*6*/	{ "fidiv",  WORD,  op2(ST,STI),	"fdivrp" },
+/*7*/	{ "fidivr", WORD,  op2(ST,STI),	"fdivp" },
 };
 
-struct finst db_Escf[] = {
-/*0*/	{ "fild",   LONG,  0,		0 },
-/*1*/	{ "",       LONG,  0,		0 },
-/*2*/	{ "fist",   LONG,  0,		0 },
-/*3*/	{ "fistp",  LONG,  0,		0 },
-/*4*/	{ "fbld",   NONE,  op1(XA),	(char *)db_Escf4 },
-/*5*/	{ "fld",    QUAD,  0,		0 },
+static const struct finst db_Escf[] = {
+/*0*/	{ "fild",   WORD,  0,		0 },
+/*1*/	{ "",       NONE,  0,		0 },
+/*2*/	{ "fist",   WORD,  0,		0 },
+/*3*/	{ "fistp",  WORD,  0,		0 },
+/*4*/	{ "fbld",   NONE,  op1(XA),	db_Escf4 },
+/*5*/	{ "fild",   QUAD,  0,		0 },
 /*6*/	{ "fbstp",  NONE,  0,		0 },
-/*7*/	{ "fstp",   QUAD,  0,		0 },
+/*7*/	{ "fistp",  QUAD,  0,		0 },
 };
 
-struct finst *db_Esc_inst[] = {
+static const struct finst * const db_Esc_inst[] = {
 	db_Esc8, db_Esc9, db_Esca, db_Escb,
 	db_Escc, db_Escd, db_Esce, db_Escf
 };
 
-char *	db_Grp1[] = {
+static const char * const db_Grp1[] = {
 	"add",
 	"or",
 	"adc",
@@ -458,7 +471,7 @@ char *	db_Grp1[] = {
 	"cmp"
 };
 
-char *	db_Grp2[] = {
+static const char * const db_Grp2[] = {
 	"rol",
 	"ror",
 	"rcl",
@@ -469,7 +482,7 @@ char *	db_Grp2[] = {
 	"sar"
 };
 
-struct inst db_Grp3[] = {
+static const struct inst db_Grp3[] = {
 	{ "test",  TRUE, NONE, op2(I,E), 0 },
 	{ "test",  TRUE, NONE, op2(I,E), 0 },
 	{ "not",   TRUE, NONE, op1(E),   0 },
@@ -480,7 +493,7 @@ struct inst db_Grp3[] = {
 	{ "idiv",  TRUE, NONE, op2(E,A), 0 },
 };
 
-struct inst	db_Grp4[] = {
+static const struct inst db_Grp4[] = {
 	{ "inc",   TRUE, BYTE, op1(E),   0 },
 	{ "dec",   TRUE, BYTE, op1(E),   0 },
 	{ "",      TRUE, NONE, 0,	 0 },
@@ -491,23 +504,23 @@ struct inst	db_Grp4[] = {
 	{ "",      TRUE, NONE, 0,	 0 }
 };
 
-struct inst	db_Grp5[] = {
+static const struct inst db_Grp5[] = {
 	{ "inc",   TRUE, LONG, op1(E),   0 },
 	{ "dec",   TRUE, LONG, op1(E),   0 },
-	{ "call",  TRUE, NONE, op1(Eind),0 },
-	{ "lcall", TRUE, NONE, op1(Eind),0 },
-	{ "jmp",   TRUE, NONE, op1(Eind),0 },
-	{ "ljmp",  TRUE, NONE, op1(Eind),0 },
+	{ "call",  TRUE, LONG, op1(Eind),0 },
+	{ "lcall", TRUE, LONG, op1(Eind),0 },
+	{ "jmp",   TRUE, LONG, op1(Eind),0 },
+	{ "ljmp",  TRUE, LONG, op1(Eind),0 },
 	{ "push",  TRUE, LONG, op1(E),   0 },
 	{ "",      TRUE, NONE, 0,	 0 }
 };
 
-struct inst db_inst_table[256] = {
+static const struct inst db_inst_table[256] = {
 /*00*/	{ "add",   TRUE,  BYTE,  op2(R, E),  0 },
 /*01*/	{ "add",   TRUE,  LONG,  op2(R, E),  0 },
 /*02*/	{ "add",   TRUE,  BYTE,  op2(E, R),  0 },
 /*03*/	{ "add",   TRUE,  LONG,  op2(E, R),  0 },
-/*04*/	{ "add",   FALSE, BYTE,  op2(Is, A), 0 },
+/*04*/	{ "add",   FALSE, BYTE,  op2(I, A),  0 },
 /*05*/	{ "add",   FALSE, LONG,  op2(Is, A), 0 },
 /*06*/	{ "push",  FALSE, NONE,  op1(Si),    0 },
 /*07*/	{ "pop",   FALSE, NONE,  op1(Si),    0 },
@@ -525,7 +538,7 @@ struct inst db_inst_table[256] = {
 /*11*/	{ "adc",   TRUE,  LONG,  op2(R, E),  0 },
 /*12*/	{ "adc",   TRUE,  BYTE,  op2(E, R),  0 },
 /*13*/	{ "adc",   TRUE,  LONG,  op2(E, R),  0 },
-/*14*/	{ "adc",   FALSE, BYTE,  op2(Is, A), 0 },
+/*14*/	{ "adc",   FALSE, BYTE,  op2(I, A),  0 },
 /*15*/	{ "adc",   FALSE, LONG,  op2(Is, A), 0 },
 /*16*/	{ "push",  FALSE, NONE,  op1(Si),    0 },
 /*17*/	{ "pop",   FALSE, NONE,  op1(Si),    0 },
@@ -534,7 +547,7 @@ struct inst db_inst_table[256] = {
 /*19*/	{ "sbb",   TRUE,  LONG,  op2(R, E),  0 },
 /*1a*/	{ "sbb",   TRUE,  BYTE,  op2(E, R),  0 },
 /*1b*/	{ "sbb",   TRUE,  LONG,  op2(E, R),  0 },
-/*1c*/	{ "sbb",   FALSE, BYTE,  op2(Is, A), 0 },
+/*1c*/	{ "sbb",   FALSE, BYTE,  op2(I, A),  0 },
 /*1d*/	{ "sbb",   FALSE, LONG,  op2(Is, A), 0 },
 /*1e*/	{ "push",  FALSE, NONE,  op1(Si),    0 },
 /*1f*/	{ "pop",   FALSE, NONE,  op1(Si),    0 },
@@ -546,13 +559,13 @@ struct inst db_inst_table[256] = {
 /*24*/	{ "and",   FALSE, BYTE,  op2(I, A),  0 },
 /*25*/	{ "and",   FALSE, LONG,  op2(I, A),  0 },
 /*26*/	{ "",      FALSE, NONE,  0,	     0 },
-/*27*/	{ "aaa",   FALSE, NONE,  0,	     0 },
+/*27*/	{ "daa",   FALSE, NONE,  0,	     0 },
 
 /*28*/	{ "sub",   TRUE,  BYTE,  op2(R, E),  0 },
 /*29*/	{ "sub",   TRUE,  LONG,  op2(R, E),  0 },
 /*2a*/	{ "sub",   TRUE,  BYTE,  op2(E, R),  0 },
 /*2b*/	{ "sub",   TRUE,  LONG,  op2(E, R),  0 },
-/*2c*/	{ "sub",   FALSE, BYTE,  op2(Is, A), 0 },
+/*2c*/	{ "sub",   FALSE, BYTE,  op2(I, A),  0 },
 /*2d*/	{ "sub",   FALSE, LONG,  op2(Is, A), 0 },
 /*2e*/	{ "",      FALSE, NONE,  0,	     0 },
 /*2f*/	{ "das",   FALSE, NONE,  0,	     0 },
@@ -564,13 +577,13 @@ struct inst db_inst_table[256] = {
 /*34*/	{ "xor",   FALSE, BYTE,  op2(I, A),  0 },
 /*35*/	{ "xor",   FALSE, LONG,  op2(I, A),  0 },
 /*36*/	{ "",      FALSE, NONE,  0,	     0 },
-/*37*/	{ "daa",   FALSE, NONE,  0,	     0 },
+/*37*/	{ "aaa",   FALSE, NONE,  0,	     0 },
 
 /*38*/	{ "cmp",   TRUE,  BYTE,  op2(R, E),  0 },
 /*39*/	{ "cmp",   TRUE,  LONG,  op2(R, E),  0 },
 /*3a*/	{ "cmp",   TRUE,  BYTE,  op2(E, R),  0 },
 /*3b*/	{ "cmp",   TRUE,  LONG,  op2(E, R),  0 },
-/*3c*/	{ "cmp",   FALSE, BYTE,  op2(Is, A), 0 },
+/*3c*/	{ "cmp",   FALSE, BYTE,  op2(I, A),  0 },
 /*3d*/	{ "cmp",   FALSE, LONG,  op2(Is, A), 0 },
 /*3e*/	{ "",      FALSE, NONE,  0,	     0 },
 /*3f*/	{ "aas",   FALSE, NONE,  0,	     0 },
@@ -614,7 +627,7 @@ struct inst db_inst_table[256] = {
 /*60*/	{ "pusha", FALSE, LONG,  0,	     0 },
 /*61*/	{ "popa",  FALSE, LONG,  0,	     0 },
 /*62*/  { "bound", TRUE,  LONG,  op2(E, R),  0 },
-/*63*/	{ "arpl",  TRUE,  NONE,  op2(Ew,Rw), 0 },
+/*63*/	{ "arpl",  TRUE,  NONE,  op2(Rw,Ew), 0 },
 
 /*64*/	{ "",      FALSE, NONE,  0,	     0 },
 /*65*/	{ "",      FALSE, NONE,  0,	     0 },
@@ -623,7 +636,7 @@ struct inst db_inst_table[256] = {
 
 /*68*/	{ "push",  FALSE, LONG,  op1(I),     0 },
 /*69*/  { "imul",  TRUE,  LONG,  op3(I,E,R), 0 },
-/*6a*/	{ "push",  FALSE, LONG,  op1(Ib),    0 },
+/*6a*/	{ "push",  FALSE, LONG,  op1(Ibs),   0 },
 /*6b*/  { "imul",  TRUE,  LONG,  op3(Ibs,E,R),0 },
 /*6c*/	{ "ins",   FALSE, BYTE,  op2(DX, DI), 0 },
 /*6d*/	{ "ins",   FALSE, LONG,  op2(DX, DI), 0 },
@@ -648,10 +661,10 @@ struct inst db_inst_table[256] = {
 /*7e*/	{ "jle",   FALSE, NONE,  op1(Db),     0 },
 /*7f*/	{ "jnle",  FALSE, NONE,  op1(Db),     0 },
 
-/*80*/  { "",	   TRUE,  BYTE,  op2(I, E),   (char *)db_Grp1 },
-/*81*/  { "",	   TRUE,  LONG,  op2(I, E),   (char *)db_Grp1 },
-/*82*/  { "",	   TRUE,  BYTE,  op2(Is,E),   (char *)db_Grp1 },
-/*83*/  { "",	   TRUE,  LONG,  op2(Ibs,E),  (char *)db_Grp1 },
+/*80*/  { "",	   TRUE,  BYTE,  op2(I, E),   db_Grp1 },
+/*81*/  { "",	   TRUE,  LONG,  op2(I, E),   db_Grp1 },
+/*82*/  { "",	   TRUE,  BYTE,  op2(I, E),   db_Grp1 },
+/*83*/  { "",	   TRUE,  LONG,  op2(Ibs,E),  db_Grp1 },
 /*84*/	{ "test",  TRUE,  BYTE,  op2(R, E),   0 },
 /*85*/	{ "test",  TRUE,  LONG,  op2(R, E),   0 },
 /*86*/	{ "xchg",  TRUE,  BYTE,  op2(R, E),   0 },
@@ -720,8 +733,8 @@ struct inst db_inst_table[256] = {
 /*be*/	{ "mov",   FALSE, LONG,  op2(I, Ri),  0 },
 /*bf*/	{ "mov",   FALSE, LONG,  op2(I, Ri),  0 },
 
-/*c0*/	{ "",	   TRUE,  BYTE,  op2(Ib, E),  (char *)db_Grp2 },
-/*c1*/	{ "",	   TRUE,  LONG,  op2(Ib, E),  (char *)db_Grp2 },
+/*c0*/	{ "",	   TRUE,  BYTE,  op2(Ib, E),  db_Grp2 },
+/*c1*/	{ "",	   TRUE,  LONG,  op2(Ib, E),  db_Grp2 },
 /*c2*/	{ "ret",   FALSE, NONE,  op1(Iw),     0 },
 /*c3*/	{ "ret",   FALSE, NONE,  0,	      0 },
 /*c4*/	{ "les",   TRUE,  LONG,  op2(E, R),   0 },
@@ -729,7 +742,7 @@ struct inst db_inst_table[256] = {
 /*c6*/	{ "mov",   TRUE,  BYTE,  op2(I, E),   0 },
 /*c7*/	{ "mov",   TRUE,  LONG,  op2(I, E),   0 },
 
-/*c8*/	{ "enter", FALSE, NONE,  op2(Ib, Iw), 0 },
+/*c8*/	{ "enter", FALSE, NONE,  op2(Iw, Ib), 0 },
 /*c9*/	{ "leave", FALSE, NONE,  0,           0 },
 /*ca*/	{ "lret",  FALSE, NONE,  op1(Iw),     0 },
 /*cb*/	{ "lret",  FALSE, NONE,  0,	      0 },
@@ -738,23 +751,23 @@ struct inst db_inst_table[256] = {
 /*ce*/	{ "into",  FALSE, NONE,  0,	      0 },
 /*cf*/	{ "iret",  FALSE, NONE,  0,	      0 },
 
-/*d0*/	{ "",	   TRUE,  BYTE,  op2(o1, E),  (char *)db_Grp2 },
-/*d1*/	{ "",	   TRUE,  LONG,  op2(o1, E),  (char *)db_Grp2 },
-/*d2*/	{ "",	   TRUE,  BYTE,  op2(CL, E),  (char *)db_Grp2 },
-/*d3*/	{ "",	   TRUE,  LONG,  op2(CL, E),  (char *)db_Grp2 },
-/*d4*/	{ "aam",   TRUE,  NONE,  0,	      0 },
-/*d5*/	{ "aad",   TRUE,  NONE,  0,	      0 },
-/*d6*/	{ "",      FALSE, NONE,  0,	      0 },
+/*d0*/	{ "",	   TRUE,  BYTE,  op2(o1, E),  db_Grp2 },
+/*d1*/	{ "",	   TRUE,  LONG,  op2(o1, E),  db_Grp2 },
+/*d2*/	{ "",	   TRUE,  BYTE,  op2(CL, E),  db_Grp2 },
+/*d3*/	{ "",	   TRUE,  LONG,  op2(CL, E),  db_Grp2 },
+/*d4*/	{ "aam",   FALSE, NONE,  op1(Iba),    0 },
+/*d5*/	{ "aad",   FALSE, NONE,  op1(Iba),    0 },
+/*d6*/	{ ".byte\t0xd6", FALSE, NONE, 0,      0 },
 /*d7*/	{ "xlat",  FALSE, BYTE,  op1(BX),     0 },
 
-/*d8*/  { "",      TRUE,  NONE,  0,	      (char *)db_Esc8 },
-/*d9*/  { "",      TRUE,  NONE,  0,	      (char *)db_Esc9 },
-/*da*/  { "",      TRUE,  NONE,  0,	      (char *)db_Esca },
-/*db*/  { "",      TRUE,  NONE,  0,	      (char *)db_Escb },
-/*dc*/  { "",      TRUE,  NONE,  0,	      (char *)db_Escc },
-/*dd*/  { "",      TRUE,  NONE,  0,	      (char *)db_Escd },
-/*de*/  { "",      TRUE,  NONE,  0,	      (char *)db_Esce },
-/*df*/  { "",      TRUE,  NONE,  0,	      (char *)db_Escf },
+/*d8*/  { "",      TRUE,  NONE,  0,	      db_Esc8 },
+/*d9*/  { "",      TRUE,  NONE,  0,	      db_Esc9 },
+/*da*/  { "",      TRUE,  NONE,  0,	      db_Esca },
+/*db*/  { "",      TRUE,  NONE,  0,	      db_Escb },
+/*dc*/  { "",      TRUE,  NONE,  0,	      db_Escc },
+/*dd*/  { "",      TRUE,  NONE,  0,	      db_Escd },
+/*de*/  { "",      TRUE,  NONE,  0,	      db_Esce },
+/*df*/  { "",      TRUE,  NONE,  0,	      db_Escf },
 
 /*e0*/	{ "loopne",FALSE, NONE,  op1(Db),     0 },
 /*e1*/	{ "loope", FALSE, NONE,  op1(Db),     0 },
@@ -775,13 +788,13 @@ struct inst db_inst_table[256] = {
 /*ef*/	{ "out",   FALSE, LONG,  op2(A, DX) , 0 },
 
 /*f0*/	{ "",      FALSE, NONE,  0,	     0 },
-/*f1*/	{ "",      FALSE, NONE,  0,	     0 },
+/*f1*/	{ ".byte\t0xf1", FALSE, NONE, 0,     0 },
 /*f2*/	{ "",      FALSE, NONE,  0,	     0 },
 /*f3*/	{ "",      FALSE, NONE,  0,	     0 },
 /*f4*/	{ "hlt",   FALSE, NONE,  0,	     0 },
 /*f5*/	{ "cmc",   FALSE, NONE,  0,	     0 },
-/*f6*/	{ "",      TRUE,  BYTE,  0,	     (char *)db_Grp3 },
-/*f7*/	{ "",	   TRUE,  LONG,  0,	     (char *)db_Grp3 },
+/*f6*/	{ "",      TRUE,  BYTE,  0,	     db_Grp3 },
+/*f7*/	{ "",	   TRUE,  LONG,  0,	     db_Grp3 },
 
 /*f8*/	{ "clc",   FALSE, NONE,  0,	     0 },
 /*f9*/	{ "stc",   FALSE, NONE,  0,	     0 },
@@ -789,11 +802,11 @@ struct inst db_inst_table[256] = {
 /*fb*/	{ "sti",   FALSE, NONE,  0,	     0 },
 /*fc*/	{ "cld",   FALSE, NONE,  0,	     0 },
 /*fd*/	{ "std",   FALSE, NONE,  0,	     0 },
-/*fe*/	{ "",	   TRUE,  NONE,  0,	     (char *)db_Grp4 },
-/*ff*/	{ "",	   TRUE,  NONE,  0,	     (char *)db_Grp5 },
+/*fe*/	{ "",	   TRUE,  NONE,  0,	     db_Grp4 },
+/*ff*/	{ "",	   TRUE,  NONE,  0,	     db_Grp5 },
 };
 
-struct inst	db_bad_inst =
+static const struct inst db_bad_inst =
 	{ "???",   FALSE, NONE,  0,	      0 }
 ;
 
@@ -808,12 +821,12 @@ struct inst	db_bad_inst =
 struct i_addr {
 	int		is_reg;	/* if reg, reg number is in 'disp' */
 	int		disp;
-	char *		base;
-	char *		index;
+	const char *	base;
+	const char *	index;
 	int		ss;
 };
 
-char *	db_index_reg_16[8] = {
+static const char * const db_index_reg_16[8] = {
 	"%bx,%si",
 	"%bx,%di",
 	"%bp,%si",
@@ -824,20 +837,20 @@ char *	db_index_reg_16[8] = {
 	"%bx"
 };
 
-char *	db_reg[3][8] = {
+static const char * const db_reg[3][8] = {
 	"%al",  "%cl",  "%dl",  "%bl",  "%ah",  "%ch",  "%dh",  "%bh",
 	"%ax",  "%cx",  "%dx",  "%bx",  "%sp",  "%bp",  "%si",  "%di",
 	"%eax", "%ecx", "%edx", "%ebx", "%esp", "%ebp", "%esi", "%edi"
 };
 
-char *	db_seg_reg[8] = {
+static const char * const db_seg_reg[8] = {
 	"%es", "%cs", "%ss", "%ds", "%fs", "%gs", "", ""
 };
 
 /*
  * lengths for size attributes
  */
-int db_lengths[] = {
+static const int db_lengths[] = {
 	1,	/* BYTE */
 	2,	/* WORD */
 	4,	/* LONG */
@@ -851,15 +864,24 @@ int db_lengths[] = {
 	result = db_get_value((loc), (size), (is_signed)); \
 	(loc) += (size);
 
+static db_addr_t
+		db_disasm_esc __P((db_addr_t loc, int inst, int short_addr,
+				   int size, const char *seg));
+static void	db_print_address __P((const char *seg, int size,
+				      struct i_addr *addrp));
+static db_addr_t
+		db_read_address __P((db_addr_t loc, int short_addr,
+				     int regmodrm, struct i_addr *addrp));
+
 /*
  * Read address at location and return updated location.
  */
-db_addr_t
+static db_addr_t
 db_read_address(loc, short_addr, regmodrm, addrp)
 	db_addr_t	loc;
 	int		short_addr;
 	int		regmodrm;
-	struct i_addr	*addrp;		/* out */
+	struct i_addr *	addrp;		/* out */
 {
 	int		mod, rm, sib, index, disp;
 
@@ -880,7 +902,7 @@ db_read_address(loc, short_addr, regmodrm, addrp)
 	    switch (mod) {
 		case 0:
 		    if (rm == 6) {
-			get_value_inc(disp, loc, 2, TRUE);
+			get_value_inc(disp, loc, 2, FALSE);
 			addrp->disp = disp;
 			addrp->base = 0;
 		    }
@@ -891,11 +913,12 @@ db_read_address(loc, short_addr, regmodrm, addrp)
 		    break;
 		case 1:
 		    get_value_inc(disp, loc, 1, TRUE);
+		    disp &= 0xFFFF;
 		    addrp->disp = disp;
 		    addrp->base = db_index_reg_16[rm];
 		    break;
 		case 2:
-		    get_value_inc(disp, loc, 2, TRUE);
+		    get_value_inc(disp, loc, 2, FALSE);
 		    addrp->disp = disp;
 		    addrp->base = db_index_reg_16[rm];
 		    break;
@@ -939,11 +962,11 @@ db_read_address(loc, short_addr, regmodrm, addrp)
 	return (loc);
 }
 
-void
+static void
 db_print_address(seg, size, addrp)
-	char *		seg;
+	const char *	seg;
 	int		size;
-	struct i_addr	*addrp;
+	struct i_addr *	addrp;
 {
 	if (addrp->is_reg) {
 	    db_printf("%s", db_reg[size][addrp->disp]);
@@ -969,24 +992,28 @@ db_print_address(seg, size, addrp)
  * Disassemble floating-point ("escape") instruction
  * and return updated location.
  */
-db_addr_t
+static db_addr_t
 db_disasm_esc(loc, inst, short_addr, size, seg)
 	db_addr_t	loc;
 	int		inst;
 	int		short_addr;
 	int		size;
-	char *		seg;
+	const char *	seg;
 {
 	int		regmodrm;
-	struct finst	*fp;
+	const struct finst *	fp;
 	int		mod;
 	struct i_addr	address;
-	char *		name;
+	const char *	name;
 
 	get_value_inc(regmodrm, loc, 1, FALSE);
 	fp = &db_Esc_inst[inst - 0xd8][f_reg(regmodrm)];
 	mod = f_mod(regmodrm);
 	if (mod != 3) {
+	    if (*fp->f_name == '\0') {
+		db_printf("<bad instruction>");
+		return (loc);
+	    }
 	    /*
 	     * Normal address modes.
 	     */
@@ -1035,13 +1062,19 @@ db_disasm_esc(loc, inst, short_addr, size, seg)
 		    db_printf("%s\t%%st(%d)",name, f_rm(regmodrm));
 		    break;
 		case op1(X):
-		    db_printf("%s", ((char **)fp->f_rrname)[f_rm(regmodrm)]);
+		    name = ((const char * const *)fp->f_rrname)[f_rm(regmodrm)];
+		    if (*name == '\0')
+			goto bad;
+		    db_printf("%s", name);
 		    break;
 		case op1(XA):
-		    db_printf("%s\t%%ax",
-				 ((char **)fp->f_rrname)[f_rm(regmodrm)]);
+		    name = ((const char * const *)fp->f_rrname)[f_rm(regmodrm)];
+		    if (*name == '\0')
+			goto bad;
+		    db_printf("%s\t%%ax", name);
 		    break;
 		default:
+		bad:
 		    db_printf("<bad instruction>");
 		    break;
 	    }
@@ -1063,9 +1096,9 @@ db_disasm(loc, altfmt)
 	int	inst;
 	int	size;
 	int	short_addr;
-	char *	seg;
-	struct inst *	ip;
-	char *	i_name;
+	const char *	seg;
+	const struct inst *	ip;
+	const char *	i_name;
 	int	i_size;
 	int	i_mode;
 	int	regmodrm = 0;
@@ -1158,22 +1191,19 @@ db_disasm(loc, altfmt)
 	i_size = ip->i_size;
 	i_mode = ip->i_mode;
 
-	if (ip->i_extra == (char *)db_Grp1 ||
-	    ip->i_extra == (char *)db_Grp2 ||
-	    ip->i_extra == (char *)db_Grp6 ||
-	    ip->i_extra == (char *)db_Grp7 ||
-	    ip->i_extra == (char *)db_Grp8) {
-	    i_name = ((char **)ip->i_extra)[f_reg(regmodrm)];
+	if (ip->i_extra == db_Grp1 || ip->i_extra == db_Grp2 ||
+	    ip->i_extra == db_Grp6 || ip->i_extra == db_Grp7 ||
+	    ip->i_extra == db_Grp8 || ip->i_extra == db_Grp9) {
+	    i_name = ((const char * const *)ip->i_extra)[f_reg(regmodrm)];
 	}
-	else if (ip->i_extra == (char *)db_Grp3) {
-	    ip = (struct inst *)ip->i_extra;
+	else if (ip->i_extra == db_Grp3) {
+	    ip = ip->i_extra;
 	    ip = &ip[f_reg(regmodrm)];
 	    i_name = ip->i_name;
 	    i_mode = ip->i_mode;
 	}
-	else if (ip->i_extra == (char *)db_Grp4 ||
-		 ip->i_extra == (char *)db_Grp5) {
-	    ip = (struct inst *)ip->i_extra;
+	else if (ip->i_extra == db_Grp4 || ip->i_extra == db_Grp5) {
+	    ip = ip->i_extra;
 	    ip = &ip[f_reg(regmodrm)];
 	    i_name = ip->i_name;
 	    i_mode = ip->i_mode;
@@ -1184,7 +1214,7 @@ db_disasm(loc, altfmt)
 	    if (size == WORD)
 		db_printf(i_name);
 	    else
-		db_printf(ip->i_extra);
+		db_printf((const char *)ip->i_extra);
 	}
 	else {
 	    db_printf(i_name);
@@ -1222,6 +1252,10 @@ db_disasm(loc, altfmt)
 		    db_print_address(seg, size, &address);
 		    break;
 
+		case El:
+		    db_print_address(seg, LONG, &address);
+		    break;
+
 		case Ew:
 		    db_print_address(seg, WORD, &address);
 		    break;
@@ -1242,6 +1276,10 @@ db_disasm(loc, altfmt)
 		    db_printf("%s", db_reg[size][f_rm(inst)]);
 		    break;
 
+		case Ril:
+		    db_printf("%s", db_reg[LONG][f_rm(inst)]);
+		    break;
+
 		case S:
 		    db_printf("%s", db_seg_reg[f_reg(regmodrm)]);
 		    break;
@@ -1292,57 +1330,67 @@ db_disasm(loc, altfmt)
 
 		case I:
 		    len = db_lengths[size];
-		    get_value_inc(imm, loc, len, FALSE);/* unsigned */
+		    get_value_inc(imm, loc, len, FALSE);
 		    db_printf("$%#n", imm);
 		    break;
 
 		case Is:
 		    len = db_lengths[size];
-		    get_value_inc(imm, loc, len, TRUE);	/* signed */
-		    db_printf("$%#r", imm);
+		    get_value_inc(imm, loc, len, FALSE);
+		    /*
+		     * XXX the + in this format doesn't seem to work right.
+		     * `Is' is equivalent to `I'.
+		     */
+		    db_printf("$%+#n", imm);
 		    break;
 
 		case Ib:
-		    get_value_inc(imm, loc, 1, FALSE);	/* unsigned */
+		    get_value_inc(imm, loc, 1, FALSE);
 		    db_printf("$%#n", imm);
 		    break;
 
+		case Iba:
+		    get_value_inc(imm, loc, 1, FALSE);
+		    if (imm != 0x0a)
+			db_printf("$%#n", imm);
+		    break;
+
 		case Ibs:
-		    get_value_inc(imm, loc, 1, TRUE);	/* signed */
-		    db_printf("$%#r", imm);
+		    get_value_inc(imm, loc, 1, TRUE);
+		    if (size == WORD)
+			imm &= 0xFFFF;
+		    db_printf("$%+#n", imm);
 		    break;
 
 		case Iw:
-		    get_value_inc(imm, loc, 2, FALSE);	/* unsigned */
-		    db_printf("$%#n", imm);
-		    break;
-
-		case Il:
-		    get_value_inc(imm, loc, 4, FALSE);
+		    get_value_inc(imm, loc, 2, FALSE);
 		    db_printf("$%#n", imm);
 		    break;
 
 		case O:
-		    if (short_addr) {
-			get_value_inc(displ, loc, 2, TRUE);
-		    }
-		    else {
-			get_value_inc(displ, loc, 4, TRUE);
-		    }
+		    len = (short_addr ? 2 : 4);
+		    get_value_inc(displ, loc, len, FALSE);
 		    if (seg)
-			db_printf("%s:%#r",seg, displ);
+			db_printf("%s:%+#n",seg, displ);
 		    else
 			db_printsym((db_addr_t)displ, DB_STGY_ANY);
 		    break;
 
 		case Db:
 		    get_value_inc(displ, loc, 1, TRUE);
-		    db_printsym((db_addr_t)(displ + loc), DB_STGY_XTRN);
+		    displ += loc;
+		    if (size == WORD)
+			displ &= 0xFFFF;
+		    db_printsym((db_addr_t)displ, DB_STGY_XTRN);
 		    break;
 
 		case Dl:
-		    get_value_inc(displ, loc, 4, TRUE);
-		    db_printsym((db_addr_t)(displ + loc), DB_STGY_XTRN);
+		    len = db_lengths[size];
+		    get_value_inc(displ, loc, len, FALSE);
+		    displ += loc;
+		    if (size == WORD)
+			displ &= 0xFFFF;
+		    db_printsym((db_addr_t)displ, DB_STGY_XTRN);
 		    break;
 
 		case o1:
@@ -1354,22 +1402,13 @@ db_disasm(loc, altfmt)
 		    break;
 
 		case OS:
-		    get_value_inc(imm, loc, 4, FALSE);	/* offset */
+		    len = db_lengths[size];
+		    get_value_inc(imm, loc, len, FALSE);	/* offset */
 		    get_value_inc(imm2, loc, 2, FALSE);	/* segment */
 		    db_printf("$%#n,%#n", imm2, imm);
 		    break;
 	    }
 	}
-
-	if (altfmt == 0) {
-	    if (inst == 0xe9 || inst == 0xeb) {
-		/*
-		 * GAS pads to longword boundary after unconditional jumps.
-		 */
-		loc = (loc + (4-1)) & ~(4-1);
-	    }
-	}
 	db_printf("\n");
 	return (loc);
 }
-
