@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)genassym.c	5.11 (Berkeley) 5/10/91
- * $FreeBSD: stable/2.1/sys/i386/i386/genassym.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/genassym.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <stdio.h>
@@ -50,6 +50,7 @@
 #include <sys/mount.h>
 #endif
 #include <sys/mbuf.h>
+#include <sys/socket.h>
 #include <sys/msgbuf.h>
 #include <machine/cpu.h>
 #include <machine/trap.h>
@@ -57,13 +58,20 @@
 #include <machine/reg.h>
 #include <machine/bootinfo.h>
 #include <sys/syscall.h>
+#include <sys/vmmeter.h>
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 #include <sys/user.h>
 #include <net/if.h>
 #include <netinet/in.h>
 #include <nfs/nfsv2.h>
 #include <nfs/nfsdiskless.h>
 
+extern int	main __P((void));
+
 int
 main()
 {
@@ -78,9 +86,8 @@ main()
 	struct sigframe *sigf = (struct sigframe *)0;
 	struct bootinfo *bootinfo = (struct bootinfo *)0;
 
-	printf("#define\tUDOT_SZ %d\n", sizeof(struct user));
-	printf("#define\tP_FORW %p\n", &p->p_forw);
-	printf("#define\tP_BACK %p\n", &p->p_back);
+	printf("#define\tP_FORW %p\n", &p->p_procq.tqe_next);
+	printf("#define\tP_BACK %p\n", &p->p_procq.tqe_prev);
 	printf("#define\tP_VMSPACE %p\n", &p->p_vmspace);
 	printf("#define\tVM_PMAP %p\n", &vms->vm_pmap);
 	printf("#define\tP_ADDR %p\n", &p->p_addr);
@@ -97,30 +104,19 @@ main()
 	printf("#define\tV_SYSCALL %p\n", &vm->v_syscall);
 	printf("#define\tV_INTR %p\n", &vm->v_intr);
 	printf("#define\tUPAGES %d\n", UPAGES);
-	printf("#define\tCLSIZE %d\n", CLSIZE);
-	printf("#define\tNBPG %d\n", NBPG);
+	printf("#define\tPAGE_SIZE %d\n", PAGE_SIZE);
 	printf("#define\tNPTEPG %d\n", NPTEPG);
+	printf("#define\tNPDEPG %d\n", NPDEPG);
 	printf("#define\tPDESIZE %d\n", PDESIZE);
 	printf("#define\tPTESIZE %d\n", PTESIZE);
 	printf("#define\tNKPDE %d\n", NKPDE);
 	printf("#define\tNKPT %d\n", NKPT);
-	printf("#define\tKPTDI 0x%x\n", KPTDI);
-	printf("#define\tKSTKPTDI 0x%x\n", KSTKPTDI);
-	printf("#define\tKSTKPTEOFF 0x%x\n", KSTKPTEOFF);
-	printf("#define\tPTDPTDI 0x%x\n", PTDPTDI);
-	printf("#define\tAPTDPTDI 0x%x\n", APTDPTDI);
-	printf("#define\tPGSHIFT %d\n", PGSHIFT);
+	printf("#define\tPAGE_SHIFT %d\n", PAGE_SHIFT);
+	printf("#define\tPAGE_MASK %d\n", PAGE_MASK);
 	printf("#define\tPDRSHIFT %d\n", PDRSHIFT);
-	printf("#define\tSYSPTSIZE %d\n", SYSPTSIZE);
-	printf("#define\tUSRPTSIZE %d\n", USRPTSIZE);
-	printf("#define\tUSRIOSIZE %d\n", USRIOSIZE);
-#ifdef SYSVSHM
-	printf("#define\tSHMMAXPGS %d\n", SHMMAXPGS);
-#endif
 	printf("#define\tUSRSTACK 0x%lx\n", USRSTACK);
 	printf("#define\tVM_MAXUSER_ADDRESS 0x%lx\n", VM_MAXUSER_ADDRESS);
 	printf("#define\tKERNBASE 0x%x\n", KERNBASE);
-	printf("#define\tMSGBUFPTECNT %d\n", btoc(sizeof (struct msgbuf)));
 	printf("#define\tMCLBYTES %d\n", MCLBYTES);
 	printf("#define\tPCB_LINK %p\n", &pcb->pcb_tss.tss_link);
 	printf("#define\tPCB_ESP0 %p\n", &pcb->pcb_tss.tss_esp0);
@@ -158,9 +154,7 @@ main()
 	printf("#define\tRU_MINFLT %p\n", &rup->ru_minflt);
 	printf("#define\tPCB_FLAGS %p\n", &pcb->pcb_flags);
 	printf("#define\tPCB_SAVEFPU %p\n", &pcb->pcb_savefpu);
-	printf("#define\tFP_USESEMC %d\n", FP_USESEMC);
-	printf("#define\tPCB_SAVEEMC %p\n", &pcb->pcb_saveemc);
-	printf("#define\tPCB_INL %p\n", &pcb->pcb_inl);
+	printf("#define\tPCB_SAVEFPU_SIZE %d\n", sizeof pcb->pcb_savefpu);
 	printf("#define\tPCB_ONFAULT %p\n", &pcb->pcb_onfault);
 
 	printf("#define\tTF_ES %p\n", &tf->tf_es);
