@@ -37,7 +37,7 @@
  *	@(#)procfs_machdep.c	8.3 (Berkeley) 1/27/94
  *
  * From:
- * $FreeBSD: stable/2.1/sys/i386/i386/procfs_machdep.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/procfs_machdep.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -62,8 +62,6 @@
  * procfs_sstep(proc)
  *	Arrange for the process to trap after executing a single instruction.
  *
- * procfs_fix_sstep(proc)
- *	Cleanup process state after executing a single-step instruction.
  */
 
 #include <sys/param.h>
@@ -72,7 +70,6 @@
 #include <sys/kernel.h>
 #include <sys/proc.h>
 #include <sys/ptrace.h>
-#include <sys/user.h>
 #include <sys/vnode.h>
 #include <machine/psl.h>
 #include <machine/reg.h>
@@ -80,6 +77,15 @@
 #include <machine/md_var.h>
 #include <miscfs/procfs/procfs.h>
 
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
+
+#include <sys/user.h>
+
 int
 procfs_read_regs(p, regs)
 	struct proc *p;
@@ -135,9 +141,3 @@ procfs_sstep(p)
 		return (EIO);
 	return (ptrace_single_step(p));
 }
-
-void
-procfs_fix_sstep(p)
-	struct proc *p;
-{
-}
