@@ -35,30 +35,41 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)trap.c	7.4 (Berkeley) 5/13/91
- * $FreeBSD: stable/2.1/sys/i386/i386/trap.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/trap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
  * 386 Trap and System call handling
  */
 
+#include "opt_cpu.h"
+#include "opt_ddb.h"
+#include "opt_ktrace.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/acct.h>
 #include <sys/kernel.h>
 #include <sys/syscall.h>
 #include <sys/sysent.h>
+#include <sys/queue.h>
+#include <sys/vmmeter.h>
 #ifdef KTRACE
 #include <sys/ktrace.h>
 #endif
 
+#include <vm/vm.h>
 #include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
 #include <vm/pmap.h>
 #include <vm/vm_kern.h>
 #include <vm/vm_map.h>
 #include <vm/vm_page.h>
+#include <vm/vm_extern.h>
+
+#include <sys/user.h>
 
 #include <machine/cpu.h>
 #include <machine/md_var.h>
@@ -68,17 +79,18 @@
 #include <machine/../isa/isa_device.h>
 
 #ifdef POWERFAIL_NMI
-# include <syslog.h>
-# include <machine/clock.h>
+#include <sys/syslog.h>
+#include <machine/clock.h>
 #endif
 
 #include "isa.h"
 #include "npx.h"
 
+int (*pmath_emulate) __P((struct trapframe *));
+
 extern void trap __P((struct trapframe frame));
 extern int trapwrite __P((unsigned addr));
 extern void syscall __P((struct trapframe frame));
-extern void linux_syscall __P((struct trapframe frame));
 
 static int trap_pfault __P((struct trapframe *, int));
 static void trap_fatal __P((struct trapframe *));
@@ -86,8 +98,8 @@ void dblfault_handler __P((void));
 
 extern inthand_t IDTVEC(syscall);
 
-#define MAX_TRAP_MSG		27
-char *trap_msg[] = {
+#define MAX_TRAP_MSG		28
+static char *trap_msg[] = {
 	"",					/*  0 unused */
 	"privileged instruction fault",		/*  1 T_PRIVINFLT */
 	"",					/*  2 unused */
@@ -116,12 +128,18 @@ char *trap_msg[] = {
 	"invalid TSS fault",			/* 25 T_TSSFLT */
 	"segment not present fault",		/* 26 T_SEGNPFLT */
 	"stack fault",				/* 27 T_STKFLT */
+	"machine check trap",			/* 28 T_MCHK */
 };
 
 static void userret __P((struct proc *p, struct trapframe *frame,
 			 u_quad_t oticks));
 
-static inline void
+#if defined(I586_CPU) && !defined(NO_F00F_HACK)
+extern struct gate_descriptor *t_idt;
+extern int has_f00f_bug;
+#endif
+
+static __inline void
 userret(p, frame, oticks)
 	struct proc *p;
 	struct trapframe *frame;
@@ -141,7 +159,7 @@ userret(p, frame, oticks)
 		 * mi_switch()'ed, we might not be on the queue indicated by
 		 * our priority.
 		 */
-		s = splclock();
+		s = splhigh();
 		setrunqueue(p);
 		p->p_stats->p_ru.ru_nivcsw++;
 		mi_switch();
@@ -152,21 +170,10 @@ userret(p, frame, oticks)
 	/*
 	 * Charge system time if profiling.
 	 */
-	if (p->p_flag & P_PROFIL) {
-		u_quad_t ticks = p->p_sticks - oticks;
+	if (p->p_flag & P_PROFIL)
+		addupc_task(p, frame->tf_eip,
+			    (u_int)(p->p_sticks - oticks) * psratio);
 
-		if (ticks) {
-#ifdef PROFTIMER
-			extern int profscale;
-			addupc(frame->tf_eip, &p->p_stats->p_prof,
-			    ticks * profscale);
-#else
-			extern int psratio;
-			addupc(frame->tf_eip, &p->p_stats->p_prof,
-			    ticks * psratio);
-#endif
-		}
-	}
 	curpriority = p->p_priority;
 }
 
@@ -184,10 +191,13 @@ trap(frame)
 	struct proc *p = curproc;
 	u_quad_t sticks = 0;
 	int i = 0, ucode = 0, type, code;
-#ifdef DIAGNOSTIC
+#ifdef DEBUG
 	u_long eva;
 #endif
 
+#if defined(I586_CPU) && !defined(NO_F00F_HACK)
+restart:
+#endif
 	type = frame.tf_trapno;
 	code = frame.tf_err;
 
@@ -218,8 +228,9 @@ trap(frame)
 			astoff();
 			cnt.v_soft++;
 			if (p->p_flag & P_OWEUPC) {
-				addupc(frame.tf_eip, &p->p_stats->p_prof, 1);
 				p->p_flag &= ~P_OWEUPC;
+				addupc_task(p, p->p_stats->p_prof.pr_addr,
+					    p->p_stats->p_prof.pr_ticks);
 			}
 			goto out;
 
@@ -237,6 +248,10 @@ trap(frame)
 			i = trap_pfault(&frame, TRUE);
 			if (i == -1)
 				return;
+#if defined(I586_CPU) && !defined(NO_F00F_HACK)
+			if (i == -2)
+				goto restart;
+#endif
 			if (i == 0)
 				goto out;
 
@@ -280,10 +295,13 @@ trap(frame)
 			/* if a transparent fault (due to context switch "late") */
 			if (npxdna())
 				return;
-#endif	/* NNPX > 0 */
-
-#if defined(MATH_EMULATE) || defined(GPL_MATH_EMULATE)
-			i = math_emulate(&frame);
+#endif
+			if (!pmath_emulate) {
+				i = SIGFPE;
+				ucode = FPE_FPU_NP_TRAP;
+				break;
+			}
+			i = (*pmath_emulate)(&frame);
 			if (i == 0) {
 				if (!(frame.tf_eflags & PSL_T))
 					return;
@@ -291,10 +309,6 @@ trap(frame)
 				i = SIGTRAP;
 			}
 			/* else ucode = emulator_only_knows() XXX */
-#else	/* MATH_EMULATE || GPL_MATH_EMULATE */
-			i = SIGFPE;
-			ucode = FPE_FPU_NP_TRAP;
-#endif	/* MATH_EMULATE || GPL_MATH_EMULATE */
 			break;
 
 		case T_FPOPFLT:		/* FPU operand fetch fault */
@@ -310,6 +324,18 @@ trap(frame)
 			(void) trap_pfault(&frame, FALSE);
 			return;
 
+		case T_DNA:
+#if NNPX > 0
+			/*
+			 * The kernel is apparently using npx for copying.
+			 * XXX this should be fatal unless the kernel has
+			 * registered such use.
+			 */
+			if (npxdna())
+				return;
+#endif
+			break;
+
 		case T_PROTFLT:		/* general protection fault */
 		case T_SEGNPFLT:	/* segment not present fault */
 			/*
@@ -337,10 +363,10 @@ trap(frame)
 						   doreti_popl_ds_fault);
 				MAYBE_DORETI_FAULT(doreti_popl_es,
 						   doreti_popl_es_fault);
-			}
-			if (curpcb && curpcb->pcb_onfault) {
-				frame.tf_eip = (int)curpcb->pcb_onfault;
-				return;
+				if (curpcb && curpcb->pcb_onfault) {
+					frame.tf_eip = (int)curpcb->pcb_onfault;
+					return;
+				}
 			}
 			break;
 
@@ -428,6 +454,10 @@ trap(frame)
 		return;
 	}
 
+	/* Translate fault for emulators (e.g. Linux) */
+	if (*p->p_sysent->sv_transtrap)
+		i = (*p->p_sysent->sv_transtrap)(i, type);
+
 	trapsignal(p, i, ucode);
 
 #ifdef DEBUG
@@ -453,7 +483,7 @@ out:
  * to be made safe are the iBCS2 code and the process tracing/
  * debugging code.
  */
-int
+static int
 trap_pfault(frame, usermode)
 	struct trapframe *frame;
 	int usermode;
@@ -476,11 +506,12 @@ trap_pfault(frame, usermode)
 
 	if (va < VM_MIN_KERNEL_ADDRESS) {
 		vm_offset_t v;
-		vm_page_t ptepg;
+		vm_page_t mpte;
 
 		if (p == NULL ||
 		    (!usermode && va < VM_MAXUSER_ADDRESS &&
-		    (curpcb == NULL || curpcb->pcb_onfault == NULL))) {
+		     (intr_nesting_level != 0 || curpcb == NULL ||
+		      curpcb->pcb_onfault == NULL))) {
 			trap_fatal(frame);
 			return (-1);
 		}
@@ -514,22 +545,9 @@ trap_pfault(frame, usermode)
 			}
 		}
 
-		/*
-		 * Check if page table is mapped, if not,
-		 *	fault it first
-		 */
-		v = (vm_offset_t) vtopte(va);
-
-		/* Fault the pte only if needed: */
-		if (*((int *)vtopte(v)) == 0)
-			(void) vm_fault(map, trunc_page(v), VM_PROT_WRITE, FALSE);
-
-		pmap_use_pt( vm_map_pmap(map), va);
-
 		/* Fault in the user page: */
-		rv = vm_fault(map, va, ftype, FALSE);
-
-		pmap_unuse_pt( vm_map_pmap(map), va);
+		rv = vm_fault(map, va, ftype,
+			(ftype & VM_PROT_WRITE) ? VM_FAULT_DIRTY : 0);
 
 		--p->p_lock;
 	} else {
@@ -551,7 +569,7 @@ trap_pfault(frame, usermode)
 		return (0);
 nogo:
 	if (!usermode) {
-		if (curpcb && curpcb->pcb_onfault) {
+		if (intr_nesting_level == 0 && curpcb && curpcb->pcb_onfault) {
 			frame->tf_eip = (int)curpcb->pcb_onfault;
 			return (0);
 		}
@@ -585,7 +603,18 @@ trap_pfault(frame, usermode)
 	if (va >= KERNBASE) {
 		/*
 		 * Don't allow user-mode faults in kernel address space.
+		 * An exception:  if the faulting address is the invalid
+		 * instruction entry in the IDT, then the Intel Pentium
+		 * F00F bug workaround was triggered, and we need to
+		 * treat it is as an illegal instruction, and not a page
+		 * fault.
 		 */
+#if defined(I586_CPU) && !defined(NO_F00F_HACK)
+		if ((eva == (unsigned int)&t_idt[6]) && has_f00f_bug) {
+			frame->tf_trapno = T_PRIVINFLT;
+			return -2;
+		}
+#endif
 		if (usermode)
 			goto nogo;
 
@@ -611,9 +640,6 @@ trap_pfault(frame, usermode)
 		ftype = VM_PROT_READ;
 
 	if (map != kernel_map) {
-		vm_offset_t v;
-		vm_page_t ptepg;
-
 		/*
 		 * Keep swapout from messing with us during this
 		 *	critical time.
@@ -632,29 +658,14 @@ trap_pfault(frame, usermode)
 			}
 		}
 
-		/*
-		 * Check if page table is mapped, if not,
-		 *	fault it first
-		 */
-		v = (vm_offset_t) vtopte(va);
-
-		/* Fault the pte only if needed: */
-		if (*((int *)vtopte(v)) == 0)
-			(void) vm_fault(map, trunc_page(v), VM_PROT_WRITE, FALSE);
-
-		pmap_use_pt( vm_map_pmap(map), va);
-
 		/* Fault in the user page: */
-		rv = vm_fault(map, va, ftype, FALSE);
-
-		pmap_unuse_pt( vm_map_pmap(map), va);
+		rv = vm_fault(map, va, ftype,
+			(ftype & VM_PROT_WRITE) ? VM_FAULT_DIRTY : 0);
 
 		--p->p_lock;
 	} else {
 		/*
-		 * Since we know that kernel virtual address addresses
-		 * always have pte pages mapped, we just have to fault
-		 * the page.
+		 * Don't have to worry about process locking or stacks in the kernel.
 		 */
 		rv = vm_fault(map, va, ftype, FALSE);
 	}
@@ -663,7 +674,7 @@ trap_pfault(frame, usermode)
 		return (0);
 nogo:
 	if (!usermode) {
-		if (curpcb && curpcb->pcb_onfault) {
+		if (intr_nesting_level == 0 && curpcb && curpcb->pcb_onfault) {
 			frame->tf_eip = (int)curpcb->pcb_onfault;
 			return (0);
 		}
@@ -677,11 +688,11 @@ nogo:
 	return((rv == KERN_PROTECTION_FAILURE) ? SIGBUS : SIGSEGV);
 }
 
-void
+static void
 trap_fatal(frame)
 	struct trapframe *frame;
 {
-	int code, type, eva;
+	int code, type, eva, ss, esp;
 	struct soft_segment_descriptor softseg;
 
 	code = frame->tf_err;
@@ -700,14 +711,25 @@ trap_fatal(frame)
 			code & PGEX_W ? "write" : "read",
 			code & PGEX_P ? "protection violation" : "page not present");
 	}
-	printf("instruction pointer	= 0x%x:0x%x\n", frame->tf_cs & 0xffff, frame->tf_eip);
+	printf("instruction pointer	= 0x%x:0x%x\n",
+	       frame->tf_cs & 0xffff, frame->tf_eip);
+	if (ISPL(frame->tf_cs) == SEL_UPL) {
+		ss = frame->tf_ss & 0xffff;
+		esp = frame->tf_esp;
+	} else {
+		ss = GSEL(GDATA_SEL, SEL_KPL);
+		esp = (int)&frame->tf_esp;
+	}
+	printf("stack pointer	        = 0x%x:0x%x\n", ss, esp);
+	printf("frame pointer	        = 0x%x:0x%x\n", ss, frame->tf_ebp);
 	printf("code segment		= base 0x%x, limit 0x%x, type 0x%x\n",
-	    softseg.ssd_base, softseg.ssd_limit, softseg.ssd_type);
+	       softseg.ssd_base, softseg.ssd_limit, softseg.ssd_type);
 	printf("			= DPL %d, pres %d, def32 %d, gran %d\n",
-	    softseg.ssd_dpl, softseg.ssd_p, softseg.ssd_def32, softseg.ssd_gran);
+	       softseg.ssd_dpl, softseg.ssd_p, softseg.ssd_def32,
+	       softseg.ssd_gran);
 	printf("processor eflags	= ");
 	if (frame->tf_eflags & PSL_T)
-		printf("trace/trap, ");
+		printf("trace trap, ");
 	if (frame->tf_eflags & PSL_I)
 		printf("interrupt enabled, ");
 	if (frame->tf_eflags & PSL_NT)
@@ -732,6 +754,8 @@ trap_fatal(frame)
 		printf("tty ");
 	if ((cpl & bio_imask) == bio_imask)
 		printf("bio ");
+	if ((cpl & cam_imask) == cam_imask)
+		printf("cam ");
 	if (cpl == 0)
 		printf("none");
 	printf("\n");
@@ -787,7 +811,7 @@ int trapwrite(addr)
 	unsigned addr;
 {
 	struct proc *p;
-	vm_offset_t va, v;
+	vm_offset_t va;
 	struct vmspace *vm;
 	int rv;
 
@@ -811,26 +835,10 @@ int trapwrite(addr)
 		}
 	}
 
-	v = trunc_page(vtopte(va));
-
-	/*
-	 * wire the pte page
-	 */
-	if (va < USRSTACK) {
-		vm_map_pageable(&vm->vm_map, v, round_page(v+1), FALSE);
-	}
-
 	/*
 	 * fault the data page
 	 */
-	rv = vm_fault(&vm->vm_map, va, VM_PROT_READ|VM_PROT_WRITE, FALSE);
-
-	/*
-	 * unwire the pte page
-	 */
-	if (va < USRSTACK) {
-		vm_map_pageable(&vm->vm_map, v, round_page(v+1), TRUE);
-	}
+	rv = vm_fault(&vm->vm_map, va, VM_PROT_READ|VM_PROT_WRITE, VM_FAULT_DIRTY);
 
 	--p->p_lock;
 
@@ -864,22 +872,26 @@ syscall(frame)
 	p->p_md.md_regs = (int *)&frame;
 	params = (caddr_t)frame.tf_esp + sizeof(int);
 	code = frame.tf_eax;
-	/*
-	 * Need to check if this is a 32 bit or 64 bit syscall.
-	 */
-	if (code == SYS_syscall) {
+	if (p->p_sysent->sv_prepsyscall) {
+		(*p->p_sysent->sv_prepsyscall)(&frame, args, &code, &params);
+	} else {
 		/*
-		 * Code is first argument, followed by actual args.
+		 * Need to check if this is a 32 bit or 64 bit syscall.
 		 */
-		code = fuword(params);
-		params += sizeof(int);
-	} else if (code == SYS___syscall) {
-		/*
-		 * Like syscall, but code is a quad, so as to maintain
-		 * quad alignment for the rest of the arguments.
-		 */
-		code = fuword(params);
-		params += sizeof(quad_t);
+		if (code == SYS_syscall) {
+			/*
+			 * Code is first argument, followed by actual args.
+			 */
+			code = fuword(params);
+			params += sizeof(int);
+		} else if (code == SYS___syscall) {
+			/*
+			 * Like syscall, but code is a quad, so as to maintain
+			 * quad alignment for the rest of the arguments.
+			 */
+			code = fuword(params);
+			params += sizeof(quad_t);
+		}
 	}
 
  	if (p->p_sysent->sv_mask)
@@ -890,7 +902,7 @@ syscall(frame)
   	else
  		callp = &p->p_sysent->sv_table[code];
 
-	if ((i = callp->sy_narg * sizeof(int)) &&
+	if (params && (i = callp->sy_narg * sizeof(int)) &&
 	    (error = copyin(params, (caddr_t)args, (u_int)i))) {
 #ifdef KTRACE
 		if (KTRPOINT(p, KTR_SYSCALL))
@@ -922,9 +934,10 @@ syscall(frame)
 
 	case ERESTART:
 		/*
-		 * Reconstruct pc, assuming lcall $X,y is 7 bytes.
+		 * Reconstruct pc, assuming lcall $X,y is 7 bytes,
+		 * int 0x80 is 2 bytes. We saved this in tf_err.
 		 */
-		frame.tf_eip -= 7;
+		frame.tf_eip -= frame.tf_err;
 		break;
 
 	case EJUSTRETURN:
@@ -955,99 +968,3 @@ bad:
 		ktrsysret(p->p_tracep, code, error, rval[0]);
 #endif
 }
-
-#ifdef COMPAT_LINUX
-void
-linux_syscall(frame)
-	struct trapframe frame;
-{
-	int i;
-	struct proc *p = curproc;
-	struct sysent *callp;
-	u_quad_t sticks;
-	int error;
-	int rval[2];
-	u_int code;
-	struct linux_syscall_args {
-		int arg1;
-		int arg2;
-		int arg3;
-		int arg4;
-		int arg5;
-	} args;
-
-	args.arg1 = frame.tf_ebx;
-	args.arg2 = frame.tf_ecx;
-	args.arg3 = frame.tf_edx;
-	args.arg4 = frame.tf_esi;
-	args.arg5 = frame.tf_edi;
-
-	sticks = p->p_sticks;
-	if (ISPL(frame.tf_cs) != SEL_UPL)
-		panic("linux syscall");
-
-	p->p_md.md_regs = (int *)&frame;
-	code = frame.tf_eax;
-
-	if (p->p_sysent->sv_mask)
-		code &= p->p_sysent->sv_mask;
-
-	if (code >= p->p_sysent->sv_size)
-		callp = &p->p_sysent->sv_table[0];
-	else
-		callp = &p->p_sysent->sv_table[code];
-
-#ifdef KTRACE
-	if (KTRPOINT(p, KTR_SYSCALL))
-		ktrsyscall(p->p_tracep, code, callp->sy_narg, (int *)&args);
-#endif
-
-	rval[0] = 0;
-
-	error = (*callp->sy_call)(p, &args, rval);
-
-	switch (error) {
-
-	case 0:
-		/*
-		 * Reinitialize proc pointer `p' as it may be different
-		 * if this is a child returning from fork syscall.
-		 */
-		p = curproc;
-		frame.tf_eax = rval[0];
-		frame.tf_eflags &= ~PSL_C;
-		break;
-
-	case ERESTART:
-		/* Reconstruct pc, subtract size of int 0x80 */
-		frame.tf_eip -= 2;
-		break;
-
-	case EJUSTRETURN:
-		break;
-
-	default:
- 		if (p->p_sysent->sv_errsize)
- 			if (error >= p->p_sysent->sv_errsize)
-  				error = -1;	/* XXX */
-   			else
-  				error = p->p_sysent->sv_errtbl[error];
-		frame.tf_eax = -error;
-		frame.tf_eflags |= PSL_C;
-		break;
-	}
-
-	if (frame.tf_eflags & PSL_T) {
-		/* Traced syscall. */
-		frame.tf_eflags &= ~PSL_T;
-		trapsignal(p, SIGTRAP, 0);
-	}
-
-	userret(p, &frame, sticks);
-
-#ifdef KTRACE
-	if (KTRPOINT(p, KTR_SYSRET))
-		ktrsysret(p->p_tracep, code, error, rval[0]);
-#endif
-}
-#endif /* COMPAT_LINUX */
