@@ -46,7 +46,7 @@
  ** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  ** THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  **
- ** $FreeBSD: stable/2.1/sys/i386/i386/userconfig.c 50950 1999-09-05 08:39:22Z peter $
+ ** $FreeBSD: stable/2.2/sys/i386/i386/userconfig.c 50950 1999-09-05 08:39:22Z peter $
  **/
 
 /**
@@ -99,22 +99,27 @@
  ** - Implement page up/down (as what?)
  ** - Wizard mode (no restrictions)
  ** - Find out how to put syscons back into low-intensity mode so that the
- **   !b escape is useful on the console.
+ **   !b escape is useful on the console.  (It seems to be that it actually
+ **   gets low/high intensity backwards. That looks OK.)
  **
  ** - Only display headings with devices under them. (difficult)
  **/
 
+#include "opt_userconfig.h"
 #include "pci.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
 #include <sys/malloc.h>
+#include <sys/reboot.h>
 
 #include <machine/cons.h>
 #include <machine/md_var.h>
 
 #include <i386/isa/isa_device.h>
+#include <i386/isa/pnp.h> /* XXX */
+#include "pnp.h"     
 
 #include <pci/pcivar.h>
 
@@ -122,6 +127,7 @@ static struct isa_device *isa_devlist;	/* list read by dset to extract changes *
 
 #ifdef USERCONFIG_BOOT
 char userconfig_from_boot[512] = "";
+static int userconfig_boot_parsing;	/* set if we are reading from the boot instructions */
 
 static int
 getchar(void)
@@ -137,8 +143,10 @@ getchar(void)
 	}
     } 
     if (*next) {
+	userconfig_boot_parsing = 1;
 	return (*next++);
     } else {
+	userconfig_boot_parsing = 0;
 	return cngetc();
     }
 }
@@ -157,10 +165,6 @@ getchar(void)
 static struct isa_device *devtabs[] = { isa_devtab_bio, isa_devtab_tty, isa_devtab_net,
 				     isa_devtab_null, NULL };
 
-
-
-
-
 typedef struct
 {
     char	dev[16];		/* device basename */
@@ -221,6 +225,7 @@ static DEVCLASS_INFO devclass_names[] = {
 
 static DEV_INFO device_info[] = {
 /*---Name-----   ---Description---------------------------------------------- */
+{"dpt",         "DPT RAID, Caching SCSI HBA", 0, CLS_STORAGE},
 {"bt",          "Buslogic SCSI controller",		0,		CLS_STORAGE},
 {"ahc",         "Adaptec 274x/284x/294x SCSI controller",	0,	CLS_STORAGE},
 {"ahb",         "Adaptec 174x SCSI controller",		0,		CLS_STORAGE},
@@ -230,17 +235,20 @@ static DEV_INFO device_info[] = {
 {"nca",         "ProAudio Spectrum SCSI and compatibles",	0,	CLS_STORAGE},
 {"sea",         "Seagate ST01/ST02 SCSI and compatibles",	0,	CLS_STORAGE},
 {"wds",         "Western Digitial WD7000 SCSI controller",	0,	CLS_STORAGE},
-{"ncr",         "NCR 53C810 SCSI controller",		FLG_FIXED,	CLS_STORAGE},
+{"ncr",         "NCR/Symbios 53C810/15/25/60/75 SCSI controller",FLG_FIXED,CLS_STORAGE},
 {"wdc",         "IDE/ESDI/MFM disk controller",		0,		CLS_STORAGE},
 {"fdc",         "Floppy disk controller",		FLG_FIXED,	CLS_STORAGE},
 {"mcd",         "Mitsumi CD-ROM",			0,		CLS_STORAGE},
 {"scd",         "Sony CD-ROM",				0,		CLS_STORAGE},
 {"matcdc",       "Matsushita/Panasonic/Creative CDROM",	0,		CLS_STORAGE},
 {"wt",          "Wangtek/Archive QIC-02 Tape drive",	0,		CLS_STORAGE},
+{"amd",		"Tekram DC-390(T) / AMD 53c974 based PCI SCSI",	FLG_FIXED, CLS_STORAGE},
 
+{"cs",          "IBM EtherJet, CS89x0-based Ethernet adapters",0,	CLS_NETWORK},
 {"ed",          "NE1000,NE2000,3C503,WD/SMC80xx Ethernet adapters",0,	CLS_NETWORK},
 {"el",          "3C501 Ethernet adapter",		0,		CLS_NETWORK},
 {"ep",          "3C509 Ethernet adapter",		0,		CLS_NETWORK},
+{"ex",          "Intel EtherExpress Pro/10 Ethernet adapter",	0,	CLS_NETWORK},
 {"fe",          "Fujitsu MD86960A/MB869685A Ethernet adapters",	0,	CLS_NETWORK},
 {"fea",         "DEC DEFEA EISA FDDI adapter",		0,		CLS_NETWORK},
 {"fxp",         "Intel EtherExpress Pro/100B Ethernet adapter",	0,	CLS_NETWORK},
@@ -248,7 +256,10 @@ static DEV_INFO device_info[] = {
 {"ix",          "Intel EtherExpress Ethernet adapter",	0,		CLS_NETWORK},
 {"le",          "DEC Etherworks 2 and 3 Ethernet adapters",	0,	CLS_NETWORK},
 {"lnc",         "Isolan, Novell NE2100/NE32-VL Ethernet adapters",	0,CLS_NETWORK},
+{"tl",          "TI ThunderLAN PCI Ethernet adapter",		0,	CLS_NETWORK},
+{"tx",          "SMC 9432TX Ethernet adapter",			0,	CLS_NETWORK},
 {"vx",          "3COM 3C590/3C595 Ethernet adapters",		0,	CLS_NETWORK},
+{"xl",          "3COM 3C90x/3C90xB/3C980 Ethernet adapters",	0,	CLS_NETWORK},
 {"ze",          "IBM/National Semiconductor PCMCIA Ethernet adapter",0,	CLS_NETWORK},
 {"zp",          "3COM PCMCIA Etherlink III Ethernet adapter",	0,	CLS_NETWORK},
 {"de",          "DEC DC21040 Ethernet adapter",		FLG_FIXED,	CLS_NETWORK},
@@ -272,6 +283,8 @@ static DEV_INFO device_info[] = {
 {"vt",          "PCVT console driver",			FLG_FIXED,	CLS_INPUT},
 {"sc",          "Syscons console driver",		FLG_FIXED,	CLS_INPUT},
 
+{"bktr",        "Brooktree BT848 based frame grabber/tuner card",	0,CLS_MMEDIA},
+{"pcm",         "New Luigi audio driver for all supported sound cards",	0,CLS_MMEDIA},
 {"sb",          "Soundblaster PCM (SB, SBPro, SB16, ProAudio Spectrum)",0,CLS_MMEDIA},
 {"sbxvi",       "Soundblaster 16",			0,		CLS_MMEDIA},
 {"sbmidi",      "Soundblaster MIDI interface",		0,		CLS_MMEDIA},
@@ -338,6 +351,8 @@ typedef struct _devlist_struct
 #define KEY_ZOOM	10	/* these for zoom all/collapse all */
 #define KEY_UNZOOM	11
 
+#define KEY_HELP	12	/* duh? */
+
 static void redraw(void);
 static void insdev(DEV_LIST *dev, DEV_LIST *list);
 static int  devinfo(DEV_LIST *dev);
@@ -544,6 +559,8 @@ findspot(DEV_LIST *dev, DEV_LIST *list)
 	{
 	    if (ap->comment != DEV_DEVICE)			/* ignore comments */
 		continue;
+	    if (ap->iobase == -2)				/* don't group with a PCI device */
+		continue;
 	    if (!strcmp(dev->dev,ap->dev))			/* same base device */
 	    {
 		if ((dev->unit <= ap->unit)			/* belongs before (equal is bad) */
@@ -944,12 +961,12 @@ collapselist(DEV_LIST *list)
  ** On a device in the active list
  **
  ** [Enter] Edit device parameters  [DEL] Disable device
- ** [TAB]   Change fields           [Q]   Save and Exit
+ ** [TAB]   Change fields           [Q]   Save and Exit             [?] Help
  **
  ** On a device in the inactive list
  **
  ** [Enter] Enable device
- ** [TAB]   Change fields           [Q]   Save and Exit
+ ** [TAB]   Change fields           [Q]   Save and Exit             [?] Help
  **
  ** While editing parameters
  **
@@ -1289,7 +1306,7 @@ drawlist(int row, int num, int detail, DEV_LIST *list)
     {
 	if (list)
 	{
-	    drawline(row+ofs,detail,list,0,"");
+	    drawline(row+ofs,detail,list,0,NULL);	/* NULL -> don't draw empty help string */
 	    list = nextent(list);			/* move down visible list */
 	}else{
 	    erase(0,row+ofs,80,1);
@@ -1310,7 +1327,7 @@ redrawactive(void)
 
     if (conflicts)
     {
-	sprintf(cbuf,"!i%d conflict%s",conflicts,(conflicts>1)?"s":"");
+	sprintf(cbuf,"!i%d conflict%s-",conflicts,(conflicts>1)?"s":"");
 	putxy(45,0,cbuf);
     }else{
 	putxyl(45,0,lines,16);
@@ -1347,7 +1364,7 @@ redraw(void)
     putxy(63,9,"!bDev");
     putxy(0,17,lines);
     putxy(0,21,lines);
-    masterhelp("  [!bTAB!n]   Change fields           [!bQ!n]   Save and Exit");
+    masterhelp("  [!bTAB!n]   Change fields           [!bQ!n]   Save and Exit             [!b?!n] Help");
 
     redrawactive();
     redrawinactive();
@@ -1416,7 +1433,8 @@ showparams(DEV_LIST *dev)
 	putxy(1,18,buf);
     } else {
 	if (dev->iobase == -2)			/* a PCI device */
-	    putmsg(" PCI devices are automatically configured.");
+	    putmsg(" PCI devices are displayed for informational purposes only, and\n"
+		   " cannot be disabled or configured here.");
     }
 	    
     if (dev->irq > 0)
@@ -1673,8 +1691,8 @@ editparams(DEV_LIST *dev)
     ep_iobase:
 	if (dev->iobase > 0)
 	{
-	    puthelp("  IO Port address (Hexadecimal, 0x1-0x2000)");
-	    ret = editval(18,18,5,1,0x1,0x2000,&(dev->iobase),(dev->attrib & FLG_FIXIOBASE));
+	    puthelp("  IO Port address (Hexadecimal, 0x1-0xffff)");
+	    ret = editval(18,18,5,1,0x1,0xffff,&(dev->iobase),(dev->attrib & FLG_FIXIOBASE));
 	    switch(ret)
 	    {
 	    case KEY_EXIT:
@@ -1840,6 +1858,126 @@ editparams(DEV_LIST *dev)
     dev->changed = 1;					/* mark as changed */
 }
 
+static char *helptext[] =
+{
+    "                Using the UserConfig kernel settings editor",
+    "                -------------------------------------------",
+    "",
+    "VISUAL MODE:",
+    "",
+    "- - Layout -",
+    "",
+    "The screen displays a list of available drivers, divided into two",
+    "scrolling lists: Active Drivers, and Inactive Drivers.  Each list is",
+    "by default collapsed and can be expanded to show all the drivers",
+    "available in each category.  The parameters for the currently selected",
+    "driver are shown at the bottom of the screen.",
+    "",
+    "- - Moving around -",
+    "",
+    "To move in the current list, use the UP and DOWN cursor keys to select",
+    "an item (the selected item will be highlighted).  If the item is a",
+    "category name, you may alternatively expand or collapse the list of",
+    "drivers for that category by pressing [!bENTER!n].  Once the category is",
+    "expanded, you can select each driver in the same manner and either:",
+    "",
+    "  - change its parameters using [!bENTER!n]",
+    "  - move it to the Inactive list using [!bDEL!n]",
+    "",
+    "Use the [!bTAB!n] key to toggle between the Active and Inactive list; if",
+    "you need to move a driver from the Inactive list back to the Active",
+    "one, select it in the Inactive list, using [!bTAB!n] to change lists if",
+    "necessary, and press [!bENTER!n] -- the device will be moved back to",
+    "its place in the Active list.",
+    "",
+    "- - Altering the list/parameters -",
+    "",
+    "Any drivers for devices not installed in your system should be moved",
+    "to the Inactive list, until there are no remaining parameter conflicts",
+    "between the drivers, as indicated at the top.",
+    "",
+    "Once the list of Active drivers only contains entries for the devices",
+    "present in your system, you can set their parameters (Interrupt, DMA",
+    "channel, I/O addresses).  To do this, select the driver and press",
+    "[!bENTER!n]: it is now possible to edit the settings the settings at the",
+    "bottom of the screen.  Use [!bTAB!n] to change fields, and when you are",
+    "finished, use [!bQ!n] to return to the list.",
+    "",
+    "- - Saving changes -",
+    "",
+    "When all settings seem correct, and you wish to proceed with the",
+    "kernel device probing and boot, press [!bQ!n] -- you will be asked to",
+    "confirm your choice.",
+    "",
+    NULL
+};
+
+
+/**
+ ** helpscreen
+ **
+ ** Displays help text onscreen for people that are confused, using a simple
+ ** pager.
+ **/
+static void
+helpscreen(void) 
+{
+    int		topline = 0;			/* where we are in the text */
+    int		line, c, delta = 1;
+    char	prompt[80];
+
+    for (;;)					/* loop until user quits */
+    {
+	/* display help text */
+	if (delta) 
+	{
+	    clear();					/* remove everything else */
+	    for (line = topline; 
+		 (line < (topline + 24)) && (helptext[line]); 
+		 line++)
+		putxy(0,line-topline,helptext[line]);
+	    delta = 0;
+	}
+	
+	/* prompt */
+	sprintf(prompt,"!i --%s-- [U]p [D]own [Q]uit !n",helptext[line] ? "MORE" : "END");
+	putxy(0,24,prompt);
+	
+	c = getchar();				/* so what do they say? */
+	
+	switch (c)
+	{
+	case 'u':
+	case 'U':
+	case 'b':
+	case 'B':				/* wired into 'more' users' fingers */
+	    if (topline > 0)			/* room to go up? */
+	    {
+		topline -= 24;
+		if (topline < 0)		/* don't go too far */
+		    topline = 0;
+		delta = 1;
+	    }
+	    break;
+
+	case 'd':
+	case 'D':
+	case ' ':				/* expected by most people */
+	    if (helptext[line]) 		/* maybe more below? */
+	    {
+		topline += 24;
+		delta = 1;
+	    }
+	    break;
+	    
+	case 'q':
+	case 'Q':
+	    redraw();				/* restore the screen */
+	    return;
+	}
+    }
+}
+
 
 /** 
  ** High-level control functions
@@ -1957,6 +2095,10 @@ dolist(int row, int num, int detail, int *ofs, DEV_LIST **list, char *dhelp)
 		
 	    case '\014':			/* ^L, redraw */
 		return(KEY_REDRAW);
+		
+	    case '?':				/* helptext */
+		return(KEY_HELP);
+		
 	    }
 	}
     }		
@@ -2053,7 +2195,7 @@ visuserconfig(void)
 			{
 			    masterhelp("  [!bTAB!n]   Change fields           [!bQ!n]   Save device parameters");
 			    editparams(dp);
-			    masterhelp("  [!bTAB!n]   Change fields           [!bQ!n]   Save and Exit");
+			    masterhelp("  [!bTAB!n]   Change fields           [!bQ!n]   Save and Exit             [!b?!n] Help");
 			    putxy(0,17,lines);
 			    conflicts = findconflict(active);	/* update conflict tags */
 			}
@@ -2155,9 +2297,15 @@ visuserconfig(void)
 	default:
 	    mode = 0;				/* shouldn't happen... */
 	}
-	if (ret == KEY_EXIT)
-	{
-	    i = yesnocancel(" Save these parameters before exiting? (Yes/No/Cancel) ");
+
+	/* handle returns that are the same for both modes */
+	switch (ret) {
+	case KEY_HELP:
+	    helpscreen();
+	    break;
+	    
+	case KEY_EXIT:
+	    i = yesnocancel(" Save these parameters before exiting? ([!bY!n]es/[!bN!n]o/[!bC!n]ancel) ");
 	    switch(i)
 	    {
 	    case 2:				/* cancel */
@@ -2175,6 +2323,7 @@ visuserconfig(void)
 		clear();
 		return(1);
 	    }
+	    break;
 	}
     }
 }
@@ -2222,7 +2371,7 @@ visuserconfig(void)
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/i386/userconfig.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/userconfig.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "scbus.h"
@@ -2232,6 +2381,7 @@ visuserconfig(void)
 #define PARM_DEVSPEC	0x1
 #define PARM_INT	0x2
 #define PARM_ADDR	0x3
+#define PARM_STRING	0x4
 
 typedef struct _cmdparm {
     int type;
@@ -2276,12 +2426,27 @@ static int set_device_enable(CmdParm *);
 static int set_device_disable(CmdParm *);
 static int quitfunc(CmdParm *);
 static int helpfunc(CmdParm *);
-#if defined(USERCONFIG_BOOT) && defined(VISUAL_USERCONFIG)
+#if defined(USERCONFIG_BOOT)
 static int introfunc(CmdParm *);
 #endif
 
+#if NPNP > 0
+static int lspnp(void);
+static int set_pnp_parms(CmdParm *);
+#endif
+
 static int lineno;
 
+#include "eisa.h"
+
+#if NEISA > 0
+
+#include <i386/eisa/eisaconf.h>
+
+static int set_num_eisa_slots(CmdParm *);
+
+#endif /* NEISA > 0 */
+
 static CmdParm addr_parms[] = {
     { PARM_DEVSPEC, {} },
     { PARM_ADDR, {} },
@@ -2299,21 +2464,41 @@ static CmdParm dev_parms[] = {
     { -1, {} },
 };
 
+static CmdParm string_arg[] = {
+    { PARM_STRING, {} },
+    { -1, {} },
+};  
+
+
+
+#if NEISA > 0
+static CmdParm int_arg[] = {
+    { PARM_INT, {} },
+    { -1, {} },
+};
+#endif /* NEISA > 0 */
+
 static Cmd CmdList[] = {
     { "?", 	helpfunc, 		NULL },		/* ? (help)	*/
     { "di",	set_device_disable,	dev_parms },	/* disable dev	*/
     { "dr",	set_device_drq,		int_parms },	/* drq dev #	*/
+#if NEISA > 0
+    { "ei",	set_num_eisa_slots,	int_arg },	/* # EISA slots */
+#endif /* NEISA > 0 */
     { "en",	set_device_enable,	dev_parms },	/* enable dev	*/
     { "ex", 	quitfunc, 		NULL },		/* exit (quit)	*/
     { "f",	set_device_flags,	int_parms },	/* flags dev mask */
     { "h", 	helpfunc, 		NULL },		/* help		*/
-#if defined(USERCONFIG_BOOT) && defined(VISUAL_USERCONFIG)
+#if defined(USERCONFIG_BOOT)
     { "intro", 	introfunc, 		NULL },		/* intro screen	*/
 #endif
     { "iom",	set_device_mem,		addr_parms },	/* iomem dev addr */
     { "ios",	set_device_iosize,	int_parms },	/* iosize dev size */
     { "ir",	set_device_irq,		int_parms },	/* irq dev #	*/
     { "l",	list_devices,		NULL },		/* ls, list	*/
+#if NPNP > 0
+    { "pn",    set_pnp_parms,          string_arg },   /* pnp ... */
+#endif
     { "po",	set_device_ioaddr,	int_parms },	/* port dev addr */
     { "res",	(CmdFunc)cpu_reset,	NULL },		/* reset CPU	*/
     { "q", 	quitfunc, 		NULL },		/* quit		*/
@@ -2441,6 +2626,10 @@ parse_args(char *cmd, CmdParm *parms)
 	    ++parms;
 	    continue;
 	}
+        if (parms->type == PARM_STRING) {
+            parms->parm.aparm = (void *)cmd ;
+            return 0;
+        }
     }
     return 0;
 }
@@ -2453,6 +2642,12 @@ list_devices(CmdParm *parms)
     if (lsdevtab(&isa_devtab_tty[0])) return 0;
     if (lsdevtab(&isa_devtab_net[0])) return 0;
     if (lsdevtab(&isa_devtab_null[0])) return 0;
+#if NPNP > 0
+    if (lspnp()) return 0;
+#endif
+#if NEISA > 0
+    printf("\nNumber of EISA slots to probe: %d\n", num_eisa_slots);
+#endif /* NEISA > 0 */
     return 0;
 }
 
@@ -2538,9 +2733,118 @@ set_device_disable(CmdParm *parms)
     return 0;
 }
 
+#if NPNP > 0 
+/*  
+ * this function sets the kernel table to override bios PnP
+ * configuration.
+ */
+static int
+set_pnp_parms(CmdParm *parms)
+{   
+    u_long idx, val, ldn, csn;
+    int i;     
+    char *q, *p=parms[0].parm.aparm;
+    struct pnp_cinfo d;
+
+    csn=strtoul(p,&q, 0);
+    ldn=strtoul(q,&q, 0);
+    for (p=q; *p && (*p==' ' || *p=='\t'); p++) ;      
+    if (csn < 1 || csn > MAX_PNP_CARDS || ldn >= MAX_PNP_LDN) {
+       printf("bad csn/ldn %d:%d\n", csn, ldn);
+       return 0;
+    }  
+    for (i=0; i < MAX_PNP_LDN; i++) {
+       if (pnp_ldn_overrides[i].csn == csn &&
+           pnp_ldn_overrides[i].ldn == ldn)
+              break;
+    }
+    if (i==MAX_PNP_LDN) {
+       for (i=0; i < MAX_PNP_LDN; i++) {
+           if (pnp_ldn_overrides[i].csn <1 ||
+                pnp_ldn_overrides[i].csn > MAX_PNP_CARDS)
+                break;
+       }
+    }
+    if (i==MAX_PNP_LDN) { 
+       printf("sorry, no PnP entries available, try delete one\n");
+       return 0 ;
+    }
+    d = pnp_ldn_overrides[i] ;
+    d.csn = csn;
+    d.ldn = ldn ;
+    while (*p) {
+       idx = 0;
+       val = 0;
+       if (!strncmp(p,"irq",3)) {
+           idx=strtoul(p+3,&q, 0);
+           val=strtoul(q,&q, 0);
+           if (idx >=0 && idx < 2) d.irq[idx] = val;
+       } else if (!strncmp(p,"flags",5)) {
+           idx=strtoul(p+5,&q, 0);
+           d.flags = idx;
+       } else if (!strncmp(p,"drq",3)) {
+           idx=strtoul(p+3,&q, 0);
+           val=strtoul(q,&q, 0);
+           if (idx >=0 && idx < 2) d.drq[idx] = val;
+       } else if (!strncmp(p,"port",4)) {
+           idx=strtoul(p+4,&q, 0);
+           val=strtoul(q,&q, 0);
+           if (idx >=0 && idx < 8) d.port[idx] = val;
+       } else if (!strncmp(p,"mem",3)) {
+           idx=strtoul(p+3,&q, 0);
+           val=strtoul(q,&q, 0);
+           if (idx >=0 && idx < 4) d.mem[idx].base = val;
+       } else if (!strncmp(p,"bios",4)) {
+           q = p+ 4;
+           d.override = 0 ;
+       } else if (!strncmp(p,"os",2)) {
+           q = p+2 ;
+           d.override = 1 ;
+       } else if (!strncmp(p,"disable",7)) {
+           q = p+7 ;
+           d.enable = 0 ;
+       } else if (!strncmp(p,"enable",6)) {
+           q = p+6;
+           d.enable = 1 ;
+       } else if (!strncmp(p,"delete",6)) {
+           bzero(&pnp_ldn_overrides[i], sizeof (pnp_ldn_overrides[i]));
+           if (i==0) pnp_ldn_overrides[i].csn = 255;/* not reinit */
+           return 0;
+       } else {
+           printf("unknown command <%s>\n", p);
+           break;
+       }
+       for (p=q; *p && (*p==' ' || *p=='\t'); p++) ;
+    }
+    pnp_ldn_overrides[i] = d ;
+    return 0;
+}
+#endif /* NPNP */
+
+#if NEISA > 0
+static int
+set_num_eisa_slots(CmdParm *parms)
+{
+    int num_slots;
+
+    num_slots = parms[0].parm.iparm;
+    num_eisa_slots = (num_slots <= 16 ? num_slots : 10);
+    return 0;
+}
+#endif /* NEISA > 0 */
+
 static int
 quitfunc(CmdParm *parms)
 {
+#ifdef USERCONFIG_BOOT
+    /*
+     * If kernel config supplied, and we are parsing it, and -c also supplied,
+     * ignore a quit command,  This provides a safety mechanism to allow
+     * recovery from a damaged/buggy kernel config.
+     */
+    if ((boothowto & RB_CONFIG) && userconfig_boot_parsing)
+	return 0;
+#endif
     return 1;
 }
 
@@ -2558,6 +2862,19 @@ helpfunc(CmdParm *parms)
     printf("flags <devname> <mask>\tSet device flags\n");
     printf("enable <devname>\tEnable device\n");
     printf("disable <devname>\tDisable device (will not be probed)\n");
+#if NPNP > 0
+    printf(
+    "pnp <csn> <ldn> [enable|disable]\tenable/disable device\n"
+    "pnp <csn> <ldn> [os|bios]\tset parameters using FreeBSD or BIOS\n"
+    "pnp <csn> <ldn> [portX <addr>]\tset addr for port X (0..7)\n"
+    "pnp <csn> <ldn> [memX <maddr>]\tset addr for memory range X (0..3)\n"
+    "pnp <csn> <ldn> [irqX <number>]\tset irq X (0..1) to number, 0=unused\n"
+    "pnp <csn> <ldn> [drqX <number>]\tset drq X (0..1) to number, 4=unused\n"
+    "pnp <csn> <ldn> [flags <number>]\tset pnp flags (driver-specific)\n");
+#endif
+#if NEISA > 0
+    printf("eisa <number>\t\tSet the number of EISA slots to probe\n");
+#endif /* NEISA > 0 */
     printf("quit\t\t\tExit this configuration utility\n");
     printf("reset\t\t\tReset CPU\n");
 #ifdef VISUAL_USERCONFIG
@@ -2568,22 +2885,25 @@ helpfunc(CmdParm *parms)
     return 0;
 }
 
-#if defined(USERCONFIG_BOOT) && defined(VISUAL_USERCONFIG)
+#if defined(USERCONFIG_BOOT) 
 
+#if defined (VISUAL_USERCONFIG)
 static void
 center(int y, char *str)
 {
     putxy((80 - strlen(str)) / 2, y, str);
 }
+#endif
 
 static int
 introfunc(CmdParm *parms)
 {
-    int curr_item, first_time;
+#if defined (VISUAL_USERCONFIG)
+    int curr_item, first_time, extended = 0;
     static char *choices[] = {
 	" Skip kernel configuration and continue with installation ",
-	" Start kernel configuration in Visual mode                ",
-	" Start kernel configuration in CLI mode (experts only)    ",
+	" Start kernel configuration in full-screen visual mode    ",
+	" Start kernel configuration in CLI mode                   ",
     };
 
     clear();
@@ -2593,32 +2913,34 @@ introfunc(CmdParm *parms)
     first_time = 1;
     while (1) {
 	char tmp[80];
-	int c, i, extended = 0;
+	int c, i;
 
-	for (i = 0; i < 3; i++) {
-	    tmp[0] = '\0';
-	    if (curr_item == i)
-		strcpy(tmp, "!i");
-	    strcat(tmp, choices[i]);
-	    if (curr_item == i)
-		strcat(tmp, "!n");
-	    putxy(10, 5 + i, tmp);
+	if (!extended) { 
+	    for (i = 0; i < 3; i++) {
+		tmp[0] = '\0';
+		if (curr_item == i)
+		    strcpy(tmp, "!i");
+		strcat(tmp, choices[i]);
+		if (curr_item == i)
+		    strcat(tmp, "!n");
+		putxy(10, 5 + i, tmp);
+	    }
+
+	    if (first_time) {
+		putxy(2, 10, "Here you have the chance to go into kernel configuration mode, making");
+		putxy(2, 11, "any changes which may be necessary to properly adjust the kernel to");
+		putxy(2, 12, "match your hardware configuration.");
+		putxy(2, 14, "If you are installing FreeBSD for the first time, select Visual Mode");
+		putxy(2, 15, "(press Down-Arrow then ENTER).");
+		putxy(2, 17, "If you need to do more specialized kernel configuration and are an");
+		putxy(2, 18, "experienced FreeBSD user, select CLI mode.");
+		putxy(2, 20, "If you are !icertain!n that you do not need to configure your kernel");
+		putxy(2, 21, "then simply press ENTER or Q now.");
+		first_time = 0;
+	    }
+	    
+	    move(0, 0);	/* move the cursor out of the way */
 	}
-
-	if (first_time) {
-	    putxy(2, 10, "Here you have the chance to go into kernel configuration mode, making");
-	    putxy(2, 11, "any changes which may be necessary to properly adjust the kernel to");
-	    putxy(2, 12, "match your hardware configuration.");
-	    putxy(2, 14, "If you are installing FreeBSD for the first time, select Visual Mode");
-	    putxy(2, 15, "(press Down-Arrow then ENTER).");
-	    putxy(2, 17, "If you need to do more specialized kernel configuration and are an");
-	    putxy(2, 18, "experienced FreeBSD user, select CLI mode.");
-	    putxy(2, 20, "If you are !icertain!n that you do not need to configure your kernel");
-	    putxy(2, 21, "then simply press ENTER or Q now.");
-	    first_time = 0;
-	}
-
-	move(0, 0);	/* move the cursor out of the way */
 	c = getchar();
 	if ((extended == 2) || (c == 588) || (c == 596)) {	/* console gives "alternative" codes */
 	    extended = 0;		/* no longer */
@@ -2672,9 +2994,72 @@ introfunc(CmdParm *parms)
 	    }
 	}
     }
+#endif
+    return(0);
 }
 #endif
 
+#if NPNP > 0   
+static int     
+lspnp ()       
+{                  
+    struct pnp_cinfo *c;
+    int i, first = 1;
+               
+    for (i=0; i< MAX_PNP_LDN; i++) {
+       c = &pnp_ldn_overrides[i];
+       if (c->csn >0 && c->csn != 255) {
+           int pmax, mmax;
+           static char pfmt[] =
+               "port 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x ";
+           static char mfmt[] =
+               "mem 0x%x 0x%x 0x%x 0x%x";
+           char buf[256];
+           if (lineno >= 23) {
+                   printf("<More> ");
+                   if (getchar() == 'q') {
+                           printf("quit\n");
+                           return (1);
+                   }
+                   printf("\n");
+                   lineno = 0;
+           }
+           if (lineno == 0 || first)
+               printf("CSN LDN conf en irqs  drqs others (PnP devices)\n");
+           first = 0 ;
+           printf("%3d %3d %4s %2s %2d %-2d %2d %-2d ",
+               c->csn, c->ldn,
+               c->override ? "OS  ":"BIOS",
+               c->enable ? "Y":"N",
+               c->irq[0], c->irq[1], c->drq[0], c->drq[1]);
+           if (c->flags)
+               printf("flags 0x%08lx ",c->flags);
+           for (pmax = 7; pmax >=0 ; pmax--)
+               if (c->port[pmax]!=0) break;
+           for (mmax = 3; mmax >=0 ; mmax--)
+               if (c->mem[mmax].base!=0) break;
+           if (pmax>=0) {
+               strcpy(buf, pfmt);
+               buf[10 + 5*pmax]='\0';
+               printf(buf,
+                   c->port[0], c->port[1], c->port[2], c->port[3],
+                   c->port[4], c->port[5], c->port[6], c->port[7]);
+           }
+           if (mmax>=0) {
+               strcpy(buf, mfmt);
+               buf[8 + 5*mmax]='\0';
+               printf(buf,
+                   c->mem[0].base, c->mem[1].base,
+                   c->mem[2].base, c->mem[3].base);
+           }
+           printf("\n");
+       }
+    }
+    return 0 ;
+}
+#endif /* NPNP */
+
+
 static int
 lsdevtab(struct isa_device *dt)
 {
@@ -2908,7 +3293,7 @@ lsscsi(void)
 
     for (i = 0; scsi_cinit[i].driver; i++)
     {
-	id_put("controller scbus", scsi_cinit[i].bus);
+	id_put("controller scbus", scsi_cinit[i].scbus);
 
 	if (scsi_cinit[i].unit != -1)
 	{
@@ -2929,7 +3314,7 @@ lsscsi(void)
 		id_put(" lun ", scsi_dinit[i].lun);
 
 		if (scsi_dinit[i].flags)
-	    	printf("flags 0x%x\n", scsi_dinit[i].flags);
+	    	printf(" flags 0x%x\n", scsi_dinit[i].flags);
 
 		printf("\n");
     }
