@@ -38,59 +38,81 @@
  *
  *	from: @(#)vm_machdep.c	7.3 (Berkeley) 5/13/91
  *	Utah $Hdr: vm_machdep.c 1.16.1.1 89/06/23$
- * $FreeBSD: stable/2.1/sys/i386/i386/vm_machdep.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/i386/vm_machdep.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "npx.h"
+#include "opt_bounce.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/vnode.h>
-#include <sys/user.h>
+#include <sys/vmmeter.h>
 
 #include <machine/clock.h>
-#include <machine/cpu.h>
 #include <machine/md_var.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
 #include <vm/vm_kern.h>
+#include <vm/vm_page.h>
+#include <vm/vm_map.h>
+#include <vm/vm_extern.h>
 
+#include <sys/user.h>
+
+#ifdef PC98
+#include <pc98/pc98/pc98.h>
+#else
 #include <i386/isa/isa.h>
+#endif
 
 #ifdef BOUNCE_BUFFERS
-vm_map_t	io_map;
-volatile int	kvasfreecnt;
+static vm_offset_t
+		vm_bounce_kva __P((int size, int waitok));
+static void	vm_bounce_kva_free __P((vm_offset_t addr, vm_offset_t size,
+					int now));
+static vm_offset_t
+		vm_bounce_page_find __P((int count));
+static void	vm_bounce_page_free __P((vm_offset_t pa, int count));
 
+static volatile int	kvasfreecnt;
 
 caddr_t		bouncememory;
-int		bouncepages, bpwait;
-vm_offset_t	*bouncepa;
-int		bmwait, bmfreeing;
+int		bouncepages;
+static int	bpwait;
+static vm_offset_t	*bouncepa;
+static int		bmwait, bmfreeing;
 
 #define BITS_IN_UNSIGNED (8*sizeof(unsigned))
-int		bounceallocarraysize;
-unsigned	*bounceallocarray;
-int		bouncefree;
+static int		bounceallocarraysize;
+static unsigned	*bounceallocarray;
+static int		bouncefree;
 
+#if defined(PC98) && defined (EPSON_BOUNCEDMA)
+#define SIXTEENMEG (3840*4096)			/* 15MB boundary */
+#else
 #define SIXTEENMEG (4096*4096)
+#endif
 #define MAXBKVA 1024
-int		maxbkva = MAXBKVA*NBPG;
+int		maxbkva = MAXBKVA*PAGE_SIZE;
 
 /* special list that can be used at interrupt time for eventual kva free */
-struct kvasfree {
+static struct kvasfree {
 	vm_offset_t addr;
 	vm_offset_t size;
 } kvaf[MAXBKVA];
 
-
-vm_offset_t vm_bounce_kva();
 /*
  * get bounce buffer pages (count physically contiguous)
  * (only 1 inplemented now)
  */
-vm_offset_t
+static vm_offset_t
 vm_bounce_page_find(count)
 	int count;
 {
@@ -118,7 +140,7 @@ retry:
 	goto retry;
 }
 
-void
+static void
 vm_bounce_kva_free(addr, size, now)
 	vm_offset_t addr;
 	vm_offset_t size;
@@ -148,7 +170,7 @@ vm_bounce_kva_free(addr, size, now)
 /*
  * free count bounce buffer pages
  */
-void
+static void
 vm_bounce_page_free(pa, count)
 	vm_offset_t pa;
 	int count;
@@ -183,7 +205,7 @@ vm_bounce_page_free(pa, count)
 /*
  * allocate count bounce buffer kva pages
  */
-vm_offset_t
+static vm_offset_t
 vm_bounce_kva(size, waitok)
 	int size;
 	int waitok;
@@ -196,7 +218,7 @@ more:
 	if (!bmfreeing && kvasfreecnt) {
 		bmfreeing = 1;
 		for (i = 0; i < kvasfreecnt; i++) {
-			for(off=0;off<kvaf[i].size;off+=NBPG) {
+			for(off=0;off<kvaf[i].size;off+=PAGE_SIZE) {
 				pmap_kremove( kvaf[i].addr + off);
 			}
 			kmem_free_wakeup(io_map, kvaf[i].addr,
@@ -212,13 +234,13 @@ more:
 
 	if( size == 0) {
 		splx(s);
-		return NULL;
+		return 0;
 	}
 
 	if ((kva = kmem_alloc_pageable(io_map, size)) == 0) {
 		if( !waitok) {
 			splx(s);
-			return NULL;
+			return 0;
 		}
 		bmwait = 1;
 		tsleep((caddr_t) io_map, PRIBIO, "bmwait", 0);
@@ -239,13 +261,13 @@ int count;
 	vm_offset_t kva;
 	vm_offset_t pa;
 	if( bouncepages == 0) {
-		kva = (vm_offset_t) malloc(count*NBPG, M_TEMP, M_WAITOK);
+		kva = (vm_offset_t) malloc(count*PAGE_SIZE, M_TEMP, M_WAITOK);
 		return kva;
 	}
-	kva = vm_bounce_kva(count*NBPG, 1);
+	kva = vm_bounce_kva(count*PAGE_SIZE, 1);
 	for(i=0;i<count;i++) {
 		pa = vm_bounce_page_find(1);
-		pmap_kenter(kva + i * NBPG, pa);
+		pmap_kenter(kva + i * PAGE_SIZE, pa);
 	}
 	return kva;
 }
@@ -265,10 +287,10 @@ vm_bounce_kva_alloc_free(kva, count)
 		return;
 	}
 	for(i = 0; i < count; i++) {
-		pa = pmap_kextract(kva + i * NBPG);
+		pa = pmap_kextract(kva + i * PAGE_SIZE);
 		vm_bounce_page_free(pa, 1);
 	}
-	vm_bounce_kva_free(kva, count*NBPG, 0);
+	vm_bounce_kva_free(kva, count*PAGE_SIZE, 0);
 }
 
 /*
@@ -313,9 +335,9 @@ vm_bounce_alloc(bp)
 	vastart = (vm_offset_t) bp->b_data;
 	vaend = (vm_offset_t) bp->b_data + bp->b_bufsize;
 
-	vapstart = i386_trunc_page(vastart);
-	vapend = i386_round_page(vaend);
-	countvmpg = (vapend - vapstart) / NBPG;
+	vapstart = trunc_page(vastart);
+	vapend = round_page(vaend);
+	countvmpg = (vapend - vapstart) / PAGE_SIZE;
 
 /*
  * if any page is above 16MB, then go into bounce-buffer mode
@@ -327,7 +349,7 @@ vm_bounce_alloc(bp)
 			++dobounceflag;
 		if( pa == 0)
 			panic("vm_bounce_alloc: Unmapped page");
-		va += NBPG;
+		va += PAGE_SIZE;
 	}
 	if (dobounceflag == 0)
 		return;
@@ -338,7 +360,7 @@ vm_bounce_alloc(bp)
 /*
  * allocate a replacement kva for b_addr
  */
-	kva = vm_bounce_kva(countvmpg*NBPG, 1);
+	kva = vm_bounce_kva(countvmpg*PAGE_SIZE, 1);
 #if 0
 	printf("%s: vapstart: %x, vapend: %x, countvmpg: %d, kva: %x ",
 		(bp->b_flags & B_READ) ? "read":"write",
@@ -352,7 +374,7 @@ vm_bounce_alloc(bp)
 			 * allocate a replacement page
 			 */
 			vm_offset_t bpa = vm_bounce_page_find(1);
-			pmap_kenter(kva + (NBPG * i), bpa);
+			pmap_kenter(kva + (PAGE_SIZE * i), bpa);
 #if 0
 			printf("r(%d): (%x,%x,%x) ", i, va, pa, bpa);
 #endif
@@ -360,15 +382,15 @@ vm_bounce_alloc(bp)
 			 * if we are writing, the copy the data into the page
 			 */
 			if ((bp->b_flags & B_READ) == 0) {
-				bcopy((caddr_t) va, (caddr_t) kva + (NBPG * i), NBPG);
+				bcopy((caddr_t) va, (caddr_t) kva + (PAGE_SIZE * i), PAGE_SIZE);
 			}
 		} else {
 			/*
 			 * use original page
 			 */
-			pmap_kenter(kva + (NBPG * i), pa);
+			pmap_kenter(kva + (PAGE_SIZE * i), pa);
 		}
-		va += NBPG;
+		va += PAGE_SIZE;
 	}
 
 /*
@@ -383,7 +405,7 @@ vm_bounce_alloc(bp)
  * put our new kva into the buffer (offset by original offset)
  */
 	bp->b_data = (caddr_t) (((vm_offset_t) kva) |
-				((vm_offset_t) bp->b_savekva & (NBPG - 1)));
+				((vm_offset_t) bp->b_savekva & PAGE_MASK));
 #if 0
 	printf("b_savekva: %x, newva: %x\n", bp->b_savekva, bp->b_data);
 #endif
@@ -427,13 +449,13 @@ vm_bounce_free(bp)
 		vm_offset_t mybouncepa;
 		vm_offset_t copycount;
 
-		copycount = i386_round_page(bouncekva + 1) - bouncekva;
-		mybouncepa = pmap_kextract(i386_trunc_page(bouncekva));
+		copycount = round_page(bouncekva + 1) - bouncekva;
+		mybouncepa = pmap_kextract(trunc_page(bouncekva));
 
 /*
  * if this is a bounced pa, then process as one
  */
-		if ( mybouncepa != pmap_kextract( i386_trunc_page( origkva))) {
+		if ( mybouncepa != pmap_kextract( trunc_page( origkva))) {
 			vm_offset_t tocopy = copycount;
 			if (i + tocopy > bp->b_bufsize)
 				tocopy = bp->b_bufsize - i;
@@ -464,11 +486,11 @@ vm_bounce_free(bp)
  * add the old kva into the "to free" list
  */
 
-	bouncekva= i386_trunc_page((vm_offset_t) bp->b_data);
-	bouncekvaend= i386_round_page((vm_offset_t)bp->b_data + bp->b_bufsize);
+	bouncekva= trunc_page((vm_offset_t) bp->b_data);
+	bouncekvaend= round_page((vm_offset_t)bp->b_data + bp->b_bufsize);
 
 /*
-	printf("freeva: %d\n", (bouncekvaend - bouncekva) / NBPG);
+	printf("freeva: %d\n", (bouncekvaend - bouncekva) / PAGE_SIZE);
 */
 	vm_bounce_kva_free( bouncekva, (bouncekvaend - bouncekva), 0);
 	bp->b_data = bp->b_savekva;
@@ -508,7 +530,7 @@ vm_bounce_init()
 
 	for(i=0;i<bouncepages;i++) {
 		vm_offset_t pa;
-		if( (pa = pmap_kextract((vm_offset_t) bouncememory + i * NBPG)) >= SIXTEENMEG)
+		if( (pa = pmap_kextract((vm_offset_t) bouncememory + i * PAGE_SIZE)) >= SIXTEENMEG)
 			panic("bounce memory out of range");
 		if( pa == 0)
 			panic("bounce memory not resident");
@@ -519,22 +541,21 @@ vm_bounce_init()
 
 }
 #endif /* BOUNCE_BUFFERS */
+
 /*
  * quick version of vm_fault
  */
-
 void
-vm_fault_quick( v, prot)
-	vm_offset_t v;
+vm_fault_quick(v, prot)
+	caddr_t v;
 	int prot;
 {
 	if (prot & VM_PROT_WRITE)
-		subyte((char *)v, fubyte((char *)v));
+		subyte(v, fubyte(v));
 	else
-		(void) fubyte((char *)v);
+		fubyte(v);
 }
 
-
 /*
  * Finish a fork operation, with process p2 nearly set up.
  * Copy and update the kernel stack and pcb, making the child
@@ -548,8 +569,12 @@ int
 cpu_fork(p1, p2)
 	register struct proc *p1, *p2;
 {
-	register struct user *up = p2->p_addr;
-	int offset;
+	struct pcb *pcb2 = &p2->p_addr->u_pcb;
+	int sp, offset;
+	volatile int retval;
+#ifdef USER_LDT
+	struct pcb *pcb = &p2->p_addr->u_pcb;
+#endif
 
 	/*
 	 * Copy pcb and stack from proc p1 to p2.
@@ -557,46 +582,71 @@ cpu_fork(p1, p2)
 	 * part of the stack.  The stack and pcb need to agree;
 	 * this is tricky, as the final pcb is constructed by savectx,
 	 * but its frame isn't yet on the stack when the stack is copied.
-	 * swtch compensates for this when the child eventually runs.
 	 * This should be done differently, with a single call
 	 * that copies and updates the pcb+stack,
 	 * replacing the bcopy and savectx.
 	 */
-	p2->p_addr->u_pcb = p1->p_addr->u_pcb;
-	offset = mvesp() - (int)kstack;
+
+	__asm __volatile("movl %%esp,%0" : "=r" (sp));
+	offset = sp - (int)kstack;
+
+	retval = 1;		/* return 1 in child */
 	bcopy((caddr_t)kstack + offset, (caddr_t)p2->p_addr + offset,
 	    (unsigned) ctob(UPAGES) - offset);
 	p2->p_md.md_regs = p1->p_md.md_regs;
 
-	pmap_activate(&p2->p_vmspace->vm_pmap, &up->u_pcb);
+	*pcb2 = p1->p_addr->u_pcb;
+	pcb2->pcb_cr3 = vtophys(p2->p_vmspace->vm_pmap.pm_pdir);
 
-	/*
-	 * Return (0) in parent, (1) in child.
-	 */
-	return (savectx(&up->u_pcb));
+#ifdef USER_LDT
+        /* Copy the LDT, if necessary. */
+        if (pcb->pcb_ldt != 0) {
+                union descriptor *new_ldt;
+                size_t len = pcb->pcb_ldt_len * sizeof(union descriptor);
+
+                new_ldt = (union descriptor *)kmem_alloc(kernel_map, len);
+                bcopy(pcb->pcb_ldt, new_ldt, len);
+                pcb->pcb_ldt = (caddr_t)new_ldt;
+        }
+#endif
+
+	retval = 0;		/* return 0 in parent */
+	savectx(pcb2);
+	return (retval);
 }
 
 void
 cpu_exit(p)
 	register struct proc *p;
 {
+#ifdef USER_LDT
+	struct pcb *pcb;
+#endif
 
 #if NNPX > 0
 	npxexit(p);
 #endif	/* NNPX */
+#ifdef USER_LDT
+	pcb = &p->p_addr->u_pcb; 
+	if (pcb->pcb_ldt != 0) {
+		if (pcb == curpcb)
+			lldt(GSEL(GUSERLDT_SEL, SEL_KPL));
+		kmem_free(kernel_map, (vm_offset_t)pcb->pcb_ldt,
+			pcb->pcb_ldt_len * sizeof(union descriptor));
+		pcb->pcb_ldt_len = (int)pcb->pcb_ldt = 0;
+	}
+#endif
 	cnt.v_swtch++;
 	cpu_switch(p);
 	panic("cpu_exit");
 }
 
 void
-cpu_wait(p) struct proc *p; {
-/*	extern vm_map_t upages_map; */
-
+cpu_wait(p)
+	struct proc *p;
+{
 	/* drop per-process resources */
- 	pmap_remove(vm_map_pmap(u_map), (vm_offset_t) p->p_addr,
-		((vm_offset_t) p->p_addr) + ctob(UPAGES));
-	kmem_free(u_map, (vm_offset_t)p->p_addr, ctob(UPAGES));
+	pmap_dispose_proc(p);
 	vmspace_free(p->p_vmspace);
 }
 
@@ -615,10 +665,8 @@ cpu_coredump(p, vp, cred)
 	    p));
 }
 
-/*
- * Set a red zone in the kernel stack after the u. area.
- */
-void
+#ifdef notyet
+static void
 setredzone(pte, vaddr)
 	u_short *pte;
 	caddr_t vaddr;
@@ -632,35 +680,7 @@ setredzone(pte, vaddr)
    used by sched (that has physical memory mapped 1:1 at bottom)
    and take the dump while still in mapped mode */
 }
-
-/*
- * Move pages from one kernel virtual address to another.
- * Both addresses are assumed to reside in the Sysmap,
- * and size must be a multiple of CLSIZE.
- */
-
-void
-pagemove(from, to, size)
-	register caddr_t from, to;
-	int size;
-{
-	register vm_offset_t pa;
-
-	if (size & CLOFSET)
-		panic("pagemove");
-	while (size > 0) {
-		pa = pmap_kextract((vm_offset_t)from);
-		if (pa == 0)
-			panic("pagemove 2");
-		if (pmap_kextract((vm_offset_t)to) != 0)
-			panic("pagemove 3");
-		pmap_kremove((vm_offset_t)from);
-		pmap_kenter((vm_offset_t)to, pa);
-		from += PAGE_SIZE;
-		to += PAGE_SIZE;
-		size -= PAGE_SIZE;
-	}
-}
+#endif
 
 /*
  * Convert kernel VA to physical address
@@ -744,25 +764,35 @@ vunmapbuf(bp)
  */
 void
 cpu_reset() {
-
+#ifdef PC98
+	/*
+	 * Attempt to do a CPU reset via CPU reset port.
+	 */
+	asm("cli");
+	if ((inb(0x35) & 0xa0) != 0xa0) {
+		outb(0x37, 0x0f);		/* SHUT0 = 0. */
+		outb(0x37, 0x0b);		/* SHUT1 = 0. */
+	}
+	outb(0xf0, 0x00);		/* Reset. */
+#else
 	/*
 	 * Attempt to do a CPU reset via the keyboard controller,
 	 * do not turn of the GateA20, as any machine that fails
 	 * to do the reset here would then end up in no man's land.
 	 */
 
-#ifndef BROKEN_KEYBOARD_RESET
+#if !defined(BROKEN_KEYBOARD_RESET)
 	outb(IO_KBD + 4, 0xFE);
 	DELAY(500000);	/* wait 0.5 sec to see if that did it */
 	printf("Keyboard reset did not work, attempting CPU shutdown\n");
 	DELAY(1000000);	/* wait 1 sec for printf to complete */
 #endif
-
+#endif /* PC98 */
 	/* force a shutdown by unmapping entire address space ! */
-	bzero((caddr_t) PTD, NBPG);
+	bzero((caddr_t) PTD, PAGE_SIZE);
 
 	/* "good night, sweet prince .... <THUNK!>" */
-	pmap_update();
+	invltlb();
 	/* NOTREACHED */
 	while(1);
 }
@@ -811,7 +841,7 @@ grow(p, sp)
 			grow_amount = MAXSSIZ - (vm->vm_ssize << PAGE_SHIFT);
 		}
 		if ((grow_amount == 0) || (vm_map_find(&vm->vm_map, NULL, 0, (vm_offset_t *)&v,
-		    grow_amount, FALSE) != KERN_SUCCESS)) {
+		    grow_amount, FALSE, VM_PROT_ALL, VM_PROT_ALL, 0) != KERN_SUCCESS)) {
 			return (0);
 		}
 		vm->vm_ssize += grow_amount >> PAGE_SHIFT;
@@ -819,3 +849,67 @@ grow(p, sp)
 
 	return (1);
 }
+
+/*
+ * prototype routine to implement the pre-zeroed page mechanism
+ * this routine is called from the idle loop.
+ */
+int
+vm_page_zero_idle() {
+	vm_page_t m;
+	static int free_rover = 0;
+	if ((cnt.v_free_count > cnt.v_interrupt_free_min) &&
+		(m = vm_page_list_find(PQ_FREE, free_rover))) {
+		--(*vm_page_queues[m->queue].lcnt);
+		TAILQ_REMOVE(vm_page_queues[m->queue].pl, m, pageq);
+		m->queue = PQ_NONE;
+		enable_intr();
+		pmap_zero_page(VM_PAGE_TO_PHYS(m));
+		disable_intr();
+		m->queue = PQ_ZERO + m->pc;
+		++(*vm_page_queues[m->queue].lcnt);
+		TAILQ_INSERT_HEAD(vm_page_queues[m->queue].pl, m, pageq);
+		free_rover = (free_rover + PQ_PRIME3) & PQ_L2_MASK;
+		++vm_page_zero_count;
+		return 1;
+	}
+	return 0;
+}
+
+/*
+ * Software interrupt handler for queued VM system processing.
+ */
+void
+swi_vm()
+{
+	if (busdma_swi_pending != 0)
+		busdma_swi();
+}
+
+/*
+ * Tell whether this address is in some physical memory region.
+ * Currently used by the kernel coredump code in order to avoid
+ * dumping the ``ISA memory hole'' which could cause indefinite hangs,
+ * or other unpredictable behaviour.
+ */
+
+#include "isa.h"
+
+int
+is_physical_memory(addr)
+	vm_offset_t addr;
+{
+
+#if NISA > 0
+	/* The ISA ``memory hole''. */
+	if (addr >= 0xa0000 && addr < 0x100000)
+		return 0;
+#endif
+
+	/*
+	 * stuff other tests for known memory-mapped devices (PCI?)
+	 * here
+	 */
+
+	return 1;
+}
