@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/ibcs2/imgact_coff.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/ibcs2/imgact_coff.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -39,11 +39,28 @@
 #include <sys/file.h>
 #include <sys/malloc.h>
 #include <sys/mount.h>
+#include <sys/namei.h>
 #include <sys/sysent.h>
+#include <sys/vnode.h>
+
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 #include <vm/vm_kern.h>
-#include "coff.h"
-#include "ibcs2.h"
+#include <vm/vm_extern.h>
+
+#include <i386/ibcs2/coff.h>
+#include <i386/ibcs2/ibcs2_util.h>
+
+extern struct sysentvec ibcs2_svr3_sysvec;
+
+extern int coff_load_file __P((struct proc *p, char *name));
+extern int exec_coff_imgact __P((struct image_params *imgp));
+
+static int load_coff_section __P((struct vmspace *vmspace, struct vnode *vp, vm_offset_t offset, caddr_t vmaddr, size_t memsz, size_t filsz, vm_prot_t prot));
 
 static int
 load_coff_section(vmspace, vp, offset, vmaddr, memsz, filsz, prot)
@@ -80,18 +97,16 @@ load_coff_section(vmspace, vp, offset, vmaddr, memsz, filsz, prot)
 		map_len = round_page(offset + filsz) - trunc_page(map_offset);
 	}
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d):  vm_mmap(&vmspace->vm_map, &0x%08lx, 0x%x, 0x%x, "
-	"VM_PROT_ALL, MAP_FILE | MAP_PRIVATE | MAP_FIXED, vp, 0x%x)\n",
-	__FILE__, __LINE__, map_addr, map_len, prot, map_offset);
-}
+	DPRINTF(("%s(%d):  vm_mmap(&vmspace->vm_map, &0x%08lx, 0x%x, 0x%x, "
+		"VM_PROT_ALL, MAP_PRIVATE | MAP_FIXED, vp, 0x%x)\n",
+		__FILE__, __LINE__, map_addr, map_len, prot, map_offset));
 
 	if (error = vm_mmap(&vmspace->vm_map,
 			     &map_addr,
 			     map_len,
 			     prot,
 			     VM_PROT_ALL,
-			     MAP_FILE | MAP_PRIVATE | MAP_FIXED,
+			     MAP_PRIVATE | MAP_FIXED,
 			     (caddr_t) vp,
 			     map_offset))
 		return error;
@@ -111,16 +126,13 @@ printf("%s(%d):  vm_mmap(&vmspace->vm_map, &0x%08lx, 0x%x, 0x%x, "
 
 	copy_len = (offset + filsz) - trunc_page(offset + filsz);
 	map_addr = trunc_page(vmaddr + filsz);
-	map_len = round_page(memsz) - trunc_page(filsz);
+	map_len = round_page(vmaddr + memsz) - map_addr;
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): vm_map_find(&vmspace->vm_map, NULL, 0, &0x%08lx, 0x%x, FALSE)\n",
-	__FILE__, __LINE__, map_addr, map_len);
-}
+	DPRINTF(("%s(%d): vm_map_find(&vmspace->vm_map, NULL, 0, &0x%08lx,0x%x, FALSE, VM_PROT_ALL, VM_PROT_ALL, 0)\n", __FILE__, __LINE__, map_addr, map_len));
 
 	if (map_len != 0) {
 		error = vm_map_find(&vmspace->vm_map, NULL, 0, &map_addr,
-				    map_len, FALSE);
+				    map_len, FALSE, VM_PROT_ALL, VM_PROT_ALL, 0);
 		if (error)
 			return error;
 	}
@@ -130,19 +142,19 @@ printf("%s(%d): vm_map_find(&vmspace->vm_map, NULL, 0, &0x%08lx, 0x%x, FALSE)\n"
 			    PAGE_SIZE,
 			    VM_PROT_READ,
 			    VM_PROT_READ,
-			    MAP_FILE,
+			    0,
 			    (caddr_t) vp,
 			    trunc_page(offset + filsz)))
 		return error;
 
-	bcopy(data_buf, (caddr_t) map_addr, copy_len);
+	error = copyout(data_buf, (caddr_t) map_addr, copy_len);
 
 	if (vm_map_remove(kernel_map,
 			  (vm_offset_t) data_buf,
 			  (vm_offset_t) data_buf + PAGE_SIZE))
 		panic("load_coff_section vm_map_remove failed");
 
-	return 0;
+	return error;
 }
 
 int
@@ -151,7 +163,7 @@ coff_load_file(struct proc *p, char *name)
   	struct vmspace *vmspace = p->p_vmspace;
   	int error;
   	struct nameidata nd;
-  	struct vnode *vnodep;
+  	struct vnode *vp;
   	struct vattr attr;
   	struct filehdr *fhdr;
   	struct aouthdr *ahdr;
@@ -170,19 +182,19 @@ coff_load_file(struct proc *p, char *name)
   	if (error)
     		return error;
 
-  	vnodep = nd.ni_vp;
-  	if (vnodep == NULL)
+  	vp = nd.ni_vp;
+  	if (vp == NULL)
     		return ENOEXEC;
 
-  	if (vnodep->v_writecount) {
+  	if (vp->v_writecount) {
     		error = ETXTBSY;
     		goto fail;
   	}
 
-  	if (error = VOP_GETATTR(vnodep, &attr, p->p_ucred, p))
+  	if (error = VOP_GETATTR(vp, &attr, p->p_ucred, p))
     		goto fail;
 
-  	if ((vnodep->v_mount->mnt_flag & MNT_NOEXEC)
+  	if ((vp->v_mount->mnt_flag & MNT_NOEXEC)
 	    || ((attr.va_mode & 0111) == 0)
 	    || (attr.va_type != VREG))
     		goto fail;
@@ -192,25 +204,25 @@ coff_load_file(struct proc *p, char *name)
     		goto fail;
   	}
 
-  	if (error = VOP_ACCESS(vnodep, VEXEC, p->p_ucred, p))
+  	if (error = VOP_ACCESS(vp, VEXEC, p->p_ucred, p))
     		goto fail;
 
-  	if (error = VOP_OPEN(vnodep, FREAD, p->p_ucred, p))
+  	if (error = VOP_OPEN(vp, FREAD, p->p_ucred, p))
     		goto fail;
 
 	/*
 	 * Lose the lock on the vnode. It's no longer needed, and must not
 	 * exist for the pagefault paging to work below.
 	 */
-	VOP_UNLOCK(vnodep);
+	VOP_UNLOCK(vp);
 
   	if (error = vm_mmap(kernel_map,
 			    (vm_offset_t *) &ptr,
 			    PAGE_SIZE,
 			    VM_PROT_READ,
 		       	    VM_PROT_READ,
-			    MAP_FILE,
-			    (caddr_t) vnodep,
+			    0,
+			    (caddr_t) vp,
 			    0))
     	goto fail;
 
@@ -253,13 +265,13 @@ coff_load_file(struct proc *p, char *name)
     		}
   	}
 
-  	if (error = load_coff_section(vmspace, vnodep, text_offset,
+  	if (error = load_coff_section(vmspace, vp, text_offset,
 				      (caddr_t)text_address,
 				      text_size, text_size,
 				      VM_PROT_READ | VM_PROT_EXECUTE)) {
     		goto dealloc_and_fail;
   	}
-  	if (error = load_coff_section(vmspace, vnodep, data_offset,
+  	if (error = load_coff_section(vmspace, vp, data_offset,
 				      (caddr_t)data_address,
 				      data_size + bss_size, data_size,
 				      VM_PROT_ALL)) {
@@ -281,36 +293,26 @@ coff_load_file(struct proc *p, char *name)
 }
 
 int
-exec_coff_imgact(iparams)
-	struct image_params *iparams;
+exec_coff_imgact(imgp)
+	struct image_params *imgp;
 {
-	struct filehdr *fhdr = (struct filehdr*)iparams->image_header;
-	struct aouthdr *ahdr;
-	struct scnhdr *scns;
+	const struct filehdr *fhdr = (const struct filehdr*)imgp->image_header;
+	const struct aouthdr *ahdr;
+	const struct scnhdr *scns;
 	int i;
-	struct vmspace *vmspace = iparams->proc->p_vmspace;
-	unsigned long vmaddr;
+	struct vmspace *vmspace = imgp->proc->p_vmspace;
 	int nscns;
-	int error, len;
-	extern struct sysentvec ibcs2_svr3_sysvec;
+	int error;
 	unsigned long text_offset = 0, text_address = 0, text_size = 0;
 	unsigned long data_offset = 0, data_address = 0, data_size = 0;
 	unsigned long bss_size = 0;
-	int need_hack_p;
-	unsigned long data_end;
-	unsigned long data_map_start, data_map_len, data_map_addr = 0;
-	unsigned long bss_address, bss_map_start, data_copy_len, bss_map_len;
-	unsigned char *data_buf = 0;
 	caddr_t hole;
 
 	if (fhdr->f_magic != I386_COFF ||
 	    !(fhdr->f_flags & F_EXEC)) {
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): return -1\n", __FILE__, __LINE__);
-}
-
-	  	return -1;
+		 DPRINTF(("%s(%d): return -1\n", __FILE__, __LINE__));
+		 return -1;
 	}
 
 	nscns = fhdr->f_nscns;
@@ -320,37 +322,30 @@ printf("%s(%d): return -1\n", __FILE__, __LINE__);
 	   	 * read in all of the section structures.
 	   	 */
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): return -1\n", __FILE__, __LINE__);
-}
-	  	return -1;
+		DPRINTF(("%s(%d): return -1\n", __FILE__, __LINE__));
+		return -1;
 	}
 
-	ahdr = (struct aouthdr*)((char*)(iparams->image_header) +
-				 sizeof(struct filehdr));
-	iparams->entry_addr = ahdr->entry;
+	ahdr = (const struct aouthdr*)
+	       ((const char*)(imgp->image_header) + sizeof(struct filehdr));
+	imgp->entry_addr = ahdr->entry;
 
-	scns = (struct scnhdr*)((char*)(iparams->image_header) +
-				sizeof(struct filehdr) +
-				sizeof(struct aouthdr));
+	scns = (const struct scnhdr*)
+	       ((const char*)(imgp->image_header) + sizeof(struct filehdr) +
+		sizeof(struct aouthdr));
 
-	if (error = exec_extract_strings(iparams)) {
-
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d):  return %d\n", __FILE__, __LINE__, error);
-}
-	  	return error;
+	if (error = exec_extract_strings(imgp)) {
+		DPRINTF(("%s(%d):  return %d\n", __FILE__, __LINE__, error));
+		return error;
 	}
 
-	exec_new_vmspace(iparams);
+	exec_new_vmspace(imgp);
 
 	for (i = 0; i < nscns; i++) {
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("i = %d, scns[i].s_name = %s, scns[i].s_vaddr = %08lx, "
-       "scns[i].s_scnptr = %d\n", i, scns[i].s_name,
-       scns[i].s_vaddr, scns[i].s_scnptr);
-}
+	  DPRINTF(("i = %d, scns[i].s_name = %s, scns[i].s_vaddr = %08lx, "
+		   "scns[i].s_scnptr = %d\n", i, scns[i].s_name,
+		   scns[i].s_vaddr, scns[i].s_scnptr));
 	  if (scns[i].s_flags & STYP_NOLOAD) {
 	    	/*
 	     	 * A section that is not loaded, for whatever
@@ -371,7 +366,7 @@ printf("i = %d, scns[i].s_name = %s, scns[i].s_vaddr = %08lx, "
 	    	/* .bss section */
 	    	bss_size = scns[i].s_size;
 	  } else if (scns[i].s_flags & STYP_LIB) {
-	    	char *buf = 0, *ptr;
+	    	char *buf = 0;
 	    	int foff = trunc_page(scns[i].s_scnptr);
 	    	int off = scns[i].s_scnptr - foff;
 	    	int len = round_page(scns[i].s_size + PAGE_SIZE);
@@ -382,23 +377,37 @@ printf("i = %d, scns[i].s_name = %s, scns[i].s_vaddr = %08lx, "
 				    len,
 				    VM_PROT_READ,
 				    VM_PROT_READ,
-				    MAP_FILE,
-				    (caddr_t) iparams->vnodep,
+				    0,
+				    (caddr_t) imgp->vp,
 				    foff)) {
 	      		return ENOEXEC;
 	    	}
-	    	for (j = off; j < scns[i].s_size + off; j++) {
-	      		char *libname;
-	      		libname = buf + j + 4 * *(long*)(buf + j + 4);
-	      		j += 4* *(long*)(buf + j);
+		if(scns[i].s_size) {
+			char *libbuf;
+			int emul_path_len = strlen(ibcs2_emul_path);
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d):  shared library %s\n", __FILE__, __LINE__, libname);
-}
-	      		error = coff_load_file(iparams->proc, libname);
-	      		if (error)
-			break;
-	    	}
+			libbuf = malloc(MAXPATHLEN + emul_path_len,
+					M_TEMP, M_WAITOK);
+			strcpy(libbuf, ibcs2_emul_path);
+
+		    	for (j = off; j < scns[i].s_size + off; j++) {
+	      			char *libname;
+
+		      		libname = buf + j + 4 * *(long*)(buf + j + 4);
+		      		j += 4* *(long*)(buf + j);
+
+				DPRINTF(("%s(%d):  shared library %s\n",
+					 __FILE__, __LINE__, libname));
+				strcpy(&libbuf[emul_path_len], libname);
+		      		error = coff_load_file(imgp->proc, libbuf);
+		      		if (error)
+	      				error = coff_load_file(imgp->proc,
+							       libname);
+		      		if (error)
+					break;
+		    	}
+			free(libbuf, M_TEMP);
+		}
 		if (vm_map_remove(kernel_map,
 				  (vm_offset_t) buf,
 				  (vm_offset_t) buf + len))
@@ -411,20 +420,15 @@ printf("%s(%d):  shared library %s\n", __FILE__, __LINE__, libname);
 	 * Map in .text now
 	 */
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d):  load_coff_section(vmspace, "
-	"iparams->vnodep, %08lx, %08lx, 0x%x, 0x%x, 0x%x)\n",
-	__FILE__, __LINE__, text_offset, text_address,
-	text_size, text_size, VM_PROT_READ | VM_PROT_EXECUTE);
-}
-	if (error = load_coff_section(vmspace, iparams->vnodep,
+	DPRINTF(("%s(%d):  load_coff_section(vmspace, "
+		"imgp->vp, %08lx, %08lx, 0x%x, 0x%x, 0x%x)\n",
+		__FILE__, __LINE__, text_offset, text_address,
+		text_size, text_size, VM_PROT_READ | VM_PROT_EXECUTE));
+	if (error = load_coff_section(vmspace, imgp->vp,
 				      text_offset, (caddr_t)text_address,
 				      text_size, text_size,
 				      VM_PROT_READ | VM_PROT_EXECUTE)) {
-
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): error = %d\n", __FILE__, __LINE__, error);
-}
+		DPRINTF(("%s(%d): error = %d\n", __FILE__, __LINE__, error));
 		return error;
        	}
 	/*
@@ -432,25 +436,21 @@ printf("%s(%d): error = %d\n", __FILE__, __LINE__, error);
 	 */
 
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): load_coff_section(vmspace, "
-	"iparams->vnodep, 0x%08lx, 0x%08lx, 0x%x, 0x%x, 0x%x)\n",
-	__FILE__, __LINE__, data_offset, data_address,
-	data_size + bss_size, data_size, VM_PROT_ALL);
-}
-	if (error = load_coff_section(vmspace, iparams->vnodep,
+	DPRINTF(("%s(%d): load_coff_section(vmspace, "
+		"imgp->vp, 0x%08lx, 0x%08lx, 0x%x, 0x%x, 0x%x)\n",
+		__FILE__, __LINE__, data_offset, data_address,
+		data_size + bss_size, data_size, VM_PROT_ALL));
+	if (error = load_coff_section(vmspace, imgp->vp,
 				      data_offset, (caddr_t)data_address,
 				      data_size + bss_size, data_size,
 				      VM_PROT_ALL)) {
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): error = %d\n", __FILE__, __LINE__, error);
-}
+		DPRINTF(("%s(%d): error = %d\n", __FILE__, __LINE__, error));
 		return error;
 	}
 
-	iparams->interpreted = 0;
-	iparams->proc->p_sysent = &ibcs2_svr3_sysvec;
+	imgp->interpreted = 0;
+	imgp->proc->p_sysent = &ibcs2_svr3_sysvec;
 
 	vmspace->vm_tsize = round_page(text_size) >> PAGE_SHIFT;
 	vmspace->vm_dsize = round_page(data_size + bss_size) >> PAGE_SHIFT;
@@ -460,22 +460,21 @@ printf("%s(%d): error = %d\n", __FILE__, __LINE__, error);
 	hole = (caddr_t)trunc_page(vmspace->vm_daddr) + ctob(vmspace->vm_dsize);
 
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("%s(%d): vm_map_find(&vmspace->vm_map, NULL, 0, &0x%08lx, PAGE_SIZE, FALSE)\n",
-	__FILE__, __LINE__, hole);
-printf("imgact: error = %d\n", error);
-}
+	DPRINTF(("%s(%d): vm_map_find(&vmspace->vm_map, NULL, 0, &0x%08lx, PAGE_SIZE, FALSE, VM_PROT_ALL, VM_PROT_ALL, 0)\n",
+		__FILE__, __LINE__, hole));
+        DPRINTF(("imgact: error = %d\n", error));
+
 	error = vm_map_find(&vmspace->vm_map, NULL, 0,
-			    (vm_offset_t *) &hole, PAGE_SIZE, FALSE);
+			    (vm_offset_t *) &hole, PAGE_SIZE, FALSE,
+				VM_PROT_ALL, VM_PROT_ALL, 0);
+
+	DPRINTF(("IBCS2: start vm_dsize = 0x%x, vm_daddr = 0x%x end = 0x%x\n",
+		ctob(vmspace->vm_dsize), vmspace->vm_daddr,
+		ctob(vmspace->vm_dsize) + vmspace->vm_daddr ));
+	DPRINTF(("%s(%d):  returning successfully!\n", __FILE__, __LINE__));
 
-if (ibcs2_trace & IBCS2_TRACE_COFF) {
-printf("IBCS2: start vm_dsize = 0x%x, vm_daddr = 0x%x end = 0x%x\n",
-	ctob(vmspace->vm_dsize), vmspace->vm_daddr,
-	ctob(vmspace->vm_dsize) + vmspace->vm_daddr );
-printf("%s(%d):  returning successfully!\n", __FILE__, __LINE__);
-}
 	/* Indicate that this file should not be modified */
-	iparams->vnodep->v_flag |= VTEXT;
+	imgp->vp->v_flag |= VTEXT;
 	return 0;
 }
 
