@@ -12,17 +12,27 @@
  *
  * Aug, 1994	Implemented on FreeBSD 1.1.5.1R (Toshiba AVS001WD)
  *
- * $FreeBSD: stable/2.1/sys/i386/include/apm_bios.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/apm_bios.h 50950 1999-09-05 08:39:22Z peter $
  */
 
-#ifndef _MACHINE_APM_BIOS_H_
-#define _MACHINE_APM_BIOS_H_	1
+#ifndef	_MACHINE_APM_BIOS_H_
+#define	_MACHINE_APM_BIOS_H_
+
+#ifndef KERNEL
+#include <sys/types.h>
+#endif
+#include <sys/ioccom.h>
 
 #ifdef KERNEL
 
 /* BIOS id */
+#ifdef PC98
+#define APM_BIOS		0x9a
+#define SYSTEM_BIOS		0x1f
+#else
 #define APM_BIOS		0x53
 #define SYSTEM_BIOS		0x15
+#endif
 
 /* APM flags */
 #define APM_16BIT_SUPPORT	0x01
@@ -50,11 +60,19 @@
 #define APM_SETPWSTATE		0x07
 #define APM_ENABLEDISABLEPM	0x08
 #define APM_RESTOREDEFAULT	0x09
+#ifdef PC98
+#define	APM_GETPWSTATUS		0x3a
+#else
 #define	APM_GETPWSTATUS		0x0a
+#endif
 #define APM_GETPMEVENT		0x0b
 #define APM_GETPWSTATE		0x0c
 #define APM_ENABLEDISABLEDPM	0x0d
+#ifdef PC98
+#define APM_DRVVERSION		0x3e
+#else
 #define APM_DRVVERSION		0x0e
+#endif
 #define APM_ENGAGEDISENGAGEPM	0x0f
 #define APM_OEMFUNC		0x80
 
@@ -183,8 +201,8 @@ typedef struct apm_info {
 #define APMIO_DISABLE		_IO('P', 6)
 #define APMIO_HALTCPU		_IO('P', 7)
 #define APMIO_NOTHALTCPU	_IO('P', 8)
-#define APMIO_DISPLAYOFF	_IO('P', 9)
+#define APMIO_DISPLAY		_IOW('P', 9, int)
 
 #endif /* !ASSEMBLER && !INITIALIZER */
 
-#endif /* _MACHINE_APM_BIOS_H_ */
+#endif /* !_MACHINE_APM_BIOS_H_ */
