@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1991-1995 Sren Schmidt
+ * Copyright (c) 1991-1996 Sren Schmidt
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,7 +12,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software withough specific prior written permission
+ *    derived from this software without specific prior written permission
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -25,13 +25,15 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/console.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/console.h 50950 1999-09-05 08:39:22Z peter $
  */
 
-#ifndef	_CONSOLE_H_
-#define	_CONSOLE_H_
+#ifndef	_MACHINE_CONSOLE_H_
+#define	_MACHINE_CONSOLE_H_
 
+#ifndef KERNEL
 #include <sys/types.h>
+#endif
 #include <sys/ioccom.h>
 
 #define KDGKBMODE 	_IOR('K',  6, int)
@@ -56,6 +58,8 @@
 #define PIO_SCRNMAP	_IOW('k', 3, scrmap_t)
 #define GIO_KEYMAP 	_IOR('k', 6, keymap_t)
 #define PIO_KEYMAP 	_IOW('k', 7, keymap_t)
+#define GIO_DEADKEYMAP 	_IOR('k', 8, accentmap_t)
+#define PIO_DEADKEYMAP 	_IOW('k', 9, accentmap_t)
 
 #define GIO_ATTR	_IOR('a', 0, int)
 #define GIO_COLOR	_IOR('c', 0, int)
@@ -78,6 +82,10 @@
 #define CONS_GETINFO    _IOWR('c', 73, vid_info_t)
 #define CONS_GETVERS	_IOR('c', 74, int)
 
+#ifdef PC98
+#define ADJUST_CLOCK		_IO('t',100)		/* for 98note resume */
+#endif
+
 #define VT_OPENQRY	_IOR('v', 1, int)
 #define VT_SETMODE	_IOW('v', 2, vtmode_t)
 #define VT_GETMODE	_IOR('v', 3, vtmode_t)
@@ -92,6 +100,7 @@
 
 #define VT_AUTO		0		/* switching is automatic 	*/
 #define VT_PROCESS	1		/* switching controlled by prog */
+#define VT_KERNEL	255		/* switching controlled in kernel */
 
 struct vt_mode {
 	char	mode;
@@ -101,23 +110,48 @@ struct vt_mode {
 	short	frsig;			/* not implemented yet	SOS	*/
 };
 
-#define MOUSE_SHOW	0x01
-#define MOUSE_HIDE	0x02
-#define MOUSE_MOVEABS	0x03
-#define MOUSE_MOVEREL	0x04
-#define MOUSE_GETPOS	0x05
+struct mouse_data {
+	int	x;
+	int 	y;
+	int 	z;
+	int 	buttons;
+};
+
+struct mouse_mode {
+	int	mode;
+	int	signal;
+};
+
+struct mouse_event {
+	int	id;			/* one based */
+	int	value;
+};
+
+#define MOUSE_SHOW		0x01
+#define MOUSE_HIDE		0x02
+#define MOUSE_MOVEABS		0x03
+#define MOUSE_MOVEREL		0x04
+#define MOUSE_GETINFO		0x05
+#define MOUSE_MODE		0x06
+#define MOUSE_ACTION		0x07
+#define MOUSE_MOTION_EVENT	0x08
+#define MOUSE_BUTTON_EVENT	0x09
 
 struct mouse_info {
 	int	operation;
-	int	x;
-	int 	y;
+	union {
+		struct mouse_data data;
+		struct mouse_mode mode;
+		struct mouse_event event;
+	}u;
 };
 
 #define KD_MONO		1		/* monochrome adapter        	*/
 #define KD_HERCULES	2		/* hercules adapter          	*/
 #define KD_CGA		3		/* color graphics adapter    	*/
 #define KD_EGA		4		/* enhanced graphics adapter 	*/
-#define KD_VGA		5		/* video graohics adapter    	*/
+#define KD_VGA		5		/* video graphics adapter    	*/
+#define KD_PC98		6		/* PC-98 display            	*/
 
 #define KD_TEXT		0		/* set text mode restore fonts  */
 #define KD_TEXT0	0		/* ditto			*/
@@ -126,6 +160,7 @@ struct mouse_info {
 
 #define K_RAW		0		/* keyboard returns scancodes	*/
 #define K_XLATE		1		/* keyboard returns ascii 	*/
+#define K_CODE		2		/* keyboard returns keycodes 	*/
 
 #define KB_84		1		/* 'old' 84 key AT-keyboard	*/
 #define KB_101		2		/* MF-101 or MF-102 keyboard	*/
@@ -159,6 +194,19 @@ struct keymap {
 	struct key_t key[NUM_KEYS];
 };
 
+#define NUM_DEADKEYS	15		/* number of accent keys */
+#define NUM_ACCENTCHARS	52		/* max number of accent chars */
+
+struct acc_t {
+	u_char accchar;
+	u_char map[NUM_ACCENTCHARS][2];
+};
+
+struct accentmap {
+	u_short n_accs;
+	struct acc_t acc[NUM_DEADKEYS];
+};
+
 #define MAXFK		16
 #define NUM_FKEYS	96
 
@@ -199,6 +247,7 @@ struct ssaver	{
 };
 
 typedef struct keymap keymap_t;
+typedef struct accentmap accentmap_t;
 typedef struct fkeytab fkeytab_t;
 typedef struct fkeyarg fkeyarg_t;
 typedef struct vid_info vid_info_t;
@@ -234,28 +283,35 @@ typedef struct ssaver ssaver_t;
 #define RBT		0x85		/* boot machine			*/
 #define DBG		0x86		/* call debugger		*/
 #define SUSP		0x87		/* suspend power (APM)		*/
+#define SPSC		0x88		/* toggle splash/text screen	*/
+
+#define F_ACC		DGRA		/* first accent key		*/
+#define DGRA		0x89		/* grave			*/
+#define DACU		0x8a		/* acute			*/
+#define DCIR		0x8b		/* circumflex			*/
+#define DTIL		0x8c		/* tilde			*/
+#define DMAC		0x8d		/* macron			*/
+#define DBRE		0x8e		/* breve			*/
+#define DDOT		0x8f		/* dot				*/
+#define DUML		0x90		/* umlaut/diaresis		*/
+#define DDIA		0x90		/* diaresis			*/
+#define DSLA		0x91		/* slash			*/
+#define DRIN		0x92		/* ring				*/
+#define DCED		0x93		/* cedilla			*/
+#define DAPO		0x94		/* apostrophe			*/
+#define DDAC		0x95		/* double acute			*/
+#define DOGO		0x96		/* ogonek			*/
+#define DCAR		0x97		/* caron			*/
+#define L_ACC		DCAR		/* last accent key		*/
 
 #define F(x)		((x)+F_FN-1)
 #define	S(x)		((x)+F_SCR-1)
+#define ACC(x)		((x)+F_ACC)
 #define NOKEY		0x100		/* no key pressed marker 	*/
-#define FKEY		0x200		/* funtion key marker 		*/
+#define FKEY		0x200		/* function key marker 		*/
 #define MKEY		0x400		/* meta key marker (prepend ESC)*/
 #define BKEY		0x800		/* backtab (ESC [ Z)		*/
 
-#define	KB_DATA		0x60		/* kbd data port 		*/
-#define	KB_STAT		0x64		/* kbd status port 		*/
-#define	KB_BUF_FULL	0x01		/* kbd has char pending 	*/
-#define	KB_READY	0x02		/* kbd ready for command 	*/
-#define KB_MODE		0x4D		/* kbd mode (trans, ints enable)*/
-#define KB_WRITE	0x60		/* kbd write command 		*/
-#define KB_RESET_DONE	0xAA		/* kbd reset command completed  */
-#define KB_SETLEDS	0xED		/* kbd set leds 		*/
-#define KB_ECHO		0xEE		/* kbd set leds 		*/
-#define KB_SETRAD	0xF3		/* kbd set repeat&delay command */
-#define KB_ACK		0xFA		/* kbd acknowledge answer 	*/
-#define KB_RESEND	0xFE		/* kbd resend cmd answer      	*/
-#define KB_RESET	0xFF		/* kbd reset 			*/
-
 /* video mode definitions */
 #define M_B40x25	0	/* black & white 40 columns */
 #define M_C40x25	1	/* color 40 columns */
@@ -292,13 +348,21 @@ typedef struct ssaver ssaver_t;
 #define M_VGA_M80x30	33	/* vga 8x16 font on color */
 #define M_VGA_C80x60	34	/* vga 8x8 font on color */
 #define M_VGA_M80x60	35	/* vga 8x8 font on color */
+#define M_VGA_CG640	36	/* vga 640x400 256 color */
+#define M_VGA_MODEX	37	/* vga 320x240 256 color */
 
 #define M_ENH_B80x43	0x70	/* ega black & white 80x43 */
 #define M_ENH_C80x43	0x71	/* ega color 80x43 */
+
+#define M_PC98_80x25	98	/* PC98 80x25 */
+#define M_PC98_80x30	99	/* PC98 80x30 */
+
 #define M_HGC_P0	0xe0	/* hercules graphics - page 0 @ B0000 */
 #define M_HGC_P1	0xe1	/* hercules graphics - page 1 @ B8000 */
 #define M_MCA_MODE	0xff	/* monochrome adapter mode */
 
+#define SW_PC98_80x25	_IO('S', M_PC98_80x25)
+#define SW_PC98_80x30	_IO('S', M_PC98_80x30)
 #define SW_B40x25 	_IO('S', M_B40x25)
 #define SW_C40x25  	_IO('S', M_C40x25)
 #define SW_B80x25  	_IO('S', M_B80x25)
@@ -335,5 +399,7 @@ typedef struct ssaver ssaver_t;
 #define SW_CG640x480	_IO('S', M_VGA12)
 #define SW_VGA13	_IO('S', M_VGA13)
 #define SW_VGA_CG320	_IO('S', M_VGA13)
+#define SW_VGA_CG640	_IO('S', M_VGA_CG640)
+#define SW_VGA_MODEX	_IO('S', M_VGA_MODEX)
 
-#endif
+#endif /* !_MACHINE_CONSOLE_H_ */
