@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/cpufunc.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/cpufunc.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -43,22 +43,13 @@
 #include <sys/cdefs.h>
 #include <sys/types.h>
 
-#include <machine/spl.h>	/* XXX belongs elsewhere */
-
 #ifdef	__GNUC__
 
-#ifdef BDE_DEBUGGER
-extern int	bdb_exists;
-
-static __inline int
-bdb(void)
+static __inline void
+breakpoint(void)
 {
-	if (!bdb_exists)
-		return (0);
 	__asm __volatile("int $3");
-	return (1);
 }
-#endif /* BDE_DEBUGGER */
 
 static __inline void
 disable_intr(void)
@@ -93,6 +84,17 @@ ffs(int mask)
 	return (result);
 }
 
+#define	HAVE_INLINE_FLS
+
+static __inline int
+fls(int mask)
+{
+	int	result;
+	__asm __volatile("testl %0,%0; je 1f; bsrl %0,%0; incl %0; 1:"
+			 : "=r" (result) : "0" (mask));
+	return (result);
+}
+
 #if __GNUC__ < 2
 
 #define	inb(port)		inbv(port)
@@ -110,18 +112,25 @@ ffs(int mask)
  * Use an expression-statement instead of a conditional expression
  * because gcc-2.6.0 would promote the operands of the conditional
  * and produce poor code for "if ((inb(var) & const1) == const2)".
+ *
+ * The unnecessary test `(port) < 0x10000' is to generate a warning if
+ * the `port' has type u_short or smaller.  Such types are pessimal.
+ * This actually only works for signed types.  The range check is
+ * careful to avoid generating warnings.
  */
-#define	inb(port)	({						\
+#define	inb(port) __extension__ ({					\
 	u_char	_data;							\
-	if (__builtin_constant_p((int) (port)) && (port) < 256ul)	\
+	if (__builtin_constant_p(port) && ((port) & 0xffff) < 0x100	\
+	    && (port) < 0x10000)					\
 		_data = inbc(port);					\
 	else								\
 		_data = inbv(port);					\
 	_data; })
 
-#define	outb(port, data) \
-	(__builtin_constant_p((int) (port)) && (port) < 256ul \
-	 ? outbc(port, data) : outbv(port, data))
+#define	outb(port, data) (						\
+	__builtin_constant_p(port) && ((port) & 0xffff) < 0x100		\
+	&& (port) < 0x10000						\
+	? outbc(port, data) : outbv(port, data))
 
 static __inline u_char
 inbc(u_int port)
@@ -186,6 +195,30 @@ insl(u_int port, void *addr, size_t cnt)
 			 : "di", "cx", "memory");
 }
 
+static __inline void
+invd(void)
+{
+	__asm __volatile("invd");
+}
+
+static __inline void
+invlpg(u_int addr)
+{
+	__asm __volatile("invlpg (%0)" : : "r" (addr) : "memory");
+}
+
+static __inline void
+invltlb(void)
+{
+	u_long	temp;
+	/*
+	 * This should be implemented as load_cr3(rcr3()) when load_cr3()
+	 * is inlined.
+	 */
+	__asm __volatile("movl %%cr3, %0; movl %0, %%cr3" : "=r" (temp)
+			 : : "memory");
+}
+
 static __inline u_short
 inw(u_int port)
 {
@@ -195,7 +228,7 @@ inw(u_int port)
 	return (data);
 }
 
-static __inline unsigned
+static __inline u_int
 loadandclear(u_int *addr)
 {
 	u_int	result;
@@ -231,7 +264,7 @@ outl(u_int port, u_long data)
 }
 
 static __inline void
-outsb(u_int port, void *addr, size_t cnt)
+outsb(u_int port, const void *addr, size_t cnt)
 {
 	__asm __volatile("cld; rep; outsb"
 			 : : "d" (port), "S" (addr), "c" (cnt)
@@ -239,7 +272,7 @@ outsb(u_int port, void *addr, size_t cnt)
 }
 
 static __inline void
-outsw(u_int port, void *addr, size_t cnt)
+outsw(u_int port, const void *addr, size_t cnt)
 {
 	__asm __volatile("cld; rep; outsw"
 			 : : "d" (port), "S" (addr), "c" (cnt)
@@ -247,7 +280,7 @@ outsw(u_int port, void *addr, size_t cnt)
 }
 
 static __inline void
-outsl(u_int port, void *addr, size_t cnt)
+outsl(u_int port, const void *addr, size_t cnt)
 {
 	__asm __volatile("cld; rep; outsl"
 			 : : "d" (port), "S" (addr), "c" (cnt)
@@ -260,18 +293,6 @@ outw(u_int port, u_short data)
 	__asm __volatile("outw %0,%%dx" : : "a" (data), "d" (port));
 }
 
-static __inline void
-pmap_update(void)
-{
-	u_long	temp;
-	/*
-	 * This should be implemented as load_cr3(rcr3()) when load_cr3()
-	 * is inlined.
-	 */
-	__asm __volatile("movl %%cr3, %0; movl %0, %%cr3" : "=r" (temp)
-			 : : "memory");
-}
-
 static __inline u_long
 rcr2(void)
 {
@@ -290,45 +311,60 @@ read_eflags(void)
 	return (ef);
 }
 
+static __inline quad_t
+rdmsr(u_int msr)
+{
+	quad_t rv;
+
+	__asm __volatile(".byte 0x0f, 0x32" : "=A" (rv) : "c" (msr));
+	return (rv);
+}
+
+static __inline quad_t
+rdpmc(u_int pmc)
+{
+	quad_t rv;
+
+	__asm __volatile(".byte 0x0f, 0x33" : "=A" (rv) : "c" (pmc));
+	return (rv);
+}
+
+static __inline quad_t
+rdtsc(void)
+{
+	quad_t rv;
+
+	__asm __volatile(".byte 0x0f, 0x31" : "=A" (rv));
+	return (rv);
+}
+
+static __inline void
+setbits(volatile unsigned *addr, u_int bits)
+{
+	__asm __volatile("orl %1,%0" : "=m" (*addr) : "ir" (bits));
+}
+
+static __inline void
+wbinvd(void)
+{
+	__asm __volatile("wbinvd");
+}
+
 static __inline void
 write_eflags(u_long ef)
 {
 	__asm __volatile("pushl %0; popfl" : : "r" (ef));
 }
 
-/*
- * XXX queue stuff belongs elsewhere.
- */
-struct quehead {
-	struct quehead *qh_link;
-	struct quehead *qh_rlink;
-};
-
 static __inline void
-insque(void *a, void *b)
+wrmsr(u_int msr, quad_t newval)
 {
-	struct quehead *element = (struct quehead *)a,
-			*head = (struct quehead *)b;
-
-	element->qh_link = head->qh_link;
-	element->qh_rlink = head;
-	head->qh_link = element;
-	element->qh_link->qh_rlink = element;
-}
-
-static __inline void
-remque(void *a)
-{
-	struct quehead *element = (struct quehead *)a;
-
-	element->qh_link->qh_rlink = element->qh_rlink;
-	element->qh_rlink->qh_link = element->qh_link;
-	element->qh_rlink = 0;
+	__asm __volatile(".byte 0x0f, 0x30" : : "A" (newval), "c" (msr));
 }
 
 #else /* !__GNUC__ */
 
-int	bdb		__P((void));
+int	breakpoint	__P((void));
 void	disable_intr	__P((void));
 void	enable_intr	__P((void));
 u_char	inb		__P((u_int port));
@@ -336,6 +372,9 @@ u_long	inl		__P((u_int port));
 void	insb		__P((u_int port, void *addr, size_t cnt));
 void	insl		__P((u_int port, void *addr, size_t cnt));
 void	insw		__P((u_int port, void *addr, size_t cnt));
+void	invd		__P((void));
+void	invlpg		__P((u_int addr));
+void	invltlb		__P((void));
 u_short	inw		__P((u_int port));
 u_int	loadandclear	__P((u_int *addr));
 void	outb		__P((u_int port, u_char data));
@@ -344,110 +383,24 @@ void	outsb		__P((u_int port, void *addr, size_t cnt));
 void	outsl		__P((u_int port, void *addr, size_t cnt));
 void	outsw		__P((u_int port, void *addr, size_t cnt));
 void	outw		__P((u_int port, u_short data));
-void	pmap_update	__P((void));
-u_long	read_eflags	__P((void));
 u_long	rcr2		__P((void));
+quad_t	rdmsr		__P((u_int msr));
+quad_t	rdpmc		__P((u_int pmc));
+quad_t	rdtsc		__P((void));
+u_long	read_eflags	__P((void));
+void	setbits		__P((volatile unsigned *addr, u_int bits));
+void	wbinvd		__P((void));
 void	write_eflags	__P((u_long ef));
-
-void	insque		__P((void *a, void *b));
-void	remque		__P((void *a));
+void	wrmsr		__P((u_int msr, quad_t newval));
 
 #endif	/* __GNUC__ */
 
-/*
- * XXX the following declarations document garbage in support.s.
- * gcc hasn't needed _divsi* for years.
- * bcopy[bwx]() was used by pccons but isn't used now.
- */
-int	__divsi3	__P((int factor1, int factor2));
-u_int	__udivsi3	__P((u_int factor1, u_int factor2));
-void	bcopyb		__P((const void *from, void *to, size_t len));
-void	bcopyw		__P((const void *from, void *to, size_t len));
-void	bcopyx		__P((const void *from, void *to, size_t len,
-			     int stride));
-
-#if 0
-/*
- * These functions in support.s are declared elsewhere.
- */
-void	bcopy		__P((const void *from, void *to, size_t len));
-void	blkclr		__P((void *buf, size_t len));
-void	bzero		__P((void *buf, size_t len));
-int	copyin		__P((void *udaddr, void *kaddr, size_t len));
-int	copyinstr	__P((void *udaddr, void *kaddr, size_t len,
-			     size_t *lencopied));
-int	copyout		__P((void *kaddr, void *udaddr, size_t len));
-int	copystr		__P((void *kfaddr, void *kdaddr, size_t len,
-			     size_t *lencopied));
-int	fubyte		__P((void *base));
-int	fuswintr	__P((void *base));
-int	fuibyte		__P((void *base));
-int	fuword		__P((void *base));
-struct	region_descriptor;
-void	lgdt		__P((struct region_descriptor *rdp));
-void	lidt		__P((struct region_descriptor *rdp));
-void	lldt		__P((u_short sel));
-/*
- * longjmp() and setjmp() are only used by ddb.  They probably shouldn't
- * shouldn't be supported in the kernel.
- */
-#include <setjmp.h>
-void	longjmp		__P((jmp_buf jb, int rv));
-void	ovbcopy		__P((const void *from, void *to, size_t len);
-int	setjmp		__P((jmp_buf jb));
-struct soft_segment_descriptor;
-union descriptor;
-int	ssdtosd		__P((struct soft_segment_descriptor *ssdp,
-			     union descriptor *sdp));
-int	subyte		__P((void *base, int byte));
-int	suibyte		__P((void *base, int byte));
-int	suswintr	__P((void *base, int word));
-int	suword		__P((void *base, int word));
-
-/*
- * These functions in support.s are declared elsewhere, but never used.
- * A silly amount of effort went into copyoutstr().  It's not worth
- * maintaining, since the string length is usually known so copyout
- * works better, or is easy to find so copyout() can be used.
- */
-int	copyoutstr	__P((void *kaddr, void *udaddr, size_t len,
-			     size_t *lencopied));
-int	fuiword		__P((void *base));
-int	suiword		__P((void *base, int word));
-
-/*
- * These functions in support.s are also in libkern.a and are declared in
- * libkern.h.
- * ffs() is built in to gcc-2 and was buggy in gcc-2.4.5 so we may may the
- * buggy version if we don't replace it by an inline.
- */
-int	bcmp		__P((const void *b1, const void *b2, size_t length));
-int	ffs		__P((int mask));
-#endif /* 0 */
-
-/*
- * These variables and functions in support.s are used.
- */
-extern u_int atdevbase;	/* offset in virtual memory of ISA io mem */
-
-void	filli		__P((int pat, void *base, size_t cnt));
-void	fillw		__P((int /*u_short*/ pat, void *base, size_t cnt));
-int	fusword		__P((void *base));
 void	load_cr0	__P((u_long cr0));
 void	load_cr3	__P((u_long cr3));
 void	ltr		__P((u_short sel));
 u_int	rcr0		__P((void));
 u_long	rcr3		__P((void));
-int	rtcin		__P((int val));
 
-/*
- * These functions are NOT in support.s and should be declared elsewhere.
- */
-void	Debugger	__P((const char *msg));
-u_long	kvtop		__P((void *addr));
-typedef void alias_for_inthand_t __P((u_int cs, u_int ef, u_int esp,
-				      u_int ss));
-void	setidt		__P((int idx, alias_for_inthand_t *func, int typ,
-			     int dpl, int selec));
+#include <machine/spl.h>	/* XXX belongs elsewhere */
 
 #endif /* !_MACHINE_CPUFUNC_H_ */
