@@ -11,7 +11,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software withough specific prior written permission
+ *    derived from this software without specific prior written permission
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/cputypes.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/cputypes.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _MACHINE_CPUTYPES_H_
@@ -38,6 +38,7 @@
 #define	CPUCLASS_386	1
 #define	CPUCLASS_486	2
 #define	CPUCLASS_586	3
+#define CPUCLASS_686	4
 
 /*
  *	Kinds of Processor
@@ -50,5 +51,12 @@
 #define	CPU_486		4	/* Intel 80486DX */
 #define	CPU_586		5	/* Intel P.....m (I hate lawyers; it's TM) */
 #define	CPU_486DLC	6	/* Cyrix 486DLC */
-
+#define CPU_686		7	/* Pentium Pro */
+#define	CPU_M1SC	8	/* Cyrix M1sc (aka 5x86) */
+#define	CPU_M1		9	/* Cyrix M1 (aka 6x86) */
+#define	CPU_BLUE	10	/* IBM BlueLighting CPU */
+#define	CPU_M2		11	/* Cyrix M2 (aka enhanced 6x86 with MMX */
+#define CPU_NX586	12	/* NexGen (now AMD) 586 */
+#define	CPU_CY486DX	13	/* Cyrix 486S/DX/DX2/DX4 */
+#define CPU_PII		14	/* Intel Pentium II */
 #endif /* _MACHINE_CPUTYPES_H_ */
