@@ -11,15 +11,14 @@
  *
  *	Geoff Rehmet, Rhodes University, South Africa <csgr@cs.ru.ac.za>
  *
- * $FreeBSD: stable/2.1/sys/i386/include/lpt.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/lpt.h 50950 1999-09-05 08:39:22Z peter $
  */
 
-#ifndef _LPT_PRINTER_H_
-#define _LPT_PRINTER_H_
+#ifndef	_MACHINE_LPT_H_
+#define	_MACHINE_LPT_H_
 
-#include <sys/types.h>
-#include <sys/ioctl.h>
+#include <sys/ioccom.h>
 
 #define	LPT_IRQ		_IOW('p', 1, long)	/* set interrupt status */
 
-#endif
+#endif /* !_MACHINE_LPT_H_ */
