@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/md_var.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/md_var.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _MACHINE_MD_VAR_H_
@@ -37,23 +37,27 @@
  */
 
 extern	int	Maxmem;
+extern	u_int	atdevbase;	/* offset in virtual memory of ISA io mem */
 extern	u_long	cpu_feature;
 extern	u_long	cpu_high;
 extern	u_long	cpu_id;
+extern	u_long	cyrix_did;
 extern	char	cpu_vendor[];
 extern	char	etext[];
-extern	vm_offset_t isaphysmem;
 extern	char	kstack[];
 extern	void	(*netisrs[32]) __P((void));
 extern	int	nfs_diskless_valid;
-extern	int	sigcode;
+extern	char	sigcode[];
 extern	int	szsigcode;
+extern	int	busdma_swi_pending;
 
+typedef void alias_for_inthand_t __P((u_int cs, u_int ef, u_int esp, u_int ss));
 struct	proc;
 struct	reg;
 
-void	configure __P((void));
+void	bcopyb __P((const void *from, void *to, size_t len));
 void	cpu_reset __P((void));
+void	cpu_halt __P((void));
 void	doreti_iret __P((void)) __asm(__STRING(doreti_iret));
 void	doreti_iret_fault __P((void)) __asm(__STRING(doreti_iret_fault));
 void	doreti_popl_ds __P((void)) __asm(__STRING(doreti_popl_ds));
@@ -61,9 +65,21 @@ void	doreti_popl_ds_fault __P((void)) __asm(__STRING(doreti_popl_ds_fault));
 void	doreti_popl_es __P((void)) __asm(__STRING(doreti_popl_es));
 void	doreti_popl_es_fault __P((void)) __asm(__STRING(doreti_popl_es_fault));
 int	fill_regs __P((struct proc *p, struct reg *regs));
-int	mvesp __P((void));
-void	setconf __P((void));
+void	fillw __P((int /*u_short*/ pat, void *base, size_t cnt));
+int	fusword __P((void *base));
+int	is_physical_memory __P((vm_offset_t addr));
+u_long	kvtop __P((void *addr));
+void	setidt __P((int idx, alias_for_inthand_t *func, int typ, int dpl,
+		    int selec));
+void	swi_vm __P((void));
 void	userconfig __P((void));
 void	vm_bounce_init __P((void));
+int	vm_page_zero_idle __P((void));
+void	busdma_swi __P((void));
+
+#ifdef PC98
+extern int need_pre_dma_flush;
+extern int need_post_dma_flush;
+#endif
 
 #endif /* !_MACHINE_MD_VAR_H_ */
