@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)npx.h	5.3 (Berkeley) 1/18/91
- * $FreeBSD: stable/2.1/sys/i386/include/npx.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/npx.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -85,13 +85,6 @@ struct	save87 {
 	u_char	sv_pad[64];	/* padding; used by emulators */
 };
 
-/* Cyrix EMC memory - mapped coprocessor context switch information */
-struct	emcsts {
-	long	em_msw;		/* memory mapped status register when swtched */
-	long	em_tar;		/* memory mapped temp A register when swtched */
-	long	em_dl;		/* memory mapped D low register when swtched */
-};
-
 /* Intel prefers long real (53 bit) precision */
 #define	__iBCS_NPXCW__		0x262
 /* wfj prefers temporary real (64 bit) precision */
@@ -150,7 +143,6 @@ struct proc;
 int	npxdna __P((void));
 void	npxexit __P((struct proc *p));
 void	npxinit __P((int control));
-void	npxintr __P((struct intrframe frame));
 void	npxsave __P((struct save87 *addr));
 #endif
 
