@@ -12,7 +12,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software withough specific prior written permission
+ *    derived from this software without specific prior written permission
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -25,11 +25,13 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/pcaudioio.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/pcaudioio.h 50950 1999-09-05 08:39:22Z peter $
  */
 
-#ifndef _PCAUDIOIO_H_
-#define	_PCAUDIOIO_H_
+#ifndef	_MACHINE_PCAUDIOIO_H_
+#define	_MACHINE_PCAUDIOIO_H_
+
+#include <sys/ioccom.h>
 
 typedef struct audio_prinfo {
 	unsigned	sample_rate;	/* samples per second */
@@ -73,4 +75,8 @@ typedef struct audio_info {
 #define	AUDIO_DRAIN		_IO('A', 3)
 #define	AUDIO_FLUSH		_IO('A', 4)
 
-#endif /*!_PCAUDIOIO_H*/
+/* compatibility to /dev/audio */
+#define AUDIO_COMPAT_DRAIN	_IO('P', 1)
+#define AUDIO_COMPAT_FLUSH	_IO('P', 0)
+
+#endif /* !_MACHINE_PCAUDIOIO_H_ */
