@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/include/spl.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/include/spl.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _MACHINE_IPL_H_
@@ -45,6 +45,9 @@
  */
 #define	SWI_TTY		(NHWI + 0)
 #define	SWI_NET		(NHWI + 1)
+#define	SWI_CAMNET	(NHWI + 2)
+#define	SWI_CAMBIO	(NHWI + 3)
+#define	SWI_VM		(NHWI + 4)
 #define	SWI_CLOCK	30
 #define	SWI_AST		31
 
@@ -53,6 +56,9 @@
  */
 #define	SWI_TTY_PENDING		(1 << SWI_TTY)
 #define	SWI_NET_PENDING		(1 << SWI_NET)
+#define	SWI_CAMNET_PENDING	(1 << SWI_CAMNET)
+#define	SWI_CAMBIO_PENDING	(1 << SWI_CAMBIO)
+#define	SWI_VM_PENDING		(1 << SWI_VM)
 #define	SWI_CLOCK_PENDING	(1 << SWI_CLOCK)
 #define	SWI_AST_PENDING		(1 << SWI_AST)
 
@@ -70,13 +76,22 @@
  */
 #define	SWI_TTY_MASK	(SWI_TTY_PENDING | SWI_CLOCK_MASK | SWI_NET_MASK)
 #define	SWI_NET_MASK	(SWI_NET_PENDING | SWI_CLOCK_MASK)
+#define	SWI_CAMNET_MASK	(SWI_CAMNET_PENDING | SWI_CLOCK_MASK)
+#define	SWI_CAMBIO_MASK	(SWI_CAMBIO_PENDING | SWI_CLOCK_MASK)
+#define	SWI_VM_MASK	(SWI_VM_PENDING | SWI_CLOCK_MASK)
 #define	SWI_CLOCK_MASK	(SWI_CLOCK_PENDING | SWI_AST_MASK)
 #define	SWI_AST_MASK	SWI_AST_PENDING
 #define	SWI_MASK	(~HWI_MASK)
 
 #ifndef	LOCORE
 
+/*
+ * cpl is preserved by interrupt handlers so it is effectively nonvolatile.
+ * ipending and idelayed are changed by interrupt handlers so they are
+ * volatile.
+ */
 extern	unsigned bio_imask;	/* group of interrupts masked with splbio() */
+extern	unsigned cam_imask;	/* group of interrupts masked with splcam() */
 extern	unsigned cpl;		/* current priority level mask */
 extern	volatile unsigned idelayed;	/* interrupts to become pending */
 extern	volatile unsigned ipending;	/* active interrupts masked by cpl */
@@ -85,18 +100,25 @@ extern	unsigned stat_imask;	/* interrupts masked with splstatclock() */
 extern	unsigned tty_imask;	/* group of interrupts masked with spltty() */
 
 /*
- * ipending has to be volatile so that it is read every time it is accessed
- * in splx() and spl0(), but we don't want it to be read nonatomically when
- * it is changed.  Pretending that ipending is a plain int happens to give
- * suitable atomic code for "ipending |= constant;".
+ * The volatile bitmap variables must be set atomically.  This normally
+ * involves using a machine-dependent bit-set or `or' instruction.
  */
-#define	setdelayed()	(*(unsigned *)&ipending |= loadandclear(&idelayed))
-#define	setsoftast()	(*(unsigned *)&ipending |= SWI_AST_PENDING)
-#define	setsoftclock()	(*(unsigned *)&ipending |= SWI_CLOCK_PENDING)
-#define	setsoftnet()	(*(unsigned *)&ipending |= SWI_NET_PENDING)
-#define	setsofttty()	(*(unsigned *)&ipending |= SWI_TTY_PENDING)
+#define	setdelayed()	setbits(&ipending, loadandclear(&idelayed))
+#define	setsoftast()	setbits(&ipending, SWI_AST_PENDING)
+#define	setsoftclock()	setbits(&ipending, SWI_CLOCK_PENDING)
+#define	setsoftnet()	setbits(&ipending, SWI_NET_PENDING)
+#define	setsofttty()	setbits(&ipending, SWI_TTY_PENDING)
+#define	setsoftcamnet()	setbits(&ipending, SWI_CAMNET_PENDING)
+#define	setsoftcambio()	setbits(&ipending, SWI_CAMBIO_PENDING)
+#define	setsoftvm()	setbits(&ipending, SWI_VM_PENDING)
 
-#define	schedsofttty()	(*(unsigned *)&idelayed |= SWI_TTY_PENDING)
+#define	schedsofttty()		setbits(&idelayed, SWI_TTY_PENDING)
+#define	schedsoftnet()		setbits(&idelayed, SWI_NET_PENDING)
+#define	schedsoftcamnet()	setbits(&idelayed, SWI_CAMNET_PENDING)
+#define	schedsoftcambio()	setbits(&idelayed, SWI_CAMBIO_PENDING)
+#define	schedsoftvm()		setbits(&idelayed, SWI_VM_PENDING)
+
+#define	softclockpending()	(ipending & SWI_CLOCK_PENDING)
 
 #ifdef __GNUC__
 
@@ -118,10 +140,16 @@ GENSPL(splclock, cpl = HWI_MASK | SWI_MASK)
 GENSPL(splhigh, cpl = HWI_MASK | SWI_MASK)
 GENSPL(splimp, cpl |= net_imask)
 GENSPL(splnet, cpl |= SWI_NET_MASK)
+GENSPL(splcam, cpl |= cam_imask)
+GENSPL(splsoftcam, cpl |= SWI_CAMBIO_MASK | SWI_CAMNET_MASK)
+GENSPL(splsoftcambio, cpl |= SWI_CAMBIO_MASK) 
+GENSPL(splsoftcamnet, cpl |= SWI_CAMNET_MASK)
 GENSPL(splsoftclock, cpl = SWI_CLOCK_MASK)
 GENSPL(splsofttty, cpl |= SWI_TTY_MASK)
 GENSPL(splstatclock, cpl |= stat_imask)
 GENSPL(spltty, cpl |= tty_imask)
+GENSPL(splvm, cpl |= net_imask | bio_imask)
+GENSPL(splsoftvm, cpl |= SWI_VM_MASK)
 
 static __inline void
 spl0(void)
