@@ -12,7 +12,7 @@
  * on the understanding that TFS is not responsible for the correct
  * functioning of this software in any circumstances.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/aha1542.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/aha1542.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -32,7 +32,6 @@
 #include <sys/proc.h>
 
 #include <machine/clock.h>
-#include <machine/cpu.h>	/* XXX for bootverbose: a funny place */
 #include <machine/stdarg.h>
 
 #include <vm/vm.h>
@@ -43,7 +42,6 @@
 #endif	/* KERNEL */
 #include <scsi/scsi_all.h>
 #include <scsi/scsiconf.h>
-#include <sys/devconf.h>
 
 #ifdef	KERNEL
 #include <sys/kernel.h>
@@ -299,7 +297,7 @@ int     aha_debug = 1;
 #endif /*AHADEBUG */
 
 static struct aha_data {
-	short   aha_base;	/* base port for each board */
+	int   aha_base;		/* base port for each board */
 	/*
 	 * xor this with a physaddr to get a kv addr and visa versa
 	 * for items in THIS STRUCT only.
@@ -326,7 +324,10 @@ static struct aha_data {
 
 static u_int32_t	aha_adapter_info __P((int unit));
 static int	ahaattach __P((struct isa_device *dev));
+#ifdef	TUNE_1542
 static int	aha_bus_speed_check __P((struct aha_data *aha, int speed));
+static int	aha_set_bus_speed __P((struct aha_data *aha));
+#endif
 static int	aha_cmd __P((struct aha_data *aha, int icnt, int ocnt, int wait,
 			     u_char *retval, u_char opcode, ...));
 static void	aha_done __P((struct aha_data *aha, struct aha_ccb *ccb));
@@ -340,9 +341,7 @@ static void	ahaminphys __P((struct buf *bp));
 static int	aha_poll __P((struct aha_data *aha, struct scsi_xfer *xs,
 			      struct aha_ccb *ccb));
 static int	ahaprobe __P((struct isa_device *dev));
-static void	aha_registerdev __P((struct isa_device *id));
 static int32_t	aha_scsi_cmd __P((struct scsi_xfer *xs));
-static int	aha_set_bus_speed __P((struct aha_data *aha));
 static timeout_t
 		aha_timeout;
 static char	*board_rev __P((struct aha_data *aha, int type));
@@ -380,27 +379,6 @@ struct isa_driver ahadriver =
     "aha"
 };
 
-static struct kern_devconf kdc_aha[NAHA] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"aha", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* always start out here */
-	"Adaptec 154x-series SCSI host adapter",
-	DC_CLS_MISC		/* SCSI host adapters aren't special */
-} };
-
-static inline void
-aha_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_aha[id->id_unit] = kdc_aha[0];
-	kdc_aha[id->id_unit].kdc_unit = id->id_unit;
-	kdc_aha[id->id_unit].kdc_parentdata = id;
-	dev_attach(&kdc_aha[id->id_unit]);
-}
-
 #endif	/* KERNEL */
 
 static int ahaunit = 0;
@@ -602,12 +580,9 @@ ahaprobe(dev)
 	}
 	bzero(aha, sizeof(struct aha_data));
 	ahadata[unit] = aha;
+	aha->unit = unit;
 	aha->aha_base = dev->id_iobase;
 
-#ifndef DEV_LKM
-	aha_registerdev(dev);
-#endif
-
 	/*
 	 * Try initialise a unit at this location
 	 * sets up dma and bus speed, loads aha->aha_int
@@ -674,7 +649,6 @@ ahaattach(dev)
 	/*
 	 * ask the adapter what subunits are present
 	 */
-	kdc_aha[unit].kdc_state = DC_BUSY; /* host adapters are always busy */
 	scsi_attachdevs(scbus);
 
 	return 1;
@@ -913,7 +887,9 @@ aha_done(aha, ccb)
 	 */
 	if (!(xs->flags & INUSE)) {
 		printf("aha%d: exiting but not in use!\n", aha->unit);
-		Debugger("aha1542");
+#ifdef DIAGNOSTIC
+		panic("aha1542 exiting but not in use");
+#endif
 	}
 	xs->status = ccb->target_stat;
 	xs->resid = 0;
@@ -1869,7 +1845,9 @@ aha_timeout(void *arg1)
 	 */
 	if (ccb->mbx->cmd != AHA_MBO_FREE) {
 		printf("\nadapter not taking commands.. frozen?!\n");
-		Debugger("aha1542");
+#ifdef DIAGNOSTIC
+		panic("aha1542 frozen");
+#endif
 	}
 	/*
 	 * If it has been through before, then
