@@ -31,7 +31,7 @@
  */
 
 /*
- * $FreeBSD: stable/2.1/sys/i386/isa/aic6360.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/aic6360.c 50950 1999-09-05 08:39:22Z peter $
  *
  * Acknowledgements: Many of the algorithms used in this driver are
  * inspired by the work of Julian Elischer (julian@tfs.com) and
@@ -47,6 +47,10 @@
  * 4) Rewrite it to use malloc for the acb structs instead of static alloc.?
  */
 
+/*
+ * PC-9801-100/AHA-1030P support by URATA S.
+ */
+
 /*
  * A few customizable items:
  */
@@ -108,43 +112,26 @@
 #error "I said not yet! Start paying attention... grumble"
 #endif
 
-#ifdef __FreeBSD__
-#include <aic.h>
-#endif
+#include "opt_ddb.h"
+#include "aic.h"
 
 #include <sys/types.h>
 #include <sys/param.h>
-#ifdef __FreeBSD__
 #include <sys/kernel.h>
-#endif
 #include <sys/systm.h>
 #include <sys/errno.h>
 #include <sys/ioctl.h>
-#ifdef __FreeBSD__
 #include <sys/malloc.h>
-#else
-#include <sys/device.h>
-#endif
 #include <sys/buf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/queue.h>
-#ifndef __FreeBSD__
-#include <machine/pio.h>
-#endif
 #include <scsi/scsi_all.h>
 #include <scsi/scsiconf.h>
 
-#ifdef __FreeBSD__
-#include <sys/devconf.h>
 #include <machine/clock.h>
 #include <i386/isa/isa_device.h>
 
 #include <sys/kernel.h>
-#else
-#include <i386/isa/isavar.h>
-#include <i386/isa/icu.h>
-#endif
 
 /* Definitions, most of them has turned out to be unneccesary, but here they
  * are anyway.
@@ -200,6 +187,9 @@
 #define ST_MASK			0x3e /* bit 0,6,7 is reserved */
 
 /* AIC6360 definitions */
+#ifdef PC98
+#include <i386/isa/aic_98.h>
+#else
 #define SCSISEQ		(iobase + 0x00) /* SCSI sequence control */
 #define SXFRCTL0	(iobase + 0x01) /* SCSI transfer control 0 */
 #define SXFRCTL1	(iobase + 0x02) /* SCSI transfer control 1 */
@@ -239,6 +229,7 @@
 #define STACK		(iobase + 0x1d) /* Stack */
 #define TEST		(iobase + 0x1e) /* Test register */
 #define ID		(iobase + 0x1f) /* ID register */
+#endif
 
 #define IDSTRING "(C)1991ADAPTECAIC6360           "
 
@@ -510,11 +501,7 @@ struct aic_dma_seg {
 	long		len;
 };
 
-#ifdef __FreeBSD__
 #define DELAYCOUNT	16
-#else
-extern int delaycount;
-#endif
 
 #define FUDGE(X)	((X)>>1) 	/* get 1 ms spincount */
 #define MINIFUDGE(X)	((X)>>4) 	/* get (approx) 125us spincount */
@@ -565,7 +552,7 @@ struct aic_tinfo {
 	u_char  persgst;	/* Period suggestion */
 	u_char  offsgst;	/* Offset suggestion */
 	u_char  syncdata;	/* True negotiated synch parameters */
-} tinfo_t;
+};
 
 /* Register a linenumber (for debugging) */
 #if AIC_DEBUG
@@ -578,24 +565,11 @@ struct aic_tinfo {
 #define LOGLINE(p)
 #endif
 
-#ifdef __FreeBSD__
-struct aic_data { /* One of these per adapter */
-#else
-struct aic_softc { /* One of these per adapter */
-	/* Auto config stuff */
-	struct device   sc_dev; /* This one has to go first! */
-	struct isadev   sc_id;
-	struct intrhand sc_ih;
-	struct scsi_link sc_link;       /* prototype for subdevs */
-	int             id_irq;         /* IRQ on the EISA bus */
-	int             id_drq;         /* DRQ on the EISA bus */
-#endif
+static struct aic_data { /* One of these per adapter */
 	u_short		iobase;		/* Base I/O port */
-#ifdef __FreeBSD__
 	struct scsi_link sc_link;	/* prototype for subdevs */
 	int		aic_int;	/* IRQ on the EISA bus */
 	int		aic_dma;	/* DRQ on the EISA bus */
-#endif
 	/* Lists of command blocks */
 	TAILQ_HEAD(acb_list, acb) free_list, ready_list, nexus_list;
 	struct acb *nexus;	/* current command */
@@ -643,11 +617,10 @@ struct aic_softc { /* One of these per adapter */
 	u_char	imess[AIC_MAX_MSG_LEN + 1];
 	u_char	*imp;		/* Message pointer (for multibyte messages) */
 	u_char	imlen;
-#ifdef __FreeBSD__
-} *aicdata[NAIC];
-#else
-};
+#ifdef PC98
+	int		*aicport;	/* I/O port information */
 #endif
+} *aicdata[NAIC];
 
 #define AIC_SHOWACBS 0x01
 #define AIC_SHOWINTS 0x02
@@ -655,7 +628,7 @@ struct aic_softc { /* One of these per adapter */
 #define AIC_SHOWMISC 0x08
 #define AIC_SHOWTRAC 0x10
 #define AIC_SHOWSTART 0x20
-int aic_debug = 0; /* AIC_SHOWSTART|AIC_SHOWMISC|AIC_SHOWTRAC; */
+static int aic_debug = 0; /* AIC_SHOWSTART|AIC_SHOWMISC|AIC_SHOWTRAC; */
 
 #if AIC_DEBUG
 #define AIC_ACBS(str)  do {if (aic_debug & AIC_SHOWACBS) printf str;} while (0)
@@ -673,75 +646,50 @@ int aic_debug = 0; /* AIC_SHOWSTART|AIC_SHOWMISC|AIC_SHOWTRAC; */
 #define AIC_START(str)
 #endif
 
-#ifdef __FreeBSD__
-int	aicprobe	__P((struct isa_device *));
-int	aicattach	__P((struct isa_device *));
-#else
-int	aicprobe        __P((struct device *, struct device *, void *));
-void	aicattach       __P((struct device *, struct device *, void *));
-#endif
-void	aic_minphys	__P((struct buf *));
-#ifdef __FreeBSD__
-u_int32_t aic_adapter_info __P((int));
-int	aicintr		__P((int));
-void 	aic_init	__P((struct aic_data *));
-int 	aic_find	__P((struct aic_data *));
-#else
-u_int	aic_adapter_info __P((struct aic_softc *));
-int	aicintr         __P((struct aic_softc *));
-void	aic_init        __P((struct aic_softc *));
-#endif
-void	aic_done	__P((struct acb *));
-#ifdef __FreeBSD__
-int32_t	aic_scsi_cmd	__P((struct scsi_xfer *));
-int	aic_poll	__P((int, struct acb *));
-#else
-int	aic_scsi_cmd    __P((struct scsi_xfer *));
-int	aic_poll        __P((struct aic_softc *, struct acb *));
-#endif
+static int	aicprobe	__P((struct isa_device *));
+static int	aicattach	__P((struct isa_device *));
+static void	aic_minphys	__P((struct buf *));
+static u_int32_t	aic_adapter_info __P((int));
+static void 	aic_init	__P((struct aic_data *));
+static int 	aic_find	__P((struct aic_data *));
+static void	aic_done	__P((struct acb *));
+static void	aic_dataout	__P((struct aic_data *aic));
+static void	aic_datain	__P((struct aic_data *aic));
+static int32_t	aic_scsi_cmd	__P((struct scsi_xfer *));
+static int	aic_poll	__P((struct aic_data *aic, struct acb *));
 void	aic_add_timeout __P((struct acb *, int));
 void	aic_remove_timeout __P((struct acb *));
-#ifdef __FreeBSD__
-timeout_t aic_timeout;
-void	aic_sched	__P((struct aic_data *));
-void	aic_scsi_reset	__P((struct aic_data *));
-#else
-void	aic_timeout     __P((void *arg));
-int	aic_find        __P((struct aic_softc *));
-void	aic_sched       __P((struct aic_softc *));
-void	aic_scsi_reset  __P((struct aic_softc *));
-#endif
+static	void	aic6360_reset	__P((struct aic_data *aic));
+static	u_short	aicphase	__P((struct aic_data *aic));
+static	void	aic_msgin	__P((struct aic_data *aic));
+static	void	aic_msgout	__P((struct aic_data *aic));
+static timeout_t aic_timeout;
+static void	aic_sched	__P((struct aic_data *));
+static void	aic_scsi_reset	__P((struct aic_data *));
 #if AIC_DEBUG
-void	aic_print_active_acb();
-void	aic_dump_driver();
-void	aic_dump6360();
+void	aic_print_active_acb	__P((void));
+void	aic_dump6360		__P((void));
+void	aic_dump_driver		__P((void));
 #endif
 
 /* Linkup to the rest of the kernel */
-#ifdef __FreeBSD__
 struct isa_driver aicdriver = {
     aicprobe, aicattach, "aic"
 };
 
 static int aicunit = 0;
-#else
-struct cfdriver aiccd = {
-	NULL, "aic", aicprobe, aicattach, DV_DULL, sizeof(struct aic_softc)
-#endif
 
-struct scsi_adapter aic_switch = {
+static struct scsi_adapter aic_switch = {
 	aic_scsi_cmd,
 	aic_minphys,
 	0,
 	0,
 	aic_adapter_info,
 	"aic"
-#ifdef __FreeBSD__
 	,0 , 0
-#endif
 };
 
-struct scsi_device aic_dev = {
+static struct scsi_device aic_dev = {
 	NULL,			/* Use default error handler */
 	NULL,			/* have a queue, served by this */
 	NULL,			/* have no async handler */
@@ -750,27 +698,89 @@ struct scsi_device aic_dev = {
 	0
 };
 
-static struct kern_devconf kdc_aic[NAIC] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"aic", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* start out in unconfig state */
-	"Adaptec AIC-6360 SCSI host adapter chipset",
-	DC_CLS_MISC		/* host adapters aren't special */
-} };
 
-static inline void
-aic_registerdev(struct isa_device *id)
+/* PCCARD suport */
+#include "card.h"
+#if NCARD > 0
+#include <sys/select.h>
+#include <pccard/cardinfo.h>
+#include <pccard/driver.h>
+#include <pccard/slot.h>
+
+static int aicinit(struct pccard_devinfo *);		/* init device */
+void aicunload(struct pccard_devinfo *);		/* Disable driver */
+static int aic_card_intr(struct pccard_devinfo *);	/* Interrupt handler */
+
+static struct pccard_device aic_info = {
+	"aic",
+	aicinit,
+	aicunload,
+	aic_card_intr,
+	0,			/* Attributes - presently unused */
+	&bio_imask		/* Interrupt mask for device */
+};
+
+DATA_SET(pccarddrv_set, aic_info);
+
+/*
+ * Initialize the device - called from Slot manager.
+ */
+int
+aicinit(struct pccard_devinfo *devi)
 {
-	if(id->id_unit)
-		kdc_aic[id->id_unit] = kdc_aic[0];
-	kdc_aic[id->id_unit].kdc_unit = id->id_unit;
-	kdc_aic[id->id_unit].kdc_parentdata = id;
-	dev_attach(&kdc_aic[id->id_unit]);
+	static int already_aicinit[NAIC];
+
+	/* validate unit number */
+	if (devi->isahd.id_unit >= NAIC)
+		return(ENODEV);
+	/* Make sure it isn't already initialised */
+	if (already_aicinit[devi->isahd.id_unit] == 1) {
+		if (aicattach(&devi->isahd) == 0)
+			return(ENXIO);
+		return(0);
+	}
+	/*
+	 * Probe the device. If a value is returned, the
+	 * device was found at the location.
+	 */
+	if (aicprobe(&devi->isahd) == 0)
+		return(ENXIO);
+
+	if (aicattach(&devi->isahd) == 0)
+		return(ENXIO);
+	already_aicinit[devi->isahd.id_unit] = 1;
+	return(0);
 }
 
+/*
+ * aicunload - unload the driver and clear the table.
+ * XXX TODO:
+ * This is called usually when the card is ejected, but
+ * can be caused by the modunload of a controller driver.
+ * The idea is reset the driver's view of the device
+ * and ensure that any driver entry points such as
+ * read and write do not hang.
+ */
+void
+aicunload(struct pccard_devinfo *devi)
+{
+	printf("aic%d: unload\n", devi->isahd.id_unit);
+#if 0
+	aicstop(devi->isahd.id_unit);
+#endif
+}
+
+/*
+ * card_intr - Shared interrupt called from front end of PC-Card handler.
+ */
+static int
+aic_card_intr(struct pccard_devinfo *devi)
+{
+	aicintr(devi->isahd.id_unit);
+	return(1);
+}
+#endif /* NCARD > 0 */ 
+
 /*
  * INITIALIZATION ROUTINES (probe, attach ++)
  */
@@ -779,34 +789,32 @@ aic_registerdev(struct isa_device *id)
  * aicprobe: probe for AIC6360 SCSI-controller
  * returns non-zero value if a controller is found.
  */
-int
-#ifdef __FreeBSD__
+static int
 aicprobe(dev)
 	struct isa_device *dev;
-#else
-aicprobe(parent, self, aux)
-	struct device *parent, *self;
-	void *aux;
-#endif
 {
-#ifdef __FreeBSD__
-	int	unit = aicunit;
 	struct aic_data *aic;
+#if NCARD > 0
+	int     unit = dev->id_unit;
+	int     aic_reg_drv[NAIC];
 #else
-	struct aic_softc *aic = (void *)self;
-	struct isa_attach_args *ia = aux;
+	int	unit = aicunit;
 #endif
-	int i, len, ic;
 
-#ifdef __FreeBSD__
 	if (unit >= NAIC) {
 		printf("aic%d: unit number too high\n", unit);
 		return 0;
 	}
 	dev->id_unit = unit;
+
+#if NCARD > 0
+	if (!aic_reg_drv[unit])
+		aic_reg_drv[unit] = 1;
+#endif
+
 	/*
-	* Allocate a storage area for us
-	*/
+	 * Allocate a storage area for us
+	 */
 	if (aicdata[unit]) {
 	       printf("aic%d: memory already allocated\n", unit);
 	       return 0;
@@ -819,9 +827,15 @@ aicprobe(parent, self, aux)
 	bzero(aic, sizeof(struct aic_data));
 	aicdata[unit] = aic;
 	aic->iobase = dev->id_iobase;
-#ifndef DEV_LKM
-	aic_registerdev(dev);
-#endif /* not DEV_LKM */
+#ifdef PC98
+	if (AIC_TYPE98(dev->id_flags) == AIC98_100) {
+		/* PC-9801-100 */
+		aic->aicport = aicport_100;
+	} else {
+		/* generic card */
+		aic->aicport = aicport_generic;
+	}
+#endif
 
 	if (aic_find(aic) != 0) {
 		aicdata[unit] = NULL;
@@ -829,53 +843,23 @@ aicprobe(parent, self, aux)
 		return 0;
 	}
 	aicunit++;
-	return 0x20;
-#else
-#ifdef NEWCONFIG
-	if (ia->ia_iobase == IOBASEUNK)
-		return 0;
-#endif
-	aic->iobase = ia->ia_iobase;
-	if (aic_find(aic) != 0)
-		return 0;
-#ifdef NEWCONFIG
-	if (ia->ia_irq == IRQUNK)
-		ia->ia_irq = (1 << aic->aic_int);
-	else if (ia->ia_irq != (1 << aic->aic_int)) {
-		printf("aic%d: irq mismatch, %x != %x\n",
-			aic->sc_dev.dv_unit, ia->ia_irq, 1 << aic->aic_int);
-		return 0;
-	}
 
-	if (ia->ia_drq == DRQUNK)
-		ia->ia_drq = aic->aic_dma;
-	else if (ia->ia_drq != aic->aic_dma) {
-		printf("aic%d: drq mismatch, %x != %x\n",
-			aic->sc_dev.dv_unit, ia->ia_drq, aic->aic_dma);
-		return 0;
-#endif
-		}
-	ia->ia_msize = 0;
-	ia->ia_iosize = 0x20;
-	return 1;
-	}
+#ifdef PC98
+	if (AIC_TYPE98(dev->id_flags) == AIC98_100)
+		return 0x40;
 #endif
+	return 0x20;
 }
 
 /* Do the real search-for-device.
  * Prerequisite: aic->iobase should be set to the proper value
  */
-int
+static int
 aic_find(aic)
-#ifdef __FreeBSD__
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 {
 	u_short iobase = aic->iobase;
 	char chip_id[sizeof(IDSTRING)];	/* For chips that support it */
-	char *start;
 	int i;
 
 	/* Remove aic6360 from possible powerdown mode */
@@ -916,35 +900,16 @@ aic_find(aic)
 	return 0;
 }
 
-#ifndef __FreeBSD__
-int
-aicprint()
-{
-}
-#endif
-
 /*
  * Attach the AIC6360, fill out some high and low level data structures
  */
-#ifdef __FreeBSD__
-int
+static int
 aicattach(dev)
 	struct isa_device *dev;
-#else
-void
-aicattach(parent, self, aux)
-	struct device *parent, *self;
-	void *aux;
-#endif
 {
-#ifdef __FreeBSD__
 	int unit = dev->id_unit;
 	struct aic_data *aic = aicdata[unit];
 	struct scsibus_data *scbus;
-#else
-	struct isa_attach_args *ia = aux;
-	struct aic_softc *aic = (void *)self;
-#endif
 
 	AIC_TRACE(("aicattach\n"));
 	aic->state = 0;
@@ -954,16 +919,12 @@ aicattach(parent, self, aux)
 	/*
 	 * Fill in the prototype scsi_link
 	 */
-#ifdef __FreeBSD__
 	aic->sc_link.adapter_unit = unit;
-#else
-	aic->sc_link.adapter_softc = aic;
-#endif
 	aic->sc_link.adapter_targ = AIC_SCSI_HOSTID;
+	aic->sc_link.adapter_softc = aic;
 	aic->sc_link.adapter = &aic_switch;
 	aic->sc_link.device = &aic_dev;
 
-#ifdef __FreeBSD__
 	/*
 	 * Prepare the scsibus_data area for the upperlevel
 	 * scsi code.
@@ -976,22 +937,9 @@ aicattach(parent, self, aux)
 	/*
 	 * ask the adapter what subunits are present
 	 */
-	kdc_aic[unit].kdc_state = DC_BUSY; /* host adapters are always busy */
 	scsi_attachdevs(scbus);
 
 	return 1;
-#else
-	printf("\n");
-#ifdef NEWCONFIG
-	isa_establish(&aic->sc_id, &aic->sc_dev);
-#endif
-	aic->sc_ih.ih_fun = aicintr;
-	aic->sc_ih.ih_arg = aic;
-	aic->sc_ih.ih_level = IPL_BIO;
-	intr_establish(ia->ia_irq, &aic->sc_ih);
-
-	config_found(self, &aic->sc_link, aicprint);
-#endif
 }
 
 
@@ -1002,11 +950,7 @@ aicattach(parent, self, aux)
  */
 static void
 aic6360_reset(aic)
-#ifdef __FreeBSD__
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 {
 	u_short iobase = aic->iobase;
 
@@ -1038,41 +982,25 @@ aic6360_reset(aic)
 }
 
 /* Pull the SCSI RST line for 500 us */
-void
+static void
 aic_scsi_reset(aic)
-#ifdef __FreeBSD__
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 {
 	u_short iobase = aic->iobase;
 
 	outb(SCSISEQ, SCSIRSTO);
-#ifdef __FreeBSD__
 	DELAY(500);
-#else
-	delay(500);
-#endif
 	outb(SCSISEQ, 0);
-#ifdef __FreeBSD__
 	DELAY(50);
-#else
-	delay(50);
-#endif
 }
 
 /*
  * Initialize aic SCSI driver, also (conditonally) reset the SCSI bus.
  * The reinitialization is still buggy (e.g. on SCSI resets).
  */
-void
+static void
 aic_init(aic)
-#ifdef __FreeBSD__
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 {
 	u_short iobase = aic->iobase;
 	struct acb *acb;
@@ -1154,26 +1082,17 @@ aic_init(aic)
  * This function is called by the higher level SCSI-driver to queue/run
  * SCSI-commands.
  */
-#ifdef __FreeBSD__
-int32_t
-#else
-int
-#endif
+static int32_t
 aic_scsi_cmd(xs)
 	struct scsi_xfer *xs;
 {
 	struct scsi_link *sc = xs->sc_link;
-#ifdef __FreeBSD__
-	int	unit = sc->adapter_unit;
-	struct aic_data *aic = aicdata[unit];
-#else
-	struct aic_softc *aic = sc->adapter_softc;
-#endif
+	struct aic_data *aic;
 	struct acb 	*acb;
 	int s = 0;
 	int flags;
-	u_short iobase = aic->iobase;
 
+	aic = (struct aic_data *)sc->adapter_softc;
 	SC_DEBUG(sc, SDEV_DB2, ("aic_scsi_cmd\n"));
 	AIC_TRACE(("aic_scsi_cmd\n"));
 	AIC_MISC(("[0x%x, %d]->%d ", (int)xs->cmd->opcode, xs->cmdlen,
@@ -1228,36 +1147,26 @@ aic_scsi_cmd(xs)
 	}
 
 	/* Not allowed to use interrupts, use polling instead */
-#ifdef __FreeBSD__
-	return aic_poll(unit, acb);
-#else
 	return aic_poll(aic, acb);
-#endif
 }
 
 /*
  * Adjust transfer size in buffer structure
  */
-void
+static void
 aic_minphys(bp)
 	struct buf *bp;
 {
 
 	AIC_TRACE(("aic_minphys\n"));
-	if (bp->b_bcount > (AIC_NSEG << PGSHIFT))
-		bp->b_bcount = (AIC_NSEG << PGSHIFT);
+	if (bp->b_bcount > (AIC_NSEG << PAGE_SHIFT))
+		bp->b_bcount = (AIC_NSEG << PAGE_SHIFT);
 }
 
 
-#ifdef __FreeBSD__
-u_int32_t
+static u_int32_t
 aic_adapter_info(unit)
 	int	unit;
-#else
-u_int
-aic_adapter_info(aic)
-	struct aic_softc *aic;
-#endif
 {
 
 	AIC_TRACE(("aic_adapter_info\n"));
@@ -1267,19 +1176,11 @@ aic_adapter_info(aic)
 /*
  * Used when interrupt driven I/O isn't allowed, e.g. during boot.
  */
-int
-#ifdef __FreeBSD__
-aic_poll(unit, acb)
-	int	unit;
-#else
+static int
 aic_poll(aic, acb)
-	struct aic_softc *aic;
-#endif
+	struct aic_data *aic;
 	struct acb *acb;
 {
-#ifdef __FreeBSD__
-	struct aic_data *aic = aicdata[unit];
-#endif
 	register u_short iobase = aic->iobase;
 	struct scsi_xfer *xs = acb->xs;
 	int count = xs->timeout * 10;
@@ -1287,18 +1188,10 @@ aic_poll(aic, acb)
 	AIC_TRACE(("aic_poll\n"));
 	while (count) {
 		if (inb(DMASTAT) & INTSTAT)
-#ifdef __FreeBSD__
-			aicintr(unit);
-#else
-			aicintr(aic);
-#endif
+			aicintr(xs->sc_link->adapter_unit);
 		if (xs->flags & ITSDONE)
 			break;
-#ifdef __FreeBSD__
 		DELAY(100);
-#else
-		delay(100);
-#endif
 		count--;
 	}
 	if (count == 0) {
@@ -1325,11 +1218,7 @@ aic_poll(aic, acb)
  */
 static inline u_short
 aicphase(aic)
-#ifdef __FreeBSD__
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 {
 	register u_short iobase = aic->iobase;
 	register u_char sstat0, sstat1, scsisig;
@@ -1355,19 +1244,14 @@ aicphase(aic)
  * save us an unecessary interrupt just to get things going.  Should only be
  * called when state == AIC_IDLE and at bio pl.
  */
-void
+static void
 aic_sched(aic)
-#ifdef __FreeBSD__
 	register struct aic_data *aic;
-#else
-	register struct aic_softc *aic;
-#endif
 {
-	struct scsi_xfer *xs;
 	struct scsi_link *sc;
 	struct acb *acb;
 	u_short iobase = aic->iobase;
-	int t, l;
+	int t;
 	u_char simode0, simode1, scsiseq;
 
 	AIC_TRACE(("aic_sched\n"));
@@ -1415,20 +1299,13 @@ aic_sched(aic)
 /*
  * POST PROCESSING OF SCSI_CMD (usually current)
  */
-void
+static void
 aic_done(acb)
 	struct acb *acb;
 {
 	struct scsi_xfer *xs = acb->xs;
 	struct scsi_link *sc = xs->sc_link;
-#ifdef __FreeBSD__
-	int unit = sc->adapter_unit;
-	struct aic_data *aic = aicdata[unit];
-#else
-	struct aic_softc *aic = sc->adapter_softc;
-#endif
-	u_short iobase = aic->iobase;
-	struct acb *acb2;
+	struct aic_data *aic = (struct aic_data *)sc->adapter_softc;
 
 	AIC_TRACE(("aic_done "));
 
@@ -1500,6 +1377,10 @@ aic_done(acb)
 	 * longer busy.  This code is sickening, but it works.
 	 */
 	if (acb == aic->nexus) {
+#if NAPM > 0 && NCARD > 0
+		/* SlimSCSI dies without this when it resumes from suspend */
+		aic->nexus = NULL;
+#endif
 		aic->state = AIC_IDLE;
 		aic->tinfo[sc->target].lubusy &= ~(1<<sc->lun);
 		aic_sched(aic);
@@ -1520,12 +1401,8 @@ aic_done(acb)
 		else if (acb->chain.tqe_next) {
 			TAILQ_REMOVE(&aic->ready_list, acb, chain);
 		} else {
-#ifdef __FreeBSD__
-			printf("aic%d: can't find matching acb\n", unit);
-#else
-			printf("%s: can't find matching acb\n",
-			    aic->sc_dev.dv_xname);
-#endif
+			printf("aic%d: can't find matching acb\n", 
+				xs->sc_link->adapter_unit);
 			Debugger("aic6360");
 			fatal_if_no_DDB();
 		}
@@ -1582,11 +1459,7 @@ aic_done(acb)
  */
 static void
 aic_msgin(aic)
-#ifdef __FreeBSD__
 	register struct aic_data *aic;
-#else
-	register struct aic_softc *aic;
-#endif
 {
 	register u_short iobase = aic->iobase;
 	int spincount, extlen;
@@ -1657,11 +1530,7 @@ aic_msgin(aic)
 		 * Therefore we will spinwait for some small amount of time
 		 * waiting for the next byte.
 		 */
-#ifdef __FreeBSD__
 		spincount = DELAYCOUNT * AIC_MSGI_SPIN;
-#else
-		spincount = MINIFUDGE(delaycount) * AIC_MSGI_SPIN;
-#endif
 		LOGLINE(aic);
 		while (spincount-- && !((sstat1 = inb(SSTAT1)) & REQINIT))
 			;
@@ -1844,16 +1713,11 @@ aic_msgin(aic)
  */
 static void
 aic_msgout(aic)
-#ifdef __FreeBSD__
 	register struct aic_data *aic;
-#else
-	register struct aic_softc *aic;
-#endif
 {
 	register u_short iobase = aic->iobase;
 	struct aic_tinfo *ti;
 	struct acb *acb;
-	u_char dmastat, scsisig;
 
 	/* First determine what to send. If we haven't seen a
 	 * phasechange this is a retransmission request.
@@ -1958,13 +1822,12 @@ aic_msgout(aic)
  * This new revision has been optimized (I tried) to make the common case fast,
  * and the rarer cases (as a result) somewhat more comlex
  */
-void
+static void
 aic_dataout(aic)
 	register struct aic_data *aic;
 {
 	register u_short iobase = aic->iobase;
 	register u_char dmastat;
-	struct acb *acb = aic->nexus;
 	int amount, olddleft = aic->dleft;
 #define DOUTAMOUNT 128		/* Full FIFO */
 
@@ -2075,18 +1938,13 @@ phasechange:
  * transferred.  This, is OK for fast targets, but not so smart for slow
  * targets which don't disconnect or for huge transfers.
  */
-void
+static void
 aic_datain(aic)
-#ifdef __FreeBSD__
 	register struct aic_data *aic;
-#else
-	register struct aic_softc *aic;
-#endif
 {
 	register u_short iobase = aic->iobase;
 	register u_char dmastat;
-	struct acb *acb = aic->nexus;
-	int amount, olddleft = aic->dleft;
+	int olddleft = aic->dleft;
 #define DINAMOUNT 128		/* Default amount of data to transfer */
 
 	/* Enable DATA IN transfers */
@@ -2203,26 +2061,15 @@ aic_datain(aic)
  * 2) doesn't support synchronous transfers properly (yet)
  */
 
-int
-#ifdef __FreeBSD__
+void
 aicintr(int unit)
-#else
-aicintr(aic)
-	register struct aic_softc *aic;
-#endif
 {
-#ifdef __FreeBSD__
 	struct aic_data *aic = aicdata[unit];
-#else
-	struct scsi_xfer *xs;
-#endif
 	register struct acb *acb;
 	register struct scsi_link *sc;
 	register u_short iobase = aic->iobase;
 	struct aic_tinfo *ti;
-	int done, amount;
-	u_char sstat0, sstat1, scsisig, dmastat, sstat2;
-	u_char scsiseq, simode0, simode1, sxfrctl0;
+	u_char sstat0, sstat1,  sstat2, sxfrctl0;
 
 
 	LOGLINE(aic);
@@ -2245,7 +2092,7 @@ aicintr(aic)
 			aic_init(aic); /* Restart everything */
 			LOGLINE(aic);
 			outb(DMACNTRL0, INTEN);
-			return 1;
+			return;
 		} else {
 			printf("aic: SCSI bus parity error\n");
 			outb(CLRSINT1, CLRSCSIPERR);
@@ -2307,7 +2154,7 @@ aicintr(aic)
 			if (aic->phase & PH_PSBIT) {
 				LOGLINE(aic);
 				outb(DMACNTRL0, INTEN);
-				return 1; /* Come back when REQ is set */
+				return;	/* Come back when REQ is set */
 			}
 			if (aic->phase == PH_MSGI)
 				aic_msgin(aic);	/* Handle identify message */
@@ -2319,13 +2166,13 @@ aicintr(aic)
 				Debugger("aic6360");
 				fatal_if_no_DDB();
 				aic_init(aic);
-				return 1;
+				return;
 			}
 			if (aic->state != AIC_HASNEXUS) {/* IDENTIFY fail?! */
 				printf("aic at line %d: identify failed\n",
 				    __LINE__);
 				aic_init(aic);
-				return 1;
+				return;
 			} else {
 				outb(SIMODE1,
 				    ENSCSIRST|ENBUSFREE|ENSCSIPERR|ENREQINIT);
@@ -2375,17 +2222,13 @@ aicintr(aic)
 				 * a "spurious" selection. Shouldn't happen.
 				 */
 				printf("aic: unexpected busfree\n");
-#ifdef __FreeBSD__
 				acb->xs->error = XS_DRIVER_STUFFUP;
-#else
-				xs->error = XS_DRIVER_STUFFUP;
-#endif
 				untimeout(aic_timeout, (caddr_t)acb);
 				aic_done(acb);
 			}
 			LOGLINE(aic);
 			outb(DMACNTRL0, INTEN);
-			return 1;
+			return;
 		} else if (sstat1 & SELTO) {
 			/* Selection timed out. What to do:
 			 * Disable selections out and fail the command with
@@ -2406,7 +2249,7 @@ aicintr(aic)
 			aic_done(acb);
 			LOGLINE(aic);
 			outb(DMACNTRL0, INTEN);
-			return 1;
+			return;
 		} else {
 			/* Assume a bus free interrupt.  What to do:
 			 * Start selecting.
@@ -2419,7 +2262,7 @@ aicintr(aic)
 #endif
 			LOGLINE(aic);
 			outb(DMACNTRL0, INTEN);
-			return 1;
+			return;
 		}
 	}
 	/* Driver is now in state AIC_HASNEXUS, i.e. we have a current command
@@ -2487,7 +2330,7 @@ aicintr(aic)
 			untimeout(aic_timeout, (caddr_t)acb);
 			aic_done(acb);
 			aic_init(aic);
-			return 1;
+			return;
 		}
 		outb(SXFRCTL0, CHEN);	/* Clear SCSIEN & DMAEN */
 		outb(SIMODE0, 0);
@@ -2505,7 +2348,7 @@ aicintr(aic)
 			untimeout(aic_timeout, (caddr_t)acb);
 			aic_done(acb);
 			aic_init(aic);
-			return 1;
+			return;
 		}
 		outb(SIMODE1, ENSCSIRST|ENBUSFREE|ENSCSIPERR|ENREQINIT);
 		/* Enabled ints: BUSFREE, SCSIPERR, SCSIRSTI (unexpected)
@@ -2571,26 +2414,18 @@ aicintr(aic)
 	}
 	LOGLINE(aic);
 	outb(DMACNTRL0, INTEN);
-	return 1;
+	return;
 }
 
-void
+static void
 aic_timeout(void *arg1) {
 	int s = splbio();
 	struct acb *acb = (struct acb *)arg1;
-#ifdef __FreeBSD__
 	int     unit;
 	struct aic_data *aic;
-#else
-	struct aic_softc *aic;
-#endif
 
-#ifdef __FreeBSD__
 	unit = acb->xs->sc_link->adapter_unit;
 	aic = aicdata[unit];
-#else
-	aic = acb->xs->sc_link->adapter_softc;
-#endif
 	sc_print_addr(acb->xs->sc_link);
 	acb->xs->error = XS_TIMEOUT;
 	printf("timed out\n");
@@ -2640,11 +2475,7 @@ void
 aic_print_active_acb()
 {
 	struct acb *acb;
-#ifdef __FreeBSD__
 	struct aic_data *aic = aicdata[0];
-#else
-	struct aic_softc *aic = aiccd.cd_devs[0];
-#endif
 
 	printf("ready list:\n");
 	for (acb = aic->ready_list.tqh_first; acb; acb = acb->chain.tqe_next)
@@ -2676,11 +2507,7 @@ aic_dump6360()
 void
 aic_dump_driver()
 {
-#ifdef __FreeBSD__
 	struct aic_data *aic = aicdata[0];
-#else
-	struct aic_softc *aic = aiccd.cd_devs[0];
-#endif
 	struct aic_tinfo *ti;
 	int i;
 
