@@ -2,10 +2,10 @@
  *
  * Current version supports:
  *
- * 	- Trust AmiScan BW (GI1904 chipset)
+ * 	- AmiScan (Mustek) Color and BW hand scanners (GI1904 chipset)
  *
  * Copyright (c) 1995 Gunther Schadow.  All rights reserved.
- * Copyright (c) 1995 Luigi Rizzo.  All rights reserved.
+ * Copyright (c) 1995,1996,1997 Luigi Rizzo.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,7 +34,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * $FreeBSD: stable/2.1/sys/i386/isa/asc.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/asc.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "asc.h"
@@ -43,16 +43,11 @@
 #include "param.h"
 #include "systm.h"
 #include "proc.h"
-#include "user.h"
 #include "buf.h"
 #include "malloc.h"
 #include "kernel.h"
 #include "ioctl.h"
-#include "tty.h"
-#include "uio.h"
-#include "syslog.h"
 
-#include "i386/isa/isa.h"
 #include "i386/isa/isa_device.h"
 #include "i386/isa/ascreg.h"
 
@@ -60,27 +55,21 @@
 #else
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/buf.h>
 #include <sys/malloc.h>
 #include <sys/kernel.h>
 #include <sys/ioctl.h>
-#include <sys/tty.h>
-#include <sys/uio.h>
-#include <sys/syslog.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <machine/asc_ioctl.h>
 
-#include <sys/devconf.h>
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/ascreg.h>
-#ifdef  DEVFS
-#include <sys/devfsext.h>
-
-extern d_open_t ascopen;
-#endif
 
 #endif /* FREEBSD_1_X */
 
@@ -90,7 +79,9 @@ extern d_open_t ascopen;
  ***/
 
 #define PROBE_FAIL    0
-#define PROBE_SUCCESS 5	/* number of io ports... */
+#define PROBE_SUCCESS IO_ASCSIZE
+#define ATTACH_FAIL   0
+#define ATTACH_SUCCESS 1
 #define SUCCESS       0
 #define FAIL         -1
 #define INVALID       FAIL
@@ -98,7 +89,7 @@ extern d_open_t ascopen;
 #define DMA1_READY  0x08
 #define ASCDEBUG
 #ifdef ASCDEBUG
-#	define lprintf if(scu->flags & DEBUG) printf
+#	define lprintf if(scu->flags & FLAG_DEBUG) printf
 #else
 #	define lprintf (void)
 #endif
@@ -115,7 +106,7 @@ extern d_open_t ascopen;
 #define DBUG_MASK 0x20
 #define FRMT_MASK 0x18    /* output format */
 #define FRMT_RAW  0x00    /* output bits as read from scanner */
-#define FRMT_GRAY 0x10    /* output graymap (not implemented yet) */
+#define FRMT_GRAY 0x1     /* output gray mode for color scanner */
 #define FRMT_PBM  0x08    /* output pbm format */
 #define FRMT_PGM  0x18
 
@@ -123,6 +114,7 @@ extern d_open_t ascopen;
  *** THE GEMOMETRY TABLE
  ***/
 
+#define GREY_LINE 826 /* 825, or 826 , or 550 ??? */
 static const struct asc_geom {
   int dpi;     /* dots per inch */
   int dpl;     /* dots per line */
@@ -137,6 +129,8 @@ static const struct asc_geom {
   { 300, 1240, 155, ASC_RES_300},
   { 200, 832, 104, ASC_RES_200},
   { 100, 416, 52, ASC_RES_100},
+  { 200, 3*GREY_LINE, 3*GREY_LINE, 0 /* returned by color scanner */},
+  { 200, GREY_LINE, GREY_LINE, 0 /* color scanner, grey mode */},
   { INVALID, 416, 52, INVALID } /* terminator */
 };
 
@@ -153,12 +147,14 @@ struct _sbuf {
 };
 
 struct asc_unit {
+  long thedev;	/* XXX */
   int base;		/* base address */
   int dma_num;		/* dma number */
   char    dma_byte;       /* mask of byte for setting DMA value */
   char    int_byte;       /* mask of byte for setting int value */
   char    cfg_byte;       /* mirror of byte written to config reg (ASC_CFG). */
   char    cmd_byte;       /* mirror of byte written to cmd port (ASC_CMD)*/
+  char   portf_byte;
   int flags;
 #define ATTACHED 	0x01
 #define OPEN     	0x02
@@ -167,7 +163,7 @@ struct asc_unit {
 #define SLEEPING	0x10
 #define SEL_COLL	0x20
 #define PBM_MODE	0x40
-#define DEBUG    	0x80
+#define FLAG_DEBUG    	0x80
   int     geometry;       /* resolution as geomtab index */
   int     linesize;       /* length of one scan line (from geom.table) */
   int     blen;           /* length of buffer in lines */
@@ -181,48 +177,51 @@ struct asc_unit {
 #endif
   int     height;         /* height, for pnm modes */
   size_t  bcount;         /* bytes to read, for pnm modes */
-} unittab[NASC];                                 
+#ifdef DEVFS
+  void *devfs_asc;	  /* storage for devfs tokens (handles) */
+  void *devfs_ascp;
+  void *devfs_ascd;
+  void *devfs_ascpd;
+#endif
+};
+
+static struct asc_unit unittab[NASC];                                 
 
 /*** I could not find a reasonable buffer size limit other than by
  *** experiments. MAXPHYS is obviously too much, while DEV_BSIZE and
  *** PAGE_SIZE are really too small. There must be something wrong
  *** with isa_dmastart/isa_dmarangecheck HELP!!!
+ ***
+ *** Note, must be DEFAULT_BLEN * samples_per_line <= MAX_BUFSIZE
  ***/
-#define MAX_BUFSIZE 0x3000 
-#define DEFAULT_BLEN 20
+#define MAX_BUFSIZE 0xb000 /* XXX was 0x3000 */
+#define DEFAULT_BLEN 16
 
 /***
  *** THE PER-DRIVER RECORD FOR ISA.C
  ***/
-int ascprobe (struct isa_device *isdp);
-int ascattach(struct isa_device *isdp);
+static int ascprobe (struct isa_device *isdp);
+static int ascattach(struct isa_device *isdp);
 struct isa_driver ascdriver = { ascprobe, ascattach, "asc" };
 
 #ifndef FREEBSD_1_X
-struct asc_softc {
-    struct isa_device *dev;
-} asc_softc[NASC];
 
-static struct kern_devconf kdc_asc[NASC] = { {
-	0, 0, 0,                /* filled in by dev_attach */
-	"asc", 0, { MDDT_ISA, 0, "tty" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,        /* state */
-	"GI1904 Hand scanner",
-	DC_CLS_MISC
-} };
+static d_open_t		ascopen;
+static d_close_t	ascclose;
+static d_read_t		ascread;
+static d_ioctl_t	ascioctl;
+static d_select_t	ascselect;
 
-static inline void
-asc_registerdev(struct isa_device *id)
-{
-        if(id->id_unit)
-                kdc_asc[id->id_unit] = kdc_asc[0];
-        kdc_asc[id->id_unit].kdc_unit = id->id_unit;
-        kdc_asc[id->id_unit].kdc_isa = id;
-        dev_attach(&kdc_asc[id->id_unit]);
-}
+#define CDEV_MAJOR 71
+
+static struct cdevsw asc_cdevsw = 
+	{ ascopen,      ascclose,       ascread,        nowrite,        /*71*/
+	  ascioctl,     nostop,         nullreset,      nodevtotty, /* asc */   
+	  ascselect,    nommap,         NULL,	"asc",	NULL,	-1 };
+
+#define STATIC static
+#else
+#define STATIC
 #endif /* ! FREEBSD_1_X */
 
 /***
@@ -267,6 +266,12 @@ get_resolution(struct asc_unit *scu)
 	lprintf("asc.get_resolution: %d dpi\n",geomtab[i].dpi);
 	scu->geometry = i;
     }
+    scu->portf_byte=0; /* default */
+    if (geomtab[scu->geometry].g_res==0 && !(scu->thedev&FRMT_GRAY)) {
+	/* color scanner seems to require this */
+	scu->portf_byte=2;
+	/* scu->geometry++; */
+    }
     scu->linesize = geomtab[scu->geometry].bpl;
     scu->height = geomtab[scu->geometry].dpl; /* default... */
 }
@@ -274,6 +279,7 @@ get_resolution(struct asc_unit *scu)
 /***
  *** buffer_allocate
  ***	allocate/reallocate a buffer
+ ***	Now just checks that the preallocated buffer is large enough.
  ***/
 
 static int
@@ -290,24 +296,6 @@ buffer_allocate(struct asc_unit *scu)
       size= ( (MAX_BUFSIZE+scu->linesize-1) / scu->linesize)*scu->linesize;
       lprintf("asc.buffer_allocate: 0x%x bytes are too much, try 0x%x\n",
 	  size1, size);
-      /* return ENOMEM; */
-  }
-
-  if ( scu->sbuf.base != NULL )
-    if ( scu->sbuf.size == size ) {
-	lprintf("asc.buffer_allocate: keep old buffer\n");
-	return SUCCESS;
-    } else {
-	lprintf("asc.buffer_allocate: release old buffer\n");
-	free( scu->sbuf.base, M_DEVBUF );
-    }
-
-  scu->sbuf.base = (char *)malloc(size, M_DEVBUF, M_WAITOK);
-
-  if ( scu->sbuf.base == NULL ) {
-      lprintf("asc.buffer_allocate: "
-	     "buffer allocatation failed for size = 0x%x\n",
-	     scu->sbuf.size);
       return ENOMEM;
   }
 
@@ -328,6 +316,21 @@ static void
 dma_restart(struct asc_unit *scu)
 {
     unsigned char al=scu->cmd_byte;
+
+    if (geomtab[scu->geometry].g_res==0) {/* color */
+	isa_dmastart(B_READ, scu->sbuf.base+scu->sbuf.wptr,
+	    scu->linesize + 90 /* XXX */ , scu->dma_num);
+	/*
+	 * looks like we have to set and then clear this
+	 * bit to enable the scanner to send interrupts
+	 */
+	outb( ASC_CMD, al |= 4 ); /* seems to disable interrupts */
+#if 0
+	outb( ASC_CMD, al |= 8 ); /* ??? seems useless */
+#endif
+	outb( ASC_CMD, al &= 0xfb );
+	scu->cmd_byte = al;
+    } else {					/* normal */
     isa_dmastart(B_READ, scu->sbuf.base+scu->sbuf.wptr,
 	scu->linesize, scu->dma_num);
     /*** this is done in sub_20, after dmastart ? ***/  
@@ -339,6 +342,7 @@ dma_restart(struct asc_unit *scu)
 #else
     outb( ASC_CMD, ASC_OPERATE); 
 #endif
+    }
     scu->flags |= DMA_ACTIVE;
 }
 
@@ -349,7 +353,7 @@ dma_restart(struct asc_unit *scu)
 /*** asc_reset
  ***	resets the scanner and the config bytes...
  ***/
-void
+static void
 asc_reset(struct asc_unit *scu)
 {
   scu->cfg_byte = 0 ; /* clear... */
@@ -370,7 +374,7 @@ asc_reset(struct asc_unit *scu)
  ***	- if address group matches (status byte has reasonable value)
  ***	  cannot check interrupt/dma, only clear the config byte.
  ***/
-int
+static int
 ascprobe (struct isa_device *isdp)
 {
   int unit = isdp->id_unit;
@@ -378,7 +382,7 @@ ascprobe (struct isa_device *isdp)
   int stb;
 
   scu->base = isdp->id_iobase; /*** needed by the following macros ***/
-  scu->flags = DEBUG;
+  scu->flags = FLAG_DEBUG;
 
   if ( isdp->id_iobase < 0 ) {
       lprintf("asc%d.probe: no iobase given\n", unit);
@@ -391,14 +395,34 @@ ascprobe (struct isa_device *isdp)
       return PROBE_FAIL;
   }
 
+/*
+ * NOTE NOTE NOTE
+ * the new AmiScan Color board uses int 10,11,12 instead of 3,5,10
+ * respectively. This means that the driver must act accordingly.
+ * Unfortunately there is no easy way of telling which board one has,
+ * other than trying to get an interrupt and noticing that it is
+ * missing. use "option ASC_NEW_BOARD" if you have a new board.
+ *
+ */
+
+#if ASC_NEW_BOARD
+#define	ASC_IRQ_A	10
+#define	ASC_IRQ_B	11
+#define	ASC_IRQ_C	12
+#else
+#define	ASC_IRQ_A	3
+#define	ASC_IRQ_B	5
+#define	ASC_IRQ_C	10
+#endif
+
   switch(ffs(isdp->id_irq) - 1) {
-    case 3:
+    case ASC_IRQ_A :
       scu->int_byte = ASC_CNF_IRQ3;
       break;
-    case 5:
+    case ASC_IRQ_B :
       scu->int_byte = ASC_CNF_IRQ5;
       break;
-    case 10:
+    case ASC_IRQ_C :
       scu->int_byte = ASC_CNF_IRQ10;
       break;
 #if 0
@@ -409,7 +433,7 @@ ascprobe (struct isa_device *isdp)
 #endif
     default:
       lprintf("asc%d.probe: unsupported INT %d (only 3, 5, 10)\n",
-		unit, isdp->id_irq);
+		unit, ffs(isdp->id_irq) - 1 );
       return PROBE_FAIL;
   }
   scu->dma_num = isdp->id_drq;
@@ -421,15 +445,15 @@ ascprobe (struct isa_device *isdp)
       scu->dma_byte = ASC_CNF_DMA3;
       break;
     default:
-      lprintf("asc%d.probe: unsupported DMA %d (only 1 or 3)\n", scu->dma_num);
+      lprintf("asc%d.probe: unsupported DMA %d (only 1 or 3)\n", 
+		unit, scu->dma_num);
       return PROBE_FAIL;
   }
   asc_reset(scu);
 /*  lprintf("asc%d.probe: ok\n", unit); */
 
-  scu->flags &= ~DEBUG;
+  scu->flags &= ~FLAG_DEBUG;
   scu->icnt = 0;
-
   return PROBE_SUCCESS;
 }
 
@@ -439,51 +463,67 @@ ascprobe (struct isa_device *isdp)
  ***	finish initialization of unit structure, get geometry value (?)
  ***/
 
-int
+static int
 ascattach(struct isa_device *isdp)
 {
   int unit = isdp->id_unit;
   struct asc_unit *scu = unittab + unit;
-#ifdef DEVFS
-  char buf[32];
-  void *x;
+
+  scu->flags |= FLAG_DEBUG;
+  printf("asc%d: [GI1904/Trust Ami-Scan Grey/Color]\n", unit);
+
+  /*
+   * Initialize buffer structure.
+   * XXX this must be done early to give a good chance of getting a
+   * contiguous buffer.  This wastes memory.
+   */
+#ifdef FREEBSD_1_X
+  /*
+   * The old contigmalloc() didn't have a `low/minpa' arg, and took masks
+   * instead of multipliers for the alignments.
+   */
+  scu->sbuf.base = contigmalloc((unsigned long)MAX_BUFSIZE, M_DEVBUF, M_NOWAIT,
+			        0xfffffful, 0ul, 0xfffful);
+#else
+  scu->sbuf.base = contigmalloc((unsigned long)MAX_BUFSIZE, M_DEVBUF, M_NOWAIT,
+				0ul, 0xfffffful, 1ul, 0x10000ul);
 #endif
-
-  scu->flags |= DEBUG;
-  printf("asc%d: [GI1904/Trust Ami-Scan Grey, type S2]\n", unit);
-
-  /* initialize buffer structure */
-  scu->sbuf.base = NULL;
+  if ( scu->sbuf.base == NULL )
+    {
+      lprintf("asc%d.attach: buffer allocation failed\n", unit);
+      return ATTACH_FAIL;	/* XXX attach must not fail */
+    }
   scu->sbuf.size = INVALID;
   scu->sbuf.rptr  = INVALID;
 
   scu->flags |= ATTACHED;
 /*  lprintf("asc%d.attach: ok\n", unit); */
-  scu->flags &= ~DEBUG;
+  scu->flags &= ~FLAG_DEBUG;
 
 #ifdef FREEBSD_1_X
   scu->selp = (pid_t)0;
 #else
     scu->selp.si_flags=0;
     scu->selp.si_pid=(pid_t)0;
-    kdc_asc[isdp->id_unit].kdc_state = DC_BUSY; /* XXX don't know better */
-    asc_registerdev(isdp);
 #endif
 #ifdef DEVFS
-    sprintf(buf,"asc%d",unit);
-/*            path      name  devsw    minor    type   uid gid perm*/
-   x=dev_add("/misc",   buf,  ascopen, unit<<6, DV_CHR, 0,  0, 0666);
-    sprintf(buf,"asc%dp",unit);
-   x=dev_add("/misc",   buf,  ascopen, ((unit<<6) + FRMT_PBM),
-						DV_CHR, 0,  0, 0666);
-    sprintf(buf,"asc%dd",unit);
-   x=dev_add("/misc",   buf,  ascopen, ((unit<<6) + DBUG_MASK),
-						DV_CHR, 0,  0, 0666);
-    sprintf(buf,"asc%dpd",unit);
-   x=dev_add("/misc",   buf,  ascopen, ((unit<<6) + DBUG_MASK + FRMT_PBM),
-						DV_CHR, 0,  0, 0666);
+#define ASC_UID 0
+#define ASC_GID 13
+    scu->devfs_asc = 
+		devfs_add_devswf(&asc_cdevsw, unit<<6, DV_CHR, ASC_UID,
+				 ASC_GID, 0666, "asc%d", unit);
+    scu->devfs_ascp = 
+		devfs_add_devswf(&asc_cdevsw, ((unit<<6) + FRMT_PBM), DV_CHR, 
+				 ASC_UID,  ASC_GID, 0666, "asc%dp", unit);
+    scu->devfs_ascd = 
+		devfs_add_devswf(&asc_cdevsw, ((unit<<6) + DBUG_MASK), DV_CHR, 
+				 ASC_UID,  ASC_GID, 0666, "asc%dd", unit);
+    scu->devfs_ascpd = 
+		devfs_add_devswf(&asc_cdevsw, ((unit<<6) + DBUG_MASK+FRMT_PBM),
+				 DV_CHR, ASC_UID, ASC_GID, 0666, "asc%dpd", 
+				 unit);
 #endif /*DEVFS*/
-  return 1; /* attach must not fail */
+  return ATTACH_SUCCESS;
 }
 
 /**************************************************************************
@@ -544,27 +584,33 @@ ascintr(int unit)
  ***	don't switch scanner on, wait until first read or ioctls go before
  ***/
 
-int
+STATIC int
 ascopen(dev_t dev, int flags, int fmt, struct proc *p)
 {
-  int unit = UNIT(minor(dev)) & UNIT_MASK;
-  struct asc_unit *scu = unittab + unit;
-  
-  lprintf("asc%d.open: minor %d icnt %d\n", unit, minor(dev), scu->icnt);
+  struct asc_unit *scu;
+  int unit;
 
-  if ( unit >= NASC || !( scu->flags & ATTACHED ) ) {
-      lprintf("asc%d.open: unit was not attached successfully, flags 0x%04x\n",
+  unit = UNIT(minor(dev)) & UNIT_MASK;
+  if ( unit >= NASC )
+    {
+#ifdef ASCDEBUG
+      /* XXX lprintf isn't valid here since there is no scu. */
+      printf("asc%d.open: unconfigured unit number (max %d)\n", unit, NASC);
+#endif
+      return ENXIO;
+    }
+  scu = unittab + unit;
+  if ( !( scu->flags & ATTACHED ) )
+    {
+      lprintf("asc%d.open: unit was not attached successfully 0x%04x\n",
 	     unit, scu->flags);
       return ENXIO;
-  }
-  
-  if ( scu->flags & OPEN ) {
-      lprintf("asc%d.open: already open", unit);
-      return EBUSY;
-  }
-  scu->flags = ATTACHED | OPEN;      
+    }
 
-  if ( minor(dev) & DBUG_MASK ) scu->flags |= DEBUG;
+  if ( minor(dev) & DBUG_MASK )
+    scu->flags |= FLAG_DEBUG;
+  else
+    scu->flags &= ~FLAG_DEBUG;
 
   switch(minor(dev) & FRMT_MASK) {
   case FRMT_PBM:
@@ -580,12 +626,23 @@ ascopen(dev_t dev, int flags, int fmt, struct proc *p)
     return ENXIO;
   }
   
+  lprintf("asc%d.open: minor %d icnt %ld\n", unit, minor(dev), scu->icnt);
+
+  if ( scu->flags & OPEN ) {
+      lprintf("asc%d.open: already open", unit);
+      return EBUSY;
+  }
+  if (isa_dma_acquire(scu->dma_num))
+      return(EBUSY);
+
+  scu->flags = ATTACHED | OPEN;      
+
   asc_reset(scu);
   get_resolution(scu);
   return SUCCESS;
 }
 
-int
+static int
 asc_startread(struct asc_unit *scu)
 {
     /*** from here on, things can be delayed to the first read/ioctl ***/
@@ -593,8 +650,12 @@ asc_startread(struct asc_unit *scu)
   scu->cfg_byte= scu->cmd_byte=0;	/* init scanner */
   outb(ASC_CMD, scu->cmd_byte);
     /*** this was done in sub_16, set scan len... ***/
-  outb(ASC_BOH, 0 );
+  outb(ASC_BOH, scu->portf_byte );
+  if (geomtab[scu->geometry].g_res==0) {		/* color */
+	scu->cmd_byte = 0x00 ;
+  } else {
   scu->cmd_byte = 0x90 ;
+  }
   outb(ASC_CMD, scu->cmd_byte);
   outb(ASC_LEN_L, scu->linesize & 0xff /* len_low */);
   outb(ASC_LEN_H, (scu->linesize >>8) & 0xff /* len_high */);
@@ -618,7 +679,7 @@ asc_startread(struct asc_unit *scu)
  ***	should probably terminate dma ops, release int and dma. lr 12mar95
  ***/
 
-int
+STATIC int
 ascclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
   int unit = UNIT(minor(dev));
@@ -642,15 +703,13 @@ ascclose(dev_t dev, int flags, int fmt, struct proc *p)
   scu->cfg_byte &= ~ scu->int_byte ; /* disable scanner int */
   outb(ASC_CFG, scu->cfg_byte);
     /* --- disable dma controller ? --- */
+  isa_dma_release(scu->dma_num);
     /* --- disable interrupts on the controller (sub_24) --- */
 
-  if ( scu->sbuf.base != NULL ) free( scu->sbuf.base, M_DEVBUF );
-
-  scu->sbuf.base = NULL;
   scu->sbuf.size = INVALID;
   scu->sbuf.rptr  = INVALID;
 
-  scu->flags &= ~(DEBUG | OPEN | READING);
+  scu->flags &= ~(FLAG_DEBUG | OPEN | READING);
   
   return SUCCESS;
 }
@@ -668,15 +727,17 @@ pbm_init(struct asc_unit *scu)
     scu->sbuf.rptr=scu->sbuf.size-l;
     bcopy(scu->sbuf.base, scu->sbuf.base+scu->sbuf.rptr,l);
     scu->sbuf.count = l;
+    if (geomtab[scu->geometry].g_res!=0) { /* BW scanner */
     for(p = scu->sbuf.base + scu->sbuf.rptr; l; p++, l--)
 	*p = ~*p;
 }
+}
 /**************************************************************************
  ***
  *** ascread
  ***/
 
-int
+STATIC int
 ascread(dev_t dev, struct uio *uio, int ioflag)
 {
   int unit = UNIT(minor(dev));
@@ -718,7 +779,7 @@ ascread(dev_t dev, struct uio *uio, int ioflag)
       if ( res == 0 ) res = SUCCESS;
   }
   splx(sps); /* lower priority... */
-  if (scu->flags & DEBUG)
+  if (scu->flags & FLAG_DEBUG)
       tsleep((caddr_t)scu, ASCPRI | PCATCH, "ascdly",hz);
   lprintf("asc%d.read(after): "
       "sz 0x%x, rptr 0x%x, wptr 0x%x, cnt 0x%x bcnt 0x%x flags 0x%x icnt %d\n",
@@ -734,10 +795,11 @@ ascread(dev_t dev, struct uio *uio, int ioflag)
   if ( (scu->flags & PBM_MODE) )
       nbytes = min( nbytes, scu->bcount );
   lprintf("asc%d.read: transferring 0x%x bytes\n", unit, nbytes);
-  
+  if (geomtab[scu->geometry].g_res!=0) { /* BW scanner */
   lprintf("asc%d.read: invert buffer\n",unit);
   for(p = scu->sbuf.base + scu->sbuf.rptr, res=nbytes; res; p++, res--)
 	*p = ~*p;
+  }
   res = uiomove(scu->sbuf.base + scu->sbuf.rptr, nbytes, uio);
   if ( res != SUCCESS ) {
       lprintf("asc%d.read: uiomove failed %d", unit, res);
@@ -764,7 +826,7 @@ ascread(dev_t dev, struct uio *uio, int ioflag)
  *** ascioctl
  ***/
 
-int
+STATIC int
 ascioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
 {
   int unit = UNIT(minor(dev));
@@ -774,7 +836,7 @@ ascioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
 	 unit, minor(dev));
 
   if ( unit >= NASC || !( scu->flags & ATTACHED ) ) {
-      lprintf("asc%d.ioctl: unit was not attached successfully 0x04x\n",
+      lprintf("asc%d.ioctl: unit was not attached successfully 0x%04x\n",
 	     unit, scu->flags);
       return ENXIO;
   }
@@ -829,7 +891,7 @@ ascioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
   return SUCCESS;
 }
 
-int
+STATIC int
 ascselect(dev_t dev, int rw, struct proc *p)
 {
     int unit = UNIT(minor(dev));
@@ -849,14 +911,28 @@ ascselect(dev_t dev, int rw, struct proc *p)
 	scu->flags |= SEL_COLL;
     }
 #else
-    
-    if (scu->selp.si_pid && (p1=pfind(scu->selp.si_pid))
-	    && p1->p_wchan == (caddr_t)&selwait)
-	scu->selp.si_flags = SI_COLL;
-    else
-	scu->selp.si_pid = p->p_pid;
+    selrecord(p, &(scu->selp) );
 #endif
     splx(sps);
     return 0;
 }
+
+
+static asc_devsw_installed = 0;
+
+static void 
+asc_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! asc_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR,0);
+		cdevsw_add(&dev,&asc_cdevsw,NULL);
+		asc_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(ascdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,asc_drvinit,NULL)
+
+
 #endif /* NASC > 0 */
