@@ -49,32 +49,19 @@
 #if NBQU > 0
 
 #include <sys/param.h>
-#include <sys/systm.h> /* for min(), SELWAIT */
-/* #include "proc.h" /*  for pid_t */
-/* #include "user.h" */
-/* #include "buf.h" */
-/* #include "kernel.h"  */
+#include <sys/systm.h>
+#include <sys/proc.h>
 #include <sys/uio.h>
-#include <sys/devconf.h>
-#include <i386/isa/isa.h>
-/* #include "sys/ioctl.h" */
-/* #include "syslog.h" */
+#include <sys/conf.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
+#include <machine/clock.h>
+
+#include <i386/isa/b004.h>
 #include <i386/isa/isa_device.h>
-#include <sys/errno.h>
-
-#include "b004.h"
-
-static struct kern_devconf kdc_bqu[NBQU] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"bqu", 0, { MDDT_ISA, 0 },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* always start here */
-	"B004-compatible Transputer board",
-	DC_CLS_MISC
-} };
 
 #define IOCTL_OUT(arg, ret)             *(int*)arg = ret
 
@@ -89,6 +76,9 @@ static struct kern_devconf kdc_bqu[NBQU] = { {
 #undef B004_DEBUG_2
 
 #ifdef B004_DEBUG
+static u_char d_inb(u_int port);
+static void d_outb(u_int port, u_char data);
+
 #define out(port,data)	d_outb(port, data)
 #define in(a)		d_inb(((u_int)a))
 #else
@@ -110,20 +100,27 @@ static struct kern_devconf kdc_bqu[NBQU] = { {
 #define DEB2(x)
 #endif
 
-int bquclose(dev_t dev, int flag);
-int bquopen(dev_t dev, int flag);
-int bquwrite(dev_t dev, struct uio *uio, int flag);
-int bquread(dev_t dev, struct uio *uio, int flag);
-int bquioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p);
-int bquselect(dev_t dev, int rw, struct proc *p);
-int bquprobe(struct isa_device *idp);
-int bquattach(struct isa_device *idp);
+static int bquprobe(struct isa_device *idp);
+static int bquattach(struct isa_device *idp);
 
 
 struct isa_driver bqudriver = {
    bquprobe, bquattach, "bqu"
 };
 
+static	d_open_t	bquopen;
+static	d_close_t	bquclose;
+static	d_read_t	bquread;
+static	d_write_t	bquwrite;
+static	d_ioctl_t	bquioctl;
+static	d_select_t	bquselect;
+
+#define CDEV_MAJOR 8
+static struct cdevsw bqu_cdevsw = 
+	{ bquopen,      bquclose,       bquread,        bquwrite,       /*8*/
+	  bquioctl,     nostop,         nullreset,      nodevtotty,/* tputer */
+	  bquselect,    nommap,         NULL,	"bqu",	NULL,	-1 };
+
 static int b004_sleep; /* wait address */
 
 static struct b004_struct b004_table[NBQU];
@@ -139,8 +136,8 @@ b004_base_addresses[B004_CHANCE] = {
     0x150, 0x190, 0, 0, 0, 0, 0, 0
 };
 
-
-void
+#ifdef B004_DEBUG
+static void
 d_outb(u_int port, u_char data)
 {
 
@@ -148,13 +145,15 @@ d_outb(u_int port, u_char data)
 	outb(port,data);
 }
 
-u_char d_inb(u_int port)
+static u_char
+d_inb(u_int port)
 {
 u_char ap;
 	ap=inb(port);
 	printf("INPUT 0x%x FROM 0x%x\n",ap,port);
 	return(ap);
 }
+#endif
 
 static int
 detected(int base)
@@ -238,7 +237,7 @@ bquanalyse( const int dev_min )
  *
  *****************************************************************************/
 
-int
+static int
 bquread(dev_t dev, struct uio *uio, int flag)
 {
     unsigned int dev_min = minor(dev) & 7;
@@ -329,7 +328,7 @@ bquread(dev_t dev, struct uio *uio, int flag)
  * int bquwrite() - write to the link interface.
  */
 
-int
+static int
 bquwrite(dev_t dev, struct uio *uio, int flag)
 {
     unsigned int dev_min = minor(dev) & 7;
@@ -426,8 +425,8 @@ bquwrite(dev_t dev, struct uio *uio, int flag)
  *
  */
 
-int
-bquopen(dev_t dev, int flag)
+static int
+bquopen(dev_t dev, int flags, int fmt, struct proc *p)
 {
     unsigned int dev_min = minor(dev) & 7;
 
@@ -444,7 +443,6 @@ bquopen(dev_t dev, int flag)
 	return EBUSY;
     }
     B004_F(dev_min) |= B004_BUSY;
-    kdc_bqu[dev_min].kdc_state = DC_BUSY;
     B004_TIMEOUT(dev_min) = 0;
     DEB(printf( "B004 opened, minor = %d.\n", dev_min );)
     return 0;
@@ -455,8 +453,8 @@ bquopen(dev_t dev, int flag)
  * int b004close() -- close the link device.
  */
 
-int
-bquclose(dev_t dev, int flag)
+static int
+bquclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
     unsigned int dev_min = minor(dev) & 7;
 
@@ -465,12 +463,11 @@ bquclose(dev_t dev, int flag)
 	return ENXIO;
     }
     B004_F(dev_min) &= ~B004_BUSY;
-    kdc_bqu[dev_min].kdc_state = DC_IDLE;
     DEB(printf("B004(%d) released.\n", dev_min );)
     return 0;
 }
 
-int
+static int
 bquselect(dev_t dev, int rw, struct proc *p)
 {
     /* still unimplemented */
@@ -487,7 +484,7 @@ bquselect(dev_t dev, int rw, struct proc *p)
  * - set timeout
  */
 
-int
+static int
 bquioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 {
     unsigned int dev_min = minor(dev) & 7;
@@ -531,20 +528,36 @@ bquioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 } /* bquioctl() */
 
 
-static inline void
-bqu_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_bqu[id->id_unit] = kdc_bqu[0];
-	kdc_bqu[id->id_unit].kdc_unit = id->id_unit;
-	kdc_bqu[id->id_unit].kdc_parentdata = id;
-	dev_attach(&kdc_bqu[id->id_unit]);
-}
-
-int
+static int
 bquattach(struct isa_device *idp)
 {
-	kdc_bqu[idp->id_unit].kdc_state = DC_IDLE;
+	int unit = idp->id_unit;
+	struct b004_struct *bp;
+	int	i;
+
+#ifdef DEVFS
+#define BQU_UID 66
+#define BQU_GID 66
+#define BQU_PERM 0600
+	bp = &b004_table[unit];
+	for ( i = 0; i < 8; i++) {
+#ifdef NOTYET
+	/*	if (we've done all the ports found) break; */
+#endif
+		bp->devfs_token[i][0]=
+			devfs_add_devswf(&bqu_cdevsw, i, DV_CHR, BQU_UID, 
+				 	 BQU_GID, BQU_PERM, "ttyba%d", i);
+		bp->devfs_token[i][0]=
+			devfs_add_devswf(&bqu_cdevsw, i+64, DV_CHR, BQU_UID, 
+					 BQU_GID, BQU_PERM, "ttybb%d", i);
+		bp->devfs_token[i][0]=
+			devfs_add_devswf(&bqu_cdevsw, i+128, DV_CHR, BQU_UID, 
+					 BQU_GID, BQU_PERM, "ttybc%d", i);
+		bp->devfs_token[i][0]=
+			devfs_add_devswf(&bqu_cdevsw, i+192, DV_CHR, BQU_UID, 
+					 BQU_GID, BQU_PERM, "ttybd%d", unit);
+	}
+#endif
 	return 1;
 }
 
@@ -555,7 +568,7 @@ bquattach(struct isa_device *idp)
  * and sets up all relevant data-structures.
  */
 
-int
+static int
 bquprobe(struct isa_device *idp)
 {
     unsigned int test;
@@ -566,7 +579,10 @@ bquprobe(struct isa_device *idp)
        register
     */
 #ifdef undef
-printf("bquprobe::\nIOBASE 0x%x\nIRQ %d\nDRQ %d\nMSIZE %d\nUNIT %d\nFLAGS x0%x\nALIVE %d\n",idp->id_iobase,idp->id_irq,idp->id_drq,idp->id_msize,idp->id_unit,idp->id_flags,idp->id_alive);
+	printf(
+	  "bquprobe::\nIOBASE 0x%x\nIRQ %d\nDRQ %d\nMSIZE %d\nUNIT %d\nFLAGS"
+	  "x0%x\nALIVE %d\n",idp->id_iobase,idp->id_irq,
+	  idp->id_drq,idp->id_msize,idp->id_unit,idp->id_flags,idp->id_alive);
 #endif
     if(first_time){
 	for(i=0;i<NBQU;i++) B004_F(i) &= ~B004_EXIST;
@@ -576,9 +592,6 @@ printf("bquprobe::\nIOBASE 0x%x\nIRQ %d\nDRQ %d\nMSIZE %d\nUNIT %d\nFLAGS x0%x\n
     if(dev_min >= NBQU) return (0);	/* No more descriptors */
     if ((idp->id_iobase < 0x100) || (idp->id_iobase >= 0x1000))
 	idp->id_iobase=0;		/* Dangerous isa addres ) */
-#ifndef DEV_LKM
-    bqu_registerdev(idp);
-#endif /* not DEV_LKM */
 
     for (test = 0; (test < B004_CHANCE); test++) {
 	if((idp->id_iobase==0)&&((!b004_base_addresses[test])||
@@ -621,7 +634,7 @@ printf("bquprobe::\nIOBASE 0x%x\nIRQ %d\nDRQ %d\nMSIZE %d\nUNIT %d\nFLAGS x0%x\n
     }
 
     if (!found){
-	printf("b004probe(): no B004-board found.\n");
+	DEB(printf("b004probe(): no B004-board found.\n"));
 	return (0);
     }
 
@@ -633,4 +646,22 @@ printf("bquprobe::\nIOBASE 0x%x\nIRQ %d\nDRQ %d\nMSIZE %d\nUNIT %d\nFLAGS x0%x\n
 	return(20);
 } /* bquprobe() */
 
+
+static bqu_devsw_installed = 0;
+
+static void
+bqu_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! bqu_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&bqu_cdevsw, NULL);
+		bqu_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(bqudev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,bqu_drvinit,NULL)
+
+
 #endif /* NBQU */
