@@ -1,6 +1,3 @@
-#ifndef _B004_H
-#define _B004_H
-
 /*
  * b004.h
  *
@@ -41,7 +38,10 @@
  * Works for FreeBSD 1.1.5
  */
 
-#include <sys/ioctl.h>
+#ifndef	_I386_ISA_B004_H_
+#define	_I386_ISA_B004_H_
+
+#include <sys/ioccom.h>
 
 /*
  * device status FLAGS
@@ -93,6 +93,7 @@ struct b004_struct {
 	int osr;		/* address of the output status register */
 	unsigned int timeout;	/* timeout for writing/reading the link */
 	int boardtype;		/* what kind of board is installed */
+	void *devfs_token[8][4]; /* tokens for 4 types for 8 ports */
 };
 
 /*
@@ -115,7 +116,7 @@ struct b004_struct {
 #define B004_BOARDTYPE(minor)		b004_table[minor].boardtype
 
 /*
- * Additonal defines for B008-boards
+ * Additional defines for B008-boards
  */
 #define B008_DMA(minor)			b004_table[minor].int
 #define B008_INT(minor)			b004_table[minor].dma
@@ -150,4 +151,4 @@ struct b004_struct {
 #define B004_ASSERT_ANALYSE	0x01	/* switch transputer to analyse-mode */
 #define B004_DEASSERT_ANALYSE	0x00
 
-#endif
+#endif /* !_I386_ISA_B004_H_ */
