@@ -34,7 +34,11 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)clock.c	7.2 (Berkeley) 5/12/91
- * $FreeBSD: stable/2.1/sys/i386/isa/clock.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/clock.c 50950 1999-09-05 08:39:22Z peter $
+ */
+
+/*
+ * Routines to handle clock hardware.
  */
 
 /*
@@ -44,15 +48,22 @@
  * reintroduced and updated by Chris Stenton <chris@gnome.co.uk> 8/10/94
  */
 
-/*
- * Primitive clock interrupt routines.
- */
+#include "opt_clock.h"
+#include "opt_cpu.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/time.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
+
 #include <machine/clock.h>
+#ifdef CLK_CALIBRATION_LOOP
+#include <machine/cons.h>
+#endif
+#include <machine/cpu.h>
 #include <machine/frame.h>
+
 #include <i386/isa/icu.h>
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
@@ -66,11 +77,7 @@
 #define	LEAPYEAR(y) ((u_int)(y) % 4 == 0)
 #define DAYSPERYEAR   (31+28+31+30+31+30+31+31+30+31+30+31)
 
-/* X-tals being what they are, it's nice to be able to fudge this one... */
-#ifndef TIMER_FREQ
-#define	TIMER_FREQ	1193182	/* XXX - should be in isa.h */
-#endif
-#define TIMER_DIV(x) ((TIMER_FREQ+(x)/2)/(x))
+#define	TIMER_DIV(x) ((timer_freq + (x) / 2) / (x))
 
 /*
  * Time in timer cycles that it takes for microtime() to disable interrupts
@@ -82,63 +89,76 @@
 #define	TIMER0_LATCH_COUNT	20
 
 /*
- * Minimum maximum count that we are willing to program into timer0.
- * Must be large enough to guarantee that the timer interrupt handler
- * returns before the next timer interrupt.  Must be larger than
- * TIMER0_LATCH_COUNT so that we don't have to worry about underflow in
- * the calculation of timer0_overflow_threshold.
+ * Maximum frequency that we are willing to allow for timer0.  Must be
+ * low enough to guarantee that the timer interrupt handler returns
+ * before the next timer interrupt.  Must result in a lower TIMER_DIV
+ * value than TIMER0_LATCH_COUNT so that we don't have to worry about
+ * underflow in the calculation of timer0_overflow_threshold.
  */
-#define	TIMER0_MIN_MAX_COUNT	TIMER_DIV(20000)
+#define	TIMER0_MAX_FREQ		20000
 
-int	adjkerntz = 0;		/* offset from CMOS clock */
-int	disable_rtc_set	= 0;	/* disable resettodr() if != 0 */
+int	adjkerntz;		/* local offset	from GMT in seconds */
+int	disable_rtc_set;	/* disable resettodr() if != 0 */
 u_int	idelayed;
-#ifdef I586_CPU
-int	pentium_mhz;
+#if defined(I586_CPU) || defined(I686_CPU)
+u_int	i586_ctr_bias;
+u_int	i586_ctr_comultiplier;
+u_int	i586_ctr_freq;
+u_int	i586_ctr_multiplier;
 #endif
 int	statclock_disable;
 u_int	stat_imask = SWI_CLOCK_MASK;
-int 	timer0_max_count;
-u_int 	timer0_overflow_threshold;
-u_int 	timer0_prescaler_count;
+#ifdef TIMER_FREQ
+u_int	timer_freq = TIMER_FREQ;
+#else
+u_int	timer_freq = 1193182;
+#endif
+int	timer0_max_count;
+u_int	timer0_overflow_threshold;
+u_int	timer0_prescaler_count;
+int	wall_cmos_clock;	/* wall	CMOS clock assumed if != 0 */
 
 static	int	beeping = 0;
 static	u_int	clk_imask = HWI_MASK | SWI_MASK;
 static	const u_char daysinmonth[] = {31,28,31,30,31,30,31,31,30,31,30,31};
-static 	u_int	hardclock_max_count;
+static	u_int	hardclock_max_count;
 /*
  * XXX new_function and timer_func should not handle clockframes, but
  * timer_func currently needs to hold hardclock to handle the
  * timer0_state == 0 case.  We should use register_intr()/unregister_intr()
  * to switch between clkintr() and a slightly different timerintr().
- * This will require locking when acquiring and releasing timer0 - the
- * current (nonexistent) locking doesn't seem to be adequate even now.
  */
-static 	void	(*new_function) __P((struct clockframe *frame));
-static 	u_int	new_rate;
+static	void	(*new_function) __P((struct clockframe *frame));
+static	u_int	new_rate;
 static	u_char	rtc_statusa = RTCSA_DIVIDER | RTCSA_NOPROF;
 static	u_char	rtc_statusb = RTCSB_24HR | RTCSB_PINTR;
-static 	char	timer0_state = 0;
-static	char	timer2_state = 0;
-static 	void	(*timer_func) __P((struct clockframe *frame)) = hardclock;
 
-#if 0
-void
-clkintr(struct clockframe frame)
-{
-	hardclock(&frame);
-	setdelayed();
-}
-#else
-void
+/* Values for timerX_state: */
+#define	RELEASED	0
+#define	RELEASE_PENDING	1
+#define	ACQUIRED	2
+#define	ACQUIRE_PENDING	3
+
+static	u_char	timer0_state;
+static	u_char	timer2_state;
+static	void	(*timer_func) __P((struct clockframe *frame)) = hardclock;
+
+#if defined(I586_CPU) || defined(I686_CPU)
+static	void	set_i586_ctr_freq(u_int i586_freq, u_int i8254_freq);
+#endif
+static	void	set_timer_freq(u_int freq, int intr_freq);
+
+static void
 clkintr(struct clockframe frame)
 {
 	timer_func(&frame);
 	switch (timer0_state) {
-	case 0:
+
+	case RELEASED:
 		setdelayed();
 		break;
-	case 1:
+
+	case ACQUIRED:
 		if ((timer0_prescaler_count += timer0_max_count)
 		    >= hardclock_max_count) {
 			hardclock(&frame);
@@ -146,7 +166,8 @@ clkintr(struct clockframe frame)
 			timer0_prescaler_count -= hardclock_max_count;
 		}
 		break;
-	case 2:
+
+	case ACQUIRE_PENDING:
 		setdelayed();
 		timer0_max_count = TIMER_DIV(new_rate);
 		timer0_overflow_threshold =
@@ -158,9 +179,10 @@ clkintr(struct clockframe frame)
 		enable_intr();
 		timer0_prescaler_count = 0;
 		timer_func = new_function;
-		timer0_state = 1;
+		timer0_state = ACQUIRED;
 		break;
-	case 3:
+
+	case RELEASE_PENDING:
 		if ((timer0_prescaler_count += timer0_max_count)
 		    >= hardclock_max_count) {
 			hardclock(&frame);
@@ -177,59 +199,104 @@ clkintr(struct clockframe frame)
 			/*
 			 * See microtime.s for this magic.
 			 */
-			time.tv_usec += (27645 *
+			time.tv_usec += (27465 *
 				(timer0_prescaler_count - hardclock_max_count))
 				>> 15;
 			if (time.tv_usec >= 1000000)
 				time.tv_usec -= 1000000;
 			timer0_prescaler_count = 0;
-			timer_func = hardclock;;
-			timer0_state = 0;
+			timer_func = hardclock;
+			timer0_state = RELEASED;
 		}
 		break;
 	}
 }
-#endif
 
+/*
+ * The acquire and release functions must be called at ipl >= splclock().
+ */
 int
 acquire_timer0(int rate, void (*function) __P((struct clockframe *frame)))
 {
-	if (timer0_state || TIMER_DIV(rate) < TIMER0_MIN_MAX_COUNT ||
-	    !function)
-		return -1;
+	static int old_rate;
+
+	if (rate <= 0 || rate > TIMER0_MAX_FREQ)
+		return (-1);
+	switch (timer0_state) {
+
+	case RELEASED:
+		timer0_state = ACQUIRE_PENDING;
+		break;
+
+	case RELEASE_PENDING:
+		if (rate != old_rate)
+			return (-1);
+		/*
+		 * The timer has been released recently, but is being
+		 * re-acquired before the release completed.  In this
+		 * case, we simply reclaim it as if it had not been
+		 * released at all.
+		 */
+		timer0_state = ACQUIRED;
+		break;
+
+	default:
+		return (-1);	/* busy */
+	}
 	new_function = function;
-	new_rate = rate;
-	timer0_state = 2;
-	return 0;
+	old_rate = new_rate = rate;
+	return (0);
 }
 
 int
 acquire_timer2(int mode)
 {
-	if (timer2_state)
-		return -1;
-	timer2_state = 1;
-	outb(TIMER_MODE, TIMER_SEL2 | (mode &0x3f));
-	return 0;
+
+	if (timer2_state != RELEASED)
+		return (-1);
+	timer2_state = ACQUIRED;
+
+	/*
+	 * This access to the timer registers is as atomic as possible
+	 * because it is a single instruction.  We could do better if we
+	 * knew the rate.  Use of splclock() limits glitches to 10-100us,
+	 * and this is probably good enough for timer2, so we aren't as
+	 * careful with it as with timer0.
+	 */
+	outb(TIMER_MODE, TIMER_SEL2 | (mode & 0x3f));
+
+	return (0);
 }
 
 int
 release_timer0()
 {
-	if (!timer0_state)
-		return -1;
-	timer0_state = 3;
-	return 0;
+	switch (timer0_state) {
+
+	case ACQUIRED:
+		timer0_state = RELEASE_PENDING;
+		break;
+
+	case ACQUIRE_PENDING:
+		/* Nothing happened yet, release quickly. */
+		timer0_state = RELEASED;
+		break;
+
+	default:
+		return (-1);
+	}
+	return (0);
 }
 
 int
 release_timer2()
 {
-	if (!timer2_state)
-		return -1;
-	timer2_state = 0;
-	outb(TIMER_MODE, TIMER_SEL2|TIMER_SQWAVE|TIMER_16BIT);
-	return 0;
+
+	if (timer2_state != ACQUIRED)
+		return (-1);
+	timer2_state = RELEASED;
+	outb(TIMER_MODE, TIMER_SEL2 | TIMER_SQWAVE | TIMER_16BIT);
+	return (0);
 }
 
 /*
@@ -253,74 +320,54 @@ release_timer2()
  * Stat clock ticks can still be lost, causing minor loss of accuracy
  * in the statistics, but the stat clock will no longer stop.
  */
-void
+static void
 rtcintr(struct clockframe frame)
 {
 	while (rtcin(RTC_INTR) & RTCIR_PERIOD)
 		statclock(&frame);
 }
 
+#include "opt_ddb.h"
 #ifdef DDB
-static void
-printrtc(void)
+#include <ddb/ddb.h>
+
+DB_SHOW_COMMAND(rtc, rtc)
 {
 	printf("%02x/%02x/%02x %02x:%02x:%02x, A = %02x, B = %02x, C = %02x\n",
 	       rtcin(RTC_YEAR), rtcin(RTC_MONTH), rtcin(RTC_DAY),
 	       rtcin(RTC_HRS), rtcin(RTC_MIN), rtcin(RTC_SEC),
 	       rtcin(RTC_STATUSA), rtcin(RTC_STATUSB), rtcin(RTC_INTR));
 }
-#endif
+#endif /* DDB */
 
 static int
 getit(void)
 {
+	u_long ef;
 	int high, low;
 
+	ef = read_eflags();
 	disable_intr();
-	/* select timer0 and latch counter value */
-	outb(TIMER_MODE, TIMER_SEL0);
+
+	/* Select timer0 and latch counter value. */
+	outb(TIMER_MODE, TIMER_SEL0 | TIMER_LATCH);
+
 	low = inb(TIMER_CNTR0);
 	high = inb(TIMER_CNTR0);
-	enable_intr();
+
+	write_eflags(ef);
 	return ((high << 8) | low);
 }
 
-#ifdef I586_CPU
-static long long cycles_per_sec = 0;
-
-/*
- * Figure out how fast the cyclecounter runs.  This must be run with
- * clock interrupts disabled, but with the timer/counter programmed
- * and running.
- */
-void
-calibrate_cyclecounter(void)
-{
-	/*
-	 * Don't need volatile; should always use unsigned if 2's
-	 * complement arithmetic is desired.
-	 */
-	unsigned long long count, last_count;
-
-	__asm __volatile(".byte 0xf,0x31" : "=A" (last_count));
-	DELAY(1000000);
-	__asm __volatile(".byte 0xf,0x31" : "=A" (count));
-	/*
-	 * XX lose if the clock rate is not nearly a multiple of 1000000.
-	 */
-	pentium_mhz = ((count - last_count) + 500000) / 1000000;
-}
-#endif
-
 /*
  * Wait "n" microseconds.
- * Relies on timer 1 counting down from (TIMER_FREQ / hz)
+ * Relies on timer 1 counting down from (timer_freq / hz)
  * Note: timer had better have been programmed before this is first used!
  */
 void
 DELAY(int n)
 {
-	int prev_tick, tick, ticks_left, sec, usec;
+	int delta, prev_tick, tick, ticks_left;
 
 #ifdef DELAYDEBUG
 	int getit_calls = 1;
@@ -336,6 +383,13 @@ DELAY(int n)
 	if (state == 1)
 		printf("DELAY(%d)...", n);
 #endif
+	/*
+	 * Guard against the timer being uninitialized if we are called
+	 * early for console i/o.
+	 */
+	if (timer0_max_count == 0)
+		set_timer_freq(timer_freq, hz);
+
 	/*
 	 * Read the counter first, so that the rest of the setup overhead is
 	 * counted.  Guess the initial overhead is 20 usec (on most systems it
@@ -344,28 +398,50 @@ DELAY(int n)
 	 * multiplications and divisions to scale the count take a while).
 	 */
 	prev_tick = getit();
-	n -= 20;
+	n -= 0;			/* XXX actually guess no initial overhead */
 	/*
-	 * Calculate (n * (TIMER_FREQ / 1e6)) without using floating point
+	 * Calculate (n * (timer_freq / 1e6)) without using floating point
 	 * and without any avoidable overflows.
 	 */
-	sec = n / 1000000;
-	usec = n - sec * 1000000;
-	ticks_left = sec * TIMER_FREQ
-		     + usec * (TIMER_FREQ / 1000000)
-		     + usec * ((TIMER_FREQ % 1000000) / 1000) / 1000
-		     + usec * (TIMER_FREQ % 1000) / 1000000;
+	if (n <= 0)
+		ticks_left = 0;
+	else if (n < 256)
+		/*
+		 * Use fixed point to avoid a slow division by 1000000.
+		 * 39099 = 1193182 * 2^15 / 10^6 rounded to nearest.
+		 * 2^15 is the first power of 2 that gives exact results
+		 * for n between 0 and 256.
+		 */
+		ticks_left = ((u_int)n * 39099 + (1 << 15) - 1) >> 15;
+	else
+		/*
+		 * Don't bother using fixed point, although gcc-2.7.2
+		 * generates particularly poor code for the long long
+		 * division, since even the slow way will complete long
+		 * before the delay is up (unless we're interrupted).
+		 */
+		ticks_left = ((u_int)n * (long long)timer_freq + 999999)
+			     / 1000000;
 
 	while (ticks_left > 0) {
 		tick = getit();
 #ifdef DELAYDEBUG
 		++getit_calls;
 #endif
-		if (tick > prev_tick)
-			ticks_left -= prev_tick - (tick - timer0_max_count);
-		else
-			ticks_left -= prev_tick - tick;
+		delta = prev_tick - tick;
 		prev_tick = tick;
+		if (delta < 0) {
+			delta += timer0_max_count;
+			/*
+			 * Guard against timer0_max_count being wrong.
+			 * This shouldn't happen in normal operation,
+			 * but it may happen if set_timer_freq() is
+			 * traced.
+			 */
+			if (delta < 0)
+				delta = 0;
+		}
+		ticks_left -= delta;
 	}
 #ifdef DELAYDEBUG
 	if (state == 1)
@@ -385,47 +461,168 @@ sysbeepstop(void *chan)
 int
 sysbeep(int pitch, int period)
 {
+	int x = splclock();
 
 	if (acquire_timer2(TIMER_SQWAVE|TIMER_16BIT))
-		return -1;
+		if (!beeping) {
+			/* Something else owns it. */
+			splx(x);
+			return (-1); /* XXX Should be EBUSY, but nobody cares anyway. */
+		}
 	disable_intr();
 	outb(TIMER_CNTR2, pitch);
 	outb(TIMER_CNTR2, (pitch>>8));
 	enable_intr();
 	if (!beeping) {
-	outb(IO_PPI, inb(IO_PPI) | 3);	/* enable counter2 output to speaker */
+		/* enable counter2 output to speaker */
+		outb(IO_PPI, inb(IO_PPI) | 3);
 		beeping = period;
 		timeout(sysbeepstop, (void *)NULL, period);
 	}
-	return 0;
+	splx(x);
+	return (0);
 }
 
 /*
  * RTC support routines
  */
-static int
-bcd2int(int bcd)
+
+int
+rtcin(reg)
+	int reg;
 {
-	return(bcd/16 *	10 + bcd%16);
+	u_char val;
+
+	outb(IO_RTC, reg);
+	inb(0x84);
+	val = inb(IO_RTC + 1);
+	inb(0x84);
+	return (val);
 }
 
-static int
-int2bcd(int dez)
-{
-	return(dez/10 *	16 + dez%10);
-}
-
-static inline void
+static __inline void
 writertc(u_char reg, u_char val)
 {
 	outb(IO_RTC, reg);
 	outb(IO_RTC + 1, val);
 }
 
-static int
+static __inline int
 readrtc(int port)
 {
-	return(bcd2int(rtcin(port)));
+	return(bcd2bin(rtcin(port)));
+}
+
+static u_int
+calibrate_clocks(void)
+{
+	u_int count, prev_count, tot_count;
+	int sec, start_sec, timeout;
+
+	if (bootverbose)
+	        printf("Calibrating clock(s) ... ");
+	if (!(rtcin(RTC_STATUSD) & RTCSD_PWR))
+		goto fail;
+	timeout = 100000000;
+
+	/* Read the mc146818A seconds counter. */
+	for (;;) {
+		if (!(rtcin(RTC_STATUSA) & RTCSA_TUP)) {
+			sec = rtcin(RTC_SEC);
+			break;
+		}
+		if (--timeout == 0)
+			goto fail;
+	}
+
+	/* Wait for the mC146818A seconds counter to change. */
+	start_sec = sec;
+	for (;;) {
+		if (!(rtcin(RTC_STATUSA) & RTCSA_TUP)) {
+			sec = rtcin(RTC_SEC);
+			if (sec != start_sec)
+				break;
+		}
+		if (--timeout == 0)
+			goto fail;
+	}
+
+	/* Start keeping track of the i8254 counter. */
+	prev_count = getit();
+	if (prev_count == 0 || prev_count > timer0_max_count)
+		goto fail;
+	tot_count = 0;
+
+#if defined(I586_CPU) || defined(I686_CPU)
+	if (cpu_class == CPUCLASS_586 || cpu_class == CPUCLASS_686)
+		wrmsr(0x10, 0LL);	/* XXX 0x10 is the MSR for the TSC */
+#endif
+
+	/*
+	 * Wait for the mc146818A seconds counter to change.  Read the i8254
+	 * counter for each iteration since this is convenient and only
+	 * costs a few usec of inaccuracy. The timing of the final reads
+	 * of the counters almost matches the timing of the initial reads,
+	 * so the main cause of inaccuracy is the varying latency from 
+	 * inside getit() or rtcin(RTC_STATUSA) to the beginning of the
+	 * rtcin(RTC_SEC) that returns a changed seconds count.  The
+	 * maximum inaccuracy from this cause is < 10 usec on 486's.
+	 */
+	start_sec = sec;
+	for (;;) {
+		if (!(rtcin(RTC_STATUSA) & RTCSA_TUP))
+			sec = rtcin(RTC_SEC);
+		count = getit();
+		if (count == 0 || count > timer0_max_count)
+			goto fail;
+		if (count > prev_count)
+			tot_count += prev_count - (count - timer0_max_count);
+		else
+			tot_count += prev_count - count;
+		prev_count = count;
+		if (sec != start_sec)
+			break;
+		if (--timeout == 0)
+			goto fail;
+	}
+
+#if defined(I586_CPU) || defined(I686_CPU)
+	/*
+	 * Read the cpu cycle counter.  The timing considerations are
+	 * similar to those for the i8254 clock.
+	 */
+	if (cpu_class == CPUCLASS_586 || cpu_class == CPUCLASS_686) {
+		set_i586_ctr_freq((u_int)rdtsc(), tot_count);
+		if (bootverbose)
+		        printf("i586 clock: %u Hz, ", i586_ctr_freq);
+	}
+#endif
+
+	if (bootverbose)
+	        printf("i8254 clock: %u Hz\n", tot_count);
+	return (tot_count);
+
+fail:
+	if (bootverbose)
+	        printf("failed, using default i8254 clock of %u Hz\n",
+		       timer_freq);
+	return (timer_freq);
+}
+
+static void
+set_timer_freq(u_int freq, int intr_freq)
+{
+	u_long ef;
+
+	ef = read_eflags();
+	disable_intr();
+	timer_freq = freq;
+	timer0_max_count = hardclock_max_count = TIMER_DIV(intr_freq);
+	timer0_overflow_threshold = timer0_max_count - TIMER0_LATCH_COUNT;
+	outb(TIMER_MODE, TIMER_SEL0 | TIMER_RATEGEN | TIMER_16BIT);
+	outb(TIMER_CNTR0, timer0_max_count & 0xff);
+	outb(TIMER_CNTR0, timer0_max_count >> 8);
+	write_eflags(ef);
 }
 
 /*
@@ -435,11 +632,73 @@ readrtc(int port)
 void
 startrtclock()
 {
-	timer0_max_count = hardclock_max_count = TIMER_DIV(hz);
-	timer0_overflow_threshold = timer0_max_count - TIMER0_LATCH_COUNT;
-	outb(TIMER_MODE, TIMER_SEL0 | TIMER_RATEGEN | TIMER_16BIT);
-	outb(TIMER_CNTR0, timer0_max_count & 0xff);
-	outb(TIMER_CNTR0, timer0_max_count >> 8);
+	u_int delta, freq;
+
+	writertc(RTC_STATUSA, rtc_statusa);
+	writertc(RTC_STATUSB, RTCSB_24HR);
+
+	set_timer_freq(timer_freq, hz);
+	freq = calibrate_clocks();
+#ifdef CLK_CALIBRATION_LOOP
+	if (bootverbose) {
+		printf(
+		"Press a key on the console to abort clock calibration\n");
+		while (cncheckc() == -1)
+			calibrate_clocks();
+	}
+#endif
+
+	/*
+	 * Use the calibrated i8254 frequency if it seems reasonable.
+	 * Otherwise use the default, and don't use the calibrated i586
+	 * frequency.
+	 */
+	delta = freq > timer_freq ? freq - timer_freq : timer_freq - freq;
+	if (delta < timer_freq / 100) {
+#ifndef CLK_USE_I8254_CALIBRATION
+		if (bootverbose)
+			printf(
+"CLK_USE_I8254_CALIBRATION not specified - using default frequency\n");
+		freq = timer_freq;
+#endif
+		timer_freq = freq;
+	} else {
+		if (bootverbose)
+			printf(
+		    "%d Hz differs from default of %d Hz by more than 1%%\n",
+			       freq, timer_freq);
+#if defined(I586_CPU) || defined(I686_CPU)
+		i586_ctr_freq = 0;
+#endif
+	}
+
+	set_timer_freq(timer_freq, hz);
+
+#if defined(I586_CPU) || defined(I686_CPU)
+#ifndef CLK_USE_I586_CALIBRATION
+	if (i586_ctr_freq != 0) {
+		if (bootverbose)
+			printf(
+"CLK_USE_I586_CALIBRATION not specified - using old calibration method\n");
+		i586_ctr_freq = 0;
+	}
+#endif
+	if (i586_ctr_freq == 0 &&
+	    (cpu_class == CPUCLASS_586 || cpu_class == CPUCLASS_686)) {
+		/*
+		 * Calibration of the i586 clock relative to the mc146818A
+		 * clock failed.  Do a less accurate calibration relative
+		 * to the i8254 clock.
+		 */
+		wrmsr(0x10, 0LL);	/* XXX */
+		DELAY(1000000);
+		set_i586_ctr_freq((u_int)rdtsc(), timer_freq);
+#ifdef CLK_USE_I586_CALIBRATION
+		if (bootverbose)
+			printf("i586 clock: %u Hz\n", i586_ctr_freq);
+#endif
+	}
+#endif
 }
 
 /*
@@ -493,7 +752,7 @@ inittodr(time_t base)
 	/* sec now contains the	number of seconds, since Jan 1 1970,
 	   in the local	time zone */
 
-	sec += tz.tz_minuteswest * 60 + adjkerntz;
+	sec += tz.tz_minuteswest * 60 + (wall_cmos_clock ? adjkerntz : 0);
 
 	s = splclock();
 	time.tv_sec = sec;
@@ -512,7 +771,7 @@ void
 resettodr()
 {
 	unsigned long	tm;
-	int		y, m, fd, r, s;
+	int		y, m, s;
 
 	if (disable_rtc_set)
 		return;
@@ -526,35 +785,37 @@ resettodr()
 
 	/* Calculate local time	to put in RTC */
 
-	tm -= tz.tz_minuteswest * 60 + adjkerntz;
+	tm -= tz.tz_minuteswest * 60 + (wall_cmos_clock ? adjkerntz : 0);
 
-	writertc(RTC_SEC, int2bcd(tm%60)); tm /= 60;	/* Write back Seconds */
-	writertc(RTC_MIN, int2bcd(tm%60)); tm /= 60;	/* Write back Minutes */
-	writertc(RTC_HRS, int2bcd(tm%24)); tm /= 24;	/* Write back Hours   */
+	writertc(RTC_SEC, bin2bcd(tm%60)); tm /= 60;	/* Write back Seconds */
+	writertc(RTC_MIN, bin2bcd(tm%60)); tm /= 60;	/* Write back Minutes */
+	writertc(RTC_HRS, bin2bcd(tm%24)); tm /= 24;	/* Write back Hours   */
 
 	/* We have now the days	since 01-01-1970 in tm */
 	writertc(RTC_WDAY, (tm+4)%7);			/* Write back Weekday */
-	for (y=1970;; y++)
-		if ((tm	- DAYSPERYEAR -	LEAPYEAR(y)) > tm)
-			break;
-		else
-			tm -= DAYSPERYEAR + LEAPYEAR(y);
+	for (y = 1970, m = DAYSPERYEAR + LEAPYEAR(y);
+	     tm >= m;
+	     y++,      m = DAYSPERYEAR + LEAPYEAR(y))
+	     tm -= m;
 
 	/* Now we have the years in y and the day-of-the-year in tm */
-	writertc(RTC_YEAR, int2bcd(y%100));		/* Write back Year    */
+	writertc(RTC_YEAR, bin2bcd(y%100));		/* Write back Year    */
 #ifdef USE_RTC_CENTURY
-	writertc(RTC_CENTURY, int2bcd(y/100));		/* ... and Century    */
+	writertc(RTC_CENTURY, bin2bcd(y/100));		/* ... and Century    */
 #endif
-	if (LEAPYEAR(y)	&& (tm >= 31+29))
-		tm--;					/* Subtract Feb-29 */
-	for (m=1;; m++)
-		if (tm - daysinmonth[m-1] > tm)
-			break;
-		else
-			tm -= daysinmonth[m-1];
+	for (m = 0; ; m++) {
+		int ml;
 
-	writertc(RTC_MONTH, int2bcd(m));		/* Write back Month   */
-	writertc(RTC_DAY, int2bcd(tm+1));		/* Write back Day     */
+		ml = daysinmonth[m];
+		if (m == 1 && LEAPYEAR(y))
+			ml++;
+		if (tm < ml)
+			break;
+		tm -= ml;
+	}
+
+	writertc(RTC_MONTH, bin2bcd(m + 1));            /* Write back Month   */
+	writertc(RTC_DAY, bin2bcd(tm + 1));             /* Write back Month Day */
 
 	/* Reenable RTC updates and interrupts. */
 	writertc(RTC_STATUSB, rtc_statusb);
@@ -588,6 +849,13 @@ cpu_initclocks()
 		      /* XXX */ (inthand2_t *)clkintr, &clk_imask,
 		      /* unit */ 0);
 	INTREN(IRQ0);
+#if defined(I586_CPU) || defined(I686_CPU)
+	/*
+	 * Finish setting up anti-jitter measures.
+	 */
+	if (i586_ctr_freq != 0)
+		i586_ctr_bias = rdtsc();
+#endif
 
 	/* Initialize RTC. */
 	writertc(RTC_STATUSA, rtc_statusa);
@@ -615,3 +883,70 @@ setstatclockrate(int newhz)
 		rtc_statusa = RTCSA_DIVIDER | RTCSA_NOPROF;
 	writertc(RTC_STATUSA, rtc_statusa);
 }
+
+static int
+sysctl_machdep_i8254_freq SYSCTL_HANDLER_ARGS
+{
+	int error;
+	u_int freq;
+
+	/*
+	 * Use `i8254' instead of `timer' in external names because `timer'
+	 * is is too generic.  Should use it everywhere.
+	 */
+	freq = timer_freq;
+	error = sysctl_handle_opaque(oidp, &freq, sizeof freq, req);
+	if (error == 0 && req->newptr != NULL) {
+		if (timer0_state != 0)
+			return (EBUSY);	/* too much trouble to handle */
+		set_timer_freq(freq, hz);
+#if defined(I586_CPU) || defined(I686_CPU)
+		set_i586_ctr_freq(i586_ctr_freq, timer_freq);
+#endif
+	}
+	return (error);
+}
+
+SYSCTL_PROC(_machdep, OID_AUTO, i8254_freq, CTLTYPE_INT | CTLFLAG_RW,
+	    0, sizeof(u_int), sysctl_machdep_i8254_freq, "I", "");
+
+#if defined(I586_CPU) || defined(I686_CPU)
+static void
+set_i586_ctr_freq(u_int i586_freq, u_int i8254_freq)
+{
+	u_int comultiplier, multiplier;
+	u_long ef;
+
+	if (i586_freq == 0) {
+		i586_ctr_freq = i586_freq;
+		return;
+	}
+	comultiplier = ((unsigned long long)i586_freq
+			<< I586_CTR_COMULTIPLIER_SHIFT) / i8254_freq;
+	multiplier = (1000000LL << I586_CTR_MULTIPLIER_SHIFT) / i586_freq;
+	ef = read_eflags();
+	disable_intr();
+	i586_ctr_freq = i586_freq;
+	i586_ctr_comultiplier = comultiplier;
+	i586_ctr_multiplier = multiplier;
+	write_eflags(ef);
+}
+
+static int
+sysctl_machdep_i586_freq SYSCTL_HANDLER_ARGS
+{
+	int error;
+	u_int freq;
+
+	if (cpu_class != CPUCLASS_586 && cpu_class != CPUCLASS_686)
+		return (EOPNOTSUPP);
+	freq = i586_ctr_freq;
+	error = sysctl_handle_opaque(oidp, &freq, sizeof freq, req);
+	if (error == 0 && req->newptr != NULL)
+		set_i586_ctr_freq(freq, timer_freq);
+	return (error);
+}
+
+SYSCTL_PROC(_machdep, OID_AUTO, i586_freq, CTLTYPE_INT | CTLFLAG_RW,
+	    0, sizeof(u_int), sysctl_machdep_i586_freq, "I", "");
+#endif /* defined(I586_CPU) || defined(I686_CPU) */
