@@ -1,8 +1,8 @@
 /*
  * Low-level subroutines for Cronyx-Sigma adapter.
  *
- * Copyright (C) 1994 Cronyx Ltd.
- * Author: Serge Vakulenko, <vak@zebub.msk.su>
+ * Copyright (C) 1994-95 Cronyx Ltd.
+ * Author: Serge Vakulenko, <vak@cronyx.ru>
  *
  * This software is distributed with NO WARRANTIES, not even the implied
  * warranties for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -11,7 +11,7 @@
  * or modify this software as long as this message is kept with the software,
  * all derivative works or modified versions.
  *
- * Version 1.2, Mon Nov 28 16:12:18 MSK 1994
+ * Version 1.6, Wed May 31 16:03:20 MSD 1995
  */
 #if defined (MSDOS) || defined (__MSDOS__)
 #   include <string.h>
@@ -26,14 +26,13 @@
 #   include "cxreg.h"
 #else
 #   include <sys/param.h>
+#   include <sys/systm.h>
 #   include <sys/socket.h>
 #   include <net/if.h>
 #   include <vm/vm.h>
-#   ifdef __FreeBSD__
-#      if __FreeBSD__ < 2
-#         include <machine/pio.h>
-#      endif
-#   else
+#   include <vm/vm_param.h>
+#   include <vm/pmap.h>
+#   ifndef __FreeBSD__
 #      include <machine/inline.h>
 #   endif
 #   include <machine/cronyx.h>
@@ -59,15 +58,18 @@ static unsigned char dmamask [] = {
 	BCR0_DMA_DIS,	BCR0_DMA_5,	BCR0_DMA_6,	BCR0_DMA_7,
 };
 
-long cx_rxbaud = CX_SPEED_DFLT;	/* receiver baud rate */
-long cx_txbaud = CX_SPEED_DFLT;	/* transmitter baud rate */
+static long cx_rxbaud = CX_SPEED_DFLT;	/* receiver baud rate */
+static long cx_txbaud = CX_SPEED_DFLT;	/* transmitter baud rate */
 
-int cx_univ_mode = M_ASYNC;	/* univ. chan. mode: async or sync */
-int cx_sync_mode = M_HDLC;	/* sync. chan. mode: HDLC, Bisync or X.21 */
-int cx_iftype = 0;		/* univ. chan. interface: upper/lower */
+static int cx_univ_mode = M_ASYNC;	/* univ. chan. mode: async or sync */
+static int cx_sync_mode = M_HDLC;	/* sync. chan. mode: HDLC, Bisync or X.21 */
+static int cx_iftype = 0;		/* univ. chan. interface: upper/lower */
 
 static int cx_probe_chip (int base);
 static void cx_setup_chip (cx_chip_t *c);
+static void cx_init_board (cx_board_t *b, int num, int port, int irq, int dma,
+	int chain, int rev, int osc, int rev2, int osc2);
+static void cx_reinit_board (cx_board_t *b);
 
 /*
  * Wait for CCR to clear.
@@ -159,7 +161,8 @@ static int cx_probe_chained_board (int port, int *c0, int *c1)
 /*
  * Check if the CD2400 board is present at the given base port.
  */
-int cx_probe_board (int port)
+int
+cx_probe_board (int port)
 {
 	int c0, c1, c2=0, c3=0, result;
 
@@ -223,7 +226,7 @@ static int cx_probe_chip (int base)
 	newrev = inb (GFRCR(base));
 	if (newrev != rev)
 		return (0);	/* revision changed */
-	
+
 	/* Yes, we really have CD2400 chip here. */
 	return (1);
 }
@@ -246,7 +249,8 @@ void cx_init (cx_board_t *b, int num, int port, int irq, int dma)
 /*
  * Initialize the board structure, given the type of the board.
  */
-void cx_init_board (cx_board_t *b, int num, int port, int irq, int dma,
+static void
+cx_init_board (cx_board_t *b, int num, int port, int irq, int dma,
 	int chain, int rev, int osc, int rev2, int osc2)
 {
 	cx_chan_t *c;
@@ -367,6 +371,7 @@ void cx_init_board (cx_board_t *b, int num, int port, int irq, int dma,
 		c->num = i;
 		c->board = b;
 		c->chip = b->chip + i*NCHIP/NCHAN;
+		c->stat = b->stat + i;
 		c->type = T_NONE;
 	}
 
@@ -504,7 +509,8 @@ void cx_init_board (cx_board_t *b, int num, int port, int irq, int dma,
 /*
  * Reinitialize all channels, using new options and baud rate.
  */
-void cx_reinit_board (cx_board_t *b)
+static void
+cx_reinit_board (cx_board_t *b)
 {
 	cx_chan_t *c;
 
@@ -771,20 +777,26 @@ void cx_chan_dtr (cx_chan_t *c, int on)
 		/* Channels 4..7 and 12..15 in syncronous mode
 		 * have no DTR signal. */
 		break;
-	case 0: case 1: case 2:  case 3:
+
+	case 1: case 2:  case 3:
+		if (c->type == T_UNIV_RS232)
+			break;
+	case 0:
 		if (on)
 			c->board->bcr1 |= 0x100 << c->num;
 		else
 			c->board->bcr1 &= ~(0x100 << c->num);
-		outb (CAR(c->chip->port), c->num);
 		outw (BCR1(c->board->port), c->board->bcr1);
 		break;
-	case 8: case 9: case 10: case 11:
+
+	case 9: case 10: case 11:
+		if (c->type == T_UNIV_RS232)
+			break;
+	case 8:
 		if (on)
 			c->board->bcr1b |= 0x100 << (c->num & 3);
 		else
 			c->board->bcr1b &= ~(0x100 << (c->num & 3));
-		outb (CAR(c->chip->port), c->num & 3);
 		outw (BCR1(c->board->port+0x10), c->board->bcr1b);
 		break;
 	}
@@ -794,45 +806,20 @@ void cx_chan_dtr (cx_chan_t *c, int on)
  * Control RTS signal for the channel.
  * Turn it on/off.
  */
-void cx_chan_rts (cx_chan_t *c, int on)
+void
+cx_chan_rts (cx_chan_t *c, int on)
 {
 	c->rts = on ? 1 : 0;
 	outb (CAR(c->chip->port), c->num & 3);
 	outb (MSVR_RTS(c->chip->port), on ? MSV_RTS : 0);
 }
 
-/*
- * Get the state of DSR signal of the channel.
- */
-int cx_chan_dsr (cx_chan_t *c)
-{
-	unsigned char sigval;
-
-	if (c->mode == M_ASYNC) {
-		outb (CAR(c->chip->port), c->num & 3);
-		return (inb (MSVR(c->chip->port)) & MSV_DSR ? 1 : 0);
-	}
-
-	/*
-	 * Channels 4..7 and 12..15 don't have DSR signal available.
-	 */
-	switch (c->num) {
-	default:
-		return (1);
-	case 0: case 1: case 2:  case 3:
-		sigval = inw (BSR(c->board->port)) >> 8;
-		break;
-	case 8: case 9: case 10: case 11:
-		sigval = inw (BSR(c->board->port+0x10)) >> 8;
-		break;
-	}
-	return (~sigval >> (c->num & 3) & 1);
-}
 
 /*
  * Get the state of CARRIER signal of the channel.
  */
-int cx_chan_cd (cx_chan_t *c)
+int
+cx_chan_cd (cx_chan_t *c)
 {
 	unsigned char sigval;
 
@@ -847,10 +834,18 @@ int cx_chan_cd (cx_chan_t *c)
 	switch (c->num) {
 	default:
 		return (1);
-	case 0: case 1: case 2:  case 3:
+
+	case 1: case 2:  case 3:
+		if (c->type == T_UNIV_RS232)
+			return (1);
+	case 0:
 		sigval = inw (BSR(c->board->port)) >> 8;
 		break;
-	case 8: case 9: case 10: case 11:
+
+	case 9: case 10: case 11:
+		if (c->type == T_UNIV_RS232)
+			return (1);
+	case 8:
 		sigval = inw (BSR(c->board->port+0x10)) >> 8;
 		break;
 	}
@@ -872,24 +867,18 @@ void cx_clock (long hz, long ba, int *clk, int *div)
 		}
 	}
 	/* Incorrect baud rate.  Return some meaningful values. */
-	*clk = 0; 
+	*clk = 0;
 	*div = 255;
 }
 
-void cx_disable_dma (cx_board_t *b)
-{
-	/* Disable DMA channel. */
-	outb (DMA_MASK, (b->dma & 3) | DMA_MASK_CLEAR);
-}
-
-cx_chan_opt_t chan_opt_dflt = { /* mode-independent options */
+static cx_chan_opt_t chan_opt_dflt = { /* mode-independent options */
 	{			/* cor4 */
 		7,		/* FIFO threshold, odd is better */
 		0,
 		0,              /* don't detect 1 to 0 on CTS */
 		1,		/* detect 1 to 0 on CD */
 		0,              /* detect 1 to 0 on DSR */
-	}, 
+	},
 	{			/* cor5 */
 		0,		/* receive flow control FIFO threshold */
 		0,
@@ -914,13 +903,13 @@ cx_chan_opt_t chan_opt_dflt = { /* mode-independent options */
 	},
 };
 
-cx_opt_async_t opt_async_dflt = { /* default async options */
+static cx_opt_async_t opt_async_dflt = { /* default async options */
 	{			/* cor1 */
 		8-1,		/* 8-bit char length */
 		0,		/* don't ignore parity */
 		PARM_NOPAR,	/* no parity */
 		PAR_EVEN,	/* even parity */
-	}, 
+	},
 	{			/* cor2 */
 		0,              /* disable automatic DSR */
 		1,              /* enable automatic CTS */
@@ -930,7 +919,7 @@ cx_opt_async_t opt_async_dflt = { /* default async options */
 		0,              /* disable embedded cmds */
 		0,		/* disable XON/XOFF */
 		0,		/* disable XANY */
-	}, 
+	},
 	{			/* cor3 */
 		STOPB_1,	/* 1 stop bit */
 		0,
@@ -957,7 +946,7 @@ cx_opt_async_t opt_async_dflt = { /* default async options */
 	0, 0, 0, 0, 0, 0, 0,	/* clear schr1-4, scrl, scrh, lnxt */
 };
 
-cx_opt_hdlc_t opt_hdlc_dflt = { /* default hdlc options */
+static cx_opt_hdlc_t opt_hdlc_dflt = { /* default hdlc options */
 	{			/* cor1 */
 		2,              /* 2 inter-frame flags */
 		0,		/* no-address mode */
@@ -986,13 +975,13 @@ cx_opt_hdlc_t opt_hdlc_dflt = { /* default hdlc options */
 	POLY_V41,		/* use V.41 CRC polynomial */
 };
 
-cx_opt_bisync_t opt_bisync_dflt = { /* default bisync options */
+static cx_opt_bisync_t opt_bisync_dflt = { /* default bisync options */
 	{			/* cor1 */
 		8-1,            /* 8-bit char length */
 		0,		/* don't ignore parity */
 		PARM_NOPAR,	/* no parity */
 		PAR_EVEN,	/* even parity */
-	}, 
+	},
 	{			/* cor2 */
 		3-2,            /* send three SYN chars */
 		CRC_DONT_INVERT,/* don't invert CRC (CRC-16) */
@@ -1014,13 +1003,13 @@ cx_opt_bisync_t opt_bisync_dflt = { /* default bisync options */
 	POLY_16,		/* use CRC-16 polynomial */
 };
 
-cx_opt_x21_t opt_x21_dflt = {   /* default x21 options */
+static cx_opt_x21_t opt_x21_dflt = {   /* default x21 options */
 	{			/* cor1 */
 		8-1,            /* 8-bit char length */
 		0,		/* don't ignore parity */
 		PARM_NOPAR,	/* no parity */
 		PAR_EVEN,	/* even parity */
-	}, 
+	},
 	{			/* cor2 */
 		0,
 		0,		/* disable embedded transmitter cmds */
