@@ -35,12 +35,13 @@
  *
  *	from: @(#)ufs_disksubr.c	7.16 (Berkeley) 5/4/91
  *	from: ufs_disksubr.c,v 1.8 1994/06/07 01:21:39 phk Exp $
- * $FreeBSD: stable/2.1/sys/i386/isa/diskslice_machdep.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/diskslice_machdep.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <stddef.h>
 #include <sys/param.h>
 #include <sys/buf.h>
+#include <sys/conf.h>
 #include <sys/disklabel.h>
 #define	DOSPTYP_EXTENDED	5
 #define	DOSPTYP_ONTRACK		84
@@ -48,7 +49,6 @@
 #include <sys/malloc.h>
 #include <sys/syslog.h>
 #include <sys/systm.h>
-#include <machine/cpu.h>
 
 #define TRACE(str)	do { if (dsi_debug) printf str; } while (0)
 
@@ -100,14 +100,16 @@ check_part(sname, dp, offset, nsectors, ntracks, mbr_offset )
 	/*
 	 * If ssector1 is on a cylinder >= 1024, then ssector can't be right.
 	 * Allow the C/H/S for it to be 1023/ntracks-1/nsectors, or correct
-	 * apart from the cylinder being reduced modulo 1024.
+	 * apart from the cylinder being reduced modulo 1024.  Always allow
+	 * 1023/255/63.
 	 */
 	if (ssector < ssector1
 	    && ((chs_ssect == nsectors && dp->dp_shd == ntracks - 1
 		 && chs_scyl == 1023)
-		|| (ssector1 - ssector) % (1024 * secpercyl) == 0)
-		|| (dp->dp_scyl == 255 && dp->dp_shd == 255
-		    && dp->dp_ssect == 255)) {
+		|| (secpercyl != 0
+		    && (ssector1 - ssector) % (1024 * secpercyl) == 0))
+	    || (dp->dp_scyl == 255 && dp->dp_shd == 255
+		&& dp->dp_ssect == 255)) {
 		TRACE(("%s: C/H/S start %d/%d/%d, start %lu: allow\n",
 		       sname, chs_scyl, dp->dp_shd, chs_ssect, ssector1));
 		ssector = ssector1;
@@ -123,9 +125,10 @@ check_part(sname, dp, offset, nsectors, ntracks, mbr_offset )
 	if (esector < esector1
 	    && ((chs_esect == nsectors && dp->dp_ehd == ntracks - 1
 		 && chs_ecyl == 1023)
-		|| (esector1 - esector) % (1024 * secpercyl) == 0)
-		|| (dp->dp_ecyl == 255 && dp->dp_ehd == 255
-		    && dp->dp_esect == 255)) {
+		|| (secpercyl != 0
+		    && (esector1 - esector) % (1024 * secpercyl) == 0))
+	    || (dp->dp_ecyl == 255 && dp->dp_ehd == 255
+		&& dp->dp_esect == 255)) {
 		TRACE(("%s: C/H/S end %d/%d/%d, end %lu: allow\n",
 		       sname, chs_ecyl, dp->dp_ehd, chs_esect, esector1));
 		esector = esector1;
@@ -167,7 +170,6 @@ dsinit(dname, dev, strat, lp, sspp)
 	u_long	mbr_offset;
 	char	partname[2];
 	u_long	secpercyl;
-	int	slice;
 	char	*sname;
 	struct diskslice *sp;
 	struct diskslices *ssp;
@@ -194,7 +196,7 @@ reread_mbr:
 	bp->b_dev = dkmodpart(dkmodslice(dev, WHOLE_DISK_SLICE), RAW_PART);
 	bp->b_blkno = mbr_offset;
 	bp->b_bcount = lp->d_secsize;
-	bp->b_flags = B_BUSY | B_READ;
+	bp->b_flags |= B_BUSY | B_READ;
 	(*strat)(bp);
 	if (biowait(bp) != 0) {
 		diskerr(bp, dname, "error reading primary partition table",
@@ -209,8 +211,9 @@ reread_mbr:
 	sname = dsname(dname, dkunit(dev), WHOLE_DISK_SLICE, RAW_PART,
 		       partname);
 	if (cp[0x1FE] != 0x55 || cp[0x1FF] != 0xAA) {
-		printf("%s: invalid primary partition table: no magic\n",
-		       sname);
+		if (bootverbose)
+			printf("%s: invalid primary partition table: no magic\n",
+			       sname);
 		error = EINVAL;
 		goto done;
 	}
@@ -222,7 +225,7 @@ reread_mbr:
 			if (bootverbose)
 				printf(
 	    "%s: Found \"Ontrack Disk Manager\" on this disk.\n", sname);
-			bp->b_flags = B_INVAL | B_AGE;
+			bp->b_flags |= B_INVAL | B_AGE;
 			brelse(bp);
 			mbr_offset = 63;
 			goto reread_mbr;
@@ -347,7 +350,7 @@ reread_mbr:
 				 max_nsectors, max_ntracks, mbr_offset);
 
 done:
-	bp->b_flags = B_INVAL | B_AGE;
+	bp->b_flags |= B_INVAL | B_AGE;
 	brelse(bp);
 	if (error == EINVAL)
 		error = 0;
@@ -373,7 +376,6 @@ extended(dname, dev, strat, lp, ssp, ext_offset, ext_size, base_ext_offset,
 	u_char	*cp;
 	int	dospart;
 	struct dos_partition *dp;
-	int	end_slice;
 	u_long	ext_offsets[NDOSPART];
 	u_long	ext_sizes[NDOSPART];
 	char	partname[2];
@@ -386,7 +388,7 @@ extended(dname, dev, strat, lp, ssp, ext_offset, ext_size, base_ext_offset,
 	bp->b_dev = dev;
 	bp->b_blkno = ext_offset;
 	bp->b_bcount = lp->d_secsize;
-	bp->b_flags = B_BUSY | B_READ;
+	bp->b_flags |= B_BUSY | B_READ;
 	(*strat)(bp);
 	if (biowait(bp) != 0) {
 		diskerr(bp, dname, "error reading extended partition table",
@@ -400,8 +402,9 @@ extended(dname, dev, strat, lp, ssp, ext_offset, ext_size, base_ext_offset,
 	if (cp[0x1FE] != 0x55 || cp[0x1FF] != 0xAA) {
 		sname = dsname(dname, dkunit(dev), WHOLE_DISK_SLICE, RAW_PART,
 			       partname);
-		printf("%s: invalid extended partition table: no magic\n",
-		       sname);
+		if (bootverbose)
+			printf("%s: invalid extended partition table: no magic\n",
+			       sname);
 		goto done;
 	}
 
@@ -453,6 +456,6 @@ extended(dname, dev, strat, lp, ssp, ext_offset, ext_size, base_ext_offset,
 				 mbr_offset);
 
 done:
-	bp->b_flags = B_INVAL | B_AGE;
+	bp->b_flags |= B_INVAL | B_AGE;
 	brelse(bp);
 }
