@@ -1,4 +1,4 @@
-/*-
+/*
  * Copyright (c) 1990 The Regents of the University of California.
  * All rights reserved.
  *
@@ -43,7 +43,7 @@
  * SUCH DAMAGE.
  *
  *	from:	@(#)fd.c	7.4 (Berkeley) 5/25/91
- * $FreeBSD: stable/2.1/sys/i386/isa/fd.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/fd.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -52,6 +52,7 @@
 #undef NFDC
 #endif
 #include "fd.h"
+#include "opt_fdc.h"
 
 #if NFDC > 0
 
@@ -64,14 +65,14 @@
 #include <machine/clock.h>
 #include <machine/ioctl_fd.h>
 #include <sys/disklabel.h>
-#include <sys/diskslice.h>
 #include <sys/buf.h>
 #include <sys/uio.h>
 #include <sys/malloc.h>
 #include <sys/proc.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
+#ifdef notyet
 #include <sys/dkstat.h>
+#endif
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/fdreg.h>
@@ -82,86 +83,19 @@
 #include <sys/ftape.h>
 #include <i386/isa/ftreg.h>
 #endif
-
-static int fd_goaway(struct kern_devconf *, int);
-static int fdc_goaway(struct kern_devconf *, int);
-static int
-fd_externalize(struct proc *, struct kern_devconf *, void *, size_t);
-
-/*
- * Templates for the kern_devconf structures used when we attach.
- */
-static struct kern_devconf kdc_fd[NFD] = { {
-	0, 0, 0,		/* filled in by kern_devconf.c */
-	"fd", 0, { MDDT_DISK, 0 },
-	fd_externalize, 0, fd_goaway, DISK_EXTERNALLEN,
-	0,			/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"floppy disk",
-	DC_CLS_DISK		/* class */
-} };
-
-struct kern_devconf kdc_fdc[NFDC] = { {
-	0, 0, 0,		/* filled in by kern_devconf.c */
-	"fdc", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, fdc_goaway, ISA_EXTERNALLEN,
-	0,			/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"floppy disk/tape controller",
-	DC_CLS_MISC		/* class */
-} };
-
-static inline void
-fd_registerdev(int ctlr, int unit)
-{
-	if(unit != 0)
-		kdc_fd[unit] = kdc_fd[0];
-
-	kdc_fd[unit].kdc_unit = unit;
-	kdc_fd[unit].kdc_parent = &kdc_fdc[ctlr];
-	kdc_fd[unit].kdc_parentdata = 0;
-	dev_attach(&kdc_fd[unit]);
-}
-
-static inline void
-fdc_registerdev(struct isa_device *dvp)
-{
-	int unit = dvp->id_unit;
-
-	if(unit != 0)
-		kdc_fdc[unit] = kdc_fdc[0];
-
-	kdc_fdc[unit].kdc_unit = unit;
-	kdc_fdc[unit].kdc_parent = &kdc_isa0;
-	kdc_fdc[unit].kdc_parentdata = dvp;
-	dev_attach(&kdc_fdc[unit]);
-}
-
-static int
-fdc_goaway(struct kern_devconf *kdc, int force)
-{
-	if(force) {
-		dev_detach(kdc);
-		return 0;
-	} else {
-		return EBUSY;	/* XXX fix */
-	}
-}
-
-static int
-fd_goaway(struct kern_devconf *kdc, int force)
-{
-	dev_detach(kdc);
-	return 0;
-}
-
-#define	b_cylin	b_resid		/* XXX now spelled b_cylinder elsewhere */
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif
 
 /* misuse a flag to identify format operation */
 #define B_FORMAT B_XXX
 
+/* configuration flags */
+#define FDC_PRETEND_D0	(1 << 0)	/* pretend drive 0 to be there */
+
+/* internally used only, not really from CMOS: */
+#define RTCFDT_144M_PRETENDED	0x1000
+
 /*
  * this biotab field doubles as a field for the physical unit number
  * on the controller
@@ -196,7 +130,7 @@ fd_goaway(struct kern_devconf *kdc, int force)
 #define FD_360in5_25    14
 
 
-struct fd_type fd_types[NUMTYPES] =
+static struct fd_type fd_types[NUMTYPES] =
 {
 { 21,2,0xFF,0x04,82,3444,1,FDC_500KBPS,2,0x0C,2 }, /* 1.72M in HD 3.5in */
 { 18,2,0xFF,0x1B,82,2952,1,FDC_500KBPS,2,0x6C,1 }, /* 1.48M in HD 3.5in */
@@ -226,7 +160,7 @@ struct fdc_data fdc_data[NFDC];
 * Per drive structure.							*
 * N per controller  (DRVS_PER_CTLR)					*
 \***********************************************************************/
-struct fd_data {
+static struct fd_data {
 	struct	fdc_data *fdc;	/* pointer to controller structure */
 	int	fdsu;		/* this units number on this controller */
 	int	type;		/* Drive type (FD_1440...) */
@@ -241,7 +175,13 @@ struct fd_data {
 #define FD_NO_TRACK -2
 	int	track;		/* where we think the head is */
 	int	options;	/* user configurable options, see ioctl_fd.h */
+#ifdef notyet
 	int	dkunit;		/* disk stats unit number */
+#endif
+#ifdef DEVFS
+	void	*bdevs[1 + NUMDENS + MAXPARTITIONS];
+	void	*cdevs[1 + NUMDENS + MAXPARTITIONS];
+#endif
 } fd_data[NFD];
 
 /***********************************************************************\
@@ -268,14 +208,6 @@ int ftattach(struct isa_device *, struct isa_device *, int);
 static int fdprobe(struct isa_device *);
 static int fdattach(struct isa_device *);
 
-/* exported functions */
-int fdsize (dev_t);
-void fdintr(fdcu_t);
-int Fdopen(dev_t, int, int, struct proc *);
-int fdclose(dev_t, int, int, struct proc *);
-void fdstrategy(struct buf *);
-int fdioctl(dev_t, int, caddr_t, int, struct proc *);
-
 /* needed for ft driver, thus exported */
 int in_fdc(fdcu_t);
 int out_fdc(fdcu_t, int);
@@ -296,6 +228,10 @@ static int fdstate(fdcu_t, fdc_p);
 static int retrier(fdcu_t);
 static int fdformat(dev_t, struct fd_formb *, struct proc *);
 
+static int enable_fifo(fdc_p fdc);
+
+static int fifo_threshold = 8;	/* XXX: should be accessible via sysctl */
+
 
 #define DEVIDLE		0
 #define FINDWORK	1
@@ -310,8 +246,8 @@ static int fdformat(dev_t, struct fd_formb *, struct proc *);
 #define	MOTORWAIT	10
 #define	IOTIMEDOUT	11
 
-#ifdef	DEBUG
-char *fdstates[] =
+#ifdef	FDC_DEBUG
+static char const * const fdstates[] =
 {
 "DEVIDLE",
 "FINDWORK",
@@ -328,13 +264,13 @@ char *fdstates[] =
 };
 
 /* CAUTION: fd_debug causes huge amounts of logging output */
-int	fd_debug = 0;
+static int volatile fd_debug = 0;
 #define TRACE0(arg) if(fd_debug) printf(arg)
 #define TRACE1(arg1, arg2) if(fd_debug) printf(arg1, arg2)
-#else /* DEBUG */
+#else /* FDC_DEBUG */
 #define TRACE0(arg)
 #define TRACE1(arg1, arg2)
-#endif /* DEBUG */
+#endif /* FDC_DEBUG */
 
 /* autoconfig structure */
 
@@ -342,34 +278,32 @@ struct	isa_driver fdcdriver = {
 	fdprobe, fdattach, "fdc",
 };
 
-struct isa_device *fdcdevs[NFDC];
+static	d_open_t	Fdopen;	/* NOTE, not fdopen */
+static	d_close_t	fdclose;
+static	d_ioctl_t	fdioctl;
+static	d_strategy_t	fdstrategy;
 
-/*
- * Provide hw.devconf information.
- */
-static int
-fd_externalize(struct proc *p, struct kern_devconf *kdc,
-	       void *userp, size_t len)
-{
-	return disk_externalize(fd_data[kdc->kdc_unit].fdsu, userp, &len);
-}
+#define CDEV_MAJOR 9
+#define BDEV_MAJOR 2
+static struct cdevsw fd_cdevsw;
+static struct bdevsw fd_bdevsw = 
+	{ Fdopen,	fdclose,	fdstrategy,	fdioctl,	/*2*/
+	  nodump,	nopsize,	0,	"fd",	&fd_cdevsw,	-1 };
 
-static int
-fdc_externalize(struct proc *p, struct kern_devconf *kdc,
-		void *userp, size_t len)
-{
-	return isa_externalize(fdcdevs[kdc->kdc_unit], userp, &len);
-}
+
+static struct isa_device *fdcdevs[NFDC];
 
 static int
 fdc_err(fdcu_t fdcu, const char *s)
 {
 	fdc_data[fdcu].fdc_errs++;
-	if(fdc_data[fdcu].fdc_errs < FDC_ERRMAX)
-		printf("fdc%d: %s", fdcu, s);
-	else if(fdc_data[fdcu].fdc_errs == FDC_ERRMAX)
-		printf("fdc%d: too many errors, not logging any more\n",
-		       fdcu);
+	if(s) {
+		if(fdc_data[fdcu].fdc_errs < FDC_ERRMAX)
+			printf("fdc%d: %s", fdcu, s);
+		else if(fdc_data[fdcu].fdc_errs == FDC_ERRMAX)
+			printf("fdc%d: too many errors, not logging any more\n",
+			       fdcu);
+	}
 
 	return FD_FAILED;
 }
@@ -381,7 +315,7 @@ fdc_err(fdcu_t fdcu, const char *s)
  * # of input bytes, input bytes as ints ...
  */
 
-int
+static int
 fd_cmd(fdcu_t fdcu, int n_out, ...)
 {
 	u_char cmd;
@@ -421,7 +355,47 @@ fd_cmd(fdcu_t fdcu, int n_out, ...)
 	return 0;
 }
 
-int
+static int 
+enable_fifo(fdc_p fdc)
+{
+	int i, j;
+
+	if ((fdc->flags & FDC_HAS_FIFO) == 0) {
+		
+		/*
+		 * XXX: 
+		 * Cannot use fd_cmd the normal way here, since
+		 * this might be an invalid command. Thus we send the
+		 * first byte, and check for an early turn of data directon.
+		 */
+		
+		if (out_fdc(fdc->fdcu, I8207X_CONFIGURE) < 0)
+			return fdc_err(fdc->fdcu, "Enable FIFO failed\n");
+		
+		/* If command is invalid, return */
+		j = 100000;
+		while ((i = inb(fdc->baseport + FDSTS) & (NE7_DIO | NE7_RQM))
+		       != NE7_RQM && j-- > 0)
+			if (i == (NE7_DIO | NE7_RQM)) {
+				fdc_reset(fdc);
+				return FD_FAILED;
+			}
+		if (j<0 || 
+		    fd_cmd(fdc->fdcu, 3,
+			   0, (fifo_threshold - 1) & 0xf, 0, 0) < 0) {
+			fdc_reset(fdc);
+			return fdc_err(fdc->fdcu, "Enable FIFO failed\n");
+		}
+		fdc->flags |= FDC_HAS_FIFO;
+		return 0;
+	}
+	if (fd_cmd(fdc->fdcu, 4,
+		   I8207X_CONFIGURE, 0, (fifo_threshold - 1) & 0xf, 0, 0) < 0)
+		return fdc_err(fdc->fdcu, "Re-enable FIFO failed\n");
+	return 0;
+}
+
+static int
 fd_sense_drive_status(fdc_p fdc, int *st3p)
 {
 	int st3;
@@ -436,7 +410,7 @@ fd_sense_drive_status(fdc_p fdc, int *st3p)
 	return 0;
 }
 
-int
+static int
 fd_sense_int(fdc_p fdc, int *st0p, int *cylp)
 {
 	int st0, cyl;
@@ -473,7 +447,7 @@ fd_sense_int(fdc_p fdc, int *st0p, int *cylp)
 }
 
 
-int
+static int
 fd_read_status(fdc_p fdc, int fdsu)
 {
 	int i, ret;
@@ -521,10 +495,6 @@ fdprobe(struct isa_device *dev)
 	fdcdevs[fdcu] = dev;
 	fdc_data[fdcu].baseport = dev->id_iobase;
 
-#ifndef DEV_LKM
-	fdc_registerdev(dev);
-#endif
-
 	/* First - lets reset the floppy controller */
 	outb(dev->id_iobase+FDOUT, 0);
 	DELAY(100);
@@ -537,7 +507,6 @@ fdprobe(struct isa_device *dev)
 	{
 		return(0);
 	}
-	kdc_fdc[fdcu].kdc_state = DC_IDLE;
 	return (IO_FDCSIZE);
 }
 
@@ -552,20 +521,28 @@ fdattach(struct isa_device *dev)
 	fdcu_t	fdcu = dev->id_unit;
 	fdc_p	fdc = fdc_data + fdcu;
 	fd_p	fd;
-	int	fdsu, st0, st3, i, unithasfd;
+	int	fdsu, st0, st3, i;
+#if NFT > 0
+	int	unithasfd;
+#endif
 	struct isa_device *fdup;
 	int ic_type = 0;
-#ifdef	DEVFS
-	char	name[64];
-	caddr_t key;
-#endif	/* DEVFS */
+#ifdef DEVFS
+	int	mynor;
+	int	typemynor;
+	int	typesize;
+#endif
 
 	fdc->fdcu = fdcu;
 	fdc->flags |= FDC_ATTACHED;
 	fdc->dmachan = dev->id_drq;
+	/* Acquire the DMA channel forever, The driver will do the rest */
+	isa_dma_acquire(fdc->dmachan);
+	isa_dmainit(fdc->dmachan, 128 << 3 /* XXX max secsize */);
 	fdc->state = DEVIDLE;
 	/* reset controller, turn motor off, clear fdout mirror reg */
 	outb(fdc->baseport + FDOUT, ((fdc->fdout = 0)));
+	bufq_init(&fdc->head);
 
 	/* check for each floppy drive */
 	for (fdup = isa_biotab_fdc; fdup->id_driver != 0; fdup++) {
@@ -578,7 +555,10 @@ fdattach(struct isa_device *dev)
 		fdsu = fdup->id_physid;
 		/* look up what bios thinks we have */
 		switch (fdu) {
-			case 0: fdt = (rtcin(RTC_FDISKETTE) & 0xf0);
+			case 0: if (dev->id_flags & FDC_PRETEND_D0)
+					fdt = RTCFDT_144M | RTCFDT_144M_PRETENDED;
+				else
+					fdt = (rtcin(RTC_FDISKETTE) & 0xf0);
 				break;
 			case 1: fdt = ((rtcin(RTC_FDISKETTE) << 4) & 0xf0);
 				break;
@@ -617,32 +597,43 @@ fdattach(struct isa_device *dev)
 		if (ic_type == 0 &&
 		    fd_cmd(fdcu, 1, NE7CMD_VERSION, 1, &ic_type) == 0)
 		{
+#ifdef FDC_PRINT_BOGUS_CHIPTYPE
 			printf("fdc%d: ", fdcu);
+#endif
 			ic_type = (u_char)ic_type;
 			switch( ic_type ) {
 			case 0x80:
+#ifdef FDC_PRINT_BOGUS_CHIPTYPE
 				printf("NEC 765\n");
+#endif
 				fdc->fdct = FDC_NE765;
-				kdc_fdc[fdcu].kdc_description =
-					"NEC 765 floppy disk/tape controller";
 				break;
 			case 0x81:
+#ifdef FDC_PRINT_BOGUS_CHIPTYPE
 				printf("Intel 82077\n");
+#endif
 				fdc->fdct = FDC_I82077;
-				kdc_fdc[fdcu].kdc_description =
-					"Intel 82077 floppy disk/tape controller";
 				break;
 			case 0x90:
+#ifdef FDC_PRINT_BOGUS_CHIPTYPE
 				printf("NEC 72065B\n");
+#endif
 				fdc->fdct = FDC_NE72065;
-				kdc_fdc[fdcu].kdc_description =
-					"NEC 72065B floppy disk/tape controller";
 				break;
 			default:
+#ifdef FDC_PRINT_BOGUS_CHIPTYPE
 				printf("unknown IC type %02x\n", ic_type);
+#endif
 				fdc->fdct = FDC_UNKNOWN;
 				break;
 			}
+			if (fdc->fdct != FDC_NE765 &&
+			    fdc->fdct != FDC_UNKNOWN && 
+			    enable_fifo(fdc) == 0) {
+				printf("fdc%d: FIFO enabled", fdcu);
+				printf(", %d bytes threshold\n", 
+				       fifo_threshold);
+			}
 		}
 		if ((fd_cmd(fdcu, 2, NE7CMD_SENSED, fdsu, 1, &st3) == 0) &&
 		    (st3 & NE7_ST3_T0)) {
@@ -695,69 +686,101 @@ fdattach(struct isa_device *dev)
 		fd->options = 0;
 		printf("fd%d: ", fdu);
 
-		fd_registerdev(fdcu, fdu);
 		switch (fdt) {
 		case RTCFDT_12M:
 			printf("1.2MB 5.25in\n");
 			fd->type = FD_1200;
-			kdc_fd[fdu].kdc_description =
-				"1.2MB (1200K) 5.25in floppy disk drive";
-#ifdef	DEVFS
-			sprintf(name,"fd%d.1200",fdu);
-#endif	/* DEVFS */
 			break;
+		case RTCFDT_144M | RTCFDT_144M_PRETENDED:
+			printf("config-pretended ");
+			fdt = RTCFDT_144M;
+			/* fallthrough */
 		case RTCFDT_144M:
 			printf("1.44MB 3.5in\n");
 			fd->type = FD_1440;
-			kdc_fd[fdu].kdc_description =
-				"1.44MB (1440K) 3.5in floppy disk drive";
-#ifdef	DEVFS
-			sprintf(name,"fd%d.1440",fdu);
-#endif	/* DEVFS */
 			break;
 		case RTCFDT_288M:
 		case RTCFDT_288M_1:
 			printf("2.88MB 3.5in - 1.44MB mode\n");
 			fd->type = FD_1440;
-			kdc_fd[fdu].kdc_description =
-				"2.88MB (2880K) 3.5in floppy disk drive in 1.44 mode";
-#ifdef	DEVFS
-			sprintf(name,"fd%d.1440",fdu);
-#endif	/* DEVFS */
 			break;
 		case RTCFDT_360K:
 			printf("360KB 5.25in\n");
 			fd->type = FD_360;
-			kdc_fd[fdu].kdc_description =
-				"360KB 5.25in floppy disk drive";
-#ifdef	DEVFS
-			sprintf(name,"fd%d.360",fdu);
-#endif	/* DEVFS */
 			break;
 		case RTCFDT_720K:
 			printf("720KB 3.5in\n");
 			fd->type = FD_720;
-			kdc_fd[fdu].kdc_description =
-				"720KB 3.5in floppy disk drive";
-#ifdef	DEVFS
-			sprintf(name,"fd%d.720",fdu);
-#endif	/* DEVFS */
 			break;
 		default:
 			printf("unknown\n");
 			fd->type = NO_TYPE;
-#ifdef	DEVFS
-			sprintf(name,"fd%d.xxxx",fdu);
-#endif	/* DEVFS */
-			break;
+			continue;
 		}
-		kdc_fd[fdu].kdc_state = DC_IDLE;
 #ifdef DEVFS
-		key = dev_add("/disks/rfloppy",name,(caddr_t)Fdopen,fdu * 8,
-			0,0,0,0644);
-		key = dev_add("/disks/floppy",name,(caddr_t)Fdopen,fdu * 8,
-			1,0,0,0644);
+		mynor = fdu << 6;
+		fd->bdevs[0] = devfs_add_devswf(&fd_bdevsw, mynor, DV_BLK,
+						UID_ROOT, GID_OPERATOR, 0640,
+						"fd%d", fdu);
+		fd->cdevs[0] = devfs_add_devswf(&fd_cdevsw, mynor, DV_CHR,
+						UID_ROOT, GID_OPERATOR, 0640,
+						"rfd%d", fdu);
+		for (i = 1; i < 1 + NUMDENS; i++) {
+			/*
+			 * XXX this and the lookup in Fdopen() should be
+			 * data driven.
+			 */
+			switch (fd->type) {
+			case FD_360:
+				if (i != FD_360)
+					continue;
+				break;
+			case FD_720:
+				if (i != FD_720 && i != FD_800 && i != FD_820)
+					continue;
+				break;
+			case FD_1200:
+				if (i != FD_360 && i != FD_720 && i != FD_800
+				    && i != FD_820 && i != FD_1200
+				    && i != FD_1440 && i != FD_1480)
+					continue;
+				break;
+			case FD_1440:
+				if (i != FD_720 && i != FD_800 && i != FD_820
+				    && i != FD_1200 && i != FD_1440
+				    && i != FD_1480 && i != FD_1720)
+					continue;
+				break;
+			}
+			typemynor = mynor | i;
+			typesize = fd_types[i - 1].size / 2;
+			/*
+			 * XXX all these conversions give bloated code and
+			 * confusing names.
+			 */
+			if (typesize == 1476)
+				typesize = 1480;
+			if (typesize == 1722)
+				typesize = 1720;
+			fd->bdevs[i] =
+				devfs_add_devswf(&fd_bdevsw, typemynor, DV_BLK,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "fd%d.%d", fdu, typesize);
+			fd->cdevs[i] =
+				devfs_add_devswf(&fd_cdevsw, typemynor, DV_CHR,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "rfd%d.%d", fdu, typesize);
+		}
+		for (i = 0; i < MAXPARTITIONS; i++) {
+			fd->bdevs[1 + NUMDENS + i] =
+				devfs_link(fd->bdevs[0],
+					   "fd%d%c", fdu, 'a' + i);
+			fd->cdevs[1 + NUMDENS + i] =
+				devfs_link(fd->cdevs[0],
+					   "rfd%d%c", fdu, 'a' + i);
+		}
 #endif /* DEVFS */
+#ifdef notyet
 		if (dk_ndrive < DK_NDRIVE) {
 			sprintf(dk_names[dk_ndrive], "fd%d", fdu);
 			fd->dkunit = dk_ndrive++;
@@ -768,17 +791,12 @@ fdattach(struct isa_device *dev)
 		} else {
 			fd->dkunit = -1;
 		}
+#endif
 	}
 
 	return (1);
 }
 
-int
-fdsize(dev_t dev)
-{
-	return(0);
-}
-
 /****************************************************************************/
 /*                            motor control stuff                           */
 /*		remember to not deselect the drive we're working on         */
@@ -808,7 +826,6 @@ set_motor(fdcu_t fdcu, int fdsu, int turnon)
 
 	outb(fdc_data[fdcu].baseport+FDOUT, fdout);
 	fdc_data[fdcu].fdout = fdout;
-	kdc_fdc[fdcu].kdc_state = (fdout & FDO_FRST)? DC_BUSY: DC_IDLE;
 	TRACE1("[0x%x->FDOUT]", fdout);
 
 	if(needspecify) {
@@ -821,10 +838,11 @@ set_motor(fdcu_t fdcu, int fdsu, int turnon)
 		(void)fd_cmd(fdcu, 3, NE7CMD_SPECIFY,
 			     NE7_SPEC_1(3, 240), NE7_SPEC_2(2, 0),
 			     0);
+		if (fdc_data[fdcu].flags & FDC_HAS_FIFO)
+			(void) enable_fifo(&fdc_data[fdcu]);
 	}
 }
 
-/* ARGSUSED */
 static void
 fd_turnoff(void *arg1)
 {
@@ -850,7 +868,6 @@ fd_turnoff(void *arg1)
 	splx(s);
 }
 
-/* ARGSUSED */
 static void
 fd_motor_on(void *arg1)
 {
@@ -899,6 +916,8 @@ fdc_reset(fdc_p fdc)
 	(void)fd_cmd(fdcu, 3, NE7CMD_SPECIFY,
 		     NE7_SPEC_1(3, 240), NE7_SPEC_2(2, 0),
 		     0);
+	if (fdc->flags & FDC_HAS_FIFO)
+		(void) enable_fifo(fdc);
 }
 
 /****************************************************************************/
@@ -914,14 +933,14 @@ in_fdc(fdcu_t fdcu)
 		if (i == NE7_RQM)
 			return fdc_err(fdcu, "ready for output in input\n");
 	if (j <= 0)
-		return fdc_err(fdcu, "input ready timeout\n");
-#ifdef	DEBUG
+		return fdc_err(fdcu, bootverbose? "input ready timeout\n": 0);
+#ifdef	FDC_DEBUG
 	i = inb(baseport+FDDATA);
 	TRACE1("[FDDATA->0x%x]", (unsigned char)i);
 	return(i);
-#else
+#else	/* !FDC_DEBUG */
 	return inb(baseport+FDDATA);
-#endif
+#endif	/* FDC_DEBUG */
 }
 
 /*
@@ -937,18 +956,18 @@ fd_in(fdcu_t fdcu, int *ptr)
 		if (i == NE7_RQM)
 			return fdc_err(fdcu, "ready for output in input\n");
 	if (j <= 0)
-		return fdc_err(fdcu, "input ready timeout\n");
-#ifdef	DEBUG
+		return fdc_err(fdcu, bootverbose? "input ready timeout\n": 0);
+#ifdef	FDC_DEBUG
 	i = inb(baseport+FDDATA);
 	TRACE1("[FDDATA->0x%x]", (unsigned char)i);
 	*ptr = i;
 	return 0;
-#else
+#else	/* !FDC_DEBUG */
 	i = inb(baseport+FDDATA);
 	if (ptr)
 		*ptr = i;
 	return 0;
-#endif
+#endif	/* FDC_DEBUG */
 }
 
 int
@@ -965,7 +984,8 @@ out_fdc(fdcu_t fdcu, int x)
 	/* Check that the floppy controller is ready for a command */
 	i = 100000;
 	while ((inb(baseport+FDSTS) & NE7_RQM) == 0 && i-- > 0);
-	if (i <= 0) return fdc_err(fdcu, "output ready timeout\n");
+	if (i <= 0)
+		return fdc_err(fdcu, bootverbose? "output ready timeout\n": 0);
 
 	/* Send the command and return */
 	outb(baseport+FDDATA, x);
@@ -982,7 +1002,6 @@ Fdopen(dev_t dev, int flags, int mode, struct proc *p)
  	fdu_t fdu = FDUNIT(minor(dev));
 	int type = FDTYPE(minor(dev));
 	fdc_p	fdc;
-	int     st3;
 
 #if NFT > 0
 	/* check for a tape open */
@@ -1049,7 +1068,6 @@ Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 	}
 	fd_data[fdu].ft = fd_types + type - 1;
 	fd_data[fdu].flags |= FD_OPEN;
-	kdc_fd[fdu].kdc_state = DC_BUSY;
 
 	return 0;
 }
@@ -1067,7 +1085,6 @@ fdclose(dev_t dev, int flags, int mode, struct proc *p)
 #endif
 	fd_data[fdu].flags &= ~FD_OPEN;
 	fd_data[fdu].options &= ~FDOPT_NORETRY;
-	kdc_fd[fdu].kdc_state = DC_IDLE;
 
 	return(0);
 }
@@ -1079,8 +1096,7 @@ fdclose(dev_t dev, int flags, int mode, struct proc *p)
 void
 fdstrategy(struct buf *bp)
 {
-	register struct buf *dp;
-	long nblocks, blknum;
+	unsigned nblocks, blknum, cando;
  	int	s;
  	fdcu_t	fdcu;
  	fdu_t	fdu;
@@ -1127,21 +1143,33 @@ fdstrategy(struct buf *bp)
 	/*
 	 * Set up block calculations.
 	 */
-	blknum = (unsigned long) bp->b_blkno * DEV_BSIZE/fdblk;
- 	nblocks = fd->ft->size;
-	if (blknum + (bp->b_bcount / fdblk) > nblocks) {
-		if (blknum == nblocks) {
-			bp->b_resid = bp->b_bcount;
-		} else {
-			bp->b_error = ENOSPC;
-			bp->b_flags |= B_ERROR;
-		}
+	if (bp->b_blkno > 20000000) {
+		/*
+		 * Reject unreasonably high block number, prevent the
+		 * multiplication below from overflowing.
+		 */
+		bp->b_error = EINVAL;
+		bp->b_flags |= B_ERROR;
 		goto bad;
 	}
- 	bp->b_cylin = blknum / (fd->ft->sectrac * fd->ft->heads);
-	dp = &(fdc->head);
+	blknum = (unsigned) bp->b_blkno * DEV_BSIZE/fdblk;
+ 	nblocks = fd->ft->size;
+	bp->b_resid = 0;
+	if (blknum + (bp->b_bcount / fdblk) > nblocks) {
+		if (blknum <= nblocks) {
+			cando = (nblocks - blknum) * fdblk;
+			bp->b_resid = bp->b_bcount - cando;
+			if (cando == 0)
+				goto bad;	/* not actually bad but EOF */
+		} else {
+			bp->b_error = EINVAL;
+			bp->b_flags |= B_ERROR;
+			goto bad;
+		}
+	}
+ 	bp->b_pblkno = bp->b_blkno;
 	s = splbio();
-	disksort(dp, bp);
+	bufqdisksort(&fdc->head, bp);
 	untimeout(fd_turnoff, (caddr_t)fdu); /* a good idea */
 	fdstart(fdcu);
 	splx(s);
@@ -1173,18 +1201,16 @@ fdstart(fdcu_t fdcu)
 	splx(s);
 }
 
-/* ARGSUSED */
 static void
 fd_timeout(void *arg1)
 {
 	fdcu_t fdcu = (fdcu_t)arg1;
 	fdu_t fdu = fdc_data[fdcu].fdu;
 	int baseport = fdc_data[fdcu].baseport;
-	struct buf *dp, *bp;
+	struct buf *bp;
 	int s;
 
-	dp = &fdc_data[fdcu].head;
-	bp = dp->b_actf;
+	bp = bufq_first(&fdc_data[fdcu].head);
 
 	/*
 	 * Due to IBM's brain-dead design, the FDC has a faked ready
@@ -1225,7 +1251,6 @@ fd_timeout(void *arg1)
 }
 
 /* just ensure it has the right spl */
-/* ARGSUSED */
 static void
 fd_pseudointr(void *arg1)
 {
@@ -1264,18 +1289,16 @@ fdintr(fdcu_t fdcu)
 static int
 fdstate(fdcu_t fdcu, fdc_p fdc)
 {
-	int read, format, head, sec = 0, i = 0, sectrac, st0, cyl, st3;
-	unsigned long blknum;
+	int read, format, head, sec = 0, sectrac, st0, cyl, st3;
+	unsigned blknum = 0, b_cylinder = 0;
 	fdu_t fdu = fdc->fdu;
 	fd_p fd;
-	register struct buf *dp, *bp;
+	register struct buf *bp;
 	struct fd_formb *finfo = NULL;
 	size_t fdblk;
 
-	dp = &(fdc->head);
-	bp = dp->b_actf;
-	if(!bp)
-	{
+	bp = bufq_first(&fdc->head);
+	if(!bp) {
 		/***********************************************\
 		* nothing left for this controller to do	*
 		* Force into the IDLE state,			*
@@ -1300,8 +1323,16 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 	}
 	read = bp->b_flags & B_READ;
 	format = bp->b_flags & B_FORMAT;
-	if(format)
+	if(format) {
 		finfo = (struct fd_formb *)bp->b_un.b_addr;
+		fd->skip = (char *)&(finfo->fd_formb_cylno(0))
+			- (char *)finfo;
+	}
+	if (fdc->state == DOSEEK || fdc->state == SEEKCOMPLETE) {
+		blknum = (unsigned) bp->b_blkno * DEV_BSIZE/fdblk +
+			fd->skip/fdblk;
+		b_cylinder = blknum / (fd->ft->sectrac * fd->ft->heads);
+	}
 	TRACE1("fd%d", fdu);
 	TRACE1("[%s]", fdstates[fdc->state]);
 	TRACE1("(0x%x)", fd->flags);
@@ -1342,13 +1373,13 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 		fdc->state = DOSEEK;
 		break;
 	case DOSEEK:
-		if (bp->b_cylin == fd->track)
+		if (b_cylinder == (unsigned)fd->track)
 		{
 			fdc->state = SEEKCOMPLETE;
 			break;
 		}
 		if (fd_cmd(fdcu, 3, NE7CMD_SEEK,
-			   fd->fdsu, bp->b_cylin * fd->ft->steptrac,
+			   fd->fdsu, b_cylinder * fd->ft->steptrac,
 			   0))
 		{
 			/*
@@ -1370,7 +1401,7 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 		/* Make sure seek really happened*/
 		if(fd->track == FD_NO_TRACK)
 		{
-			int descyl = bp->b_cylin * fd->ft->steptrac;
+			int descyl = b_cylinder * fd->ft->steptrac;
 			do {
 				/*
 				 * This might be a "ready changed" interrupt,
@@ -1434,14 +1465,9 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 			}
 		}
 
-		fd->track = bp->b_cylin;
-		if(format)
-			fd->skip = (char *)&(finfo->fd_formb_cylno(0))
-				- (char *)finfo;
+		fd->track = b_cylinder;
 		isa_dmastart(bp->b_flags, bp->b_un.b_addr+fd->skip,
 			format ? bp->b_bcount : fdblk, fdc->dmachan);
-		blknum = (unsigned long)bp->b_blkno*DEV_BSIZE/fdblk
-			+ fd->skip/fdblk;
 		sectrac = fd->ft->sectrac;
 		sec = blknum %  (sectrac * fd->ft->heads);
 		head = sec / sectrac;
@@ -1559,21 +1585,16 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 		}
 		/* All OK */
 		fd->skip += fdblk;
-		if (!format && fd->skip < bp->b_bcount)
+		if (!format && fd->skip < bp->b_bcount - bp->b_resid)
 		{
 			/* set up next transfer */
-			blknum = (unsigned long)bp->b_blkno*DEV_BSIZE/fdblk
-				+ fd->skip/fdblk;
-			bp->b_cylin =
-				(blknum / (fd->ft->sectrac * fd->ft->heads));
 			fdc->state = DOSEEK;
 		}
 		else
 		{
 			/* ALL DONE */
 			fd->skip = 0;
-			bp->b_resid = 0;
-			dp->b_actf = bp->b_actf;
+			bufq_remove(&fdc->head, bp);
 			biodone(bp);
 			fdc->fd = (fd_p) 0;
 			fdc->fdu = -1;
@@ -1681,10 +1702,9 @@ retrier(fdcu)
 	fdcu_t fdcu;
 {
 	fdc_p fdc = fdc_data + fdcu;
-	register struct buf *dp, *bp;
+	register struct buf *bp;
 
-	dp = &(fdc->head);
-	bp = dp->b_actf;
+	bp = bufq_first(&fdc->head);
 
 	if(fd_data[FDUNIT(minor(bp->b_dev))].options & FDOPT_NORETRY)
 		goto fail;
@@ -1727,8 +1747,8 @@ retrier(fdcu)
 		}
 		bp->b_flags |= B_ERROR;
 		bp->b_error = EIO;
-		bp->b_resid = bp->b_bcount - fdc->fd->skip;
-		dp->b_actf = bp->b_actf;
+		bp->b_resid += bp->b_bcount - fdc->fd->skip;
+		bufq_remove(&fdc->head, bp);
 		fdc->fd->skip = 0;
 		biodone(bp);
 		fdc->state = FINDWORK;
@@ -1850,7 +1870,8 @@ fdioctl(dev, cmd, addr, flag, p)
 		dl->d_secpercyl = fdt->size / fdt->tracks;
 		dl->d_type = DTYPE_FLOPPY;
 
-		if (readdisklabel(dev, fdstrategy, dl, NULL, 0) == NULL)
+		if (readdisklabel(dkmodpart(dev, RAW_PART), fdstrategy, dl)
+		    == NULL)
 			error = 0;
 		else
 			error = EINVAL;
@@ -1877,8 +1898,8 @@ fdioctl(dev, cmd, addr, flag, p)
 
 		dl = (struct disklabel *)addr;
 
-		if ((error =
-		     setdisklabel ((struct disklabel *)buffer, dl, 0)))
+		if ((error = setdisklabel((struct disklabel *)buffer, dl,
+					  (u_long)0)) != 0)
 			break;
 
 		error = writedisklabel(dev, fdstrategy,
@@ -1921,6 +1942,20 @@ fdioctl(dev, cmd, addr, flag, p)
 	return (error);
 }
 
+
+static fd_devsw_installed = 0;
+
+static void 	fd_drvinit(void *notused )
+{
+
+	if( ! fd_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &fd_bdevsw);
+		fd_devsw_installed = 1;
+	}
+}
+
+SYSINIT(fddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,fd_drvinit,NULL)
+
 #endif
 /*
  * Hello emacs, these are the
