@@ -17,7 +17,7 @@
  *  POSSIBILITY OF SUCH DAMAGE.
  *
  *  ft.c - QIC-40/80 floppy tape driver
- * $FreeBSD: stable/2.1/sys/i386/isa/ft.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/ft.c 50950 1999-09-05 08:39:22Z peter $
  *
  *  01/19/95 ++sg
  *  Cleaned up recalibrate/seek code at attach time for FreeBSD 2.x.
@@ -78,17 +78,21 @@
 #include <sys/buf.h>
 #include <sys/uio.h>
 #include <sys/ftape.h>
-#include <sys/devconf.h>
 
 #include <machine/clock.h>
 
 #include <i386/isa/isa_device.h>
+#ifdef PC98
+#include <pc98/pc98/fdreg.h>
+#else
 #include <i386/isa/fdreg.h>
-#include <i386/isa/fdc.h>
-#include <i386/isa/icu.h>
 #include <i386/isa/rtc.h>
+#endif
+#include <i386/isa/fdc.h>
 #include <i386/isa/ftreg.h>
 
+extern int ftintr __P((ftu_t ftu));
+
 /* Enable or disable debugging messages. */
 #define FTDBGALL 0			/* 1 if you want everything */
 /*#define DPRT(a) printf a		*/
@@ -145,7 +149,7 @@ enum {
 };
 
 /* Tape geometries table */
-QIC_Geom ftgtbl[] = {
+static QIC_Geom ftgtbl[] = {
 	{ 0, 0, "Unformatted", "Unknown", 0,  0,     0,   0,     0 }, /* XXX */
 	{ 1, 1, "QIC-40",  "205/550",	20,  68,  2176, 128, 21760 },
 	{ 1, 2, "QIC-40",  "307.5/550",	20, 102,  3264, 128, 32640 },
@@ -165,7 +169,7 @@ QIC_Geom ftgtbl[] = {
 };
 #define NGEOM	(sizeof(ftgtbl) / sizeof(QIC_Geom))
 
-QIC_Geom *ftg = NULL;			/* Current tape's geometry */
+static QIC_Geom *ftg = NULL;			/* Current tape's geometry */
 
 /*
  *  things relating to asynchronous commands
@@ -237,7 +241,7 @@ extern struct fdc_data fdc_data[NFDC];
 /***********************************************************************\
 * Per tape drive structure.						*
 \***********************************************************************/
-struct ft_data {
+static struct ft_data {
 	struct	fdc_data *fdc;	/* pointer to controller structure */
 	int	ftsu;		/* this units number on this controller */
 	int	type;		/* Drive type (Mountain, Colorado) */
@@ -274,7 +278,7 @@ struct ft_data {
 	int moving;		/* TRUE if tape is moving	  */
 	int rid[7];		/* read_id return values	  */
 
-} ft_data[NFT];
+} *ft_data[NFT];
 
 /***********************************************************************\
 * Throughout this file the following conventions will be used:		*
@@ -292,10 +296,8 @@ struct ft_data {
 
 int ftopen(dev_t, int);
 int ftclose(dev_t, int);
-void ftstrategy(struct buf *);
 int ftioctl(dev_t, int, caddr_t, int, struct proc *);
-int ftdump(dev_t);
-int ftsize(dev_t);
+int ftattach(struct isa_device *, struct isa_device *, int);
 static timeout_t ft_timeout;
 static void async_cmd(ftu_t);
 static void async_req(ftu_t, int);
@@ -401,41 +403,6 @@ segio_free(ft_p ft, SegReq *sp)
   DPRT(("segio_free: nfree=%d ndone=%d nreq=%d\n", ft->nfreelist, ft->ndoneq, ft->nsegq));
 }
 
-static int ft_externalize(struct proc *, struct kern_devconf *, void *,
-			  size_t);
-
-extern struct kern_devconf kdc_fdc[];
-static struct kern_devconf kdc_ft[NFT] = { {
-	0, 0, 0,		/* filled in by kern_devconf.c */
-	"ft", 0, { MDDT_DISK, 0 },
-	ft_externalize, 0, 0, DISK_EXTERNALLEN,
-	0,			/* parent */
-	0,			/* parentdata */
-	DC_IDLE,		/* state */
-	"floppy tape",
-	DC_CLS_TAPE		/* class */
-} };
-
-static inline void
-ft_registerdev(int ctlr, int unit)
-{
-	if(unit != 0)
-		kdc_ft[unit] = kdc_ft[0];
-
-	kdc_ft[unit].kdc_unit = unit;
-	kdc_ft[unit].kdc_parent = &kdc_fdc[ctlr];
-	kdc_ft[unit].kdc_parentdata = 0;
-	dev_attach(&kdc_ft[unit]);
-}
-
-
-static int
-ft_externalize(struct proc *p, struct kern_devconf *kdc, void *userp,
-	       size_t len)
-{
-	return disk_externalize(ft_data[kdc->kdc_unit].ftsu, userp, &len);
-}
-
 /*
  *  Probe/attach floppy tapes.
  */
@@ -453,7 +420,8 @@ ftattach(isadev, fdup, unithasfd)
   char *manu;
 
   if (ftu >= NFT) return 0;
-  ft = &ft_data[ftu];
+  ft = ft_data[ftu] = malloc(sizeof *ft, M_DEVBUF, M_NOWAIT);
+  bzero(ft, sizeof *ft);
 
   /* Probe for tape */
   ft->attaching = 1;
@@ -524,49 +492,38 @@ out:
   tape_end(ftu);
   if (ft->type != NO_TYPE) {
 	fdc->flags |= FDC_HASFTAPE;
-	ft_registerdev(fdcu, ftu);
 	switch(hw.hw_make) {
 	    case 0x0000:
 		if (ft->type == FT_COLORADO) {
 			manu = "Colorado";
-			kdc_ft[ftu].kdc_description = "Colorado floppy tape";
 		} else if (ft->type == FT_INSIGHT) {
 			manu = "Insight";
-			kdc_ft[ftu].kdc_description = "Insight floppy tape";
 		} else if (ft->type == FT_MOUNTAIN && hw.hw_model == 0x05) {
 			manu = "Archive";
-			kdc_ft[ftu].kdc_description = "Archive floppy tape";
 		} else if (ft->type == FT_MOUNTAIN) {
 			manu = "Mountain";
-			kdc_ft[ftu].kdc_description = "Mountain floppy tape";
 		} else {
 			manu = "Unknown";
 		}
 		break;
 	    case 0x0001:
 		manu = "Colorado";
-		kdc_ft[ftu].kdc_description = "Colorado floppy tape";
 		break;
 	    case 0x0005:
 		if (hw.hw_model >= 0x09) {
 			manu = "Conner";
-			kdc_ft[ftu].kdc_description = "Conner floppy tape";
 		} else {
 			manu = "Archive";
-			kdc_ft[ftu].kdc_description = "Archive floppy tape";
 		}
 		break;
 	    case 0x0006:
 		manu = "Mountain";
-		kdc_ft[ftu].kdc_description = "Mountain floppy tape";
 		break;
 	    case 0x0007:
 		manu = "Wangtek";
-		kdc_ft[ftu].kdc_description = "Wangtek floppy tape";
 		break;
 	    case 0x0222:
 		manu = "IOMega";
-		kdc_ft[ftu].kdc_description = "IOMega floppy tape";
 		break;
 	    default:
 		manu = "Unknown";
@@ -584,7 +541,7 @@ out:
  */
 static void
 async_cmd(ftu_t ftu) {
-	ft_p	ft = &ft_data[ftu];
+	ft_p	ft = ft_data[ftu];
 	fdcu_t	fdcu = ft->fdc->fdcu;
 	int cmd, i, st0, st3, pcn;
 	static int bitn, retval, retpos, nbits, newcn;
@@ -1007,7 +964,7 @@ complete:
 static void
 async_req(ftu_t ftu, int from)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   SegReq *sp;
   static int over_async, lastreq;
   int cmd;
@@ -1136,7 +1093,7 @@ restate:
 static void
 async_read(ftu_t ftu, int from)
 {
-  ft_p ft = &ft_data[ftu];
+  ft_p ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
   int i, rddta[7];
   int where;
@@ -1257,7 +1214,7 @@ restate:
 static void
 async_write(ftu_t ftu, int from)
 {
-  ft_p ft = &ft_data[ftu];
+  ft_p ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
   int i, rddta[7];
   int where;
@@ -1382,7 +1339,7 @@ int
 ftintr(ftu_t ftu)
 {
   int st0, pcn, i;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
   int s = splbio();
 
@@ -1460,7 +1417,7 @@ ft_timeout(void *arg1)
 {
   int s;
   ftu_t ftu = (ftu_t)arg1;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   s = splbio();
   if (ft->active) {
@@ -1487,7 +1444,7 @@ static int
 ftintr_wait(ftu_t ftu, int cmd, int ticks)
 {
   int retries, st0, pcn;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
 
   ft->cmd_wait = cmd;
@@ -1549,14 +1506,22 @@ static int
 tape_recal(ftu_t ftu, int totape)
 {
   int s;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
 
   DPRT(("tape_recal start\n"));
 
+#ifdef PC98
+  outb(0xbe, FDP_FDDEXC | FDP_PORTEXC);
+#endif
   out_fdc(fdcu, NE7CMD_SPECIFY);
+#ifdef PC98
+  out_fdc(fdcu, (totape) ? 0xEF : 0xCF);
+  out_fdc(fdcu, 0x02);
+#else
   out_fdc(fdcu, (totape) ? 0xAD : 0xDF);
   out_fdc(fdcu, 0x02);
+#endif
 
   s = splbio();
   out_fdc(fdcu, NE7CMD_RECAL);
@@ -1570,8 +1535,13 @@ tape_recal(ftu_t ftu, int totape)
   splx(s);
 
   out_fdc(fdcu, NE7CMD_SPECIFY);
+#ifdef PC98
+  out_fdc(fdcu, (totape) ? 0xEF : 0xCF);
+  out_fdc(fdcu, 0x02);
+#else
   out_fdc(fdcu, (totape) ? 0xFD : 0xDF);
   out_fdc(fdcu, 0x02);
+#endif
 
   DPRT(("tape_recal end\n"));
   return(0);
@@ -1610,7 +1580,7 @@ tape_cmd(ftu_t ftu, int cmd)
   int newcn;
   int retries = 0;
   int s;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
 
   DPRT(("===> tape_cmd: %d\n",cmd));
@@ -1651,7 +1621,7 @@ static int
 tape_status(ftu_t ftu)
 {
   int r, err, tries;
-  ft_p ft = &ft_data[ftu];
+  ft_p ft = ft_data[ftu];
   int max = (ft->attaching) ? 2 : 3;
 
   for (r = -1, tries = 0; r < 0 && tries < max; tries++)
@@ -1696,24 +1666,34 @@ recheck:
 static void
 tape_start(ftu_t ftu, int motor)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdc_p	fdc = ft->fdc;
   int s, mbits;
+#ifndef PC98
   static int mbmotor[] = { FDO_MOEN0, FDO_MOEN1, FDO_MOEN2, FDO_MOEN3 };
+#endif
 
   s = splbio();
   DPRT(("tape_start start\n"));
 
   /* reset, dma disable */
+#ifdef PC98
+  outb(fdc->baseport+FDOUT, FDO_RST | FDO_FRY | FDO_AIE | FDO_MTON);
+#else
   outb(fdc->baseport+FDOUT, 0x00);
+#endif
   (void)ftintr_wait(ftu, FTCMD_RESET, hz/10);
 
   /* raise reset, enable DMA, motor on if needed */
+#ifdef PC98
+  outb(fdc->baseport+FDOUT, FDO_DMAE | FDO_MTON);
+#else
   mbits = ftu & 3;
   if (motor && ftu < 4)
 	mbits |= mbmotor[ftu];
 
   outb(fdc->baseport+FDOUT, FDO_FRST | FDO_FDMAEN | mbits);
+#endif
   (void)ftintr_wait(ftu, FTCMD_RESET, hz/10);
 
   splx(s);
@@ -1721,8 +1701,10 @@ tape_start(ftu_t ftu, int motor)
   tape_recal(ftu, 1);
 
   /* set transfer speed */
+#ifndef PC98
   outb(fdc->baseport+FDCTL, FDC_500KBPS);
   DELAY(10);
+#endif
 
   DPRT(("tape_start end\n"));
 }
@@ -1734,7 +1716,7 @@ tape_start(ftu_t ftu, int motor)
 static void
 tape_end(ftu_t ftu)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdc_p	fdc = ft->fdc;
   int s;
 
@@ -1744,18 +1726,28 @@ tape_end(ftu_t ftu)
   s = splbio();
 
   /* reset, dma disable */
+#ifdef PC98
+  outb(fdc->baseport+FDOUT, FDO_RST | FDO_FRY | FDO_AIE | FDO_MTON);
+#else
   outb(fdc->baseport+FDOUT, 0x00);
+#endif
   (void)ftintr_wait(ftu, FTCMD_RESET, hz/10);
 
   /* raise reset, enable DMA */
+#ifdef PC98
+  outb(fdc->baseport+FDOUT, FDO_DMAE | FDO_MTON);
+#else
   outb(fdc->baseport+FDOUT, FDO_FRST | FDO_FDMAEN);
+#endif
   (void)ftintr_wait(ftu, FTCMD_RESET, hz/10);
 
   splx(s);
 
   /* set transfer speed */
+#ifndef PC98
   outb(fdc->baseport+FDCTL, FDC_500KBPS);
   DELAY(10);
+#endif
   fdc->flags &= ~FDC_TAPE_BUSY;
 
   DPRT(("tape_end end\n"));
@@ -1768,7 +1760,7 @@ tape_end(ftu_t ftu)
 static void
 tape_inactive(ftu_t ftu)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   int s = splbio();
 
   if (ft->segh != NULL) {
@@ -1801,7 +1793,7 @@ ftgetgeom(ftu_t ftu)
   int r, i, tries;
   int cfg, qic80, ext;
   int sts, fmt, len;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   r = tape_status(ftu);
 
@@ -1895,7 +1887,7 @@ static int
 set_fdcmode(dev_t dev, int newmode)
 {
   ftu_t ftu = FDUNIT(minor(dev));
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdc_p	fdc = ft->fdc;
   static int havebufs = 0;
   int i;
@@ -2031,7 +2023,7 @@ static int
 qic_status(ftu_t ftu, int cmd, int nbits)
 {
   int st3, r, i;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   fdcu_t fdcu = ft->fdc->fdcu;		/* fdc active unit */
 
   if (tape_cmd(ftu, cmd)) {
@@ -2091,8 +2083,10 @@ ftopen(dev_t dev, int arg2) {
   /* check bounds */
   if (ftu >= NFT)
 	return(ENXIO);
-  fdc = ft_data[ftu].fdc;
-  if ((fdc == NULL) || (ft_data[ftu].type == NO_TYPE))
+  if (!ft_data[ftu])
+	return(ENXIO);
+  fdc = ft_data[ftu]->fdc;
+  if ((fdc == NULL) || (ft_data[ftu]->type == NO_TYPE))
 	  return(ENXIO);
   /* check for controller already busy with tape */
   if (fdc->flags & FDC_TAPE_BUSY)
@@ -2102,7 +2096,6 @@ ftopen(dev_t dev, int arg2) {
 	return(ENODEV);
   fdc->fdu = ftu;
   fdc->flags |= FDC_TAPE_BUSY;
-  kdc_ft[ftu].kdc_state = DC_BUSY;
   return(set_fdcmode(dev, FDC_TAPE_MODE)); /* try to switch to tape */
 }
 
@@ -2114,7 +2107,7 @@ int
 ftclose(dev_t dev, int flags)
 {
   ftu_t ftu = FDUNIT(minor(dev));
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
 
   /* Wait for any remaining I/O activity to complete. */
@@ -2124,24 +2117,9 @@ ftclose(dev_t dev, int flags)
   tape_cmd(ftu, QC_PRIMARY);
   tape_state(ftu, 0, QS_READY, 60);
   ftreq_rewind(ftu);
-  kdc_ft[ftu].kdc_state = DC_IDLE;
   return(set_fdcmode(dev, FDC_DISK_MODE));	/* Otherwise, close tape */
 }
 
-
-/*
- *  Perform strategy on a given buffer (not!).  Changed so that the
- *  driver will at least return 'Operation not supported'.
- */
-void
-ftstrategy(struct buf *bp)
-{
-  bp->b_error = ENODEV;
-  bp->b_flags |= B_ERROR;
-  biodone(bp);
-}
-
-
 /*
  *  Read or write a segment.
  */
@@ -2153,7 +2131,7 @@ ftreq_rw(ftu_t ftu, int cmd, QIC_Segment *sr, struct proc *p)
   int s;
   long blk, bad, seg;
   unsigned char *cp, *cp2;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   if (!ft->active && ft->segh == NULL) {
 	r = tape_status(ftu);
@@ -2305,7 +2283,7 @@ rddone:
 static int
 ftreq_rewind(ftu_t ftu)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   tape_cmd(ftu, QC_STOP);
@@ -2328,7 +2306,7 @@ static int
 ftreq_trkpos(ftu_t ftu, int req)
 {
   int curtrk, r, cmd;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   tape_cmd(ftu, QC_STOP);
@@ -2360,7 +2338,7 @@ static int
 ftreq_trkset(ftu_t ftu, int *trk)
 {
   int r;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   tape_cmd(ftu, QC_STOP);
@@ -2386,7 +2364,7 @@ ftreq_trkset(ftu_t ftu, int *trk)
 static int
 ftreq_lfwd(ftu_t ftu)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   tape_cmd(ftu, QC_STOP);
@@ -2403,7 +2381,7 @@ ftreq_lfwd(ftu_t ftu)
 static int
 ftreq_stop(ftu_t ftu)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   tape_cmd(ftu, QC_STOP);
@@ -2420,7 +2398,7 @@ static int
 ftreq_setmode(ftu_t ftu, int cmd)
 {
   int r;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   tape_inactive(ftu);
   r = tape_status(ftu);
@@ -2451,7 +2429,7 @@ ftreq_setmode(ftu_t ftu, int cmd)
 static int
 ftreq_status(ftu_t ftu, int cmd, int *sts, struct proc *p)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   if (ft->active)
 	*sts = ft->laststs & ~QS_READY;
@@ -2468,7 +2446,7 @@ static int
 ftreq_config(ftu_t ftu, int cmd, int *cfg, struct proc *p)
 {
   int r, tries;
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
 
   if (ft->active)
 	r = ft->lastcfg;
@@ -2531,7 +2509,7 @@ ftreq_hwinfo(ftu_t ftu, QIC_HWInfo *hwp)
 static int
 ftreq_hdr(ftu_t ftu, int cmd, QIC_Segment *sp)
 {
-  ft_p	ft = &ft_data[ftu];
+  ft_p	ft = ft_data[ftu];
   QIC_Header *h = (QIC_Header *)ft->hdr->buff;
 
   if (sp == NULL || sp->sg_data == NULL) return(EINVAL);
@@ -2605,21 +2583,4 @@ badreq:
   return(ENXIO);
 }
 
-/*
- *  Not implemented
- */
-int
-ftdump(dev_t dev)
-{
-  return(EINVAL);
-}
-
-/*
- *  Not implemented
- */
-int
-ftsize(dev_t dev)
-{
-  return(EINVAL);
-}
 #endif
