@@ -6,7 +6,7 @@
  *   of this software, nor does the author assume any responsibility
  *   for damages incurred with its use.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/if_edreg.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_edreg.h 50950 1999-09-05 08:39:22Z peter $
  */
 /*
  * National Semiconductor DS8390 NIC register definitions
@@ -211,7 +211,7 @@
 
 /*
  * OVW: OverWrite. Indicates a receive ring-buffer overrun. Incoming network
- *	would exceed (has exceeded?) the boundry pointer, resulting in data
+ *	would exceed (has exceeded?) the boundary pointer, resulting in data
  *	that was previously received and not yet read from the buffer to be
  *	overwritten.
  */
@@ -266,7 +266,7 @@
 
 /*
  * OVWE: OverWrite error interrupt Enable. If set, an interrupt is generated whenever
- *	the receive ring-buffer is overrun. i.e. when the boundry pointer is exceeded.
+ *	the receive ring-buffer is overrun. i.e. when the boundary pointer is exceeded.
  */
 #define ED_IMR_OVWE	0x10
 
@@ -318,7 +318,7 @@
 /*
  * AR: Auto-initialize Remote. When 0, data must be removed from ring-buffer
  *	under program control. When 1, remote DMA is automatically initiated
- *	and the boundry pointer is automatically updated
+ *	and the boundary pointer is automatically updated
  */
 #define ED_DCR_AR	0x10
 
@@ -500,7 +500,7 @@
 
 /*
  * FAE: Frame Alignment Error. Indicates that the incoming packet did not end on
- *	a byte boundry and the CRC did not match at the last byte boundry.
+ *	a byte boundary and the CRC did not match at the last byte boundary.
  */
 #define ED_RSR_FAE	0x04
 
@@ -513,7 +513,7 @@
 /*
  * MPA: Missed Packet. Indicates that the received packet couldn't be stored in
  *	the ring-buffer because of insufficient buffer space (exceeding the
- *	boundry pointer), or because the transfer to the ring-buffer was inhibited
+ *	boundary pointer), or because the transfer to the ring-buffer was inhibited
  *	by RCR_MON - monitor mode.
  */
 #define ED_RSR_MPA	0x10
@@ -526,7 +526,7 @@
 #define ED_RSR_PHY	0x20
 
 /*
- * DIS: Receiver Disabled. Set to indicate that the receiver has enetered monitor
+ * DIS: Receiver Disabled. Set to indicate that the receiver has entered monitor
  *	mode. Cleared when the receiver exits monitor mode.
  */
 #define ED_RSR_DIS	0x40
@@ -539,7 +539,7 @@
 #define ED_RSR_DFR	0x80
 
 /*
- * receive ring discriptor
+ * receive ring descriptor
  *
  * The National Semiconductor DS8390 Network interface controller uses
  * the following receive ring headers.  The way this works is that the
@@ -568,12 +568,14 @@ struct ed_ring	{
 #define ED_VENDOR_WD_SMC	0x00		/* Western Digital/SMC */
 #define ED_VENDOR_3COM		0x01		/* 3Com */
 #define ED_VENDOR_NOVELL	0x02		/* Novell */
+#define ED_VENDOR_PCCARD	0x03		/* PCMCIA/PCCARD */
+#define ED_VENDOR_HP		0x04		/* Hewlett Packard */
 
 /*
  * Compile-time config flags
  */
 /*
- * this sets the default for enabling/disablng the tranceiver
+ * this sets the default for enabling/disabling the transceiver
  */
 #define ED_FLAGS_DISABLE_TRANCEIVER	0x0001
 
@@ -784,9 +786,22 @@ struct ed_ring	{
 #define ED_WD_ROM_CHECKSUM_TOTAL	0xFF
 #endif
 
+#ifdef PC98
+/*
+ * SMC EtherEZ98(SMC8498BTA)
+ */
+#define ED_WD_NIC_OFFSET	0x100		/* I/O base offset to NIC */
+#define ED_WD_ASIC_OFFSET	0		/* I/O base offset to ASIC */
+/*
+ * XXX - The I/O address range is fragmented in the EtherEZ98;
+ *	this is the number of regs at iobase.
+ */
+#define ED_WD_IO_PORTS		16		/* # of i/o addresses used */
+#else
 #define ED_WD_NIC_OFFSET	0x10		/* I/O base offset to NIC */
 #define ED_WD_ASIC_OFFSET	0		/* I/O base offset to ASIC */
 #define ED_WD_IO_PORTS		32		/* # of i/o addresses used */
+#endif /* PC98 */
 
 #define ED_WD_PAGE_OFFSET	0	/* page offset for NIC access to mem */
 
@@ -981,3 +996,105 @@ struct ed_ring	{
  * Reset register; reading from this register causes a board reset
  */
 #define ED_NOVELL_RESET		0x0f
+
+/*
+ *		Definitions for PCCARD
+ */
+#define ED_PC_PAGE_OFFSET	0x40	/* page offset for NIC access to mem */
+#define ED_PC_IO_PORTS		32
+#define ED_PC_RESET		0x1f
+#define ED_PC_MISC		0x18
+
+/* 
+ * if_ze.h  constants
+ */
+
+#define ZE_PAGE_OFFSET		0x40	/* mem buffer starts at 0x4000 */
+
+#define ZE_DATA_IO	0x10
+#define ZE_MISC		0x18
+#define ZE_RESET	0x1F
+
+/*
+ * Definitions for HP PC LAN Adapter Plus; based on the CRYNWR packet
+ * driver for the card.
+ */
+
+#define	ED_HPP_ASIC_OFFSET	0x00	/* Offset to ASIC registers */
+#define	ED_HPP_NIC_OFFSET	0x10	/* Offset to 8390 registers */
+
+#define	ED_HPP_ID		0x00	/* ID register, always 0x4850 */
+#define	ED_HPP_PAGING		0x02	/* Page select register */
+#define	ED_HPP_OPTION		0x04	/* Bitmask of supported options */
+#define	ED_HPP_PAGE_0		0x08	/* Page 0 */
+#define	ED_HPP_PAGE_2		0x0A	/* Page 2 */
+#define ED_HPP_PAGE_4		0x0C	/* Page 4 */
+#define	ED_HPP_PAGE_6		0x0E	/* Page 6 */
+
+/* PERF PAGE */
+#define	ED_HPP_OUT_ADDR		ED_HPP_PAGE_0	/* I/O output location */
+#define	ED_HPP_IN_ADDR		ED_HPP_PAGE_2	/* I/O input location */
+#define	ED_HPP_DATAPORT		ED_HPP_PAGE_4	/* I/O data transfer */
+/* MAC PAGE */
+#define ED_HPP_MAC_ADDR		0x08	/* Offset of MAC address in MAC page */
+
+#define	ED_HPP_IO_PORTS		32	/* Number of IO ports */
+
+#define	ED_HPP_TX_PAGE_OFFSET	0x00	/* first page of TX buffer */
+#define ED_HPP_RX_PAGE_START	0x06	/* start at page 6 */
+#define	ED_HPP_RX_PAGE_STOP	0x80	/* end at page 128 */
+
+/*
+ * Register pages supported.
+ */
+
+#define	ED_HPP_PAGE_PERF	0	/* Normal operation */
+#define	ED_HPP_PAGE_MAC		1	/* The ethernet address and checksum */
+#define	ED_HPP_PAGE_HW		2	/* Hardware parameters in EEPROM */
+#define	ED_HPP_PAGE_LAN		4	/* Transciever selection etc */
+#define	ED_HPP_PAGE_ID		6	/* ID */
+
+/*
+ * Options supported.
+ */
+
+#define	ED_HPP_OPTION_NIC_RESET		0x0001	/* active low */
+#define	ED_HPP_OPTION_CHIP_RESET	0x0002	/* active low */
+#define	ED_HPP_OPTION_ENABLE_IRQ	0x0004
+#define	ED_HPP_OPTION_FAKE_INTR		0x0008
+#define	ED_HPP_OPTION_BOOT_ROM_ENB	0x0010
+#define	ED_HPP_OPTION_IO_ENB		0x0020
+#define	ED_HPP_OPTION_MEM_ENABLE	0x0040
+#define	ED_HPP_OPTION_ZERO_WAIT		0x0080
+#define	ED_HPP_OPTION_MEM_DISABLE	0x1000
+
+/*
+ * Page ID configuration.
+ */
+
+#define	ED_HPP_ID_REVISION_MASK		0x0300	/* revision id */
+#define ED_HPP_ID_SOFT_MODEL_MASK	0xFC00	/* soft model number */
+#define ED_HPP_ID_16_BIT_ACCESS		0x0010	/* if set use 16 bit accesses */
+#define	ED_HPP_ID_TWISTED_PAIR		0x0040	
+
+/*
+ * Hardware configuration.
+ */
+
+#define	ED_HPP_HW_MEM_MAP	0x09	/* low mem map location in HW page */
+#define ED_HPP_HW_ID		0x0C	/* revision number, capabilities */
+#define ED_HPP_HW_IRQ		0x0D	/* IRQ channel register in HW page */
+#define	ED_HPP_HW_WRAP		0x0E	/* mem wrap page for rcv */
+
+/*
+ * Lan configuration
+ */
+
+#define ED_HPP_LAN_AUI		0x01	/* Use AUI */
+#define ED_HPP_LAN_TL		0x40	/* Don't use AUI */
+
+/*
+ * Card types.
+ */
+
+#define ED_TYPE_HP_PCLANPLUS	0x00
