@@ -26,8 +26,6 @@
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/sys/i386/isa/if_egreg.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -63,7 +61,7 @@
 
 /*
  * Host Status Register bits
- * EG_STAT_HRDY - Data Register ready
+ * EG_STAT_HRDY - Data Register ready 
  * EG_STAT_HCRE - Host Command Register empty
  * EG_STAT_ACRF - Adapter Command register full
  * EG_STAT_DIR  - Direction flag, 0 = host -> adapter, 1 = adapter -> host
@@ -82,11 +80,11 @@
 #define EG_STAT_ASF2 0x02
 #define EG_STAT_ASF1 0x01
 
-#define EG_PCB_ACCEPT	1
-#define EG_PCB_REJECT	2
-#define EG_PCB_DONE	3
-#define EG_PCB_STAT(x)	((x)&3)
-#define EG_PCB_MASK(x)	((x)&~3)
+#define	EG_PCB_NULL	0x00
+#define EG_PCB_ACCEPT	0x01 
+#define EG_PCB_REJECT	0x02
+#define EG_PCB_DONE	0x03
+#define EG_PCB_STAT	0x03
 
 #define EG_CMD_CONFIG82586	0x02
 #define EG_CMD_GETEADDR		0x03
