@@ -6,7 +6,7 @@
  *
  * Questions, comments, bug reports and fixes to kimmel@cs.umass.edu.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/if_el.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_el.c 50950 1999-09-05 08:39:22Z peter $
  */
 /* Except of course for the portions of code lifted from other FreeBSD
  * drivers (mainly elread, elget and el_ioctl)
@@ -20,17 +20,16 @@
  *	- Does not currently support multicasts
  */
 #include "el.h"
-#if NEL > 0
 #include "bpfilter.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/errno.h>
 #include <sys/ioctl.h>
 #include <sys/mbuf.h>
 #include <sys/socket.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
 
 #include <net/if.h>
 #include <net/if_dl.h>
@@ -44,6 +43,11 @@
 #include <netinet/if_ether.h>
 #endif
 
+#ifdef IPX
+#include <netipx/ipx.h>
+#include <netipx/ipx_if.h>
+#endif
+
 #ifdef NS
 #include <netns/ns.h>
 #include <netns/ns_if.h>
@@ -56,14 +60,9 @@
 
 #include <machine/clock.h>
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
-#include <i386/isa/icu.h>
 #include <i386/isa/if_elreg.h>
 
-#define ETHER_MIN_LEN	64
-#define ETHER_MAX_LEN	1518
-
 /* For debugging convenience */
 #ifdef EL_DEBUG
 #define dprintf(x) printf x
@@ -72,22 +71,20 @@
 #endif
 
 /* el_softc: per line info and status */
-struct el_softc {
+static struct el_softc {
 	struct arpcom arpcom;	/* Ethernet common */
 	u_short el_base;	/* Base I/O addr */
-	caddr_t bpf;		/* BPF magic cookie */
 	char el_pktbuf[EL_BUFSIZ]; 	/* Frame buffer */
 } el_softc[NEL];
 
 /* Prototypes */
-int el_attach(struct isa_device *);
-void el_init(int);
-void elintr(int);
-int el_ioctl(struct ifnet *,int,caddr_t);
-int el_probe(struct isa_device *);
-void el_start(struct ifnet *);
-void el_reset(int);
-void el_watchdog(int);
+static int el_attach(struct isa_device *);
+static void el_init(int);
+static int el_ioctl(struct ifnet *,int,caddr_t);
+static int el_probe(struct isa_device *);
+static void el_start(struct ifnet *);
+static void el_reset(int);
+static void el_watchdog(struct ifnet *);
 
 static void el_stop(int);
 static int el_xmit(struct el_softc *,int);
@@ -100,29 +97,9 @@ struct isa_driver eldriver = {
 	el_probe, el_attach, "el"
 };
 
-static struct kern_devconf kdc_el[NEL] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"el", 0, { MDDT_ISA, 0, "net" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"Ethernet adapter: 3Com 3C501",
-	DC_CLS_NETIF		/* class */
-} };
-
-static inline void
-el_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_el[id->id_unit] = kdc_el[0];
-	kdc_el[id->id_unit].kdc_unit = id->id_unit;
-	kdc_el[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_el[id->id_unit]);
-}
-
 /* Probe routine.  See if the card is there and at the right place. */
-int el_probe(struct isa_device *idev)
+static int
+el_probe(struct isa_device *idev)
 {
 	struct el_softc *sc;
 	u_short base; /* Just for convenience */
@@ -134,10 +111,6 @@ int el_probe(struct isa_device *idev)
 	sc->el_base = idev->id_iobase;
 	base = sc->el_base;
 
-#ifndef DEV_LKM
-	el_registerdev(idev);
-#endif
-
 	/* First check the base */
 	if((base < 0x280) || (base > 0x3f0)) {
 		printf("el%d: ioaddr must be between 0x280 and 0x3f0\n",
@@ -161,7 +134,7 @@ int el_probe(struct isa_device *idev)
 		outb(base+EL_GPBL,i);
 		station_addr[i] = inb(base+EL_EAW);
 	}
-	dprintf(("Address is %s\n",ether_sprintf(station_addr)));
+	dprintf(("Address is %6D\n",station_addr, ":"));
 
 	/* If the vendor code is ok, return a 1.  We'll assume that
 	 * whoever configured this system is right about the IRQ.
@@ -182,14 +155,14 @@ int el_probe(struct isa_device *idev)
  * this is called, we know that the card exists at the given I/O address.
  * We still assume that the IRQ given is correct.
  */
-int el_attach(struct isa_device *idev)
+static int
+el_attach(struct isa_device *idev)
 {
 	struct el_softc *sc;
 	struct ifnet *ifp;
 	struct ifaddr *ifa;
 	struct sockaddr_dl *sdl;
 	u_short base;
-	int t;
 
 	dprintf(("Attaching el%d...\n",idev->id_unit));
 
@@ -203,21 +176,20 @@ int el_attach(struct isa_device *idev)
 	el_hardreset(idev->id_unit);
 
 	/* Initialize ifnet structure */
+	ifp->if_softc = sc;
 	ifp->if_unit = idev->id_unit;
 	ifp->if_name = "el";
 	ifp->if_mtu = ETHERMTU;
-	ifp->if_init = el_init;
 	ifp->if_output = ether_output;
 	ifp->if_start = el_start;
 	ifp->if_ioctl = el_ioctl;
-	ifp->if_reset = el_reset;
 	ifp->if_watchdog = el_watchdog;
-	ifp->if_flags = (IFF_BROADCAST | IFF_SIMPLEX | IFF_NOTRAILERS);
+	ifp->if_flags = (IFF_BROADCAST | IFF_SIMPLEX);
 
 	/* Now we can attach the interface */
 	dprintf(("Attaching interface...\n"));
 	if_attach(ifp);
-	kdc_el[idev->id_unit].kdc_state = DC_BUSY;
+	ether_ifattach(ifp);
 
 	/* Put the station address in the ifa address list's AF_LINK
 	 * entry, if any.
@@ -235,13 +207,13 @@ int el_attach(struct isa_device *idev)
 	}
 
 	/* Print out some information for the user */
-	printf("el%d: 3c501 address %s\n",idev->id_unit,
-	  ether_sprintf(sc->arpcom.ac_enaddr));
+	printf("el%d: 3c501 address %6D\n",idev->id_unit,
+	  sc->arpcom.ac_enaddr, ":");
 
 	/* Finally, attach to bpf filter if it is present. */
 #if NBPFILTER > 0
 	dprintf(("Attaching to BPF...\n"));
-	bpfattach(&sc->bpf,ifp,DLT_EN10MB,sizeof(struct ether_header));
+	bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));
 #endif
 
 	dprintf(("el_attach() finished.\n"));
@@ -249,7 +221,8 @@ int el_attach(struct isa_device *idev)
 }
 
 /* This routine resets the interface. */
-void el_reset(int unit)
+static void 
+el_reset(int unit)
 {
 	int s;
 
@@ -291,7 +264,8 @@ static inline void el_hardreset(int unit)
 }
 
 /* Initialize interface.  */
-void el_init(int unit)
+static void 
+el_init(int unit)
 {
 	struct el_softc *sc;
 	struct ifnet *ifp;
@@ -343,7 +317,8 @@ void el_init(int unit)
  * them, giving the receiver a chance between datagrams.  Call only
  * from splimp or interrupt level!
  */
-void el_start(struct ifnet *ifp)
+static void
+el_start(struct ifnet *ifp)
 {
 	struct el_softc *sc;
 	u_short base;
@@ -351,7 +326,7 @@ void el_start(struct ifnet *ifp)
 	int s, i, len, retries, done;
 
 	/* Get things pointing in the right directions */
-	sc = &el_softc[ifp->if_unit];
+	sc = ifp->if_softc;
 	base = sc->el_base;
 
 	dprintf(("el_start()...\n"));
@@ -394,8 +369,8 @@ void el_start(struct ifnet *ifp)
 
 		/* Give the packet to the bpf, if any */
 #if NBPFILTER > 0
-		if(sc->bpf)
-			bpf_tap(sc->bpf,sc->el_pktbuf,len);
+		if(sc->arpcom.ac_if.if_bpf)
+			bpf_tap(&sc->arpcom.ac_if, sc->el_pktbuf, len);
 #endif
 
 		/* Transfer datagram to board */
@@ -547,8 +522,8 @@ void elintr(int unit)
 		insb(base+EL_BUF,sc->el_pktbuf,len);
 		outb(base+EL_RBC,0);
 		outb(base+EL_AC,EL_AC_RX);
-		dprintf(("%s-->",ether_sprintf(sc->el_pktbuf+6)));
-		dprintf(("%s\n",ether_sprintf(sc->el_pktbuf)));
+		dprintf(("%6D-->",sc->el_pktbuf+6,":"));
+		dprintf(("%6D\n",sc->el_pktbuf,":"));
 
 		/* Pass data up to upper levels */
 		len -= sizeof(struct ether_header);
@@ -582,8 +557,9 @@ static inline void elread(struct el_softc *sc,caddr_t buf,int len)
 	 * Check if there's a bpf filter listening on this interface.
 	 * If so, hand off the raw packet to bpf.
 	 */
-	if(sc->bpf) {
-		bpf_tap(sc->bpf,buf,len+sizeof(struct ether_header));
+	if(sc->arpcom.ac_if.if_bpf) {
+		bpf_tap(&sc->arpcom.ac_if, buf, 
+			len + sizeof(struct ether_header));
 
 		/*
 		 * Note that the interface cannot be in promiscuous mode if
@@ -621,7 +597,7 @@ elget(buf, totlen, off0, ifp)
         int totlen, off0;
         struct ifnet *ifp;
 {
-        struct mbuf *top, **mp, *m, *p;
+        struct mbuf *top, **mp, *m;
         int off = off0, len;
         register caddr_t cp = buf;
         char *epkt;
@@ -686,14 +662,14 @@ elget(buf, totlen, off0, ifp)
  * Process an ioctl request. This code needs some work - it looks
  *	pretty ugly.
  */
-int
+static int
 el_ioctl(ifp, command, data)
 	register struct ifnet *ifp;
 	int command;
 	caddr_t data;
 {
 	register struct ifaddr *ifa = (struct ifaddr *)data;
-	struct el_softc *sc = &el_softc[ifp->if_unit];
+	struct el_softc *sc = ifp->if_softc;
 	struct ifreq *ifr = (struct ifreq *)data;
 	int s, error = 0;
 
@@ -711,6 +687,32 @@ el_ioctl(ifp, command, data)
 			arp_ifinit((struct arpcom *)ifp, ifa);
 			break;
 #endif
+#ifdef IPX
+		/*
+		 * XXX - This code is probably wrong
+		 */
+		case AF_IPX:
+		    {
+			register struct ipx_addr *ina = &(IA_SIPX(ifa)->sipx_addr);
+
+			if (ipx_nullhost(*ina))
+				ina->x_host =
+					*(union ipx_host *)(sc->arpcom.ac_enaddr);
+			else {
+				/* 
+				 * 
+				 */
+				bcopy((caddr_t)ina->x_host.c_host,
+				      (caddr_t)sc->arpcom.ac_enaddr,
+				      sizeof(sc->arpcom.ac_enaddr));
+			}
+			/*
+			 * Set new address
+			 */
+			el_init(ifp->if_unit);
+			break;
+		    }
+#endif
 #ifdef NS
 		/*
 		 * XXX - This code is probably wrong
@@ -790,14 +792,10 @@ el_ioctl(ifp, command, data)
 }
 
 /* Device timeout routine */
-void el_watchdog(int unit)
+static void
+el_watchdog(struct ifnet *ifp)
 {
-	struct el_softc *sc;
-
-	sc = &el_softc[unit];
-
-	log(LOG_ERR,"el%d: device timeout\n",unit);
-	sc->arpcom.ac_if.if_oerrors++;
-	el_reset(unit);
+	log(LOG_ERR,"el%d: device timeout\n", ifp->if_unit);
+	ifp->if_oerrors++;
+	el_reset(ifp->if_unit);
 }
-#endif
