@@ -38,7 +38,7 @@
  */
 
 /*
- * $FreeBSD: stable/2.1/sys/i386/isa/if_ep.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_ep.c 50950 1999-09-05 08:39:22Z peter $
  *
  *  Promiscuous mode added and interrupt logic slightly changed
  *  to reduce the number of adapter failures. Transceiver select
@@ -50,6 +50,12 @@
  *          babkin@hq.icb.chel.su
  */
 
+/*
+ * Pccard support for 3C589 by:
+ *		HAMADA Naoki
+ *		nao@tom-yam.or.jp
+ */
+
 #include "ep.h"
 #if NEP > 0
 
@@ -60,7 +66,6 @@
 #include <sys/systm.h>
 #include <sys/kernel.h>
 #include <sys/conf.h>
-#include <sys/devconf.h>
 #endif
 #include <sys/mbuf.h>
 #include <sys/socket.h>
@@ -98,13 +103,15 @@
 #include <net/bpfdesc.h>
 #endif
 
+#ifdef BRIDGE
+#include <net/bridge.h>
+#endif
+
 #if defined(__FreeBSD__)
 #include <machine/clock.h>
 #endif
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
-#include <i386/isa/icu.h>
 #include <i386/isa/if_epreg.h>
 #include <i386/isa/elink.h>
 
@@ -118,20 +125,18 @@ static	int eeprom_rdy __P((struct ep_softc *sc));
 static	int ep_isa_probe __P((struct isa_device *));
 static struct ep_board * ep_look_for_board_at __P((struct isa_device *is));
 static	int ep_isa_attach __P((struct isa_device *));
-static	void ep_isa_registerdev __P((struct ep_softc *sc,
-				     struct isa_device *id));
 static	int epioctl __P((struct ifnet * ifp, int, caddr_t));
-static	void epmbuffill __P((caddr_t, int));
-static	void epmbufempty __P((struct ep_softc *));
 
 static	void epinit __P((struct ep_softc *));
 static	void epread __P((struct ep_softc *));
 void	epreset __P((int));
 static	void epstart __P((struct ifnet *));
 static	void epstop __P((struct ep_softc *));
-static	void epwatchdog __P((int unit));
+static	void epwatchdog __P((struct ifnet *));
 
+#if 0
 static	int send_ID_sequence __P((int));
+#endif
 static	int get_eeprom_data __P((int, int));
 
 static	struct ep_softc* ep_softc[NEP];
@@ -149,42 +154,150 @@ struct isa_driver epdriver = {
     0
 };
 
-static struct kern_devconf kdc_isa_ep = {
-    0, 0, 0,			/* filled in by dev_attach */
-    "ep", 0, { MDDT_ISA, 0, "net" },
-    isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-    &kdc_isa0,		/* parent */
-    0,			/* parentdata */
-    DC_UNCONFIGURED,		/* state */
-    "3Com 3C509 Ethernet adapter",
-    DC_CLS_NETIF		/* class */
+#include "card.h"
+
+#if NCARD > 0
+#include "apm.h"
+#include <sys/select.h>
+#include <pccard/cardinfo.h>
+#include <pccard/driver.h>
+#include <pccard/slot.h>
+
+/*
+ * PC-Card (PCMCIA) specific code.
+ */
+static int ep_pccard_init __P((struct pccard_devinfo *));
+static int ep_pccard_attach  __P((struct pccard_devinfo *));
+static void ep_unload __P((struct pccard_devinfo *));
+static int card_intr __P((struct pccard_devinfo *));
+
+static struct pccard_device ep_info = {
+    "ep",
+    ep_pccard_init,
+    ep_unload,
+    card_intr,
+    0,                      /* Attributes - presently unused */
+    &net_imask
 };
 
-static void
-ep_isa_registerdev(sc, id)
-    struct ep_softc *sc;
-    struct isa_device *id;
+DATA_SET(pccarddrv_set, ep_info);
+
+/*
+ * Initialize the device - called from Slot manager.
+ */
+static int
+ep_pccard_init(devi)
+    struct pccard_devinfo *devi;
 {
-    sc->kdc = (struct kern_devconf *)malloc(sizeof(struct kern_devconf),
-					    M_DEVBUF, M_NOWAIT);
-    if (!sc->kdc) {
-	printf("WARNING: ep_isa_registerdev unable to malloc! "
-	       "Device kdc will not be registerd\n");
+    struct isa_device *is = &devi->isahd;
+    struct ep_softc *sc = ep_softc[is->id_unit];
+    struct ep_board *epb;
+    int i;
+
+    epb = &ep_board[is->id_unit];
+
+    if (sc == 0) {
+	if ((sc = ep_alloc(is->id_unit, epb)) == 0) {
+	    return (ENXIO);
+	}
+	ep_unit++;
+    }
+
+    /* get_e() requires these. */
+    sc->ep_io_addr = is->id_iobase;
+    sc->unit = is->id_unit;
+
+    epb->epb_addr = is->id_iobase;
+    epb->epb_used = 1;
+    epb->prod_id = get_e(sc, EEPROM_PROD_ID);
+
+    /* 3C589's product id? */
+    if (epb->prod_id != 0x9058) {
+	printf("ep%d: failed to come ready.\n", is->id_unit);
+	return (ENXIO);
+    }
+
+    epb->res_cfg = get_e(sc, EEPROM_RESOURCE_CFG);
+    for (i = 0; i < 3; i++)
+	sc->epb->eth_addr[i] = get_e(sc, EEPROM_NODE_ADDR_0 + i);
+
+    if (ep_pccard_attach(devi) == 0)
+	return (ENXIO);
+
+    sc->arpcom.ac_if.if_snd.ifq_maxlen = ifqmaxlen;
+    return (0);
+}
+
+static int
+ep_pccard_attach(devi)
+    struct pccard_devinfo *devi;
+{
+    struct isa_device *is = &devi->isahd;
+    struct ep_softc *sc = ep_softc[is->id_unit];
+    u_short config;
+
+    sc->ep_connectors = 0;
+    config = inw(IS_BASE + EP_W0_CONFIG_CTRL);
+    if (config & IS_BNC) {
+	sc->ep_connectors |= BNC;
+    }
+    if (config & IS_UTP) {
+	sc->ep_connectors |= UTP;
+    }
+    if (!(sc->ep_connectors & 7))
+	printf("no connectors!");
+    sc->ep_connector = inw(BASE + EP_W0_ADDRESS_CFG) >> ACF_CONNECTOR_BITS;
+
+    /* ROM size = 0, ROM base = 0 */
+    /* For now, ignore AUTO SELECT feature of 3C589B and later. */
+    outw(BASE + EP_W0_ADDRESS_CFG, get_e(sc, EEPROM_ADDR_CFG) & 0xc000);
+
+    /* Fake IRQ must be 3 */
+    outw(BASE + EP_W0_RESOURCE_CFG, (sc->epb->res_cfg & 0x0fff) | 0x3000);
+
+    outw(BASE + EP_W0_PRODUCT_ID, sc->epb->prod_id);
+
+    ep_attach(sc);
+
+    return 1;
+}
+
+static void
+ep_unload(devi)
+    struct pccard_devinfo *devi;
+{
+    struct ep_softc *sc = ep_softc[devi->isahd.id_unit];
+
+    if (sc->gone) {
+        printf("ep%d: already unloaded\n", devi->isahd.id_unit);
 	return;
     }
-    bcopy(&kdc_isa_ep, sc->kdc, sizeof(kdc_isa_ep));
-    sc->kdc->kdc_unit = sc->unit;
-    sc->kdc->kdc_parentdata = id;
-    dev_attach(sc->kdc);
+    sc->arpcom.ac_if.if_flags &= ~IFF_RUNNING;
+    sc->gone = 1;
+    printf("ep%d: unload\n", devi->isahd.id_unit);
 }
 
+/*
+ * card_intr - Shared interrupt called from
+ * front end of PC-Card handler.
+ */
+static int
+card_intr(devi)
+    struct pccard_devinfo *devi;
+{
+    epintr(devi->isahd.id_unit);
+    return(1);
+}
+#endif /* NCARD > 0 */
+
 static int
 eeprom_rdy(sc)
     struct ep_softc *sc;
 {
     int i;
 
-    for (i = 0; is_eeprom_busy(BASE) && i < MAX_EEPROMBUSY; i++);
+    for (i = 0; is_eeprom_busy(BASE) && i < MAX_EEPROMBUSY; i++)
+	continue;
     if (i >= MAX_EEPROMBUSY) {
 	printf("ep%d: eeprom failed to come ready.\n", sc->unit);
 	return (0);
@@ -196,7 +309,7 @@ static struct ep_board *
 ep_look_for_board_at(is)
     struct isa_device *is;
 {
-    int data, i, j, io_base, id_port = ELINK_ID_PORT;
+    int data, i, j, id_port = ELINK_ID_PORT;
     int count = 0;
 
     if (ep_current_tag == (EP_LAST_TAG + 1)) {
@@ -238,12 +351,17 @@ ep_look_for_board_at(is)
 		get_eeprom_data(id_port, EEPROM_PROD_ID);
 
 	    ep_board[ep_boards].epb_used = 0;
+#ifdef PC98
+	    ep_board[ep_boards].epb_addr =
+			(get_eeprom_data(id_port, EEPROM_ADDR_CFG) & 0x1f) * 0x100 + 0x40d0;
+#else
 	    ep_board[ep_boards].epb_addr =
 			(get_eeprom_data(id_port, EEPROM_ADDR_CFG) & 0x1f) * 0x10 + 0x200;
 
-	    if(ep_board[ep_boards].epb_addr > 0x3E0)
+	    if (ep_board[ep_boards].epb_addr > 0x3E0)
 		/* Board in EISA configuration mode */
 		continue;
+#endif /* PC98 */
 
 	    outb(id_port, ep_current_tag);	/* tags board */
 	    outb(id_port, ACTIVATE_ADAPTER_TO_CONFIG);
@@ -272,25 +390,30 @@ ep_look_for_board_at(is)
      *
      */
 
-    if(IS_BASE==-1) { /* port? */
-	for (i = 0; ep_board[i].epb_addr && ep_board[i].epb_used; i++);
-	if(ep_board[i].epb_addr==0)
+    if (IS_BASE == -1) { /* port? */
+	for (i = 0; ep_board[i].epb_addr && ep_board[i].epb_used; i++)
+	    ;
+	if (ep_board[i].epb_addr == 0)
 	    return 0;
 
-	IS_BASE=ep_board[i].epb_addr;
-	ep_board[i].epb_used=1;
+	IS_BASE = ep_board[i].epb_addr;
+	ep_board[i].epb_used = 1;
 
 	return &ep_board[i];
     } else {
-	for (i=0; ep_board[i].epb_addr && ep_board[i].epb_addr != IS_BASE; i++);
+	for (i = 0;
+	     ep_board[i].epb_addr && ep_board[i].epb_addr != IS_BASE;
+	     i++)
+	    ;
 
-	if( ep_board[i].epb_used || ep_board[i].epb_addr != IS_BASE) 
+	if (ep_board[i].epb_used || ep_board[i].epb_addr != IS_BASE)
 	    return 0;
 
-	if (inw(IS_BASE + EP_W0_EEPROM_COMMAND) & EEPROM_TST_MODE)
+	if (inw(IS_BASE + EP_W0_EEPROM_COMMAND) & EEPROM_TST_MODE) {
 	    printf("ep%d: 3c5x9 at 0x%x in PnP mode. Disable PnP mode!\n",
 		   is->id_unit, IS_BASE);
-	ep_board[i].epb_used=1;
+	}
+	ep_board[i].epb_used = 1;
 
 	return &ep_board[i];
     }
@@ -335,7 +458,7 @@ ep_alloc(unit, epb)
     }
 
     sc = malloc(sizeof(struct ep_softc), M_DEVBUF, M_NOWAIT);
-    if(!sc) {
+    if (!sc) {
 	printf("ep%d: cannot malloc!\n", unit);
 	return NULL;
     }
@@ -364,29 +487,31 @@ ep_isa_probe(is)
     struct ep_softc *sc;
     struct ep_board *epb;
     u_short k;
-    int i;
 
-    if(( epb=ep_look_for_board_at(is) )==0)
+    if ((epb = ep_look_for_board_at(is)) == 0)
 	return (0);
 
     /*
      * Allocate a storage area for us
      */
     sc = ep_alloc(ep_unit, epb); 
-    if( !sc )
+    if (!sc)
 	return (0);
 
     is->id_unit = ep_unit++;
 
-    ep_isa_registerdev(sc, is);
-
     /*
      * The iobase was found and MFG_ID was 0x6d50. PROD_ID should be
-     * 0x9[0-f]50
+     * 0x9[0-f]50	(IBM-PC)
+     * 0x9[0-f]5[0-f]	(PC-98)
      */
     GO_WINDOW(0);
     k = sc->epb->prod_id;
+#ifdef PC98
+    if ((k & 0xf0f0) != (PROD_ID & 0xf0f0)) {
+#else
     if ((k & 0xf0ff) != (PROD_ID & 0xf0ff)) {
+#endif
 	printf("ep_isa_probe: ignoring model %04x\n", k);
 	ep_free(sc);
 	return (0);
@@ -406,8 +531,8 @@ ep_isa_probe(is)
      *
      */
 
-    if(is->id_irq==0) { /* irq? */
-	is->id_irq= 1 << ( (k==2) ? 9 : k );
+    if (is->id_irq == 0) { /* irq? */
+	is->id_irq = 1 << ((k == 2) ? 9 : k);
     }
 
     sc->stat = 0;	/* 16 bit access */
@@ -444,15 +569,13 @@ ep_isa_attach(is)
      */
 
     irq = ffs(is->id_irq) - 1;
-    if(irq == -1) {
+    if (irq == -1) {
 	printf(" invalid irq... cannot attach\n");
 	return 0;
     }
 
     GO_WINDOW(0);
-    if(irq == 9)
-	irq = 2;
-    outw(BASE + EP_W0_RESOURCE_CFG, SET_IRQ(irq));
+    SET_IRQ(BASE, irq);
 
     ep_attach(sc);
     return 1;
@@ -467,22 +590,26 @@ ep_attach(sc)
     struct sockaddr_dl *sdl;
     u_short *p;
     int i;
+    int attached;
+
+    sc->gone = 0;
+    attached = (ifp->if_softc != 0);
 
     printf("ep%d: ", sc->unit);
     /*
      * Current media type
      */
-    if(sc->ep_connectors & AUI) {
+    if (sc->ep_connectors & AUI) {
 	printf("aui");
-	if(sc->ep_connectors & ~AUI)
+	if (sc->ep_connectors & ~AUI)
 		printf("/");
     }
-    if(sc->ep_connectors & UTP) {
+    if (sc->ep_connectors & UTP) {
 	printf("utp");
-	if(sc->ep_connectors & BNC)
+	if (sc->ep_connectors & BNC)
 		printf("/");
     }
-    if(sc->ep_connectors & BNC) {
+    if (sc->ep_connectors & BNC) {
 	printf("bnc");
     }
 
@@ -497,8 +624,9 @@ ep_attach(sc)
 	p[i] = htons(sc->epb->eth_addr[i]);
 	outw(BASE + EP_W2_ADDR_0 + (i * 2), ntohs(p[i]));
     }
-    printf(" address %s\n", ether_sprintf(sc->arpcom.ac_enaddr));
+    printf(" address %6D\n", sc->arpcom.ac_enaddr, ":");
 
+    ifp->if_softc = sc;
     ifp->if_unit = sc->unit;
     ifp->if_name = "ep";
     ifp->if_mtu = ETHERMTU;
@@ -508,11 +636,12 @@ ep_attach(sc)
     ifp->if_ioctl = epioctl;
     ifp->if_watchdog = epwatchdog;
 
-    if_attach(ifp);
-    ether_ifattach(ifp);
+    if (!attached) {
+	if_attach(ifp);
+	ether_ifattach(ifp);
+    }
 
     /* device attach does transition from UNCONFIGURED to IDLE state */
-    sc->kdc->kdc_state=DC_IDLE;
 
     /*
      * Fill the hardware address into ifa_addr if we find an AF_LINK entry.
@@ -531,23 +660,7 @@ ep_attach(sc)
 	sdl->sdl_slen = 0;
 	bcopy(sc->arpcom.ac_enaddr, LLADDR(sdl), ETHER_ADDR_LEN);
     }
-    /* we give some initial parameters */
-    sc->rx_avg_pkt = 128;
 
-    /*
-     * NOTE: In all this I multiply everything by 64.
-     * W_s = the speed the CPU is able to write to the TX FIFO.
-     * T_s = the speed the board sends the info to the Ether.
-     * W_s/T_s = 16   (represents 16/64) =>    W_s = 25 % of T_s.
-     * This will give us for a packet of 1500 bytes
-     * tx_start_thresh=1125 and for a pkt of 64 bytes tx_start_threshold=48.
-     * We prefer to start thinking the CPU is much slower than the Ethernet
-     * transmission.
-     */
-    sc->tx_rate = TX_INIT_RATE;
-    sc->tx_counter = 0;
-    sc->rx_latency = RX_INIT_LATENCY;
-    sc->rx_early_thresh = RX_INIT_EARLY_THRESH;
 #ifdef EP_LOCAL_STATS
     sc->rx_no_first = sc->rx_no_mbuf =
 	sc->rx_bpf_disc = sc->rx_overrunf = sc->rx_overrunl =
@@ -557,7 +670,9 @@ ep_attach(sc)
     sc->top = sc->mcur = 0;
 
 #if NBPFILTER > 0
-    bpfattach(&sc->bpf, ifp, DLT_EN10MB, sizeof(struct ether_header));
+    if (!attached) {
+	bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));
+    }
 #endif
     return 0;
 }
@@ -574,6 +689,9 @@ epinit(sc)
     register struct ifnet *ifp = &sc->arpcom.ac_if;
     int s, i, j;
 
+    if (sc->gone)
+	return;
+
 	/*
     if (ifp->if_addrlist == (struct ifaddr *) 0)
 	return;
@@ -602,6 +720,7 @@ epinit(sc)
 
     outw(BASE + EP_COMMAND, RX_RESET);
     outw(BASE + EP_COMMAND, TX_RESET);
+    while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
 
     /* Window 1 is operating window */
     GO_WINDOW(1);
@@ -615,64 +734,65 @@ epinit(sc)
 
     outw(BASE + EP_COMMAND, SET_INTR_MASK | S_5_INTS);
 
-	if(ifp->if_flags & IFF_PROMISC)
-		outw(BASE + EP_COMMAND, SET_RX_FILTER | FIL_INDIVIDUAL |
-		 FIL_GROUP | FIL_BRDCST | FIL_ALL);
-	else
-		outw(BASE + EP_COMMAND, SET_RX_FILTER | FIL_INDIVIDUAL |
-		 FIL_GROUP | FIL_BRDCST);
+    if (ifp->if_flags & IFF_PROMISC)
+	outw(BASE + EP_COMMAND, SET_RX_FILTER | FIL_INDIVIDUAL |
+	 FIL_GROUP | FIL_BRDCST | FIL_ALL);
+    else
+	outw(BASE + EP_COMMAND, SET_RX_FILTER | FIL_INDIVIDUAL |
+	 FIL_GROUP | FIL_BRDCST);
 
-	 /*
-	  * S.B.
-	  *
-	  * Now behavior was slightly changed:
-	  *
-	  * if any of flags link[0-2] is used and its connector is
-	  * physically present the following connectors are used:
-	  *
-	  *   link0 - AUI * highest precedence
-	  *   link1 - BNC
-	  *   link2 - UTP * lowest precedence
-	  *
-	  * If none of them is specified then
-	  * connector specified in the EEPROM is used
-	  * (if present on card or AUI if not).
-	  *
-	  */
+     /*
+      * S.B.
+      *
+      * Now behavior was slightly changed:
+      *
+      * if any of flags link[0-2] is used and its connector is
+      * physically present the following connectors are used:
+      *
+      *   link0 - AUI * highest precedence
+      *   link1 - BNC
+      *   link2 - UTP * lowest precedence
+      *
+      * If none of them is specified then
+      * connector specified in the EEPROM is used
+      * (if present on card or AUI if not).
+      *
+      */
 
-    if(ifp->if_flags & IFF_LINK0 && sc->ep_connectors & AUI) {
-	/* nothing */
-    } else if(ifp->if_flags & IFF_LINK1 && sc->ep_connectors & BNC) {
-	outw(BASE + EP_COMMAND, START_TRANSCEIVER);
-	DELAY(1000);
-    } else if(ifp->if_flags & IFF_LINK2 && sc->ep_connectors & UTP) {
-	GO_WINDOW(4);
-	outw(BASE + EP_W4_MEDIA_TYPE, ENABLE_UTP);
-	GO_WINDOW(1);
+    /* Set the xcvr. */
+    if (ifp->if_flags & IFF_LINK0 && sc->ep_connectors & AUI) {
+	i = ACF_CONNECTOR_AUI;
+    } else if (ifp->if_flags & IFF_LINK1 && sc->ep_connectors & BNC) {
+	i = ACF_CONNECTOR_BNC;
+    } else if (ifp->if_flags & IFF_LINK2 && sc->ep_connectors & UTP) {
+	i = ACF_CONNECTOR_UTP;
     } else {
-	GO_WINDOW(1);
-	switch(sc->ep_connector) {
-	    case ACF_CONNECTOR_UTP:
-		if(sc->ep_connectors & UTP) {
-		    GO_WINDOW(4);
-		    outw(BASE + EP_W4_MEDIA_TYPE, ENABLE_UTP);
-		    GO_WINDOW(1);
-		}
-		break;
-	    case ACF_CONNECTOR_BNC:
-		if(sc->ep_connectors & BNC) {
-		    outw(BASE + EP_COMMAND, START_TRANSCEIVER);
-		    DELAY(1000);
-		}
-		break;
-	    case ACF_CONNECTOR_AUI:
-		/* nothing to do */
-		break;
-	    default:
-		printf("ep%d: strange connector type in EEPROM: assuming AUI\n",
-		    sc->unit);
-		break;
+	i = sc->ep_connector;
+    }
+    GO_WINDOW(0);
+    j = inw(BASE + EP_W0_ADDRESS_CFG) & 0x3fff;
+    outw(BASE + EP_W0_ADDRESS_CFG, j | (i << ACF_CONNECTOR_BITS));
+
+    switch(i) {
+      case ACF_CONNECTOR_UTP:
+	if (sc->ep_connectors & UTP) {
+	    GO_WINDOW(4);
+	    outw(BASE + EP_W4_MEDIA_TYPE, ENABLE_UTP);
 	}
+	break;
+      case ACF_CONNECTOR_BNC:
+	if (sc->ep_connectors & BNC) {
+	    outw(BASE + EP_COMMAND, START_TRANSCEIVER);
+	    DELAY(1000);
+	}
+	break;
+      case ACF_CONNECTOR_AUI:
+	/* nothing to do */
+	break;
+      default:
+	printf("ep%d: strange connector type in EEPROM: assuming AUI\n",
+	       sc->unit);
+	break;
     }
 
     outw(BASE + EP_COMMAND, RX_ENABLE);
@@ -681,31 +801,17 @@ epinit(sc)
     ifp->if_flags |= IFF_RUNNING;
     ifp->if_flags &= ~IFF_OACTIVE;	/* just in case */
 
-    sc->tx_rate = TX_INIT_RATE;
-    sc->tx_counter = 0;
-    sc->rx_latency = RX_INIT_LATENCY;
-    sc->rx_early_thresh = RX_INIT_EARLY_THRESH;
 #ifdef EP_LOCAL_STATS
     sc->rx_no_first = sc->rx_no_mbuf =
 	sc->rx_bpf_disc = sc->rx_overrunf = sc->rx_overrunl =
 	sc->tx_underrun = 0;
 #endif
     ep_fset(F_RX_FIRST);
-    ep_frst(F_RX_TRAILER);
     if (sc->top) {
 	m_freem(sc->top);
 	sc->top = sc->mcur = 0;
     }
-    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | sc->rx_early_thresh);
-
-    /*
-     * These clever computations look very interesting
-     * but the fixed threshold gives near no output errors
-     * and if it as low as 16 bytes it gives the max. throughput.
-     * We think that processor is anyway quicker than Ethernet
-	 * (and this should be true for any 386 and higher)
-     */
-
+    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | RX_INIT_EARLY_THRESH);
     outw(BASE + EP_COMMAND, SET_TX_START_THRESH | 16);
 
     /*
@@ -713,10 +819,8 @@ epinit(sc)
      * any that we had in case we're being called from intr or somewhere
      * else.
      */
-    sc->last_mb = 0;
-    sc->next_mb = 0;
-    epmbuffill((caddr_t) sc, 0);
 
+    GO_WINDOW(1);
     epstart(ifp);
 
     splx(s);
@@ -728,13 +832,18 @@ static void
 epstart(ifp)
     struct ifnet *ifp;
 {
-    register struct ep_softc *sc = ep_softc[ifp->if_unit];
+    register struct ep_softc *sc = ifp->if_softc;
     register u_int len;
     register struct mbuf *m;
     struct mbuf *top;
     int s, pad;
 
+    if (sc->gone) {
+	return;
+    }
+
     s = splimp();
+    while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
     if (ifp->if_flags & IFF_OACTIVE) {
 	splx(s);
 	return;
@@ -766,30 +875,20 @@ startagain:
     if (inw(BASE + EP_W1_FREE_TX) < len + pad + 4) {
 	/* no room in FIFO */
 	outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH | (len + pad + 4));
-	ifp->if_flags |= IFF_OACTIVE;
-	splx(s);
-	return;
+	/* make sure */
+	if (inw(BASE + EP_W1_FREE_TX) < len + pad + 4) {
+	    ifp->if_flags |= IFF_OACTIVE;
+	    splx(s);
+	    return;
+	}
     }
     IF_DEQUEUE(&ifp->if_snd, m);
 
-    outw(BASE + EP_W1_TX_PIO_WR_1, len);
+    outw(BASE + EP_W1_TX_PIO_WR_1, len); 
     outw(BASE + EP_W1_TX_PIO_WR_1, 0x0);	/* Second dword meaningless */
 
-    /* compute the Tx start threshold for this packet */
-    sc->tx_start_thresh = len =
-	(((len * (64 - sc->tx_rate)) >> 6) & ~3) + 16;
-#if 0
-    /*
-     * The following string does something strange with the card and
-     * we get a lot of output errors due to it so it's commented out
-     * and we use fixed threshold (see above)
-     */
-
-    outw(BASE + EP_COMMAND, SET_TX_START_THRESH | len);
-#endif
-
     for (top = m; m != 0; m = m->m_next)
-	if(ep_ftst(F_ACCESS_32_BITS)) {
+	if (ep_ftst(F_ACCESS_32_BITS)) {
 	    outsl(BASE + EP_W1_TX_PIO_WR_1, mtod(m, caddr_t),
 		  m->m_len / 4);
 	    if (m->m_len & 3)
@@ -807,21 +906,14 @@ startagain:
 	outb(BASE + EP_W1_TX_PIO_WR_1, 0);	/* Padding */
 
 #if NBPFILTER > 0
-    if (sc->bpf) {
-	bpf_mtap(sc->bpf, top);
+    if (ifp->if_bpf) {
+	bpf_mtap(ifp, top);
     }
 #endif
 
-    ifp->if_timer=2;
+    ifp->if_timer = 2;
     ifp->if_opackets++;
     m_freem(top);
-    /*
-     * Every 1024*4 packets we increment the tx_rate if we haven't had
-     * errors, that in the case it has abnormaly goten too low
-     */
-    if (!(++sc->tx_counter & (1024 * 4 - 1)) &&
-	sc->tx_rate < TX_INIT_MAX_RATE)
-	sc->tx_rate++;
 
     /*
      * Is another packet coming in? We don't want to overflow the tiny RX
@@ -834,8 +926,7 @@ readcheck:
 	 * back later
 	 */
 	if (ifp->if_snd.ifq_head) {
-	    outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH |
-		 sc->tx_start_thresh);
+	    outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH | 8);
 	}
 	splx(s);
 	return;
@@ -848,6 +939,11 @@ epintr(unit)
     int unit;
 {
     register struct ep_softc *sc = ep_softc[unit];
+
+    if (sc->gone) {
+	return;
+    }
+
     ep_intr(sc);
 }
 
@@ -860,7 +956,7 @@ ep_intr(arg)
     struct ifnet *ifp;
     int x;
 
-    x=splbio();
+    x = splbio();
 
     sc = (struct ep_softc *)arg;
 
@@ -881,14 +977,14 @@ rescan:
 	}
 	if (status & S_TX_AVAIL) {
 	    /* we need ACK */
-	    ifp->if_timer=0;
+	    ifp->if_timer = 0;
 	    ifp->if_flags &= ~IFF_OACTIVE;
 	    GO_WINDOW(1);
 	    inw(BASE + EP_W1_FREE_TX);
 	    epstart(ifp);
 	}
 	if (status & S_CARD_FAILURE) {
-	    ifp->if_timer=0;
+	    ifp->if_timer = 0;
 #ifdef EP_LOCAL_STATS
 	    printf("\nep%d:\n\tStatus: %x\n", sc->unit, status);
 	    GO_WINDOW(4);
@@ -913,7 +1009,7 @@ rescan:
 	    return;
 	}
 	if (status & S_TX_COMPLETE) {
-	    ifp->if_timer=0;
+	    ifp->if_timer = 0;
 	    /* we  need ACK. we do it at the end */
 	    /*
 	     * We need to read TX_STATUS until we get a 0 status in order to
@@ -924,10 +1020,6 @@ rescan:
 		else if (status & (TXS_UNDERRUN | TXS_JABBER | TXS_MAX_COLLISION)) {
 		    outw(BASE + EP_COMMAND, TX_RESET);
 		    if (status & TXS_UNDERRUN) {
-			if (sc->tx_rate > 1) {
-			    sc->tx_rate--;	/* Actually in steps of 1/64 */
-			    sc->tx_counter = 0;	/* We reset it */
-			}
 #ifdef EP_LOCAL_STATS
 			sc->tx_underrun++;
 #endif
@@ -998,24 +1090,17 @@ read_again:
 	    else
 		sc->rx_overrunl++;
 #endif
-	    if (sc->rx_latency < ETHERMTU)
-		sc->rx_latency += 16;
 	}
 	goto out;
     }
     rx_fifo = rx_fifo2 = status & RX_BYTES_MASK;
 
     if (ep_ftst(F_RX_FIRST)) {
-	if (m = sc->mb[sc->next_mb]) {
-	    sc->mb[sc->next_mb] = 0;
-	    sc->next_mb = (sc->next_mb + 1) % MAX_MBS;
-	    m->m_data = m->m_pktdat;
-	    m->m_flags = M_PKTHDR;
-	} else {
-	    MGETHDR(m, M_DONTWAIT, MT_DATA);
-	    if (!m)
-		goto out;
-	}
+	MGETHDR(m, M_DONTWAIT, MT_DATA);
+	if (!m)
+	    goto out;
+	if (rx_fifo >= MINCLSIZE)
+	    MCLGET(m, M_DONTWAIT);
 	sc->top = sc->mcur = top = m;
 #define EROUND  ((sizeof(struct ether_header) + 3) & ~3)
 #define EOFF    (EROUND - sizeof(struct ether_header))
@@ -1032,9 +1117,6 @@ read_again:
 	top = sc->top;
 	m = sc->mcur;
 	sc->cur_len += rx_fifo2;
-	if (ep_ftst(F_RX_TRAILER))
-	    /* We don't read the trailer */
-	    rx_fifo -= sizeof(struct ether_header);
     }
 
     /* Reads what is left in the RX FIFO */
@@ -1042,15 +1124,9 @@ read_again:
 	lenthisone = min(rx_fifo, M_TRAILINGSPACE(m));
 	if (lenthisone == 0) {	/* no room in this one */
 	    mcur = m;
-	    if (m = sc->mb[sc->next_mb]) {
-		sc->mb[sc->next_mb] = 0;
-		sc->next_mb = (sc->next_mb + 1) % MAX_MBS;
-	    } else {
-		MGET(m, M_DONTWAIT, MT_DATA);
-		if (!m)
-		    goto out;
-	    }
-
+	    MGET(m, M_DONTWAIT, MT_DATA);
+	    if (!m)
+		goto out;
 	    if (rx_fifo >= MINCLSIZE)
 		MCLGET(m, M_DONTWAIT);
 	    m->m_len = 0;
@@ -1076,119 +1152,78 @@ read_again:
 	rx_fifo -= lenthisone;
     }
 
-    if (ep_ftst(F_RX_TRAILER)) {/* reads the trailer */
-	if (m = sc->mb[sc->next_mb]) {
-	    sc->mb[sc->next_mb] = 0;
-	    sc->next_mb = (sc->next_mb + 1) % MAX_MBS;
-	    m->m_data = m->m_pktdat;
-	    m->m_flags = M_PKTHDR;
-	} else {
-	    MGETHDR(m, M_DONTWAIT, MT_DATA);
-	    if (!m)
-		goto out;
-	}
-	insw(BASE + EP_W1_RX_PIO_RD_1, mtod(m, caddr_t),
-	     sizeof(struct ether_header));
-	m->m_len = sizeof(struct ether_header);
-	m->m_next = top;
-	sc->top = top = m;
-	/* XXX Accomodate for type and len from beginning of trailer */
-	sc->cur_len -= (2 * sizeof(u_short));
-	ep_frst(F_RX_TRAILER);
-	goto all_pkt;
-    }
-
     if (status & ERR_RX_INCOMPLETE) {	/* we haven't received the complete
 					 * packet */
 	sc->mcur = m;
 #ifdef EP_LOCAL_STATS
 	sc->rx_no_first++;	/* to know how often we come here */
 #endif
-	/*
-	 * Re-compute rx_latency, the factor used is 1/4 to go up and 1/32 to
-	 * go down
-	 */
-	delta = rx_fifo2 - sc->rx_early_thresh;	/* last latency seen LLS */
-	delta -= sc->rx_latency;/* LLS - estimated_latency */
-	if (delta >= 0)
-	    sc->rx_latency += (delta / 4);
-	else
-	    sc->rx_latency += (delta / 32);
 	ep_frst(F_RX_FIRST);
 	if (!((status = inw(BASE + EP_W1_RX_STATUS)) & ERR_RX_INCOMPLETE)) {
 	    /* we see if by now, the packet has completly arrived */
 	    goto read_again;
 	}
-	/* compute rx_early_threshold */
-	delta = (sc->rx_avg_pkt - sc->cur_len - sc->rx_latency - 16) & ~3;
-	if (delta < MIN_RX_EARLY_THRESHL)
-	    delta = MIN_RX_EARLY_THRESHL;
-
-	outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH |
-	     (sc->rx_early_thresh = delta));
+	outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | RX_NEXT_EARLY_THRESH);
 	return;
     }
 all_pkt:
     outw(BASE + EP_COMMAND, RX_DISCARD_TOP_PACK);
-    /*
-     * recompute average packet's length, the factor used is 1/8 to go down
-     * and 1/32 to go up
-     */
-    delta = sc->cur_len - sc->rx_avg_pkt;
-    if (delta > 0)
-	sc->rx_avg_pkt += (delta / 32);
-    else
-	sc->rx_avg_pkt += (delta / 8);
-    delta = (sc->rx_avg_pkt - sc->rx_latency - 16) & ~3;
-    if (delta < MIN_RX_EARLY_THRESHF)
-	delta = MIN_RX_EARLY_THRESHF;
-    sc->rx_early_thresh = delta;
     ++ifp->if_ipackets;
     ep_fset(F_RX_FIRST);
-    ep_frst(F_RX_TRAILER);
     top->m_pkthdr.rcvif = &sc->arpcom.ac_if;
     top->m_pkthdr.len = sc->cur_len;
 
 #if NBPFILTER > 0
-    if (sc->bpf) {
-	bpf_mtap(sc->bpf, top);
-
-	/*
-	 * Note that the interface cannot be in promiscuous mode if there are
-	 * no BPF listeners.  And if we are in promiscuous mode, we have to
-	 * check if this packet is really ours.
-	 */
-	eh = mtod(top, struct ether_header *);
-	if ((ifp->if_flags & IFF_PROMISC) &&
-	    (eh->ether_dhost[0] & 1) == 0 &&
-	    bcmp(eh->ether_dhost, sc->arpcom.ac_enaddr,
-		 sizeof(eh->ether_dhost)) != 0 &&
-	    bcmp(eh->ether_dhost, etherbroadcastaddr,
-		 sizeof(eh->ether_dhost)) != 0) {
-	    if (sc->top) {
-		m_freem(sc->top);
-		sc->top = 0;
-	    }
-	    ep_fset(F_RX_FIRST);
-	    ep_frst(F_RX_TRAILER);
-#ifdef EP_LOCAL_STATS
-	    sc->rx_bpf_disc++;
+    if (ifp->if_bpf)
+	bpf_mtap(ifp, top);
 #endif
-	    while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
-	    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | delta);
-	    return;
-	}
+#ifdef BRIDGE
+    if (do_bridge) {
+	struct ifnet * bdg_ifp ;
+
+	bdg_ifp = bridge_in(top);
+	if (bdg_ifp == BDG_DROP)
+	    goto dropit ;
+	if (bdg_ifp != BDG_LOCAL)
+	    bdg_forward(&(sc->top), bdg_ifp);
+	if (bdg_ifp !=BDG_LOCAL && bdg_ifp !=BDG_BCAST && bdg_ifp !=BDG_MCAST)
+	    goto dropit ;
+	/* all others accepted locally */
+	goto getit ;
     }
 #endif
 
+    /*
+     * If we are in promiscuous mode, we have to
+     * check if this packet is really ours.
+     */
+    eh = mtod(top, struct ether_header *);
+    if ((ifp->if_flags & IFF_PROMISC) &&
+	(eh->ether_dhost[0] & 1) == 0 &&
+	bcmp(eh->ether_dhost, sc->arpcom.ac_enaddr,
+	     sizeof(eh->ether_dhost)) != 0 &&
+	bcmp(eh->ether_dhost, etherbroadcastaddr,
+	     sizeof(eh->ether_dhost)) != 0) {
+dropit:
+	if (sc->top) {
+	    m_freem(sc->top);
+	    sc->top = 0;
+	}
+	ep_fset(F_RX_FIRST);
+#ifdef EP_LOCAL_STATS
+	sc->rx_bpf_disc++;
+#endif
+	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
+	outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | RX_INIT_EARLY_THRESH);
+	return;
+    }
+getit:
     eh = mtod(top, struct ether_header *);
     m_adj(top, sizeof(struct ether_header));
     ether_input(ifp, eh, top);
-    if (!sc->mb[sc->next_mb])
-	epmbuffill((caddr_t) sc, 0);
     sc->top = 0;
     while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
-    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | delta);
+    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | RX_INIT_EARLY_THRESH);
     return;
 
 out:
@@ -1200,14 +1235,9 @@ out:
 	sc->rx_no_mbuf++;
 #endif
     }
-    delta = (sc->rx_avg_pkt - sc->rx_latency - 16) & ~3;
-    if (delta < MIN_RX_EARLY_THRESHF)
-	delta = MIN_RX_EARLY_THRESHF;
     ep_fset(F_RX_FIRST);
-    ep_frst(F_RX_TRAILER);
     while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
-    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH |
-	 (sc->rx_early_thresh = delta));
+    outw(BASE + EP_COMMAND, SET_RX_EARLY_THRESH | RX_INIT_EARLY_THRESH);
 }
 
 /*
@@ -1220,7 +1250,7 @@ epioctl(ifp, cmd, data)
     caddr_t data;
 {
     register struct ifaddr *ifa = (struct ifaddr *) data;
-    struct ep_softc *sc = ep_softc[ifp->if_unit];
+    struct ep_softc *sc = ifp->if_softc;
     struct ifreq *ifr = (struct ifreq *) data;
     int s, error = 0;
 
@@ -1231,7 +1261,6 @@ epioctl(ifp, cmd, data)
 	ifp->if_flags |= IFF_UP;
 
 	/* netifs are BUSY when UP */
-	sc->kdc->kdc_state=DC_BUSY;
 
 	switch (ifa->ifa_addr->sa_family) {
 #ifdef INET
@@ -1291,15 +1320,10 @@ epioctl(ifp, cmd, data)
 	}
 	break;
       case SIOCSIFFLAGS:
-	/* UP controls BUSY/IDLE */
-	sc->kdc->kdc_state= ( (ifp->if_flags & IFF_UP)
-		? DC_BUSY
-		: DC_IDLE );
 
 	if ((ifp->if_flags & IFF_UP) == 0 && ifp->if_flags & IFF_RUNNING) {
 	    ifp->if_flags &= ~IFF_RUNNING;
 	    epstop(sc);
-	    epmbufempty(sc);
 	    break;
 	} else {
 	    /* reinitialize card on any parameter change */
@@ -1332,7 +1356,7 @@ epioctl(ifp, cmd, data)
 	     * multicast filters. If some day it will gain this
 	     * support this part of code must be extended.
 	     */
-	    error=0;
+	    error = 0;
 	    break;
       default:
 		error = EINVAL;
@@ -1344,11 +1368,11 @@ epioctl(ifp, cmd, data)
 }
 
 static void
-epwatchdog(unit)
-    int unit;
+epwatchdog(ifp)
+    struct ifnet *ifp;
 {
-    struct ep_softc *sc = ep_softc[unit];
-    struct ifnet *ifp = &sc->arpcom.ac_if;
+    struct ep_softc *sc = ifp->if_softc;
+
     /*
     printf("ep: watchdog\n");
 
@@ -1356,15 +1380,23 @@ epwatchdog(unit)
     ifp->if_oerrors++;
     */
 
+    if (sc->gone) {
+	return;
+    }
+
     ifp->if_flags &= ~IFF_OACTIVE;
     epstart(ifp);
-    ep_intr(sc);
+    ep_intr(ifp->if_softc);
 }
 
 static void
 epstop(sc)
     struct ep_softc *sc;
 {
+    if (sc->gone) {
+	return;
+    }
+
     outw(BASE + EP_COMMAND, RX_DISABLE);
     outw(BASE + EP_COMMAND, RX_DISCARD_TOP_PACK);
     while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
@@ -1372,6 +1404,7 @@ epstop(sc)
     outw(BASE + EP_COMMAND, STOP_TRANSCEIVER);
     outw(BASE + EP_COMMAND, RX_RESET);
     outw(BASE + EP_COMMAND, TX_RESET);
+    while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
     outw(BASE + EP_COMMAND, C_INTR_LATCH);
     outw(BASE + EP_COMMAND, SET_RD_0_MASK);
     outw(BASE + EP_COMMAND, SET_INTR_MASK);
@@ -1379,6 +1412,7 @@ epstop(sc)
 }
 
 
+#if 0
 static int
 send_ID_sequence(port)
     int port;
@@ -1393,6 +1427,7 @@ send_ID_sequence(port)
     }
     return (1);
 }
+#endif
 
 
 /*
@@ -1420,43 +1455,4 @@ get_eeprom_data(id_port, offset)
     return (data);
 }
 
-/*
- * We suppose this is always called inside a splimp(){...}splx() region
- */
-static void
-epmbuffill(sp, dummy_arg)
-    caddr_t sp;
-    int dummy_arg;
-{
-    struct ep_softc *sc = (struct ep_softc *) sp;
-    int i;
-
-    i = sc->last_mb;
-    do {
-	if (sc->mb[i] == NULL)
-	    MGET(sc->mb[i], M_DONTWAIT, MT_DATA);
-	if (sc->mb[i] == NULL)
-	    break;
-	i = (i + 1) % MAX_MBS;
-    } while (i != sc->next_mb);
-    sc->last_mb = i;
-}
-
-static void
-epmbufempty(sc)
-    struct ep_softc *sc;
-{
-    int s, i;
-
-    s = splimp();
-    for (i = 0; i < MAX_MBS; i++) {
-	if (sc->mb[i]) {
-	    m_freem(sc->mb[i]);
-	    sc->mb[i] = NULL;
-	}
-    }
-    sc->last_mb = sc->next_mb = 0;
-    splx(s);
-}
-
 #endif				/* NEP > 0 */
