@@ -31,7 +31,7 @@
 
  */
 /*
- * $FreeBSD: stable/2.1/sys/i386/isa/if_epreg.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_epreg.h 50950 1999-09-05 08:39:22Z peter $
  *
  *  Promiscuous mode added and interrupt logic slightly changed
  *  to reduce the number of adapter failures. Transceiver select
@@ -43,31 +43,25 @@
  *          babkin@hq.icb.chel.su
  */
 
+/*
+ * Pccard support for 3C589 by:
+ *		HAMADA Naoki
+ *		nao@tom-yam.or.jp
+ */
+
 /*
  * Ethernet software status per interface.
  */
 struct ep_softc {
     struct arpcom arpcom;	/* Ethernet common part		 */
-    short ep_io_addr;		/* i/o bus address		 */
-#define MAX_MBS  8		/* # of mbufs we keep around	 */
-    struct mbuf *mb[MAX_MBS];	/* spare mbuf storage.		 */
-    int next_mb;		/* Which mbuf to use next. 	 */
-    int last_mb;		/* Last mbuf.			 */
+    int ep_io_addr;		/* i/o bus address		 */
     struct mbuf *top, *mcur;
-    short tx_start_thresh;	/* Current TX_start_thresh.	 */
-    short tx_rate;
-    short tx_counter;
-    short rx_early_thresh;	/* Current RX_early_thresh.     */
-    short rx_latency;
-    short rx_avg_pkt;
     short cur_len;
-    caddr_t bpf;		/* BPF  "magic cookie"		 */
     u_short ep_connectors;	/* Connectors on this card.	 */
     u_char ep_connector;	/* Configured connector.	 */
     int stat;			/* some flags */
+    int gone;			/* adapter is not present (for PCCARD) */
 #define         F_RX_FIRST   0x1
-#define         F_WAIT_TRAIL 0x2
-#define         F_RX_TRAILER 0x4
 #define		F_PROMISC    0x8
 
 #define         F_ACCESS_32_BITS 0x100
@@ -76,8 +70,6 @@ struct ep_softc {
 
     int unit;
 
-    struct kern_devconf* kdc;
-
 #ifdef  EP_LOCAL_STATS
     short tx_underrun;
     short rx_no_first;
@@ -101,22 +93,31 @@ struct ep_board {
 /*
  * Some global constants
  */
-#define ETHER_MIN_LEN	     64
-#define ETHER_MAX_LEN	   1518
-#define ETHER_ADDR_LEN	      6
-
 #define TX_INIT_RATE         16
 #define TX_INIT_MAX_RATE     64
 #define RX_INIT_LATENCY      64
-#define RX_INIT_EARLY_THRESH 64
-#define MIN_RX_EARLY_THRESHF   16 /* not less than ether_header */
-#define MIN_RX_EARLY_THRESHL   4
+#define RX_INIT_EARLY_THRESH 208 /* not less than MINCLSIZE */
+#define RX_NEXT_EARLY_THRESH 500
 
 #define EEPROMSIZE      0x40
 #define MAX_EEPROMBUSY  1000
 #define EP_LAST_TAG     0xd7
 #define EP_MAX_BOARDS   16
-#define EP_ID_PORT      0x100
+/*
+ * This `ID' port is a mere hack.  There's currently no chance to register
+ * it with config's idea of the ports that are in use.
+ *
+ * "After the automatic configuration is completed, the IDS is in it's initial
+ * state (ID-WAIT), and it monitors all write access to I/O port 01x0h, where
+ * 'x' is any hex digit.  If a zero is written to any one of these ports, then
+ * that address is remembered and becomes the ID port.  A second zero written
+ * to that port resets the ID sequence to its initial state.  The IDS watches
+ * for the ID sequence to be written to the ID port."
+ *
+ * We prefer 0x110 over 0x100 so to not conflict with the Plaque&Pray
+ * ports.
+ */
+#define EP_ID_PORT      0x110
 #define EP_IOSIZE	16	/* 16 bytes of I/O space used. */
 
 /*
@@ -364,7 +365,9 @@ struct ep_board {
  *
  */
 
-#define SET_IRQ(i)	(((i)<<12) | 0xF00) /* set IRQ i */
+#define SET_IRQ(base,irq)     outw((base) + EP_W0_RESOURCE_CFG, \
+                              ((inw((base) + EP_W0_RESOURCE_CFG) & 0x0fff) | \
+                              ((u_short)(irq)<<12))  ) /* set IRQ i */
 
 /*
  * FIFO Registers.
@@ -447,8 +450,6 @@ struct ep_board {
 #define BNC 				0x2
 #define UTP 				0x4
 
-#define ETHER_ADDR_LEN			6
-#define ETHER_MAX			1536
 #define RX_BYTES_MASK			(u_short) (0x07ff)
 
 extern	struct ep_board ep_board[];
