@@ -21,7 +21,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/if_le.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_le.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -40,6 +40,7 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/mbuf.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
@@ -48,7 +49,6 @@
 #include <sys/errno.h>
 #include <sys/malloc.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
 
 #include <net/if.h>
 #include <net/if_types.h>
@@ -65,6 +65,11 @@
 #include <netinet/if_ether.h>
 #endif
 
+#ifdef IPX
+#include <netipx/ipx.h>
+#include <netipx/ipx_if.h>
+#endif
+
 #ifdef NS
 #include <netns/ns.h>
 #include <netns/ns_if.h>
@@ -72,11 +77,12 @@
 
 #include <machine/clock.h>
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/icu.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
 
 #if NBPFILTER > 0
 #include <net/bpf.h>
@@ -92,10 +98,10 @@ typedef u_short le_mcbits_t;
 #define LE_MC_NBPW		(1 << LE_MC_NBPW_LOG2)
 #if __FreeBSD__ > 1
 #define	IF_RESET_ARGS	int unit
-#define	LE_RESET(ifp)	(((sc)->le_if.if_reset)((sc)->le_if.if_unit))
+#define	LE_RESET(ifp)	(((sc)->if_reset)((sc)->le_if.if_unit))
 #else
 #define	IF_RESET_ARGS	int unit, int dummy
-#define	LE_RESET(ifp)	(((sc)->le_if.if_reset)((sc)->le_if.if_unit, 0))
+#define	LE_RESET(ifp)	(((sc)->if_reset)((sc)->le_if.if_unit, 0))
 #endif
 
 #if !defined(LE_NOLEMAC)
@@ -215,6 +221,8 @@ static void (*le_intrvec[NLE])(le_softc_t *sc);
  */
 struct le_softc {
     struct arpcom le_ac;		/* Common Ethernet/ARP Structure */
+    void (*if_init) __P((int));		/* Interface init routine */
+    void (*if_reset) __P((int));	/* Interface reset routine */
     caddr_t le_membase;			/* Starting memory address (virtual) */
     unsigned le_iobase;			/* Starting I/O base address */
     unsigned le_irq;			/* Interrupt Request Value */
@@ -223,9 +231,6 @@ struct le_softc {
     u_int le_mcmask;			/* bit mask for CRC-32 for multicast hash */
     le_mcbits_t *le_mctbl;		/* pointer to multicast table */
     const char *le_prodname;		/* product name DE20x-xx */
-#if NBPFILTER > 0
-    caddr_t le_bpf;			/* BPF context */
-#endif
     u_char le_hwaddr[6];		/* local copy of hwaddr */
     unsigned le_scast_drops;		/* singlecast drops */
     unsigned le_mcast_drops;		/* multicast drops */
@@ -245,7 +250,6 @@ struct le_softc {
 static int le_probe(struct isa_device *dvp);
 static int le_attach(struct isa_device *dvp);
 static int le_ioctl(struct ifnet *ifp, int command, caddr_t data);
-extern int le_intr(int unit);
 static void le_input(le_softc_t *sc, caddr_t seg1, size_t total_len,
 		     size_t len2, caddr_t seg2);
 static void le_multi_filter(le_softc_t *sc);
@@ -259,9 +263,9 @@ struct le_board {
 };
 
 
-le_softc_t le_softc[NLE];
+static le_softc_t le_softc[NLE];
 
-const le_board_t le_boards[] = {
+static const le_board_t le_boards[] = {
 #if !defined(LE_NOLEMAC)
     { lemac_probe },			/* DE20[345] */
 #endif
@@ -278,7 +282,7 @@ struct isa_driver ledriver = {
     le_probe, le_attach, "le",
 };
 
-unsigned le_intrs[NLE];
+static unsigned le_intrs[NLE];
 
 #define	LE_ADDREQUAL(a1, a2) \
 	(((u_short *)a1)[0] == ((u_short *)a2)[0] \
@@ -320,26 +324,6 @@ unsigned le_intrs[NLE];
 #define	MEMSET(where, what, howmuch)	bzero(where, howmuch)
 #define	MEMCMP(l, r, len)		bcmp(l, r, len)
 
-static struct kern_devconf kdc_le[NLE] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"le", 0, { MDDT_ISA, 0, "net" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"Ethernet adapter: DEC EtherWorks II or EtherWorks III",
-	DC_CLS_NETIF		/* class */
-} };
-
-static inline void
-le_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_le[id->id_unit] = kdc_le[0];
-	kdc_le[id->id_unit].kdc_unit = id->id_unit;
-	kdc_le[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_le[id->id_unit]);
-}
 
 static int
 le_probe(
@@ -355,8 +339,6 @@ le_probe(
 	return 0;
     }
 
-    le_registerdev(dvp);
-
     sc->le_iobase = dvp->id_iobase;
     sc->le_membase = (u_char *) dvp->id_maddr;
     sc->le_irq = dvp->id_irq;
@@ -384,16 +366,15 @@ le_attach(
 {
     le_softc_t *sc = &le_softc[dvp->id_unit];
     struct ifnet *ifp = &sc->le_if;
-    struct ifaddr *ifa = ifp->if_addrlist;
 
+    ifp->if_softc = sc;
     ifp->if_mtu = ETHERMTU;
-    printf("%s%d: %s ethernet address %s\n",
+    printf("%s%d: %s ethernet address %6D\n",
 	   ifp->if_name, ifp->if_unit,
 	   sc->le_prodname,
-	   ether_sprintf(sc->le_ac.ac_enaddr));
+	   sc->le_ac.ac_enaddr, ":");
 
-    ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_NOTRAILERS;
-    ifp->if_flags  |= IFF_MULTICAST;
+    ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
     ifp->if_output = ether_output;
     ifp->if_ioctl = le_ioctl;
     ifp->if_type = IFT_ETHER;
@@ -401,30 +382,16 @@ le_attach(
     ifp->if_hdrlen = 14;
 
 #if NBPFILTER > 0
-    bpfattach(&sc->le_bpf, ifp, DLT_EN10MB, sizeof(struct ether_header));
+    bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));
 #endif
 
     if_attach(ifp);
-    kdc_le[dvp->id_unit].kdc_state = DC_IDLE;
+    ether_ifattach(ifp);
 
-    while (ifa && ifa->ifa_addr && ifa->ifa_addr->sa_family != AF_LINK)
-	ifa = ifa->ifa_next;
-
-    if (ifa != NULL && ifa->ifa_addr != NULL) {
-	struct sockaddr_dl *sdl;
-	/*
-	 * Provide our ether address to the higher layers
-	 */
-	sdl = (struct sockaddr_dl *) ifa->ifa_addr;
-	sdl->sdl_type = IFT_ETHER;
-	sdl->sdl_alen = 6;
-	sdl->sdl_slen = 0;
-	MEMCPY(LLADDR(sdl), sc->le_ac.ac_enaddr, 6);
-    }
     return 1;
 }
 
-int
+void
 le_intr(
     int unit)
 {
@@ -434,7 +401,6 @@ le_intr(
     (*le_intrvec[unit])(&le_softc[unit]);
 
     splx(s);
-    return unit;
 }
 
 #define	LE_XTRA		0
@@ -458,8 +424,8 @@ le_input(
     MEMCPY(&eh, seg1, sizeof(eh));
 
 #if NBPFILTER > 0
-    if (sc->le_bpf != NULL && seg2 == NULL) {
-	bpf_tap(sc->le_bpf, seg1, total_len);
+    if (sc->le_if.if_bpf != NULL && seg2 == NULL) {
+	bpf_tap(&sc->le_if, seg1, total_len);
 	/*
 	 * If this is single cast but not to us
 	 * drop it!
@@ -515,8 +481,8 @@ le_input(
     if (seg2 != NULL)
 	MEMCPY(mtod(m, caddr_t) + len1, seg2, total_len - len1);
 #if NBPFILTER > 0
-    if (sc->le_bpf != NULL && seg2 != NULL) {
-	bpf_mtap(sc->le_bpf, m);
+    if (sc->le_if.if_bpf != NULL && seg2 != NULL) {
+	bpf_mtap(&sc->le_if, m);
 	/*
 	 * If this is single cast but not to us
 	 * drop it!
@@ -549,7 +515,7 @@ le_ioctl(
     int cmd,
     caddr_t data)
 {
-    le_softc_t *sc = &le_softc[ifp->if_unit];
+    le_softc_t *sc = ifp->if_softc;
     int s, error = 0;
 
     if ((sc->le_flags & IFF_UP) == 0)
@@ -565,12 +531,31 @@ le_ioctl(
 	    switch(ifa->ifa_addr->sa_family) {
 #ifdef INET
 		case AF_INET: {
-		    (*ifp->if_init)(ifp->if_unit);
+		    (*sc->if_init)(ifp->if_unit);
 		    arp_ifinit((struct arpcom *)ifp, ifa);
 		    break;
 		}
 #endif /* INET */
+#ifdef IPX
+		/* This magic copied from if_is.c; I don't use XNS,
+		 * so I have no way of telling if this actually
+		 * works or not.
+		 */
+		case AF_IPX: {
+		    struct ipx_addr *ina = &(IA_SIPX(ifa)->sipx_addr);
+		    if (ipx_nullhost(*ina)) {
+			ina->x_host = *(union ipx_host *)(sc->le_ac.ac_enaddr);
+		    } else {
+			ifp->if_flags &= ~IFF_RUNNING;
+			bcopy((caddr_t)ina->x_host.c_host,
+			      (caddr_t)sc->le_ac.ac_enaddr,
+			      sizeof sc->le_ac.ac_enaddr);
+		    }
 
+		    (*sc->if_init)(ifp->if_unit);
+		    break;
+		}
+#endif /* IPX */
 #ifdef NS
 		/* This magic copied from if_is.c; I don't use XNS,
 		 * so I have no way of telling if this actually
@@ -587,13 +572,12 @@ le_ioctl(
 			      sizeof sc->le_ac.ac_enaddr);
 		    }
 
-		    (*ifp->if_init)(ifp->if_unit);
+		    (*sc->if_init)(ifp->if_unit);
 		    break;
 		}
 #endif /* NS */
-
 		default: {
-		    (*ifp->if_init)(ifp->if_unit);
+		    (*sc->if_init)(ifp->if_unit);
 		    break;
 		}
 	    }
@@ -601,7 +585,7 @@ le_ioctl(
 	}
 
 	case SIOCSIFFLAGS: {
-	    (*ifp->if_init)(ifp->if_unit);
+	    (*sc->if_init)(ifp->if_unit);
 	    break;
 	}
 
@@ -617,7 +601,7 @@ le_ioctl(
 
 	    if (error == ENETRESET) {
 		/* reset multicast filtering */
-		(*ifp->if_init)(ifp->if_unit);
+		(*sc->if_init)(ifp->if_unit);
 		error = 0;
 	    }
 	    break;
@@ -629,8 +613,6 @@ le_ioctl(
     }
 
     splx(s);
-    kdc_le[ifp->if_unit].kdc_state = (ifp->if_flags & IFF_UP)
-      ? DC_BUSY : DC_IDLE;
     return error;
 }
 
@@ -713,10 +695,10 @@ le_multi_filter(
 	return;
     }
     sc->le_flags &= ~IFF_MULTICAST;
-    if (sc->le_ac.ac_ipaddr.s_addr != 0) {
+    /* if (interface has had an address assigned) { */
 	le_multi_op(sc, etherbroadcastaddr, TRUE);
 	sc->le_flags |= LE_BRDCSTONLY|IFF_MULTICAST;
-    }
+    /* } */
 #ifdef ISO
     le_multi_op(sc, all_es_snpa, TRUE);
 #endif
@@ -816,13 +798,13 @@ static const int lemac_irqs[] = { IRQ5, IRQ10, IRQ11, IRQ15 };
 /*
  * Some tuning/monitoring variables.
  */
-unsigned lemac_deftxmax = 16;	/* see lemac_max above */
-unsigned lemac_txnospc = 0;	/* total # of tranmit starvations */
+static unsigned lemac_deftxmax = 16;	/* see lemac_max above */
+static unsigned lemac_txnospc = 0;	/* total # of tranmit starvations */
 
-unsigned lemac_tne_intrs = 0;	/* total # of tranmit done intrs */
-unsigned lemac_rne_intrs = 0;	/* total # of receive done intrs */
-unsigned lemac_txd_intrs = 0;	/* total # of tranmit error intrs */
-unsigned lemac_rxd_intrs = 0;	/* total # of receive error intrs */
+static unsigned lemac_tne_intrs = 0;	/* total # of tranmit done intrs */
+static unsigned lemac_rne_intrs = 0;	/* total # of receive done intrs */
+static unsigned lemac_txd_intrs = 0;	/* total # of tranmit error intrs */
+static unsigned lemac_rxd_intrs = 0;	/* total # of receive error intrs */
 
 
 static int
@@ -831,8 +813,7 @@ lemac_probe(
     const le_board_t *bd,
     int *msize)
 {
-    int irq, portval, cksum;
-    long membase;
+    int irq, portval;
 
     LE_OUTB(sc, LEMAC_REG_IOP, LEMAC_IOP_EEINIT);
     DELAY(LEMAC_EEP_DELAY);
@@ -865,9 +846,9 @@ lemac_probe(
     /*
      * Try to reset the unit
      */
-    sc->le_if.if_init = lemac_init;
+    sc->if_init = lemac_init;
     sc->le_if.if_start = lemac_start;
-    sc->le_if.if_reset = lemac_reset;
+    sc->if_reset = lemac_reset;
     sc->lemac_memmode = 2;
     LE_RESET(sc);
     if ((sc->le_flags & IFF_UP) == 0)
@@ -959,7 +940,7 @@ lemac_init(
     int unit)
 {
     le_softc_t *sc = &le_softc[unit];
-    int regval, s;
+    int s;
 
     if ((sc->le_flags & IFF_UP) == 0)
 	return;
@@ -1059,7 +1040,6 @@ lemac_rne_intr(
     le_softc_t *sc)
 {
     int rxcount, rxlen, rxpg;
-    struct mbuf *m;
     u_char *rxptr;
 
     lemac_rne_intrs++;
@@ -1081,7 +1061,6 @@ lemac_rne_intr(
 	} else { /* end if (*rxptr & LEMAC_RX_OK) */
 	    sc->le_if.if_ierrors++;
 	}
-next:
 	LE_OUTB(sc, LEMAC_REG_FMQ, rxpg);  /* Return this page to Free Memory Queue */
     }  /* end while (recv_count--) */
 
@@ -1194,8 +1173,8 @@ lemac_start(
 	LE_OUTB(sc, LEMAC_REG_TQ, tx_pg);	/* tell chip to transmit this packet */
 
 #if NBPFILTER > 0
-	if (sc->le_bpf)
-		bpf_mtap(sc->le_bpf, m);
+	if (sc->le_if.if_bpf)
+		bpf_mtap(&sc->le_if, m);
 #endif
 
 	m_freem(m);			/* free the mbuf */
@@ -1333,7 +1312,7 @@ static int  lance_tx_intr(le_softc_t *sc);
 #define	LN_DESC_MAX		128
 
 #if LN_DOSTATS
-struct {
+static struct {
     unsigned lance_rx_misses;
     unsigned lance_rx_badcrc;
     unsigned lance_rx_badalign;
@@ -1542,8 +1521,8 @@ depca_probe(
     if (!lance_init_adapmem(sc))
 	return 0;
 
-    sc->le_if.if_reset = lance_reset;
-    sc->le_if.if_init = lance_init;
+    sc->if_reset = lance_reset;
+    sc->if_init = lance_init;
     sc->le_if.if_start = lance_start;
     DEPCA_WRNICSR(sc, DEPCA_NICSR_SHE | DEPCA_NICSR_ENABINTR);
     LE_RESET(sc);
