@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1995
+ * Copyright (c) 1995, 1996
  *	Paul Richards.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@
  */
 
 /*
+#define LNC_MULTICAST
 #define DIAGNOSTIC
 #define DEBUG
  *
@@ -59,6 +60,7 @@
  *
  */
 
+#include "pci.h"
 #include "lnc.h"
 #if NLNC > 0
 
@@ -66,20 +68,18 @@
 
 /* Some defines that should really be in generic locations */
 #define FCS_LEN 4
-#define ETHER_ADDR_LEN 6
-#define ETHER_HDR_LEN 14
-#define MULTICAST_ADDR_LEN 8
-#define ETHER_MIN_LEN 64
+#define MULTICAST_FILTER_LEN 8
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/errno.h>
 #include <sys/ioccom.h>
 #include <sys/sockio.h>
+#include <sys/malloc.h>
 #include <sys/mbuf.h>
 #include <sys/socket.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
 
 #include <net/if.h>
 #include <net/if_dl.h>
@@ -97,6 +97,15 @@
 #include <net/bpfdesc.h>
 #endif
 
+#ifdef BRIDGE
+#include <net/bridge.h>
+#endif
+
+#ifdef PC98
+#include <machine/clock.h>
+#endif
+#include <machine/md_var.h>
+
 #include <i386/isa/isa_device.h>
 #include <i386/isa/if_lnc.h>
 
@@ -117,123 +126,192 @@ struct lnc_softc {
 	int initialised;
 	int rap;
 	int rdp;
-	struct kern_devconf kdc;
+	int bdp;
 #ifdef DEBUG
 	int lnc_debug;
-#endif
-#if NBPFILTER > 0
-	caddr_t bpf;	/* XXX bpf magic cookie - move to arpcom */
 #endif
 	LNCSTATS_STRUCT
-} lnc_softc[NLNC];
+};
 
-/* Function prototypes */
-int bicc_probe(struct isa_device *);
-int depca_probe(struct isa_device *);
-int lance_probe(int);
-int ne2100_probe(struct isa_device *);
-int pcnet_probe(int);
-void lnc_init(int);
-void lnc_start(struct ifnet *);
-int  lnc_ioctl(struct ifnet *, int, caddr_t);
-void lnc_watchdog(int);
-int  lnc_probe(struct isa_device *);
-int  lnc_attach(struct isa_device *);
-void lnc_dump_state(int);
+static struct lnc_softc lnc_softc[NLNC];
+
+static char const * const nic_ident[] = {
+	"Unknown",
+	"BICC",
+	"NE2100",
+	"DEPCA",
+	"CNET98S",	/* PC-98 */
+};
+
+static char const * const ic_ident[] = {
+	"Unknown",
+	"LANCE",
+	"C-LANCE",
+	"PCnet-ISA",
+	"PCnet-ISA+",
+	"PCnet-ISA II",
+	"PCnet-32 VL-Bus",
+	"PCnet-PCI",
+	"PCnet-PCI II",
+	"PCnet-FAST",
+	"PCnet-FAST+",
+};
+
+#ifdef LNC_MULTICAST
+static void lnc_setladrf __P((struct lnc_softc *sc));
+#endif
+static void lnc_stop __P((struct lnc_softc *sc));
+static void lnc_reset __P((struct lnc_softc *sc));
+static void lnc_free_mbufs __P((struct lnc_softc *sc));
+static int alloc_mbuf_cluster __P((struct lnc_softc *sc, struct host_ring_entry *desc));
+static struct mbuf *chain_mbufs __P((struct lnc_softc *sc, int start_of_packet, int pkt_len));
+static struct mbuf *mbuf_packet __P((struct lnc_softc *sc, int start_of_packet, int pkt_len));
+static void lnc_rint __P((struct lnc_softc *sc));
+static void lnc_tint __P((struct lnc_softc *sc));
+static int lnc_probe __P((struct isa_device *isa_dev));
+#ifdef PC98
+static int cnet98s_probe __P((struct lnc_softc *sc, unsigned iobase));
+#endif
+static int ne2100_probe __P((struct lnc_softc *sc, unsigned iobase));
+static int bicc_probe __P((struct lnc_softc *sc, unsigned iobase));
+static int dec_macaddr_extract __P((u_char ring[], struct lnc_softc *sc));
+static int depca_probe __P((struct lnc_softc *sc, unsigned iobase));
+static int lance_probe __P((struct lnc_softc *sc));
+static int pcnet_probe __P((struct lnc_softc *sc));
+static int lnc_attach_sc __P((struct lnc_softc *sc, int unit));
+static int lnc_attach __P((struct isa_device *isa_dev));
+static void lnc_init __P((struct lnc_softc *sc));
+static int mbuf_to_buffer __P((struct mbuf *m, char *buffer));
+static struct mbuf *chain_to_cluster __P((struct mbuf *m));
+static void lnc_start __P((struct ifnet *ifp));
+static int lnc_ioctl __P((struct ifnet *ifp, int command, caddr_t data));
+static void lnc_watchdog __P((struct ifnet *ifp));
+#ifdef DEBUG
+static void lnc_dump_state __P((struct lnc_softc *sc));
+static void mbuf_dump_chain __P((struct mbuf *m));
+#endif
+
+#if NPCI > 0
+void *lnc_attach_ne2100_pci __P((int unit, unsigned iobase));
+#endif
+void lncintr_sc __P((struct lnc_softc *sc));
 
 struct isa_driver lncdriver = {lnc_probe, lnc_attach, "lnc"};
 
-static struct kern_devconf kdc_lnc = {
-	0, 0, 0,		/* filled in by dev_attach */
-	"lnc", 0, { MDDT_ISA, 0, "net" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,
-	"",
-	DC_CLS_NETIF
-};
-
-static inline void
-write_csr(int unit, u_short port, u_short val)
+static __inline void
+write_csr(struct lnc_softc *sc, u_short port, u_short val)
 {
-	outw(lnc_softc[unit].rap, port);
-	outw(lnc_softc[unit].rdp, val);
+	outw(sc->rap, port);
+	outw(sc->rdp, val);
 }
 
-static inline u_short
-read_csr(int unit, u_short port)
+static __inline u_short
+read_csr(struct lnc_softc *sc, u_short port)
 {
-	outw(lnc_softc[unit].rap, port);
-	return (inw(lnc_softc[unit].rdp));
+	outw(sc->rap, port);
+	return (inw(sc->rdp));
 }
 
-static inline void
-lnc_registerdev(struct isa_device *isa_dev)
+static __inline void
+write_bcr(struct lnc_softc *sc, u_short port, u_short val)
 {
-	struct lnc_softc *sc = &lnc_softc[isa_dev->id_unit];
-	struct kern_devconf *kdc = &sc->kdc;
-	*kdc = kdc_lnc;
-	kdc->kdc_unit = isa_dev->id_unit;
-	kdc->kdc_parentdata = isa_dev;
+	outw(sc->rap, port);
+	outw(sc->bdp, val);
+}
 
-	switch(sc->nic.ic) {
-		case LANCE:
-			if (sc->nic.ident == BICC)
-				kdc->kdc_description = "BICC (LANCE) Ethernet controller";
-			else if (sc->nic.ident == NE2100)
-				kdc->kdc_description = "NE2100 (LANCE) Ethernet controller";
-			else if (sc->nic.ident == DEPCA)
-				kdc->kdc_description = "DEPCA (LANCE) Ethernet controller";
-			break;
-		case C_LANCE:
-			if (sc->nic.ident == BICC)
-				kdc->kdc_description = "BICC (C-LANCE) Ethernet controller";
-			else if (sc->nic.ident == NE2100)
-				kdc->kdc_description = "NE2100 (C-LANCE) Ethernet controller";
-			else if (sc->nic.ident == DEPCA)
-				kdc->kdc_description = "DEPCA (C-LANCE) Ethernet controller";
-			break;
-		case PCnet_ISA:
-			kdc->kdc_description = "PCnet-ISA Ethernet controller";
-			break;
-		case PCnet_ISAplus:
-			kdc->kdc_description = "PCnet-ISA+ Ethernet controller";
-			break;
-		case PCnet_32:
-			kdc->kdc_description = "PCnet-32 VL-Bus Ethernet controller";
-			break;
-		case PCnet_PCI:
-			kdc->kdc_description = "PCnet-PCI Ethernet controller";
-			break;
-		default:
-			break;
+static __inline u_short
+read_bcr(struct lnc_softc *sc, u_short port)
+{
+	outw(sc->rap, port);
+	return (inw(sc->bdp));
+}
+
+#ifdef LNC_MULTICAST
+static __inline u_long
+ether_crc(u_char *ether_addr)
+{
+#define POLYNOMIAL 0x04c11db6
+	u_long crc = 0xffffffffL;
+	int i, j, carry;
+	u_char b;
+
+	for (i = ETHER_ADDR_LEN; --i >= 0;) {
+		b = *ether_addr++;
+		for (j = 8; --j >= 0;) {
+			carry  = ((crc & 0x80000000L) ? 1 : 0) ^ (b & 0x01);
+			crc <<= 1;
+			b >>= 1;
+			if (carry)
+				crc = ((crc ^ POLYNOMIAL) | carry);
+		}
 	}
-
-	dev_attach(kdc);
+	return crc;
+#undef POLYNOMIAL
 }
 
-
-void
-lnc_setladrf(struct ifnet *ifp, struct lnc_softc *sc)
+/*
+ * Set up the logical address filter for multicast packets
+ */
+static void
+lnc_setladrf(struct lnc_softc *sc)
 {
+	struct ifnet *ifp = &sc->arpcom.ac_if;
+	struct ether_multistep step;
+	struct ether_multi *enm;
+	u_long index;
+	int i;
 
+	/* If promiscuous mode is set then all packets are accepted anyway */
+	if (ifp->if_flags & IFF_PROMISC) {
+		ifp->if_flags |= IFF_ALLMULTI;
+		for (i = 0; i < MULTICAST_FILTER_LEN; i++)
+			sc->init_block->ladrf[i] = 0xff;
+		return;
+	}   
+
+/*
+ * For each multicast address, calculate a crc for that address and
+ * then use the high order 6 bits of the crc as a hash code where
+ * bits 3-5 select the byte of the address filter and bits 0-2 select
+ * the bit within that byte.
+ */
+
+	bzero(sc->init_block->ladrf, MULTICAST_FILTER_LEN);
+	ETHER_FIRST_MULTI(step, &sc->arpcom, enm);
+	while (enm != NULL) {
+		if (bcmp(enm->enm_addrlo, enm->enm_addrhi, ETHER_ADDR_LEN) != 0) {
+			/*
+			 * A range of multicast addresses should be accepted but
+			 * but for now just accept all multicasts. Only currently
+			 * used by multicast routing where the range would require
+			 * all bits to be set anyway.
+			 */
+			ifp->if_flags |= IFF_ALLMULTI;
+			for (i = 0; i < MULTICAST_FILTER_LEN; i++)
+				sc->init_block->ladrf[i] = 0xff;
+			return;
+		}
+
+		index = ether_crc(enm->enm_addrlo) >> 26;
+		sc->init_block->ladrf[index >> 3] |= 1 << (index & 7);
+		ETHER_NEXT_MULTI(step, enm);
+	}
 }
+#endif /* LNC_MULTICAST */
 
-void
-lnc_stop(int unit)
+static void
+lnc_stop(struct lnc_softc *sc)
 {
-	write_csr(unit, CSR0, STOP);
+	write_csr(sc, CSR0, STOP);
 }
 
-void
-lnc_reset(int unit)
+static void
+lnc_reset(struct lnc_softc *sc)
 {
-	int s;
-	lnc_init(unit);
+	lnc_init(sc);
 }
 
-void
+static void
 lnc_free_mbufs(struct lnc_softc *sc)
 {
 	int i;
@@ -255,7 +333,7 @@ lnc_free_mbufs(struct lnc_softc *sc)
 		m_freem(sc->mbufs);
 }
 
-inline int
+static __inline int
 alloc_mbuf_cluster(struct lnc_softc *sc, struct host_ring_entry *desc)
 {
 	register struct mds *md = desc->md;
@@ -288,7 +366,7 @@ alloc_mbuf_cluster(struct lnc_softc *sc, struct host_ring_entry *desc)
 	return(0);
 }
 
-inline struct mbuf *
+static __inline struct mbuf *
 chain_mbufs(struct lnc_softc *sc, int start_of_packet, int pkt_len)
 {
 	struct mbuf *head, *m;
@@ -322,7 +400,7 @@ chain_mbufs(struct lnc_softc *sc, int start_of_packet, int pkt_len)
 	return(head);
 }
 
-inline struct mbuf *
+static __inline struct mbuf *
 mbuf_packet(struct lnc_softc *sc, int start_of_packet, int pkt_len)
 {
 
@@ -397,10 +475,9 @@ mbuf_packet(struct lnc_softc *sc, int start_of_packet, int pkt_len)
 }
 
 
-inline void
-lnc_rint(int unit)
+static __inline void
+lnc_rint(struct lnc_softc *sc)
 {
-	register struct lnc_softc *sc = &lnc_softc[unit];
 	struct host_ring_entry *next, *start;
 	int start_of_packet;
 	struct mbuf *head;
@@ -420,13 +497,15 @@ lnc_rint(int unit)
 
 #ifdef DIAGNOSTIC
 	if ((sc->recv_ring + sc->recv_next)->md->md1 & OWN) {
+		int unit = sc->arpcom.ac_if.if_unit;
 		log(LOG_ERR, "lnc%d: Receive interrupt with buffer still owned by controller -- Resetting\n", unit);
-		lnc_reset(unit);
+		lnc_reset(sc);
 		return;
 	}
 	if (!((sc->recv_ring + sc->recv_next)->md->md1 & STP)) {
+		int unit = sc->arpcom.ac_if.if_unit;
 		log(LOG_ERR, "lnc%d: Receive interrupt but not start of packet -- Resetting\n", unit);
-		lnc_reset(unit);
+		lnc_reset(sc);
 		return;
 	}
 #endif
@@ -456,8 +535,9 @@ lnc_rint(int unit)
 			} while (!(flags & (STP | OWN | ENP | MDERR)));
 
 			if (flags & STP) {
+				int unit = sc->arpcom.ac_if.if_unit;
 				log(LOG_ERR, "lnc%d: Start of packet found before end of previous in receive ring -- Resetting\n", unit);
-				lnc_reset(unit);
+				lnc_reset(sc);
 				return;
 			}
 			if (flags & OWN) {
@@ -469,8 +549,9 @@ lnc_rint(int unit)
 					sc->recv_next = start_of_packet;
 					break;
 				} else {
+					int unit = sc->arpcom.ac_if.if_unit;
 					log(LOG_ERR, "lnc%d: End of received packet not found-- Resetting\n", unit);
-					lnc_reset(unit);
+					lnc_reset(sc);
 					return;
 				}
 			}
@@ -483,6 +564,7 @@ lnc_rint(int unit)
 		next = sc->recv_ring + sc->recv_next;
 
 		if (flags & MDERR) {
+			int unit = sc->arpcom.ac_if.if_unit;
 			if (flags & RBUFF) {
 				LNCSTATS(rbuff)
 				log(LOG_ERR, "lnc%d: Receive buffer error\n", unit);
@@ -494,13 +576,14 @@ lnc_rint(int unit)
 					log(LOG_ERR, "lnc%d: Receive overflow error \n", unit);
 				}
 			} else if (flags & ENP) {
+			    if ((sc->arpcom.ac_if.if_flags & IFF_PROMISC)==0) {
 				/*
 				 * FRAM and CRC are valid only if ENP
 				 * is set and OFLO is not.
 				 */
 				if (flags & FRAM) {
 					LNCSTATS(fram)
-					log(LOG_ERR, "lnc%d: Framming error\n", unit);
+					log(LOG_ERR, "lnc%d: Framing error\n", unit);
 					/*
 					 * FRAM is only set if there's a CRC
 					 * error so avoid multiple messages
@@ -509,6 +592,7 @@ lnc_rint(int unit)
 					LNCSTATS(crc)
 					log(LOG_ERR, "lnc%d: Receive CRC error\n", unit);
 				}
+			    }
 			}
 
 			/* Drop packet */
@@ -537,7 +621,7 @@ lnc_rint(int unit)
 				 * ethernet and packet headers
 				 */
 				head->m_pkthdr.rcvif = &sc->arpcom.ac_if;
-				head->m_pkthdr.len = pkt_len - sizeof *eh;
+				head->m_pkthdr.len = pkt_len ;
 
 				/*
 				 * BPF expects the ether header to be in the first
@@ -549,9 +633,26 @@ lnc_rint(int unit)
 				eh = (struct ether_header *) head->m_data;
 
 #if NBPFILTER > 0
-				if (sc->bpf)
-					bpf_mtap(sc->bpf, head);
+				if (sc->arpcom.ac_if.if_bpf)
+					bpf_mtap(&sc->arpcom.ac_if, head);
+#endif
+#ifdef BRIDGE
+                                if (do_bridge) {
+                                    struct ifnet *bdg_ifp ;
 
+                                    bdg_ifp = bridge_in(head);
+                                    if (bdg_ifp == BDG_DROP)
+                                        m_freem(head);
+                                    else {
+                                        if (bdg_ifp != BDG_LOCAL)
+                                            bdg_forward(&head, bdg_ifp);
+                                        if (bdg_ifp == BDG_LOCAL || bdg_ifp == BDG_BCAST || bdg_ifp == BDG_MCAST)
+                                            goto getit;
+                                        else if (head)
+                                            m_freem(head);
+                                    }
+                                } else
+#endif
 				/* Check this packet is really for us */
 
 				if ((sc->arpcom.ac_if.if_flags & IFF_PROMISC) &&
@@ -560,16 +661,18 @@ lnc_rint(int unit)
 							sizeof(eh->ether_dhost))))
 						m_freem(head);
 				else
-#endif
 				{
+getit:
 					/* Skip over the ether header */
 					head->m_data += sizeof *eh;
 					head->m_len -= sizeof *eh;
+					head->m_pkthdr.len -= sizeof *eh;
 
 					ether_input(&sc->arpcom.ac_if, eh, head);
 				}
 
 			} else {
+				int unit = sc->arpcom.ac_if.if_unit;
 				log(LOG_ERR,"lnc%d: Packet dropped, no mbufs\n",unit);
 				LNCSTATS(drop_packet)
 			}
@@ -587,10 +690,9 @@ lnc_rint(int unit)
 	outw(sc->rdp, RINT | INEA);
 }
 
-inline void
-lnc_tint(int unit)
+static __inline void
+lnc_tint(struct lnc_softc *sc)
 {
-	register struct lnc_softc *sc = &lnc_softc[unit];
 	struct host_ring_entry *next, *start;
 	int start_of_packet;
 	int lookahead;
@@ -612,8 +714,9 @@ lnc_tint(int unit)
 
 #ifdef DIAGNOSTIC
 	if ((sc->trans_ring + sc->trans_next)->md->md1 & OWN) {
+		int unit = sc->arpcom.ac_if.if_unit;
 		log(LOG_ERR, "lnc%d: Transmit interrupt with buffer still owned by controller -- Resetting\n", unit);
-		lnc_reset(unit);
+		lnc_reset(sc);
 		return;
 	}
 #endif
@@ -641,8 +744,9 @@ lnc_tint(int unit)
 
 #ifdef DIAGNOSTIC
 	if (!(next->md->md1 & STP)) {
+		int unit = sc->arpcom.ac_if.if_unit;
 		log(LOG_ERR, "lnc%d: Transmit interrupt but not start of packet -- Resetting\n", unit);
-		lnc_reset(unit);
+		lnc_reset(sc);
 		return;
 	}
 #endif
@@ -658,8 +762,9 @@ lnc_tint(int unit)
 			} while (!(next->md->md1 & (STP | OWN | ENP | MDERR)));
 
 			if (next->md->md1 & STP) {
+				int unit = sc->arpcom.ac_if.if_unit;
 				log(LOG_ERR, "lnc%d: Start of packet found before end of previous in transmit ring -- Resetting\n", unit);
-				lnc_reset(unit);
+				lnc_reset(sc);
 				return;
 			}
 			if (next->md->md1 & OWN) {
@@ -671,8 +776,9 @@ lnc_tint(int unit)
 					sc->trans_next = start_of_packet;
 					break;
 				} else {
+					int unit = sc->arpcom.ac_if.if_unit;
 					log(LOG_ERR, "lnc%d: End of transmitted packet not found -- Resetting\n", unit);
-					lnc_reset(unit);
+					lnc_reset(sc);
 					return;
 				}
 			}
@@ -683,6 +789,8 @@ lnc_tint(int unit)
 		 */
 		if (next->md->md1 & MDERR) {
 
+			int unit = sc->arpcom.ac_if.if_unit;
+
 			LNCSTATS(terr)
 				sc->arpcom.ac_if.if_oerrors++;
 
@@ -731,7 +839,7 @@ lnc_tint(int unit)
 					log(LOG_ERR, "lnc%d: Transmit buffer error -- Resetting\n", unit);
 				} else
 					log(LOG_ERR, "lnc%d: Transmit underflow error -- Resetting\n", unit);
-				lnc_reset(unit);
+				lnc_reset(sc);
 				return;
 			}
 			do {
@@ -822,44 +930,122 @@ lnc_tint(int unit)
 
 }
 
-int
+static int
 lnc_probe(struct isa_device * isa_dev)
 {
-int nports;
-int vsw;
+	int nports;
+	int unit = isa_dev->id_unit;
+	struct lnc_softc *sc = &lnc_softc[unit];
+	unsigned iobase = isa_dev->id_iobase;
 
 #ifdef DIAGNOSTIC
+	int vsw;
 	vsw = inw(isa_dev->id_iobase + PCNET_VSW);
 	printf("Vendor Specific Word = %x\n", vsw);
 #endif
 
-
-	if (nports = bicc_probe(isa_dev))
-		return (nports);
-	if (nports = ne2100_probe(isa_dev))
-		return (nports);
-	if (nports = depca_probe(isa_dev))
-		return (nports);
-
-	/*
-	 * Register device even though probe has
-	 * failed so it can be reconfigured later.
-	 */
-
-	lnc_registerdev(isa_dev);
-	return (0);
+	nports = bicc_probe(sc, iobase);
+	if (nports == 0)
+		nports = ne2100_probe(sc, iobase);
+	if (nports == 0)
+		nports = depca_probe(sc, iobase);
+#ifdef PC98
+	if (nports == 0)
+		nports = cnet98s_probe(sc, iobase);
+#endif
+	return (nports);
 }
 
-int
-ne2100_probe(struct isa_device * isa_dev)
+#ifdef PC98
+/* ISA Bus Configuration Registers */
+/* XXX - Should be in ic/Am7990.h */
+#define	MSRDA	0x0000	/* ISACSR0: Master Mode Read Activity */
+#define	MSWRA	0x0001	/* ISACSR1: Master Mode Write Activity */
+#define	MC	0x0002	/* ISACSR2: Miscellaneous Configuration */
+
+#define	LED1	0x0005	/* ISACSR5: LED1 Status */
+#define	LED2	0x0006	/* ISACSR6: LED2 Status */
+#define	LED3	0x0007	/* ISACSR7: LED3 Status */
+
+#define	LED_PSE		0x0080	/* Pulse Stretcher */
+#define	LED_XMTE	0x0010	/* Transmit Status */
+#define	LED_RVPOLE	0x0008	/* Receive Polarity */
+#define	LED_RCVE	0x0004	/* Receive Status */
+#define	LED_JABE	0x0002	/* Jabber */
+#define	LED_COLE	0x0001	/* Collision */
+
+static int
+cnet98s_probe(struct lnc_softc *sc, unsigned iobase)
+{
+	int i;
+	ushort tmp;
+
+	sc->rap = iobase + CNET98S_RAP;
+	sc->rdp = iobase + CNET98S_RDP;
+
+	/* Reset */
+	tmp = inw(iobase + CNET98S_RESET);
+	outw(iobase + CNET98S_RESET, tmp);
+	DELAY(500);
+
+	sc->nic.ic = pcnet_probe(sc);
+	if ((sc->nic.ic == UNKNOWN) || (sc->nic.ic > PCnet_32)) {
+		return (0);
+	}
+
+	sc->nic.ident = CNET98S;
+	sc->nic.mem_mode = DMA_FIXED;
+
+	/* XXX - For now just use the defines */
+	sc->nrdre = NRDRE;
+	sc->ntdre = NTDRE;
+
+	/* Extract MAC address from PROM */
+	for (i = 0; i < ETHER_ADDR_LEN; i++) {
+		sc->arpcom.ac_enaddr[i] = inb(iobase + (i * 2));
+	}
+
+	/*
+	 * ISA Configuration
+	 *
+	 * XXX - Following parameters are Contec C-NET(98)S only.
+	 *       So, check the Ethernet address here.
+	 *
+	 *       Contec uses 00 80 4c ?? ?? ??
+	 */ 
+	if (sc->arpcom.ac_enaddr[0] == (u_char)0x00
+	&&  sc->arpcom.ac_enaddr[1] == (u_char)0x80
+	&&  sc->arpcom.ac_enaddr[2] == (u_char)0x4c) {
+        	outw(sc->rap, MSRDA);
+        	outw(iobase + CNET98S_IDP, 0x0006);
+        	outw(sc->rap, MSWRA);
+        	outw(iobase + CNET98S_IDP, 0x0006);
+#ifdef DIAGNOSTIC
+        	outw(sc->rap, MC);
+		printf("ISACSR2 = %x\n", inw(iobase + CNET98S_IDP));
+#endif
+        	outw(sc->rap, LED1);
+        	outw(iobase + CNET98S_IDP, LED_PSE | LED_XMTE);
+        	outw(sc->rap, LED2);
+        	outw(iobase + CNET98S_IDP, LED_PSE | LED_RCVE);
+        	outw(sc->rap, LED3);
+        	outw(iobase + CNET98S_IDP, LED_PSE | LED_COLE);
+	}
+		
+	return (CNET98S_IOSIZE);
+}
+#endif
+
+static int
+ne2100_probe(struct lnc_softc *sc, unsigned iobase)
 {
-	struct lnc_softc *sc = &lnc_softc[isa_dev->id_unit];
 	int i;
 
-	sc->rap = isa_dev->id_iobase + PCNET_RAP;
-	sc->rdp = isa_dev->id_iobase + PCNET_RDP;
+	sc->rap = iobase + PCNET_RAP;
+	sc->rdp = iobase + PCNET_RDP;
 
-	if (sc->nic.ic = pcnet_probe(isa_dev->id_unit)) {
+	sc->nic.ic = pcnet_probe(sc);
+	if ((sc->nic.ic > 0) && (sc->nic.ic < PCnet_PCI)) {
 		sc->nic.ident = NE2100;
 		sc->nic.mem_mode = DMA_FIXED;
 
@@ -869,19 +1055,17 @@ ne2100_probe(struct isa_device * isa_dev)
 
 		/* Extract MAC address from PROM */
 		for (i = 0; i < ETHER_ADDR_LEN; i++)
-			sc->arpcom.ac_enaddr[i] = inb(isa_dev->id_iobase + i);
-		lnc_registerdev(isa_dev);
+			sc->arpcom.ac_enaddr[i] = inb(iobase + i);
 		return (NE2100_IOSIZE);
 	} else {
 		return (0);
 	}
 }
 
-int
-bicc_probe(struct isa_device * isa_dev)
+static int
+bicc_probe(struct lnc_softc *sc, unsigned iobase)
 {
-struct lnc_softc *sc = &lnc_softc[isa_dev->id_unit];
-int i;
+	int i;
 
 	/*
 	 * There isn't any way to determine if a NIC is a BICC. Basically, if
@@ -890,12 +1074,12 @@ int i;
 	 *
 	 */
 
-	sc->rap = isa_dev->id_iobase + BICC_RAP;
-	sc->rdp = isa_dev->id_iobase + BICC_RDP;
+	sc->rap = iobase + BICC_RAP;
+	sc->rdp = iobase + BICC_RDP;
 
 	/* I think all these cards us the Am7990 */
 
-	if (sc->nic.ic = lance_probe(isa_dev->id_unit)) {
+	if ((sc->nic.ic = lance_probe(sc))) {
 		sc->nic.ident = BICC;
 		sc->nic.mem_mode = DMA_FIXED;
 
@@ -905,9 +1089,8 @@ int i;
 
 		/* Extract MAC address from PROM */
 		for (i = 0; i < ETHER_ADDR_LEN; i++)
-			sc->arpcom.ac_enaddr[i] = inb(isa_dev->id_iobase + (i * 2));
+			sc->arpcom.ac_enaddr[i] = inb(iobase + (i * 2));
 
-		lnc_registerdev(isa_dev);
 		return (BICC_IOSIZE);
 	} else {
 		return (0);
@@ -920,7 +1103,7 @@ int i;
  * you get the next byte in the ring. The mac address is stored after a
  * signature so keep searching for the signature first.
  */
-int
+static int
 dec_macaddr_extract(u_char ring[], struct lnc_softc * sc)
 {
 	const unsigned char signature[] = {0xff, 0x00, 0x55, 0xaa, 0xff, 0x00, 0x55, 0xaa};
@@ -946,44 +1129,42 @@ dec_macaddr_extract(u_char ring[], struct lnc_softc * sc)
 	return (0);
 }
 
-int
-depca_probe(struct isa_device * isa_dev)
+static int
+depca_probe(struct lnc_softc *sc, unsigned iobase)
 {
 	int i;
-	struct lnc_softc *sc = &lnc_softc[isa_dev->id_unit];
 	unsigned char maddr_ring[DEPCA_ADDR_ROM_SIZE];
 
-	sc->rap = isa_dev->id_iobase + DEPCA_RAP;
-	sc->rdp = isa_dev->id_iobase + DEPCA_RDP;
+	sc->rap = iobase + DEPCA_RAP;
+	sc->rdp = iobase + DEPCA_RDP;
 
-	if (sc->nic.ic = lance_probe(isa_dev->id_unit)) {
+	if ((sc->nic.ic = lance_probe(sc))) {
 		sc->nic.ident = DEPCA;
 		sc->nic.mem_mode = SHMEM;
 
 		/* Extract MAC address from PROM */
 		for (i = 0; i < DEPCA_ADDR_ROM_SIZE; i++)
-			maddr_ring[i] = inb(isa_dev->id_iobase + DEPCA_ADP);
+			maddr_ring[i] = inb(iobase + DEPCA_ADP);
 		if (dec_macaddr_extract(maddr_ring, sc)) {
-			lnc_registerdev(isa_dev);
 			return (DEPCA_IOSIZE);
 		}
 	}
 	return (0);
 }
 
-int
-lance_probe(int unit)
+static int
+lance_probe(struct lnc_softc *sc)
 {
-	write_csr(unit, CSR0, STOP);
+	write_csr(sc, CSR0, STOP);
 
-	if ((inw(lnc_softc[unit].rdp) & STOP) && !(read_csr(unit, CSR3))) {
+	if ((inw(sc->rdp) & STOP) && !(read_csr(sc, CSR3))) {
 		/*
 		 * Check to see if it's a C-LANCE. For the LANCE the INEA bit
 		 * cannot be set while the STOP bit is. This restriction is
 		 * removed for the C-LANCE.
 		 */
-		write_csr(unit, CSR0, INEA);
-		if (read_csr(unit, CSR0) & INEA)
+		write_csr(sc, CSR0, INEA);
+		if (read_csr(sc, CSR0) & INEA)
 			return (C_LANCE);
 		else
 			return (LANCE);
@@ -991,8 +1172,8 @@ lance_probe(int unit)
 		return (UNKNOWN);
 }
 
-int
-pcnet_probe(int unit)
+static int
+pcnet_probe(struct lnc_softc *sc)
 {
 	u_long chip_id;
 	int type;
@@ -1003,11 +1184,11 @@ pcnet_probe(int unit)
 	 * though so the probe is just a matter of reading it.
 	 */
 
-	if (type = lance_probe(unit)) {
+	if ((type = lance_probe(sc))) {
 
-		chip_id = read_csr(unit, CSR89);
+		chip_id = read_csr(sc, CSR89);
 		chip_id <<= 16;
-		chip_id |= read_csr(unit, CSR88);
+		chip_id |= read_csr(sc, CSR88);
 		if (chip_id & AMD_MASK) {
 			chip_id >>= 12;
 			switch (chip_id & PART_MASK) {
@@ -1015,10 +1196,18 @@ pcnet_probe(int unit)
 				return (PCnet_ISA);
 			case Am79C961:
 				return (PCnet_ISAplus);
+			case Am79C961A:
+				return (PCnet_ISA_II);
 			case Am79C965:
 				return (PCnet_32);
 			case Am79C970:
 				return (PCnet_PCI);
+			case Am79C970A:
+				return (PCnet_PCI_II);
+			case Am79C971:
+				return (PCnet_FAST);
+			case Am79C972:
+				return (PCnet_FASTplus);
 			default:
 				break;
 			}
@@ -1027,10 +1216,9 @@ pcnet_probe(int unit)
 	return (type);
 }
 
-int
-lnc_attach(struct isa_device * isa_dev)
+static int
+lnc_attach_sc(struct lnc_softc *sc, int unit)
 {
-	struct lnc_softc *sc = &lnc_softc[isa_dev->id_unit];
 	int lnc_mem_size;
 
 	/*
@@ -1061,35 +1249,36 @@ lnc_attach(struct isa_device * isa_dev)
 	sc->recv_ring = malloc(lnc_mem_size, M_DEVBUF, M_NOWAIT);
 
 	if (!sc->recv_ring) {
-		log(LOG_ERR, "lnc%d: Couldn't allocate memory for NIC\n", isa_dev->id_unit);
+		log(LOG_ERR, "lnc%d: Couldn't allocate memory for NIC\n", unit);
 		return (0);	/* XXX -- attach failed -- not tested in
 				 * calling routines */
 	}
+	/*
+	 * XXX - Shouldn't this be skipped for the EISA and PCI versions ???
+	 *       Print the message but do not return for the PCnet_PCI !
+	 */
 	if ((sc->nic.mem_mode != SHMEM) && (kvtop(sc->recv_ring) > 0x1000000)) {
-		log(LOG_ERR, "lnc%d: Memory allocated above 16Mb limit\n", isa_dev->id_unit);
-		return (0);
+		log(LOG_ERR, "lnc%d: Memory allocated above 16Mb limit\n", unit);
+		if ((sc->nic.ic != PCnet_PCI) &&
+		    (sc->nic.ic != PCnet_PCI_II) &&
+		    (sc->nic.ic != PCnet_FAST))
+			return (0);
 	}
 
-	if ((sc->nic.mem_mode != SHMEM) &&
-		 (sc->nic.ic != PCnet_32) &&
-		 (sc->nic.ic != PCnet_PCI))
-		isa_dmacascade(isa_dev->id_drq);
-
 	/* Set default mode */
 	sc->nic.mode = NORMAL;
 
 	/* Fill in arpcom structure entries */
 
+	sc->arpcom.ac_if.if_softc = sc;
 	sc->arpcom.ac_if.if_name = lncdriver.name;
-	sc->arpcom.ac_if.if_unit = isa_dev->id_unit;
+	sc->arpcom.ac_if.if_unit = unit;
 	sc->arpcom.ac_if.if_mtu = ETHERMTU;
-	sc->arpcom.ac_if.if_flags = IFF_BROADCAST | IFF_NOTRAILERS | IFF_SIMPLEX;
+	sc->arpcom.ac_if.if_flags = IFF_BROADCAST | IFF_SIMPLEX;
 	sc->arpcom.ac_if.if_timer = 0;
-	sc->arpcom.ac_if.if_init = lnc_init;
 	sc->arpcom.ac_if.if_output = ether_output;
 	sc->arpcom.ac_if.if_start = lnc_start;
 	sc->arpcom.ac_if.if_ioctl = lnc_ioctl;
-	sc->arpcom.ac_if.if_reset = lnc_reset;
 	sc->arpcom.ac_if.if_watchdog = lnc_watchdog;
 	sc->arpcom.ac_if.if_type = IFT_ETHER;
 	sc->arpcom.ac_if.if_addrlen = ETHER_ADDR_LEN;
@@ -1100,27 +1289,92 @@ lnc_attach(struct isa_device * isa_dev)
 	 */
 
 	if_attach(&sc->arpcom.ac_if);
-	sc->kdc.kdc_state = DC_IDLE;
+	ether_ifattach(&sc->arpcom.ac_if);
 
-	printf("lnc%d: %s, address %s\n",
-	       isa_dev->id_unit,
-		   sc->kdc.kdc_description,
-	       ether_sprintf(sc->arpcom.ac_enaddr));
+	printf("lnc%d: ", unit);
+	if (sc->nic.ic == LANCE || sc->nic.ic == C_LANCE)
+		printf("%s (%s)",
+		       nic_ident[sc->nic.ident], ic_ident[sc->nic.ic]);
+	else
+		printf("%s", ic_ident[sc->nic.ic]);
+	printf(" address %6D\n", sc->arpcom.ac_enaddr, ":");
 
 #if NBPFILTER > 0
-	bpfattach(&sc->bpf, &sc->arpcom.ac_if, DLT_EN10MB, sizeof(struct ether_header));
+	bpfattach(&sc->arpcom.ac_if, DLT_EN10MB, sizeof(struct ether_header));
 #endif
 
 	return (1);
 }
 
-void
-lnc_init(int unit)
+static int
+lnc_attach(struct isa_device * isa_dev)
 {
+	int unit = isa_dev->id_unit;
 	struct lnc_softc *sc = &lnc_softc[unit];
+
+	int result = lnc_attach_sc (sc, unit);
+	if (result == 0)
+		return (0);
+
+#ifndef PC98
+	/*
+	 * XXX - is it safe to call isa_dmacascade() after if_attach() 
+	 *       and ether_ifattach() have been called in lnc_attach() ???
+	 */
+	if ((sc->nic.mem_mode != SHMEM) &&
+		 (sc->nic.ic < PCnet_32))
+		isa_dmacascade(isa_dev->id_drq);
+#endif
+
+	return result;
+}
+
+#if NPCI > 0
+void *
+lnc_attach_ne2100_pci(int unit, unsigned iobase)
+{
+	int i;
+	struct lnc_softc *sc = malloc(sizeof *sc, M_DEVBUF, M_NOWAIT);
+
+	if (sc) {
+		bzero (sc, sizeof *sc);
+
+		sc->rap = iobase + PCNET_RAP;
+		sc->rdp = iobase + PCNET_RDP;
+		sc->bdp = iobase + PCNET_BDP;
+
+		sc->nic.ic = pcnet_probe(sc);
+		if (sc->nic.ic >= PCnet_PCI) {
+			sc->nic.ident = NE2100;
+			sc->nic.mem_mode = DMA_FIXED;
+
+			/* XXX - For now just use the defines */
+			sc->nrdre = NRDRE;
+			sc->ntdre = NTDRE;
+
+			/* Extract MAC address from PROM */
+			for (i = 0; i < ETHER_ADDR_LEN; i++)
+				sc->arpcom.ac_enaddr[i] = inb(iobase + i);
+
+			if (lnc_attach_sc(sc, unit) == 0) {
+				free(sc, M_DEVBUF);
+				sc = NULL;
+			}
+		}
+		else {
+			free(sc, M_DEVBUF);
+			sc = NULL;
+		}
+	}
+	return sc;
+}
+#endif
+
+static void
+lnc_init(struct lnc_softc *sc)
+{
 	int s, i;
 	char *lnc_mem;
-	struct mbuf *m = 0;
 
 	/* Check that interface has valid address */
 
@@ -1130,8 +1384,8 @@ lnc_init(int unit)
 	/* Shut down interface */
 
 	s = splimp();
-	lnc_stop(unit);
-	sc->arpcom.ac_if.if_flags |= IFF_BROADCAST | IFF_NOTRAILERS | IFF_SIMPLEX;
+	lnc_stop(sc);
+	sc->arpcom.ac_if.if_flags |= IFF_BROADCAST | IFF_SIMPLEX; /* XXX??? */
 
 	/*
 	 * This sets up the memory area for the controller. Memory is set up for
@@ -1224,8 +1478,12 @@ lnc_init(int unit)
 	for (i = 0; i < ETHER_ADDR_LEN; i++)
 		sc->init_block->padr[i] = sc->arpcom.ac_enaddr[i];
 
-	for (i = 0; i < MULTICAST_ADDR_LEN; i++)
+#ifdef LNC_MULTICAST
+			lnc_setladrf(sc);
+#else
+	for (i = 0; i < MULTICAST_FILTER_LEN; i++)
 		sc->init_block->ladrf[i] = MULTI_INIT_ADDR;
+#endif
 
 	sc->init_block->rdra = kvtop(sc->recv_ring->md);
 	sc->init_block->rlen = ((kvtop(sc->recv_ring->md) >> 16) & 0xff) | (sc->nrdre << 13);
@@ -1240,8 +1498,8 @@ lnc_init(int unit)
 
 	/* Give the LANCE the physical address of the initialisation block */
 
-	write_csr(unit, CSR1, kvtop(sc->init_block));
-	write_csr(unit, CSR2, (kvtop(sc->init_block) >> 16) & 0xff);
+	write_csr(sc, CSR1, kvtop(sc->init_block));
+	write_csr(sc, CSR2, (kvtop(sc->init_block) >> 16) & 0xff);
 
 	/*
 	 * Depending on which controller this is, CSR3 has different meanings.
@@ -1251,13 +1509,13 @@ lnc_init(int unit)
 	 *
 	 */
 
-	write_csr(unit, CSR3, 0);
+	write_csr(sc, CSR3, 0);
 
 	/* Let's see if it starts */
 
-	write_csr(unit, CSR0, INIT);
+	write_csr(sc, CSR0, INIT);
 	for (i = 0; i < 1000; i++)
-		if (read_csr(unit, CSR0) & IDON)
+		if (read_csr(sc, CSR0) & IDON)
 			break;
 
 	/*
@@ -1267,17 +1525,18 @@ lnc_init(int unit)
 	 * time.
 	 */
 
-	if (read_csr(unit, CSR0) & IDON) {
+	if (read_csr(sc, CSR0) & IDON) {
 		/*
 		 * Enable interrupts, start the LANCE, mark the interface as
 		 * running and transmit any pending packets.
 		 */
-		write_csr(unit, CSR0, STRT | INEA);
+		write_csr(sc, CSR0, STRT | INEA);
 		sc->arpcom.ac_if.if_flags |= IFF_RUNNING;
 		sc->arpcom.ac_if.if_flags &= ~IFF_OACTIVE;
 		lnc_start(&sc->arpcom.ac_if);
 	} else
-		log(LOG_ERR, "lnc%d: Initialisation failed\n", unit);
+		log(LOG_ERR, "lnc%d: Initialisation failed\n", 
+		    sc->arpcom.ac_if.if_unit);
 
 	splx(s);
 }
@@ -1304,9 +1563,9 @@ lnc_init(int unit)
  */
 
 void
-lncintr(int unit)
+lncintr_sc(struct lnc_softc *sc)
 {
-	struct lnc_softc *sc = &lnc_softc[unit];
+	int unit = sc->arpcom.ac_if.if_unit;
 	u_short csr0;
 
 	/*
@@ -1324,7 +1583,8 @@ lncintr(int unit)
 		 * be missed.
 		 */
 
-		outw(sc->rdp, IDON | CERR | BABL | MISS | MERR | RINT | TINT | INEA);
+		/*outw(sc->rdp, IDON | CERR | BABL | MISS | MERR | RINT | TINT | INEA);*/
+		outw(sc->rdp, csr0);
 
 		/* We don't do anything with the IDON flag */
 
@@ -1346,18 +1606,18 @@ lncintr(int unit)
 			if (csr0 & MERR) {
 				log(LOG_ERR, "lnc%d: Memory error  -- Resetting\n", unit);
 				LNCSTATS(merr)
-				lnc_reset(unit);
+				lnc_reset(sc);
 				continue;
 			}
 		}
 		if (csr0 & RINT) {
 			LNCSTATS(rint)
-			lnc_rint(unit);
+			lnc_rint(sc);
 		}
 		if (csr0 & TINT) {
 			LNCSTATS(tint)
 			sc->arpcom.ac_if.if_timer = 0;
-			lnc_tint(unit);
+			lnc_tint(sc);
 		}
 
 		/*
@@ -1370,10 +1630,17 @@ lncintr(int unit)
 	}
 }
 
+void
+lncintr(int unit)
+{
+	struct lnc_softc *sc = &lnc_softc[unit];
+	lncintr_sc (sc);
+}
 
 
 
-inline int
+
+static __inline int
 mbuf_to_buffer(struct mbuf *m, char *buffer)
 {
 
@@ -1388,7 +1655,7 @@ mbuf_to_buffer(struct mbuf *m, char *buffer)
 	return(len);
 }
 
-inline struct mbuf *
+static __inline struct mbuf *
 chain_to_cluster(struct mbuf *m)
 {
 	struct mbuf *new;
@@ -1413,17 +1680,16 @@ chain_to_cluster(struct mbuf *m)
  * flags should be ok at those points too.
  */
 
-void
+static void
 lnc_start(struct ifnet *ifp)
 {
 
-	struct lnc_softc *sc = &lnc_softc[ifp->if_unit];
+	struct lnc_softc *sc = ifp->if_softc;
 	struct host_ring_entry *desc;
 	int tmp;
 	int end_of_packet;
 	struct mbuf *head, *m;
 	int len, chunk;
-	char *buffer;
 	int addr;
 	int no_entries_needed;
 
@@ -1457,7 +1723,7 @@ lnc_start(struct ifnet *ifp)
 			if (no_entries_needed > (NDESC(sc->ntdre) - sc->pending_transmits))
 				if (!(head = chain_to_cluster(head))) {
 					log(LOG_ERR, "lnc%d: Couldn't get mbuf for transmit packet -- Resetting \n ",ifp->if_unit);
-					lnc_reset(ifp->if_unit);
+					lnc_reset(sc);
 					return;
 				}
 			else if ((sc->nic.ic == LANCE) || (sc->nic.ic == C_LANCE)) {
@@ -1538,7 +1804,7 @@ lnc_start(struct ifnet *ifp)
 			desc = sc->trans_ring + sc->next_to_send;
 			len = mbuf_to_buffer(head, desc->buff.data);
 			desc->md->md3 = 0;
-			desc->md->md2 = -max(len, ETHER_MIN_LEN);
+			desc->md->md2 = -max(len, ETHER_MIN_LEN - ETHER_CRC_LEN);
 			desc->md->md1 |= OWN | STP | ENP;
 			INC_MD_PTR(sc->next_to_send, sc->ntdre)
 		}
@@ -1554,8 +1820,8 @@ lnc_start(struct ifnet *ifp)
 		ifp->if_timer = 2;
 
 #if NBPFILTER > 0
-		if (sc->bpf)
-			bpf_mtap(sc->bpf, head);
+		if (sc->arpcom.ac_if.if_bpf)
+			bpf_mtap(&sc->arpcom.ac_if, head);
 #endif
 
 		if (sc->nic.mem_mode != DMA_MBUF)
@@ -1572,11 +1838,11 @@ lnc_start(struct ifnet *ifp)
 	LNCSTATS(trans_ring_full)
 }
 
-int
+static int
 lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 {
 
-	struct lnc_softc *sc = &lnc_softc[ifp->if_unit];
+	struct lnc_softc *sc = ifp->if_softc;
 	struct ifaddr  *ifa = (struct ifaddr *) data;
 	struct ifreq *ifr = (struct ifreq *) data;
 	int s, error = 0;
@@ -1586,17 +1852,16 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 	switch (command) {
 	case SIOCSIFADDR:
 		ifp->if_flags |= IFF_UP;
-		sc->kdc.kdc_state = DC_BUSY;
 
 		switch (ifa->ifa_addr->sa_family) {
 #ifdef INET
 		case AF_INET:
-			lnc_init(ifp->if_unit);
+			lnc_init(sc);
 			arp_ifinit((struct arpcom *)ifp, ifa);
 			break;
 #endif
 		default:
-			lnc_init(ifp->if_unit);
+			lnc_init(sc);
 			break;
 		}
 		break;
@@ -1611,11 +1876,11 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 		if (ifp->if_flags & IFF_PROMISC) {
 			if (!(sc->nic.mode & PROM)) {
 				sc->nic.mode |= PROM;
-				lnc_init(ifp->if_unit);
+				lnc_init(sc);
 			}
 		} else if (sc->nic.mode & PROM) {
 			sc->nic.mode &= ~PROM;
-			lnc_init(ifp->if_unit);
+			lnc_init(sc);
 		}
 		if ((ifp->if_flags & IFF_UP) == 0 &&
 		    (ifp->if_flags & IFF_RUNNING) != 0) {
@@ -1623,7 +1888,7 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 			 * If interface is marked down and it is running,
 			 * then stop it.
 			 */
-			lnc_stop(ifp->if_unit);
+			lnc_stop(sc);
 			ifp->if_flags &= ~IFF_RUNNING;
 		} else if ((ifp->if_flags & IFF_UP) != 0 &&
 			   (ifp->if_flags & IFF_RUNNING) == 0) {
@@ -1631,12 +1896,10 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 			 * If interface is marked up and it is stopped, then
 			 * start it.
 			 */
-			lnc_init(ifp->if_unit);
+			lnc_init(sc);
 		}
-		sc->kdc.kdc_state =
-			((ifp->if_flags & IFF_UP) ? DC_BUSY : DC_IDLE);
 		break;
-#ifdef notyet
+#ifdef LNC_MULTICAST
 	case SIOCADDMULTI:
 	case SIOCDELMULTI:
 		error = (command == SIOCADDMULTI) ?
@@ -1644,7 +1907,7 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 					ether_delmulti(ifr, &sc->arpcom);
 
 		if (error == ENETRESET) {
-			lnc_setladrf(ifp,sc);
+			lnc_setladrf(sc);
 			error = 0;
 		}
 		break;
@@ -1666,22 +1929,21 @@ lnc_ioctl(struct ifnet * ifp, int command, caddr_t data)
 	return error;
 }
 
-void
-lnc_watchdog(int unit)
+static void
+lnc_watchdog(struct ifnet *ifp)
 {
-	log(LOG_ERR, "lnc%d: Device timeout -- Resetting\n", unit);
-	++lnc_softc[unit].arpcom.ac_if.if_oerrors;
-	lnc_reset(unit);
+	log(LOG_ERR, "lnc%d: Device timeout -- Resetting\n", ifp->if_unit);
+	ifp->if_oerrors++;
+	lnc_reset(ifp->if_softc);
 }
 
 #ifdef DEBUG
-void
-lnc_dump_state(int unit)
+static void
+lnc_dump_state(struct lnc_softc *sc)
 {
-	struct lnc_softc *sc = &lnc_softc[unit];
 	int             i;
 
-	printf("\nDriver/NIC [%d] state dump\n", unit);
+	printf("\nDriver/NIC [%d] state dump\n", sc->arpcom.ac_if.if_unit);
 	printf("Memory access mode: %b\n", sc->nic.mem_mode, MEM_MODES);
 	printf("Host memory\n");
 	printf("-----------\n");
@@ -1721,12 +1983,13 @@ lnc_dump_state(int unit)
 		    ((sc->trans_ring + i)->md->md1 >> 8), TRANS_MD1,
 		    ((sc->trans_ring + i)->md->md3 >> 10), TRANS_MD3);
 	printf("\nnext_to_send = %x\n", sc->next_to_send);
-	printf("\n CSR0 = %b CSR1 = %x CSR2 = %x CSR3 = %x\n\n", read_csr(unit, CSR0), CSR0_FLAGS, read_csr(unit, CSR1), read_csr(unit, CSR2), read_csr(unit, CSR3));
+	printf("\n CSR0 = %b CSR1 = %x CSR2 = %x CSR3 = %x\n\n", read_csr(sc, CSR0), CSR0_FLAGS, read_csr(sc, CSR1), read_csr(sc, CSR2), read_csr(sc, CSR3));
+
 	/* Set RAP back to CSR0 */
 	outw(sc->rap, CSR0);
 }
 
-void
+static void
 mbuf_dump_chain(struct mbuf * m)
 {
 
