@@ -47,15 +47,27 @@
  */
 
 /*
- * $FreeBSD: stable/2.1/sys/i386/isa/if_ze.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/if_ze.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+/* XXX - Don't mix different PCCARD support code */
+#include "card.h"
+#include "pcic.h"
+#if NCARD > 0 || NPCIC > 0
+#ifndef LINT_PCCARD_HACK
+#error "Dedicated PCMCIA drivers and generic PCMCIA support can't be mixed"
+#else
+#warning "Dedicated PCMCIA drivers and generic PCMCIA support can't be mixed"
+#endif
+#endif
+
 #include "ze.h"
 #if	NZE > 0
 #include "bpfilter.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/errno.h>
 #include <sys/ioctl.h>
 #include <sys/mbuf.h>
@@ -75,6 +87,11 @@
 #include <netinet/if_ether.h>
 #endif
 
+#ifdef IPX
+#include <netipx/ipx.h>
+#include <netipx/ipx_if.h>
+#endif
+
 #ifdef NS
 #include <netns/ns.h>
 #include <netns/ns_if.h>
@@ -86,11 +103,12 @@
 #endif
 
 #include <machine/clock.h>
+#include <machine/md_var.h>
 
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/icu.h>
-#include <i386/isa/if_zereg.h>
+#include <i386/isa/if_edreg.h>
 #include <i386/isa/pcic.h>
 
 #include "apm.h"
@@ -105,13 +123,13 @@
 /*
  * ze_softc: per line info and status
  */
-struct	ze_softc {
+static struct	ze_softc {
+
+	struct	arpcom arpcom;	/* ethernet common */
 
 	caddr_t  maddr;
 	u_long  iobase, irq;
 
-	struct	arpcom arpcom;	/* ethernet common */
-
 	char	*type_str;	/* pointer to type string */
 	char    *mau;		/* type of media access unit */
 	u_short	nic_addr;	/* NIC (DS8390) I/O bus address */
@@ -121,8 +139,6 @@ struct	ze_softc {
 	u_long	smem_size;	/* total shared memory size */
 	caddr_t	smem_ring;	/* start of RX ring-buffer (in smem) */
 
-	caddr_t	bpf;		/* BPF "magic cookie" */
-
 	u_char	memwidth;	/* width of access to card mem 8 or 16 */
 	u_char	xmit_busy;	/* transmitter is busy */
 	u_char	txb_cnt;	/* Number of transmit buffers */
@@ -143,14 +159,24 @@ struct	ze_softc {
 #endif /* NAPM > 0 */
 } ze_softc[NZE];
 
-int	ze_attach(), ze_ioctl(), ze_probe();
-void	ze_init(), ze_start(), ze_stop(), ze_intr();
-void	ze_reset(), ze_watchdog(), ze_get_packet();
-
-void 	ze_setup __P((struct ze_softc *sc));
-static inline void ze_rint();
-static inline void ze_xmit();
-static inline char *ze_ring_copy();
+static int ze_check_cis __P((unsigned char *scratch));
+static int ze_find_adapter __P((unsigned char *scratch, int reconfig));
+static int ze_probe __P((struct isa_device *isa_dev));
+static void ze_setup __P((struct ze_softc *sc));
+static int ze_suspend __P((void *visa_dev));
+static int ze_resume __P((void *visa_dev));
+static int ze_attach __P((struct isa_device *isa_dev));
+static void ze_reset __P((int unit));
+static void ze_stop __P((int unit));
+static void ze_watchdog __P((struct ifnet *ifp));
+static void ze_init __P((int unit));
+static __inline void ze_xmit __P((struct ifnet *ifp));
+static void ze_start __P((struct ifnet *ifp));
+static __inline void ze_rint __P((int unit));
+static int ze_ioctl __P((struct ifnet *ifp, int command, caddr_t data));
+static void ze_get_packet __P((struct ze_softc *sc, char *buf, int len));
+static __inline char *ze_ring_copy __P((struct ze_softc *sc, char *src, char *dst, int amount));
+static struct mbuf *ze_ring_to_mbuf __P((struct ze_softc *sc, char *src, struct mbuf *dst, int total_len));
 
 struct isa_driver zedriver = {
 	ze_probe,
@@ -158,11 +184,6 @@ struct isa_driver zedriver = {
 	"ze"
 };
 
-#define	ETHER_MIN_LEN	64
-#define ETHER_MAX_LEN	1518
-#define	ETHER_ADDR_LEN	6
-#define	ETHER_HDR_SIZE	14
-
 static unsigned char enet_addr[6];
 static unsigned char card_info[256];
 
@@ -209,11 +230,11 @@ ze_check_cis (unsigned char *scratch)
 		card_info[k++] = scratch[j] == '\0' ? '~' : scratch[j];
 	    card_info[k++] = '\0';
 #if 0
-	    return (memcmp (card_info, CARD_INFO, sizeof(CARD_INFO)-1) == 0);
+	    return (bcmp (card_info, CARD_INFO, sizeof(CARD_INFO)-1) == 0);
 #else
-	    if ((memcmp (card_info, CARD_INFO, sizeof(CARD_INFO)-1) == 0) ||
-		(memcmp (card_info, CARD2_INFO, sizeof(CARD2_INFO)-1) == 0) ||
-		(memcmp (card_info, CARD3_INFO, sizeof(CARD3_INFO)-1) == 0)) {
+	    if ((bcmp (card_info, CARD_INFO, sizeof(CARD_INFO)-1) == 0) ||
+		(bcmp (card_info, CARD2_INFO, sizeof(CARD2_INFO)-1) == 0) ||
+		(bcmp (card_info, CARD3_INFO, sizeof(CARD3_INFO)-1) == 0)) {
 		return 1;
 	    }
 	    return 0;
@@ -329,18 +350,18 @@ ze_find_adapter (unsigned char *scratch, int reconfig)
  *	or # of i/o addresses used (if found)
 	pcic(
  */
-int
+static int
 ze_probe(isa_dev)
 	struct isa_device *isa_dev;
 {
 	struct ze_softc *sc = &ze_softc[isa_dev->id_unit];
-	int i, x, re_init_flag;
+	int i;
 	u_int memsize;
-	u_char iptr, memwidth, sum, tmp;
+	u_char tmp;
 	int slot;
 
         if ((slot = ze_find_adapter (isa_dev->id_maddr, isa_dev->id_reconfig)) < 0)
-	    return NULL;
+	    return 0;
 
 	/*
 	 * okay, we found a card, so set it up
@@ -396,12 +417,12 @@ ze_probe(isa_dev)
 	sc->memwidth = 16;
 
 	/* allocate 1 xmit buffer */
-	sc->smem_ring = sc->smem_start + (ZE_PAGE_SIZE * ZE_TXBUF_SIZE);
+	sc->smem_ring = sc->smem_start + (ED_PAGE_SIZE * ED_TXBUF_SIZE);
 	sc->txb_cnt = 1;
-	sc->rec_page_start = ZE_TXBUF_SIZE + ZE_PAGE_OFFSET;
+	sc->rec_page_start = ED_TXBUF_SIZE + ZE_PAGE_OFFSET;
 	sc->smem_size = memsize;
 	sc->smem_end = sc->smem_start + memsize;
-	sc->rec_page_stop = memsize / ZE_PAGE_SIZE + ZE_PAGE_OFFSET;
+	sc->rec_page_stop = memsize / ED_PAGE_SIZE + ZE_PAGE_OFFSET;
 	sc->tx_page_start = ZE_PAGE_OFFSET;
 
 	/* get station address */
@@ -419,7 +440,7 @@ ze_probe(isa_dev)
 }
 
 
-void
+static void
 ze_setup(struct ze_softc *sc)
 {
 	int re_init_flag = 0,tmp,slot = sc->slot;
@@ -532,9 +553,10 @@ re_init:
 
 #if NAPM > 0
 static int
-ze_suspend(isa_dev)
-	struct isa_device *isa_dev;
+ze_suspend(visa_dev)
+	void *visa_dev;
 {
+	struct isa_device *isa_dev = visa_dev;
 	struct ze_softc *sc = &ze_softc[isa_dev->id_unit];
 
 	pcic_power_off(sc->slot);
@@ -542,9 +564,11 @@ ze_suspend(isa_dev)
 }
 
 static int
-ze_resume(isa_dev)
-	struct isa_device *isa_dev;
+ze_resume(visa_dev)
+	void *visa_dev;
 {
+	struct isa_device *isa_dev = visa_dev;
+
 #if 0
 	printf("Resume ze:\n");
 #endif
@@ -558,14 +582,12 @@ ze_resume(isa_dev)
  * Install interface into kernel networking data structures
  */
 
-int
+static int
 ze_attach(isa_dev)
 	struct isa_device *isa_dev;
 {
 	struct ze_softc *sc = &ze_softc[isa_dev->id_unit];
 	struct ifnet *ifp = &sc->arpcom.ac_if;
-	struct ifaddr *ifa;
-	struct sockaddr_dl *sdl;
 	int pl;
 
 	/* PCMCIA card can be offlined. Reconfiguration is required */
@@ -600,69 +622,37 @@ ze_attach(isa_dev)
 	/*
 	 * Initialize ifnet structure
 	 */
+	ifp->if_softc = sc;
 	ifp->if_unit = isa_dev->id_unit;
 	ifp->if_name = "ze" ;
 	ifp->if_mtu = ETHERMTU;
-	ifp->if_init = ze_init;
 	ifp->if_output = ether_output;
 	ifp->if_start = ze_start;
 	ifp->if_ioctl = ze_ioctl;
-	ifp->if_reset = ze_reset;
 	ifp->if_watchdog = ze_watchdog;
 
-	/*
-	 * Set default state for IIF_LINK0 flag (used to disable the tranceiver
-	 *	for AUI operation), based on compile-time config option.
-	 */
-	if (isa_dev->id_flags & ZE_FLAGS_DISABLE_TRANCEIVER)
-		ifp->if_flags = (IFF_BROADCAST | IFF_SIMPLEX | IFF_NOTRAILERS
-			| IFF_LINK0);
-	else
-		ifp->if_flags = (IFF_BROADCAST | IFF_SIMPLEX | IFF_NOTRAILERS);
+	ifp->if_flags = (IFF_BROADCAST | IFF_SIMPLEX);
 
 	/*
 	 * Attach the interface
 	 */
 	if_attach(ifp);
-
-	/*
-	 * Search down the ifa address list looking for the AF_LINK type entry
-	 */
- 	ifa = ifp->if_addrlist;
-	while ((ifa != 0) && (ifa->ifa_addr != 0) &&
-	    (ifa->ifa_addr->sa_family != AF_LINK))
-		ifa = ifa->ifa_next;
-	/*
-	 * If we find an AF_LINK type entry we fill in the hardware address.
-	 *	This is useful for netstat(1) to keep track of which interface
-	 *	is which.
-	 */
-	if ((ifa != 0) && (ifa->ifa_addr != 0)) {
-		/*
-		 * Fill in the link-level address for this interface
-		 */
-		sdl = (struct sockaddr_dl *)ifa->ifa_addr;
-		sdl->sdl_type = IFT_ETHER;
-		sdl->sdl_alen = ETHER_ADDR_LEN;
-		sdl->sdl_slen = 0;
-		bcopy(sc->arpcom.ac_enaddr, LLADDR(sdl), ETHER_ADDR_LEN);
-	}
+	ether_ifattach(ifp);
 
 	/*
 	 * Print additional info when attached
 	 */
-	printf("ze%d: address %s, type %s (%dbit)%s, MAU %s\n",
+	printf("ze%d: address %6D, type %s (%dbit), MAU %s\n",
 	       isa_dev->id_unit,
-	       ether_sprintf(sc->arpcom.ac_enaddr), sc->type_str,
+	       sc->arpcom.ac_enaddr, ":", sc->type_str,
 	       sc->memwidth,
-	       (ifp->if_flags & IFF_LINK0 ? " [tranceiver disabled]" : ""),
 	       sc->mau);
 
 	/*
 	 * If BPF is in the kernel, call the attach for it
 	 */
 #if NBPFILTER > 0
-	bpfattach(&sc->bpf, ifp, DLT_EN10MB, sizeof(struct ether_header));
+	bpfattach(ifp, DLT_EN10MB, sizeof(struct ether_header));
 #endif
 
 #if NAPM > 0
@@ -684,7 +674,7 @@ ze_attach(isa_dev)
 /*
  * Reset interface.
  */
-void
+static void
 ze_reset(unit)
 	int unit;
 {
@@ -704,7 +694,7 @@ ze_reset(unit)
 /*
  * Take interface offline.
  */
-void
+static void
 ze_stop(unit)
 	int unit;
 {
@@ -714,14 +704,14 @@ ze_stop(unit)
 	/*
 	 * Stop everything on the interface, and select page 0 registers.
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STP);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STP);
 
 	/*
 	 * Wait for interface to enter stopped state, but limit # of checks
 	 *	to 'n' (about 5ms). It shouldn't even take 5us on modern
 	 *	DS8390's, but just in case it's an old one.
 	 */
-	while (((inb(sc->nic_addr + ZE_P0_ISR) & ZE_ISR_RST) == 0) && --n);
+	while (((inb(sc->nic_addr + ED_P0_ISR) & ED_ISR_RST) == 0) && --n);
 	pcic_power_off(sc->slot);
 
 }
@@ -730,53 +720,52 @@ ze_stop(unit)
  * Device timeout/watchdog routine. Entered if the device neglects to
  *	generate an interrupt after a transmit has been started on it.
  */
-void
-ze_watchdog(unit)
-	int unit;
+static void
+ze_watchdog(ifp)
+	struct ifnet *ifp;
 {
 #if 1
-    struct ze_softc *sc = &ze_softc[unit];
+    struct ze_softc *sc = (struct ze_softc *)ifp;
     u_char isr, imr;
     u_short imask;
 
-    if(!(sc->arpcom.ac_if.if_flags & IFF_UP))
+    if(!(ifp->if_flags & IFF_UP))
 	return;
     /* select page zero */
-    outb (sc->nic_addr + ZE_P0_CR,
-	  (inb (sc->nic_addr + ZE_P0_CR) & 0x3f) | ZE_CR_PAGE_0);
+    outb (sc->nic_addr + ED_P0_CR,
+	  (inb (sc->nic_addr + ED_P0_CR) & 0x3f) | ED_CR_PAGE_0);
 
     /* read interrupt status register */
-    isr = inb (sc->nic_addr + ZE_P0_ISR) & 0xff;
+    isr = inb (sc->nic_addr + ED_P0_ISR) & 0xff;
 
     /* select page two */
-    outb (sc->nic_addr + ZE_P0_CR,
-	  (inb (sc->nic_addr + ZE_P0_CR) & 0x3f) | ZE_CR_PAGE_2);
+    outb (sc->nic_addr + ED_P0_CR,
+	  (inb (sc->nic_addr + ED_P0_CR) & 0x3f) | ED_CR_PAGE_2);
 
     /* read interrupt mask register */
-    imr = inb (sc->nic_addr + ZE_P2_IMR) & 0xff;
+    imr = inb (sc->nic_addr + ED_P2_IMR) & 0xff;
 
     imask = inb(IO_ICU2) << 8 | inb(IO_ICU1);
 
     log (LOG_ERR, "ze%d: device timeout, isr=%02x, imr=%02x, imask=%04x\n",
-	 unit, isr, imr, imask);
+	 ifp->if_unit, isr, imr, imask);
 #else
-    log(LOG_ERR, "ze%d: device timeout\n", unit);
+    log(LOG_ERR, "ze%d: device timeout\n", ifp->if_unit);
 #endif
 
-    ze_reset(unit);
+    ze_reset(ifp->if_unit);
 }
 
 /*
  * Initialize device.
  */
-void
+static void
 ze_init(unit)
 	int unit;
 {
 	struct ze_softc *sc = &ze_softc[unit];
 	struct ifnet *ifp = &sc->arpcom.ac_if;
 	int i, s;
-	u_char	command;
 
 
 	pcic_power_on(sc->slot);
@@ -807,54 +796,54 @@ ze_init(unit)
 	/*
 	 * Set interface for page 0, Remote DMA complete, Stopped
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STP);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STP);
 
 	if (sc->memwidth == 16) {
 		/*
 		 * Set FIFO threshold to 8, No auto-init Remote DMA,
 		 *	byte order=80x86, word-wide DMA xfers
 		 */
-		outb(sc->nic_addr + ZE_P0_DCR, ZE_DCR_FT1|ZE_DCR_WTS);
+		outb(sc->nic_addr + ED_P0_DCR, ED_DCR_FT1|ED_DCR_WTS);
 	} else {
 		/*
 		 * Same as above, but byte-wide DMA xfers
 		 */
-		outb(sc->nic_addr + ZE_P0_DCR, ZE_DCR_FT1);
+		outb(sc->nic_addr + ED_P0_DCR, ED_DCR_FT1);
 	}
 
 	/*
 	 * Clear Remote Byte Count Registers
 	 */
-	outb(sc->nic_addr + ZE_P0_RBCR0, 0);
-	outb(sc->nic_addr + ZE_P0_RBCR1, 0);
+	outb(sc->nic_addr + ED_P0_RBCR0, 0);
+	outb(sc->nic_addr + ED_P0_RBCR1, 0);
 
 	/*
 	 * Enable reception of broadcast packets
 	 */
-	outb(sc->nic_addr + ZE_P0_RCR, ZE_RCR_AB);
+	outb(sc->nic_addr + ED_P0_RCR, ED_RCR_AB);
 
 	/*
 	 * Place NIC in internal loopback mode
 	 */
-	outb(sc->nic_addr + ZE_P0_TCR, ZE_TCR_LB0);
+	outb(sc->nic_addr + ED_P0_TCR, ED_TCR_LB0);
 
 	/*
 	 * Initialize transmit/receive (ring-buffer) Page Start
 	 */
-	outb(sc->nic_addr + ZE_P0_TPSR, sc->tx_page_start);
-	outb(sc->nic_addr + ZE_P0_PSTART, sc->rec_page_start);
+	outb(sc->nic_addr + ED_P0_TPSR, sc->tx_page_start);
+	outb(sc->nic_addr + ED_P0_PSTART, sc->rec_page_start);
 
 	/*
 	 * Initialize Receiver (ring-buffer) Page Stop and Boundry
 	 */
-	outb(sc->nic_addr + ZE_P0_PSTOP, sc->rec_page_stop);
-	outb(sc->nic_addr + ZE_P0_BNRY, sc->rec_page_start);
+	outb(sc->nic_addr + ED_P0_PSTOP, sc->rec_page_stop);
+	outb(sc->nic_addr + ED_P0_BNRY, sc->rec_page_start);
 
 	/*
 	 * Clear all interrupts. A '1' in each bit position clears the
 	 *	corresponding flag.
 	 */
-	outb(sc->nic_addr + ZE_P0_ISR, 0xff);
+	outb(sc->nic_addr + ED_P0_ISR, 0xff);
 
 	/*
 	 * Enable the following interrupts: receive/transmit complete,
@@ -862,19 +851,19 @@ ze_init(unit)
 	 *
 	 * Counter overflow and Remote DMA complete are *not* enabled.
 	 */
-	outb(sc->nic_addr + ZE_P0_IMR,
-		ZE_IMR_PRXE|ZE_IMR_PTXE|ZE_IMR_RXEE|ZE_IMR_TXEE|ZE_IMR_OVWE);
+	outb(sc->nic_addr + ED_P0_IMR,
+		ED_IMR_PRXE|ED_IMR_PTXE|ED_IMR_RXEE|ED_IMR_TXEE|ED_IMR_OVWE);
 
 	/*
 	 * Program Command Register for page 1
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_PAGE_1|ZE_CR_RD2|ZE_CR_STP);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_PAGE_1|ED_CR_RD2|ED_CR_STP);
 
 	/*
 	 * Copy out our station address
 	 */
 	for (i = 0; i < ETHER_ADDR_LEN; ++i)
-		outb(sc->nic_addr + ZE_P1_PAR0 + i, sc->arpcom.ac_enaddr[i]);
+		outb(sc->nic_addr + ED_P1_PAR0 + i, sc->arpcom.ac_enaddr[i]);
 
 #if NBPFILTER > 0
 	/*
@@ -882,24 +871,24 @@ ze_init(unit)
 	 *	 all multicasts (only used when in promiscuous mode)
 	 */
 	for (i = 0; i < 8; ++i)
-		outb(sc->nic_addr + ZE_P1_MAR0 + i, 0xff);
+		outb(sc->nic_addr + ED_P1_MAR0 + i, 0xff);
 #endif
 
 	/*
 	 * Set Current Page pointer to next_packet (initialized above)
 	 */
-	outb(sc->nic_addr + ZE_P1_CURR, sc->next_packet);
+	outb(sc->nic_addr + ED_P1_CURR, sc->next_packet);
 
 	/*
 	 * Set Command Register for page 0, Remote DMA complete,
 	 * 	and interface Start.
 	 */
-	outb(sc->nic_addr + ZE_P1_CR, ZE_CR_RD2|ZE_CR_STA);
+	outb(sc->nic_addr + ED_P1_CR, ED_CR_RD2|ED_CR_STA);
 
 	/*
 	 * Take interface out of loopback
 	 */
-	outb(sc->nic_addr + ZE_P0_TCR, 0);
+	outb(sc->nic_addr + ED_P0_TCR, 0);
 
 	/*
 	 * Set 'running' flag, and clear output active flag.
@@ -918,34 +907,34 @@ ze_init(unit)
 /*
  * This routine actually starts the transmission on the interface
  */
-static inline void
+static __inline void
 ze_xmit(ifp)
 	struct ifnet *ifp;
 {
-	struct ze_softc *sc = &ze_softc[ifp->if_unit];
+	struct ze_softc *sc = ifp->if_softc;
 	u_short len = sc->txb_next_len;
 
 	/*
 	 * Set NIC for page 0 register access
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STA);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STA);
 
 	/*
 	 * Set TX buffer start page
 	 */
-	outb(sc->nic_addr + ZE_P0_TPSR, sc->tx_page_start +
-		sc->txb_next * ZE_TXBUF_SIZE);
+	outb(sc->nic_addr + ED_P0_TPSR, sc->tx_page_start +
+		sc->txb_next * ED_TXBUF_SIZE);
 
 	/*
 	 * Set TX length
 	 */
-	outb(sc->nic_addr + ZE_P0_TBCR0, len & 0xff);
-	outb(sc->nic_addr + ZE_P0_TBCR1, len >> 8);
+	outb(sc->nic_addr + ED_P0_TBCR0, len & 0xff);
+	outb(sc->nic_addr + ED_P0_TBCR1, len >> 8);
 
 	/*
 	 * Set page 0, Remote DMA complete, Transmit Packet, and *Start*
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_TXP|ZE_CR_STA);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_TXP|ED_CR_STA);
 
 	sc->xmit_busy = 1;
 	sc->data_buffered = 0;
@@ -973,15 +962,14 @@ ze_xmit(ifp)
  *  2) that the IFF_OACTIVE flag is checked before this code is called
  *     (i.e. that the output part of the interface is idle)
  */
-void
+static void
 ze_start(ifp)
 	struct ifnet *ifp;
 {
-	struct ze_softc *sc = &ze_softc[ifp->if_unit];
+	struct ze_softc *sc = ifp->if_softc;
 	struct mbuf *m0, *m;
 	caddr_t buffer;
 	int len;
-	u_char laar_tmp;
 
 outloop:
 	/*
@@ -1027,7 +1015,7 @@ outloop:
 	 * Copy the mbuf chain into the transmit buffer
 	 */
 
-	buffer = sc->smem_start + (sc->txb_next * ZE_TXBUF_SIZE * ZE_PAGE_SIZE);
+	buffer = sc->smem_start + (sc->txb_next * ED_TXBUF_SIZE * ED_PAGE_SIZE);
 	len = 0;
 	for (m0 = m; m != 0; m = m->m_next) {
 		bcopy(mtod(m, caddr_t), buffer, m->m_len);
@@ -1049,8 +1037,8 @@ outloop:
 	 * If there is BPF support in the configuration, tap off here.
 	 */
 #if NBPFILTER > 0
-	if (sc->bpf) {
-		bpf_mtap(sc->bpf, m0);
+	if (ifp->if_bpf) {
+		bpf_mtap(ifp, m0);
 	}
 #endif
 
@@ -1071,19 +1059,19 @@ outloop:
 /*
  * Ethernet interface receiver interrupt.
  */
-static inline void /* only called from one place, so may as well integrate */
+static __inline void /* only called from one place, so may as well integrate */
 ze_rint(unit)
 	int unit;
 {
 	register struct ze_softc *sc = &ze_softc[unit];
-	u_char boundry, current;
+	u_char boundry;
 	u_short len;
-	struct ze_ring *packet_ptr;
+	struct ed_ring *packet_ptr;
 
 	/*
 	 * Set NIC to page 1 registers to get 'current' pointer
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_PAGE_1|ZE_CR_RD2|ZE_CR_STA);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_PAGE_1|ED_CR_RD2|ED_CR_STA);
 
 	/*
 	 * 'sc->next_packet' is the logical beginning of the ring-buffer - i.e.
@@ -1093,11 +1081,11 @@ ze_rint(unit)
 	 *	We loop here until the logical beginning equals the logical
 	 *	end (or in other words, until the ring-buffer is empty).
 	 */
-	while (sc->next_packet != inb(sc->nic_addr + ZE_P1_CURR)) {
+	while (sc->next_packet != inb(sc->nic_addr + ED_P1_CURR)) {
 
 		/* get pointer to this buffer header structure */
-		packet_ptr = (struct ze_ring *)(sc->smem_ring +
-			 (sc->next_packet - sc->rec_page_start) * ZE_PAGE_SIZE);
+		packet_ptr = (struct ed_ring *)(sc->smem_ring +
+			 (sc->next_packet - sc->rec_page_start) * ED_PAGE_SIZE);
 
 		/*
 		 * The byte count includes the FCS - Frame Check Sequence (a
@@ -1141,15 +1129,15 @@ ze_rint(unit)
 		/*
 		 * Set NIC to page 0 registers to update boundry register
 		 */
-		outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STA);
+		outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STA);
 
-		outb(sc->nic_addr + ZE_P0_BNRY, boundry);
+		outb(sc->nic_addr + ED_P0_BNRY, boundry);
 
 		/*
 		 * Set NIC to page 1 registers before looping to top (prepare to
 		 *	get 'CURR' current pointer)
 		 */
-		outb(sc->nic_addr + ZE_P0_CR, ZE_CR_PAGE_1|ZE_CR_RD2|ZE_CR_STA);
+		outb(sc->nic_addr + ED_P0_CR, ED_CR_PAGE_1|ED_CR_RD2|ED_CR_STA);
 	}
 }
 
@@ -1168,19 +1156,19 @@ zeintr(unit)
 	/*
 	 * Set NIC to page 0 registers
 	 */
-	outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STA);
+	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STA);
 
 	/*
 	 * loop until there are no more new interrupts
 	 */
-	while (isr = inb(sc->nic_addr + ZE_P0_ISR)) {
+	while (isr = inb(sc->nic_addr + ED_P0_ISR)) {
 
 		/*
 		 * reset all the bits that we are 'acknowleging'
 		 *	by writing a '1' to each bit position that was set
 		 * (writing a '1' *clears* the bit)
 		 */
-		outb(sc->nic_addr + ZE_P0_ISR, isr);
+		outb(sc->nic_addr + ED_P0_ISR, isr);
 
 		/*
 		 * Transmit error. If a TX completed with an error, we end up
@@ -1190,14 +1178,14 @@ zeintr(unit)
 		 *	the flow. Of course, with UDP we're screwed, but this is
 		 *	expected when a network is heavily loaded.
 		 */
-		if (isr & ZE_ISR_TXE) {
-			u_char tsr = inb(sc->nic_addr + ZE_P0_TSR);
-			u_char ncr = inb(sc->nic_addr + ZE_P0_NCR);
+		if (isr & ED_ISR_TXE) {
+			u_char tsr = inb(sc->nic_addr + ED_P0_TSR);
+			u_char ncr = inb(sc->nic_addr + ED_P0_NCR);
 
 			/*
 			 * Excessive collisions (16)
 			 */
-			if ((tsr & ZE_TSR_ABT) && (ncr == 0)) {
+			if ((tsr & ED_TSR_ABT) && (ncr == 0)) {
 				/*
 				 *    When collisions total 16, the P0_NCR will
 				 * indicate 0, and the TSR_ABT is set.
@@ -1228,11 +1216,11 @@ zeintr(unit)
 		 * Receiver Error. One or more of: CRC error, frame alignment error
 		 *	FIFO overrun, or missed packet.
 		 */
-		if (isr & ZE_ISR_RXE) {
+		if (isr & ED_ISR_RXE) {
 			++sc->arpcom.ac_if.if_ierrors;
 #ifdef ZE_DEBUG
 			printf("ze%d: receive error %b\n", unit,
-				inb(sc->nic_addr + ZE_P0_RSR),
+				inb(sc->nic_addr + ED_P0_RSR),
 			       "\20\8DEF\7REC DISAB\6PHY/MC\5MISSED\4OVR\3ALIGN\2FCS\1RCVD");
 #endif
 		}
@@ -1246,7 +1234,7 @@ zeintr(unit)
 		 *	seen only with early rev chips - Methinks this
 		 *	bug was fixed in later revs. -DG
 		 */
-		if (isr & ZE_ISR_OVW) {
+		if (isr & ED_ISR_OVW) {
 			++sc->arpcom.ac_if.if_ierrors;
 			/*
 			 * Stop/reset/re-init NIC
@@ -1257,7 +1245,7 @@ zeintr(unit)
 		/*
 		 * Transmission completed normally.
 		 */
-		if (isr & ZE_ISR_PTX) {
+		if (isr & ED_ISR_PTX) {
 
 			/*
 			 * reset tx busy and output active flags
@@ -1281,7 +1269,7 @@ zeintr(unit)
 			 *	transmission.
 			 */
 			sc->arpcom.ac_if.if_collisions += inb(sc->nic_addr +
-				ZE_P0_TBCR0);
+				ED_P0_TBCR0);
 		}
 
 		/*
@@ -1290,7 +1278,7 @@ zeintr(unit)
 		 *	   shouldn't be any data to get (we've configured the
 		 *	   interface to not accept packets with errors).
 		 */
-		if (isr & (ZE_ISR_PRX|ZE_ISR_RXE)) {
+		if (isr & (ED_ISR_PRX|ED_ISR_RXE)) {
 			ze_rint (unit);
 		}
 
@@ -1311,7 +1299,7 @@ zeintr(unit)
 		 *	in the transmit routine, is *okay* - it is 'edge'
 		 *	triggered from low to high)
 		 */
-		outb(sc->nic_addr + ZE_P0_CR, ZE_CR_RD2|ZE_CR_STA);
+		outb(sc->nic_addr + ED_P0_CR, ED_CR_RD2|ED_CR_STA);
 
 		/*
 		 * If the Network Talley Counters overflow, read them to
@@ -1319,10 +1307,10 @@ zeintr(unit)
 		 *	clear the ISR flag otherwise - resulting in an
 		 *	infinite loop.
 		 */
-		if (isr & ZE_ISR_CNT) {
-			(void) inb(sc->nic_addr + ZE_P0_CNTR0);
-			(void) inb(sc->nic_addr + ZE_P0_CNTR1);
-			(void) inb(sc->nic_addr + ZE_P0_CNTR2);
+		if (isr & ED_ISR_CNT) {
+			(void) inb(sc->nic_addr + ED_P0_CNTR0);
+			(void) inb(sc->nic_addr + ED_P0_CNTR1);
+			(void) inb(sc->nic_addr + ED_P0_CNTR2);
 		}
 	}
 }
@@ -1331,15 +1319,14 @@ zeintr(unit)
  * Process an ioctl request. This code needs some work - it looks
  *	pretty ugly.
  */
-int
+static int
 ze_ioctl(ifp, command, data)
 	register struct ifnet *ifp;
 	int command;
 	caddr_t data;
 {
 	register struct ifaddr *ifa = (struct ifaddr *)data;
-	struct ze_softc *sc = &ze_softc[ifp->if_unit];
-	struct ifreq *ifr = (struct ifreq *)data;
+	struct ze_softc *sc = ifp->if_softc;
 	int s, error = 0;
 
 	s = splnet();
@@ -1356,6 +1343,32 @@ ze_ioctl(ifp, command, data)
 			arp_ifinit((struct arpcom*) ifp, ifa);
 			break;
 #endif
+#ifdef IPX
+		/*
+		 * XXX - This code is probably wrong
+		 */
+		case AF_IPX:
+		    {
+			register struct ipx_addr *ina = &(IA_SIPX(ifa)->sipx_addr);
+
+			if (ipx_nullhost(*ina))
+				ina->x_host =
+					*(union ipx_host *)(sc->arpcom.ac_enaddr);
+			else {
+				/* 
+				 * 
+				 */
+				bcopy((caddr_t)ina->x_host.c_host,
+				    (caddr_t)sc->arpcom.ac_enaddr,
+					sizeof(sc->arpcom.ac_enaddr));
+			}
+			/*
+			 * Set new address
+			 */
+			ze_init(ifp->if_unit);
+			break;
+		    }
+#endif
 #ifdef NS
 		/*
 		 * XXX - This code is probably wrong
@@ -1424,15 +1437,15 @@ ze_ioctl(ifp, command, data)
 			 *		hashing array. For now we assume that
 			 *		this was done in ze_init().
 			 */
-			outb(sc->nic_addr + ZE_P0_RCR,
-				ZE_RCR_PRO|ZE_RCR_AM|ZE_RCR_AB);
+			outb(sc->nic_addr + ED_P0_RCR,
+				ED_RCR_PRO|ED_RCR_AM|ED_RCR_AB);
 		} else {
 			/*
 			 * XXX - for multicasts to work, we would need to
 			 *	rewrite the multicast hashing array with the
 			 *	proper hash (would have been destroyed above).
 			 */
-			outb(sc->nic_addr + ZE_P0_RCR, ZE_RCR_AB);
+			outb(sc->nic_addr + ED_P0_RCR, ED_RCR_AB);
 		}
 #endif
 		break;
@@ -1458,21 +1471,14 @@ ze_ioctl(ifp, command, data)
  * Retreive packet from shared memory and send to the next level up via
  *	ether_input(). If there is a BPF listener, give a copy to BPF, too.
  */
-void
+static void
 ze_get_packet(sc, buf, len)
 	struct ze_softc *sc;
 	char *buf;
 	u_short len;
 {
 	struct ether_header *eh;
-    	struct mbuf *m, *head = NULL, *ze_ring_to_mbuf();
-	u_short off;
-	int resid;
-	u_short etype;
-	struct trailer_header {
-		u_short	trail_type;
-		u_short trail_residual;
-	} trailer_header;
+    	struct mbuf *m, *head = NULL;
 
 	/* Allocate a header mbuf */
 	MGETHDR(m, M_DONTWAIT, MT_DATA);
@@ -1511,8 +1517,8 @@ ze_get_packet(sc, buf, len)
 	 * Check if there's a BPF listener on this interface.
 	 * If so, hand off the raw packet to bpf.
 	 */
-	if (sc->bpf) {
-		bpf_mtap(sc->bpf, head);
+	if (sc->arpcom.ac_if.if_bpf) {
+		bpf_mtap(&sc->arpcom.ac_if, head);
 
 		/*
 		 * Note that the interface cannot be in promiscuous mode if
@@ -1555,7 +1561,7 @@ bad:	if (head)
  *	the ring buffer into a linear destination buffer. Takes into account
  *	ring-wrap.
  */
-static inline char *
+static __inline char *
 ze_ring_copy(sc,src,dst,amount)
 	struct ze_softc *sc;
 	char	*src;
@@ -1587,7 +1593,7 @@ ze_ring_copy(sc,src,dst,amount)
  * dst = pointer to last mbuf in mbuf chain to copy to
  * amount = amount of data to copy
  */
-struct mbuf *
+static struct mbuf *
 ze_ring_to_mbuf(sc,src,dst,total_len)
 	struct ze_softc *sc;
 	char *src;
