@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)isa.c	7.2 (Berkeley) 5/13/91
- * $FreeBSD: stable/2.1/sys/i386/isa/isa.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/isa.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -47,26 +47,22 @@
  * isa_dmastart()
  */
 
+#include "opt_auto_eoi.h"
+
 #include <sys/param.h>
-#include <sys/systm.h>		/* isn't it a joy */
-#include <sys/kernel.h>		/* to have three of these */
-#include <sys/proc.h>
-#include <sys/conf.h>
-#include <sys/file.h>
+#include <sys/systm.h>
 #include <sys/buf.h>
-#include <sys/uio.h>
 #include <sys/syslog.h>
 #include <sys/malloc.h>
-#include <sys/rlist.h>
+#include <machine/md_var.h>
 #include <machine/segments.h>
 #include <vm/vm.h>
-#include <machine/spl.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/isa.h>
 #include <i386/isa/icu.h>
 #include <i386/isa/ic/i8237.h>
-#include <i386/isa/ic/i8042.h>
-#include <sys/devconf.h>
 #include "vector.h"
 
 /*
@@ -85,33 +81,12 @@
 #define	DMA2_MODE	(IO_DMA2 + 2*11)	/* mode register */
 #define	DMA2_FFC	(IO_DMA2 + 2*12)	/* clear first/last FF */
 
-/*
- * XXX these defines should be in a central place.
- */
-#define	read_eflags()		({u_long ef; \
-				  __asm("pushfl; popl %0" : "=a" (ef)); \
-				  ef; })
-#define	write_eflags(ef)	__asm("pushl %0; popfl" : : "a" ((u_long)(ef)))
-
 u_long	*intr_countp[ICU_LEN];
 inthand2_t *intr_handler[ICU_LEN];
 u_int	intr_mask[ICU_LEN];
 u_int*	intr_mptr[ICU_LEN];
 int	intr_unit[ICU_LEN];
 
-extern struct kern_devconf kdc_cpu0;
-
-struct kern_devconf kdc_isa0 = {
-	0, 0, 0,		/* filled in by dev_attach */
-	"isa", 0, { MDDT_BUS, 0 },
-	0, 0, 0, BUS_EXTERNALLEN,
-	&kdc_cpu0,		/* parent is the CPU */
-	0,			/* no parentdata */
-	DC_BUSY,		/* busses are always busy */
-	"ISA bus",
-	DC_CLS_BUS		/* class */
-};
-
 static inthand_t *fastintr[ICU_LEN] = {
 	&IDTVEC(fastintr0), &IDTVEC(fastintr1),
 	&IDTVEC(fastintr2), &IDTVEC(fastintr3),
@@ -138,6 +113,7 @@ static void conflict __P((struct isa_device *dvp, struct isa_device *tmpdvp,
 			  char const *format));
 static int haveseen __P((struct isa_device *dvp, struct isa_device *tmpdvp,
 			 u_int checkbits));
+static int isa_dmarangecheck __P((caddr_t va, u_int length, int chan));
 static inthand2_t isa_strayintr;
 static void register_imask __P((struct isa_device *dvp, u_int mask));
 
@@ -240,7 +216,6 @@ haveseen(dvp, tmpdvp, checkbits)
  * Search through all the isa_devtab_* tables looking for anything that
  * conflicts with the current device.
  */
-
 int
 haveseen_isadev(dvp, checkbits)
 	struct isa_device *dvp;
@@ -259,6 +234,11 @@ haveseen_isadev(dvp, checkbits)
 		if (status)
 			return status;
 	}
+	for (tmpdvp = isa_devtab_cam; tmpdvp->id_driver; tmpdvp++) {
+		status |= haveseen(dvp, tmpdvp, checkbits);
+		if (status)
+			return status;
+	}
 	for (tmpdvp = isa_devtab_net; tmpdvp->id_driver; tmpdvp++) {
 		status |= haveseen(dvp, tmpdvp, checkbits);
 		if (status)
@@ -279,8 +259,6 @@ void
 isa_configure() {
 	struct isa_device *dvp;
 
-	dev_attach(&kdc_isa0);
-
 	splhigh();
 	printf("Probing for devices on the ISA bus:\n");
 	/* First probe all the sensitive probes */
@@ -290,6 +268,9 @@ isa_configure() {
 	for (dvp = isa_devtab_bio; dvp->id_driver; dvp++)
 		if (dvp->id_driver->sensitive_hw)
 			config_isadev(dvp, &bio_imask);
+	for (dvp = isa_devtab_cam; dvp->id_driver; dvp++)
+		if (dvp->id_driver->sensitive_hw)
+			config_isadev(dvp, &cam_imask);
 	for (dvp = isa_devtab_net; dvp->id_driver; dvp++)
 		if (dvp->id_driver->sensitive_hw)
 			config_isadev(dvp, &net_imask);
@@ -304,6 +285,9 @@ isa_configure() {
 	for (dvp = isa_devtab_bio; dvp->id_driver; dvp++)
 		if (!dvp->id_driver->sensitive_hw)
 			config_isadev(dvp, &bio_imask);
+	for (dvp = isa_devtab_cam; dvp->id_driver; dvp++)
+		if (!dvp->id_driver->sensitive_hw)
+			config_isadev(dvp, &cam_imask);
 	for (dvp = isa_devtab_net; dvp->id_driver; dvp++)
 		if (!dvp->id_driver->sensitive_hw)
 			config_isadev(dvp, &net_imask);
@@ -327,17 +311,17 @@ isa_configure() {
  * during spltty.
  */
 #include "sl.h"
-#include "ppp.h"
-
-#if (NSL > 0) || (NPPP > 0)
+#if NSL > 0
 	net_imask |= tty_imask;
 	tty_imask = net_imask;
 #endif
+
 	/* bio_imask |= tty_imask ;  can some tty devices use buffers? */
-#ifdef DIAGNOSTIC
-	printf("bio_imask %x tty_imask %x net_imask %x\n",
-	       bio_imask, tty_imask, net_imask);
-#endif
+
+	if (bootverbose)
+		printf("imasks: bio %x, tty %x, net %x\n",
+		       bio_imask, tty_imask, net_imask);
+
 	/*
 	 * Finish initializing intr_mask[].  Note that the partly
 	 * constructed masks aren't actually used since we're at splhigh.
@@ -349,6 +333,8 @@ isa_configure() {
 		register_imask(dvp, tty_imask);
 	for (dvp = isa_devtab_bio; dvp->id_driver; dvp++)
 		register_imask(dvp, bio_imask);
+	for (dvp = isa_devtab_cam; dvp->id_driver; dvp++)
+		register_imask(dvp, cam_imask);
 	for (dvp = isa_devtab_net; dvp->id_driver; dvp++)
 		register_imask(dvp, net_imask);
 	for (dvp = isa_devtab_null; dvp->id_driver; dvp++)
@@ -388,19 +374,17 @@ config_isadev_c(isdp, mp, reconfig)
 	int last_alive;
 	struct isa_driver *dp = isdp->id_driver;
 
- 	checkbits = 0;
-	checkbits |= CC_DRQ;
-	checkbits |= CC_IOADDR;
-	checkbits |= CC_MEMADDR;
 	if (!isdp->id_enabled) {
-		printf("%s%d: disabled, not probed.\n",
-			dp->name, isdp->id_unit);
+		if (bootverbose)
+			printf("%s%d: disabled, not probed.\n",
+				dp->name, isdp->id_unit);
 		return;
 	}
+	checkbits = CC_DRQ | CC_IOADDR | CC_MEMADDR;
 	if (!reconfig && haveseen_isadev(isdp, checkbits))
 		return;
 	if (!reconfig && isdp->id_maddr) {
-		isdp->id_maddr -= 0xa0000; /* XXX should be a define */
+		isdp->id_maddr -= ISA_HOLE_START;
 		isdp->id_maddr += atdevbase;
 	}
 	if (reconfig) {
@@ -423,10 +407,10 @@ config_isadev_c(isdp, mp, reconfig)
 		if (!isdp->id_reconfig) {
 			printf("%s%d", dp->name, isdp->id_unit);
 			if (id_alive != -1) {
- 				printf(" at 0x%x", isdp->id_iobase);
- 				if ((isdp->id_iobase + id_alive - 1) !=
- 				     isdp->id_iobase) {
- 					printf("-0x%x",
+				printf(" at 0x%x", isdp->id_iobase);
+				if (isdp->id_iobase + id_alive - 1 !=
+				    isdp->id_iobase) {
+					printf("-0x%x",
 					       isdp->id_iobase + id_alive - 1);
 				}
 			}
@@ -457,7 +441,7 @@ config_isadev_c(isdp, mp, reconfig)
 			 * *dvp to avoid conflicts if given a chance.  We
 			 * already skip the early check for IRQs and force
 			 * a check for IRQs in the next group of checks.
-		 	 */
+			 */
 			checkbits |= CC_IRQ;
 			if (haveseen_isadev(isdp, checkbits))
 				return;
@@ -478,7 +462,8 @@ config_isadev_c(isdp, mp, reconfig)
 		}
 		if (!last_alive) {
 			if (!isdp->id_reconfig) {
-				printf("%s%d not found", dp->name, isdp->id_unit);
+				printf("%s%d not found",
+				       dp->name, isdp->id_unit);
 				if (isdp->id_iobase) {
 					printf(" at 0x%x", isdp->id_iobase);
 				}
@@ -498,55 +483,6 @@ config_isadev_c(isdp, mp, reconfig)
 	}
 }
 
-/*
- * Provide ISA-specific device information to user programs using the
- * hw.devconf interface.
- */
-int
-isa_externalize(struct isa_device *id, void *userp, size_t *maxlen)
-{
-	if(*maxlen < sizeof *id) {
-		return ENOMEM;
-	}
-
-	*maxlen -= sizeof *id;
-	return copyout(id, userp, sizeof *id);
-}
-
-/*
- * This is used to forcibly reconfigure an ISA device.  It currently just
- * returns an error 'cos you can't do that yet.  It is here to demonstrate
- * what the `internalize' routine is supposed to do.
- */
-int
-isa_internalize(struct isa_device *id, void **userpp, size_t *len)
-{
-	struct isa_device myid;
-	char *userp = *userpp;
-	int rv;
-
-	if(*len < sizeof *id) {
-		return EINVAL;
-	}
-
-	rv = copyin(userp, &myid, sizeof myid);
-	if(rv) return rv;
-	*userpp = userp + sizeof myid;
-	*len -= sizeof myid;
-
-	rv = EOPNOTSUPP;
-	/* code would go here to validate the configuration request */
-	/* code would go here to actually perform the reconfiguration */
-	return rv;
-}
-
-int
-isa_generic_externalize(struct proc *p, struct kern_devconf *kdc,
-			void *userp, size_t l)
-{
-	return isa_externalize(kdc->kdc_isa, userp, &l);
-}
-
 /*
  * Fill in default interrupt table (in case of spuruious interrupt
  * during configuration of kernel, setup interrupt control unit
@@ -585,24 +521,119 @@ isa_defaultirq()
 	outb(IO_ICU2, 0x0a);		/* default to IRR on read */
 }
 
-/* region of physical memory known to be contiguous */
-vm_offset_t isaphysmem;
-static caddr_t dma_bounce[8];		/* XXX */
-static char bounced[8];		/* XXX */
-#define MAXDMASZ 512		/* XXX */
+static caddr_t	dma_bouncebuf[8];
+static u_int	dma_bouncebufsize[8];
+static u_int8_t	dma_bounced = 0;
+static u_int8_t	dma_busy = 0;		/* Used in isa_dmastart() */
+static u_int8_t	dma_inuse = 0;		/* User for acquire/release */
+static u_int8_t        dma_auto_mode = 0;
+
+#define VALID_DMA_MASK (7)
 
 /* high byte of address is stored in this port for i-th dma channel */
-static short dmapageport[8] =
-	{ 0x87, 0x83, 0x81, 0x82, 0x8f, 0x8b, 0x89, 0x8a };
+static int dmapageport[8] = { 0x87, 0x83, 0x81, 0x82, 0x8f, 0x8b, 0x89, 0x8a };
+
+/*
+ * Setup a DMA channel's bounce buffer.
+ */
+void
+isa_dmainit(chan, bouncebufsize)
+	int chan;
+	u_int bouncebufsize;
+{
+	void *buf;
+
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dmainit: channel out of range");
+
+	if (dma_bouncebuf[chan] != NULL)
+		panic("isa_dmainit: impossible request"); 
+#endif
+
+	dma_bouncebufsize[chan] = bouncebufsize;
+
+	/* Try malloc() first.  It works better if it works. */
+	buf = malloc(bouncebufsize, M_DEVBUF, M_NOWAIT);
+	if (buf != NULL) {
+		if (isa_dmarangecheck(buf, bouncebufsize, chan) == 0) {
+			dma_bouncebuf[chan] = buf;
+			return;
+		}
+		free(buf, M_DEVBUF);
+	}
+	buf = contigmalloc(bouncebufsize, M_DEVBUF, M_NOWAIT, 0ul, 0xfffffful,
+			   1ul, chan & 4 ? 0x20000ul : 0x10000ul);
+	if (buf == NULL)
+		printf("isa_dmainit(%d, %d) failed\n", chan, bouncebufsize);
+	else
+		dma_bouncebuf[chan] = buf;
+}
+
+/*
+ * Register a DMA channel's usage.  Usually called from a device driver
+ * in open() or during it's initialization.
+ */
+int
+isa_dma_acquire(chan)
+	int chan;
+{
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dma_acquire: channel out of range");
+#endif
+
+	if (dma_inuse & (1 << chan)) {
+		printf("isa_dma_acquire: channel %d already in use\n", chan);
+		return (EBUSY);
+	}
+	dma_inuse |= (1 << chan);
+	dma_auto_mode &= ~(1 << chan);
+
+	return (0);
+}
+
+/*
+ * Unregister a DMA channel's usage.  Usually called from a device driver
+ * during close() or during it's shutdown.
+ */
+void
+isa_dma_release(chan)
+	int chan;
+{
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dma_release: channel out of range");
+
+	if ((dma_inuse & (1 << chan)) == 0)
+		printf("isa_dma_release: channel %d not in use\n", chan);
+#endif
+
+	if (dma_busy & (1 << chan)) {
+		dma_busy &= ~(1 << chan);
+		/* 
+		 * XXX We should also do "dma_bounced &= (1 << chan);"
+		 * because we are acting on behalf of isa_dmadone() which
+		 * was not called to end the last DMA operation.  This does
+		 * not matter now, but it may in the future.
+		 */
+	}
+
+	dma_inuse &= ~(1 << chan);
+	dma_auto_mode &= ~(1 << chan);
+}
 
 /*
  * isa_dmacascade(): program 8237 DMA controller channel to accept
  * external dma control by a board.
  */
-void isa_dmacascade(unsigned chan)
+void isa_dmacascade(chan)
+	int chan;
 {
-	if (chan > 7)
-		panic("isa_dmacascade: impossible request");
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dmacascade: channel out of range");
+#endif
 
 	/* set dma channel mode, and set dma channel mode */
 	if ((chan & 4) == 0) {
@@ -614,31 +645,122 @@ void isa_dmacascade(unsigned chan)
 	}
 }
 
-static int
-isa_dmarangecheck(caddr_t va, unsigned length, unsigned chan);
+/*
+ * Query the progress of a transfer on a DMA channel.
+ *
+ * To avoid having to interrupt a transfer in progress, we sample
+ * each of the high and low databytes twice, and apply the following
+ * logic to determine the correct count.
+ *
+ * Reads are performed with interrupts disabled, thus it is to be
+ * expected that the time between reads is very small.  At most
+ * one rollover in the low count byte can be expected within the
+ * four reads that are performed.
+ *
+ * There are three gaps in which a rollover can occur :
+ *
+ * - read low1 
+ *              gap1
+ * - read high1
+ *              gap2
+ * - read low2
+ *              gap3
+ * - read high2
+ *
+ * If a rollover occurs in gap1 or gap2, the low2 value will be
+ * greater than the low1 value.  In this case, low2 and high2 are a
+ * corresponding pair.
+ *     
+ * In any other case, low1 and high1 can be considered to be correct.
+ *             
+ * The function returns the number of bytes remaining in the transfer,
+ * or -1 if the channel requested is not active.
+ *
+ */    
+int
+isa_dmastatus(int chan) 
+{      
+         u_long  cnt = 0;
+        int     ffport, waport, s;
+         u_long  low, high, low2, high2;
+ 
+         /* channel active? */
+         if ((dma_inuse & (1 << chan)) == 0) {
+                 printf("isa_dmastatus: channel %d not in use\n", chan);
+                 return(-1);
+         }
+  
+         /* 
+         * do not print an error message if the chan is not busy,
+         * it might just be a race condition.
+         */
+         if ( !(dma_busy & (1 << chan)) && !(dma_auto_mode & (1<<chan)) )
+                 return(0); 
+  
+         if (chan < 4) {                 /* low DMA controller */
+                 ffport = DMA1_FFC; 
+                 waport = DMA1_CHN(chan) + 1;
+         } else {                        /* high DMA controller */
+                 ffport = DMA2_FFC;
+                 waport = DMA2_CHN(chan - 4) + 2;
+         }       
+  
+         disable_intr();                  /* no interrupts Mr Jones! */
+         outb(ffport, 0);                /* clear register LSB flipflop */
+         low = inb(waport);
+         high = inb(waport);
+         outb(ffport, 0);                /* clear again (paranoia? */
+         low2 = inb(waport);
+         high2 = inb(waport);
+         enable_intr();
+                
+         /* now decide if a wrap has tried to skew our results */
+         if (low >= low2)
+                 cnt = low + (high << 8) ;
+         else
+                 cnt = low2 + (high2 << 8) ;
+ 
+        cnt = (cnt + 1) & 0xffff ;
+  
+        if (chan >=4)   /* 16-bit chans transfer words */
+                cnt *= 2 ;
+         return(cnt);
+}
+
 
 /*
  * isa_dmastart(): program 8237 DMA controller channel, avoid page alignment
  * problems by using a bounce buffer.
  */
-void isa_dmastart(int flags, caddr_t addr, unsigned nbytes, unsigned chan)
-{	vm_offset_t phys;
+void isa_dmastart(int flags, caddr_t addr, u_int nbytes, int chan)
+{
+	vm_offset_t phys;
 	int waport;
 	caddr_t newaddr;
 
-	if (    chan > 7
-	    || (chan < 4 && nbytes > (1<<16))
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dmastart: channel out of range");
+
+	if ((chan < 4 && nbytes > (1<<16))
 	    || (chan >= 4 && (nbytes > (1<<17) || (u_int)addr & 1)))
 		panic("isa_dmastart: impossible request");
+#endif
+	if ((dma_inuse & (1 << chan)) == 0)
+		printf("isa_dmastart: channel %d not acquired\n", chan);
+
+    if (!(flags & B_RAW))
+	if (dma_busy & (1 << chan))
+		printf("isa_dmastart: channel %d busy\n", chan);
+
+	dma_busy |= (1 << chan);
 
 	if (isa_dmarangecheck(addr, nbytes, chan)) {
-		if (dma_bounce[chan] == 0)
-			dma_bounce[chan] =
-				/*(caddr_t)malloc(MAXDMASZ, M_TEMP, M_WAITOK);*/
-				(caddr_t) isaphysmem + NBPG*chan;
-		bounced[chan] = 1;
-		newaddr = dma_bounce[chan];
-		*(int *) newaddr = 0;	/* XXX */
+		if (dma_bouncebuf[chan] == NULL
+		    || dma_bouncebufsize[chan] < nbytes)
+			panic("isa_dmastart: bad bounce buffer"); 
+		dma_bounced |= (1 << chan);
+		newaddr = dma_bouncebuf[chan];
 
 		/* copy bounce buffer on write */
 		if (!(flags & B_READ))
@@ -649,6 +771,12 @@ void isa_dmastart(int flags, caddr_t addr, unsigned nbytes, unsigned chan)
 	/* translate to physical */
 	phys = pmap_extract(pmap_kernel(), (vm_offset_t)addr);
 
+        if (flags & B_RAW) 
+                dma_auto_mode |= (1 << chan);
+        else
+                dma_auto_mode &= ~(1 << chan);
+
+
 	if ((chan & 4) == 0) {
 		/*
 		 * Program one of DMA channels 0..3.  These are
@@ -719,15 +847,46 @@ void isa_dmastart(int flags, caddr_t addr, unsigned nbytes, unsigned chan)
 	}
 }
 
-void isa_dmadone(int flags, caddr_t addr, int nbytes, int chan)
+/*
+ * this stops the dma channel and returns the residual count
+ * derived calling isa_dmastatus
+ */    
+int isa_dmastop(int chan)
 {
+     if ( !(dma_inuse & (1 << chan)) )
+        printf("isa_dmastop: channel %d not acquired\n", chan);
+     if ( ! (dma_busy & (1 << chan))  &&
+         ! (dma_auto_mode & (1 << chan)) )
+        printf("isa_dmastop: channel %d not active\n", chan);
+     if ( chan & 4 )
+        outb(DMA2_SMSK, (chan & 3) | 4 /* disable mask */);
+     else   
+        outb(DMA1_SMSK, (chan & 3) | 4 /* disable mask */);
+     return isa_dmastatus(chan);
+}      
 
-	/* copy bounce buffer on read */
-	/*if ((flags & (B_PHYS|B_READ)) == (B_PHYS|B_READ))*/
-	if (bounced[chan]) {
-		bcopy(dma_bounce[chan], addr, nbytes);
-		bounced[chan] = 0;
+void isa_dmadone(int flags, caddr_t addr, int nbytes, int chan)
+{  
+#ifdef DIAGNOSTIC
+	if (chan & ~VALID_DMA_MASK)
+		panic("isa_dmadone: channel out of range");
+
+	if ((dma_inuse & (1 << chan)) == 0)
+		printf("isa_dmadone: channel %d not acquired\n", chan);
+#endif
+
+        if ( ! (dma_busy & (1 << chan))  &&
+             ! (dma_auto_mode & (1 << chan)) )
+                printf("isa_dmadone: channel %d not active\n", chan);
+
+	if (dma_bounced & (1 << chan)) {
+		/* copy bounce buffer on read */
+		if (flags & B_READ)
+			bcopy(dma_bouncebuf[chan], addr, nbytes);
+
+		dma_bounced &= ~(1 << chan);
 	}
+	dma_busy &= ~(1 << chan);
 }
 
 /*
@@ -738,12 +897,12 @@ void isa_dmadone(int flags, caddr_t addr, int nbytes, int chan)
  */
 
 static int
-isa_dmarangecheck(caddr_t va, unsigned length, unsigned chan) {
+isa_dmarangecheck(caddr_t va, u_int length, int chan) {
 	vm_offset_t phys, priorpage = 0, endva;
 	u_int dma_pgmsk = (chan & 4) ?  ~(128*1024-1) : ~(64*1024-1);
 
 	endva = (vm_offset_t)round_page(va + length);
-	for (; va < (caddr_t) endva ; va += NBPG) {
+	for (; va < (caddr_t) endva ; va += PAGE_SIZE) {
 		phys = trunc_page(pmap_extract(pmap_kernel(), (vm_offset_t)va));
 #define ISARAM_END	RAM_END
 		if (phys == 0)
@@ -751,7 +910,7 @@ isa_dmarangecheck(caddr_t va, unsigned length, unsigned chan) {
 		if (phys >= ISARAM_END)
 			return (1);
 		if (priorpage) {
-			if (priorpage + NBPG != phys)
+			if (priorpage + PAGE_SIZE != phys)
 				return (1);
 			/* check if crossing a DMA page boundary */
 			if (((u_int)priorpage ^ (u_int)phys) & dma_pgmsk)
@@ -762,48 +921,6 @@ isa_dmarangecheck(caddr_t va, unsigned length, unsigned chan) {
 	return (0);
 }
 
-/* head of queue waiting for physmem to become available */
-struct buf isa_physmemq;
-
-/* blocked waiting for resource to become free for exclusive use */
-static isaphysmemflag;
-/* if waited for and call requested when free (B_CALL) */
-static void (*isaphysmemunblock)(); /* needs to be a list */
-
-/*
- * Allocate contiguous physical memory for transfer, returning
- * a *virtual* address to region. May block waiting for resource.
- * (assumed to be called at splbio())
- */
-caddr_t
-isa_allocphysmem(caddr_t va, unsigned length, void (*func)()) {
-
-	isaphysmemunblock = func;
-	while (isaphysmemflag & B_BUSY) {
-		isaphysmemflag |= B_WANTED;
-		tsleep((caddr_t)&isaphysmemflag, PRIBIO, "isaphys", 0);
-	}
-	isaphysmemflag |= B_BUSY;
-
-	return((caddr_t)isaphysmem);
-}
-
-/*
- * Free contiguous physical memory used for transfer.
- * (assumed to be called at splbio())
- */
-void
-isa_freephysmem(caddr_t va, unsigned length) {
-
-	isaphysmemflag &= ~B_BUSY;
-	if (isaphysmemflag & B_WANTED) {
-		isaphysmemflag &= B_WANTED;
-		wakeup((caddr_t)&isaphysmemflag);
-		if (isaphysmemunblock)
-			(*isaphysmemunblock)();
-	}
-}
-
 #define NMI_PARITY (1 << 7)
 #define NMI_IOCHAN (1 << 6)
 #define ENMI_WATCHDOG (1 << 7)
@@ -820,20 +937,32 @@ isa_nmi(cd)
 {
 	int isa_port = inb(0x61);
 	int eisa_port = inb(0x461);
-	if(isa_port & NMI_PARITY) {
+
+	if (isa_port & NMI_PARITY)
 		panic("RAM parity error, likely hardware failure.");
-	} else if(isa_port & NMI_IOCHAN) {
+
+	if (isa_port & NMI_IOCHAN)
 		panic("I/O channel check, likely hardware failure.");
-	} else if(eisa_port & ENMI_WATCHDOG) {
-		panic("EISA watchdog timer expired, likely hardware failure.");
-	} else if(eisa_port & ENMI_BUSTIMER) {
-		panic("EISA bus timeout, likely hardware failure.");
-	} else if(eisa_port & ENMI_IOSTATUS) {
-		panic("EISA I/O port status error.");
-	} else {
-		printf("\nNMI ISA %x, EISA %x\n", isa_port, eisa_port);
+
+	/*
+	 * On a real EISA machine, this will never happen.  However it can
+	 * happen on ISA machines which implement XT style floating point
+	 * error handling (very rare).  Save them from a meaningless panic.
+	 */
+	if (eisa_port == 0xff)
 		return(0);
-	}
+
+	if (eisa_port & ENMI_WATCHDOG)
+		panic("EISA watchdog timer expired, likely hardware failure.");
+
+	if (eisa_port & ENMI_BUSTIMER)
+		panic("EISA bus timeout, likely hardware failure.");
+
+	if (eisa_port & ENMI_IOSTATUS)
+		panic("EISA I/O port status error.");
+
+	printf("\nNMI ISA %x, EISA %x\n", isa_port, eisa_port);
+	return(0);
 }
 
 /*
@@ -868,6 +997,25 @@ isa_strayintr(d)
 		    "too many stray irq %d's; not logging any more\n", d);
 }
 
+/*
+ * Find the highest priority enabled display device.  Since we can't
+ * distinguish display devices from ttys, depend on display devices
+ * being sensitive and before sensitive non-display devices (if any)
+ * in isa_devtab_tty.
+ *
+ * XXX we should add capability flags IAMDISPLAY and ISUPPORTCONSOLES.
+ */
+struct isa_device *
+find_display()
+{
+	struct isa_device *dvp;
+
+	for (dvp = isa_devtab_tty; dvp->id_driver != NULL; dvp++)
+		if (dvp->id_driver->sensitive_hw && dvp->id_enabled)
+			return (dvp);
+	return (NULL);
+}
+
 /*
  * find an ISA device in a given isa_devtab_* table, given
  * the table to search, the expected id_driver entry, and the unit number.
@@ -897,18 +1045,18 @@ struct isa_device *find_isadev(table, driverp, unit)
 }
 
 /*
- * Return nonzero if a (masked) irq is pending for a given device.
+ * Return a bitmap of the current interrupt requests.  This is 8259-specific
+ * and is only suitable for use at probe time.
  */
-int
-isa_irq_pending(dvp)
-	struct isa_device *dvp;
+u_int
+isa_irq_pending()
 {
-	unsigned id_irq;
+	u_char irr1;
+	u_char irr2;
 
-	id_irq = dvp->id_irq;
-	if (id_irq & 0xff)
-		return (inb(IO_ICU1) & id_irq);
-	return (inb(IO_ICU2) & (id_irq >> 8));
+	irr1 = inb(IO_ICU1);
+	irr2 = inb(IO_ICU2);
+	return ((irr2 << 8) | irr1);
 }
 
 int
@@ -916,6 +1064,7 @@ update_intr_masks(void)
 {
 	int intr, n=0;
 	u_int mask,*maskptr;
+
 	for (intr=0; intr < ICU_LEN; intr ++) {
 		if (intr==2) continue;
 		maskptr = intr_mptr[intr];
