@@ -31,11 +31,11 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)isa_device.h	7.1 (Berkeley) 5/9/91
- * $FreeBSD: stable/2.1/sys/i386/isa/isa_device.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/isa_device.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _I386_ISA_ISA_DEVICE_H_
-#define _I386_ISA_ISA_DEVICE_H_ 1
+#define	_I386_ISA_ISA_DEVICE_H_
 
 /*
  * ISA Bus Autoconfiguration
@@ -116,13 +116,18 @@ struct isa_driver {
 extern char eintrnames[];	/* end of intrnames[] */
 extern u_long intrcnt[];	/* counts for for each device and stray */
 extern char intrnames[];	/* string table containing device names */
-extern u_long *intr_countp[];	/* indirectors into intrcnt[] */
+extern u_long *intr_countp[];	/* pointers into intrcnt[] */
 extern inthand2_t *intr_handler[];	/* C entry points of intr handlers */
 extern u_int intr_mask[];	/* sets of intrs masked during handling of 1 */
 extern int intr_unit[];		/* cookies to pass to intr handlers */
 
-extern struct isa_device isa_devtab_bio[], isa_devtab_tty[], isa_devtab_net[],
-		isa_devtab_null[], isa_biotab_wdc[], isa_biotab_fdc[];
+extern struct isa_device isa_biotab_fdc[];
+extern struct isa_device isa_biotab_wdc[];
+extern struct isa_device isa_devtab_bio[];
+extern struct isa_device isa_devtab_cam[];
+extern struct isa_device isa_devtab_net[];
+extern struct isa_device isa_devtab_null[];
+extern struct isa_device isa_devtab_tty[];
 
 inthand_t
 	IDTVEC(fastintr0), IDTVEC(fastintr1),
@@ -133,34 +138,35 @@ inthand_t
 	IDTVEC(fastintr10), IDTVEC(fastintr11),
 	IDTVEC(fastintr12), IDTVEC(fastintr13),
 	IDTVEC(fastintr14), IDTVEC(fastintr15);
-struct isa_device *find_isadev __P((struct isa_device *table,
-				    struct isa_driver *driverp, int unit));
 inthand_t
 	IDTVEC(intr0), IDTVEC(intr1), IDTVEC(intr2), IDTVEC(intr3),
 	IDTVEC(intr4), IDTVEC(intr5), IDTVEC(intr6), IDTVEC(intr7),
 	IDTVEC(intr8), IDTVEC(intr9), IDTVEC(intr10), IDTVEC(intr11),
 	IDTVEC(intr12), IDTVEC(intr13), IDTVEC(intr14), IDTVEC(intr15);
+struct isa_device *
+	find_display __P((void));
+struct isa_device *
+	find_isadev __P((struct isa_device *table, struct isa_driver *driverp,
+			 int unit));
+int	haveseen_isadev __P((struct isa_device *dvp, u_int checkbits));
+void	isa_configure __P((void));
+void	isa_defaultirq __P((void));
+void	isa_dmacascade __P((int chan));
+void	isa_dmadone __P((int flags, caddr_t addr, int nbytes, int chan));
+void	isa_dmainit __P((int chan, u_int bouncebufsize));
+void	isa_dmastart __P((int flags, caddr_t addr, u_int nbytes, int chan));
+int	isa_dma_acquire __P((int chan));
+void	isa_dma_release __P((int chan));
+int	isa_dmastatus __P((int chan));
+int	isa_dmastop __P((int chan));
+u_int	isa_irq_pending __P((void));
+int	isa_nmi __P((int cd));
+void	reconfig_isadev __P((struct isa_device *isdp, u_int *mp));
+int	register_intr __P((int intr, int device_id, u_int flags,
+			   inthand2_t *handler, u_int *maskptr, int unit));
+int	unregister_intr __P((int intr, inthand2_t *handler));
+int	update_intr_masks __P((void));
 
-void isa_configure __P((void));
-int haveseen_isadev __P((struct isa_device *dvp, u_int checkbits));
-void isa_defaultirq __P((void));
-void isa_dmacascade __P((unsigned chan));
-void isa_dmadone __P((int, caddr_t, int, int));
-void isa_dmastart __P((int, caddr_t, unsigned, unsigned));
-int isa_irq_pending __P((struct isa_device *dvp));
-int isa_nmi __P((int cd));
-void reconfig_isadev __P((struct isa_device *isdp, u_int *mp));
-int register_intr __P((int intr, int device_id, u_int flags,
-		       inthand2_t *handler, u_int *maskptr, int unit));
-int unregister_intr __P((int intr, inthand2_t *handler));
-int update_intr_masks __P((void));
-
-extern int isa_externalize(struct isa_device *, void *, size_t *);
-extern int isa_internalize(struct isa_device *, void **, size_t *);
-
-struct kern_devconf;
-extern int isa_generic_externalize(struct proc *, struct kern_devconf *, void *, size_t);
-extern struct kern_devconf kdc_isa0;
 #endif /* KERNEL */
 
-#endif /* _I386_ISA_ISA_DEVICE_H_ */
+#endif /* !_I386_ISA_ISA_DEVICE_H_ */
