@@ -33,7 +33,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/istallion.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/istallion.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*****************************************************************************/
@@ -51,7 +51,6 @@
 #include <sys/file.h>
 #include <sys/uio.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
 #include <machine/cpu.h>
 #include <machine/clock.h>
 #include <vm/vm.h>
@@ -68,7 +67,7 @@
  *	appropriate bits of code. By default this will compile for a 2.1
  *	level kernel.
  */
-#define	VFREEBSD	210
+#define	VFREEBSD	220
 
 #if VFREEBSD >= 220
 #define	STATIC		static
@@ -2631,7 +2630,7 @@ static char *stli_ecpgetmemptr(stlibrd_t *brdp, unsigned long offset, int line)
 		ptr = 0;
 		val = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % ECP_ATPAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % ECP_ATPAGESIZE);
 		val = (unsigned char) (offset / ECP_ATPAGESIZE);
 	}
 	outb((brdp->iobase + ECP_ATMEMPR), val);
@@ -2721,7 +2720,7 @@ static char *stli_ecpeigetmemptr(stlibrd_t *brdp, unsigned long offset, int line
 		ptr = 0;
 		val = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % ECP_EIPAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % ECP_EIPAGESIZE);
 		if (offset < ECP_EIPAGESIZE)
 			val = ECP_EIENABLE;
 		else
@@ -2773,7 +2772,7 @@ static char *stli_ecpmcgetmemptr(stlibrd_t *brdp, unsigned long offset, int line
 		ptr = 0;
 		val = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % ECP_MCPAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % ECP_MCPAGESIZE);
 		val = ((unsigned char) (offset / ECP_MCPAGESIZE)) | ECP_MCENABLE;
 	}
 	outb((brdp->iobase + ECP_MCCONFR), val);
@@ -2854,7 +2853,7 @@ static char *stli_onbgetmemptr(stlibrd_t *brdp, unsigned long offset, int line)
 			__LINE__, brdp->brdnr);
 		ptr = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % ONB_ATPAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % ONB_ATPAGESIZE);
 	}
 	return(ptr);
 }
@@ -2945,7 +2944,7 @@ static char *stli_onbegetmemptr(stlibrd_t *brdp, unsigned long offset, int line)
 		ptr = 0;
 		val = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % ONB_EIPAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % ONB_EIPAGESIZE);
 		if (offset < ONB_EIPAGESIZE)
 			val = ONB_EIENABLE;
 		else
@@ -3014,7 +3013,7 @@ static char *stli_bbygetmemptr(stlibrd_t *brdp, unsigned long offset, int line)
 		ptr = 0;
 		val = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % BBY_PAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % BBY_PAGESIZE);
 		val = (unsigned char) (offset / BBY_PAGESIZE);
 	}
 	outb((brdp->iobase + BBY_ATCONFR), val);
@@ -3074,7 +3073,7 @@ static char *stli_stalgetmemptr(stlibrd_t *brdp, unsigned long offset, int line)
 			__LINE__, brdp->brdnr);
 		ptr = 0;
 	} else {
-		ptr = brdp->vaddr + (offset % STAL_PAGESIZE);
+		ptr = (char *) brdp->vaddr + (offset % STAL_PAGESIZE);
 	}
 	return(ptr);
 }
@@ -3090,7 +3089,7 @@ static void stli_stalreset(stlibrd_t *brdp)
 	printf("stli_stalreset(brdp=%x)\n", (int) brdp);
 #endif
 
-	vecp = (volatile unsigned long *) (brdp->vaddr + 0x30);
+	vecp = (volatile unsigned long *) ((char *) brdp->vaddr + 0x30);
 	*vecp = 0xffff0000;
 	outb(brdp->iobase, 0);
 	for (i = 0; (i < 1000); i++)
