@@ -46,17 +46,24 @@
 #include <sys/time.h>
 
 #include <sys/systm.h>
-#include <sys/devconf.h>
 
 #include <sys/kernel.h>
 #include <sys/malloc.h>
 #include <sys/errno.h>
 #include <sys/buf.h>
+#define b_actf	b_act.tqe_next
 #include <sys/dataacq.h>
+#include <sys/conf.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
-#include <machine/devconf.h>
 #include <machine/clock.h>
 
+#include <i386/isa/isa_device.h>
+
+
 
 /* Miniumum timeout:
  */
@@ -139,7 +146,9 @@ struct ctlr
 
 	/* Device configuration structure:
 	 */
-	struct kern_devconf kdc;
+#ifdef DEVFS
+	void *devfs_token;
+#endif
 };
 
 #ifdef LOUTB
@@ -155,7 +164,7 @@ static inline void loutb(int port, u_char val)
 #define loutb(port, val) outb(port, val)
 #endif
 
-struct ctlr **labpcs;	/* XXX: Should be dynamic */
+static struct ctlr **labpcs;	/* XXX: Should be dynamic */
 
 /* CR_EXPR: A macro that sets the shadow register in addition to
  * sending out the data.
@@ -266,11 +275,21 @@ struct ctlr **labpcs;	/* XXX: Should be dynamic */
 
 #define DCR(LABPC) ((LABPC)->base + 0x13)
 
-extern int labpcattach(struct isa_device *dev);
-extern int labpcdetach(struct isa_device *dev);
-extern int labpcprobe(struct isa_device *dev);
+static int labpcattach(struct isa_device *dev);
+static int labpcprobe(struct isa_device *dev);
 struct isa_driver labpcdriver =
-	{ labpcprobe, labpcattach, "labpc", 0 /* , labpcdetach */ };
+	{ labpcprobe, labpcattach, "labpc", 0  };
+
+static	d_open_t	labpcopen;
+static	d_close_t	labpcclose;
+static	d_ioctl_t	labpcioctl;
+static	d_strategy_t	labpcstrategy;
+
+#define CDEV_MAJOR 66
+static struct cdevsw labpc_cdevsw = 
+	{ labpcopen,	labpcclose,	rawread,	rawwrite,	/*66*/
+	  labpcioctl,	nostop,		nullreset,	nodevtotty,/* labpc */
+	  seltrue,	nommap,		labpcstrategy, "labpc",	NULL,	-1 };
 
 static void start(struct ctlr *ctlr);
 
@@ -346,36 +365,6 @@ reset(struct ctlr *ctlr)
 	ad_clear(ctlr);
 }
 
-static int
-labpc_goaway(struct kern_devconf *kdc, int force)
-{
-	if(force) {
-		dev_detach(kdc);
-		return 0;
-	} else {
-		return EBUSY;   /* XXX fix */
-	}
-}
-
-static struct kern_devconf kdc_template = {
-      0, 0, 0,                /* filled in by dev_attach */
-      "labpc", 0, { MDDT_ISA, 0, "tty" },
-      isa_generic_externalize, 0, labpc_goaway, ISA_EXTERNALLEN,
-      &kdc_isa0,              /* parent */
-      0,                      /* parentdata */
-      DC_UNKNOWN,
-      "?"                     /* Description (filled in later ) */
-};
-
-static inline void
-labpc_registerdev(struct isa_device *id)
-{
-	struct kern_devconf *kdc = &labpcs[id->id_unit]->kdc;
-	kdc->kdc_unit = id->id_unit;
-	kdc->kdc_parentdata = id;
-	dev_attach(kdc);
-}
-
 /* overrun: slam the start convert register and OVERRUN should get set:
  */
 static u_char
@@ -408,7 +397,8 @@ labpcinit(void)
 	return 0;
 }
 
-int labpcprobe(struct isa_device *dev)
+static int
+labpcprobe(struct isa_device *dev)
 {
 	static unit;
 	struct ctlr scratch, *ctlr;
@@ -461,14 +451,6 @@ int labpcprobe(struct isa_device *dev)
 		l->base = ctlr->base;
 		dev->id_unit = l->unit = unit;
 
-		l->kdc = kdc_template;
-		l->kdc.kdc_state = DC_IDLE;
-
-		if ((status & LABPCPLUS) == 0)
-			l->kdc.kdc_description = "National Instrument's LabPC+";
-		else
-			l->kdc.kdc_description = "National Instrument's LabPC";
-
 		unit++;
 		return 0x20;
 	}
@@ -481,12 +463,13 @@ int labpcprobe(struct isa_device *dev)
 
 /* attach: Set things in a normal state.
  */
-int labpcattach(struct isa_device *dev)
+static int
+labpcattach(struct isa_device *dev)
 {
 	struct ctlr *ctlr = labpcs[dev->id_unit];
+
 	ctlr->sample_us = (1000000.0 / (double)LABPC_DEFAULT_HERZ) + .50;
 	reset(ctlr);
-    labpc_registerdev(dev);
 
 	ctlr->min_tmo = LABPC_MIN_TMO;
 
@@ -494,19 +477,16 @@ int labpcattach(struct isa_device *dev)
 	ctlr->dcr_is = 0x80;
 	loutb(DCR(ctlr), ctlr->dcr_val);
 
+#ifdef DEVFS
+	ctlr->devfs_token = 
+		devfs_add_devswf(&labpc_cdevsw, 0, DV_CHR, 
+                                 /* what  UID GID PERM */
+				 0, 0, 0600, 
+				 "labpc%d", dev->id_unit);
+#endif
 	return 1;
 }
 
-int
-labpcdetach(struct isa_device *id)
-{
-	struct ctlr *ctlr = labpcs[id->id_unit];
-	CR_CLR(ctlr, 3);
-	reset(ctlr);
-	dev_detach(&ctlr->kdc);
-	return 0;
-}
-
 /* Null handlers:
  */
 static void null_intr (struct ctlr *ctlr)             { }
@@ -723,7 +703,7 @@ lockout_multiple_open(dev_t current, dev_t next)
 	return ! (DIGITAL(current) && DIGITAL(next));
 }
 
-int
+static	int
 labpcopen(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	u_short unit = UNIT(dev);
@@ -744,7 +724,6 @@ labpcopen(dev_t dev, int flags, int fmt, struct proc *p)
 	if ( (ctlr->flags & BUSY) == 0)
 	{
 		ctlr->flags |= BUSY;
-		ctlr->kdc.kdc_state = DC_BUSY;
 
 		reset(ctlr);
 
@@ -761,14 +740,13 @@ labpcopen(dev_t dev, int flags, int fmt, struct proc *p)
 	return 0;
 }
 
-int
+static	int
 labpcclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct ctlr *ctlr = labpcs[UNIT(dev)];
 
 	(*ctlr->stop)(ctlr);
 
-	ctlr->kdc.kdc_state = DC_IDLE;
 	ctlr->flags &= ~BUSY;
 
 	return 0;
@@ -789,7 +767,7 @@ start(struct ctlr *ctlr)
 		 */
 		CR_EXPR(ctlr, 3, &= ~(FIFOINTEN|ERRINTEN));
 		ctlr->cleared_intr = 1;
-		ctlr->start_queue.b_active = 0;
+		ctlr->start_queue.b_bcount = 0;
 		return;
 	}
 
@@ -833,14 +811,14 @@ ad_strategy(struct buf *bp, struct ctlr *ctlr)
 	s = spltty();
 	bp->b_actf = NULL;
 
-	if (ctlr->start_queue.b_active)
+	if (ctlr->start_queue.b_bcount)
 	{
 		ctlr->last->b_actf = bp;
 		ctlr->last = bp;
 	}
 	else
 	{
-		ctlr->start_queue.b_active = 1;
+		ctlr->start_queue.b_bcount = 1;
 		ctlr->start_queue.b_actf = bp;
 		ctlr->last = bp;
 		start(ctlr);
@@ -860,7 +838,8 @@ ad_strategy(struct buf *bp, struct ctlr *ctlr)
  *
  * 2. No interrupt support yet.
  */
-void da_strategy(struct buf *bp, struct ctlr *ctlr)
+static void
+da_strategy(struct buf *bp, struct ctlr *ctlr)
 {
 	int len;
 	u_char *data;
@@ -996,7 +975,7 @@ digital_in_strategy(struct buf *bp, struct ctlr *ctlr)
 }
 
 
-void
+static	void
 labpcstrategy(struct buf *bp)
 {
 	struct ctlr *ctlr = labpcs[UNIT(bp->b_dev)];
@@ -1034,7 +1013,7 @@ labpcstrategy(struct buf *bp)
 	}
 }
 
-int
+static	int
 labpcioctl(dev_t dev, int cmd, caddr_t arg, int mode, struct proc *p)
 {
 	struct ctlr *ctlr = labpcs[UNIT(dev)];
@@ -1094,3 +1073,21 @@ labpcioctl(dev_t dev, int cmd, caddr_t arg, int mode, struct proc *p)
 			return ENOTTY;
 	}
 }
+
+
+static labpc_devsw_installed = 0;
+
+static void 	labpc_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! labpc_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR,0);
+		cdevsw_add(&dev,&labpc_cdevsw,NULL);
+		labpc_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(labpcdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,labpc_drvinit,NULL)
+
+
