@@ -46,7 +46,7 @@
  * SUCH DAMAGE.
  *
  *	from: unknown origin, 386BSD 0.1
- * $FreeBSD: stable/2.1/sys/i386/isa/lpt.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/lpt.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -112,7 +112,6 @@
 #include <sys/ioctl.h>
 #include <sys/uio.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
 #ifdef DEVFS
 #include <sys/devfsext.h>
 #endif /*DEVFS*/
@@ -195,14 +194,14 @@
 #define lprintf (void)
 #else
 #define lprintf		if (lptflag) printf
-int lptflag = 1;
+static int volatile lptflag = 1;
 #endif
 
 #define	LPTUNIT(s)	((s)&0x03)
 #define	LPTFLAGS(s)	((s)&0xfc)
 
 static struct lpt_softc {
-	short	sc_port;
+	int	sc_port;
 	short	sc_state;
 	/* default case: negative prime, negative ack, handshake strobe,
 	   prime once */
@@ -228,9 +227,10 @@ static struct lpt_softc {
 	struct  ifnet	sc_if;
 	u_char		*sc_ifbuf;
 	int		sc_iferrs;
-#endif /* ENDIF */
-#ifdef	DEVFS
+#endif
+#ifdef DEVFS
 	void	*devfs_token;
+	void	*devfs_token_ctl;
 #endif
 } lpt_sc[NLPT] ;
 
@@ -288,46 +288,23 @@ struct	isa_driver lptdriver = {
 	lptprobe, lptattach, "lpt"
 };
 
-d_open_t	lptopen;
-d_close_t	lptclose;
-d_write_t	lptwrite;
-d_ioctl_t	lptioctl;
-void		lptintr(int unit);
+static	d_open_t	lptopen;
+static	d_close_t	lptclose;
+static	d_write_t	lptwrite;
+static	d_ioctl_t	lptioctl;
 
 #define CDEV_MAJOR 16
-#if 0
 static struct cdevsw lpt_cdevsw = 
 	{ lptopen,	lptclose,	noread,		lptwrite,	/*16*/
 	  lptioctl,	nullstop,	nullreset,	nodevtotty,/* lpt */
 	  seltrue,	nommap,		nostrat,	"lpt",	NULL,	-1 };
-#endif
 
-static struct kern_devconf kdc_lpt[NLPT] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"lpt", 0, { MDDT_ISA, 0, "tty" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"Parallel printer adapter",
-	DC_CLS_PARALLEL | DC_CLS_NETIF /* class */
-} };
-
-static inline void
-lpt_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_lpt[id->id_unit] = kdc_lpt[0];
-	kdc_lpt[id->id_unit].kdc_unit = id->id_unit;
-	kdc_lpt[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_lpt[id->id_unit]);
-}
 
 /*
  * Internal routine to lptprobe to do port tests of one byte value
  */
 static int
-lpt_port_test (short port, u_char data, u_char mask)
+lpt_port_test (int port, u_char data, u_char mask)
 {
 	int	temp, timeout;
 
@@ -391,15 +368,19 @@ lpt_port_test (short port, u_char data, u_char mask)
 int
 lptprobe(struct isa_device *dvp)
 {
-	short		port;
+	int		port;
 	static short	next_bios_lpt = 0;
 	int		status;
-	u_char		data;
-	u_char		mask;
+	static u_char	testbyte[18] = {
+		0x55,			/* alternating zeros */
+		0xaa,			/* alternating ones */
+		0xfe, 0xfd, 0xfb, 0xf7,
+		0xef, 0xdf, 0xbf, 0x7f,	/* walking zero */
+		0x01, 0x02, 0x04, 0x08,
+		0x10, 0x20, 0x40, 0x80	/* walking one */
+	};
 	int		i;
 
-	lpt_registerdev(dvp);
-
 	/*
 	 * Make sure there is some way for lptopen to see that
 	 * the port is not configured
@@ -420,27 +401,12 @@ lptprobe(struct isa_device *dvp)
 
 	/* Port was explicitly specified */
 	/* This allows probing of ports unknown to the BIOS */
-
 	port = dvp->id_iobase + lpt_data;
-	mask = 0xff;
-	data = 0x55;				/* Alternating zeros */
-	if (!lpt_port_test(port, data, mask))
-		{ status = 0 ; goto end_probe ; }
-
-	data = 0xaa;				/* Alternating ones */
-	if (!lpt_port_test(port, data, mask))
-		{ status = 0 ; goto end_probe ; }
-
-	for (i = 0; i < 8; i++)	{		/* Walking zero */
-		data = ~(1 << i);
-		if (!lpt_port_test(port, data, mask))
-			{ status = 0 ; goto end_probe ; }
-	}
-
-	for (i = 0; i < 8; i++)	{		/* Walking one */
-		data = (1 << i);
-		if (!lpt_port_test(port, data, mask))
-			{ status = 0 ; goto end_probe ; }
+	for (i = 0; i < 18; i++) {
+		if (!lpt_port_test(port, testbyte[i], 0xff)) {
+			status = 0;
+			goto end_probe;
+		}
 	}
 
 end_probe:
@@ -457,7 +423,6 @@ lptattach(struct isa_device *isdp)
 {
 	struct	lpt_softc	*sc;
 	int	unit;
-	char	name[32];
 
 	unit = isdp->id_unit;
 	sc = lpt_sc + unit;
@@ -479,14 +444,14 @@ lptattach(struct isa_device *isdp)
 	}
 	lprintf("irq %x\n", sc->sc_irq);
 
-	kdc_lpt[unit].kdc_state = DC_IDLE;
-
 #ifdef DEVFS
-/* XXX */ /* what to do about the flags in the minor number? */
-	sprintf(name,"lpt%d",unit);
-			/*	path name devsw minor type  uid gid perm*/
-	sc->devfs_token = devfs_add_devsw( "/", name, &lpt_cdevsw, unit,
-					DV_CHR, 0, 0, 0600);
+	/* XXX what to do about the flags in the minor number? */
+	sc->devfs_token = devfs_add_devswf(&lpt_cdevsw,
+		unit, DV_CHR,
+		UID_ROOT, GID_WHEEL, 0600, "lpt%d", unit);
+	sc->devfs_token_ctl = devfs_add_devswf(&lpt_cdevsw,
+		unit | LP_BYPASS, DV_CHR,
+		UID_ROOT, GID_WHEEL, 0600, "lpctl%d", unit);
 #endif
 	return (1);
 }
@@ -497,7 +462,7 @@ lptattach(struct isa_device *isdp)
  *	printer -- this is just used for passing ioctls.
  */
 
-int
+static	int
 lptopen (dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct lpt_softc *sc;
@@ -583,7 +548,6 @@ lptopen (dev_t dev, int flags, int fmt, struct proc *p)
 	outb(port+lpt_control, sc->sc_control);
 
 	sc->sc_state = OPEN;
-	kdc_lpt[unit].kdc_state = DC_BUSY;
 	sc->sc_inbuf = geteblk(BUFSIZE);
 	sc->sc_xfercnt = 0;
 	splx(s);
@@ -635,7 +599,7 @@ lptout (struct lpt_softc * sc)
  * Check for interrupted write call added.
  */
 
-int
+static	int
 lptclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct lpt_softc *sc = lpt_sc + LPTUNIT(minor(dev));
@@ -645,7 +609,6 @@ lptclose(dev_t dev, int flags, int fmt, struct proc *p)
 		goto end_close;
 
 	sc->sc_state &= ~OPEN;
-	kdc_lpt[minor(dev)].kdc_state = DC_IDLE;
 
 	/* if the last write was interrupted, don't complete it */
 	if((!(sc->sc_state  & INTERRUPTED)) && (sc->sc_irq & LP_USE_IRQ))
@@ -734,7 +697,7 @@ pushbytes(struct lpt_softc * sc)
  * Flagging of interrupted write added.
  */
 
-int
+static	int
 lptwrite(dev_t dev, struct uio * uio, int ioflag)
 {
 	register unsigned n;
@@ -844,7 +807,7 @@ lptintr(int unit)
 	lprintf("sts %x ", sts);
 }
 
-int
+static	int
 lptioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
 {
 	int	error = 0;
@@ -893,6 +856,7 @@ lpattach (struct lpt_softc *sc, int unit)
 {
 	struct ifnet *ifp = &sc->sc_if;
 
+	ifp->if_softc = sc;
 	ifp->if_name = "lp";
 	ifp->if_unit = unit;
 	ifp->if_mtu = LPMTU;
@@ -907,7 +871,7 @@ lpattach (struct lpt_softc *sc, int unit)
 	printf("lp%d: TCP/IP capable interface\n", unit);
 
 #if NBPFILTER > 0
-	bpfattach(&ifp->if_bpf, ifp, DLT_NULL, LPIPHDRLEN);
+	bpfattach(ifp, DLT_NULL, LPIPHDRLEN);
 #endif
 }
 /*
@@ -953,7 +917,6 @@ lpinittables (void)
  * Process an ioctl request.
  */
 
-/* ARGSUSED */
 static int
 lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
 {
@@ -1030,7 +993,7 @@ lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
     return 0;
 }
 
-static inline int
+static __inline int
 clpoutbyte (u_char byte, int spin, int data_port, int status_port)
 {
 	outb(data_port, ctxmitl[byte]);
@@ -1046,7 +1009,7 @@ clpoutbyte (u_char byte, int spin, int data_port, int status_port)
 	return 0;
 }
 
-static inline int
+static __inline int
 clpinbyte (int spin, int data_port, int status_port)
 {
 	int c, cl;
@@ -1176,7 +1139,7 @@ lpintr (int unit)
 	    }
 #if NBPFILTER > 0
 	    if (sc->sc_if.if_bpf) {
-		bpf_tap(sc->sc_if.if_bpf, sc->sc_ifbuf, len);
+		bpf_tap(&sc->sc_if, sc->sc_ifbuf, len);
 	    }
 #endif
 	    len -= LPIPHDRLEN;
@@ -1212,7 +1175,7 @@ lpintr (int unit)
 	return;
 }
 
-static inline int
+static __inline int
 lpoutbyte (u_char byte, int spin, int data_port, int status_port)
 {
     outb(data_port, txmith[byte]);
@@ -1378,7 +1341,7 @@ lpoutput (struct ifnet *ifp, struct mbuf *m,
 	    m0.m_len = 2;
 	    m0.m_data = (char *)&hdr;
 
-	    bpf_mtap(ifp->if_bpf, &m0);
+	    bpf_mtap(ifp, &m0);
 	}
 #endif
     }
@@ -1395,3 +1358,19 @@ lpoutput (struct ifnet *ifp, struct mbuf *m,
 }
 
 #endif /* INET */
+
+static lpt_devsw_installed = 0;
+
+static void 	lpt_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! lpt_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&lpt_cdevsw, NULL);
+		lpt_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(lptdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,lpt_drvinit,NULL)
+
