@@ -40,9 +40,9 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/mcd.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/mcd.c 50950 1999-09-05 08:39:22Z peter $
  */
-static char COPYRIGHT[] = "mcd-driver (C)1993 by H.Veit & B.Moore";
+static const char COPYRIGHT[] = "mcd-driver (C)1993 by H.Veit & B.Moore";
 
 #include "mcd.h"
 #if NMCD > 0
@@ -60,13 +60,16 @@ static char COPYRIGHT[] = "mcd-driver (C)1993 by H.Veit & B.Moore";
 #include <sys/errno.h>
 #include <sys/dkbad.h>
 #include <sys/disklabel.h>
-#include <sys/devconf.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
+#include <machine/spl.h>
 #include <machine/clock.h>
 
 #include <i386/i386/cons.h>
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/mcdreg.h>
 
@@ -85,16 +88,15 @@ static char COPYRIGHT[] = "mcd-driver (C)1993 by H.Veit & B.Moore";
 #define RAW_PART        2
 
 /* flags */
-#define MCDOPEN		0x0001	/* device opened */
-#define MCDVALID	0x0002	/* parameters loaded */
-#define MCDINIT		0x0004	/* device is init'd */
-#define MCDNEWMODEL     0x0008  /* device is new model */
-#define MCDLABEL	0x0010	/* label is read */
-#define	MCDPROBING	0x0020	/* probing */
-#define	MCDREADRAW	0x0040	/* read raw mode (2352 bytes) */
-#define	MCDVOLINFO	0x0080	/* already read volinfo */
-#define	MCDTOC		0x0100	/* already read toc */
-#define	MCDMBXBSY	0x0200	/* local mbx is busy */
+#define MCDVALID        0x0001  /* parameters loaded */
+#define MCDINIT         0x0002  /* device is init'd */
+#define MCDNEWMODEL     0x0004  /* device is new model */
+#define MCDLABEL        0x0008  /* label is read */
+#define MCDPROBING      0x0010  /* probing */
+#define MCDREADRAW      0x0020  /* read raw mode (2352 bytes) */
+#define MCDVOLINFO      0x0040  /* already read volinfo */
+#define MCDTOC          0x0080  /* already read toc */
+#define MCDMBXBSY       0x0100  /* local mbx is busy */
 
 /* status */
 #define	MCDAUDIOBSY	MCD_ST_AUDIOBSY		/* playing audio */
@@ -117,8 +119,9 @@ static char COPYRIGHT[] = "mcd-driver (C)1993 by H.Veit & B.Moore";
 #define	MCD_TYPE_UNKNOWN	0
 #define	MCD_TYPE_LU002S		1
 #define	MCD_TYPE_LU005S		2
-#define	MCD_TYPE_FX001		3
-#define	MCD_TYPE_FX001D		4
+#define MCD_TYPE_LU006S         3
+#define MCD_TYPE_FX001          4
+#define MCD_TYPE_FX001D         5
 
 struct mcd_mbx {
 	short		unit;
@@ -133,7 +136,7 @@ struct mcd_mbx {
 	short           mode;
 };
 
-struct mcd_data {
+static struct mcd_data {
 	short	type;
 	char	*name;
 	short	config;
@@ -152,8 +155,14 @@ struct mcd_data {
 	short   curr_mode;
 	struct mcd_read2 lastpb;
 	short	debug;
-	struct buf head;		/* head of buf queue */
+	struct buf_queue_head head;		/* head of buf queue */
 	struct mcd_mbx mbx;
+#ifdef	DEVFS
+	void *ra_devfs_token;		/* store the devfs handle here */
+	void *rc_devfs_token;		/* store the devfs handle here */
+	void *a_devfs_token;		/* store the devfs handle here */
+	void *c_devfs_token;		/* store the devfs handle here */
+#endif
 } mcd_data[NMCD];
 
 /* reader state machine */
@@ -164,23 +173,17 @@ struct mcd_data {
 #define MCD_S_WAITREAD	4
 
 /* prototypes */
-int	mcdopen(dev_t dev, int flags, int fmt, struct proc *p);
-int	mcdclose(dev_t dev, int flags, int fmt, struct proc *p);
-void	mcdstrategy(struct buf *bp);
-int	mcdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p);
-int	mcdsize(dev_t dev);
-static	void	mcd_done(struct mcd_mbx *mbx);
 static	void	mcd_start(int unit);
 static	int	mcd_getdisklabel(int unit);
+#ifdef NOTYET
 static	void	mcd_configure(struct mcd_data *cd);
+#endif
 static	int	mcd_get(int unit, char *buf, int nmax);
 static  int     mcd_setflags(int unit,struct mcd_data *cd);
 static	int	mcd_getstat(int unit,int sflg);
 static	int	mcd_send(int unit, int cmd,int nretrys);
-static	int	bcd2bin(bcd_t b);
-static	bcd_t	bin2bcd(int b);
 static	void	hsg2msf(int hsg, bcd_t *msf);
-static	int	msf2hsg(bcd_t *msf);
+static  int     msf2hsg(bcd_t *msf, int relative);
 static	int	mcd_volinfo(int unit);
 static	int	mcd_waitrdy(int port,int dly);
 static 	void	mcd_doread(int state, struct mcd_mbx *mbxin);
@@ -194,19 +197,33 @@ static	int	mcd_read_toc(int unit);
 static  int     mcd_toc_entrys(int unit, struct ioc_read_toc_entry *te);
 static	int	mcd_stop(int unit);
 static  int     mcd_eject(int unit);
+static  int     mcd_inject(int unit);
 static	int	mcd_playtracks(int unit, struct ioc_play_track *pt);
 static	int	mcd_play(int unit, struct mcd_read2 *pb);
 static  int     mcd_playmsf(int unit, struct ioc_play_msf *pt);
+static  int     mcd_playblocks(int unit, struct ioc_play_blocks *);
 static	int	mcd_pause(int unit);
 static	int	mcd_resume(int unit);
 static  int     mcd_lock_door(int unit, int lock);
 static  int     mcd_close_tray(int unit);
 
-extern	int	hz;
-extern	int	mcd_probe(struct isa_device *dev);
-extern	int	mcd_attach(struct isa_device *dev);
+static	int	mcd_probe(struct isa_device *dev);
+static	int	mcd_attach(struct isa_device *dev);
 struct	isa_driver	mcddriver = { mcd_probe, mcd_attach, "mcd" };
 
+static	d_open_t	mcdopen;
+static	d_close_t	mcdclose;
+static	d_ioctl_t	mcdioctl;
+static	d_psize_t	mcdsize;
+static	d_strategy_t	mcdstrategy;
+
+#define CDEV_MAJOR 29
+#define BDEV_MAJOR 7
+static struct cdevsw mcd_cdevsw;
+static struct bdevsw mcd_bdevsw = 
+	{ mcdopen,	mcdclose,	mcdstrategy,	mcdioctl,	/*7*/
+	  nodump,	mcdsize,	0,	"mcd",	&mcd_cdevsw,	-1 };
+
 #define mcd_put(port,byte)	outb(port,byte)
 
 #define MCD_RETRYS	5
@@ -224,44 +241,39 @@ struct	isa_driver	mcddriver = { mcd_probe, mcd_attach, "mcd" };
 #define MIN_DELAY       15
 #define DELAY_GETREPLY  5000000
 
-static struct kern_devconf kdc_mcd[NMCD] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"mcd", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* status */
-	"Mitsumi CD-ROM controller", /* properly filled later */
-	DC_CLS_RDISK
-} };
-
-static inline void
-mcd_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_mcd[id->id_unit] = kdc_mcd[0];
-	kdc_mcd[id->id_unit].kdc_unit = id->id_unit;
-	kdc_mcd[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_mcd[id->id_unit]);
-}
-
 int mcd_attach(struct isa_device *dev)
 {
-	struct mcd_data *cd = mcd_data + dev->id_unit;
-	int i;
+	int	unit = dev->id_unit;
+	struct mcd_data *cd = mcd_data + unit;
 
 	cd->iobase = dev->id_iobase;
 	cd->flags |= MCDINIT;
-	mcd_soft_reset(dev->id_unit);
+	mcd_soft_reset(unit);
+	bufq_init(&cd->head);
 
 #ifdef NOTYET
 	/* wire controller for interrupts and dma */
 	mcd_configure(cd);
 #endif
-	kdc_mcd[dev->id_unit].kdc_state = DC_IDLE;
 	/* name filled in probe */
-	kdc_mcd[dev->id_unit].kdc_description = mcd_data[dev->id_unit].name;
-
+#ifdef DEVFS
+	cd->ra_devfs_token = 
+		devfs_add_devswf(&mcd_cdevsw, dkmakeminor(unit, 0, 0),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rmcd%da", unit);
+	cd->rc_devfs_token = 
+		devfs_add_devswf(&mcd_cdevsw, dkmakeminor(unit, 0, RAW_PART),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rmcd%dc", unit);
+	cd->a_devfs_token = 
+		devfs_add_devswf(&mcd_bdevsw, dkmakeminor(unit, 0, 0),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "mcd%da", unit);
+	cd->c_devfs_token = 
+		devfs_add_devswf(&mcd_bdevsw, dkmakeminor(unit, 0, RAW_PART),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "mcd%dc", unit);
+#endif
 	return 1;
 }
 
@@ -286,19 +298,29 @@ int mcdopen(dev_t dev, int flags, int fmt, struct proc *p)
 	if (!(cd->flags & MCDVALID) && cd->openflags)
 		return ENXIO;
 
-	if (mcd_close_tray(unit) == EIO)  /* detect disk change too */
+	if (mcd_getstat(unit,1) == -1)
 		return EIO;
 
 	if (    (cd->status & (MCDDSKCHNG|MCDDOOROPEN))
 	    || !(cd->status & MCDDSKIN))
 		for (retry = 0; retry < DISK_SENSE_SECS * WAIT_FRAC; retry++) {
-			(void) tsleep((caddr_t)cd, PSOCK | PCATCH, "mcdsns", hz/WAIT_FRAC);
+			(void) tsleep((caddr_t)cd, PSOCK | PCATCH, "mcdsn1", hz/WAIT_FRAC);
 			if ((r = mcd_getstat(unit,1)) == -1)
 				return EIO;
 			if (r != -2)
 				break;
 		}
 
+	if ((   (cd->status & (MCDDOOROPEN|MCDDSKCHNG))
+	     || !(cd->status & MCDDSKIN)
+	    )
+	    && major(dev) == CDEV_MAJOR && part == RAW_PART
+	   ) {
+		cd->openflags |= (1<<part);
+		if (phys)
+			cd->partflags[part] |= MCDREADRAW;
+		return 0;
+	}
 	if (cd->status & MCDDOOROPEN) {
 		printf("mcd%d: door is open\n", unit);
 		return ENXIO;
@@ -313,6 +335,12 @@ int mcdopen(dev_t dev, int flags, int fmt, struct proc *p)
 	}
 
 	if (mcdsize(dev) < 0) {
+		if (major(dev) == CDEV_MAJOR && part == RAW_PART) {
+			cd->openflags |= (1<<part);
+			if (phys)
+				cd->partflags[part] |= MCDREADRAW;
+			return 0;
+		}
 		printf("mcd%d: failed to get disk size\n",unit);
 		return ENXIO;
 	} else
@@ -327,11 +355,9 @@ MCD_TRACE("open: partition=%d, disksize = %ld, blksize=%d\n",
 	if (part == RAW_PART ||
 		(part < cd->dlabel.d_npartitions &&
 		cd->dlabel.d_partitions[part].p_fstype != FS_UNUSED)) {
-		cd->partflags[part] |= MCDOPEN;
 		cd->openflags |= (1<<part);
-		if (part == RAW_PART && phys != 0)
+		if (part == RAW_PART && phys)
 			cd->partflags[part] |= MCDREADRAW;
-		kdc_mcd[unit].kdc_state = DC_BUSY;
 		(void) mcd_lock_door(unit, MCD_LK_LOCK);
 		if (!(cd->flags & MCDVALID))
 			return ENXIO;
@@ -343,7 +369,7 @@ MCD_TRACE("open: partition=%d, disksize = %ld, blksize=%d\n",
 
 int mcdclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
-	int unit,part,phys;
+	int unit,part;
 	struct mcd_data *cd;
 
 	unit = mcd_unit(dev);
@@ -352,22 +378,16 @@ int mcdclose(dev_t dev, int flags, int fmt, struct proc *p)
 
 	cd = mcd_data + unit;
 	part = mcd_part(dev);
-	phys = mcd_phys(dev);
 
-	if (!(cd->flags & MCDINIT))
+	if (!(cd->flags & MCDINIT) || !(cd->openflags & (1<<part)))
 		return ENXIO;
 
-	kdc_mcd[unit].kdc_state = DC_IDLE;
-	(void) mcd_lock_door(unit, MCD_LK_UNLOCK);
-
-	if (!(cd->flags & MCDVALID))
-		return 0;
-
-	/* close channel */
-	cd->partflags[part] &= ~(MCDOPEN|MCDREADRAW);
-	cd->openflags &= ~(1<<part);
 	MCD_TRACE("close: partition=%d\n", part);
 
+	(void) mcd_lock_door(unit, MCD_LK_UNLOCK);
+	cd->openflags &= ~(1<<part);
+	cd->partflags[part] &= ~MCDREADRAW;
+
 	return 0;
 }
 
@@ -375,7 +395,6 @@ void
 mcdstrategy(struct buf *bp)
 {
 	struct mcd_data *cd;
-	struct buf *qp;
 	int s;
 
 	int unit = mcd_unit(bp->b_dev);
@@ -388,7 +407,7 @@ mcdstrategy(struct buf *bp)
 	if (unit >= NMCD || bp->b_blkno < 0) {
 		printf("mcdstrategy: unit = %d, blkno = %ld, bcount = %ld\n",
 			unit, bp->b_blkno, bp->b_bcount);
-		pg("mcd: mcdstratregy failure");
+		printf("mcd: mcdstratregy failure");
 		bp->b_error = EINVAL;
 		bp->b_flags |= B_ERROR;
 		goto bad;
@@ -427,9 +446,8 @@ MCD_TRACE("strategy: drive not valid\n");
 	}
 
 	/* queue it */
-	qp = &cd->head;
 	s = splbio();
-	disksort(qp,bp);
+	bufqdisksort(&cd->head, bp);
 	splx(s);
 
 	/* now check whether we can perform processing */
@@ -447,9 +465,8 @@ done:
 static void mcd_start(int unit)
 {
 	struct mcd_data *cd = mcd_data + unit;
-	struct buf *bp, *qp = &cd->head;
 	struct partition *p;
-	int part;
+	struct buf *bp;
 	register s = splbio();
 
 	if (cd->flags & MCDMBXBSY) {
@@ -457,10 +474,11 @@ static void mcd_start(int unit)
 		return;
 	}
 
-	if ((bp = qp->b_actf) != 0) {
+	bp = bufq_first(&cd->head);
+	if (bp != 0) {
 		/* block found to process, dequeue */
 		/*MCD_TRACE("mcd_start: found block bp=0x%x\n",bp,0,0,0);*/
-	    qp->b_actf = bp->b_actf; /* changed from: bp->av_forw <se> */
+		bufq_remove(&cd->head, bp);
 		splx(s);
 	} else {
 		/* nothing to do */
@@ -494,7 +512,7 @@ static void mcd_start(int unit)
 int mcdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 {
 	struct mcd_data *cd;
-	int unit,part;
+	int unit,part,retry,r;
 
 	unit = mcd_unit(dev);
 	part = mcd_part(dev);
@@ -505,66 +523,6 @@ int mcdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 MCD_TRACE("ioctl called 0x%x\n", cmd);
 
 	switch (cmd) {
-	case DIOCSBAD:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return EINVAL;
-	case DIOCGDINFO:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		*(struct disklabel *) addr = cd->dlabel;
-		return 0;
-	case DIOCGPART:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		((struct partinfo *) addr)->disklab = &cd->dlabel;
-		((struct partinfo *) addr)->part =
-		    &cd->dlabel.d_partitions[mcd_part(dev)];
-		return 0;
-
-		/*
-		 * a bit silly, but someone might want to test something on a
-		 * section of cdrom.
-		 */
-	case DIOCWDINFO:
-	case DIOCSDINFO:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		if ((flags & FWRITE) == 0)
-			return EBADF;
-		else {
-			return setdisklabel(&cd->dlabel,
-			    (struct disklabel *) addr,
-			    0);
-		}
-	case DIOCWLABEL:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return EBADF;
-	case CDIOCPLAYTRACKS:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_playtracks(unit, (struct ioc_play_track *) addr);
-	case CDIOCPLAYBLOCKS:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return EINVAL;
-	case CDIOCPLAYMSF:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_playmsf(unit, (struct ioc_play_msf *) addr);
-	case CDIOCREADSUBCHANNEL:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_subchan(unit, (struct ioc_read_subchannel *) addr);
-	case CDIOREADTOCHEADER:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_toc_header(unit, (struct ioc_toc_header *) addr);
-	case CDIOREADTOCENTRYS:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_toc_entrys(unit, (struct ioc_read_toc_entry *) addr);
 	case CDIOCSETPATCH:
 	case CDIOCGETVOL:
 	case CDIOCSETVOL:
@@ -574,22 +532,6 @@ MCD_TRACE("ioctl called 0x%x\n", cmd);
 	case CDIOCSETLEFT:
 	case CDIOCSETRIGHT:
 		return EINVAL;
-	case CDIOCRESUME:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_resume(unit);
-	case CDIOCPAUSE:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_pause(unit);
-	case CDIOCSTART:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return EINVAL;
-	case CDIOCSTOP:
-		if (!(cd->flags & MCDVALID))
-			return ENXIO;
-		return mcd_stop(unit);
 	case CDIOCEJECT:
 		return mcd_eject(unit);
 	case CDIOCSETDEBUG:
@@ -602,6 +544,90 @@ MCD_TRACE("ioctl called 0x%x\n", cmd);
 		return mcd_hard_reset(unit);
 	case CDIOCALLOW:
 		return mcd_lock_door(unit, MCD_LK_UNLOCK);
+	case CDIOCPREVENT:
+		return mcd_lock_door(unit, MCD_LK_LOCK);
+	case CDIOCCLOSE:
+		return mcd_inject(unit);
+	}
+
+	if (!(cd->flags & MCDVALID)) {
+		if (   major(dev) != CDEV_MAJOR
+		    || part != RAW_PART
+		    || !(cd->openflags & (1<<RAW_PART))
+		   )
+			return ENXIO;
+		if (    (cd->status & (MCDDSKCHNG|MCDDOOROPEN))
+		    || !(cd->status & MCDDSKIN))
+			for (retry = 0; retry < DISK_SENSE_SECS * WAIT_FRAC; retry++) {
+				(void) tsleep((caddr_t)cd, PSOCK | PCATCH, "mcdsn2", hz/WAIT_FRAC);
+				if ((r = mcd_getstat(unit,1)) == -1)
+					return EIO;
+				if (r != -2)
+					break;
+			}
+		if (   (cd->status & (MCDDOOROPEN|MCDDSKCHNG))
+		    || !(cd->status & MCDDSKIN)
+		    || mcdsize(dev) < 0
+		   )
+			return ENXIO;
+		cd->flags |= MCDVALID;
+		mcd_getdisklabel(unit);
+		if (mcd_phys(dev))
+			cd->partflags[part] |= MCDREADRAW;
+		(void) mcd_lock_door(unit, MCD_LK_LOCK);
+		if (!(cd->flags & MCDVALID))
+			return ENXIO;
+	}
+
+	switch (cmd) {
+	case DIOCSBAD:
+		return EINVAL;
+	case DIOCGDINFO:
+		*(struct disklabel *) addr = cd->dlabel;
+		return 0;
+	case DIOCGPART:
+		((struct partinfo *) addr)->disklab = &cd->dlabel;
+		((struct partinfo *) addr)->part =
+		    &cd->dlabel.d_partitions[mcd_part(dev)];
+		return 0;
+
+		/*
+		 * a bit silly, but someone might want to test something on a
+		 * section of cdrom.
+		 */
+	case DIOCWDINFO:
+	case DIOCSDINFO:
+		if ((flags & FWRITE) == 0)
+			return EBADF;
+		else {
+			return setdisklabel(&cd->dlabel,
+			    (struct disklabel *) addr,
+			    0);
+		}
+	case DIOCWLABEL:
+		return EBADF;
+	case CDIOCPLAYTRACKS:
+		return mcd_playtracks(unit, (struct ioc_play_track *) addr);
+	case CDIOCPLAYBLOCKS:
+		return mcd_playblocks(unit, (struct ioc_play_blocks *) addr);
+	case CDIOCPLAYMSF:
+		return mcd_playmsf(unit, (struct ioc_play_msf *) addr);
+	case CDIOCREADSUBCHANNEL:
+		return mcd_subchan(unit, (struct ioc_read_subchannel *) addr);
+	case CDIOREADTOCHEADER:
+		return mcd_toc_header(unit, (struct ioc_toc_header *) addr);
+	case CDIOREADTOCENTRYS:
+		return mcd_toc_entrys(unit, (struct ioc_read_toc_entry *) addr);
+	case CDIOCRESUME:
+		return mcd_resume(unit);
+	case CDIOCPAUSE:
+		return mcd_pause(unit);
+	case CDIOCSTART:
+		if (mcd_setmode(unit, MCD_MD_COOKED) != 0)
+			return EIO;
+		return 0;
+	case CDIOCSTOP:
+		return mcd_stop(unit);
 	default:
 		return ENOTTY;
 	}
@@ -655,7 +681,7 @@ int mcdsize(dev_t dev)
 
 	if (mcd_volinfo(unit) == 0) {
 		cd->blksize = MCDBLK;
-		size = msf2hsg(cd->volinfo.vol_msf);
+		size = msf2hsg(cd->volinfo.vol_msf, 0);
 		cd->disksize = size * (MCDBLK/DEV_BSIZE);
 		return 0;
 	}
@@ -679,11 +705,13 @@ drqs[] = {
 };
 #endif
 
+#ifdef NOT_YET
 static void
 mcd_configure(struct mcd_data *cd)
 {
 	outb(cd->iobase+mcd_config,cd->config);
 }
+#endif
 
 /* Wait for non-busy - return 0 on timeout */
 static int
@@ -695,7 +723,8 @@ twiddle_thumbs(int port, int unit, int count, char *whine)
 		if (!(inb(port+MCD_FLAGS) & MFL_STATUS_NOT_AVAIL))
 			return 1;
 		}
-	printf("mcd%d: timeout %s\n", unit, whine);
+	if (bootverbose)
+		printf("mcd%d: timeout %s\n", unit, whine);
 	return 0;
 }
 
@@ -707,10 +736,8 @@ mcd_probe(struct isa_device *dev)
 	int port = dev->id_iobase;
 	int unit = dev->id_unit;
 	int i, j;
-	int status;
 	unsigned char stbytes[3];
 
-	mcd_registerdev(dev);
 	mcd_data[unit].flags = MCDPROBING;
 
 #ifdef NOTDEF
@@ -750,12 +777,15 @@ mcd_probe(struct isa_device *dev)
 	mcd_data[unit].read_command = MCD_CMDSINGLESPEEDREAD;
 	switch (stbytes[1]) {
 	case 'M':
-		if (mcd_data[unit].flags & MCDNEWMODEL)	{
+		if (stbytes[2] <= 2) {
+			mcd_data[unit].type = MCD_TYPE_LU002S;
+			mcd_data[unit].name = "Mitsumi LU002S";
+		} else if (stbytes[2] <= 5) {
 			mcd_data[unit].type = MCD_TYPE_LU005S;
 			mcd_data[unit].name = "Mitsumi LU005S";
 		} else {
-			mcd_data[unit].type = MCD_TYPE_LU002S;
-			mcd_data[unit].name = "Mitsumi LU002S";
+			mcd_data[unit].type = MCD_TYPE_LU006S;
+			mcd_data[unit].name = "Mitsumi LU006S";
 		}
 		break;
 	case 'F':
@@ -796,7 +826,6 @@ mcd_waitrdy(int port,int dly)
 static int
 mcd_getreply(int unit,int dly)
 {
-	int	i;
 	struct	mcd_data *cd = mcd_data + unit;
 	int	port = cd->iobase;
 
@@ -854,7 +883,6 @@ mcd_setflags(int unit, struct mcd_data *cd)
 static int
 mcd_get(int unit, char *buf, int nmax)
 {
-	int port = mcd_data[unit].iobase;
 	int i,k;
 
 	for (i=0; i<nmax; i++) {
@@ -892,41 +920,28 @@ mcd_send(int unit, int cmd,int nretrys)
 	return 0;
 }
 
-static int
-bcd2bin(bcd_t b)
-{
-	return (b >> 4) * 10 + (b & 15);
-}
-
-static bcd_t
-bin2bcd(int b)
-{
-	return ((b / 10) << 4) | (b % 10);
-}
-
 static void
 hsg2msf(int hsg, bcd_t *msf)
 {
 	hsg += 150;
-	M_msf(msf) = bin2bcd(hsg / 4500);
-	hsg %= 4500;
-	S_msf(msf) = bin2bcd(hsg / 75);
 	F_msf(msf) = bin2bcd(hsg % 75);
+	hsg /= 75;
+	S_msf(msf) = bin2bcd(hsg % 60);
+	hsg /= 60;
+	M_msf(msf) = bin2bcd(hsg);
 }
 
 static int
-msf2hsg(bcd_t *msf)
+msf2hsg(bcd_t *msf, int relative)
 {
-	return (bcd2bin(M_msf(msf)) * 60 +
-		bcd2bin(S_msf(msf))) * 75 +
-		bcd2bin(F_msf(msf)) - 150;
+	return (bcd2bin(M_msf(msf)) * 60 + bcd2bin(S_msf(msf))) * 75 +
+		bcd2bin(F_msf(msf)) - (!relative) * 150;
 }
 
 static int
 mcd_volinfo(int unit)
 {
 	struct mcd_data *cd = mcd_data + unit;
-	int i;
 
 	/* Just return if we already have it */
 	if (cd->flags & MCDVOLINFO) return 0;
@@ -1238,7 +1253,7 @@ mcd_eject(int unit)
 	if (mcd_getstat(unit,1) == -1)    /* detect disk change too */
 		return EIO;
 	if (cd->status & MCDDOOROPEN)
-		return mcd_close_tray(unit);
+		return 0;
 	if ((r = mcd_stop(unit)) == EIO)
 		return r;
 	outb(port+mcd_command, MCD_CMDEJECTDISK);
@@ -1247,6 +1262,18 @@ mcd_eject(int unit)
 	return 0;
 }
 
+static int
+mcd_inject(int unit)
+{
+	struct mcd_data *cd = mcd_data + unit;
+
+	if (mcd_getstat(unit,1) == -1)    /* detect disk change too */
+		return EIO;
+	if (cd->status & MCDDOOROPEN)
+		return mcd_close_tray(unit);
+	return 0;
+}
+
 static int
 mcd_hard_reset(int unit)
 {
@@ -1265,7 +1292,6 @@ mcd_soft_reset(int unit)
 	struct mcd_data *cd = mcd_data + unit;
 	int i;
 
-	cd->openflags = 0;
 	cd->flags &= (MCDINIT|MCDPROBING|MCDNEWMODEL);
 	cd->curr_mode = MCD_MD_UNKNOWN;
 	for (i=0; i<MAXPARTITIONS; i++) cd->partflags[i] = 0;
@@ -1310,9 +1336,11 @@ mcd_toc_header(int unit, struct ioc_toc_header *th)
 	if ((r = mcd_volinfo(unit)) != 0)
 		return r;
 
-	th->len = msf2hsg(cd->volinfo.vol_msf);
 	th->starting_track = bcd2bin(cd->volinfo.trk_low);
 	th->ending_track = bcd2bin(cd->volinfo.trk_high);
+	th->len = 2 * sizeof(u_char) /* start & end tracks */ +
+		  (th->ending_track + 1 - th->starting_track + 1) *
+		  sizeof(struct cd_toc_entry);
 
 	return 0;
 }
@@ -1367,9 +1395,10 @@ mcd_read_toc(int unit)
 
 	/* add a fake last+1 */
 	idx = th.ending_track + 1;
-	cd->toc[idx].ctrl_adr = cd->toc[idx-1].ctrl_adr;
+	cd->toc[idx].control = cd->toc[idx-1].control;
+	cd->toc[idx].addr_type = cd->toc[idx-1].addr_type;
 	cd->toc[idx].trk_no = 0;
-	cd->toc[idx].idx_no = 0xAA;
+	cd->toc[idx].idx_no = MCD_LASTPLUS1;
 	cd->toc[idx].hd_pos_msf[0] = cd->volinfo.vol_msf[0];
 	cd->toc[idx].hd_pos_msf[1] = cd->volinfo.vol_msf[1];
 	cd->toc[idx].hd_pos_msf[2] = cd->volinfo.vol_msf[2];
@@ -1379,7 +1408,8 @@ mcd_read_toc(int unit)
 	for (i = th.starting_track; i <= idx; i++)
 		printf("mcd%d: trk %d idx %d pos %d %d %d\n",
 			unit, i,
-			cd->toc[i].idx_no,
+			cd->toc[i].idx_no > 0x99 ? cd->toc[i].idx_no :
+			bcd2bin(cd->toc[i].idx_no),
 			bcd2bin(cd->toc[i].hd_pos_msf[0]),
 			bcd2bin(cd->toc[i].hd_pos_msf[1]),
 			bcd2bin(cd->toc[i].hd_pos_msf[2]));
@@ -1396,27 +1426,12 @@ mcd_toc_entrys(int unit, struct ioc_read_toc_entry *te)
 	struct mcd_data *cd = mcd_data + unit;
 	struct cd_toc_entry entries[MCD_MAXTOCS];
 	struct ioc_toc_header th;
-	int rc, i, len = te->data_len;
+	int rc, n, trk, len;
 
-	/* Make sure we have a valid toc */
-	if ((rc=mcd_read_toc(unit)) != 0)
-		return rc;
-
-	/* find the toc to copy*/
-	i = te->starting_track;
-	if (i == MCD_LASTPLUS1)
-		i = bcd2bin(cd->volinfo.trk_high) + 1;
-
-	/* verify starting track */
-	if (i < bcd2bin(cd->volinfo.trk_low) ||
-		i > bcd2bin(cd->volinfo.trk_high)+1) {
-		return EINVAL;
-	}
-
-	/* do we have room */
-	if (   len > sizeof(entries)
-	    || len < sizeof(struct cd_toc_entry)
-	    || (len % sizeof(struct cd_toc_entry)) != 0
+	if (   te->data_len < sizeof(entries[0])
+	    || (te->data_len % sizeof(entries[0])) != 0
+	    || te->address_format != CD_MSF_FORMAT
+	    && te->address_format != CD_LBA_FORMAT
 	   )
 		return EINVAL;
 
@@ -1424,27 +1439,52 @@ mcd_toc_entrys(int unit, struct ioc_read_toc_entry *te)
 	if ((rc = mcd_toc_header(unit, &th)) != 0)
 		return rc;
 
-	do {
-		/* copy the toc data */
-		entries[i-1].control = cd->toc[i].ctrl_adr;
-		entries[i-1].addr_type = te->address_format;
-		entries[i-1].track = i;
-		if (te->address_format == CD_MSF_FORMAT) {
-			entries[i-1].addr.msf.unused = 0;
-			entries[i-1].addr.msf.minute = bcd2bin(cd->toc[i].hd_pos_msf[0]);
-			entries[i-1].addr.msf.second = bcd2bin(cd->toc[i].hd_pos_msf[1]);
-			entries[i-1].addr.msf.frame = bcd2bin(cd->toc[i].hd_pos_msf[2]);
+	/* verify starting track */
+	trk = te->starting_track;
+	if (trk == 0)
+		trk = th.starting_track;
+	else if (trk == MCD_LASTPLUS1)
+		trk = th.ending_track + 1;
+	else if (trk < th.starting_track || trk > th.ending_track + 1)
+		return EINVAL;
+
+	len = ((th.ending_track + 1 - trk) + 1) *
+		sizeof(entries[0]);
+	if (te->data_len < len)
+		len = te->data_len;
+	if (len > sizeof(entries))
+		return EINVAL;
+
+	/* Make sure we have a valid toc */
+	if ((rc=mcd_read_toc(unit)) != 0)
+		return rc;
+
+	/* Copy the TOC data. */
+	for (n = 0; len > 0 && trk <= th.ending_track + 1; trk++) {
+		if (cd->toc[trk].idx_no == 0)
+			continue;
+		entries[n].control = cd->toc[trk].control;
+		entries[n].addr_type = cd->toc[trk].addr_type;
+		entries[n].track =
+			cd->toc[trk].idx_no > 0x99 ? cd->toc[trk].idx_no :
+			bcd2bin(cd->toc[trk].idx_no);
+		switch (te->address_format) {
+		case CD_MSF_FORMAT:
+			entries[n].addr.msf.unused = 0;
+			entries[n].addr.msf.minute = bcd2bin(cd->toc[trk].hd_pos_msf[0]);
+			entries[n].addr.msf.second = bcd2bin(cd->toc[trk].hd_pos_msf[1]);
+			entries[n].addr.msf.frame = bcd2bin(cd->toc[trk].hd_pos_msf[2]);
+			break;
+		case CD_LBA_FORMAT:
+			entries[n].addr.lba = htonl(msf2hsg(cd->toc[trk].hd_pos_msf, 0));
+			break;
 		}
 		len -= sizeof(struct cd_toc_entry);
-		i++;
+		n++;
 	}
-	while (len > 0 && i <= th.ending_track + 2);
 
 	/* copy the data back */
-	if (copyout(entries, te->data, (i - 1) * sizeof(struct cd_toc_entry)) != 0)
-		return EFAULT;
-
-	return 0;
+	return copyout(entries, te->data, n * sizeof(struct cd_toc_entry));
 }
 
 static int
@@ -1454,9 +1494,11 @@ mcd_stop(int unit)
 
 	/* Verify current status */
 	if (cd->audio_status != CD_AS_PLAY_IN_PROGRESS &&
-	    cd->audio_status != CD_AS_PLAY_PAUSED) {
+	    cd->audio_status != CD_AS_PLAY_PAUSED &&
+	    cd->audio_status != CD_AS_PLAY_COMPLETED) {
 		if (cd->debug)
-			printf("mcd%d: stop attempted when not playing\n", unit);
+			printf("mcd%d: stop attempted when not playing, audio status %d\n",
+				unit, cd->audio_status);
 		return EINVAL;
 	}
 	if (cd->audio_status == CD_AS_PLAY_IN_PROGRESS)
@@ -1476,9 +1518,10 @@ mcd_getqchan(int unit, struct mcd_qchninfo *q)
 	if (mcd_get(unit, (char *) q, sizeof(struct mcd_qchninfo)) < 0)
 		return -1;
 	if (cd->debug) {
-		printf("mcd%d: getqchan ctrl_adr=0x%x trk=%d ind=%d ttm=%d:%d.%d dtm=%d:%d.%d\n",
+		printf("mcd%d: getqchan control=0x%x addr_type=0x%x trk=%d ind=%d ttm=%d:%d.%d dtm=%d:%d.%d\n",
 		unit,
-		q->ctrl_adr, bcd2bin(q->trk_no), bcd2bin(q->idx_no),
+		q->control, q->addr_type, bcd2bin(q->trk_no),
+		bcd2bin(q->idx_no),
 		bcd2bin(q->trk_size_msf[0]), bcd2bin(q->trk_size_msf[1]),
 		bcd2bin(q->trk_size_msf[2]),
 		bcd2bin(q->hd_pos_msf[0]), bcd2bin(q->hd_pos_msf[1]),
@@ -1493,6 +1536,7 @@ mcd_subchan(int unit, struct ioc_read_subchannel *sc)
 	struct mcd_data *cd = mcd_data + unit;
 	struct mcd_qchninfo q;
 	struct cd_sub_channel_info data;
+	int lba;
 
 	if (cd->debug)
 		printf("mcd%d: subchan af=%d, df=%d\n", unit,
@@ -1503,7 +1547,8 @@ mcd_subchan(int unit, struct ioc_read_subchannel *sc)
 	    sc->address_format != CD_LBA_FORMAT)
 		return EINVAL;
 
-	if (sc->data_format != CD_CURRENT_POSITION)
+	if (sc->data_format != CD_CURRENT_POSITION &&
+	    sc->data_format != CD_MEDIA_CATALOG)
 		return EINVAL;
 
 	if (mcd_setmode(unit, MCD_MD_COOKED) != 0)
@@ -1513,45 +1558,79 @@ mcd_subchan(int unit, struct ioc_read_subchannel *sc)
 		return EIO;
 
 	data.header.audio_status = cd->audio_status;
-	data.what.position.data_format = CD_CURRENT_POSITION;
-	data.what.position.addr_type = q.ctrl_adr;
-	data.what.position.control = q.ctrl_adr >> 4;
-	data.what.position.track_number = bcd2bin(q.trk_no);
-	data.what.position.index_number = bcd2bin(q.idx_no);
-	if (sc->address_format == CD_MSF_FORMAT) {
-		data.what.position.reladdr.msf.unused = 0;
-		data.what.position.reladdr.msf.minute = bcd2bin(q.trk_size_msf[0]);
-		data.what.position.reladdr.msf.second = bcd2bin(q.trk_size_msf[1]);
-		data.what.position.reladdr.msf.frame = bcd2bin(q.trk_size_msf[2]);
-		data.what.position.absaddr.msf.unused = 0;
-		data.what.position.absaddr.msf.minute = bcd2bin(q.hd_pos_msf[0]);
-		data.what.position.absaddr.msf.second = bcd2bin(q.hd_pos_msf[1]);
-		data.what.position.absaddr.msf.frame = bcd2bin(q.hd_pos_msf[2]);
-	} else if (sc->address_format == CD_LBA_FORMAT) {
-		data.what.position.reladdr.lba = msf2hsg(q.trk_size_msf);
-		data.what.position.absaddr.lba = msf2hsg(q.hd_pos_msf);
+	data.what.position.data_format = sc->data_format;
+
+	switch (sc->data_format) {
+	case CD_MEDIA_CATALOG:
+		data.what.media_catalog.mc_valid = 1;
+		data.what.media_catalog.mc_number[0] = '\0';
+		break;
+
+	case CD_CURRENT_POSITION:
+		data.what.position.control = q.control;
+		data.what.position.addr_type = q.addr_type;
+		data.what.position.track_number = bcd2bin(q.trk_no);
+		data.what.position.index_number = bcd2bin(q.idx_no);
+		switch (sc->address_format) {
+		case CD_MSF_FORMAT:
+			data.what.position.reladdr.msf.unused = 0;
+			data.what.position.reladdr.msf.minute = bcd2bin(q.trk_size_msf[0]);
+			data.what.position.reladdr.msf.second = bcd2bin(q.trk_size_msf[1]);
+			data.what.position.reladdr.msf.frame = bcd2bin(q.trk_size_msf[2]);
+			data.what.position.absaddr.msf.unused = 0;
+			data.what.position.absaddr.msf.minute = bcd2bin(q.hd_pos_msf[0]);
+			data.what.position.absaddr.msf.second = bcd2bin(q.hd_pos_msf[1]);
+			data.what.position.absaddr.msf.frame = bcd2bin(q.hd_pos_msf[2]);
+			break;
+		case CD_LBA_FORMAT:
+			lba = msf2hsg(q.trk_size_msf, 1);
+			/*
+			 * Pre-gap has index number of 0, and decreasing MSF
+			 * address.  Must be converted to negative LBA, per
+			 * SCSI spec.
+			 */
+			if (data.what.position.index_number == 0)
+				lba = -lba;
+			data.what.position.reladdr.lba = htonl(lba);
+			data.what.position.absaddr.lba = htonl(msf2hsg(q.hd_pos_msf, 0));
+			break;
+		}
+		break;
 	}
 
-	if (copyout(&data, sc->data, min(sizeof(struct cd_sub_channel_info), sc->data_len))!=0)
-		return EFAULT;
-	return 0;
+	return copyout(&data, sc->data, min(sizeof(struct cd_sub_channel_info), sc->data_len));
 }
 
 static int
-mcd_playmsf(int unit, struct ioc_play_msf *pt)
+mcd_playmsf(int unit, struct ioc_play_msf *p)
 {
+	struct mcd_data *cd = mcd_data + unit;
 	struct mcd_read2 pb;
 
+	if (cd->debug)
+		printf("mcd%d: playmsf: from %d:%d.%d to %d:%d.%d\n",
+		    unit,
+		    p->start_m, p->start_s, p->start_f,
+		    p->end_m, p->end_s, p->end_f);
+
+	if ((p->start_m * 60 * 75 + p->start_s * 75 + p->start_f) >=
+	    (p->end_m * 60 * 75 + p->end_s * 75 + p->end_f) ||
+	    (p->end_m * 60 * 75 + p->end_s * 75 + p->end_f) >
+	    M_msf(cd->volinfo.vol_msf) * 60 * 75 +
+	    S_msf(cd->volinfo.vol_msf) * 75 +
+	    F_msf(cd->volinfo.vol_msf))
+		return EINVAL;
+
+	pb.start_msf[0] = bin2bcd(p->start_m);
+	pb.start_msf[1] = bin2bcd(p->start_s);
+	pb.start_msf[2] = bin2bcd(p->start_f);
+	pb.end_msf[0] = bin2bcd(p->end_m);
+	pb.end_msf[1] = bin2bcd(p->end_s);
+	pb.end_msf[2] = bin2bcd(p->end_f);
+
 	if (mcd_setmode(unit, MCD_MD_COOKED) != 0)
 		return EIO;
 
-	pb.start_msf[0] = bin2bcd(pt->start_m);
-	pb.start_msf[1] = bin2bcd(pt->start_s);
-	pb.start_msf[2] = bin2bcd(pt->start_f);
-	pb.end_msf[0] = bin2bcd(pt->end_m);
-	pb.end_msf[1] = bin2bcd(pt->end_s);
-	pb.end_msf[2] = bin2bcd(pt->end_f);
-
 	return mcd_play(unit, &pb);
 }
 
@@ -1583,6 +1662,33 @@ mcd_playtracks(int unit, struct ioc_play_track *pt)
 		pb.end_msf[i] = cd->toc[z+1].hd_pos_msf[i];
 	}
 
+	if (mcd_setmode(unit, MCD_MD_COOKED) != 0)
+		return EIO;
+
+	return mcd_play(unit, &pb);
+}
+
+static int
+mcd_playblocks(int unit, struct ioc_play_blocks *p)
+{
+	struct mcd_data *cd = mcd_data + unit;
+	struct mcd_read2 pb;
+
+	if (cd->debug)
+		printf("mcd%d: playblocks: blkno %d length %d\n",
+		    unit, p->blk, p->len);
+
+	if (p->blk > cd->disksize || p->len > cd->disksize ||
+	    p->blk < 0 || p->len < 0 ||
+	    (p->blk + p->len) > cd->disksize)
+		return EINVAL;
+
+	hsg2msf(p->blk, pb.start_msf);
+	hsg2msf(p->blk + p->len, pb.end_msf);
+
+	if (mcd_setmode(unit, MCD_MD_COOKED) != 0)
+		return EIO;
+
 	return mcd_play(unit, &pb);
 }
 
@@ -1634,9 +1740,11 @@ mcd_pause(int unit)
 	int rc;
 
 	/* Verify current status */
-	if (cd->audio_status != CD_AS_PLAY_IN_PROGRESS) {
+	if (cd->audio_status != CD_AS_PLAY_IN_PROGRESS &&
+	    cd->audio_status != CD_AS_PLAY_PAUSED) {
 		if (cd->debug)
-			printf("mcd%d: pause attempted when not playing\n", unit);
+			printf("mcd%d: pause attempted when not playing, audio status %d\n",
+			       unit, cd->audio_status);
 		return EINVAL;
 	}
 
@@ -1667,4 +1775,20 @@ mcd_resume(int unit)
 		return EINVAL;
 	return mcd_play(unit, &cd->lastpb);
 }
+
+
+static mcd_devsw_installed = 0;
+
+static void 	mcd_drvinit(void *unused)
+{
+
+	if( ! mcd_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &mcd_bdevsw);
+		mcd_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(mcddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,mcd_drvinit,NULL)
+
+
 #endif /* NMCD > 0 */
