@@ -11,7 +11,7 @@
  * this software for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/mse.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/mse.c 50950 1999-09-05 08:39:22Z peter $
  */
 /*
  * Driver for the Logitech and ATI Inport Bus mice for use with 386bsd and
@@ -48,47 +48,70 @@
 #if NMSE > 0
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/buf.h>
 #include <sys/kernel.h>
 #include <sys/ioctl.h>
 #include <sys/uio.h>
-#include <sys/devconf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <machine/clock.h>
-#include <machine/cpu.h>
+#include <machine/mouse.h>
 
 #include <i386/isa/isa_device.h>
-#include <i386/isa/icu.h>
+
+/* driver configuration flags (config) */
+#define MSE_CONFIG_ACCEL	0x00f0  /* acceleration factor */
+#define MSE_CONFIG_FLAGS	(MSE_CONFIG_ACCEL)
 
 static int mseprobe(struct isa_device *);
 static int mseattach(struct isa_device *);
-void mseintr(int);
 
 struct	isa_driver msedriver = {
 	mseprobe, mseattach, "mse"
 };
 
+static	d_open_t	mseopen;
+static	d_close_t	mseclose;
+static	d_read_t	mseread;
+static  d_ioctl_t	mseioctl;
+static	d_select_t	mseselect;
+
+#define CDEV_MAJOR 27
+static struct cdevsw mse_cdevsw = 
+	{ mseopen,	mseclose,	mseread,	nowrite,	/*27*/
+	  mseioctl,	nostop,		nullreset,	nodevtotty,/* mse */
+	  mseselect,	nommap,		NULL,	"mse",	NULL,	-1 };
+
+
 /*
  * Software control structure for mouse. The sc_enablemouse(),
  * sc_disablemouse() and sc_getmouse() routines must be called spl'd().
  */
-#define	PROTOBYTES	5
-struct mse_softc {
+static struct mse_softc {
 	int	sc_flags;
 	int	sc_mousetype;
 	struct	selinfo sc_selp;
 	u_int	sc_port;
-	void	(*sc_enablemouse)();
-	void	(*sc_disablemouse)();
-	void	(*sc_getmouse)();
+	void	(*sc_enablemouse) __P((u_int port));
+	void	(*sc_disablemouse) __P((u_int port));
+	void	(*sc_getmouse) __P((u_int port, int *dx, int *dy, int *but));
 	int	sc_deltax;
 	int	sc_deltay;
 	int	sc_obuttons;
 	int	sc_buttons;
 	int	sc_bytesread;
-	u_char	sc_bytes[PROTOBYTES];
+	u_char	sc_bytes[MOUSE_SYS_PACKETSIZE];
+	mousehw_t	hw;
+	mousemode_t	mode;
+	mousestatus_t	status;
+#ifdef DEVFS
+	void 	*devfs_token;
+	void	*n_devfs_token;
+#endif
 } mse_sc[NMSE];
 
 /* Flags */
@@ -96,6 +119,7 @@ struct mse_softc {
 #define	MSESC_WANT	0x2
 
 /* and Mouse Types */
+#define	MSE_NONE	0	/* don't move this! */
 #define	MSE_LOGITECH	0x1
 #define	MSE_ATIINPORT	0x2
 #define	MSE_LOGI_SIG	0xA5
@@ -145,8 +169,10 @@ struct mse_softc {
 #define	MSE_DISINTR	0x10
 #define MSE_INTREN	0x00
 
-static int mse_probelogi();
-static void mse_enablelogi(), mse_disablelogi(), mse_getlogi();
+static int mse_probelogi __P((struct isa_device *idp));
+static void mse_disablelogi __P((u_int port));
+static void mse_getlogi __P((u_int port, int *dx, int *dy, int *but));
+static void mse_enablelogi __P((u_int port));
 
 /*
  * ATI Inport mouse definitions
@@ -159,8 +185,10 @@ static void mse_enablelogi(), mse_disablelogi(), mse_getlogi();
 #define	MSE_INPORT_HOLD		0x20
 #define	MSE_INPORT_INTREN	0x09
 
-static int mse_probeati();
-static void mse_enableati(), mse_disableati(), mse_getati();
+static int mse_probeati __P((struct isa_device *idp));
+static void mse_enableati __P((u_int port));
+static void mse_disableati __P((u_int port));
+static void mse_getati __P((u_int port, int *dx, int *dy, int *but));
 
 #define	MSEPRI	(PZERO + 3)
 
@@ -169,39 +197,32 @@ static void mse_enableati(), mse_disableati(), mse_getati();
  * Keep the Logitech last, since I haven't figured out how to probe it
  * properly yet. (Someday I'll have the documentation.)
  */
-struct mse_types {
+static struct mse_types {
 	int	m_type;		/* Type of bus mouse */
-	int	(*m_probe)();	/* Probe routine to test for it */
-	void	(*m_enable)();	/* Start routine */
-	void	(*m_disable)();	/* Disable interrupts routine */
-	void	(*m_get)();	/* and get mouse status */
+	int	(*m_probe) __P((struct isa_device *idp));
+				/* Probe routine to test for it */
+	void	(*m_enable) __P((u_int port));
+				/* Start routine */
+	void	(*m_disable) __P((u_int port));
+				/* Disable interrupts routine */
+	void	(*m_get) __P((u_int port, int *dx, int *dy, int *but));
+				/* and get mouse status */
+	mousehw_t   m_hw;	/* buttons iftype type model hwid */
+	mousemode_t m_mode;	/* proto rate res accel level size mask */
 } mse_types[] = {
-	{ MSE_ATIINPORT, mse_probeati, mse_enableati, mse_disableati, mse_getati },
-	{ MSE_LOGITECH, mse_probelogi, mse_enablelogi, mse_disablelogi, mse_getlogi },
+	{ MSE_ATIINPORT, 
+	  mse_probeati, mse_enableati, mse_disableati, mse_getati,
+	  { 2, MOUSE_IF_INPORT, MOUSE_MOUSE, MOUSE_MODEL_GENERIC, 0, },
+	  { MOUSE_PROTO_INPORT, -1, -1, 0, 0, MOUSE_MSC_PACKETSIZE, 
+	    { MOUSE_MSC_SYNCMASK, MOUSE_MSC_SYNC, }, }, },
+	{ MSE_LOGITECH, 
+	  mse_probelogi, mse_enablelogi, mse_disablelogi, mse_getlogi,
+	  { 2, MOUSE_IF_BUS, MOUSE_MOUSE, MOUSE_MODEL_GENERIC, 0, },
+	  { MOUSE_PROTO_BUS, -1, -1, 0, 0, MOUSE_MSC_PACKETSIZE, 
+	    { MOUSE_MSC_SYNCMASK, MOUSE_MSC_SYNC, }, }, },
 	{ 0, },
 };
 
-static struct kern_devconf kdc_mse[NMSE] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"mse", 0, { MDDT_ISA, 0, "tty" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"ATI or Logitech bus mouse adapter",
-	DC_CLS_MISC		/* class */
-} };
-
-static inline void
-mse_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_mse[id->id_unit] = kdc_mse[0];
-	kdc_mse[id->id_unit].kdc_unit = id->id_unit;
-	kdc_mse[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_mse[id->id_unit]);
-}
-
 int
 mseprobe(idp)
 	register struct isa_device *idp;
@@ -209,7 +230,6 @@ mseprobe(idp)
 	register struct mse_softc *sc = &mse_sc[idp->id_unit];
 	register int i;
 
-	mse_registerdev(idp);
 	/*
 	 * Check for each mouse type in the table.
 	 */
@@ -220,6 +240,8 @@ mseprobe(idp)
 			sc->sc_enablemouse = mse_types[i].m_enable;
 			sc->sc_disablemouse = mse_types[i].m_disable;
 			sc->sc_getmouse = mse_types[i].m_get;
+			sc->hw = mse_types[i].m_hw;
+			sc->mode = mse_types[i].m_mode;
 			return (1);
 		}
 		i++;
@@ -231,17 +253,26 @@ int
 mseattach(idp)
 	struct isa_device *idp;
 {
-	struct mse_softc *sc = &mse_sc[idp->id_unit];
+	int unit = idp->id_unit;
+	struct mse_softc *sc = &mse_sc[unit];
 
 	sc->sc_port = idp->id_iobase;
-	kdc_mse[idp->id_unit].kdc_state = DC_IDLE;
+	sc->mode.accelfactor = (idp->id_flags & MSE_CONFIG_ACCEL) >> 4;
+#ifdef	DEVFS
+	sc->devfs_token = 
+		devfs_add_devswf(&mse_cdevsw, unit << 1, DV_CHR, 0, 0, 
+				 0600, "mse%d", unit);
+	sc->n_devfs_token = 
+		devfs_add_devswf(&mse_cdevsw, (unit<<1)+1, DV_CHR,0, 0, 
+				 0600, "nmse%d", unit);
+#endif
 	return (1);
 }
 
 /*
  * Exclusive open the mouse, initialize it and enable interrupts.
  */
-int
+static	int
 mseopen(dev, flags, fmt, p)
 	dev_t dev;
 	int flags;
@@ -254,13 +285,18 @@ mseopen(dev, flags, fmt, p)
 	if (MSE_UNIT(dev) >= NMSE)
 		return (ENXIO);
 	sc = &mse_sc[MSE_UNIT(dev)];
+	if (sc->sc_mousetype == MSE_NONE)
+		return (ENXIO);
 	if (sc->sc_flags & MSESC_OPEN)
 		return (EBUSY);
 	sc->sc_flags |= MSESC_OPEN;
-	kdc_mse[MSE_UNIT(dev)].kdc_state = DC_BUSY;
-	sc->sc_obuttons = sc->sc_buttons = 0x7;
+	sc->sc_obuttons = sc->sc_buttons = MOUSE_MSC_BUTTONS;
 	sc->sc_deltax = sc->sc_deltay = 0;
-	sc->sc_bytesread = PROTOBYTES;
+	sc->sc_bytesread = sc->mode.packetsize = MOUSE_MSC_PACKETSIZE;
+	sc->mode.level = 0;
+	sc->status.flags = 0;
+	sc->status.button = sc->status.obutton = 0;
+	sc->status.dx = sc->status.dy = sc->status.dz = 0;
 
 	/*
 	 * Initialize mouse interface and enable interrupts.
@@ -274,7 +310,7 @@ mseopen(dev, flags, fmt, p)
 /*
  * mseclose: just turn off mouse innterrupts.
  */
-int
+static	int
 mseclose(dev, flags, fmt, p)
 	dev_t dev;
 	int flags;
@@ -287,7 +323,6 @@ mseclose(dev, flags, fmt, p)
 	s = spltty();
 	(*sc->sc_disablemouse)(sc->sc_port);
 	sc->sc_flags &= ~MSESC_OPEN;
-	kdc_mse[MSE_UNIT(dev)].kdc_state = DC_IDLE;
 	splx(s);
 	return(0);
 }
@@ -297,7 +332,7 @@ mseclose(dev, flags, fmt, p)
  * using bytes 4 and 5.
  * (Yes this is cheesy, but it makes the X386 server happy, so...)
  */
-int
+static	int
 mseread(dev, uio, ioflag)
 	dev_t dev;
 	struct uio *uio;
@@ -311,7 +346,7 @@ mseread(dev, uio, ioflag)
 	 * packet.
 	 */
 	s = spltty(); /* XXX Should be its own spl, but where is imlXX() */
-	if (sc->sc_bytesread >= PROTOBYTES) {
+	if (sc->sc_bytesread >= sc->mode.packetsize) {
 		while (sc->sc_deltax == 0 && sc->sc_deltay == 0 &&
 		       (sc->sc_obuttons ^ sc->sc_buttons) == 0) {
 			if (MSE_NBLOCKIO(dev)) {
@@ -331,7 +366,8 @@ mseread(dev, uio, ioflag)
 		 * For some reason X386 expects 5 bytes but never uses
 		 * the fourth or fifth?
 		 */
-		sc->sc_bytes[0] = 0x80 | (sc->sc_buttons & ~0xf8);
+		sc->sc_bytes[0] = sc->mode.syncmask[1] 
+		    | (sc->sc_buttons & ~sc->mode.syncmask[0]);
 		if (sc->sc_deltax > 127)
 			sc->sc_deltax = 127;
 		if (sc->sc_deltax < -127)
@@ -344,22 +380,141 @@ mseread(dev, uio, ioflag)
 		sc->sc_bytes[1] = sc->sc_deltax;
 		sc->sc_bytes[2] = sc->sc_deltay;
 		sc->sc_bytes[3] = sc->sc_bytes[4] = 0;
+		sc->sc_bytes[5] = sc->sc_bytes[6] = 0;
+		sc->sc_bytes[7] = MOUSE_SYS_EXTBUTTONS;
 		sc->sc_obuttons = sc->sc_buttons;
 		sc->sc_deltax = sc->sc_deltay = 0;
 		sc->sc_bytesread = 0;
 	}
 	splx(s);
-	xfer = min(uio->uio_resid, PROTOBYTES - sc->sc_bytesread);
+	xfer = min(uio->uio_resid, sc->mode.packetsize - sc->sc_bytesread);
 	if (error = uiomove(&sc->sc_bytes[sc->sc_bytesread], xfer, uio))
 		return (error);
 	sc->sc_bytesread += xfer;
 	return(0);
 }
 
+/*
+ * mseioctl: process ioctl commands.
+ */
+static int
+mseioctl(dev, cmd, addr, flag, p)
+	dev_t dev;
+	int cmd;
+	caddr_t addr;
+	int flag;
+	struct proc *p;
+{
+	register struct mse_softc *sc = &mse_sc[MSE_UNIT(dev)];
+	mousestatus_t status;
+	int err = 0;
+	int s;
+
+	switch (cmd) {
+
+	case MOUSE_GETHWINFO:
+		s = spltty();
+		*(mousehw_t *)addr = sc->hw;
+		if (sc->mode.level == 0)
+			((mousehw_t *)addr)->model = MOUSE_MODEL_GENERIC;
+		splx(s);
+		break;
+
+	case MOUSE_GETMODE:
+		s = spltty();
+		*(mousemode_t *)addr = sc->mode;
+		switch (sc->mode.level) {
+		case 0:
+			break;
+		case 1:
+			((mousemode_t *)addr)->protocol = MOUSE_PROTO_SYSMOUSE;
+	    		((mousemode_t *)addr)->syncmask[0] = MOUSE_SYS_SYNCMASK;
+	    		((mousemode_t *)addr)->syncmask[1] = MOUSE_SYS_SYNC;
+			break;
+		}
+		splx(s);
+		break;
+
+	case MOUSE_SETMODE:
+		switch (((mousemode_t *)addr)->level) {
+		case 0:
+		case 1:
+			break;
+		default:
+			return (EINVAL);
+		}
+		if (((mousemode_t *)addr)->accelfactor < -1)
+			return (EINVAL);
+		else if (((mousemode_t *)addr)->accelfactor >= 0)
+			sc->mode.accelfactor = 
+			    ((mousemode_t *)addr)->accelfactor;
+		sc->mode.level = ((mousemode_t *)addr)->level;
+		switch (sc->mode.level) {
+		case 0:
+			sc->sc_bytesread = sc->mode.packetsize 
+			    = MOUSE_MSC_PACKETSIZE;
+			break;
+		case 1:
+			sc->sc_bytesread = sc->mode.packetsize 
+			    = MOUSE_SYS_PACKETSIZE;
+			break;
+		}
+		break;
+
+	case MOUSE_GETLEVEL:
+		*(int *)addr = sc->mode.level;
+		break;
+
+	case MOUSE_SETLEVEL:
+		switch (*(int *)addr) {
+		case 0:
+			sc->mode.level = *(int *)addr;
+			sc->sc_bytesread = sc->mode.packetsize 
+			    = MOUSE_MSC_PACKETSIZE;
+			break;
+		case 1:
+			sc->mode.level = *(int *)addr;
+			sc->sc_bytesread = sc->mode.packetsize 
+			    = MOUSE_SYS_PACKETSIZE;
+			break;
+		default:
+			return (EINVAL);
+		}
+		break;
+
+	case MOUSE_GETSTATUS:
+		s = spltty();
+		status = sc->status;
+		sc->status.flags = 0;
+		sc->status.obutton = sc->status.button;
+		sc->status.button = 0;
+		sc->status.dx = 0;
+		sc->status.dy = 0;
+		sc->status.dz = 0;
+		splx(s);
+		*(mousestatus_t *)addr = status;
+		break;
+
+	case MOUSE_READSTATE:
+	case MOUSE_READDATA:
+		return (ENODEV);
+
+#if (defined(MOUSE_GETVARS))
+	case MOUSE_GETVARS:
+	case MOUSE_SETVARS:
+		return (ENODEV);
+#endif
+
+	default:
+		return (ENOTTY);
+	}
+	return (err);
+}
+
 /*
  * mseselect: check for mouse input to be processed.
  */
-int
+static	int
 mseselect(dev, rw, p)
 	dev_t dev;
 	int rw;
@@ -369,7 +524,7 @@ mseselect(dev, rw, p)
 	int s;
 
 	s = spltty();
-	if (sc->sc_bytesread != PROTOBYTES || sc->sc_deltax != 0 ||
+	if (sc->sc_bytesread != sc->mode.packetsize || sc->sc_deltax != 0 ||
 	    sc->sc_deltay != 0 || (sc->sc_obuttons ^ sc->sc_buttons) != 0) {
 		splx(s);
 		return (1);
@@ -391,8 +546,23 @@ void
 mseintr(unit)
 	int unit;
 {
+	/*
+	 * the table to turn MouseSystem button bits (MOUSE_MSC_BUTTON?UP)
+	 * into `mousestatus' button bits (MOUSE_BUTTON?DOWN).
+	 */
+	static int butmap[8] = {
+		0, 
+		MOUSE_BUTTON3DOWN, 
+		MOUSE_BUTTON2DOWN, 
+		MOUSE_BUTTON2DOWN | MOUSE_BUTTON3DOWN, 
+		MOUSE_BUTTON1DOWN, 
+		MOUSE_BUTTON1DOWN | MOUSE_BUTTON3DOWN, 
+		MOUSE_BUTTON1DOWN | MOUSE_BUTTON2DOWN,
+        	MOUSE_BUTTON1DOWN | MOUSE_BUTTON2DOWN | MOUSE_BUTTON3DOWN
+	};
 	register struct mse_softc *sc = &mse_sc[unit];
-	pid_t p;
+	int dx, dy, but;
+	int sign;
 
 #ifdef DEBUG
 	static int mse_intrcnt = 0;
@@ -402,7 +572,31 @@ mseintr(unit)
 	if ((sc->sc_flags & MSESC_OPEN) == 0)
 		return;
 
-	(*sc->sc_getmouse)(sc->sc_port, &sc->sc_deltax, &sc->sc_deltay, &sc->sc_buttons);
+	(*sc->sc_getmouse)(sc->sc_port, &dx, &dy, &but);
+	if (sc->mode.accelfactor > 0) {
+		sign = (dx < 0);
+		dx = dx * dx / sc->mode.accelfactor;
+		if (dx == 0)
+			dx = 1;
+		if (sign)
+			dx = -dx;
+		sign = (dy < 0);
+		dy = dy * dy / sc->mode.accelfactor;
+		if (dy == 0)
+			dy = 1;
+		if (sign)
+			dy = -dy;
+	}
+	sc->sc_deltax += dx;
+	sc->sc_deltay += dy;
+	sc->sc_buttons = but;
+
+	but = butmap[~but & MOUSE_MSC_BUTTONS];
+	sc->status.dx += dx;
+	sc->status.dy += dy;
+	sc->status.flags |= ((dx || dy) ? MOUSE_POSCHANGED : 0)
+	    | (sc->status.button ^ but);
+	sc->status.button = but;
 
 	/*
 	 * If mouse state has changed, wake up anyone wanting to know.
@@ -486,7 +680,7 @@ mse_getlogi(port, dx, dy, but)
 
 	outb(port + MSE_PORTC, MSE_HOLD | MSE_RXLOW);
 	x = inb(port + MSE_PORTA);
-	*but = (x >> 5) & 0x7;
+	*but = (x >> 5) & MOUSE_MSC_BUTTONS;
 	x &= 0xf;
 	outb(port + MSE_PORTC, MSE_HOLD | MSE_RXHIGH);
 	x |= (inb(port + MSE_PORTA) << 4);
@@ -494,8 +688,8 @@ mse_getlogi(port, dx, dy, but)
 	y = (inb(port + MSE_PORTA) & 0xf);
 	outb(port + MSE_PORTC, MSE_HOLD | MSE_RYHIGH);
 	y |= (inb(port + MSE_PORTA) << 4);
-	*dx += x;
-	*dy += y;
+	*dx = x;
+	*dy = y;
 	outb(port + MSE_PORTC, MSE_INTREN);
 }
 
@@ -558,14 +752,31 @@ mse_getati(port, dx, dy, but)
 	outb(port + MSE_PORTA, MSE_INPORT_MODE);
 	outb(port + MSE_PORTB, MSE_INPORT_HOLD);
 	outb(port + MSE_PORTA, MSE_INPORT_STATUS);
-	*but = ~(inb(port + MSE_PORTB) & 0x7);
+	*but = ~inb(port + MSE_PORTB) & MOUSE_MSC_BUTTONS;
 	outb(port + MSE_PORTA, MSE_INPORT_DX);
 	byte = inb(port + MSE_PORTB);
-	*dx += byte;
+	*dx = byte;
 	outb(port + MSE_PORTA, MSE_INPORT_DY);
 	byte = inb(port + MSE_PORTB);
-	*dy += byte;
+	*dy = byte;
 	outb(port + MSE_PORTA, MSE_INPORT_MODE);
 	outb(port + MSE_PORTB, MSE_INPORT_INTREN);
 }
+
+static mse_devsw_installed = 0;
+
+static void 	mse_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! mse_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&mse_cdevsw, NULL);
+		mse_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(msedev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,mse_drvinit,NULL)
+
+
 #endif /* NMSE */
