@@ -45,8 +45,6 @@
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
-#include <sys/devconf.h>
 
 #include <machine/clock.h>
 
@@ -131,7 +129,7 @@ typedef struct scb {
 	struct scb *next;               /* in free list */
 	struct scsi_xfer *xfer;		/* the scsi_xfer for this cmd */
 	u_char *data;                   /* position in data buffer so far */
-	int32_t datalen;		/* bytes remaining to transfer */;
+	int32_t datalen;			/* bytes remaining to transfer */;
 } scb_t;
 
 typedef enum {
@@ -196,7 +194,7 @@ typedef struct {
 	target_t target[8];             /* target state data */
 } adapter_t;
 
-adapter_t ncadata[NNCA];
+static adapter_t ncadata[NNCA];
 
 #define IS_BUSY(a,b)    ((a)->target[(b)->xfer->sc_link->target].busy &\
 				(1 << (b)->xfer->sc_link->lun))
@@ -215,7 +213,6 @@ adapter_t ncadata[NNCA];
 	if (cnt == -1 && msg)\
 		printf ("nca: %s timeout\n", msg); }
 
-int ncaintr (int unit);
 static int nca_probe (struct isa_device *dev);
 static int nca_attach (struct isa_device *dev);
 static int32_t nca_scsi_cmd (struct scsi_xfer *xs);
@@ -246,14 +243,6 @@ static struct scsi_adapter nca_switch = {
 static struct scsi_device nca_dev = { NULL, NULL, NULL, NULL, "nca", 0, {0} };
 struct isa_driver ncadriver = { nca_probe, nca_attach, "nca" };
 
-static char nca_description [80];
-static struct kern_devconf nca_kdc[NNCA] = {{
-	0, 0, 0, "nca", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN, &kdc_isa0, 0,
-	DC_UNCONFIGURED, nca_description,
-	DC_CLS_MISC		/* host adapters aren't special */
-}};
-
 /*
  * Check if the device can be found at the port given and if so,
  * detect the type of board. Set it up ready for further work.
@@ -265,12 +254,6 @@ int nca_probe (struct isa_device *dev)
 	adapter_t *z = &ncadata[dev->id_unit];
 	int i;
 
-	if (dev->id_unit)
-		nca_kdc[dev->id_unit] = nca_kdc[0];
-	nca_kdc[dev->id_unit].kdc_unit = dev->id_unit;
-	nca_kdc[dev->id_unit].kdc_isa = dev;
-	dev_attach (&nca_kdc[dev->id_unit]);
-
 	/* Init fields used by our routines */
 	z->parity = (dev->id_flags & FLAG_NOPARITY) ? 0 :
 		MR_ENABLE_PARITY_CHECKING;
@@ -497,13 +480,13 @@ int nca_attach (struct isa_device *dev)
 	adapter_t *z = &ncadata[unit];
 	struct scsibus_data *scbus;
 
-	sprintf (nca_description, "%s SCSI controller", z->name);
 	printf ("nca%d: type %s%s\n", unit, z->name,
 		(dev->id_flags & FLAG_NOPARITY) ? ", no parity" : "");
 
 	/* fill in the prototype scsi_link */
 	z->sc_link.adapter_unit = unit;
 	z->sc_link.adapter_targ = z->scsi_addr;
+	z->sc_link.adapter_softc = z;
 	z->sc_link.adapter = &nca_switch;
 	z->sc_link.device = &nca_dev;
 
@@ -517,7 +500,6 @@ int nca_attach (struct isa_device *dev)
 	scbus->adapter_link = &z->sc_link;
 
 	/* ask the adapter what subunits are present */
-	nca_kdc[unit].kdc_state = DC_BUSY;
 	scsi_attachdevs (scbus);
 
 	return (1);
@@ -539,7 +521,7 @@ void ncaminphys (struct buf *bp)
 /*
  * Catch an interrupt from the adaptor.
  */
-int ncaintr (int unit)
+void ncaintr (int unit)
 {
 	adapter_t *z = &ncadata[unit];
 
@@ -548,7 +530,6 @@ int ncaintr (int unit)
 	nca_start (z);
 	/* Reset interrupt state. */
 	inb (z->RPIR);
-	return (1);
 }
 
 /*
@@ -579,7 +560,7 @@ void nca_tick (void *arg)
 int32_t nca_scsi_cmd (struct scsi_xfer *xs)
 {
 	int unit = xs->sc_link->adapter_unit, flags = xs->flags, x = 0;
-	adapter_t *z = &ncadata[unit];
+	adapter_t *z = (adapter_t *)xs->sc_link->adapter_softc;
 	scb_t *scb;
 
 	/* PRINT (("nca%d/%d/%d command 0x%x\n", unit, xs->sc_link->target,
@@ -717,7 +698,7 @@ void nca_timeout (void *arg)
 {
 	scb_t *scb = (scb_t*) arg;
 	int unit = scb->xfer->sc_link->adapter_unit;
-	adapter_t *z = &ncadata[unit];
+	adapter_t *z = (adapter_t *)scb->xfer->sc_link->adapter_softc;
 	int x = splbio ();
 
 	if (! (scb->xfer->flags & SCSI_NOMASK))
