@@ -70,7 +70,9 @@
 #include "vt.h"
 #if NVT > 0
 
-#include "pcvt_hdr.h"		/* global include */
+#include <i386/isa/pcvt/pcvt_hdr.h>	/* global include */
+
+#define ISSIGVALID(sig) ((sig) > 0 && (sig) < NSIG)
 
 static int  s3testwritable( void );
 static int  et4000_col( int );
@@ -2505,6 +2507,13 @@ usl_vt_ioctl(Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 			return EBUSY; /* already in use on this VT */
 		}
 
+		if (!ISSIGVALID(newmode.relsig) || !ISSIGVALID(newmode.acqsig)
+		    || !ISSIGVALID(newmode.frsig))
+		{
+			splx(opri);
+			return EINVAL;
+		}
+
 		vsp->smode = newmode;
 		vsp->proc = p;
 		vsp->pid = p->p_pid;
@@ -2592,8 +2601,8 @@ usl_vt_ioctl(Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 					if(pcvt_is_console)
 						cons_unavail = 0;
 #endif
-					return 0;
 				}
+				return 0;
 			}
 			break;
 
@@ -2642,20 +2651,8 @@ usl_vt_ioctl(Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 			return 0;
 
 		if(i == -1)
-		{
-			/* xxx Is this what it is supposed to do? */
-			int x = spltty();
-			i = current_video_screen;
-			error = 0;
-			while (current_video_screen == i &&
-			       (error == 0 || error == ERESTART)) {
-				vs[i].vt_status |= VT_WAIT_ACT;
-				error = tsleep((caddr_t)&vs[i].smode,
-					       PZERO | PCATCH, "waitvt", 0);
-			}
-			splx(x);
-		}
-		else
+			i = minor(dev);
+
 		{
 			int x = spltty();
 			error = 0;
@@ -2672,6 +2669,7 @@ usl_vt_ioctl(Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 
 	case KDENABIO:
 		/* grant the process IO access; only allowed if euid == 0 */
+		/* and insecure */
 	{
 
 #if PCVT_NETBSD > 9 || PCVT_FREEBSD >= 200
@@ -2682,7 +2680,10 @@ usl_vt_ioctl(Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 		struct syscframe *fp = (struct syscframe *)p->p_regs;
 #endif
 
-		if(suser(p->p_ucred, &p->p_acflag) != 0)
+		error = suser(p->p_ucred, &p->p_acflag);
+		if (error != 0)
+			return (error);
+		if (securelevel > 0)
 			return (EPERM);
 
 #if PCVT_NETBSD || (PCVT_FREEBSD && PCVT_FREEBSD > 102)
