@@ -66,13 +66,17 @@
 #define	PCVT_REL "3.20-b24"	/* driver attach announcement	*/
 				/* see also: pcvt_ioctl.h	*/
 
+#include "opt_pcvt.h"
+#if defined(__FreeBSD__) && !defined(PCVT_FREEBSD)
+#  define PCVT_FREEBSD 210
+#endif
+
 #if PCVT_FREEBSD >= 200
 
 #include <sys/param.h>
 #include <sys/conf.h>
-#include <sys/ioctl.h>
 #include <sys/proc.h>
-#include <sys/user.h>
+#include <sys/signalvar.h>
 #include <sys/tty.h>
 #include <sys/uio.h>
 #include <sys/callout.h>
@@ -83,14 +87,14 @@
 #include <sys/time.h>
 #if PCVT_FREEBSD > 210
 #include <machine/random.h>
-#endif  /* PCVT_FREEBSD > 210 */
+#endif	/* PCVT_FREEBSD > 210 */
 #else /* ! PCVT_FREEBSD >= 200 */
 
 #include "param.h"
 #include "conf.h"
 #include "ioctl.h"
 #include "proc.h"
-#include "user.h"
+#include "signalvar.h"
 #include "tty.h"
 #include "uio.h"
 #include "callout.h"
@@ -102,7 +106,8 @@
 
 #endif /* PCVT_FREEBSD >= 200 */
 
-#include "pcvt_conf.h"
+#include <i386/isa/pcvt/pcvt_conf.h>
+#include <i386/isa/kbdio.h>
 
 #if PCVT_NETBSD > 9
 #include "device.h"
@@ -134,7 +139,7 @@
 #if PCVT_NETBSD > 9
 #include "dev/cons.h"
 #elif PCVT_FREEBSD >= 200
-#include <i386/i386/cons.h>
+#include <machine/cons.h>
 #else
 #include "i386/i386/cons.h"
 #endif
@@ -149,14 +154,8 @@
 #endif /* PCVT_FREEBSD >= 200 */
 #endif /* PCVT_NETBSD <= 9 */
 
-#if PCVT_FREEBSD >= 200
-#include <machine/stdarg.h>
-#else
-#include "machine/stdarg.h"
-#endif
-
 #if PCVT_NETBSD > 9
-#include "i386/isa/pcvt/pcvt_ioctl.h"
+#include <i386/isa/pcvt/pcvt_ioctl.h>
 #elif PCVT_FREEBSD >= 200
 #include <machine/pcvt_ioctl.h>
 #else
@@ -169,16 +168,10 @@
 #include <machine/clock.h>
 #include <machine/md_var.h>
 #endif
-#include <vm/vm_kern.h>
 #else /* PCVT_FREEBSD >= 200 */
 #include "machine/pc/display.h"
-#include "vm/vm_kern.h"
 #endif /* PCVT_FREEBSD >= 200 */
 
-#if PCVT_FREEBSD > 205
-#include <sys/devconf.h>
-#endif
-
 /* setup irq disable function to use */
 
 #if !(PCVT_SLOW_INTERRUPT) && (PCVT_NETBSD > 9)
@@ -267,6 +260,8 @@ in the config file"
  *	Keyboard and Keyboard Controller
  *---------------------------------------------------------------------------*/
 
+#ifndef _I386_ISA_KBDIO_H_
+
 #define CONTROLLER_CTRL	0x64	/* W - command, R - status	*/
 #define CONTROLLER_DATA	0x60	/* R/W - data			*/
 
@@ -313,6 +308,8 @@ in the config file"
 #define KEYB_C_ECHO	0xee	/* diagnostic, echo 0xee */
 #define KEYB_C_LEDS	0xed	/* set/reset numlock,capslock & scroll lock */
 
+#endif /* _I386_ISA_KBDIO_H_ */
+
 /* responses from the KEYBOARD (via the 8042 controller on mainboard..) */
 
 #define	KEYB_R_OVERRUN0	0x00	/* keyboard buffer overflow */
@@ -731,9 +728,9 @@ in the config file"
 
 /* arguments to async_update() */
 
-#define UPDATE_START	0	/* do cursor update and requeue */
-#define UPDATE_STOP	1	/* suspend cursor updates */
-#define UPDATE_KERN	2	/* do cursor updates for kernel output */
+#define UPDATE_START	((void *)0)	/* do cursor update and requeue */
+#define UPDATE_STOP	((void *)1)	/* suspend cursor updates */
+#define UPDATE_KERN	((void *)2)	/* do cursor updates for kernel output */
 
 /* variables */
 
@@ -825,8 +822,8 @@ typedef struct video_state {
 	u_short	*sc_G1;			/* save G1 ptr */
 	u_short	*sc_G2;			/* save G2 ptr */
 	u_short	*sc_G3;			/* save G3 ptr */
-	u_short	*sc_GL;			/* save GL ptr */
-	u_short	*sc_GR;			/* save GR ptr */
+	u_short	**sc_GL;		/* save GL ptr */
+	u_short	**sc_GR;		/* save GR ptr */
 	u_char	sc_sel;			/* selective erase state */
 	u_char	ufkl[8][17];		/* user fkey-labels */
 	u_char	sfkl[8][17];		/* system fkey-labels */
@@ -866,8 +863,8 @@ typedef struct video_state {
 	struct sixels sixel;		/* structure for storing char sixels */
 	u_char	selchar;		/* true = selective attribute on */
 	u_int	decsca[MAXDECSCA];	/* Select Character Attrib bit array */
-	u_short *GL;			/* ptr to current GL conversion table*/
-	u_short *GR;			/* ptr to current GR conversion table*/
+	u_short **GL;			/* ptr to current GL conversion table*/
+	u_short **GR;			/* ptr to current GR conversion table*/
 	u_short *G0;			/* ptr to current G0 conversion table*/
 	u_short *G1;			/* ptr to current G1 conversion table*/
 	u_char force24;			/* force 24 lines in DEC 25 and HP 28*/
@@ -877,7 +874,7 @@ typedef struct video_state {
 	u_char	which[DSCS_LENGTH+1];	/* which set to designate */
 	u_char	whichi;			/* index into which ..	*/
 	u_char  ss;			/* flag, single shift G2 / G3 -> GL */
-	u_short *Gs;			/* ptr to cur. G2/G3 conversion table*/
+	u_short **Gs;			/* ptr to cur. G2/G3 conversion table*/
 	u_char	udkbuf[MAXUDKDEF];	/* buffer for user defined keys */
 	struct udkentry ukt;		/* index & length for each udk */
 	u_char	udkff;			/* index into buffer first free entry*/
@@ -945,7 +942,7 @@ int pcprobe ( struct isa_device *dev );
 int pcattach ( struct isa_device *dev );
 
 struct	isa_driver vtdriver = {		/* driver routines */
-	pcprobe, pcattach, "vt",
+	pcprobe, pcattach, "vt", 1,
 };
 
 #endif /* PCVT_NETBSD > 9 */
@@ -961,7 +958,8 @@ u_char bgansitopc[] = {			/* background ANSI color -> pc */
 };
 
 #if !PCVT_NETBSD
-u_short *Crtat	=	(u_short *)MONO_BUF;	/* screen start address */
+/* XXX Crtat is shared with syscons. */
+u_short *Crtat;			/* screen start address */
 #if !(PCVT_FREEBSD > 110 && PCVT_FREEBSD < 200)
 struct tty *pcconsp =	&pccons[0];		/* ptr to current device */
 #else /* PCVT_FREEBSD > 110 */
@@ -1002,6 +1000,10 @@ u_char	chargen_access		= 0;		/* synchronize access */
 u_char	keyboard_type		= KB_UNKNOWN;	/* type of keyboard */
 u_char	keyboard_is_initialized = 0;		/* for ddb sanity */
 u_char	kbd_polling		= 0;		/* keyboard is being polled */
+#ifdef _I386_ISA_KBDIO_H_
+u_char	reset_keyboard		= 0;		/* OK to reset keyboard */
+KBDC	kbdc			= NULL;		/* keyboard controller */
+#endif /* _I386_ISA_KBDIO_H_ */
 
 #if PCVT_SHOWKEYS
 u_char	keyboard_show		= 0;		/* normal display */
@@ -1120,7 +1122,6 @@ extern int		vt_switch_pending;
 
 extern u_int		addr_6845;
 extern u_short		*Crtat;
-extern struct isa_driver vtdriver;
 extern u_char		do_initialization;
 extern u_char		pcvt_is_console;
 extern u_char		bgansitopc[];
@@ -1140,6 +1141,10 @@ extern u_char		can_do_132col;
 extern u_char		vga_family;
 extern u_char		keyboard_is_initialized;
 extern u_char		kbd_polling;
+#ifdef _I386_ISA_KBDIO_H_
+extern u_char		reset_keyboard;
+extern KBDC		kbdc;
+#endif /* _I386_ISA_KBDIO_H_ */
 
 #if PCVT_SHOWKEYS
 extern u_char		keyboard_show;
@@ -1196,14 +1201,6 @@ extern u_char		*saved_charsets[NVGAFONTS];
 #define Dev_t	dev_t
 #endif
 
-/* in FreeBSD > 102 arguments for timeout()/untimeout() are a special type */
-
-#if PCVT_FREEBSD > 102
-#define	TIMEOUT_FUNC_T	timeout_func_t
-#else
-#define	TIMEOUT_FUNC_T	void *
-#endif
-
 #if !PCVT_FREEBSD || (PCVT_FREEBSD < 210)
 extern void bcopyb(void *from, void *to, u_int length);
 #endif
@@ -1212,20 +1209,10 @@ extern void bcopyb(void *from, void *to, u_int length);
 extern void fillw(U_short value, void *addr, u_int length);
 #endif
 
-int	pcopen ( Dev_t dev, int flag, int mode, struct proc *p );
-int	pcclose ( Dev_t dev, int flag, int mode, struct proc *p );
-int	pcread ( Dev_t dev, struct uio *uio, int flag );
-int	pcwrite ( Dev_t dev, struct uio *uio, int flag );
-int	pcioctl ( Dev_t dev, int cmd, caddr_t data, int flag, struct proc *p );
-int	pcmmap ( Dev_t dev, int offset, int nprot );
-#if PCVT_FREEBSD > 205
-struct tty *pcdevtotty ( Dev_t dev );
-#endif /* PCVT_FREEBSD > 205 */
-int	pcrint ( void );
 int	pcparam ( struct tty *tp, struct termios *t );
 
 /*
- * In FreeBSD > 2.0.6, driver console functions are declared in i386/cons.h
+ * In FreeBSD > 2.0.6, driver console functions are declared in machine/cons.h
  * and some return void, so don't declare them here.
  */
 #if PCVT_FREEBSD <= 205
@@ -1267,11 +1254,9 @@ void 	pcvt_scrnsv_reset ( void );
 void 	pcvt_set_scrnsv_tmo ( int );
 #endif /* PCVT_SCREENSAVER && defined(XSERVER) */
 
-#ifdef XSERVER
 void	vga_move_charset ( unsigned n, unsigned char *b, int save_it);
-#endif /* XSERVER */
 
-void	async_update ( int arg );
+void	async_update ( void *arg );
 void	clr_parms ( struct video_state *svsp );
 void	cons_highlight ( void );
 void	cons_normal ( void );
@@ -1282,7 +1267,10 @@ void	fkl_on ( struct video_state *svsp );
 struct tty *get_pccons ( Dev_t dev );
 void	init_sfkl ( struct video_state *svsp );
 void	init_ufkl ( struct video_state *svsp );
+#ifndef _I386_ISA_KBDIO_H_
 int	kbd_cmd ( int val );
+int	kbd_response ( void );
+#endif /* _I386_ISA_KBDIO_H_ */
 void	kbd_code_init ( void );
 void	kbd_code_init1 ( void );
 
@@ -1408,12 +1396,12 @@ static __inline void vt_selattr(struct video_state *svsp)
 				/* 0x84 to produce keyboard controller    */
 				/* access delays                          */
 #define PCVT_KBD_DELAY()          \
-	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); }
+	{ (void)inb(0x84); } \
+	{ (void)inb(0x84); } \
+	{ (void)inb(0x84); } \
+	{ (void)inb(0x84); } \
+	{ (void)inb(0x84); } \
+	{ (void)inb(0x84); }
 
 #else /* PCVT_PORTIO_DELAY */
 				/* use system supplied delay function for */
