@@ -67,9 +67,13 @@
  *---------------------------------------------------------------------------*/
 
 #include "vt.h"
+#include "opt_ddb.h"
+
 #if NVT > 0
 
-#include "pcvt_hdr.h"		/* global include */
+#include <i386/isa/pcvt/pcvt_hdr.h>	/* global include */
+
+#define LEDSTATE_UPDATE_PENDING	(1 << 3)
 
 static void fkey1(void), fkey2(void),  fkey3(void),  fkey4(void);
 static void fkey5(void), fkey6(void),  fkey7(void),  fkey8(void);
@@ -87,14 +91,18 @@ static void	doreset ( void );
 static void	ovlinit ( int force );
 static void 	settpmrate ( int rate );
 static void	setlockkeys ( int snc );
+#ifndef _I386_ISA_KBDIO_H_
 static int	kbc_8042cmd ( int val );
+#else
+static int 	set_keyboard_param( int command, int data );
+#endif /* !_I386_ISA_KBDIO_H_ */
 static int	getokeydef ( unsigned key, struct kbd_ovlkey *thisdef );
 static int 	getckeydef ( unsigned key, struct kbd_ovlkey *thisdef );
 static int	rmkeydef ( int key );
 static int	setkeydef ( struct kbd_ovlkey *data );
 static u_char *	xlatkey2ascii( U_short key );
 
-static int	ledstate  = 0;	/* keyboard led's */
+static int	ledstate  = LEDSTATE_UPDATE_PENDING;	/* keyboard led's */
 static int	tpmrate   = KBD_TPD500|KBD_TPM100;
 static u_char	altkpflag = 0;
 static u_short	altkpval  = 0;
@@ -103,7 +111,7 @@ static u_short	altkpval  = 0;
 u_char rawkeybuf[80];
 #endif
 
-#include "pcvt_kbd.h"		/* tables etc */
+#include <i386/isa/pcvt/pcvt_kbd.h>	/* tables etc */
 
 #if PCVT_SHOWKEYS
 /*---------------------------------------------------------------------------*
@@ -162,7 +170,7 @@ do_vgapage(int page)
 
 #define PCVT_UPDLED_LOSES_INTR	0	/* disabled for now */
 
-#if PCVT_UPDLED_LOSES_INTR
+#if PCVT_UPDLED_LOSES_INTR || defined(_I386_ISA_KBDIO_H_)
 
 /*---------------------------------------------------------------------------*
  *	check for lost keyboard interrupts
@@ -175,6 +183,9 @@ do_vgapage(int page)
  * the interrupts sometimes gets lost (I'm not sure of the details of
  * how and why and what hardware this happens with).
  *
+ * This may have had something to do with spltty() previously not being
+ * called before the kbd_cmd() calls in update_led().
+ *
  * This is a real problem, because normally the keyboard is only polled
  * by pcrint(), and no more interrupts will be generated until the ACK
  * has been read.  So the keyboard is hung.  This code polls a little
@@ -190,16 +201,31 @@ static int lost_intr_timeout_queued = 0;
 static void
 check_for_lost_intr (void *arg)
 {
+#ifndef _I386_ISA_KBDIO_H_
 	lost_intr_timeout_queued = 0;
 	if (inb(CONTROLLER_CTRL) & STATUS_OUTPBF)
 	{
 		int opri = spltty ();
-		(void) pcrint ();
+		pcrint ();
 		splx (opri);
 	}
+#else
+	int opri;
+
+	lost_intr_timeout_queued = 0;
+	if (kbdc_lock(kbdc, TRUE)) {
+		opri = spltty ();
+		kbdc_lock(kbdc, FALSE);
+		if (kbdc_data_ready(kbdc)) 
+			pcrint (0);
+		splx (opri);
+	}
+	timeout(check_for_lost_intr, (void *)NULL, hz);
+	lost_intr_timeout_queued = 1;
+#endif /* !_I386_ISA_KBDIO_H_ */
 }
 
-#endif /* PCVT_UPDLED_LOSES_INTR */
+#endif /* PCVT_UPDLED_LOSES_INTR || defined(_I386_ISA_KBDIO_H_) */
 
 /*---------------------------------------------------------------------------*
  *	update keyboard led's
@@ -207,39 +233,94 @@ check_for_lost_intr (void *arg)
 void
 update_led(void)
 {
-
 #if !PCVT_NO_LED_UPDATE
 
 	/* Don't update LED's unless necessary. */
 
-	int new_ledstate = ((vsp->scroll_lock) |
-			    (vsp->num_lock * 2) |
-			    (vsp->caps_lock * 4));
+	int opri, new_ledstate, response1, response2;
+
+	opri = spltty();
+	new_ledstate = (vsp->scroll_lock) |
+		       (vsp->num_lock * 2) |
+		       (vsp->caps_lock * 4);
 
 	if (new_ledstate != ledstate)
 	{
+#ifndef _I386_ISA_KBDIO_H_
+
+		ledstate = LEDSTATE_UPDATE_PENDING;
+
 		if(kbd_cmd(KEYB_C_LEDS) != 0)
 		{
 			printf("Keyboard LED command timeout\n");
+			splx(opri);
 			return;
 		}
 
+		/*
+		 * For some keyboards or keyboard controllers, it is an
+		 * error to issue a command without waiting long enough
+		 * for an ACK for the previous command.  The keyboard
+		 * gets confused, and responds with KEYB_R_RESEND, but
+		 * we ignore that.  Wait for the ACK here.  The busy
+		 * waiting doesn't matter much, since we lose anyway by
+		 * busy waiting to send the command.
+		 *
+		 * XXX actually wait for any response, since we can't
+		 * handle normal scancodes here.
+		 *
+		 * XXX all this should be interrupt driven.  Issue only
+		 * one command at a time wait for a ACK before proceeding.
+		 * Retry after a timeout or on receipt of a KEYB_R_RESEND.
+		 * KEYB_R_RESENDs seem to be guaranteed by working
+		 * keyboard controllers with broken (or disconnected)
+		 * keyboards.  There is another code for keyboard
+		 * reconnects.  The keyboard hardware is very simple and
+		 * well designed :-).
+		 */
+		response1 = kbd_response();
+
 		if(kbd_cmd(new_ledstate) != 0) {
 			printf("Keyboard LED data timeout\n");
+			splx(opri);
 			return;
 		}
+		response2 = kbd_response();
 
-		ledstate = new_ledstate;
+		if (response1 == KEYB_R_ACK && response2 == KEYB_R_ACK)
+			ledstate = new_ledstate;
+		else
+			printf(
+			"Keyboard LED command not ACKed (responses %#x %#x)\n",
+			       response1, response2);
+#else /* _I386_ISA_KBDIO_H_ */
+
+		if (kbdc == NULL) {
+			ledstate = new_ledstate;
+			splx(opri);
+		} else {
+			ledstate = LEDSTATE_UPDATE_PENDING;
+			splx(opri);
+			if (set_keyboard_param(KBDC_SET_LEDS, new_ledstate) == 0) 
+				ledstate = new_ledstate;
+		}
+
+#endif /* !_I386_ISA_KBDIO_H_ */
 
 #if PCVT_UPDLED_LOSES_INTR
 		if (lost_intr_timeout_queued)
-			untimeout (check_for_lost_intr, (void *)NULL);
+			untimeout(check_for_lost_intr, NULL);
 
-		timeout (check_for_lost_intr, (void *)NULL, hz);
+		timeout(check_for_lost_intr, NULL, hz);
 		lost_intr_timeout_queued = 1;
 #endif /* PCVT_UPDLED_LOSES_INTR */
 
 	}
+
+#ifndef _I386_ISA_KBDIO_H_
+	splx(opri);
+#endif 
+
 #endif /* !PCVT_NO_LED_UPDATE */
 }
 
@@ -249,13 +330,20 @@ update_led(void)
 static void
 settpmrate(int rate)
 {
+#ifndef _I386_ISA_KBDIO_H_
 	tpmrate = rate & 0x7f;
 	if(kbd_cmd(KEYB_C_TYPEM) != 0)
 		printf("Keyboard TYPEMATIC command timeout\n");
 	else if(kbd_cmd(tpmrate) != 0)
 		printf("Keyboard TYPEMATIC data timeout\n");
+#else
+	tpmrate = rate & 0x7f;
+	if (set_keyboard_param(KBDC_SET_TYPEMATIC, tpmrate) != 0)
+		printf("pcvt: failed to set keyboard TYPEMATIC.\n");
+#endif /* !_I386_ISA_KBDIO_H_ */
 }
 
+#ifndef _I386_ISA_KBDIO_H_
 /*---------------------------------------------------------------------------*
  *	Pass command to keyboard controller (8042)
  *---------------------------------------------------------------------------*/
@@ -317,6 +405,62 @@ kbd_response(void)
 
 	return ch;
 }
+#else
+static int
+set_keyboard_param(int command, int data)
+{
+    int s;
+    int c;
+
+    if (kbdc == NULL)
+	return 1;
+
+    /* prevent the timeout routine from polling the keyboard */
+    if (!kbdc_lock(kbdc, TRUE)) 
+	return 1;
+
+    /* disable the keyboard and mouse interrupt */
+    s = spltty();
+#if 0
+    c = get_controller_command_byte(kbdc);
+    if ((c == -1) 
+	|| !set_controller_command_byte(kbdc, 
+            kbdc_get_device_mask(kbdc),
+            KBD_DISABLE_KBD_PORT | KBD_DISABLE_KBD_INT
+                | KBD_DISABLE_AUX_PORT | KBD_DISABLE_AUX_INT)) {
+	/* CONTROLLER ERROR */
+        kbdc_lock(kbdc, FALSE);
+	splx(s);
+	return 1;
+    }
+    /* 
+     * Now that the keyboard controller is told not to generate 
+     * the keyboard and mouse interrupts, call `splx()' to allow 
+     * the other tty interrupts. The clock interrupt may also occur, 
+     * but the timeout routine (`scrn_timer()') will be blocked 
+     * by the lock flag set via `kbdc_lock()'
+     */
+    splx(s);
+#endif
+
+    if (send_kbd_command_and_data(kbdc, command, data) != KBD_ACK)
+        send_kbd_command(kbdc, KBDC_ENABLE_KBD);
+
+#if 0
+    /* restore the interrupts */
+    if (!set_controller_command_byte(kbdc,
+            kbdc_get_device_mask(kbdc),
+	    c & (KBD_KBD_CONTROL_BITS | KBD_AUX_CONTROL_BITS))) { 
+	/* CONTROLLER ERROR */
+    }
+#else
+    splx(s);
+#endif
+    kbdc_lock(kbdc, FALSE);
+
+    return 0;
+}
+#endif /* !_I386_ISA_KBDIO_H_ */
 
 #if PCVT_SCANSET > 1
 /*---------------------------------------------------------------------------*
@@ -325,6 +469,7 @@ kbd_response(void)
 void
 kbd_emulate_pc(int do_emulation)
 {
+#ifndef _I386_ISA_KBDIO_H_
 	int cmd, timeo = 10000;
 
 	cmd = COMMAND_SYSFLG|COMMAND_IRQEN; /* common base cmd */
@@ -341,6 +486,10 @@ kbd_emulate_pc(int do_emulation)
 		if (--timeo == 0)
 			break;
 	outb(CONTROLLER_DATA, cmd);
+#else
+	set_controller_command_byte(kbdc, KBD_TRANSLATION, 
+		(do_emulation) ? KBD_TRANSLATION : 0);
+#endif /* !_I386_ISA_KBDIO_H_ */
 }
 
 #endif /* PCVT_SCANSET > 1 */
@@ -356,6 +505,7 @@ kbd_emulate_pc(int do_emulation)
 static
 void doreset(void)
 {
+#ifndef _I386_ISA_KBDIO_H_
 	int again = 0;
 	int once = 0;
 	int response, opri;
@@ -388,17 +538,29 @@ void doreset(void)
 
 	/*
 	 * Discard any stale keyboard activity.  The 0.1 boot code isn't
-	 * very careful and sometimes leaves a KEYB_R_RESEND.
+	 * very careful and sometimes leaves a KEYB_R_RESEND.  Versions
+	 * between 1992 and Oct 1996 didn't have the delay and sometimes
+	 * left a KEYB_R_RESEND.
 	 */
-	while (inb(CONTROLLER_CTRL) & STATUS_OUTPBF)
-		kbd_response();
+	while (1) {
+		if (inb(CONTROLLER_CTRL) & STATUS_OUTPBF)
+			kbd_response();
+		else {
+			DELAY(10000);
+			if (!(inb(CONTROLLER_CTRL) & STATUS_OUTPBF))
+				break;
+		}
+	}
 
 	/* Start keyboard reset */
 
 	opri = spltty ();
 
 	if (kbd_cmd(KEYB_C_RESET) != 0)
+	{
 		printf("pcvt: doreset() - timeout for keyboard reset command\n");
+		outb(CONTROLLER_DATA, KEYB_C_RESET);	/* force */
+	}
 
 	/* Wait for the first response to reset and handle retries */
 	while ((response = kbd_response()) != KEYB_R_ACK)
@@ -409,11 +571,13 @@ void doreset(void)
 				printf("pcvt: doreset() - response != ack and response < 0 [one time only msg]\n");
 			response = KEYB_R_RESEND;
 		}
-		if (response == KEYB_R_RESEND)
+		else if (response == KEYB_R_RESEND)
 		{
 			if(!again)	/* print message only once ! */
 				printf("pcvt: doreset() - got KEYB_R_RESEND response ... [one time only msg]\n");
-
+		}
+		if (response == KEYB_R_RESEND)
+		{
 			if(++again > PCVT_NONRESP_KEYB_TRY)
 			{
 				printf("pcvt: doreset() - Caution - no PC keyboard detected!\n");
@@ -426,6 +590,7 @@ void doreset(void)
 			{
 				once++;		/* print message only once ! */
 				printf("pcvt: doreset() - timeout for loop keyboard reset command [one time only msg]\n");
+				outb(CONTROLLER_DATA, KEYB_C_RESET);	/* force */
 			}
 		}
 	}
@@ -501,6 +666,160 @@ r_entry:
 
 #endif /* PCVT_KEYBDID */
 
+#else /* _I386_ISA_KBDIO_H_ */
+	int c;
+	int m;
+	int s;
+
+	if (!reset_keyboard)	/* no, we are not ready to reset */
+		return;
+
+	if (lost_intr_timeout_queued) {
+		untimeout(check_for_lost_intr, (void *)NULL);
+		lost_intr_timeout_queued = 0;
+	}
+
+	if (kbdc == NULL)
+		kbdc = kbdc_open(IO_KBD);
+
+	if (!kbdc_lock(kbdc, TRUE))	/* strange, somebody got there first */
+		return;
+
+	/* remove any noise */
+	empty_both_buffers(kbdc, 10);
+
+	s = spltty();
+
+	/* save the current controller command byte */
+	m = kbdc_get_device_mask(kbdc) & ~KBD_KBD_CONTROL_BITS;
+	c = get_controller_command_byte(kbdc);
+	if (c == -1) {
+		/* CONTROLLER ERROR */
+		kbdc_set_device_mask(kbdc, m);
+		kbdc_lock(kbdc, FALSE);
+		kbdc = NULL;
+		splx(s);
+    		printf("pcvt: unable to get the command byte.\n");
+		return;
+	}
+
+#if PCVT_USEKBDSEC		/* security enabled */
+
+#  if PCVT_SCANSET == 2
+#    define KBDINITCMD 0
+#  else /* PCVT_SCANSET != 2 */
+#    define KBDINITCMD KBD_TRANSLATION
+#  endif /* PCVT_SCANSET == 2 */
+
+#else /* ! PCVT_USEKBDSEC */	/* security disabled */
+
+#  if PCVT_SCANSET == 2
+#    define KBDINITCMD KBD_OVERRIDE_KBD_LOCK
+#  else /* PCVT_SCANSET != 2 */
+#    define KBDINITCMD KBD_TRANSLATION | KBD_OVERRIDE_KBD_LOCK
+#  endif /* PCVT_SCANSET == 2 */
+
+#endif /* PCVT_USEKBDSEC */
+
+	/* disable the keyboard interrupt and the aux port and interrupt */
+        if (!set_controller_command_byte(kbdc,
+                KBD_KBD_CONTROL_BITS | KBD_TRANSLATION | KBD_OVERRIDE_KBD_LOCK,
+                KBD_ENABLE_KBD_PORT | KBD_DISABLE_KBD_INT | KBDINITCMD)) {
+    		/* CONTROLLER ERROR: there is very little we can do... */
+		kbdc_set_device_mask(kbdc, m);
+		kbdc_lock(kbdc, FALSE);
+		kbdc = NULL;
+		splx(s);
+    		printf("pcvt: unable to set the command byte.\n");
+		return;
+        }
+	splx(s);
+    
+        /* reset keyboard hardware */
+	ledstate = LEDSTATE_UPDATE_PENDING;
+        if (!reset_kbd(kbdc)) {
+		/* KEYBOARD ERROR */
+    		empty_both_buffers(kbdc, 10);
+    		test_controller(kbdc);
+    		test_kbd_port(kbdc);
+    		/* 
+		 * We could disable the keyboard port and interrupt now... 
+		 * but, the keyboard may still exist. 
+    		 */
+    		printf("pcvt: failed to reset the keyboard.\n"); 
+		/* try to restore the original command byte */
+		set_controller_command_byte(kbdc, 0xff, c);
+		kbdc_set_device_mask(kbdc, m);
+		kbdc_lock(kbdc, FALSE);
+		kbdc = NULL;
+		return;
+	}
+    
+#if PCVT_KEYBDID
+
+	keyboard_type = KB_UNKNOWN;
+	if (send_kbd_command(kbdc, KBDC_SEND_DEV_ID) == KBD_ACK) {
+		DELAY(10000);	/* 10msec delay */
+		switch (read_kbd_data(kbdc)) {
+		case KEYB_R_MF2ID1:
+			switch (read_kbd_data(kbdc)) {
+			case KEYB_R_MF2ID2:
+			case KEYB_R_MF2ID2HP:
+				keyboard_type = KB_MFII;
+				break;
+			case -1:
+			default:
+				break;
+			}
+			break;
+		case -1:
+			keyboard_type = KB_AT;
+			/* fall through */
+		default:
+			/* XXX: should we read the second byte? */
+    			empty_both_buffers(kbdc, 10);	/* XXX */
+			break;
+		}
+	} else {
+		/* 
+		 * The send ID command failed. This error is considered
+		 * benign, but may need recovery. 
+		 */ 
+    		empty_both_buffers(kbdc, 10);
+    		test_controller(kbdc);
+    		test_kbd_port(kbdc);
+	}
+
+#else /* PCVT_KEYBDID */
+
+	keyboard_type = KB_MFII;	/* force it .. */
+
+#endif /* PCVT_KEYBDID */
+
+	/* enable the keyboard port and intr. */
+	if (!set_controller_command_byte(kbdc, 
+        	KBD_KBD_CONTROL_BITS, 
+    	        KBD_ENABLE_KBD_PORT | KBD_ENABLE_KBD_INT)) {
+    		/* CONTROLLER ERROR 
+    		 * This is serious; we are left with the disabled 
+		 * keyboard intr. 
+    		 */
+    		printf("pcvt: failed to enable the keyboard port and intr.\n");
+		kbdc_set_device_mask(kbdc, m);
+		kbdc_lock(kbdc, FALSE);
+		kbdc = NULL;
+		return;
+	}
+    
+	kbdc_set_device_mask(kbdc, m | KBD_KBD_CONTROL_BITS);
+	kbdc_lock(kbdc, FALSE);
+
+	update_led();
+
+	timeout(check_for_lost_intr, (void *)NULL, hz);
+	lost_intr_timeout_queued = 1;
+
+#endif /* !_I386_ISA_KBDIO_H_ */
 }
 
 /*---------------------------------------------------------------------------*
@@ -658,7 +977,7 @@ xlatkey2ascii(U_short key)
 #endif
 	static Ovl_tbl	thisdef;
 	int		n;
-	void		(*fnc)();
+	void		(*fnc)(void);
 
 	if(key==0)			/* ignore the NON-KEY */
 		return 0;
@@ -922,19 +1241,24 @@ sgetc(int noblock)
 	static char	keybuf[2] = {0}; /* the second 0 is a delimiter! */
 #endif /* XSERVER */
 
+#ifdef _I386_ISA_KBDIO_H_
+	int 		c;
+#endif /* _I386_ISA_KBDIO_H_ */
+
 loop:
 
 #ifdef XSERVER
 
+#ifndef _I386_ISA_KBDIO_H_
+
 #if PCVT_KBD_FIFO
 
 	/* see if there is data from the keyboard available either from */
 	/* the keyboard fifo or from the 8042 keyboard controller	*/
 
-	if ((( noblock) && (pcvt_kbd_count)) ||
-	    ((!noblock) && (inb(CONTROLLER_CTRL) & STATUS_OUTPBF)))
+	if (pcvt_kbd_count || (inb(CONTROLLER_CTRL) & STATUS_OUTPBF))
 	{
-		if (!noblock)		/* source = 8042 */
+		if (!pcvt_kbd_count)	/* source = 8042 */
 		{
 			PCVT_KBD_DELAY();	/* 7 us delay */
 			dt = inb(CONTROLLER_DATA);	/* get from obuf */
@@ -960,6 +1284,32 @@ loop:
 
 #endif /* !PCVT_KBD_FIFO */
 
+#else /* _I386_ISA_KBDIO_H_ */
+
+#if PCVT_KBD_FIFO
+	if (pcvt_kbd_count) {
+		dt = pcvt_kbd_fifo[pcvt_kbd_rptr++];
+		PCVT_DISABLE_INTR();
+		pcvt_kbd_count--;
+		PCVT_ENABLE_INTR();
+		if (pcvt_kbd_rptr >= PCVT_KBD_FIFO_SZ)
+			pcvt_kbd_rptr = 0;
+	} else
+#endif /* PCVT_KBD_FIFO */
+	if (!noblock) {
+		while ((c = read_kbd_data(kbdc)) == -1)
+			;
+		dt = c;
+	} else {
+		if ((c = read_kbd_data_no_wait(kbdc)) == -1)
+			return NULL;
+		dt = c;
+	}
+
+	{
+
+#endif /* !_I386_ISA_KBDIO_H_ */
+
 		/*
 		 * If x mode is active, only care for locking keys, then
 		 * return the scan code instead of any key translation.
@@ -984,7 +1334,7 @@ loop:
 
 #if PCVT_FREEBSD > 210
 			add_keyboard_randomness(dt);
-#endif  /* PCVT_FREEBSD > 210 */
+#endif	/* PCVT_FREEBSD > 210 */
 
 #if !PCVT_USL_VT_COMPAT
 			if ((dt & 0x80) == 0)
@@ -1206,7 +1556,7 @@ loop:
 					 * happen "recently", i.e. before
 					 * less than half a second
 					 */
-					now = time;
+					gettime(&now);
 					timevalsub(&now, &mouse.lastmove);
 					mouse.lastmove = time;
 					accel = (now.tv_sec == 0
@@ -1236,15 +1586,16 @@ no_mouse_event:
 
 #else /* !XSERVER */
 
+#ifndef  _I386_ISA_KBDIO_H_
+
 #  if PCVT_KBD_FIFO
 
 	/* see if there is data from the keyboard available either from */
 	/* the keyboard fifo or from the 8042 keyboard controller	*/
 
-	if ((( noblock) && (pcvt_kbd_count)) ||
-	    ((!noblock) && (inb(CONTROLLER_CTRL) & STATUS_OUTPBF)))
+	if (pcvt_kbd_count || (inb(CONTROLLER_CTRL) & STATUS_OUTPBF))
 	{
-		if (!noblock)		/* source = 8042 */
+		if (!noblock || kbd_polling)	/* source = 8042 */
 		{
 			PCVT_KBD_DELAY();	/* 7 us delay */
 			dt = inb(CONTROLLER_DATA);
@@ -1272,8 +1623,33 @@ no_mouse_event:
 
 #endif /* !PCVT_KBD_FIFO */
 
+#else /* _I386_ISA_KBDIO_H_ */
+
+#if PCVT_KBD_FIFO
+	if (pcvt_kbd_count) {
+		dt = pcvt_kbd_fifo[pcvt_kbd_rptr++];
+		PCVT_DISABLE_INTR();
+		pcvt_kbd_count--;
+		PCVT_ENABLE_INTR();
+		if (pcvt_kbd_rptr >= PCVT_KBD_FIFO_SZ)
+			pcvt_kbd_rptr = 0;
+	} else
+#endif /* PCVT_KBD_FIFO */
+	if (!noblock) {
+		while ((c = read_kbd_data(kbdc)) == -1)
+			;
+		dt = c;
+	} else {
+		if ((c = read_kbd_data_no_wait(kbdc)) == -1)
+			return NULL;
+		dt = c;
+	}
+
+#endif /* !_I386_ISA_KBDIO_H_ */
+
 #endif /* !XSERVER */
 
+#ifndef _I386_ISA_KBDIO_H_
 	else
 	{
 		if(noblock)
@@ -1281,6 +1657,7 @@ no_mouse_event:
 		else
 			goto loop;
 	}
+#endif /* !_I386_ISA_KBDIO_H_ */
 
 #if PCVT_SHOWKEYS
 	showkey (' ', dt);
@@ -1330,7 +1707,7 @@ regular:
 
 #if PCVT_FREEBSD > 210
 	add_keyboard_randomness(dt);
-#endif  /* PCVT_FREEBSD > 210 */
+#endif	/* PCVT_FREEBSD > 210 */
 
 #if PCVT_SCANSET == 1
 	kbd_status.breakseen = dt & 0x80 ? 1 : 0;
@@ -1351,7 +1728,7 @@ regular:
 
 #if PCVT_CTRL_ALT_DEL		/*   Check for cntl-alt-del	*/
 	if((key == 76) && ctrl_down && (meta_down||altgr_down))
-		cpu_reset();
+		shutdown_nice();
 #endif /* PCVT_CTRL_ALT_DEL */
 
 #if !(PCVT_NETBSD || PCVT_FREEBSD >= 200)
