@@ -64,11 +64,15 @@
 
 #define PCVT_INCLUDE_VT_SELATTR	/* get inline function from pcvt_hdr.h */
 
-#include "pcvt_hdr.h"		/* global include */
+#include <i386/isa/pcvt/pcvt_hdr.h>	/* global include */
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
 
 extern u_short csd_ascii[];	/* pcvt_tbl.h */
 extern u_short csd_supplemental[];
 
+static void write_char (struct video_state *svsp, int attrib, int ch);
 static void check_scroll ( struct video_state *svsp );
 static void hp_entry ( U_char ch, struct video_state *svsp );
 static void vt_coldinit ( void );
@@ -84,17 +88,17 @@ static void writefkl ( int num, u_char *string, struct video_state *svsp );
 
 static __inline void write_char (svsp, attrib, ch)
 struct	video_state *svsp;
-u_short	attrib, ch;
+u_short	attrib, ch;		/* XXX inefficient interface */
 {
 	if ((ch >= 0x20) && (ch <= 0x7f))	/* use GL if ch >= 0x20 */
 	{
 		if(!svsp->ss)		/* single shift G2/G3 -> GL ? */
 		{
-			*video = attrib | svsp->GL[ch-0x20];
+			*video = attrib | (*svsp->GL)[ch-0x20];
 		}
 		else
 		{
-			*video = attrib | svsp->Gs[ch-0x20];
+			*video = attrib | (*svsp->Gs)[ch-0x20];
 			svsp->ss = 0;
 		}
 	}
@@ -106,7 +110,7 @@ u_short	attrib, ch;
 		{
 			if(ch >= 0xA0)		/* use GR if ch >= 0xA0 */
 			{
-				*video = attrib | svsp->GR[ch-0xA0];
+				*video = attrib | (*svsp->GR)[ch-0xA0];
 			}
 			else
 			{
@@ -174,10 +178,10 @@ sput (u_char *s, U_char kernel, int len, int page)
 	if(svsp->sevenbit)
 		ch &= 0x7f;
 
-	if((ch <= 0x1f) && (svsp->transparent == 0))
+	if(((ch <= 0x1f) || (ch == 0x7f)) && (svsp->transparent == 0))
 	{
 
-	/* always process control-chars in the range 0x00..0x1f !!! */
+	/* always process control-chars in the range 0x00..0x1f, 0x7f !!! */
 
 		if(svsp->dis_fnc)
 		{
@@ -280,11 +284,11 @@ sput (u_char *s, U_char kernel, int len, int page)
 					break;
 
 				case 0x0e:	/* SO */
-					svsp->GL = svsp->G1;
+					svsp->GL = &svsp->G1;
 					break;
 
 				case 0x0f:	/* SI */
-					svsp->GL = svsp->G0;
+					svsp->GL = &svsp->G0;
 					break;
 
 				case 0x10:	/* DLE */
@@ -319,6 +323,7 @@ sput (u_char *s, U_char kernel, int len, int page)
 				case 0x1d:	/* GS */
 				case 0x1e:	/* RS */
 				case 0x1f:	/* US */
+				case 0x7f:	/* DEL */
 				break;
 			}
 		}
@@ -326,7 +331,8 @@ sput (u_char *s, U_char kernel, int len, int page)
 	else
 	{
 
-	/* char range 0x20...0xff processing depends on current state */
+	/* char range 0x20...0x73, 0x80...0xff processing */
+	/* depends on current state */
 
 		switch(svsp->state)
 		{
@@ -465,13 +471,13 @@ sput (u_char *s, U_char kernel, int len, int page)
 						break;
 
 					case 'N':	/* SINGLE SHIFT G2 */
-						svsp->Gs = svsp->G2;
+						svsp->Gs = &svsp->G2;
 						svsp->ss = 1;
 						svsp->state = STATE_INIT;
 						break;
 
 					case 'O':	/* SINGLE SHIFT G3 */
-						svsp->Gs = svsp->G3;
+						svsp->Gs = &svsp->G3;
 						svsp->ss = 1;
 						svsp->state = STATE_INIT;
 						break;
@@ -520,27 +526,27 @@ sput (u_char *s, U_char kernel, int len, int page)
 						break;
 #endif /* PCVT_SETCOLOR */
 					case 'n': /* Lock Shift G2 -> GL */
-						svsp->GL = svsp->G2;
+						svsp->GL = &svsp->G2;
 						svsp->state = STATE_INIT;
 						break;
 
 					case 'o': /* Lock Shift G3 -> GL */
-						svsp->GL = svsp->G3;
+						svsp->GL = &svsp->G3;
 						svsp->state = STATE_INIT;
 						break;
 
 					case '}': /* Lock Shift G2 -> GR */
-						svsp->GR = svsp->G2;
+						svsp->GR = &svsp->G2;
 						svsp->state = STATE_INIT;
 						break;
 
 					case '|': /* Lock Shift G3 -> GR */
-						svsp->GR = svsp->G3;
+						svsp->GR = &svsp->G3;
 						svsp->state = STATE_INIT;
 						break;
 
 					case '~': /* Lock Shift G1 -> GR */
-						svsp->GR = svsp->G1;
+						svsp->GR = &svsp->G1;
 						svsp->state = STATE_INIT;
 						break;
 
@@ -892,13 +898,17 @@ sput (u_char *s, U_char kernel, int len, int page)
 static void
 vt_coldinit(void)
 {
-	u_short volatile *cp = Crtat + (CGA_BUF-MONO_BUF)/CHR;
+	u_short volatile *cp;
 	u_short was;
 	int nscr, charset;
 	int equipment;
-	u_short *SaveCrtat = Crtat;
+	u_short *SaveCrtat;
 	struct video_state *svsp;
 
+	Crtat = (u_short *)MONO_BUF;	/* XXX assume static relocation works */
+	SaveCrtat = Crtat;
+	cp = Crtat + (CGA_BUF-MONO_BUF)/CHR;
+
 	do_initialization = 0;		/* reset init necessary flag */
 
 	/* get the equipment byte from the RTC chip */
@@ -1061,6 +1071,7 @@ vt_coldinit(void)
 		}
 
 #ifdef FAT_CURSOR
+		svsp->cursor_start = 0;
 		svsp->cursor_end = 15;		/* cursor lower scanline */
 #endif
 
@@ -1073,8 +1084,8 @@ vt_coldinit(void)
 		svsp->G1 = csd_ascii;		/* G1 = ascii	*/
 		svsp->G2 = csd_supplemental;	/* G2 = supplemental */
 		svsp->G3 = csd_supplemental;	/* G3 = supplemental */
-		svsp->GL = svsp->G0;		/* GL = G0 */
-		svsp->GR = svsp->G2;		/* GR = G2 */
+		svsp->GL = &svsp->G0;		/* GL = G0 */
+		svsp->GR = &svsp->G2;		/* GR = G2 */
 		svsp->whichi = 0;		/* char set designate init */
 		svsp->which[0] = '\0';		/* char set designate init */
 		svsp->hp_state = SHP_INIT;	/* init HP mode state machine*/
@@ -1121,6 +1132,16 @@ vt_coldinit(void)
 			outb(addr_6845, CRTC_CURSORL);
 			cursorat |= inb(addr_6845+1);
 
+			/*
+			 * Reject cursors that are more than one row off a
+			 * 25-row screen.  syscons sets the cursor offset
+			 * to 0xffff. The scroll up fixup fails for this
+			 * because the assignment to svsp->row overflows
+			 * and perhaps for other reasons.
+			 */
+			if (cursorat > 25 * svsp->maxcol)
+				cursorat = 25 * svsp->maxcol;
+
 			svsp->cur_offset = cursorat;
 			svsp->row = cursorat / svsp->maxcol;
 			svsp->col = cursorat % svsp->maxcol;
@@ -1223,10 +1244,6 @@ vt_coldinit(void)
 
 	if(!keyboard_is_initialized)
 		kbd_code_init1();
-
-	/* update keyboard led's */
-
-	update_led();
 }
 
 /*---------------------------------------------------------------------------*
@@ -1869,7 +1886,7 @@ clr_parms(struct video_state *svsp)
  *	partial HP 2392 ANSI mode Emulator
  *	==================================
  *
- *	this part tooks over the emulation of some escape sequences
+ *	this part takes over the emulation of some escape sequences
  *	needed to handle the function key labels
  *
  *	They are modeled after the corresponding escape sequences
