@@ -62,8 +62,8 @@
 
 #define PCVT_INCLUDE_VT_SELATTR	/* get inline function from pcvt_hdr.h */
 
-#include "pcvt_hdr.h"		/* global include */
-#include "pcvt_tbl.h"		/* character set conversion tables */
+#include <i386/isa/pcvt/pcvt_hdr.h>	/* global include */
+#include <i386/isa/pcvt/pcvt_tbl.h>	/* character set conversion tables */
 
 static void clear_dld ( struct video_state *svsp );
 static void init_dld ( struct video_state *svsp );
@@ -491,8 +491,8 @@ vt_str(struct video_state *svsp)
 		svsp->G1 = cse_ascii;		/* G1 = ascii	*/
 		svsp->G2 = cse_supplemental;	/* G2 = supplemental */
 		svsp->G3 = cse_supplemental;	/* G3 = supplemental */
-		svsp->GL = svsp->G0;		/* GL = G0 */
-		svsp->GR = svsp->G2;		/* GR = G2 */
+		svsp->GL = &svsp->G0;		/* GL = G0 */
+		svsp->GR = &svsp->G2;		/* GR = G2 */
 	}
 	else
 	{
@@ -500,8 +500,8 @@ vt_str(struct video_state *svsp)
 		svsp->G1 = csd_ascii;		/* G1 = ascii	*/
 		svsp->G2 = csd_supplemental;	/* G2 = supplemental */
 		svsp->G3 = csd_supplemental;	/* G3 = supplemental */
-		svsp->GL = svsp->G0;		/* GL = G0 */
-		svsp->GR = svsp->G2;		/* GR = G2 */
+		svsp->GL = &svsp->G0;		/* GL = G0 */
+		svsp->GR = &svsp->G2;		/* GR = G2 */
 	}
 
 	svsp->vtsgr = VT_NORMAL;		/* no attributes */
