@@ -46,7 +46,6 @@
 #include	<machine/cpufunc.h>
 #ifdef	KERNEL
 #include	<sys/systm.h>
-#include	<sys/devconf.h>
 #include	<machine/clock.h>
 #include	<machine/qcam.h>
 #else	/* user mode version of driver */
@@ -63,7 +62,7 @@
 #include	"qcam.h"
 #endif	/* bsdi */
 
-#ifdef	LINUX
+#ifdef	__linux__
 #include	<linux/kernel.h>
 #include	<linux/sched.h>
 #include	<linux/string.h>
@@ -71,7 +70,7 @@
 #include	<asm/io.h>
 #include	"qcam-linux.h"
 #include	"qcam.h"
-#endif	/* LINUX */
+#endif	/* __linux__ */
 
 #ifdef	_SCO_DS
 #include	<limits.h>
@@ -81,8 +80,13 @@
 #include	"qcam.h"
 #endif
 
+#ifdef __FreeBSD__
+#include	<i386/isa/qcamreg.h>
+#include	<i386/isa/qcamdefs.h>
+#else
 #include	"qcamreg.h"
 #include	"qcamdefs.h"
+#endif
 
 /*
  * There should be _NO_ operating system dependant code or definitions
