@@ -30,7 +30,9 @@
  *
  */
 
+#include "opt_comconsole.h"
 #include "rc.h"
+
 #if NRC > 0
 
 /*#define RCDEBUG*/
@@ -40,36 +42,37 @@
 #include <sys/ioctl.h>
 #include <sys/tty.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/conf.h>
 #include <sys/dkstat.h>
 #include <sys/file.h>
 #include <sys/uio.h>
 #include <sys/kernel.h>
 #include <sys/syslog.h>
-#include <sys/devconf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <machine/clock.h>
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/sioreg.h>
 
 #include <i386/isa/ic/cd180.h>
 #include <i386/isa/rcreg.h>
 
-/* Prototypes */
-int     rcprobe         __P((struct isa_device *));
-int     rcattach        __P((struct isa_device *));
 
-int     rcopen          __P((dev_t, int, int, struct proc *));
-int     rcclose         __P((dev_t, int, int, struct proc *));
-int     rcread          __P((dev_t, struct uio *, int));
-int     rcwrite         __P((dev_t, struct uio *, int));
-void    rcintr          __P((int));
+/* Prototypes */
+static int     rcprobe         __P((struct isa_device *));
+static int     rcattach        __P((struct isa_device *));
+
+/*-
+ * This space intentionally left blank to stop __LINE__ from screwing up
+ * regression tests :-(.
+ *
+ *
+ *
+ */
 void    rcpoll          __P((void));
-void    rcstop          __P((struct tty *, int));
-int     rcioctl         __P((dev_t, int, caddr_t, int, struct proc *));
 
 #define rcin(port)      RC_IN  (nec, port)
 #define rcout(port,v)   RC_OUT (nec, port, v)
@@ -97,6 +100,20 @@ struct isa_driver rcdriver = {
 	rcprobe, rcattach, "rc"
 };
 
+static	d_open_t	rcopen;
+static	d_close_t	rcclose;
+static	d_read_t	rcread;
+static	d_write_t	rcwrite;
+static	d_ioctl_t	rcioctl;
+static	d_stop_t	rcstop;
+static	d_devtotty_t	rcdevtotty;
+
+#define CDEV_MAJOR 63
+static struct cdevsw rc_cdevsw = 
+	{ rcopen,       rcclose,        rcread,         rcwrite,        /*63*/
+	  rcioctl,      rcstop,         noreset,        rcdevtotty,/* rc */
+	  ttselect,	nommap,		NULL,	"rc",	NULL,	-1 };
+
 /* Per-board structure */
 static struct rc_softc {
 	u_int           rcb_probed;     /* 1 - probed, 2 - attached */
@@ -126,13 +143,16 @@ static struct rc_chans  {
 	u_char          *rc_obufend;            /* end of output buf    */
 	u_char           rc_ibuf[4 * RC_IBUFSIZE];  /* input buffer         */
 	u_char           rc_obuf[RC_OBUFSIZE];  /* output buffer        */
+#ifdef	DEVFS
+	void	*devfs_token;
+#endif
 } rc_chans[NRC * CD180_NCHAN];
 
 static int rc_scheduled_event = 0;
 
 /* for pstat -t */
-struct tty rc_tty[NRC * CD180_NCHAN];
-int        nrc_tty = NRC * CD180_NCHAN;
+static struct tty rc_tty[NRC * CD180_NCHAN];
+static const int  nrc_tty = NRC * CD180_NCHAN;
 
 /* Flags */
 #define RC_DTR_OFF      0x0001          /* DTR wait, for close/open     */
@@ -166,7 +186,6 @@ static void rc_hardclose        __P((struct rc_chans *));
 static int  rc_modctl           __P((struct rc_chans *, int, int));
 static void rc_start            __P((struct tty *));
 static int  rc_param            __P((struct tty *, struct termios *));
-static void rc_registerdev      __P((struct isa_device *id));
 static void rc_reinit           __P((struct rc_softc *));
 #ifdef RCDEBUG
 static void printrcflags();
@@ -179,7 +198,8 @@ static void rc_wait0            __P((int nec, int unit, int chan, int line));
 /**********************************************/
 
 /* Quick device probing */
-int rcprobe(dvp)
+static int
+rcprobe(dvp)
 	struct  isa_device      *dvp;
 {
 	int             irq = ffs(dvp->id_irq) - 1;
@@ -207,36 +227,11 @@ int rcprobe(dvp)
 	return 0xF;
 }
 
-static struct kern_devconf kdc_rc[NRC] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"rc", 0, { MDDT_ISA, 0, "tty" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,        /* state */
-	"RISCom/8 multiport card",
-	DC_CLS_SERIAL		/* class */
-} };
-
-static void
-rc_registerdev(id)
-	struct isa_device *id;
-{
-	int	unit;
-
-	unit = id->id_unit;
-	if (unit != 0)
-		kdc_rc[unit] = kdc_rc[0];
-	kdc_rc[unit].kdc_unit = unit;
-	kdc_rc[unit].kdc_isa = id;
-	kdc_rc[unit].kdc_state = DC_UNKNOWN;
-	dev_attach(&kdc_rc[unit]);
-}
-
-int rcattach(dvp)
+static int
+rcattach(dvp)
 	struct  isa_device      *dvp;
 {
-	register int            i, chan, nec = dvp->id_iobase;
+	register int            chan, nec = dvp->id_iobase;
 	struct rc_softc         *rcb = &rc_softc[dvp->id_unit];
 	struct rc_chans         *rc  = &rc_chans[dvp->id_unit * CD180_NCHAN];
 	static int              rc_wakeup_started = 0;
@@ -248,12 +243,11 @@ int rcattach(dvp)
 	rcb->rcb_addr   = nec;
 	rcb->rcb_dtr    = 0;
 	rcb->rcb_baserc = rc;
+	rcb->rcb_unit	= dvp->id_unit;
 	/*rcb->rcb_chipid = 0x10 + dvp->id_unit;*/
-	printf("rc%d: %d chans, firmware rev. %c\n", dvp->id_unit,
+	printf("rc%d: %d chans, firmware rev. %c\n", rcb->rcb_unit,
 		CD180_NCHAN, (rcin(CD180_GFRCR) & 0xF) + 'A');
 
-	rc_registerdev(dvp);
-
 	for (chan = 0; chan < CD180_NCHAN; chan++, rc++) {
 		rc->rc_rcb     = rcb;
 		rc->rc_chan    = chan;
@@ -266,11 +260,19 @@ int rcattach(dvp)
 		rc->rc_dtrwait = 3 * hz;
 		rc->rc_dcdwaits= 0;
 		rc->rc_hotchar = 0;
-		tp = rc->rc_tp = &rc_tty[chan];
+		tp = rc->rc_tp = &rc_tty[chan + (dvp->id_unit * CD180_NCHAN)];
 		ttychars(tp);
 		tp->t_lflag = tp->t_iflag = tp->t_oflag = 0;
 		tp->t_cflag = TTYDEF_CFLAG;
 		tp->t_ispeed = tp->t_ospeed = TTYDEF_SPEED;
+#ifdef DEVFS
+/* FIX THIS to reflect real devices */
+		rc->devfs_token = 
+			devfs_add_devswf(&rc_cdevsw,
+					 (dvp->id_unit * CD180_NCHAN) + chan,
+					 DV_CHR, 0, 0, 0600, "rc%d.%d", 
+					 dvp->id_unit, chan);
+#endif
 	}
 	rcb->rcb_probed = RC_ATTACHED;
 	if (!rc_wakeup_started) {
@@ -565,9 +567,8 @@ void rcpoll()
 	register struct rc_chans *rc;
 	register struct rc_softc *rcb;
 	register u_char        *tptr, *eptr;
-	register int            s;
 	register struct tty    *tp;
-	register int            chan, icnt, c, nec, unit;
+	register int            chan, icnt, nec, unit;
 
 	if (rc_scheduled_event == 0)
 		return;
@@ -685,7 +686,8 @@ done1:
 		goto repeat;
 }
 
-void rcstop(tp, rw)
+static	void
+rcstop(tp, rw)
 	register struct tty     *tp;
 	int                     rw;
 {
@@ -718,7 +720,8 @@ void rcstop(tp, rw)
 	enable_intr();
 }
 
-int rcopen(dev, flag, mode, p)
+static	int
+rcopen(dev, flag, mode, p)
 	dev_t           dev;
 	int             flag, mode;
 	struct proc    *p;
@@ -798,6 +801,7 @@ again:
 		goto again;
 	}
 	error = (*linesw[tp->t_line].l_open)(dev, tp);
+	disc_optim(tp, &tp->t_termios, rc);
 	if ((tp->t_state & TS_ISOPEN) && CALLOUT(dev))
 		rc->rc_flags |= RC_ACTOUT;
 out:
@@ -809,7 +813,8 @@ out:
 	return error;
 }
 
-int rcclose(dev, flag, mode, p)
+static	int
+rcclose(dev, flag, mode, p)
 	dev_t           dev;
 	int             flag, mode;
 	struct proc    *p;
@@ -827,6 +832,7 @@ int rcclose(dev, flag, mode, p)
 #endif
 	s = spltty();
 	(*linesw[tp->t_line].l_close)(tp, flag);
+	disc_optim(tp, &tp->t_termios, rc);
 	rcstop(tp, FREAD | FWRITE);
 	rc_hardclose(rc);
 	ttyclose(tp);
@@ -866,7 +872,8 @@ register struct rc_chans *rc;
 }
 
 /* Read from line */
-int rcread(dev, uio, flag)
+static	int
+rcread(dev, uio, flag)
 	dev_t           dev;
 	struct uio      *uio;
 	int             flag;
@@ -877,7 +884,8 @@ int rcread(dev, uio, flag)
 }
 
 /* Write to line */
-int rcwrite(dev, uio, flag)
+static	int
+rcwrite(dev, uio, flag)
 	dev_t           dev;
 	struct uio      *uio;
 	int             flag;
@@ -1073,7 +1081,7 @@ static void rc_reinit(rcb)
 struct rc_softc         *rcb;
 {
 	register struct rc_chans       *rc, *rce;
-	register int                    i, nec;
+	register int                    nec;
 
 	nec = rcb->rcb_addr;
 	rc_hwreset(rcb->rcb_unit, nec, RC_FAKEID);
@@ -1083,7 +1091,8 @@ struct rc_softc         *rcb;
 		(void) rc_param(rc->rc_tp, &rc->rc_tp->t_termios);
 }
 
-int rcioctl(dev, cmd, data, flag, p)
+static	int
+rcioctl(dev, cmd, data, flag, p)
 dev_t           dev;
 int             cmd, flag;
 caddr_t         data;
@@ -1097,6 +1106,7 @@ struct proc     *p;
 	if (error >= 0)
 		return (error);
 	error = ttioctl(tp, cmd, data, flag);
+	disc_optim(tp, &tp->t_termios, rc);
 	if (error >= 0)
 		return (error);
 	s = spltty();
@@ -1233,7 +1243,7 @@ int rc_test(nec, unit)
 	register int    nec;
 	int             unit;
 {
-	int     chan = 0, nopt = 0;
+	int     chan = 0;
 	int     i = 0, rcnt, old_level;
 	unsigned int    iack, chipid;
 	unsigned short  divs;
@@ -1401,7 +1411,7 @@ char             *comment;
 }
 #endif /* RCDEBUG */
 
-struct tty *
+static	struct tty *
 rcdevtotty(dev)
 	dev_t	dev;
 {
@@ -1443,8 +1453,6 @@ static void
 rc_wakeup(chan)
 	void	*chan;
 {
-	int		unit;
-
 	timeout(rc_wakeup, (caddr_t)NULL, 1);
 
 	if (rc_scheduled_event != 0) {
@@ -1492,4 +1500,21 @@ rc_wait0(nec, unit, chan, line)
 		printf("rc%d/%d: channel command timeout, rc.c line: %d\n",
 		      unit, chan, line);
 }
+
+static rc_devsw_installed = 0;
+
+static void 	rc_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! rc_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&rc_cdevsw, NULL);
+		rc_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(rcdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,rc_drvinit,NULL)
+
+
 #endif /* NRC */
