@@ -41,7 +41,7 @@
  */
 
 
-/* $FreeBSD: stable/2.1/sys/i386/isa/scd.c 50950 1999-09-05 08:39:22Z peter $ */
+/* $FreeBSD: stable/2.2/sys/i386/isa/scd.c 50950 1999-09-05 08:39:22Z peter $ */
 
 /* Please send any comments to micke@dynas.se */
 
@@ -63,15 +63,18 @@
 #include <sys/errno.h>
 #include <sys/dkbad.h>
 #include <sys/disklabel.h>
-#include <sys/devconf.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <machine/clock.h>
 #include <machine/stdarg.h>
 
-#include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/scdreg.h>
 
+
 #define scd_part(dev)	((minor(dev)) & 7)
 #define scd_unit(dev)	(((minor(dev)) & 0x38) >> 3)
 #define scd_phys(dev)	(((minor(dev)) & 0x40) >> 6)
@@ -100,7 +103,7 @@
 #define	SCDBLKSIZE	2048
 
 #ifdef SCD_DEBUG
-   int scd_debuglevel = SCD_DEBUG;
+   static int scd_debuglevel = SCD_DEBUG;
 #  define XDEBUG(level, data) {if (scd_debuglevel >= level) printf data;}
 #else
 #  define XDEBUG(level, data)
@@ -118,7 +121,7 @@ struct scd_mbx {
 	short		count;
 };
 
-struct scd_data {
+static struct scd_data {
 	int	iobase;
 	char	double_speed;
 	char	*name;
@@ -137,26 +140,23 @@ struct scd_data {
 	struct	ioc_play_msf last_play;
 
 	short	audio_status;
-	struct buf head;		/* head of buf queue */
+	struct buf_queue_head head;		/* head of buf queue */
 	struct scd_mbx mbx;
+#ifdef	DEVFS
+	void	*ra_devfs_token;
+	void	*rc_devfs_token;
+	void	*a_devfs_token;
+	void	*c_devfs_token;
+#endif
 } scd_data[NSCD];
 
 /* prototypes */
-int	scdopen(dev_t dev, int flags, int fmt, struct proc *p);
-int	scdclose(dev_t dev, int flags, int fmt, struct proc *p);
-void	scdstrategy(struct buf *bp);
-int	scdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p);
-int	scdsize(dev_t dev);
-
-static	int	bcd2bin(bcd_t b);
-static	bcd_t	bin2bcd(int b);
 static	void	hsg2msf(int hsg, bcd_t *msf);
 static	int	msf2hsg(bcd_t *msf);
 
 static void process_attention(unsigned unit);
-static inline void write_control(unsigned port, unsigned data);
+static __inline void write_control(unsigned port, unsigned data);
 static int waitfor_status_bits(int unit, int bits_set, int bits_clear);
-static int waitfor_attention(int unit);
 static int send_cmd(u_int unit, u_char cmd, u_int nargs, ...);
 static void init_drive(unsigned unit);
 static int spin_up(unsigned unit);
@@ -182,54 +182,60 @@ static int scd_toc_header(int unit, struct ioc_toc_header *th);
 static int scd_toc_entrys(int unit, struct ioc_read_toc_entry *te);
 #define SCD_LASTPLUS1 170 /* don't ask, xcdplayer passes this in */
 
-extern	int	hz;
-
-int	scd_probe(struct isa_device *dev);
-int	scd_attach(struct isa_device *dev);
+static int	scd_probe(struct isa_device *dev);
+static int	scd_attach(struct isa_device *dev);
 struct	isa_driver	scddriver = { scd_probe, scd_attach, "scd" };
 
-static struct kern_devconf kdc_scd[NSCD] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"scd", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* status */
-	"Sony CD-ROM drive",	/* properly filled later */
-	DC_CLS_RDISK		/* class */
-} };
+static	d_open_t	scdopen;
+static	d_close_t	scdclose;
+static	d_ioctl_t	scdioctl;
+static	d_strategy_t	scdstrategy;
 
-static inline void
-scd_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_scd[id->id_unit] = kdc_scd[0];
-	kdc_scd[id->id_unit].kdc_unit = id->id_unit;
-	kdc_scd[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_scd[id->id_unit]);
-}
+#define CDEV_MAJOR 45
+#define BDEV_MAJOR 16
+static struct cdevsw scd_cdevsw;
+static struct bdevsw scd_bdevsw = 
+	{ scdopen,	scdclose,	scdstrategy,	scdioctl,	/*16*/
+	  nodump,	nopsize,	0, "scd",	&scd_cdevsw,	-1 };
 
 int scd_attach(struct isa_device *dev)
 {
-	struct scd_data *cd = scd_data + dev->id_unit;
-	int i;
+	int	unit = dev->id_unit;
+	struct scd_data *cd = scd_data + unit;
 
 	cd->iobase = dev->id_iobase;	/* Already set by probe, but ... */
 
-	kdc_scd[dev->id_unit].kdc_state = DC_IDLE;
 	/* name filled in probe */
-	kdc_scd[dev->id_unit].kdc_description = scd_data[dev->id_unit].name;
 	printf("scd%d: <%s>\n", dev->id_unit, scd_data[dev->id_unit].name);
 
 	init_drive(dev->id_unit);
 
 	cd->flags = SCDINIT;
 	cd->audio_status = CD_AS_AUDIO_INVALID;
+	bufq_init(&cd->head);
 
+#ifdef DEVFS
+	cd->ra_devfs_token = 
+		devfs_add_devswf(&scd_cdevsw, dkmakeminor(unit, 0, 0),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rscd%da", unit);
+	cd->rc_devfs_token = 
+		devfs_add_devswf(&scd_cdevsw, dkmakeminor(unit, 0, RAW_PART),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rscd%dc", unit);
+	cd->a_devfs_token = 
+		devfs_add_devswf(&scd_bdevsw, dkmakeminor(unit, 0, 0),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "scd%da", unit);
+	cd->c_devfs_token = 
+		devfs_add_devswf(&scd_bdevsw, dkmakeminor(unit, 0, RAW_PART),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "scd%dc", unit);
+#endif
 	return 1;
 }
 
-int
+static	int
 scdopen(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	int unit,part,phys;
@@ -277,18 +283,15 @@ scdopen(dev_t dev, int flags, int fmt, struct proc *p)
 
 	cd->openflag = 1;
 	cd->flags |= SCDVALID;
-	kdc_scd[unit].kdc_state = DC_BUSY;
 
 	return 0;
 }
 
-int
+static	int
 scdclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	int unit,part,phys;
 	struct scd_data *cd;
-	int rlen;
-	char rdata[10];
 
 	unit = scd_unit(dev);
 	if (unit >= NSCD)
@@ -306,7 +309,6 @@ scdclose(dev_t dev, int flags, int fmt, struct proc *p)
 		cd->flags &= ~SCDSPINNING;
 	}
 
-	kdc_scd[unit].kdc_state = DC_IDLE;
 
 	/* close channel */
 	cd->openflag = 0;
@@ -314,11 +316,10 @@ scdclose(dev_t dev, int flags, int fmt, struct proc *p)
 	return 0;
 }
 
-void
+static	void
 scdstrategy(struct buf *bp)
 {
 	struct scd_data *cd;
-	struct buf *qp;
 	int s;
 	int unit = scd_unit(bp->b_dev);
 
@@ -363,9 +364,8 @@ scdstrategy(struct buf *bp)
 	bp->b_resid = 0;
 
 	/* queue it */
-	qp = &cd->head;
 	s = splbio();
-	disksort(qp,bp);
+	bufqdisksort(&cd->head, bp);
 	splx(s);
 
 	/* now check whether we can perform processing */
@@ -384,9 +384,8 @@ static void
 scd_start(int unit)
 {
 	struct scd_data *cd = scd_data + unit;
-	struct buf *bp, *qp = &cd->head;
+	struct buf *bp;
 	struct partition *p;
-	int part;
 	register s = splbio();
 
 	if (cd->flags & SCDMBXBSY) {
@@ -394,9 +393,10 @@ scd_start(int unit)
 		return;
 	}
 
-	if ((bp = qp->b_actf) != 0) {
+	bp = bufq_first(&cd->head);
+	if (bp != 0) {
 		/* block found to process, dequeue */
-		qp->b_actf = bp->b_actf;
+		bufq_remove(&cd->head, bp);
 		cd->flags |= SCDMBXBSY;
 		splx(s);
 	} else {
@@ -418,7 +418,7 @@ scd_start(int unit)
 	return;
 }
 
-int
+static	int
 scdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 {
 	struct scd_data *cd;
@@ -494,18 +494,6 @@ scdioctl(dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 	}
 }
 
-int
-scdsize(dev_t dev)
-{
-	return -1;
-}
-
-void
-scdintr()
-{
-	return;
-}
-
 /***************************************************************
  * lower level of driver starts here
  **************************************************************/
@@ -517,7 +505,7 @@ scd_playtracks(int unit, struct ioc_play_track *pt)
 	struct ioc_play_msf msf;
 	int a = pt->start_track;
 	int z = pt->end_track;
-	int rc, i;
+	int rc;
 
 	if (!(cd->flags & SCDTOC) && (rc = read_toc(unit)) != 0) {
 		if (rc == -ERR_NOT_SPINNING) {
@@ -644,7 +632,6 @@ static int
 scd_eject(int unit)
 {
 	struct scd_data *cd = scd_data + unit;
-	int port = cd->iobase;
 
 	cd->audio_status = CD_AS_AUDIO_INVALID;
 	cd->flags &= ~(SCDSPINNING|SCDTOC);
@@ -710,8 +697,6 @@ scd_probe(struct isa_device *dev)
 
 	bzero(&drive_config, sizeof(drive_config));
 
-	scd_registerdev(dev);
-
 again:
 	/* Reset drive */
 	write_control(dev->id_iobase, CBIT_RESET_DRIVE);
@@ -800,10 +785,9 @@ scd_doread(int state, struct scd_mbx *mbxin)
 	int	port = mbx->port;
 	struct	buf *bp = mbx->bp;
 	struct	scd_data *cd = scd_data + unit;
-	int	reg,i,k,c;
+	int	reg,i;
 	int	blknum;
 	caddr_t	addr;
-	char	rdata[10];
 	static char sdata[3];	/* Must be preserved between calls to this function */
 
 loop:
@@ -842,7 +826,6 @@ trystat:
 
 		mbx->sz = cd->blksize;
 
-firstblock:
 		/* for first block */
 		mbx->nblk = (bp->b_bcount + (mbx->sz-1)) / mbx->sz;
 		mbx->skip = 0;
@@ -1063,18 +1046,6 @@ changed:
 	goto harderr;
 }
 
-static int
-bcd2bin(bcd_t b)
-{
-	return (b >> 4) * 10 + (b & 15);
-}
-
-static bcd_t
-bin2bcd(int b)
-{
-	return ((b / 10) << 4) | (b % 10);
-}
-
 static void
 hsg2msf(int hsg, bcd_t *msf)
 {
@@ -1099,7 +1070,6 @@ process_attention(unsigned unit)
 	unsigned port = scd_data[unit].iobase;
 	unsigned char code;
 	int count = 0;
-	int i;
 
 	while (IS_ATTENTION(port) && count++ < 30) {
 		write_control(port, CBIT_ATTENTION_CLEAR);
@@ -1154,7 +1124,7 @@ spin_up(unsigned unit)
 	int loop_count = 0;
 
 again:
-	rc = send_cmd(unit, CMD_SPIN_UP, NULL, 0, res_reg, &res_size);
+	rc = send_cmd(unit, CMD_SPIN_UP, 0, 0, res_reg, &res_size);
 	if (rc != 0) {
 		XDEBUG(2, ("scd%d: CMD_SPIN_UP error 0x%x\n", unit, rc));
 		return rc;
@@ -1297,7 +1267,7 @@ read_toc(dev_t dev)
 	return 0;
 }
 
-static inline void
+static __inline void
 write_control(unsigned port, unsigned data)
 {
 	outb(port + OREG_CONTROL, data);
@@ -1320,7 +1290,6 @@ get_result(u_int unit, int result_len, u_char *result)
 {
 	unsigned int port = scd_data[unit].iobase;
 	unsigned int res_reg = port + IREG_RESULT;
-	unsigned char c;
 	int loop_index = 2; /* send_cmd() reads two bytes ... */
 
 	XDEBUG(1, ("scd%d: DEBUG: get_result: bytes=%d\n", unit, result_len));
@@ -1381,7 +1350,8 @@ send_cmd(u_int unit, u_char cmd, u_int nargs, ...)
 
 	outb(port+OREG_COMMAND, cmd);
 
-	if (rc = waitfor_status_bits(unit, SBIT_RESULT_READY, SBIT_BUSY))
+	rc = waitfor_status_bits(unit, SBIT_RESULT_READY, SBIT_BUSY);
+	if (rc)
 		return -0x100;
 
 	reg = port + IREG_RESULT;
@@ -1540,4 +1510,19 @@ scd_toc_entrys (int unit, struct ioc_read_toc_entry *te)
 	return 0;
 }
 
+
+static scd_devsw_installed = 0;
+
+static void 	scd_drvinit(void *unused)
+{
+
+	if( ! scd_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &scd_bdevsw);
+		scd_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(scddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,scd_drvinit,NULL)
+
+
 #endif /* NSCD > 0 */
