@@ -60,7 +60,7 @@
  *               that category, with the possible exception of scanners and
  *               some of the older MO drives.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/seagate.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/seagate.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -121,11 +121,13 @@
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
-#include <sys/devconf.h>
 
 #include <machine/clock.h>
 
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
+
 #include <i386/isa/isa_device.h>
 
 #include <scsi/scsi_all.h>
@@ -280,7 +282,7 @@ typedef struct adapter {
 	target_t target[8];             /* target state data */
 } adapter_t;
 
-adapter_t seadata[NSEA];
+static adapter_t seadata[NSEA];
 
 #define IS_BUSY(a,b)    ((a)->target[(b)->xfer->sc_link->target].busy &\
 				(1 << (b)->xfer->sc_link->lun))
@@ -330,7 +332,6 @@ adapter_t seadata[NSEA];
 		(t)->ndelay.op = 1;\
 	(t)->init.op = 1; }
 
-int seaintr (int unit);
 static int sea_probe (struct isa_device *dev);
 static int sea_detect (adapter_t *z, struct isa_device *dev);
 static int sea_attach (struct isa_device *dev);
@@ -360,14 +361,6 @@ static struct scsi_adapter sea_switch = {
 static struct scsi_device sea_dev = { NULL, NULL, NULL, NULL, "sea", 0, {0} };
 struct isa_driver seadriver = { sea_probe, sea_attach, "sea" };
 
-static char sea_description [80]; /* XXX BOGUS!!! */
-static struct kern_devconf sea_kdc[NSEA] = {{
-	0, 0, 0, "sea", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN, &kdc_isa0, 0,
-	DC_UNCONFIGURED, sea_description,
-	DC_CLS_MISC		/* host adapters aren't special */
-} };
-
 /* FD TMC885's can't handle detach & re-attach */
 static int sea_select_cmd = CMD_DRVR_ENABLE | CMD_ATTN;
 
@@ -385,12 +378,6 @@ int sea_probe (struct isa_device *dev)
 	};
 	int i;
 
-	if (dev->id_unit)
-		sea_kdc[dev->id_unit] = sea_kdc[0];
-	sea_kdc[dev->id_unit].kdc_unit = dev->id_unit;
-	sea_kdc[dev->id_unit].kdc_isa = dev;
-	dev_attach (&sea_kdc[dev->id_unit]);
-
 	/* Init fields used by our routines */
 	z->parity = (dev->id_flags & FLAG_NOPARITY) ? 0 : CMD_EN_PARITY;
 	z->scsi_addr = HOST_SCSI_ADDR;
@@ -549,14 +536,13 @@ int sea_attach (struct isa_device *dev)
 	adapter_t *z = &seadata[unit];
 	struct scsibus_data *scbus;
 
-	sea_kdc[unit].kdc_state = DC_BUSY; /* host adapters are always busy */
-	sprintf (sea_description, "%s SCSI controller", z->name);
 	printf ("\nsea%d: type %s%s\n", unit, z->name,
 		(dev->id_flags & FLAG_NOPARITY) ? ", no parity" : "");
 
 	/* fill in the prototype scsi_link */
 	z->sc_link.adapter_unit = unit;
 	z->sc_link.adapter_targ = z->scsi_addr;
+	z->sc_link.adapter_softc = z;
 	z->sc_link.adapter = &sea_switch;
 	z->sc_link.device = &sea_dev;
 
@@ -591,13 +577,12 @@ void seaminphys (struct buf *bp)
 /*
  * Catch an interrupt from the adaptor.
  */
-int seaintr (int unit)
+void seaintr (int unit)
 {
 	adapter_t *z = &seadata[unit];
 
 	PRINT (("sea%d: interrupt status=%b\n", unit, *z->STATUS, STAT_BITS));
 	sea_start (z);
-	return (1);
 }
 
 /*
@@ -625,8 +610,8 @@ void sea_tick (void *arg)
  */
 int32_t sea_scsi_cmd (struct scsi_xfer *xs)
 {
-	int unit = xs->sc_link->adapter_unit, flags = xs->flags, x = 0;
-	adapter_t *z = &seadata[unit];
+	int flags = xs->flags, x = 0;
+	adapter_t *z = (adapter_t *)xs->sc_link->adapter_softc;
 	scb_t *scb;
 
 	PRINT (("sea%d/%d/%d command 0x%x\n", unit, xs->sc_link->target,
@@ -634,15 +619,16 @@ int32_t sea_scsi_cmd (struct scsi_xfer *xs)
 	if (xs->bp)
 		flags |= SCSI_NOSLEEP;
 	if (flags & ITSDONE) {
-		printf ("sea%d: already done?", unit);
+		printf ("sea%d: already done?", xs->sc_link->adapter_unit);
 		xs->flags &= ~ITSDONE;
 	}
 	if (! (flags & INUSE)) {
-		printf ("sea%d: not in use?", unit);
+		printf ("sea%d: not in use?", xs->sc_link->adapter_unit);
 		xs->flags |= INUSE;
 	}
 	if (flags & SCSI_RESET)
-		printf ("sea%d: SCSI_RESET not implemented\n", unit);
+		printf ("sea%d: SCSI_RESET not implemented\n",
+			xs->sc_link->adapter_unit);
 
 	if (! (flags & SCSI_NOMASK))
 		x = splbio ();
@@ -697,7 +683,8 @@ int32_t sea_scsi_cmd (struct scsi_xfer *xs)
 			return (SUCCESSFULLY_QUEUED);
 		timeout (sea_timeout, (caddr_t) scb, (xs->timeout * hz) / 1000);
 		scb->flags |= SCB_TIMECHK;
-		PRINT (("sea%d/%d/%d command queued\n", unit,
+		PRINT (("sea%d/%d/%d command queued\n",
+			xs->sc_link->adapter_unit,
 			xs->sc_link->target, xs->sc_link->lun));
 		return (SUCCESSFULLY_QUEUED);
 	}
@@ -718,7 +705,7 @@ int32_t sea_scsi_cmd (struct scsi_xfer *xs)
 			 * this time there is no clock queue entry to remove. */
 			sea_timeout ((void*) scb);
 	}
-	PRINT (("sea%d/%d/%d command %s\n", unit,
+	PRINT (("sea%d/%d/%d command %s\n", xs->sc_link->adapter_unit,
 		xs->sc_link->target, xs->sc_link->lun,
 		xs->error ? "failed" : "done")); 
 	return (xs->error ? HAD_ERROR : COMPLETE);
@@ -763,12 +750,12 @@ again:
 void sea_timeout (void *arg)
 {
 	scb_t *scb = (scb_t*) arg;
-	int unit = scb->xfer->sc_link->adapter_unit;
-	adapter_t *z = &seadata[unit];
+	adapter_t *z = (adapter_t *)scb->xfer->sc_link->adapter_softc;
 	int x = splbio ();
 
 	if (! (scb->xfer->flags & SCSI_NOMASK))
-		printf ("sea%d/%d/%d (%s%d) timed out\n", unit,
+		printf ("sea%d/%d/%d (%s%d) timed out\n",
+			scb->xfer->sc_link->adapter_unit,
 			scb->xfer->sc_link->target,
 			scb->xfer->sc_link->lun,
 			scb->xfer->sc_link->device->name,
