@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)comreg.h	7.2 (Berkeley) 5/9/91
- * $FreeBSD: stable/2.1/sys/i386/isa/sioreg.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/sioreg.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 
@@ -59,10 +59,10 @@
 #define	FIFO_RCV_RST	0x02
 #define	FIFO_XMT_RST	0x04
 #define	FIFO_DMA_MODE	0x08
-#define	FIFO_TRIGGER_1	0x00
-#define	FIFO_TRIGGER_4	0x40
-#define	FIFO_TRIGGER_8	0x80
-#define	FIFO_TRIGGER_14	0xc0
+#define	FIFO_RX_LOW	0x00
+#define	FIFO_RX_MEDL	0x40
+#define	FIFO_RX_MEDH	0x80
+#define	FIFO_RX_HIGH	0xc0
 
 /* character format control register */
 #define	CFCR_DLAB	0x80
@@ -119,3 +119,7 @@
 #ifndef CONUNIT
 #define	CONUNIT	(0)
 #endif
+
+#ifndef CONSPEED
+#define CONSPEED (9600)
+#endif
