@@ -280,6 +280,9 @@ struct sound_timer_operations {
 #ifndef EXCLUDE_AUDIO
 		{SNDCARD_SB16,	"SoundBlaster16",	sb16_dsp_init, sb16_dsp_detect},
 #endif
+#ifndef EXCLUDE_AWE32
+		{SNDCARD_AWE32,	"AWE32 Synth",	attach_awe_obsolete, probe_awe_obsolete},
+#endif
 #ifndef EXCLUDE_MIDI
 		{SNDCARD_SB16MIDI,"SB16 MIDI",	attach_sb16midi, probe_sb16midi},
 #endif
@@ -301,6 +304,11 @@ struct sound_timer_operations {
 		{SNDCARD_TRXPRO, "MediaTriX AudioTriX Pro",	attach_trix_wss, probe_trix_wss},
 		{SNDCARD_TRXPRO_SB, "AudioTriX (SB mode)",	attach_trix_sb, probe_trix_sb},
 		{SNDCARD_TRXPRO_MPU, "AudioTriX MIDI",	attach_trix_mpu, probe_trix_mpu},
+#endif
+#ifdef PC98
+#ifndef EXCLUDE_PCM86
+	  {SNDCARD_PCM86,	"PC-9801-86/73",	attach_pcm86, probe_pcm86},
+#endif
 #endif
 		{0,			"*?*",			NULL, NULL}
 	};
@@ -372,6 +380,9 @@ struct sound_timer_operations {
 #ifndef EXCLUDE_MIDI
 		{SNDCARD_SB16MIDI,{SB16MIDI_BASE, SBC_IRQ, 0}, SND_DEFAULT_ENABLE},
 #endif
+#ifndef EXCLUDE_AWE32
+		{SNDCARD_AWE32,{AWE32_BASE, 0, 0}, SND_DEFAULT_ENABLE},
+#endif
 #endif
 
 #ifndef EXCLUDE_GUS
@@ -383,6 +394,11 @@ struct sound_timer_operations {
 
 #ifndef EXCLUDE_YM3812
 		{SNDCARD_ADLIB, {FM_MONO, 0, 0}, SND_DEFAULT_ENABLE},
+#endif
+#ifdef PC98
+#ifndef EXCLUDE_PCM86
+	     {SNDCARD_PCM86,	{0, 0, 0}, SND_DEFAULT_ENABLE},
+#endif
 #endif
 		{0, {0}, 0}
 	};
@@ -408,14 +424,12 @@ struct sound_timer_operations {
 	extern int num_sound_drivers;
 	extern struct card_info snd_installed_cards[];
 	extern int num_sound_cards;
+#endif	/* _DEV_TABLE_C_ */
 
 int sndtable_probe(int unit, struct address_info *hw_config);
 int sndtable_init_card(int unit, struct address_info *hw_config);
-long sndtable_init(long mem_start);
 int sndtable_get_cardcount (void);
 struct address_info *sound_getconf(int card_type);
-void sound_chconf(int card_type, int ioaddr, int irq, int dma);
 int snd_find_driver(int type);
 
-#endif	/* _DEV_TABLE_C_ */
 #endif	/* _DEV_TABLE_H_ */
