@@ -27,7 +27,7 @@
  *
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
 
 #ifdef CONFIGURE_SOUNDCARD
 
@@ -44,6 +44,10 @@ static struct dma_buffparms dmaps[MAX_AUDIO_DEV] =
 				   * Needs dynamic run-time alloction.
 				 */
 
+static int space_in_queue (int);
+static void reorganize_buffers (int);
+static void dma_init_buffers (int);
+
 static void
 reorganize_buffers (int dev)
 {
@@ -73,10 +77,10 @@ reorganize_buffers (int dev)
 	  sz = 8;
 	}
 
-      sz /= 8;			/* #bits -> #bytes */
-
       sz = sr * nc * sz;
 
+      sz /= 8;			/* #bits -> #bytes */
+
       /*
          * Compute a buffer size for time not exeeding 1 second.
          * Usually this algorithm gives a buffer size for 0.5 to 1.0 seconds
@@ -92,11 +96,11 @@ reorganize_buffers (int dev)
 
       if (dmap->subdivision == 0)	/* Not already set */
 	dmap->subdivision = 1;	/* Init to default value */
+      else
+	bsz /= dmap->subdivision;
 
-      bsz /= dmap->subdivision;
-
-      if (bsz < 64)
-	bsz = 4096;		/* Just a sanity check */
+      if (bsz < 16)
+	bsz = 16;		/* Just a sanity check */
 
       while ((dsp_dev->buffsize * dsp_dev->buffcount) / bsz > MAX_SUB_BUFFERS)
 	bsz *= 2;
@@ -109,11 +113,13 @@ reorganize_buffers (int dev)
          * The process has specified the buffer sice with SNDCTL_DSP_SETFRAGMENT or
          * the buffer sice computation has already been done.
        */
-      if (dmap->fragment_size > audio_devs[dev]->buffsize)
-	dmap->fragment_size = audio_devs[dev]->buffsize;
+      if (dmap->fragment_size > (audio_devs[dev]->buffsize / 2))
+	dmap->fragment_size = (audio_devs[dev]->buffsize / 2);
       bsz = dmap->fragment_size;
     }
 
+  bsz &= ~0x03;			/* Force size which is multiple of 4 bytes */
+
   /*
    * Now computing addresses for the logical buffers
    */
@@ -156,6 +162,8 @@ dma_init_buffers (int dev)
 
   dmap->flags = DMA_BUSY;	/* Other flags off */
   dmap->qlen = dmap->qhead = dmap->qtail = 0;
+  dmap->nbufs = 1;
+  dmap->bytes_in_use = audio_devs[dev]->buffsize;
 
   dmap->dma_mode = DMODE_NONE;
 }
@@ -357,6 +365,7 @@ DMAbuf_getrdbuffer (int dev, char **buf, int *len, int dontblock)
       if (TIMED_OUT (dev_sleeper[dev], dev_sleep_flag[dev]))
 	{
 	  printk ("Sound: DMA timed out - IRQ/DRQ config error?\n");
+	  dma_reset (dev);
 	  err = EIO;
 	  SET_ABORT_FLAG (dev_sleeper[dev], dev_sleep_flag[dev]);
 	}
@@ -505,14 +514,48 @@ DMAbuf_ioctl (int dev, unsigned int cmd, unsigned int arg, int local)
     case SNDCTL_DSP_GETOSPACE:
       if (!local)
 	return RET_ERROR (EINVAL);
+      else
+	{
+	  audio_buf_info *info = (audio_buf_info *) arg;
 
-      {
-	audio_buf_info *info = (audio_buf_info *) arg;
+	  if (!(dmap->flags & DMA_ALLOC_DONE))
+	    reorganize_buffers (dev);
 
-	info->fragments = dmap->qlen;
-	info->fragsize = dmap->fragment_size;
-	info->bytes = dmap->qlen * dmap->fragment_size;
-      }
+	  info->fragstotal = dmap->nbufs;
+
+	  if (cmd == SNDCTL_DSP_GETISPACE)
+	    info->fragments = dmap->qlen;
+	  else
+	    {
+	      if (!space_in_queue (dev))
+		info->fragments = 0;
+	      else
+		{
+		  info->fragments = dmap->nbufs - dmap->qlen;
+		  if (audio_devs[dev]->local_qlen)
+		    {
+		      int             tmp = audio_devs[dev]->local_qlen (dev);
+
+		      if (tmp & info->fragments)
+			tmp--;	/*
+				   * This buffer has been counted twice
+				 */
+		      info->fragments -= tmp;
+		    }
+		}
+	    }
+
+	  if (info->fragments < 0)
+	    info->fragments = 0;
+	  else if (info->fragments > dmap->nbufs)
+	    info->fragments = dmap->nbufs;
+
+	  info->fragsize = dmap->fragment_size;
+	  info->bytes = info->fragments * dmap->fragment_size;
+
+	  if (cmd == SNDCTL_DSP_GETISPACE && dmap->qlen)
+	    info->bytes -= dmap->counts[dmap->qhead];
+	}
       return 0;
 
     default:
@@ -527,7 +570,7 @@ space_in_queue (int dev)
   int             len, max, tmp;
   struct dma_buffparms *dmap = audio_devs[dev]->dmap;
 
-  if (dmap->qlen == dmap->nbufs)	/* No space at all */
+  if (dmap->qlen >= dmap->nbufs)	/* No space at all */
     return 0;
 
   /*
@@ -586,20 +629,19 @@ DMAbuf_getwrbuffer (int dev, char **buf, int *size, int dontblock)
 	return err;
     }
 
-
-  if (dontblock)
-#if defined(__FreeBSD__)
-    return RET_ERROR (EWOULDBLOCK);
-#else
-    return RET_ERROR (EAGAIN);
-#endif
-
   DISABLE_INTR (flags);
 
   abort = 0;
   while (!space_in_queue (dev) &&
 	 !abort)
     {
+
+      if (dontblock)
+	{
+	  RESTORE_INTR (flags);
+	  return RET_ERROR (EAGAIN);
+	}
+
       /*
        * Wait for free space
        */
@@ -607,6 +649,7 @@ DMAbuf_getwrbuffer (int dev, char **buf, int *size, int dontblock)
       if (TIMED_OUT (dev_sleeper[dev], dev_sleep_flag[dev]))
 	{
 	  printk ("Sound: DMA timed out - IRQ/DRQ config error?\n");
+	  dma_reset (dev);
 	  err = EIO;
 	  abort = 1;
 	  SET_ABORT_FLAG (dev_sleeper[dev], dev_sleep_flag[dev]);
@@ -673,8 +716,9 @@ DMAbuf_start_dma (int dev, unsigned long physaddr, int count, int dma_mode)
 {
   int             chan = audio_devs[dev]->dmachan;
   struct dma_buffparms *dmap = audio_devs[dev]->dmap;
+#ifdef linux
   unsigned long   flags;
-
+#endif
   /*
    * This function is not as portable as it should be.
    */
@@ -827,6 +871,10 @@ DMAbuf_outputintr (int dev, int event_type)
       dmap->qhead = (dmap->qhead + 1) % dmap->nbufs;
       dmap->flags &= ~DMA_ACTIVE;
 
+#ifdef __FreeBSD__
+      isa_dmadone(0, 0, 0, audio_devs[dev]->dmachan);
+#endif
+
       if (dmap->qlen)
 	{
 	  audio_devs[dev]->output_block (dev, dmap->buf_phys[dmap->qhead],
@@ -867,6 +915,10 @@ DMAbuf_inputintr (int dev)
 #if defined(SVR42)
   snd_dma_intr (audio_devs[dev]->dmachan);
 #endif /* SVR42 */
+ 
+#ifdef __FreeBSD__
+      isa_dmadone(0, 0, 0, audio_devs[dev]->dmachan);
+#endif
 
   if (dmap->qlen == (dmap->nbufs - 1))
     {
@@ -915,7 +967,10 @@ DMAbuf_open_dma (int dev)
 
   if (ALLOC_DMA_CHN (chan, audio_devs[dev]->name))
     {
+#if 0
+      /* Enough already!  This error is reported twice elsewhere */
       printk ("Unable to grab DMA%d for the audio driver\n", chan);
+#endif
       return RET_ERROR (EBUSY);
     }
 
@@ -941,11 +996,6 @@ DMAbuf_close_dma (int dev)
 void
 DMAbuf_reset_dma (int dev)
 {
-#if 0
-  int             chan = audio_devs[dev]->dmachan;
-
-  disable_dma (chan);
-#endif
 }
 
 #ifdef ALLOW_SELECT
@@ -961,9 +1011,9 @@ DMAbuf_select (int dev, struct fileinfo *file, int sel_type, select_table * wait
       if (dmap->dma_mode != DMODE_INPUT)
 	return 0;
 
+      DISABLE_INTR (flags);
       if (!dmap->qlen)
 	{
-	  DISABLE_INTR (flags);
 #if defined(__FreeBSD__)
 	  selrecord(wait, &selinfo[dev]);
 #else
@@ -973,6 +1023,7 @@ DMAbuf_select (int dev, struct fileinfo *file, int sel_type, select_table * wait
 	  RESTORE_INTR (flags);
 	  return 0;
 	}
+      RESTORE_INTR (flags);
       return 1;
       break;
 
@@ -983,9 +1034,9 @@ DMAbuf_select (int dev, struct fileinfo *file, int sel_type, select_table * wait
       if (dmap->dma_mode == DMODE_NONE)
 	return 1;
 
+      DISABLE_INTR (flags);
       if (!space_in_queue (dev))
 	{
-	  DISABLE_INTR (flags);
 #if defined(__FreeBSD__)
 	  selrecord(wait, &selinfo[dev]);
 #else
@@ -995,6 +1046,7 @@ DMAbuf_select (int dev, struct fileinfo *file, int sel_type, select_table * wait
 	  RESTORE_INTR (flags);
 	  return 0;
 	}
+      RESTORE_INTR (flags);
       return 1;
       break;
 
