@@ -27,9 +27,14 @@
  *
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
 #include <machine/ultrasound.h>
-#include "gus_hw.h"
+#include <i386/isa/sound/gus_hw.h>
+
+static unsigned char gus_look8 __P((int reg));
+static unsigned short gus_read16 __P((int reg));
+static void gus_write_addr __P((int reg, unsigned long address, int is16bit));
+static void gus_write16 __P((int reg, unsigned int data));
 
 #if defined(CONFIGURE_SOUNDCARD) && !defined(EXCLUDE_GUS)
 
@@ -78,17 +83,17 @@ extern int      gus_base;
 extern int      gus_irq, gus_dma;
 static long     gus_mem_size = 0;
 static long     free_mem_ptr = 0;
-       int	gus_busy[MAX_AUDIO_DEV], gus_dspnum=0;
+static int	gus_busy[MAX_AUDIO_DEV], gus_dspnum=0;
 static int	gus_dma_read=0;
 static int      nr_voices = 0;
-       int      gus_devnum = 0;
+static int      gus_devnum = 0;
 static int      volume_base, volume_scale, volume_method;
 static int      gus_recmask = SOUND_MASK_MIC;
 static int      recording_active = 0;
 static int      only_read_access = 0;
 
 int             gus_wave_volume = 60;
-int             gus_pcm_volume = 80;
+static int      gus_pcm_volume = 80;
 int             have_gus_max = 0;
 static int      gus_line_vol = 100, gus_mic_vol = 0;
 static unsigned char mix_image = 0x00;
@@ -126,7 +131,11 @@ static unsigned long pcm_current_buf;
 static int      pcm_current_count;
 static int      pcm_current_intrflag;
 
-struct voice_info voices[32];
+#if defined(__FreeBSD__)
+static char    *gus_copy_buf;
+#endif
+
+static struct voice_info voices[32];
 
 static int      freq_div_table[] =
 {
@@ -253,7 +262,7 @@ gus_write8 (int reg, unsigned int data)
   RESTORE_INTR (flags);
 }
 
-unsigned char
+static unsigned char
 gus_read8 (int reg)
 {				/* Reads from an indirect register (8 bit). Offset 0x80. */
   unsigned long   flags;
@@ -267,7 +276,7 @@ gus_read8 (int reg)
   return val;
 }
 
-unsigned char
+static unsigned char
 gus_look8 (int reg)
 {				/* Reads from an indirect register (8 bit). No additional offset. */
   unsigned long   flags;
@@ -281,7 +290,7 @@ gus_look8 (int reg)
   return val;
 }
 
-void
+static void
 gus_write16 (int reg, unsigned int data)
 {				/* Writes to an indirect register (16 bit) */
   unsigned long   flags;
@@ -296,7 +305,7 @@ gus_write16 (int reg, unsigned int data)
   RESTORE_INTR (flags);
 }
 
-unsigned short
+static unsigned short
 gus_read16 (int reg)
 {				/* Reads from an indirect register (16 bit). Offset 0x80. */
   unsigned long   flags;
@@ -314,7 +323,7 @@ gus_read16 (int reg)
   return ((hi << 8) & 0xff00) | lo;
 }
 
-void
+static void
 gus_write_addr (int reg, unsigned long address, int is16bit)
 {				/* Writes an 24 bit memory address */
   unsigned long   hold_address;
@@ -1998,11 +2007,13 @@ static int
 gus_sampling_open (int dev, int mode)
 {
   int dev_flag;
+  int init_flag;
 #ifdef GUS_NO_DMA
   printk ("GUS: DMA mode not enabled. Device not supported\n");
   return RET_ERROR (ENXIO);
 #endif
   dev_flag = 0;
+  init_flag = (gus_busy[gus_devnum] == 0 && gus_busy[gus_dspnum] == 0);
   if(mode & OPEN_WRITE)
    {
      if (gus_busy[gus_devnum])
@@ -2024,18 +2035,20 @@ gus_sampling_open (int dev, int mode)
      }
    }
 
-  if(gus_busy[gus_devnum] == 0 && gus_busy[gus_dspnum] == 0)
-    gus_initialize ();
+  if(init_flag)
+    {
+      gus_initialize ();
 
-  active_device = 0;
+      active_device = 0;
 
-  gus_reset ();
-  reset_sample_memory ();
-  gus_select_max_voices (14);
+      gus_reset ();
+      reset_sample_memory ();
+      gus_select_max_voices (14);
 
-  pcm_active = 0;
-  dma_active = 0;
-  pcm_opened = 1;
+      pcm_active = 0;
+      dma_active = 0;
+      pcm_opened = 1;
+    }
   if (mode & OPEN_READ)
     {
       recording_active = 1;
@@ -2050,7 +2063,7 @@ static void
 gus_sampling_close (int dev)
 {
   gus_busy[dev] = 0;
-  if (gus_busy[gus_devnum] == 0 && gus_busy[gus_dspnum]) {
+  if (gus_busy[gus_devnum] == 0 && gus_busy[gus_dspnum] == 0) {
     active_device = 0;
     gus_reset();
     pcm_opened = 0;
@@ -2387,6 +2400,47 @@ gus_copy_from_user (int dev, char *localbuf, int localoffs,
       COPY_FROM_USER (&localbuf[localoffs], userbuf, useroffs, len);
     }
   else if (gus_sampling_bits == 8)
+#if defined(__FreeBSD__)
+    {
+      char           *in_left = gus_copy_buf;
+      char           *in_right = in_left + 1;
+      char           *out_left = localbuf + (localoffs / 2);
+      char           *out_right = out_left + pcm_bsize;
+      int             i;
+
+      COPY_FROM_USER (gus_copy_buf, userbuf, useroffs, len);
+
+      len /= 2;
+
+      for (i = 0; i < len; i++)
+	{
+	  *out_left++ = *in_left++;
+	  in_left++;
+	  *out_right++ = *in_right++;
+	  in_right++;
+	}
+    }
+  else
+    {
+      short          *in_left = (short *)gus_copy_buf;
+      short          *in_right = in_left + 1;
+      short          *out_left = (short *)localbuf + (localoffs / 4);
+      short          *out_right = out_left + (pcm_bsize / 2);
+      int             i;
+
+      COPY_FROM_USER (gus_copy_buf, userbuf, useroffs, len);
+
+      len /= 4;
+
+      for (i = 0; i < len; i++)
+	{
+	  *out_left++ = *in_left++;
+	  in_left++;
+	  *out_right++ = *in_right++;
+	  in_right++;
+	}
+    }
+#else
     {
       int             in_left = useroffs;
       int             in_right = useroffs + 1;
@@ -2434,6 +2488,7 @@ gus_copy_from_user (int dev, char *localbuf, int localoffs,
 #endif
 	}
     }
+#endif
 }
 
 static struct audio_operations gus_sampling_operations =
@@ -3050,6 +3105,10 @@ gus_wave_init (long mem_start, int irq, int dma, int dma_read)
       synth_devs[num_synths++] = &guswave_operations;
     }
 
+#if defined(__FreeBSD__)
+  PERMANENT_MALLOC (char *, gus_copy_buf, DSP_BUFFSIZE, mem_start);
+#endif
+
   PERMANENT_MALLOC (struct patch_info *, samples,
 	                   (MAX_SAMPLE + 1) * sizeof (*samples), mem_start);
 
