@@ -3,7 +3,12 @@
  * $Id: local.h,v 1.11 1994/11/01 17:26:50 ache Exp
  */
 
+#ifdef PC98
+/* If a soundcard uses DMA AUTO mode, DPS_BUFFSIZE must be < 64KB. */
+#define DSP_BUFFSIZE 61440
+#else
 #define DSP_BUFFSIZE 65536
+#endif
 #define SELECTED_SOUND_OPTIONS  0xffffffff
 #define SOUND_CONFIG_DATE "Sun Feb 5 14:38:12 EST 1995"
 #define SOUND_CONFIG_BY "freebsd-hackers"
@@ -57,7 +62,7 @@
 # define GUSMAX_MIXER
 #endif
 
-#include <sb.h>
+#include "sb.h"
 #if NSB == 0 && !defined(EXCLUDE_SB)
 #define EXCLUDE_SB
 #endif
@@ -72,7 +77,12 @@
 #define EXCLUDE_SB16MIDI
 #endif
 
-#include <pas.h>
+#include "awe.h"
+#if NAWE == 0 && !defined(EXCLUDE_AWE32)
+#define EXCLUDE_AWE32
+#endif
+
+#include "pas.h"
 #if NPAS == 0 && !defined(EXCLUDE_PAS)
 #define EXCLUDE_PAS
 #endif
@@ -92,6 +102,14 @@
 #define EXCLUDE_UART6850
 #endif
 
+#ifdef PC98
+#include "pcm.h"
+#if NPCM == 0 && !defined(EXCLUDE_PCM86)
+#define EXCLUDE_PCM86
+#endif
+#endif
+
+
 /* nothing but a sequencer (Adlib/OPL) ? */
 #if NGUS == 0 && NSB == 0 && NSBMIDI == 0 && NPAS == 0 && NMPU == 0 && \
     NUART == 0 && NMSS == 0
@@ -99,9 +117,11 @@
 #define EXCLUDE_MIDI
 #endif
 #ifndef EXCLUDE_AUDIO
+#if !defined(PC98) || defined(EXCLUDE_PCM86) && defined(EXCLUDE_MSS)
 #define EXCLUDE_AUDIO
 #endif
 #endif
+#endif
 
 /* nothing but a Midi (MPU/UART) ? */
 #if NGUS == 0 && NSB == 0 && NSBMIDI == 0 && NPAS == 0 && NOPL == 0 && \
@@ -111,6 +131,8 @@
 #define EXCLUDE_SEQUENCER
 #endif
 #ifndef EXCLUDE_AUDIO
+#if !defined(PC98) || defined(EXCLUDE_PCM86) && defined(EXCLUDE_MSS)
 #define EXCLUDE_AUDIO
 #endif
 #endif
+#endif
