@@ -27,7 +27,9 @@
  *
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
+
+static void drain_midi_queue __P((int dev));
 
 #if defined(CONFIGURE_SOUNDCARD) && !defined(EXCLUDE_MIDI)
 
@@ -87,7 +89,7 @@ static volatile int open_devs = 0;
 	  RESTORE_INTR(flags); \
 	}
 
-void
+static void
 drain_midi_queue (int dev)
 {
 
