@@ -22,9 +22,9 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/conf.h>
 #include <sys/file.h>
+#include <sys/filio.h>
 #include <sys/uio.h>
 #include <sys/kernel.h>
 #include <sys/syslog.h>
@@ -33,8 +33,7 @@
 #include <sys/buf.h>
 #include <i386/isa/isa_device.h>
 #include <machine/cpufunc.h>
-
-/* These few lines are used by 386BSD (only??). */
+#include <sys/signalvar.h>
 
 #if NSND > 0
 #define CONFIGURE_SOUNDCARD
@@ -117,13 +116,13 @@ typedef struct uio snd_rw_buf;
 /*
  * The way how the ioctl arguments are passed is another nonportable thing.
  * In Linux the argument is just a pointer directly to the user segment. On
- * 386bsd the data is already moved to the kernel space. The following
+ * FreeBSD the data is already moved to the kernel space. The following
  * macros should handle the difference.
  */
 
 /*
  * IOCTL_FROM_USER is used to copy a record pointed by the argument to
- * a buffer in the kernel space. On 386bsd it can be done just by calling
+ * a buffer in the kernel space. On FreeBSD it can be done just by calling
  * memcpy. With Linux a memcpy_from_fs should be called instead.
  * Parameters of the following macros are like in the COPY_*_USER macros.
  */
@@ -175,30 +174,30 @@ struct snd_wait {
  * is aborts the process. This macro is called from close() to see if the
  * buffers should be discarded. If this kind info is not available, a constant
  * 1 or 0 could be returned (1 should be better than 0).
- * I'm not sure if the following is correct for 386BSD.
  */
-#define PROCESS_ABORTING(q, f) (f.aborting | curproc->p_siglist)
+#define PROCESS_ABORTING(q, f) (f.aborting || CURSIG(curproc))
 
 /*
- * The following macro calls sleep. It should be implemented such that
- * the process is resumed if it receives a signal. The following is propably
- * not the way how it should be done on 386bsd.
- * The on_what parameter is a wait_queue defined with DEFINE_WAIT_QUEUE(),
+ * The following macro calls tsleep. It should be implemented such that
+ * the process is resumed if it receives a signal.
+ * The q parameter is a wait_queue defined with DEFINE_WAIT_QUEUE(),
  * and the second is a workarea parameter. The third is a timeout 
  * in ticks. Zero means no timeout.
  */
 #define DO_SLEEP(q, f, time_limit)	\
 	{ \
-	  int flag, chn; \
+	  int flag; \
 	  f.mode = WK_SLEEP; \
-	  q = &chn; \
-	  flag=tsleep((caddr_t) &chn, (PRIBIO-5)|PCATCH, "sndint", time_limit); \
-	  if(flag == ERESTART) f.aborting = 1;\
-	  else f.aborting = 0;\
+	  flag=tsleep(&q, (PRIBIO-5)|PCATCH, "sndint", time_limit); \
 	  f.mode &= ~WK_SLEEP; \
+	  if (flag == EWOULDBLOCK) { \
+		f.mode |= WK_TIMEOUT; \
+		f.aborting = 0; \
+	  } else \
+		f.aborting = flag; \
 	}
 /* An the following wakes up a process */
-#define WAKE_UP(q, f)	{f.mode = WK_WAKEUP;wakeup((caddr_t) q);}
+#define WAKE_UP(q, f)   wakeup(&q)
 
 /*
  * Timing macros. This driver assumes that there is a timer running in the
@@ -248,14 +247,19 @@ extern unsigned long get_time(void);
  * The outb(0, 0x80) is just for slowdown. It's bit unsafe since
  * this address could be used for something usefull.
  */
-#define OUTB(addr, data)	{outb(data, addr);outb(0, 0x80);}
-#define OUTW(addr, data)	{outw(data, addr);outb(0, 0x80);}
+#ifdef PC98
+#define OUTB(addr, data)	{outb(data, addr);outb(0x5f, 0);}
+#define OUTW(addr, data)	{outw(data, addr);outb(0x5f, 0);}
+#else /* IBM-PC */
+#define OUTB(addr, data)	{outb(data, addr);outb(0x80, 0);}
+#define OUTW(addr, data)	{outw(data, addr);outb(0x80, 0);}
+#endif /* PC98 */
 #else
 #define OUTB(addr, data)	outb(data, addr)
 #define OUTW(addr, data)	outw(data, addr)
 #endif
 
-/* memcpy() was not defined og 386bsd. Lets define it here */
+/* memcpy() was not defined on FreeBSD. Lets define it here */
 #define memcpy(d, s, c)		bcopy(s, d, c)
 
 /*
@@ -309,8 +313,8 @@ extern unsigned long get_time(void);
  * The rest of this file is not complete yet. The functions using these
  * macros will not work
  */
-#define ALLOC_DMA_CHN(chn,deviceID) ({0; } )
-#define RELEASE_DMA_CHN(chn) ({ 0; })
+#define ALLOC_DMA_CHN(chn,deviceID) (isa_dma_acquire(chn))
+#define RELEASE_DMA_CHN(chn) (isa_dma_release(chn))
 #define DMA_MODE_READ		0
 #define DMA_MODE_WRITE		1
 #define RELEASE_IRQ(irq_no)
