@@ -33,12 +33,12 @@
  *      Code added for MV ProSonic/Jazz 16 in 16 bit mode
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
 
 #if defined(CONFIGURE_SOUNDCARD) && !defined(EXCLUDE_SB)
 
-#include "sb.h"
-#include "sb_mixer.h"
+#include <i386/isa/sound/sb_defs.h>
+#include <i386/isa/sound/sb_mixer.h>
 #undef SB_TEST_IRQ
 
 int             sbc_base = 0;
@@ -901,7 +901,7 @@ static int
 initialize_smw (void)
 {
 #ifdef SMW_MIDI0001_INCLUDED
-#include "smw-midi0001.h"
+#include <i386/isa/sound/smw-midi0001.h>
 #else
   unsigned char   smw_ucode[1];
   int             smw_ucodeLen = 0;
@@ -1067,6 +1067,30 @@ sb_dsp_detect (struct address_info *hw_config)
     return 0;
 #endif
 
+#ifdef PC98
+  switch (sbc_irq)
+    {
+    case 3:
+      sb_setmixer (IRQ_NR, 1);
+      break;
+    case 5:
+      sb_setmixer (IRQ_NR, 8);
+      break;
+    case 10:
+      sb_setmixer (IRQ_NR, 2);
+      break;
+    }
+  switch (hw_config->dma)
+    {
+    case 0:
+      sb_setmixer (DMA_NR, 1);
+      break;
+    case 3:
+      sb_setmixer (DMA_NR, 2);
+      break;
+    }
+#endif
+
   return 1;			/*
 				 * Detected
 				 */
@@ -1137,8 +1161,13 @@ sb_dsp_init (long mem_start, struct address_info *hw_config)
 
 #ifndef EXCLUDE_YM3812
 
+#ifdef PC98
+  if (sbc_major > 3 ||
+      (sbc_major == 3 && INB (0x28d2) == 0x00))
+#else
   if (sbc_major > 3 ||
       (sbc_major == 3 && INB (0x388) == 0x00))	/* Should be 0x06 if not OPL-3 */
+#endif
     enable_opl3_mode (OPL3_LEFT, OPL3_RIGHT, OPL3_BOTH);
 #endif
 
@@ -1191,7 +1220,9 @@ sb_dsp_init (long mem_start, struct address_info *hw_config)
       {
 	audio_devs[my_dev = num_audiodevs++] = &sb_dsp_operations;
 	audio_devs[my_dev]->buffcount = DSP_BUFFCOUNT;
-	audio_devs[my_dev]->buffsize = DSP_BUFFSIZE;
+	audio_devs[my_dev]->buffsize = (
+		(sbc_major > 2 || sbc_major == 2 && sbc_minor > 0) ?
+		16 : 8) * 1024;
 	audio_devs[my_dev]->dmachan = hw_config->dma;
       }
     else
