@@ -28,8 +28,10 @@
  */
 
 #define SEQUENCER_C
-#include "sound_config.h"
-#include "midi_ctrl.h"
+#include <i386/isa/sound/sound_config.h>
+#include <i386/isa/sound/midi_ctrl.h>
+
+extern void seq_drain_midi_queues __P((void));
 
 #ifdef CONFIGURE_SOUNDCARD
 
@@ -65,11 +67,11 @@ static int      midi_opened[MAX_MIDI_DEV] =
 static int      midi_written[MAX_MIDI_DEV] =
 {0};
 
-unsigned long   prev_input_time = 0;
-int             prev_event_time;
-unsigned long   seq_time = 0;
+static unsigned long   prev_input_time = 0;
+static int             prev_event_time;
+static unsigned long   seq_time = 0;
 
-#include "tuning.h"
+#include <i386/isa/sound/tuning.h>
 
 #define EV_SZ	8
 #define IEV_SZ	8
