@@ -98,7 +98,6 @@ long soundcard_init(long mem_start);
 void tenmicrosec(void);
 void request_sound_timer (int count);
 void sound_stop_timer(void);
-int snd_ioctl_return(int *addr, int value);
 int snd_set_irq_handler (int interrupt_level, INT_HANDLER_PROTO(), char *name);
 void snd_release_irq(int vect);
 void sound_dma_malloc(int dev);
@@ -153,6 +152,10 @@ int probe_sb(struct address_info *hw_config);
 long attach_adlib_card(long mem_start, struct address_info *hw_config);
 int probe_adlib(struct address_info *hw_config);
 
+/* from awe_wave.c */
+long attach_awe_obsolete(long mem_start, struct address_info *hw_config);
+int probe_awe_obsolete(struct address_info *hw_config);
+
 /*	From pas_card.c	*/
 long attach_pas_card(long mem_start, struct address_info *hw_config);
 int probe_pas(struct address_info *hw_config);
@@ -176,7 +179,6 @@ void pas_midi_interrupt(void);
 long attach_gus_card(long mem_start, struct address_info * hw_config);
 int probe_gus(struct address_info *hw_config);
 int gus_set_midi_irq(int num);
-void gusintr(INT_HANDLER_PARMS(irq, dummy));
 long attach_gus_db16(long mem_start, struct address_info * hw_config);
 int probe_gus_db16(struct address_info *hw_config);
 
@@ -184,7 +186,6 @@ int probe_gus_db16(struct address_info *hw_config);
 int gus_wave_detect(int baseaddr);
 long gus_wave_init(long mem_start, int irq, int dma, int dma_read);
 void gus_voice_irq(void);
-unsigned char gus_read8 (int reg);
 void gus_write8(int reg, unsigned int data);
 void guswave_dma_irq(void);
 void gus_delay(void);
@@ -252,6 +253,12 @@ int pss_ioctl (int dev, struct fileinfo *file,
 int pss_lseek (int dev, struct fileinfo *file, off_t offset, int orig);
 long pss_init(long mem_start);
 
+#ifdef PC98
+/*	From pcm86.c */
+int probe_pcm86(struct address_info *hw_config);
+long attach_pcm86(long mem_start, struct address_info *hw_config);
+#endif PC98
+
 /* From aedsp16.c */
 int InitAEDSP16_SBPRO(struct address_info *hw_config);
 int InitAEDSP16_MSS(struct address_info *hw_config);
