@@ -26,27 +26,27 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/sound/soundcard.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/sound/soundcard.c 50950 1999-09-05 08:39:22Z peter $
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
+#include <vm/vm.h>
+#include <vm/vm_extern.h>
 
 #ifdef CONFIGURE_SOUNDCARD
 
-#include "dev_table.h"
+#include <i386/isa/sound/dev_table.h>
 #include <i386/isa/isa_device.h>
+#include <sys/conf.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
-u_int	snd1_imask;
-u_int	snd2_imask;
-u_int	snd3_imask;
-u_int	snd4_imask;
-u_int	snd5_imask;
-u_int	snd6_imask;
-u_int	snd7_imask;
-u_int	snd8_imask;
-u_int	snd9_imask;
-
-#define FIX_RETURN(ret) {if ((ret)<0) return -(ret); else return 0;}
+#define FIX_RETURN(ret) { \
+			  int tmp_ret = (ret); \
+			  if (tmp_ret<0) return -tmp_ret; else return 0; \
+			}
 
 static int      timer_running = 0;
 
@@ -55,22 +55,36 @@ static int      soundcards_installed = 0;	/* Number of installed
 static int      soundcard_configured = 0;
 
 static struct fileinfo files[SND_NDEVS];
+
+#ifdef DEVFS
+static void * snd_devfs_token[SND_NDEVS];
+static void * sndstat_devfs_token;
+#endif 
+
 struct selinfo selinfo[SND_NDEVS >> 4];
 
-int             sndprobe (struct isa_device *dev);
-int             sndattach (struct isa_device *dev);
-int             sndopen (dev_t dev, int flags, int fmt, struct proc *p);
-int             sndclose (dev_t dev, int flags, int fmt, struct proc *p);
-int             sndioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p);
-int             sndread (dev_t dev, struct uio *uio, int ioflag);
-int             sndwrite (dev_t dev, struct uio *uio, int ioflag);
-int             sndselect (dev_t dev, int rw, struct proc *p);
+static int	sndprobe (struct isa_device *dev);
+static int	sndattach (struct isa_device *dev);
 static void	sound_mem_init(void);
 
+static d_open_t		sndopen;
+static d_close_t	sndclose;
+static d_read_t		sndread;
+static d_write_t	sndwrite;
+static d_ioctl_t	sndioctl;
+static d_select_t	sndselect;
+
+#define CDEV_MAJOR 30
+static struct cdevsw snd_cdevsw = 
+	{ sndopen,	sndclose,	sndread,	sndwrite,	/*30*/
+  	  sndioctl,	nostop,		nullreset,	nodevtotty,/* sound */
+  	  sndselect,	nommap,		NULL,	"snd", NULL, -1 };
+
 struct isa_driver opldriver	= {sndprobe, sndattach, "opl"};
 struct isa_driver sbdriver	= {sndprobe, sndattach, "sb"};
 struct isa_driver sbxvidriver	= {sndprobe, sndattach, "sbxvi"};
 struct isa_driver sbmididriver	= {sndprobe, sndattach, "sbmidi"};
+struct isa_driver awedriver	= {sndprobe, sndattach, "awe"};
 struct isa_driver pasdriver	= {sndprobe, sndattach, "pas"};
 struct isa_driver mpudriver	= {sndprobe, sndattach, "mpu"};
 struct isa_driver gusdriver	= {sndprobe, sndattach, "gus"};
@@ -78,6 +92,9 @@ struct isa_driver gusxvidriver	= {sndprobe, sndattach, "gusxvi"};
 struct isa_driver gusmaxdriver	= {sndprobe, sndattach, "gusmax"};
 struct isa_driver uartdriver	= {sndprobe, sndattach, "uart"};
 struct isa_driver mssdriver	= {sndprobe, sndattach, "mss"};
+#ifdef PC98
+struct isa_driver pcmdriver	= {sndprobe, sndattach, "pcm"};
+#endif
 
 static unsigned short
 ipri_to_irq (unsigned short ipri);
@@ -118,7 +135,7 @@ int x;
 }
  
 
-int
+static int
 sndread (dev_t dev, struct uio *buf, int ioflag)
 {
   int             count = buf->uio_resid;
@@ -128,7 +145,7 @@ sndread (dev_t dev, struct uio *buf, int ioflag)
   FIX_RETURN (sound_read_sw (dev, &files[dev], buf, count));
 }
 
-int
+static int
 sndwrite (dev_t dev, struct uio *buf, int ioflag)
 {
   int             count = buf->uio_resid;
@@ -138,11 +155,9 @@ sndwrite (dev_t dev, struct uio *buf, int ioflag)
   FIX_RETURN (sound_write_sw (dev, &files[dev], buf, count));
 }
 
-int
+static int
 sndopen (dev_t dev, int flags, int fmt, struct proc *p)
 {
-  int             retval;
-
   dev = minor (dev);
 
   if (!soundcard_configured && dev)
@@ -166,7 +181,7 @@ sndopen (dev_t dev, int flags, int fmt, struct proc *p)
   FIX_RETURN(sound_open_sw (dev, &files[dev]));
 }
 
-int
+static int
 sndclose (dev_t dev, int flags, int fmt, struct proc *p)
 {
 
@@ -176,7 +191,7 @@ sndclose (dev_t dev, int flags, int fmt, struct proc *p)
   FIX_RETURN (0);
 }
 
-int
+static int
 sndioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
 {
   dev = minor (dev);
@@ -184,7 +199,7 @@ sndioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
   FIX_RETURN (sound_ioctl_sw (dev, &files[dev], cmd, (unsigned int) arg));
 }
 
-int
+static int
 sndselect (dev_t dev, int rw, struct proc *p)
 {
   dev = minor (dev);
@@ -257,17 +272,23 @@ driver_to_voxunit(struct isa_driver *driver)
     return(SNDCARD_SB16);
   else if(driver == &sbmididriver)
     return(SNDCARD_SB16MIDI);
+  else if(driver == &awedriver)
+    return(SNDCARD_AWE32);
   else if(driver == &uartdriver)
     return(SNDCARD_UART6850);
   else if(driver == &gusdriver)
     return(SNDCARD_GUS16);
   else if(driver == &mssdriver)
     return(SNDCARD_MSS);
+#ifdef PC98
+  else if(driver == &pcmdriver)
+    return(SNDCARD_PCM86);
+#endif
   else
     return(0);
 }
 
-int
+static int
 sndprobe (struct isa_device *dev)
 {
   struct address_info hw_config;
@@ -280,18 +301,30 @@ sndprobe (struct isa_device *dev)
   hw_config.dma_read = dev->id_flags;	/* misuse the flags field for read dma*/
   
   if(unit)
+#ifdef PC98
+    if(unit == SNDCARD_PCM86) {
+      int result;
+
+      result = sndtable_probe (unit, &hw_config);
+      dev->id_iobase = hw_config.io_base;
+      dev->id_irq = (1 << hw_config.irq);
+      dev->id_drq = hw_config.dma;
+
+      return result;
+    }
+    else
+#endif
     return sndtable_probe (unit, &hw_config);
   else
     return 0;
 }
 
-int
+static int
 sndattach (struct isa_device *dev)
 {
-  int             i, unit;
+  int             unit;
   static int      midi_initialized = 0;
   static int      seq_initialized = 0;
-  static int 	  generic_midi_initialized = 0; 
   unsigned long	  mem_start = 0xefffffffUL;
   struct address_info hw_config;
 
@@ -348,6 +381,57 @@ sndattach (struct isa_device *dev)
     }
 #endif
 
+#ifdef DEVFS
+/* XXX */ /* find out where to store the tokens.. */
+/* XXX */ /* should only create devices if that card has them */
+#define SND_UID 0
+#define SND_GID 13
+
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_CTL, DV_CHR, 
+				 SND_UID,  SND_GID, 0660, "mixer%d", unit);
+
+#ifndef EXCLUDE_SEQUENCER
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_SEQ, DV_CHR,
+				 SND_UID,  SND_GID, 0660, "sequencer%d", unit);
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_SEQ2, DV_CHR,
+				 SND_UID,  SND_GID, 0660, "music%d", unit);
+#endif
+
+#ifndef EXCLUDE_MIDI
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_MIDIN,
+				 DV_CHR, SND_UID,  SND_GID, 0660, "midi%d", 
+				 unit);
+#endif
+
+#ifndef EXCLUDE_AUDIO
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_DSP, DV_CHR,
+				 SND_UID,  SND_GID, 0660, "dsp%d", unit);
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_AUDIO,
+				 DV_CHR, SND_UID,  SND_GID, 0660, "audio%d", 
+				 unit);
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_DSP16,
+				 DV_CHR, SND_UID,  SND_GID, 0660, "dspW%d", 
+				 unit);
+#endif
+
+    snd_devfs_token[unit]=
+		devfs_add_devswf(&snd_cdevsw, (unit << 4)+SND_DEV_SNDPROC,
+		 		 DV_CHR, SND_UID,  SND_GID, 0660, "pss%d", 
+				 unit);
+
+    if ( ! sndstat_devfs_token) {
+        sndstat_devfs_token = 
+		devfs_add_devswf(&snd_cdevsw, 6, DV_CHR, SND_UID, SND_GID, 
+				 0660, "sndstat");
+    }
+#endif /* DEVFS */
   return TRUE;
 }
 
@@ -400,14 +484,18 @@ sound_stop_timer (void)
 static void
 sound_mem_init (void)
 {
-  int             i, dev;
+  int             dev;
   unsigned long   dma_pagesize;
   struct dma_buffparms *dmap;
   static unsigned long dsp_init_mask = 0;
 
   for (dev = 0; dev < num_audiodevs; dev++)	/* Enumerate devices */
     if (!(dsp_init_mask & (1 << dev)))	/* Not already done */
+#ifdef PC98
+      if (audio_devs[dev]->buffcount > 0 && audio_devs[dev]->dmachan >= 0)
+#else
       if (audio_devs[dev]->buffcount > 0 && audio_devs[dev]->dmachan > 0)
+#endif
 	{
 	  dsp_init_mask |= (1 << dev);
 	  dmap = audio_devs[dev]->dmap;
@@ -415,6 +503,8 @@ sound_mem_init (void)
 	  if (audio_devs[dev]->flags & DMA_AUTOMODE)
 	    audio_devs[dev]->buffcount = 1;
 
+	  audio_devs[dev]->buffsize &= ~0xfff;	/* Truncate to n*4k */
+
 	  if (audio_devs[dev]->dmachan > 3 && audio_devs[dev]->buffsize > 65536)
 	    dma_pagesize = 131072;	/* 128k */
 	  else
@@ -424,7 +514,6 @@ sound_mem_init (void)
 
 	  if (audio_devs[dev]->buffsize > dma_pagesize)
 	    audio_devs[dev]->buffsize = dma_pagesize;
-	  audio_devs[dev]->buffsize &= ~0xfff;	/* Truncate to n*4k */
 	  if (audio_devs[dev]->buffsize < 4096)
 	    audio_devs[dev]->buffsize = 4096;
 
@@ -436,7 +525,7 @@ sound_mem_init (void)
 
 	      if (tmpbuf == NULL)
 		{
-		  printk ("snd: Unable to allocate %d bytes of buffer\n",
+		  printk ("snd: Unable to allocate %ld bytes of buffer\n",
 			  audio_devs[dev]->buffsize);
 		  return;
 		}
@@ -455,15 +544,6 @@ sound_mem_init (void)
 
 #endif
 
-int
-snd_ioctl_return (int *addr, int value)
-{
-  if (value < 0)
-    return value;		/* Error */
-  suword (addr, value);
-  return 0;
-}
-
 int
 snd_set_irq_handler (int interrupt_level, INT_HANDLER_PROTO(), char *name)
 {
@@ -476,4 +556,21 @@ snd_release_irq(int vect)
 {
 }
 
+static snd_devsw_installed = 0;
+
+static void 
+snd_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! snd_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&snd_cdevsw, NULL);
+		snd_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(snddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,snd_drvinit,NULL)
+
 #endif
+
