@@ -27,11 +27,11 @@
  *
  */
 
-#include "sound_config.h"
+#include <i386/isa/sound/sound_config.h>
 
 #if defined(CONFIGURE_SOUNDCARD) && !defined(EXCLUDE_SSCAPE)
 
-#include "coproc.h"
+#include <i386/isa/sound/coproc.h>
 
 /*
  *    I/O ports
