@@ -40,7 +40,7 @@
  *
  *	ftp.cs.uwm.edu://pub/FreeBSD/spigot/spigot.tar.gz
  *
- * Version 1.5, August 30, 1995.
+ * Version 1.7, December 1995.
  *
  */
 
@@ -54,28 +54,35 @@ error "Can only have 1 spigot configured."
 #include	<sys/param.h>
 #include	<sys/systm.h>
 #include	<sys/kernel.h>
+#include	<sys/conf.h>
 #include	<sys/ioctl.h>
 #include	<sys/proc.h>
-#include	<sys/user.h>
+#include	<sys/signalvar.h>
 #include	<sys/file.h>
 #include	<sys/uio.h>
 #include	<sys/malloc.h>
-#include	<sys/devconf.h>
 #include	<sys/errno.h>
 #include	<sys/mman.h>
+#ifdef DEVFS
+#include	<sys/devfsext.h>
+#endif /* DEVFS */
 
+#include	<machine/frame.h>
+#include	<machine/md_var.h>
 #include	<machine/spigot.h>
 #include	<machine/psl.h>
 
-#include	<i386/isa/isa.h>
 #include	<i386/isa/isa_device.h>
 
-struct spigot_softc {
+static struct spigot_softc {
 	u_long		flags;
 	u_long	 	maddr;
 	struct proc	*p;
 	u_long		signal_num;
 	u_short		irq;
+#ifdef	DEVFS
+	void	*devfs_token;
+#endif
 } spigot_softc[NSPIGOT];
 
 /* flags in softc */
@@ -84,44 +91,26 @@ struct spigot_softc {
 
 #define	UNIT(dev) minor(dev)
 
-int	spigot_probe(struct isa_device *id);
-int	spigot_attach(struct isa_device *id);
+static int	spigot_probe(struct isa_device *id);
+static int	spigot_attach(struct isa_device *id);
 
 struct isa_driver	spigotdriver = {spigot_probe, spigot_attach, "spigot"};
 
-static struct kern_devconf kdc_spigot[NSPIGOT] = { {
-	0,			/* kdc_next -> filled in by dev_attach() */
-	0,			/* kdc_rlink -> filled in by dev_attach() */
-	0,			/* kdc_number -> filled in by dev_attach() */
-	"spigot",			/* kdc_name */
-	0,				/* kdc_unit */
-	{ 				/* kdc_md */
-  	   MDDT_ISA,			/* mddc_devtype */
-	   0				/* mddc_flags */
-					/* mddc_imask[4] */
-	},
-	isa_generic_externalize,	/* kdc_externalize */
-	0,				/* kdc_internalize */
-	0,				/* kdc_goaway */
-	ISA_EXTERNALLEN,		/* kdc_datalen */
-	&kdc_isa0,			/* kdc_parent */
-	0,				/* kdc_parentdata */
-	DC_UNCONFIGURED,	/* kdc_state - not supported */
-	"Video Spigot frame grabber",	/* kdc_description */
-	DC_CLS_MISC		/* class */
-} };
+static	d_open_t	spigot_open;
+static	d_close_t	spigot_close;
+static	d_read_t	spigot_read;
+static	d_write_t	spigot_write;
+static	d_ioctl_t	spigot_ioctl;
+static	d_select_t	spigot_select;
+static	d_mmap_t	spigot_mmap;
 
-static inline void
-spigot_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_spigot[id->id_unit] = kdc_spigot[0];
-	kdc_spigot[id->id_unit].kdc_unit = id->id_unit;
-	kdc_spigot[id->id_unit].kdc_isa = id;
-	dev_attach(&kdc_spigot[id->id_unit]);
-}
+#define CDEV_MAJOR 11
+static struct cdevsw spigot_cdevsw = 
+	{ spigot_open,	spigot_close,	spigot_read,	spigot_write,	/*11*/
+	  spigot_ioctl,	nostop,		nullreset,	nodevtotty,/* Spigot */
+	  spigot_select, spigot_mmap,	NULL,	"spigot",	NULL,	-1  };
 
-int
+static int
 spigot_probe(struct isa_device *devp)
 {
 int			status;
@@ -136,26 +125,29 @@ struct	spigot_softc	*ss=(struct spigot_softc *)&spigot_softc[devp->id_unit];
 	else {
 		status = 1;	/* found */
 		ss->flags |= ALIVE;
-		spigot_registerdev(devp);
 	}
 
 	return(status);
 }
 
-int
+static int
 spigot_attach(struct isa_device *devp)
 {
-struct	spigot_softc	*ss=(struct spigot_softc *)&spigot_softc[devp->id_unit];
-
-	kdc_spigot[devp->id_unit].kdc_state = DC_UNKNOWN;
+	int	unit;
+	struct	spigot_softc	*ss= &spigot_softc[unit = devp->id_unit];
 
 	ss->maddr = kvtop(devp->id_maddr);
 	ss->irq = devp->id_irq;
+#ifdef DEVFS
+	ss->devfs_token = 
+		devfs_add_devswf(&spigot_cdevsw, unit, DV_CHR, 0, 0, 0644,
+				 "spigot%d", unit);
+#endif
 
 	return 1;
 }
 
-int
+static	int
 spigot_open(dev_t dev, int flags, int fmt, struct proc *p)
 {
 struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
@@ -166,6 +158,12 @@ struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
 	if(ss->flags & OPEN)
 		return EBUSY;
 
+#if !defined(SPIGOT_UNSECURE)
+	/* Since we can't map the i/o page, don't allow open unless suser */
+	if(suser(p->p_ucred, &p->p_acflag) != 0)
+		return EPERM;
+#endif
+
 	ss->flags |= OPEN;
 	ss->p = 0;
 	ss->signal_num = 0;
@@ -173,7 +171,7 @@ struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
 	return 0;
 }
 
-int
+static	int
 spigot_close(dev_t dev, int flags, int fmt, struct proc *p)
 {
 struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
@@ -187,26 +185,23 @@ struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
 	return 0;
 }
 
-int
+static	int
 spigot_write(dev_t dev, struct uio *uio, int ioflag)
 {
-struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
-
 	return ENXIO;
 }
 
-int
+static	int
 spigot_read(dev_t dev, struct uio *uio, int ioflag)
 {
-struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
-
 	return ENXIO;
 }
 
 
-int
+static	int
 spigot_ioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
 {
+int			error;
 struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
 struct	trapframe	*fp;
 struct	spigot_info	*info;
@@ -218,6 +213,11 @@ struct	spigot_info	*info;
 		ss->signal_num = *((int *)data);
 		break;
 	case	SPIGOT_IOPL_ON:	/* allow access to the IO PAGE */
+#if !defined(SPIGOT_UNSECURE)
+		error = suser(p->p_ucred, &p->p_acflag);
+		if (error != 0)
+			return error;
+#endif
 		fp=(struct trapframe *)p->p_md.md_regs;
 		fp->tf_eflags |= PSL_IOPL;
 		break;
@@ -237,12 +237,9 @@ struct	spigot_info	*info;
 	return 0;
 }
 
-int
+static	int
 spigot_select(dev_t dev, int rw, struct proc *p)
 {
-struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[UNIT(dev)];
-int			s;
-int			r;
 
 	return ENXIO;
 }
@@ -260,8 +257,8 @@ struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[unit];
 		psignal(ss->p, ss->signal_num);
 }
 
-int
-spigot_mmap(dev_t dev, int offset, int nprot)
+static	int
+spigot_mmap(dev_t dev, vm_offset_t offset, int nprot)
 {
 struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[0];
 
@@ -276,4 +273,21 @@ struct	spigot_softc	*ss = (struct spigot_softc *)&spigot_softc[0];
 	return i386_btop(ss->maddr);
 }
 
+
+static spigot_devsw_installed = 0;
+
+static void 	spigot_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! spigot_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&spigot_cdevsw, NULL);
+		spigot_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(spigotdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,spigot_drvinit,NULL)
+
+
 #endif /* NSPIGOT */
