@@ -4,7 +4,7 @@
  * v1.4 by Eric S. Raymond (esr@snark.thyrsus.com) Aug 1993
  * modified for FreeBSD by Andrew A. Chernov <ache@astral.msk.su>
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/spkr.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/spkr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "speaker.h"
@@ -14,57 +14,63 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
-#include <sys/errno.h>
 #include <sys/buf.h>
-#include <sys/proc.h>
 #include <sys/uio.h>
+#include <sys/conf.h>
 #include <i386/isa/isa.h>
 #include <i386/isa/timerreg.h>
 #include <machine/clock.h>
 #include <machine/speaker.h>
 
+#ifdef	DEVFS
+#include <sys/devfsext.h>
+void	*devfs_token;
+#endif
+
+static	d_open_t	spkropen;
+static	d_close_t	spkrclose;
+static	d_write_t	spkrwrite;
+static	d_ioctl_t	spkrioctl;
+
+#define CDEV_MAJOR 26
+static struct cdevsw spkr_cdevsw = 
+	{ spkropen,     spkrclose,      noread,         spkrwrite,      /*26*/
+	  spkrioctl,    nostop,         nullreset,      nodevtotty,/* spkr */
+	  seltrue,	nommap,		NULL,	"spkr",	NULL,	-1 };
+
 /**************** MACHINE DEPENDENT PART STARTS HERE *************************
  *
  * This section defines a function tone() which causes a tone of given
- * frequency and duration from the 80x86's console speaker.
+ * frequency and duration from the ISA console speaker.
  * Another function endtone() is defined to force sound off, and there is
  * also a rest() entry point to do pauses.
  *
  * Audible sound is generated using the Programmable Interval Timer (PIT) and
- * Programmable Peripheral Interface (PPI) attached to the 80x86's speaker. The
+ * Programmable Peripheral Interface (PPI) attached to the ISA speaker. The
  * PPI controls whether sound is passed through at all; the PIT's channel 2 is
  * used to generate clicks (a square wave) of whatever frequency is desired.
  */
 
 /*
- * PIT and PPI port addresses and control values
- *
- * Most of the magic is hidden in the TIMER_PREP value, which selects PIT
- * channel 2, frequency LSB first, square-wave mode and binary encoding.
- * The encoding is as follows:
- *
- * +----------+----------+---------------+-----+
- * |  1    0  |  1    1  |  0    1    1  |  0  |
- * | SC1  SC0 | RW1  RW0 | M2   M1   M0  | BCD |
- * +----------+----------+---------------+-----+
- *   Counter     Write        Mode 3      Binary
- *  Channel 2  LSB first,  (Square Wave) Encoding
- *             MSB second
+ * PPI control values.
+ * XXX should be in a header and used in clock.c.
  */
 #define PPI_SPKR	0x03	/* turn these PPI bits on to pass sound */
-#define PIT_MODE	0xB6	/* set timer mode for sound generation */
-
-/*
- * Magic numbers for timer control.
- */
-#define TIMER_CLK	1193180L	/* corresponds to 18.2 MHz tick rate */
 
 #define SPKRPRI PSOCK
 static char endtone, endrest;
 
-static void tone(thz, ticks)
+static void tone __P((unsigned int thz, unsigned int ticks));
+static void rest __P((int ticks));
+static void playinit __P((void));
+static void playtone __P((int pitch, int value, int sustain));
+static int abs __P((int n));
+static void playstring __P((char *cp, size_t slen));
+
 /* emit tone of frequency thz for given number of ticks */
-unsigned int thz, ticks;
+static void
+tone(thz, ticks)
+	unsigned int thz, ticks;
 {
     unsigned int divisor;
     int sps;
@@ -72,22 +78,25 @@ unsigned int thz, ticks;
     if (thz <= 0)
 	return;
 
-    divisor = TIMER_CLK / thz;
+    divisor = timer_freq / thz;
 
 #ifdef DEBUG
     (void) printf("tone: thz=%d ticks=%d\n", thz, ticks);
 #endif /* DEBUG */
 
     /* set timer to generate clicks at given frequency in Hertz */
-    sps = spltty();
+    sps = splclock();
 
-    if (acquire_timer2(PIT_MODE)) {
+    if (acquire_timer2(TIMER_SEL2 | TIMER_SQWAVE | TIMER_16BIT)) {
 	/* enter list of waiting procs ??? */
+	splx(sps);
 	return;
     }
+    splx(sps);
+    disable_intr();
     outb(TIMER_CNTR2, (divisor & 0xff));	/* send lo byte */
     outb(TIMER_CNTR2, (divisor >> 8));	/* send hi byte */
-    splx(sps);
+    enable_intr();
 
     /* turn the speaker on */
     outb(IO_PPI, inb(IO_PPI) | PPI_SPKR);
@@ -100,12 +109,15 @@ unsigned int thz, ticks;
     if (ticks > 0)
 	tsleep((caddr_t)&endtone, SPKRPRI | PCATCH, "spkrtn", ticks);
     outb(IO_PPI, inb(IO_PPI) & ~PPI_SPKR);
+    sps = splclock();
     release_timer2();
+    splx(sps);
 }
 
-static void rest(ticks)
 /* rest for given number of ticks */
-int	ticks;
+static void
+rest(ticks)
+	int	ticks;
 {
     /*
      * Set timeout to endrest function, then give up the timeslice.
@@ -182,7 +194,8 @@ static int pitchtab[] =
 /* 6 */ 4186, 4435, 4698, 4978, 5274, 5588, 5920, 6272, 6644, 7040, 7459, 7902,
 };
 
-static void playinit()
+static void
+playinit()
 {
     octave = DFLT_OCTAVE;
     whole = (hz * SECS_PER_MIN * WHOLE_NOTE) / DFLT_TEMPO;
@@ -192,9 +205,10 @@ static void playinit()
     octprefix = TRUE;	/* act as though there was an initial O(n) */
 }
 
-static void playtone(pitch, value, sustain)
 /* play tone of proper duration for current rhythm signature */
-int	pitch, value, sustain;
+static void
+playtone(pitch, value, sustain)
+	int	pitch, value, sustain;
 {
     register int	sound, silence, snum = 1, sdenom = 1;
 
@@ -228,8 +242,9 @@ int	pitch, value, sustain;
     }
 }
 
-static int abs(n)
-int n;
+static int
+abs(n)
+	int n;
 {
     if (n < 0)
 	return(-n);
@@ -237,10 +252,11 @@ int n;
 	return(n);
 }
 
-static void playstring(cp, slen)
 /* interpret and play an item from a notation string */
-char	*cp;
-size_t	slen;
+static void
+playstring(cp, slen)
+	char	*cp;
+	size_t	slen;
 {
     int		pitch, oldfill, lastpitch = OCTAVE_NOTES * DFLT_OCTAVE;
 
@@ -437,11 +453,12 @@ size_t	slen;
 static int spkr_active = FALSE; /* exclusion flag */
 static struct buf *spkr_inbuf;  /* incoming buf */
 
-int spkropen(dev, flags, fmt, p)
-dev_t		dev;
-int		flags;
-int		fmt;
-struct proc	*p;
+int
+spkropen(dev, flags, fmt, p)
+	dev_t		dev;
+	int		flags;
+	int		fmt;
+	struct proc	*p;
 {
 #ifdef DEBUG
     (void) printf("spkropen: entering with dev = %x\n", dev);
@@ -463,10 +480,11 @@ struct proc	*p;
     }
 }
 
-int spkrwrite(dev, uio, ioflag)
-dev_t		dev;
-struct uio	*uio;
-int		ioflag;
+int
+spkrwrite(dev, uio, ioflag)
+	dev_t		dev;
+	struct uio	*uio;
+	int		ioflag;
 {
 #ifdef DEBUG
     printf("spkrwrite: entering with dev = %x, count = %d\n",
@@ -475,7 +493,7 @@ int		ioflag;
 
     if (minor(dev) != 0)
 	return(ENXIO);
-    else if (uio->uio_resid > DEV_BSIZE)     /* prevent system crashes */
+    else if (uio->uio_resid > (DEV_BSIZE - 1))     /* prevent system crashes */
 	return(E2BIG);
     else
     {
@@ -485,17 +503,21 @@ int		ioflag;
 
 	n = uio->uio_resid;
 	cp = spkr_inbuf->b_un.b_addr;
-	if (!(error = uiomove(cp, n, uio)))
+	error = uiomove(cp, n, uio);
+	if (!error) {
+		cp[n] = '\0';
 		playstring(cp, n);
+	}
 	return(error);
     }
 }
 
-int spkrclose(dev, flags, fmt, p)
-dev_t		dev;
-int		flags;
-int		fmt;
-struct proc	*p;
+int
+spkrclose(dev, flags, fmt, p)
+	dev_t		dev;
+	int		flags;
+	int		fmt;
+	struct proc	*p;
 {
 #ifdef DEBUG
     (void) printf("spkrclose: entering with dev = %x\n", dev);
@@ -513,12 +535,13 @@ struct proc	*p;
     }
 }
 
-int spkrioctl(dev, cmd, cmdarg, flags, p)
-dev_t		dev;
-int		cmd;
-caddr_t		cmdarg;
-int		flags;
-struct proc	*p;
+int
+spkrioctl(dev, cmd, cmdarg, flags, p)
+	dev_t		dev;
+	int		cmd;
+	caddr_t		cmdarg;
+	int		flags;
+	struct proc	*p;
 {
 #ifdef DEBUG
     (void) printf("spkrioctl: entering with dev = %x, cmd = %x\n");
@@ -558,5 +581,28 @@ struct proc	*p;
     return(EINVAL);
 }
 
+
+static spkr_devsw_installed = 0;
+
+static void
+spkr_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! spkr_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&spkr_cdevsw, NULL);
+		spkr_devsw_installed = 1;
+#ifdef DEVFS
+		devfs_token = devfs_add_devswf(&spkr_cdevsw, 0, DV_CHR,
+					       UID_ROOT, GID_WHEEL, 0600,
+					       "speaker");
+#endif
+    	}
+}
+
+SYSINIT(spkrdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,spkr_drvinit,NULL)
+
+
 #endif  /* NSPEAKER > 0 */
 /* spkr.c ends here */
