@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1995 Sren Schmidt
+ * Copyright (c) 1995-1997 Sren Schmidt
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,7 +12,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software withough specific prior written permission
+ *    derived from this software without specific prior written permission
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/syscons.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/syscons.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _I386_ISA_SYSCONS_H_
@@ -56,19 +56,33 @@
 #define LED_MASK	0x00007
 #define UNKNOWN_MODE	0x00010
 #define KBD_RAW_MODE	0x00020
-#define SWITCH_WAIT_REL	0x00040
-#define SWITCH_WAIT_ACQ	0x00080
-#define BUFFER_SAVED	0x00100
-#define CURSOR_ENABLED 	0x00200
-#define CURSOR_SHOWN 	0x00400
+#define KBD_CODE_MODE	0x00040
+#define SWITCH_WAIT_REL	0x00080
+#define SWITCH_WAIT_ACQ	0x00100
+#define BUFFER_SAVED	0x00200
+#define CURSOR_ENABLED 	0x00400
 #define MOUSE_ENABLED	0x00800
-#define UPDATE_MOUSE	0x01000
+#define MOUSE_MOVED	0x01000
+#define MOUSE_CUTTING	0x02000
+#define MOUSE_VISIBLE	0x04000
 
 /* configuration flags */
 #define VISUAL_BELL	0x00001
 #define BLINK_CURSOR	0x00002
 #define CHAR_CURSOR	0x00004
+#define DETECT_KBD	0x00008
 #define XT_KEYBD	0x00010
+#define KBD_NORESET	0x00020
+#define QUIET_BELL	0x00040
+
+/* attribute flags */
+#define NORMAL_ATTR             0x00
+#define BLINK_ATTR              0x01
+#define BOLD_ATTR               0x02
+#define UNDERLINE_ATTR          0x04
+#define REVERSE_ATTR            0x08
+#define FOREGROUND_CHANGED      0x10
+#define BACKGROUND_CHANGED      0x20
 
 /* video hardware memory addresses */
 #define VIDEOMEM	0x000A0000
@@ -86,10 +100,10 @@
 #define TIMER_FREQ	1193182			/* should be in isa.h */
 #define CONSOLE_BUFSIZE 1024
 #define PCBURST		128
-#define FONT_8		0x001
-#define FONT_14		0x002
-#define FONT_16		0x004
-#define HISTORY_SIZE	100*80
+#define FONT_NONE	1
+#define FONT_8		2
+#define FONT_14		4
+#define FONT_16		8
 
 /* defines related to hardware addresses */
 #define	MONO_BASE	0x3B4			/* crt controller base mono */
@@ -120,36 +134,49 @@ typedef struct term_stat {
 	int 		num_param;		/* # of parameters to ESC */
 	int	 	last_param;		/* last parameter # */
 	int 		param[MAX_ESC_PAR];	/* contains ESC parameters */
-	int 		cur_attr;		/* current attributes */
-	int 		std_attr;		/* normal attributes */
-	int 		rev_attr;		/* reverse attributes */
+	int             cur_attr;               /* current hardware attr word */
+	int             attr_mask;              /* current logical attr mask */
+	int             cur_color;              /* current hardware color */
+	int             std_color;              /* normal hardware color */
+	int             rev_color;              /* reverse hardware color */
 } term_stat;
 
 typedef struct scr_stat {
 	u_short 	*scr_buf;		/* buffer when off screen */
 	int 		xpos;			/* current X position */
 	int 		ypos;			/* current Y position */
-	int 		xsize;			/* X size */
-	int 		ysize;			/* Y size */
+	int             saved_xpos;             /* saved X position */
+	int             saved_ypos;             /* saved Y position */
+	int 		xsize;			/* X text size */
+	int 		ysize;			/* Y text size */
+	int 		xpixel;			/* X graphics size */
+	int 		ypixel;			/* Y graphics size */
+	int		font_size;		/* fontsize in Y direction */
 	int		start;			/* modified area start */
 	int		end;			/* modified area end */
 	term_stat 	term;			/* terminal emulation stuff */
 	int	 	status;			/* status (bitfield) */
 	u_short 	*cursor_pos;		/* cursor buffer position */
+	u_short 	*cursor_oldpos;		/* cursor old buffer position */
 	u_short		cursor_saveunder;	/* saved chars under cursor */
 	char		cursor_start;		/* cursor start line # */
 	char		cursor_end;		/* cursor end line # */
 	u_short		*mouse_pos;		/* mouse buffer position */
 	u_short		*mouse_oldpos;		/* mouse old buffer position */
-	u_short		mouse_saveunder[4];	/* saved chars under mouse */
 	short		mouse_xpos;		/* mouse x coordinate */
 	short		mouse_ypos;		/* mouse y coordinate */
+	short		mouse_buttons;		/* mouse buttons */
 	u_char		mouse_cursor[128];	/* mouse cursor bitmap store */
+	u_short		*mouse_cut_start;	/* mouse cut start pos */
+	u_short		*mouse_cut_end;		/* mouse cut end pos */
+	struct proc 	*mouse_proc;		/* proc* of controlling proc */
+	pid_t 		mouse_pid;		/* pid of controlling proc */
+	int		mouse_signal;		/* signal # to report with */
 	u_short		bell_duration;
 	u_short		bell_pitch;
 	u_char		border;			/* border color */
+	u_char	 	initial_mode;		/* initial mode */
 	u_char	 	mode;			/* mode */
-	u_char		font;			/* font on this screen */
 	pid_t 		pid;			/* pid of controlling proc */
 	struct proc 	*proc;			/* proc* of controlling proc */
 	struct vt_mode 	smode;			/* switch mode */
@@ -162,55 +189,16 @@ typedef struct scr_stat {
 } scr_stat;
 
 typedef struct default_attr {
-	int             std_attr;               /* normal attributes */
-	int 		rev_attr;		/* reverse attributes */
+	int             std_color;              /* normal hardware color */
+	int             rev_color;              /* reverse hardware color */
 } default_attr;
 
-/* function prototypes */
-int scprobe(struct isa_device *dev);
-int scattach(struct isa_device *dev);
-int scopen(dev_t dev, int flag, int mode, struct proc *p);
-int scclose(dev_t dev, int flag, int mode, struct proc *p);
-int scread(dev_t dev, struct uio *uio, int flag);
-int scwrite(dev_t dev, struct uio *uio, int flag);
-int scparam(struct tty *tp, struct termios *t);
-int scioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p);
-void scstart(struct tty *tp);
-void scintr(int unit);
-int pcmmap(dev_t dev, int offset, int nprot);
-static void scinit(void);
-static u_int scgetc(int noblock);
-       struct tty *scdevtotty(dev_t dev);
-static scr_stat *get_scr_stat(dev_t dev);
-static scr_stat *alloc_scp(void);
-static void init_scp(scr_stat *scp);
-static int get_scr_num(void);
-static void scrn_timer(void);
-static void clear_screen(scr_stat *scp);
-static int switch_scr(scr_stat *scp, u_int next_scr);
-static void exchange_scr(void);
-static inline void move_crsr(scr_stat *scp, int x, int y);
-static void scan_esc(scr_stat *scp, u_char c);
-static inline void draw_cursor(scr_stat *scp, int show);
-static void ansi_put(scr_stat *scp, u_char *buf, int len);
-static u_char *get_fstr(u_int c, u_int *len);
-static void update_leds(int which);
-static void history_to_screen(scr_stat *scp);
-static int history_up_line(scr_stat *scp);
-static int history_down_line(scr_stat *scp);
-static void kbd_wait(void);
-static void kbd_cmd(u_char command);
-static void set_mode(scr_stat *scp);
-       void set_border(int color);
-static void set_vgaregs(char *modetable);
-static void set_font_mode(void);
-static void set_normal_mode(void);
-static void copy_font(int operation, int font_type, char* font_image);
-static void set_destructive_cursor(scr_stat *scp, int force);
-static void draw_mouse_image(scr_stat *scp);
-static void save_palette(void);
-       void load_palette(void);
-static void do_bell(scr_stat *scp, int pitch, int duration);
-static void blink_screen(scr_stat *scp);
+/* misc prototypes used by different syscons related LKM's */
+void set_border(u_char color);
+void set_mode(scr_stat *scp);
+void copy_font(int operation, int font_type, char* font_image);
+void load_palette(char *palette);
+int add_scrn_saver(void (*this)(int));
+int remove_scrn_saver(void (*this)(int));
 
 #endif /* !_I386_ISA_SYSCONS_H_ */
