@@ -22,13 +22,13 @@
  * today: Fri Jun  2 17:21:03 EST 1994
  * added 24F support  ++sg
  *
- * $FreeBSD: stable/2.1/sys/i386/isa/ultra14f.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/ultra14f.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/types.h>
 
 #ifdef	KERNEL			/* don't laugh.. this compiles to a program too.. look */
-#include <uha.h>
+#include "uha.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -37,11 +37,13 @@
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
-#include <sys/devconf.h>
 
 #include <machine/clock.h>
 
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
+
 #include <i386/isa/isa_device.h>
 #endif /*KERNEL */
 #include <scsi/scsi_all.h>
@@ -90,7 +92,7 @@ struct uha_reg
 	int ogmptr;		/* outgoing mail ptr		*/
 	int icmcmd;		/* incoming mail command	*/
 	int icmptr;		/* incoming mail ptr		*/
-} *uhareg[NUHA];
+};
 
 struct uha_bits
 {
@@ -112,7 +114,7 @@ struct uha_bits
 	/* uha_sint (write) */
 	unsigned char abort_ack;
 	unsigned char icm_ack;
-} *uhabits[NUHA];
+};
 
 
 /*
@@ -248,36 +250,49 @@ struct mscp {
 	long int hashkey;
 };
 
-struct uha_data {
-	int     flags;
+static struct uha_data {
+	int	flags;
 #define UHA_INIT        0x01
 #define UHA_24F		0x02
-	int     baseport;
-	struct mscp *mscphash[MSCP_HASH_SIZE];
-	struct mscp *free_mscp;
-	int     our_id;		/* our scsi id */
-	int     vect;
-	int     dma;
-	int     nummscps;
-	struct scsi_link sc_link;
+	int	baseport;
+	struct	mscp *mscphash[MSCP_HASH_SIZE];
+	struct	mscp *free_mscp;
+	int	unit;
+	int	our_id;		/* our scsi id */
+	int	vect;
+	int	dma;
+	int	nummscps;
+	struct	scsi_link sc_link;
+	struct	uha_reg *ur;
+	struct	uha_bits *ub;
 } *uhadata[NUHA];
 
-int     uhaprobe();
-int     uha_attach();
-int     uhaintr();
-int32_t uha_scsi_cmd();
-timeout_t uha_timeout;
-void	uha_free_mscp();
-int     uha_abort();
-void    uhaminphys();
-void    uha_done();
-u_int32_t uha_adapter_info();
-struct mscp *uha_mscp_phys_kv();
-int	uha_init __P((int unit));
-int	uha24_init __P((int unit));
+static int	uha_abort __P((struct uha_data *uha, struct mscp *mscp));
+static u_int32_t	uha_adapter_info __P((int unit));
+static int	uha_attach __P((struct isa_device *dev));
+static void	uha_done __P((struct uha_data *uha, struct mscp *mscp));
+static void	uha_free_mscp __P((struct uha_data *uha, struct mscp *mscp,
+				   int flags));
+static struct mscp *
+		uha_get_mscp __P((struct uha_data *uha, int flags));
+static int	uha_init __P((struct uha_data *uha));
+static int	uha24_init __P((struct uha_data *uha));
+static void	uhaminphys __P((struct buf *bp));
+static struct mscp *
+		uha_mscp_phys_kv __P((struct uha_data *uha, long mscp_phys));
+static int	uha_poll __P((struct uha_data *uha, int wait));
+#ifdef UHADEBUG
+static void	uha_print_active_mscp __P((struct uha_data *uha));
+static void	uha_print_mscp __P((struct mscp *mscp));
+#endif
+static int	uhaprobe __P((struct isa_device *dev));
+static int32_t	uha_scsi_cmd __P((struct scsi_xfer *xs));
+static void	uha_send_mbox __P((struct uha_data *uha, struct mscp *mscp));
+static timeout_t
+		uha_timeout;
 
 static	struct mscp *cheat;
-unsigned long int scratch;
+static unsigned long int scratch;
 #define	EISA_MAX_SLOTS	16	/* XXX This should go into a comon header */
 static	uha_slot = 0;		/* slot last board was found in */
 static  uha_unit = 0;
@@ -297,7 +312,7 @@ struct isa_driver uhadriver =
 	"uha"
 };
 
-struct scsi_adapter uha_switch =
+static struct scsi_adapter uha_switch =
 {
 	uha_scsi_cmd,
 	uhaminphys,
@@ -309,7 +324,7 @@ struct scsi_adapter uha_switch =
 };
 
 /* the below structure is so we have a default dev struct for out link struct */
-struct scsi_device uha_dev =
+static struct scsi_device uha_dev =
 {
     NULL,			/* Use default error handler */
     NULL,			/* have a queue, served by this */
@@ -320,27 +335,6 @@ struct scsi_device uha_dev =
     { 0, 0 }
 };
 
-static struct kern_devconf kdc_uha[NUHA] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"uha", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"UltraStore 14F or 34F SCSI host adapter",
-	DC_CLS_MISC		/* host adapters aren't special */
-} };
-
-static inline void
-uha_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_uha[id->id_unit] = kdc_uha[0];
-	kdc_uha[id->id_unit].kdc_unit = id->id_unit;
-	kdc_uha[id->id_unit].kdc_parentdata = id;
-	dev_attach(&kdc_uha[id->id_unit]);
-}
-
 #endif /*KERNEL */
 
 #ifndef	KERNEL
@@ -354,14 +348,12 @@ main()
 /*
  * Function to send a command out through a mailbox
  */
-void
-uha_send_mbox(int unit, struct mscp *mscp)
+static void
+uha_send_mbox(struct uha_data *uha, struct mscp *mscp)
 {
-	struct uha_data *uha = uhadata[unit];
-	int     port = uha->baseport;
 	int     spincount = 100000;	/* 1s should be enough */
-	struct uha_reg *ur = uhareg[unit];
-	struct uha_bits *ub = uhabits[unit];
+	struct uha_reg *ur = uha->ur;
+	struct uha_bits *ub = uha->ub;
 	int     s = splbio();
 
 	while (--spincount) {
@@ -370,7 +362,8 @@ uha_send_mbox(int unit, struct mscp *mscp)
 		DELAY(100);
 	}
 	if (spincount == 0) {
-		printf("uha%d: uha_send_mbox, board is not responding\n", unit);
+		printf("uha%d: uha_send_mbox, board is not responding\n",
+			uha->unit);
 		Debugger("ultra14f");
 	}
 	outl(ur->ogmptr, KVTOPHYS(mscp));
@@ -383,14 +376,12 @@ uha_send_mbox(int unit, struct mscp *mscp)
  * Function to send abort to 14f
  */
 int
-uha_abort(int unit, struct mscp *mscp)
+uha_abort(struct uha_data *uha, struct mscp *mscp)
 {
-	struct uha_data *uha = uhadata[unit];
-	int     port = uha->baseport;
-	int     spincount = 100;	/* 1 mSec */
-	int     abortcount = 200000;	/*2 secs */
-	struct uha_reg *ur = uhareg[unit];
-	struct uha_bits *ub = uhabits[unit];
+	int	spincount = 100;	/* 1 mSec */
+	int	abortcount = 200000;	/*2 secs */
+	struct	uha_reg *ur = uha->ur;
+	struct	uha_bits *ub = uha->ub;
 	int     s = splbio();
 
 	while (--spincount) {
@@ -399,7 +390,7 @@ uha_abort(int unit, struct mscp *mscp)
 		DELAY(10);
 	}
 	if (spincount == 0) {
-		printf("uha%d: uha_abort, board is not responding\n", unit);
+		printf("uha%d: uha_abort, board is not responding\n", uha->unit);
 		Debugger("ultra14f");
 	}
 	outl(ur->ogmptr,KVTOPHYS(mscp));
@@ -412,7 +403,7 @@ uha_abort(int unit, struct mscp *mscp)
 		DELAY(10);
 	}
 	if (abortcount == 0) {
-		printf("uha%d: uha_abort, board is not responding\n", unit);
+		printf("uha%d: uha_abort, board is not responding\n", uha->unit);
 		Debugger("ultra14f");
 	}
 	if ((inb(ur->sint) & 0x10) != 0) {
@@ -431,26 +422,23 @@ uha_abort(int unit, struct mscp *mscp)
  *
  *	wait = timeout in msec
  */
-int
-uha_poll(int unit, int wait)
+static int
+uha_poll(struct uha_data *uha, int wait)
 {
-	struct	uha_data *uha = uhadata[unit];
-	int	port = uha->baseport;
-	struct uha_reg *ur = uhareg[unit];
-	struct uha_bits *ub = uhabits[unit];
+	struct	uha_reg *ur = uha->ur;
+	struct	uha_bits *ub = uha->ub;
 	int	stport = ur->sint;
 
-      retry:
 	while (--wait) {
 		if (inb(stport) & ub->sintp)
 			break;
 		DELAY(1000);	/* 1 mSec per loop */
 	}
 	if (wait == 0) {
-		printf("uha%d: uha_poll, board is not responding\n", unit);
+		printf("uha%d: uha_poll, board is not responding\n", uha->unit);
 		return (EIO);
 	}
-	uhaintr(unit);
+	uhaintr(uha->unit);
 	return (0);
 }
 
@@ -465,8 +453,6 @@ uhaprobe(dev)
 {
 	int     unit = uha_unit;
 	struct uha_data *uha;
-	struct uha_reg *ur;
-	struct uha_bits *ub;
 
 	dev->id_unit = unit;	/* XXX */
 
@@ -493,35 +479,32 @@ uhaprobe(dev)
 	}
 	bzero(uha, sizeof(struct uha_data));
 
-	ur = malloc(sizeof(struct uha_reg), M_TEMP, M_NOWAIT);
-	if (!ur) {
+	uha->ur = malloc(sizeof(struct uha_reg), M_TEMP, M_NOWAIT);
+	if (!uha->ur) {
 		printf("uha%d: cannot malloc!\n", unit);
 		return 0;
 	}
-	bzero(ur, sizeof(struct uha_reg));
+	bzero(uha->ur, sizeof(struct uha_reg));
 
-	ub = malloc(sizeof(struct uha_bits), M_TEMP, M_NOWAIT);
-	if (!ub) {
+	uha->ub = malloc(sizeof(struct uha_bits), M_TEMP, M_NOWAIT);
+	if (!uha->ub) {
 		printf("uha%d: cannot malloc!\n", unit);
 		return 0;
 	}
-	bzero(ub, sizeof(struct uha_bits));
+	bzero(uha->ub, sizeof(struct uha_bits));
 
-	uha_registerdev(dev);
-
-	uhareg[unit] = ur;
-	uhabits[unit] = ub;
 	uhadata[unit] = uha;
+	uha->unit = unit;
 	uha->baseport = dev->id_iobase;
 	/*
 	 * Try initialise a unit at this location
 	 * sets up dma and bus speed, loads uha->vect
 	 */
-	if (uha_init(unit) != 0 && uha24_init(unit) != 0) {
+	if (uha_init(uha) != 0 && uha24_init(uha) != 0) {
 		uhadata[unit] = NULL;
+		free(uha->ur, M_TEMP);
+		free(uha->ub, M_TEMP);
 		free(uha, M_TEMP);
-		free(ur, M_TEMP);
-		free(ub, M_TEMP);
 		return (0);
 	}
 	/* if it's there put in its interrupt and DRQ vectors */
@@ -549,6 +532,7 @@ uha_attach(dev)
 	 */
 	uha->sc_link.adapter_unit = unit;
 	uha->sc_link.adapter_targ = uha->our_id;
+	uha->sc_link.adapter_softc = uha;
 	uha->sc_link.adapter = &uha_switch;
 	uha->sc_link.device = &uha_dev;
 	uha->sc_link.flags = SDEV_BOUNCE;
@@ -562,7 +546,6 @@ uha_attach(dev)
                 return 0;
 	scbus->adapter_link = &uha->sc_link;
 
-	kdc_uha[unit].kdc_state = DC_BUSY;
 	/*
 	 * ask the adapter what subunits are present
 	 */
@@ -585,7 +568,7 @@ uha_adapter_info(unit)
 /*
  * Catch an interrupt from the adaptor
  */
-int
+void
 uhaintr(unit)
 	int unit;
 {
@@ -593,10 +576,13 @@ uhaintr(unit)
 	struct mscp *mscp;
 	u_char  uhastat;
 	unsigned long int mboxval;
-	struct uha_reg *ur = uhareg[unit];
-	struct uha_bits *ub = uhabits[unit];
+	struct uha_reg *ur;
+	struct uha_bits *ub;
+	int     port;
 
-	int     port = uha->baseport;
+	ur = uha->ur;
+	ub = uha->ub;
+	port = uha->baseport;
 
 #ifdef	UHADEBUG
 	printf("uhaintr ");
@@ -621,13 +607,12 @@ uhaintr(unit)
 		mscp = uha_mscp_phys_kv(uha, mboxval);
 		if (!mscp) {
 			printf("uha: BAD MSCP RETURNED\n");
-			return (0);	/* whatever it was, it'll timeout */
+			return;		/* whatever it was, it'll timeout */
 		}
 		untimeout(uha_timeout, (caddr_t)mscp);
 
-		uha_done(unit, mscp);
+		uha_done(uha, mscp);
 	}
-	return (1);
 }
 
 /*
@@ -635,11 +620,10 @@ uhaintr(unit)
  * how the operation went.
  */
 void
-uha_done(unit, mscp)
-	int	unit;
+uha_done(uha, mscp)
+	struct	uha_data *uha;
 	struct	mscp *mscp;
 {
-	struct uha_data *uha = uhadata[unit];
 	struct scsi_sense_data *s1, *s2;
 	struct scsi_xfer *xs = mscp->xs;
 
@@ -671,7 +655,7 @@ uha_done(unit, mscp)
 			default:	/* Other scsi protocol messes */
 				xs->error = XS_DRIVER_STUFFUP;
 				printf("uha%d: unexpected ha_status 0x%x (target status 0x%x)\n",
-					unit, mscp->ha_status,
+					uha->unit, mscp->ha_status,
 					mscp->targ_status);
 				break;
 			}
@@ -689,7 +673,7 @@ uha_done(unit, mscp)
 				break;
 			default:
 				printf("uha%d: unexpected targ_status 0x%x\n",
-					unit, mscp->targ_status);
+					uha->unit, mscp->targ_status);
 				xs->error = XS_DRIVER_STUFFUP;
 				break;
 			}
@@ -700,9 +684,8 @@ uha_done(unit, mscp)
 		xs->resid = 0;
 		xs->error = 0;
 	}
-    done:
 	xs->flags |= ITSDONE;
-	uha_free_mscp(unit, mscp, xs->flags);
+	uha_free_mscp(uha, mscp, xs->flags);
 	scsi_done(xs);
 }
 
@@ -710,12 +693,11 @@ uha_done(unit, mscp)
  * A mscp (and hence a mbx-out) is put onto the free list.
  */
 void
-uha_free_mscp(unit, mscp, flags)
-	int unit;
+uha_free_mscp(uha, mscp, flags)
+	struct uha_data *uha;
 	struct mscp *mscp;
 	int flags;
 {
-	struct uha_data *uha = uhadata[unit];
 	unsigned int opri = 0;
 
 	if (!(flags & SCSI_NOMASK))
@@ -741,11 +723,11 @@ uha_free_mscp(unit, mscp, flags)
  * If there are none, see if we can allocate a new one.  If so, put it in the
  * hash table too otherwise either return an error or sleep.
  */
-struct mscp *
-uha_get_mscp(unit, flags)
-	int	unit, flags;
+static struct mscp *
+uha_get_mscp(uha, flags)
+	struct	uha_data *uha;
+	int	flags;
 {
-	struct uha_data *uha = uhadata[unit];
 	unsigned opri = 0;
 	struct mscp *mscpp;
 	int     hashnum;
@@ -773,7 +755,7 @@ uha_get_mscp(unit, flags)
 				mscpp->nexthash = uha->mscphash[hashnum];
 				uha->mscphash[hashnum] = mscpp;
 			} else {
-				printf("uha%d: Can't malloc MSCP\n", unit);
+				printf("uha%d: Can't malloc MSCP\n", uha->unit);
 			}
 			goto gottit;
 		} else {
@@ -798,7 +780,7 @@ uha_get_mscp(unit, flags)
 /*
  * given a physical address, find the mscp that it corresponds to.
  */
-struct mscp *
+static struct mscp *
 uha_mscp_phys_kv(uha, mscp_phys)
 	struct	uha_data *uha;
 	long	int mscp_phys;
@@ -818,11 +800,9 @@ uha_mscp_phys_kv(uha, mscp_phys)
  * Start the board, ready for normal operation
  */
 int
-uha_init(unit)
-	int     unit;
+uha_init(uha)
+	struct	uha_data *uha;
 {
-	struct uha_data *uha = uhadata[unit];
-	unsigned char ad[4];
 	volatile unsigned char model;
 	volatile unsigned char submodel;
 	unsigned char config_reg1;
@@ -831,10 +811,9 @@ uha_init(unit)
 	unsigned char irq_ch;
 	unsigned char uha_id;
 	int     port = uha->baseport;
-	int     i;
 	int     resetcount = 4000;	/* 4 secs? */
-	struct uha_reg *ur = uhareg[unit];
-	struct uha_bits *ub = uhabits[unit];
+	struct uha_reg *ur = uha->ur;
+	struct uha_bits *ub = uha->ub;
 
 	/*
 	 *  Prepare to use a 14/34F.
@@ -867,7 +846,7 @@ uha_init(unit)
 	model = inb(ur->id);
 	submodel = inb(ur->id + 1);
 	if ((model != 0x56) & (submodel != 0x40)) return(ENXIO);
-	printf("uha%d: reading board settings, ", unit);
+	printf("uha%d: reading board settings, ", uha->unit);
 
 	config_reg1 = inb(ur->config);
 	config_reg2 = inb(ur->config + 1);
@@ -928,7 +907,7 @@ uha_init(unit)
 		DELAY(1000);	/* 1 mSec per loop */
 	}
 	if (resetcount == 0) {
-		printf("uha%d: board timed out during reset\n", unit);
+		printf("uha%d: board timed out during reset\n", uha->unit);
 		return (ENXIO);
 	}
 	outb(ur->smask, 0x81);		/* make sure interrupts are enabled */
@@ -941,16 +920,15 @@ uha_init(unit)
  *  Initialize an Ultrastor 24F
  */
 int
-uha24_init(unit)
-int	unit;
+uha24_init(uha)
+	struct uha_data *uha;
 {
   unsigned char p0, p1, p2, p3, p5, p7;
   unsigned char id[7], rev, haid;
-  int port = 0, irq, i;
+  int port = 0, irq;
   int resetcount = 4000;
-  struct uha_data *uha = uhadata[unit];
-  struct uha_reg *ur = uhareg[unit];
-  struct uha_bits *ub = uhabits[unit];
+  struct uha_reg *ur = uha->ur;
+  struct uha_bits *ub = uha->ub;
 
   /* Search for the 24F's product ID */
   uha_slot++;
@@ -1004,9 +982,9 @@ int	unit;
 	id[0] = 0x40 + ((p0 >> 2) & 0x1f);
 	id[1] = 0x40 + (((p0 & 0x03) << 3) | ((p1 >> 5) & 0x07));
 	id[2] = 0x40 + (p1 & 0x1f);
-	id[3] = "0123456789abcdef"[(p2 >> 4) & 0x0f];
-	id[4] = "0123456789abcdef"[p2 & 0x0f];
-	id[5] = "0123456789abcdef"[(p3 >> 4) & 0x0f];
+	id[3] = hex2ascii((p2 >> 4) & 0x0f);
+	id[4] = hex2ascii(p2 & 0x0f);
+	id[5] = hex2ascii((p3 >> 4) & 0x0f);
 	id[6] = '\0';
 	rev = p3 & 0xf;
 
@@ -1026,12 +1004,12 @@ int	unit;
      case 0x40: irq = 11; break;
      case 0x80: irq = 10; break;
      default:
-	printf("uha%d: bad 24F irq\n", unit);
+	printf("uha%d: bad 24F irq\n", uha->unit);
 	return(ENXIO);
   }
 
   haid = (p7 & 0x07);
-  printf("uha%d: UltraStor 24F int=%d id=%d\n", unit, irq, haid);
+  printf("uha%d: UltraStor 24F int=%d id=%d\n", uha->unit, irq, haid);
 
   /* Issue SCSI and adapter reset */
   outb(ur->lint, ub->asrst);
@@ -1041,7 +1019,7 @@ int	unit;
 	DELAY(1000);	/* 1 mSec per loop */
   }
   if (resetcount == 0) {
-	printf("uha%d: board timed out during reset\n", unit);
+	printf("uha%d: board timed out during reset\n", uha->unit);
 	return (ENXIO);
   }
   outb(ur->smask, 0xc2);	/* make sure interrupts are enabled */
@@ -1070,27 +1048,21 @@ uhaminphys(bp)
  * start a scsi operation given the command and the data address.  Also
  * needs the unit, target and lu.
  */
-int32_t
+static int32_t
 uha_scsi_cmd(xs)
 	struct scsi_xfer *xs;
 {
-	struct	scsi_sense_data *s1, *s2;
 	struct	mscp *mscp;
 	struct	uha_dma_seg *sg;
 	int	seg;		/* scatter gather seg being worked on */
-	int	i = 0;
-	int     rc = 0;
 	int     thiskv;
 	unsigned long int thisphys, nextphys;
-	int     unit = xs->sc_link->adapter_unit;
 	int     bytes_this_seg, bytes_this_page, datalen, flags;
-	struct iovec *iovp;
-	struct uha_data *uha = uhadata[unit];
+	struct	uha_data *uha;
 	int     s;
-	unsigned int stat;
-	int     port = uha->baseport;
 	unsigned long int templen;
 
+	uha = (struct uha_data *)xs->sc_link->adapter_softc;
 	SC_DEBUG(xs->sc_link, SDEV_DB2, ("uha_scsi_cmd\n"));
 	/*
 	 * get a mscp (mbox-out) to use. If the transfer
@@ -1101,14 +1073,14 @@ uha_scsi_cmd(xs)
 	if (xs->bp)
 		flags |= (SCSI_NOSLEEP);	/* just to be sure */
 	if (flags & ITSDONE) {
-		printf("uha%d: Already done?", unit);
+		printf("uha%d: Already done?", uha->unit);
 		xs->flags &= ~ITSDONE;
 	}
 	if (!(flags & INUSE)) {
-		printf("uha%d: Not in use?", unit);
+		printf("uha%d: Not in use?", uha->unit);
 		xs->flags |= INUSE;
 	}
-	if (!(mscp = uha_get_mscp(unit, flags))) {
+	if (!(mscp = uha_get_mscp(uha, flags))) {
 		xs->error = XS_DRIVER_STUFFUP;
 		return (TRY_AGAIN_LATER);
 	}
@@ -1135,7 +1107,7 @@ uha_scsi_cmd(xs)
 #ifdef	GOTTABEJOKING
 	if (xs->sc_link->lun != 0) {
 		xs->error = XS_DRIVER_STUFFUP;
-		uha_free_mscp(unit, mscp, flags);
+		uha_free_mscp(uha, mscp, flags);
 		return (HAD_ERROR);
 	}
 #endif
@@ -1263,9 +1235,9 @@ uha_scsi_cmd(xs)
 		SC_DEBUGN(xs->sc_link, SDEV_DB4, ("\n"));
 		if (datalen) {	/* there's still data, must have run out of segs! */
 			printf("uha%d: uha_scsi_cmd, more than %d DMA segs\n",
-			    unit, UHA_NSEG);
+			    uha->unit, UHA_NSEG);
 			xs->error = XS_DRIVER_STUFFUP;
-			uha_free_mscp(unit, mscp, flags);
+			uha_free_mscp(uha, mscp, flags);
 			return (HAD_ERROR);
 		}
 	} else {		/* No data xfer, use non S/G values */
@@ -1292,7 +1264,7 @@ uha_scsi_cmd(xs)
 	 */
 	if (!(flags & SCSI_NOMASK)) {
 		s = splbio();
-		uha_send_mbox(unit, mscp);
+		uha_send_mbox(uha, mscp);
 		timeout(uha_timeout, (caddr_t)mscp, (xs->timeout * hz) / 1000);
 		splx(s);
 		SC_DEBUG(xs->sc_link, SDEV_DB3, ("cmd_sent\n"));
@@ -1302,15 +1274,16 @@ uha_scsi_cmd(xs)
 	/*
 	 * If we can't use interrupts, poll on completion
 	 */
-	uha_send_mbox(unit, mscp);
+	uha_send_mbox(uha, mscp);
 	SC_DEBUG(xs->sc_link, SDEV_DB3, ("cmd_wait\n"));
 	do {
-		if (uha_poll(unit, xs->timeout)) {
+		if (uha_poll(uha, xs->timeout)) {
 			if (!(xs->flags & SCSI_SILENT))
-				printf("uha%d: cmd fail\n", unit);
-			if (!(uha_abort(unit, mscp))) {
-				printf("uha%d: abort failed in wait\n", unit);
-				uha_free_mscp(unit, mscp, flags);
+				printf("uha%d: cmd fail\n", uha->unit);
+			if (!(uha_abort(uha, mscp))) {
+				printf("uha%d: abort failed in wait\n",
+					uha->unit);
+				uha_free_mscp(uha, mscp, flags);
 			}
 			xs->error = XS_DRIVER_STUFFUP;
 			return (HAD_ERROR);
@@ -1323,32 +1296,29 @@ uha_scsi_cmd(xs)
 	return (COMPLETE);
 }
 
-void
+static void
 uha_timeout(arg1)
 	void	*arg1;
 {
 	struct mscp *mscp = (struct mscp *)arg1;
-	int     unit;
 	struct uha_data *uha;
 	int     s = splbio();
-	/*int   port = uha->baseport; */
 
-	unit = mscp->xs->sc_link->adapter_unit;
-	uha = uhadata[unit];
-	printf("uha%d:%d:%d (%s%d) timed out ", unit
+	uha = (struct uha_data *)mscp->xs->sc_link->adapter_softc;
+	printf("uha%d:%d:%d (%s%d) timed out ", uha->unit
 	    ,mscp->xs->sc_link->target
 	    ,mscp->xs->sc_link->lun
 	    ,mscp->xs->sc_link->device->name
 	    ,mscp->xs->sc_link->dev_unit);
 
 #ifdef	UHADEBUG
-	uha_print_active_mscp(unit);
+	uha_print_active_mscp(uha);
 #endif /*UHADEBUG */
 
-	if ((uha_abort(unit, mscp) != 1) || (mscp->flags = MSCP_ABORTED)) {
+	if ((uha_abort(uha, mscp) != 1) || (mscp->flags = MSCP_ABORTED)) {
 		printf("AGAIN");
 		mscp->xs->retries = 0;	/* I MEAN IT ! */
-		uha_done(unit, mscp);
+		uha_done(uha, mscp);
 	} else {		/* abort the operation that has timed out */
 		printf("\n");
 		timeout(uha_timeout, (caddr_t)mscp, 2 * hz);
@@ -1378,9 +1348,8 @@ uha_print_mscp(mscp)
 }
 
 void
-uha_print_active_mscp(int unit)
+uha_print_active_mscp(struct uha_data *uha)
 {
-	struct uha_data *uha = uhadata[unit];
 	struct mscp *mscp;
 	int     i = 0;
 
