@@ -12,11 +12,13 @@
  * or modify this software as long as this message is kept with the software,
  * all derivative works or modified versions.
  *
- * Version 1.8, Thu Sep 28 21:04:16 MSK 1995
+ * Version 1.9, Mon Oct  9 20:27:42 MSK 1995
  */
 
 #include "wdc.h"
 #include "wcd.h"
+#include "opt_atapi.h"
+
 #if NWCD > 0 && NWDC > 0 && defined (ATAPI)
 
 #include <sys/param.h>
@@ -26,22 +28,49 @@
 #include <sys/malloc.h>
 #include <sys/buf.h>
 #include <sys/ioctl.h>
-#include <sys/devconf.h>
 #include <sys/disklabel.h>
 #include <sys/cdio.h>
-#include <i386/include/cpufunc.h>
+#include <sys/conf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
+
+#include <machine/cpufunc.h>
+
 #include <i386/isa/atapi.h>
 
+static	d_open_t	wcdropen;
+static	d_open_t	wcdbopen;
+static	d_close_t	wcdrclose;
+static	d_close_t	wcdbclose;
+static	d_ioctl_t	wcdioctl;
+static	d_strategy_t	wcdstrategy;
+
+#define CDEV_MAJOR 69
+#define BDEV_MAJOR 19
+extern	struct cdevsw wcd_cdevsw;
+static struct bdevsw wcd_bdevsw = 
+	{ wcdbopen,	wcdbclose,	wcdstrategy,	wcdioctl,	/*19*/
+	  nodump,	nopsize,	0,	"wcd",	&wcd_cdevsw,	-1 };
+
+static struct cdevsw wcd_cdevsw = 
+	{ wcdropen,	wcdrclose,	rawread,	nowrite,	/*69*/
+	  wcdioctl,	nostop,		nullreset,	nodevtotty,/* atapi */
+	  seltrue,	nommap,		wcdstrategy,	"wcd",
+	  &wcd_bdevsw,	-1 };
+
+#ifndef ATAPI_STATIC
+static
+#endif
+int  wcdattach(struct atapi*, int, struct atapi_params*, int);
+
 #define NUNIT   (NWDC*2)                /* Max. number of devices */
 #define UNIT(d) ((minor(d) >> 3) & 3)   /* Unit part of minor device number */
 #define SECSIZE 2048                    /* CD-ROM sector size in bytes */
 
-#define F_OPEN          0x0001          /* The drive os opened */
+#define F_BOPEN         0x0001          /* The block device is opened */
 #define F_MEDIA_CHANGED 0x0002          /* The media have changed since open */
-#define F_DEBUG         0x0004          /* The media have changed since open */
-#define F_NOPLAYCD      0x0008          /* The PLAY_CD op not supported */
-
-#define WCD_LASTPLUS1	170		/* Don't ask, xcdplayer uses this */
+#define F_DEBUG         0x0004          /* Print debug info */
 
 /*
  * Disc table of contents.
@@ -186,7 +215,8 @@ struct wcd {
 	int unit;                       /* IDE bus drive unit */
 	int lun;                        /* Logical device unit */
 	int flags;                      /* Device state flags */
-	struct buf queue;               /* Queue of i/o requests */
+	int refcnt;                     /* The number of raw opens */
+	struct buf_queue_head buf_queue;               /* Queue of i/o requests */
 	struct atapi_params *param;     /* Drive parameters table */
 	struct toc toc;                 /* Table of disc contents */
 	struct volinfo info;            /* Volume size info */
@@ -194,8 +224,14 @@ struct wcd {
 	struct cappage cap;             /* Capabilities page info */
 	struct audiopage aumask;        /* Audio page mask */
 	struct subchan subchan;         /* Subchannel info */
-	struct kern_devconf cf;         /* Driver configuration info */
 	char description[80];           /* Device description */
+	int starting_lba ;
+#ifdef	DEVFS
+	void	*ra_devfs_token;
+	void	*rc_devfs_token;
+	void	*a_devfs_token;
+	void	*c_devfs_token;
+#endif
 };
 
 struct wcd *wcdtab[NUNIT];      /* Drive info by unit number */
@@ -209,17 +245,11 @@ static int wcd_read_toc (struct wcd *t);
 static int wcd_request_wait (struct wcd *t, u_char cmd, u_char a1, u_char a2,
 	u_char a3, u_char a4, u_char a5, u_char a6, u_char a7, u_char a8,
 	u_char a9, char *addr, int count);
-static int wcd_externalize (struct proc*, struct kern_devconf*, void*, size_t);
-static int wcd_goaway (struct kern_devconf *kdc, int force);
 static void wcd_describe (struct wcd *t);
+static int wcd_open(dev_t dev, int rawflag);
 static int wcd_setchan (struct wcd *t,
 	u_char c0, u_char c1, u_char c2, u_char c3);
-
-static struct kern_devconf cftemplate = {
-	0, 0, 0, "wcd", 0, { MDDT_DISK, 0 },
-	wcd_externalize, 0, wcd_goaway, DISK_EXTERNALLEN,
-	0, 0, DC_IDLE, "ATAPI compact disc",
-};
+static int wcd_eject (struct wcd *t, int closeit);
 
 /*
  * Dump the array in hexadecimal format for debugging purposes.
@@ -234,40 +264,40 @@ static void wcd_dump (int lun, char *label, void *data, int len)
 	printf ("\n");
 }
 
-static int wcd_externalize (struct proc *p, struct kern_devconf *kdc,
-	void *userp, size_t len)
-{
-	return disk_externalize (wcdtab[kdc->kdc_unit]->unit, userp, &len);
-}
-
-static int wcd_goaway (struct kern_devconf *kdc, int force)
-{
-	dev_detach (kdc);
-	return 0;
-}
-
-void wcdattach (struct atapi *ata, int unit, struct atapi_params *ap, int debug,
-	struct kern_devconf *parent)
+#ifndef ATAPI_STATIC
+static
+#endif
+int 
+wcdattach (struct atapi *ata, int unit, struct atapi_params *ap, int debug)
 {
 	struct wcd *t;
 	struct atapires result;
+	int lun;
 
 	if (wcdnlun >= NUNIT) {
 		printf ("wcd: too many units\n");
-		return;
+		return (0);
+	}
+	if (!atapi_request_immediate) {
+		printf("wcd: configuration error, ATAPI core code not present!\n");
+		printf("wcd: check `options ATAPI_STATIC' in your kernel config file!\n");
+		return (0);
 	}
 	t = malloc (sizeof (struct wcd), M_TEMP, M_NOWAIT);
 	if (! t) {
 		printf ("wcd: out of memory\n");
-		return;
+		return (0);
 	}
 	wcdtab[wcdnlun] = t;
 	bzero (t, sizeof (struct wcd));
+	bufq_init(&t->buf_queue);
 	t->ata = ata;
 	t->unit = unit;
-	t->lun = wcdnlun++;
+	lun = t->lun = wcdnlun++;
 	t->param = ap;
 	t->flags = F_MEDIA_CHANGED;
+	t->starting_lba = 0;
+	t->refcnt = 0;
 	if (debug) {
 		t->flags |= F_DEBUG;
 		/* Print params. */
@@ -297,16 +327,26 @@ void wcdattach (struct atapi *ata, int unit, struct atapi_params *ap, int debug,
 			wcd_dump (t->lun, "cap", &t->cap, sizeof t->cap);
 	}
 
-	/* Register driver */
-	t->cf = cftemplate;
-	t->cf.kdc_unit = t->lun;
-	t->cf.kdc_parent = parent;
-	t->cf.kdc_description = t->description;
-	strcpy (t->description, cftemplate.kdc_description);
-	strcat (t->description, ": ");
-	strncpy (t->description + strlen(t->description),
-		ap->model, sizeof(ap->model));
-	dev_attach (&t->cf);
+
+#ifdef DEVFS
+	t->ra_devfs_token = 
+		devfs_add_devswf(&wcd_cdevsw, dkmakeminor(lun, 0, 0),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rwcd%da", lun);
+	t->rc_devfs_token = 
+		devfs_add_devswf(&wcd_cdevsw, dkmakeminor(lun, 0, RAW_PART),
+				 DV_CHR, UID_ROOT, GID_OPERATOR, 0640,
+				 "rwcd%dc", lun);
+	t->a_devfs_token = 
+		devfs_add_devswf(&wcd_bdevsw, dkmakeminor(lun, 0, 0),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "wcd%da", lun);
+	t->c_devfs_token = 
+		devfs_add_devswf(&wcd_bdevsw, dkmakeminor(lun, 0, RAW_PART),
+				 DV_BLK, UID_ROOT, GID_OPERATOR, 0640,
+				 "wcd%dc", lun);
+#endif
+	return (1);
 }
 
 void wcd_describe (struct wcd *t)
@@ -321,9 +361,9 @@ void wcd_describe (struct wcd *t)
 	printf ("wcd%d: ", t->lun);
 	if (t->cap.cur_speed != t->cap.max_speed)
 		printf ("%d/", t->cap.cur_speed * 1000 / 1024);
-	printf ("%dKb/sec", t->cap.max_speed * 1000 / 1024);
+	printf ("%dKB/sec", t->cap.max_speed * 1000 / 1024);
 	if (t->cap.buf_size)
-		printf (", %dKb cache", t->cap.buf_size);
+		printf (", %dKB cache", t->cap.buf_size);
 
 	if (t->cap.audio_play)
 		printf (", audio play");
@@ -367,89 +407,78 @@ void wcd_describe (struct wcd *t)
 	printf ("\n");
 }
 
-int wcdopen (dev_t dev, int flags, int fmt, struct proc *p)
+static int 
+wcd_open (dev_t dev, int rawflag)
 {
 	int lun = UNIT(dev);
+	int track = dkslice(dev); /* XXX */
 	struct wcd *t;
-	struct atapires result;
 
-	/* Check the unit is legal. */
-	if (lun >= wcdnlun)
+	/* Check that the device number is legal
+	 * and the ATAPI driver is loaded. */
+	if (lun >= wcdnlun || ! atapi_request_immediate)
 		return (ENXIO);
 	t = wcdtab[lun];
 
-	/* If already opened, that's all. */
-	if (t->flags & F_OPEN) {
-		/* If it's been invalidated, forbid re-entry.
-		 * (may have changed media) */
-		if (t->flags & F_MEDIA_CHANGED)
-			return (ENXIO);
-		return (0);
+	/* On the first open, read the table of contents. */
+	if (! (t->flags & F_BOPEN) && ! t->refcnt) {
+		/* Read table of contents. */
+		if (wcd_read_toc (t) < 0)
+			return (EIO);
+
+		/* Lock the media. */
+		wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+			0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0);
 	}
-
-	/* On the first open: check for the media.
-	 * Do it twice to avoid the stale media changed state. */
-	result = atapi_request_wait (t->ata, t->unit, ATAPI_TEST_UNIT_READY,
-		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-
-	if (result.code == RES_ERR &&
-	    (result.error & AER_SKEY) == AER_SK_UNIT_ATTENTION) {
-		t->flags |= F_MEDIA_CHANGED;
-		result = atapi_request_wait (t->ata, t->unit,
-			ATAPI_TEST_UNIT_READY, 0, 0, 0, 0, 0, 0, 0, 0,
-			0, 0, 0, 0, 0, 0, 0, 0, 0);
+	if (rawflag)
+		++t->refcnt;
+	else
+		t->flags |= F_BOPEN;
+	t->starting_lba = ntohl(t->toc.tab[track].addr.lba) ;
+	if (track != 0) { 
+		printf("Warning, opening track %d at %d\n",
+			track, t->starting_lba);
 	}
-	if (result.code) {
-		wcd_error (t, result);
-		return (ENXIO);
-	}
-
-	/* Read table of contents. */
-	if (wcd_read_toc (t) != 0)
-		bzero (&t->toc, sizeof (t->toc));
-
-	/* Read disc capacity. */
-	if (wcd_request_wait (t, ATAPI_READ_CAPACITY, 0, 0, 0, 0, 0, 0,
-	    0, sizeof(t->info), 0, (char*)&t->info, sizeof(t->info)) != 0)
-		bzero (&t->info, sizeof (t->info));
-	t->info.volsize = ntohl (t->info.volsize);
-	t->info.blksize = ntohl (t->info.blksize);
-
-	/* Print the disc description string on every disc change.
-	 * It would help to track the history of disc changes. */
-	if (t->info.volsize && t->toc.hdr.ending_track &&
-	    (t->flags & F_MEDIA_CHANGED) && (t->flags & F_DEBUG)) {
-		printf ("wcd%d: ", t->lun);
-		if (t->toc.tab[0].control & 4)
-			printf ("%ldMB ", t->info.volsize / 512);
-		else
-			printf ("%ld:%ld audio ", t->info.volsize/75/60,
-				t->info.volsize/75%60);
-		printf ("(%ld sectors), %d tracks\n", t->info.volsize,
-			t->toc.hdr.ending_track - t->toc.hdr.starting_track + 1);
-	}
-	/* Lock the media. */
-	wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
-		0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0);
-
-	t->flags &= ~F_MEDIA_CHANGED;
-	t->flags |= F_OPEN;
 	return (0);
 }
 
+int wcdbopen (dev_t dev, int flags, int fmt, struct proc *p)
+{
+	return wcd_open (dev, 0);
+}
+
+int wcdropen (dev_t dev, int flags, int fmt, struct proc *p)
+{
+	return wcd_open (dev, 1);
+}
+
 /*
  * Close the device.  Only called if we are the LAST
  * occurence of an open device.
  */
-int wcdclose (dev_t dev, int flags, int fmt, struct proc *p)
+int wcdbclose (dev_t dev, int flags, int fmt, struct proc *p)
 {
 	int lun = UNIT(dev);
 	struct wcd *t = wcdtab[lun];
 
 	/* If we were the last open of the entire device, release it. */
-	wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
-		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-	t->flags &= ~F_OPEN;
+	if (! t->refcnt)
+		wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+	t->flags &= ~F_BOPEN;
+	return (0);
+}
+
+int wcdrclose (dev_t dev, int flags, int fmt, struct proc *p)
+{
+	int lun = UNIT(dev);
+	struct wcd *t = wcdtab[lun];
+
+	/* If we were the last open of the entire device, release it. */
+	if (! (t->flags & F_BOPEN) && t->refcnt == 1)
+		wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+	--t->refcnt;
 	return (0);
 }
 
@@ -464,15 +493,6 @@ void wcdstrategy (struct buf *bp)
 	struct wcd *t = wcdtab[lun];
 	int x;
 
-	/* If the device has been made invalid, error out
-	 * maybe the media changed. */
-	if (t->flags & F_MEDIA_CHANGED) {
-		bp->b_error = EIO;
-		bp->b_flags |= B_ERROR;
-		biodone (bp);
-		return;
-	}
-
 	/* Can't ever write to a CD. */
 	if (! (bp->b_flags & B_READ)) {
 		bp->b_error = EROFS;
@@ -491,10 +511,11 @@ void wcdstrategy (struct buf *bp)
 	/* Process transfer request. */
 	bp->b_pblkno = bp->b_blkno;
 	bp->b_resid = bp->b_bcount;
+
 	x = splbio();
 
 	/* Place it in the queue of disk activities for this disk. */
-	disksort (&t->queue, bp);
+	bufqdisksort (&t->buf_queue, bp);
 
 	/* Tell the device to get going on the transfer if it's
 	 * not doing anything, otherwise just wait for completion. */
@@ -512,7 +533,7 @@ void wcdstrategy (struct buf *bp)
  */
 static void wcd_start (struct wcd *t)
 {
-	struct buf *bp = t->queue.b_actf;
+	struct buf *bp = bufq_first(&t->buf_queue);
 	u_long blkno, nblk;
 
 	/* See if there is a buf to do and we are not already doing one. */
@@ -520,7 +541,7 @@ static void wcd_start (struct wcd *t)
 		return;
 
 	/* Unqueue the request. */
-	t->queue.b_actf = bp->b_actf;
+	bufq_remove(&t->buf_queue, bp);
 
 	/* Should reject all queued entries if media have changed. */
 	if (t->flags & F_MEDIA_CHANGED) {
@@ -534,14 +555,13 @@ static void wcd_start (struct wcd *t)
 	 * First, translate the block to absolute and put it in terms of the
 	 * logical blocksize of the device.
 	 * What if something asks for 512 bytes not on a 2k boundary? */
-	blkno = bp->b_blkno / (SECSIZE / 512);
+	blkno = t->starting_lba + bp->b_blkno / (SECSIZE / 512);
 	nblk = (bp->b_bcount + (SECSIZE - 1)) / SECSIZE;
 
 	atapi_request_callback (t->ata, t->unit, ATAPI_READ_BIG, 0,
 		blkno>>24, blkno>>16, blkno>>8, blkno, 0, nblk>>8, nblk, 0, 0,
 		0, 0, 0, 0, 0, (u_char*) bp->b_un.b_addr, bp->b_bcount,
 		wcd_done, t, bp);
-	t->cf.kdc_state = DC_BUSY;
 }
 
 static void wcd_done (struct wcd *t, struct buf *bp, int resid,
@@ -554,7 +574,6 @@ static void wcd_done (struct wcd *t, struct buf *bp, int resid,
 	} else
 		bp->b_resid = resid;
 	biodone (bp);
-	t->cf.kdc_state = DC_IDLE;
 	wcd_start (t);
 }
 
@@ -598,11 +617,9 @@ static int wcd_request_wait (struct wcd *t, u_char cmd, u_char a1, u_char a2,
 {
 	struct atapires result;
 
-	t->cf.kdc_state = DC_BUSY;
 	result = atapi_request_wait (t->ata, t->unit, cmd,
 		a1, a2, a3, a4, a5, a6, a7, a8, a9, 0, 0, 0, 0, 0, 0,
 		addr, count);
-	t->cf.kdc_state = DC_IDLE;
 	if (result.code) {
 		wcd_error (t, result);
 		return (EIO);
@@ -610,35 +627,7 @@ static int wcd_request_wait (struct wcd *t, u_char cmd, u_char a1, u_char a2,
 	return (0);
 }
 
-static int wcd_play (struct wcd *t, u_char cmd, u_char a1, u_char a2,
-	u_char a3, u_char a4, u_char a5, u_char a6, u_char a7, u_char a8,
-	u_char a9, char *addr, int count)
-{
-	struct atapires result;
-
-	if (! (t->flags & F_NOPLAYCD)) {
-		t->cf.kdc_state = DC_BUSY;
-		result = atapi_request_wait (t->ata, t->unit, ATAPI_PLAY_CD,
-			cmd == ATAPI_PLAY_MSF ? 2 : 0, a2, a3, a4, a5, a6,
-			a7, a8, a9, 0, 0, 0, 0, 0, 0, addr, count);
-		t->cf.kdc_state = DC_IDLE;
-		if (result.code == RES_ERR &&
-		    (result.error & AER_SKEY) == AER_SK_ILLEGAL_REQUEST) {
-			/* Some drives don't support a PLAY_CD command.
-			 * Remember this and use PLAY_MSF instead. */
-			t->flags |= F_NOPLAYCD;
-			result.code = 0;
-		}
-		if (result.code) {
-			wcd_error (t, result);
-			return (EIO);
-		}
-	}
-	return wcd_request_wait (t, cmd, a1, a2, a3, a4, a5, a6,
-		a7, a8, a9, addr, count);
-}
-
-static inline void lba2msf (int lba, u_char *m, u_char *s, u_char *f)
+static __inline void lba2msf (int lba, u_char *m, u_char *s, u_char *f)
 {
 	lba += 150;             /* offset of first logical frame */
 	lba &= 0xffffff;        /* negative lbas use only 24 bits */
@@ -652,15 +641,28 @@ static inline void lba2msf (int lba, u_char *m, u_char *s, u_char *f)
  * Perform special action on behalf of the user.
  * Knows about the internals of this device
  */
-int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
+int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 {
 	int lun = UNIT(dev);
 	struct wcd *t = wcdtab[lun];
 	int error = 0;
 
-	/* If the device is not valid.. abandon ship.  */
 	if (t->flags & F_MEDIA_CHANGED)
-		return (EIO);
+		switch (cmd) {
+		case CDIOCSETDEBUG:
+		case CDIOCCLRDEBUG:
+		case CDIOCRESET:
+			/* These ops are media change transparent. */
+			break;
+		default:
+			/* Read table of contents. */
+			wcd_read_toc (t);
+
+			/* Lock the media. */
+			wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+				0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0);
+			break;
+		}
 	switch (cmd) {
 	default:
 		return (ENOTTY);
@@ -710,17 +712,16 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 
 	case CDIOCEJECT:
-		/* Stop the disc. */
-		error = wcd_request_wait (t, ATAPI_START_STOP,
-			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-		if (error)
-			return (error);
-		/* Give it some time to stop spinning. */
-		tsleep ((caddr_t)&lbolt, PRIBIO, "wcdejct", 0);
-		tsleep ((caddr_t)&lbolt, PRIBIO, "wcdejct", 0);
-		/* Eject. */
-		return wcd_request_wait (t, ATAPI_START_STOP,
-			0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0);
+		/* Don't allow eject if the device is opened
+		 * by somebody (not us) in block mode. */
+		if ((t->flags & F_BOPEN) && t->refcnt)
+			return (EBUSY);
+		return wcd_eject (t, 0);
+
+	case CDIOCCLOSE:
+		if ((t->flags & F_BOPEN) && t->refcnt)
+			return (0);
+		return wcd_eject (t, 1);
 
 	case CDIOREADTOCHEADER:
 		if (! t->toc.hdr.ending_track)
@@ -744,22 +745,22 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 		    || te->address_format != CD_MSF_FORMAT
 		    && te->address_format != CD_LBA_FORMAT
 		   )
-			return (EINVAL);
+			return EINVAL;
 
 		if (starting_track == 0)
 			starting_track = toc->hdr.starting_track;
-		else if (starting_track == WCD_LASTPLUS1)
+		else if (starting_track == 170) /* Handle leadout request */
 			starting_track = toc->hdr.ending_track + 1;
 		else if (starting_track < toc->hdr.starting_track ||
 			 starting_track > toc->hdr.ending_track + 1)
-			return (EINVAL);
+                        return (EINVAL);
 
-		len = (toc->hdr.ending_track + 2 - starting_track) *
+		len = ((toc->hdr.ending_track + 1 - starting_track) + 1) *
 			sizeof(toc->tab[0]);
 		if (te->data_len < len)
 			len = te->data_len;
 		if (len > sizeof(toc->tab))
-			return (EINVAL);
+			return EINVAL;
 
 		/* Convert to MSF format, if needed. */
 		if (te->address_format == CD_MSF_FORMAT) {
@@ -767,10 +768,10 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 
 			buf = t->toc;
 			toc = &buf;
-			e = toc->tab + toc->hdr.ending_track + 2 -
-				    toc->hdr.starting_track;
+			e = toc->tab + (toc->hdr.ending_track + 1 -
+					toc->hdr.starting_track) + 1;
 			while (--e >= toc->tab)
-				lba2msf (e->addr.lba, &e->addr.msf.minute,
+				lba2msf (ntohl(e->addr.lba), &e->addr.msf.minute,
 				    &e->addr.msf.second, &e->addr.msf.frame);
 		}
 		return copyout (toc->tab + starting_track -
@@ -794,14 +795,14 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 		if (t->flags & F_DEBUG)
 			wcd_dump (t->lun, "subchan", &t->subchan, sizeof t->subchan);
 
-		abslba = ntohl (t->subchan.abslba);
-		rellba = ntohl (t->subchan.rellba);
+		abslba = t->subchan.abslba;
+		rellba = t->subchan.rellba;
 		if (args->address_format == CD_MSF_FORMAT) {
-			lba2msf (abslba,
+			lba2msf (ntohl(abslba),
 				&data.what.position.absaddr.msf.minute,
 				&data.what.position.absaddr.msf.second,
 				&data.what.position.absaddr.msf.frame);
-			lba2msf (rellba,
+			lba2msf (ntohl(rellba),
 				&data.what.position.reladdr.msf.minute,
 				&data.what.position.reladdr.msf.second,
 				&data.what.position.reladdr.msf.frame);
@@ -811,24 +812,23 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 		}
 		data.header.audio_status = t->subchan.audio_status;
 		data.what.position.control = t->subchan.control & 0xf;
+		data.what.position.addr_type = t->subchan.control >> 4;
 		data.what.position.track_number = t->subchan.track;
 		data.what.position.index_number = t->subchan.indx;
 
-		if (copyout (&data, args->data, len) != 0)
-			error = EFAULT;
-		break;
+		return copyout (&data, args->data, len);
 	}
 	case CDIOCPLAYMSF: {
 		struct ioc_play_msf *args = (struct ioc_play_msf*) addr;
 
-		return wcd_play (t, ATAPI_PLAY_MSF, 0, 0,
+		return wcd_request_wait (t, ATAPI_PLAY_MSF, 0, 0,
 			args->start_m, args->start_s, args->start_f,
 			args->end_m, args->end_s, args->end_f, 0, 0, 0);
 	}
 	case CDIOCPLAYBLOCKS: {
 		struct ioc_play_blocks *args = (struct ioc_play_blocks*) addr;
 
-		return wcd_play (t, ATAPI_PLAY_BIG, 0,
+		return wcd_request_wait (t, ATAPI_PLAY_BIG, 0,
 			args->blk >> 24 & 0xff, args->blk >> 16 & 0xff,
 			args->blk >> 8 & 0xff, args->blk & 0xff,
 			args->len >> 24 & 0xff, args->len >> 16 & 0xff,
@@ -852,10 +852,10 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
 		t2 = args->end_track - t->toc.hdr.starting_track;
 		if (t1 < 0 || t2 < 0)
 			return (EINVAL);
-		start = t->toc.tab[t1].addr.lba;
-		len = t->toc.tab[t2].addr.lba - start;
+		start = ntohl(t->toc.tab[t1].addr.lba);
+		len = ntohl(t->toc.tab[t2].addr.lba) - start;
 
-		return wcd_play (t, ATAPI_PLAY_BIG, 0,
+		return wcd_request_wait (t, ATAPI_PLAY_BIG, 0,
 			start >> 24 & 0xff, start >> 16 & 0xff,
 			start >> 8 & 0xff, start & 0xff,
 			len >> 24 & 0xff, len >> 16 & 0xff,
@@ -944,30 +944,78 @@ int wcdioctl (dev_t dev, int cmd, caddr_t addr, int flags, struct proc *p)
  */
 static int wcd_read_toc (struct wcd *t)
 {
-	int ntracks, len, i;
+	int ntracks, len;
+	struct atapires result;
+
+	bzero (&t->toc, sizeof (t->toc));
+	bzero (&t->info, sizeof (t->info));
+
+	/* Check for the media.
+	 * Do it twice to avoid the stale media changed state. */
+	result = atapi_request_wait (t->ata, t->unit, ATAPI_TEST_UNIT_READY,
+		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+
+	if (result.code == RES_ERR &&
+	    (result.error & AER_SKEY) == AER_SK_UNIT_ATTENTION) {
+		t->flags |= F_MEDIA_CHANGED;
+		result = atapi_request_wait (t->ata, t->unit,
+			ATAPI_TEST_UNIT_READY, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 0, 0, 0, 0, 0, 0, 0, 0);
+	}
+	if (result.code) {
+		wcd_error (t, result);
+		return (EIO);
+	}
+	t->flags &= ~F_MEDIA_CHANGED;
 
 	/* First read just the header, so we know how long the TOC is. */
 	len = sizeof(struct ioc_toc_header) + sizeof(struct cd_toc_entry);
 	if (wcd_request_wait (t, ATAPI_READ_TOC, 0, 0, 0, 0, 0, 0,
-	    len >> 8, len & 0xff, 0, (char*)&t->toc, len) != 0)
-		return (EIO);
+	    len >> 8, len & 0xff, 0, (char*)&t->toc, len) != 0) {
+err:            bzero (&t->toc, sizeof (t->toc));
+		return (0);
+	}
 
 	ntracks = t->toc.hdr.ending_track - t->toc.hdr.starting_track + 1;
-	if (ntracks <= 0)
-		return (EIO);
-	if (ntracks > MAXTRK)
-		ntracks = MAXTRK;
+	if (ntracks <= 0 || ntracks > MAXTRK)
+		goto err;
 
 	/* Now read the whole schmeer. */
 	len = sizeof(struct ioc_toc_header) +
-		(ntracks+1) * sizeof(struct cd_toc_entry);
+		ntracks * sizeof(struct cd_toc_entry);
 	if (wcd_request_wait (t, ATAPI_READ_TOC, 0, 0, 0, 0, 0, 0,
 	    len >> 8, len & 0xff, 0, (char*)&t->toc, len) & 0xff)
-		return (EIO);
+		goto err;
 
-	t->toc.hdr.len = ntohs (t->toc.hdr.len);
-	for (i=0; i<=ntracks; i++)
-		t->toc.tab[i].addr.lba = ntohl (t->toc.tab[i].addr.lba);
+	NTOHS(t->toc.hdr.len);
+
+	/* Read disc capacity. */
+	if (wcd_request_wait (t, ATAPI_READ_CAPACITY, 0, 0, 0, 0, 0, 0,
+	    0, sizeof(t->info), 0, (char*)&t->info, sizeof(t->info)) != 0)
+		bzero (&t->info, sizeof (t->info));
+
+	/* make fake leadout entry */
+	t->toc.tab[ntracks].control = t->toc.tab[ntracks-1].control;
+	t->toc.tab[ntracks].addr_type = t->toc.tab[ntracks-1].addr_type;
+	t->toc.tab[ntracks].track = 170; /* magic */
+	t->toc.tab[ntracks].addr.lba = t->info.volsize;
+
+	NTOHL(t->info.volsize);
+	NTOHL(t->info.blksize);
+
+	/* Print the disc description string on every disc change.
+	 * It would help to track the history of disc changes. */
+	if (t->info.volsize && t->toc.hdr.ending_track &&
+	    (t->flags & F_MEDIA_CHANGED) && (t->flags & F_DEBUG)) {
+		printf ("wcd%d: ", t->lun);
+		if (t->toc.tab[0].control & 4)
+			printf ("%ldMB ", t->info.volsize / 512);
+		else
+			printf ("%ld:%ld audio ", t->info.volsize/75/60,
+				t->info.volsize/75%60);
+		printf ("(%ld sectors), %d tracks\n", t->info.volsize,
+			t->toc.hdr.ending_track - t->toc.hdr.starting_track + 1);
+	}
 	return (0);
 }
 
@@ -1000,4 +1048,176 @@ static int wcd_setchan (struct wcd *t,
 		0, 0, 0, 0, 0, sizeof (t->au) >> 8, sizeof (t->au),
 		0, (char*) &t->au, - sizeof (t->au));
 }
+
+static int wcd_eject (struct wcd *t, int closeit)
+{
+	struct atapires result;
+
+	/* Try to stop the disc. */
+	result = atapi_request_wait (t->ata, t->unit, ATAPI_START_STOP,
+		1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+
+	if (result.code == RES_ERR &&
+	    ((result.error & AER_SKEY) == AER_SK_NOT_READY ||
+	    (result.error & AER_SKEY) == AER_SK_UNIT_ATTENTION)) {
+		int err;
+
+		if (!closeit)
+			return (0);
+		/*
+		 * The disc was unloaded.
+		 * Load it (close tray).
+		 * Read the table of contents.
+		 */
+		err = wcd_request_wait (t, ATAPI_START_STOP,
+			0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0);
+		if (err)
+			return (err);
+
+		/* Read table of contents. */
+		wcd_read_toc (t);
+
+		/* Lock the media. */
+		wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+			0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0);
+
+		return (0);
+	}
+
+	if (result.code) {
+		wcd_error (t, result);
+		return (EIO);
+	}
+
+	if (closeit)
+		return (0);
+
+	/* Give it some time to stop spinning. */
+	tsleep ((caddr_t)&lbolt, PRIBIO, "wcdej1", 0);
+	tsleep ((caddr_t)&lbolt, PRIBIO, "wcdej2", 0);
+
+	/* Unlock. */
+	wcd_request_wait (t, ATAPI_PREVENT_ALLOW,
+		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+
+	/* Eject. */
+	t->flags |= F_MEDIA_CHANGED;
+	return wcd_request_wait (t, ATAPI_START_STOP,
+		0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0);
+}
+
+#ifdef WCD_MODULE
+/*
+ * Loadable ATAPI CD-ROM driver stubs.
+ */
+#include <sys/exec.h>
+#include <sys/sysent.h>
+#include <sys/lkm.h>
+
+/*
+ * Construct lkm_dev structures (see lkm.h).
+ * Our bdevsw/cdevsw slot numbers are 19/69.
+ */
+
+
+MOD_DEV(wcd, LM_DT_BLOCK, BDEV_MAJOR, &wcd_bdevsw);
+MOD_DEV(rwcd, LM_DT_CHAR, CDEV_MAJOR, &wcd_cdevsw);
+
+/*
+ * Function called when loading the driver.
+ */
+int wcd_load (struct lkm_table *lkmtp, int cmd)
+{
+	struct atapi *ata;
+	int n, u;
+
+	if (! atapi_start)
+		/* No ATAPI driver available. */
+		return EPROTONOSUPPORT;
+	n = 0;
+	for (ata=atapi_tab; ata<atapi_tab+2; ++ata)
+		if (ata->port)
+			for (u=0; u<2; ++u)
+				/* Probing controller ata->ctrlr, unit u. */
+				if (ata->params[u] && ! ata->attached[u] &&
+				    wcdattach (ata, u, ata->params[u],
+				    ata->debug) >= 0)
+				{
+					/* Drive found. */
+					ata->attached[u] = 1;
+					++n;
+				}
+	if (! n)
+		/* No IDE CD-ROMs found. */
+		return ENXIO;
+	return 0;
+}
+
+/*
+ * Function called when unloading the driver.
+ */
+int wcd_unload (struct lkm_table *lkmtp, int cmd)
+{
+	struct wcd **t;
+
+	for (t=wcdtab; t<wcdtab+wcdnlun; ++t)
+		if (((*t)->flags & F_BOPEN) || (*t)->refcnt)
+			/* The device is opened, cannot unload the driver. */
+			return EBUSY;
+	for (t=wcdtab; t<wcdtab+wcdnlun; ++t) {
+		(*t)->ata->attached[(*t)->unit] = 0;
+		free (*t, M_TEMP);
+	}
+	wcdnlun = 0;
+	bzero (wcdtab, sizeof(wcdtab));
+	return 0;
+}
+
+/*
+ * Dispatcher function for the module (load/unload/stat).
+ */
+int wcd_mod (struct lkm_table *lkmtp, int cmd, int ver)
+{
+	int err = 0;
+
+	if (ver != LKM_VERSION)
+		return EINVAL;
+
+	if (cmd == LKM_E_LOAD)
+		err = wcd_load (lkmtp, cmd);
+	else if (cmd == LKM_E_UNLOAD)
+		err = wcd_unload (lkmtp, cmd);
+	if (err)
+		return err;
+
+	/* Register the cdevsw entry. */
+	lkmtp->private.lkm_dev = &rwcd_module;
+	err = lkmdispatch (lkmtp, cmd);
+	if (err)
+		return err;
+
+	/* Register the bdevsw entry. */
+	lkmtp->private.lkm_dev = &wcd_module;
+	return lkmdispatch (lkmtp, cmd);
+}
+#endif /* WCD_MODULE */
+
+static wcd_devsw_installed = 0;
+
+static void 	wcd_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! wcd_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&wcd_cdevsw, NULL);
+		dev = makedev(BDEV_MAJOR, 0);
+		bdevsw_add(&dev,&wcd_bdevsw, NULL);
+		wcd_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(wcddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,wcd_drvinit,NULL)
+
+
 #endif /* NWCD && NWDC && ATAPI */
