@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)wd.c	7.2 (Berkeley) 5/9/91
- * $FreeBSD: stable/2.1/sys/i386/isa/wd.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/wd.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /* TODO:
@@ -61,6 +61,9 @@
 #endif
 
 #include "wdc.h"
+#include "opt_wd.h"
+#include "opt_atapi.h"
+
 #if     NWDC > 0
 
 #include <sys/param.h>
@@ -68,31 +71,37 @@
 #include <sys/systm.h>
 #include <sys/kernel.h>
 #include <sys/conf.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <sys/ioctl.h>
 #include <sys/disklabel.h>
 #include <sys/diskslice.h>
 #include <sys/buf.h>
-#include <sys/proc.h>
-#include <sys/uio.h>
 #include <sys/malloc.h>
-#include <sys/devconf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 #include <machine/bootinfo.h>
 #include <machine/clock.h>
+#include <machine/cons.h>
 #include <machine/md_var.h>
-#include <i386/i386/cons.h>
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/wdreg.h>
 #include <sys/syslog.h>
 #include <sys/dkstat.h>
 #include <vm/vm.h>
+#include <vm/vm_prot.h>
+#include <vm/pmap.h>
+
 
 #ifdef ATAPI
 #include <i386/isa/atapi.h>
 #endif
 
+#ifdef CMD640
+#include <i386/isa/wdc_p.h>
+#endif /*CMD640*/
+
+extern void wdstart(int ctrlr);
+
 #define TIMEOUT		10000
 #define	RETRIES		5	/* number of retries before giving up */
 #define RECOVERYTIME	500000	/* usec for controller to recover after err */
@@ -101,80 +110,9 @@
 				/* can't handle that in all cases */
 #define WDOPT_32BIT	0x8000
 #define WDOPT_SLEEPHACK	0x4000
+#define WDOPT_FORCEHD(x)	(((x)&0x0f00)>>8)
 #define WDOPT_MULTIMASK	0x00ff
 
-static int wd_goaway(struct kern_devconf *, int);
-static int wdc_goaway(struct kern_devconf *, int);
-static int wd_externalize(struct proc *, struct kern_devconf *, void *, size_t);
-static int wdc_externalize(struct proc *, struct kern_devconf *, void *, size_t);
-
-/*
- * Templates for the kern_devconf structures used when we attach.
- */
-static struct kern_devconf kdc_wd[NWD] = { {
-	0, 0, 0,		/* filled in by kern_devconf.c */
-	"wd", 0, { MDDT_DISK, 0 },
-	wd_externalize, 0, wd_goaway, DISK_EXTERNALLEN,
-	0,			/* parent */
-	0,			/* parentdata */
-	DC_UNKNOWN,		/* state */
-	"ST506/ESDI/IDE disk",	/* description */
-	DC_CLS_DISK		/* class */
-} };
-
-struct kern_devconf kdc_wdc[NWDC] = { {
-	0, 0, 0,		/* filled in by kern_devconf.c */
-	"wdc", 0, { MDDT_ISA, 0 },
-	isa_generic_externalize, 0, wdc_goaway, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"ST506/ESDI/IDE disk controller",
-	DC_CLS_MISC		/* just an ordinary device */
-} };
-
-static inline void
-wd_registerdev(int ctlr, int unit)
-{
-	if(unit != 0)
-		kdc_wd[unit] = kdc_wd[0];
-
-	kdc_wd[unit].kdc_unit = unit;
-	kdc_wd[unit].kdc_parent = &kdc_wdc[ctlr];
-	dev_attach(&kdc_wd[unit]);
-}
-
-static inline void
-wdc_registerdev(struct isa_device *dvp)
-{
-	int unit = dvp->id_unit;
-
-	if(unit != 0)
-		kdc_wdc[unit] = kdc_wdc[0];
-
-	kdc_wdc[unit].kdc_unit = unit;
-	kdc_wdc[unit].kdc_parentdata = dvp;
-	dev_attach(&kdc_wdc[unit]);
-}
-
-static int
-wdc_goaway(struct kern_devconf *kdc, int force)
-{
-	if(force) {
-		dev_detach(kdc);
-		return 0;
-	} else {
-		return EBUSY;	/* XXX fix */
-	}
-}
-
-static int
-wd_goaway(struct kern_devconf *kdc, int force)
-{
-	dev_detach(kdc);
-	return 0;
-}
-
 /*
  * This biotab field doubles as a field for the physical unit number on
  * the controller.
@@ -190,6 +128,8 @@ wd_goaway(struct kern_devconf *kdc, int force)
 #define	RECAL		2	/* doing restore */
 #define	OPEN		3	/* done with open */
 
+#define PRIMARY		0
+
 /*
  * Disk geometry.  A small part of struct disklabel.
  * XXX disklabel.5 contains an old clone of disklabel.h.
@@ -210,15 +150,21 @@ struct diskgeom {
 struct disk {
 	long	dk_bc;		/* byte count left */
 	short	dk_skip;	/* blocks already transferred */
-	char	dk_ctrlr;	/* physical controller number */
-	char	dk_unit;	/* physical unit number */
-	char	dk_lunit;	/* logical unit number */
+	int	dk_ctrlr;	/* physical controller number */
+#ifdef CMD640
+	int	dk_ctrlr_cmd640;/* controller number for CMD640 quirk */
+#endif
+	int	dk_unit;	/* physical unit number */
+	int	dk_lunit;	/* logical unit number */
 	char	dk_state;	/* control state */
 	u_char	dk_status;	/* copy of status reg. */
 	u_char	dk_error;	/* copy of error reg. */
 	u_char	dk_timeout;	/* countdown to next timeout */
-	short	dk_port;	/* i/o port base */
-
+	int	dk_port;	/* i/o port base */
+#ifdef	DEVFS
+	void	*dk_bdev;	/* devfs token for whole disk */
+	void	*dk_cdev;	/* devfs token for raw whole disk */
+#endif
 	u_long	cfg_flags;	/* configured characteristics */
 	short	dk_flags;	/* drive characteristics found */
 #define	DKFL_SINGLE	0x00004	/* sector at a time mode */
@@ -239,8 +185,22 @@ struct disk {
 static int wdtest = 0;
 
 static struct disk *wddrives[NWD];	/* table of units */
+static struct buf_queue_head drive_queue[NWD];	/* head of queue per drive */
+static struct {
+	int	b_errcnt;
+	int	b_active;
+} wdutab[NWD];
+/*
 static struct buf wdtab[NWDC];
-static struct buf wdutab[NWD];	/* head of queue per drive */
+*/
+static struct {
+	struct	buf_queue_head controller_queue;
+	int	b_errcnt;
+	int	b_active;
+} wdtab[NWDC];
+
+struct wddma wddma;
+
 #ifdef notyet
 static struct buf rwdbuf[NWD];	/* buffers for raw IO */
 #endif
@@ -248,12 +208,13 @@ static struct buf rwdbuf[NWD];	/* buffers for raw IO */
 static int wdprobe(struct isa_device *dvp);
 static int wdattach(struct isa_device *dvp);
 static void wdustart(struct disk *du);
-void wdstart(int ctrlr);
 static int wdcontrol(struct buf *bp);
 static int wdcommand(struct disk *du, u_int cylinder, u_int head,
 		     u_int sector, u_int count, u_int command);
 static int wdsetctlr(struct disk *du);
+#if 0
 static int wdwsetctlr(struct disk *du);
+#endif
 static int wdgetctlr(struct disk *du);
 static void wderror(struct buf *bp, struct disk *du, char *mesg);
 static void wdflushirq(struct disk *du, int old_ipl);
@@ -264,19 +225,44 @@ static timeout_t wdtimeout;
 static int wdunwedge(struct disk *du);
 static int wdwait(struct disk *du, u_char bits_wanted, int timeout);
 
-/*
- * Provide hw.devconf information.
- */
-static int
-wd_externalize(struct proc *p, struct kern_devconf *kdc, void *userp, size_t len)
-{
-	return disk_externalize(wddrives[kdc->kdc_unit]->dk_unit, userp, &len);
-}
-
 struct isa_driver wdcdriver = {
 	wdprobe, wdattach, "wdc",
 };
 
+static	d_open_t	wdopen;
+static	d_close_t	wdclose;
+static	d_strategy_t	wdstrategy;
+static	d_ioctl_t	wdioctl;
+static	d_dump_t	wddump;
+static	d_psize_t	wdsize;
+
+#define CDEV_MAJOR 3
+#define BDEV_MAJOR 0
+static struct cdevsw wd_cdevsw;
+static struct bdevsw wd_bdevsw = 
+	{ wdopen,	wdclose,	wdstrategy,	wdioctl,	/*0*/
+	  wddump,	wdsize,		0,	"wd",	&wd_cdevsw,	-1 };
+
+#ifdef CMD640
+static int      atapictrlr;
+static int      eide_quirks;
+#endif
+
+
+/*
+ *  Here we use the pci-subsystem to find out, whether there is
+ *  a cmd640b-chip attached on this pci-bus. This public routine
+ *  will be called by wdc_p.c .
+ */
+
+#ifdef CMD640
+void
+wdc_pci(int quirks)
+{
+	eide_quirks = quirks;
+}
+#endif
+
 /*
  * Probe for controller.
  */
@@ -296,7 +282,6 @@ wdprobe(struct isa_device *dvp)
 	du->dk_ctrlr = dvp->id_unit;
 	du->dk_port = dvp->id_iobase;
 
-	wdc_registerdev(dvp);
 
 	/* check if we have registers that work */
 	outb(du->dk_port + wd_sdh, WDSD_IBM);   /* set unit 0 */
@@ -348,7 +333,7 @@ reset_ok:
 	 */
 	du->dk_error = inb(du->dk_port + wd_error);
 	/* printf("Error : %x\n", du->dk_error); */
-	if(du->dk_error != 0x01) {
+	if(du->dk_error != 0x01 && du->dk_error != 0) {
 		if(du->dk_error & 0x80) { /* drive 1 failure */
 
 			/* first set the DRV bit */
@@ -364,8 +349,12 @@ reset_ok:
 			/* Get status for drive 1 */
 			du->dk_error = inb(du->dk_port + wd_error);
 			/* printf("Error (drv 1) : %x\n", du->dk_error); */
-
-			if(du->dk_error != 0x01)
+			/*
+			 * Sometimes (apparently mostly with ATAPI
+			 * drives involved) 0x81 really means 0x81
+			 * (drive 0 OK, drive 1 failed).
+			 */
+			if(du->dk_error != 0x01 && du->dk_error != 0x81)
 				goto nodevice;
 		} else	/* drive 0 fail */
 			goto nodevice;
@@ -386,6 +375,9 @@ nodevice:
 static int
 wdattach(struct isa_device *dvp)
 {
+#ifdef DEVFS
+	int	mynor;
+#endif
 	int	unit, lunit;
 	struct isa_device *wdup;
 	struct disk *du;
@@ -393,7 +385,18 @@ wdattach(struct isa_device *dvp)
 	if (dvp->id_unit >= NWDC)
 		return (0);
 
-	kdc_wdc[dvp->id_unit].kdc_state = DC_UNKNOWN; /* XXX */
+#ifdef CMD640
+	if (eide_quirks & Q_CMD640B) {
+		if (dvp->id_unit == PRIMARY) {
+			printf("wdc0: CMD640B workaround enabled\n");
+			bufq_init(&wdtab[PRIMARY].controller_queue);
+		}
+	} else
+		bufq_init(&wdtab[dvp->id_unit].controller_queue);
+
+#else
+	bufq_init(&wdtab[dvp->id_unit].controller_queue);
+#endif
 
 	for (wdup = isa_biotab_wdc; wdup->id_driver != 0; wdup++) {
 		if (wdup->id_iobase != dvp->id_iobase)
@@ -401,6 +404,7 @@ wdattach(struct isa_device *dvp)
 		lunit = wdup->id_unit;
 		if (lunit >= NWD)
 			continue;
+
 		unit = wdup->id_physid;
 
 		du = malloc(sizeof *du, M_TEMP, M_NOWAIT);
@@ -409,8 +413,16 @@ wdattach(struct isa_device *dvp)
 		if (wddrives[lunit] != NULL)
 			panic("drive attached twice");
 		wddrives[lunit] = du;
+		bufq_init(&drive_queue[lunit]);
 		bzero(du, sizeof *du);
 		du->dk_ctrlr = dvp->id_unit;
+#ifdef CMD640
+		if (eide_quirks & Q_CMD640B) {
+			du->dk_ctrlr_cmd640 = PRIMARY;
+		} else {
+			du->dk_ctrlr_cmd640 = du->dk_ctrlr;
+		}
+#endif
 		du->dk_unit = unit;
 		du->dk_lunit = lunit;
 		du->dk_port = dvp->id_iobase;
@@ -423,18 +435,14 @@ wdattach(struct isa_device *dvp)
 			((dvp->id_flags) >> (16 * unit));
 
 		if (wdgetctlr(du) == 0) {
-			char buf[sizeof du->dk_params.wdp_model + 1];
-
 			/*
 			 * Print out description of drive.
-			 * wdp_model may not be null terminated, and printf
-			 * doesn't support "%.*s" :-(, so copy wdp_model
-			 * and add a null before printing.
+			 * wdp_model may not be null terminated.
 			 */
-			bcopy(du->dk_params.wdp_model, buf, sizeof buf - 1);
-			buf[sizeof buf - 1] = '\0';
-			printf("wdc%d: unit %d (wd%d): <%s>",
-			       dvp->id_unit, unit, lunit, buf);
+			printf("wdc%d: unit %d (wd%d): <%.*s>",
+				dvp->id_unit, unit, lunit,
+				sizeof du->dk_params.wdp_model,
+				du->dk_params.wdp_model);
 			if (du->dk_flags & DKFL_32BIT)
 				printf(", 32-bit");
 			if (du->dk_multi > 1)
@@ -450,7 +458,7 @@ wdattach(struct isa_device *dvp)
 "wd%d: %luMB (%lu sectors), %lu cyls, %lu heads, %lu S/T, %lu B/S\n",
 			       lunit,
 			       du->dk_dd.d_secperunit
-			       * du->dk_dd.d_secsize / (1024 * 1024),
+			       / ((1024L * 1024L) / du->dk_dd.d_secsize),
 			       du->dk_dd.d_secperunit,
 			       du->dk_dd.d_ncylinders,
 			       du->dk_dd.d_ntracks,
@@ -463,7 +471,18 @@ wdattach(struct isa_device *dvp)
 			 */
 			wdtimeout(du);
 
-			wd_registerdev(dvp->id_unit, lunit);
+#ifdef DEVFS
+			mynor = dkmakeminor(unit, WHOLE_DISK_SLICE, RAW_PART);
+			du->dk_bdev = devfs_add_devswf(&wd_bdevsw, mynor,
+						       DV_BLK, UID_ROOT,
+						       GID_OPERATOR, 0640,
+						       "wd%d", unit);
+			du->dk_cdev = devfs_add_devswf(&wd_cdevsw, mynor,
+						       DV_CHR, UID_ROOT,
+						       GID_OPERATOR, 0640,
+						       "rwd%d", unit);
+#endif
+
 			if (dk_ndrive < DK_NDRIVE) {
 				sprintf(dk_names[dk_ndrive], "wd%d", lunit);
 				/*
@@ -487,19 +506,33 @@ wdattach(struct isa_device *dvp)
 	 * Probe all free IDE units, searching for ATAPI drives.
 	 */
 	for (unit=0; unit<2; ++unit) {
-		for (lunit=0; lunit<NWD && wddrives[lunit]; ++lunit)
-			if (wddrives[lunit]->dk_ctrlr == dvp->id_unit &&
+		for (lunit=0; lunit<NWD; ++lunit)
+			if (wddrives[lunit] &&
+			    wddrives[lunit]->dk_ctrlr == dvp->id_unit &&
 			    wddrives[lunit]->dk_unit == unit)
 				goto next;
-		atapi_attach (dvp->id_unit, unit, dvp->id_iobase,
-			&kdc_wdc[dvp->id_unit]);
-next:   }
+#ifdef CMD640
+		if (atapi_attach (dvp->id_unit, unit, dvp->id_iobase))
+			atapictrlr = dvp->id_unit;
+#else
+		atapi_attach (dvp->id_unit, unit, dvp->id_iobase);
+#endif
+next: ;
+	}
 #endif
 	/*
 	 * Discard any interrupts generated by wdgetctlr().  wdflushirq()
 	 * doesn't work now because the ambient ipl is too high.
 	 */
+#ifdef CMD640
+	if (eide_quirks & Q_CMD640B) {
+		wdtab[PRIMARY].b_active = 2;
+	} else {
+		wdtab[dvp->id_unit].b_active = 2;
+	}
+#else
 	wdtab[dvp->id_unit].b_active = 2;
+#endif
 
 	return (1);
 }
@@ -512,7 +545,6 @@ next:   }
 void
 wdstrategy(register struct buf *bp)
 {
-	register struct buf *dp;
 	struct disk *du;
 	int	lunit = dkunit(bp->b_dev);
 	int	s;
@@ -527,7 +559,7 @@ wdstrategy(register struct buf *bp)
 	}
 
 	/*
-	 * Do bounds checking, adjust transfer, set b_cylin and b_pbklno.
+	 * Do bounds checking, adjust transfer, and set b_pblkno.
 	 */
 	if (dscheck(bp, du->dk_slices) <= 0)
 		goto done;
@@ -562,12 +594,11 @@ wdstrategy(register struct buf *bp)
 	}
 
 	/* queue transfer on drive, activate drive and controller if idle */
-	dp = &wdutab[lunit];
 	s = splbio();
 
-	disksort(dp, bp);
+	bufqdisksort(&drive_queue[lunit], bp);
 
-	if (dp->b_active == 0)
+	if (wdutab[lunit].b_active == 0)
 		wdustart(du);	/* start drive */
 
 	/* Pick up changes made by readdisklabel(). */
@@ -576,7 +607,11 @@ wdstrategy(register struct buf *bp)
 		du->dk_state = WANTOPEN;
 	}
 
+#ifdef CMD640
+	if (wdtab[du->dk_ctrlr_cmd640].b_active == 0)
+#else
 	if (wdtab[du->dk_ctrlr].b_active == 0)
+#endif
 		wdstart(du->dk_ctrlr);	/* start controller */
 
 	if (du->dk_dkunit >= 0) {
@@ -623,30 +658,29 @@ wdstrategy1(struct buf *bp)
 static void
 wdustart(register struct disk *du)
 {
-	register struct buf *bp, *dp = &wdutab[du->dk_lunit];
+	register struct buf *bp;
+#ifdef CMD640
+	int	ctrlr = du->dk_ctrlr_cmd640;
+#else
 	int	ctrlr = du->dk_ctrlr;
+#endif
 
 	/* unit already active? */
-	if (dp->b_active)
+	if (wdutab[du->dk_lunit].b_active)
 		return;
 
-	/* anything to start? */
-	bp = dp->b_actf;
-	if (bp == NULL)
-		return;
 
-	dp->b_actf = bp->b_actf;
-	bp->b_actf = NULL;
-	/* link onto controller queue */
-	if (wdtab[ctrlr].b_actf == NULL) {
-		wdtab[ctrlr].b_actf = bp;
-	} else {
-		*wdtab[ctrlr].b_actb = bp;
+	bp = bufq_first(&drive_queue[du->dk_lunit]);
+	if (bp == NULL) {	/* yes, an assign */
+		return;
 	}
-	wdtab[ctrlr].b_actb = &bp->b_actf;
+	bufq_remove(&drive_queue[du->dk_lunit], bp);
+
+	/* link onto controller queue */
+	bufq_insert_tail(&wdtab[ctrlr].controller_queue, bp);
 
 	/* mark the drive unit as busy */
-	dp->b_active = 1;
+	wdutab[du->dk_lunit].b_active = 1;
 }
 
 /*
@@ -663,11 +697,20 @@ wdstart(int ctrlr)
 	register struct disk *du;
 	register struct buf *bp;
 	struct diskgeom *lp;	/* XXX sic */
-	struct buf *dp;
 	long	blknum;
 	long	secpertrk, secpercyl;
 	int	lunit;
 	int	count;
+#ifdef CMD640
+	int	ctrlr_atapi;
+
+	if (eide_quirks & Q_CMD640B) {
+		ctrlr = PRIMARY;
+		ctrlr_atapi = atapictrlr;
+	} else {
+		ctrlr_atapi = ctrlr;
+	}
+#endif
 
 #ifdef ATAPI
 	if (wdtab[ctrlr].b_active == 2)
@@ -675,14 +718,18 @@ wdstart(int ctrlr)
 	if (wdtab[ctrlr].b_active)
 		return;
 #endif
-loop:
 	/* is there a drive for the controller to do a transfer with? */
-	bp = wdtab[ctrlr].b_actf;
+	bp = bufq_first(&wdtab[ctrlr].controller_queue);
 	if (bp == NULL) {
 #ifdef ATAPI
-		if (atapi_start (ctrlr))
+#ifdef CMD640
+		if (atapi_start && atapi_start (ctrlr_atapi))
+			wdtab[ctrlr].b_active = 3;
+#else
+		if (atapi_start && atapi_start (ctrlr))
 			/* mark controller active in ATAPI mode */
 			wdtab[ctrlr].b_active = 3;
+#endif
 #endif
 		return;
 	}
@@ -744,7 +791,7 @@ loop:
 	/* if starting a multisector transfer, or doing single transfers */
 	if (du->dk_skip == 0 || (du->dk_flags & DKFL_SINGLE)) {
 		u_int	command;
-		u_int	count;
+		u_int	count1;
 		long	cylin, head, sector;
 
 		cylin = blknum / secpercyl;
@@ -753,14 +800,14 @@ loop:
 
 		if (wdtab[ctrlr].b_errcnt && (bp->b_flags & B_READ) == 0)
 			du->dk_bc += DEV_BSIZE;
-		count = howmany( du->dk_bc, DEV_BSIZE);
+		count1 = howmany( du->dk_bc, DEV_BSIZE);
 
 		du->dk_flags &= ~DKFL_MULTI;
 
 #ifdef B_FORMAT
 		if (bp->b_flags & B_FORMAT) {
 			command = WDCC_FORMAT;
-			count = lp->d_nsectors;
+			count1 = lp->d_nsectors;
 			sector = lp->d_gap3 - 1;	/* + 1 later */
 		} else
 #endif
@@ -769,10 +816,10 @@ loop:
 			if (du->dk_flags & DKFL_SINGLE) {
 				command = (bp->b_flags & B_READ)
 					  ? WDCC_READ : WDCC_WRITE;
-				count = 1;
+				count1 = 1;
 				du->dk_currentiosize = 1;
 			} else {
-				if( (count > 1) && (du->dk_multi > 1)) {
+				if( (count1 > 1) && (du->dk_multi > 1)) {
 					du->dk_flags |= DKFL_MULTI;
 					if( bp->b_flags & B_READ) {
 						command = WDCC_READ_MULTI;
@@ -780,8 +827,8 @@ loop:
 						command = WDCC_WRITE_MULTI;
 					}
 					du->dk_currentiosize = du->dk_multi;
-					if( du->dk_currentiosize > count)
-						du->dk_currentiosize = count;
+					if( du->dk_currentiosize > count1)
+						du->dk_currentiosize = count1;
 				} else {
 					if( bp->b_flags & B_READ) {
 						command = WDCC_READ;
@@ -808,7 +855,7 @@ loop:
 		if(du->dk_dkunit >= 0) {
 			dk_busy |= 1 << du->dk_dkunit;
 		}
-		while (wdcommand(du, cylin, head, sector, count, command)
+		while (wdcommand(du, cylin, head, sector, count1, command)
 		       != 0) {
 			wderror(bp, du,
 				"wdstart: timeout waiting to give command");
@@ -896,7 +943,18 @@ void
 wdintr(int unit)
 {
 	register struct	disk *du;
-	register struct buf *bp, *dp;
+	register struct buf *bp;
+
+#ifdef CMD640
+	int ctrlr_atapi;
+
+	if (eide_quirks & Q_CMD640B) {
+		unit = PRIMARY;
+		ctrlr_atapi = atapictrlr;
+	} else {
+		ctrlr_atapi = unit;
+	}
+#endif
 
 	if (wdtab[unit].b_active == 2)
 		return;		/* intr in wdflushirq() */
@@ -914,7 +972,11 @@ wdintr(int unit)
 #ifdef ATAPI
 	if (wdtab[unit].b_active == 3) {
 		/* process an ATAPI interrupt */
-		if (atapi_intr (unit))
+#ifdef CMD640
+		if (atapi_intr && atapi_intr (ctrlr_atapi))
+#else
+		if (atapi_intr && atapi_intr (unit))
+#endif
 			/* ATAPI op continues */
 			return;
 		/* controller is free, start new op */
@@ -923,9 +985,8 @@ wdintr(int unit)
 		return;
 	}
 #endif
-	bp = wdtab[unit].b_actf;
+	bp = bufq_first(&wdtab[unit].controller_queue);
 	du = wddrives[dkunit(bp->b_dev)];
-	dp = &wdutab[du->dk_lunit];
 	du->dk_timeout = 0;
 
 	if (wdwait(du, 0, TIMEOUT) < 0) {
@@ -954,7 +1015,7 @@ oops:
 		 * XXX bogus inb() here, register 0 is assumed and intr status
 		 * is reset.
 		 */
-		if( (du->dk_status & DKFL_MULTI) && (inb(du->dk_port) & WDERR_ABORT)) {
+		if( (du->dk_flags & DKFL_MULTI) && (inb(du->dk_port) & WDERR_ABORT)) {
 			wderror(bp, du, "reverting to non-multi sector mode");
 			du->dk_multi = 1;
 		}
@@ -1065,11 +1126,11 @@ outt:
 done: ;
 		/* done with this transfer, with or without error */
 		du->dk_flags &= ~DKFL_SINGLE;
-		wdtab[unit].b_actf = bp->b_actf;
+		bufq_remove(&wdtab[unit].controller_queue, bp);
 		wdtab[unit].b_errcnt = 0;
 		bp->b_resid = bp->b_bcount - du->dk_skip * DEV_BSIZE;
-		dp->b_active = 0;
-		dp->b_errcnt = 0;
+		wdutab[du->dk_lunit].b_active = 0;
+		wdutab[du->dk_lunit].b_errcnt = 0;
 		du->dk_skip = 0;
 		biodone(bp);
 	}
@@ -1086,7 +1147,7 @@ done: ;
 	/* anything more for controller to do? */
 #ifndef ATAPI
 	/* This is not valid in ATAPI mode. */
-	if (wdtab[unit].b_actf)
+	if (bufq_first(&wdtab[unit].controller_queue) != NULL)
 #endif
 		wdstart(unit);
 }
@@ -1100,9 +1161,6 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 	register unsigned int lunit;
 	register struct disk *du;
 	int	error;
-	int	part = dkpart(dev), mask = 1 << part;
-	struct partition *pp;
-	char	*msg;
 
 	lunit = dkunit(dev);
 	if (lunit >= NWD || dktype(dev) != 0)
@@ -1112,8 +1170,13 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 		return (ENXIO);
 
 	/* Finish flushing IRQs left over from wdattach(). */
+#ifdef CMD640
+	if (wdtab[du->dk_ctrlr_cmd640].b_active == 2)
+		wdtab[du->dk_ctrlr_cmd640].b_active = 0;
+#else
 	if (wdtab[du->dk_ctrlr].b_active == 2)
 		wdtab[du->dk_ctrlr].b_active = 0;
+#endif
 
 	du->dk_flags &= ~DKFL_BADSCAN;
 
@@ -1123,7 +1186,6 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 	wdsleep(du->dk_ctrlr, "wdopn1");
 	du->dk_flags |= DKFL_LABELLING;
 	du->dk_state = WANTOPEN;
-	wdutab[lunit].b_actf = NULL;
 	{
 	struct disklabel label;
 
@@ -1135,7 +1197,7 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 	label.d_secpercyl = du->dk_dd.d_secpercyl;
 	label.d_secperunit = du->dk_dd.d_secperunit;
 	error = dsopen("wd", dev, fmt, &du->dk_slices, &label, wdstrategy1,
-		       (ds_setgeom_t *)NULL);
+		       (ds_setgeom_t *)NULL, &wd_bdevsw, &wd_cdevsw);
 	}
 	du->dk_flags &= ~DKFL_LABELLING;
 	wdsleep(du->dk_ctrlr, "wdopn2");
@@ -1143,8 +1205,8 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 #else
 	if ((du->dk_flags & DKFL_BSDLABEL) == 0) {
 		/*
-		 * wdtab[ctrlr].b_active != 0 implies
-		 * wdutab[lunit].b_actf == NULL (?)
+		 * wdtab[ctrlr].b_active != 0 implies  XXX applicable now ??
+		 * drive_queue[lunit].b_act == NULL (?)  XXX applicable now ??
 		 * so the following guards most things (until the next i/o).
 		 * It doesn't guard against a new i/o starting and being
 		 * affected by the label being changed.  Sigh.
@@ -1153,7 +1215,6 @@ wdopen(dev_t dev, int flags, int fmt, struct proc *p)
 
 		du->dk_flags |= DKFL_LABELLING;
 		du->dk_state = WANTOPEN;
-		wdutab[lunit].b_actf = NULL;
 
 		error = dsinit(dkmodpart(dev, RAW_PART), wdstrategy,
 			       &du->dk_dd, &du->dk_slices);
@@ -1280,7 +1341,11 @@ wdcontrol(register struct buf *bp)
 	int	ctrlr;
 
 	du = wddrives[dkunit(bp->b_dev)];
+#ifdef CMD640
+	ctrlr = du->dk_ctrlr_cmd640;
+#else
 	ctrlr = du->dk_ctrlr;
+#endif
 
 	switch (du->dk_state) {
 	case WANTOPEN:
@@ -1329,9 +1394,41 @@ wdcommand(struct disk *du, u_int cylinder, u_int head, u_int sector,
 	u_int	wdc;
 
 	wdc = du->dk_port;
-	if (du->cfg_flags & WDOPT_SLEEPHACK)
-		if(inb(wdc + wd_status) == WDCS_BUSY)
+	if (du->cfg_flags & WDOPT_SLEEPHACK) {
+		/* OK, so the APM bios has put the disk into SLEEP mode,
+		 * how can we tell ?  Uhm, we can't.  There is no 
+		 * standardized way of finding out, and the only way to
+		 * wake it up is to reset it.  Bummer.
+		 *
+		 * All the many and varied versions of the IDE/ATA standard
+		 * explicitly tells us not to look at these registers if
+		 * the disk is in SLEEP mode.  Well, too bad really, we
+		 * have to find out if it's in sleep mode before we can 
+		 * avoid reading the registers.
+		 *
+		 * I have reason to belive that most disks will return
+		 * either 0xff or 0x00 in all but the status register 
+		 * when in SLEEP mode, but I have yet to see one return 
+		 * 0x00, so we don't check for that yet.
+		 *
+		 * The check for WDCS_BUSY is for the case where the
+		 * bios spins up the disk for us, but doesn't initialize
+		 * it correctly					/phk
+		 */
+		if(inb(wdc + wd_precomp) + inb(wdc + wd_cyl_lo) +
+		    inb(wdc + wd_cyl_hi) + inb(wdc + wd_sdh) +
+		    inb(wdc + wd_sector) + inb(wdc + wd_seccnt) == 6 * 0xff) {
+			if (bootverbose)
+				printf("wd(%d,%d): disk aSLEEP\n",
+					du->dk_ctrlr, du->dk_unit);
 			wdunwedge(du);
+		} else if(inb(wdc + wd_status) == WDCS_BUSY) {
+			if (bootverbose)
+				printf("wd(%d,%d): disk is BUSY\n",
+					du->dk_ctrlr, du->dk_unit);
+			wdunwedge(du);
+		}
+	}
 
 	if (wdwait(du, 0, TIMEOUT) < 0)
 		return (1);
@@ -1352,6 +1449,30 @@ wdcommand(struct disk *du, u_int cylinder, u_int head, u_int sector,
 	return (0);
 }
 
+static void
+wdsetmulti(struct disk *du)
+{
+	/*
+	 * The config option flags low 8 bits define the maximum multi-block
+	 * transfer size.  If the user wants the maximum that the drive
+	 * is capable of, just set the low bits of the config option to
+	 * 0x00ff.
+	 */
+	if ((du->cfg_flags & WDOPT_MULTIMASK) != 0 && (du->dk_multi > 1)) {
+		int configval = du->cfg_flags & WDOPT_MULTIMASK;
+		du->dk_multi = min(du->dk_multi, configval);
+		if (wdcommand(du, 0, 0, 0, du->dk_multi, WDCC_SET_MULTI)) {
+			du->dk_multi = 1;
+		} else {
+		    	if (wdwait(du, WDCS_READY, TIMEOUT) < 0) {
+				du->dk_multi = 1;
+			}
+		}
+	} else {
+		du->dk_multi = 1;
+	}
+}
+
 /*
  * issue IDC to drive to tell it just what geometry it is to be.
  */
@@ -1389,7 +1510,11 @@ wdsetctlr(struct disk *du)
 		error = 1;
 	}
 	if (error) {
+#ifdef CMD640 
+		wdtab[du->dk_ctrlr_cmd640].b_errcnt += RETRIES;
+#else
 		wdtab[du->dk_ctrlr].b_errcnt += RETRIES;
+#endif
 		return (1);
 	}
 	if (wdcommand(du, du->dk_dd.d_ncylinders, du->dk_dd.d_ntracks - 1, 0,
@@ -1398,9 +1523,22 @@ wdsetctlr(struct disk *du)
 		wderror((struct buf *)NULL, du, "wdsetctlr failed");
 		return (1);
 	}
+
+	wdsetmulti(du);
+
+#ifdef NOTYET
+/* set read caching and write caching */
+	wdcommand(du, 0, 0, 0, WDFEA_RCACHE, WDCC_FEATURES);
+	wdwait(du, WDCS_READY, TIMEOUT);
+
+	wdcommand(du, 0, 0, 0, WDFEA_WCACHE, WDCC_FEATURES);
+	wdwait(du, WDCS_READY, TIMEOUT);
+#endif
+
 	return (0);
 }
 
+#if 0
 /*
  * Wait until driver is inactive, then set up controller.
  */
@@ -1417,6 +1555,7 @@ wdwsetctlr(struct disk *du)
 	splx(x);
 	return (stat);
 }
+#endif
 
 /*
  * issue READP to drive to ask it what it is.
@@ -1449,6 +1588,10 @@ again:
 			return (1);
 		outb(du->dk_port + wd_sdh, WDSD_IBM | (du->dk_unit << 4));
 		DELAY(5000);	/* usually unnecessary; drive select is fast */
+		/*
+		 * Do this twice: may get a false WDCS_READY the first time.
+		 */
+		inb(du->dk_port + wd_status);
 		if ((inb(du->dk_port + wd_status) & (WDCS_BUSY | WDCS_READY))
 		    != WDCS_READY
 		    || wdcommand(du, 0, 0, 0, 0, WDCC_RESTORE | WD_STEP) != 0
@@ -1563,22 +1706,41 @@ failed:
 		wp->wdp_model[i] = '\0';
 	}
 
+	/*
+	 * find out the drives maximum multi-block transfer capability
+	 */
+	du->dk_multi = wp->wdp_nsecperint & 0xff;
+	wdsetmulti(du);
+
 #ifdef WDDEBUG
 	printf(
 "\nwd(%d,%d): wdgetctlr: gc %x cyl %d trk %d sec %d type %d sz %d model %s\n",
-	       du->dk_ctrlr, du->dk_unit, wp->wdp_config,
-	       wp->wdp_fixedcyl + wp->wdp_removcyl, wp->wdp_heads,
-	       wp->wdp_sectors, wp->wdp_cntype, wp->wdp_cnsbsz,
-	       wp->wdp_model);
+	       du->dk_ctrlr, du->dk_unit, wp->wdp_config, wp->wdp_cylinders,
+	       wp->wdp_heads, wp->wdp_sectors, wp->wdp_buffertype,
+	       wp->wdp_buffersize, wp->wdp_model);
 #endif
 
 	/* update disklabel given drive information */
 	du->dk_dd.d_secsize = DEV_BSIZE;
-	du->dk_dd.d_ncylinders = wp->wdp_fixedcyl + wp->wdp_removcyl /*+- 1*/ ;
+	du->dk_dd.d_ncylinders = wp->wdp_cylinders;	/* +- 1 */
 	du->dk_dd.d_ntracks = wp->wdp_heads;
 	du->dk_dd.d_nsectors = wp->wdp_sectors;
 	du->dk_dd.d_secpercyl = du->dk_dd.d_ntracks * du->dk_dd.d_nsectors;
 	du->dk_dd.d_secperunit = du->dk_dd.d_secpercyl * du->dk_dd.d_ncylinders;
+	if (wp->wdp_cylinders == 16383 &&
+            du->dk_dd.d_secperunit < wp->wdp_lbasize) { 
+        	du->dk_dd.d_secperunit = wp->wdp_lbasize;
+        	du->dk_dd.d_ncylinders = 
+			du->dk_dd.d_secperunit / du->dk_dd.d_secpercyl; 
+        }       
+
+	if (WDOPT_FORCEHD(du->cfg_flags)) {
+		du->dk_dd.d_ntracks = WDOPT_FORCEHD(du->cfg_flags);
+		du->dk_dd.d_secpercyl = 
+		    du->dk_dd.d_ntracks * du->dk_dd.d_nsectors;
+		du->dk_dd.d_ncylinders =
+		    du->dk_dd.d_secperunit / du->dk_dd.d_secpercyl;
+	}
 #if 0
 	du->dk_dd.d_partitions[RAW_PART].p_size = du->dk_dd.d_secperunit;
 	/* dubious ... */
@@ -1589,37 +1751,9 @@ failed:
 	du->dk_dd.d_subtype |= DSTYPE_GEOMETRY;
 #endif
 
-	/*
-	 * find out the drives maximum multi-block transfer capability
-	 */
-	du->dk_multi = wp->wdp_nsecperint & 0xff;
-
-	/*
-	 * The config option flags low 8 bits define the maximum multi-block
-	 * transfer size.  If the user wants the maximum that the drive
-	 * is capable of, just set the low bits of the config option to
-	 * 0x00ff.
-	 */
-	if ((flags & WDOPT_MULTIMASK) != 0 && (du->dk_multi > 1)) {
-		if (du->dk_multi > (flags & WDOPT_MULTIMASK))
-			du->dk_multi = flags & WDOPT_MULTIMASK;
-		if (wdcommand(du, 0, 0, 0, du->dk_multi, WDCC_SET_MULTI)) {
-			du->dk_multi = 1;
-		}
-	} else {
-		du->dk_multi = 1;
-	}
-
-#ifdef NOTYET
-/* set read caching and write caching */
-	wdcommand(du, 0, 0, 0, WDFEA_RCACHE, WDCC_FEATURES);
-	wdcommand(du, 0, 0, 0, WDFEA_WCACHE, WDCC_FEATURES);
-#endif
-
 	return (0);
 }
 
-/* ARGSUSED */
 int
 wdclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
@@ -1728,7 +1862,7 @@ wddump(dev_t dev)
 	static int wddoingadump = 0;
 
 	/* Toss any characters present prior to dump. */
-	while (cncheckc())
+	while (cncheckc() != -1)
 		;
 
 	/* Check for acceptable device. */
@@ -1741,7 +1875,7 @@ wddump(dev_t dev)
 		return (ENXIO);
 
 	/* Size of memory to dump, in disk sectors. */
-	num = (u_long)Maxmem * NBPG / du->dk_dd.d_secsize;
+	num = (u_long)Maxmem * PAGE_SIZE / du->dk_dd.d_secsize;
 
 	secpertrk = du->dk_dd.d_nsectors;
 	secpercyl = du->dk_dd.d_secpercyl;
@@ -1842,8 +1976,12 @@ out:
 			return (EIO);
 		}
 		while (blkcnt != 0) {
-			pmap_enter(kernel_pmap, (vm_offset_t)CADDR1, trunc_page(addr),
-				   VM_PROT_READ, TRUE);
+			if (is_physical_memory((vm_offset_t)addr))
+				pmap_enter(kernel_pmap, (vm_offset_t)CADDR1,
+					   trunc_page(addr), VM_PROT_READ, TRUE);
+			else
+				pmap_enter(kernel_pmap, (vm_offset_t)CADDR1,
+					   trunc_page(0), VM_PROT_READ, TRUE);
 
 			/* Ready to send data? */
 			DELAY(5);	/* ATA spec */
@@ -1855,15 +1993,24 @@ out:
 			}
 			if (du->dk_flags & DKFL_32BIT)
 				outsl(du->dk_port + wd_data,
-				      CADDR1 + ((int)addr & (NBPG - 1)),
+				      CADDR1 + ((int)addr & PAGE_MASK),
 				      DEV_BSIZE / sizeof(long));
 			else
 				outsw(du->dk_port + wd_data,
-				      CADDR1 + ((int)addr & (NBPG - 1)),
+				      CADDR1 + ((int)addr & PAGE_MASK),
 				      DEV_BSIZE / sizeof(short));
 			addr += DEV_BSIZE;
-			if ((unsigned)addr % (1024 * 1024) == 0)
+			/*
+			 * If we are dumping core, it may take a while.
+			 * So reassure the user and hold off any watchdogs.
+			 */
+			if ((unsigned)addr % (1024 * 1024) == 0) {
+#ifdef	HW_WDOG
+				if (wdog_tickler)
+					(*wdog_tickler)();
+#endif /* HW_WDOG */
 				printf("%ld ", num / (1024 * 1024 / DEV_BSIZE));
+			}
 			num--;
 			blkcnt--;
 		}
@@ -1889,7 +2036,7 @@ out:
 		blknum = blknext;
 
 		/* Operator aborting dump? */
-		if (cncheckc())
+		if (cncheckc() != -1)
 			return (EINTR);
 	}
 	return (0);
@@ -1914,10 +2061,17 @@ wderror(struct buf *bp, struct disk *du, char *mesg)
 static void
 wdflushirq(struct disk *du, int old_ipl)
 {
+#ifdef CMD640
+	wdtab[du->dk_ctrlr_cmd640].b_active = 2;
+	splx(old_ipl);
+	(void)splbio();
+	wdtab[du->dk_ctrlr_cmd640].b_active = 0;
+#else
 	wdtab[du->dk_ctrlr].b_active = 2;
 	splx(old_ipl);
 	(void)splbio();
 	wdtab[du->dk_ctrlr].b_active = 0;
+#endif  
 }
 
 /*
@@ -1956,8 +2110,14 @@ wdreset(struct disk *du)
 static void
 wdsleep(int ctrlr, char *wmesg)
 {
+	int s = splbio();
+#ifdef CMD640
+	if (eide_quirks & Q_CMD640B)
+		ctrlr = PRIMARY;
+#endif
 	while (wdtab[ctrlr].b_active)
 		tsleep((caddr_t)&wdtab[ctrlr].b_active, PZERO - 1, wmesg, 1);
+	splx(s);
 }
 
 static void
@@ -2100,4 +2260,18 @@ wdwait(struct disk *du, u_char bits_wanted, int timeout)
 	return (-1);
 }
 
+static wd_devsw_installed = 0;
+
+static void 	wd_drvinit(void *unused)
+{
+
+	if( ! wd_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &wd_bdevsw);
+		wd_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(wddev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,wd_drvinit,NULL)
+
+
 #endif /* NWDC > 0 */
