@@ -57,31 +57,21 @@
 #include <sys/ioctl.h>
 #include <sys/buf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/dkbad.h>
 #include <sys/disklabel.h>
 
 #include <scsi/scsi_all.h>
 #include <scsi/scsiconf.h>
-#include <sys/devconf.h>
 
 #include <machine/clock.h>
-#include <machine/cpu.h>
+
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
 
 #include <i386/isa/isa_device.h>
 
-static struct kern_devconf kdc_wds[NWDS] = { {
-  0, 0, 0,
-  "wds", 0, {MDDT_ISA, 0, "bio"},
-  isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-  &kdc_isa0,
-  0,
-  DC_UNCONFIGURED,		/* state */
-  "Western Digital WD-7000 SCSI host adapter",
-  DC_CLS_MISC			/* class */
-} };
-
-struct scsi_device wds_dev =
+static struct scsi_device wds_dev =
 {
  	NULL,
 	NULL,
@@ -211,29 +201,28 @@ struct wds_setup {
 #define WDS_NIMB	8
 #define MAXSIMUL	8
 
-int wdsunit=0;
+static int wdsunit=0;
 
-u_char wds_data[NWDS][BUFSIZ];
-u_char wds_data_in_use[NWDS];
+static u_char wds_data[NWDS][BUFSIZ];
+static u_char wds_data_in_use[NWDS];
 
-struct wds {
+static struct wds {
   int addr;
   struct wds_req wdsr[MAXSIMUL];
   struct wds_mb ombs[WDS_NOMB], imbs[WDS_NIMB];
   struct scsi_link sc_link;
 } wds[NWDS];
 
-int wdsprobe(struct isa_device *);
-void wds_minphys(struct buf *);
-struct wds_req *wdsr_alloc(int);
-int32_t wds_scsi_cmd(struct scsi_xfer *);
-u_int32_t wds_adapter_info(int);
-int wdsintr(int);
-int wds_done(int, struct wds_cmd *, u_char);
-int wdsattach(struct isa_device *);
-int wds_init(struct isa_device *);
-int wds_cmd(int, u_char *, int);
-void wds_wait(int, int, int);
+static int wdsprobe(struct isa_device *);
+static void wds_minphys(struct buf *);
+static struct wds_req *wdsr_alloc(int);
+static int32_t wds_scsi_cmd(struct scsi_xfer *);
+static u_int32_t wds_adapter_info(int);
+static int wds_done(int, struct wds_cmd *, u_char);
+static int wdsattach(struct isa_device *);
+static int wds_init(struct isa_device *);
+static int wds_cmd(int, u_char *, int);
+static void wds_wait(int, int, int);
 
 struct isa_driver wdsdriver =
 {
@@ -242,7 +231,7 @@ struct isa_driver wdsdriver =
   "wds"
 };
 
-struct scsi_adapter wds_switch =
+static struct scsi_adapter wds_switch =
 {
 	wds_scsi_cmd,
 	wds_minphys,
@@ -262,12 +251,6 @@ wdsprobe(struct isa_device *dev)
   dev->id_unit = wdsunit;	/* XXX WRONG! */
   wds[wdsunit].addr = dev->id_iobase;
 
-  if(dev->id_unit)
-    kdc_wds[dev->id_unit] = kdc_wds[0];
-  kdc_wds[dev->id_unit].kdc_unit = dev->id_unit;
-  kdc_wds[dev->id_unit].kdc_parentdata = dev;
-  dev_attach(&kdc_wds[dev->id_unit]);
-
   if(wds_init(dev) != 0)
     return 0;
   wdsunit++;
@@ -327,7 +310,7 @@ wds_scsi_cmd(struct scsi_xfer *sxp)
   struct wds_req *r;
   int unit = sxp->sc_link->adapter_unit;
   int base;
-  u_char c, *p;
+  u_char c;
   int i;
 
   base = wds[unit].addr;
@@ -445,7 +428,7 @@ wds_adapter_info(int unit)
   return 1;
 }
 
-int
+void
 wdsintr(int unit)
 {
   struct wds_cmd *pc, *vc;
@@ -456,7 +439,7 @@ wdsintr(int unit)
   if(!inb(wds[unit].addr+WDS_STAT) & WDS_IRQ)
   {
     outb(wds[unit].addr + WDS_IRQACK, 0);
-    return 1;
+    return;
   }
 
   c = inb(wds[unit].addr + WDS_IRQSTAT);
@@ -474,7 +457,6 @@ wdsintr(int unit)
 
     outb(wds[unit].addr + WDS_IRQACK, 0);
   }
-  return 1;
 }
 
 int
@@ -482,7 +464,6 @@ wds_done(int unit, struct wds_cmd *c, u_char stat)
 {
   struct wds_req *r;
   int i;
-  char slask[80];
 
   r = (struct wds_req *)NULL;
 
@@ -551,12 +532,12 @@ wds_done(int unit, struct wds_cmd *c, u_char stat)
   return 0;
 }
 
-int
+static int
 wds_getvers(int unit)
 {
   struct wds_req *r;
   int base;
-  u_char c, *p;
+  u_char c;
   int i;
 
   base = wds[unit].addr;
@@ -612,9 +593,7 @@ int
 wdsattach(struct isa_device *dev)
 {
   int masunit;
-  static int firstswitch[NWDS];
   static u_long versprobe=0;	/* max 32 controllers */
-  int r;
   int unit = dev->id_unit;
   struct scsibus_data *scbus;
 
@@ -644,8 +623,6 @@ wdsattach(struct isa_device *dev)
     return 0;
   scbus->adapter_link = &wds[unit].sc_link;
 
-  kdc_wds[unit].kdc_state = DC_BUSY;
-
   scsi_attachdevs(scbus);
 
   return 1;
@@ -656,7 +633,6 @@ wds_init(struct isa_device *dev)
 {
   struct wds_setup init;
   int base;
-  u_char *p, c;
   int unit, i;
   struct wds_cmd wc;
 
@@ -730,7 +706,6 @@ int
 wds_cmd(int base, u_char *p, int l)
 {
   int s=splbio();
-  u_char c;
 
   while(l--)
   {
