@@ -1,3 +1,4 @@
+
 /*
  * Streamer tape driver for 386bsd and FreeBSD.
  * Supports Archive and Wangtek compatible QIC-02/QIC-36 boards.
@@ -19,7 +20,7 @@
  * the original CMU copyright notice.
  *
  * Version 1.3, Thu Nov 11 12:09:13 MSK 1993
- * $FreeBSD: stable/2.1/sys/i386/isa/wt.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/wt.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -68,18 +69,23 @@
 #include <sys/ioctl.h>
 #include <sys/mtio.h>
 #include <sys/proc.h>
-#include <sys/devconf.h>
+#include <sys/conf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 #include <vm/vm_param.h>
 
 #include <machine/clock.h>
+#include <machine/wtio.h>
 
 #include <i386/isa/isa_device.h>
 #include <i386/isa/wtreg.h>
 
+
 /*
  * Uncomment this to enable internal device tracing.
  */
-#define DEBUG(s)                /* printf s */
+#define TRACE(s)                /* printf s */
 
 #define WTPRI                   (PZERO+10)      /* sleep priority */
 
@@ -158,9 +164,13 @@ typedef struct {
 	unsigned short DATAPORT, CMDPORT, STATPORT, CTLPORT, SDMAPORT, RDMAPORT;
 	unsigned char BUSY, NOEXCEP, RESETMASK, RESETVAL;
 	unsigned char ONLINE, RESET, REQUEST, IEN;
+#ifdef	DEVFS
+	void	*devfs_token;
+	void	*devfs_token_r;
+#endif
 } wtinfo_t;
 
-wtinfo_t wttab[NWT];                    /* tape info by unit number */
+static wtinfo_t wttab[NWT];                    /* tape info by unit number */
 
 static int wtwait (wtinfo_t *t, int catch, char *msg);
 static int wtcmd (wtinfo_t *t, int cmd);
@@ -176,37 +186,30 @@ static int wtreadfm (wtinfo_t *t);
 static int wtwritefm (wtinfo_t *t);
 static int wtpoll (wtinfo_t *t, int mask, int bits);
 
-static struct kern_devconf kdc_wt[NWT] = { {
-	0, 0, 0,		/* filled in by dev_attach */
-	"wt", 0, { MDDT_ISA, 0, "bio" },
-	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-	&kdc_isa0,		/* parent */
-	0,			/* parentdata */
-	DC_UNCONFIGURED,	/* state */
-	"Archive or Wangtek QIC-02/QIC-36 tape controller",
-	DC_CLS_TAPE		/* class */
-} };
+static	d_open_t	wtopen;
+static	d_close_t	wtclose;
+static	d_ioctl_t	wtioctl;
+static	d_dump_t	wtdump;
+static	d_psize_t	wtsize;
+static	d_strategy_t	wtstrategy;
 
-static inline void
-wt_registerdev(struct isa_device *id)
-{
-	if(id->id_unit)
-		kdc_wt[id->id_unit] = kdc_wt[0];
-	kdc_wt[id->id_unit].kdc_unit = id->id_unit;
-	kdc_wt[id->id_unit].kdc_parentdata = id;
-	dev_attach(&kdc_wt[id->id_unit]);
-}
+#define CDEV_MAJOR 10
+#define BDEV_MAJOR 3
+
+static struct cdevsw wt_cdevsw;
+static struct bdevsw wt_bdevsw = 
+	{ wtopen,	wtclose,	wtstrategy,	wtioctl,	/*3*/
+	  wtdump,	wtsize,		B_TAPE,	"wt",	&wt_cdevsw,	-1 };
 
 
 /*
  * Probe for the presence of the device.
  */
-int wtprobe (struct isa_device *id)
+static int 
+wtprobe (struct isa_device *id)
 {
 	wtinfo_t *t = wttab + id->id_unit;
 
-	wt_registerdev(id);
-
 	t->unit = id->id_unit;
 	t->chan = id->id_drq;
 	t->port = id->id_iobase;
@@ -247,7 +250,8 @@ int wtprobe (struct isa_device *id)
 /*
  * Device is found, configure it.
  */
-int wtattach (struct isa_device *id)
+static int
+wtattach (struct isa_device *id)
 {
 	wtinfo_t *t = wttab + id->id_unit;
 
@@ -258,19 +262,30 @@ int wtattach (struct isa_device *id)
 		printf ("wt%d: type <Wangtek>\n", t->unit);
 	t->flags = TPSTART;                     /* tape is rewound */
 	t->dens = -1;                           /* unknown density */
-	kdc_wt[id->id_unit].kdc_state = DC_IDLE;
+	isa_dmainit(t->chan, 1024);
+
+#ifdef DEVFS
+	t->devfs_token_r = 
+		devfs_add_devswf(&wt_cdevsw, id->id_unit, DV_CHR, 0, 0, 
+				 0600, "rwt%d", id->id_unit);
+	t->devfs_token = 
+		devfs_add_devswf(&wt_bdevsw, id->id_unit, DV_BLK, 0, 0, 
+				 0600, "wt%d", id->id_unit);
+#endif
 	return (1);
 }
 
 struct isa_driver wtdriver = { wtprobe, wtattach, "wt", };
 
-int wtdump (dev_t dev)
+int
+wtdump (dev_t dev)
 {
 	/* Not implemented */
 	return (EINVAL);
 }
 
-int wtsize (dev_t dev)
+int
+wtsize (dev_t dev)
 {
 	/* Not implemented */
 	return (-1);
@@ -279,7 +294,8 @@ int wtsize (dev_t dev)
 /*
  * Open routine, called on every device open.
  */
-int wtopen (dev_t dev, int flag, int fmt, struct proc *p)
+int
+wtopen (dev_t dev, int flag, int fmt, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -344,8 +360,10 @@ int wtopen (dev_t dev, int flag, int fmt, struct proc *p)
 	if (! t->buf)
 		return (EAGAIN);
 
+	if (isa_dma_acquire(t->chan))
+		return(EBUSY);
+
 	t->flags = TPINUSE;
-	kdc_wt[u].kdc_state = DC_BUSY;
 
 	if (flag & FREAD)
 		t->flags |= TPREAD;
@@ -357,7 +375,8 @@ int wtopen (dev_t dev, int flag, int fmt, struct proc *p)
 /*
  * Close routine, called on last device close.
  */
-int wtclose (dev_t dev, int flags, int fmt, struct proc *p)
+int
+wtclose (dev_t dev, int flags, int fmt, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -394,8 +413,8 @@ int wtclose (dev_t dev, int flags, int fmt, struct proc *p)
 		wtreadfm (t);
 done:
 	t->flags &= TPREW | TPRMARK | TPSTART | TPTIMER;
-	kdc_wt[u].kdc_state = DC_IDLE;
 	free (t->buf, M_TEMP);
+	isa_dma_release(t->chan);
 	return (0);
 }
 
@@ -405,7 +424,8 @@ done:
  * ioctl (int fd, MTIOCGET, struct mtget *buf)  -- get status
  * ioctl (int fd, MTIOCTOP, struct mtop *buf)   -- do BSD-like op
  */
-int wtioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
+int
+wtioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -495,20 +515,28 @@ int wtioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
 /*
  * Strategy routine.
  */
-void wtstrategy (struct buf *bp)
+void
+wtstrategy (struct buf *bp)
 {
 	int u = minor (bp->b_dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
 	int s;
 
 	bp->b_resid = bp->b_bcount;
-	if (u >= NWT || t->type == UNKNOWN)
-		goto errxit;
+	if (u >= NWT || t->type == UNKNOWN) {
+		bp->b_error = ENXIO;
+		goto err2xit;
+	}
 
 	/* at file marks and end of tape, we just return '0 bytes available' */
 	if (t->flags & TPVOL)
 		goto xit;
 
+	if (bp->b_bcount % t->bsize != 0) {
+		bp->b_error = EINVAL;
+		goto err2xit;
+	}
+
 	if (bp->b_flags & B_READ) {
 		/* Check read access and no previous write to this tape. */
 		if (! (t->flags & TPREAD) || (t->flags & TPWANY))
@@ -555,8 +583,8 @@ void wtstrategy (struct buf *bp)
 	splx (s);
 
 	if (t->flags & TPEXCEP) {
-errxit:         bp->b_flags |= B_ERROR;
-		bp->b_error = EIO;
+errxit:		bp->b_error = EIO;
+err2xit:	bp->b_flags |= B_ERROR;
 	}
 xit:    biodone (bp);
 	return;
@@ -565,20 +593,21 @@ xit:    biodone (bp);
 /*
  * Interrupt routine.
  */
-void wtintr (int u)
+void
+wtintr (int u)
 {
 	wtinfo_t *t = wttab + u;
 	unsigned char s;
 
 	if (u >= NWT || t->type == UNKNOWN) {
-		DEBUG (("wtintr() -- device not configured\n"));
+		TRACE (("wtintr() -- device not configured\n"));
 		return;
 	}
 
 	s = inb (t->STATPORT);                  /* get status */
-	DEBUG (("wtintr() status=0x%x -- ", s));
+	TRACE (("wtintr() status=0x%x -- ", s));
 	if ((s & (t->BUSY | t->NOEXCEP)) == (t->BUSY | t->NOEXCEP)) {
-		DEBUG (("busy\n"));
+		TRACE (("busy\n"));
 		return;                         /* device is busy */
 	}
 
@@ -586,7 +615,7 @@ void wtintr (int u)
 	 * Check if rewind finished.
 	 */
 	if (t->flags & TPREW) {
-		DEBUG (((s & (t->BUSY | t->NOEXCEP)) == (t->BUSY | t->NOEXCEP) ?
+		TRACE (((s & (t->BUSY | t->NOEXCEP)) == (t->BUSY | t->NOEXCEP) ?
 			"rewind busy?\n" : "rewind finished\n"));
 		t->flags &= ~TPREW;             /* Rewind finished. */
 		wtsense (t, 1, TP_WRP);
@@ -598,7 +627,7 @@ void wtintr (int u)
 	 * Check if writing/reading of file mark finished.
 	 */
 	if (t->flags & (TPRMARK | TPWMARK)) {
-		DEBUG (((s & (t->BUSY | t->NOEXCEP)) == (t->BUSY | t->NOEXCEP) ?
+		TRACE (((s & (t->BUSY | t->NOEXCEP)) == (t->BUSY | t->NOEXCEP) ?
 			"marker r/w busy?\n" : "marker r/w finished\n"));
 		if (! (s & t->NOEXCEP))         /* operation failed */
 			wtsense (t, 1, (t->flags & TPRMARK) ? TP_WRP : 0);
@@ -611,11 +640,9 @@ void wtintr (int u)
 	 * Do we started any i/o?  If no, just return.
 	 */
 	if (! (t->flags & TPACTIVE)) {
-		DEBUG (("unexpected interrupt\n"));
+		TRACE (("unexpected interrupt\n"));
 		return;
 	}
-	t->flags &= ~TPACTIVE;
-	t->dmacount += t->bsize;                /* increment counter */
 
 	/*
 	 * Clean up dma.
@@ -628,11 +655,15 @@ void wtintr (int u)
 	} else
 		isa_dmadone (t->dmaflags, t->dmavaddr, t->bsize, t->chan);
 
+	t->flags &= ~TPACTIVE;
+	t->dmacount += t->bsize;
+	t->dmavaddr = (char *)t->dmavaddr + t->bsize;
+
 	/*
 	 * On exception, check for end of file and end of volume.
 	 */
 	if (! (s & t->NOEXCEP)) {
-		DEBUG (("i/o exception\n"));
+		TRACE (("i/o exception\n"));
 		wtsense (t, 1, (t->dmaflags & B_READ) ? TP_WRP : 0);
 		if (t->error.err & (TP_EOM | TP_FIL))
 			t->flags |= TPVOL;      /* end of file */
@@ -643,19 +674,19 @@ void wtintr (int u)
 	}
 
 	if (t->dmacount < t->dmatotal) {        /* continue i/o */
-		t->dmavaddr += t->bsize;
 		wtdma (t);
-		DEBUG (("continue i/o, %d\n", t->dmacount));
+		TRACE (("continue i/o, %d\n", t->dmacount));
 		return;
 	}
 	if (t->dmacount > t->dmatotal)          /* short last block */
 		t->dmacount = t->dmatotal;
 	wakeup ((caddr_t)t);	/* wake up user level */
-	DEBUG (("i/o finished, %d\n", t->dmacount));
+	TRACE (("i/o finished, %d\n", t->dmacount));
 }
 
 /* start the rewind operation */
-static void wtrewind (wtinfo_t *t)
+static void
+wtrewind (wtinfo_t *t)
 {
 	int rwmode = (t->flags & (TPRO | TPWO));
 
@@ -675,7 +706,8 @@ static void wtrewind (wtinfo_t *t)
 }
 
 /* start the `read marker' operation */
-static int wtreadfm (wtinfo_t *t)
+static int
+wtreadfm (wtinfo_t *t)
 {
 	t->flags &= ~(TPRO | TPWO | TPVOL);
 	if (! wtcmd (t, QIC_READFM)) {
@@ -689,7 +721,8 @@ static int wtreadfm (wtinfo_t *t)
 }
 
 /* write marker to the tape */
-static int wtwritefm (wtinfo_t *t)
+static int
+wtwritefm (wtinfo_t *t)
 {
 	tsleep ((caddr_t)wtwritefm, WTPRI, "wtwfm", hz); /* timeout: 1 second */
 	t->flags &= ~(TPRO | TPWO);
@@ -703,7 +736,8 @@ static int wtwritefm (wtinfo_t *t)
 }
 
 /* while controller status & mask == bits continue waiting */
-static int wtpoll (wtinfo_t *t, int mask, int bits)
+static int
+wtpoll (wtinfo_t *t, int mask, int bits)
 {
 	int s, i;
 
@@ -729,11 +763,12 @@ static int wtpoll (wtinfo_t *t, int mask, int bits)
 }
 
 /* execute QIC command */
-static int wtcmd (wtinfo_t *t, int cmd)
+static int
+wtcmd (wtinfo_t *t, int cmd)
 {
 	int s, x;
 
-	DEBUG (("wtcmd() cmd=0x%x\n", cmd));
+	TRACE (("wtcmd() cmd=0x%x\n", cmd));
 	x = splbio();
 	s = wtpoll (t, t->BUSY | t->NOEXCEP, t->BUSY | t->NOEXCEP); /* ready? */
 	if (! (s & t->NOEXCEP)) {                       /* error */
@@ -752,11 +787,12 @@ static int wtcmd (wtinfo_t *t, int cmd)
 }
 
 /* wait for the end of i/o, seeking marker or rewind operation */
-static int wtwait (wtinfo_t *t, int catch, char *msg)
+static int
+wtwait (wtinfo_t *t, int catch, char *msg)
 {
 	int error;
 
-	DEBUG (("wtwait() `%s'\n", msg));
+	TRACE (("wtwait() `%s'\n", msg));
 	while (t->flags & (TPACTIVE | TPREW | TPRMARK | TPWMARK))
 		if (error = tsleep ((caddr_t)t, WTPRI | catch, msg, 0))
 			return (error);
@@ -764,7 +800,8 @@ static int wtwait (wtinfo_t *t, int catch, char *msg)
 }
 
 /* initialize dma for the i/o operation */
-static void wtdma (wtinfo_t *t)
+static void
+wtdma (wtinfo_t *t)
 {
 	t->flags |= TPACTIVE;
 	wtclock (t);
@@ -780,11 +817,12 @@ static void wtdma (wtinfo_t *t)
 }
 
 /* start i/o operation */
-static int wtstart (wtinfo_t *t, unsigned flags, void *vaddr, unsigned len)
+static int
+wtstart (wtinfo_t *t, unsigned flags, void *vaddr, unsigned len)
 {
 	int s, x;
 
-	DEBUG (("wtstart()\n"));
+	TRACE (("wtstart()\n"));
 	x = splbio();
 	s = wtpoll (t, t->BUSY | t->NOEXCEP, t->BUSY | t->NOEXCEP); /* ready? */
 	if (! (s & t->NOEXCEP)) {
@@ -803,7 +841,8 @@ static int wtstart (wtinfo_t *t, unsigned flags, void *vaddr, unsigned len)
 }
 
 /* start timer */
-static void wtclock (wtinfo_t *t)
+static void
+wtclock (wtinfo_t *t)
 {
 	if (! (t->flags & TPTIMER)) {
 		t->flags |= TPTIMER;
@@ -818,7 +857,8 @@ static void wtclock (wtinfo_t *t)
  * This is necessary in case interrupts get eaten due to
  * multiple devices on a single IRQ line.
  */
-static void wtimer (void *xt)
+static void
+wtimer (void *xt)
 {
 	wtinfo_t *t = (wtinfo_t *)xt;
 	int s;
@@ -830,7 +870,7 @@ static void wtimer (void *xt)
 	/* If i/o going, simulate interrupt. */
 	s = splbio ();
 	if ((inb (t->STATPORT) & (t->BUSY | t->NOEXCEP)) != (t->BUSY | t->NOEXCEP)) {
-		DEBUG (("wtimer() -- "));
+		TRACE (("wtimer() -- "));
 		wtintr (t->unit);
 	}
 	splx (s);
@@ -841,7 +881,8 @@ static void wtimer (void *xt)
 }
 
 /* reset the controller */
-static int wtreset (wtinfo_t *t)
+static int
+wtreset (wtinfo_t *t)
 {
 	/* Perform QIC-02 and QIC-36 compatible reset sequence. */
 	/* Thanks to Mikael Hybsch <micke@dynas.se>. */
@@ -869,12 +910,13 @@ static int wtreset (wtinfo_t *t)
 
 /* get controller status information */
 /* return 0 if user i/o request should receive an i/o error code */
-static int wtsense (wtinfo_t *t, int verb, int ignor)
+static int
+wtsense (wtinfo_t *t, int verb, int ignor)
 {
 	char *msg = 0;
 	int err;
 
-	DEBUG (("wtsense() ignor=0x%x\n", ignor));
+	TRACE (("wtsense() ignor=0x%x\n", ignor));
 	t->flags &= ~(TPRO | TPWO);
 	if (! wtstatus (t))
 		return (0);
@@ -909,7 +951,8 @@ static int wtsense (wtinfo_t *t, int verb, int ignor)
 }
 
 /* get controller status information */
-static int wtstatus (wtinfo_t *t)
+static int
+wtstatus (wtinfo_t *t)
 {
 	char *p;
 	int x;
@@ -941,4 +984,21 @@ static int wtstatus (wtinfo_t *t)
 	splx(x);
 	return (1);
 }
+
+
+static wt_devsw_installed = 0;
+
+static void 
+wt_drvinit(void *unused)
+{
+
+	if( ! wt_devsw_installed ) {
+		bdevsw_add_generic(BDEV_MAJOR,CDEV_MAJOR, &wt_bdevsw);
+		wt_devsw_installed = 1;
+    	}
+}
+
+SYSINIT(wtdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,wt_drvinit,NULL)
+
+
 #endif /* NWT */
