@@ -9,7 +9,7 @@
  * This software is distributed with NO WARRANTIES, not even the implied
  * warranties for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * Authors grant any other persons or organisations permission to use
+ * Authors grant any other persons or organizations permission to use
  * or modify this software as long as this message is kept with the software,
  * all derivative works or modified versions.
  *
@@ -19,7 +19,7 @@
  * the original CMU copyright notice.
  *
  * Version 1.3, Thu Nov 11 12:09:13 MSK 1993
- * $FreeBSD: stable/2.1/sys/i386/isa/wtreg.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/isa/wtreg.h 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -104,10 +104,6 @@
 #define TP_ST1          0x8000  /* Status byte 1 bits */
 #define TP_ST1MASK      0xff00  /* Status byte 1 mask */
 
-/* formats for printing flags and error values */
-#define WTDS_BITS "\20\1inuse\2read\3write\4start\5rmark\6wmark\7rew\10excep\11vol\12wo\13ro\14wany\15rany\16wp\17timer\20active"
-#define WTER_BITS "\20\1eof\2bnl\3uda\4eom\5wrp\6usl\7cni\11por\12erm\13bpe\14bom\15mbd\16ndt\17ill"
-
 /* device minor number */
 #define WT_BSIZE        0100    /* long block flag */
 #define WT_DENSEL       0070    /* density select mask */
