@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/i386/scsi/aic7xxx.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/scsi/aic7xxx.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _AIC7XXX_H_
@@ -38,6 +38,7 @@
 
 #if defined(__FreeBSD__)
 #include "ahc.h"                /* for NAHC from config */
+#include "opt_aic7xxx.h"	/* for config options */
 #endif
 
 #if defined(__NetBSD__)
@@ -312,8 +313,6 @@ extern int ahc_debug; /* Initialized in i386/scsi/aic7xxx.c */
 #endif
 
 #if defined(__FreeBSD__)
-extern int bootverbose;
-
 char *ahc_name __P((struct ahc_softc *ahc));
 
 struct ahc_softc *ahc_alloc __P((int unit, u_int32_t io_base,
