@@ -12,7 +12,7 @@
  * on the understanding that TFS is not responsible for the correct
  * functioning of this software in any circumstances.
  *
- * $FreeBSD: stable/2.1/sys/i386/scsi/bt.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/i386/scsi/bt.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -37,6 +37,7 @@
 #include <sys/buf.h>
 #include <sys/kernel.h>
 #include <sys/proc.h>
+#include <sys/sysctl.h>
  
 #include <machine/clock.h>
 #include <machine/stdarg.h>
@@ -217,6 +218,7 @@ struct bt_sync_value {
 #define	BT_SHOWCMDS 0x04
 #define	BT_SHOWMISC 0x08
 static int		bt_debug = 0;
+SYSCTL_INT(_debug, OID_AUTO, bt_debug, CTLFLAG_RW, &bt_debug, 0, "");
 
 static u_int32_t bt_adapter_info __P((int unit));
 static struct bt_ccb *
@@ -1304,6 +1306,8 @@ bt_scsi_cmd(xs)
 		seg = 0;
 #ifdef	TFS
 		if (flags & SCSI_DATA_UIO) {
+			struct iovec *iovp;
+
 			iovp = ((struct uio *) xs->data)->uio_iov;
 			datalen = ((struct uio *) xs->data)->uio_iovcnt;
 			xs->datalen = 0;
