@@ -36,9 +36,11 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_bmap.c	8.3 (Berkeley) 1/23/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_bmap.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_bmap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#define CD9660 1  /* bogus dependency in sys/mount.h */
+
 #include <sys/param.h>
 #include <sys/namei.h>
 #include <sys/buf.h>
@@ -62,6 +64,7 @@ cd9660_bmap(ap)
 		struct vnode **a_vpp;
 		daddr_t *a_bnp;
 		int *a_runp;
+		int *a_runb;
 	} */ *ap;
 {
 	struct iso_node *ip = VTOI(ap->a_vp);
@@ -99,5 +102,9 @@ cd9660_bmap(ap)
 			*ap->a_runp = nblk;
 	}
 
+	if (ap->a_runb) {
+		*ap->a_runb = 0;
+	}
+
 	return 0;
 }
