@@ -38,9 +38,11 @@
  *	from: @(#)ufs_lookup.c	7.33 (Berkeley) 5/19/91
  *
  *	@(#)cd9660_lookup.c	8.2 (Berkeley) 1/23/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_lookup.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_lookup.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#define CD9660 1  /* bogus dependency in sys/mount.h */
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/namei.h>
@@ -154,7 +156,7 @@ cd9660_lookup(ap)
 	 * we are looking for is known already.
 	 */
 	if ((error = cache_lookup(vdp, vpp, cnp))) {
-		int vpid;	/* capability number of vnode */
+		u_long vpid;	/* capability number of vnode */
 
 		if (error == ENOENT)
 			return (error);
