@@ -36,9 +36,11 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_node.c	8.2 (Berkeley) 1/23/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_node.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_node.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#define CD9660 1  /* bogus dependency in sys/mount.h */
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/mount.h>
@@ -80,6 +82,8 @@ union iso_dhead {
 } iso_dhead[DNOHSZ];
 #endif
 
+static unsigned	cd9660_chars2ui __P((unsigned char *begin, int len));
+
 /*
  * Initialize hash links for inodes and dnodes.
  */
@@ -420,9 +424,16 @@ iso_ilock(ip)
 {
 
 	while (ip->i_flag & ILOCKED) {
+		if (ip->i_spare0 == curproc->p_pid) {
+			if((ip->i_flag & IRECURSE) == 0)
+				panic("locking against myself");
+			else {
+				++ip->i_lockcount;
+				return(0);
+			}
+				
+		}
 		ip->i_flag |= IWANT;
-		if (ip->i_spare0 == curproc->p_pid)
-			panic("locking against myself");
 		ip->i_spare1 = curproc->p_pid;
 		(void) tsleep((caddr_t)ip, PINOD, "isoilk", 0);
 	}
@@ -440,10 +451,14 @@ iso_iunlock(ip)
 	register struct iso_node *ip;
 {
 
+	if(ip->i_lockcount > 0) {
+		--ip->i_lockcount;
+		return(0);
+	}
 	if ((ip->i_flag & ILOCKED) == 0)
 		vprint("iso_iunlock: unlocked inode", ITOV(ip));
 	ip->i_spare0 = 0;
-	ip->i_flag &= ~ILOCKED;
+	ip->i_flag &= ~(ILOCKED|IRECURSE);
 	if (ip->i_flag&IWANT) {
 		ip->i_flag &= ~IWANT;
 		wakeup((caddr_t)ip);
@@ -581,8 +596,8 @@ enum ISO_FTYPE ftype;
 		tz = 0;
 
 	if (y < 1970) {
-		pu->ts_sec  = 0;
-		pu->ts_nsec = 0;
+		pu->tv_sec  = 0;
+		pu->tv_nsec = 0;
 		return 0;
 	} else {
 #ifdef	ORIGINAL
@@ -602,8 +617,8 @@ enum ISO_FTYPE ftype;
 		if (-48 <= tz && tz <= 52)
 			crtime -= tz * 15 * 60;
 	}
-	pu->ts_sec  = crtime;
-	pu->ts_nsec = 0;
+	pu->tv_sec  = crtime;
+	pu->tv_nsec = 0;
 	return 1;
 }
 
