@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_node.h	8.2 (Berkeley) 1/23/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_node.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_node.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -95,6 +95,7 @@ struct iso_node {
 	long iso_start;		/* actual start of data of file (may be different */
 				/* from iso_extent, if file has extended attributes) */
 	ISO_RRIP_INODE	inode;
+	int		i_lockcount; /* Process lock count (recursion) */
 };
 
 #define	i_forw		i_chain[0]
@@ -104,6 +105,7 @@ struct iso_node {
 #define	ILOCKED		0x0001		/* inode is locked */
 #define	IWANT		0x0002		/* some process waiting on lock */
 #define	IACC		0x0020		/* inode access time to be updated */
+#define	IRECURSE	0x0400		/* Recursion expected */
 
 #define VTOI(vp) ((struct iso_node *)(vp)->v_data)
 #define ITOV(ip) ((ip)->i_vnode)
@@ -111,32 +113,16 @@ struct iso_node {
 #define ISO_ILOCK(ip)	iso_ilock(ip)
 #define ISO_IUNLOCK(ip)	iso_iunlock(ip)
 
-extern int	(**cd9660_fifoop_p)();
-extern int	(**cd9660_specop_p)();
+extern vop_t	**cd9660_fifoop_p;
+extern vop_t	**cd9660_specop_p;
 
 /*
  * Prototypes for ISOFS vnode operations
  */
 int cd9660_lookup __P((struct vop_lookup_args *));
-int cd9660_open __P((struct vop_open_args *));
-int cd9660_close __P((struct vop_close_args *));
-int cd9660_access __P((struct vop_access_args *));
-int cd9660_getattr __P((struct vop_getattr_args *));
-int cd9660_read __P((struct vop_read_args *));
-int cd9660_ioctl __P((struct vop_ioctl_args *));
-int cd9660_select __P((struct vop_select_args *));
-int cd9660_mmap __P((struct vop_mmap_args *));
-int cd9660_seek __P((struct vop_seek_args *));
-int cd9660_readdir __P((struct vop_readdir_args *));
-int cd9660_abortop __P((struct vop_abortop_args *));
 int cd9660_inactive __P((struct vop_inactive_args *));
 int cd9660_reclaim __P((struct vop_reclaim_args *));
 int cd9660_bmap __P((struct vop_bmap_args *));
-int cd9660_lock __P((struct vop_lock_args *));
-int cd9660_unlock __P((struct vop_unlock_args *));
-int cd9660_strategy __P((struct vop_strategy_args *));
-int cd9660_print __P((struct vop_print_args *));
-int cd9660_islocked __P((struct vop_islocked_args *));
 void cd9660_defattr __P((struct iso_directory_record *,
 			struct iso_node *, struct buf *, enum ISO_FTYPE));
 void cd9660_deftstamp __P((struct iso_directory_record *,
