@@ -36,9 +36,11 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_vfsops.c	8.3 (Berkeley) 1/31/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_vfsops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_vfsops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#define CD9660 1  /* bogus dependency in sys/mount.h */
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/namei.h>
@@ -48,6 +50,8 @@
 #include <miscfs/specfs/specdev.h>
 #include <sys/mount.h>
 #include <sys/buf.h>
+#include <sys/cdio.h>
+#include <sys/conf.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
 #include <sys/errno.h>
@@ -57,7 +61,23 @@
 #include <isofs/cd9660/cd9660_node.h>
 #include <isofs/cd9660/iso_rrip.h>
 
-struct vfsops cd9660_vfsops = {
+
+static int cd9660_mount __P((struct mount *,
+	    char *, caddr_t, struct nameidata *, struct proc *));
+static int cd9660_start __P((struct mount *, int, struct proc *));
+static int cd9660_unmount __P((struct mount *, int, struct proc *));
+static int cd9660_root __P((struct mount *, struct vnode **));
+static int cd9660_quotactl __P((struct mount *, int, uid_t, caddr_t, 
+	    struct proc *));
+static int cd9660_statfs __P((struct mount *, struct statfs *, struct proc *));
+static int cd9660_sync __P((struct mount *, int, struct ucred *, 
+	    struct proc *));
+static int cd9660_vget __P((struct mount *, ino_t, struct vnode **));
+static int cd9660_fhtovp __P((struct mount *, struct fid *, struct mbuf *,
+	    struct vnode **, int *, struct ucred **));
+static int cd9660_vptofh __P((struct vnode *, struct fid *));
+
+static struct vfsops cd9660_vfsops = {
 	cd9660_mount,
 	cd9660_start,
 	cd9660_unmount,
@@ -80,11 +100,53 @@ VFS_SET(cd9660_vfsops, cd9660, MOUNT_CD9660, VFCF_READONLY);
  */
 #define ROOTNAME	"root_device"
 
+static int iso_get_ssector __P((dev_t dev, struct proc *p));
 static int iso_mountfs __P((struct vnode *devvp, struct mount *mp,
 			    struct proc *p, struct iso_args *argp));
 
+/*
+ * Try to find the start of the last data track on this CD-ROM.  This
+ * is used to mount the last session of a multi-session CD.  Bail out
+ * and return 0 if we fail, this is always a safe bet.
+ */
+static int
+iso_get_ssector(dev, p)
+	dev_t dev;
+	struct proc *p;
+{
+	struct ioc_toc_header h;
+	struct ioc_read_toc_single_entry t;
+	int i;
+	struct bdevsw *bd;
+	d_ioctl_t *ioctlp;
+
+	bd = bdevsw[major(dev)];
+	ioctlp = bd->d_ioctl;
+	if (ioctlp == NULL)
+		return 0;
+
+	if (ioctlp(dev, CDIOREADTOCHEADER, (caddr_t)&h, FREAD, p) != 0)
+		return 0;
+
+	for (i = h.ending_track; i >= 0; i--) {
+		t.address_format = CD_LBA_FORMAT;
+		t.track = i;
+		if (ioctlp(dev, CDIOREADTOCENTRY, (caddr_t)&t, FREAD, p) != 0)
+			return 0;
+		if ((t.entry.control & 4) != 0)
+			/* found a data track */
+			break;
+	}
+
+	if (i < 0)
+		return 0;
+
+	return ntohl(t.entry.addr.lba);
+}
+
 int
-cd9660_mountroot()
+cd9660_mountroot(dummy)
+	void *dummy;
 {
 	register struct mount *mp;
 	struct proc *p = curproc;	/* XXX */
@@ -104,6 +166,10 @@ cd9660_mountroot()
 	mp->mnt_op = &cd9660_vfsops;
 	mp->mnt_flag = MNT_RDONLY;
 	args.flags = ISOFSMNT_ROOT;
+	args.ssector = iso_get_ssector(rootdev, p);
+	if (bootverbose)
+		printf("cd9660_mountroot(): using session at block %d\n",
+		       args.ssector);
 	if ((error = iso_mountfs(rootvp, mp, p, &args))) {
 		free(mp, M_MOUNT);
 		return (error);
@@ -113,7 +179,7 @@ cd9660_mountroot()
 		free(mp, M_MOUNT);
 		return (error);
 	}
-	TAILQ_INSERT_TAIL(&mountlist, mp, mnt_list);
+	CIRCLEQ_INSERT_TAIL(&mountlist, mp, mnt_list);
 	mp->mnt_flag |= MNT_ROOTFS;
 	mp->mnt_vnodecovered = NULLVP;
 	imp = VFSTOISOFS(mp);
@@ -132,14 +198,14 @@ cd9660_mountroot()
 /*
  * Flag to allow forcible unmounting.
  */
-int iso_doforce = 1;
+static int iso_doforce = 1;
 
 /*
  * VFS Operations.
  *
  * mount system call
  */
-int
+static int
 cd9660_mount(mp, path, data, ndp, p)
 	register struct mount *mp;
 	char *path;
@@ -260,7 +326,9 @@ iso_mountfs(devvp, mp, p, argp)
 	 */
 	iso_bsize = ISO_DEFAULT_BLOCK_SIZE;
 
-	for (iso_blknum = 16; iso_blknum < 100; iso_blknum++) {
+	for (iso_blknum = 16 + argp->ssector;
+	     iso_blknum < 100 + argp->ssector;
+	     iso_blknum++) {
 		if ((error = bread (devvp, btodb(iso_blknum * iso_bsize),
 				    iso_bsize, NOCRED, &bp)))
 			goto out;
@@ -316,6 +384,15 @@ iso_mountfs(devvp, mp, p, argp)
 		isonum_733 (high_sierra?
 			    pri_sierra->volume_space_size:
 			    pri->volume_space_size);
+	/*
+	 * Since an ISO9660 multi-session CD can also access previous
+	 * sessions, we have to include them into the space consider-
+	 * ations.  This doesn't yield a very accurate number since
+	 * parts of the old sessions might be inaccessible now, but we
+	 * can't do much better.  This is also important for the NFS
+	 * filehandle validation.
+	 */
+	isomp->volume_space_size += argp->ssector;
 	bcopy (rootp, isomp->root, sizeof isomp->root);
 	isomp->root_extent = isonum_733 (rootp->extent);
 	isomp->root_size = isonum_733 (rootp->size);
@@ -384,6 +461,7 @@ iso_mountfs(devvp, mp, p, argp)
 
 	return 0;
 out:
+	devvp->v_specflags &= ~SI_MOUNTEDON;
 	if (bp)
 		brelse(bp);
 	if (needclose)
@@ -400,7 +478,7 @@ out:
  * Nothing to do at the moment.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_start(mp, flags, p)
 	struct mount *mp;
 	int flags;
@@ -412,7 +490,7 @@ cd9660_start(mp, flags, p)
 /*
  * unmount system call
  */
-int
+static int
 cd9660_unmount(mp, mntflags, p)
 	struct mount *mp;
 	int mntflags;
@@ -453,7 +531,7 @@ cd9660_unmount(mp, mntflags, p)
 /*
  * Return root of a filesystem
  */
-int
+static int
 cd9660_root(mp, vpp)
 	struct mount *mp;
 	struct vnode **vpp;
@@ -491,7 +569,7 @@ cd9660_root(mp, vpp)
  * Do operations associated with quotas, not supported
  */
 /* ARGSUSED */
-int
+static int
 cd9660_quotactl(mp, cmd, uid, arg, p)
 	struct mount *mp;
 	int cmd;
@@ -536,7 +614,7 @@ cd9660_statfs(mp, sbp, p)
 }
 
 /* ARGSUSED */
-int
+static int
 cd9660_sync(mp, waitfor, cred, p)
 	struct mount *mp;
 	int waitfor;
@@ -551,7 +629,7 @@ cd9660_sync(mp, waitfor, cred, p)
  * Currently unsupported.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_vget(mp, ino, vpp)
 	struct mount *mp;
 	ino_t ino;
