@@ -36,9 +36,11 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_vnops.c	8.3 (Berkeley) 1/23/94
- * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/isofs/cd9660/cd9660_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#define CD9660 1  /* bogus dependency in sys/mount.h */
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/namei.h>
@@ -60,11 +62,36 @@
 #include <isofs/cd9660/cd9660_node.h>
 #include <isofs/cd9660/iso_rrip.h>
 
+static int cd9660_setattr __P((struct vop_setattr_args *));
+static int cd9660_open __P((struct vop_open_args *));
+static int cd9660_close __P((struct vop_close_args *));
+static int cd9660_access __P((struct vop_access_args *));
+static int cd9660_getattr __P((struct vop_getattr_args *));
+static int cd9660_read __P((struct vop_read_args *));
+static int cd9660_ioctl __P((struct vop_ioctl_args *));
+static int cd9660_select __P((struct vop_select_args *));
+static int cd9660_mmap __P((struct vop_mmap_args *));
+static int cd9660_seek __P((struct vop_seek_args *));
+struct isoreaddir;
+static int iso_uiodir __P((struct isoreaddir *idp, struct dirent *dp,
+			   off_t off));
+static int iso_shipdir __P((struct isoreaddir *idp));
+static int cd9660_readdir __P((struct vop_readdir_args *));
+static int cd9660_readlink __P((struct vop_readlink_args *ap));
+static int cd9660_abortop __P((struct vop_abortop_args *));
+static int cd9660_lock __P((struct vop_lock_args *));
+static int cd9660_unlock __P((struct vop_unlock_args *));
+static int cd9660_strategy __P((struct vop_strategy_args *));
+static int cd9660_print __P((struct vop_print_args *));
+static int cd9660_enotsupp __P((void));
+static int cd9660_islocked __P((struct vop_islocked_args *));
+
 #if 0
 /*
  * Mknod vnode call
  *  Actually remap the device number
  */
+static int
 cd9660_mknod(ndp, vap, cred, p)
 	struct nameidata *ndp;
 	struct ucred *cred;
@@ -119,7 +146,7 @@ cd9660_mknod(ndp, vap, cred, p)
 /*
  * Setattr call. Only allowed for block and character special devices.
  */
-int
+static int
 cd9660_setattr(ap)
 	struct vop_setattr_args /* {
 		struct vnodeop_desc *a_desc;
@@ -132,11 +159,11 @@ cd9660_setattr(ap)
 	struct vnode *vp = ap->a_vp;
 	struct vattr *vap = ap->a_vap;
 
-  	if (vap->va_flags != VNOVAL || vap->va_uid != (uid_t)VNOVAL ||
-	    vap->va_gid != (gid_t)VNOVAL || vap->va_atime.ts_sec != VNOVAL ||
-	    vap->va_mtime.ts_sec != VNOVAL || vap->va_mode != (mode_t)VNOVAL)
+  	if (vap->va_flags != (u_long)VNOVAL || vap->va_uid != (uid_t)VNOVAL ||
+	    vap->va_gid != (gid_t)VNOVAL || vap->va_atime.tv_sec != VNOVAL ||
+	    vap->va_mtime.tv_sec != VNOVAL || vap->va_mode != (mode_t)VNOVAL)
 		return (EROFS);
-	if (vap->va_size != VNOVAL) {
+	if (vap->va_size != (u_quad_t)VNOVAL) {
  		switch (vp->v_type) {
  		case VDIR:
  			return (EISDIR);
@@ -159,7 +186,7 @@ cd9660_setattr(ap)
  * Nothing to do.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_open(ap)
 	struct vop_open_args /* {
 		struct vnode *a_vp;
@@ -177,7 +204,7 @@ cd9660_open(ap)
  * Update the times on the inode on writeable file systems.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_close(ap)
 	struct vop_close_args /* {
 		struct vnode *a_vp;
@@ -195,7 +222,7 @@ cd9660_close(ap)
  * super user is granted all permissions.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_access(ap)
 	struct vop_access_args /* {
 		struct vnode *a_vp;
@@ -223,7 +250,7 @@ cd9660_access(ap)
 	return (0);
 }
 
-int
+static int
 cd9660_getattr(ap)
 	struct vop_getattr_args /* {
 		struct vnode *a_vp;
@@ -255,10 +282,11 @@ cd9660_getattr(ap)
 	vap->va_blocksize = ip->i_mnt->logical_block_size;
 	vap->va_bytes	= (u_quad_t) ip->i_size;
 	vap->va_type	= vp->v_type;
+	vap->va_filerev	= 0;
 	return (0);
 }
 
-#if ISO_DEFAULT_BLOCK_SIZE >= NBPG
+#if ISO_DEFAULT_BLOCK_SIZE >= PAGE_SIZE
 #ifdef DEBUG
 extern int doclusterread;
 #else
@@ -266,13 +294,17 @@ extern int doclusterread;
 #endif
 #else
 /* XXX until cluster routines can handle block sizes less than one page */
+#if defined(__FreeBSD__)
+#define doclusterread 1
+#else
 #define doclusterread 0
 #endif
+#endif
 
 /*
  * Vnode op for reading.
  */
-int
+static int
 cd9660_read(ap)
 	struct vop_read_args /* {
 		struct vnode *a_vp;
@@ -311,7 +343,7 @@ cd9660_read(ap)
 		rablock = lbn + 1;
 		if (doclusterread) {
 			if (iso_lblktosize(imp, rablock) <= ip->i_size)
-				error = cluster_read(vp, (off_t)ip->i_size,
+				error = cluster_read(vp, ip->i_size,
 						     lbn, size, NOCRED, &bp);
 			else
 				error = bread(vp, lbn, size, NOCRED, &bp);
@@ -331,14 +363,18 @@ cd9660_read(ap)
 			return (error);
 		}
 
+		if (uio->uio_segflg != UIO_NOCOPY)
+			ip->i_flag |= IRECURSE;
 		error = uiomove(bp->b_un.b_addr + on, (int)n, uio);
+		if (uio->uio_segflg != UIO_NOCOPY)
+			ip->i_flag &= ~IRECURSE;
 		brelse(bp);
 	} while (error == 0 && uio->uio_resid > 0 && n != 0);
 	return (error);
 }
 
 /* ARGSUSED */
-int
+static int
 cd9660_ioctl(ap)
 	struct vop_ioctl_args /* {
 		struct vnode *a_vp;
@@ -354,7 +390,7 @@ cd9660_ioctl(ap)
 }
 
 /* ARGSUSED */
-int
+static int
 cd9660_select(ap)
 	struct vop_select_args /* {
 		struct vnode *a_vp;
@@ -377,7 +413,7 @@ cd9660_select(ap)
  * NB Currently unsupported.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_mmap(ap)
 	struct vop_mmap_args /* {
 		struct vnode *a_vp;
@@ -396,7 +432,7 @@ cd9660_mmap(ap)
  * Nothing to do, so just return.
  */
 /* ARGSUSED */
-int
+static int
 cd9660_seek(ap)
 	struct vop_seek_args /* {
 		struct vnode *a_vp;
@@ -513,7 +549,7 @@ assoc = (cl > 1) && (*cname == ASSOCCHAR);
  * XXX make sure everything still works now that eofflagp and cookiep
  * are no longer args.
  */
-int
+static int
 cd9660_readdir(ap)
 	struct vop_readdir_args /* {
 		struct vnode *a_vp;
@@ -711,7 +747,7 @@ cd9660_readdir(ap)
 typedef struct iso_directory_record ISODIR;
 typedef struct iso_node		    ISONODE;
 typedef struct iso_mnt		    ISOMNT;
-int
+static int
 cd9660_readlink(ap)
 	struct vop_readlink_args /* {
 		struct vnode *a_vp;
@@ -767,7 +803,7 @@ cd9660_readlink(ap)
 	 *   1: Check not cross boundary on block
 	 */
 	if ((ip->i_number & imp->im_bmask) + isonum_711(dirp->length)
-	    > imp->logical_block_size) {
+	    > (unsigned)imp->logical_block_size) {
 		brelse(bp);
 		return EINVAL;
 	}
@@ -805,7 +841,7 @@ cd9660_readlink(ap)
  * Ufs abort op, called after namei() when a CREATE/DELETE isn't actually
  * done. If a buffer has been saved in anticipation of a CREATE, delete it.
  */
-int
+static int
 cd9660_abortop(ap)
 	struct vop_abortop_args /* {
 		struct vnode *a_dvp;
@@ -820,7 +856,7 @@ cd9660_abortop(ap)
 /*
  * Lock an inode.
  */
-int
+static int
 cd9660_lock(ap)
 	struct vop_lock_args /* {
 		struct vnode *a_vp;
@@ -835,7 +871,7 @@ cd9660_lock(ap)
 /*
  * Unlock an inode.
  */
-int
+static int
 cd9660_unlock(ap)
 	struct vop_unlock_args /* {
 		struct vnode *a_vp;
@@ -852,7 +888,7 @@ cd9660_unlock(ap)
 /*
  * Check for a locked inode.
  */
-int
+static int
 cd9660_islocked(ap)
 	struct vop_islocked_args /* {
 		struct vnode *a_vp;
@@ -868,7 +904,7 @@ cd9660_islocked(ap)
  * Calculate the logical to physical mapping if not done already,
  * then call the device strategy routine.
  */
-int
+static int
 cd9660_strategy(ap)
 	struct vop_strategy_args /* {
 		struct buf *a_bp;
@@ -884,7 +920,7 @@ cd9660_strategy(ap)
 		panic("cd9660_strategy: spec");
 	if (bp->b_blkno == bp->b_lblkno) {
 		if ((error =
-		    VOP_BMAP(vp, bp->b_lblkno, NULL, &bp->b_blkno, NULL))) {
+		    VOP_BMAP(vp, bp->b_lblkno, NULL, &bp->b_blkno, NULL, NULL))) {
 			bp->b_error = error;
 			bp->b_flags |= B_ERROR;
 			biodone(bp);
@@ -906,7 +942,7 @@ cd9660_strategy(ap)
 /*
  * Print out the contents of an inode.
  */
-int
+static int
 cd9660_print(ap)
 	struct vop_print_args /* {
 		struct vnode *a_vp;
@@ -919,7 +955,7 @@ cd9660_print(ap)
 /*
  * Unsupported operation
  */
-int
+static int
 cd9660_enotsupp()
 {
 
@@ -965,153 +1001,154 @@ cd9660_enotsupp()
 /*
  * Global vfs data structures for nfs
  */
-int (**cd9660_vnodeop_p)();
-struct vnodeopv_entry_desc cd9660_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, cd9660_lookup },	/* lookup */
-	{ &vop_create_desc, cd9660_create },	/* create */
-	{ &vop_mknod_desc, cd9660_mknod },	/* mknod */
-	{ &vop_open_desc, cd9660_open },	/* open */
-	{ &vop_close_desc, cd9660_close },	/* close */
-	{ &vop_access_desc, cd9660_access },	/* access */
-	{ &vop_getattr_desc, cd9660_getattr },	/* getattr */
-	{ &vop_setattr_desc, cd9660_setattr },	/* setattr */
-	{ &vop_read_desc, cd9660_read },	/* read */
-	{ &vop_write_desc, cd9660_write },	/* write */
-	{ &vop_ioctl_desc, cd9660_ioctl },	/* ioctl */
-	{ &vop_select_desc, cd9660_select },	/* select */
-	{ &vop_mmap_desc, cd9660_mmap },	/* mmap */
-	{ &vop_fsync_desc, cd9660_fsync },	/* fsync */
-	{ &vop_seek_desc, cd9660_seek },	/* seek */
-	{ &vop_remove_desc, cd9660_remove },	/* remove */
-	{ &vop_link_desc, cd9660_link },	/* link */
-	{ &vop_rename_desc, cd9660_rename },	/* rename */
-	{ &vop_mkdir_desc, cd9660_mkdir },	/* mkdir */
-	{ &vop_rmdir_desc, cd9660_rmdir },	/* rmdir */
-	{ &vop_symlink_desc, cd9660_symlink },	/* symlink */
-	{ &vop_readdir_desc, cd9660_readdir },	/* readdir */
-	{ &vop_readlink_desc, cd9660_readlink },/* readlink */
-	{ &vop_abortop_desc, cd9660_abortop },	/* abortop */
-	{ &vop_inactive_desc, cd9660_inactive },/* inactive */
-	{ &vop_reclaim_desc, cd9660_reclaim },	/* reclaim */
-	{ &vop_lock_desc, cd9660_lock },	/* lock */
-	{ &vop_unlock_desc, cd9660_unlock },	/* unlock */
-	{ &vop_bmap_desc, cd9660_bmap },	/* bmap */
-	{ &vop_strategy_desc, cd9660_strategy },/* strategy */
-	{ &vop_print_desc, cd9660_print },	/* print */
-	{ &vop_islocked_desc, cd9660_islocked },/* islocked */
-	{ &vop_pathconf_desc, cd9660_pathconf },/* pathconf */
-	{ &vop_advlock_desc, cd9660_advlock },	/* advlock */
-	{ &vop_blkatoff_desc, cd9660_blkatoff },/* blkatoff */
-	{ &vop_valloc_desc, cd9660_valloc },	/* valloc */
-	{ &vop_vfree_desc, cd9660_vfree },	/* vfree */
-	{ &vop_truncate_desc, cd9660_truncate },/* truncate */
-	{ &vop_update_desc, cd9660_update },	/* update */
-	{ &vop_bwrite_desc, vn_bwrite },
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+vop_t **cd9660_vnodeop_p;
+static struct vnodeopv_entry_desc cd9660_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)cd9660_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)cd9660_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)cd9660_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)cd9660_open },		/* open */
+	{ &vop_close_desc, (vop_t *)cd9660_close },		/* close */
+	{ &vop_access_desc, (vop_t *)cd9660_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)cd9660_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)cd9660_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)cd9660_read },		/* read */
+	{ &vop_write_desc, (vop_t *)cd9660_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)cd9660_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)cd9660_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)cd9660_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)cd9660_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)cd9660_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)cd9660_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)cd9660_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)cd9660_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)cd9660_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)cd9660_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)cd9660_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)cd9660_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)cd9660_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)cd9660_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)cd9660_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)cd9660_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)cd9660_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)cd9660_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)cd9660_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)cd9660_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)cd9660_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)cd9660_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)cd9660_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)cd9660_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)cd9660_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)cd9660_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)cd9660_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)cd9660_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)cd9660_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)vn_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc cd9660_vnodeop_opv_desc =
+static struct vnodeopv_desc cd9660_vnodeop_opv_desc =
 	{ &cd9660_vnodeop_p, cd9660_vnodeop_entries };
 VNODEOP_SET(cd9660_vnodeop_opv_desc);
 
 /*
  * Special device vnode ops
  */
-int (**cd9660_specop_p)();
-struct vnodeopv_entry_desc cd9660_specop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, spec_lookup },	/* lookup */
-	{ &vop_create_desc, cd9660_create },	/* create */
-	{ &vop_mknod_desc, cd9660_mknod },	/* mknod */
-	{ &vop_open_desc, spec_open },		/* open */
-	{ &vop_close_desc, spec_close },	/* close */
-	{ &vop_access_desc, cd9660_access },	/* access */
-	{ &vop_getattr_desc, cd9660_getattr },	/* getattr */
-	{ &vop_setattr_desc, cd9660_setattr },	/* setattr */
-	{ &vop_read_desc, spec_read },		/* read */
-	{ &vop_write_desc, spec_write },	/* write */
-	{ &vop_ioctl_desc, spec_ioctl },	/* ioctl */
-	{ &vop_select_desc, spec_select },	/* select */
-	{ &vop_mmap_desc, spec_mmap },		/* mmap */
-	{ &vop_fsync_desc, spec_fsync },	/* fsync */
-	{ &vop_seek_desc, spec_seek },		/* seek */
-	{ &vop_remove_desc, cd9660_remove },	/* remove */
-	{ &vop_link_desc, cd9660_link },	/* link */
-	{ &vop_rename_desc, cd9660_rename },	/* rename */
-	{ &vop_mkdir_desc, cd9660_mkdir },	/* mkdir */
-	{ &vop_rmdir_desc, cd9660_rmdir },	/* rmdir */
-	{ &vop_symlink_desc, cd9660_symlink },	/* symlink */
-	{ &vop_readdir_desc, spec_readdir },	/* readdir */
-	{ &vop_readlink_desc, spec_readlink },	/* readlink */
-	{ &vop_abortop_desc, spec_abortop },	/* abortop */
-	{ &vop_inactive_desc, cd9660_inactive },/* inactive */
-	{ &vop_reclaim_desc, cd9660_reclaim },	/* reclaim */
-	{ &vop_lock_desc, cd9660_lock },	/* lock */
-	{ &vop_unlock_desc, cd9660_unlock },	/* unlock */
-	{ &vop_bmap_desc, spec_bmap },		/* bmap */
-	{ &vop_strategy_desc, spec_strategy },  /* strategy */
-	{ &vop_print_desc, cd9660_print },	/* print */
-	{ &vop_islocked_desc, cd9660_islocked },/* islocked */
-	{ &vop_pathconf_desc, spec_pathconf },	/* pathconf */
-	{ &vop_advlock_desc, spec_advlock },	/* advlock */
-	{ &vop_blkatoff_desc, spec_blkatoff },	/* blkatoff */
-	{ &vop_valloc_desc, spec_valloc },	/* valloc */
-	{ &vop_vfree_desc, spec_vfree },	/* vfree */
-	{ &vop_truncate_desc, spec_truncate },	/* truncate */
-	{ &vop_update_desc, cd9660_update },	/* update */
-	{ &vop_bwrite_desc, vn_bwrite },
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+vop_t **cd9660_specop_p;
+static struct vnodeopv_entry_desc cd9660_specop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)spec_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)cd9660_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)cd9660_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)spec_open },			/* open */
+	{ &vop_close_desc, (vop_t *)spec_close },		/* close */
+	{ &vop_access_desc, (vop_t *)cd9660_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)cd9660_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)cd9660_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)spec_read },			/* read */
+	{ &vop_write_desc, (vop_t *)spec_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)spec_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)spec_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)spec_mmap },			/* mmap */
+	{ &vop_fsync_desc, (vop_t *)spec_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)spec_seek },			/* seek */
+	{ &vop_remove_desc, (vop_t *)cd9660_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)cd9660_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)cd9660_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)cd9660_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)cd9660_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)cd9660_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)spec_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)spec_readlink },		/* readlink */
+	{ &vop_abortop_desc, (vop_t *)spec_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)cd9660_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)cd9660_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)cd9660_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)cd9660_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)spec_bmap },			/* bmap */
+	{ &vop_strategy_desc, (vop_t *)spec_strategy },		/* strategy */
+	{ &vop_print_desc, (vop_t *)cd9660_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)cd9660_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)spec_pathconf },		/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)spec_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)spec_blkatoff },		/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)spec_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)spec_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)spec_truncate },		/* truncate */
+	{ &vop_update_desc, (vop_t *)cd9660_update },		/* update */
+ 	{ &vop_getpages_desc, (vop_t *)spec_getpages},		/* getpages */
+	{ &vop_bwrite_desc, (vop_t *)vn_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc cd9660_specop_opv_desc =
+static struct vnodeopv_desc cd9660_specop_opv_desc =
 	{ &cd9660_specop_p, cd9660_specop_entries };
 VNODEOP_SET(cd9660_specop_opv_desc);
 
-int (**cd9660_fifoop_p)();
-struct vnodeopv_entry_desc cd9660_fifoop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, fifo_lookup },	/* lookup */
-	{ &vop_create_desc, cd9660_create },	/* create */
-	{ &vop_mknod_desc, cd9660_mknod },	/* mknod */
-	{ &vop_open_desc, fifo_open },		/* open */
-	{ &vop_close_desc, fifo_close },	/* close */
-	{ &vop_access_desc, cd9660_access },	/* access */
-	{ &vop_getattr_desc, cd9660_getattr },	/* getattr */
-	{ &vop_setattr_desc, cd9660_setattr },	/* setattr */
-	{ &vop_read_desc, fifo_read },		/* read */
-	{ &vop_write_desc, fifo_write },	/* write */
-	{ &vop_ioctl_desc, fifo_ioctl },	/* ioctl */
-	{ &vop_select_desc, fifo_select },	/* select */
-	{ &vop_mmap_desc, fifo_mmap },		/* mmap */
-	{ &vop_fsync_desc, fifo_fsync },	/* fsync */
-	{ &vop_seek_desc, fifo_seek },		/* seek */
-	{ &vop_remove_desc, cd9660_remove },	/* remove */
-	{ &vop_link_desc, cd9660_link },	/* link */
-	{ &vop_rename_desc, cd9660_rename },	/* rename */
-	{ &vop_mkdir_desc, cd9660_mkdir },	/* mkdir */
-	{ &vop_rmdir_desc, cd9660_rmdir },	/* rmdir */
-	{ &vop_symlink_desc, cd9660_symlink },	/* symlink */
-	{ &vop_readdir_desc, fifo_readdir },	/* readdir */
-	{ &vop_readlink_desc, fifo_readlink },	/* readlink */
-	{ &vop_abortop_desc, fifo_abortop },	/* abortop */
-	{ &vop_inactive_desc, cd9660_inactive },/* inactive */
-	{ &vop_reclaim_desc, cd9660_reclaim },	/* reclaim */
-	{ &vop_lock_desc, cd9660_lock },	/* lock */
-	{ &vop_unlock_desc, cd9660_unlock },	/* unlock */
-	{ &vop_bmap_desc, fifo_bmap },		/* bmap */
-	{ &vop_strategy_desc, fifo_badop },	/* strategy */
-	{ &vop_print_desc, cd9660_print },	/* print */
-	{ &vop_islocked_desc, cd9660_islocked },/* islocked */
-	{ &vop_pathconf_desc, fifo_pathconf },	/* pathconf */
-	{ &vop_advlock_desc, fifo_advlock },	/* advlock */
-	{ &vop_blkatoff_desc, fifo_blkatoff },	/* blkatoff */
-	{ &vop_valloc_desc, fifo_valloc },	/* valloc */
-	{ &vop_vfree_desc, fifo_vfree },	/* vfree */
-	{ &vop_truncate_desc, fifo_truncate },	/* truncate */
-	{ &vop_update_desc, cd9660_update },	/* update */
-	{ &vop_bwrite_desc, vn_bwrite },
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+vop_t **cd9660_fifoop_p;
+static struct vnodeopv_entry_desc cd9660_fifoop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)fifo_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)cd9660_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)cd9660_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)fifo_open },			/* open */
+	{ &vop_close_desc, (vop_t *)fifo_close },		/* close */
+	{ &vop_access_desc, (vop_t *)cd9660_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)cd9660_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)cd9660_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)fifo_read },			/* read */
+	{ &vop_write_desc, (vop_t *)fifo_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)fifo_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)fifo_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)fifo_mmap },			/* mmap */
+	{ &vop_fsync_desc, (vop_t *)fifo_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)fifo_seek },			/* seek */
+	{ &vop_remove_desc, (vop_t *)cd9660_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)cd9660_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)cd9660_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)cd9660_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)cd9660_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)cd9660_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)fifo_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)fifo_readlink },		/* readlink */
+	{ &vop_abortop_desc, (vop_t *)fifo_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)cd9660_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)cd9660_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)cd9660_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)cd9660_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)fifo_bmap },			/* bmap */
+	{ &vop_strategy_desc, (vop_t *)fifo_badop },		/* strategy */
+	{ &vop_print_desc, (vop_t *)cd9660_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)cd9660_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)fifo_pathconf },		/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)fifo_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)fifo_blkatoff },		/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)fifo_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)fifo_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)fifo_truncate },		/* truncate */
+	{ &vop_update_desc, (vop_t *)cd9660_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)vn_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc cd9660_fifoop_opv_desc =
+static struct vnodeopv_desc cd9660_fifoop_opv_desc =
 	{ &cd9660_fifoop_p, cd9660_fifoop_entries };
 
 VNODEOP_SET(cd9660_fifoop_opv_desc);
