@@ -7,7 +7,7 @@
  * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
  * ----------------------------------------------------------------------------
  *
- * $FreeBSD: stable/2.1/sys/kern/inflate.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/inflate.c 50950 1999-09-05 08:39:22Z peter $
  *
  *
  */
@@ -20,11 +20,6 @@
 #include <sys/mman.h>
 #include <sys/malloc.h>
 
-#ifdef KERNEL
-#include <vm/vm.h>
-#include <vm/vm_kern.h>
-#endif
-
 /* needed to make inflate() work */
 #define	uch u_char
 #define	ush u_short
