@@ -36,40 +36,61 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_descrip.c	8.6 (Berkeley) 4/19/94
- * $FreeBSD: stable/2.1/sys/kern/kern_descrip.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_descrip.c 56600 2000-01-25 18:09:10Z dillon $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
+#include <sys/conf.h>
 #include <sys/filedesc.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
 #include <sys/vnode.h>
 #include <sys/proc.h>
 #include <sys/file.h>
-#include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <sys/stat.h>
-#include <sys/ioctl.h>
+#include <sys/filio.h>
+#include <sys/ttycom.h>
 #include <sys/fcntl.h>
 #include <sys/malloc.h>
-#include <sys/syslog.h>
 #include <sys/unistd.h>
 #include <sys/resourcevar.h>
-#include <vm/vm.h>
+#include <sys/pipe.h>
 
-int finishdup(struct filedesc *fdp, int old, int new, int *retval);
+#include <vm/vm.h>
+#include <vm/vm_extern.h>
+
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
+
+static	 d_open_t  fdopen;
+#define NUMFDESC 64
+
+#define CDEV_MAJOR 22
+static struct cdevsw fildesc_cdevsw = 
+	{ fdopen,	noclose,	noread,		nowrite,	/*22*/
+	  noioc,	nostop,		nullreset,	nodevtotty,/*fd(!=Fd)*/
+	  noselect,	nommap,		nostrat };
+
+static int finishdup(struct filedesc *fdp, int old, int new, int *retval);
 /*
  * Descriptor management.
  */
-struct file *filehead;	/* head of list of open files */
-int nfiles;		/* actual number of open files */
+struct filelist filehead;	/* head of list of open files */
+int nfiles;			/* actual number of open files */
+extern int cmask;	
 
 /*
  * System calls on descriptors.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct getdtablesize_args {
 	int	dummy;
 };
+#endif
 /* ARGSUSED */
 int
 getdtablesize(p, uap, retval)
@@ -85,10 +106,12 @@ getdtablesize(p, uap, retval)
 /*
  * Duplicate a file descriptor to a particular value.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct dup2_args {
 	u_int	from;
 	u_int	to;
 };
+#endif
 /* ARGSUSED */
 int
 dup2(p, uap, retval)
@@ -128,9 +151,11 @@ dup2(p, uap, retval)
 /*
  * Duplicate a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct dup_args {
 	u_int	fd;
 };
+#endif
 /* ARGSUSED */
 int
 dup(p, uap, retval)
@@ -162,11 +187,13 @@ dup(p, uap, retval)
 /*
  * The file control system call.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fcntl_args {
 	int	fd;
 	int	cmd;
 	int	arg;
 };
+#endif
 /* ARGSUSED */
 int
 fcntl(p, uap, retval)
@@ -231,7 +258,7 @@ fcntl(p, uap, retval)
 			return (0);
 		}
 		error = (*fp->f_ops->fo_ioctl)
-			(fp, (int)TIOCGPGRP, (caddr_t)retval, p);
+			(fp, TIOCGPGRP, (caddr_t)retval, p);
 		*retval = -*retval;
 		return (error);
 
@@ -249,7 +276,7 @@ fcntl(p, uap, retval)
 			uap->arg = p1->p_pgrp->pg_id;
 		}
 		return ((*fp->f_ops->fo_ioctl)
-			(fp, (int)TIOCSPGRP, (caddr_t)&uap->arg, p));
+			(fp, TIOCSPGRP, (caddr_t)&uap->arg, p));
 
 	case F_SETLKW:
 		flg |= F_WAIT;
@@ -295,6 +322,9 @@ fcntl(p, uap, retval)
 		error = copyin((caddr_t)uap->arg, (caddr_t)&fl, sizeof (fl));
 		if (error)
 			return (error);
+		if (fl.l_type != F_RDLCK && fl.l_type != F_WRLCK &&
+		    fl.l_type != F_UNLCK)
+			return (EINVAL);
 		if (fl.l_whence == SEEK_CUR)
 			fl.l_start += fp->f_offset;
 		if ((error = VOP_ADVLOCK(vp,(caddr_t)p,F_GETLK,&fl,F_POSIX)))
@@ -310,7 +340,7 @@ fcntl(p, uap, retval)
 /*
  * Common code for dup, dup2, and fcntl(F_DUPFD).
  */
-int
+static int
 finishdup(fdp, old, new, retval)
 	register struct filedesc *fdp;
 	register int old, new, *retval;
@@ -330,6 +360,11 @@ finishdup(fdp, old, new, retval)
 /*
  * Close a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
+struct close_args {
+        int     fd;
+};
+#endif
 /* ARGSUSED */
 int
 close(p, uap, retval)
@@ -361,10 +396,12 @@ close(p, uap, retval)
 /*
  * Return status information about a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ofstat_args {
 	int	fd;
 	struct	ostat *sb;
 };
+#endif
 /* ARGSUSED */
 int
 ofstat(p, uap, retval)
@@ -383,6 +420,7 @@ ofstat(p, uap, retval)
 		return (EBADF);
 	switch (fp->f_type) {
 
+	case DTYPE_FIFO:
 	case DTYPE_VNODE:
 		error = vn_stat((struct vnode *)fp->f_data, &ub, p);
 		break;
@@ -391,6 +429,12 @@ ofstat(p, uap, retval)
 		error = soo_stat((struct socket *)fp->f_data, &ub);
 		break;
 
+#ifndef OLD_PIPE
+	case DTYPE_PIPE:
+		error = pipe_stat((struct pipe *)fp->f_data, &ub);
+		break;
+#endif
+
 	default:
 		panic("ofstat");
 		/*NOTREACHED*/
@@ -405,10 +449,12 @@ ofstat(p, uap, retval)
 /*
  * Return status information about a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fstat_args {
 	int	fd;
 	struct	stat *sb;
 };
+#endif
 /* ARGSUSED */
 int
 fstat(p, uap, retval)
@@ -426,6 +472,7 @@ fstat(p, uap, retval)
 		return (EBADF);
 	switch (fp->f_type) {
 
+	case DTYPE_FIFO:
 	case DTYPE_VNODE:
 		error = vn_stat((struct vnode *)fp->f_data, &ub, p);
 		break;
@@ -434,6 +481,12 @@ fstat(p, uap, retval)
 		error = soo_stat((struct socket *)fp->f_data, &ub);
 		break;
 
+#ifndef OLD_PIPE
+	case DTYPE_PIPE:
+		error = pipe_stat((struct pipe *)fp->f_data, &ub);
+		break;
+#endif
+
 	default:
 		panic("fstat");
 		/*NOTREACHED*/
@@ -446,10 +499,12 @@ fstat(p, uap, retval)
 /*
  * Return pathconf information about a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fpathconf_args {
 	int	fd;
 	int	name;
 };
+#endif
 /* ARGSUSED */
 int
 fpathconf(p, uap, retval)
@@ -466,12 +521,16 @@ fpathconf(p, uap, retval)
 		return (EBADF);
 	switch (fp->f_type) {
 
+#ifndef OLD_PIPE
+	case DTYPE_PIPE:
+#endif
 	case DTYPE_SOCKET:
 		if (uap->name != _PC_PIPE_BUF)
 			return (EINVAL);
 		*retval = PIPE_BUF;
 		return (0);
 
+	case DTYPE_FIFO:
 	case DTYPE_VNODE:
 		vp = (struct vnode *)fp->f_data;
 		return (VOP_PATHCONF(vp, uap->name, retval));
@@ -485,7 +544,8 @@ fpathconf(p, uap, retval)
 /*
  * Allocate a file descriptor for the process.
  */
-int fdexpand;
+static int fdexpand;
+SYSCTL_INT(_debug, OID_AUTO, fdexpand, CTLFLAG_RD, &fdexpand, 0, "");
 
 int
 fdalloc(p, want, result)
@@ -564,13 +624,15 @@ fdavail(p, n)
 {
 	register struct filedesc *fdp = p->p_fd;
 	register struct file **fpp;
-	register int i, lim;
+	register int i, lim, last;
 
 	lim = min((int)p->p_rlimit[RLIMIT_NOFILE].rlim_cur, maxfilesperproc);
 	if ((i = lim - fdp->fd_nfiles) > 0 && (n -= i) <= 0)
 		return (1);
+
+	last = min(fdp->fd_nfiles, lim);
 	fpp = &fdp->fd_ofiles[fdp->fd_freefile];
-	for (i = fdp->fd_nfiles - fdp->fd_freefile; --i >= 0; fpp++)
+	for (i = last - fdp->fd_freefile; --i >= 0; fpp++)
 		if (*fpp == NULL && --n <= 0)
 			return (1);
 	return (0);
@@ -586,7 +648,7 @@ falloc(p, resultfp, resultfd)
 	struct file **resultfp;
 	int *resultfd;
 {
-	register struct file *fp, *fq, **fpp;
+	register struct file *fp, *fq;
 	int error, i;
 
 	if ((error = fdalloc(p, 0, &i)))
@@ -604,16 +666,12 @@ falloc(p, resultfp, resultfd)
 	nfiles++;
 	MALLOC(fp, struct file *, sizeof(struct file), M_FILE, M_WAITOK);
 	bzero(fp, sizeof(struct file));
-	if ((fq = p->p_fd->fd_ofiles[0]))
-		fpp = &fq->f_filef;
-	else
-		fpp = &filehead;
+	if ((fq = p->p_fd->fd_ofiles[0])) {
+		LIST_INSERT_AFTER(fq, fp, f_list);
+	} else {
+		LIST_INSERT_HEAD(&filehead, fp, f_list);
+	}
 	p->p_fd->fd_ofiles[i] = fp;
-	if ((fq = *fpp))
-		fq->f_fileb = &fp->f_filef;
-	fp->f_filef = fq;
-	fp->f_fileb = fpp;
-	*fpp = fp;
 	fp->f_count = 1;
 	fp->f_cred = p->p_ucred;
 	crhold(fp->f_cred);
@@ -631,21 +689,58 @@ void
 ffree(fp)
 	register struct file *fp;
 {
-	register struct file *fq;
-
-	if ((fq = fp->f_filef))
-		fq->f_fileb = fp->f_fileb;
-	*fp->f_fileb = fq;
+	LIST_REMOVE(fp, f_list);
 	crfree(fp->f_cred);
 #ifdef DIAGNOSTIC
-	fp->f_filef = NULL;
-	fp->f_fileb = NULL;
 	fp->f_count = 0;
 #endif
 	nfiles--;
 	FREE(fp, M_FILE);
 }
 
+/*
+ * Build a new filedesc structure.
+ */
+struct filedesc *
+fdinit(p)
+	struct proc *p;
+{
+	register struct filedesc0 *newfdp;
+	register struct filedesc *fdp = p->p_fd;
+
+	MALLOC(newfdp, struct filedesc0 *, sizeof(struct filedesc0),
+	    M_FILEDESC, M_WAITOK);
+	bzero(newfdp, sizeof(struct filedesc0));
+	newfdp->fd_fd.fd_cdir = fdp->fd_cdir;
+	VREF(newfdp->fd_fd.fd_cdir);
+	newfdp->fd_fd.fd_rdir = fdp->fd_rdir;
+	if (newfdp->fd_fd.fd_rdir)
+		VREF(newfdp->fd_fd.fd_rdir);
+
+	/* Create the file descriptor table. */
+	newfdp->fd_fd.fd_refcnt = 1;
+	newfdp->fd_fd.fd_cmask = cmask;
+	newfdp->fd_fd.fd_ofiles = newfdp->fd_dfiles;
+	newfdp->fd_fd.fd_ofileflags = newfdp->fd_dfileflags;
+	newfdp->fd_fd.fd_nfiles = NDFILE;
+
+	newfdp->fd_fd.fd_freefile = 0;
+	newfdp->fd_fd.fd_lastfile = 0;
+
+	return (&newfdp->fd_fd);
+}
+
+/*
+ * Share a filedesc structure.
+ */
+struct filedesc *
+fdshare(p)
+	struct proc *p;
+{
+	p->p_fd->fd_refcnt++;
+	return (p->p_fd);
+}
+
 /*
  * Copy a filedesc structure.
  */
@@ -811,10 +906,12 @@ closef(fp, p)
  * Just attempt to get a record lock of the requested type on
  * the entire file (l_whence = SEEK_SET, l_start = 0, l_len = 0).
  */
+#ifndef _SYS_SYSPROTO_H_
 struct flock_args {
 	int	fd;
 	int	how;
 };
+#endif
 /* ARGSUSED */
 int
 flock(p, uap, retval)
@@ -862,7 +959,7 @@ flock(p, uap, retval)
  * references to this file will be direct to the other driver.
  */
 /* ARGSUSED */
-int
+static int
 fdopen(dev, mode, type, p)
 	dev_t dev;
 	int mode, type;
@@ -881,6 +978,63 @@ fdopen(dev, mode, type, p)
 	return (ENODEV);
 }
 
+ /*
+ * For setuid/setgid programs we don't want to people to use that setuidness
+ * to generate error messages which write to a file which otherwise would 
+ * otherwise be off limits to the proces.
+ * 
+ * This is a gross hack to plug the hole.  A better solution would involve
+ * a special vop or other form of generalized access control mechanism.  We
+ * go ahead and just reject all procfs file systems accesses as dangerous.
+ *
+ * Since setugidsafety calls this only for fd 0, 1 and 2, this check is
+ * sufficient.  We also don't for setugidness since we know we are.
+ */
+static int
+is_unsafe(struct file *fp)
+{
+       if (fp->f_type == DTYPE_VNODE &&
+           ((struct vnode *)(fp->f_data))->v_tag == VT_PROCFS)
+               return (1);
+       return (0);
+}   
+
+/*
+ * Make this setguid thing safe, if at all possible.
+ */
+void
+setugidsafety(p)  
+       struct proc *p;
+{
+       struct filedesc *fdp = p->p_fd;
+       struct file **fpp;
+       char *fdfp;
+       register int i;
+       
+       /* Certain daemons might not have file descriptors. */
+       if (fdp == NULL)
+               return;   
+                       
+       fpp = fdp->fd_ofiles;
+       fdfp = fdp->fd_ofileflags;
+       for (i = 0; i <= fdp->fd_lastfile; i++, fpp++, fdfp++) {
+               if (i > 2)
+                       break;
+               if (*fpp != NULL && is_unsafe(*fpp)) {
+                       if (*fdfp & UF_MAPPED)
+                               (void) munmapfd(p, i);
+                       (void) closef(*fpp, p); 
+                       *fpp = NULL;
+                       *fdfp = 0;
+                       if (i < fdp->fd_freefile)
+                               fdp->fd_freefile = i;
+               }
+       }
+       while (fdp->fd_lastfile > 0 && fdp->fd_ofiles[fdp->fd_lastfile] == NULL)
+               fdp->fd_lastfile--;
+}
+
+
 /*
  * Duplicate the specified descriptor to a free descriptor.
  */
@@ -960,3 +1114,89 @@ dupfdopen(fdp, indx, dfd, mode, error)
 	}
 	/* NOTREACHED */
 }
+
+/*
+ * Get file structures.
+ */
+static int
+sysctl_kern_file SYSCTL_HANDLER_ARGS
+{
+	int error;
+	struct file *fp;
+
+	if (!req->oldptr) {
+		/*
+		 * overestimate by 10 files
+		 */
+		return (SYSCTL_OUT(req, 0, sizeof(filehead) + 
+				(nfiles + 10) * sizeof(struct file)));
+	}
+
+	error = SYSCTL_OUT(req, (caddr_t)&filehead, sizeof(filehead));
+	if (error)
+		return (error);
+
+	/*
+	 * followed by an array of file structures
+	 */
+	for (fp = filehead.lh_first; fp != NULL; fp = fp->f_list.le_next) {
+		error = SYSCTL_OUT(req, (caddr_t)fp, sizeof (struct file));
+		if (error)
+			return (error);
+	}
+	return (0);
+}
+
+SYSCTL_PROC(_kern, KERN_FILE, file, CTLTYPE_OPAQUE|CTLFLAG_RD,
+	0, 0, sysctl_kern_file, "S,file", "");
+
+SYSCTL_INT(_kern, KERN_MAXFILESPERPROC, maxfilesperproc,
+	CTLFLAG_RW, &maxfilesperproc, 0, "");
+
+SYSCTL_INT(_kern, KERN_MAXFILES, maxfiles, CTLFLAG_RW, &maxfiles, 0, "");
+
+static fildesc_devsw_installed = 0;
+#ifdef DEVFS
+static	void *devfs_token_stdin;
+static	void *devfs_token_stdout;
+static	void *devfs_token_stderr;
+static	void *devfs_token_fildesc[NUMFDESC];
+#endif
+
+static void 	fildesc_drvinit(void *unused)
+{
+	dev_t dev;
+#ifdef DEVFS
+	int fd;
+#endif
+
+	if( ! fildesc_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR,0);
+		cdevsw_add(&dev,&fildesc_cdevsw,NULL);
+		fildesc_devsw_installed = 1;
+#ifdef DEVFS
+		for (fd = 0; fd < NUMFDESC; fd++)
+			devfs_token_fildesc[fd] =
+				devfs_add_devswf(&fildesc_cdevsw, fd, DV_CHR,
+						 UID_BIN, GID_BIN, 0666,
+						 "fd/%d", fd);
+		devfs_token_stdin =
+			devfs_add_devswf(&fildesc_cdevsw, 0, DV_CHR,
+					 UID_ROOT, GID_WHEEL, 0666,
+					 "stdin", fd);
+		devfs_token_stdout =
+			devfs_add_devswf(&fildesc_cdevsw, 1, DV_CHR,
+					 UID_ROOT, GID_WHEEL, 0666,
+					 "stdout", fd);
+		devfs_token_stderr =
+			devfs_add_devswf(&fildesc_cdevsw, 2, DV_CHR,
+					 UID_ROOT, GID_WHEEL, 0666,
+					 "stderr", fd);
+#endif
+    	}
+}
+
+SYSINIT(fildescdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,
+					fildesc_drvinit,NULL)
+
+
