@@ -23,11 +23,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/kern/kern_exec.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_exec.c 78373 2001-06-16 23:43:32Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/signalvar.h>
 #include <sys/kernel.h>
 #include <sys/mount.h>
@@ -36,32 +37,58 @@
 #include <sys/acct.h>
 #include <sys/exec.h>
 #include <sys/imgact.h>
+#include <sys/imgact_elf.h>
 #include <sys/wait.h>
+#include <sys/proc.h>
 #include <sys/malloc.h>
+#include <sys/namei.h>
 #include <sys/sysent.h>
 #include <sys/syslog.h>
 #include <sys/shm.h>
+#include <sys/sysctl.h>
+#include <sys/vnode.h>
+#include <sys/buf.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 #include <vm/vm_kern.h>
+#include <vm/vm_extern.h>
+#include <vm/vm_object.h>
 
 #include <machine/reg.h>
 
-int *exec_copyout_strings __P((struct image_params *));
+static int *exec_copyout_strings __P((struct image_params *));
 
 static int exec_check_permissions(struct image_params *);
 
 /*
- * Exported to userland via sysctl..
+ * XXX trouble here if sizeof(caddr_t) != sizeof(int), other parts
+ * of the sysctl code also assumes this, and sizeof(int) == sizeof(long).
  */
-struct ps_strings *exec_ps_strings = PS_STRINGS;
-int exec_usrstack = USRSTACK;
+static struct ps_strings *ps_strings = PS_STRINGS;
+SYSCTL_INT(_kern, KERN_PS_STRINGS, ps_strings, 0, &ps_strings, 0, "");
+
+static caddr_t usrstack = (caddr_t)USRSTACK;
+SYSCTL_INT(_kern, KERN_USRSTACK, usrstack, 0, &usrstack, 0, "");
 
 /*
  * execsw_set is constructed for us by the linker.  Each of the items
  * is a pointer to a `const struct execsw', hence the double pointer here.
  */
-const struct execsw **execsw = (const struct execsw **)&execsw_set.ls_items[0];
+static const struct execsw **execsw = 
+	(const struct execsw **)&execsw_set.ls_items[0];
+
+#ifndef _SYS_SYSPROTO_H_
+struct execve_args {
+        char    *fname; 
+        char    **argv;
+        char    **envv; 
+};
+#endif
 
 /*
  * execve() system call.
@@ -77,6 +104,7 @@ execve(p, uap, retval)
 	int error, len, i;
 	struct image_params image_params, *imgp;
 	struct vattr attr;
+	struct buf *bp = NULL;
 
 	imgp = &image_params;
 
@@ -88,10 +116,12 @@ execve(p, uap, retval)
 	imgp->attr = &attr;
 	imgp->image_header = NULL;
 	imgp->argc = imgp->envc = 0;
+	imgp->argv0 = NULL;
 	imgp->entry_addr = 0;
 	imgp->vmspace_destroyed = 0;
 	imgp->interpreted = 0;
 	imgp->interpreter_name[0] = '\0';
+	imgp->auxargs = NULL;
 
 	/*
 	 * Allocate temporary demand zeroed space for argument and
@@ -121,43 +151,56 @@ interpret:
 		goto exec_fail;
 	}
 
-	imgp->vnodep = ndp->ni_vp;
-	if (imgp->vnodep == NULL) {
-		error = ENOEXEC;
-		goto exec_fail_dealloc;
-	}
+	imgp->vp = ndp->ni_vp;
 
 	/*
 	 * Check file permissions (also 'opens' file)
 	 */
 	error = exec_check_permissions(imgp);
-
-	/*
-	 * Lose the lock on the vnode. It's no longer needed, and must not
-	 * exist for the pagefault paging to work below.
-	 */
-	VOP_UNLOCK(imgp->vnodep);
-
-	if (error)
-		goto exec_fail_dealloc;
-
-	/*
-	 * Map the image header (first page) of the file into
-	 *	kernel address space
-	 */
-	error = vm_mmap(kernel_map,			/* map */
-			(vm_offset_t *)&imgp->image_header, /* address */
-			PAGE_SIZE,			/* size */
-			VM_PROT_READ, 			/* protection */
-			VM_PROT_READ, 			/* max protection */
-			0,	 			/* flags */
-			(caddr_t)imgp->vnodep,		/* vnode */
-			0);				/* offset */
 	if (error) {
-		uprintf("mmap failed: %d\n",error);
+		VOP_UNLOCK(imgp->vp);
 		goto exec_fail_dealloc;
 	}
 
+	/* XXX temporary hack for CODA filesystem XXX */
+#ifndef	CFS
+	/*
+	 * Get the image header, which we define here as meaning the first
+	 * page of the executable.
+	 */
+	if (imgp->vp->v_object && imgp->vp->v_mount &&
+	    imgp->vp->v_mount->mnt_stat.f_iosize >= PAGE_SIZE &&
+	    imgp->vp->v_object->un_pager.vnp.vnp_size >=
+	    imgp->vp->v_mount->mnt_stat.f_iosize) {
+		/*
+		 * Get a buffer with (at least) the first page.
+		 */
+		error = bread(imgp->vp, 0, imgp->vp->v_mount->mnt_stat.f_iosize,
+		     p->p_ucred, &bp);
+		imgp->image_header = bp->b_data;
+	} else 
+#endif
+	{
+		int resid;
+
+		/*
+		 * The filesystem block size is too small, so do this the hard
+		 * way. Malloc some space and read PAGE_SIZE worth of the image
+		 * header into it.
+		 */
+		imgp->image_header = malloc(PAGE_SIZE, M_TEMP, M_WAITOK);
+		error = vn_rdwr(UIO_READ, imgp->vp, (void *)imgp->image_header, PAGE_SIZE, 0,
+		    UIO_SYSSPACE, IO_NODELOCKED, p->p_ucred, &resid, p);
+		/*
+		 * Clear out any remaining junk.
+		 */
+		if (!error && resid)
+			bzero((char *)imgp->image_header + PAGE_SIZE - resid, resid);
+	}
+	VOP_UNLOCK(imgp->vp);
+	if (error)
+		goto exec_fail_dealloc;
+
 	/*
 	 * Loop through list of image activators, calling each one.
 	 *	If there is no match, the activator returns -1. If there
@@ -171,19 +214,21 @@ interpret:
 			error = (*execsw[i]->ex_imgact)(imgp);
 		else
 			continue;
-
 		if (error == -1)
 			continue;
 		if (error)
 			goto exec_fail_dealloc;
 		if (imgp->interpreted) {
+			/* free old bp/image_header */
+			if (bp != NULL) {
+				brelse(bp);
+				bp = NULL;
+			} else
+				free((void *)imgp->image_header, M_TEMP);
+			imgp->image_header = NULL;
 			/* free old vnode and name buffer */
 			vrele(ndp->ni_vp);
 			FREE(ndp->ni_cnd.cn_pnbuf, M_NAMEI);
-			if (vm_map_remove(kernel_map, (vm_offset_t)imgp->image_header,
-			    (vm_offset_t)imgp->image_header + PAGE_SIZE))
-				panic("execve: header dealloc failed (1)");
-
 			/* set new name to that of the interpreter */
 			NDINIT(ndp, LOOKUP, LOCKLEAF | FOLLOW | SAVENAME,
 			    UIO_SYSSPACE, imgp->interpreter_name, p);
@@ -213,6 +258,18 @@ interpret:
 	else
 		suword(--stack_base, imgp->argc);
 
+	/*
+	 * For security and other reasons, the file descriptor table cannot
+	 * be shared after an exec.
+	 */
+	if (p->p_fd->fd_refcnt > 1) {
+		struct filedesc *tmp;
+
+		tmp = fdcopy(p);
+		fdfree(p);
+		p->p_fd = tmp;
+	}
+
 	/* close files on exec */
 	fdcloseexec(p);
 
@@ -225,7 +282,7 @@ interpret:
 	p->p_comm[len] = 0;
 
 	/*
-	 * mark as executable, wakeup any process that was vforked and tell
+	 * mark as execed, wakeup the process that vforked (if any) and tell
 	 * it that it now has it's own resources back
 	 */
 	p->p_flag |= P_EXEC;
@@ -235,15 +292,21 @@ interpret:
 	}
 
 	/*
-	 * Implement image setuid/setgid. Disallow if the process is
-	 * being traced.
+	 * Implement image setuid/setgid.
+	 *
+	 * Don't honor setuid/setgid if the filesystem prohibits it or if
+	 * the process is being traced.
 	 */
-	if ((attr.va_mode & (VSUID | VSGID)) &&
+	if ((attr.va_mode & VSUID && p->p_ucred->cr_uid != attr.va_uid ||
+	     attr.va_mode & VSGID && p->p_ucred->cr_gid != attr.va_gid) &&
+	    (imgp->vp->v_mount->mnt_flag & MNT_NOSUID) == 0 &&
 	    (p->p_flag & P_TRACED) == 0) {
 		/*
 		 * Turn off syscall tracing for set-id programs, except for
-		 * root.
+		 * root.  Record any set-id flags first to make sure that
+		 * we do not regain any tracing during a possible block.
 		 */
+		p->p_flag |= P_SUGID;
 		if (p->p_tracep && suser(p->p_ucred, &p->p_acflag)) {
 			p->p_traceflag = 0;
 			vrele(p->p_tracep);
@@ -256,8 +319,8 @@ interpret:
 		if (attr.va_mode & VSUID)
 			p->p_ucred->cr_uid = attr.va_uid;
 		if (attr.va_mode & VSGID)
-			p->p_ucred->cr_groups[0] = attr.va_gid;
-		p->p_flag |= P_SUGID;
+			p->p_ucred->cr_gid = attr.va_gid;
+		setugidsafety(p);
 	} else {
 	        if (p->p_ucred->cr_uid == p->p_cred->p_ruid &&
 		    p->p_ucred->cr_gid == p->p_cred->p_rgid)
@@ -295,9 +358,10 @@ interpret:
 	 * free various allocated resources
 	 */
 	kmem_free_wakeup(exec_map, (vm_offset_t)imgp->stringbase, ARG_MAX);
-	if (vm_map_remove(kernel_map, (vm_offset_t)imgp->image_header,
-	    (vm_offset_t)imgp->image_header + PAGE_SIZE))
-		panic("execve: header dealloc failed (2)");
+	if (bp != NULL)
+		brelse(bp);
+	else if (imgp->image_header != NULL)
+		free((void *)imgp->image_header, M_TEMP);
 	vrele(ndp->ni_vp);
 	FREE(ndp->ni_cnd.cn_pnbuf, M_NAMEI);
 
@@ -306,10 +370,10 @@ interpret:
 exec_fail_dealloc:
 	if (imgp->stringbase != NULL)
 		kmem_free_wakeup(exec_map, (vm_offset_t)imgp->stringbase, ARG_MAX);
-	if (imgp->image_header && imgp->image_header != (char *)-1)
-		if (vm_map_remove(kernel_map, (vm_offset_t)imgp->image_header,
-		    (vm_offset_t)imgp->image_header + PAGE_SIZE))
-			panic("execve: header dealloc failed (3)");
+	if (bp != NULL)
+		brelse(bp);
+	else if (imgp->image_header != NULL)
+		free((void *)imgp->image_header, M_TEMP);
 	if (ndp->ni_vp) {
 		vrele(ndp->ni_vp);
 		FREE(ndp->ni_cnd.cn_pnbuf, M_NAMEI);
@@ -342,15 +406,14 @@ exec_new_vmspace(imgp)
 	imgp->vmspace_destroyed = 1;
 
 	/* Blow away entire process VM */
-#ifdef SYSVSHM
 	if (vmspace->vm_shm)
 		shmexit(imgp->proc);
-#endif
+	pmap_remove_pages(&vmspace->vm_pmap, 0, USRSTACK);
 	vm_map_remove(&vmspace->vm_map, 0, USRSTACK);
 
 	/* Allocate a new stack */
 	error = vm_map_find(&vmspace->vm_map, NULL, 0, (vm_offset_t *)&stack_addr,
-	    SGROWSIZ, FALSE);
+	    SGROWSIZ, FALSE, VM_PROT_ALL, VM_PROT_ALL, 0);
 	if (error)
 		return(error);
 
@@ -381,20 +444,29 @@ exec_extract_strings(imgp)
 	argv = imgp->uap->argv;
 
 	if (argv) {
-		while ((argp = (caddr_t) fuword(argv++))) {
-			if (argp == (caddr_t) -1)
-				return (EFAULT);
-			if ((error = copyinstr(argp, imgp->stringp,
-			    imgp->stringspace, &length))) {
-				if (error == ENAMETOOLONG)
-					return(E2BIG);
-				return (error);
-			}
-			imgp->stringspace -= length;
-			imgp->stringp += length;
-			imgp->argc++;
+		argp = (caddr_t) fuword(argv);
+		if (argp == (caddr_t) -1)
+			return (EFAULT);
+		if (argp)
+			argv++;
+		if (imgp->argv0)
+			argp = imgp->argv0;
+		if (argp) {
+			do {
+				if (argp == (caddr_t) -1)
+					return (EFAULT);
+				if ((error = copyinstr(argp, imgp->stringp,
+				    imgp->stringspace, &length))) {
+					if (error == ENAMETOOLONG)
+						return(E2BIG);
+					return (error);
+				}
+				imgp->stringspace -= length;
+				imgp->stringp += length;
+				imgp->argc++;
+			} while ((argp = (caddr_t) fuword(argv++)));
 		}
-	}
+	}	
 
 	/*
 	 * extract environment strings
@@ -435,20 +507,43 @@ exec_copyout_strings(imgp)
 	char *stringp, *destp;
 	int *stack_base;
 	struct ps_strings *arginfo;
+	int szsigcode;
 
 	/*
 	 * Calculate string base and vector table pointers.
+	 * Also deal with signal trampoline code for this exec type.
 	 */
 	arginfo = PS_STRINGS;
-	destp =	(caddr_t)arginfo - SPARE_USRSPACE -
+	szsigcode = *(imgp->proc->p_sysent->sv_szsigcode);
+	destp =	(caddr_t)arginfo - szsigcode - SPARE_USRSPACE -
 		roundup((ARG_MAX - imgp->stringspace), sizeof(char *));
 
 	/*
-	 * The '+ 2' is for the null pointers at the end of each of the
-	 *	arg and	env vector sets
+	 * install sigcode
 	 */
-	vectp = (char **) (destp -
-		(imgp->argc + imgp->envc + 2) * sizeof(char *));
+	if (szsigcode)
+		copyout(imgp->proc->p_sysent->sv_sigcode,
+			((caddr_t)arginfo - szsigcode), szsigcode);
+
+	/*
+	 * If we have a valid auxargs ptr, prepare some room
+	 * on the stack.
+	 */
+	if (imgp->auxargs)
+	/*
+	 * The '+ 2' is for the null pointers at the end of each of the
+	 * arg and env vector sets, and 'AT_COUNT*2' is room for the
+	 * ELF Auxargs data.
+	 */
+		vectp = (char **)(destp - (imgp->argc + imgp->envc + 2 +
+				  AT_COUNT*2) * sizeof(char*));
+	else 
+	/*
+	 * The '+ 2' is for the null pointers at the end of each of the
+	 * arg and env vector sets
+	 */
+		vectp = (char **)
+			(destp - (imgp->argc + imgp->envc + 2) * sizeof(char*));
 
 	/*
 	 * vectp also becomes our initial stack base
@@ -481,7 +576,7 @@ exec_copyout_strings(imgp)
 	}
 
 	/* a null vector table pointer seperates the argp's from the envp's */
-	suword(vectp++, NULL);
+	suword(vectp++, 0);
 
 	suword(&arginfo->ps_envstr, (int)vectp);
 	suword(&arginfo->ps_nenvstr, envc);
@@ -497,7 +592,7 @@ exec_copyout_strings(imgp)
 	}
 
 	/* end of vector table is a null pointer */
-	suword(vectp, NULL);
+	suword(vectp, 0);
 
 	return (stack_base);
 }
@@ -511,18 +606,10 @@ exec_check_permissions(imgp)
 	struct image_params *imgp;
 {
 	struct proc *p = imgp->proc;
-	struct vnode *vp = imgp->vnodep;
+	struct vnode *vp = imgp->vp;
 	struct vattr *attr = imgp->attr;
 	int error;
 
-	/*
-	 * Check number of open-for-writes on the file and deny execution
-	 *	if there are any.
-	 */
-	if (vp->v_writecount) {
-		return (ETXTBSY);
-	}
-
 	/* Get file attributes */
 	error = VOP_GETATTR(vp, attr, p->p_ucred, p);
 	if (error)
@@ -548,22 +635,24 @@ exec_check_permissions(imgp)
 	if (attr->va_size == 0)
 		return (ENOEXEC);
 
-	/*
-	 * Disable setuid/setgid if the filesystem prohibits it or if
-	 *	the process is being traced.
-	 */
-        if ((vp->v_mount->mnt_flag & MNT_NOSUID) || (p->p_flag & P_TRACED))
-		attr->va_mode &= ~(VSUID | VSGID);
-
 	/*
 	 *  Check for execute permission to file based on current credentials.
-	 *	Then call filesystem specific open routine (which does nothing
-	 *	in the general case).
 	 */
 	error = VOP_ACCESS(vp, VEXEC, p->p_ucred, p);
 	if (error)
 		return (error);
 
+	/*
+	 * Check number of open-for-writes on the file and deny execution
+	 * if there are any.
+	 */
+	if (vp->v_writecount)
+		return (ETXTBSY);
+
+	/*
+	 * Call filesystem specific open routine (which does nothing in the
+	 * general case).
+	 */
 	error = VOP_OPEN(vp, FREAD, p->p_ucred, p);
 	if (error)
 		return (error);
