@@ -36,11 +36,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_exit.c	8.7 (Berkeley) 2/12/94
- * $FreeBSD: stable/2.1/sys/kern/kern_exit.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_exit.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_ktrace.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/sysent.h>
 #include <sys/ioctl.h>
 #include <sys/proc.h>
@@ -57,41 +60,51 @@
 #include <sys/resourcevar.h>
 #include <sys/signalvar.h>
 #include <sys/ptrace.h>
+#include <sys/acct.h>		/* for acct_process() function prototype */
 #include <sys/filedesc.h>
-
-#ifdef SYSVSHM
 #include <sys/shm.h>
-#endif
-#ifdef SYSVSEM
 #include <sys/sem.h>
-#endif
 
-#include <machine/cpu.h>
 #ifdef COMPAT_43
 #include <machine/reg.h>
 #include <machine/psl.h>
 #endif
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 #include <vm/vm_kern.h>
 
+static int wait1 __P((struct proc *, struct wait_args *, int [], int));
+
+/*
+ * callout list for things to do at exit time
+ */
+typedef struct exit_list_element {
+	struct exit_list_element *next;
+	exitlist_fn function;
+} *ele_p;
+
+static ele_p exit_list;
+
 /*
  * exit --
  *	Death of process.
  */
-struct rexit_args {
-	int	rval;
-};
-__dead void
+void
 exit(p, uap, retval)
 	struct proc *p;
-	struct rexit_args *uap;
+	struct rexit_args /* {
+		int	rval;
+	} */ *uap;
 	int *retval;
 {
 
 	exit1(p, W_EXITCODE(uap->rval, 0));
 	/* NOTREACHED */
-	while (1);
 }
 
 /*
@@ -99,14 +112,14 @@ exit(p, uap, retval)
  * to zombie, and unlink proc from allproc and parent's lists.  Save exit
  * status and rusage for wait().  Check for child processes and orphan them.
  */
-__dead void
+void
 exit1(p, rv)
 	register struct proc *p;
 	int rv;
 {
 	register struct proc *q, *nq;
-	register struct proc **pp;
 	register struct vmspace *vm;
+	ele_p ep = exit_list;
 
 	if (p->p_pid == 1) {
 		printf("init died (signal %d, exit %d)\n",
@@ -116,6 +129,16 @@ exit1(p, rv)
 #ifdef PGINPROF
 	vmsizmon();
 #endif
+	/* 
+	 * Check if any LKMs need anything done at process exit.
+	 * e.g. SYSV IPC stuff
+	 * XXX what if one of these generates an error?
+	 */
+	while (ep) {
+		(*ep->function)(p);
+		ep = ep->next;
+	}
+
 	if (p->p_flag & P_PROFIL)
 		stopprofclock(p);
 	MALLOC(p->p_ru, struct rusage *, sizeof(struct rusage),
@@ -136,16 +159,21 @@ exit1(p, rv)
 	 */
 	fdfree(p);
 
-#ifdef SYSVSEM
+	/*
+	 * Delete select() buffers
+	 */
+	if (p->p_selbits)
+		free (p->p_selbits, M_SELECT);
+
+	/*
+	 * XXX Shutdown SYSV semaphores
+	 */
 	semexit(p);
-#endif
 
 	/* The next two chunks should probably be moved to vmspace_exit. */
 	vm = p->p_vmspace;
-#ifdef SYSVSHM
 	if (vm->vm_shm)
 		shmexit(p);
-#endif
 	/*
 	 * Release user portion of address space.
 	 * This releases references to vnodes,
@@ -154,9 +182,12 @@ exit1(p, rv)
 	 * Can't free the entire vmspace as the kernel stack
 	 * may be mapped within that space also.
 	 */
-	if (vm->vm_refcnt == 1)
+	if (vm->vm_refcnt == 1) {
+		pmap_remove_pages(&vm->vm_pmap, VM_MIN_ADDRESS,
+		    VM_MAXUSER_ADDRESS);
 		(void) vm_map_remove(&vm->vm_map, VM_MIN_ADDRESS,
 		    VM_MAXUSER_ADDRESS);
+	}
 
 	if (SESS_LEADER(p)) {
 		register struct session *sp = p->p_session;
@@ -168,7 +199,7 @@ exit1(p, rv)
 			 * drain controlling terminal
 			 * and revoke access to controlling terminal.
 			 */
-			if (sp->s_ttyp->t_session == sp) {
+			if (sp->s_ttyp && (sp->s_ttyp->t_session == sp)) {
 				if (sp->s_ttyp->t_pgrp)
 					pgsignal(sp->s_ttyp->t_pgrp, SIGHUP, 1);
 				(void) ttywait(sp->s_ttyp);
@@ -191,6 +222,11 @@ exit1(p, rv)
 		sp->s_leader = NULL;
 	}
 	fixjobc(p, p->p_pgrp, 0);
+	if (p->p_limit->p_refcnt > 1 &&
+	    (p->p_limit->p_lflags & PL_SHAREMOD) == 0) {
+		p->p_limit->p_refcnt--;
+		p->p_limit = limcopy(p->p_limit);
+	}
 	p->p_rlimit[RLIMIT_FSIZE].rlim_cur = RLIM_INFINITY;
 	(void)acct_process(p);
 #ifdef KTRACE
@@ -205,34 +241,19 @@ exit1(p, rv)
 	 * Remove proc from allproc queue and pidhash chain.
 	 * Place onto zombproc.  Unlink from parent's child list.
 	 */
-	if ((*p->p_prev = p->p_next))
-		p->p_next->p_prev = p->p_prev;
-	if ((p->p_next = zombproc))
-		p->p_next->p_prev = &p->p_next;
-	p->p_prev = &zombproc;
-	zombproc = p;
+	LIST_REMOVE(p, p_list);
+	LIST_INSERT_HEAD(&zombproc, p, p_list);
 	p->p_stat = SZOMB;
 
-	for (pp = &pidhash[PIDHASH(p->p_pid)]; *pp; pp = &(*pp)->p_hash)
-		if (*pp == p) {
-			*pp = p->p_hash;
-			goto done;
-		}
-	panic("exit");
-done:
+	LIST_REMOVE(p, p_hash);
 
-	if (p->p_cptr)		/* only need this if any child is S_ZOMB */
+	q = p->p_children.lh_first;
+	if (q)		/* only need this if any child is S_ZOMB */
 		wakeup((caddr_t) initproc);
-	for (q = p->p_cptr; q != NULL; q = nq) {
-		nq = q->p_osptr;
-		if (nq != NULL)
-			nq->p_ysptr = NULL;
-		if (initproc->p_cptr)
-			initproc->p_cptr->p_ysptr = q;
-		q->p_osptr = initproc->p_cptr;
-		q->p_ysptr = NULL;
-		initproc->p_cptr = q;
-
+	for (; q != 0; q = nq) {
+		nq = q->p_sibling.le_next;
+		LIST_REMOVE(q, p_sibling);
+		LIST_INSERT_HEAD(&initproc->p_children, q, p_sibling);
 		q->p_pptr = initproc;
 		/*
 		 * Traced processes are killed
@@ -243,7 +264,6 @@ done:
 			psignal(q, SIGKILL);
 		}
 	}
-	p->p_cptr = NULL;
 
 	/*
 	 * Save exit status and final rusage info, adding in child rusage
@@ -285,13 +305,13 @@ done:
 	 * The address space is released by "vmspace_free(p->p_vmspace)";
 	 * This is machine-dependent, as we may have to change stacks
 	 * or ensure that the current one isn't reallocated before we
-	 * finish.  cpu_exit will end with a call to cpu_swtch(), finishing
+	 * finish.  cpu_exit will end with a call to cpu_switch(), finishing
 	 * our execution (pun intended).
 	 */
 	cpu_exit(p);
 }
 
-#if defined(COMPAT_43) || defined(COMPAT_IBCS2)
+#ifdef COMPAT_43
 #if defined(hp300) || defined(luna68k)
 #include <machine/frame.h>
 #define GETPS(rp)	((struct frame *)(rp))->f_sr
@@ -302,27 +322,30 @@ done:
 int
 owait(p, uap, retval)
 	struct proc *p;
-	register struct wait_args *uap;
+	register struct owait_args /* {
+		int     dummy;
+	} */ *uap;
 	int *retval;
 {
+	struct wait_args w;
 
 #ifdef PSL_ALLCC
 	if ((GETPS(p->p_md.md_regs) & PSL_ALLCC) != PSL_ALLCC) {
-		uap->options = 0;
-		uap->rusage = NULL;
+		w.options = 0;
+		w.rusage = NULL;
 	} else {
-		uap->options = p->p_md.md_regs[R0];
-		uap->rusage = (struct rusage *)p->p_md.md_regs[R1];
+		w.options = p->p_md.md_regs[R0];
+		w.rusage = (struct rusage *)p->p_md.md_regs[R1];
 	}
 #else
-	uap->options = 0;
-	uap->rusage = NULL;
+	w.options = 0;
+	w.rusage = NULL;
 #endif
-	uap->pid = WAIT_ANY;
-	uap->status = NULL;
-	uap->compat = 1;
-	return (wait1(p, uap, retval));
+	w.pid = WAIT_ANY;
+	w.status = NULL;
+	return (wait1(p, &w, retval, 1));
 }
+#endif /* COMPAT_43 */
 
 int
 wait4(p, uap, retval)
@@ -331,22 +354,24 @@ wait4(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat = 0;
-	return (wait1(p, uap, retval));
+	return (wait1(p, uap, retval, 0));
 }
-#else
-#define	wait1	wait4
-#endif
 
-int
-wait1(q, uap, retval)
+static int
+wait1(q, uap, retval, compat)
 	register struct proc *q;
-	register struct wait_args *uap;
+	register struct wait_args /* {
+		int pid;
+		int *status;
+		int options;
+		struct rusage *rusage;
+	} */ *uap;
 	int retval[];
+	int compat;
 {
 	register int nfound;
 	register struct proc *p, *t;
-	int status, error, sig;
+	int status, error;
 
 	if (uap->pid == 0)
 		uap->pid = -q->p_pgid;
@@ -356,20 +381,11 @@ wait1(q, uap, retval)
 #endif
 loop:
 	nfound = 0;
-	for (p = q->p_cptr; p; p = p->p_osptr) {
+	for (p = q->p_children.lh_first; p != 0; p = p->p_sibling.le_next) {
 		if (uap->pid != WAIT_ANY &&
 		    p->p_pid != uap->pid && p->p_pgid != -uap->pid)
 			continue;
 		nfound++;
-#if defined(COMPAT_43) || defined(COMPAT_IBCS2)
-		if (q->p_sysent->sv_sigtbl) {
-			if ((p->p_xstat & 0xff) < q->p_sysent->sv_sigsize)
-				sig = q->p_sysent->sv_sigtbl[p->p_xstat & 0xff];
-			else
-				sig = q->p_sysent->sv_sigsize + 1;
-		} else
-			sig = p->p_xstat;
-#endif
 		if (p->p_stat == SZOMB) {
 			/* charge childs scheduling cpu usage to parent */
 			if (curproc->p_pid != 1) {
@@ -378,9 +394,9 @@ loop:
 			}
 
 			retval[0] = p->p_pid;
-#if defined(COMPAT_43) || defined(COMPAT_IBCS2)
-			if (uap->compat)
-				retval[1] = sig;
+#ifdef COMPAT_43
+			if (compat)
+				retval[1] = p->p_xstat;
 			else
 #endif
 			if (uap->status) {
@@ -413,6 +429,12 @@ loop:
 			 */
 			(void)chgproccnt(p->p_cred->p_ruid, -1);
 
+			/*
+			 * Release reference to text vnode
+			 */
+			if (p->p_textvp)
+				vrele(p->p_textvp);
+
 			/*
 			 * Free up credentials.
 			 */
@@ -422,25 +444,13 @@ loop:
 				p->p_cred = NULL;
 			}
 
-			/*
-			 * Release reference to text vnode
-			 */
-			if (p->p_textvp)
-				vrele(p->p_textvp);
-
 			/*
 			 * Finally finished with old proc entry.
 			 * Unlink it from its process group and free it.
 			 */
 			leavepgrp(p);
-			if ((*p->p_prev = p->p_next))	/* off zombproc */
-				p->p_next->p_prev = p->p_prev;
-			if ((q = p->p_ysptr))
-				q->p_osptr = p->p_osptr;
-			if ((q = p->p_osptr))
-				q->p_ysptr = p->p_ysptr;
-			if ((q = p->p_pptr)->p_cptr == p)
-				q->p_cptr = p->p_osptr;
+			LIST_REMOVE(p, p_list);	/* off zombproc */
+			LIST_REMOVE(p, p_sibling);
 
 			/*
 			 * Give machine-dependent layer a chance
@@ -456,9 +466,9 @@ loop:
 		    (p->p_flag & P_TRACED || uap->options & WUNTRACED)) {
 			p->p_flag |= P_WAITED;
 			retval[0] = p->p_pid;
-#if defined(COMPAT_43) || defined(COMPAT_IBCS2)
-			if (uap->compat) {
-				retval[1] = W_STOPCODE(sig);
+#ifdef COMPAT_43
+			if (compat) {
+				retval[1] = W_STOPCODE(p->p_xstat);
 				error = 0;
 			} else
 #endif
@@ -490,28 +500,65 @@ proc_reparent(child, parent)
 	register struct proc *child;
 	register struct proc *parent;
 {
-	register struct proc *o;
-	register struct proc *y;
 
 	if (child->p_pptr == parent)
 		return;
 
-	/* fix up the child linkage for the old parent */
-	o = child->p_osptr;
-	y = child->p_ysptr;
-	if (y)
-		y->p_osptr = o;
-	if (o)
-		o->p_ysptr = y;
-	if (child->p_pptr->p_cptr == child)
-		child->p_pptr->p_cptr = o;
-
-	/* fix up child linkage for new parent */
-	o = parent->p_cptr;
-	if (o)
-		o->p_ysptr = child;
-	child->p_osptr = o;
-	child->p_ysptr = NULL;
-	parent->p_cptr = child;
+	LIST_REMOVE(child, p_sibling);
+	LIST_INSERT_HEAD(&parent->p_children, child, p_sibling);
 	child->p_pptr = parent;
 }
+
+/*
+ * The next two functions are to handle adding/deleting items on the
+ * exit callout list
+ * 
+ * at_exit():
+ * Take the arguments given and put them onto the exit callout list,
+ * However first make sure that it's not already there.
+ * returns 0 on success.
+ */
+int
+at_exit(exitlist_fn function)
+{
+	ele_p ep;
+
+	/* Be noisy if the programmer has lost track of things */
+	if (rm_at_exit(function)) 
+		printf("exit callout entry already present\n");
+	ep = malloc(sizeof(*ep), M_TEMP, M_NOWAIT);
+	if (ep == NULL)
+		return (ENOMEM);
+	ep->next = exit_list;
+	ep->function = function;
+	exit_list = ep;
+	return (0);
+}
+/*
+ * Scan the exit callout list for the given items and remove them.
+ * Returns the number of items removed.
+ * Logically this can only be 0 or 1.
+ */
+int
+rm_at_exit(exitlist_fn function)
+{
+	ele_p *epp, ep;
+	int count;
+
+	count = 0;
+	epp = &exit_list;
+	ep = *epp;
+	while (ep) {
+		if (ep->function == function) {
+			*epp = ep->next;
+			free(ep, M_TEMP);
+			count++;
+		} else {
+			epp = &ep->next;
+		}
+		ep = *epp;
+	}
+	return (count);
+}
+
+
