@@ -30,18 +30,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/kern/kern_lkm.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_lkm.c 50950 1999-09-05 08:39:22Z peter $
  */
 
-/*
- * XXX it's not really safe to unload *any* of the types which are
- * currently loadable; e.g. you could unload a syscall which was being
- * blocked in, etc.  In the long term, a solution should be come up
- * with, but "not right now." -- cgd
- */
+#include "opt_no_lkm.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/ioctl.h>
 #include <sys/tty.h>
 #include <sys/conf.h>
@@ -55,10 +51,16 @@
 #include <sys/exec.h>
 #include <sys/imgact.h>
 #include <sys/lkm.h>
+#include <sys/vnode.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <vm/vm.h>
 #include <vm/vm_param.h>
 #include <vm/vm_kern.h>
+#include <vm/vm_extern.h>
+
 
 #define PAGESIZE 1024		/* kmem_alloc() allocation quantum */
 
@@ -81,8 +83,25 @@ static int	lkm_state = LKMS_IDLE;
 static struct lkm_table	lkmods[MAXLKMS];	/* table of loaded modules */
 static struct lkm_table	*curp;			/* global for in-progress ops */
 
+static int	_lkm_dev __P((struct lkm_table *lkmtp, int cmd));
+static int	_lkm_exec __P((struct lkm_table *lkmtp, int cmd));
+static int	_lkm_vfs __P((struct lkm_table *lkmtp, int cmd));
+static int	_lkm_syscall __P((struct lkm_table *lkmtp, int cmd));
+static void	lkmunreserve __P((void));
+
+static	d_open_t	lkmcopen;
+static	d_close_t	lkmcclose;
+static	d_ioctl_t	lkmcioctl;
+
+#define CDEV_MAJOR 32
+static struct cdevsw lkmc_cdevsw = 
+	{ lkmcopen,	lkmcclose,	noread,		nowrite,	/*32*/
+	  lkmcioctl,	nostop,		nullreset,	nodevtotty,
+	  noselect,	nommap,		NULL,	"lkm",	NULL,	-1 };
+
+
 /*ARGSUSED*/
-int
+static	int
 lkmcopen(dev, flag, devtype, p)
 	dev_t dev;
 	int flag;
@@ -132,7 +151,7 @@ lkmunreserve()
 	 * Actually unreserve the memory
 	 */
 	if (curp && curp->area) {
-		kmem_free(kmem_map, curp->area, curp->size);/**/
+		kmem_free(kernel_map, curp->area, curp->size);/**/
 		curp->area = 0;
 		if (curp->private.lkm_any != NULL)
 			curp->private.lkm_any = NULL;
@@ -141,7 +160,7 @@ lkmunreserve()
 	lkm_state = LKMS_IDLE;
 }
 
-int
+static	int
 lkmcclose(dev, flag, mode, p)
 	dev_t dev;
 	int flag;
@@ -173,7 +192,7 @@ lkmcclose(dev, flag, mode, p)
 }
 
 /*ARGSUSED*/
-int
+static	int
 lkmcioctl(dev, cmd, data, flag, p)
 	dev_t dev;
 	int cmd;
@@ -191,8 +210,8 @@ lkmcioctl(dev, cmd, data, flag, p)
 
 	switch(cmd) {
 	case LMRESERV:		/* reserve pages for a module */
-		if ((flag & FWRITE) == 0 || securelevel > 0)
-			/* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0) 
+			/* only allow this if writing and insecure */
 			return EPERM;
 
 		resrvp = (struct lmc_resrv *)data;
@@ -217,7 +236,7 @@ lkmcioctl(dev, cmd, data, flag, p)
 		 */
 		curp->size = resrvp->size;
 
-		curp->area = kmem_alloc(kmem_map, curp->size);/**/
+		curp->area = kmem_alloc(kernel_map, curp->size);/**/
 
 		curp->offset = 0;		/* load offset */
 
@@ -233,7 +252,7 @@ lkmcioctl(dev, cmd, data, flag, p)
 
 	case LMLOADBUF:		/* Copy in; stateful, follows LMRESERV */
 		if ((flag & FWRITE) == 0 || securelevel > 0)
-			/* only allow this if writing */
+			/* only allow this if writing and insecure */
 			return EPERM;
 
 		loadbufp = (struct lmc_loadbuf *)data;
@@ -269,7 +288,7 @@ lkmcioctl(dev, cmd, data, flag, p)
 
 	case LMUNRESRV:		/* discard reserved pages for a module */
 		if ((flag & FWRITE) == 0 || securelevel > 0)
-			/* only allow this if writing */
+			/* only allow this if writing and insecure */
 			return EPERM;
 
 		lkmunreserve();	/* coerce state to LKM_IDLE */
@@ -280,7 +299,7 @@ lkmcioctl(dev, cmd, data, flag, p)
 
 	case LMREADY:		/* module loaded: call entry */
 		if ((flag & FWRITE) == 0 || securelevel > 0)
-			/* only allow this if writing */
+			/* only allow this if writing or insecure */
 			return EPERM;
 
 		switch (lkm_state) {
@@ -299,7 +318,9 @@ lkmcioctl(dev, cmd, data, flag, p)
 			return ENXIO;
 		}
 
-		curp->entry = (int (*)()) (*((int *) (data)));
+		/* XXX gack */
+		curp->entry = (int (*) __P((struct lkm_table *, int, int)))
+			      (*((int *)data));
 
 		/* call entry(load)... (assigns "private" portion) */
 		err = (*(curp->entry))(curp, LKM_E_LOAD, LKM_VERSION);
@@ -343,7 +364,7 @@ lkmcioctl(dev, cmd, data, flag, p)
 
 	case LMUNLOAD:		/* unload a module */
 		if ((flag & FWRITE) == 0 || securelevel > 0)
-			/* only allow this if writing */
+			/* only allow this if writing and insecure */
 			return EPERM;
 
 		unloadp = (struct lmc_unload *)data;
@@ -479,24 +500,6 @@ lkmnosys(p, args, retval)
 	return(nosys(p, args, retval));
 }
 
-/*
- * Acts like "enodev", but can be identified in cdevsw and bdevsw for
- * dynamic driver major number assignment for a limited number of
- * drivers.
- *
- * Place holder for device switch slots reserved for loadable modules.
- */
-int
-lkmenodev(dev, flags, fmt, p)
-	dev_t dev;
-	int flags;
-	int fmt;
-	struct proc *p;
-{
-
-	return(enodev());
-}
-
 int
 lkmexists(lkmtp)
 	struct lkm_table *lkmtp;
@@ -544,7 +547,8 @@ _lkm_syscall(lkmtp, cmd)
 			 * Search the table looking for a slot...
 			 */
 			for (i = 0; i < aout_sysvec.sv_size; i++)
-				if (aout_sysvec.sv_table[i].sy_call == lkmnosys)
+				if (aout_sysvec.sv_table[i].sy_call ==
+				    (sy_call_t *)lkmnosys)
 					break;		/* found it! */
 			/* out of allocable slots? */
 			if (i == aout_sysvec.sv_size) {
@@ -634,8 +638,8 @@ _lkm_vfs(lkmtp, cmd)
 
 		/* like in vfs_op_init */
 		for(i = 0; args->lkm_vnodeops->ls_items[i]; i++) {
-			struct vnodeopv_desc *opv =
-			  (struct vnodeopv_desc *)args->lkm_vnodeops->ls_items[i];
+			const struct vnodeopv_desc *opv =
+				args->lkm_vnodeops->ls_items[i];
 			*(opv->opv_desc_vector_p) = NULL;
 		}
 		vfs_opv_init((struct vnodeopv_desc **)args->lkm_vnodeops->ls_items);
@@ -679,6 +683,7 @@ _lkm_dev(lkmtp, cmd)
 {
 	struct lkm_dev *args = lkmtp->private.lkm_dev;
 	int i;
+	dev_t descrip;
 	int err = 0;
 
 	switch(cmd) {
@@ -688,64 +693,27 @@ _lkm_dev(lkmtp, cmd)
 			return(EEXIST);
 		switch(args->lkm_devtype) {
 		case LM_DT_BLOCK:
-			if ((i = args->lkm_offset) == -1) {	/* auto */
-				/*
-				 * Search the table looking for a slot...
-				 */
-				for (i = 0; i < nblkdev; i++)
-					if (bdevsw[i].d_open == lkmenodev)
-						break;		/* found it! */
-				/* out of allocable slots? */
-				if (i == nblkdev) {
-					err = ENFILE;
-					break;
-				}
-			} else {				/* assign */
-				if (i < 0 || i >= nblkdev) {
-					err = EINVAL;
-					break;
-				}
+			if ((i = args->lkm_offset) == -1)
+				descrip = (dev_t) -1;
+			else
+				descrip = makedev(args->lkm_offset,0);
+			if ( err = bdevsw_add(&descrip, args->lkm_dev.bdev,
+					&(args->lkm_olddev.bdev))) {
+				break;
 			}
-
-			/* save old */
-			bcopy(&bdevsw[i], &(args->lkm_olddev.bdev), sizeof(struct bdevsw));
-
-			/* replace with new */
-			bcopy(args->lkm_dev.bdev, &bdevsw[i], sizeof(struct bdevsw));
-
-			/* done! */
-			args->lkm_offset = i;	/* slot in bdevsw[] */
+			args->lkm_offset = major(descrip) ;
 			break;
 
 		case LM_DT_CHAR:
-			if ((i = args->lkm_offset) == -1) {	/* auto */
-				/*
-				 * Search the table looking for a slot...
-				 */
-				for (i = 0; i < nchrdev; i++)
-					if (cdevsw[i].d_open == lkmenodev)
-						break;		/* found it! */
-				/* out of allocable slots? */
-				if (i == nchrdev) {
-					err = ENFILE;
-					break;
-				}
-			} else {				/* assign */
-				if (i < 0 || i >= nchrdev) {
-					err = EINVAL;
-					break;
-				}
+			if ((i = args->lkm_offset) == -1)
+				descrip = (dev_t) -1;
+			else
+				descrip = makedev(args->lkm_offset,0);
+			if ( err = cdevsw_add(&descrip, args->lkm_dev.cdev,
+					&(args->lkm_olddev.cdev))) {
+				break;
 			}
-
-			/* save old */
-			bcopy(&cdevsw[i], &(args->lkm_olddev.cdev), sizeof(struct cdevsw));
-
-			/* replace with new */
-			bcopy(args->lkm_dev.cdev, &cdevsw[i], sizeof(struct cdevsw));
-
-			/* done! */
-			args->lkm_offset = i;	/* slot in cdevsw[] */
-
+			args->lkm_offset = major(descrip) ;
 			break;
 
 		default:
@@ -757,16 +725,17 @@ _lkm_dev(lkmtp, cmd)
 	case LKM_E_UNLOAD:
 		/* current slot... */
 		i = args->lkm_offset;
+		descrip = makedev(i,0);
 
 		switch(args->lkm_devtype) {
 		case LM_DT_BLOCK:
 			/* replace current slot contents with old contents */
-			bcopy(&(args->lkm_olddev.bdev), &bdevsw[i], sizeof(struct bdevsw));
+			bdevsw_add(&descrip, args->lkm_olddev.bdev,NULL);
 			break;
 
 		case LM_DT_CHAR:
 			/* replace current slot contents with old contents */
-			bcopy(&(args->lkm_olddev.cdev), &cdevsw[i], sizeof(struct cdevsw));
+			cdevsw_add(&descrip, args->lkm_olddev.cdev,NULL);
 			break;
 
 		default:
@@ -877,11 +846,15 @@ _lkm_exec(lkmtp, cmd)
 	return(err);
 }
 
-static const struct execsw lkm_exec_dummy = { NULL, "lkm" };
-TEXT_SET(execsw_set, lkm_exec_dummy);
-TEXT_SET(execsw_set, lkm_exec_dummy);
-TEXT_SET(execsw_set, lkm_exec_dummy);
-TEXT_SET(execsw_set, lkm_exec_dummy);
+/* XXX: This is bogus.  we should find a better method RSN! */
+static const struct execsw lkm_exec_dummy1 = { NULL, "lkm" };
+static const struct execsw lkm_exec_dummy2 = { NULL, "lkm" };
+static const struct execsw lkm_exec_dummy3 = { NULL, "lkm" };
+static const struct execsw lkm_exec_dummy4 = { NULL, "lkm" };
+TEXT_SET(execsw_set, lkm_exec_dummy1);
+TEXT_SET(execsw_set, lkm_exec_dummy2);
+TEXT_SET(execsw_set, lkm_exec_dummy3);
+TEXT_SET(execsw_set, lkm_exec_dummy4);
 
 /*
  * This code handles the per-module type "wiring-in" of loadable modules
@@ -934,3 +907,37 @@ lkmdispatch(lkmtp, cmd)
 
 	return(err);
 }
+
+int
+lkm_nullcmd(lkmtp, cmd)
+	struct lkm_table *lkmtp;
+	int cmd;
+{
+
+	return (0);
+}
+
+static lkm_devsw_installed = 0;
+#ifdef DEVFS
+static void	*lkmc_devfs_token;
+#endif
+
+static void 	lkm_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! lkm_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&lkmc_cdevsw, NULL);
+		lkm_devsw_installed = 1;
+#ifdef DEVFS
+		lkmc_devfs_token = devfs_add_devswf(&lkmc_cdevsw, 0, DV_CHR,
+						    UID_ROOT, GID_WHEEL, 0644,
+						    "lkm");
+#endif
+    	}
+}
+
+#ifndef NO_LKM
+SYSINIT(lkmdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,lkm_drvinit,NULL)
+#endif
