@@ -34,14 +34,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)ufs_lockf.c	8.3 (Berkeley) 1/6/94
- * $FreeBSD: stable/2.1/sys/kern/kern_lockf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_lockf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
-#include <sys/file.h>
 #include <sys/proc.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/malloc.h>
 #include <sys/fcntl.h>
@@ -61,6 +61,16 @@ int	lockf_debug = 0;
 #define NOLOCKF (struct lockf *)0
 #define SELF	0x1
 #define OTHERS	0x2
+static void	 lf_addblock __P((struct lockf *, struct lockf *));
+static int	 lf_clearlock __P((struct lockf *));
+static int	 lf_findoverlap __P((struct lockf *,
+	    struct lockf *, int, struct lockf ***, struct lockf **));
+static struct lockf *
+	 lf_getblock __P((struct lockf *));
+static int	 lf_getlock __P((struct lockf *, struct flock *));
+static int	 lf_setlock __P((struct lockf *));
+static void	 lf_split __P((struct lockf *, struct lockf *));
+static void	 lf_wakelock __P((struct lockf *));
 
 /*
  * Advisory record locking support
@@ -82,15 +92,6 @@ lf_advlock(ap, head, size)
 	off_t start, end;
 	int error;
 
-	/*
-	 * Avoid the common case of unlocking when inode has no locks.
-	 */
-	if (*head == (struct lockf *)0) {
-		if (ap->a_op != F_SETLK) {
-			fl->l_type = F_UNLCK;
-			return (0);
-		}
-	}
 	/*
 	 * Convert the flock structure into a start and end.
 	 */
@@ -116,8 +117,20 @@ lf_advlock(ap, head, size)
 		return (EINVAL);
 	if (fl->l_len == 0)
 		end = -1;
-	else
+	else {
 		end = start + fl->l_len - 1;
+		if (end < start)
+			return (EINVAL);
+	}
+	/*
+	 * Avoid the common case of unlocking when inode has no locks.
+	 */
+	if (*head == (struct lockf *)0) {
+		if (ap->a_op != F_SETLK) {
+			fl->l_type = F_UNLCK;
+			return (0);
+		}
+	}
 	/*
 	 * Create the lockf structure
 	 */
@@ -157,7 +170,7 @@ lf_advlock(ap, head, size)
 /*
  * Set a byte-range lock.
  */
-int
+static int
 lf_setlock(lock)
 	register struct lockf *lock;
 {
@@ -258,13 +271,6 @@ lf_setlock(lock)
 				block->lf_block = block->lf_block->lf_block;
 				break;
 			}
-			/*
-			 * If we did not find ourselves on the list, but
-			 * are still linked onto a lock list, then something
-			 * is very wrong.
-			 */
-			if (block == NOLOCKF && lock->lf_next != NOLOCKF)
-				panic("lf_setlock: lost lock");
 			free(lock, M_LOCKF);
 			return (error);
 		}
@@ -399,7 +405,7 @@ lf_setlock(lock)
  * Generally, find the lock (or an overlap to that lock)
  * and remove it (or shrink it), then wakeup anyone we can.
  */
-int
+static int
 lf_clearlock(unlock)
 	register struct lockf *unlock;
 {
@@ -468,7 +474,7 @@ lf_clearlock(unlock)
  * Check whether there is a blocking lock,
  * and if so return its process identifier.
  */
-int
+static int
 lf_getlock(lock, fl)
 	register struct lockf *lock;
 	register struct flock *fl;
@@ -502,7 +508,7 @@ lf_getlock(lock, fl)
  * Walk the list of locks for an inode and
  * return the first blocking lock.
  */
-struct lockf *
+static struct lockf *
 lf_getblock(lock)
 	register struct lockf *lock;
 {
@@ -532,7 +538,7 @@ lf_getblock(lock)
  * NOTE: this returns only the FIRST overlapping lock.  There
  *	 may be more than one.
  */
-int
+static int
 lf_findoverlap(lf, lock, type, prev, overlap)
 	register struct lockf *lf;
 	struct lockf *lock;
@@ -641,7 +647,7 @@ lf_findoverlap(lf, lock, type, prev, overlap)
 /*
  * Add a lock to the end of the blocked list.
  */
-void
+static void
 lf_addblock(blocklist, lock)
 	struct lockf *blocklist;
 	struct lockf *lock;
@@ -670,7 +676,7 @@ lf_addblock(blocklist, lock)
  * Split a lock and a contained region into
  * two or three locks as necessary.
  */
-void
+static void
 lf_split(lock1, lock2)
 	register struct lockf *lock1;
 	register struct lockf *lock2;
@@ -717,7 +723,7 @@ lf_split(lock1, lock2)
 /*
  * Wakeup a blocklist
  */
-void
+static void
 lf_wakelock(listhead)
 	struct lockf *listhead;
 {
