@@ -17,6 +17,9 @@
 /*
  * Modification history kern_ntptime.c
  *
+ * 24 Sep 94	David L. Mills
+ *	Tightened code at exits.
+ *
  * 24 Mar 94	David L. Mills
  *	Revised syscall interface to include new variables for PPS
  *	time discipline.
@@ -32,7 +35,7 @@
  */
 /*
  * ntp_gettime(), ntp_adjtime() - precision time interface for SunOS
- * 4.1.1 and 4.1.3
+ * V4.1.1 and V4.1.3
  *
  * These routines consitute the Network Time Protocol (NTP) interfaces
  * for user and daemon application programs. The ntp_gettime() routine
@@ -45,10 +48,10 @@
  */
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/kernel.h>
 #include <sys/proc.h>
 #include <sys/timex.h>
-#include <vm/vm.h>
 #include <sys/sysctl.h>
 
 /*
@@ -80,23 +83,13 @@ extern long pps_errcnt;		/* calibration errors */
 extern long pps_stbcnt;		/* stability limit exceeded */
 #endif /* PPS_SYNC */
 
-int
-ntp_sysctl(int *name, u_int namelen, void *oldp, size_t *oldlenp,
-	   void *newp, size_t newlen, struct proc *p)
+static int
+ntp_sysctl SYSCTL_HANDLER_ARGS
 {
 	struct timeval atv;
 	struct ntptimeval ntv;
 	int s;
 
-	/* All names at this level are terminal. */
-	if (namelen != 1) {
-		return ENOTDIR;
-	}
-
-	if (name[0] != NTP_PLL_GETTIME) {
-		return EOPNOTSUPP;
-	}
-
 	s = splclock();
 #ifdef EXT_CLOCK
 	/*
@@ -162,15 +155,22 @@ ntp_sysctl(int *name, u_int namelen, void *oldp, size_t *oldlenp,
 	    time_status & (STA_PPSWANDER | STA_PPSERROR)) {
 		ntv.time_state = TIME_ERROR;
 	}
-	return(sysctl_rdstruct(oldp, oldlenp, newp, &ntv, sizeof ntv));
+	return (sysctl_handle_opaque(oidp, &ntv, sizeof ntv, req));
 }
 
+SYSCTL_NODE(_kern, KERN_NTP_PLL, ntp_pll, CTLFLAG_RW, 0,
+	"NTP kernel PLL related stuff");
+SYSCTL_PROC(_kern_ntp_pll, NTP_PLL_GETTIME, gettime, CTLTYPE_OPAQUE|CTLFLAG_RD,
+	0, sizeof(struct ntptimeval) , ntp_sysctl, "S,ntptimeval", "");
+
 /*
  * ntp_adjtime() - NTP daemon application interface
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ntp_adjtime_args {
   struct timex *tp;
 };
+#endif
 
 int
 ntp_adjtime(struct proc *p, struct ntp_adjtime_args *uap, int *retval)
