@@ -30,13 +30,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)kern_proc.c	8.4 (Berkeley) 1/4/94
- * $FreeBSD: stable/2.1/sys/kern/kern_proc.c 50950 1999-09-05 08:39:22Z peter $
+ *	@(#)kern_proc.c	8.7 (Berkeley) 2/14/95
+ * $FreeBSD: stable/2.2/sys/kern/kern_proc.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
 #include <sys/proc.h>
 #include <sys/buf.h>
 #include <sys/acct.h>
@@ -49,35 +50,55 @@
 #include <sys/ioctl.h>
 #include <sys/tty.h>
 #include <sys/signalvar.h>
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
+#include <sys/user.h>
 
 struct prochd qs[NQS];		/* as good a place as any... */
 struct prochd rtqs[NQS];	/* Space for REALTIME queues too */
 struct prochd idqs[NQS];	/* Space for IDLE queues too */
 
-volatile struct proc *allproc;	/* all processes */
-struct proc *zombproc;		/* just zombies */
-
-void pgdelete	__P((struct pgrp *));
+static void pgdelete	__P((struct pgrp *));
 
 /*
  * Structure associated with user cacheing.
  */
 struct uidinfo {
-	struct	uidinfo *ui_next;
-	struct	uidinfo **ui_prev;
+	LIST_ENTRY(uidinfo) ui_hash;
 	uid_t	ui_uid;
 	long	ui_proccnt;
-} **uihashtbl;
-u_long	uihash;		/* size of hash table - 1 */
-#define	UIHASH(uid)	((uid) & uihash)
+};
+#define	UIHASH(uid)	(&uihashtbl[(uid) & uihash])
+LIST_HEAD(uihashhead, uidinfo) *uihashtbl;
+static u_long uihash;		/* size of hash table - 1 */
+
+static void	orphanpg __P((struct pgrp *pg));
 
 /*
- * Allocate a hash table.
+ * Other process lists
+ */
+struct pidhashhead *pidhashtbl;
+u_long pidhash;
+struct pgrphashhead *pgrphashtbl;
+u_long pgrphash;
+struct proclist allproc;
+struct proclist zombproc;
+
+/*
+ * Initialize global process hashing structures.
  */
 void
-usrinfoinit()
+procinit()
 {
 
+	LIST_INIT(&allproc);
+	LIST_INIT(&zombproc);
+	pidhashtbl = hashinit(maxproc / 4, M_PROC, &pidhash);
+	pgrphashtbl = hashinit(maxproc / 4, M_PROC, &pgrphash);
 	uihashtbl = hashinit(maxproc / 16, M_PROC, &uihash);
 }
 
@@ -90,10 +111,11 @@ chgproccnt(uid, diff)
 	uid_t	uid;
 	int	diff;
 {
-	register struct uidinfo **uipp, *uip, *uiq;
+	register struct uidinfo *uip;
+	register struct uihashhead *uipp;
 
-	uipp = &uihashtbl[UIHASH(uid)];
-	for (uip = *uipp; uip; uip = uip->ui_next)
+	uipp = UIHASH(uid);
+	for (uip = uipp->lh_first; uip != 0; uip = uip->ui_hash.le_next)
 		if (uip->ui_uid == uid)
 			break;
 	if (uip) {
@@ -102,9 +124,7 @@ chgproccnt(uid, diff)
 			return (uip->ui_proccnt);
 		if (uip->ui_proccnt < 0)
 			panic("chgproccnt: procs < 0");
-		if ((uiq = uip->ui_next))
-			uiq->ui_prev = uip->ui_prev;
-		*uip->ui_prev = uiq;
+		LIST_REMOVE(uip, ui_hash);
 		FREE(uip, M_PROC);
 		return (0);
 	}
@@ -114,11 +134,7 @@ chgproccnt(uid, diff)
 		panic("chgproccnt: lost user");
 	}
 	MALLOC(uip, struct uidinfo *, sizeof(*uip), M_PROC, M_WAITOK);
-	if ((uiq = *uipp))
-		uiq->ui_prev = &uip->ui_next;
-	uip->ui_next = uiq;
-	uip->ui_prev = uipp;
-	*uipp = uip;
+	LIST_INSERT_HEAD(uipp, uip, ui_hash);
 	uip->ui_uid = uid;
 	uip->ui_proccnt = diff;
 	return (diff);
@@ -147,22 +163,7 @@ pfind(pid)
 {
 	register struct proc *p;
 
-	for (p = pidhash[PIDHASH(pid)]; p != NULL; p = p->p_hash)
-		if (p->p_pid == pid)
-			return (p);
-	return (NULL);
-}
-
-/*
- * Locate a zombie process by number
- */
-struct proc *
-zpfind(pid)
-	pid_t pid;
-{
-	struct proc *p;
-
-	for (p = zombproc; p != NULL; p = p->p_next) 
+	for (p = PIDHASH(pid)->lh_first; p != 0; p = p->p_hash.le_next)
 		if (p->p_pid == pid)
 			return (p);
 	return (NULL);
@@ -177,8 +178,8 @@ pgfind(pgid)
 {
 	register struct pgrp *pgrp;
 
-	for (pgrp = pgrphash[PIDHASH(pgid)];
-	    pgrp != NULL; pgrp = pgrp->pg_hforw)
+	for (pgrp = PGRPHASH(pgid)->lh_first; pgrp != 0;
+	     pgrp = pgrp->pg_hash.le_next)
 		if (pgrp->pg_id == pgid)
 			return (pgrp);
 	return (NULL);
@@ -194,8 +195,6 @@ enterpgrp(p, pgid, mksess)
 	int mksess;
 {
 	register struct pgrp *pgrp = pgfind(pgid);
-	register struct proc **pp;
-	int n;
 
 #ifdef DIAGNOSTIC
 	if (pgrp != NULL && mksess)	/* firewalls */
@@ -214,7 +213,7 @@ enterpgrp(p, pgid, mksess)
 			panic("enterpgrp: new pgrp and pid != pgid");
 #endif
 		MALLOC(pgrp, struct pgrp *, sizeof(struct pgrp), M_PGRP,
-		       M_WAITOK);
+		    M_WAITOK);
 		if ((np = pfind(savepid)) == NULL || np != p)
 			return (ESRCH);
 		if (mksess) {
@@ -224,7 +223,7 @@ enterpgrp(p, pgid, mksess)
 			 * new session
 			 */
 			MALLOC(sess, struct session *, sizeof(struct session),
-				M_SESSION, M_WAITOK);
+			    M_SESSION, M_WAITOK);
 			sess->s_leader = p;
 			sess->s_count = 1;
 			sess->s_ttyvp = NULL;
@@ -242,10 +241,9 @@ enterpgrp(p, pgid, mksess)
 			pgrp->pg_session->s_count++;
 		}
 		pgrp->pg_id = pgid;
-		pgrp->pg_hforw = pgrphash[n = PIDHASH(pgid)];
-		pgrphash[n] = pgrp;
+		LIST_INIT(&pgrp->pg_members);
+		LIST_INSERT_HEAD(PGRPHASH(pgid), pgrp, pg_hash);
 		pgrp->pg_jobc = 0;
-		pgrp->pg_mem = NULL;
 	} else if (pgrp == p->p_pgrp)
 		return (0);
 
@@ -257,30 +255,11 @@ enterpgrp(p, pgid, mksess)
 	fixjobc(p, pgrp, 1);
 	fixjobc(p, p->p_pgrp, 0);
 
-	/*
-	 * unlink p from old process group
-	 */
-	for (pp = &p->p_pgrp->pg_mem; *pp; pp = &(*pp)->p_pgrpnxt) {
-		if (*pp == p) {
-			*pp = p->p_pgrpnxt;
-			break;
-		}
-	}
-#ifdef DIAGNOSTIC
-	if (pp == NULL)
-		panic("enterpgrp: can't find p on old pgrp");
-#endif
-	/*
-	 * delete old if empty
-	 */
-	if (p->p_pgrp->pg_mem == 0)
+	LIST_REMOVE(p, p_pglist);
+	if (p->p_pgrp->pg_members.lh_first == 0)
 		pgdelete(p->p_pgrp);
-	/*
-	 * link into new one
-	 */
 	p->p_pgrp = pgrp;
-	p->p_pgrpnxt = pgrp->pg_mem;
-	pgrp->pg_mem = p;
+	LIST_INSERT_HEAD(&pgrp->pg_members, p, p_pglist);
 	return (0);
 }
 
@@ -291,19 +270,9 @@ int
 leavepgrp(p)
 	register struct proc *p;
 {
-	register struct proc **pp = &p->p_pgrp->pg_mem;
 
-	for (; *pp; pp = &(*pp)->p_pgrpnxt) {
-		if (*pp == p) {
-			*pp = p->p_pgrpnxt;
-			break;
-		}
-	}
-#ifdef DIAGNOSTIC
-	if (pp == NULL)
-		panic("leavepgrp: can't find p in pgrp");
-#endif
-	if (!p->p_pgrp->pg_mem)
+	LIST_REMOVE(p, p_pglist);
+	if (p->p_pgrp->pg_members.lh_first == 0)
 		pgdelete(p->p_pgrp);
 	p->p_pgrp = 0;
 	return (0);
@@ -312,32 +281,20 @@ leavepgrp(p)
 /*
  * delete a process group
  */
-void
+static void
 pgdelete(pgrp)
 	register struct pgrp *pgrp;
 {
-	register struct pgrp **pgp = &pgrphash[PIDHASH(pgrp->pg_id)];
 
 	if (pgrp->pg_session->s_ttyp != NULL &&
 	    pgrp->pg_session->s_ttyp->t_pgrp == pgrp)
 		pgrp->pg_session->s_ttyp->t_pgrp = NULL;
-	for (; *pgp; pgp = &(*pgp)->pg_hforw) {
-		if (*pgp == pgrp) {
-			*pgp = pgrp->pg_hforw;
-			break;
-		}
-	}
-#ifdef DIAGNOSTIC
-	if (pgp == NULL)
-		panic("pgdelete: can't find pgrp on hash chain");
-#endif
+	LIST_REMOVE(pgrp, pg_hash);
 	if (--pgrp->pg_session->s_count == 0)
 		FREE(pgrp->pg_session, M_SESSION);
 	FREE(pgrp, M_PGRP);
 }
 
-static void orphanpg();
-
 /*
  * Adjust pgrp jobc counters when specified process changes process group.
  * We count the number of processes in each process group that "qualify"
@@ -373,7 +330,7 @@ fixjobc(p, pgrp, entering)
 	 * their process groups; if so, adjust counts for children's
 	 * process groups.
 	 */
-	for (p = p->p_cptr; p; p = p->p_osptr)
+	for (p = p->p_children.lh_first; p != 0; p = p->p_sibling.le_next)
 		if ((hispgrp = p->p_pgrp) != pgrp &&
 		    hispgrp->pg_session == mysession &&
 		    p->p_stat != SZOMB)
@@ -394,9 +351,10 @@ orphanpg(pg)
 {
 	register struct proc *p;
 
-	for (p = pg->pg_mem; p; p = p->p_pgrpnxt) {
+	for (p = pg->pg_members.lh_first; p != 0; p = p->p_pglist.le_next) {
 		if (p->p_stat == SSTOP) {
-			for (p = pg->pg_mem; p; p = p->p_pgrpnxt) {
+			for (p = pg->pg_members.lh_first; p != 0;
+			    p = p->p_pglist.le_next) {
 				psignal(p, SIGHUP);
 				psignal(p, SIGCONT);
 			}
@@ -405,28 +363,235 @@ orphanpg(pg)
 	}
 }
 
-#ifdef debug
-/* DEBUG */
-pgrpdump()
+#include "opt_ddb.h"
+#ifdef DDB
+#include <ddb/ddb.h>
+
+DB_SHOW_COMMAND(pgrpdump, pgrpdump)
 {
 	register struct pgrp *pgrp;
 	register struct proc *p;
 	register i;
 
-	for (i=0; i<PIDHSZ; i++) {
-		if (pgrphash[i]) {
-		  printf("\tindx %d\n", i);
-		  for (pgrp=pgrphash[i]; pgrp; pgrp=pgrp->pg_hforw) {
-		    printf("\tpgrp %x, pgid %d, sess %x, sesscnt %d, mem %x\n",
-			pgrp, pgrp->pg_id, pgrp->pg_session,
-			pgrp->pg_session->s_count, pgrp->pg_mem);
-		    for (p=pgrp->pg_mem; p; p=p->p_pgrpnxt) {
-			printf("\t\tpid %d addr %x pgrp %x\n",
-				p->p_pid, p, p->p_pgrp);
-		    }
-		  }
-
+	for (i = 0; i <= pgrphash; i++) {
+		if (pgrp = pgrphashtbl[i].lh_first) {
+			printf("\tindx %d\n", i);
+			for (; pgrp != 0; pgrp = pgrp->pg_hash.le_next) {
+				printf("\tpgrp %x, pgid %d, sess %x, sesscnt %d, mem %x\n",
+				    pgrp, pgrp->pg_id, pgrp->pg_session,
+				    pgrp->pg_session->s_count,
+				    pgrp->pg_members.lh_first);
+				for (p = pgrp->pg_members.lh_first; p != 0;
+				    p = p->p_pglist.le_next) {
+					printf("\t\tpid %d addr %x pgrp %x\n", 
+					    p->p_pid, p, p->p_pgrp);
+				}
+			}
 		}
 	}
 }
-#endif /* debug */
+#endif /* DDB */
+
+/*
+ * Fill in an eproc structure for the specified process.
+ */
+void
+fill_eproc(p, ep)
+	register struct proc *p;
+	register struct eproc *ep;
+{
+	register struct tty *tp;
+
+	bzero(ep, sizeof(*ep));
+
+	ep->e_paddr = p;
+	if (p->p_cred) {
+		ep->e_pcred = *p->p_cred;
+		if (p->p_ucred)
+			ep->e_ucred = *p->p_ucred;
+	}
+	if (p->p_stat != SIDL && p->p_stat != SZOMB && p->p_vmspace != NULL) {
+		register struct vmspace *vm = p->p_vmspace;
+
+#ifdef pmap_resident_count
+		ep->e_vm.vm_rssize = pmap_resident_count(&vm->vm_pmap); /*XXX*/
+#else
+		ep->e_vm.vm_rssize = vm->vm_rssize;
+#endif
+		ep->e_vm.vm_tsize = vm->vm_tsize;
+		ep->e_vm.vm_dsize = vm->vm_dsize;
+		ep->e_vm.vm_ssize = vm->vm_ssize;
+		ep->e_vm.vm_taddr = vm->vm_taddr;
+		ep->e_vm.vm_daddr = vm->vm_daddr;
+		ep->e_vm.vm_minsaddr = vm->vm_minsaddr;
+		ep->e_vm.vm_maxsaddr = vm->vm_maxsaddr;
+#ifndef sparc
+		ep->e_vm.vm_pmap = vm->vm_pmap;
+#endif
+	}
+	if (p->p_pptr)
+		ep->e_ppid = p->p_pptr->p_pid;
+	if (p->p_pgrp) {
+		ep->e_pgid = p->p_pgrp->pg_id;
+		ep->e_jobc = p->p_pgrp->pg_jobc;
+		ep->e_sess = p->p_pgrp->pg_session;
+
+		if (ep->e_sess) {
+			bcopy(ep->e_sess->s_login, ep->e_login, sizeof(ep->e_login));
+			if (ep->e_sess->s_ttyvp)
+				ep->e_flag = EPROC_CTTY;
+			if (p->p_session && SESS_LEADER(p))
+				ep->e_flag |= EPROC_SLEADER;
+		}
+	}
+	if ((p->p_flag & P_CONTROLT) &&
+	    (ep->e_sess != NULL) &&
+	    ((tp = ep->e_sess->s_ttyp) != NULL)) {
+		ep->e_tdev = tp->t_dev;
+		ep->e_tpgid = tp->t_pgrp ? tp->t_pgrp->pg_id : NO_PID;
+		ep->e_tsess = tp->t_session;
+	} else
+		ep->e_tdev = NODEV;
+	if (p->p_wmesg) {
+		strncpy(ep->e_wmesg, p->p_wmesg, WMESGLEN);
+		ep->e_wmesg[WMESGLEN] = 0;
+	}
+}
+
+static struct proc *
+zpfind(pid_t pid)
+{
+	struct proc *p;
+
+	for (p = zombproc.lh_first; p != 0; p = p->p_list.le_next)
+		if (p->p_pid == pid)
+			return (p);
+	return (NULL);
+}
+
+
+static int
+sysctl_out_proc(struct proc *p, struct sysctl_req *req, int doingzomb)
+{
+	struct eproc eproc;
+	int error;
+	pid_t pid = p->p_pid;
+
+	fill_eproc(p, &eproc);
+	error = SYSCTL_OUT(req,(caddr_t)p, sizeof(struct proc));
+	if (error)
+		return (error);
+	error = SYSCTL_OUT(req,(caddr_t)&eproc, sizeof(eproc));
+	if (error)
+		return (error);
+	if (!doingzomb && pid && (pfind(pid) != p))
+		return EAGAIN;
+	if (doingzomb && zpfind(pid) != p)
+		return EAGAIN;
+	return (0);
+}
+
+static int
+sysctl_kern_proc SYSCTL_HANDLER_ARGS
+{
+	int *name = (int*) arg1;
+	u_int namelen = arg2;
+	struct proc *p;
+	int doingzomb;
+	int error = 0;
+
+	if (oidp->oid_number == KERN_PROC_PID) {
+		if (namelen != 1) 
+			return (EINVAL);
+		p = pfind((pid_t)name[0]);
+		if (!p)
+			return (0);
+		error = sysctl_out_proc(p, req, 0);
+		return (error);
+	}
+	if (oidp->oid_number == KERN_PROC_ALL && !namelen)
+		;
+	else if (oidp->oid_number != KERN_PROC_ALL && namelen == 1)
+		;
+	else
+		return (EINVAL);
+	
+	if (!req->oldptr) {
+		/* overestimate by 5 procs */
+		error = SYSCTL_OUT(req, 0, sizeof (struct kinfo_proc) * 5);
+		if (error)
+			return (error);
+	}
+	for (doingzomb=0 ; doingzomb < 2 ; doingzomb++) {
+		if (!doingzomb)
+			p = allproc.lh_first;
+		else
+			p = zombproc.lh_first;
+		for (; p != 0; p = p->p_list.le_next) {
+			/*
+			 * Skip embryonic processes.
+			 */
+			if (p->p_stat == SIDL)
+				continue;
+			/*
+			 * TODO - make more efficient (see notes below).
+			 * do by session.
+			 */
+			switch (oidp->oid_number) {
+
+			case KERN_PROC_PGRP:
+				/* could do this by traversing pgrp */
+				if (p->p_pgrp == NULL || 
+				    p->p_pgrp->pg_id != (pid_t)name[0])
+					continue;
+				break;
+
+			case KERN_PROC_TTY:
+				if ((p->p_flag & P_CONTROLT) == 0 ||
+				    p->p_session == NULL ||
+				    p->p_session->s_ttyp == NULL ||
+				    p->p_session->s_ttyp->t_dev != (dev_t)name[0])
+					continue;
+				break;
+
+			case KERN_PROC_UID:
+				if (p->p_ucred == NULL || 
+				    p->p_ucred->cr_uid != (uid_t)name[0])
+					continue;
+				break;
+
+			case KERN_PROC_RUID:
+				if (p->p_ucred == NULL || 
+				    p->p_cred->p_ruid != (uid_t)name[0])
+					continue;
+				break;
+			}
+
+			error = sysctl_out_proc(p, req, doingzomb);
+			if (error)
+				return (error);
+		}
+	}
+	return (0);
+}
+
+
+SYSCTL_NODE(_kern, KERN_PROC, proc, CTLFLAG_RD,  0, "Process table");
+
+SYSCTL_PROC(_kern_proc, KERN_PROC_ALL, all, CTLFLAG_RD|CTLTYPE_STRUCT,
+	0, 0, sysctl_kern_proc, "S,proc", "");
+
+SYSCTL_NODE(_kern_proc, KERN_PROC_PGRP, pgrp, CTLFLAG_RD, 
+	sysctl_kern_proc, "Process table");
+
+SYSCTL_NODE(_kern_proc, KERN_PROC_TTY, tty, CTLFLAG_RD, 
+	sysctl_kern_proc, "Process table");
+
+SYSCTL_NODE(_kern_proc, KERN_PROC_UID, uid, CTLFLAG_RD, 
+	sysctl_kern_proc, "Process table");
+
+SYSCTL_NODE(_kern_proc, KERN_PROC_RUID, ruid, CTLFLAG_RD, 
+	sysctl_kern_proc, "Process table");
+
+SYSCTL_NODE(_kern_proc, KERN_PROC_PID, pid, CTLFLAG_RD, 
+	sysctl_kern_proc, "Process table");
