@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_prot.c	8.6 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/kern_prot.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_prot.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -46,21 +46,25 @@
 #include <sys/param.h>
 #include <sys/acct.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/ucred.h>
 #include <sys/proc.h>
 #include <sys/timeb.h>
 #include <sys/times.h>
 #include <sys/malloc.h>
+#include <sys/unistd.h>
 
-struct args {
+#ifndef _SYS_SYSPROTO_H_
+struct getpid_args {
 	int	dummy;
 };
+#endif
 
 /* ARGSUSED */
 int
 getpid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getpid_args *uap;
 	int *retval;
 {
 
@@ -71,11 +75,16 @@ getpid(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
+struct getppid_args {
+        int     dummy;
+};
+#endif
 /* ARGSUSED */
 int
 getppid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getppid_args *uap;
 	int *retval;
 {
 
@@ -84,10 +93,16 @@ getppid(p, uap, retval)
 }
 
 /* Get process group ID; note that POSIX getpgrp takes no parameter */
+#ifndef _SYS_SYSPROTO_H_
+struct getpgrp_args {
+        int     dummy;
+};
+#endif
+
 int
 getpgrp(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getpgrp_args *uap;
 	int *retval;
 {
 
@@ -95,11 +110,17 @@ getpgrp(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
+struct getuid_args {
+        int     dummy;
+};
+#endif
+
 /* ARGSUSED */
 int
 getuid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getuid_args *uap;
 	int *retval;
 {
 
@@ -110,11 +131,17 @@ getuid(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
+struct geteuid_args {
+        int     dummy;
+};
+#endif
+
 /* ARGSUSED */
 int
 geteuid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct geteuid_args *uap;
 	int *retval;
 {
 
@@ -122,11 +149,17 @@ geteuid(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
+struct getgid_args {
+        int     dummy;
+};
+#endif
+
 /* ARGSUSED */
 int
 getgid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getgid_args *uap;
 	int *retval;
 {
 
@@ -142,11 +175,17 @@ getgid(p, uap, retval)
  * via getgroups.  This syscall exists because it is somewhat painful to do
  * correctly in a library function.
  */
+#ifndef _SYS_SYSPROTO_H_
+struct getegid_args {
+        int     dummy;
+};
+#endif
+
 /* ARGSUSED */
 int
 getegid(p, uap, retval)
 	struct proc *p;
-	struct args *uap;
+	struct getegid_args *uap;
 	int *retval;
 {
 
@@ -154,10 +193,12 @@ getegid(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct getgroups_args {
 	u_int	gidsetsize;
 	gid_t	*gidset;
 };
+#endif
 int
 getgroups(p, uap, retval)
 	struct proc *p;
@@ -182,11 +223,17 @@ getgroups(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
+struct setsid_args {
+        int     dummy;
+};
+#endif
+
 /* ARGSUSED */
 int
 setsid(p, uap, retval)
 	register struct proc *p;
-	struct args *uap;
+	struct setsid_args *uap;
 	int *retval;
 {
 
@@ -212,10 +259,12 @@ setsid(p, uap, retval)
  * 	there must exist some pid in same session having pgid (EPERM)
  * pid must not be session leader (EPERM)
  */
+#ifndef _SYS_SYSPROTO_H_
 struct setpgid_args {
 	int	pid;	/* target process id */
 	int	pgid;	/* target pgrp id */
 };
+#endif
 /* ARGSUSED */
 int
 setpgid(curp, uap, retval)
@@ -226,6 +275,8 @@ setpgid(curp, uap, retval)
 	register struct proc *targp;		/* target process */
 	register struct pgrp *pgrp;		/* target pgrp */
 
+	if (uap->pgid < 0)
+		return (EINVAL);
 	if (uap->pid != 0 && uap->pid != curp->p_pid) {
 		if ((targp = pfind(uap->pid)) == 0 || !inferior(targp))
 			return (ESRCH);
@@ -246,9 +297,23 @@ setpgid(curp, uap, retval)
 	return (enterpgrp(targp, uap->pgid, 0));
 }
 
+/*
+ * Use the clause in B.4.2.2 that allows setuid/setgid to be 4.2/4.3BSD
+ * compatable.  It says that setting the uid/gid to euid/egid is a special
+ * case of "appropriate privilege".  Once the rules are expanded out, this
+ * basically means that setuid(nnn) sets all three id's, in all permitted
+ * cases unless _POSIX_SAVED_IDS is enabled.  In that case, setuid(getuid())
+ * does not set the saved id - this is dangerous for traditional BSD
+ * programs.  For this reason, we *really* do not want to set
+ * _POSIX_SAVED_IDS and do not want to clear POSIX_APPENDIX_B_4_2_2.
+ */
+#define POSIX_APPENDIX_B_4_2_2
+
+#ifndef _SYS_SYSPROTO_H_
 struct setuid_args {
 	uid_t	uid;
 };
+#endif
 /* ARGSUSED */
 int
 setuid(p, uap, retval)
@@ -260,32 +325,90 @@ setuid(p, uap, retval)
 	register uid_t uid;
 	int error;
 
+	/*
+	 * See if we have "permission" by POSIX 1003.1 rules.
+	 *
+	 * Note that setuid(geteuid()) is a special case of 
+	 * "appropriate privileges" in appendix B.4.2.2.  We need
+	 * to use this clause to be compatable with traditional BSD
+	 * semantics.  Basically, it means that "setuid(xx)" sets all
+	 * three id's (assuming you have privs).
+	 *
+	 * Notes on the logic.  We do things in three steps.
+	 * 1: We determine if the euid is going to change, and do EPERM
+	 *    right away.  We unconditionally change the euid later if this
+	 *    test is satisfied, simplifying that part of the logic.
+	 * 2: We determine if the real and/or saved uid's are going to
+	 *    change.  Determined by compile options.
+	 * 3: Change euid last. (after tests in #2 for "appropriate privs")
+	 */
 	uid = uap->uid;
-	if (uid != pc->p_ruid && uid != pc->p_svuid &&
+	if (uid != pc->p_ruid &&		/* allow setuid(getuid()) */
+#ifdef _POSIX_SAVED_IDS
+	    uid != pc->p_svuid &&		/* allow setuid(saved gid) */
+#endif
+#ifdef POSIX_APPENDIX_B_4_2_2	/* Use BSD-compat clause from B.4.2.2 */
+	    uid != pc->pc_ucred->cr_uid &&	/* allow setuid(geteuid()) */
+#endif
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
+
+#ifdef _POSIX_SAVED_IDS
 	/*
-	 * Everything's okay, do it.
-	 * Transfer proc count to new user.
+	 * Do we have "appropriate privileges" (are we root or uid == euid)
+	 * If so, we are changing the real uid and/or saved uid.
+	 */
+	if (
+#ifdef POSIX_APPENDIX_B_4_2_2	/* Use the clause from B.4.2.2 */
+	    uid == pc->pc_ucred->cr_uid ||
+#endif
+	    suser(pc->pc_ucred, &p->p_acflag) == 0) /* we are using privs */
+#endif
+	{
+		/*
+		 * Transfer proc count to new user.
+		 */
+		if (uid != pc->p_ruid) {
+			(void)chgproccnt(pc->p_ruid, -1);
+			(void)chgproccnt(uid, 1);
+		}
+		/*
+		 * Set real uid
+		 */
+		if (uid != pc->p_ruid) {
+			p->p_flag |= P_SUGID;
+			pc->p_ruid = uid;
+		}
+		/*
+		 * Set saved uid
+		 *
+		 * XXX always set saved uid even if not _POSIX_SAVED_IDS, as
+		 * the security of seteuid() depends on it.  B.4.2.2 says it
+		 * is important that we should do this.
+		 */
+		if (pc->p_svuid != uid) {
+			p->p_flag |= P_SUGID;
+			pc->p_svuid = uid;
+		}
+	}
+
+	/*
+	 * In all permitted cases, we are changing the euid.
 	 * Copy credentials so other references do not see our changes.
 	 */
-	if (pc->pc_ucred->cr_uid == 0 && uid != pc->p_ruid) {
-		(void)chgproccnt(pc->p_ruid, -1);
-		(void)chgproccnt(uid, 1);
+	if (pc->pc_ucred->cr_uid != uid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
+		pc->pc_ucred->cr_uid = uid;
+		p->p_flag |= P_SUGID;
 	}
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	if (pc->pc_ucred->cr_uid == 0) {
-		pc->p_ruid = uid;
-		pc->p_svuid = uid;
-	}
-	pc->pc_ucred->cr_uid = uid;
-	p->p_flag |= P_SUGID;
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct seteuid_args {
 	uid_t	euid;
 };
+#endif
 /* ARGSUSED */
 int
 seteuid(p, uap, retval)
@@ -298,22 +421,27 @@ seteuid(p, uap, retval)
 	int error;
 
 	euid = uap->euid;
-	if (euid != pc->p_ruid && euid != pc->p_svuid &&
+	if (euid != pc->p_ruid &&		/* allow seteuid(getuid()) */
+	    euid != pc->p_svuid &&		/* allow seteuid(saved uid) */
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
 	/*
 	 * Everything's okay, do it.  Copy credentials so other references do
 	 * not see our changes.
 	 */
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	pc->pc_ucred->cr_uid = euid;
-	p->p_flag |= P_SUGID;
+	if (pc->pc_ucred->cr_uid != euid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
+		pc->pc_ucred->cr_uid = euid;
+		p->p_flag |= P_SUGID;
+	}
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setgid_args {
 	gid_t	gid;
 };
+#endif
 /* ARGSUSED */
 int
 setgid(p, uap, retval)
@@ -325,23 +453,76 @@ setgid(p, uap, retval)
 	register gid_t gid;
 	int error;
 
+	/*
+	 * See if we have "permission" by POSIX 1003.1 rules.
+	 *
+	 * Note that setgid(getegid()) is a special case of
+	 * "appropriate privileges" in appendix B.4.2.2.  We need
+	 * to use this clause to be compatable with traditional BSD
+	 * semantics.  Basically, it means that "setgid(xx)" sets all
+	 * three id's (assuming you have privs).
+	 *
+	 * For notes on the logic here, see setuid() above.
+	 */
 	gid = uap->gid;
-	if (gid != pc->p_rgid && gid != pc->p_svgid &&
+	if (gid != pc->p_rgid &&		/* allow setgid(getgid()) */
+#ifdef _POSIX_SAVED_IDS
+	    gid != pc->p_svgid &&		/* allow setgid(saved gid) */
+#endif
+#ifdef POSIX_APPENDIX_B_4_2_2	/* Use BSD-compat clause from B.4.2.2 */
+	    gid != pc->pc_ucred->cr_groups[0] && /* allow setgid(getegid()) */
+#endif
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	pc->pc_ucred->cr_groups[0] = gid;
-	if (pc->pc_ucred->cr_uid == 0) {
-		pc->p_rgid = gid;
-		pc->p_svgid = gid;
+
+#ifdef _POSIX_SAVED_IDS
+	/*
+	 * Do we have "appropriate privileges" (are we root or gid == egid)
+	 * If so, we are changing the real uid and saved gid.
+	 */
+	if (
+#ifdef POSIX_APPENDIX_B_4_2_2	/* use the clause from B.4.2.2 */
+	    gid == pc->pc_ucred->cr_groups[0] ||
+#endif
+	    suser(pc->pc_ucred, &p->p_acflag) == 0) /* we are using privs */
+#endif
+	{
+		/*
+		 * Set real gid
+		 */
+		if (pc->p_rgid != gid) {
+			p->p_flag |= P_SUGID;
+			pc->p_rgid = gid;
+		}
+		/*
+		 * Set saved gid
+		 *
+		 * XXX always set saved gid even if not _POSIX_SAVED_IDS, as
+		 * the security of setegid() depends on it.  B.4.2.2 says it
+		 * is important that we should do this.
+		 */
+		if (pc->p_svgid != gid) {
+			p->p_flag |= P_SUGID;
+			pc->p_svgid = gid;
+		}
+	}
+	/*
+	 * In all cases permitted cases, we are changing the egid.
+	 * Copy credentials so other references do not see our changes.
+	 */
+	if (pc->pc_ucred->cr_groups[0] != gid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
+		pc->pc_ucred->cr_groups[0] = gid;
+		p->p_flag |= P_SUGID;
 	}
-	p->p_flag |= P_SUGID;
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setegid_args {
 	gid_t	egid;
 };
+#endif
 /* ARGSUSED */
 int
 setegid(p, uap, retval)
@@ -354,19 +535,24 @@ setegid(p, uap, retval)
 	int error;
 
 	egid = uap->egid;
-	if (egid != pc->p_rgid && egid != pc->p_svgid &&
+	if (egid != pc->p_rgid &&		/* allow setegid(getgid()) */
+	    egid != pc->p_svgid &&		/* allow setegid(saved gid) */
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	pc->pc_ucred->cr_groups[0] = egid;
-	p->p_flag |= P_SUGID;
+	if (pc->pc_ucred->cr_groups[0] != egid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
+		pc->pc_ucred->cr_groups[0] = egid;
+		p->p_flag |= P_SUGID;
+	}
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setgroups_args {
 	u_int	gidsetsize;
 	gid_t	*gidset;
 };
+#endif
 /* ARGSUSED */
 int
 setgroups(p, uap, retval)
@@ -381,21 +567,37 @@ setgroups(p, uap, retval)
 	if ((error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
 	ngrp = uap->gidsetsize;
-	if (ngrp < 1 || ngrp > NGROUPS)
+	if (ngrp > NGROUPS)
 		return (EINVAL);
+	/*
+	 * XXX A little bit lazy here.  We could test if anything has
+	 * changed before crcopy() and setting P_SUGID.
+	 */
 	pc->pc_ucred = crcopy(pc->pc_ucred);
-	if ((error = copyin((caddr_t)uap->gidset,
-	    (caddr_t)pc->pc_ucred->cr_groups, ngrp * sizeof(gid_t))))
-		return (error);
-	pc->pc_ucred->cr_ngroups = ngrp;
+	if (ngrp < 1) {
+		/*
+		 * setgroups(0, NULL) is a legitimate way of clearing the
+		 * groups vector on non-BSD systems (which generally do not
+		 * have the egid in the groups[0]).  We risk security holes
+		 * when running non-BSD software if we do not do the same.
+		 */
+		pc->pc_ucred->cr_ngroups = 1;
+	} else {
+		if ((error = copyin((caddr_t)uap->gidset,
+		    (caddr_t)pc->pc_ucred->cr_groups, ngrp * sizeof(gid_t))))
+			return (error);
+		pc->pc_ucred->cr_ngroups = ngrp;
+	}
 	p->p_flag |= P_SUGID;
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setreuid_args {
 	uid_t	ruid;
 	uid_t	euid;
 };
+#endif
 /* ARGSUSED */
 int
 setreuid(p, uap, retval)
@@ -410,28 +612,36 @@ setreuid(p, uap, retval)
 	ruid = uap->ruid;
 	euid = uap->euid;
 	if ((ruid != (uid_t)-1 && ruid != pc->p_ruid && ruid != pc->p_svuid ||
-	     euid != (uid_t)-1 && euid != pc->p_ruid && euid != pc->p_svuid) &&
+	     euid != (uid_t)-1 && euid != pc->pc_ucred->cr_uid &&
+	     euid != pc->p_ruid && euid != pc->p_svuid) &&
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
 
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	if (euid != (uid_t)-1)
+	if (euid != (uid_t)-1 && pc->pc_ucred->cr_uid != euid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
 		pc->pc_ucred->cr_uid = euid;
-	if (ruid != (uid_t)-1 && ruid != pc->p_ruid) {
+		p->p_flag |= P_SUGID;
+	}
+	if (ruid != (uid_t)-1 && pc->p_ruid != ruid) {
 		(void)chgproccnt(pc->p_ruid, -1);
 		(void)chgproccnt(ruid, 1);
 		pc->p_ruid = ruid;
+		p->p_flag |= P_SUGID;
 	}
-	if (ruid != (uid_t)-1 || pc->pc_ucred->cr_uid != pc->p_ruid)
+	if ((ruid != (uid_t)-1 || pc->pc_ucred->cr_uid != pc->p_ruid) &&
+	    pc->p_svuid != pc->pc_ucred->cr_uid) {
 		pc->p_svuid = pc->pc_ucred->cr_uid;
-	p->p_flag |= P_SUGID;
+		p->p_flag |= P_SUGID;
+	}
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setregid_args {
 	gid_t	rgid;
 	gid_t	egid;
 };
+#endif
 /* ARGSUSED */
 int
 setregid(p, uap, retval)
@@ -446,18 +656,50 @@ setregid(p, uap, retval)
 	rgid = uap->rgid;
 	egid = uap->egid;
 	if ((rgid != (gid_t)-1 && rgid != pc->p_rgid && rgid != pc->p_svgid ||
-	     egid != (gid_t)-1 && egid != pc->p_rgid && egid != pc->p_svgid) &&
+	     egid != (gid_t)-1 && egid != pc->pc_ucred->cr_groups[0] &&
+	     egid != pc->p_rgid && egid != pc->p_svgid) &&
 	    (error = suser(pc->pc_ucred, &p->p_acflag)))
 		return (error);
 
-	pc->pc_ucred = crcopy(pc->pc_ucred);
-	if (egid != (gid_t)-1)
+	if (egid != (gid_t)-1 && pc->pc_ucred->cr_groups[0] != egid) {
+		pc->pc_ucred = crcopy(pc->pc_ucred);
 		pc->pc_ucred->cr_groups[0] = egid;
-	if (rgid != (gid_t)-1)
+		p->p_flag |= P_SUGID;
+	}
+	if (rgid != (gid_t)-1 && pc->p_rgid != rgid) {
 		pc->p_rgid = rgid;
-	if (rgid != (gid_t)-1 || pc->pc_ucred->cr_groups[0] != pc->p_rgid)
+		p->p_flag |= P_SUGID;
+	}
+	if ((rgid != (gid_t)-1 || pc->pc_ucred->cr_groups[0] != pc->p_rgid) &&
+	    pc->p_svgid != pc->pc_ucred->cr_groups[0]) {
 		pc->p_svgid = pc->pc_ucred->cr_groups[0];
-	p->p_flag |= P_SUGID;
+		p->p_flag |= P_SUGID;
+	}
+	return (0);
+}
+
+#ifndef _SYS_SYSPROTO_H_
+struct issetugid_args {
+	int dummy;
+};
+#endif
+/* ARGSUSED */
+int
+issetugid(p, uap, retval)
+	register struct proc *p;
+	struct issetugid_args *uap;
+	int *retval;
+{
+	/*
+	 * Note: OpenBSD sets a P_SUGIDEXEC flag set at execve() time,
+	 * we use P_SUGID because we consider changing the owners as
+	 * "tainting" as well.
+	 * This is significant for procs that start as root and "become"
+	 * a user without an exec - programs cannot know *everything*
+	 * that libc *might* have put in their data segment.
+	 */
+	if (p->p_flag & P_SUGID)
+		return (1);
 	return (0);
 }
 
@@ -564,10 +806,12 @@ crdup(cr)
 /*
  * Get login name, if available.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct getlogin_args {
 	char	*namebuf;
 	u_int	namelen;
 };
+#endif
 /* ARGSUSED */
 int
 getlogin(p, uap, retval)
@@ -585,9 +829,11 @@ getlogin(p, uap, retval)
 /*
  * Set login name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct setlogin_args {
 	char	*namebuf;
 };
+#endif
 /* ARGSUSED */
 int
 setlogin(p, uap, retval)
