@@ -36,11 +36,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_resource.c	8.5 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/kern_resource.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_resource.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_rlimit.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/kernel.h>
 #include <sys/file.h>
 #include <sys/resourcevar.h>
@@ -48,18 +51,25 @@
 #include <sys/proc.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 
-int	donice __P((struct proc *, struct proc *, int));
-int	dosetrlimit __P((struct proc *, u_int, struct rlimit *));
+int	donice __P((struct proc *curp, struct proc *chgp, int n));
+int	dosetrlimit __P((struct proc *p, u_int which, struct rlimit *limp));
 
 /*
  * Resource controls and accounting.
  */
 
+#ifndef _SYS_SYSPROTO_H_
 struct getpriority_args {
 	int	which;
 	int	who;
 };
+#endif
 int
 getpriority(curp, uap, retval)
 	struct proc *curp;
@@ -88,7 +98,8 @@ getpriority(curp, uap, retval)
 			pg = curp->p_pgrp;
 		else if ((pg = pgfind(uap->who)) == NULL)
 			break;
-		for (p = pg->pg_mem; p != NULL; p = p->p_pgrpnxt) {
+		for (p = pg->pg_members.lh_first; p != 0;
+		     p = p->p_pglist.le_next) {
 			if (p->p_nice < low)
 				low = p->p_nice;
 		}
@@ -98,11 +109,10 @@ getpriority(curp, uap, retval)
 	case PRIO_USER:
 		if (uap->who == 0)
 			uap->who = curp->p_ucred->cr_uid;
-		for (p = (struct proc *)allproc; p != NULL; p = p->p_next) {
+		for (p = allproc.lh_first; p != 0; p = p->p_list.le_next)
 			if (p->p_ucred->cr_uid == uap->who &&
 			    p->p_nice < low)
 				low = p->p_nice;
-		}
 		break;
 
 	default:
@@ -114,11 +124,13 @@ getpriority(curp, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct setpriority_args {
 	int	which;
 	int	who;
 	int	prio;
 };
+#endif
 /* ARGSUSED */
 int
 setpriority(curp, uap, retval)
@@ -149,7 +161,8 @@ setpriority(curp, uap, retval)
 			pg = curp->p_pgrp;
 		else if ((pg = pgfind(uap->who)) == NULL)
 			break;
-		for (p = pg->pg_mem; p != NULL; p = p->p_pgrpnxt) {
+		for (p = pg->pg_members.lh_first; p != 0;
+		    p = p->p_pglist.le_next) {
 			error = donice(curp, p, uap->prio);
 			found++;
 		}
@@ -159,7 +172,7 @@ setpriority(curp, uap, retval)
 	case PRIO_USER:
 		if (uap->who == 0)
 			uap->who = curp->p_ucred->cr_uid;
-		for (p = (struct proc *)allproc; p != NULL; p = p->p_next)
+		for (p = allproc.lh_first; p != 0; p = p->p_list.le_next)
 			if (p->p_ucred->cr_uid == uap->who) {
 				error = donice(curp, p, uap->prio);
 				found++;
@@ -197,11 +210,13 @@ donice(curp, chgp, n)
 }
 
 /* rtprio system call */
+#ifndef _SYS_SYSPROTO_H_
 struct rtprio_args {
 	int		function;
 	pid_t		pid;
-	struct rtprio	*rtprio;
+	struct rtprio	*rtp;
 };
+#endif
 
 /*
  * Set realtime priority
@@ -219,7 +234,7 @@ rtprio(curp, uap, retval)
 	struct rtprio rtp;
 	int error;
 
-	error = copyin(uap->rtprio, &rtp, sizeof(struct rtprio));
+	error = copyin(uap->rtp, &rtp, sizeof(struct rtprio));
 	if (error)
 		return (error);
 
@@ -233,7 +248,7 @@ rtprio(curp, uap, retval)
 
 	switch (uap->function) {
 	case RTP_LOOKUP:
-		return (copyout(&p->p_rtprio, uap->rtprio, sizeof(struct rtprio)));
+		return (copyout(&p->p_rtprio, uap->rtp, sizeof(struct rtprio)));
 	case RTP_SET:
 		if (pcred->pc_ucred->cr_uid && pcred->p_ruid &&
 		    pcred->pc_ucred->cr_uid != p->p_ucred->cr_uid &&
@@ -245,7 +260,17 @@ rtprio(curp, uap, retval)
 			if (uap->pid)
 				return (EPERM);
 			/* can't set realtime priority */
+/*
+ * Realtime priority has to be restricted for reasons which should be
+ * obvious. However, for idle priority, there is a potential for
+ * system deadlock if an idleprio process gains a lock on a resource
+ * that other processes need (and the idleprio process can't run
+ * due to a CPU-bound normal process). Fix me! XXX
+ */
+#if 0
 			if (rtp.type == RTP_PRIO_REALTIME)
+#endif
+			if (rtp.type != RTP_PRIO_NORMAL)
 				return (EPERM);
 		}
 		switch (rtp.type) {
@@ -266,15 +291,17 @@ rtprio(curp, uap, retval)
 }
 
 #if defined(COMPAT_43) || defined(COMPAT_SUNOS)
-struct setrlimit_args {
+#ifndef _SYS_SYSPROTO_H_
+struct osetrlimit_args {
 	u_int	which;
-	struct	orlimit *lim;
+	struct	orlimit *rlp;
 };
+#endif
 /* ARGSUSED */
 int
 osetrlimit(p, uap, retval)
 	struct proc *p;
-	register struct setrlimit_args *uap;
+	register struct osetrlimit_args *uap;
 	int *retval;
 {
 	struct orlimit olim;
@@ -282,22 +309,24 @@ osetrlimit(p, uap, retval)
 	int error;
 
 	if ((error =
-	    copyin((caddr_t)uap->lim, (caddr_t)&olim, sizeof(struct orlimit))))
+	    copyin((caddr_t)uap->rlp, (caddr_t)&olim, sizeof(struct orlimit))))
 		return (error);
 	lim.rlim_cur = olim.rlim_cur;
 	lim.rlim_max = olim.rlim_max;
 	return (dosetrlimit(p, uap->which, &lim));
 }
 
-struct getrlimit_args {
+#ifndef _SYS_SYSPROTO_H_
+struct ogetrlimit_args {
 	u_int	which;
 	struct	orlimit *rlp;
 };
+#endif
 /* ARGSUSED */
 int
 ogetrlimit(p, uap, retval)
 	struct proc *p;
-	register struct getrlimit_args *uap;
+	register struct ogetrlimit_args *uap;
 	int *retval;
 {
 	struct orlimit olim;
@@ -314,10 +343,12 @@ ogetrlimit(p, uap, retval)
 }
 #endif /* COMPAT_43 || COMPAT_SUNOS */
 
+#ifndef _SYS_SYSPROTO_H_
 struct __setrlimit_args {
 	u_int	which;
-	struct	rlimit *lim;
+	struct	rlimit *rlp;
 };
+#endif
 /* ARGSUSED */
 int
 setrlimit(p, uap, retval)
@@ -329,7 +360,7 @@ setrlimit(p, uap, retval)
 	int error;
 
 	if ((error =
-	    copyin((caddr_t)uap->lim, (caddr_t)&alim, sizeof (struct rlimit))))
+	    copyin((caddr_t)uap->rlp, (caddr_t)&alim, sizeof (struct rlimit))))
 		return (error);
 	return (dosetrlimit(p, uap->which, &alim));
 }
@@ -426,10 +457,12 @@ dosetrlimit(p, which, limp)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct __getrlimit_args {
 	u_int	which;
 	struct	rlimit *rlp;
 };
+#endif
 /* ARGSUSED */
 int
 getrlimit(p, uap, retval)
@@ -487,10 +520,8 @@ calcru(p, up, sp, ip)
 	}
 	totusec = (quad_t)sec * 1000000 + usec;
 	if (totusec < 0) {
-#ifdef DIAGNOSTIC
 		/* XXX no %qd in kernel.  Truncate. */
 		printf("calcru: negative time: %ld usec\n", (long)totusec);
-#endif
 		totusec = 0;
 	}
 	u = totusec;
@@ -507,10 +538,12 @@ calcru(p, up, sp, ip)
 	}
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct getrusage_args {
 	int	who;
 	struct	rusage *rusage;
 };
+#endif
 /* ARGSUSED */
 int
 getrusage(p, uap, retval)
