@@ -36,11 +36,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_sig.c	8.7 (Berkeley) 4/18/94
- * $FreeBSD: stable/2.1/sys/kern/kern_sig.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/kern_sig.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_ktrace.h"
+
 #define	SIGPROP		/* include signal properties table */
 #include <sys/param.h>
+#include <sys/sysproto.h>
 #include <sys/signalvar.h>
 #include <sys/resourcevar.h>
 #include <sys/namei.h>
@@ -57,14 +60,21 @@
 #include <sys/ktrace.h>
 #include <sys/syslog.h>
 #include <sys/stat.h>
+#include <sys/sysent.h>
 
 #include <machine/cpu.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 #include <sys/user.h>		/* for coredump */
 
-void setsigvec	__P((struct proc *, int, struct sigaction *));
-void stop	__P((struct proc *));
+static int coredump     __P((struct proc *p));
+static int killpg1	__P((struct proc *cp, int signum, int pgid, int all));
+static void stop	__P((struct proc *));
 
 /*
  * Can process p, with pcred pc, send the signal signum to process q?
@@ -77,11 +87,13 @@ void stop	__P((struct proc *));
 	    (pc)->pc_ucred->cr_uid == (q)->p_ucred->cr_uid || \
 	    ((signum) == SIGCONT && (q)->p_session == (p)->p_session))
 
+#ifndef _SYS_SYSPROTO_H_
 struct sigaction_args {
 	int	signum;
 	struct	sigaction *nsa;
 	struct	sigaction *osa;
 };
+#endif
 /* ARGSUSED */
 int
 sigaction(p, uap, retval)
@@ -96,8 +108,7 @@ sigaction(p, uap, retval)
 	int bit, error;
 
 	signum = uap->signum;
-	if (signum <= 0 || signum >= NSIG ||
-	    signum == SIGKILL || signum == SIGSTOP)
+	if (signum <= 0 || signum >= NSIG)
 		return (EINVAL);
 	sa = &vec;
 	if (uap->osa) {
@@ -109,7 +120,11 @@ sigaction(p, uap, retval)
 			sa->sa_flags |= SA_ONSTACK;
 		if ((ps->ps_sigintr & bit) == 0)
 			sa->sa_flags |= SA_RESTART;
-		if (p->p_flag & P_NOCLDSTOP)
+		if ((ps->ps_sigreset & bit) != 0)
+			sa->sa_flags |= SA_RESETHAND;
+		if ((ps->ps_signodefer & bit) != 0)
+			sa->sa_flags |= SA_NODEFER;
+		if (signum == SIGCHLD && p->p_flag & P_NOCLDSTOP)
 			sa->sa_flags |= SA_NOCLDSTOP;
 		if ((error = copyout((caddr_t)sa, (caddr_t)uap->osa,
 		    sizeof (vec))))
@@ -119,6 +134,9 @@ sigaction(p, uap, retval)
 		if ((error = copyin((caddr_t)uap->nsa, (caddr_t)sa,
 		    sizeof (vec))))
 			return (error);
+		if ((signum == SIGKILL || signum == SIGSTOP) &&
+		    sa->sa_handler != SIG_DFL)
+			return (EINVAL);
 		setsigvec(p, signum, sa);
 	}
 	return (0);
@@ -148,6 +166,14 @@ setsigvec(p, signum, sa)
 		ps->ps_sigonstack |= bit;
 	else
 		ps->ps_sigonstack &= ~bit;
+	if (sa->sa_flags & SA_RESETHAND)
+		ps->ps_sigreset |= bit;
+	else
+		ps->ps_sigreset &= ~bit;
+	if (sa->sa_flags & SA_NODEFER)
+		ps->ps_signodefer |= bit;
+	else
+		ps->ps_signodefer &= ~bit;
 #ifdef COMPAT_SUNOS
 	if (sa->sa_flags & SA_USERTRAMP)
 		ps->ps_usertramp |= bit;
@@ -227,7 +253,7 @@ execsigs(p)
 	 * Reset stack state to the user stack.
 	 * Clear set of signals caught on the signal stack.
 	 */
-	ps->ps_sigstk.ss_flags = SA_DISABLE;
+	ps->ps_sigstk.ss_flags = SS_DISABLE;
 	ps->ps_sigstk.ss_size = 0;
 	ps->ps_sigstk.ss_sp = 0;
 	ps->ps_flags = 0;
@@ -239,10 +265,12 @@ execsigs(p)
  * and return old mask as return value;
  * the library stub does the rest.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct sigprocmask_args {
 	int	how;
 	sigset_t mask;
 };
+#endif
 int
 sigprocmask(p, uap, retval)
 	register struct proc *p;
@@ -275,9 +303,11 @@ sigprocmask(p, uap, retval)
 	return (error);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct sigpending_args {
 	int	dummy;
 };
+#endif
 /* ARGSUSED */
 int
 sigpending(p, uap, retval)
@@ -294,11 +324,13 @@ sigpending(p, uap, retval)
 /*
  * Generalized interface signal handler, 4.3-compatible.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct osigvec_args {
 	int	signum;
 	struct	sigvec *nsv;
 	struct	sigvec *osv;
 };
+#endif
 /* ARGSUSED */
 int
 osigvec(p, uap, retval)
@@ -313,8 +345,7 @@ osigvec(p, uap, retval)
 	int bit, error;
 
 	signum = uap->signum;
-	if (signum <= 0 || signum >= NSIG ||
-	    signum == SIGKILL || signum == SIGSTOP)
+	if (signum <= 0 || signum >= NSIG)
 		return (EINVAL);
 	sv = &vec;
 	if (uap->osv) {
@@ -326,9 +357,13 @@ osigvec(p, uap, retval)
 			sv->sv_flags |= SV_ONSTACK;
 		if ((ps->ps_sigintr & bit) != 0)
 			sv->sv_flags |= SV_INTERRUPT;
+		if ((ps->ps_sigreset & bit) != 0)
+			sv->sv_flags |= SV_RESETHAND;
+		if ((ps->ps_signodefer & bit) != 0)
+			sv->sv_flags |= SV_NODEFER;
 #ifndef COMPAT_SUNOS
-		if (p->p_flag & P_NOCLDSTOP)
-			sv->sv_flags |= SA_NOCLDSTOP;
+		if (signum == SIGCHLD && p->p_flag & P_NOCLDSTOP)
+			sv->sv_flags |= SV_NOCLDSTOP;
 #endif
 		if ((error = copyout((caddr_t)sv, (caddr_t)uap->osv,
 		    sizeof (vec))))
@@ -338,14 +373,10 @@ osigvec(p, uap, retval)
 		if ((error = copyin((caddr_t)uap->nsv, (caddr_t)sv,
 		    sizeof (vec))))
 			return (error);
-#ifdef COMPAT_SUNOS
-		/*
-		 * SunOS uses this bit (4, aka SA_DISABLE) as SV_RESETHAND,
-		 * `reset to SIG_DFL on delivery'. We have no such option
-		 * now or ever!
-		 */
-		if (sv->sv_flags & SA_DISABLE)
+		if ((signum == SIGKILL || signum == SIGSTOP) &&
+		    sv->sv_handler != SIG_DFL)
 			return (EINVAL);
+#ifdef COMPAT_SUNOS
 		sv->sv_flags |= SA_USERTRAMP;
 #endif
 		sv->sv_flags ^= SA_RESTART;	/* opposite of SV_INTERRUPT */
@@ -354,9 +385,11 @@ osigvec(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct osigblock_args {
 	int	mask;
 };
+#endif
 int
 osigblock(p, uap, retval)
 	register struct proc *p;
@@ -371,9 +404,11 @@ osigblock(p, uap, retval)
 	return (0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct osigsetmask_args {
 	int	mask;
 };
+#endif
 int
 osigsetmask(p, uap, retval)
 	struct proc *p;
@@ -394,9 +429,11 @@ osigsetmask(p, uap, retval)
  * in the meantime.  Note nonstandard calling convention:
  * libc stub passes mask, not pointer, to save a copyin.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct sigsuspend_args {
 	sigset_t mask;
 };
+#endif
 /* ARGSUSED */
 int
 sigsuspend(p, uap, retval)
@@ -423,10 +460,12 @@ sigsuspend(p, uap, retval)
 }
 
 #if defined(COMPAT_43) || defined(COMPAT_SUNOS)
+#ifndef _SYS_SYSPROTO_H_
 struct osigstack_args {
 	struct	sigstack *nss;
 	struct	sigstack *oss;
 };
+#endif
 /* ARGSUSED */
 int
 osigstack(p, uap, retval)
@@ -440,7 +479,7 @@ osigstack(p, uap, retval)
 
 	psp = p->p_sigacts;
 	ss.ss_sp = psp->ps_sigstk.ss_sp;
-	ss.ss_onstack = psp->ps_sigstk.ss_flags & SA_ONSTACK;
+	ss.ss_onstack = psp->ps_sigstk.ss_flags & SS_ONSTACK;
 	if (uap->oss && (error = copyout((caddr_t)&ss, (caddr_t)uap->oss,
 	    sizeof (struct sigstack))))
 		return (error);
@@ -448,17 +487,19 @@ osigstack(p, uap, retval)
 	    sizeof (ss))) == 0) {
 		psp->ps_sigstk.ss_sp = ss.ss_sp;
 		psp->ps_sigstk.ss_size = 0;
-		psp->ps_sigstk.ss_flags |= ss.ss_onstack & SA_ONSTACK;
+		psp->ps_sigstk.ss_flags |= ss.ss_onstack & SS_ONSTACK;
 		psp->ps_flags |= SAS_ALTSTACK;
 	}
 	return (error);
 }
 #endif /* COMPAT_43 || COMPAT_SUNOS */
 
+#ifndef _SYS_SYSPROTO_H_
 struct sigaltstack_args {
 	struct	sigaltstack *nss;
 	struct	sigaltstack *oss;
 };
+#endif
 /* ARGSUSED */
 int
 sigaltstack(p, uap, retval)
@@ -472,7 +513,7 @@ sigaltstack(p, uap, retval)
 
 	psp = p->p_sigacts;
 	if ((psp->ps_flags & SAS_ALTSTACK) == 0)
-		psp->ps_sigstk.ss_flags |= SA_DISABLE;
+		psp->ps_sigstk.ss_flags |= SS_DISABLE;
 	if (uap->oss && (error = copyout((caddr_t)&psp->ps_sigstk,
 	    (caddr_t)uap->oss, sizeof (struct sigaltstack))))
 		return (error);
@@ -480,8 +521,8 @@ sigaltstack(p, uap, retval)
 		return (0);
 	if ((error = copyin((caddr_t)uap->nss, (caddr_t)&ss, sizeof (ss))))
 		return (error);
-	if (ss.ss_flags & SA_DISABLE) {
-		if (psp->ps_sigstk.ss_flags & SA_ONSTACK)
+	if (ss.ss_flags & SS_DISABLE) {
+		if (psp->ps_sigstk.ss_flags & SS_ONSTACK)
 			return (EINVAL);
 		psp->ps_flags &= ~SAS_ALTSTACK;
 		psp->ps_sigstk.ss_flags = ss.ss_flags;
@@ -512,7 +553,7 @@ killpg1(cp, signum, pgid, all)
 		/*
 		 * broadcast
 		 */
-		for (p = (struct proc *)allproc; p != NULL; p = p->p_next) {
+		for (p = allproc.lh_first; p != 0; p = p->p_list.le_next) {
 			if (p->p_pid <= 1 || p->p_flag & P_SYSTEM ||
 			    p == cp || !CANSIGNAL(cp, pc, p, signum))
 				continue;
@@ -531,7 +572,8 @@ killpg1(cp, signum, pgid, all)
 			if (pgrp == NULL)
 				return (ESRCH);
 		}
-		for (p = pgrp->pg_mem; p != NULL; p = p->p_pgrpnxt) {
+		for (p = pgrp->pg_members.lh_first; p != 0;
+		     p = p->p_pglist.le_next) {
 			if (p->p_pid <= 1 || p->p_flag & P_SYSTEM ||
 			    p->p_stat == SZOMB ||
 			    !CANSIGNAL(cp, pc, p, signum))
@@ -544,10 +586,12 @@ killpg1(cp, signum, pgid, all)
 	return (nfound ? 0 : ESRCH);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct kill_args {
 	int	pid;
 	int	signum;
 };
+#endif
 /* ARGSUSED */
 int
 kill(cp, uap, retval)
@@ -582,10 +626,12 @@ kill(cp, uap, retval)
 }
 
 #if defined(COMPAT_43) || defined(COMPAT_SUNOS)
+#ifndef _SYS_SYSPROTO_H_
 struct okillpg_args {
 	int	pgid;
 	int	signum;
 };
+#endif
 /* ARGSUSED */
 int
 okillpg(p, uap, retval)
@@ -625,7 +671,8 @@ pgsignal(pgrp, signum, checkctty)
 	register struct proc *p;
 
 	if (pgrp)
-		for (p = pgrp->pg_mem; p != NULL; p = p->p_pgrpnxt)
+		for (p = pgrp->pg_members.lh_first; p != 0;
+		     p = p->p_pglist.le_next)
 			if (checkctty == 0 || p->p_flag & P_CONTROLT)
 				psignal(p, signum);
 }
@@ -639,7 +686,7 @@ void
 trapsignal(p, signum, code)
 	struct proc *p;
 	register int signum;
-	u_int code;
+	u_long code;
 {
 	register struct sigacts *ps = p->p_sigacts;
 	int mask;
@@ -653,10 +700,22 @@ trapsignal(p, signum, code)
 			ktrpsig(p->p_tracep, signum, ps->ps_sigact[signum],
 				p->p_sigmask, code);
 #endif
-		sendsig(ps->ps_sigact[signum], signum, p->p_sigmask, code);
-		p->p_sigmask |= ps->ps_catchmask[signum] | mask;
+		(*p->p_sysent->sv_sendsig)(ps->ps_sigact[signum], signum,
+						p->p_sigmask, code);
+		p->p_sigmask |= ps->ps_catchmask[signum] |
+				(mask & ~ps->ps_signodefer);
+		if ((ps->ps_sigreset & mask) != 0) {
+			/*
+			 * See setsigvec() for origin of this code.
+			 */
+			p->p_sigcatch &= ~mask;
+			if (signum != SIGCONT && sigprop[signum] & SA_IGNORE)
+				p->p_sigignore |= mask;
+			ps->ps_sigact[signum] = SIG_DFL;
+		}
 	} else {
 		ps->ps_code = code;	/* XXX for core dump/debugger */
+		ps->ps_sig = signum;	/* XXX to verify code */
 		psignal(p, signum);
 	}
 }
@@ -1088,7 +1147,17 @@ postsig(signum)
 			ps->ps_flags &= ~SAS_OLDMASK;
 		} else
 			returnmask = p->p_sigmask;
-		p->p_sigmask |= ps->ps_catchmask[signum] | mask;
+		p->p_sigmask |= ps->ps_catchmask[signum] |
+				(mask & ~ps->ps_signodefer);
+		if ((ps->ps_sigreset & mask) != 0) {
+			/*
+			 * See setsigvec() for origin of this code.
+			 */
+			p->p_sigcatch &= ~mask;
+			if (signum != SIGCONT && sigprop[signum] & SA_IGNORE)
+				p->p_sigignore |= mask;
+			ps->ps_sigact[signum] = SIG_DFL;
+		}
 		(void) spl0();
 		p->p_stats->p_ru.ru_nsignals++;
 		if (ps->ps_sig != signum) {
@@ -1096,8 +1165,9 @@ postsig(signum)
 		} else {
 			code = ps->ps_code;
 			ps->ps_code = 0;
+			ps->ps_sig = 0;
 		}
-		sendsig(action, signum, returnmask, code);
+		(*p->p_sysent->sv_sendsig)(action, signum, returnmask, code);
 	}
 }
 
@@ -1137,12 +1207,13 @@ sigexit(p, signum)
 		 * these messages.)
 		 * XXX : Todo, as well as euid, write out ruid too
 		 */
-		log(LOG_INFO, "pid %d (%s), uid %d: exited on signal %d\n",
-			p->p_pid, p->p_comm,
-			p->p_cred && p->p_ucred ? p->p_ucred->cr_uid : -1,
-			signum);
 		if (coredump(p) == 0)
 			signum |= WCOREFLAG;
+		log(LOG_INFO, "pid %d (%s), uid %d: exited on signal %d%s\n",
+			p->p_pid, p->p_comm,
+			p->p_cred && p->p_ucred ? p->p_ucred->cr_uid : -1,
+			signum &~ WCOREFLAG,
+			signum & WCOREFLAG ? " (core dumped)" : "");
 	}
 	exit1(p, W_EXITCODE(0, signum));
 	/* NOTREACHED */
@@ -1152,7 +1223,7 @@ sigexit(p, signum)
  * Dump core, into a file named "progname.core", unless the process was
  * setuid/setgid.
  */
-int
+static int
 coredump(p)
 	register struct proc *p;
 {
@@ -1170,9 +1241,9 @@ coredump(p)
 	    p->p_rlimit[RLIMIT_CORE].rlim_cur)
 		return (EFAULT);
 	sprintf(name, "%s.core", p->p_comm);
-	NDINIT(&nd, LOOKUP, FOLLOW, UIO_SYSSPACE, name, p);
+	NDINIT(&nd, LOOKUP, NOFOLLOW, UIO_SYSSPACE, name, p);
 	if ((error = vn_open(&nd,
-	    O_CREAT | FWRITE, S_IRUSR | S_IWUSR)))
+	    O_CREAT | FWRITE | O_NOFOLLOW, S_IRUSR | S_IWUSR)))
 		return (error);
 	vp = nd.ni_vp;
 
@@ -1212,9 +1283,11 @@ out:
  * Nonexistent system call-- signal process (may want to handle it).
  * Flag error in case process won't see signal immediately (blocked or ignored).
  */
+#ifndef _SYS_SYSPROTO_H_
 struct nosys_args {
 	int	dummy;
 };
+#endif
 /* ARGSUSED */
 int
 nosys(p, args, retval)
