@@ -35,10 +35,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)kern_synch.c	8.6 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/kern_synch.c 50950 1999-09-05 08:39:22Z peter $
+ *	@(#)kern_synch.c	8.9 (Berkeley) 5/19/95
+ * $FreeBSD: stable/2.2/sys/kern/kern_synch.c 53946 1999-11-30 09:05:00Z peter $
  */
 
+#include "opt_ktrace.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
@@ -47,17 +49,52 @@
 #include <sys/signalvar.h>
 #include <sys/resourcevar.h>
 #include <sys/signalvar.h>
+#include <sys/vmmeter.h>
+#include <sys/sysctl.h>
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_extern.h>
 #ifdef KTRACE
 #include <sys/ktrace.h>
 #endif
 
 #include <machine/cpu.h>
 
+static void rqinit __P((void *));
+SYSINIT(runqueue, SI_SUB_RUN_QUEUE, SI_ORDER_FIRST, rqinit, NULL)
+
 u_char	curpriority;		/* usrpri of curproc */
 int	lbolt;			/* once a second sleep address */
 
-void	endtsleep __P((void *));
+extern void	endtsleep __P((void *));
+extern void	updatepri __P((struct proc *p));
+
+#define MAXIMUM_SCHEDULE_QUANTUM	(1000000) /* arbitrary limit */
+#ifndef DEFAULT_SCHEDULE_QUANTUM
+#define DEFAULT_SCHEDULE_QUANTUM 10
+#endif
+static int quantum = DEFAULT_SCHEDULE_QUANTUM; /* default value */
+
+static int
+sysctl_kern_quantum SYSCTL_HANDLER_ARGS
+{
+	int error;
+	int new_val = quantum;
+
+	new_val = quantum;
+	error = sysctl_handle_int(oidp, &new_val, 0, req);
+	if (error == 0) {
+		if ((new_val > 0) && (new_val < MAXIMUM_SCHEDULE_QUANTUM)) {
+			quantum = new_val;
+		} else {
+			error = EINVAL;
+		}
+	}
+	return (error);
+}
+
+SYSCTL_PROC(_kern, OID_AUTO, quantum, CTLTYPE_INT|CTLFLAG_RW,
+	0, sizeof quantum, sysctl_kern_quantum, "I", "");
 
 /*
  * Force switch among equal priority processes every 100ms.
@@ -69,7 +106,7 @@ roundrobin(arg)
 {
 
 	need_resched();
-	timeout(roundrobin, NULL, hz / 10);
+	timeout(roundrobin, NULL, hz / quantum);
 }
 
 /*
@@ -79,7 +116,7 @@ roundrobin(arg)
  *          Note that, as ps(1) mentions, this can let percentages
  *          total over 100% (I've seen 137.9% for 3 processes).
  *
- * Note that hardclock updates p_estcpu and p_cpticks independently.
+ * Note that statclock updates p_estcpu and p_cpticks independently.
  *
  * We wish to decay away 90% of p_estcpu in (5 * loadavg) seconds.
  * That is, the system wants to compute a value of decay such
@@ -171,7 +208,7 @@ schedcpu(arg)
 	register unsigned int newcpu;
 
 	wakeup((caddr_t)&lbolt);
-	for (p = (struct proc *)allproc; p != NULL; p = p->p_next) {
+	for (p = allproc.lh_first; p != 0; p = p->p_list.le_next) {
 		/*
 		 * Increment time in/out of memory and sleep time
 		 * (if sleeping).  We ignore overflow; with 16-bit int's
@@ -187,7 +224,7 @@ schedcpu(arg)
 		 */
 		if (p->p_slptime > 1)
 			continue;
-		s = splstatclock();	/* prevent state changes */
+		s = splhigh();	/* prevent state changes and protect run queue */
 		/*
 		 * p_pctcpu is only for ps.
 		 */
@@ -251,11 +288,8 @@ updatepri(p)
  * of 2.  Shift right by 8, i.e. drop the bottom 256 worth.
  */
 #define TABLESIZE	128
-#define LOOKUP(x)	(((int)(x) >> 8) & (TABLESIZE - 1))
-struct slpque {
-	struct proc *sq_head;
-	struct proc **sq_tailp;
-} slpque[TABLESIZE];
+TAILQ_HEAD(slpquehead, proc) slpque[TABLESIZE];
+#define LOOKUP(x)	(((long)(x) >> 8) & (TABLESIZE - 1))
 
 /*
  * During autoconfiguration or after a panic, a sleep will simply
@@ -268,6 +302,15 @@ struct slpque {
  */
 int safepri;
 
+void
+sleepinit()
+{
+	int i;
+
+	for (i = 0; i < TABLESIZE; i++)
+		TAILQ_INIT(&slpque[i]);
+}
+
 /*
  * General sleep call.  Suspends the current process until a wakeup is
  * performed on the specified identifier.  The process will then be made
@@ -285,13 +328,11 @@ tsleep(ident, priority, wmesg, timo)
 	int priority, timo;
 	char *wmesg;
 {
-	register struct proc *p = curproc;
-	register struct slpque *qp;
-	register s;
-	int sig, catch = priority & PCATCH;
+	struct proc *p = curproc;
+	int s, sig, catch = priority & PCATCH;
 
 #ifdef KTRACE
-	if (KTRPOINT(p, KTR_CSW))
+	if (p && KTRPOINT(p, KTR_CSW))
 		ktrcsw(p->p_tracep, 1, 0);
 #endif
 	s = splhigh();
@@ -307,19 +348,19 @@ tsleep(ident, priority, wmesg, timo)
 		return (0);
 	}
 #ifdef DIAGNOSTIC
-	if (ident == NULL || p->p_stat != SRUN || p->p_back)
+	if(p == NULL) 
+		panic("tsleep1");
+	if (ident == NULL || p->p_stat != SRUN)
 		panic("tsleep");
+	/* XXX This is not exhaustive, just the most common case */
+	if ((p->p_procq.tqe_prev != NULL) && (*p->p_procq.tqe_prev == p))
+		panic("sleeping process already on another queue");
 #endif
 	p->p_wchan = ident;
 	p->p_wmesg = wmesg;
 	p->p_slptime = 0;
 	p->p_priority = priority & PRIMASK;
-	qp = &slpque[LOOKUP(ident)];
-	if (qp->sq_head == 0)
-		qp->sq_head = p;
-	else
-		*qp->sq_tailp = p;
-	*(qp->sq_tailp = &p->p_forw) = 0;
+	TAILQ_INSERT_TAIL(&slpque[LOOKUP(ident)], p, p_procq);
 	if (timo)
 		timeout(endtsleep, (void *)p, timo);
 	/*
@@ -404,66 +445,6 @@ endtsleep(arg)
 	splx(s);
 }
 
-/*
- * Short-term, non-interruptable sleep.
- */
-void
-sleep(ident, priority)
-	void *ident;
-	int priority;
-{
-	register struct proc *p = curproc;
-	register struct slpque *qp;
-	register s;
-
-#ifdef DIAGNOSTIC
-	if (priority > PZERO) {
-		printf("sleep called with priority %d > PZERO, wchan: %p\n",
-		    priority, ident);
-		panic("old sleep");
-	}
-#endif
-	s = splhigh();
-	if (cold || panicstr) {
-		/*
-		 * After a panic, or during autoconfiguration,
-		 * just give interrupts a chance, then just return;
-		 * don't run any other procs or panic below,
-		 * in case this is the idle process and already asleep.
-		 */
-		splx(safepri);
-		splx(s);
-		return;
-	}
-#ifdef DIAGNOSTIC
-	if (ident == NULL || p->p_stat != SRUN || p->p_back)
-		panic("sleep");
-#endif
-	p->p_wchan = ident;
-	p->p_wmesg = NULL;
-	p->p_slptime = 0;
-	p->p_priority = priority;
-	qp = &slpque[LOOKUP(ident)];
-	if (qp->sq_head == 0)
-		qp->sq_head = p;
-	else
-		*qp->sq_tailp = p;
-	*(qp->sq_tailp = &p->p_forw) = 0;
-	p->p_stat = SSLEEP;
-	p->p_stats->p_ru.ru_nvcsw++;
-#ifdef KTRACE
-	if (KTRPOINT(p, KTR_CSW))
-		ktrcsw(p->p_tracep, 1, 0);
-#endif
-	mi_switch();
-#ifdef KTRACE
-	if (KTRPOINT(p, KTR_CSW))
-		ktrcsw(p->p_tracep, 0, 0);
-#endif
-	curpriority = p->p_usrpri;
-	splx(s);
-}
-
 /*
  * Remove a process from its wait queue
  */
@@ -471,18 +452,11 @@ void
 unsleep(p)
 	register struct proc *p;
 {
-	register struct slpque *qp;
-	register struct proc **hp;
 	int s;
 
 	s = splhigh();
 	if (p->p_wchan) {
-		hp = &(qp = &slpque[LOOKUP(p->p_wchan)])->sq_head;
-		while (*hp != p)
-			hp = &(*hp)->p_forw;
-		*hp = p->p_forw;
-		if (qp->sq_tailp == &p->p_forw)
-			qp->sq_tailp = hp;
+		TAILQ_REMOVE(&slpque[LOOKUP(p->p_wchan)], p, p_procq);
 		p->p_wchan = 0;
 	}
 	splx(s);
@@ -495,46 +469,83 @@ void
 wakeup(ident)
 	register void *ident;
 {
-	register struct slpque *qp;
-	register struct proc *p, **q;
+	register struct slpquehead *qp;
+	register struct proc *p;
 	int s;
 
 	s = splhigh();
 	qp = &slpque[LOOKUP(ident)];
 restart:
-	for (q = &qp->sq_head; *q; ) {
-		p = *q;
+	for (p = qp->tqh_first; p != NULL; p = p->p_procq.tqe_next) {
 #ifdef DIAGNOSTIC
-		if (p->p_back || (p->p_stat != SSLEEP && p->p_stat != SSTOP))
+		if (p->p_stat != SSLEEP && p->p_stat != SSTOP)
 			panic("wakeup");
 #endif
 		if (p->p_wchan == ident) {
+			TAILQ_REMOVE(qp, p, p_procq);
 			p->p_wchan = 0;
-			*q = p->p_forw;
-			if (qp->sq_tailp == &p->p_forw)
-				qp->sq_tailp = q;
 			if (p->p_stat == SSLEEP) {
 				/* OPTIMIZED EXPANSION OF setrunnable(p); */
 				if (p->p_slptime > 1)
 					updatepri(p);
 				p->p_slptime = 0;
 				p->p_stat = SRUN;
-				if (p->p_flag & P_INMEM)
+				if (p->p_flag & P_INMEM) {
 					setrunqueue(p);
-				/*
-				 * Since curpriority is a user priority,
-				 * p->p_priority is always better than
-				 * curpriority.
-				 */
-				if ((p->p_flag & P_INMEM) == 0)
-					wakeup((caddr_t)&proc0);
-				else
 					need_resched();
+				} else {
+					p->p_flag |= P_SWAPINREQ;
+					wakeup((caddr_t)&proc0);
+				}
 				/* END INLINE EXPANSION */
 				goto restart;
 			}
-		} else
-			q = &p->p_forw;
+		}
+	}
+	splx(s);
+}
+
+/*
+ * Make a process sleeping on the specified identifier runnable.
+ * May wake more than one process if a target prcoess is currently
+ * swapped out.
+ */
+void
+wakeup_one(ident)
+	register void *ident;
+{
+	register struct slpquehead *qp;
+	register struct proc *p;
+	int s;
+
+	s = splhigh();
+	qp = &slpque[LOOKUP(ident)];
+
+	for (p = qp->tqh_first; p != NULL; p = p->p_procq.tqe_next) {
+#ifdef DIAGNOSTIC
+		if (p->p_stat != SSLEEP && p->p_stat != SSTOP)
+			panic("wakeup_one");
+#endif
+		if (p->p_wchan == ident) {
+			TAILQ_REMOVE(qp, p, p_procq);
+			p->p_wchan = 0;
+			if (p->p_stat == SSLEEP) {
+				/* OPTIMIZED EXPANSION OF setrunnable(p); */
+				if (p->p_slptime > 1)
+					updatepri(p);
+				p->p_slptime = 0;
+				p->p_stat = SRUN;
+				if (p->p_flag & P_INMEM) {
+					setrunqueue(p);
+					need_resched();
+					break;
+				} else {
+					p->p_flag |= P_SWAPINREQ;
+					wakeup((caddr_t)&proc0);
+				}
+				/* END INLINE EXPANSION */
+			}
+		}
 	}
 	splx(s);
 }
@@ -549,8 +560,32 @@ mi_switch()
 	register struct proc *p = curproc;	/* XXX */
 	register struct rlimit *rlim;
 	register long s, u;
+	int x;
 	struct timeval tv;
 
+	/*
+	 * XXX this spl is almost unnecessary.  It is partly to allow for
+	 * sloppy callers that don't do it (issignal() via CURSIG() is the
+	 * main offender).  It is partly to work around a bug in the i386
+	 * cpu_switch() (the ipl is not preserved).  We ran for years
+	 * without it.  I think there was only a interrupt latency problem.
+	 * The main caller, tsleep(), does an splx() a couple of instructions
+	 * after calling here.  The buggy caller, issignal(), usually calls
+	 * here at spl0() and sometimes returns at splhigh().  The process
+	 * then runs for a little too long at splhigh().  The ipl gets fixed
+	 * when the process returns to user mode (or earlier).
+	 *
+	 * It would probably be better to always call here at spl0(). Callers
+	 * are prepared to give up control to another process, so they must
+	 * be prepared to be interrupted.  The clock stuff here may not
+	 * actually need splstatclock().
+	 */
+	x = splstatclock();
+
+#ifdef DEBUG
+	if (p->p_simple_locks)
+		panic("sleep: holding simple lock");
+#endif
 	/*
 	 * Compute the amount of time during which the current
 	 * process was running, and add that to its total so far.
@@ -570,8 +605,7 @@ mi_switch()
 
 	/*
 	 * Check if the process exceeds its cpu resource allocation.
-	 * If over max, kill it.  In any case, if it has run for more
-	 * than 10 minutes, reduce priority to give others a chance.
+	 * If over max, kill it.
 	 */
 	if (p->p_stat != SZOMB) {
 		rlim = &p->p_rlimit[RLIMIT_CPU];
@@ -584,10 +618,6 @@ mi_switch()
 					rlim->rlim_cur += 5;
 			}
 		}
-		if (s > 10 * 60 && p->p_ucred->cr_uid && p->p_nice == NZERO) {
-			p->p_nice = NZERO + 4;
-			resetpriority(p);
-		}
 	}
 
 	/*
@@ -596,14 +626,17 @@ mi_switch()
 	cnt.v_swtch++;
 	cpu_switch(p);
 	microtime(&runtime);
+	splx(x);
 }
 
 /*
  * Initialize the (doubly-linked) run queues
  * to be empty.
  */
-void
-rqinit()
+/* ARGSUSED*/
+static void
+rqinit(dummy)
+	void *dummy;
 {
 	register int i;
 
@@ -647,8 +680,10 @@ setrunnable(p)
 	if (p->p_slptime > 1)
 		updatepri(p);
 	p->p_slptime = 0;
-	if ((p->p_flag & P_INMEM) == 0)
+	if ((p->p_flag & P_INMEM) == 0) {
+		p->p_flag |= P_SWAPINREQ;
 		wakeup((caddr_t)&proc0);
+	}
 	else if (p->p_priority < curpriority)
 		need_resched();
 }
