@@ -43,11 +43,15 @@
  *	from: wd.c,v 1.55 1994/10/22 01:57:12 phk Exp $
  *	from: @(#)ufs_disksubr.c	7.16 (Berkeley) 5/4/91
  *	from: ufs_disksubr.c,v 1.8 1994/06/07 01:21:39 phk Exp $
- * $FreeBSD: stable/2.1/sys/kern/subr_diskslice.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/subr_diskslice.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/buf.h>
+#include <sys/conf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif
 #include <sys/disklabel.h>
 #include <sys/diskslice.h>
 #include <sys/dkbad.h>
@@ -59,10 +63,9 @@
 #include <sys/systm.h>
 #include <sys/vnode.h>
 
-#define b_cylinder	b_resid
+#include <ufs/ffs/fs.h>
 
-#define	FALSE	0
-#define	TRUE	1
+#define b_cylinder	b_resid
 
 #define TRACE(str)	do { if (ds_debug) printf str; } while (0)
 
@@ -73,12 +76,22 @@ static volatile bool_t ds_debug;
 static void dsiodone __P((struct buf *bp));
 static char *fixlabel __P((char *sname, struct diskslice *sp,
 			   struct disklabel *lp, int writeflag));
+static void free_ds_label __P((struct diskslices *ssp, int slice));
+#ifdef DEVFS
+static void free_ds_labeldevs __P((struct diskslices *ssp, int slice));
+#endif
 static void partition_info __P((char *sname, int part, struct partition *pp));
 static void slice_info __P((char *sname, struct diskslice *sp));
 static void set_ds_bad __P((struct diskslices *ssp, int slice,
 			    struct dkbad_intern *btp));
 static void set_ds_label __P((struct diskslices *ssp, int slice,
 			      struct disklabel *lp));
+#ifdef DEVFS
+static void set_ds_labeldevs __P((char *dname, dev_t dev,
+				  struct diskslices *ssp));
+static void set_ds_labeldevs_unaliased __P((char *dname, dev_t dev,
+					    struct diskslices *ssp));
+#endif
 static void set_ds_wlabel __P((struct diskslices *ssp, int slice,
 			       int wlabel));
 
@@ -107,6 +120,12 @@ dscheck(bp, ssp)
 	struct diskslice *sp;
 	long	sz;
 
+	if (bp->b_blkno < 0) {
+		Debugger("Slice code got negative blocknumber");
+		bp->b_error = EINVAL;
+		goto bad;
+	}
+
 	sp = &ssp->dss_slices[dkslice(bp->b_dev)];
 	lp = sp->ds_label;
 	sz = (bp->b_bcount + DEV_BSIZE - 1) >> DEV_BSHIFT;
@@ -261,10 +280,13 @@ dsgone(sspp)
 			free(sp->ds_bad, M_DEVBUF);
 			set_ds_bad(ssp, slice, (struct dkbad_intern *)NULL);
 		}
-		if (sp->ds_label != NULL) {
-			free(sp->ds_label, M_DEVBUF);
-			set_ds_label(ssp, slice, (struct disklabel *)NULL);
-		}
+#ifdef DEVFS
+		if (sp->ds_bdev != NULL)
+			devfs_remove_dev(sp->ds_bdev);
+		if (sp->ds_cdev != NULL)
+			devfs_remove_dev(sp->ds_cdev);
+#endif
+		free_ds_label(ssp, slice);
 	}
 	free(ssp, M_DEVBUF);
 	*sspp = NULL;
@@ -288,6 +310,8 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 	int	error;
 	struct disklabel *lp;
 	int	old_wlabel;
+	u_char	openmask;
+	int	part;
 	int	slice;
 	struct diskslice *sp;
 	struct diskslices *ssp;
@@ -321,7 +345,8 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 		return (0);
 
 	case DIOCGSLICEINFO:
-		*(struct diskslices *)data = *ssp;
+		bcopy(ssp, data, (char *)&ssp->dss_slices[ssp->dss_nslices] -
+				 (char *)ssp);
 		return (0);
 
 	case DIOCSBAD:
@@ -346,12 +371,29 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 			bzero(lp, sizeof *lp);
 		else
 			bcopy(sp->ds_label, lp, sizeof *lp);
+		if (sp->ds_label == NULL)
+			openmask = 0;
+		else {
+			openmask = sp->ds_openmask;
+			if (slice == COMPATIBILITY_SLICE)
+				openmask |= ssp->dss_slices[
+				    ssp->dss_first_bsd_slice].ds_openmask;
+			else if (slice == ssp->dss_first_bsd_slice)
+				openmask |= ssp->dss_slices[
+				    COMPATIBILITY_SLICE].ds_openmask;
+		}
 		error = setdisklabel(lp, (struct disklabel *)data,
-				     sp->ds_label != NULL
-				     ? sp->ds_openmask : (u_long)0);
+				     (u_long)openmask);
 		/* XXX why doesn't setdisklabel() check this? */
 		if (error == 0 && lp->d_partitions[RAW_PART].p_offset != 0)
 			error = EINVAL;
+		if (error == 0) {
+			if (lp->d_secperunit > sp->ds_size)
+				error = ENOSPC;
+			for (part = 0; part < lp->d_npartitions; part++)
+				if (lp->d_partitions[part].p_size > sp->ds_size)
+					error = ENOSPC;
+		}
 #if 0 /* XXX */
 		if (error != 0 && setgeom != NULL)
 			error = setgeom(lp);
@@ -360,9 +402,11 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 			free(lp, M_DEVBUF);
 			return (error);
 		}
-		if (sp->ds_label != NULL)
-			free(sp->ds_label, M_DEVBUF);
+		free_ds_label(ssp, slice);
 		set_ds_label(ssp, slice, lp);
+#ifdef DEVFS
+		set_ds_labeldevs(dname, dev, ssp);
+#endif
 		return (0);
 
 	case DIOCSYNCSLICEINFO:
@@ -391,7 +435,8 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 		error = dsopen(dname, dev,
 			       ssp->dss_slices[WHOLE_DISK_SLICE].ds_copenmask
 			       & (1 << RAW_PART) ? S_IFCHR : S_IFBLK,
-			       sspp, lp, strat, setgeom);
+			       sspp, lp, strat, setgeom, ssp->dss_bdevsw,
+			       ssp->dss_cdevsw);
 		if (error != 0) {
 			free(lp, M_DEVBUF);
 			*sspp = ssp;
@@ -404,9 +449,6 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 		 * if anything fails.
 		 */
 		for (slice = 0; slice < ssp->dss_nslices; slice++) {
-			u_char	openmask;
-			int	part;
-
 			for (openmask = ssp->dss_slices[slice].ds_bopenmask,
 			     part = 0; openmask; openmask >>= 1, part++) {
 				if (!(openmask & 1))
@@ -415,9 +457,12 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 					       dkmodslice(dkmodpart(dev, part),
 							  slice),
 					       S_IFBLK, sspp, lp, strat,
-					       setgeom);
+					       setgeom, ssp->dss_bdevsw,
+					       ssp->dss_cdevsw);
 				if (error != 0) {
+					/* XXX should free devfs toks. */
 					free(lp, M_DEVBUF);
+					/* XXX should restore devfs toks. */
 					*sspp = ssp;
 					return (EBUSY);
 				}
@@ -430,15 +475,19 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 					       dkmodslice(dkmodpart(dev, part),
 							  slice),
 					       S_IFCHR, sspp, lp, strat,
-					       setgeom);
+					       setgeom, ssp->dss_bdevsw,
+					       ssp->dss_cdevsw);
 				if (error != 0) {
+					/* XXX should free devfs toks. */
 					free(lp, M_DEVBUF);
+					/* XXX should restore devfs toks. */
 					*sspp = ssp;
 					return (EBUSY);
 				}
 			}
 		}
 
+		/* XXX devfs tokens? */
 		free(lp, M_DEVBUF);
 		dsgone(&ssp);
 		return (0);
@@ -454,7 +503,7 @@ dsioctl(dname, dev, cmd, data, flags, sspp, strat, setgeom)
 		 */
 		old_wlabel = sp->ds_wlabel;
 		set_ds_wlabel(ssp, slice, TRUE);
-		error = correct_writedisklabel(dev, strat, sp->ds_label);
+		error = writedisklabel(dev, strat, sp->ds_label);
 		/* XXX should invalidate in-core label if write failed. */
 		set_ds_wlabel(ssp, slice, old_wlabel);
 		return (error);
@@ -540,7 +589,7 @@ dsname(dname, unit, slice, part, partname)
  * strategy routine must be special to allow activity.
  */
 int
-dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
+dsopen(dname, dev, mode, sspp, lp, strat, setgeom, bdevsw, cdevsw)
 	char	*dname;
 	dev_t	dev;
 	int	mode;
@@ -548,11 +597,18 @@ dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
 	struct disklabel *lp;
 	d_strategy_t *strat;
 	ds_setgeom_t *setgeom;
+	struct bdevsw *bdevsw;
+	struct cdevsw *cdevsw;
 {
+	struct dkbad *btp;
+	dev_t	dev1;
 	int	error;
 	struct disklabel *lp1;
 	char	*msg;
 	u_char	mask;
+#ifdef DEVFS
+	int	mynor;
+#endif
 	bool_t	need_init;
 	int	part;
 	char	partname[2];
@@ -577,9 +633,13 @@ dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
 			dsgone(sspp);
 			return (error);
 		}
-		lp->d_npartitions = RAW_PART + 1;
+		lp->d_npartitions = MAXPARTITIONS;
 		lp->d_partitions[RAW_PART].p_size = lp->d_secperunit;
 		ssp = *sspp;
+#ifdef DEVFS
+		ssp->dss_bdevsw = bdevsw;
+		ssp->dss_cdevsw = cdevsw;
+#endif
 
 		/*
 		 * If there are no real slices, then make the compatiblity
@@ -590,8 +650,8 @@ dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
 				= lp->d_secperunit;
 
 		/* Point the compatibility slice at the BSD slice, if any. */
-		for (slice = BASE_SLICE, sp = &ssp->dss_slices[BASE_SLICE];
-		     slice < ssp->dss_nslices; slice++, sp++)
+		for (slice = BASE_SLICE; slice < ssp->dss_nslices; slice++) {
+			sp = &ssp->dss_slices[slice];
 			if (sp->ds_type == DOSPTYP_386BSD /* XXX */) {
 				ssp->dss_first_bsd_slice = slice;
 				ssp->dss_slices[COMPATIBILITY_SLICE].ds_offset
@@ -602,9 +662,26 @@ dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
 					= sp->ds_type;
 				break;
 			}
+		}
 
 		lp1 = malloc(sizeof *lp1, M_DEVBUF, M_WAITOK);
 		*lp1 = *lp;
+
+		/*
+		 * Initialize defaults for the label for the whole disk so
+		 * that it can be used as a template for disklabel(8).
+		 * d_rpm = 3600 is unlikely to be correct for a modern
+		 * disk, but d_rpm is normally irrelevant.
+		 */
+		if (lp1->d_rpm == 0)
+			lp1->d_rpm = 3600;
+		if (lp1->d_interleave == 0)
+			lp1->d_interleave = 1;
+		if (lp1->d_bbsize == 0)
+			lp1->d_bbsize = BBSIZE;
+		if (lp1->d_sbsize == 0)
+			lp1->d_sbsize = SBSIZE;
+
 		ssp->dss_slices[WHOLE_DISK_SLICE].ds_label = lp1;
 		ssp->dss_slices[WHOLE_DISK_SLICE].ds_wlabel = TRUE;
 		if (setgeom != NULL) {
@@ -616,67 +693,88 @@ dsopen(dname, dev, mode, sspp, lp, strat, setgeom)
 		}
 	}
 
-	slice = dkslice(dev);
-	if (slice >= ssp->dss_nslices)
-		return (ENXIO);
-	sp = &ssp->dss_slices[slice];
-	part = dkpart(dev);
 	unit = dkunit(dev);
-	if (sp->ds_label == NULL) {
+
+	/*
+	 * Initialize secondary info for all slices.  It is needed for more
+	 * than the current slice in the DEVFS case.
+	 */
+	for (slice = 0; slice < ssp->dss_nslices; slice++) {
+		sp = &ssp->dss_slices[slice];
+		if (sp->ds_label != NULL)
+			continue;
+		dev1 = dkmodslice(dkmodpart(dev, RAW_PART), slice);
+		sname = dsname(dname, unit, slice, RAW_PART, partname);
+#ifdef DEVFS
+		if (slice != COMPATIBILITY_SLICE && sp->ds_bdev == NULL
+		    && sp->ds_size != 0) {
+			mynor = minor(dev1);
+			sp->ds_bdev =
+				devfs_add_devswf(bdevsw, mynor, DV_BLK,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "%s", sname);
+			sp->ds_cdev =
+				devfs_add_devswf(cdevsw, mynor, DV_CHR,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "r%s", sname);
+		}
+#endif
+		/*
+		 * XXX this should probably only be done for the need_init
+		 * case, but there may be a problem with DIOCSYNCSLICEINFO.
+		 */
 		set_ds_wlabel(ssp, slice, TRUE);	/* XXX invert */
 		lp1 = malloc(sizeof *lp1, M_DEVBUF, M_WAITOK);
 		*lp1 = *lp;
-		lp = lp1;
 		TRACE(("readdisklabel\n"));
-		msg = correct_readdisklabel(dkmodpart(dev, RAW_PART), strat, lp);
+		msg = readdisklabel(dev1, strat, lp1);
 #if 0 /* XXX */
-		if (msg == NULL && setgeom != NULL && setgeom(lp) != 0)
+		if (msg == NULL && setgeom != NULL && setgeom(lp1) != 0)
 			msg = "setgeom failed";
 #endif
-		sname = dsname(dname, unit, slice, RAW_PART, partname);
 		if (msg == NULL)
-			msg = fixlabel(sname, sp, lp, FALSE);
+			msg = fixlabel(sname, sp, lp1, FALSE);
 		if (msg != NULL) {
-			free(lp, M_DEVBUF);
+			free(lp1, M_DEVBUF);
 			if (sp->ds_type == DOSPTYP_386BSD /* XXX */)
 				log(LOG_WARNING, "%s: cannot find label (%s)\n",
 				    sname, msg);
-			if (part == RAW_PART)
-				goto out;
-			return (EINVAL);	/* XXX needs translation */
+			continue;
 		}
-		if (lp->d_flags & D_BADSECT) {
-			struct dkbad *btp;
-
+		if (lp1->d_flags & D_BADSECT) {
 			btp = malloc(sizeof *btp, M_DEVBUF, M_WAITOK);
 			TRACE(("readbad144\n"));
-			msg = readbad144(dev, strat, lp, btp);
+			msg = readbad144(dev1, strat, lp1, btp);
 			if (msg != NULL) {
 				log(LOG_WARNING,
 				    "%s: cannot find bad sector table (%s)\n",
 				    sname, msg);
 				free(btp, M_DEVBUF);
-				free(lp, M_DEVBUF);
-				if (part == RAW_PART)
-					goto out;
-				return (EINVAL);  /* XXX needs translation */
+				free(lp1, M_DEVBUF);
+				continue;
 			}
-			set_ds_bad(ssp, slice, internbad144(btp, lp));
+			set_ds_bad(ssp, slice, internbad144(btp, lp1));
 			free(btp, M_DEVBUF);
 			if (sp->ds_bad == NULL) {
-				free(lp, M_DEVBUF);
-				if (part == RAW_PART)
-					goto out;
-				return (EINVAL);  /* XXX needs translation */
+				free(lp1, M_DEVBUF);
+				continue;
 			}
 		}
-		set_ds_label(ssp, slice, lp);
+		set_ds_label(ssp, slice, lp1);
+#ifdef DEVFS
+		set_ds_labeldevs(dname, dev1, ssp);
+#endif
 		set_ds_wlabel(ssp, slice, FALSE);
 	}
+
+	slice = dkslice(dev);
+	if (slice >= ssp->dss_nslices)
+		return (ENXIO);
+	sp = &ssp->dss_slices[slice];
+	part = dkpart(dev);
 	if (part != RAW_PART
 	    && (sp->ds_label == NULL || part >= sp->ds_label->d_npartitions))
 		return (EINVAL);	/* XXX needs translation */
-out:
 	mask = 1 << part;
 	switch (mode) {
 	case S_IFBLK:
@@ -718,6 +816,56 @@ dssize(dev, sspp, dopen, dclose)
 	return ((int)lp->d_partitions[part].p_size);
 }
 
+static void
+free_ds_label(ssp, slice)
+	struct diskslices *ssp;
+	int	slice;
+{
+	struct disklabel *lp;
+	struct diskslice *sp;
+
+	sp = &ssp->dss_slices[slice];
+	lp = sp->ds_label;
+	if (lp == NULL)
+		return;
+#ifdef DEVFS
+	free_ds_labeldevs(ssp, slice);
+	if (slice == COMPATIBILITY_SLICE)
+		free_ds_labeldevs(ssp, ssp->dss_first_bsd_slice);
+	else if (slice == ssp->dss_first_bsd_slice)
+		free_ds_labeldevs(ssp, COMPATIBILITY_SLICE);
+#endif
+	free(lp, M_DEVBUF);
+	set_ds_label(ssp, slice, (struct disklabel *)NULL);
+}
+
+#ifdef DEVFS
+static void
+free_ds_labeldevs(ssp, slice)
+	struct diskslices *ssp;
+	int	slice;
+{
+	struct disklabel *lp;
+	int	part;
+	struct diskslice *sp;
+
+	sp = &ssp->dss_slices[slice];
+	lp = sp->ds_label;
+	if (lp == NULL)
+		return;
+	for (part = 0; part < lp->d_npartitions; part++) {
+		if (sp->ds_bdevs[part] != NULL) {
+			devfs_remove_dev(sp->ds_bdevs[part]);
+			sp->ds_bdevs[part] = NULL;
+		}
+		if (sp->ds_cdevs[part] != NULL) {
+			devfs_remove_dev(sp->ds_cdevs[part]);
+			sp->ds_cdevs[part] = NULL;
+		}
+	}
+}
+#endif
+
 static char *
 fixlabel(sname, sp, lp, writeflag)
 	char	*sname;
@@ -856,6 +1004,74 @@ set_ds_label(ssp, slice, lp)
 		ssp->dss_slices[COMPATIBILITY_SLICE].ds_label = lp;
 }
 
+#ifdef DEVFS
+static void
+set_ds_labeldevs(dname, dev, ssp)
+	char	*dname;
+	dev_t	dev;
+	struct diskslices *ssp;
+{
+	int	slice;
+
+	set_ds_labeldevs_unaliased(dname, dev, ssp);
+	if (ssp->dss_first_bsd_slice == COMPATIBILITY_SLICE)
+		return;
+	slice = dkslice(dev);
+	if (slice == COMPATIBILITY_SLICE)
+		set_ds_labeldevs_unaliased(dname,
+			dkmodslice(dev, ssp->dss_first_bsd_slice), ssp);
+	else if (slice == ssp->dss_first_bsd_slice)
+		set_ds_labeldevs_unaliased(dname,
+			dkmodslice(dev, COMPATIBILITY_SLICE), ssp);
+}
+
+static void
+set_ds_labeldevs_unaliased(dname, dev, ssp)
+	char	*dname;
+	dev_t	dev;
+	struct diskslices *ssp;
+{
+	struct disklabel *lp;
+	int	mynor;
+	int	part;
+	char	partname[2];
+	struct partition *pp;
+	int	slice;
+	char	*sname;
+	struct diskslice *sp;
+ 
+	slice = dkslice(dev);
+	sp = &ssp->dss_slices[slice];
+	if (sp->ds_size == 0)
+		return;
+	lp = sp->ds_label;
+	for (part = 0; part < lp->d_npartitions; part++) {
+		pp = &lp->d_partitions[part];
+		if (pp->p_size == 0)
+			continue;
+		sname = dsname(dname, dkunit(dev), slice, part, partname);
+		if (part == RAW_PART && sp->ds_bdev != NULL) {
+			sp->ds_bdevs[part] =
+				devfs_link(sp->ds_bdev,
+					   "%s%s", sname, partname);
+			sp->ds_cdevs[part] =
+				devfs_link(sp->ds_cdev,
+					   "r%s%s", sname, partname);
+		} else {
+			mynor = minor(dkmodpart(dev, part));
+			sp->ds_bdevs[part] =
+				devfs_add_devswf(ssp->dss_bdevsw, mynor, DV_BLK,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "%s%s", sname, partname);
+			sp->ds_cdevs[part] =
+				devfs_add_devswf(ssp->dss_cdevsw, mynor, DV_CHR,
+						 UID_ROOT, GID_OPERATOR, 0640,
+						 "r%s%s", sname, partname);
+		}
+	}
+}
+#endif /* DEVFS */
+
 static void
 set_ds_wlabel(ssp, slice, wlabel)
 	struct diskslices *ssp;
