@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)subr_log.c	8.1 (Berkeley) 6/10/93
- * $FreeBSD: stable/2.1/sys/kern/subr_log.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/subr_log.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -40,19 +40,36 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/proc.h>
 #include <sys/vnode.h>
 #include <sys/ioctl.h>
 #include <sys/msgbuf.h>
 #include <sys/file.h>
 #include <sys/signalvar.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #define LOG_RDPRI	(PZERO + 1)
 
 #define LOG_ASYNC	0x04
 #define LOG_RDWAIT	0x08
 
-struct logsoftc {
+static	d_open_t	logopen;
+static	d_close_t	logclose;
+static	d_read_t	logread;
+static	d_ioctl_t	logioctl;
+static	d_select_t	logselect;
+
+#define CDEV_MAJOR 7
+static struct cdevsw log_cdevsw = 
+	{ logopen,	logclose,	logread,	nowrite,	/*7*/
+	  logioctl,	nostop,		nullreset,	nodevtotty,/* klog */
+	  logselect,	nommap,		NULL,	"log",	NULL,	-1 };
+
+static struct logsoftc {
 	int	sc_state;		/* see above for possibilities */
 	struct	selinfo sc_selp;	/* process waiting on select call */
 	int	sc_pgid;		/* process/group for async I/O */
@@ -61,14 +78,12 @@ struct logsoftc {
 int	log_open;			/* also used in log() */
 
 /*ARGSUSED*/
-int
+static	int
 logopen(dev, flags, mode, p)
 	dev_t dev;
 	int flags, mode;
 	struct proc *p;
 {
-	register struct msgbuf *mbp = msgbufp;
-
 	if (log_open)
 		return (EBUSY);
 	log_open = 1;
@@ -77,7 +92,7 @@ logopen(dev, flags, mode, p)
 }
 
 /*ARGSUSED*/
-int
+static	int
 logclose(dev, flag, mode, p)
 	dev_t dev;
 	int flag, mode;
@@ -90,7 +105,7 @@ logclose(dev, flag, mode, p)
 }
 
 /*ARGSUSED*/
-int
+static	int
 logread(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -120,23 +135,23 @@ logread(dev, uio, flag)
 	while (uio->uio_resid > 0) {
 		l = mbp->msg_bufx - mbp->msg_bufr;
 		if (l < 0)
-			l = MSG_BSIZE - mbp->msg_bufr;
+			l = mbp->msg_size - mbp->msg_bufr;
 		l = min(l, uio->uio_resid);
 		if (l == 0)
 			break;
-		error = uiomove((caddr_t)&mbp->msg_bufc[mbp->msg_bufr],
-			(int)l, uio);
+		error = uiomove((caddr_t)msgbufp->msg_ptr + mbp->msg_bufr,
+		    (int)l, uio);
 		if (error)
 			break;
 		mbp->msg_bufr += l;
-		if (mbp->msg_bufr >= MSG_BSIZE)
+		if (mbp->msg_bufr >= mbp->msg_size)
 			mbp->msg_bufr = 0;
 	}
 	return (error);
 }
 
 /*ARGSUSED*/
-int
+static	int
 logselect(dev, rw, p)
 	dev_t dev;
 	int rw;
@@ -179,7 +194,7 @@ logwakeup()
 }
 
 /*ARGSUSED*/
-int
+static	int
 logioctl(dev, com, data, flag, p)
 	dev_t dev;
 	int com;
@@ -198,7 +213,7 @@ logioctl(dev, com, data, flag, p)
 		l = msgbufp->msg_bufx - msgbufp->msg_bufr;
 		splx(s);
 		if (l < 0)
-			l += MSG_BSIZE;
+			l += msgbufp->msg_size;
 		*(int *)data = l;
 		break;
 
@@ -225,3 +240,29 @@ logioctl(dev, com, data, flag, p)
 	}
 	return (0);
 }
+
+static log_devsw_installed = 0;
+#ifdef DEVFS
+static	void	*log_devfs_token;
+#endif
+
+static void
+log_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! log_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR,0);
+		cdevsw_add(&dev,&log_cdevsw,NULL);
+		log_devsw_installed = 1;
+#ifdef DEVFS
+		log_devfs_token = devfs_add_devswf(&log_cdevsw, 0, DV_CHR,
+						   UID_ROOT, GID_WHEEL, 0600,
+						   "klog");
+#endif
+    	}
+}
+
+SYSINIT(logdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,log_drvinit,NULL)
+
+
