@@ -36,15 +36,16 @@
  * SUCH DAMAGE.
  *
  *	@(#)subr_prf.c	8.3 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/subr_prf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/subr_prf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_ddb.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/reboot.h>
+#include <sys/kernel.h>
 #include <sys/msgbuf.h>
 #include <sys/proc.h>
-#include <sys/vnode.h>
 #include <sys/tty.h>
 #include <sys/tprintf.h>
 #include <sys/syslog.h>
@@ -57,64 +58,21 @@
  */
 #include <machine/stdarg.h>
 
-
 #define TOCONS	0x01
 #define TOTTY	0x02
 #define TOLOG	0x04
 
 struct	tty *constty;			/* pointer to console "window" tty */
 
-void	(*v_putc)(int) = cnputc;	/* routine to putc on virtual console */
-
-void  logpri __P((int level));
-static void  putchar __P((int ch, int flags, struct tty *tp));
+static void (*v_putc)(int) = cnputc;	/* routine to putc on virtual console */
+static void  logpri __P((int level));
+static void  msglogchar(int c, void *dummyarg);
+struct putchar_arg {int flags; struct tty *tty; };
+static void  putchar __P((int ch, void *arg));
 static char *ksprintn __P((u_long num, int base, int *len));
 
-int consintr = 1;			/* Ok to handle console interrupts? */
-
-/*
- * Variable panicstr contains argument to first call to panic; used as flag
- * to indicate that the kernel has already called panic.
- */
-const char *panicstr;
-
-/*
- * Panic is called on unresolvable fatal errors.  It prints "panic: mesg",
- * and then reboots.  If we are called twice, then we avoid trying to sync
- * the disks as this often leads to recursive panics.
- */
-#ifdef __GNUC__
-__dead			/* panic() does not return */
-#endif
-void
-#ifdef __STDC__
-panic(const char *fmt, ...)
-#else
-panic(fmt, va_alist)
-	char *fmt;
-#endif
-{
-	int bootopt;
-	va_list ap;
-
-	bootopt = RB_AUTOBOOT | RB_DUMP;
-	if (panicstr)
-		bootopt |= RB_NOSYNC;
-	else
-		panicstr = fmt;
-
-	va_start(ap, fmt);
-	printf("panic: %r\n", fmt, ap);
-	va_end(ap);
-
-#ifdef KGDB
-	kgdb_panic();
-#endif
-#ifdef DDB
-	Debugger ("panic");
-#endif
-	boot(bootopt);
-}
+static int consintr = 1;		/* Ok to handle console interrupts? */
+static int msgbufmapped;		/* Set when safe to use msgbuf */
 
 /*
  * Warn that a system table is full.
@@ -133,19 +91,17 @@ tablefull(tab)
  * the queue does not clear in a reasonable time.
  */
 void
-#ifdef __STDC__
 uprintf(const char *fmt, ...)
-#else
-uprintf(fmt, va_alist)
-	char *fmt;
-#endif
 {
-	register struct proc *p = curproc;
+	struct proc *p = curproc;
 	va_list ap;
+	struct putchar_arg pca;
 
 	if (p->p_flag & P_CONTROLT && p->p_session->s_ttyvp) {
 		va_start(ap, fmt);
-		kprintf(fmt, TOTTY, p->p_session->s_ttyp, ap);
+		pca.tty = p->p_session->s_ttyp;
+		pca.flags = TOTTY;
+		kvprintf(fmt, putchar, &pca, 10, ap);
 		va_end(ap);
 	}
 }
@@ -176,18 +132,13 @@ tprintf_close(sess)
  * with the given session.
  */
 void
-#ifdef __STDC__
 tprintf(tpr_t tpr, const char *fmt, ...)
-#else
-tprintf(tpr, fmt, va_alist)
-	tpr_t tpr;
-	char *fmt;
-#endif
 {
 	register struct session *sess = (struct session *)tpr;
 	struct tty *tp = NULL;
 	int flags = TOLOG;
 	va_list ap;
+	struct putchar_arg pca;
 
 	logpri(LOG_INFO);
 	if (sess && sess->s_ttyvp && ttycheckoutq(sess->s_ttyp, 0)) {
@@ -195,7 +146,9 @@ tprintf(tpr, fmt, va_alist)
 		tp = sess->s_ttyp;
 	}
 	va_start(ap, fmt);
-	kprintf(fmt, flags, tp, ap);
+	pca.tty = tp;
+	pca.flags = flags;
+	kvprintf(fmt, putchar, &pca, 10, ap);
 	va_end(ap);
 	logwakeup();
 }
@@ -206,18 +159,14 @@ tprintf(tpr, fmt, va_alist)
  * be revoke(2)'d away.  Other callers should use tprintf.
  */
 void
-#ifdef __STDC__
 ttyprintf(struct tty *tp, const char *fmt, ...)
-#else
-ttyprintf(tp, fmt, va_alist)
-	struct tty *tp;
-	char *fmt;
-#endif
 {
 	va_list ap;
-
+	struct putchar_arg pca;
 	va_start(ap, fmt);
-	kprintf(fmt, TOTTY, tp, ap);
+	pca.tty = tp;
+	pca.flags = TOTTY;
+	kvprintf(fmt, putchar, &pca, 10, ap);
 	va_end(ap);
 }
 
@@ -229,13 +178,7 @@ extern	int log_open;
  * log yet, it writes to the console also.
  */
 void
-#ifdef __STDC__
 log(int level, const char *fmt, ...)
-#else
-log(level, fmt, va_alist)
-	int level;
-	char *fmt;
-#endif
 {
 	register int s;
 	va_list ap;
@@ -243,72 +186,157 @@ log(level, fmt, va_alist)
 	s = splhigh();
 	logpri(level);
 	va_start(ap, fmt);
-	kprintf(fmt, TOLOG, NULL, ap);
-	splx(s);
+
+	kvprintf(fmt, msglogchar, NULL, 10, ap);
 	va_end(ap);
+
+	splx(s);
 	if (!log_open) {
+		struct putchar_arg pca;
 		va_start(ap, fmt);
-		kprintf(fmt, TOCONS, NULL, ap);
+		pca.tty = NULL;
+		pca.flags = TOCONS;
+		kvprintf(fmt, putchar, &pca, 10, ap);
 		va_end(ap);
 	}
 	logwakeup();
 }
 
-void
+static void
 logpri(level)
 	int level;
 {
 	register char *p;
 
-	putchar('<', TOLOG, NULL);
+	msglogchar('<', NULL);
 	for (p = ksprintn((u_long)level, 10, NULL); *p;)
-		putchar(*p--, TOLOG, NULL);
-	putchar('>', TOLOG, NULL);
+		msglogchar(*p--, NULL);
+	msglogchar('>', NULL);
 }
 
-void
-#ifdef __STDC__
+int
 addlog(const char *fmt, ...)
-#else
-addlog(fmt, va_alist)
-	char *fmt;
-#endif
 {
 	register int s;
 	va_list ap;
+	int retval;
 
 	s = splhigh();
 	va_start(ap, fmt);
-	kprintf(fmt, TOLOG, NULL, ap);
+	retval = kvprintf(fmt, msglogchar, NULL, 10, ap);
 	splx(s);
 	va_end(ap);
 	if (!log_open) {
+		struct putchar_arg pca;
 		va_start(ap, fmt);
-		kprintf(fmt, TOCONS, NULL, ap);
+		pca.tty = NULL;
+		pca.flags = TOCONS;
+		kvprintf(fmt, putchar, &pca, 10, ap);
 		va_end(ap);
 	}
 	logwakeup();
+	return (retval);
 }
 
-void
-#ifdef __STDC__
+int
 printf(const char *fmt, ...)
-#else
-printf(fmt, va_alist)
-	char *fmt;
-#endif
 {
 	va_list ap;
 	register int savintr;
+	struct putchar_arg pca;
+	int retval;
 
 	savintr = consintr;		/* disable interrupts */
 	consintr = 0;
 	va_start(ap, fmt);
-	kprintf(fmt, TOCONS | TOLOG, NULL, ap);
+	pca.tty = NULL;
+	pca.flags = TOCONS | TOLOG;
+	retval = kvprintf(fmt, putchar, &pca, 10, ap);
 	va_end(ap);
 	if (!panicstr)
 		logwakeup();
 	consintr = savintr;		/* reenable interrupts */
+	return retval;
+}
+
+void
+vprintf(const char *fmt, va_list ap)
+{
+	register int savintr;
+	struct putchar_arg pca;
+
+	savintr = consintr;		/* disable interrupts */
+	consintr = 0;
+	pca.tty = NULL;
+	pca.flags = TOCONS | TOLOG;
+	kvprintf(fmt, putchar, &pca, 10, ap);
+	if (!panicstr)
+		logwakeup();
+	consintr = savintr;		/* reenable interrupts */
+}
+
+/*
+ * Print a character on console or users terminal.  If destination is
+ * the console then the last bunch of characters are saved in msgbuf for
+ * inspection later.
+ */
+static void
+putchar(int c, void *arg)
+{
+	struct putchar_arg *ap = (struct putchar_arg*) arg;
+	int flags = ap->flags;
+	struct tty *tp = ap->tty;
+	if (panicstr)
+		constty = NULL;
+	if ((flags & TOCONS) && tp == NULL && constty) {
+		tp = constty;
+		flags |= TOTTY;
+	}
+	if ((flags & TOTTY) && tp && tputchar(c, tp) < 0 &&
+	    (flags & TOCONS) && tp == constty)
+		constty = NULL;
+	if ((flags & TOLOG))
+		msglogchar(c, NULL);
+	if ((flags & TOCONS) && constty == NULL && c != '\0')
+		(*v_putc)(c);
+}
+
+/*
+ * Scaled down version of sprintf(3).
+ */
+int
+sprintf(char *buf, const char *cfmt, ...)
+{
+	int retval;
+	va_list ap;
+
+	va_start(ap, cfmt);
+	retval = kvprintf(cfmt, NULL, (void *)buf, 10, ap);
+	buf[retval] = '\0';
+	va_end(ap);
+	return retval;
+}
+
+/*
+ * Put a number (base <= 16) in a buffer in reverse order; return an
+ * optional length and a pointer to the NULL terminated (preceded?)
+ * buffer.
+ */
+static char *
+ksprintn(ul, base, lenp)
+	register u_long ul;
+	register int base, *lenp;
+{					/* A long in base 8, plus NULL. */
+	static char buf[sizeof(long) * NBBY / 3 + 2];
+	register char *p;
+
+	p = buf;
+	do {
+		*++p = hex2ascii(ul % base);
+	} while (ul /= base);
+	if (lenp)
+		*lenp = p - buf;
+	return (p);
 }
 
 /*
@@ -327,71 +355,100 @@ printf(fmt, va_alist)
  * the next characters (up to a control character, i.e. a character <= 32),
  * give the name of the register.  Thus:
  *
- *	kprintf("reg=%b\n", 3, "\10\2BITTWO\1BITONE\n");
+ *	kvprintf("reg=%b\n", 3, "\10\2BITTWO\1BITONE\n");
  *
  * would produce output:
  *
  *	reg=3<BITTWO,BITONE>
  *
- * The format %r passes an additional format string and argument list
- * recursively.  Its usage is:
- *
- * fn(char *fmt, ...)
- * {
- *	va_list ap;
- *	va_start(ap, fmt);
- *	printf("prefix: %r: suffix\n", fmt, ap);
- *	va_end(ap);
- * }
- *
- * Space or zero padding and a field width are supported for the numeric
- * formats only.
+ * XXX:  %D  -- Hexdump, takes pointer and separator string:
+ *		("%6D", ptr, ":")   -> XX:XX:XX:XX:XX:XX
+ *		("%*D", len, ptr, " " -> XX XX XX XX ...
  */
-void
-kprintf(fmt, flags, tp, ap)
-	register const char *fmt;
-	int flags;
-	struct tty *tp;
-	va_list ap;
+int
+kvprintf(char const *fmt, void (*func)(int, void*), void *arg, int radix, va_list ap)
 {
-	register char *p, *q;
-	register int ch, n;
+#define PCHAR(c) {int cc=(c); if (func) (*func)(cc,arg); else *d++ = cc; retval++; }
+	char *p, *q, *d;
+	u_char *up;
+	int ch, n;
 	u_long ul;
-	int base, lflag, tmp, width;
+	int base, lflag, tmp, width, ladjust, sharpflag, neg, sign, dot;
+	int dwidth;
 	char padc;
+	int retval = 0;
+
+	if (!func)
+		d = (char *) arg;
+	else
+		d = NULL;
 
 	if (fmt == NULL)
 		fmt = "(fmt null)\n";
+
+	if (radix < 2 || radix > 36)
+		radix = 10;
+
 	for (;;) {
 		padc = ' ';
 		width = 0;
-		while ((ch = *(u_char *)fmt++) != '%') {
-			if (ch == '\0')
-				return;
-			putchar(ch, flags, tp);
+		while ((ch = (u_char)*fmt++) != '%') {
+			if (ch == '\0') 
+				return retval;
+			PCHAR(ch);
 		}
-		lflag = 0;
-reswitch:	switch (ch = *(u_char *)fmt++) {
-		case '0':
-			padc = '0';
+		lflag = 0; ladjust = 0; sharpflag = 0; neg = 0;
+		sign = 0; dot = 0; dwidth = 0;
+reswitch:	switch (ch = (u_char)*fmt++) {
+		case '.':
+			dot = 1;
 			goto reswitch;
-		case '1': case '2': case '3': case '4':
-		case '5': case '6': case '7': case '8': case '9':
-			for (width = 0;; ++fmt) {
-				width = width * 10 + ch - '0';
-				ch = *fmt;
-				if (ch < '0' || ch > '9')
-					break;
+		case '#':
+			sharpflag = 1;
+			goto reswitch;
+		case '+':
+			sign = 1;
+			goto reswitch;
+		case '-':
+			ladjust = 1;
+			goto reswitch;
+		case '%':
+			PCHAR(ch);
+			break;
+		case '*':
+			if (!dot) {
+				width = va_arg(ap, int);
+				if (width < 0) {
+					ladjust = !ladjust;
+					width = -width;
+				}
+			} else {
+				dwidth = va_arg(ap, int);
 			}
 			goto reswitch;
-		case 'l':
-			lflag = 1;
+		case '0':
+			if (!dot) {
+				padc = '0';
+				goto reswitch;
+			}
+		case '1': case '2': case '3': case '4':
+		case '5': case '6': case '7': case '8': case '9':
+				for (n = 0;; ++fmt) {
+					n = n * 10 + ch - '0';
+					ch = *fmt;
+					if (ch < '0' || ch > '9')
+						break;
+				}
+			if (dot)
+				dwidth = n;
+			else
+				width = n;
 			goto reswitch;
 		case 'b':
 			ul = va_arg(ap, int);
 			p = va_arg(ap, char *);
 			for (q = ksprintn(ul, *p++, NULL); *q;)
-				putchar(*q--, flags, tp);
+				PCHAR(*q--);
 
 			if (!ul)
 				break;
@@ -399,49 +456,77 @@ reswitch:	switch (ch = *(u_char *)fmt++) {
 			for (tmp = 0; *p;) {
 				n = *p++;
 				if (ul & (1 << (n - 1))) {
-					putchar(tmp ? ',' : '<', flags, tp);
+					PCHAR(tmp ? ',' : '<');
 					for (; (n = *p) > ' '; ++p)
-						putchar(n, flags, tp);
+						PCHAR(n);
 					tmp = 1;
 				} else
 					for (; *p > ' '; ++p)
 						continue;
 			}
 			if (tmp)
-				putchar('>', flags, tp);
+				PCHAR('>');
 			break;
 		case 'c':
-			putchar(va_arg(ap, int), flags, tp);
+			PCHAR(va_arg(ap, int));
 			break;
-		case 'r':
+		case 'D':
+			up = va_arg(ap, u_char *);
 			p = va_arg(ap, char *);
-			kprintf(p, flags, tp, va_arg(ap, va_list));
+			if (!width)
+				width = 16;
+			while(width--) {
+				PCHAR(hex2ascii(*up >> 4));
+				PCHAR(hex2ascii(*up & 0x0f));
+				up++;
+				if (width)
+					for (q=p;*q;q++)
+						PCHAR(*q);
+			}
 			break;
+		case 'd':
+			ul = lflag ? va_arg(ap, long) : va_arg(ap, int);
+			sign = 1;
+			base = 10;
+			goto number;
+		case 'l':
+			lflag = 1;
+			goto reswitch;
+		case 'n':
+			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
+			base = radix;
+			goto number;
+		case 'o':
+			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
+			base = 8;
+			goto number;
+		case 'p':
+			ul = (u_long)va_arg(ap, void *);
+			base = 16;
+			PCHAR('0');
+			PCHAR('x');
+			goto number;
 		case 's':
 			p = va_arg(ap, char *);
 			if (p == NULL)
 				p = "(null)";
-			while (*p)
-				putchar(*p++, flags, tp);
+			if (!dot)
+				n = strlen (p);
+			else
+				for (n = 0; n < dwidth && p[n]; n++)
+					continue;
+
+			width -= n;
+
+			if (!ladjust && width > 0)
+				while (width--)
+					PCHAR(padc);
+			while (n--)
+				PCHAR(*p++);
+			if (ladjust && width > 0)
+				while (width--)
+					PCHAR(padc);
 			break;
-		case 'd':
-			ul = lflag ? va_arg(ap, long) : va_arg(ap, int);
-			if ((long)ul < 0) {
-				putchar('-', flags, tp);
-				ul = -(long)ul;
-			}
-			base = 10;
-			goto number;
-		case 'o':
-			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
-			base = 8;
-			goto number;
-		case 'p':
-			ul = (u_long)va_arg(ap, void *);
-			base = 16;
-			putchar('0', flags, tp);
-			putchar('x', flags, tp);
-			goto number;
 		case 'u':
 			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
 			base = 10;
@@ -449,167 +534,111 @@ reswitch:	switch (ch = *(u_char *)fmt++) {
 		case 'x':
 			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
 			base = 16;
-number:			p = ksprintn(ul, base, &tmp);
-			if (width && (width -= tmp) > 0)
+number:			if (sign && (long)ul < 0L) {
+				neg = 1;
+				ul = -(long)ul;
+			}
+			p = ksprintn(ul, base, &tmp);
+			if (sharpflag && ul != 0) {
+				if (base == 8)
+					tmp++;
+				else if (base == 16)
+					tmp += 2;
+			}
+			if (neg)
+				tmp++;
+
+			if (!ladjust && width && (width -= tmp) > 0)
 				while (width--)
-					putchar(padc, flags, tp);
+					PCHAR(padc);
+			if (neg)
+				PCHAR('-');
+			if (sharpflag && ul != 0) {
+				if (base == 8) {
+					PCHAR('0');
+				} else if (base == 16) {
+					PCHAR('0');
+					PCHAR('x');
+				}
+			}
+
 			while (*p)
-				putchar(*p--, flags, tp);
+				PCHAR(*p--);
+
+			if (ladjust && width && (width -= tmp) > 0)
+				while (width--)
+					PCHAR(padc);
+
 			break;
 		default:
-			putchar('%', flags, tp);
+			PCHAR('%');
 			if (lflag)
-				putchar('l', flags, tp);
-			/* FALLTHROUGH */
-		case '%':
-			putchar(ch, flags, tp);
+				PCHAR('l');
+			PCHAR(ch);
+			break;
 		}
 	}
+#undef PCHAR
 }
 
 /*
- * Print a character on console or users terminal.  If destination is
- * the console then the last MSGBUFS characters are saved in msgbuf for
- * inspection later.
+ * Put character in log buffer.
  */
 static void
-putchar(c, flags, tp)
-	register int c;
-	int flags;
-	struct tty *tp;
+msglogchar(int c, void *dummyarg)
 {
-	register struct msgbuf *mbp;
+	struct msgbuf *mbp;
 
-	if (panicstr)
-		constty = NULL;
-	if ((flags & TOCONS) && tp == NULL && constty) {
-		tp = constty;
-		flags |= TOTTY;
-	}
-	if ((flags & TOTTY) && tp && tputchar(c, tp) < 0 &&
-	    (flags & TOCONS) && tp == constty)
-		constty = NULL;
-	if ((flags & TOLOG) &&
-	    c != '\0' && c != '\r' && c != 0177 && msgbufmapped) {
+	if (c != '\0' && c != '\r' && c != 0177 && msgbufmapped) {
 		mbp = msgbufp;
-		if (mbp->msg_magic != MSG_MAGIC ||
-		    mbp->msg_bufx >= MSG_BSIZE ||
-		    mbp->msg_bufr >= MSG_BSIZE) {
-			bzero(mbp, sizeof(struct msgbuf));
-			mbp->msg_magic = MSG_MAGIC;
-		}
-		mbp->msg_bufc[mbp->msg_bufx++] = c;
-		if (mbp->msg_bufx >= MSG_BSIZE)
+		mbp->msg_ptr[mbp->msg_bufx++] = c;
+		if (mbp->msg_bufx >= mbp->msg_size)
 			mbp->msg_bufx = 0;
 		/* If the buffer is full, keep the most recent data. */
 		if (mbp->msg_bufr == mbp->msg_bufx) {
-			if (++mbp->msg_bufr >= MSG_BSIZE)
+			if (++mbp->msg_bufr >= mbp->msg_size)
 				mbp->msg_bufr = 0;
 		}
 	}
-	if ((flags & TOCONS) && constty == NULL && c != '\0')
-		(*v_putc)(c);
 }
 
-/*
- * Scaled down version of sprintf(3).
- */
-#ifdef __STDC__
-int
-sprintf(char *buf, const char *cfmt, ...)
-#else
-int
-sprintf(buf, cfmt, va_alist)
-	char *buf, *cfmt;
-#endif
+void
+msgbufinit(void *ptr, size_t size)
 {
-	register const char *fmt = cfmt;
-	register char *p, *bp;
-	register int ch, base;
-	u_long ul;
-	int lflag;
-	va_list ap;
+	char *cp;
 
-	va_start(ap, cfmt);
-	for (bp = buf; ; ) {
-		while ((ch = *(u_char *)fmt++) != '%')
-			if ((*bp++ = ch) == '\0')
-				return ((bp - buf) - 1);
-
-		lflag = 0;
-reswitch:	switch (ch = *(u_char *)fmt++) {
-		case 'l':
-			lflag = 1;
-			goto reswitch;
-		case 'c':
-			*bp++ = va_arg(ap, int);
-			break;
-		case 's':
-			p = va_arg(ap, char *);
-			while (*p)
-				*bp++ = *p++;
-			break;
-		case 'd':
-			ul = lflag ? va_arg(ap, long) : va_arg(ap, int);
-			if ((long)ul < 0) {
-				*bp++ = '-';
-				ul = -(long)ul;
-			}
-			base = 10;
-			goto number;
-			break;
-		case 'o':
-			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
-			base = 8;
-			goto number;
-			break;
-		case 'p':
-			ul = (u_long)va_arg(ap, void *);
-			base = 16;
-			*bp++ = '0';
-			*bp++ = 'x';
-			goto number;
-		case 'u':
-			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
-			base = 10;
-			goto number;
-			break;
-		case 'x':
-			ul = lflag ? va_arg(ap, u_long) : va_arg(ap, u_int);
-			base = 16;
-number:			for (p = ksprintn(ul, base, NULL); *p;)
-				*bp++ = *p--;
-			break;
-		default:
-			*bp++ = '%';
-			if (lflag)
-				*bp++ = 'l';
-			/* FALLTHROUGH */
-		case '%':
-			*bp++ = ch;
-		}
+	cp = (char *)ptr;
+	msgbufp = (struct msgbuf *) (cp + size - sizeof(*msgbufp));
+	if (msgbufp->msg_magic != MSG_MAGIC || msgbufp->msg_ptr != cp) {
+		bzero(cp, size);
+		msgbufp->msg_magic = MSG_MAGIC;
+		msgbufp->msg_size = (char *)msgbufp - cp;
+		msgbufp->msg_ptr = cp;
 	}
-	va_end(ap);
+	msgbufmapped = 1;
 }
 
-/*
- * Put a number (base <= 16) in a buffer in reverse order; return an
- * optional length and a pointer to the NULL terminated (preceded?)
- * buffer.
- */
-static char *
-ksprintn(ul, base, lenp)
-	register u_long ul;
-	register int base, *lenp;
-{					/* A long in base 8, plus NULL. */
-	static char buf[sizeof(long) * NBBY / 3 + 2];
-	register char *p;
+#include "opt_ddb.h"
+#ifdef DDB
+#include <ddb/ddb.h>
 
-	p = buf;
-	do {
-		*++p = "0123456789abcdef"[ul % base];
-	} while (ul /= base);
-	if (lenp)
-		*lenp = p - buf;
-	return (p);
+DB_SHOW_COMMAND(msgbuf, db_show_msgbuf)
+{
+	int i, j;
+
+	if (!msgbufmapped) {
+		db_printf("msgbuf not mapped yet\n");
+		return;
+	}
+	db_printf("msgbufp = %p\n", msgbufp);
+	db_printf("magic = %x, size = %d, r= %d, w = %d, ptr = %p\n",
+	    msgbufp->msg_magic, msgbufp->msg_size, msgbufp->msg_bufr,
+	    msgbufp->msg_bufx, msgbufp->msg_ptr);
+	for (i = 0; i < msgbufp->msg_size; i++) {
+		j = (i + msgbufp->msg_bufr) % msgbufp->msg_size;
+		db_printf("%c", msgbufp->msg_ptr[j]);
+	}
+	db_printf("\n");
 }
+
+#endif /* DDB */
