@@ -36,11 +36,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)sys_generic.c	8.5 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/sys_generic.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/sys_generic.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_ktrace.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/filedesc.h>
 #include <sys/ioctl.h>
 #include <sys/file.h>
@@ -57,18 +60,19 @@
 #include <sys/ktrace.h>
 #endif
 #include <vm/vm.h>
-#include <i386/ibcs2/ibcs2.h>
 
-int	selscan __P((struct proc *, fd_set *, fd_set *, int, int *));
+static int	selscan __P((struct proc *, fd_mask **, fd_mask **, int, int *));
 
 /*
  * Read system call.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct read_args {
 	int	fd;
 	char	*buf;
 	u_int	nbyte;
 };
+#endif
 /* ARGSUSED */
 int
 read(p, uap, retval)
@@ -125,11 +129,13 @@ read(p, uap, retval)
 /*
  * Scatter read system call.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct readv_args {
-	int	fdes;
+	int	fd;
 	struct	iovec *iovp;
 	u_int	iovcnt;
 };
+#endif
 int
 readv(p, uap, retval)
 	struct proc *p;
@@ -148,8 +154,8 @@ readv(p, uap, retval)
 	struct iovec *ktriov = NULL;
 #endif
 
-	if (((u_int)uap->fdes) >= fdp->fd_nfiles ||
-	    (fp = fdp->fd_ofiles[uap->fdes]) == NULL ||
+	if (((u_int)uap->fd) >= fdp->fd_nfiles ||
+	    (fp = fdp->fd_ofiles[uap->fd]) == NULL ||
 	    (fp->f_flag & FREAD) == 0)
 		return (EBADF);
 	/* note: can't use iovlen until iovcnt is validated */
@@ -197,7 +203,7 @@ readv(p, uap, retval)
 #ifdef KTRACE
 	if (ktriov != NULL) {
 		if (error == 0)
-			ktrgenio(p->p_tracep, uap->fdes, UIO_READ, ktriov,
+			ktrgenio(p->p_tracep, uap->fd, UIO_READ, ktriov,
 			    cnt, error);
 		FREE(ktriov, M_TEMP);
 	}
@@ -212,11 +218,13 @@ done:
 /*
  * Write system call
  */
+#ifndef _SYS_SYSPROTO_H_
 struct write_args {
 	int	fd;
 	char	*buf;
 	u_int	nbyte;
 };
+#endif
 int
 write(p, uap, retval)
 	struct proc *p;
@@ -272,11 +280,13 @@ write(p, uap, retval)
 /*
  * Gather write system call
  */
+#ifndef _SYS_SYSPROTO_H_
 struct writev_args {
 	int	fd;
 	struct	iovec *iovp;
 	u_int	iovcnt;
 };
+#endif
 int
 writev(p, uap, retval)
 	struct proc *p;
@@ -362,11 +372,13 @@ done:
 /*
  * Ioctl system call
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ioctl_args {
 	int	fd;
 	int	com;
 	caddr_t	data;
 };
+#endif
 /* ARGSUSED */
 int
 ioctl(p, uap, retval)
@@ -408,23 +420,11 @@ ioctl(p, uap, retval)
 	if (size > IOCPARM_MAX)
 		return (ENOTTY);
 	memp = NULL;
-#ifdef COMPAT_IBCS2
-	if (size + IBCS2_RETVAL_SIZE > sizeof (stkbuf)) {
-		memp = (caddr_t)malloc((u_long)size + IBCS2_RETVAL_SIZE,
-				       M_IOCTLOPS, M_WAITOK);
-		data = memp + IBCS2_RETVAL_SIZE;
-	} else
-		data = stkbuf + IBCS2_RETVAL_SIZE;
-	*(int *)(data - IBCS2_RETVAL_SIZE) = IBCS2_MAGIC_IN;
-	*(int *)(data - (IBCS2_RETVAL_SIZE - sizeof(int))) = 0;
-	*(int *)(data - (IBCS2_RETVAL_SIZE - 2*sizeof(int))) = 0;
-#else
 	if (size > sizeof (stkbuf)) {
 		memp = (caddr_t)malloc((u_long)size, M_IOCTLOPS, M_WAITOK);
 		data = memp;
 	} else
 		data = stkbuf;
-#endif
 	if (com&IOC_IN) {
 		if (size) {
 			error = copyin(uap->data, data, (u_int)size);
@@ -443,15 +443,6 @@ ioctl(p, uap, retval)
 		bzero(data, size);
 	else if (com&IOC_VOID)
 		*(caddr_t *)data = uap->data;
-#ifdef COMPAT_IBCS2
-	else if (com)
-		/*
-		 * Pick up such things as NIOCxx.
-		 * Any copyouts will have to be done prior
-		 * to return by their servicing code.
-		 */
-		*(caddr_t *)data = uap->data;
-#endif
 
 	switch (com) {
 
@@ -512,49 +503,73 @@ ioctl(p, uap, retval)
 			error = copyout(data, uap->data, (u_int)size);
 		break;
 	}
-#ifdef COMPAT_IBCS2
-	if ((*(int *)(data - IBCS2_RETVAL_SIZE)) == IBCS2_MAGIC_OUT) {
-		retval[0] = *(int *)(data-(IBCS2_RETVAL_SIZE - sizeof(int)));
-		retval[1] = *(int *)(data-(IBCS2_RETVAL_SIZE - 2*sizeof(int)));
-	}
-#endif
 	if (memp)
 		free(memp, M_IOCTLOPS);
 	return (error);
 }
 
-int	selwait, nselcoll;
+static int	nselcoll;
+int	selwait;
 
 /*
  * Select system call.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct select_args {
-	u_int	nd;
+	int	nd;
 	fd_set	*in, *ou, *ex;
 	struct	timeval *tv;
 };
+#endif
 int
 select(p, uap, retval)
 	register struct proc *p;
 	register struct select_args *uap;
 	int *retval;
 {
-	fd_set ibits[3], obits[3];
+	fd_mask *ibits[3], *obits[3];
 	struct timeval atv;
-	int s, ncoll, error = 0, timo;
+	int s, ncoll, error = 0, timo, i;
 	u_int ni;
 
-	bzero((caddr_t)ibits, sizeof(ibits));
-	bzero((caddr_t)obits, sizeof(obits));
-	if (uap->nd > FD_SETSIZE)
+	if (uap->nd < 0)
 		return (EINVAL);
+
 	if (uap->nd > p->p_fd->fd_nfiles)
-		uap->nd = p->p_fd->fd_nfiles;	/* forgiving; slightly wrong */
+		uap->nd = p->p_fd->fd_nfiles;   /* forgiving; slightly wrong */
+
+	/* The amount of space we need to allocate */
+	ni = howmany(roundup2 (uap->nd, FD_SETSIZE), NFDBITS) *
+		sizeof(fd_mask);
+
+	if (ni > p->p_selbits_size) {
+		if (p->p_selbits_size)
+			free (p->p_selbits, M_SELECT);
+
+		while (p->p_selbits_size < ni)
+			p->p_selbits_size += 32; /* Increase by 256 bits */
+
+		p->p_selbits = malloc(p->p_selbits_size * 6, M_SELECT,
+			M_WAITOK);
+	}
+	for (i = 0; i < 3; i++) {
+		ibits[i] = (fd_mask *)(p->p_selbits + i * p->p_selbits_size);
+		obits[i] = (fd_mask *)(p->p_selbits + (i + 3) *
+			p->p_selbits_size);
+	}
+
+	/*
+	 * This buffer is usually very small therefore it's probably faster
+	 * to just zero it, rather than calculate what needs to be zeroed.
+	 */
+	bzero (p->p_selbits, p->p_selbits_size * 6);
+
+	/* The amount of space we need to copyin/copyout */
 	ni = howmany(uap->nd, NFDBITS) * sizeof(fd_mask);
 
 #define	getbits(name, x) \
 	if (uap->name && \
-	    (error = copyin((caddr_t)uap->name, (caddr_t)&ibits[x], ni))) \
+	    (error = copyin((caddr_t)uap->name, (caddr_t)ibits[x], ni))) \
 		goto done;
 	getbits(in, 0);
 	getbits(ou, 1);
@@ -612,7 +627,7 @@ done:
 		error = 0;
 #define	putbits(name, x) \
 	if (uap->name && \
-	    (error2 = copyout((caddr_t)&obits[x], (caddr_t)uap->name, ni))) \
+	    (error2 = copyout((caddr_t)obits[x], (caddr_t)uap->name, ni))) \
 		error = error2;
 	if (error == 0) {
 		int error2;
@@ -625,10 +640,10 @@ done:
 	return (error);
 }
 
-int
+static int
 selscan(p, ibits, obits, nfd, retval)
 	struct proc *p;
-	fd_set *ibits, *obits;
+	fd_mask **ibits, **obits;
 	int nfd, *retval;
 {
 	register struct filedesc *fdp = p->p_fd;
@@ -640,14 +655,15 @@ selscan(p, ibits, obits, nfd, retval)
 
 	for (msk = 0; msk < 3; msk++) {
 		for (i = 0; i < nfd; i += NFDBITS) {
-			bits = ibits[msk].fds_bits[i/NFDBITS];
+			bits = ibits[msk][i/NFDBITS];
 			while ((j = ffs(bits)) && (fd = i + --j) < nfd) {
 				bits &= ~(1 << j);
 				fp = fdp->fd_ofiles[fd];
 				if (fp == NULL)
 					return (EBADF);
 				if ((*fp->f_ops->fo_select)(fp, flag[msk], p)) {
-					FD_SET(fd, &obits[msk]);
+					obits[msk][(fd)/NFDBITS] |= 
+						(1 << ((fd) % NFDBITS));
 					n++;
 				}
 			}
