@@ -1,4 +1,4 @@
-/* $FreeBSD: stable/2.1/sys/kern/sysv_sem.c 59837 2000-05-01 11:34:49Z peter $ */
+/* $FreeBSD: stable/2.2/sys/kern/sysv_sem.c 59836 2000-05-01 11:29:47Z peter $ */
 
 /*
  * Implementation of SVID semaphores
@@ -8,19 +8,48 @@
  * This software is provided ``AS IS'' without any warranties of any kind.
  */
 
+#include "opt_sysvipc.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/kernel.h>
 #include <sys/proc.h>
 #include <sys/sem.h>
-#include <sys/malloc.h>
+#include <sys/sysent.h>
 
-static int	semctl(), semget(), semop();
-int	(*semcalls[])() = { semctl, semget, semop };
-int	semtot = 0;
+static void seminit __P((void *));
+SYSINIT(sysv_sem, SI_SUB_SYSV_SEM, SI_ORDER_FIRST, seminit, NULL)
+
+#ifndef _SYS_SYSPROTO_H_
+struct __semctl_args;
+int __semctl __P((struct proc *p, struct __semctl_args *uap, int *retval));
+struct semget_args;
+int semget __P((struct proc *p, struct semget_args *uap, int *retval));
+struct semop_args;
+int semop __P((struct proc *p, struct semop_args *uap, int *retval));
+#endif
+
+static struct sem_undo *semu_alloc __P((struct proc *p));
+static int semundo_adjust __P((struct proc *p, struct sem_undo **supptr, 
+		int semid, int semnum, int adjval));
+static void semundo_clear __P((int semid, int semnum));
+
+/* XXX casting to (sy_call_t *) is bogus, as usual. */
+static sy_call_t *semcalls[] = {
+	(sy_call_t *)__semctl, (sy_call_t *)semget,
+	(sy_call_t *)semop
+};
+
+static int	semtot = 0;
+struct semid_ds *sema;		/* semaphore id pool */
+struct sem *sem;		/* semaphore pool */
+static struct sem_undo *semu_list; 	/* list of active undo structures */
+int	*semu;			/* undo structure pool */
 
 void
-seminit()
+seminit(dummy)
+	void *dummy;
 {
 	register int i;
 
@@ -43,21 +72,23 @@ seminit()
 /*
  * Entry point for all SEM calls
  */
-
-struct semsys_args {
-	u_int	which;
-};
-
 int
 semsys(p, uap, retval)
 	struct proc *p;
-	struct semsys_args *uap;
+	/* XXX actually varargs. */
+	struct semsys_args /* {
+		u_int	which;
+		int	a2;
+		int	a3;
+		int	a4;
+		int	a5;
+	} */ *uap;
 	int *retval;
 {
 
 	if (uap->which >= sizeof(semcalls)/sizeof(semcalls[0]))
 		return (EINVAL);
-	return ((*semcalls[uap->which])(p, &uap[1], retval));
+	return ((*semcalls[uap->which])(p, &uap->a2, retval));
 }
 
 /*
@@ -65,7 +96,7 @@ semsys(p, uap, retval)
  * (returns ptr to structure or NULL if no more room)
  */
 
-struct sem_undo *
+static struct sem_undo *
 semu_alloc(p)
 	struct proc *p;
 {
@@ -135,7 +166,7 @@ semu_alloc(p)
  * Adjust a particular entry for a particular proc
  */
 
-int
+static int
 semundo_adjust(p, supptr, semid, semnum, adjval)
 	register struct proc *p;
 	struct sem_undo **supptr;
@@ -202,7 +233,7 @@ semundo_adjust(p, supptr, semid, semnum, adjval)
 	return(0);
 }
 
-void
+static void
 semundo_clear(semid, semnum)
 	int semid, semnum;
 {
@@ -230,17 +261,22 @@ semundo_clear(semid, semnum)
 	}
 }
 
-struct semctl_args {
+/*
+ * Note that the user-mode half of this passes a union, not a pointer
+ */
+#ifndef _SYS_SYSPROTO_H_
+struct __semctl_args {
 	int	semid;
 	int	semnum;
 	int	cmd;
 	union	semun *arg;
 };
+#endif
 
 int
-semctl(p, uap, retval)
+__semctl(p, uap, retval)
 	struct proc *p;
-	register struct semctl_args *uap;
+	register struct __semctl_args *uap;
 	int *retval;
 {
 	int semid = uap->semid;
@@ -394,11 +430,13 @@ semctl(p, uap, retval)
 	return(eval);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct semget_args {
 	key_t	key;
 	int	nsems;
 	int	semflg;
 };
+#endif
 
 int
 semget(p, uap, retval)
@@ -507,11 +545,13 @@ found:
 	return(0);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct semop_args {
 	int	semid;
 	struct	sembuf *sops;
 	int	nsops;
 };
+#endif
 
 int
 semop(p, uap, retval)
