@@ -1,4 +1,4 @@
-/* $FreeBSD: stable/2.1/sys/kern/sysv_shm.c 50950 1999-09-05 08:39:22Z peter $ */
+/* $FreeBSD: stable/2.2/sys/kern/sysv_shm.c 50950 1999-09-05 08:39:22Z peter $ */
 /*	$NetBSD: sysv_shm.c,v 1.23 1994/07/04 23:25:12 glass Exp $	*/
 
 /*
@@ -31,50 +31,69 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <sys/types.h>
+#include "opt_sysvipc.h"
+#include "opt_rlimit.h"
+
 #include <sys/param.h>
+#include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/kernel.h>
 #include <sys/shm.h>
 #include <sys/proc.h>
-#include <sys/uio.h>
-#include <sys/time.h>
 #include <sys/malloc.h>
 #include <sys/mman.h>
-#include <sys/systm.h>
 #include <sys/stat.h>
+#include <sys/sysent.h>
 
 #include <vm/vm.h>
-#include <vm/vm_map.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_object.h>
 #include <vm/vm_map.h>
 #include <vm/vm_kern.h>
+#include <vm/vm_extern.h>
+#include <vm/vm_pager.h>
+#include <vm/vm_inherit.h>
 
-/*
- * Provides the following externally accessible functions:
- *
- * shminit(void);		           initialization
- * shmexit(struct proc *)                  cleanup
- * shmfork(struct proc *, struct proc *, int) fork handling
- * shmsys(arg1, arg2, arg3, arg4);         shm{at,ctl,dt,get}(arg2, arg3, arg4)
- *
- * Structures:
- * shmsegs (an array of 'struct shmid_ds')
- * per proc array of 'struct shmmap_state'
- */
+#ifndef _SYS_SYSPROTO_H_
+struct shmat_args;
+extern int shmat __P((struct proc *p, struct shmat_args *uap, int *retval));
+struct shmctl_args;
+extern int shmctl __P((struct proc *p, struct shmctl_args *uap, int *retval));
+struct shmdt_args;
+extern int shmdt __P((struct proc *p, struct shmdt_args *uap, int *retval));
+struct shmget_args;
+extern int shmget __P((struct proc *p, struct shmget_args *uap, int *retval));
+#endif
 
-int	oshmctl();
-int	shmat(), shmctl(), shmdt(), shmget();
-int	(*shmcalls[])() = { shmat, oshmctl, shmdt, shmget, shmctl };
+static void shminit __P((void *));
+SYSINIT(sysv_shm, SI_SUB_SYSV_SHM, SI_ORDER_FIRST, shminit, NULL)
+
+struct oshmctl_args;
+static int oshmctl __P((struct proc *p, struct oshmctl_args *uap, int *retval));
+static int shmget_allocate_segment __P((struct proc *p, struct shmget_args *uap, int mode, int *retval));
+static int shmget_existing __P((struct proc *p, struct shmget_args *uap, int mode, int segnum, int *retval));
+
+/* XXX casting to (sy_call_t *) is bogus, as usual. */
+sy_call_t *shmcalls[] = {
+	(sy_call_t *)shmat, (sy_call_t *)oshmctl,
+	(sy_call_t *)shmdt, (sy_call_t *)shmget,
+	(sy_call_t *)shmctl
+};
 
 #define	SHMSEG_FREE     	0x0200
 #define	SHMSEG_REMOVED  	0x0400
 #define	SHMSEG_ALLOCATED	0x0800
 #define	SHMSEG_WANTED		0x1000
 
-vm_map_t sysvshm_map;
-int shm_last_free, shm_nused, shm_committed;
+static int shm_last_free, shm_nused, shm_committed;
+struct shmid_ds	*shmsegs;
 
 struct shm_handle {
-	vm_offset_t kva;
+	/* vm_offset_t kva; */
+	vm_object_t shm_object;
 };
 
 struct shmmap_state {
@@ -126,10 +145,10 @@ shm_deallocate_segment(shmseg)
 	size_t size;
 
 	shm_handle = shmseg->shm_internal;
-	size = (shmseg->shm_segsz + CLOFSET) & ~CLOFSET;
-	(void) vm_map_remove(sysvshm_map, shm_handle->kva, shm_handle->kva + size);
+	vm_object_deallocate(shm_handle->shm_object);
 	free((caddr_t)shm_handle, M_SHM);
 	shmseg->shm_internal = NULL;
+	size = round_page(shmseg->shm_segsz);
 	shm_committed -= btoc(size);
 	shm_nused--;
 	shmseg->shm_perm.mode = SHMSEG_FREE;
@@ -146,7 +165,7 @@ shm_delete_mapping(p, shmmap_s)
 
 	segnum = IPCID_TO_IX(shmmap_s->shmid);
 	shmseg = &shmsegs[segnum];
-	size = (shmseg->shm_segsz + CLOFSET) & ~CLOFSET;
+	size = round_page(shmseg->shm_segsz);
 	result = vm_map_remove(&p->p_vmspace->vm_map, shmmap_s->va, shmmap_s->va + size);
 	if (result != KERN_SUCCESS)
 		return EINVAL;
@@ -160,9 +179,12 @@ shm_delete_mapping(p, shmmap_s)
 	return 0;
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct shmdt_args {
 	void *shmaddr;
 };
+#endif
+
 int
 shmdt(p, uap, retval)
 	struct proc *p;
@@ -184,11 +206,14 @@ shmdt(p, uap, retval)
 	return shm_delete_mapping(p, shmmap_s);
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct shmat_args {
 	int shmid;
 	void *shmaddr;
 	int shmflg;
 };
+#endif
+
 int
 shmat(p, uap, retval)
 	struct proc *p;
@@ -199,9 +224,11 @@ shmat(p, uap, retval)
 	struct ucred *cred = p->p_ucred;
 	struct shmid_ds *shmseg;
 	struct shmmap_state *shmmap_s = NULL;
+	struct shm_handle *shm_handle;
 	vm_offset_t attach_va;
 	vm_prot_t prot;
 	vm_size_t size;
+	int rv;
 
 	shmmap_s = (struct shmmap_state *)p->p_vmspace->vm_shm;
 	if (shmmap_s == NULL) {
@@ -225,7 +252,7 @@ shmat(p, uap, retval)
 	}
 	if (i >= shminfo.shmseg)
 		return EMFILE;
-	size = (shmseg->shm_segsz + CLOFSET) & ~CLOFSET;
+	size = round_page(shmseg->shm_segsz);
 	prot = VM_PROT_READ;
 	if ((uap->shmflg & SHM_RDONLY) == 0)
 		prot |= VM_PROT_WRITE;
@@ -239,13 +266,20 @@ shmat(p, uap, retval)
 		else
 			return EINVAL;
 	} else {
-		/* This is just a hint to vm_mmap() about where to put it. */
-		attach_va = round_page(p->p_vmspace->vm_daddr + MAXDSIZ);
+		/* This is just a hint to vm_map_find() about where to put it. */
+		attach_va = round_page(p->p_vmspace->vm_taddr + MAXTSIZ + MAXDSIZ);
 	}
-	error = vm_mmap(&p->p_vmspace->vm_map, &attach_va, size, prot,
-	    VM_PROT_DEFAULT, flags, (caddr_t) uap->shmid, 0);
-	if (error)
-		return error;
+
+	shm_handle = shmseg->shm_internal;
+	vm_object_reference(shm_handle->shm_object);
+	rv = vm_map_find(&p->p_vmspace->vm_map, shm_handle->shm_object,
+		0, &attach_va, size, (flags & MAP_FIXED)?0:1, prot, prot, 0);
+	if (rv != KERN_SUCCESS) {
+		return ENOMEM;
+	}
+	vm_map_inherit(&p->p_vmspace->vm_map,
+		attach_va, attach_va + size, VM_INHERIT_SHARE);
+
 	shmmap_s->va = attach_va;
 	shmmap_s->shmid = uap->shmid;
 	shmseg->shm_lpid = p->p_pid;
@@ -273,7 +307,7 @@ struct oshmctl_args {
 	struct oshmid_ds *ubuf;
 };
 
-int
+static int
 oshmctl(p, uap, retval)
 	struct proc *p;
 	struct oshmctl_args *uap;
@@ -307,7 +341,8 @@ oshmctl(p, uap, retval)
 			return error;
 		break;
 	default:
-		return shmctl(p, uap, retval);
+		/* XXX casting to (sy_call_t *) is bogus, as usual. */
+		return ((sy_call_t *)shmctl)(p, uap, retval);
 	}
 	return 0;
 #else
@@ -315,11 +350,14 @@ oshmctl(p, uap, retval)
 #endif
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct shmctl_args {
 	int shmid;
 	int cmd;
-	struct shmid_ds *ubuf;
+	struct shmid_ds *buf;
 };
+#endif
+
 int
 shmctl(p, uap, retval)
 	struct proc *p;
@@ -339,7 +377,7 @@ shmctl(p, uap, retval)
 		error = ipcperm(cred, &shmseg->shm_perm, IPC_R);
 		if (error)
 			return error;
-		error = copyout((caddr_t)shmseg, uap->ubuf, sizeof(inbuf));
+		error = copyout((caddr_t)shmseg, uap->buf, sizeof(inbuf));
 		if (error)
 			return error;
 		break;
@@ -347,7 +385,7 @@ shmctl(p, uap, retval)
 		error = ipcperm(cred, &shmseg->shm_perm, IPC_M);
 		if (error)
 			return error;
-		error = copyin(uap->ubuf, (caddr_t)&inbuf, sizeof(inbuf));
+		error = copyin(uap->buf, (caddr_t)&inbuf, sizeof(inbuf));
 		if (error)
 			return error;
 		shmseg->shm_perm.uid = inbuf.shm_perm.uid;
@@ -378,11 +416,14 @@ shmctl(p, uap, retval)
 	return 0;
 }
 
+#ifndef _SYS_SYSPROTO_H_
 struct shmget_args {
 	key_t key;
 	size_t size;
 	int shmflg;
 };
+#endif
+
 static int
 shmget_existing(p, uap, mode, segnum, retval)
 	struct proc *p;
@@ -426,7 +467,7 @@ shmget_allocate_segment(p, uap, mode, retval)
 	int mode;
 	int *retval;
 {
-	int i, segnum, result, shmid, size;
+	int i, segnum, shmid, size;
 	struct ucred *cred = p->p_ucred;
 	struct shmid_ds *shmseg;
 	struct shm_handle *shm_handle;
@@ -435,7 +476,7 @@ shmget_allocate_segment(p, uap, mode, retval)
 		return EINVAL;
 	if (shm_nused >= shminfo.shmmni) /* any shmids left? */
 		return ENOSPC;
-	size = (uap->size + CLOFSET) & ~CLOFSET;
+	size = round_page(uap->size);
 	if (shm_committed + btoc(size) > shminfo.shmall)
 		return ENOMEM;
 	if (shm_last_free < 0) {
@@ -460,16 +501,14 @@ shmget_allocate_segment(p, uap, mode, retval)
 	shm_handle = (struct shm_handle *)
 	    malloc(sizeof(struct shm_handle), M_SHM, M_WAITOK);
 	shmid = IXSEQ_TO_IPCID(segnum, shmseg->shm_perm);
-	result = vm_mmap(sysvshm_map, &shm_handle->kva, size, VM_PROT_ALL,
-	    VM_PROT_DEFAULT, MAP_ANON, (caddr_t) shmid, 0);
-	if (result != KERN_SUCCESS) {
-		shmseg->shm_perm.mode = SHMSEG_FREE;
-		shm_last_free = segnum;
-		free((caddr_t)shm_handle, M_SHM);
-		/* Just in case. */
-		wakeup((caddr_t)shmseg);
-		return ENOMEM;
-	}
+	
+	/*
+	 * We make sure that we have allocated a pager before we need
+	 * to.
+	 */
+	shm_handle->shm_object =
+		vm_pager_allocate(OBJT_SWAP, 0, OFF_TO_IDX(size),
+			VM_PROT_DEFAULT, 0);
 	shmseg->shm_internal = shm_handle;
 	shmseg->shm_perm.cuid = shmseg->shm_perm.uid = cred->cr_uid;
 	shmseg->shm_perm.cgid = shmseg->shm_perm.gid = cred->cr_gid;
@@ -518,25 +557,27 @@ shmget(p, uap, retval)
 	return shmget_allocate_segment(p, uap, mode, retval);
 }
 
-struct shmsys_args {
-	u_int	which;
-};
 int
 shmsys(p, uap, retval)
 	struct proc *p;
-	struct shmsys_args *uap;
+	/* XXX actually varargs. */
+	struct shmsys_args /* {
+		u_int	which;
+		int	a2;
+		int	a3;
+		int	a4;
+	} */ *uap;
 	int *retval;
 {
 
 	if (uap->which >= sizeof(shmcalls)/sizeof(shmcalls[0]))
 		return EINVAL;
-	return ((*shmcalls[uap->which])(p, &uap[1], retval));
+	return ((*shmcalls[uap->which])(p, &uap->a2, retval));
 }
 
 void
-shmfork(p1, p2, isvfork)
+shmfork(p1, p2)
 	struct proc *p1, *p2;
-	int isvfork;
 {
 	struct shmmap_state *shmmap_s;
 	size_t size;
@@ -567,14 +608,10 @@ shmexit(p)
 }
 
 void
-shminit()
+shminit(dummy)
+	void *dummy;
 {
 	int i;
-	vm_offset_t garbage1, garbage2;
-
-	/* actually this *should* be pageable.  SHM_{LOCK,UNLOCK} */
-	sysvshm_map = kmem_suballoc(kernel_map, &garbage1, &garbage2,
-				    shminfo.shmall * NBPG, TRUE);
 	for (i = 0; i < shminfo.shmmni; i++) {
 		shmsegs[i].shm_perm.mode = SHMSEG_FREE;
 		shmsegs[i].shm_perm.seq = 0;
