@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)tty.c	8.8 (Berkeley) 1/21/94
- * $FreeBSD: stable/2.1/sys/kern/tty.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/tty.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*-
@@ -68,6 +68,7 @@
  */
 
 #include "snp.h"
+#include "opt_uconsole.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
@@ -91,6 +92,11 @@
 #endif
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/lock.h>
+#include <vm/pmap.h>
+#include <vm/vm_map.h>
 
 static int	proc_compare __P((struct proc *p1, struct proc *p2));
 static int	ttnread __P((struct tty *tp));
@@ -101,6 +107,7 @@ static void	ttyretype __P((struct tty *tp));
 static void	ttyrub __P((int c, struct tty *tp));
 static void	ttyrubo __P((struct tty *tp, int cnt));
 static void	ttyunblock __P((struct tty *tp));
+static int	ttywflush __P((struct tty *tp));
 
 /*
  * Table with character classes and parity. The 8th bit indicates parity,
@@ -128,7 +135,7 @@ static void	ttyunblock __P((struct tty *tp));
 #define	TB	TAB
 #define	VT	VTAB
 
-char const char_type[] = {
+static u_char const char_type[] = {
 	E|CC, O|CC, O|CC, E|CC, O|CC, E|CC, E|CC, O|CC,	/* nul - bel */
 	O|BS, E|TB, E|NL, O|CC, E|VT, O|CR, O|CC, E|CC, /* bs - si */
 	O|CC, E|CC, E|CC, O|CC, E|CC, O|CC, O|CC, E|CC, /* dle - etb */
@@ -266,9 +273,10 @@ ttyclose(tp)
 }
 
 /* Is 'c' a line delimiter ("break" character)? */
-#define	TTBREAKC(c)							\
+#define	TTBREAKC(c, lflag)							\
 	((c) == '\n' || (((c) == cc[VEOF] ||				\
-	(c) == cc[VEOL] || (c) == cc[VEOL2]) && (c) != _POSIX_VDISABLE))
+	  (c) == cc[VEOL] || ((c) == cc[VEOL2] && lflag & IEXTEN)) &&	\
+	 (c) != _POSIX_VDISABLE))
 
 /*
  * Process input of a single character received on a tty.
@@ -318,14 +326,15 @@ ttyinput(c, tp)
 	err = (ISSET(c, TTY_ERRORMASK));
 	if (err) {
 		CLR(c, TTY_ERRORMASK);
-		if (ISSET(err, TTY_BI)) { /* Break. */
+		if (ISSET(err, TTY_BI)) {
 			if (ISSET(iflag, IGNBRK))
 				return (0);
-			else if (ISSET(iflag, BRKINT) &&
-			    ISSET(lflag, ISIG) &&
-			    (cc[VINTR] != _POSIX_VDISABLE))
-				c = cc[VINTR];
-			else if (ISSET(iflag, PARMRK))
+			if (ISSET(iflag, BRKINT)) {
+				ttyflush(tp, FREAD | FWRITE);
+				pgsignal(tp->t_pgrp, SIGINT, 1);
+				goto endcase;
+			}
+			if (ISSET(iflag, PARMRK))
 				goto parmrk;
 		} else if ((ISSET(err, TTY_PE) && ISSET(iflag, INPCK))
 			|| ISSET(err, TTY_FE)) {
@@ -422,7 +431,7 @@ parmrk:
 #ifdef sun4c						/* XXX */
 					(*tp->t_stop)(tp, 0);
 #else
-					(*cdevsw[major(tp->t_dev)].d_stop)(tp,
+					(*cdevsw[major(tp->t_dev)]->d_stop)(tp,
 					   0);
 #endif
 					return (0);
@@ -484,7 +493,7 @@ parmrk:
 		/*
 		 * word erase (^W)
 		 */
-		if (CCEQ(cc[VWERASE], c)) {
+		if (CCEQ(cc[VWERASE], c) && ISSET(lflag, IEXTEN)) {
 			int ctype;
 
 			/*
@@ -523,14 +532,14 @@ parmrk:
 		/*
 		 * reprint line (^R)
 		 */
-		if (CCEQ(cc[VREPRINT], c)) {
+		if (CCEQ(cc[VREPRINT], c) && ISSET(lflag, IEXTEN)) {
 			ttyretype(tp);
 			goto endcase;
 		}
 		/*
 		 * ^T - kernel info and generate SIGINFO
 		 */
-		if (CCEQ(cc[VSTATUS], c)) {
+		if (CCEQ(cc[VSTATUS], c) && ISSET(lflag, IEXTEN)) {
 			if (ISSET(lflag, ISIG))
 				pgsignal(tp->t_pgrp, SIGINFO, 1);
 			if (!ISSET(lflag, NOKERNINFO))
@@ -564,7 +573,7 @@ input_overflow:
 			ttyecho(c, tp);
 			goto endcase;
 		}
-		if (TTBREAKC(c)) {
+		if (TTBREAKC(c, lflag)) {
 			tp->t_rocount = 0;
 			catq(&tp->t_rawq, &tp->t_canq);
 			ttwakeup(tp);
@@ -729,10 +738,12 @@ ttioctl(tp, cmd, data, flag)
 	case  TIOCSETP:
 	case  TIOCSLTC:
 #endif
-		while (isbackground(curproc, tp) &&
-		    p->p_pgrp->pg_jobc && (p->p_flag & P_PPWAIT) == 0 &&
+		while (isbackground(p, tp) &&
+		    (p->p_flag & P_PPWAIT) == 0 &&
 		    (p->p_sigignore & sigmask(SIGTTOU)) == 0 &&
 		    (p->p_sigmask & sigmask(SIGTTOU)) == 0) {
+			if (p->p_pgrp->pg_jobc == 0)
+				return (EIO);
 			pgsignal(p->p_pgrp, SIGTTOU, 1);
 			error = ttysleep(tp, &lbolt, TTOPRI | PCATCH, "ttybg1",
 					 0);
@@ -961,7 +972,7 @@ ttioctl(tp, cmd, data, flag)
 #ifdef sun4c				/* XXX */
 			(*tp->t_stop)(tp, 0);
 #else
-			(*cdevsw[major(tp->t_dev)].d_stop)(tp, 0);
+			(*cdevsw[major(tp->t_dev)]->d_stop)(tp, 0);
 #endif
 		}
 		splx(s);
@@ -1062,7 +1073,7 @@ ttselect(dev, rw, p)
 	int rw;
 	struct proc *p;
 {
-	return ttyselect((*cdevsw[major(dev)].d_devtotty)(dev), rw, p);
+	return ttyselect((*cdevsw[major(dev)]->d_devtotty)(dev), rw, p);
 }
 
 /*
@@ -1122,7 +1133,7 @@ ttywait(tp)
 /*
  * Flush if successfully wait.
  */
-int
+static int
 ttywflush(tp)
 	struct tty *tp;
 {
@@ -1144,13 +1155,17 @@ ttyflush(tp, rw)
 	register int s;
 
 	s = spltty();
+#if 0
 again:
-	if (rw & FWRITE)
+#endif
+	if (rw & FWRITE) {
+		FLUSHQ(&tp->t_outq);
 		CLR(tp->t_state, TS_TTSTOP);
+	}
 #ifdef sun4c						/* XXX */
 	(*tp->t_stop)(tp, rw);
 #else
-	(*cdevsw[major(tp->t_dev)].d_stop)(tp, rw);
+	(*cdevsw[major(tp->t_dev)]->d_stop)(tp, rw);
 #endif
 	if (rw & FREAD) {
 		FLUSHQ(&tp->t_canq);
@@ -1260,6 +1275,11 @@ ttyunblock(tp)
 	ttstart(tp);
 }
 
+#ifdef notyet
+/* Not used by any current (i386) drivers. */
+/*
+ * Restart after an inter-char delay.
+ */
 void
 ttrstrt(tp_arg)
 	void *tp_arg;
@@ -1279,6 +1299,7 @@ ttrstrt(tp_arg)
 
 	splx(s);
 }
+#endif
 
 int
 ttstart(tp)
@@ -1331,7 +1352,7 @@ ttymodem(tp, flag)
 #ifdef sun4c						/* XXX */
 			(*tp->t_stop)(tp, 0);
 #else
-			(*cdevsw[major(tp->t_dev)].d_stop)(tp, 0);
+			(*cdevsw[major(tp->t_dev)]->d_stop)(tp, 0);
 #endif
 		}
 	} else if (flag == 0) {
@@ -1612,7 +1633,8 @@ slowcase:
 		/*
 		 * delayed suspend (^Y)
 		 */
-		if (CCEQ(cc[VDSUSP], c) && ISSET(lflag, ISIG)) {
+		if (CCEQ(cc[VDSUSP], c) &&
+		    ISSET(lflag, IEXTEN | ISIG) == (IEXTEN | ISIG)) {
 			pgsignal(tp->t_pgrp, SIGTSTP, 1);
 			if (first) {
 				error = ttysleep(tp, &lbolt, TTIPRI | PCATCH,
@@ -1650,7 +1672,7 @@ slowcase:
 		 * In canonical mode check for a "break character"
 		 * marking the end of a "line of input".
 		 */
-		if (ISSET(lflag, ICANON) && TTBREAKC(c))
+		if (ISSET(lflag, ICANON) && TTBREAKC(c, lflag))
 			break;
 		first = 0;
 	}
@@ -1750,8 +1772,11 @@ loop:
 	if (isbackground(p, tp) &&
 	    ISSET(tp->t_lflag, TOSTOP) && (p->p_flag & P_PPWAIT) == 0 &&
 	    (p->p_sigignore & sigmask(SIGTTOU)) == 0 &&
-	    (p->p_sigmask & sigmask(SIGTTOU)) == 0 &&
-	     p->p_pgrp->pg_jobc) {
+	    (p->p_sigmask & sigmask(SIGTTOU)) == 0) {
+		if (p->p_pgrp->pg_jobc == 0) {
+			error = EIO;
+			goto out;
+		}
 		pgsignal(p->p_pgrp, SIGTTOU, 1);
 		error = ttysleep(tp, &lbolt, TTIPRI | PCATCH, "ttybg4", 0);
 		if (error)
@@ -1801,7 +1826,7 @@ loop:
 				ce = cc;
 			else {
 				ce = cc - scanc((u_int)cc, (u_char *)cp,
-				   (u_char *)char_type, CCLASSMASK);
+						char_type, CCLASSMASK);
 				/*
 				 * If ce is zero, then we're processing
 				 * a special character through ttyoutput.
@@ -2164,11 +2189,11 @@ ttyinfo(tp)
 		ttyprintf(tp, "not a controlling terminal\n");
 	else if (tp->t_pgrp == NULL)
 		ttyprintf(tp, "no foreground process group\n");
-	else if ((p = tp->t_pgrp->pg_mem) == NULL)
+	else if ((p = tp->t_pgrp->pg_members.lh_first) == 0)
 		ttyprintf(tp, "empty foreground process group\n");
 	else {
 		/* Pick interesting process. */
-		for (pick = NULL; p != NULL; p = p->p_pgrpnxt)
+		for (pick = NULL; p != 0; p = p->p_pglist.le_next)
 			if (proc_compare(pick, p))
 				pick = p;
 
@@ -2186,7 +2211,7 @@ ttyinfo(tp)
 		ttyprintf(tp, "%d.%02ds ",
 		    stime.tv_sec, stime.tv_usec / 10000);
 
-#define	pgtok(a)	(((a) * NBPG) / 1024)
+#define	pgtok(a)	(((a) * PAGE_SIZE) / 1024)
 		/* Print percentage cpu, resident set size. */
 		tmp = (pick->p_pctcpu * 10000 + FSCALE / 2) >> FSHIFT;
 		ttyprintf(tp, "%d%% %dk\n",
@@ -2299,7 +2324,7 @@ tputchar(c, tp)
 
 /*
  * Sleep on chan, returning ERESTART if tty changed while we napped and
- * returning any errors (e.g. EINTR/ETIMEDOUT) reported by tsleep.  If
+ * returning any errors (e.g. EINTR/EWOULDBLOCK) reported by tsleep.  If
  * the tty is revoked, restarting a pending call will redo validation done
  * at the start of the call.
  */
@@ -2320,6 +2345,7 @@ ttysleep(tp, chan, pri, wmesg, timo)
 	return (tp->t_gen == gen ? 0 : ERESTART);
 }
 
+#ifdef notyet
 /*
  * XXX this is usable not useful or used.  Most tty drivers have
  * ifdefs for using ttymalloc() but assume a different interface.
@@ -2337,6 +2363,7 @@ ttymalloc()
         bzero(tp, sizeof *tp);
         return (tp);
 }
+#endif
 
 #if 0 /* XXX not yet usable: session leader holds a ref (see kern_exit.c). */
 /*
