@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)tty_compat.c	8.1 (Berkeley) 6/10/93
- * $FreeBSD: stable/2.1/sys/kern/tty_compat.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/tty_compat.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -48,13 +48,16 @@
 #include <sys/file.h>
 #include <sys/conf.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
 #include <sys/syslog.h>
 
 static int ttcompatgetflags	__P((struct tty	*tp));
 static void ttcompatsetflags	__P((struct tty	*tp, struct termios *t));
 static void ttcompatsetlflags	__P((struct tty	*tp, struct termios *t));
+static int ttcompatspeedtab	__P((int speed, struct speedtab *table));
 
-int ttydebug = 0;
+static int ttydebug = 0;
+SYSCTL_INT(_debug, OID_AUTO, ttydebug, CTLFLAG_RW, &ttydebug, 0, "");
 
 static struct speedtab compatspeeds[] = {
 #define MAX_SPEED	17
@@ -80,11 +83,11 @@ static struct speedtab compatspeeds[] = {
 };
 static int compatspcodes[] = {
 	0, 50, 75, 110, 134, 150, 200, 300, 600, 1200,
-	1800, 2400, 4800, 9600, 19200, 38400, 57600, 115200
+	1800, 2400, 4800, 9600, 19200, 38400, 57600, 115200,
 };
 
-static
-int ttcompatspeedtab(speed, table)
+static int
+ttcompatspeedtab(speed, table)
 	int speed;
 	register struct speedtab *table;
 {
@@ -96,7 +99,8 @@ int ttcompatspeedtab(speed, table)
 	return (1); /* 50, min and not hangup */
 }
 
-int ttsetcompat(tp, com, data, term)
+int
+ttsetcompat(tp, com, data, term)
 	register struct tty *tp;
 	int *com;
 	caddr_t data;
@@ -386,6 +390,7 @@ ttcompatsetflags(tp, t)
 		iflag |= ISTRIP;
 		oflag |= OPOST;
 	}
+	/* XXX don't set INPCK if RAW or PASS8? */
 	if ((flags&(EVENP|ODDP)) == EVENP) {
 		iflag |= INPCK;
 		cflag &= ~PARODD;
