@@ -30,8 +30,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)tty_pty.c	8.2 (Berkeley) 9/23/93
- * $FreeBSD: stable/2.1/sys/kern/tty_pty.c 50950 1999-09-05 08:39:22Z peter $
+ *	@(#)tty_pty.c	8.4 (Berkeley) 2/20/95
+ * $FreeBSD: stable/2.2/sys/kern/tty_pty.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -52,28 +52,74 @@
 #include <sys/vnode.h>
 #include <sys/signalvar.h>
 
-void ptyattach __P((int n));
-void ptsstart __P((struct tty *tp));
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
+
+#ifdef notyet
+static void ptyattach __P((int n));
+#endif
+static void ptsstart __P((struct tty *tp));
+static void ptcwakeup __P((struct tty *tp, int flag));
+
+static	d_open_t	ptsopen;
+static	d_close_t	ptsclose;
+static	d_read_t	ptsread;
+static	d_write_t	ptswrite;
+static	d_ioctl_t	ptyioctl;
+static	d_stop_t	ptsstop;
+static	d_devtotty_t	ptydevtotty;
+static	d_open_t	ptcopen;
+static	d_close_t	ptcclose;
+static	d_read_t	ptcread;
+static	d_write_t	ptcwrite;
+static	d_select_t	ptcselect;
+
+#define CDEV_MAJOR_S 5
+#define CDEV_MAJOR_C 6
+static struct cdevsw pts_cdevsw = 
+	{ ptsopen,	ptsclose,	ptsread,	ptswrite,	/*5*/
+	  ptyioctl,	ptsstop,	nullreset,	ptydevtotty,/* ttyp */
+	  ttselect,	nommap,		NULL,	"pts",	NULL,	-1 };
+
+static struct cdevsw ptc_cdevsw = 
+	{ ptcopen,	ptcclose,	ptcread,	ptcwrite,	/*6*/
+	  ptyioctl,	nullstop,	nullreset,	ptydevtotty,/* ptyp */
+	  ptcselect,	nommap,		NULL,	"ptc",	NULL,	-1 };
+
 
 #if NPTY == 1
 #undef NPTY
 #define	NPTY	32		/* crude XXX */
+#warning	You have only one pty defined, redefining to 32.
+#endif
+
+#ifdef DEVFS
+#define MAXUNITS (8 * 32)
+static	void	*devfs_token_pts[MAXUNITS];
+static	void	*devfs_token_ptc[MAXUNITS];
+static  const	char jnames[] = "pqrsPQRS";
+#if NPTY > MAXUNITS
+#undef NPTY
+#define NPTY MAXUNITS
+#warning	Can't have more than 256 pty's with DEVFS defined.
+#endif
 #endif
 
 #define BUFSIZ 100		/* Chunk size iomoved to/from user */
 
 /*
- * pts == /dev/tty[pqrs]?
- * ptc == /dev/pty[pqrs]?
+ * pts == /dev/tty[pqrsPQRS][0123456789abcdefghijklmnopqrstuv]
+ * ptc == /dev/pty[pqrsPQRS][0123456789abcdefghijklmnopqrstuv]
  */
-struct	tty pt_tty[NPTY];	/* XXX */
-struct	pt_ioctl {
+static struct	tty pt_tty[NPTY];	/* XXX */
+static struct	pt_ioctl {
 	int	pt_flags;
 	struct	selinfo pt_selr, pt_selw;
 	u_char	pt_send;
 	u_char	pt_ucntl;
 } pt_ioctl[NPTY];		/* XXX */
-int	npty = NPTY;		/* for pstat -t */
+static int	npty = NPTY;		/* for pstat -t */
 
 #define	PF_PKT		0x08		/* packet mode */
 #define	PF_STOPPED	0x10		/* user told stopped */
@@ -81,19 +127,16 @@ int	npty = NPTY;		/* for pstat -t */
 #define	PF_NOSTOP	0x40
 #define PF_UCNTL	0x80		/* user control mode */
 
-void	ptsstop		__P((struct tty *, int));
-void	ptcwakeup	__P((struct tty *, int));
-
+#ifdef notyet
 /*
  * Establish n (or default if n is 1) ptys in the system.
  *
  * XXX cdevsw & pstat require the array `pty[]' to be an array
  */
-void
+static void
 ptyattach(n)
 	int n;
 {
-#ifdef notyet
 	char *mem;
 	register u_long ntb;
 #define	DEFAULT_NPTY	32
@@ -108,11 +151,11 @@ ptyattach(n)
 	mem = (char *)ALIGN(mem + ntb);
 	pt_ioctl = (struct pt_ioctl *)mem;
 	npty = n;
-#endif
 }
+#endif
 
 /*ARGSUSED*/
-int
+static	int
 ptsopen(dev, flag, devtype, p)
 	dev_t dev;
 	int flag, devtype;
@@ -150,7 +193,7 @@ ptsopen(dev, flag, devtype, p)
 	return (error);
 }
 
-int
+static	int
 ptsclose(dev, flag, mode, p)
 	dev_t dev;
 	int flag, mode;
@@ -166,7 +209,7 @@ ptsclose(dev, flag, mode, p)
 	return (err);
 }
 
-int
+static	int
 ptsread(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -221,7 +264,7 @@ again:
  * Wakeups of controlling tty will happen
  * indirectly, when tty driver calls ptsstart.
  */
-int
+static	int
 ptswrite(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -239,7 +282,7 @@ ptswrite(dev, uio, flag)
  * Start output on pseudo-tty.
  * Wake up process selecting or sleeping for input from controlling tty.
  */
-void
+static void
 ptsstart(tp)
 	struct tty *tp;
 {
@@ -254,7 +297,7 @@ ptsstart(tp)
 	ptcwakeup(tp, FREAD);
 }
 
-void
+static void
 ptcwakeup(tp, flag)
 	struct tty *tp;
 	int flag;
@@ -271,7 +314,7 @@ ptcwakeup(tp, flag)
 	}
 }
 
-int
+static	int
 ptcopen(dev, flag, devtype, p)
 	dev_t dev;
 	int flag, devtype;
@@ -298,7 +341,7 @@ ptcopen(dev, flag, devtype, p)
 	return (0);
 }
 
-int
+static	int
 ptcclose(dev, flags, fmt, p)
 	dev_t dev;
 	int flags;
@@ -328,7 +371,7 @@ ptcclose(dev, flags, fmt, p)
 	return (0);
 }
 
-int
+static	int
 ptcread(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -390,7 +433,7 @@ ptcread(dev, uio, flag)
 	return (error);
 }
 
-void
+static	void
 ptsstop(tp, flush)
 	register struct tty *tp;
 	int flush;
@@ -414,7 +457,7 @@ ptsstop(tp, flush)
 	ptcwakeup(tp, flag);
 }
 
-int
+static	int
 ptcselect(dev, rw, p)
 	dev_t dev;
 	int rw;
@@ -469,7 +512,7 @@ ptcselect(dev, rw, p)
 	return (0);
 }
 
-int
+static	int
 ptcwrite(dev, uio, flag)
 	dev_t dev;
 	register struct uio *uio;
@@ -579,7 +622,7 @@ block:
 	goto again;
 }
 
-struct tty *
+static	struct tty *
 ptydevtotty(dev)
 	dev_t		dev;
 {
@@ -590,7 +633,7 @@ ptydevtotty(dev)
 }
 
 /*ARGSUSED*/
-int
+static	int
 ptyioctl(dev, cmd, data, flag, p)
 	dev_t dev;
 	int cmd;
@@ -629,7 +672,7 @@ ptyioctl(dev, cmd, data, flag, p)
 		}
 		return(0);
 	} else
-	if (cdevsw[major(dev)].d_open == ptcopen)
+	if (cdevsw[major(dev)]->d_open == ptcopen)
 		switch (cmd) {
 
 		case TIOCGPGRP:
@@ -678,7 +721,8 @@ ptyioctl(dev, cmd, data, flag, p)
 			break;
 
 		case TIOCSIG:
-			if (*(unsigned int *)data >= NSIG)
+			if (*(unsigned int *)data >= NSIG ||
+			    *(unsigned int *)data == 0)
 				return(EINVAL);
 			if ((tp->t_lflag&NOFLSH) == 0)
 				ttyflush(tp, FREAD|FWRITE);
@@ -746,3 +790,38 @@ ptyioctl(dev, cmd, data, flag, p)
 	}
 	return (error);
 }
+
+static ptc_devsw_installed = 0;
+
+static void
+ptc_drvinit(void *unused)
+{
+#ifdef DEVFS
+	int i,j,k;
+#endif
+	dev_t dev;
+
+	if( ! ptc_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR_S, 0);
+		cdevsw_add(&dev, &pts_cdevsw, NULL);
+		dev = makedev(CDEV_MAJOR_C, 0);
+		cdevsw_add(&dev, &ptc_cdevsw, NULL);
+		ptc_devsw_installed = 1;
+#ifdef DEVFS
+		for ( i = 0 ; i<NPTY ; i++ ) {
+			j = i / 32;
+			k = i % 32;
+			devfs_token_pts[i] = 
+				devfs_add_devswf(&pts_cdevsw,i,
+						DV_CHR,0,0,0666,
+						"tty%c%n",jnames[j],k);
+			devfs_token_ptc[i] =
+				devfs_add_devswf(&ptc_cdevsw,i,
+						DV_CHR,0,0,0666,
+						"pty%c%n",jnames[j],k);
+		}
+#endif
+    	}
+}
+
+SYSINIT(ptcdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR_C,ptc_drvinit,NULL)
