@@ -22,31 +22,55 @@
 #include <sys/ioctl.h>
 #include <sys/ioctl_compat.h>	/* Oooh..We need O/NTTYDISC	 */
 #include <sys/proc.h>
-#define TTYDEFCHARS
 #include <sys/tty.h>
-#undef  TTYDEFCHARS
 #include <sys/file.h>
 #include <sys/conf.h>
 #include <sys/uio.h>
 #include <sys/kernel.h>
 #include <sys/malloc.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <sys/snoop.h>
 
+static	d_open_t	snpopen;
+static	d_close_t	snpclose;
+static	d_read_t	snpread;
+static	d_write_t	snpwrite;
+static	d_ioctl_t	snpioctl;
+static	d_select_t	snpselect;
+
+#define CDEV_MAJOR 53
+static struct cdevsw snp_cdevsw = 
+	{ snpopen,	snpclose,	snpread,	snpwrite,	/*53*/
+	  snpioctl,	nostop,		nullreset,	nodevtotty,/* snoop */
+	  snpselect,	nommap,		NULL,	"snp",	NULL,	-1 };
+
+
 #ifndef MIN
 #define MIN(a,b) (((a)<(b))?(a):(b))
 #endif
 
 static struct snoop snoopsw[NSNP];
 
+static struct tty	*snpdevtotty __P((dev_t dev));
+static int		snp_detach __P((struct snoop *snp));
+
 static struct tty *
-devtotty (dev)
+snpdevtotty (dev)
 	dev_t		dev;
 {
-	if (major(dev) > nchrdev)
-		return (NULL);		/* no such device available */
+	struct cdevsw	*cdp;
+	int		maj;
 
-	return (*cdevsw[major(dev)].d_devtotty)(dev);
+	maj = major(dev);
+	if ((u_int)maj >= nchrdev)
+		return (NULL);
+	cdp = cdevsw[maj];
+	if (cdp == NULL)
+		return (NULL);
+	return ((*cdp->d_devtotty)(dev));
 }
 
 #define SNP_INPUT_BUF	5	/* This is even too  much,the maximal
@@ -54,7 +78,7 @@ devtotty (dev)
 				 * length for function keys...
 				 */
 
-int
+static	int
 snpwrite(dev, uio, flag)
 	dev_t           dev;
 	struct uio     *uio;
@@ -95,7 +119,7 @@ tty_input:
 }
 
 
-int
+static	int
 snpread(dev, uio, flag)
 	dev_t           dev;
 	struct uio     *uio;
@@ -182,7 +206,6 @@ snpin(snp, buf, n)
 	int             s, len, nblen;
 	caddr_t         from, to;
 	char           *nbuf;
-	struct tty     *tp;
 
 
 	if (n == 0)
@@ -262,7 +285,7 @@ snpin(snp, buf, n)
 	return n;
 }
 
-int
+static	int
 snpopen(dev, flag, mode, p)
 	dev_t           dev;
 	int             flag, mode;
@@ -302,7 +325,7 @@ snpopen(dev, flag, mode, p)
 }
 
 
-int
+static int
 snp_detach(snp)
 	struct snoop   *snp;
 {
@@ -338,7 +361,7 @@ detach_notty:
 	return (0);
 }
 
-int
+static	int
 snpclose(dev, flags, fmt, p)
 	dev_t           dev;
 	int             flags;
@@ -368,7 +391,7 @@ snpdown(snp)
 }
 
 
-int
+static	int
 snpioctl(dev, cmd, data, flags, p)
 	dev_t           dev;
 	int             cmd;
@@ -387,7 +410,7 @@ snpioctl(dev, cmd, data, flags, p)
 		if (tdev == -1)
 			return (snpdown(snp));
 
-		tp = devtotty(tdev);
+		tp = snpdevtotty(tdev);
 		if (!tp)
 			return (EINVAL);
 
@@ -465,13 +488,13 @@ snpioctl(dev, cmd, data, flags, p)
 }
 
 
-int
+static	int
 snpselect(dev, rw, p)
 	dev_t           dev;
 	int             rw;
 	struct proc    *p;
 {
-	int             unit = minor(dev), s;
+	int             unit = minor(dev);
 	struct snoop   *snp = &snoopsw[unit];
 
 	if (rw != FREAD)
@@ -492,4 +515,34 @@ snpselect(dev, rw, p)
 	return 0;
 }
 
+#ifdef DEVFS
+static	void	*snp_devfs_token[NSNP];
+#endif
+static snp_devsw_installed = 0;
+
+static void
+snp_drvinit(void *unused)
+{
+	dev_t dev;
+#ifdef DEVFS
+	int	i;
+#endif
+
+	if( ! snp_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&snp_cdevsw, NULL);
+		snp_devsw_installed = 1;
+#ifdef DEVFS
+		for ( i = 0 ; i < NSNP ; i++) {
+			snp_devfs_token[i] =
+				devfs_add_devswf(&snp_cdevsw, i, DV_CHR, 0, 0, 
+						0600, "snp%d", i);
+		}
+#endif
+    	}
+}
+
+SYSINIT(snpdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,snp_drvinit,NULL)
+
+
 #endif
