@@ -6,7 +6,8 @@
  * modification, are permitted provided that the following conditions
  * are met:
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *    notice unmodified, this list of conditions, and the following
+ *    disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
@@ -23,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/kern/tty_subr.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/tty_subr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -31,13 +32,17 @@
  */
 
 #include <sys/param.h>
+#include <sys/kernel.h>
 #include <sys/systm.h>
 #include <sys/ioctl.h>
 #include <sys/tty.h>
 #include <sys/clist.h>
 #include <sys/malloc.h>
 
-struct cblock *cfreelist = 0;
+static void clist_init __P((void *));
+SYSINIT(clist, SI_SUB_CLIST, SI_ORDER_FIRST, clist_init, NULL)
+
+static struct cblock *cfreelist = 0;
 int cfreecount = 0;
 static int cslushcount;
 static int ctotcount;
@@ -46,26 +51,31 @@ static int ctotcount;
 #define	INITIAL_CBLOCKS 50
 #endif
 
+static struct cblock *cblock_alloc __P((void));
 static void cblock_alloc_cblocks __P((int number));
+static void cblock_free __P((struct cblock *cblockp));
 static void cblock_free_cblocks __P((int number));
 
-#define	CBLOCK_DIAG
-#ifdef CBLOCK_DIAG
-static void
-cbstat()
+#include "opt_ddb.h"
+#ifdef DDB
+#include <ddb/ddb.h>
+
+DB_SHOW_COMMAND(cbstat, cbstat)
 {
 	printf(
 	"tot = %d (active = %d, free = %d (reserved = %d, slush = %d))\n",
 	       ctotcount * CBSIZE, ctotcount * CBSIZE - cfreecount, cfreecount,
 	       cfreecount - cslushcount * CBSIZE, cslushcount * CBSIZE);
 }
-#endif
+#endif /* DDB */
 
 /*
  * Called from init_main.c
  */
-void
-clist_init()
+/* ARGSUSED*/
+static void
+clist_init(dummy)
+	void *dummy;
 {
 	/*
 	 * Allocate an initial base set of cblocks as a 'slush'.
@@ -82,7 +92,7 @@ clist_init()
  * Remove a cblock from the cfreelist queue and return a pointer
  * to it.
  */
-static inline struct cblock *
+static __inline struct cblock *
 cblock_alloc()
 {
 	struct cblock *cblockp;
@@ -99,7 +109,7 @@ cblock_alloc()
 /*
  * Add a cblock to the cfreelist queue.
  */
-static inline void
+static __inline void
 cblock_free(cblockp)
 	struct cblock *cblockp;
 {
@@ -121,7 +131,12 @@ cblock_alloc_cblocks(number)
 	struct cblock *cbp;
 
 	for (i = 0; i < number; ++i) {
-		cbp = malloc(sizeof *cbp, M_TTYS, M_WAITOK);
+		cbp = malloc(sizeof *cbp, M_TTYS, M_NOWAIT);
+		if (cbp == NULL) {
+			printf(
+"clist_alloc_cblocks: M_NOWAIT malloc failed, trying M_WAITOK\n");
+			cbp = malloc(sizeof *cbp, M_TTYS, M_WAITOK);
+		}
 		/*
 		 * Freed cblocks have zero quotes and garbage elsewhere.
 		 * Set the may-have-quote bit to force zeroing the quotes.
