@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)tty_tty.c	8.2 (Berkeley) 9/23/93
- * $FreeBSD: stable/2.1/sys/kern/tty_tty.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/tty_tty.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -45,11 +45,29 @@
 #include <sys/tty.h>
 #include <sys/vnode.h>
 #include <sys/file.h>
+#include <sys/kernel.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
+
+static	d_open_t	cttyopen;
+static	d_read_t	cttyread;
+static	d_write_t	cttywrite;
+static	d_ioctl_t	cttyioctl;
+static	d_select_t	cttyselect;
+
+#define CDEV_MAJOR 1
+/* Don't make static, fdesc_vnops uses this. */
+struct cdevsw ctty_cdevsw = 
+	{ cttyopen,	nullclose,	cttyread,	cttywrite,	/*1*/
+	  cttyioctl,	nullstop,	nullreset,	nodevtotty,/* tty */
+	  cttyselect,	nommap,		NULL,	"ctty",	NULL,	-1 };
+
 
 #define cttyvp(p) ((p)->p_flag & P_CONTROLT ? (p)->p_session->s_ttyvp : NULL)
 
 /*ARGSUSED*/
-int
+static	int
 cttyopen(dev, flag, mode, p)
 	dev_t dev;
 	int flag, mode;
@@ -80,7 +98,7 @@ cttyopen(dev, flag, mode, p)
 }
 
 /*ARGSUSED*/
-int
+static	int
 cttyread(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -98,7 +116,7 @@ cttyread(dev, uio, flag)
 }
 
 /*ARGSUSED*/
-int
+static	int
 cttywrite(dev, uio, flag)
 	dev_t dev;
 	struct uio *uio;
@@ -116,7 +134,7 @@ cttywrite(dev, uio, flag)
 }
 
 /*ARGSUSED*/
-int
+static	int
 cttyioctl(dev, cmd, addr, flag, p)
 	dev_t dev;
 	int cmd;
@@ -141,7 +159,7 @@ cttyioctl(dev, cmd, addr, flag, p)
 }
 
 /*ARGSUSED*/
-int
+static	int
 cttyselect(dev, flag, p)
 	dev_t dev;
 	int flag;
@@ -153,3 +171,29 @@ cttyselect(dev, flag, p)
 		return (1);	/* try operation to get EOF/failure */
 	return (VOP_SELECT(ttyvp, flag, FREAD|FWRITE, NOCRED, p));
 }
+
+static ctty_devsw_installed = 0;
+#ifdef DEVFS
+static 	void	*ctty_devfs_token;
+#endif
+
+static void
+ctty_drvinit(void *unused)
+{
+	dev_t dev;
+
+	if( ! ctty_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR,0);
+		cdevsw_add(&dev,&ctty_cdevsw,NULL);
+		ctty_devsw_installed = 1;
+#ifdef DEVFS
+		ctty_devfs_token = 
+			devfs_add_devswf(&ctty_cdevsw, 0, DV_CHR, 0, 0, 
+					0666, "tty");
+#endif
+    	}
+}
+
+SYSINIT(cttydev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,ctty_drvinit,NULL)
+
+
