@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)uipc_mbuf.c	8.2 (Berkeley) 1/4/94
- * $FreeBSD: stable/2.1/sys/kern/uipc_mbuf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/uipc_mbuf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -46,23 +46,41 @@
 #include <sys/protosw.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_kern.h>
+#include <vm/vm_extern.h>
 
-extern	vm_map_t mb_map;
-struct	mbuf *mbutl;
+static void mbinit __P((void *));
+SYSINIT(mbuf, SI_SUB_MBUF, SI_ORDER_FIRST, mbinit, NULL)
+
+struct mbuf *mbutl;
 char	*mclrefcnt;
-int	mb_map_full;
+struct mbstat mbstat;
+struct mbuf *mmbfree;
+union mcluster *mclfree;
+int	max_linkhdr;
+int	max_protohdr;
+int	max_hdr;
+int	max_datalen;
 
-void
-mbinit()
+static void	m_reclaim __P((void));
+
+/* "number of clusters of pages" */
+#define NCL_INIT	1
+
+#define NMB_INIT	16
+
+/* ARGSUSED*/
+static void
+mbinit(dummy)
+	void *dummy;
 {
 	int s;
 
-#if CLBYTES < 4096
-#define NCL_INIT	(4096/CLBYTES)
-#else
-#define NCL_INIT	1
-#endif
+	mmbfree = NULL; mclfree = NULL;
 	s = splimp();
+	if (m_mballoc(NMB_INIT, M_DONTWAIT) == 0)
+		goto bad;
 	if (m_clalloc(NCL_INIT, M_DONTWAIT) == 0)
 		goto bad;
 	splx(s);
@@ -71,6 +89,47 @@ bad:
 	panic("mbinit");
 }
 
+/*
+ * Allocate at least nmb mbufs and place on mbuf free list.
+ * Must be called at splimp.
+ */
+/* ARGSUSED */
+int
+m_mballoc(nmb, nowait)
+	register int nmb;
+	int nowait;
+{
+	register caddr_t p;
+	register int i;
+	int nbytes;
+
+	/* Once we run out of map space, it will be impossible to get
+	 * any more (nothing is ever freed back to the map) (XXX which
+	 * is dumb). (however you are not dead as m_reclaim might
+	 * still be able to free a substantial amount of space).
+	 */
+	if (mb_map_full)
+		return (0);
+
+	nbytes = round_page(nmb * MSIZE);
+	p = (caddr_t)kmem_malloc(mb_map, nbytes, nowait ? M_NOWAIT : M_WAITOK);
+	/*
+	 * Either the map is now full, or this is nowait and there
+	 * are no pages left.
+	 */
+	if (p == NULL)
+		return (0);
+
+	nmb = nbytes / MSIZE;
+	for (i = 0; i < nmb; i++) {
+		((struct mbuf *)p)->m_next = mmbfree;
+		mmbfree = (struct mbuf *)p;
+		p += MSIZE;
+	}
+	mbstat.m_mbufs += nmb;
+	return (1);
+}
+
 /*
  * Allocate some number of mbuf clusters
  * and place on cluster free list.
@@ -82,7 +141,6 @@ m_clalloc(ncl, nowait)
 	register int ncl;
 	int nowait;
 {
-	static int logged;
 	register caddr_t p;
 	register int i;
 	int npg;
@@ -95,7 +153,7 @@ m_clalloc(ncl, nowait)
 	if (mb_map_full)
 		return (0);
 
-	npg = ncl * CLSIZE;
+	npg = ncl;
 	p = (caddr_t)kmem_malloc(mb_map, ctob(npg),
 				 nowait ? M_NOWAIT : M_WAITOK);
 	/*
@@ -105,7 +163,7 @@ m_clalloc(ncl, nowait)
 	if (p == NULL)
 		return (0);
 
-	ncl = ncl * CLBYTES / MCLBYTES;
+	ncl = ncl * PAGE_SIZE / MCLBYTES;
 	for (i = 0; i < ncl; i++) {
 		((union mcluster *)p)->mcl_next = mclfree;
 		mclfree = (union mcluster *)p;
@@ -126,7 +184,11 @@ m_retry(i, t)
 {
 	register struct mbuf *m;
 
-	m_reclaim();
+	/*
+	 * Must only do the reclaim if not in an interrupt context.
+	 */
+	if (i == M_WAIT)
+		m_reclaim();
 #define m_retry(i, t)	(struct mbuf *)0
 	MGET(m, i, t);
 #undef m_retry
@@ -146,7 +208,11 @@ m_retryhdr(i, t)
 {
 	register struct mbuf *m;
 
-	m_reclaim();
+	/*
+	 * Must only do the reclaim if not in an interrupt context.
+	 */
+	if (i == M_WAIT)
+		m_reclaim();
 #define m_retryhdr(i, t) (struct mbuf *)0
 	MGETHDR(m, i, t);
 #undef m_retryhdr
@@ -157,7 +223,7 @@ m_retryhdr(i, t)
 	return (m);
 }
 
-void
+static void
 m_reclaim()
 {
 	register struct domain *dp;
@@ -272,7 +338,7 @@ m_prepend(m, len, how)
  * continuing for "len" bytes.  If len is M_COPYALL, copy to end of mbuf.
  * The wait parameter is a choice of M_WAIT/M_DONTWAIT from caller.
  */
-int MCFail;
+static int MCFail;
 
 struct mbuf *
 m_copym(m, off0, len, wait)
@@ -320,7 +386,11 @@ m_copym(m, off0, len, wait)
 		n->m_len = min(len, m->m_len - off);
 		if (m->m_flags & M_EXT) {
 			n->m_data = m->m_data + off;
-			mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+			if(!m->m_ext.ext_ref)
+				mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+			else
+				(*(m->m_ext.ext_ref))(m->m_ext.ext_buf,
+							m->m_ext.ext_size);
 			n->m_ext = m->m_ext;
 			n->m_flags |= M_EXT;
 		} else
@@ -341,6 +411,61 @@ nospace:
 	return (0);
 }
 
+/*
+ * Copy an entire packet, including header (which must be present).
+ * An optimization of the common case `m_copym(m, 0, M_COPYALL, how)'.
+ */
+struct mbuf *
+m_copypacket(m, how)
+	struct mbuf *m;
+	int how;
+{
+	struct mbuf *top, *n, *o;
+
+	MGET(n, how, m->m_type);
+	top = n;
+	if (!n)
+		goto nospace;
+
+	M_COPY_PKTHDR(n, m);
+	n->m_len = m->m_len;
+	if (m->m_flags & M_EXT) {
+		n->m_data = m->m_data;
+		mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+		n->m_ext = m->m_ext;
+		n->m_flags |= M_EXT;
+	} else {
+		bcopy(mtod(m, char *), mtod(n, char *), n->m_len);
+	}
+
+	m = m->m_next;
+	while (m) {
+		MGET(o, how, m->m_type);
+		if (!o)
+			goto nospace;
+
+		n->m_next = o;
+		n = n->m_next;
+
+		n->m_len = m->m_len;
+		if (m->m_flags & M_EXT) {
+			n->m_data = m->m_data;
+			mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+			n->m_ext = m->m_ext;
+			n->m_flags |= M_EXT;
+		} else {
+			bcopy(mtod(m, char *), mtod(n, char *), n->m_len);
+		}
+
+		m = m->m_next;
+	}
+	return top;
+nospace:
+	m_freem(top);
+	MCFail++;
+	return 0;
+}
+
 /*
  * Copy data from an mbuf chain starting "off" bytes from the beginning,
  * continuing for "len" bytes, into the indicated buffer.
@@ -484,7 +609,7 @@ m_adj(mp, req_len)
  * If there is room, it will add up to max_protohdr-len extra bytes to the
  * contiguous region in an attempt to avoid being called next time.
  */
-int MPFail;
+static int MPFail;
 
 struct mbuf *
 m_pullup(n, len)
@@ -597,7 +722,11 @@ extpacket:
 	if (m->m_flags & M_EXT) {
 		n->m_flags |= M_EXT;
 		n->m_ext = m->m_ext;
-		mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+		if(!m->m_ext.ext_ref)
+			mclrefcnt[mtocl(m->m_ext.ext_buf)]++;
+		else
+			(*(m->m_ext.ext_ref))(m->m_ext.ext_buf,
+						m->m_ext.ext_size);
 		m->m_ext.ext_size = 0; /* For Accounting XXXXXX danger */
 		n->m_data = m->m_data + len;
 	} else {
@@ -617,7 +746,7 @@ m_devget(buf, totlen, off0, ifp, copy)
 	char *buf;
 	int totlen, off0;
 	struct ifnet *ifp;
-	void (*copy)();
+	void (*copy) __P((char *from, caddr_t to, u_int len));
 {
 	register struct mbuf *m;
 	struct mbuf *top = 0, **mp = &top;
