@@ -31,11 +31,12 @@
  * SUCH DAMAGE.
  *
  *	@(#)uipc_socket2.c	8.1 (Berkeley) 6/10/93
- * $FreeBSD: stable/2.1/sys/kern/uipc_socket2.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/uipc_socket2.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/kernel.h>
 #include <sys/proc.h>
 #include <sys/file.h>
 #include <sys/buf.h>
@@ -46,22 +47,18 @@
 #include <sys/socket.h>
 #include <sys/socketvar.h>
 #include <sys/signalvar.h>
+#include <sys/sysctl.h>
 
 /*
  * Primitive routines for operating on sockets and socket buffers
  */
 
-/* strings for sleep message: */
-char	netio[] = "netio";
-char	netcon[] = "netcon";
-char	netcls[] = "netcls";
+u_long	sb_max = SB_MAX;		/* XXX should be static */
+SYSCTL_INT(_kern, KERN_MAXSOCKBUF, maxsockbuf, CTLFLAG_RW, &sb_max, 0, "")
 
-/* these two are sysctl visible */
-u_long	sb_max = SB_MAX;		/* patchable */
-
-u_long	sb_efficiency = 8;		/* parameter for sbreserve() */
-
-int	sominqueue = 0;			/* minimum queue override */
+static	u_long sb_efficiency = 8;	/* parameter for sbreserve() */
+SYSCTL_INT(_kern, OID_AUTO, sockbuf_waste_factor, CTLFLAG_RW, &sb_efficiency,
+	   0, "");
 
 /*
  * Procedures to manipulate state flags of socket
@@ -110,12 +107,16 @@ soisconnected(so)
 
 	so->so_state &= ~(SS_ISCONNECTING|SS_ISDISCONNECTING|SS_ISCONFIRMING);
 	so->so_state |= SS_ISCONNECTED;
-	if (head && soqremque(so, 0)) {
-		soqinsque(head, so, 1);
+	if (head && (so->so_state & SS_INCOMP)) {
+		TAILQ_REMOVE(&head->so_incomp, so, so_list);
+		head->so_incqlen--;
+		so->so_state &= ~SS_INCOMP;
+		TAILQ_INSERT_TAIL(&head->so_comp, so, so_list);
+		so->so_state |= SS_COMP;
 		sorwakeup(head);
-		wakeup((caddr_t)&head->so_timeo);
+		wakeup_one(&head->so_timeo);
 	} else {
-		wakeup((caddr_t)&so->so_timeo);
+		wakeup(&so->so_timeo);
 		sorwakeup(so);
 		sowwakeup(so);
 	}
@@ -145,6 +146,49 @@ soisdisconnected(so)
 	sorwakeup(so);
 }
 
+/*
+ * Return a random connection that hasn't been serviced yet and
+ * is eligible for discard.  There is a one in qlen chance that
+ * we will return a null, saying that there are no dropable
+ * requests.  In this case, the protocol specific code should drop
+ * the new request.  This insures fairness.
+ *
+ * This may be used in conjunction with protocol specific queue
+ * congestion routines.
+ */
+struct socket *
+sodropablereq(head)
+	register struct socket *head;
+{
+	register struct socket *so;
+	unsigned int i, j, qlen;
+
+	static int rnd;
+	static long old_mono_secs;
+	static unsigned int cur_cnt, old_cnt;
+
+	if ((i = (mono_time.tv_sec - old_mono_secs)) != 0) {
+		old_mono_secs = mono_time.tv_sec;
+		old_cnt = cur_cnt / i;
+		cur_cnt = 0;
+	}
+
+	so = TAILQ_FIRST(&head->so_incomp);
+	if (!so)
+		return (so);
+
+	qlen = head->so_incqlen;
+	if (++cur_cnt > qlen || old_cnt > qlen) {
+		rnd = (314159 * rnd + 66329) & 0xffff;
+		j = ((qlen + 1) * rnd) >> 16;
+
+		while (j-- && so)
+		    so = TAILQ_NEXT(so, so_list);
+	}
+
+	return (so);
+}
+
 /*
  * When an attempt at a new connection is noted on a socket
  * which accepts connections, sonewconn is called.  If the
@@ -162,15 +206,14 @@ sonewconn1(head, connstatus)
 	int connstatus;
 {
 	register struct socket *so;
-	int soqueue = connstatus ? 1 : 0;
 
-	if ((head->so_qlen + head->so_q0len > 3 * head->so_qlimit / 2) &&
-	    (head->so_qlen + head->so_q0len > sominqueue))
+	if (head->so_qlen > 3 * head->so_qlimit / 2)
 		return ((struct socket *)0);
 	MALLOC(so, struct socket *, sizeof(*so), M_SOCKET, M_DONTWAIT);
 	if (so == NULL)
 		return ((struct socket *)0);
 	bzero((caddr_t)so, sizeof(*so));
+	so->so_head = head;
 	so->so_type = head->so_type;
 	so->so_options = head->so_options &~ SO_ACCEPTCONN;
 	so->so_linger = head->so_linger;
@@ -178,11 +221,25 @@ sonewconn1(head, connstatus)
 	so->so_proto = head->so_proto;
 	so->so_timeo = head->so_timeo;
 	so->so_pgid = head->so_pgid;
+	so->so_uid = head->so_uid;
 	(void) soreserve(so, head->so_snd.sb_hiwat, head->so_rcv.sb_hiwat);
-	soqinsque(head, so, soqueue);
-	if ((*so->so_proto->pr_usrreq)(so, PRU_ATTACH,
-	    (struct mbuf *)0, (struct mbuf *)0, (struct mbuf *)0)) {
-		(void) soqremque(so, soqueue);
+	if (connstatus) {
+		TAILQ_INSERT_TAIL(&head->so_comp, so, so_list);
+		so->so_state |= SS_COMP;
+	} else {
+		TAILQ_INSERT_TAIL(&head->so_incomp, so, so_list);
+		so->so_state |= SS_INCOMP;
+		head->so_incqlen++;
+	}
+	head->so_qlen++;
+	if ((*so->so_proto->pr_usrreqs->pru_attach)(so, 0)) {
+		if (so->so_state & SS_COMP) {
+			TAILQ_REMOVE(&head->so_comp, so, so_list);
+		} else {
+			TAILQ_REMOVE(&head->so_incomp, so, so_list);
+			head->so_incqlen--;
+		}
+		head->so_qlen--;
 		(void) free((caddr_t)so, M_SOCKET);
 		return ((struct socket *)0);
 	}
@@ -194,57 +251,6 @@ sonewconn1(head, connstatus)
 	return (so);
 }
 
-void
-soqinsque(head, so, q)
-	register struct socket *head, *so;
-	int q;
-{
-
-	register struct socket **prev;
-	so->so_head = head;
-	if (q == 0) {
-		head->so_q0len++;
-		so->so_q0 = 0;
-		for (prev = &(head->so_q0); *prev; )
-			prev = &((*prev)->so_q0);
-	} else {
-		head->so_qlen++;
-		so->so_q = 0;
-		for (prev = &(head->so_q); *prev; )
-			prev = &((*prev)->so_q);
-	}
-	*prev = so;
-}
-
-int
-soqremque(so, q)
-	register struct socket *so;
-	int q;
-{
-	register struct socket *head, *prev, *next;
-
-	head = so->so_head;
-	prev = head;
-	for (;;) {
-		next = q ? prev->so_q : prev->so_q0;
-		if (next == so)
-			break;
-		if (next == 0)
-			return (0);
-		prev = next;
-	}
-	if (q == 0) {
-		prev->so_q0 = next->so_q0;
-		head->so_q0len--;
-	} else {
-		prev->so_q = next->so_q;
-		head->so_qlen--;
-	}
-	next->so_q0 = next->so_q = 0;
-	next->so_head = 0;
-	return (1);
-}
-
 /*
  * Socantsendmore indicates that no more data will be sent on the
  * socket; it would normally be applied to a socket when the user
@@ -283,7 +289,7 @@ sbwait(sb)
 
 	sb->sb_flags |= SB_WAIT;
 	return (tsleep((caddr_t)&sb->sb_cc,
-	    (sb->sb_flags & SB_NOINTR) ? PSOCK : PSOCK | PCATCH, netio,
+	    (sb->sb_flags & SB_NOINTR) ? PSOCK : PSOCK | PCATCH, "sbwait",
 	    sb->sb_timeo));
 }
 
@@ -301,7 +307,7 @@ sb_lock(sb)
 		sb->sb_flags |= SB_WANT;
 		error = tsleep((caddr_t)&sb->sb_flags,
 		    (sb->sb_flags & SB_NOINTR) ? PSOCK : PSOCK|PCATCH,
-		    netio, 0);
+		    "sblock", 0);
 		if (error)
 			return (error);
 	}
@@ -487,7 +493,7 @@ sbcheck(sb)
 	for (m = sb->sb_mb; m; m = m->m_next) {
 		len += m->m_len;
 		mbcnt += MSIZE;
-		if (m->m_flags & M_EXT)
+		if (m->m_flags & M_EXT) /*XXX*/ /* pretty sure this is bogus */
 			mbcnt += m->m_ext.ext_size;
 		if (m->m_nextpkt)
 			panic("sbcheck nextpkt");
@@ -794,3 +800,223 @@ sbdroprecord(sb)
 		} while (m);
 	}
 }
+
+/*
+ * Create a "control" mbuf containing the specified data
+ * with the specified type for presentation on a socket buffer.
+ */
+struct mbuf *
+sbcreatecontrol(p, size, type, level)
+	caddr_t p;
+	register int size;
+	int type, level;
+{
+	register struct cmsghdr *cp;
+	struct mbuf *m;
+
+	if ((m = m_get(M_DONTWAIT, MT_CONTROL)) == NULL)
+		return ((struct mbuf *) NULL);
+	cp = mtod(m, struct cmsghdr *);
+	/* XXX check size? */
+	(void)memcpy(CMSG_DATA(cp), p, size);
+	size += sizeof(*cp);
+	m->m_len = size;
+	cp->cmsg_len = size;
+	cp->cmsg_level = level;
+	cp->cmsg_type = type;
+	return (m);
+}
+
+#ifdef PRU_OLDSTYLE
+/*
+ * The following routines mediate between the old-style `pr_usrreq'
+ * protocol implementations and the new-style `struct pr_usrreqs'
+ * calling convention.
+ */
+
+/* syntactic sugar */
+#define	nomb	(struct mbuf *)0
+
+static int
+old_abort(struct socket *so)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_ABORT, nomb, nomb, nomb);
+}
+
+static int
+old_accept(struct socket *so, struct mbuf *nam)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_ACCEPT, nomb,  nam, nomb);
+}
+
+static int
+old_attach(struct socket *so, int proto)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_ATTACH, nomb,
+				       (struct mbuf *)proto, /* XXX */
+				       nomb);
+}
+
+static int
+old_bind(struct socket *so, struct mbuf *nam)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_BIND, nomb, nam, nomb);
+}
+
+static int
+old_connect(struct socket *so, struct mbuf *nam)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_CONNECT, nomb, nam, nomb);
+}
+
+static int
+old_connect2(struct socket *so1, struct socket *so2)
+{
+	return so1->so_proto->pr_ousrreq(so1, PRU_CONNECT2, nomb, 
+				       (struct mbuf *)so2, nomb);
+}
+
+static int
+old_control(struct socket *so, int cmd, caddr_t data, struct ifnet *ifp)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_CONTROL, (struct mbuf *)cmd, 
+				       (struct mbuf *)data, 
+				       (struct mbuf *)ifp);
+}
+
+static int
+old_detach(struct socket *so)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_DETACH, nomb, nomb, nomb);
+}
+
+static int
+old_disconnect(struct socket *so)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_DISCONNECT, nomb, nomb, nomb);
+}
+
+static int
+old_listen(struct socket *so)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_LISTEN, nomb, nomb, nomb);
+}
+
+static int
+old_peeraddr(struct socket *so, struct mbuf *nam)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_PEERADDR, nomb, nam, nomb);
+}
+
+static int
+old_rcvd(struct socket *so, int flags)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_RCVD, nomb,
+				       (struct mbuf *)flags, /* XXX */
+				       nomb);
+}
+
+static int
+old_rcvoob(struct socket *so, struct mbuf *m, int flags)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_RCVOOB, m,
+				       (struct mbuf *)flags, /* XXX */
+				       nomb);
+}
+
+static int
+old_send(struct socket *so, int flags, struct mbuf *m, struct mbuf *addr,
+	 struct mbuf *control)
+{
+	int req;
+
+	if (flags & PRUS_OOB) {
+		req = PRU_SENDOOB;
+	} else if(flags & PRUS_EOF) {
+		req = PRU_SEND_EOF;
+	} else {
+		req = PRU_SEND;
+	}
+	return so->so_proto->pr_ousrreq(so, req, m, addr, control);
+}
+
+static int
+old_sense(struct socket *so, struct stat *sb)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_SENSE, (struct mbuf *)sb,
+				       nomb, nomb);
+}
+
+static int
+old_shutdown(struct socket *so)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_SHUTDOWN, nomb, nomb, nomb);
+}
+
+static int
+old_sockaddr(struct socket *so, struct mbuf *nam)
+{
+	return so->so_proto->pr_ousrreq(so, PRU_SOCKADDR, nomb, nam, nomb);
+}
+
+struct pr_usrreqs pru_oldstyle = {
+	old_abort, old_accept, old_attach, old_bind, old_connect,
+	old_connect2, old_control, old_detach, old_disconnect,
+	old_listen, old_peeraddr, old_rcvd, old_rcvoob, old_send,
+	old_sense, old_shutdown, old_sockaddr
+};
+
+#endif /* PRU_OLDSTYLE */
+
+/*
+ * Some routines that return EOPNOTSUPP for entry points that are not
+ * supported by a protocol.  Fill in as needed.
+ */
+int
+pru_accept_notsupp(struct socket *so, struct mbuf *nam)
+{
+	return EOPNOTSUPP;
+}
+
+int
+pru_connect2_notsupp(struct socket *so1, struct socket *so2)
+{
+	return EOPNOTSUPP;
+}
+
+int
+pru_control_notsupp(struct socket *so, int cmd, caddr_t data,
+		    struct ifnet *ifp)
+{
+	return EOPNOTSUPP;
+}
+
+int
+pru_listen_notsupp(struct socket *so)
+{
+	return EOPNOTSUPP;
+}
+
+int
+pru_rcvd_notsupp(struct socket *so, int flags)
+{
+	return EOPNOTSUPP;
+}
+
+int
+pru_rcvoob_notsupp(struct socket *so, struct mbuf *m, int flags)
+{
+	return EOPNOTSUPP;
+}
+
+/*
+ * This isn't really a ``null'' operation, but it's the default one
+ * and doesn't do anything destructive.
+ */
+int
+pru_sense_null(struct socket *so, struct stat *sb)
+{
+	sb->st_blksize = so->so_snd.sb_hiwat;
+	return 0;
+}
+
