@@ -31,11 +31,14 @@
  * SUCH DAMAGE.
  *
  *	@(#)uipc_syscalls.c	8.4 (Berkeley) 2/21/94
- * $FreeBSD: stable/2.1/sys/kern/uipc_syscalls.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/uipc_syscalls.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#include "opt_ktrace.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/filedesc.h>
 #include <sys/proc.h>
 #include <sys/file.h>
@@ -52,6 +55,18 @@
 #include <sys/ktrace.h>
 #endif
 
+extern int sendit __P((struct proc *p, int s, struct msghdr *mp, int flags,
+		       int *retsize));
+extern int recvit __P((struct proc *p, int s, struct msghdr *mp,
+		       caddr_t namelenp, int *retsize));
+  
+static int accept1 __P((struct proc *p, struct accept_args *uap, int *retval,
+			int compat));
+static int getsockname1 __P((struct proc *p, struct getsockname_args *uap,
+			     int *retval, int compat));
+static int getpeername1 __P((struct proc *p, struct getpeername_args *uap,
+			     int *retval, int compat));
+
 /*
  * System call interface to the socket abstraction.
  */
@@ -61,15 +76,14 @@
 
 extern	struct fileops socketops;
 
-struct socket_args {
-	int	domain;
-	int	type;
-	int	protocol;
-};
 int
 socket(p, uap, retval)
 	struct proc *p;
-	register struct socket_args *uap;
+	register struct socket_args /* {
+		int	domain;
+		int	type;
+		int	protocol;
+	} */ *uap;
 	int *retval;
 {
 	struct filedesc *fdp = p->p_fd;
@@ -83,7 +97,7 @@ socket(p, uap, retval)
 	fp->f_flag = FREAD|FWRITE;
 	fp->f_type = DTYPE_SOCKET;
 	fp->f_ops = &socketops;
-	error = socreate(uap->domain, &so, uap->type, uap->protocol);
+	error = socreate(uap->domain, &so, uap->type, uap->protocol, p);
 	if (error) {
 		fdp->fd_ofiles[fd] = 0;
 		ffree(fp);
@@ -94,16 +108,15 @@ socket(p, uap, retval)
 	return (error);
 }
 
-struct bind_args {
-	int	s;
-	caddr_t	name;
-	int	namelen;
-};
 /* ARGSUSED */
 int
 bind(p, uap, retval)
 	struct proc *p;
-	register struct bind_args *uap;
+	register struct bind_args /* {
+		int	s;
+		caddr_t	name;
+		int	namelen;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -121,15 +134,14 @@ bind(p, uap, retval)
 	return (error);
 }
 
-struct listen_args {
-	int	s;
-	int	backlog;
-};
 /* ARGSUSED */
 int
 listen(p, uap, retval)
 	struct proc *p;
-	register struct listen_args *uap;
+	register struct listen_args /* {
+		int	s;
+		int	backlog;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -141,29 +153,22 @@ listen(p, uap, retval)
 	return (solisten((struct socket *)fp->f_data, uap->backlog));
 }
 
-struct accept_args {
-	int	s;
-	caddr_t	name;
-	int	*anamelen;
-#ifdef COMPAT_OLDSOCK
-	int	compat_43;	/* pseudo */
-#endif
-};
-
-#ifndef COMPAT_OLDSOCK
-#  define	accept1	accept
-#endif  /* COMPAT_OLDSOCK*/
-
-int
-accept1(p, uap, retval)
+static int
+accept1(p, uap, retval, compat)
 	struct proc *p;
-	register struct accept_args *uap;
+	register struct accept_args /* {
+		int	s;
+		caddr_t	name;
+		int	*anamelen;
+	} */ *uap;
 	int *retval;
+	int compat;
 {
 	struct file *fp;
 	struct mbuf *nam;
 	int namelen, error, s;
-	register struct socket *so;
+	struct socket *head, *so;
+	short fflag;		/* type must match fp->f_flag */
 
 	if (uap->name) {
 		error = copyin((caddr_t)uap->anamelen, (caddr_t)&namelen,
@@ -175,57 +180,79 @@ accept1(p, uap, retval)
 	if (error)
 		return (error);
 	s = splnet();
-	so = (struct socket *)fp->f_data;
-	if ((so->so_options & SO_ACCEPTCONN) == 0) {
+	head = (struct socket *)fp->f_data;
+	if ((head->so_options & SO_ACCEPTCONN) == 0) {
 		splx(s);
 		return (EINVAL);
 	}
-	if ((so->so_state & SS_NBIO) && so->so_qlen == 0) {
+	if ((head->so_state & SS_NBIO) && head->so_comp.tqh_first == NULL) {
 		splx(s);
 		return (EWOULDBLOCK);
 	}
-	while (so->so_qlen == 0 && so->so_error == 0) {
-		if (so->so_state & SS_CANTRCVMORE) {
-			so->so_error = ECONNABORTED;
+	while (head->so_comp.tqh_first == NULL && head->so_error == 0) {
+		if (head->so_state & SS_CANTRCVMORE) {
+			head->so_error = ECONNABORTED;
 			break;
 		}
-		error = tsleep((caddr_t)&so->so_timeo, PSOCK | PCATCH,
-		    netcon, 0);
+		error = tsleep((caddr_t)&head->so_timeo, PSOCK | PCATCH,
+		    "accept", 0);
 		if (error) {
 			splx(s);
 			return (error);
 		}
 	}
-	if (so->so_error) {
-		error = so->so_error;
-		so->so_error = 0;
+	if (head->so_error) {
+		error = head->so_error;
+		head->so_error = 0;
 		splx(s);
 		return (error);
 	}
+
+	/*
+	 * At this point we know that there is at least one connection
+	 * ready to be accepted. Remove it from the queue prior to
+	 * allocating the file descriptor for it since falloc() may
+	 * block allowing another process to accept the connection
+	 * instead.
+	 */
+	so = head->so_comp.tqh_first;
+	TAILQ_REMOVE(&head->so_comp, so, so_list);
+	head->so_qlen--;
+
+	fflag = fp->f_flag;
 	error = falloc(p, &fp, retval);
 	if (error) {
+		/*
+		 * Probably ran out of file descriptors. Put the
+		 * unaccepted connection back onto the queue and
+		 * do another wakeup so some other process might
+		 * have a chance at it.
+		 */
+		TAILQ_INSERT_HEAD(&head->so_comp, so, so_list);
+		head->so_qlen++;
+		wakeup_one(&head->so_timeo);
 		splx(s);
 		return (error);
 	}
-	{ struct socket *aso = so->so_q;
-	  if (soqremque(aso, 1) == 0)
-		panic("accept");
-	  so = aso;
-	}
+
+	so->so_state &= ~SS_COMP;
+	so->so_head = NULL;
+
 	fp->f_type = DTYPE_SOCKET;
-	fp->f_flag = FREAD|FWRITE;
+	fp->f_flag = fflag;
 	fp->f_ops = &socketops;
 	fp->f_data = (caddr_t)so;
 	nam = m_get(M_WAIT, MT_SONAME);
 	(void) soaccept(so, nam);
 	if (uap->name) {
+		 /* check length before it is destroyed */
+		if (namelen > nam->m_len)
+			namelen = nam->m_len;
 #ifdef COMPAT_OLDSOCK
-		if (uap->compat_43)
+		if (compat)
 			mtod(nam, struct osockaddr *)->sa_family =
 			    mtod(nam, struct sockaddr *)->sa_family;
 #endif
-		if (namelen > nam->m_len)
-			namelen = nam->m_len;
 		/* SHOULD COPY OUT A CHAIN HERE */
 		error = copyout(mtod(nam, caddr_t), (caddr_t)uap->name,
 		    (u_int)namelen);
@@ -238,7 +265,6 @@ accept1(p, uap, retval)
 	return (error);
 }
 
-#ifdef COMPAT_OLDSOCK
 int
 accept(p, uap, retval)
 	struct proc *p;
@@ -246,10 +272,10 @@ accept(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat_43 = 0;
-	return (accept1(p, uap, retval));
+	return (accept1(p, uap, retval, 0));
 }
 
+#ifdef COMPAT_OLDSOCK
 int
 oaccept(p, uap, retval)
 	struct proc *p;
@@ -257,21 +283,19 @@ oaccept(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat_43 = 1;
-	return (accept1(p, uap, retval));
+	return (accept1(p, uap, retval, 1));
 }
 #endif /* COMPAT_OLDSOCK */
 
-struct connect_args {
-	int	s;
-	caddr_t	name;
-	int	namelen;
-};
 /* ARGSUSED */
 int
 connect(p, uap, retval)
 	struct proc *p;
-	register struct connect_args *uap;
+	register struct connect_args /* {
+		int	s;
+		caddr_t	name;
+		int	namelen;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -298,7 +322,7 @@ connect(p, uap, retval)
 	s = splnet();
 	while ((so->so_state & SS_ISCONNECTING) && so->so_error == 0) {
 		error = tsleep((caddr_t)&so->so_timeo, PSOCK | PCATCH,
-		    netcon, 0);
+		    "connec", 0);
 		if (error)
 			break;
 	}
@@ -315,16 +339,15 @@ bad:
 	return (error);
 }
 
-struct socketpair_args {
-	int	domain;
-	int	type;
-	int	protocol;
-	int	*rsv;
-};
 int
 socketpair(p, uap, retval)
 	struct proc *p;
-	register struct socketpair_args *uap;
+	register struct socketpair_args /* {
+		int	domain;
+		int	type;
+		int	protocol;
+		int	*rsv;
+	} */ *uap;
 	int retval[];
 {
 	register struct filedesc *fdp = p->p_fd;
@@ -332,10 +355,10 @@ socketpair(p, uap, retval)
 	struct socket *so1, *so2;
 	int fd, error, sv[2];
 
-	error = socreate(uap->domain, &so1, uap->type, uap->protocol);
+	error = socreate(uap->domain, &so1, uap->type, uap->protocol, p);
 	if (error)
 		return (error);
-	error = socreate(uap->domain, &so2, uap->type, uap->protocol);
+	error = socreate(uap->domain, &so2, uap->type, uap->protocol, p);
 	if (error)
 		goto free1;
 	error = falloc(p, &fp1, &fd);
@@ -366,8 +389,10 @@ socketpair(p, uap, retval)
 			goto free4;
 	}
 	error = copyout((caddr_t)sv, (caddr_t)uap->rsv, 2 * sizeof (int));
+#if 0	/* old pipe(2) syscall compatability, unused these days */
 	retval[0] = sv[0];		/* XXX ??? */
 	retval[1] = sv[1];		/* XXX ??? */
+#endif
 	return (error);
 free4:
 	ffree(fp2);
@@ -485,18 +510,17 @@ bad:
 	return (error);
 }
 
-struct sendto_args {
-	int	s;
-	caddr_t	buf;
-	size_t	len;
-	int	flags;
-	caddr_t	to;
-	int	tolen;
-};
 int
 sendto(p, uap, retval)
 	struct proc *p;
-	register struct sendto_args *uap;
+	register struct sendto_args /* {
+		int	s;
+		caddr_t	buf;
+		size_t	len;
+		int	flags;
+		caddr_t	to;
+		int	tolen;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -516,16 +540,15 @@ sendto(p, uap, retval)
 }
 
 #ifdef COMPAT_OLDSOCK
-struct osend_args {
-	int	s;
-	caddr_t	buf;
-	int	len;
-	int	flags;
-};
 int
 osend(p, uap, retval)
 	struct proc *p;
-	register struct osend_args *uap;
+	register struct osend_args /* {
+		int	s;
+		caddr_t	buf;
+		int	len;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -542,15 +565,14 @@ osend(p, uap, retval)
 	return (sendit(p, uap->s, &msg, uap->flags, retval));
 }
 
-struct osendmsg_args {
-	int	s;
-	caddr_t	msg;
-	int	flags;
-};
 int
 osendmsg(p, uap, retval)
 	struct proc *p;
-	register struct osendmsg_args *uap;
+	register struct osendmsg_args /* {
+		int	s;
+		caddr_t	msg;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -582,15 +604,14 @@ done:
 }
 #endif
 
-struct sendmsg_args {
-	int	s;
-	caddr_t	msg;
-	int	flags;
-};
 int
 sendmsg(p, uap, retval)
 	struct proc *p;
-	register struct sendmsg_args *uap;
+	register struct sendmsg_args /* {
+		int	s;
+		caddr_t	msg;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -623,15 +644,6 @@ done:
 	return (error);
 }
 
-struct recvfrom_args {
-	int	s;
-	caddr_t	buf;
-	size_t	len;
-	int	flags;
-	caddr_t	from;
-	int	*fromlenaddr;
-};
-
 int
 recvit(p, s, mp, namelenp, retsize)
 	register struct proc *p;
@@ -645,7 +657,8 @@ recvit(p, s, mp, namelenp, retsize)
 	register struct iovec *iov;
 	register int i;
 	int len, error;
-	struct mbuf *from = 0, *control = 0;
+	struct mbuf *m, *from = 0, *control = 0;
+	caddr_t ctlbuf;
 #ifdef KTRACE
 	struct iovec *ktriov = NULL;
 #endif
@@ -698,14 +711,15 @@ recvit(p, s, mp, namelenp, retsize)
 		if (len <= 0 || from == 0)
 			len = 0;
 		else {
+			/* save sa_len before it is destroyed by MSG_COMPAT */
+			if (len > from->m_len)
+				len = from->m_len;
+			/* else if len < from->m_len ??? */
 #ifdef COMPAT_OLDSOCK
 			if (mp->msg_flags & MSG_COMPAT)
 				mtod(from, struct osockaddr *)->sa_family =
 				    mtod(from, struct sockaddr *)->sa_family;
 #endif
-			if (len > from->m_len)
-				len = from->m_len;
-			/* else if len < from->m_len ??? */
 			error = copyout(mtod(from, caddr_t),
 			    (caddr_t)mp->msg_name, (unsigned)len);
 			if (error)
@@ -744,17 +758,29 @@ recvit(p, s, mp, namelenp, retsize)
 		}
 #endif
 		len = mp->msg_controllen;
-		if (len <= 0 || control == 0)
-			len = 0;
-		else {
-			if (len >= control->m_len)
-				len = control->m_len;
-			else
+		m = control;
+		mp->msg_controllen = 0;
+		ctlbuf = (caddr_t) mp->msg_control;
+
+		while (m && len > 0) {
+			unsigned int tocopy;
+
+			if (len >= m->m_len) 
+				tocopy = m->m_len;
+			else {
 				mp->msg_flags |= MSG_CTRUNC;
-			error = copyout((caddr_t)mtod(control, caddr_t),
-			    (caddr_t)mp->msg_control, (unsigned)len);
+				tocopy = len;
+			}
+		
+			if (error = copyout((caddr_t)mtod(m, caddr_t),
+					ctlbuf, tocopy))
+				goto out;
+
+			ctlbuf += tocopy;
+			len -= tocopy;
+			m = m->m_next;
 		}
-		mp->msg_controllen = len;
+		mp->msg_controllen = ctlbuf - mp->msg_control;
 	}
 out:
 	if (from)
@@ -764,15 +790,17 @@ out:
 	return (error);
 }
 
-struct shutdown_args {
-	int	s;
-	int	how;
-};
-
 int
 recvfrom(p, uap, retval)
 	struct proc *p;
-	register struct recvfrom_args *uap;
+	register struct recvfrom_args /* {
+		int	s;
+		caddr_t	buf;
+		size_t	len;
+		int	flags;
+		caddr_t	from;
+		int	*fromlenaddr;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -811,16 +839,15 @@ orecvfrom(p, uap, retval)
 
 
 #ifdef COMPAT_OLDSOCK
-struct orecv_args {
-	int	s;
-	caddr_t	buf;
-	int	len;
-	int	flags;
-};
 int
 orecv(p, uap, retval)
 	struct proc *p;
-	register struct orecv_args *uap;
+	register struct orecv_args /* {
+		int	s;
+		caddr_t	buf;
+		int	len;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -842,15 +869,14 @@ orecv(p, uap, retval)
  * overlays the new one, missing only the flags, and with the (old) access
  * rights where the control fields are now.
  */
-struct orecvmsg_args {
-	int	s;
-	struct	omsghdr *msg;
-	int	flags;
-};
 int
 orecvmsg(p, uap, retval)
 	struct proc *p;
-	register struct orecvmsg_args *uap;
+	register struct orecvmsg_args /* {
+		int	s;
+		struct	omsghdr *msg;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -887,15 +913,14 @@ done:
 }
 #endif
 
-struct recvmsg_args {
-	int	s;
-	struct	msghdr *msg;
-	int	flags;
-};
 int
 recvmsg(p, uap, retval)
 	struct proc *p;
-	register struct recvmsg_args *uap;
+	register struct recvmsg_args /* {
+		int	s;
+		struct	msghdr *msg;
+		int	flags;
+	} */ *uap;
 	int *retval;
 {
 	struct msghdr msg;
@@ -934,11 +959,15 @@ done:
 		FREE(iov, M_IOV);
 	return (error);
 }
+
 /* ARGSUSED */
 int
 shutdown(p, uap, retval)
 	struct proc *p;
-	register struct shutdown_args *uap;
+	register struct shutdown_args /* {
+		int	s;
+		int	how;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -950,18 +979,17 @@ shutdown(p, uap, retval)
 	return (soshutdown((struct socket *)fp->f_data, uap->how));
 }
 
-struct setsockopt_args {
-	int	s;
-	int	level;
-	int	name;
-	caddr_t	val;
-	int	valsize;
-};
 /* ARGSUSED */
 int
 setsockopt(p, uap, retval)
 	struct proc *p;
-	register struct setsockopt_args *uap;
+	register struct setsockopt_args /* {
+		int	s;
+		int	level;
+		int	name;
+		caddr_t	val;
+		int	valsize;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -988,18 +1016,17 @@ setsockopt(p, uap, retval)
 	    uap->name, m));
 }
 
-struct getsockopt_args {
-	int	s;
-	int	level;
-	int	name;
-	caddr_t	val;
-	int	*avalsize;
-};
 /* ARGSUSED */
 int
 getsockopt(p, uap, retval)
 	struct proc *p;
-	register struct getsockopt_args *uap;
+	register struct getsockopt_args /* {
+		int	s;
+		int	level;
+		int	name;
+		caddr_t	val;
+		int	*avalsize;
+	} */ *uap;
 	int *retval;
 {
 	struct file *fp;
@@ -1037,14 +1064,14 @@ getsockopt(p, uap, retval)
 	return (error);
 }
 
-struct pipe_args {
-	int	dummy;
-};
+#ifdef OLD_PIPE
 /* ARGSUSED */
 int
 pipe(p, uap, retval)
 	struct proc *p;
-	struct pipe_args *uap;
+	struct pipe_args /* {
+		int	dummy;
+	} */ *uap;
 	int retval[];
 {
 	register struct filedesc *fdp = p->p_fd;
@@ -1052,24 +1079,24 @@ pipe(p, uap, retval)
 	struct socket *rso, *wso;
 	int fd, error;
 
-	error = socreate(AF_UNIX, &rso, SOCK_STREAM, 0);
+	error = socreate(AF_UNIX, &rso, SOCK_STREAM, 0, p);
 	if (error)
 		return (error);
-	error = socreate(AF_UNIX, &wso, SOCK_STREAM, 0);
+	error = socreate(AF_UNIX, &wso, SOCK_STREAM, 0, p);
 	if (error)
 		goto free1;
 	error = falloc(p, &rf, &fd);
 	if (error)
 		goto free2;
 	retval[0] = fd;
-	rf->f_flag = FREAD;
+	rf->f_flag = FREAD | FWRITE;
 	rf->f_type = DTYPE_SOCKET;
 	rf->f_ops = &socketops;
 	rf->f_data = (caddr_t)rso;
 	error = falloc(p, &wf, &fd);
 	if (error)
 		goto free3;
-	wf->f_flag = FWRITE;
+	wf->f_flag = FREAD | FWRITE;
 	wf->f_type = DTYPE_SOCKET;
 	wf->f_ops = &socketops;
 	wf->f_data = (caddr_t)wso;
@@ -1090,29 +1117,21 @@ free1:
 	(void)soclose(rso);
 	return (error);
 }
-
+#endif
 /*
  * Get socket name.
  */
-struct getsockname_args {
-	int	fdes;
-	caddr_t	asa;
-	int	*alen;
-#ifdef COMPAT_OLDSOCK
-	int	compat_43;	/* pseudo */
-#endif
-};
-
-#ifndef COMPAT_OLDSOCK
-#define	getsockname1	getsockname
-#endif
-
 /* ARGSUSED */
-int
-getsockname1(p, uap, retval)
+static int
+getsockname1(p, uap, retval, compat)
 	struct proc *p;
-	register struct getsockname_args *uap;
+	register struct getsockname_args /* {
+		int	fdes;
+		caddr_t	asa;
+		int	*alen;
+	} */ *uap;
 	int *retval;
+	int compat;
 {
 	struct file *fp;
 	register struct socket *so;
@@ -1129,13 +1148,13 @@ getsockname1(p, uap, retval)
 	m = m_getclr(M_WAIT, MT_SONAME);
 	if (m == NULL)
 		return (ENOBUFS);
-	error = (*so->so_proto->pr_usrreq)(so, PRU_SOCKADDR, 0, m, 0);
+	error = (*so->so_proto->pr_usrreqs->pru_sockaddr)(so, m);
 	if (error)
 		goto bad;
 	if (len > m->m_len)
 		len = m->m_len;
 #ifdef COMPAT_OLDSOCK
-	if (uap->compat_43)
+	if (compat)
 		mtod(m, struct osockaddr *)->sa_family =
 		    mtod(m, struct sockaddr *)->sa_family;
 #endif
@@ -1148,7 +1167,6 @@ bad:
 	return (error);
 }
 
-#ifdef COMPAT_OLDSOCK
 int
 getsockname(p, uap, retval)
 	struct proc *p;
@@ -1156,10 +1174,10 @@ getsockname(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat_43 = 0;
-	return (getsockname1(p, uap, retval));
+	return (getsockname1(p, uap, retval, 0));
 }
 
+#ifdef COMPAT_OLDSOCK
 int
 ogetsockname(p, uap, retval)
 	struct proc *p;
@@ -1167,34 +1185,24 @@ ogetsockname(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat_43 = 1;
-	return (getsockname1(p, uap, retval));
+	return (getsockname1(p, uap, retval, 1));
 }
 #endif /* COMPAT_OLDSOCK */
 
 /*
  * Get name of peer for connected socket.
  */
-struct getpeername_args {
-	int	fdes;
-	caddr_t	asa;
-	int	*alen;
-#ifdef COMPAT_OLDSOCK
-	int	compat_43;	/* pseudo */
-#endif
-};
-
-
-#ifndef COMPAT_OLDSOCK
-#define	getpeername1	getpeername
-#endif
-
 /* ARGSUSED */
-int
-getpeername1(p, uap, retval)
+static int
+getpeername1(p, uap, retval, compat)
 	struct proc *p;
-	register struct getpeername_args *uap;
+	register struct getpeername_args /* {
+		int	fdes;
+		caddr_t	asa;
+		int	*alen;
+	} */ *uap;
 	int *retval;
+	int compat;
 {
 	struct file *fp;
 	register struct socket *so;
@@ -1213,13 +1221,13 @@ getpeername1(p, uap, retval)
 	m = m_getclr(M_WAIT, MT_SONAME);
 	if (m == NULL)
 		return (ENOBUFS);
-	error = (*so->so_proto->pr_usrreq)(so, PRU_PEERADDR, 0, m, 0);
+	error = (*so->so_proto->pr_usrreqs->pru_peeraddr)(so, m);
 	if (error)
 		goto bad;
 	if (len > m->m_len)
 		len = m->m_len;
 #ifdef COMPAT_OLDSOCK
-	if (uap->compat_43)
+	if (compat)
 		mtod(m, struct osockaddr *)->sa_family =
 		    mtod(m, struct sockaddr *)->sa_family;
 #endif
@@ -1232,7 +1240,6 @@ bad:
 	return (error);
 }
 
-#ifdef COMPAT_OLDSOCK
 int
 getpeername(p, uap, retval)
 	struct proc *p;
@@ -1240,21 +1247,22 @@ getpeername(p, uap, retval)
 	int *retval;
 {
 
-	uap->compat_43 = 0;
-	return (getpeername1(p, uap, retval));
+	return (getpeername1(p, uap, retval, 0));
 }
 
+#ifdef COMPAT_OLDSOCK
 int
 ogetpeername(p, uap, retval)
 	struct proc *p;
-	struct getpeername_args *uap;
+	struct ogetpeername_args *uap;
 	int *retval;
 {
 
-	uap->compat_43 = 1;
-	return (getpeername1(p, uap, retval));
+	/* XXX uap should have type `getpeername_args *' to begin with. */
+	return (getpeername1(p, (struct getpeername_args *)uap, retval, 1));
 }
 #endif /* COMPAT_OLDSOCK */
+
 int
 sockargs(mp, buf, buflen, type)
 	struct mbuf **mp;
