@@ -31,10 +31,11 @@
  * SUCH DAMAGE.
  *
  *	From: @(#)uipc_usrreq.c	8.3 (Berkeley) 1/4/94
- * $FreeBSD: stable/2.1/sys/kern/uipc_usrreq.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/uipc_usrreq.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
+#include <sys/queue.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
 #include <sys/filedesc.h>
@@ -59,8 +60,22 @@
  *	rethink name space problems
  *	need a proper out-of-band
  */
-struct	sockaddr sun_noname = { sizeof(sun_noname), AF_LOCAL };
-ino_t	unp_ino;			/* prototype for fake inode numbers */
+static struct	sockaddr sun_noname = { sizeof(sun_noname), AF_LOCAL };
+static ino_t	unp_ino;		/* prototype for fake inode numbers */
+
+static int     unp_attach __P((struct socket *));
+static void    unp_detach __P((struct unpcb *));
+static int     unp_bind __P((struct unpcb *,struct mbuf *, struct proc *));
+static int     unp_connect __P((struct socket *,struct mbuf *, struct proc *));
+static void    unp_disconnect __P((struct unpcb *));
+static void    unp_shutdown __P((struct unpcb *));
+static void    unp_drop __P((struct unpcb *, int));
+static void    unp_gc __P((void));
+static void    unp_scan __P((struct mbuf *, void (*)(struct file *)));
+static void    unp_mark __P((struct file *));
+static void    unp_discard __P((struct file *));
+static int     unp_internalize __P((struct mbuf *, struct proc *));
+
 
 /*ARGSUSED*/
 int
@@ -333,15 +348,17 @@ release:
  * and don't really want to reserve the sendspace.  Their recvspace should
  * be large enough for at least one max-size datagram plus address.
  */
-#define	PIPSIZ	4096
-u_long	unpst_sendspace = PIPSIZ;
-u_long	unpst_recvspace = PIPSIZ;
-u_long	unpdg_sendspace = 2*1024;	/* really max datagram size */
-u_long	unpdg_recvspace = 4*1024;
+#ifndef PIPSIZ
+#define	PIPSIZ	8192
+#endif
+static u_long	unpst_sendspace = PIPSIZ;
+static u_long	unpst_recvspace = PIPSIZ;
+static u_long	unpdg_sendspace = 2*1024;	/* really max datagram size */
+static u_long	unpdg_recvspace = 4*1024;
 
-int	unp_rights;			/* file descriptors in flight */
+static int	unp_rights;			/* file descriptors in flight */
 
-int
+static int
 unp_attach(so)
 	struct socket *so;
 {
@@ -375,7 +392,7 @@ unp_attach(so)
 	return (0);
 }
 
-void
+static void
 unp_detach(unp)
 	register struct unpcb *unp;
 {
@@ -406,7 +423,7 @@ unp_detach(unp)
 	(void) m_free(dtom(unp));
 }
 
-int
+static int
 unp_bind(unp, nam, p)
 	struct unpcb *unp;
 	struct mbuf *nam;
@@ -443,7 +460,7 @@ unp_bind(unp, nam, p)
 	}
 	VATTR_NULL(&vattr);
 	vattr.va_type = VSOCK;
-	vattr.va_mode = ACCESSPERMS;
+	vattr.va_mode = (ACCESSPERMS & ~p->p_fd->fd_cmask);
 	LEASE_CHECK(nd.ni_dvp, p, p->p_ucred, LEASE_WRITE);
 	error = VOP_CREATE(nd.ni_dvp, &nd.ni_vp, &nd.ni_cnd, &vattr);
 	if (error)
@@ -456,7 +473,7 @@ unp_bind(unp, nam, p)
 	return (0);
 }
 
-int
+static int
 unp_connect(so, nam, p)
 	struct socket *so;
 	struct mbuf *nam;
@@ -546,7 +563,7 @@ unp_connect2(so, so2)
 	return (0);
 }
 
-void
+static void
 unp_disconnect(unp)
 	struct unpcb *unp;
 {
@@ -593,7 +610,7 @@ unp_abort(unp)
 }
 #endif
 
-void
+static void
 unp_shutdown(unp)
 	struct unpcb *unp;
 {
@@ -604,7 +621,7 @@ unp_shutdown(unp)
 		socantrcvmore(so);
 }
 
-void
+static void
 unp_drop(unp, errno)
 	struct unpcb *unp;
 	int errno;
@@ -641,6 +658,9 @@ unp_externalize(rights)
 	int newfds = (cm->cmsg_len - sizeof(*cm)) / sizeof (int);
 	int f;
 
+	/*
+	 * if the new FD's will not fit, then we free them all
+	 */
 	if (!fdavail(p, newfds)) {
 		for (i = 0; i < newfds; i++) {
 			fp = *rp;
@@ -649,6 +669,12 @@ unp_externalize(rights)
 		}
 		return (EMSGSIZE);
 	}
+	/*
+	 * now change each pointer to an fd in the global table to 
+	 * an integer that is the index to the local fd table entry
+	 * that we set up to point to the global one we are transferring.
+	 * XXX this assumes a pointer and int are the same size...!
+	 */
 	for (i = 0; i < newfds; i++) {
 		if (fdalloc(p, 0, &f))
 			panic("unp_externalize");
@@ -661,7 +687,7 @@ unp_externalize(rights)
 	return (0);
 }
 
-int
+static int
 unp_internalize(control, p)
 	struct mbuf *control;
 	struct proc *p;
@@ -677,6 +703,10 @@ unp_internalize(control, p)
 	    cm->cmsg_len != control->m_len)
 		return (EINVAL);
 	oldfds = (cm->cmsg_len - sizeof (*cm)) / sizeof (int);
+	/*
+	 * check that all the FDs passed in refer to legal OPEN files
+	 * If not, reject the entire operation.
+	 */
 	rp = (struct file **)(cm + 1);
 	for (i = 0; i < oldfds; i++) {
 		fd = *(int *)rp++;
@@ -684,6 +714,11 @@ unp_internalize(control, p)
 		    fdp->fd_ofiles[fd] == NULL)
 			return (EBADF);
 	}
+	/*
+	 * Now replace the integer FDs with pointers to
+	 * the associated global file table entry..
+	 * XXX this assumes a pointer and an int are the same size!
+	 */
 	rp = (struct file **)(cm + 1);
 	for (i = 0; i < oldfds; i++) {
 		fp = fdp->fd_ofiles[*(int *)rp];
@@ -695,10 +730,9 @@ unp_internalize(control, p)
 	return (0);
 }
 
-int	unp_defer, unp_gcing;
-extern	struct domain localdomain;
+static int	unp_defer, unp_gcing;
 
-void
+static void
 unp_gc()
 {
 	register struct file *fp, *nextfp;
@@ -710,22 +744,52 @@ unp_gc()
 		return;
 	unp_gcing = 1;
 	unp_defer = 0;
-	for (fp = filehead; fp; fp = fp->f_filef)
+	/* 
+	 * before going through all this, set all FDs to 
+	 * be NOT defered and NOT externally accessible
+	 */
+	for (fp = filehead.lh_first; fp != 0; fp = fp->f_list.le_next)
 		fp->f_flag &= ~(FMARK|FDEFER);
 	do {
-		for (fp = filehead; fp; fp = fp->f_filef) {
+		for (fp = filehead.lh_first; fp != 0; fp = fp->f_list.le_next) {
+			/*
+			 * If the file is not open, skip it
+			 */
 			if (fp->f_count == 0)
 				continue;
+			/*
+			 * If we already marked it as 'defer'  in a
+			 * previous pass, then try process it this time
+			 * and un-mark it
+			 */
 			if (fp->f_flag & FDEFER) {
 				fp->f_flag &= ~FDEFER;
 				unp_defer--;
 			} else {
+				/*
+				 * if it's not defered, then check if it's
+				 * already marked.. if so skip it
+				 */
 				if (fp->f_flag & FMARK)
 					continue;
+				/* 
+				 * If all references are from messages
+				 * in transit, then skip it. it's not 
+				 * externally accessible.
+				 */ 
 				if (fp->f_count == fp->f_msgcount)
 					continue;
+				/* 
+				 * If it got this far then it must be
+				 * externally accessible.
+				 */
 				fp->f_flag |= FMARK;
 			}
+			/*
+			 * either it was defered, or it is externally 
+			 * accessible and not already marked so.
+			 * Now check if it is possibly one of OUR sockets.
+			 */ 
 			if (fp->f_type != DTYPE_SOCKET ||
 			    (so = (struct socket *)fp->f_data) == 0)
 				continue;
@@ -748,6 +812,13 @@ unp_gc()
 				goto restart;
 			}
 #endif
+			/*
+			 * So, Ok, it's one of our sockets and it IS externally
+			 * accessible (or was defered). Now we look
+			 * to see if we hold any file descriptors in it's
+			 * message buffers. Follow those links and mark them 
+			 * as accessible too.
+			 */
 			unp_scan(so->so_rcv.sb_mb, unp_mark);
 		}
 	} while (unp_defer);
@@ -791,16 +862,29 @@ unp_gc()
 	 * 91/09/19, bsy@cs.cmu.edu
 	 */
 	extra_ref = malloc(nfiles * sizeof(struct file *), M_FILE, M_WAITOK);
-	for (nunref = 0, fp = filehead, fpp = extra_ref; fp; fp = nextfp) {
-		nextfp = fp->f_filef;
+	for (nunref = 0, fp = filehead.lh_first, fpp = extra_ref; fp != 0;
+	    fp = nextfp) {
+		nextfp = fp->f_list.le_next;
+		/* 
+		 * If it's not open, skip it
+		 */
 		if (fp->f_count == 0)
 			continue;
+		/* 
+		 * If all refs are from msgs, and it's not marked accessible
+		 * then it must be referenced from some unreachable cycle
+		 * of (shut-down) FDs, so include it in our
+		 * list of FDs to remove
+		 */
 		if (fp->f_count == fp->f_msgcount && !(fp->f_flag & FMARK)) {
 			*fpp++ = fp;
 			nunref++;
 			fp->f_count++;
 		}
 	}
+	/* 
+	 * for each FD on our hit list, do the following two things
+	 */
 	for (i = nunref, fpp = extra_ref; --i >= 0; ++fpp)
 		sorflush((struct socket *)(*fpp)->f_data);
 	for (i = nunref, fpp = extra_ref; --i >= 0; ++fpp)
@@ -817,7 +901,7 @@ unp_dispose(m)
 		unp_scan(m, unp_discard);
 }
 
-void
+static void
 unp_scan(m0, op)
 	register struct mbuf *m0;
 	void (*op)(struct file *);
@@ -847,7 +931,7 @@ unp_scan(m0, op)
 	}
 }
 
-void
+static void
 unp_mark(fp)
 	struct file *fp;
 {
@@ -858,7 +942,7 @@ unp_mark(fp)
 	fp->f_flag |= (FMARK|FDEFER);
 }
 
-void
+static void
 unp_discard(fp)
 	struct file *fp;
 {
