@@ -33,7 +33,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)vfs_cluster.c	8.7 (Berkeley) 2/13/94
- * $FreeBSD: stable/2.1/sys/kern/vfs_cluster.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/vfs_cluster.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -47,28 +47,34 @@
 #include <sys/vmmeter.h>
 #include <miscfs/specfs/specdev.h>
 #include <vm/vm.h>
-#include <vm/vm_pageout.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/vm_object.h>
+#include <vm/vm_page.h>
 
+#ifdef notyet_block_reallocation_enabled
 #ifdef DEBUG
-#include <vm/vm.h>
 #include <sys/sysctl.h>
-int doreallocblks = 0;
-struct ctldebug debug13 = {"doreallocblks", &doreallocblks};
+#include <sys/kernel.h>
 
+static int	doreallocblks = 0;
+SYSCTL_INT(_debug, 13, doreallocblks, CTLFLAG_RW, &doreallocblks, 0, "");
 #else
-/* XXX for cluster_write */
-#define doreallocblks 0
+#define	doreallocblks 0
 #endif
+#endif /* notyet_block_reallocation_enabled */
 
-/*
- * Local declarations
- */
-struct buf *cluster_rbuild __P((struct vnode *, u_quad_t, struct buf *,
-    daddr_t, daddr_t, long, int, long));
-struct cluster_save *cluster_collectbufs __P((struct vnode *, struct buf *));
+#ifdef notyet_block_reallocation_enabled
+static struct cluster_save *
+	cluster_collectbufs __P((struct vnode *vp, struct buf *last_bp));
+#endif
+static struct buf *
+	cluster_rbuild __P((struct vnode *vp, u_quad_t filesize, daddr_t lbn,
+			    daddr_t blkno, long size, int run));
 
-int totreads;
-int totreadblocks;
+static int	totreads;
+static int	totreadblocks;
+extern vm_page_t	bogus_page;
 
 #ifdef DIAGNOSTIC
 /*
@@ -93,6 +99,13 @@ int totreadblocks;
 	(/* (blk) != 0 && */ ((blk) == (vp)->v_lastr + 1 || (blk) == (vp)->v_lastr))
 #endif
 
+/*
+ * allow for three entire read-aheads...  The system will
+ * adjust downwards rapidly if needed...
+ */
+#define RA_MULTIPLE_FAST	2
+#define RA_MULTIPLE_SLOW	3
+#define RA_SHIFTDOWN	1	/* approx lg2(RA_MULTIPLE) */
 /*
  * This replaces bread.  If this is a bread at the beginning of a file and
  * lastr is 0, we assume this is the first read and we'll read up to two
@@ -113,33 +126,37 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 {
 	struct buf *bp, *rbp;
 	daddr_t blkno, rablkno, origlblkno;
-	long flags;
 	int error, num_ra, alreadyincore;
+	int i;
+	int seq;
 
-	origlblkno = lblkno;
 	error = 0;
 	/*
 	 * get the requested block
 	 */
+	origlblkno = lblkno;
 	*bpp = bp = getblk(vp, lblkno, size, 0, 0);
+
+	seq = ISSEQREAD(vp, lblkno);
 	/*
 	 * if it is in the cache, then check to see if the reads have been
 	 * sequential.  If they have, then try some read-ahead, otherwise
 	 * back-off on prospective read-aheads.
 	 */
 	if (bp->b_flags & B_CACHE) {
-		int i;
-
-		if (!ISSEQREAD(vp, origlblkno)) {
+		if (!seq) {
 			vp->v_maxra = bp->b_lblkno + bp->b_bcount / size;
-			vp->v_ralen >>= 1;
+			vp->v_ralen >>= RA_SHIFTDOWN;
 			return 0;
-		} else if( vp->v_maxra >= origlblkno) {
-			if ((vp->v_ralen + 1) < (MAXPHYS / size))
-				vp->v_ralen++;
-			if ( vp->v_maxra >= (origlblkno + vp->v_ralen))
+		} else if( vp->v_maxra > lblkno) {
+			if ((vp->v_ralen + 1) < RA_MULTIPLE_FAST * (MAXPHYS / size))
+				++vp->v_ralen;
+			if ( vp->v_maxra > lblkno + vp->v_ralen ) {
 				return 0;
+			}
 			lblkno = vp->v_maxra;
+		} else {
+			lblkno += 1;
 		}
 		bp = NULL;
 	} else {
@@ -150,12 +167,8 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 		bp->b_flags |= B_READ;
 		lblkno += 1;
 		curproc->p_stats->p_ru.ru_inblock++;	/* XXX */
+		vp->v_ralen = 0;
 	}
-	/*
-	 * if ralen is "none", then try a little
-	 */
-	if (vp->v_ralen == 0)
-		vp->v_ralen = 1;
 	/*
 	 * assume no read-ahead
 	 */
@@ -165,35 +178,27 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 	/*
 	 * if we have been doing sequential I/O, then do some read-ahead
 	 */
-	if (ISSEQREAD(vp, origlblkno)) {
-		int i;
+	if (seq) {
+		alreadyincore = 0;
 
+	/*
+	 * bump ralen a bit...
+	 */
+		if ((vp->v_ralen + 1) < RA_MULTIPLE_SLOW*(MAXPHYS / size))
+			++vp->v_ralen;
 		/*
 		 * this code makes sure that the stuff that we have read-ahead
 		 * is still in the cache.  If it isn't, we have been reading
 		 * ahead too much, and we need to back-off, otherwise we might
 		 * try to read more.
 		 */
-		for (i = 0; i < vp->v_ralen; i++) {
+		for (i = 0; i < vp->v_maxra - lblkno; i++) {
 			rablkno = lblkno + i;
 			alreadyincore = (int) incore(vp, rablkno);
 			if (!alreadyincore) {
-				if (rablkno < vp->v_maxra) {
-					vp->v_maxra = rablkno;
-					vp->v_ralen >>= 1;
-					alreadyincore = 1;
-				} else {
-					if (inmem(vp, rablkno)) {
-						if( vp->v_maxra < rablkno)
-							vp->v_maxra = rablkno + 1;
-						continue;
-					}
-					if ((vp->v_ralen + 1) < MAXPHYS / size)
-						vp->v_ralen++;
-				}
-				break;
-			} else if( vp->v_maxra < rablkno) {
-				vp->v_maxra = rablkno + 1;
+				vp->v_maxra = rablkno;
+				vp->v_ralen >>= RA_SHIFTDOWN;
+				alreadyincore = 1;
 			}
 		}
 	}
@@ -202,17 +207,15 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 	 */
 	rbp = NULL;
 	if (!alreadyincore &&
-	    (rablkno + 1) * size <= filesize &&
-	    !(error = VOP_BMAP(vp, rablkno, NULL, &blkno, &num_ra)) &&
+	    ((u_quad_t)(rablkno + 1) * size) <= filesize &&
+	    !(error = VOP_BMAP(vp, rablkno, NULL, &blkno, &num_ra, NULL)) &&
 	    blkno != -1) {
-		if ((vp->v_ralen + 1) < MAXPHYS / size)
-			vp->v_ralen++;
 		if (num_ra > vp->v_ralen)
 			num_ra = vp->v_ralen;
 
 		if (num_ra) {
-			rbp = cluster_rbuild(vp, filesize,
-			    NULL, rablkno, blkno, size, num_ra, B_READ | B_ASYNC);
+			rbp = cluster_rbuild(vp, filesize, rablkno, blkno, size,
+				num_ra + 1);
 		} else {
 			rbp = getblk(vp, rablkno, size, 0, 0);
 			rbp->b_flags |= B_READ | B_ASYNC;
@@ -221,8 +224,7 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 	}
 
 	/*
-	 * if the synchronous read is a cluster, handle it, otherwise do a
-	 * simple, non-clustered read.
+	 * handle the synchronous read
 	 */
 	if (bp) {
 		if (bp->b_flags & (B_DONE | B_DELWRI))
@@ -241,11 +243,15 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 	 */
 	if (rbp) {
 		vp->v_maxra = rbp->b_lblkno + rbp->b_bcount / size;
-		if (error || (rbp->b_flags & B_CACHE)) {
+		if (error) {
 			rbp->b_flags &= ~(B_ASYNC | B_READ);
 			brelse(rbp);
+		} else if (rbp->b_flags & B_CACHE) {
+			rbp->b_flags &= ~(B_ASYNC | B_READ);
+			bqrelse(rbp);
 		} else {
-			vfs_busy_pages(rbp, 0);
+			if ((rbp->b_flags & B_CLUSTER) == 0)
+				vfs_busy_pages(rbp, 0);
 			(void) VOP_STRATEGY(rbp);
 			totreads++;
 			totreadblocks += rbp->b_bcount / size;
@@ -262,19 +268,16 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
  * read ahead.  We will read as many blocks as possible sequentially
  * and then parcel them up into logical blocks in the buffer hash table.
  */
-struct buf *
-cluster_rbuild(vp, filesize, bp, lbn, blkno, size, run, flags)
+static struct buf *
+cluster_rbuild(vp, filesize, lbn, blkno, size, run)
 	struct vnode *vp;
 	u_quad_t filesize;
-	struct buf *bp;
 	daddr_t lbn;
 	daddr_t blkno;
 	long size;
 	int run;
-	long flags;
 {
-	struct cluster_save *b_save;
-	struct buf *tbp;
+	struct buf *bp, *tbp;
 	daddr_t bn;
 	int i, inc, j;
 
@@ -283,69 +286,112 @@ cluster_rbuild(vp, filesize, bp, lbn, blkno, size, run, flags)
 		panic("cluster_rbuild: size %d != filesize %d\n",
 		    size, vp->v_mount->mnt_stat.f_iosize);
 #endif
-	if (size * (lbn + run + 1) > filesize)
+	/*
+	 * avoid a division
+	 */
+	while ((u_quad_t) size * (lbn + run) > filesize) {
 		--run;
-	if (run == 0) {
-		if (!bp) {
-			bp = getblk(vp, lbn, size, 0, 0);
-			bp->b_blkno = blkno;
-			bp->b_flags |= flags;
-		}
-		return (bp);
 	}
-	tbp = bp;
-	if (!tbp) {
-		tbp = getblk(vp, lbn, size, 0, 0);
-	}
-	if (tbp->b_flags & B_CACHE) {
-		return (tbp);
-	} else if (bp == NULL) {
-		tbp->b_flags |= B_ASYNC;
-	}
-	bp = getpbuf();
-	bp->b_flags = flags | B_CALL | B_BUSY | B_CLUSTER;
+
+	tbp = getblk(vp, lbn, size, 0, 0);
+	if (tbp->b_flags & B_CACHE)
+		return tbp;
+
+	tbp->b_blkno = blkno;
+	tbp->b_flags |= B_ASYNC | B_READ; 
+	if( (tbp->b_flags & B_MALLOC) ||
+		((tbp->b_flags & B_VMIO) == 0) || (run <= 1) )
+		return tbp;
+
+	bp = trypbuf();
+	if (bp == 0)
+		return tbp;
+
+	(vm_offset_t) bp->b_data |= ((vm_offset_t) tbp->b_data) & PAGE_MASK;
+	bp->b_flags = B_ASYNC | B_READ | B_CALL | B_BUSY | B_CLUSTER | B_VMIO;
 	bp->b_iodone = cluster_callback;
 	bp->b_blkno = blkno;
 	bp->b_lblkno = lbn;
 	pbgetvp(vp, bp);
 
-	b_save = malloc(sizeof(struct buf *) * (run + 1) + sizeof(struct cluster_save),
-	    M_SEGMENT, M_WAITOK);
-	b_save->bs_nchildren = 0;
-	b_save->bs_children = (struct buf **) (b_save + 1);
-	bp->b_saveaddr = b_save;
+	TAILQ_INIT(&bp->b_cluster.cluster_head);
 
 	bp->b_bcount = 0;
 	bp->b_bufsize = 0;
 	bp->b_npages = 0;
 
-	if (tbp->b_flags & B_VMIO)
-		bp->b_flags |= B_VMIO;
-
 	inc = btodb(size);
-	for (bn = blkno, i = 0; i <= run; ++i, bn += inc) {
+	for (bn = blkno, i = 0; i < run; ++i, bn += inc) {
 		if (i != 0) {
+			if ((bp->b_npages * PAGE_SIZE) +
+				round_page(size) > MAXPHYS)
+				break;
+
+			if (incore(vp, lbn + i))
+				break;
+
 			tbp = getblk(vp, lbn + i, size, 0, 0);
+
 			if ((tbp->b_flags & B_CACHE) ||
-			    (tbp->b_flags & B_VMIO) != (bp->b_flags & B_VMIO)) {
+				(tbp->b_flags & B_VMIO) == 0) {
+				bqrelse(tbp);
+				break;
+			}
+
+			for (j=0;j<tbp->b_npages;j++) {
+				if (tbp->b_pages[j]->valid) {
+					break;
+				}
+			}
+
+			if (j != tbp->b_npages) {
+				/*
+				 * force buffer to be re-constituted later
+				 */
+				tbp->b_flags |= B_RELBUF;
+				brelse(tbp);
+				break;
+			}
+
+			tbp->b_flags |= B_READ | B_ASYNC;
+			if (tbp->b_blkno == tbp->b_lblkno) {
+				tbp->b_blkno = bn;
+			} else if (tbp->b_blkno != bn) {
 				brelse(tbp);
 				break;
 			}
-			tbp->b_blkno = bn;
-			tbp->b_flags |= flags | B_READ | B_ASYNC;
-		} else {
-			tbp->b_flags |= flags | B_READ;
 		}
-		++b_save->bs_nchildren;
-		b_save->bs_children[i] = tbp;
+		TAILQ_INSERT_TAIL(&bp->b_cluster.cluster_head,
+			tbp, b_cluster.cluster_entry);
 		for (j = 0; j < tbp->b_npages; j += 1) {
-			bp->b_pages[j + bp->b_npages] = tbp->b_pages[j];
+			vm_page_t m;
+			m = tbp->b_pages[j];
+			++m->busy;
+			++m->object->paging_in_progress;
+			if ((bp->b_npages == 0) ||
+				(bp->b_pages[bp->b_npages-1] != m)) {
+				bp->b_pages[bp->b_npages] = m;
+				bp->b_npages++;
+			}
+			if ((m->valid & VM_PAGE_BITS_ALL) == VM_PAGE_BITS_ALL)
+				tbp->b_pages[j] = bogus_page;
 		}
-		bp->b_npages += tbp->b_npages;
-		bp->b_bcount += size;
-		bp->b_bufsize += size;
+		bp->b_bcount += tbp->b_bcount;
+		bp->b_bufsize += tbp->b_bufsize;
 	}
-	pmap_qenter((vm_offset_t) bp->b_data, (vm_page_t *)bp->b_pages, bp->b_npages);
+
+	for(j=0;j<bp->b_npages;j++) {
+		if ((bp->b_pages[j]->valid & VM_PAGE_BITS_ALL) ==
+			VM_PAGE_BITS_ALL)
+			bp->b_pages[j] = bogus_page;
+	}
+	if (bp->b_bufsize > bp->b_kvasize)
+		panic("cluster_rbuild: b_bufsize(%d) > b_kvasize(%d)\n",
+			bp->b_bufsize, bp->b_kvasize);
+	bp->b_kvasize = bp->b_bufsize;
+
+	pmap_qenter(trunc_page((vm_offset_t) bp->b_data),
+		(vm_page_t *)bp->b_pages, bp->b_npages);
 	return (bp);
 }
 
@@ -359,9 +405,7 @@ void
 cluster_callback(bp)
 	struct buf *bp;
 {
-	struct cluster_save *b_save;
-	struct buf **bpp, *tbp;
-	caddr_t cp;
+	struct buf *nbp, *tbp;
 	int error = 0;
 
 	/*
@@ -370,21 +414,21 @@ cluster_callback(bp)
 	if (bp->b_flags & B_ERROR)
 		error = bp->b_error;
 
-	b_save = (struct cluster_save *) (bp->b_saveaddr);
-	pmap_qremove((vm_offset_t) bp->b_data, bp->b_npages);
+	pmap_qremove(trunc_page((vm_offset_t) bp->b_data), bp->b_npages);
 	/*
 	 * Move memory from the large cluster buffer into the component
 	 * buffers and mark IO as done on these.
 	 */
-	for (bpp = b_save->bs_children; b_save->bs_nchildren--; ++bpp) {
-		tbp = *bpp;
+	for (tbp = bp->b_cluster.cluster_head.tqh_first;
+		tbp; tbp = nbp) {
+		nbp = tbp->b_cluster.cluster_entry.tqe_next;
 		if (error) {
 			tbp->b_flags |= B_ERROR;
 			tbp->b_error = error;
-		}
+		} else
+			tbp->b_dirtyoff = tbp->b_dirtyend = 0;
 		biodone(tbp);
 	}
-	free(b_save, M_SEGMENT);
 	relpbuf(bp);
 }
 
@@ -407,8 +451,10 @@ cluster_write(bp, filesize)
 	daddr_t lbn;
 	int maxclen, cursize;
 	int lblocksize;
+	int async;
 
 	vp = bp->b_vp;
+	async = (vp->v_mount && (vp->v_mount->mnt_flag & MNT_ASYNC));
 	lblocksize = vp->v_mount->mnt_stat.f_iosize;
 	lbn = bp->b_lblkno;
 
@@ -430,18 +476,62 @@ cluster_write(bp, filesize)
 			 * reallocating to make it sequential.
 			 */
 			cursize = vp->v_lastw - vp->v_cstart + 1;
-			cluster_wbuild(vp, NULL, lblocksize,
-			    vp->v_cstart, cursize, lbn);
+#ifndef notyet_block_reallocation_enabled
+			if (((u_quad_t)(lbn + 1) * lblocksize) != filesize ||
+				lbn != vp->v_lastw + 1 ||
+				vp->v_clen <= cursize) {
+				if (!async)
+					cluster_wbuild(vp, lblocksize,
+						vp->v_cstart, cursize);
+			}
+#else
+			if (!doreallocblks ||
+			    (lbn + 1) * lblocksize != filesize ||
+			    lbn != vp->v_lastw + 1 || vp->v_clen <= cursize) {
+				if (!async)
+					cluster_wbuild(vp, lblocksize,
+						vp->v_cstart, cursize);
+			} else {
+				struct buf **bpp, **endbp;
+				struct cluster_save *buflist;
+
+				buflist = cluster_collectbufs(vp, bp);
+				endbp = &buflist->bs_children
+				    [buflist->bs_nchildren - 1];
+				if (VOP_REALLOCBLKS(vp, buflist)) {
+					/*
+					 * Failed, push the previous cluster.
+					 */
+					for (bpp = buflist->bs_children;
+					     bpp < endbp; bpp++)
+						brelse(*bpp);
+					free(buflist, M_SEGMENT);
+					cluster_wbuild(vp, lblocksize,
+					    vp->v_cstart, cursize);
+				} else {
+					/*
+					 * Succeeded, keep building cluster.
+					 */
+					for (bpp = buflist->bs_children;
+					     bpp <= endbp; bpp++)
+						bdwrite(*bpp);
+					free(buflist, M_SEGMENT);
+					vp->v_lastw = lbn;
+					vp->v_lasta = bp->b_blkno;
+					return;
+				}
+			}
+#endif /* notyet_block_reallocation_enabled */
 		}
 		/*
 		 * Consider beginning a cluster. If at end of file, make
 		 * cluster as large as possible, otherwise find size of
 		 * existing cluster.
 		 */
-		if ((lbn + 1) * lblocksize != filesize &&
+		if (((u_quad_t) (lbn + 1) * lblocksize) != filesize &&
 		    (bp->b_blkno == bp->b_lblkno) &&
-		    (VOP_BMAP(vp, lbn, NULL, &bp->b_blkno, &maxclen) ||
-			bp->b_blkno == -1)) {
+		    (VOP_BMAP(vp, lbn, NULL, &bp->b_blkno, &maxclen, NULL) ||
+		     bp->b_blkno == -1)) {
 			bawrite(bp);
 			vp->v_clen = 0;
 			vp->v_lasta = bp->b_blkno;
@@ -450,7 +540,7 @@ cluster_write(bp, filesize)
 			return;
 		}
 		vp->v_clen = maxclen;
-		if (maxclen == 0) {	/* I/O not contiguous */
+		if (!async && maxclen == 0) {	/* I/O not contiguous */
 			vp->v_cstart = lbn + 1;
 			bawrite(bp);
 		} else {	/* Wait for rest of cluster */
@@ -461,8 +551,8 @@ cluster_write(bp, filesize)
 		/*
 		 * At end of cluster, write it out.
 		 */
-		cluster_wbuild(vp, bp, bp->b_bcount, vp->v_cstart,
-		    vp->v_clen + 1, lbn);
+		bdwrite(bp);
+		cluster_wbuild(vp, lblocksize, vp->v_cstart, vp->v_clen + 1);
 		vp->v_clen = 0;
 		vp->v_cstart = lbn + 1;
 	} else
@@ -481,68 +571,31 @@ cluster_write(bp, filesize)
  * performed.  Check to see that it doesn't fall in the middle of
  * the current block (if last_bp == NULL).
  */
-void
-cluster_wbuild(vp, last_bp, size, start_lbn, len, lbn)
+int
+cluster_wbuild(vp, size, start_lbn, len)
 	struct vnode *vp;
-	struct buf *last_bp;
 	long size;
 	daddr_t start_lbn;
 	int len;
-	daddr_t lbn;
 {
-	struct cluster_save *b_save;
-	struct buf *bp, *tbp, *pb;
-	caddr_t cp;
+	struct buf *bp, *tbp;
 	int i, j, s;
-
-#ifdef DIAGNOSTIC
-	if (size != vp->v_mount->mnt_stat.f_iosize)
-		panic("cluster_wbuild: size %d != filesize %d\n",
-		    size, vp->v_mount->mnt_stat.f_iosize);
-#endif
-redo:
-	if( (lbn != -1) || (last_bp == 0)) {
-		while ((!(tbp = incore(vp, start_lbn)) || (tbp->b_flags & B_BUSY)
-			|| (start_lbn == lbn)) && len) {
+	int totalwritten = 0;
+	int dbsize = btodb(size);
+	while (len > 0) {
+		s = splbio();
+		if ( ((tbp = gbincore(vp, start_lbn)) == NULL) ||
+			((tbp->b_flags & (B_INVAL|B_BUSY|B_DELWRI)) != B_DELWRI)) {
 			++start_lbn;
 			--len;
+			splx(s);
+			continue;
 		}
-
-		pb = trypbuf();
-		/* Get more memory for current buffer */
-		if (len <= 1 || pb == NULL) {
-			if (pb != NULL)
-				relpbuf(pb);
-			if (last_bp) {
-				bawrite(last_bp);
-			} else if (len) {
-				bp = getblk(vp, start_lbn, size, 0, 0);
-				bawrite(bp);
-			}
-			return;
-		}
-		tbp = getblk(vp, start_lbn, size, 0, 0);
-	} else {
-		tbp = last_bp;
-		if( tbp->b_flags & B_BUSY) {
-			printf("vfs_cluster: warning: buffer already busy\n");
-		}
+		bremfree(tbp);
 		tbp->b_flags |= B_BUSY;
-		last_bp = 0;
-		pb = trypbuf();
-		if (pb == NULL) {
-			bawrite(tbp);
-			return;
-		}
-	}
+		tbp->b_flags &= ~B_DONE;
+		splx(s);
 
-	if (!(tbp->b_flags & B_DELWRI)) {
-		relpbuf(pb);
-		++start_lbn;
-		--len;
-		brelse(tbp);
-		goto redo;
-	}
 	/*
 	 * Extra memory in the buffer, punt on this buffer. XXX we could
 	 * handle this in most cases, but we would have to push the extra
@@ -550,97 +603,118 @@ redo:
 	 * potentially pull it back up if the cluster was terminated
 	 * prematurely--too much hassle.
 	 */
-	if (((tbp->b_flags & B_VMIO) == 0) ||
-		(tbp->b_bcount != tbp->b_bufsize)) {
-		relpbuf(pb);
-		++start_lbn;
-		--len;
-		bawrite(tbp);
-		goto redo;
-	}
-	bp = pb;
-	b_save = malloc(sizeof(struct buf *) * (len + 1) + sizeof(struct cluster_save),
-	    M_SEGMENT, M_WAITOK);
-	b_save->bs_nchildren = 0;
-	b_save->bs_children = (struct buf **) (b_save + 1);
-	bp->b_saveaddr = b_save;
-	bp->b_bcount = 0;
-	bp->b_bufsize = 0;
-	bp->b_npages = 0;
+		if (((tbp->b_flags & (B_CLUSTEROK|B_MALLOC)) != B_CLUSTEROK) ||
+			(tbp->b_bcount != tbp->b_bufsize) ||
+			(tbp->b_bcount != size) ||
+			len == 1) {
+			totalwritten += tbp->b_bufsize;
+			bawrite(tbp);
+			++start_lbn;
+			--len;
+			continue;
+		}
 
-	if (tbp->b_flags & B_VMIO)
-		bp->b_flags |= B_VMIO;
+		bp = trypbuf();
+		if (bp == NULL) {
+			totalwritten += tbp->b_bufsize;
+			bawrite(tbp);
+			++start_lbn;
+			--len;
+			continue;
+		}
 
-	bp->b_blkno = tbp->b_blkno;
-	bp->b_lblkno = tbp->b_lblkno;
-	bp->b_flags |= B_CALL | B_BUSY | B_CLUSTER;
-	bp->b_iodone = cluster_callback;
-	pbgetvp(vp, bp);
+		TAILQ_INIT(&bp->b_cluster.cluster_head);
+		bp->b_bcount = 0;
+		bp->b_bufsize = 0;
+		bp->b_npages = 0;
+		if (tbp->b_wcred != NOCRED) {
+		    bp->b_wcred = tbp->b_wcred;
+		    crhold(bp->b_wcred);
+		}
 
-	for (i = 0; i < len; ++i, ++start_lbn) {
-		if (i != 0) {
-			/*
-			 * Block is not in core or the non-sequential block
-			 * ending our cluster was part of the cluster (in
-			 * which case we don't want to write it twice).
-			 */
-			if (!(tbp = incore(vp, start_lbn)) ||
-			    (last_bp == NULL && start_lbn == lbn))
-				break;
+		bp->b_blkno = tbp->b_blkno;
+		bp->b_lblkno = tbp->b_lblkno;
+		(vm_offset_t) bp->b_data |= ((vm_offset_t) tbp->b_data) & PAGE_MASK;
+		bp->b_flags |= B_CALL | B_BUSY | B_CLUSTER | (tbp->b_flags & (B_VMIO|B_NEEDCOMMIT));
+		bp->b_iodone = cluster_callback;
+		pbgetvp(vp, bp);
 
-			if ((tbp->b_flags & (B_INVAL | B_CLUSTEROK)) != B_CLUSTEROK)
-				break;
-
-			if ((tbp->b_npages + bp->b_npages) > (MAXPHYS / PAGE_SIZE))
-				break;
-
-			/*
-			 * Get the desired block buffer (unless it is the
-			 * final sequential block whose buffer was passed in
-			 * explictly as last_bp).
-			 */
-			if (last_bp == NULL || start_lbn != lbn) {
-				if( tbp->b_flags & B_BUSY)
-					break;
-				tbp = getblk(vp, start_lbn, size, 0, 0);
-				if (!(tbp->b_flags & B_DELWRI) ||
-				    ((tbp->b_flags & B_VMIO) != (bp->b_flags & B_VMIO))) {
-					brelse(tbp);
+		for (i = 0; i < len; ++i, ++start_lbn) {
+			if (i != 0) {
+				s = splbio();
+				if ((tbp = gbincore(vp, start_lbn)) == NULL) {
+					splx(s);
 					break;
 				}
-			} else
-				tbp = last_bp;
-		}
-		for (j = 0; j < tbp->b_npages; j += 1) {
-			bp->b_pages[j + bp->b_npages] = tbp->b_pages[j];
-		}
-		bp->b_npages += tbp->b_npages;
-		bp->b_bcount += size;
-		bp->b_bufsize += size;
 
-		tbp->b_flags &= ~(B_READ | B_DONE | B_ERROR | B_DELWRI);
-		tbp->b_flags |= B_ASYNC;
-		s = splbio();
-		reassignbuf(tbp, tbp->b_vp);	/* put on clean list */
-		++tbp->b_vp->v_numoutput;
-		splx(s);
-		b_save->bs_children[i] = tbp;
-	}
-	b_save->bs_nchildren = i;
-	pmap_qenter((vm_offset_t) bp->b_data, (vm_page_t *) bp->b_pages, bp->b_npages);
-	bawrite(bp);
+				if ((tbp->b_flags & (B_VMIO|B_CLUSTEROK|B_INVAL|B_BUSY|B_DELWRI|B_NEEDCOMMIT)) != (B_DELWRI|B_CLUSTEROK|(bp->b_flags & (B_VMIO|B_NEEDCOMMIT)))) {
+					splx(s);
+					break;
+				}
+
+				if (tbp->b_wcred != bp->b_wcred) {
+					splx(s);
+					break;
+				}
+
+				if ((tbp->b_bcount != size) ||
+					((bp->b_blkno + dbsize * i) != tbp->b_blkno) ||
+					((tbp->b_npages + bp->b_npages) > (MAXPHYS / PAGE_SIZE))) {
+					splx(s);
+					break;
+				}
+				bremfree(tbp);
+				tbp->b_flags |= B_BUSY;
+				tbp->b_flags &= ~B_DONE;
+				splx(s);
+			}
+			if (tbp->b_flags & B_VMIO) {
+				for (j = 0; j < tbp->b_npages; j += 1) {
+					vm_page_t m;
+					m = tbp->b_pages[j];
+					++m->busy;
+					++m->object->paging_in_progress;
+					if ((bp->b_npages == 0) ||
+						(bp->b_pages[bp->b_npages - 1] != m)) {
+						bp->b_pages[bp->b_npages] = m;
+						bp->b_npages++;
+					}
+				}
+			}
+			bp->b_bcount += size;
+			bp->b_bufsize += size;
+
+			tbp->b_flags &= ~(B_READ | B_DONE | B_ERROR | B_DELWRI);
+			tbp->b_flags |= B_ASYNC;
+			s = splbio();
+			reassignbuf(tbp, tbp->b_vp);	/* put on clean list */
+			++tbp->b_vp->v_numoutput;
+			splx(s);
+			TAILQ_INSERT_TAIL(&bp->b_cluster.cluster_head,
+				tbp, b_cluster.cluster_entry);
+		}
+		pmap_qenter(trunc_page((vm_offset_t) bp->b_data),
+			(vm_page_t *) bp->b_pages, bp->b_npages);
+		if (bp->b_bufsize > bp->b_kvasize)
+			panic("cluster_wbuild: b_bufsize(%d) > b_kvasize(%d)\n",
+				bp->b_bufsize, bp->b_kvasize);
+		bp->b_kvasize = bp->b_bufsize;
+		totalwritten += bp->b_bufsize;
+		bp->b_dirtyoff = 0;
+		bp->b_dirtyend = bp->b_bufsize;
+		bawrite(bp);
 
-	if (i < len) {
 		len -= i;
-		goto redo;
 	}
+	return totalwritten;
 }
 
+#ifdef notyet_block_reallocation_enabled
 /*
  * Collect together all the buffers in a cluster.
  * Plus add one additional buffer.
  */
-struct cluster_save *
+static struct cluster_save *
 cluster_collectbufs(vp, last_bp)
 	struct vnode *vp;
 	struct buf *last_bp;
@@ -661,3 +735,4 @@ cluster_collectbufs(vp, last_bp)
 	buflist->bs_nchildren = i + 1;
 	return (buflist);
 }
+#endif /* notyet_block_reallocation_enabled */
