@@ -36,15 +36,18 @@
  * SUCH DAMAGE.
  *
  *	@(#)vfs_subr.c	8.13 (Berkeley) 4/18/94
- * $FreeBSD: stable/2.1/sys/kern/vfs_subr.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/vfs_subr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
  * External virtual filesystem routines
  */
+#include "opt_ddb.h"
+#include "opt_devfs.h"
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/kernel.h>
 #include <sys/file.h>
 #include <sys/proc.h>
 #include <sys/mount.h>
@@ -60,11 +63,20 @@
 #include <sys/mbuf.h>
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_object.h>
+#include <vm/vm_extern.h>
+#include <vm/vm_pager.h>
+#include <vm/vnode_pager.h>
 #include <sys/sysctl.h>
 
 #include <miscfs/specfs/specdev.h>
 
-void insmntque __P((struct vnode *, struct mount *));
+#ifdef DDB
+extern void	printlockedvnodes __P((void));
+#endif
+extern void	vclean __P((struct vnode *vp, int flags));
+extern void	vfs_unmountroot __P((struct mount *rootfs));
 
 enum vtype iftovt_tab[16] = {
 	VNON, VFIFO, VCHR, VNON, VDIR, VNON, VBLK, VNON,
@@ -85,11 +97,17 @@ int vttoif_tab[9] = {
 }
 
 TAILQ_HEAD(freelst, vnode) vnode_free_list;	/* vnode free list */
-u_long freevnodes	= 0;
+static u_long freevnodes = 0;
 
 struct mntlist mountlist;	/* mounted filesystem list */
 
 int desiredvnodes;
+SYSCTL_INT(_kern, KERN_MAXVNODES, maxvnodes, CTLFLAG_RW, &desiredvnodes, 0, "");
+
+static void	vfs_free_addrlist __P((struct netexport *nep));
+static int	vfs_free_netcred __P((struct radix_node *rn, void *w));
+static int	vfs_hang_addrlist __P((struct mount *mp, struct netexport *nep,
+				       struct export_args *argp));
 
 /*
  * Initialize the vnode management data structures.
@@ -97,10 +115,10 @@ int desiredvnodes;
 void
 vntblinit()
 {
-	desiredvnodes = maxproc + vm_object_cache_max;
 
+	desiredvnodes = maxproc + vm_object_cache_max;
 	TAILQ_INIT(&vnode_free_list);
-	TAILQ_INIT(&mountlist);
+	CIRCLEQ_INIT(&mountlist);
 }
 
 /*
@@ -176,8 +194,7 @@ vfs_unbusy(mp)
 }
 
 void
-vfs_unmountroot(rootfs)
-	struct mount *rootfs;
+vfs_unmountroot(struct mount *rootfs)
 {
 	struct mount *mp = rootfs;
 	int error;
@@ -214,12 +231,12 @@ vfs_unmountroot(rootfs)
 void
 vfs_unmountall()
 {
-	struct mount *mp, *mp_next, *rootfs = NULL;
+	struct mount *mp, *nmp, *rootfs = NULL;
 	int error;
 
 	/* unmount all but rootfs */
-	for (mp = mountlist.tqh_first; mp != NULL; mp = mp_next) {
-		mp_next = mp->mnt_list.tqe_next;
+	for (mp = mountlist.cqh_last; mp != (void *)&mountlist; mp = nmp) {
+		nmp = mp->mnt_list.cqe_prev;
 
 		if (mp->mnt_flag & MNT_ROOTFS) {
 			rootfs = mp;
@@ -252,7 +269,8 @@ getvfs(fsid)
 {
 	register struct mount *mp;
 
-	for (mp = mountlist.tqh_first; mp != NULL; mp = mp->mnt_list.tqe_next) {
+	for (mp = mountlist.cqh_first; mp != (void *)&mountlist;
+	    mp = mp->mnt_list.cqe_next) {
 		if (mp->mnt_stat.f_fsid.val[0] == fsid->val[0] &&
 		    mp->mnt_stat.f_fsid.val[1] == fsid->val[1])
 			return (mp);
@@ -278,7 +296,7 @@ getnewfsid(mp, mtype)
 		++xxxfs_mntid;
 	tfsid.val[0] = makedev(nblkdev + mtype, xxxfs_mntid);
 	tfsid.val[1] = mtype;
-	if (mountlist.tqh_first != NULL) {
+	if (mountlist.cqh_first != (void *)&mountlist) {
 		while (getvfs(&tfsid)) {
 			tfsid.val[0]++;
 			xxxfs_mntid++;
@@ -301,9 +319,9 @@ vattr_null(vap)
 	vap->va_mode = vap->va_nlink = vap->va_uid = vap->va_gid =
 	    vap->va_fsid = vap->va_fileid =
 	    vap->va_blocksize = vap->va_rdev =
-	    vap->va_atime.ts_sec = vap->va_atime.ts_nsec =
-	    vap->va_mtime.ts_sec = vap->va_mtime.ts_nsec =
-	    vap->va_ctime.ts_sec = vap->va_ctime.ts_nsec =
+	    vap->va_atime.tv_sec = vap->va_atime.tv_nsec =
+	    vap->va_mtime.tv_sec = vap->va_mtime.tv_nsec =
+	    vap->va_ctime.tv_sec = vap->va_ctime.tv_nsec =
 	    vap->va_flags = vap->va_gen = VNOVAL;
 	vap->va_vaflags = 0;
 }
@@ -311,8 +329,7 @@ vattr_null(vap)
 /*
  * Routines having to do with the management of the vnode table.
  */
-extern int (**dead_vnodeop_p) ();
-extern void vclean();
+extern vop_t **dead_vnodeop_p;
 
 /*
  * Return the next vnode from the free list.
@@ -321,11 +338,12 @@ int
 getnewvnode(tag, mp, vops, vpp)
 	enum vtagtype tag;
 	struct mount *mp;
-	int (**vops) ();
+	vop_t **vops;
 	struct vnode **vpp;
 {
 	register struct vnode *vp;
 
+retry:
 	vp = vnode_free_list.tqh_first;
 	/*
 	 * we allocate a new vnode if
@@ -346,8 +364,12 @@ getnewvnode(tag, mp, vops, vpp)
 		numvnodes++;
 	} else {
 		TAILQ_REMOVE(&vnode_free_list, vp, v_freelist);
+		if (vp->v_usage > 0) {
+			--vp->v_usage;
+			TAILQ_INSERT_TAIL(&vnode_free_list, vp, v_freelist);
+			goto retry;
+		}
 		freevnodes--;
-
 		if (vp->v_usecount)
 			panic("free vnode isn't");
 
@@ -356,6 +378,7 @@ getnewvnode(tag, mp, vops, vpp)
 		vp->v_lease = NULL;
 		if (vp->v_type != VBAD)
 			vgone(vp);
+
 #ifdef DIAGNOSTIC
 		{
 			int s;
@@ -378,6 +401,7 @@ getnewvnode(tag, mp, vops, vpp)
 		vp->v_clen = 0;
 		vp->v_socket = 0;
 		vp->v_writecount = 0;	/* XXX */
+		vp->v_usage = 0;
 	}
 	vp->v_type = VNON;
 	cache_purge(vp);
@@ -448,7 +472,6 @@ vinvalbuf(vp, flags, cred, p, slpflag, slptimeo)
 	register struct buf *bp;
 	struct buf *nbp, *blist;
 	int s, error;
-	vm_pager_t pager;
 	vm_object_t object;
 
 	if (flags & V_SAVE) {
@@ -457,6 +480,8 @@ vinvalbuf(vp, flags, cred, p, slpflag, slptimeo)
 		if (vp->v_dirtyblkhd.lh_first != NULL)
 			panic("vinvalbuf: dirty bufs");
 	}
+
+	s = splbio();
 	for (;;) {
 		if ((blist = vp->v_cleanblkhd.lh_first) && (flags & V_SAVEMETA))
 			while (blist && blist->b_lblkno < 0)
@@ -472,20 +497,19 @@ vinvalbuf(vp, flags, cred, p, slpflag, slptimeo)
 			nbp = bp->b_vnbufs.le_next;
 			if ((flags & V_SAVEMETA) && bp->b_lblkno < 0)
 				continue;
-			s = splbio();
 			if (bp->b_flags & B_BUSY) {
 				bp->b_flags |= B_WANTED;
 				error = tsleep((caddr_t) bp,
 				    slpflag | (PRIBIO + 1), "vinvalbuf",
 				    slptimeo);
-				splx(s);
-				if (error)
+				if (error) {
+					splx(s);
 					return (error);
+				}
 				break;
 			}
 			bremfree(bp);
 			bp->b_flags |= B_BUSY;
-			splx(s);
 			/*
 			 * XXX Since there are no node locks for NFS, I
 			 * believe there is a slight chance that a delayed
@@ -501,22 +525,20 @@ vinvalbuf(vp, flags, cred, p, slpflag, slptimeo)
 		}
 	}
 
-	s = splbio();
 	while (vp->v_numoutput > 0) {
 		vp->v_flag |= VBWAIT;
 		tsleep(&vp->v_numoutput, PVM, "vnvlbv", 0);
 	}
+
 	splx(s);
 
 	/*
 	 * Destroy the copy in the VM cache, too.
 	 */
-	object = (vm_object_t) vp->v_vmdata;
+	object = vp->v_object;
 	if (object != NULL) {
-		vm_object_lock(object);
 		vm_object_page_remove(object, 0, object->size,
 		    (flags & V_SAVE) ? TRUE : FALSE);
-		vm_object_unlock(object);
 	}
 	if (!(flags & V_SAVEMETA) &&
 	    (vp->v_dirtyblkhd.lh_first || vp->v_cleanblkhd.lh_first))
@@ -620,7 +642,6 @@ reassignbuf(bp, newvp)
 	register struct buf *bp;
 	register struct vnode *newvp;
 {
-	register struct buflists *listheadp;
 	int s;
 
 	if (newvp == NULL) {
@@ -645,18 +666,19 @@ reassignbuf(bp, newvp)
 		if (!tbp || (tbp->b_lblkno > bp->b_lblkno)) {
 			bufinsvn(bp, &newvp->v_dirtyblkhd);
 		} else {
-			while (tbp->b_vnbufs.le_next && (tbp->b_vnbufs.le_next->b_lblkno < bp->b_lblkno)) {
+			while (tbp->b_vnbufs.le_next &&
+				(tbp->b_vnbufs.le_next->b_lblkno < bp->b_lblkno)) {
 				tbp = tbp->b_vnbufs.le_next;
 			}
 			LIST_INSERT_AFTER(tbp, bp, b_vnbufs);
 		}
 	} else {
-		listheadp = &newvp->v_cleanblkhd;
-		bufinsvn(bp, listheadp);
+		bufinsvn(bp, &newvp->v_cleanblkhd);
 	}
 	splx(s);
 }
 
+#ifndef DEVFS_ROOT
 /*
  * Create a vnode for a block device.
  * Used for root filesystem, argdev, and swap areas.
@@ -687,6 +709,7 @@ bdevvp(dev, vpp)
 	*vpp = vp;
 	return (0);
 }
+#endif /* !DEVFS_ROOT */
 
 /*
  * Check to see if the new vnode represents a special device
@@ -724,6 +747,7 @@ loop:
 			goto loop;
 		break;
 	}
+
 	if (vp == NULL || vp->v_tag != VT_NON) {
 		MALLOC(nvp->v_specinfo, struct specinfo *,
 		    sizeof(struct specinfo), M_VNODE, M_WAITOK);
@@ -783,8 +807,18 @@ vget(vp, lockflag)
 		freevnodes--;
 	}
 	vp->v_usecount++;
+
+	/*
+	 * Create the VM object, if needed
+	 */
+	if ((vp->v_type == VREG) &&
+		((vp->v_object == NULL) ||
+			(vp->v_object->flags & OBJ_VFS_REF) == 0)) {
+		vfs_object_create(vp, curproc, curproc->p_ucred, 0);
+	}
 	if (lockflag)
 		VOP_LOCK(vp);
+
 	return (0);
 }
 
@@ -795,10 +829,22 @@ void
 vref(vp)
 	struct vnode *vp;
 {
-
 	if (vp->v_usecount <= 0)
 		panic("vref used where vget required");
+
 	vp->v_usecount++;
+
+	if ((vp->v_type == VREG) &&
+		((vp->v_object == NULL) ||
+			((vp->v_object->flags & OBJ_VFS_REF) == 0)) ) {
+		/*
+		 * We need to lock to VP during the time that
+		 * the object is created.  This is necessary to
+		 * keep the system from re-entrantly doing it
+		 * multiple times.
+		 */
+		vfs_object_create(vp, curproc, curproc->p_ucred, 0);
+	}
 }
 
 /*
@@ -808,7 +854,6 @@ void
 vput(vp)
 	register struct vnode *vp;
 {
-
 	VOP_UNLOCK(vp);
 	vrele(vp);
 }
@@ -826,26 +871,41 @@ vrele(vp)
 	if (vp == NULL)
 		panic("vrele: null vp");
 #endif
+
 	vp->v_usecount--;
+
+	if ((vp->v_usecount == 1) &&
+		vp->v_object &&
+		(vp->v_object->flags & OBJ_VFS_REF)) {
+		vp->v_object->flags &= ~OBJ_VFS_REF;
+		vm_object_deallocate(vp->v_object);
+		return;
+	}
+
 	if (vp->v_usecount > 0)
 		return;
+
+	if (vp->v_usecount < 0) {
 #ifdef DIAGNOSTIC
-	if (vp->v_usecount != 0 /* || vp->v_writecount != 0 */ ) {
-		vprint("vrele: bad ref count", vp);
-		panic("vrele: ref cnt");
-	}
+		vprint("vrele: negative ref count", vp);
 #endif
+		panic("vrele: negative reference cnt");
+	}
 	if (vp->v_flag & VAGE) {
-		TAILQ_INSERT_HEAD(&vnode_free_list, vp, v_freelist);
+		if(vp->v_tag != VT_TFS)
+			TAILQ_INSERT_HEAD(&vnode_free_list, vp, v_freelist);
 		vp->v_flag &= ~VAGE;
+		vp->v_usage = 0;
 	} else {
-		TAILQ_INSERT_TAIL(&vnode_free_list, vp, v_freelist);
+		if(vp->v_tag != VT_TFS)
+			TAILQ_INSERT_TAIL(&vnode_free_list, vp, v_freelist);
 	}
 	freevnodes++;
 
 	VOP_INACTIVE(vp);
 }
 
+#ifdef DIAGNOSTIC
 /*
  * Page or buffer structure gets a reference.
  */
@@ -869,6 +929,7 @@ holdrele(vp)
 		panic("holdrele: holdcnt");
 	vp->v_holdcnt--;
 }
+#endif /* DIAGNOSTIC */
 
 /*
  * Remove any vnodes in the vnode table belonging to mount point mp.
@@ -879,9 +940,8 @@ holdrele(vp)
  * that are found.
  */
 #ifdef DIAGNOSTIC
-int busyprt = 0;		/* print out busy vnodes */
-struct ctldebug debug1 = {"busyprt", &busyprt};
-
+static int busyprt = 0;		/* print out busy vnodes */
+SYSCTL_INT(_debug, 1, busyprt, CTLFLAG_RW, &busyprt, 0, "");
 #endif
 
 int
@@ -921,6 +981,14 @@ loop:
 		if ((flags & WRITECLOSE) &&
 		    (vp->v_writecount == 0 || vp->v_type != VREG))
 			continue;
+
+		if (vp->v_object && (vp->v_object->flags & OBJ_VFS_REF)) {
+			vm_object_reference(vp->v_object);
+			pager_cache(vp->v_object, FALSE);
+			vp->v_object->flags &= ~OBJ_VFS_REF;
+			vm_object_deallocate(vp->v_object);
+		}
+
 		/*
 		 * With v_usecount == 0, all we need to do is clear out the
 		 * vnode data structures and we are done.
@@ -929,6 +997,7 @@ loop:
 			vgone(vp);
 			continue;
 		}
+
 		/*
 		 * If FORCECLOSE is set, forcibly close the vnode. For block
 		 * or character devices, revert to an anonymous device. For
@@ -959,9 +1028,7 @@ loop:
  * Disassociate the underlying file system from a vnode.
  */
 void
-vclean(vp, flags)
-	register struct vnode *vp;
-	int flags;
+vclean(struct vnode *vp, int flags)
 {
 	int active;
 
@@ -1089,6 +1156,11 @@ vgone(vp)
 		(void) tsleep((caddr_t) vp, PINOD, "vgone", 0);
 		return;
 	}
+
+	if (vp->v_object) {
+		vp->v_object->flags |= OBJ_VNODE_GONE;
+	}
+
 	/*
 	 * Clean out the filesystem specific data.
 	 */
@@ -1149,8 +1221,10 @@ vgone(vp)
 	if (vp->v_usecount == 0 &&
 	    vp->v_freelist.tqe_prev != (struct vnode **) 0xdeadb &&
 	    vnode_free_list.tqh_first != vp) {
-		TAILQ_REMOVE(&vnode_free_list, vp, v_freelist);
-		TAILQ_INSERT_HEAD(&vnode_free_list, vp, v_freelist);
+		if(vp->v_tag != VT_TFS) {
+			TAILQ_REMOVE(&vnode_free_list, vp, v_freelist);
+			TAILQ_INSERT_HEAD(&vnode_free_list, vp, v_freelist);
+		}
 	}
 	vp->v_type = VBAD;
 }
@@ -1253,13 +1327,14 @@ vprint(label, vp)
  * Called when debugging the kernel.
  */
 void
-printlockedvnodes()
+printlockedvnodes(void)
 {
 	register struct mount *mp;
 	register struct vnode *vp;
 
 	printf("Locked vnodes\n");
-	for (mp = mountlist.tqh_first; mp != NULL; mp = mp->mnt_list.tqe_next) {
+	for (mp = mountlist.cqh_first; mp != (void *)&mountlist;
+	    mp = mp->mnt_list.cqe_next) {
 		for (vp = mp->mnt_vnodelist.lh_first;
 		    vp != NULL;
 		    vp = vp->v_mntvnodes.le_next)
@@ -1278,30 +1353,25 @@ int kinfo_vgetfailed;
  * Copyout address of vnode followed by vnode.
  */
 /* ARGSUSED */
-int
-sysctl_vnode(where, sizep)
-	char *where;
-	size_t *sizep;
+static int
+sysctl_vnode SYSCTL_HANDLER_ARGS
 {
 	register struct mount *mp, *nmp;
 	struct vnode *vp;
-	register char *bp = where, *savebp;
-	char *ewhere;
 	int error;
 
 #define VPTRSZ	sizeof (struct vnode *)
 #define VNODESZ	sizeof (struct vnode)
-	if (where == NULL) {
-		*sizep = (numvnodes + KINFO_VNODESLOP) * (VPTRSZ + VNODESZ);
-		return (0);
-	}
-	ewhere = where + *sizep;
 
-	for (mp = mountlist.tqh_first; mp != NULL; mp = nmp) {
-		nmp = mp->mnt_list.tqe_next;
+	req->lock = 0;
+	if (!req->oldptr) /* Make an estimate */
+		return (SYSCTL_OUT(req, 0,
+			(numvnodes + KINFO_VNODESLOP) * (VPTRSZ + VNODESZ)));
+
+	for (mp = mountlist.cqh_first; mp != (void *)&mountlist; mp = nmp) {
+		nmp = mp->mnt_list.cqe_next;
 		if (vfs_busy(mp))
 			continue;
-		savebp = bp;
 again:
 		for (vp = mp->mnt_vnodelist.lh_first;
 		    vp != NULL;
@@ -1314,29 +1384,30 @@ again:
 			if (vp->v_mount != mp) {
 				if (kinfo_vdebug)
 					printf("kinfo: vp changed\n");
-				bp = savebp;
 				goto again;
 			}
-			if (bp + VPTRSZ + VNODESZ > ewhere) {
+			if ((error = SYSCTL_OUT(req, &vp, VPTRSZ)) ||
+			    (error = SYSCTL_OUT(req, vp, VNODESZ))) {
 				vfs_unbusy(mp);
-				*sizep = bp - where;
-				return (ENOMEM);
-			}
-			if ((error = copyout(&vp, bp, VPTRSZ)) ||
-			    (error = copyout(vp, bp + VPTRSZ, VNODESZ))) {
-				vfs_unbusy(mp);
-				*sizep = bp - where;
 				return (error);
 			}
-			bp += VPTRSZ + VNODESZ;
 		}
 		vfs_unbusy(mp);
 	}
 
-	*sizep = bp - where;
 	return (0);
 }
 
+/*
+ * XXX
+ * Exporting the vnode list on large systems causes them to crash.
+ * Exporting the vnode list on medium systems causes sysctl to coredump.
+ */
+#if 0
+SYSCTL_PROC(_kern, KERN_VNODE, vnode, CTLTYPE_OPAQUE|CTLFLAG_RD,
+	0, 0, sysctl_vnode, "S,vnode", "");
+#endif
+
 /*
  * Check to see if a filesystem is mounted on a block device.
  */
@@ -1365,10 +1436,8 @@ vfs_mountedon(vp)
  * Called by ufs_mount() to set up the lists of export addresses.
  */
 static int
-vfs_hang_addrlist(mp, nep, argp)
-	struct mount *mp;
-	struct netexport *nep;
-	struct export_args *argp;
+vfs_hang_addrlist(struct mount *mp, struct netexport *nep, 
+	struct export_args *argp)
 {
 	register struct netcred *np;
 	register struct radix_node_head *rnh;
@@ -1438,9 +1507,7 @@ out:
 
 /* ARGSUSED */
 static int
-vfs_free_netcred(rn, w)
-	struct radix_node *rn;
-	caddr_t w;
+vfs_free_netcred(struct radix_node *rn, void *w)
 {
 	register struct radix_node_head *rnh = (struct radix_node_head *) w;
 
@@ -1453,8 +1520,7 @@ vfs_free_netcred(rn, w)
  * Free the net address hash lists that are hanging off the mount points.
  */
 static void
-vfs_free_addrlist(nep)
-	struct netexport *nep;
+vfs_free_addrlist(struct netexport *nep)
 {
 	register int i;
 	register struct radix_node_head *rnh;
@@ -1530,23 +1596,72 @@ vfs_export_lookup(mp, nep, nam)
  */
 void
 vfs_msync(struct mount *mp, int flags) {
-	struct vnode *vp;
+	struct vnode *vp, *nvp;
 loop:
-	for (vp = mp->mnt_vnodelist.lh_first;
-	     vp != NULL;
-	     vp = vp->v_mntvnodes.le_next) {
+	for (vp = mp->mnt_vnodelist.lh_first; vp != NULL; vp = nvp) {
 
 		if (vp->v_mount != mp)
 			goto loop;
+		nvp = vp->v_mntvnodes.le_next;
 		if (VOP_ISLOCKED(vp) && (flags != MNT_WAIT))
 			continue;
-		if (vp->v_vmdata &&
-		   (((vm_object_t) vp->v_vmdata)->flags & OBJ_WRITEABLE)) {
-			if (vget(vp, 1))
-				goto loop;
-			_vm_object_page_clean( (vm_object_t) vp->v_vmdata,
-					0, 0, TRUE);
-			vput(vp);
+		if (vp->v_object &&
+		   (vp->v_object->flags & OBJ_MIGHTBEDIRTY)) {
+			vm_object_page_clean(vp->v_object, 0, 0, TRUE, TRUE);
 		}
 	}
 }
+
+/*
+ * Create the VM object needed for VMIO and mmap support.  This
+ * is done for all VREG files in the system.  Some filesystems might
+ * afford the additional metadata buffering capability of the
+ * VMIO code by making the device node be VMIO mode also.
+ */
+int
+vfs_object_create(vp, p, cred, waslocked)
+	struct vnode *vp;
+	struct proc *p;
+	struct ucred *cred;
+	int waslocked;
+{
+	struct vattr vat;
+	vm_object_t object;
+	int error = 0;
+
+retry:
+	if ((object = vp->v_object) == NULL) {
+		if (vp->v_type == VREG) {
+			if ((error = VOP_GETATTR(vp, &vat, cred, p)) != 0)
+				goto retn;
+			(void) vnode_pager_alloc(vp,
+				OFF_TO_IDX(round_page(vat.va_size)), 0, 0);
+		} else {
+			/*
+			 * This simply allocates the biggest object possible
+			 * for a VBLK vnode.  This should be fixed, but doesn't
+			 * cause any problems (yet).
+			 */
+			(void) vnode_pager_alloc(vp, INT_MAX, 0, 0);
+		}
+		vp->v_object->flags |= OBJ_VFS_REF;
+	} else {
+		if (object->flags & OBJ_DEAD) {
+			if (waslocked)
+				VOP_UNLOCK(vp);
+			tsleep(object, PVM, "vodead", 0);
+			if (waslocked)
+				VOP_LOCK(vp);
+			goto retry;
+		}
+		if ((object->flags & OBJ_VFS_REF) == 0) {
+			object->flags |= OBJ_VFS_REF;
+			vm_object_reference(object);
+		}
+	}
+	if (vp->v_object)
+		vp->v_flag |= VVMIO;
+
+retn:
+	return error;
+}
