@@ -36,16 +36,27 @@
  * SUCH DAMAGE.
  *
  *	@(#)vfs_syscalls.c	8.13 (Berkeley) 4/15/94
- * $FreeBSD: stable/2.1/sys/kern/vfs_syscalls.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/kern/vfs_syscalls.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+/*
+ * XXX - The following is required because of some magic done 
+ * in getdirentries() below which is only done if the translucent
+ * filesystem `UNION' is compiled into the kernel.  This is broken,
+ * but I don't have time to study the code deeply enough to understand
+ * what's going on and determine an appropriate fix.  -GAW
+ */
+#include "opt_union.h"
+
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/namei.h>
 #include <sys/filedesc.h>
 #include <sys/kernel.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/mount.h>
 #include <sys/proc.h>
@@ -58,10 +69,12 @@
 #endif
 
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_object.h>
+#include <vm/vm_extern.h>
 #include <sys/sysctl.h>
 
 static int change_dir	__P((struct nameidata *ndp, struct proc *p));
-int getvnode __P((struct filedesc *, int, struct file **));
 
 /*
  * Virtual File System System Calls
@@ -70,12 +83,14 @@ int getvnode __P((struct filedesc *, int, struct file **));
 /*
  * Mount a file system.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct mount_args {
 	int	type;
 	char	*path;
 	int	flags;
 	caddr_t	data;
 };
+#endif
 /* ARGSUSED */
 int
 mount(p, uap, retval)
@@ -168,9 +183,9 @@ update:
 	else if (mp->mnt_flag & MNT_RDONLY)
 		mp->mnt_flag |= MNT_WANTRDWR;
 	mp->mnt_flag &=~ (MNT_NOSUID | MNT_NOEXEC | MNT_NODEV |
-	    MNT_SYNCHRONOUS | MNT_UNION | MNT_ASYNC);
+	    MNT_SYNCHRONOUS | MNT_UNION | MNT_ASYNC | MNT_NOATIME);
 	mp->mnt_flag |= uap->flags & (MNT_NOSUID | MNT_NOEXEC | MNT_NODEV |
-	    MNT_SYNCHRONOUS | MNT_UNION | MNT_ASYNC | MNT_FORCE);
+	    MNT_SYNCHRONOUS | MNT_UNION | MNT_ASYNC | MNT_FORCE | MNT_NOATIME);
 	/*
 	 * Mount the filesystem.
 	 */
@@ -190,10 +205,12 @@ update:
 	 */
 	cache_purge(vp);
 	if (!error) {
-		TAILQ_INSERT_TAIL(&mountlist, mp, mnt_list);
+		CIRCLEQ_INSERT_TAIL(&mountlist, mp, mnt_list);
 		VOP_UNLOCK(vp);
 		vfs_unlock(mp);
 		error = VFS_START(mp, 0, p);
+		if (error)
+			vrele(vp);
 	} else {
 		mp->mnt_vnodecovered->v_mountedhere = (struct mount *)0;
 		vfs_unlock(mp);
@@ -210,10 +227,12 @@ update:
  * Note: unmount takes a path to the vnode mounted on as argument,
  * not special file (as before).
  */
+#ifndef _SYS_SYSPROTO_H_
 struct unmount_args {
 	char	*path;
 	int	flags;
 };
+#endif
 /* ARGSUSED */
 int
 unmount(p, uap, retval)
@@ -272,15 +291,20 @@ dounmount(mp, flags, p)
 {
 	struct vnode *coveredvp;
 	int error;
+	int async_flag;
 
 	coveredvp = mp->mnt_vnodecovered;
 	if (vfs_busy(mp))
 		return (EBUSY);
 	mp->mnt_flag |= MNT_UNMOUNT;
 	error = vfs_lock(mp);
-	if (error)
+	if (error) {
+		mp->mnt_flag &= ~MNT_UNMOUNT;
+		vfs_unbusy(mp);
 		return (error);
+	}
 
+	async_flag = mp->mnt_flag & MNT_ASYNC;
 	mp->mnt_flag &=~ MNT_ASYNC;
 	vfs_msync(mp, MNT_NOWAIT);
 	vnode_pager_umount(mp);	/* release cached vnodes */
@@ -289,12 +313,13 @@ dounmount(mp, flags, p)
 	    (flags & MNT_FORCE))
 		error = VFS_UNMOUNT(mp, flags, p);
 	mp->mnt_flag &= ~MNT_UNMOUNT;
+	mp->mnt_flag |= async_flag;
 	vfs_unbusy(mp);
 	if (error) {
 		vfs_unlock(mp);
 	} else {
 		vrele(coveredvp);
-		TAILQ_REMOVE(&mountlist, mp, mnt_list);
+		CIRCLEQ_REMOVE(&mountlist, mp, mnt_list);
 		mp->mnt_vnodecovered->v_mountedhere = (struct mount *)0;
 		vfs_unlock(mp);
 		mp->mnt_vfc->vfc_refcount--;
@@ -308,9 +333,11 @@ dounmount(mp, flags, p)
 /*
  * Sync each mounted filesystem.
  */
-#ifdef DIAGNOSTIC
-int syncprt = 0;
-struct ctldebug debug0 = { "syncprt", &syncprt };
+
+#ifndef _SYS_SYSPROTO_H_
+struct sync_args {
+        int     dummy;
+};
 #endif
 
 /* ARGSUSED */
@@ -323,7 +350,7 @@ sync(p, uap, retval)
 	register struct mount *mp;
 	int asyncflag;
 
-	for (mp = mountlist.tqh_first; mp != NULL; mp = mp->mnt_list.tqe_next) {
+	for (mp = mountlist.cqh_first; mp != (void *)&mountlist; mp = mp->mnt_list.cqe_next) {
 		/*
 		 * The lock check below is to avoid races with mount
 		 * and unmount.
@@ -345,12 +372,14 @@ sync(p, uap, retval)
 /*
  * Change filesystem quotas.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct quotactl_args {
 	char *path;
 	int cmd;
 	int uid;
 	caddr_t arg;
 };
+#endif
 /* ARGSUSED */
 int
 quotactl(p, uap, retval)
@@ -374,10 +403,12 @@ quotactl(p, uap, retval)
 /*
  * Get filesystem statistics.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct statfs_args {
 	char *path;
 	struct statfs *buf;
 };
+#endif
 /* ARGSUSED */
 int
 statfs(p, uap, retval)
@@ -413,10 +444,12 @@ statfs(p, uap, retval)
 /*
  * Get filesystem statistics.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fstatfs_args {
 	int fd;
 	struct statfs *buf;
 };
+#endif
 /* ARGSUSED */
 int
 fstatfs(p, uap, retval)
@@ -450,11 +483,13 @@ fstatfs(p, uap, retval)
 /*
  * Get statistics on all filesystems.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct getfsstat_args {
 	struct statfs *buf;
 	long bufsize;
 	int flags;
 };
+#endif
 int
 getfsstat(p, uap, retval)
 	struct proc *p;
@@ -468,9 +503,10 @@ getfsstat(p, uap, retval)
 
 	maxcount = uap->bufsize / sizeof(struct statfs);
 	sfsp = (caddr_t)uap->buf;
-	for (count = 0, mp = mountlist.tqh_first; mp != NULL; mp = nmp) {
+	count = 0;
+	for (mp = mountlist.cqh_first; mp != (void *)&mountlist; mp = nmp) {
 		if (vfs_busy(mp)) {
-			nmp = mp->mnt_list.tqe_next;
+			nmp = mp->mnt_list.cqe_next;
 			continue;
 		}
 		if (sfsp && count < maxcount &&
@@ -483,7 +519,7 @@ getfsstat(p, uap, retval)
 			if (((uap->flags & MNT_NOWAIT) == 0 ||
 			    (uap->flags & MNT_WAIT)) &&
 			    (error = VFS_STATFS(mp, sp, p))) {
-				nmp = mp->mnt_list.tqe_next;
+				nmp = mp->mnt_list.cqe_next;
 				vfs_unbusy(mp);
 				continue;
 			}
@@ -496,7 +532,7 @@ getfsstat(p, uap, retval)
 			sfsp += sizeof(*sp);
 		}
 		count++;
-		nmp = mp->mnt_list.tqe_next;
+		nmp = mp->mnt_list.cqe_next;
 		vfs_unbusy(mp);
 	}
 	if (sfsp && count > maxcount)
@@ -509,9 +545,11 @@ getfsstat(p, uap, retval)
 /*
  * Change current working directory to a given file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fchdir_args {
 	int	fd;
 };
+#endif
 /* ARGSUSED */
 int
 fchdir(p, uap, retval)
@@ -545,9 +583,11 @@ fchdir(p, uap, retval)
 /*
  * Change current working directory (``.'').
  */
+#ifndef _SYS_SYSPROTO_H_
 struct chdir_args {
 	char	*path;
 };
+#endif
 /* ARGSUSED */
 int
 chdir(p, uap, retval)
@@ -571,9 +611,11 @@ chdir(p, uap, retval)
 /*
  * Change notion of root (``/'') directory.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct chroot_args {
 	char	*path;
 };
+#endif
 /* ARGSUSED */
 int
 chroot(p, uap, retval)
@@ -627,11 +669,13 @@ change_dir(ndp, p)
  * Check permissions, allocate an open file structure,
  * and call the device open routine if any.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct open_args {
 	char	*path;
 	int	flags;
 	int	mode;
 };
+#endif
 int
 open(p, uap, retval)
 	struct proc *p;
@@ -650,6 +694,8 @@ open(p, uap, retval)
 	flags = FFLAGS(uap->flags);
 	if ((flags & FREAD + FWRITE) == 0)
 		return (EINVAL);
+	if (flags & O_NOFOLLOW)
+		flags &= ~O_NOFOLLOW;
 	error = falloc(p, &nfp, &indx);
 	if (error)
 		return (error);
@@ -674,8 +720,9 @@ open(p, uap, retval)
 	}
 	p->p_dupfd = 0;
 	vp = nd.ni_vp;
+
 	fp->f_flag = flags & FMASK;
-	fp->f_type = DTYPE_VNODE;
+	fp->f_type = (vp->v_type == VFIFO ? DTYPE_FIFO : DTYPE_VNODE);
 	fp->f_ops = &vnops;
 	fp->f_data = (caddr_t)vp;
 	if (flags & (O_EXLOCK | O_SHLOCK)) {
@@ -709,10 +756,12 @@ open(p, uap, retval)
 /*
  * Create a file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ocreat_args {
 	char	*path;
 	int	mode;
 };
+#endif
 int
 ocreat(p, uap, retval)
 	struct proc *p;
@@ -731,11 +780,13 @@ ocreat(p, uap, retval)
 /*
  * Create a special file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct mknod_args {
 	char	*path;
 	int	mode;
 	int	dev;
 };
+#endif
 /* ARGSUSED */
 int
 mknod(p, uap, retval)
@@ -796,10 +847,12 @@ mknod(p, uap, retval)
 /*
  * Create named pipe.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct mkfifo_args {
 	char	*path;
 	int	mode;
 };
+#endif
 /* ARGSUSED */
 int
 mkfifo(p, uap, retval)
@@ -834,10 +887,12 @@ mkfifo(p, uap, retval)
 /*
  * Make a hard file link.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct link_args {
 	char	*path;
 	char	*link;
 };
+#endif
 /* ARGSUSED */
 int
 link(p, uap, retval)
@@ -854,24 +909,13 @@ link(p, uap, retval)
 	if (error)
 		return (error);
 	vp = nd.ni_vp;
-	if (vp->v_type != VDIR ||
-	    (error = suser(p->p_ucred, &p->p_acflag)) == 0) {
-		nd.ni_cnd.cn_nameiop = CREATE;
-		nd.ni_cnd.cn_flags = LOCKPARENT;
-		if (vp->v_type == VDIR)
-			nd.ni_cnd.cn_flags |= WILLBEDIR;
-		nd.ni_dirp = uap->link;
+	if (vp->v_type == VDIR)
+		error = EPERM;		/* POSIX */
+	else {
+		NDINIT(&nd, CREATE, LOCKPARENT, UIO_USERSPACE, uap->link, p);
 		error = namei(&nd);
 		if (!error) {
-			if (nd.ni_vp != NULL)
-				error = EEXIST;
-			if (!error) {
-				LEASE_CHECK(nd.ni_dvp,
-				    p, p->p_ucred, LEASE_WRITE);
-				LEASE_CHECK(vp,
-				    p, p->p_ucred, LEASE_WRITE);
-				error = VOP_LINK(nd.ni_dvp, vp, &nd.ni_cnd);
-			} else {
+			if (nd.ni_vp != NULL) {
 				VOP_ABORTOP(nd.ni_dvp, &nd.ni_cnd);
 				if (nd.ni_dvp == nd.ni_vp)
 					vrele(nd.ni_dvp);
@@ -879,6 +923,13 @@ link(p, uap, retval)
 					vput(nd.ni_dvp);
 				if (nd.ni_vp)
 					vrele(nd.ni_vp);
+				error = EEXIST;
+			} else {
+				LEASE_CHECK(nd.ni_dvp,
+				    p, p->p_ucred, LEASE_WRITE);
+				LEASE_CHECK(vp,
+				    p, p->p_ucred, LEASE_WRITE);
+				error = VOP_LINK(nd.ni_dvp, vp, &nd.ni_cnd);
 			}
 		}
 	}
@@ -889,10 +940,12 @@ link(p, uap, retval)
 /*
  * Make a symbolic link.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct symlink_args {
 	char	*path;
 	char	*link;
 };
+#endif
 /* ARGSUSED */
 int
 symlink(p, uap, retval)
@@ -935,9 +988,11 @@ out:
 /*
  * Delete a name from the filesystem.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct unlink_args {
 	char	*path;
 };
+#endif
 /* ARGSUSED */
 int
 unlink(p, uap, retval)
@@ -957,10 +1012,13 @@ unlink(p, uap, retval)
 	LEASE_CHECK(vp, p, p->p_ucred, LEASE_WRITE);
 	VOP_LOCK(vp);
 
-	if (vp->v_type != VDIR ||
-	    (error = suser(p->p_ucred, &p->p_acflag)) == 0) {
+	if (vp->v_type == VDIR)
+		error = EPERM;		/* POSIX */
+	else {
 		/*
 		 * The root of a mounted filesystem cannot be deleted.
+		 *
+		 * XXX: can this only be a VDIR case?
 		 */
 		if (vp->v_flag & VROOT)
 			error = EBUSY;
@@ -985,12 +1043,14 @@ unlink(p, uap, retval)
 /*
  * Reposition read/write file offset.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct lseek_args {
 	int	fd;
 	int	pad;
 	off_t	offset;
 	int	whence;
 };
+#endif
 int
 lseek(p, uap, retval)
 	struct proc *p;
@@ -1032,11 +1092,13 @@ lseek(p, uap, retval)
 /*
  * Reposition read/write file offset.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct olseek_args {
 	int	fd;
 	long	offset;
 	int	whence;
 };
+#endif
 int
 olseek(p, uap, retval)
 	struct proc *p;
@@ -1050,7 +1112,7 @@ olseek(p, uap, retval)
 	nuap.fd = uap->fd;
 	nuap.offset = uap->offset;
 	nuap.whence = uap->whence;
-	error = lseek(p, &nuap, &qret);
+	error = lseek(p, &nuap, (int *)&qret);
 	*(long *)retval = qret;
 	return (error);
 }
@@ -1059,10 +1121,12 @@ olseek(p, uap, retval)
 /*
  * Check access permissions.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct access_args {
 	char	*path;
 	int	flags;
 };
+#endif
 int
 access(p, uap, retval)
 	struct proc *p;
@@ -1107,10 +1171,12 @@ out1:
 /*
  * Get file status; this version follows links.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ostat_args {
 	char	*path;
 	struct ostat *ub;
 };
+#endif
 /* ARGSUSED */
 int
 ostat(p, uap, retval)
@@ -1139,10 +1205,12 @@ ostat(p, uap, retval)
 /*
  * Get file status; this version does not follow links.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct olstat_args {
 	char	*path;
 	struct ostat *ub;
 };
+#endif
 /* ARGSUSED */
 int
 olstat(p, uap, retval)
@@ -1150,49 +1218,24 @@ olstat(p, uap, retval)
 	register struct olstat_args *uap;
 	int *retval;
 {
-	struct vnode *vp, *dvp;
-	struct stat sb, sb1;
+	struct vnode *vp;
+	struct stat sb;
 	struct ostat osb;
 	int error;
 	struct nameidata nd;
 
-	NDINIT(&nd, LOOKUP, NOFOLLOW | LOCKLEAF | LOCKPARENT, UIO_USERSPACE,
+	NDINIT(&nd, LOOKUP, NOFOLLOW | LOCKLEAF, UIO_USERSPACE,
 	    uap->path, p);
 	error = namei(&nd);
 	if (error)
 		return (error);
-	/*
-	 * For symbolic links, always return the attributes of its
-	 * containing directory, except for mode, size, and links.
-	 */
 	vp = nd.ni_vp;
-	dvp = nd.ni_dvp;
-	if (vp->v_type != VLNK) {
-		if (dvp == vp)
-			vrele(dvp);
-		else
-			vput(dvp);
-		error = vn_stat(vp, &sb, p);
-		vput(vp);
-		if (error)
-			return (error);
-	} else {
-		error = vn_stat(dvp, &sb, p);
-		vput(dvp);
-		if (error) {
-			vput(vp);
-			return (error);
-		}
-		error = vn_stat(vp, &sb1, p);
-		vput(vp);
-		if (error)
-			return (error);
-		sb.st_mode &= ~S_IFDIR;
-		sb.st_mode |= S_IFLNK;
-		sb.st_nlink = sb1.st_nlink;
-		sb.st_size = sb1.st_size;
-		sb.st_blocks = sb1.st_blocks;
-	}
+	error = vn_stat(vp, &sb, p);
+	if (vp->v_type == VLNK)
+		sb.st_mode |= S_IFLNK | ACCESSPERMS;	/* 0777 */
+	vput(vp);
+	if (error)
+		return (error);
 	cvtstat(&sb, &osb);
 	error = copyout((caddr_t)&osb, (caddr_t)uap->ub, sizeof (osb));
 	return (error);
@@ -1231,10 +1274,12 @@ cvtstat(st, ost)
 /*
  * Get file status; this version follows links.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct stat_args {
 	char	*path;
 	struct stat *ub;
 };
+#endif
 /* ARGSUSED */
 int
 stat(p, uap, retval)
@@ -1261,10 +1306,12 @@ stat(p, uap, retval)
 /*
  * Get file status; this version does not follow links.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct lstat_args {
 	char	*path;
 	struct stat *ub;
 };
+#endif
 /* ARGSUSED */
 int
 lstat(p, uap, retval)
@@ -1273,47 +1320,22 @@ lstat(p, uap, retval)
 	int *retval;
 {
 	int error;
-	struct vnode *vp, *dvp;
-	struct stat sb, sb1;
+	struct vnode *vp;
+	struct stat sb;
 	struct nameidata nd;
 
-	NDINIT(&nd, LOOKUP, NOFOLLOW | LOCKLEAF | LOCKPARENT, UIO_USERSPACE,
+	NDINIT(&nd, LOOKUP, NOFOLLOW | LOCKLEAF, UIO_USERSPACE,
 	    uap->path, p);
 	error = namei(&nd);
 	if (error)
 		return (error);
-	/*
-	 * For symbolic links, always return the attributes of its
-	 * containing directory, except for mode, size, and links.
-	 */
 	vp = nd.ni_vp;
-	dvp = nd.ni_dvp;
-	if (vp->v_type != VLNK) {
-		if (dvp == vp)
-			vrele(dvp);
-		else
-			vput(dvp);
-		error = vn_stat(vp, &sb, p);
-		vput(vp);
-		if (error)
-			return (error);
-	} else {
-		error = vn_stat(dvp, &sb, p);
-		vput(dvp);
-		if (error) {
-			vput(vp);
-			return (error);
-		}
-		error = vn_stat(vp, &sb1, p);
-		vput(vp);
-		if (error)
-			return (error);
-		sb.st_mode &= ~S_IFDIR;
-		sb.st_mode |= S_IFLNK;
-		sb.st_nlink = sb1.st_nlink;
-		sb.st_size = sb1.st_size;
-		sb.st_blocks = sb1.st_blocks;
-	}
+	error = vn_stat(vp, &sb, p);
+	if (vp->v_type == VLNK)
+		sb.st_mode |= S_IFLNK | ACCESSPERMS;	/* 0777 */
+	vput(vp);
+	if (error)
+		return (error);
 	error = copyout((caddr_t)&sb, (caddr_t)uap->ub, sizeof (sb));
 	return (error);
 }
@@ -1321,10 +1343,12 @@ lstat(p, uap, retval)
 /*
  * Get configurable pathname variables.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct pathconf_args {
 	char	*path;
 	int	name;
 };
+#endif
 /* ARGSUSED */
 int
 pathconf(p, uap, retval)
@@ -1347,11 +1371,13 @@ pathconf(p, uap, retval)
 /*
  * Return target name of a symbolic link.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct readlink_args {
 	char	*path;
 	char	*buf;
 	int	count;
 };
+#endif
 /* ARGSUSED */
 int
 readlink(p, uap, retval)
@@ -1392,10 +1418,12 @@ readlink(p, uap, retval)
 /*
  * Change flags of a file given a path name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct chflags_args {
 	char	*path;
 	int	flags;
 };
+#endif
 /* ARGSUSED */
 int
 chflags(p, uap, retval)
@@ -1413,6 +1441,15 @@ chflags(p, uap, retval)
 	if (error)
 		return (error);
 	vp = nd.ni_vp;
+	/*
+	 * Prevent non-root users from setting flags on devices.  When
+	 * a device is reused, users can retain ownership of the device
+	 * if they are allowed to set flags and programs assume that
+	 * chown can't fail when done as root.
+	 */
+	if ((vp->v_type == VCHR || vp->v_type == VBLK) &&
+	    ((error = suser(p->p_ucred, &p->p_acflag)) != 0))
+		return (error);
 	LEASE_CHECK(vp, p, p->p_ucred, LEASE_WRITE);
 	VOP_LOCK(vp);
 	VATTR_NULL(&vattr);
@@ -1425,10 +1462,12 @@ chflags(p, uap, retval)
 /*
  * Change flags of a file given a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fchflags_args {
 	int	fd;
 	int	flags;
 };
+#endif
 /* ARGSUSED */
 int
 fchflags(p, uap, retval)
@@ -1457,10 +1496,12 @@ fchflags(p, uap, retval)
 /*
  * Change mode of a file given path name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct chmod_args {
 	char	*path;
 	int	mode;
 };
+#endif
 /* ARGSUSED */
 int
 chmod(p, uap, retval)
@@ -1490,10 +1531,12 @@ chmod(p, uap, retval)
 /*
  * Change mode of a file given a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fchmod_args {
 	int	fd;
 	int	mode;
 };
+#endif
 /* ARGSUSED */
 int
 fchmod(p, uap, retval)
@@ -1522,11 +1565,13 @@ fchmod(p, uap, retval)
 /*
  * Set ownership given a path name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct chown_args {
 	char	*path;
 	int	uid;
 	int	gid;
 };
+#endif
 /* ARGSUSED */
 int
 chown(p, uap, retval)
@@ -1554,14 +1599,53 @@ chown(p, uap, retval)
 	return (error);
 }
 
+/*
+ * Set ownership given a path name, do not cross symlinks.
+ */
+#ifndef _SYS_SYSPROTO_H_
+struct lchown_args {
+	char	*path;
+	int	uid;
+	int	gid;
+};
+#endif
+/* ARGSUSED */
+int
+lchown(p, uap, retval)
+	struct proc *p;
+	register struct lchown_args *uap;
+	int *retval;
+{
+	register struct vnode *vp;
+	struct vattr vattr;
+	int error;
+	struct nameidata nd;
+
+	NDINIT(&nd, LOOKUP, NOFOLLOW, UIO_USERSPACE, uap->path, p);
+	error = namei(&nd);
+	if (error)
+		return (error);
+	vp = nd.ni_vp;
+	LEASE_CHECK(vp, p, p->p_ucred, LEASE_WRITE);
+	VOP_LOCK(vp);
+	VATTR_NULL(&vattr);
+	vattr.va_uid = uap->uid;
+	vattr.va_gid = uap->gid;
+	error = VOP_SETATTR(vp, &vattr, p->p_ucred, p);
+	vput(vp);
+	return (error);
+}
+
 /*
  * Set ownership given a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fchown_args {
 	int	fd;
 	int	uid;
 	int	gid;
 };
+#endif
 /* ARGSUSED */
 int
 fchown(p, uap, retval)
@@ -1591,10 +1675,12 @@ fchown(p, uap, retval)
 /*
  * Set the access and modification times of a file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct utimes_args {
 	char	*path;
 	struct	timeval *tptr;
 };
+#endif
 /* ARGSUSED */
 int
 utimes(p, uap, retval)
@@ -1625,10 +1711,10 @@ utimes(p, uap, retval)
 	vp = nd.ni_vp;
 	LEASE_CHECK(vp, p, p->p_ucred, LEASE_WRITE);
 	VOP_LOCK(vp);
-	vattr.va_atime.ts_sec = tv[0].tv_sec;
-	vattr.va_atime.ts_nsec = tv[0].tv_usec * 1000;
-	vattr.va_mtime.ts_sec = tv[1].tv_sec;
-	vattr.va_mtime.ts_nsec = tv[1].tv_usec * 1000;
+	vattr.va_atime.tv_sec = tv[0].tv_sec;
+	vattr.va_atime.tv_nsec = tv[0].tv_usec * 1000;
+	vattr.va_mtime.tv_sec = tv[1].tv_sec;
+	vattr.va_mtime.tv_nsec = tv[1].tv_usec * 1000;
 	error = VOP_SETATTR(vp, &vattr, p->p_ucred, p);
 	vput(vp);
 	return (error);
@@ -1637,11 +1723,13 @@ utimes(p, uap, retval)
 /*
  * Truncate a file given its path name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct truncate_args {
 	char	*path;
 	int	pad;
 	off_t	length;
 };
+#endif
 /* ARGSUSED */
 int
 truncate(p, uap, retval)
@@ -1678,11 +1766,13 @@ truncate(p, uap, retval)
 /*
  * Truncate a file given a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ftruncate_args {
 	int	fd;
 	int	pad;
 	off_t	length;
 };
+#endif
 /* ARGSUSED */
 int
 ftruncate(p, uap, retval)
@@ -1720,10 +1810,12 @@ ftruncate(p, uap, retval)
 /*
  * Truncate a file given its path name.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct otruncate_args {
 	char	*path;
 	long	length;
 };
+#endif
 /* ARGSUSED */
 int
 otruncate(p, uap, retval)
@@ -1741,10 +1833,12 @@ otruncate(p, uap, retval)
 /*
  * Truncate a file given a file descriptor.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct oftruncate_args {
 	int	fd;
 	long	length;
 };
+#endif
 /* ARGSUSED */
 int
 oftruncate(p, uap, retval)
@@ -1763,9 +1857,11 @@ oftruncate(p, uap, retval)
 /*
  * Sync an open file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct fsync_args {
 	int	fd;
 };
+#endif
 /* ARGSUSED */
 int
 fsync(p, uap, retval)
@@ -1782,10 +1878,12 @@ fsync(p, uap, retval)
 		return (error);
 	vp = (struct vnode *)fp->f_data;
 	VOP_LOCK(vp);
-	if (vp->v_vmdata) {
-		_vm_object_page_clean((vm_object_t) vp->v_vmdata, 0, 0 ,0);
+	if (vp->v_object) {
+		vm_object_page_clean(vp->v_object, 0, 0 ,0, FALSE);
 	}
-	error = VOP_FSYNC(vp, fp->f_cred, MNT_WAIT, p);
+	error = VOP_FSYNC(vp, fp->f_cred,
+		(vp->v_mount && (vp->v_mount->mnt_flag & MNT_ASYNC)) ? 
+		MNT_NOWAIT : MNT_WAIT, p);
 	VOP_UNLOCK(vp);
 	return (error);
 }
@@ -1794,10 +1892,12 @@ fsync(p, uap, retval)
  * Rename files.  Source and destination must either both be directories,
  * or both not be directories.  If target is a directory, it must be empty.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct rename_args {
 	char	*from;
 	char	*to;
 };
+#endif
 /* ARGSUSED */
 int
 rename(p, uap, retval)
@@ -1821,6 +1921,10 @@ rename(p, uap, retval)
 		tond.ni_cnd.cn_flags |= WILLBEDIR;
 	error = namei(&tond);
 	if (error) {
+		/* Translate error code for rename("dir1", "dir2/."). */
+		if (error == EISDIR && fvp->v_type == VDIR)
+			error = EINVAL;
+
 		VOP_ABORTOP(fromnd.ni_dvp, &fromnd.ni_cnd);
 		vrele(fromnd.ni_dvp);
 		vrele(fvp);
@@ -1887,10 +1991,12 @@ out1:
 /*
  * Make a directory file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct mkdir_args {
 	char	*path;
 	int	mode;
 };
+#endif
 /* ARGSUSED */
 int
 mkdir(p, uap, retval)
@@ -1931,9 +2037,11 @@ mkdir(p, uap, retval)
 /*
  * Remove a directory file.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct rmdir_args {
 	char	*path;
 };
+#endif
 /* ARGSUSED */
 int
 rmdir(p, uap, retval)
@@ -1986,12 +2094,14 @@ out:
 /*
  * Read a block of directory entries in a file system independent format.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct ogetdirentries_args {
 	int	fd;
 	char	*buf;
 	u_int	count;
 	long	*basep;
 };
+#endif
 int
 ogetdirentries(p, uap, retval)
 	struct proc *p;
@@ -2086,12 +2196,14 @@ ogetdirentries(p, uap, retval)
 /*
  * Read a block of directory entries in a file system independent format.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct getdirentries_args {
 	int	fd;
 	char	*buf;
 	u_int	count;
 	long	*basep;
 };
+#endif
 int
 getdirentries(p, uap, retval)
 	struct proc *p;
@@ -2177,9 +2289,11 @@ unionread:
 /*
  * Set the mode mask for creation of filesystem nodes.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct umask_args {
 	int	newmask;
 };
+#endif
 mode_t				/* XXX */
 umask(p, uap, retval)
 	struct proc *p;
@@ -2198,9 +2312,11 @@ umask(p, uap, retval)
  * Void all references to file by ripping underlying filesystem
  * away from vnode.
  */
+#ifndef _SYS_SYSPROTO_H_
 struct revoke_args {
 	char	*path;
 };
+#endif
 /* ARGSUSED */
 int
 revoke(p, uap, retval)
@@ -2241,15 +2357,15 @@ out:
 int
 getvnode(fdp, fd, fpp)
 	struct filedesc *fdp;
-	struct file **fpp;
 	int fd;
+	struct file **fpp;
 {
 	struct file *fp;
 
 	if ((u_int)fd >= fdp->fd_nfiles ||
 	    (fp = fdp->fd_ofiles[fd]) == NULL)
 		return (EBADF);
-	if (fp->f_type != DTYPE_VNODE)
+	if (fp->f_type != DTYPE_VNODE && fp->f_type != DTYPE_FIFO)
 		return (EINVAL);
 	*fpp = fp;
 	return (0);
