@@ -34,10 +34,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/libkern/divdi3.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/libkern/divdi3.c 50950 1999-09-05 08:39:22Z peter $
  */
 
-#include "quad.h"
+#include <libkern/quad.h>
 
 /*
  * Divide two signed quads.
