@@ -29,14 +29,29 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $FreeBSD: stable/2.1/sys/libkern/mcount.c 50950 1999-09-05 08:39:22Z peter $
  */
 
+#if !defined(lint) && !defined(KERNEL) && defined(LIBC_SCCS)
+#if 0
+static char sccsid[] = "@(#)mcount.c	8.1 (Berkeley) 6/4/93";
+#endif
+static const char rcsid[] =
+  "$FreeBSD: stable/2.2/sys/libkern/mcount.c 50950 1999-09-05 08:39:22Z peter $";
+#endif
+
 #include <sys/param.h>
 #include <sys/gmon.h>
 #ifdef KERNEL
-#include <i386/include/cpufunc.h>
+#ifndef GUPROF
+#include <sys/systm.h>
+#endif
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
+void	bintr __P((void));
+void	btrap __P((void));
+void	eintr __P((void));
+void	user __P((void));
 #endif
 
 /*
@@ -55,39 +70,109 @@
  * perform this optimization.
  */
 _MCOUNT_DECL(frompc, selfpc)	/* _mcount; may be static, inline, etc */
-	register u_long frompc, selfpc;
+	register fptrint_t frompc, selfpc;
 {
+#ifdef GUPROF
+	int delta;
+#endif
+	register fptrdiff_t frompci;
 	register u_short *frompcindex;
 	register struct tostruct *top, *prevtop;
 	register struct gmonparam *p;
 	register long toindex;
 #ifdef KERNEL
-	register int s;
-	u_long save_eflags;
+	MCOUNT_DECL(s)
 #endif
 
 	p = &_gmonparam;
+#ifndef GUPROF			/* XXX */
 	/*
 	 * check that we are profiling
 	 * and that we aren't recursively invoked.
 	 */
 	if (p->state != GMON_PROF_ON)
 		return;
+#endif
 #ifdef KERNEL
-	MCOUNT_ENTER;
+	MCOUNT_ENTER(s);
 #else
 	p->state = GMON_PROF_BUSY;
 #endif
+	frompci = frompc - p->lowpc;
+
+#ifdef KERNEL
 	/*
-	 * check that frompcindex is a reasonable pc value.
+	 * When we are called from an exception handler, frompci may be
+	 * for a user address.  Convert such frompci's to the index of
+	 * user() to merge all user counts.
+	 */
+	if (frompci >= p->textsize) {
+		if (frompci + p->lowpc
+		    >= (fptrint_t)(VM_MAXUSER_ADDRESS + UPAGES * PAGE_SIZE))
+			goto done;
+		frompci = (fptrint_t)user - p->lowpc;
+		if (frompci >= p->textsize)
+		    goto done;
+	}
+#endif /* KERNEL */
+
+#ifdef GUPROF
+	if (p->state == GMON_PROF_HIRES) {
+		/*
+		 * Count the time since cputime() was previously called
+		 * against `frompc'.  Compensate for overheads.
+		 *
+		 * cputime() sets its prev_count variable to the count when
+		 * it is called.  This in effect starts a counter for
+		 * the next period of execution (normally from now until 
+		 * the next call to mcount() or mexitcount()).  We set
+		 * cputime_bias to compensate for our own overhead.
+		 *
+		 * We use the usual sampling counters since they can be
+		 * located efficiently.  4-byte counters are usually
+		 * necessary.  gprof will add up the scattered counts
+		 * just like it does for statistical profiling.  All
+		 * counts are signed so that underflow in the subtractions
+		 * doesn't matter much (negative counts are normally
+		 * compensated for by larger counts elsewhere).  Underflow
+		 * shouldn't occur, but may be caused by slightly wrong
+		 * calibrations or from not clearing cputime_bias.
+		 */
+		delta = cputime() - cputime_bias - p->mcount_pre_overhead;
+		cputime_bias = p->mcount_post_overhead;
+		KCOUNT(p, frompci) += delta;
+		*p->cputime_count += p->cputime_overhead;
+		*p->mcount_count += p->mcount_overhead;
+	}
+#endif /* GUPROF */
+
+#ifdef KERNEL
+	/*
+	 * When we are called from an exception handler, frompc is faked
+	 * to be for where the exception occurred.  We've just solidified
+	 * the count for there.  Now convert frompci to the index of btrap()
+	 * for trap handlers and bintr() for interrupt handlers to make
+	 * exceptions appear in the call graph as calls from btrap() and
+	 * bintr() instead of calls from all over.
+	 */
+	if ((fptrint_t)selfpc >= (fptrint_t)btrap
+	    && (fptrint_t)selfpc < (fptrint_t)eintr) {
+		if ((fptrint_t)selfpc >= (fptrint_t)bintr)
+			frompci = (fptrint_t)bintr - p->lowpc;
+		else
+			frompci = (fptrint_t)btrap - p->lowpc;
+	}
+#endif /* KERNEL */
+
+	/*
+	 * check that frompc is a reasonable pc value.
 	 * for example:	signal catchers get called from the stack,
 	 *		not from text space.  too bad.
 	 */
-	frompc -= p->lowpc;
-	if (frompc > p->textsize)
+	if (frompci >= p->textsize)
 		goto done;
 
-	frompcindex = &p->froms[frompc / (p->hashfraction * sizeof(*p->froms))];
+	frompcindex = &p->froms[frompci / (p->hashfraction * sizeof(*p->froms))];
 	toindex = *frompcindex;
 	if (toindex == 0) {
 		/*
@@ -160,7 +245,7 @@ _MCOUNT_DECL(frompc, selfpc)	/* _mcount; may be static, inline, etc */
 	}
 done:
 #ifdef KERNEL
-	MCOUNT_EXIT;
+	MCOUNT_EXIT(s);
 #else
 	p->state = GMON_PROF_ON;
 #endif
@@ -168,7 +253,7 @@ done:
 overflow:
 	p->state = GMON_PROF_ERROR;
 #ifdef KERNEL
-	MCOUNT_EXIT;
+	MCOUNT_EXIT(s);
 #endif
 	return;
 }
@@ -178,3 +263,52 @@ overflow:
  * which is included by <sys/gmon.h>.
  */
 MCOUNT
+
+#ifdef GUPROF
+void
+mexitcount(selfpc)
+	fptrint_t selfpc;
+{
+	struct gmonparam *p;
+	fptrint_t selfpcdiff;
+
+	p = &_gmonparam;
+	selfpcdiff = selfpc - (fptrint_t)p->lowpc;
+	if (selfpcdiff < p->textsize) {
+		int delta;
+
+		/*
+		 * Count the time since cputime() was previously called
+		 * against `selfpc'.  Compensate for overheads.
+		 */
+		delta = cputime() - cputime_bias - p->mexitcount_pre_overhead;
+		cputime_bias = p->mexitcount_post_overhead;
+		KCOUNT(p, selfpcdiff) += delta;
+		*p->cputime_count += p->cputime_overhead;
+		*p->mexitcount_count += p->mexitcount_overhead;
+	}
+}
+
+void
+empty_loop()
+{
+	int i;
+
+	for (i = 0; i < CALIB_SCALE; i++)
+		;
+}
+
+void
+nullfunc()
+{
+}
+
+void
+nullfunc_loop()
+{
+	int i;
+
+	for (i = 0; i < CALIB_SCALE; i++)
+		nullfunc();
+}
+#endif /* GUPROF */
