@@ -34,7 +34,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $FreeBSD: stable/2.1/sys/libkern/qdivrem.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/libkern/qdivrem.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -42,7 +42,7 @@
  * section 4.3.1, pp. 257--259.
  */
 
-#include "quad.h"
+#include <libkern/quad.h>
 
 #define	B	(1 << HALF_BITS)	/* digit base */
 
