@@ -31,11 +31,20 @@
  * SUCH DAMAGE.
  *
  *	@(#)random.c	8.1 (Berkeley) 6/10/93
- * $FreeBSD: stable/2.1/sys/libkern/random.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/libkern/random.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/libkern.h>
 
+static u_long randseed = 1;
+
+void
+srandom(seed)
+	u_long seed;
+{
+	randseed = seed;
+}
+
 /*
  * Pseudo-random number generator for randomizing the profiling clock,
  * and whatever else we might use it for.  The result is uniform on
@@ -44,7 +53,6 @@
 u_long
 random()
 {
-	static u_long randseed = 1;
 	register long x, hi, lo, t;
 
 	/*
