@@ -35,7 +35,7 @@
  *
  *	@(#)fdesc.h	8.5 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/fdesc/fdesc.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/fdesc/fdesc.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifdef KERNEL
@@ -77,6 +77,6 @@ extern dev_t devctty;
 extern int fdesc_init __P((void));
 extern int fdesc_root __P((struct mount *, struct vnode **));
 extern int fdesc_allocvp __P((fdntype, int, struct mount *, struct vnode **));
-extern int (**fdesc_vnodeop_p)();
+extern vop_t **fdesc_vnodeop_p;
 extern struct vfsops fdesc_vfsops;
 #endif /* KERNEL */
