@@ -35,7 +35,7 @@
  *
  *	@(#)fdesc_vnops.c	8.9 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/fdesc/fdesc_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/fdesc/fdesc_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -44,12 +44,11 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <sys/types.h>
-#include <sys/time.h>
 #include <sys/proc.h>
 #include <sys/kernel.h>	/* boottime */
 #include <sys/resourcevar.h>
 #include <sys/filedesc.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/malloc.h>
 #include <sys/file.h>
@@ -60,8 +59,11 @@
 #include <sys/dirent.h>
 #include <sys/socketvar.h>
 #include <sys/tty.h>
+#include <sys/conf.h>
 #include <miscfs/fdesc/fdesc.h>
 
+extern	struct cdevsw ctty_cdevsw;
+
 #define cttyvp(p) ((p)->p_flag & P_CONTROLT ? (p)->p_session->s_ttyvp : NULL)
 
 #define FDL_WANT	0x01
@@ -87,6 +89,28 @@ struct fdcache {
 
 static struct fdcache fdcache[NFDCACHE];
 
+static int	fdesc_attr __P((int fd, struct vattr *vap, struct ucred *cred,
+				struct proc *p));
+static int	fdesc_badop __P((void));
+static int	fdesc_enotsupp __P((void));
+static int	fdesc_getattr __P((struct vop_getattr_args *ap));
+static struct fdcache *
+		fdesc_hash __P((int ix));
+static int	fdesc_inactive __P((struct vop_inactive_args *ap));
+static int	fdesc_ioctl __P((struct vop_ioctl_args *ap));
+static int	fdesc_lookup __P((struct vop_lookup_args *ap));
+static int	fdesc_open __P((struct vop_open_args *ap));
+static int	fdesc_pathconf __P((struct vop_pathconf_args *ap));
+static int	fdesc_print __P((struct vop_print_args *ap));
+static int	fdesc_read __P((struct vop_read_args *ap));
+static int	fdesc_readdir __P((struct vop_readdir_args *ap));
+static int	fdesc_readlink __P((struct vop_readlink_args *ap));
+static int	fdesc_reclaim __P((struct vop_reclaim_args *ap));
+static int	fdesc_select __P((struct vop_select_args *ap));
+static int	fdesc_setattr __P((struct vop_setattr_args *ap));
+static int	fdesc_vfree __P((struct vop_vfree_args *ap));
+static int	fdesc_write __P((struct vop_write_args *ap));
+
 /*
  * Initialise cache headers
  */
@@ -182,7 +206,7 @@ out:;
  * vp is the current namei directory
  * ndp is the name to locate in that directory...
  */
-int
+static int
 fdesc_lookup(ap)
 	struct vop_lookup_args /* {
 		struct vnode * a_dvp;
@@ -323,7 +347,7 @@ bad:;
 	return (error);
 }
 
-int
+static int
 fdesc_open(ap)
 	struct vop_open_args /* {
 		struct vnode *a_vp;
@@ -350,7 +374,7 @@ fdesc_open(ap)
 		break;
 
 	case Fctty:
-		error = cttyopen(devctty, ap->a_mode, 0, ap->a_p);
+		error = (*ctty_cdevsw.d_open)(devctty, ap->a_mode, 0, ap->a_p);
 		break;
 	}
 
@@ -373,6 +397,7 @@ fdesc_attr(fd, vap, cred, p)
 		return (EBADF);
 
 	switch (fp->f_type) {
+	case DTYPE_FIFO:
 	case DTYPE_VNODE:
 		error = VOP_GETATTR((struct vnode *) fp->f_data, vap, cred, p);
 		if (error == 0 && vap->va_type == VDIR) {
@@ -415,7 +440,7 @@ fdesc_attr(fd, vap, cred, p)
 	return (error);
 }
 
-int
+static int
 fdesc_getattr(ap)
 	struct vop_getattr_args /* {
 		struct vnode *a_vp;
@@ -464,8 +489,8 @@ fdesc_getattr(ap)
 		vap->va_gid = 0;
 		vap->va_fsid = vp->v_mount->mnt_stat.f_fsid.val[0];
 		vap->va_blocksize = DEV_BSIZE;
-		vap->va_atime.ts_sec = boottime.tv_sec;
-		vap->va_atime.ts_nsec = 0;
+		vap->va_atime.tv_sec = boottime.tv_sec;
+		vap->va_atime.tv_nsec = 0;
 		vap->va_mtime = vap->va_atime;
 		vap->va_ctime = vap->va_mtime;
 		vap->va_gen = 0;
@@ -490,7 +515,7 @@ fdesc_getattr(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_setattr(ap)
 	struct vop_setattr_args /* {
 		struct vnode *a_vp;
@@ -527,6 +552,7 @@ fdesc_setattr(ap)
 	 * Can setattr the underlying vnode, but not sockets!
 	 */
 	switch (fp->f_type) {
+	case DTYPE_FIFO:
 	case DTYPE_VNODE:
 		error = VOP_SETATTR((struct vnode *) fp->f_data, ap->a_vap, ap->a_cred, ap->a_p);
 		break;
@@ -556,10 +582,9 @@ static struct dirtmp {
 	{ FD_STDOUT, UIO_MX, 6, "stdout" },
 	{ FD_STDERR, UIO_MX, 6, "stderr" },
 	{ FD_CTTY, UIO_MX, 3, "tty" },
-	{ 0 }
 };
 
-int
+static int
 fdesc_readdir(ap)
 	struct vop_readdir_args /* {
 		struct vnode *a_vp;
@@ -569,20 +594,17 @@ fdesc_readdir(ap)
 {
 	struct uio *uio = ap->a_uio;
 	struct filedesc *fdp;
-	int i;
-	int error;
+	int error, i, off;
 
-	switch (VTOFDESC(ap->a_vp)->fd_type) {
-	case Fctty:
-		return (0);
-
-	case Fdesc:
-		return (ENOTDIR);
-
-	default:
-		break;
-	}
+	if (VTOFDESC(ap->a_vp)->fd_type != Froot &&
+	    VTOFDESC(ap->a_vp)->fd_type != Fdevfd)
+		panic("fdesc_readdir: not dir");
 
+	off = (int)uio->uio_offset;
+	if (off != uio->uio_offset || off < 0 || (u_int)off % UIO_MX != 0 ||
+	    uio->uio_resid < UIO_MX)
+		return (EINVAL);
+	i = (u_int)off / UIO_MX;
 	fdp = uio->uio_procp->p_fd;
 
 	if (VTOFDESC(ap->a_vp)->fd_type == Froot) {
@@ -590,17 +612,11 @@ fdesc_readdir(ap)
 		struct dirent *dp = &d;
 		struct dirtmp *dt;
 
-		i = uio->uio_offset / UIO_MX;
 		error = 0;
 
-		while (uio->uio_resid > 0) {
+		while (i < sizeof(rootent) / sizeof(rootent[0]) &&
+		    uio->uio_resid >= UIO_MX) {
 			dt = &rootent[i];
-			if (dt->d_fileno == 0) {
-				/**eofflagp = 1;*/
-				break;
-			}
-			i++;
-
 			switch (dt->d_fileno) {
 			case FD_CTTY:
 				if (cttyvp(uio->uio_procp) == NULL)
@@ -625,17 +641,14 @@ fdesc_readdir(ap)
 			error = uiomove((caddr_t) dp, UIO_MX, uio);
 			if (error)
 				break;
+			i++;
 		}
 		uio->uio_offset = i * UIO_MX;
 		return (error);
 	}
 
-	i = uio->uio_offset / UIO_MX;
 	error = 0;
-	while (uio->uio_resid > 0) {
-		if (i >= fdp->fd_nfiles)
-			break;
-
+	while (i < fdp->fd_nfiles && uio->uio_resid >= UIO_MX) {
 		if (fdp->fd_ofiles[i] != NULL) {
 			struct dirent d;
 			struct dirent *dp = &d;
@@ -660,7 +673,7 @@ fdesc_readdir(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_readlink(ap)
 	struct vop_readlink_args /* {
 		struct vnode *a_vp;
@@ -684,7 +697,7 @@ fdesc_readlink(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_read(ap)
 	struct vop_read_args /* {
 		struct vnode *a_vp;
@@ -697,7 +710,7 @@ fdesc_read(ap)
 
 	switch (VTOFDESC(ap->a_vp)->fd_type) {
 	case Fctty:
-		error = cttyread(devctty, ap->a_uio, ap->a_ioflag);
+		error = (*ctty_cdevsw.d_read)(devctty, ap->a_uio, ap->a_ioflag);
 		break;
 
 	default:
@@ -708,7 +721,7 @@ fdesc_read(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_write(ap)
 	struct vop_write_args /* {
 		struct vnode *a_vp;
@@ -721,7 +734,7 @@ fdesc_write(ap)
 
 	switch (VTOFDESC(ap->a_vp)->fd_type) {
 	case Fctty:
-		error = cttywrite(devctty, ap->a_uio, ap->a_ioflag);
+		error = (*ctty_cdevsw.d_write)(devctty, ap->a_uio, ap->a_ioflag);
 		break;
 
 	default:
@@ -732,7 +745,7 @@ fdesc_write(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_ioctl(ap)
 	struct vop_ioctl_args /* {
 		struct vnode *a_vp;
@@ -747,8 +760,8 @@ fdesc_ioctl(ap)
 
 	switch (VTOFDESC(ap->a_vp)->fd_type) {
 	case Fctty:
-		error = cttyioctl(devctty, ap->a_command, ap->a_data,
-					ap->a_fflag, ap->a_p);
+		error = (*ctty_cdevsw.d_ioctl)(devctty, ap->a_command,
+					ap->a_data, ap->a_fflag, ap->a_p);
 		break;
 
 	default:
@@ -759,7 +772,7 @@ fdesc_ioctl(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_select(ap)
 	struct vop_select_args /* {
 		struct vnode *a_vp;
@@ -773,7 +786,7 @@ fdesc_select(ap)
 
 	switch (VTOFDESC(ap->a_vp)->fd_type) {
 	case Fctty:
-		error = cttyselect(devctty, ap->a_fflags, ap->a_p);
+		error = (*ctty_cdevsw.d_select)(devctty, ap->a_fflags, ap->a_p);
 		break;
 
 	default:
@@ -784,7 +797,7 @@ fdesc_select(ap)
 	return (error);
 }
 
-int
+static int
 fdesc_inactive(ap)
 	struct vop_inactive_args /* {
 		struct vnode *a_vp;
@@ -800,7 +813,7 @@ fdesc_inactive(ap)
 	return (0);
 }
 
-int
+static int
 fdesc_reclaim(ap)
 	struct vop_reclaim_args /* {
 		struct vnode *a_vp;
@@ -818,7 +831,7 @@ fdesc_reclaim(ap)
 /*
  * Return POSIX pathconf information applicable to special devices.
  */
-int
+static int
 fdesc_pathconf(ap)
 	struct vop_pathconf_args /* {
 		struct vnode *a_vp;
@@ -856,7 +869,7 @@ fdesc_pathconf(ap)
  * Print out the contents of a /dev/fd vnode.
  */
 /* ARGSUSED */
-int
+static int
 fdesc_print(ap)
 	struct vop_print_args /* {
 		struct vnode *a_vp;
@@ -868,7 +881,7 @@ fdesc_print(ap)
 }
 
 /*void*/
-int
+static int
 fdesc_vfree(ap)
 	struct vop_vfree_args /* {
 		struct vnode *a_pvp;
@@ -883,7 +896,7 @@ fdesc_vfree(ap)
 /*
  * /dev/fd vnode unsupported operation
  */
-int
+static int
 fdesc_enotsupp()
 {
 
@@ -893,7 +906,7 @@ fdesc_enotsupp()
 /*
  * /dev/fd "should never get here" operation
  */
-int
+static int
 fdesc_badop()
 {
 
@@ -901,16 +914,6 @@ fdesc_badop()
 	/* NOTREACHED */
 }
 
-/*
- * /dev/fd vnode null operation
- */
-int
-fdesc_nullop()
-{
-
-	return (0);
-}
-
 #define fdesc_create ((int (*) __P((struct  vop_create_args *)))fdesc_enotsupp)
 #define fdesc_mknod ((int (*) __P((struct  vop_mknod_args *)))fdesc_enotsupp)
 #define fdesc_close ((int (*) __P((struct  vop_close_args *)))nullop)
@@ -944,52 +947,52 @@ fdesc_nullop()
 #define fdesc_update ((int (*) __P((struct  vop_update_args *)))fdesc_enotsupp)
 #define fdesc_bwrite ((int (*) __P((struct  vop_bwrite_args *)))fdesc_enotsupp)
 
-int (**fdesc_vnodeop_p)();
-struct vnodeopv_entry_desc fdesc_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, fdesc_lookup },	/* lookup */
-	{ &vop_create_desc, fdesc_create },	/* create */
-	{ &vop_mknod_desc, fdesc_mknod },	/* mknod */
-	{ &vop_open_desc, fdesc_open },		/* open */
-	{ &vop_close_desc, fdesc_close },	/* close */
-	{ &vop_access_desc, fdesc_access },	/* access */
-	{ &vop_getattr_desc, fdesc_getattr },	/* getattr */
-	{ &vop_setattr_desc, fdesc_setattr },	/* setattr */
-	{ &vop_read_desc, fdesc_read },		/* read */
-	{ &vop_write_desc, fdesc_write },	/* write */
-	{ &vop_ioctl_desc, fdesc_ioctl },	/* ioctl */
-	{ &vop_select_desc, fdesc_select },	/* select */
-	{ &vop_mmap_desc, fdesc_mmap },		/* mmap */
-	{ &vop_fsync_desc, fdesc_fsync },	/* fsync */
-	{ &vop_seek_desc, fdesc_seek },		/* seek */
-	{ &vop_remove_desc, fdesc_remove },	/* remove */
-	{ &vop_link_desc, fdesc_link },		/* link */
-	{ &vop_rename_desc, fdesc_rename },	/* rename */
-	{ &vop_mkdir_desc, fdesc_mkdir },	/* mkdir */
-	{ &vop_rmdir_desc, fdesc_rmdir },	/* rmdir */
-	{ &vop_symlink_desc, fdesc_symlink },	/* symlink */
-	{ &vop_readdir_desc, fdesc_readdir },	/* readdir */
-	{ &vop_readlink_desc, fdesc_readlink },	/* readlink */
-	{ &vop_abortop_desc, fdesc_abortop },	/* abortop */
-	{ &vop_inactive_desc, fdesc_inactive },	/* inactive */
-	{ &vop_reclaim_desc, fdesc_reclaim },	/* reclaim */
-	{ &vop_lock_desc, fdesc_lock },		/* lock */
-	{ &vop_unlock_desc, fdesc_unlock },	/* unlock */
-	{ &vop_bmap_desc, fdesc_bmap },		/* bmap */
-	{ &vop_strategy_desc, fdesc_strategy },	/* strategy */
-	{ &vop_print_desc, fdesc_print },	/* print */
-	{ &vop_islocked_desc, fdesc_islocked },	/* islocked */
-	{ &vop_pathconf_desc, fdesc_pathconf },	/* pathconf */
-	{ &vop_advlock_desc, fdesc_advlock },	/* advlock */
-	{ &vop_blkatoff_desc, fdesc_blkatoff },	/* blkatoff */
-	{ &vop_valloc_desc, fdesc_valloc },	/* valloc */
-	{ &vop_vfree_desc, fdesc_vfree },	/* vfree */
-	{ &vop_truncate_desc, fdesc_truncate },	/* truncate */
-	{ &vop_update_desc, fdesc_update },	/* update */
-	{ &vop_bwrite_desc, fdesc_bwrite },	/* bwrite */
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+static vop_t **fdesc_vnodeop_p;
+static struct vnodeopv_entry_desc fdesc_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)fdesc_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)fdesc_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)fdesc_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)fdesc_open },		/* open */
+	{ &vop_close_desc, (vop_t *)fdesc_close },		/* close */
+	{ &vop_access_desc, (vop_t *)fdesc_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)fdesc_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)fdesc_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)fdesc_read },		/* read */
+	{ &vop_write_desc, (vop_t *)fdesc_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)fdesc_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)fdesc_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)fdesc_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)fdesc_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)fdesc_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)fdesc_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)fdesc_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)fdesc_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)fdesc_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)fdesc_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)fdesc_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)fdesc_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)fdesc_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)fdesc_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)fdesc_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)fdesc_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)fdesc_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)fdesc_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)fdesc_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)fdesc_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)fdesc_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)fdesc_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)fdesc_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)fdesc_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)fdesc_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)fdesc_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)fdesc_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)fdesc_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)fdesc_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)fdesc_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc fdesc_vnodeop_opv_desc =
+static struct vnodeopv_desc fdesc_vnodeop_opv_desc =
 	{ &fdesc_vnodeop_p, fdesc_vnodeop_entries };
 
 VNODEOP_SET(fdesc_vnodeop_opv_desc);
