@@ -31,17 +31,15 @@
  * SUCH DAMAGE.
  *
  *	@(#)fifo.h	8.2 (Berkeley) 2/2/94
- * $FreeBSD: stable/2.1/sys/miscfs/fifofs/fifo.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/fifofs/fifo.h 50950 1999-09-05 08:39:22Z peter $
  */
 
-extern int	(**fifo_vnodeop_p)();
+extern vop_t **fifo_vnodeop_p;
 
 /*
  * Prototypes for fifo operations on vnodes.
  */
-int	fifo_badop(),
-	fifo_ebadf();
-
+int	fifo_badop __P((void));
 int	fifo_printinfo __P((struct vnode *));
 int	fifo_lookup __P((struct vop_lookup_args *));
 #define fifo_create ((int (*) __P((struct  vop_create_args *)))fifo_badop)
@@ -69,11 +67,8 @@ int	fifo_select __P((struct vop_select_args *));
 #define fifo_abortop ((int (*) __P((struct  vop_abortop_args *)))fifo_badop)
 #define fifo_inactive ((int (*) __P((struct  vop_inactive_args *)))nullop)
 #define fifo_reclaim ((int (*) __P((struct  vop_reclaim_args *)))nullop)
-int	fifo_lock __P((struct vop_lock_args *));
-int	fifo_unlock __P((struct vop_unlock_args *));
 int	fifo_bmap __P((struct vop_bmap_args *));
 #define fifo_strategy ((int (*) __P((struct  vop_strategy_args *)))fifo_badop)
-int	fifo_print __P((struct vop_print_args *));
 #define fifo_islocked ((int (*) __P((struct  vop_islocked_args *)))nullop)
 int	fifo_pathconf __P((struct vop_pathconf_args *));
 int	fifo_advlock __P((struct vop_advlock_args *));
