@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)kernfs_vnops.c	8.6 (Berkeley) 2/10/94
- * $FreeBSD: stable/2.1/sys/miscfs/kernfs/kernfs_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/kernfs/kernfs_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -48,9 +48,10 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/proc.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/malloc.h>
-#include <sys/file.h>
+#include <sys/fcntl.h>
 #include <sys/stat.h>
 #include <sys/mount.h>
 #include <sys/namei.h>
@@ -65,7 +66,7 @@
 #define	WRITE_MODE	(S_IWUSR|S_IRUSR|S_IRGRP|S_IROTH)
 #define DIR_MODE	(S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)
 
-struct kern_target {
+static struct kern_target {
 	char *kt_name;
 	void *kt_data;
 #define	KTT_NULL 1
@@ -94,15 +95,34 @@ struct kern_target {
 	{ "physmem",	&physmem,	KTT_INT,	VREAD,	VREG,	NULL },
 #if 0
 	{ "root",	0,		KTT_NULL,	VREAD,	VDIR, &rootdir},
-#endif
 	{ "rootdev",	0,		KTT_NULL,	VREAD,	VBLK, &rootvp },
 	{ "rrootdev",	0,		KTT_NULL,	VREAD,	VCHR, &rrootvp},
+#endif
 	{ "time",	0,		KTT_TIME,	VREAD,	VREG,	NULL },
 	{ "version",	version,	KTT_STRING,	VREAD,	VREG,	NULL },
 };
 
 static int nkern_targets = sizeof(kern_targets) / sizeof(kern_targets[0]);
 
+static int	kernfs_access __P((struct vop_access_args *ap));
+static int	kernfs_badop __P((void));
+static int	kernfs_enotsupp __P((void));
+static int	kernfs_getattr __P((struct vop_getattr_args *ap));
+static int	kernfs_inactive __P((struct vop_inactive_args *ap));
+static int	kernfs_lookup __P((struct vop_lookup_args *ap));
+static int	kernfs_open __P((struct vop_open_args *ap));
+static int	kernfs_pathconf __P((struct vop_pathconf_args *ap));
+static int	kernfs_print __P((struct vop_print_args *ap));
+static int	kernfs_read __P((struct vop_read_args *ap));
+static int	kernfs_readdir __P((struct vop_readdir_args *ap));
+static int	kernfs_reclaim __P((struct vop_reclaim_args *ap));
+static int	kernfs_setattr __P((struct vop_setattr_args *ap));
+static int	kernfs_vfree __P((struct vop_vfree_args *ap));
+static int	kernfs_write __P((struct vop_write_args *ap));
+static int	kernfs_xread __P((struct kern_target *kt, char *buf, int len,
+				  int *lenp));
+static int	kernfs_xwrite __P((struct kern_target *kt, char *buf, int len));
+
 static int
 kernfs_xread(kt, buf, len, lenp)
 	struct kern_target *kt;
@@ -137,7 +157,7 @@ kernfs_xread(kt, buf, len, lenp)
 
 	case KTT_HOSTNAME: {
 		char *cp = hostname;
-		int xlen = hostnamelen;
+		int xlen = strlen(hostname);
 
 		if (xlen >= (len-2))
 			return (EINVAL);
@@ -185,11 +205,11 @@ kernfs_xwrite(kt, buf, len)
 {
 	switch (kt->kt_tag) {
 	case KTT_HOSTNAME: {
+		/* XXX BOGUS !!! no check for the length */
 		if (buf[len-1] == '\n')
 			--len;
 		bcopy(buf, hostname, len);
 		hostname[len] = '\0';
-		hostnamelen = len;
 		return (0);
 	}
 
@@ -203,7 +223,7 @@ kernfs_xwrite(kt, buf, len)
  * vp is the current namei directory
  * ndp is the name to locate in that directory...
  */
-int
+static int
 kernfs_lookup(ap)
 	struct vop_lookup_args /* {
 		struct vnode * a_dvp;
@@ -325,7 +345,7 @@ bad:;
 	return (error);
 }
 
-int
+static int
 kernfs_open(ap)
 	struct vop_open_args /* {
 		struct vnode *a_vp;
@@ -389,7 +409,7 @@ kernfs_access(ap)
 }
 
 
-int
+static int
 kernfs_getattr(ap)
 	struct vop_getattr_args /* {
 		struct vnode *a_vp;
@@ -453,7 +473,7 @@ kernfs_getattr(ap)
 	return (error);
 }
 
-int
+static int
 kernfs_setattr(ap)
 	struct vop_setattr_args /* {
 		struct vnode *a_vp;
@@ -544,7 +564,7 @@ kernfs_write(ap)
 }
 
 
-int
+static int
 kernfs_readdir(ap)
 	struct vop_readdir_args /* {
 		struct vnode *a_vp;
@@ -595,7 +615,7 @@ kernfs_readdir(ap)
 	return (error);
 }
 
-int
+static int
 kernfs_inactive(ap)
 	struct vop_inactive_args /* {
 		struct vnode *a_vp;
@@ -614,7 +634,7 @@ kernfs_inactive(ap)
 	return (0);
 }
 
-int
+static int
 kernfs_reclaim(ap)
 	struct vop_reclaim_args /* {
 		struct vnode *a_vp;
@@ -634,7 +654,7 @@ kernfs_reclaim(ap)
 /*
  * Return POSIX pathconf information applicable to special devices.
  */
-int
+static int
 kernfs_pathconf(ap)
 	struct vop_pathconf_args /* {
 		struct vnode *a_vp;
@@ -672,7 +692,7 @@ kernfs_pathconf(ap)
  * Print out the contents of a kernfs vnode.
  */
 /* ARGSUSED */
-int
+static int
 kernfs_print(ap)
 	struct vop_print_args /* {
 		struct vnode *a_vp;
@@ -684,7 +704,7 @@ kernfs_print(ap)
 }
 
 /*void*/
-int
+static int
 kernfs_vfree(ap)
 	struct vop_vfree_args /* {
 		struct vnode *a_pvp;
@@ -699,7 +719,7 @@ kernfs_vfree(ap)
 /*
  * Kernfs vnode unsupported operation
  */
-int
+static int
 kernfs_enotsupp()
 {
 
@@ -709,22 +729,12 @@ kernfs_enotsupp()
 /*
  * Kernfs "should never get here" operation
  */
-int
+static int
 kernfs_badop()
 {
 	return (EIO);
 }
 
-/*
- * kernfs vnode null operation
- */
-int
-kernfs_nullop()
-{
-
-	return (0);
-}
-
 #define kernfs_create ((int (*) __P((struct  vop_create_args *)))kernfs_enotsupp)
 #define kernfs_mknod ((int (*) __P((struct  vop_mknod_args *)))kernfs_enotsupp)
 #define kernfs_close ((int (*) __P((struct  vop_close_args *)))nullop)
@@ -760,52 +770,52 @@ kernfs_nullop()
 #define kernfs_update ((int (*) __P((struct  vop_update_args *)))kernfs_enotsupp)
 #define kernfs_bwrite ((int (*) __P((struct  vop_bwrite_args *)))kernfs_enotsupp)
 
-int (**kernfs_vnodeop_p)();
-struct vnodeopv_entry_desc kernfs_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, kernfs_lookup },	/* lookup */
-	{ &vop_create_desc, kernfs_create },	/* create */
-	{ &vop_mknod_desc, kernfs_mknod },	/* mknod */
-	{ &vop_open_desc, kernfs_open },	/* open */
-	{ &vop_close_desc, kernfs_close },	/* close */
-	{ &vop_access_desc, kernfs_access },	/* access */
-	{ &vop_getattr_desc, kernfs_getattr },	/* getattr */
-	{ &vop_setattr_desc, kernfs_setattr },	/* setattr */
-	{ &vop_read_desc, kernfs_read },	/* read */
-	{ &vop_write_desc, kernfs_write },	/* write */
-	{ &vop_ioctl_desc, kernfs_ioctl },	/* ioctl */
-	{ &vop_select_desc, kernfs_select },	/* select */
-	{ &vop_mmap_desc, kernfs_mmap },	/* mmap */
-	{ &vop_fsync_desc, kernfs_fsync },	/* fsync */
-	{ &vop_seek_desc, kernfs_seek },	/* seek */
-	{ &vop_remove_desc, kernfs_remove },	/* remove */
-	{ &vop_link_desc, kernfs_link },	/* link */
-	{ &vop_rename_desc, kernfs_rename },	/* rename */
-	{ &vop_mkdir_desc, kernfs_mkdir },	/* mkdir */
-	{ &vop_rmdir_desc, kernfs_rmdir },	/* rmdir */
-	{ &vop_symlink_desc, kernfs_symlink },	/* symlink */
-	{ &vop_readdir_desc, kernfs_readdir },	/* readdir */
-	{ &vop_readlink_desc, kernfs_readlink },/* readlink */
-	{ &vop_abortop_desc, kernfs_abortop },	/* abortop */
-	{ &vop_inactive_desc, kernfs_inactive },/* inactive */
-	{ &vop_reclaim_desc, kernfs_reclaim },	/* reclaim */
-	{ &vop_lock_desc, kernfs_lock },	/* lock */
-	{ &vop_unlock_desc, kernfs_unlock },	/* unlock */
-	{ &vop_bmap_desc, kernfs_bmap },	/* bmap */
-	{ &vop_strategy_desc, kernfs_strategy },/* strategy */
-	{ &vop_print_desc, kernfs_print },	/* print */
-	{ &vop_islocked_desc, kernfs_islocked },/* islocked */
-	{ &vop_pathconf_desc, kernfs_pathconf },/* pathconf */
-	{ &vop_advlock_desc, kernfs_advlock },	/* advlock */
-	{ &vop_blkatoff_desc, kernfs_blkatoff },/* blkatoff */
-	{ &vop_valloc_desc, kernfs_valloc },	/* valloc */
-	{ &vop_vfree_desc, kernfs_vfree },	/* vfree */
-	{ &vop_truncate_desc, kernfs_truncate },/* truncate */
-	{ &vop_update_desc, kernfs_update },	/* update */
-	{ &vop_bwrite_desc, kernfs_bwrite },	/* bwrite */
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+vop_t **kernfs_vnodeop_p;
+static struct vnodeopv_entry_desc kernfs_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)kernfs_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)kernfs_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)kernfs_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)kernfs_open },		/* open */
+	{ &vop_close_desc, (vop_t *)kernfs_close },		/* close */
+	{ &vop_access_desc, (vop_t *)kernfs_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)kernfs_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)kernfs_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)kernfs_read },		/* read */
+	{ &vop_write_desc, (vop_t *)kernfs_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)kernfs_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)kernfs_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)kernfs_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)kernfs_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)kernfs_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)kernfs_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)kernfs_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)kernfs_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)kernfs_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)kernfs_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)kernfs_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)kernfs_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)kernfs_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)kernfs_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)kernfs_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)kernfs_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)kernfs_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)kernfs_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)kernfs_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)kernfs_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)kernfs_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)kernfs_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)kernfs_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)kernfs_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)kernfs_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)kernfs_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)kernfs_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)kernfs_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)kernfs_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)kernfs_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc kernfs_vnodeop_opv_desc =
+static struct vnodeopv_desc kernfs_vnodeop_opv_desc =
 	{ &kernfs_vnodeop_p, kernfs_vnodeop_entries };
 
 VNODEOP_SET(kernfs_vnodeop_opv_desc);
