@@ -35,7 +35,7 @@
  *
  *	@(#)null.h	8.2 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/nullfs/null.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/nullfs/null.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 struct null_args {
@@ -70,6 +70,6 @@ extern struct vnode *null_checkvp __P((struct vnode *vp, char *fil, int lno));
 #define	NULLVPTOLOWERVP(vp) (VTONULL(vp)->null_lowervp)
 #endif
 
-extern int (**null_vnodeop_p)();
+extern vop_t **null_vnodeop_p;
 extern struct vfsops null_vfsops;
 #endif /* KERNEL */
