@@ -35,7 +35,7 @@
  *
  *	@(#)portal.h	8.4 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/portal/portal.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/portal/portal.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 struct portal_args {
@@ -67,6 +67,6 @@ struct portalnode {
 
 #define PORTAL_ROOTFILEID	2
 
-extern int (**portal_vnodeop_p)();
+extern vop_t **portal_vnodeop_p;
 extern struct vfsops portal_vfsops;
 #endif /* KERNEL */
