@@ -35,7 +35,7 @@
  *
  *	@(#)portal_vnops.c	8.8 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/portal/portal_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/portal/portal_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -44,11 +44,13 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysproto.h>
 #include <sys/kernel.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/proc.h>
 #include <sys/filedesc.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -64,6 +66,21 @@
 
 static int portal_fileid = PORTAL_ROOTFILEID+1;
 
+static int	portal_badop __P((void));
+static void	portal_closefd __P((struct proc *p, int fd));
+static int	portal_connect __P((struct socket *so, struct socket *so2));
+static int	portal_enotsupp __P((void));
+static int	portal_getattr __P((struct vop_getattr_args *ap));
+static int	portal_inactive __P((struct vop_inactive_args *ap));
+static int	portal_lookup __P((struct vop_lookup_args *ap));
+static int	portal_open __P((struct vop_open_args *ap));
+static int	portal_pathconf __P((struct vop_pathconf_args *ap));
+static int	portal_print __P((struct vop_print_args *ap));
+static int	portal_readdir __P((struct vop_readdir_args *ap));
+static int	portal_reclaim __P((struct vop_reclaim_args *ap));
+static int	portal_setattr __P((struct vop_setattr_args *ap));
+static int	portal_vfree __P((struct vop_vfree_args *ap));
+
 static void
 portal_closefd(p, fd)
 	struct proc *p;
@@ -87,7 +104,7 @@ portal_closefd(p, fd)
  * vp is the current namei directory
  * cnp is the name to locate in that directory...
  */
-int
+static int
 portal_lookup(ap)
 	struct vop_lookup_args /* {
 		struct vnode * a_dvp;
@@ -183,7 +200,7 @@ portal_connect(so, so2)
 	return (unp_connect2(so, so2));
 }
 
-int
+static int
 portal_open(ap)
 	struct vop_open_args /* {
 		struct vnode *a_vp;
@@ -231,7 +248,7 @@ portal_open(ap)
 	/*
 	 * Create a new socket.
 	 */
-	error = socreate(AF_UNIX, &so, SOCK_STREAM, 0);
+	error = socreate(AF_UNIX, &so, SOCK_STREAM, 0, ap->a_p);
 	if (error)
 		goto bad;
 
@@ -416,7 +433,7 @@ bad:;
 	return (error);
 }
 
-int
+static int
 portal_getattr(ap)
 	struct vop_getattr_args /* {
 		struct vnode *a_vp;
@@ -463,7 +480,7 @@ portal_getattr(ap)
 	return (0);
 }
 
-int
+static int
 portal_setattr(ap)
 	struct vop_setattr_args /* {
 		struct vnode *a_vp;
@@ -486,7 +503,7 @@ portal_setattr(ap)
  * Fake readdir, just return empty directory.
  * It is hard to deal with '.' and '..' so don't bother.
  */
-int
+static int
 portal_readdir(ap)
 	struct vop_readdir_args /* {
 		struct vnode *a_vp;
@@ -498,7 +515,7 @@ portal_readdir(ap)
 	return (0);
 }
 
-int
+static int
 portal_inactive(ap)
 	struct vop_inactive_args /* {
 		struct vnode *a_vp;
@@ -508,7 +525,7 @@ portal_inactive(ap)
 	return (0);
 }
 
-int
+static int
 portal_reclaim(ap)
 	struct vop_reclaim_args /* {
 		struct vnode *a_vp;
@@ -529,7 +546,7 @@ portal_reclaim(ap)
 /*
  * Return POSIX pathconf information applicable to special devices.
  */
-int
+static int
 portal_pathconf(ap)
 	struct vop_pathconf_args /* {
 		struct vnode *a_vp;
@@ -567,7 +584,7 @@ portal_pathconf(ap)
  * Print out the contents of a Portal vnode.
  */
 /* ARGSUSED */
-int
+static int
 portal_print(ap)
 	struct vop_print_args /* {
 		struct vnode *a_vp;
@@ -579,7 +596,7 @@ portal_print(ap)
 }
 
 /*void*/
-int
+static int
 portal_vfree(ap)
 	struct vop_vfree_args /* {
 		struct vnode *a_pvp;
@@ -595,7 +612,7 @@ portal_vfree(ap)
 /*
  * Portal vnode unsupported operation
  */
-int
+static int
 portal_enotsupp()
 {
 
@@ -605,7 +622,7 @@ portal_enotsupp()
 /*
  * Portal "should never get here" operation
  */
-int
+static int
 portal_badop()
 {
 
@@ -613,16 +630,6 @@ portal_badop()
 	/* NOTREACHED */
 }
 
-/*
- * Portal vnode null operation
- */
-int
-portal_nullop()
-{
-
-	return (0);
-}
-
 #define portal_create ((int (*) __P((struct vop_create_args *)))portal_enotsupp)
 #define portal_mknod ((int (*) __P((struct  vop_mknod_args *)))portal_enotsupp)
 #define portal_close ((int (*) __P((struct  vop_close_args *)))nullop)
@@ -664,52 +671,52 @@ portal_nullop()
 #define portal_update ((int (*) __P((struct vop_update_args *)))portal_enotsupp)
 #define portal_bwrite ((int (*) __P((struct vop_bwrite_args *)))portal_enotsupp)
 
-int (**portal_vnodeop_p)();
-struct vnodeopv_entry_desc portal_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, portal_lookup },		/* lookup */
-	{ &vop_create_desc, portal_create },		/* create */
-	{ &vop_mknod_desc, portal_mknod },		/* mknod */
-	{ &vop_open_desc, portal_open },		/* open */
-	{ &vop_close_desc, portal_close },		/* close */
-	{ &vop_access_desc, portal_access },		/* access */
-	{ &vop_getattr_desc, portal_getattr },		/* getattr */
-	{ &vop_setattr_desc, portal_setattr },		/* setattr */
-	{ &vop_read_desc, portal_read },		/* read */
-	{ &vop_write_desc, portal_write },		/* write */
-	{ &vop_ioctl_desc, portal_ioctl },		/* ioctl */
-	{ &vop_select_desc, portal_select },		/* select */
-	{ &vop_mmap_desc, portal_mmap },		/* mmap */
-	{ &vop_fsync_desc, portal_fsync },		/* fsync */
-	{ &vop_seek_desc, portal_seek },		/* seek */
-	{ &vop_remove_desc, portal_remove },		/* remove */
-	{ &vop_link_desc, portal_link },		/* link */
-	{ &vop_rename_desc, portal_rename },		/* rename */
-	{ &vop_mkdir_desc, portal_mkdir },		/* mkdir */
-	{ &vop_rmdir_desc, portal_rmdir },		/* rmdir */
-	{ &vop_symlink_desc, portal_symlink },		/* symlink */
-	{ &vop_readdir_desc, portal_readdir },		/* readdir */
-	{ &vop_readlink_desc, portal_readlink },	/* readlink */
-	{ &vop_abortop_desc, portal_abortop },		/* abortop */
-	{ &vop_inactive_desc, portal_inactive },	/* inactive */
-	{ &vop_reclaim_desc, portal_reclaim },		/* reclaim */
-	{ &vop_lock_desc, portal_lock },		/* lock */
-	{ &vop_unlock_desc, portal_unlock },		/* unlock */
-	{ &vop_bmap_desc, portal_bmap },		/* bmap */
-	{ &vop_strategy_desc, portal_strategy },	/* strategy */
-	{ &vop_print_desc, portal_print },		/* print */
-	{ &vop_islocked_desc, portal_islocked },	/* islocked */
-	{ &vop_pathconf_desc, portal_pathconf },	/* pathconf */
-	{ &vop_advlock_desc, portal_advlock },		/* advlock */
-	{ &vop_blkatoff_desc, portal_blkatoff },	/* blkatoff */
-	{ &vop_valloc_desc, portal_valloc },		/* valloc */
-	{ &vop_vfree_desc, portal_vfree },		/* vfree */
-	{ &vop_truncate_desc, portal_truncate },	/* truncate */
-	{ &vop_update_desc, portal_update },		/* update */
-	{ &vop_bwrite_desc, portal_bwrite },		/* bwrite */
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+vop_t **portal_vnodeop_p;
+static struct vnodeopv_entry_desc portal_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)portal_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)portal_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)portal_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)portal_open },		/* open */
+	{ &vop_close_desc, (vop_t *)portal_close },		/* close */
+	{ &vop_access_desc, (vop_t *)portal_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)portal_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)portal_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)portal_read },		/* read */
+	{ &vop_write_desc, (vop_t *)portal_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)portal_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)portal_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)portal_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)portal_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)portal_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)portal_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)portal_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)portal_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)portal_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)portal_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)portal_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)portal_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)portal_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)portal_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)portal_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)portal_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)portal_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)portal_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)portal_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)portal_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)portal_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)portal_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)portal_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)portal_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)portal_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)portal_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)portal_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)portal_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)portal_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)portal_bwrite },		/* bwrite */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc portal_vnodeop_opv_desc =
+static struct vnodeopv_desc portal_vnodeop_opv_desc =
 	{ &portal_vnodeop_p, portal_vnodeop_entries };
 
 VNODEOP_SET(portal_vnodeop_opv_desc);
