@@ -36,7 +36,7 @@
  *
  *	@(#)procfs_fpregs.c	8.1 (Berkeley) 1/27/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/procfs/procfs_fpregs.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/procfs/procfs_fpregs.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -47,6 +47,8 @@
 #include <sys/vnode.h>
 #include <machine/reg.h>
 #include <miscfs/procfs/procfs.h>
+#include <vm/vm.h>
+#include <vm/vm_extern.h>
 
 int
 procfs_dofpregs(curp, p, pfs, uio)
@@ -70,6 +72,8 @@ procfs_dofpregs(curp, p, pfs, uio)
 	if (kl > uio->uio_resid)
 		kl = uio->uio_resid;
 
+	PHOLD(p);
+
 	if (kl < 0)
 		error = EINVAL;
 	else
@@ -82,7 +86,15 @@ procfs_dofpregs(curp, p, pfs, uio)
 		else
 			error = procfs_write_fpregs(p, &r);
 	}
+	PRELE(p);
 
 	uio->uio_offset = 0;
 	return (error);
 }
+
+int
+procfs_validfpregs(p)
+	struct proc *p;
+{
+	return ((p->p_flag & P_SYSTEM) == 0);
+}
