@@ -36,7 +36,7 @@
  *
  *	@(#)procfs_vnops.c	8.6 (Berkeley) 2/7/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/procfs/procfs_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/procfs/procfs_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -47,8 +47,9 @@
 #include <sys/systm.h>
 #include <sys/time.h>
 #include <sys/kernel.h>
-#include <sys/file.h>
+#include <sys/fcntl.h>
 #include <sys/proc.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/namei.h>
 #include <sys/malloc.h>
@@ -57,10 +58,20 @@
 #include <miscfs/procfs/procfs.h>
 #include <vm/vm.h>	/* for PAGE_SIZE */
 
-/*
- * Vnode Operations.
- *
- */
+static int	procfs_abortop __P((struct vop_abortop_args *));
+static int	procfs_access __P((struct vop_access_args *));
+static int	procfs_badop __P((void));
+static int	procfs_close __P((struct vop_close_args *));
+static int	procfs_getattr __P((struct vop_getattr_args *));
+static int	procfs_inactive __P((struct vop_inactive_args *));
+static int	procfs_ioctl __P((struct vop_ioctl_args *));
+static int	procfs_lookup __P((struct vop_lookup_args *));
+static int	procfs_open __P((struct vop_open_args *));
+static int	procfs_pathconf __P((struct vop_pathconf_args *ap));
+static int	procfs_print __P((struct vop_print_args *));
+static int	procfs_readdir __P((struct vop_readdir_args *));
+static int	procfs_reclaim __P((struct vop_reclaim_args *));
+static int	procfs_setattr __P((struct vop_setattr_args *));
 
 /*
  * This is a list of the valid names in the
@@ -71,21 +82,22 @@ static struct pfsnames {
 	u_short	d_namlen;
 	char	d_name[PROCFS_NAMELEN];
 	pfstype	d_pfstype;
+	int	(*d_valid) __P((struct proc *));
 } procent[] = {
 #define N(s) sizeof(s)-1, s
-	/* namlen, nam, type */
-	{  N("."),	Pproc },
-	{  N(".."),	Proot },
-#if 0
-	{  N("file"),	Pfile },
-#endif
-	{  N("mem"),	Pmem },
-	{  N("regs"),	Pregs },
-	{  N("fpregs"),	Pfpregs },
-	{  N("ctl"),	Pctl },
-	{  N("status"),	Pstatus },
-	{  N("note"),	Pnote },
-	{  N("notepg"),	Pnotepg },
+	/* namlen, nam, type	 validp */
+	{  N("."),	Pproc,	 NULL },
+	{  N(".."),	Proot,	 NULL },
+	{  N("file"),	Pfile,	 procfs_validfile },
+	{  N("mem"),	Pmem,	 NULL },
+	{  N("regs"),	Pregs,	 procfs_validregs },
+	{  N("fpregs"),	Pfpregs, procfs_validfpregs },
+	{  N("ctl"),	Pctl,	 NULL },
+	{  N("status"),	Pstatus, NULL },
+	{  N("note"),	Pnote,	 NULL },
+	{  N("notepg"),	Pnotepg, NULL },
+	{  N("map"), 	Pmap,	 procfs_validmap },
+	{  N("etype"),	Ptype,	 procfs_validtype },
 #undef N
 };
 #define Nprocent (sizeof(procent)/sizeof(procent[0]))
@@ -103,7 +115,7 @@ static pid_t atopid __P((const char *, u_int));
  * is to support exclusive open on process
  * memory images.
  */
-int
+static int
 procfs_open(ap)
 	struct vop_open_args *ap;
 {
@@ -143,7 +155,7 @@ procfs_open(ap)
  * nothing to do for procfs other than undo
  * any exclusive open flag (see _open above).
  */
-int
+static int
 procfs_close(ap)
 	struct vop_close_args *ap;
 {
@@ -165,7 +177,7 @@ procfs_close(ap)
  * do an ioctl operation on pfsnode (vp).
  * (vp) is not locked on entry or exit.
  */
-int
+static int
 procfs_ioctl(ap)
 	struct vop_ioctl_args *ap;
 {
@@ -188,7 +200,7 @@ procfs_ioctl(ap)
  *
  * (vp) is not locked on entry or exit.
  */
-int
+static int
 procfs_inactive(ap)
 	struct vop_inactive_args *ap;
 {
@@ -207,7 +219,7 @@ procfs_inactive(ap)
  * to free any private data and remove the node
  * from any private lists.
  */
-int
+static int
 procfs_reclaim(ap)
 	struct vop_reclaim_args *ap;
 {
@@ -220,7 +232,7 @@ procfs_reclaim(ap)
 /*
  * Return POSIX pathconf information applicable to special devices.
  */
-int
+static int
 procfs_pathconf(ap)
 	struct vop_pathconf_args /* {
 		struct vnode *a_vp;
@@ -259,7 +271,7 @@ procfs_pathconf(ap)
  * just print a readable description
  * of (vp).
  */
-int
+static int
 procfs_print(ap)
 	struct vop_print_args *ap;
 {
@@ -277,7 +289,7 @@ procfs_print(ap)
  * for undoing any side-effects caused by the lookup.
  * this will always include freeing the pathname buffer.
  */
-int
+static int
 procfs_abortop(ap)
 	struct vop_abortop_args *ap;
 {
@@ -290,7 +302,7 @@ procfs_abortop(ap)
 /*
  * generic entry point for unsupported operations
  */
-int
+static int
 procfs_badop()
 {
 
@@ -306,7 +318,7 @@ procfs_badop()
  *
  * this is relatively minimal for procfs.
  */
-int
+static int
 procfs_getattr(ap)
 	struct vop_getattr_args *ap;
 {
@@ -425,6 +437,8 @@ procfs_getattr(ap)
 		vap->va_gid = KMEM_GROUP;
 		break;
 
+	case Ptype:
+	case Pmap:
 	case Pregs:
 	case Pfpregs:
 	case Pctl:
@@ -443,7 +457,7 @@ procfs_getattr(ap)
 	return (error);
 }
 
-int
+static int
 procfs_setattr(ap)
 	struct vop_setattr_args *ap;
 {
@@ -472,7 +486,7 @@ procfs_setattr(ap)
  * but does mean that the i/o entry points need to check
  * that the operation really does make sense.
  */
-int
+static int
 procfs_access(ap)
 	struct vop_access_args *ap;
 {
@@ -525,7 +539,7 @@ found:
  * filesystem doesn't do any locking of its own.  otherwise
  * read and inwardly digest ufs_lookup().
  */
-int
+static int
 procfs_lookup(ap)
 	struct vop_lookup_args *ap;
 {
@@ -594,7 +608,8 @@ procfs_lookup(ap)
 			struct pfsnames *dp = &procent[i];
 
 			if (cnp->cn_namelen == dp->d_namlen &&
-			    bcmp(pname, dp->d_name, dp->d_namlen) == 0) {
+			    bcmp(pname, dp->d_name, dp->d_namlen) == 0 &&
+			    (dp->d_valid == NULL || (*dp->d_valid)(procp))) {
 			    	pfs_type = dp->d_pfstype;
 				goto found;
 			}
@@ -627,6 +642,16 @@ procfs_lookup(ap)
 	}
 }
 
+/*
+ * Does this process have a text file?
+ */
+int
+procfs_validfile(p)
+	struct proc *p;
+{
+	return (procfs_findtextvp(p) != NULLVP);
+}
+
 /*
  * readdir returns directory entries from pfsnode (vp).
  *
@@ -639,7 +664,7 @@ procfs_lookup(ap)
  *
  * this should just be done through read()
  */
-int
+static int
 procfs_readdir(ap)
 	struct vop_readdir_args *ap;
 {
@@ -671,6 +696,12 @@ procfs_readdir(ap)
 	 * from the procent[] table (top of this file).
 	 */
 	case Pproc: {
+		struct proc *p;
+
+		p = PFIND(pfs->pfs_pid);
+		if (p == NULL)
+			break;
+
 		while (uio->uio_resid >= UIO_MX) {
 			struct pfsnames *dt;
 
@@ -679,6 +710,12 @@ procfs_readdir(ap)
 
 			dt = &procent[i];
 
+			/* see if we should show this one. */
+			if (dt->d_valid && (*dt->d_valid)(p) == 0) {
+				i++;
+				continue;
+			}
+
 			dp->d_reclen = UIO_MX;
 			dp->d_fileno = PROCFS_FILENO(pfs->pfs_pid, dt->d_pfstype);
 			dp->d_type = DT_REG;
@@ -709,9 +746,7 @@ procfs_readdir(ap)
 #ifdef PROCFS_ZOMBIE
 		int doingzomb = 0;
 #endif
-		volatile struct proc *p;
-
-		p = allproc;
+		volatile struct proc *p = allproc.lh_first;
 
 #define PROCFS_XFILES	3	/* number of other entries, like "curproc" */
 		pcnt = PROCFS_XFILES;
@@ -744,12 +779,12 @@ procfs_readdir(ap)
 					dp->d_namlen = sprintf(dp->d_name, "%ld", (long) p->p_pid);
 				}
 
-				p = p->p_next;
+				p = p->p_list.le_next;
 
 #ifdef PROCFS_ZOMBIE
 				if (p == 0 && doingzomb == 0) {
 					doingzomb = 1;
-					p = zombproc;
+					p = zombproc.lh_first;
 				}
 #endif
 
@@ -801,54 +836,78 @@ atopid(b, len)
 	return (p);
 }
 
-/*
- * procfs vnode operations.
- */
-int (**procfs_vnodeop_p)();
-struct vnodeopv_entry_desc procfs_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, procfs_lookup },		/* lookup */
-	{ &vop_create_desc, procfs_create },		/* create */
-	{ &vop_mknod_desc, procfs_mknod },		/* mknod */
-	{ &vop_open_desc, procfs_open },		/* open */
-	{ &vop_close_desc, procfs_close },		/* close */
-	{ &vop_access_desc, procfs_access },		/* access */
-	{ &vop_getattr_desc, procfs_getattr },		/* getattr */
-	{ &vop_setattr_desc, procfs_setattr },		/* setattr */
-	{ &vop_read_desc, procfs_read },		/* read */
-	{ &vop_write_desc, procfs_write },		/* write */
-	{ &vop_ioctl_desc, procfs_ioctl },		/* ioctl */
-	{ &vop_select_desc, procfs_select },		/* select */
-	{ &vop_mmap_desc, procfs_mmap },		/* mmap */
-	{ &vop_fsync_desc, procfs_fsync },		/* fsync */
-	{ &vop_seek_desc, procfs_seek },		/* seek */
-	{ &vop_remove_desc, procfs_remove },		/* remove */
-	{ &vop_link_desc, procfs_link },		/* link */
-	{ &vop_rename_desc, procfs_rename },		/* rename */
-	{ &vop_mkdir_desc, procfs_mkdir },		/* mkdir */
-	{ &vop_rmdir_desc, procfs_rmdir },		/* rmdir */
-	{ &vop_symlink_desc, procfs_symlink },		/* symlink */
-	{ &vop_readdir_desc, procfs_readdir },		/* readdir */
-	{ &vop_readlink_desc, procfs_readlink },	/* readlink */
-	{ &vop_abortop_desc, procfs_abortop },		/* abortop */
-	{ &vop_inactive_desc, procfs_inactive },	/* inactive */
-	{ &vop_reclaim_desc, procfs_reclaim },		/* reclaim */
-	{ &vop_lock_desc, procfs_lock },		/* lock */
-	{ &vop_unlock_desc, procfs_unlock },		/* unlock */
-	{ &vop_bmap_desc, procfs_bmap },		/* bmap */
-	{ &vop_strategy_desc, procfs_strategy },	/* strategy */
-	{ &vop_print_desc, procfs_print },		/* print */
-	{ &vop_islocked_desc, procfs_islocked },	/* islocked */
-	{ &vop_pathconf_desc, procfs_pathconf },	/* pathconf */
-	{ &vop_advlock_desc, procfs_advlock },		/* advlock */
-	{ &vop_blkatoff_desc, procfs_blkatoff },	/* blkatoff */
-	{ &vop_valloc_desc, procfs_valloc },		/* valloc */
-	{ &vop_vfree_desc, procfs_vfree },		/* vfree */
-	{ &vop_truncate_desc, procfs_truncate },	/* truncate */
-	{ &vop_update_desc, procfs_update },		/* update */
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+#define procfs_create ((int (*) __P((struct vop_create_args *))) procfs_badop)
+#define procfs_mknod ((int (*) __P((struct vop_mknod_args *))) procfs_badop)
+#define	procfs_read procfs_rw
+#define	procfs_write procfs_rw
+#define procfs_select ((int (*) __P((struct vop_select_args *))) procfs_badop)
+#define procfs_mmap ((int (*) __P((struct vop_mmap_args *))) procfs_badop)
+#define procfs_fsync ((int (*) __P((struct vop_fsync_args *))) procfs_badop)
+#define procfs_seek ((int (*) __P((struct vop_seek_args *))) procfs_badop)
+#define procfs_remove ((int (*) __P((struct vop_remove_args *))) procfs_badop)
+#define procfs_link ((int (*) __P((struct vop_link_args *))) procfs_badop)
+#define procfs_rename ((int (*) __P((struct vop_rename_args *))) procfs_badop)
+#define procfs_mkdir ((int (*) __P((struct vop_mkdir_args *))) procfs_badop)
+#define procfs_rmdir ((int (*) __P((struct vop_rmdir_args *))) procfs_badop)
+#define procfs_symlink ((int (*) __P((struct vop_symlink_args *))) procfs_badop)
+#define procfs_readlink ((int (*) __P((struct vop_readlink_args *))) procfs_badop)
+#define procfs_lock ((int (*) __P((struct vop_lock_args *))) nullop)
+#define procfs_unlock ((int (*) __P((struct vop_unlock_args *))) nullop)
+#define procfs_bmap ((int (*) __P((struct vop_bmap_args *))) procfs_badop)
+#define	procfs_strategy ((int (*) __P((struct vop_strategy_args *))) procfs_badop)
+#define procfs_islocked ((int (*) __P((struct vop_islocked_args *))) nullop)
+#define procfs_advlock ((int (*) __P((struct vop_advlock_args *))) procfs_badop)
+#define procfs_blkatoff ((int (*) __P((struct vop_blkatoff_args *))) procfs_badop)
+#define procfs_valloc ((int (*) __P((struct vop_valloc_args *))) procfs_badop)
+#define procfs_vfree ((int (*) __P((struct vop_vfree_args *))) nullop)
+#define procfs_truncate ((int (*) __P((struct vop_truncate_args *))) procfs_badop)
+#define procfs_update ((int (*) __P((struct vop_update_args *))) nullop)
+
+vop_t **procfs_vnodeop_p;
+static struct vnodeopv_entry_desc procfs_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)procfs_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)procfs_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)procfs_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)procfs_open },		/* open */
+	{ &vop_close_desc, (vop_t *)procfs_close },		/* close */
+	{ &vop_access_desc, (vop_t *)procfs_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)procfs_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)procfs_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)procfs_read },		/* read */
+	{ &vop_write_desc, (vop_t *)procfs_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)procfs_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)procfs_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)procfs_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)procfs_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)procfs_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)procfs_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)procfs_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)procfs_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)procfs_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)procfs_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)procfs_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)procfs_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)procfs_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)procfs_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)procfs_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)procfs_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)procfs_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)procfs_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)procfs_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)procfs_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)procfs_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)procfs_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)procfs_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)procfs_advlock },		/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)procfs_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)procfs_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)procfs_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)procfs_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)procfs_update },		/* update */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc procfs_vnodeop_opv_desc =
+static struct vnodeopv_desc procfs_vnodeop_opv_desc =
 	{ &procfs_vnodeop_p, procfs_vnodeop_entries };
 
 VNODEOP_SET(procfs_vnodeop_opv_desc);
