@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)spec_vnops.c	8.6 (Berkeley) 4/9/94
- * $FreeBSD: stable/2.1/sys/miscfs/specfs/spec_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/specfs/spec_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -42,73 +42,83 @@
 #include <sys/buf.h>
 #include <sys/mount.h>
 #include <sys/namei.h>
+#include <sys/unistd.h>
 #include <sys/vnode.h>
 #include <sys/stat.h>
 #include <sys/errno.h>
 #include <sys/ioctl.h>
-#include <sys/file.h>
+#include <sys/fcntl.h>
 #include <sys/disklabel.h>
+#include <sys/vmmeter.h>
+
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/vm_prot.h>
+#include <vm/vm_object.h>
+#include <vm/vm_page.h>
+#include <vm/vm_pager.h>
+#include <vm/vnode_pager.h>
+#include <vm/vm_extern.h>
+
 #include <miscfs/specfs/specdev.h>
 
-/* symbolic sleep message strings for devices */
-char	devopn[] = "devopn";
-char	devio[] = "devio";
-char	devwait[] = "devwait";
-char	devin[] = "devin";
-char	devout[] = "devout";
-char	devioc[] = "devioc";
-char	devcls[] = "devcls";
+static int	spec_ebadf __P((void));
+static int	spec_getattr __P((struct  vop_getattr_args *));
 
-int (**spec_vnodeop_p)();
-struct vnodeopv_entry_desc spec_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, spec_lookup },		/* lookup */
-	{ &vop_create_desc, spec_create },		/* create */
-	{ &vop_mknod_desc, spec_mknod },		/* mknod */
-	{ &vop_open_desc, spec_open },			/* open */
-	{ &vop_close_desc, spec_close },		/* close */
-	{ &vop_access_desc, spec_access },		/* access */
-	{ &vop_getattr_desc, spec_getattr },		/* getattr */
-	{ &vop_setattr_desc, spec_setattr },		/* setattr */
-	{ &vop_read_desc, spec_read },			/* read */
-	{ &vop_write_desc, spec_write },		/* write */
-	{ &vop_ioctl_desc, spec_ioctl },		/* ioctl */
-	{ &vop_select_desc, spec_select },		/* select */
-	{ &vop_mmap_desc, spec_mmap },			/* mmap */
-	{ &vop_fsync_desc, spec_fsync },		/* fsync */
-	{ &vop_seek_desc, spec_seek },			/* seek */
-	{ &vop_remove_desc, spec_remove },		/* remove */
-	{ &vop_link_desc, spec_link },			/* link */
-	{ &vop_rename_desc, spec_rename },		/* rename */
-	{ &vop_mkdir_desc, spec_mkdir },		/* mkdir */
-	{ &vop_rmdir_desc, spec_rmdir },		/* rmdir */
-	{ &vop_symlink_desc, spec_symlink },		/* symlink */
-	{ &vop_readdir_desc, spec_readdir },		/* readdir */
-	{ &vop_readlink_desc, spec_readlink },		/* readlink */
-	{ &vop_abortop_desc, spec_abortop },		/* abortop */
-	{ &vop_inactive_desc, spec_inactive },		/* inactive */
-	{ &vop_reclaim_desc, spec_reclaim },		/* reclaim */
-	{ &vop_lock_desc, spec_lock },			/* lock */
-	{ &vop_unlock_desc, spec_unlock },		/* unlock */
-	{ &vop_bmap_desc, spec_bmap },			/* bmap */
-	{ &vop_strategy_desc, spec_strategy },		/* strategy */
-	{ &vop_print_desc, spec_print },		/* print */
-	{ &vop_islocked_desc, spec_islocked },		/* islocked */
-	{ &vop_pathconf_desc, spec_pathconf },		/* pathconf */
-	{ &vop_advlock_desc, spec_advlock },		/* advlock */
-	{ &vop_blkatoff_desc, spec_blkatoff },		/* blkatoff */
-	{ &vop_valloc_desc, spec_valloc },		/* valloc */
-	{ &vop_vfree_desc, spec_vfree },		/* vfree */
-	{ &vop_truncate_desc, spec_truncate },		/* truncate */
-	{ &vop_update_desc, spec_update },		/* update */
-	{ &vop_bwrite_desc, vn_bwrite },		/* bwrite */
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+struct vnode *speclisth[SPECHSZ];
+vop_t **spec_vnodeop_p;
+static struct vnodeopv_entry_desc spec_vnodeop_entries[] = {
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)spec_lookup },	/* lookup */
+	{ &vop_create_desc, (vop_t *)spec_create },	/* create */
+	{ &vop_mknod_desc, (vop_t *)spec_mknod },	/* mknod */
+	{ &vop_open_desc, (vop_t *)spec_open },		/* open */
+	{ &vop_close_desc, (vop_t *)spec_close },	/* close */
+	{ &vop_access_desc, (vop_t *)spec_access },	/* access */
+	{ &vop_getattr_desc, (vop_t *)spec_getattr },	/* getattr */
+	{ &vop_setattr_desc, (vop_t *)spec_setattr },	/* setattr */
+	{ &vop_read_desc, (vop_t *)spec_read },		/* read */
+	{ &vop_write_desc, (vop_t *)spec_write },	/* write */
+	{ &vop_ioctl_desc, (vop_t *)spec_ioctl },	/* ioctl */
+	{ &vop_select_desc, (vop_t *)spec_select },	/* select */
+	{ &vop_mmap_desc, (vop_t *)spec_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)spec_fsync },	/* fsync */
+	{ &vop_seek_desc, (vop_t *)spec_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)spec_remove },	/* remove */
+	{ &vop_link_desc, (vop_t *)spec_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)spec_rename },	/* rename */
+	{ &vop_mkdir_desc, (vop_t *)spec_mkdir },	/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)spec_rmdir },	/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)spec_symlink },	/* symlink */
+	{ &vop_readdir_desc, (vop_t *)spec_readdir },	/* readdir */
+	{ &vop_readlink_desc, (vop_t *)spec_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)spec_abortop },	/* abortop */
+	{ &vop_inactive_desc, (vop_t *)spec_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)spec_reclaim },	/* reclaim */
+	{ &vop_lock_desc, (vop_t *)spec_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)spec_unlock },	/* unlock */
+	{ &vop_bmap_desc, (vop_t *)spec_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)spec_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)spec_print },	/* print */
+	{ &vop_islocked_desc, (vop_t *)spec_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)spec_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)spec_advlock },	/* advlock */
+	{ &vop_blkatoff_desc, (vop_t *)spec_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)spec_valloc },	/* valloc */
+	{ &vop_vfree_desc, (vop_t *)spec_vfree },	/* vfree */
+	{ &vop_truncate_desc, (vop_t *)spec_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)spec_update },	/* update */
+	{ &vop_bwrite_desc, (vop_t *)vn_bwrite },	/* bwrite */
+	{ &vop_getpages_desc, (vop_t *)spec_getpages},	/* getpages */
+	{ NULL, NULL }
 };
-struct vnodeopv_desc spec_vnodeop_opv_desc =
+static struct vnodeopv_desc spec_vnodeop_opv_desc =
 	{ &spec_vnodeop_p, spec_vnodeop_entries };
 
 VNODEOP_SET(spec_vnodeop_opv_desc);
 
+static void spec_getpages_iodone __P((struct buf *bp));
+
 /*
  * Trivial lookup routine that always fails.
  */
@@ -154,6 +164,8 @@ spec_open(ap)
 	case VCHR:
 		if ((u_int)maj >= nchrdev)
 			return (ENXIO);
+		if ( (cdevsw[maj] == NULL) || (cdevsw[maj]->d_open == NULL))
+			return ENXIO;
 		if (ap->a_cred != FSCRED && (ap->a_mode & FWRITE)) {
 			/*
 			 * When running in very secure mode, do not allow
@@ -178,13 +190,15 @@ spec_open(ap)
 			}
 		}
 		VOP_UNLOCK(vp);
-		error = (*cdevsw[maj].d_open)(dev, ap->a_mode, S_IFCHR, ap->a_p);
+		error = (*cdevsw[maj]->d_open)(dev, ap->a_mode, S_IFCHR, ap->a_p);
 		VOP_LOCK(vp);
 		return (error);
 
 	case VBLK:
 		if ((u_int)maj >= nblkdev)
 			return (ENXIO);
+		if ( (bdevsw[maj] == NULL) || (bdevsw[maj]->d_open == NULL))
+			return ENXIO;
 		/*
 		 * When running in very secure mode, do not allow
 		 * opens for writing of any disk block devices.
@@ -199,7 +213,7 @@ spec_open(ap)
 		error = vfs_mountedon(vp);
 		if (error)
 			return (error);
-		return ((*bdevsw[maj].d_open)(dev, ap->a_mode, S_IFBLK, ap->a_p));
+		return ((*bdevsw[maj]->d_open)(dev, ap->a_mode, S_IFBLK, ap->a_p));
 	default:
 		break;
 	}
@@ -226,7 +240,8 @@ spec_read(ap)
 	daddr_t bn, nextbn;
 	long bsize, bscale;
 	struct partinfo dpart;
-	int n, on, majordev, (*ioctl)();
+	int n, on, majordev;
+	d_ioctl_t *ioctl;
 	int error = 0;
 	dev_t dev;
 
@@ -243,7 +258,7 @@ spec_read(ap)
 
 	case VCHR:
 		VOP_UNLOCK(vp);
-		error = (*cdevsw[major(vp->v_rdev)].d_read)
+		error = (*cdevsw[major(vp->v_rdev)]->d_read)
 			(vp->v_rdev, uio, ap->a_ioflag);
 		VOP_LOCK(vp);
 		return (error);
@@ -254,14 +269,14 @@ spec_read(ap)
 		bsize = BLKDEV_IOSIZE;
 		dev = vp->v_rdev;
 		if ((majordev = major(dev)) < nblkdev &&
-		    (ioctl = bdevsw[majordev].d_ioctl) != NULL &&
+		    (ioctl = bdevsw[majordev]->d_ioctl) != NULL &&
 		    (*ioctl)(dev, DIOCGPART, (caddr_t)&dpart, FREAD, p) == 0 &&
 		    dpart.part->p_fstype == FS_BSDFFS &&
 		    dpart.part->p_frag != 0 && dpart.part->p_fsize != 0)
 			bsize = dpart.part->p_frag * dpart.part->p_fsize;
-		bscale = bsize / DEV_BSIZE;
+		bscale = btodb(bsize);
 		do {
-			bn = (uio->uio_offset / DEV_BSIZE) &~ (bscale - 1);
+			bn = btodb(uio->uio_offset) & ~(bscale - 1);
 			on = uio->uio_offset % bsize;
 			n = min((unsigned)(bsize - on), uio->uio_resid);
 			if (vp->v_lastr + bscale == bn) {
@@ -321,7 +336,7 @@ spec_write(ap)
 
 	case VCHR:
 		VOP_UNLOCK(vp);
-		error = (*cdevsw[major(vp->v_rdev)].d_write)
+		error = (*cdevsw[major(vp->v_rdev)]->d_write)
 			(vp->v_rdev, uio, ap->a_ioflag);
 		VOP_LOCK(vp);
 		return (error);
@@ -332,16 +347,16 @@ spec_write(ap)
 		if (uio->uio_offset < 0)
 			return (EINVAL);
 		bsize = BLKDEV_IOSIZE;
-		if ((*bdevsw[major(vp->v_rdev)].d_ioctl)(vp->v_rdev, DIOCGPART,
+		if ((*bdevsw[major(vp->v_rdev)]->d_ioctl)(vp->v_rdev, DIOCGPART,
 		    (caddr_t)&dpart, FREAD, p) == 0) {
 			if (dpart.part->p_fstype == FS_BSDFFS &&
 			    dpart.part->p_frag != 0 && dpart.part->p_fsize != 0)
 				bsize = dpart.part->p_frag *
 				    dpart.part->p_fsize;
 		}
-		blkmask = (bsize / DEV_BSIZE) - 1;
+		blkmask = btodb(bsize) - 1;
 		do {
-			bn = (uio->uio_offset / DEV_BSIZE) &~ blkmask;
+			bn = btodb(uio->uio_offset) & ~blkmask;
 			on = uio->uio_offset % bsize;
 			n = min((unsigned)(bsize - on), uio->uio_resid);
 			if (n == bsize)
@@ -355,7 +370,8 @@ spec_write(ap)
 			}
 			error = uiomove((char *)bp->b_data + on, n, uio);
 			if (n + on == bsize) {
-				bawrite(bp);
+				/* bawrite(bp); */
+				cluster_write(bp, 0);
 			} else
 				bdwrite(bp);
 		} while (error == 0 && uio->uio_resid > 0 && n != 0);
@@ -387,16 +403,16 @@ spec_ioctl(ap)
 	switch (ap->a_vp->v_type) {
 
 	case VCHR:
-		return ((*cdevsw[major(dev)].d_ioctl)(dev, ap->a_command, ap->a_data,
+		return ((*cdevsw[major(dev)]->d_ioctl)(dev, ap->a_command, ap->a_data,
 		    ap->a_fflag, ap->a_p));
 
 	case VBLK:
 		if (ap->a_command == 0 && (int)ap->a_data == B_TAPE)
-			if (bdevsw[major(dev)].d_flags & B_TAPE)
+			if (bdevsw[major(dev)]->d_flags & B_TAPE)
 				return (0);
 			else
 				return (1);
-		return ((*bdevsw[major(dev)].d_ioctl)(dev, ap->a_command, ap->a_data,
+		return ((*bdevsw[major(dev)]->d_ioctl)(dev, ap->a_command, ap->a_data,
 		   ap->a_fflag, ap->a_p));
 
 	default:
@@ -425,7 +441,7 @@ spec_select(ap)
 
 	case VCHR:
 		dev = ap->a_vp->v_rdev;
-		return (*cdevsw[major(dev)].d_select)(dev, ap->a_which, ap->a_p);
+		return (*cdevsw[major(dev)]->d_select)(dev, ap->a_which, ap->a_p);
 	}
 }
 /*
@@ -492,7 +508,7 @@ spec_strategy(ap)
 	} */ *ap;
 {
 
-	(*bdevsw[major(ap->a_bp->b_dev)].d_strategy)(ap->a_bp);
+	(*bdevsw[major(ap->a_bp->b_dev)]->d_strategy)(ap->a_bp);
 	return (0);
 }
 
@@ -506,6 +522,8 @@ spec_bmap(ap)
 		daddr_t  a_bn;
 		struct vnode **a_vpp;
 		daddr_t *a_bnp;
+		int *a_runp;
+		int *a_runb;
 	} */ *ap;
 {
 
@@ -515,6 +533,8 @@ spec_bmap(ap)
 		*ap->a_bnp = ap->a_bn;
 	if (ap->a_runp != NULL)
 		*ap->a_runp = 0;
+	if (ap->a_runb != NULL)
+		*ap->a_runb = 0;
 	return (0);
 }
 
@@ -558,7 +578,7 @@ spec_close(ap)
 {
 	register struct vnode *vp = ap->a_vp;
 	dev_t dev = vp->v_rdev;
-	int (*devclose) __P((dev_t, int, int, struct proc *));
+	d_close_t *devclose;
 	int mode, error;
 
 	switch (vp->v_type) {
@@ -585,7 +605,7 @@ spec_close(ap)
 		 */
 		if (vcount(vp) > 1 && (vp->v_flag & VXLOCK) == 0)
 			return (0);
-		devclose = cdevsw[major(dev)].d_close;
+		devclose = cdevsw[major(dev)]->d_close;
 		mode = S_IFCHR;
 		break;
 
@@ -607,9 +627,14 @@ spec_close(ap)
 		 * sum of the reference counts on all the aliased
 		 * vnodes descends to one, we are on last close.
 		 */
-		if (vcount(vp) > 1 && (vp->v_flag & VXLOCK) == 0)
+		if ((vcount(vp) > (vp->v_object?2:1)) &&
+			(vp->v_flag & VXLOCK) == 0)
 			return (0);
-		devclose = bdevsw[major(dev)].d_close;
+
+		if (vp->v_object)
+			vnode_pager_uncache(vp);
+
+		devclose = bdevsw[major(dev)]->d_close;
 		mode = S_IFBLK;
 		break;
 
@@ -693,7 +718,7 @@ spec_advlock(ap)
 /*
  * Special device failed operation
  */
-int
+static int
 spec_ebadf()
 {
 
@@ -710,3 +735,163 @@ spec_badop()
 	panic("spec_badop called");
 	/* NOTREACHED */
 }
+
+static void
+spec_getpages_iodone(bp)
+	struct buf *bp;
+{
+
+	bp->b_flags |= B_DONE;
+	wakeup(bp);
+}
+
+int
+spec_getpages(ap)
+	struct vop_getpages_args *ap;
+{
+	vm_offset_t kva;
+	int error;
+	int i, pcount, size, s;
+	daddr_t blkno;
+	struct buf *bp;
+	vm_ooffset_t offset;
+
+	error = 0;
+	pcount = round_page(ap->a_count) / PAGE_SIZE;
+
+	/*
+	 * Calculate the offset of the transfer.
+	 */
+	offset = IDX_TO_OFF(ap->a_m[0]->pindex) + ap->a_offset;
+
+	/* XXX sanity check before we go into details. */
+	/* XXX limits should be defined elsewhere. */
+#define	DADDR_T_BIT	32
+#define	OFFSET_MAX	((1LL << (DADDR_T_BIT + DEV_BSHIFT)) - 1)
+	if (offset < 0 || offset > OFFSET_MAX) {
+		/* XXX still no %q in kernel. */
+		printf("spec_getpages: preposterous offset 0x%x%08x\n",
+		       (u_int)((u_quad_t)offset >> 32),
+		       (u_int)(offset & 0xffffffff));
+		return (VM_PAGER_ERROR);
+	}
+
+	blkno = btodb(offset);
+
+	/*
+	 * Round up physical size for real devices.
+	 */
+	size = (ap->a_count + DEV_BSIZE - 1) & ~(DEV_BSIZE - 1);
+
+	bp = getpbuf();
+	kva = (vm_offset_t)bp->b_data;
+
+	/*
+	 * Map the pages to be read into the kva.
+	 */
+	pmap_qenter(kva, ap->a_m, pcount);
+
+	/* Build a minimal buffer header. */
+	bp->b_flags = B_BUSY | B_READ | B_CALL;
+	bp->b_iodone = spec_getpages_iodone;
+
+	/* B_PHYS is not set, but it is nice to fill this in. */
+	bp->b_proc = curproc;
+	bp->b_rcred = bp->b_wcred = bp->b_proc->p_ucred;
+	if (bp->b_rcred != NOCRED)
+		crhold(bp->b_rcred);
+	if (bp->b_wcred != NOCRED)
+		crhold(bp->b_wcred);
+	bp->b_blkno = blkno;
+	bp->b_lblkno = blkno;
+	pbgetvp(ap->a_vp, bp);
+	bp->b_bcount = size;
+	bp->b_bufsize = size;
+
+	cnt.v_vnodein++;
+	cnt.v_vnodepgsin += pcount;
+
+	/* Do the input. */
+	VOP_STRATEGY(bp);
+	if (bp->b_flags & B_ASYNC)
+		return (VM_PAGER_PEND);
+
+	s = splbio();
+
+	/* We definitely need to be at splbio here. */
+	while ((bp->b_flags & B_DONE) == 0)
+		tsleep(bp, PVM, "vnread", 0);
+
+	splx(s);
+
+	if ((bp->b_flags & B_ERROR) != 0)
+		error = EIO;
+
+	if (!error && ap->a_count != pcount * PAGE_SIZE)
+		bzero((caddr_t)kva + ap->a_count,
+		      PAGE_SIZE * pcount - ap->a_count);
+	pmap_qremove(kva, pcount);
+
+	/*
+	 * Free the buffer header back to the swap buffer pool.
+	 */
+	relpbuf(bp);
+
+	for (i = 0; i < pcount; i++) {
+		ap->a_m[i]->dirty = 0;
+		ap->a_m[i]->valid = VM_PAGE_BITS_ALL;
+		ap->a_m[i]->flags &= ~PG_ZERO;
+		if (i != ap->a_reqpage) {
+			/*
+			 * Whether or not to leave the page activated is up in
+			 * the air, but we should put the page on a page queue
+			 * somewhere (it already is in the object).  Result:
+			 * It appears that emperical results show that
+			 * deactivating pages is best.
+			 */
+
+			/*
+			 * Just in case someone was asking for this page we
+			 * now tell them that it is ok to use.
+			 */
+			if (!error) {
+				vm_page_deactivate(ap->a_m[i]);
+				PAGE_WAKEUP(ap->a_m[i]);
+			} else
+				vnode_pager_freepage(ap->a_m[i]);
+		}
+	}
+	if (error)
+		printf("spec_getpages: I/O read error\n");
+	return (error ? VM_PAGER_ERROR : VM_PAGER_OK);
+}
+
+/* ARGSUSED */
+static int
+spec_getattr(ap)
+	struct vop_getattr_args /* {
+		struct vnode *a_vp;
+		struct vattr *a_vap;
+		struct ucred *a_cred;
+		struct proc *a_p;
+	} */ *ap;
+{
+	register struct vnode *vp = ap->a_vp;
+	register struct vattr *vap = ap->a_vap;
+	struct partinfo dpart;
+
+	bzero(vap, sizeof (*vap));
+
+	if (vp->v_type == VBLK)
+		vap->va_blocksize = BLKDEV_IOSIZE;
+	else if (vp->v_type == VCHR)
+		vap->va_blocksize = MAXBSIZE;
+
+	if ((*bdevsw[major(vp->v_rdev)]->d_ioctl)(vp->v_rdev, DIOCGPART,
+	    (caddr_t)&dpart, FREAD, ap->a_p) == 0) {
+		vap->va_bytes = dbtob(dpart.disklab->d_partitions
+				      [minor(vp->v_rdev)].p_size);
+		vap->va_size = vap->va_bytes;
+	}
+	return (0);
+}
