@@ -35,7 +35,7 @@
  *
  *	@(#)umap.h	8.3 (Berkeley) 1/21/94
  *
- * $FreeBSD: stable/2.1/sys/miscfs/umapfs/umap.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/umapfs/umap.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #define MAPFILEENTRIES 64
@@ -87,6 +87,6 @@ extern struct vnode *umap_checkvp __P((struct vnode *vp, char *fil, int lno));
 #define	UMAPVPTOLOWERVP(vp) (VTOUMAP(vp)->umap_lowervp)
 #endif
 
-extern int (**umap_vnodeop_p)();
+extern vop_t **umap_vnodeop_p;
 extern struct vfsops umap_vfsops;
 #endif /* KERNEL */
