@@ -35,7 +35,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)union.h	8.2 (Berkeley) 2/17/94
- * $FreeBSD: stable/2.1/sys/miscfs/union/union.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/union/union.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 struct union_args {
@@ -116,6 +116,6 @@ extern void union_newupper __P((struct union_node *, struct vnode *));
 #define	UPPERVP(vp) (VTOUNION(vp)->un_uppervp)
 #define OTHERVP(vp) (UPPERVP(vp) ? UPPERVP(vp) : LOWERVP(vp))
 
-extern int (**union_vnodeop_p)();
+extern vop_t **union_vnodeop_p;
 extern struct vfsops union_vfsops;
 #endif /* KERNEL */
