@@ -35,7 +35,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)union_vnops.c	8.6 (Berkeley) 2/17/94
- * $FreeBSD: stable/2.1/sys/miscfs/union/union_vnops.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/miscfs/union/union_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -64,6 +64,45 @@
 	} \
 }
 
+extern int	union_abortop __P((struct vop_abortop_args *ap));
+extern int	union_access __P((struct vop_access_args *ap));
+extern int	union_advlock __P((struct vop_advlock_args *ap));
+extern int	union_bmap __P((struct vop_bmap_args *ap));
+extern int	union_close __P((struct vop_close_args *ap));
+extern int	union_create __P((struct vop_create_args *ap));
+static void	union_fixup __P((struct union_node *un));
+extern int	union_fsync __P((struct vop_fsync_args *ap));
+extern int	union_getattr __P((struct vop_getattr_args *ap));
+extern int	union_inactive __P((struct vop_inactive_args *ap));
+extern int	union_ioctl __P((struct vop_ioctl_args *ap));
+extern int	union_islocked __P((struct vop_islocked_args *ap));
+extern int	union_link __P((struct vop_link_args *ap));
+extern int	union_lock __P((struct vop_lock_args *ap));
+extern int	union_lookup __P((struct vop_lookup_args *ap));
+static int	union_lookup1 __P((struct vnode *udvp, struct vnode *dvp,
+				   struct vnode **vpp,
+				   struct componentname *cnp));
+extern int	union_mkdir __P((struct vop_mkdir_args *ap));
+extern int	union_mknod __P((struct vop_mknod_args *ap));
+extern int	union_mmap __P((struct vop_mmap_args *ap));
+extern int	union_open __P((struct vop_open_args *ap));
+extern int	union_pathconf __P((struct vop_pathconf_args *ap));
+extern int	union_print __P((struct vop_print_args *ap));
+extern int	union_read __P((struct vop_read_args *ap));
+extern int	union_readdir __P((struct vop_readdir_args *ap));
+extern int	union_readlink __P((struct vop_readlink_args *ap));
+extern int	union_reclaim __P((struct vop_reclaim_args *ap));
+extern int	union_remove __P((struct vop_remove_args *ap));
+extern int	union_rename __P((struct vop_rename_args *ap));
+extern int	union_rmdir __P((struct vop_rmdir_args *ap));
+extern int	union_seek __P((struct vop_seek_args *ap));
+extern int	union_select __P((struct vop_select_args *ap));
+extern int	union_setattr __P((struct vop_setattr_args *ap));
+extern int	union_strategy __P((struct vop_strategy_args *ap));
+extern int	union_symlink __P((struct vop_symlink_args *ap));
+extern int	union_unlock __P((struct vop_lock_args *ap));
+extern int	union_write __P((struct vop_read_args *ap));
+
 static void
 union_fixup(un)
 	struct union_node *un;
@@ -898,8 +937,8 @@ union_remove(ap)
 int
 union_link(ap)
 	struct vop_link_args /* {
-		struct vnode *a_vp;
 		struct vnode *a_tdvp;
+		struct vnode *a_vp;
 		struct componentname *a_cnp;
 	} */ *ap;
 {
@@ -1337,6 +1376,7 @@ union_bmap(ap)
 		struct vnode **a_vpp;
 		daddr_t *a_bnp;
 		int *a_runp;
+		int *a_runb;
 	} */ *ap;
 {
 	int error;
@@ -1347,7 +1387,7 @@ union_bmap(ap)
 		VOP_LOCK(vp);
 	else
 		FIXUP(VTOUNION(ap->a_vp));
-	error = VOP_BMAP(vp, ap->a_bn, ap->a_vpp, ap->a_bnp, ap->a_runp);
+	error = VOP_BMAP(vp, ap->a_bn, ap->a_vpp, ap->a_bnp, ap->a_runp, ap->a_runb);
 	if (dolock)
 		VOP_UNLOCK(vp);
 
@@ -1451,52 +1491,52 @@ union_strategy(ap)
 /*
  * Global vfs data structures
  */
-int (**union_vnodeop_p)();
+vop_t **union_vnodeop_p;
 struct vnodeopv_entry_desc union_vnodeop_entries[] = {
-	{ &vop_default_desc, vn_default_error },
-	{ &vop_lookup_desc, union_lookup },		/* lookup */
-	{ &vop_create_desc, union_create },		/* create */
-	{ &vop_mknod_desc, union_mknod },		/* mknod */
-	{ &vop_open_desc, union_open },			/* open */
-	{ &vop_close_desc, union_close },		/* close */
-	{ &vop_access_desc, union_access },		/* access */
-	{ &vop_getattr_desc, union_getattr },		/* getattr */
-	{ &vop_setattr_desc, union_setattr },		/* setattr */
-	{ &vop_read_desc, union_read },			/* read */
-	{ &vop_write_desc, union_write },		/* write */
-	{ &vop_ioctl_desc, union_ioctl },		/* ioctl */
-	{ &vop_select_desc, union_select },		/* select */
-	{ &vop_mmap_desc, union_mmap },			/* mmap */
-	{ &vop_fsync_desc, union_fsync },		/* fsync */
-	{ &vop_seek_desc, union_seek },			/* seek */
-	{ &vop_remove_desc, union_remove },		/* remove */
-	{ &vop_link_desc, union_link },			/* link */
-	{ &vop_rename_desc, union_rename },		/* rename */
-	{ &vop_mkdir_desc, union_mkdir },		/* mkdir */
-	{ &vop_rmdir_desc, union_rmdir },		/* rmdir */
-	{ &vop_symlink_desc, union_symlink },		/* symlink */
-	{ &vop_readdir_desc, union_readdir },		/* readdir */
-	{ &vop_readlink_desc, union_readlink },		/* readlink */
-	{ &vop_abortop_desc, union_abortop },		/* abortop */
-	{ &vop_inactive_desc, union_inactive },		/* inactive */
-	{ &vop_reclaim_desc, union_reclaim },		/* reclaim */
-	{ &vop_lock_desc, union_lock },			/* lock */
-	{ &vop_unlock_desc, union_unlock },		/* unlock */
-	{ &vop_bmap_desc, union_bmap },			/* bmap */
-	{ &vop_strategy_desc, union_strategy },		/* strategy */
-	{ &vop_print_desc, union_print },		/* print */
-	{ &vop_islocked_desc, union_islocked },		/* islocked */
-	{ &vop_pathconf_desc, union_pathconf },		/* pathconf */
-	{ &vop_advlock_desc, union_advlock },		/* advlock */
+	{ &vop_default_desc, (vop_t *)vn_default_error },
+	{ &vop_lookup_desc, (vop_t *)union_lookup },		/* lookup */
+	{ &vop_create_desc, (vop_t *)union_create },		/* create */
+	{ &vop_mknod_desc, (vop_t *)union_mknod },		/* mknod */
+	{ &vop_open_desc, (vop_t *)union_open },		/* open */
+	{ &vop_close_desc, (vop_t *)union_close },		/* close */
+	{ &vop_access_desc, (vop_t *)union_access },		/* access */
+	{ &vop_getattr_desc, (vop_t *)union_getattr },		/* getattr */
+	{ &vop_setattr_desc, (vop_t *)union_setattr },		/* setattr */
+	{ &vop_read_desc, (vop_t *)union_read },		/* read */
+	{ &vop_write_desc, (vop_t *)union_write },		/* write */
+	{ &vop_ioctl_desc, (vop_t *)union_ioctl },		/* ioctl */
+	{ &vop_select_desc, (vop_t *)union_select },		/* select */
+	{ &vop_mmap_desc, (vop_t *)union_mmap },		/* mmap */
+	{ &vop_fsync_desc, (vop_t *)union_fsync },		/* fsync */
+	{ &vop_seek_desc, (vop_t *)union_seek },		/* seek */
+	{ &vop_remove_desc, (vop_t *)union_remove },		/* remove */
+	{ &vop_link_desc, (vop_t *)union_link },		/* link */
+	{ &vop_rename_desc, (vop_t *)union_rename },		/* rename */
+	{ &vop_mkdir_desc, (vop_t *)union_mkdir },		/* mkdir */
+	{ &vop_rmdir_desc, (vop_t *)union_rmdir },		/* rmdir */
+	{ &vop_symlink_desc, (vop_t *)union_symlink },		/* symlink */
+	{ &vop_readdir_desc, (vop_t *)union_readdir },		/* readdir */
+	{ &vop_readlink_desc, (vop_t *)union_readlink },	/* readlink */
+	{ &vop_abortop_desc, (vop_t *)union_abortop },		/* abortop */
+	{ &vop_inactive_desc, (vop_t *)union_inactive },	/* inactive */
+	{ &vop_reclaim_desc, (vop_t *)union_reclaim },		/* reclaim */
+	{ &vop_lock_desc, (vop_t *)union_lock },		/* lock */
+	{ &vop_unlock_desc, (vop_t *)union_unlock },		/* unlock */
+	{ &vop_bmap_desc, (vop_t *)union_bmap },		/* bmap */
+	{ &vop_strategy_desc, (vop_t *)union_strategy },	/* strategy */
+	{ &vop_print_desc, (vop_t *)union_print },		/* print */
+	{ &vop_islocked_desc, (vop_t *)union_islocked },	/* islocked */
+	{ &vop_pathconf_desc, (vop_t *)union_pathconf },	/* pathconf */
+	{ &vop_advlock_desc, (vop_t *)union_advlock },		/* advlock */
 #ifdef notdef
-	{ &vop_blkatoff_desc, union_blkatoff },		/* blkatoff */
-	{ &vop_valloc_desc, union_valloc },		/* valloc */
-	{ &vop_vfree_desc, union_vfree },		/* vfree */
-	{ &vop_truncate_desc, union_truncate },		/* truncate */
-	{ &vop_update_desc, union_update },		/* update */
-	{ &vop_bwrite_desc, union_bwrite },		/* bwrite */
+	{ &vop_blkatoff_desc, (vop_t *)union_blkatoff },	/* blkatoff */
+	{ &vop_valloc_desc, (vop_t *)union_valloc },		/* valloc */
+	{ &vop_vfree_desc, (vop_t *)union_vfree },		/* vfree */
+	{ &vop_truncate_desc, (vop_t *)union_truncate },	/* truncate */
+	{ &vop_update_desc, (vop_t *)union_update },		/* update */
+	{ &vop_bwrite_desc, (vop_t *)union_bwrite },		/* bwrite */
 #endif
-	{ (struct vnodeop_desc*)NULL, (int(*)())NULL }
+	{ NULL, NULL }
 };
 struct vnodeopv_desc union_vnodeop_opv_desc =
 	{ &union_vnodeop_p, union_vnodeop_entries };
