@@ -37,7 +37,7 @@
  *
  *      @(#)bpf.c	8.2 (Berkeley) 3/28/94
  *
- * $FreeBSD: stable/2.1/sys/net/bpf.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/net/bpf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "bpfilter.h"
@@ -52,12 +52,12 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
-#include <machine/cpu.h>	/* for bootverbose */
+#include <sys/conf.h>
 #include <sys/mbuf.h>
 #include <sys/buf.h>
 #include <sys/time.h>
 #include <sys/proc.h>
-#include <sys/user.h>
+#include <sys/signalvar.h>
 #include <sys/ioctl.h>
 
 #include <sys/file.h>
@@ -66,8 +66,9 @@
 #endif
 #include <sys/uio.h>
 
-#include <sys/protosw.h>
 #include <sys/socket.h>
+#include <sys/socketvar.h>
+#include <sys/protosw.h>
 #include <net/if.h>
 
 #include <net/bpf.h>
@@ -78,6 +79,12 @@
 #include <netinet/in.h>
 #include <netinet/if_ether.h>
 #include <sys/kernel.h>
+#include <sys/sysctl.h>
+#include <sys/conf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
+
 
 /*
  * Older BSDs don't have kernel malloc.
@@ -98,29 +105,21 @@ static caddr_t bpf_alloc();
 /*
  * The default read buffer size is patchable.
  */
-int bpf_bufsize = BPF_BUFSIZE;
+static int bpf_bufsize = BPF_BUFSIZE;
+SYSCTL_INT(_debug, OID_AUTO, bpf_bufsize, CTLFLAG_RW, 
+	&bpf_bufsize, 0, "");
 
 /*
  *  bpf_iflist is the list of interfaces; each corresponds to an ifnet
  *  bpf_dtab holds the descriptors, indexed by minor device #
  */
-struct bpf_if	*bpf_iflist;
-struct bpf_d	bpf_dtab[NBPFILTER];
-
-#if BSD >= 199207
-/*
- * bpfilterattach() is called at boot time in new systems.  We do
- * nothing here since old systems will not call this.
- */
-/* ARGSUSED */
-void
-bpfilterattach(n)
-	int n;
-{
-}
-#endif
+static struct bpf_if	*bpf_iflist;
+static struct bpf_d	bpf_dtab[NBPFILTER];
+static int		bpf_dtab_init;
 
 static int	bpf_allocbufs __P((struct bpf_d *));
+static void	bpf_attachd __P((struct bpf_d *d, struct bpf_if *bp));
+static void	bpf_detachd __P((struct bpf_d *d));
 static void	bpf_freed __P((struct bpf_d *));
 static void	bpf_ifname __P((struct ifnet *, struct ifreq *));
 static void	bpf_mcopy __P((const void *, void *, u_int));
@@ -132,6 +131,21 @@ static inline void
 static void	catchpacket __P((struct bpf_d *, u_char *, u_int,
 		    u_int, void (*)(const void *, void *, u_int)));
 static void	reset_d __P((struct bpf_d *));
+static int	 bpf_setf __P((struct bpf_d *, struct bpf_program *));
+
+static	d_open_t	bpfopen;
+static	d_close_t	bpfclose;
+static	d_read_t	bpfread;
+static	d_write_t	bpfwrite;
+static	d_ioctl_t	bpfioctl;
+static	d_select_t	bpfselect;
+
+#define CDEV_MAJOR 23
+static struct cdevsw bpf_cdevsw = 
+ 	{ bpfopen,	bpfclose,	bpfread,	bpfwrite,	/*23*/
+ 	  bpfioctl,	nostop,		nullreset,	nodevtotty,/* bpf */
+ 	  bpfselect,	nommap,		NULL,	"bpf",	NULL,	-1 };
+
 
 static int
 bpf_movein(uio, linktype, mp, sockp, datlen)
@@ -214,6 +228,7 @@ bpf_movein(uio, linktype, mp, sockp, datlen)
 	 * Make room for link header.
 	 */
 	if (hlen != 0) {
+		m->m_pkthdr.len -= hlen;
 		m->m_len -= hlen;
 #if BSD >= 199103
 		m->m_data += hlen; /* XXX */
@@ -250,7 +265,7 @@ bpf_attachd(d, bp)
 	d->bd_next = bp->bif_dlist;
 	bp->bif_dlist = d;
 
-	*bp->bif_driverp = bp;
+	bp->bif_ifp->if_bpf = bp;
 }
 
 /*
@@ -290,7 +305,7 @@ bpf_detachd(d)
 		/*
 		 * Let the driver know that there are no more listeners.
 		 */
-		*d->bd_bif->bif_driverp = 0;
+		d->bd_bif->bif_ifp->if_bpf = 0;
 	d->bd_bif = 0;
 }
 
@@ -309,7 +324,7 @@ bpf_detachd(d)
  * EBUSY if file is open by another process.
  */
 /* ARGSUSED */
-int
+static	int
 bpfopen(dev, flags, fmt, p)
 	dev_t dev;
 	int flags;
@@ -341,7 +356,7 @@ bpfopen(dev, flags, fmt, p)
  * deallocating its buffers, and marking it free.
  */
 /* ARGSUSED */
-int
+static	int
 bpfclose(dev, flags, fmt, p)
 	dev_t dev;
 	int flags;
@@ -413,7 +428,7 @@ bpf_sleep(d)
 /*
  *  bpfread - read next chunk of packets from buffers
  */
-int
+static	int
 bpfread(dev, uio, ioflag)
 	dev_t dev;
 	register struct uio *uio;
@@ -528,7 +543,7 @@ bpf_wakeup(d)
 #endif
 }
 
-int
+static	int
 bpfwrite(dev, uio, ioflag)
 	dev_t dev;
 	struct uio *uio;
@@ -605,7 +620,7 @@ reset_d(d)
  *  BIOCVERSION		Get filter language version.
  */
 /* ARGSUSED */
-int
+static	int
 bpfioctl(dev, cmd, addr, flags, p)
 	dev_t dev;
 	int cmd;
@@ -852,7 +867,7 @@ bpfioctl(dev, cmd, addr, flags, p)
  * Set d's packet filter program to fp.  If this file already has a filter,
  * free it and replace it.  Returns EINVAL for bogus requests.
  */
-int
+static int
 bpf_setf(d, fp)
 	struct bpf_d *d;
 	struct bpf_program *fp;
@@ -905,35 +920,20 @@ bpf_setif(d, ifr)
 	struct ifreq *ifr;
 {
 	struct bpf_if *bp;
-	char *cp;
-	int unit, s, error;
+	int s, error;
+	struct ifnet *theywant;
+
+	theywant = ifunit(ifr->ifr_name);
+	if (theywant == 0)
+		return ENXIO;
 
-	/*
-	 * Separate string into name part and unit number.  Put a null
-	 * byte at the end of the name part, and compute the number.
-	 * If the a unit number is unspecified, the default is 0,
-	 * as initialized above.  XXX This should be common code.
-	 */
-	unit = 0;
-	cp = ifr->ifr_name;
-	cp[sizeof(ifr->ifr_name) - 1] = '\0';
-	while (*cp++) {
-		if (*cp >= '0' && *cp <= '9') {
-			unit = *cp - '0';
-			*cp++ = '\0';
-			while (*cp)
-				unit = 10 * unit + *cp++ - '0';
-			break;
-		}
-	}
 	/*
 	 * Look through attached interfaces for the named one.
 	 */
 	for (bp = bpf_iflist; bp != 0; bp = bp->bif_next) {
 		struct ifnet *ifp = bp->bif_ifp;
 
-		if (ifp == 0 || unit != ifp->if_unit
-		    || strcmp(ifp->if_name, ifr->ifr_name) != 0)
+		if (ifp == 0 || ifp != theywant)
 			continue;
 		/*
 		 * We found the requested interface.
@@ -995,7 +995,7 @@ bpf_ifname(ifp, ifr)
 #if BSD >= 199103
 #define bpf_select bpfselect
 #else
-int
+static	int
 bpfselect(dev, rw)
 	register dev_t dev;
 	int rw;
@@ -1060,8 +1060,8 @@ bpf_select(dev, rw, p)
  * buffer.
  */
 void
-bpf_tap(arg, pkt, pktlen)
-	caddr_t arg;
+bpf_tap(ifp, pkt, pktlen)
+	struct ifnet *ifp;
 	register u_char *pkt;
 	register u_int pktlen;
 {
@@ -1073,7 +1073,7 @@ bpf_tap(arg, pkt, pktlen)
 	 * The only problem that could arise here is that if two different
 	 * interfaces shared any data.  This is not the case.
 	 */
-	bp = (struct bpf_if *)arg;
+	bp = ifp->if_bpf;
 	for (d = bp->bif_dlist; d != 0; d = d->bd_next) {
 		++d->bd_rcount;
 		slen = bpf_filter(d->bd_filter, pkt, pktlen, pktlen);
@@ -1102,7 +1102,7 @@ bpf_mcopy(src_arg, dst_arg, len)
 		if (m == 0)
 			panic("bpf_mcopy");
 		count = min(m->m_len, len);
-		(void)memcpy((caddr_t)dst, mtod(m, caddr_t), count);
+		bcopy(mtod(m, void *), dst, count);
 		m = m->m_next;
 		dst += count;
 		len -= count;
@@ -1113,11 +1113,11 @@ bpf_mcopy(src_arg, dst_arg, len)
  * Incoming linkage from device drivers, when packet is in an mbuf chain.
  */
 void
-bpf_mtap(arg, m)
-	caddr_t arg;
+bpf_mtap(ifp, m)
+	struct ifnet *ifp;
 	struct mbuf *m;
 {
-	struct bpf_if *bp = (struct bpf_if *)arg;
+	struct bpf_if *bp = ifp->if_bpf;
 	struct bpf_d *d;
 	u_int pktlen, slen;
 	struct mbuf *m0;
@@ -1264,33 +1264,24 @@ bpf_freed(d)
  * size of the link header (variable length headers not yet supported).
  */
 void
-bpfattach(driverp, ifp, dlt, hdrlen)
-	caddr_t *driverp;
+bpfattach(ifp, dlt, hdrlen)
 	struct ifnet *ifp;
 	u_int dlt, hdrlen;
 {
 	struct bpf_if *bp;
 	int i;
-#if BSD < 199103
-	static struct bpf_if bpf_ifs[NBPFILTER];
-	static int bpfifno;
-
-	bp = (bpfifno < NBPFILTER) ? &bpf_ifs[bpfifno++] : 0;
-#else
 	bp = (struct bpf_if *)malloc(sizeof(*bp), M_DEVBUF, M_DONTWAIT);
-#endif
 	if (bp == 0)
 		panic("bpfattach");
 
 	bp->bif_dlist = 0;
-	bp->bif_driverp = (struct bpf_if **)driverp;
 	bp->bif_ifp = ifp;
 	bp->bif_dlt = dlt;
 
 	bp->bif_next = bpf_iflist;
 	bpf_iflist = bp;
 
-	*bp->bif_driverp = 0;
+	bp->bif_ifp->if_bpf = 0;
 
 	/*
 	 * Compute the length of the bpf header.  This is not necessarily
@@ -1303,77 +1294,44 @@ bpfattach(driverp, ifp, dlt, hdrlen)
 	/*
 	 * Mark all the descriptors free if this hasn't been done.
 	 */
-	if (!D_ISFREE(&bpf_dtab[0]))
+	if (!bpf_dtab_init) {
 		for (i = 0; i < NBPFILTER; ++i)
 			D_MARKFREE(&bpf_dtab[i]);
+		bpf_dtab_init = 1;
+	}
 
 	if (bootverbose)
 		printf("bpf: %s%d attached\n", ifp->if_name, ifp->if_unit);
 }
 
-#if BSD >= 199103
-/* XXX This routine belongs in net/if.c. */
-/*
- * Set/clear promiscuous mode on interface ifp based on the truth value
- * of pswitch.  The calls are reference counted so that only the first
- * "on" request actually has an effect, as does the final "off" request.
- * Results are undefined if the "off" and "on" requests are not matched.
- */
-int
-ifpromisc(ifp, pswitch)
-	struct ifnet *ifp;
-	int pswitch;
-{
-	struct ifreq ifr;
-
-	if (pswitch) {
-		/*
-		 * If the device is not configured up, we cannot put it in
-		 * promiscuous mode.
-		 */
-		if ((ifp->if_flags & IFF_UP) == 0)
-			return (ENETDOWN);
-		if (ifp->if_pcount++ != 0)
-			return (0);
-		ifp->if_flags |= IFF_PROMISC;
-	} else {
-		if (--ifp->if_pcount > 0)
-			return (0);
-		ifp->if_flags &= ~IFF_PROMISC;
-	}
-	ifr.ifr_flags = ifp->if_flags;
-	return ((*ifp->if_ioctl)(ifp, SIOCSIFFLAGS, (caddr_t)&ifr));
-}
+#ifdef DEVFS
+static	void *bpf_devfs_token[NBPFILTER];
 #endif
 
-#if BSD < 199103
-/*
- * Allocate some memory for bpf.  This is temporary SunOS support, and
- * is admittedly a hack.
- * If resources unavaiable, return 0.
- */
-static caddr_t
-bpf_alloc(size, canwait)
-	register int size;
-	register int canwait;
+static bpf_devsw_installed = 0;
+
+static void 	bpf_drvinit(void *unused)
 {
-	register struct mbuf *m;
+	dev_t dev;
+#ifdef DEVFS
+	int i;
+#endif
 
-	if ((unsigned)size > (MCLBYTES-8))
-		return 0;
+	if( ! bpf_devsw_installed ) {
+		dev = makedev(CDEV_MAJOR, 0);
+		cdevsw_add(&dev,&bpf_cdevsw, NULL);
+		bpf_devsw_installed = 1;
+#ifdef DEVFS
 
-	MGET(m, canwait, MT_DATA);
-	if (m == 0)
-		return 0;
-	if ((unsigned)size > (MLEN-8)) {
-		MCLGET(m);
-		if (m->m_len != MCLBYTES) {
-			m_freem(m);
-			return 0;
+		for ( i = 0 ; i < NBPFILTER ; i++ ) {
+			bpf_devfs_token[i] =
+				devfs_add_devswf(&bpf_cdevsw, i, DV_CHR, 0, 0, 
+						 0600, "bpf%d", i);
 		}
-	}
-	*mtod(m, struct mbuf **) = m;
-	return mtod(m, caddr_t) + 8;
+#endif
+    	}
 }
-#endif
+
+SYSINIT(bpfdev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,bpf_drvinit,NULL)
+
 #endif
