@@ -37,7 +37,7 @@
  *
  *      @(#)bpf_filter.c	8.1 (Berkeley) 6/10/93
  *
- * $FreeBSD: stable/2.1/sys/net/bpf_filter.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/net/bpf_filter.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -82,6 +82,9 @@
 	} \
 }
 
+static int	m_xhalf __P((struct mbuf *m, int k, int *err));
+static int	m_xword __P((struct mbuf *m, int k, int *err));
+
 static int
 m_xword(m, k, err)
 	register struct mbuf *m;
