@@ -37,7 +37,7 @@
  *
  *      @(#)bpfdesc.h	8.1 (Berkeley) 6/10/93
  *
- * $FreeBSD: stable/2.1/sys/net/bpfdesc.h 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/net/bpfdesc.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _NET_BPFDESC_H_
@@ -95,14 +95,9 @@ struct bpf_d {
 struct bpf_if {
 	struct bpf_if *bif_next;	/* list of all interfaces */
 	struct bpf_d *bif_dlist;	/* descriptor list */
-	struct bpf_if **bif_driverp;	/* pointer into softc */
 	u_int bif_dlt;			/* link layer type */
 	u_int bif_hdrlen;		/* length of header (with padding) */
-	struct ifnet *bif_ifp;		/* correspoding interface */
+	struct ifnet *bif_ifp;		/* corresponding interface */
 };
 
-#ifdef KERNEL
-int	 bpf_setf __P((struct bpf_d *, struct bpf_program *));
-#endif
-
 #endif
