@@ -31,10 +31,11 @@
  * SUCH DAMAGE.
  *
  *	@(#)if.c	8.3 (Berkeley) 1/4/94
- * $FreeBSD: stable/2.1/sys/net/if.c 50950 1999-09-05 08:39:22Z peter $
+ * $FreeBSD: stable/2.2/sys/net/if.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
+#include <sys/queue.h>
 #include <sys/mbuf.h>
 #include <sys/systm.h>
 #include <sys/proc.h>
@@ -44,12 +45,26 @@
 #include <sys/kernel.h>
 #include <sys/ioctl.h>
 #include <sys/errno.h>
+#include <sys/syslog.h>
+#include <sys/sysctl.h>
 
 #include <net/if.h>
 #include <net/if_dl.h>
 #include <net/if_types.h>
 #include <net/radix.h>
-#include <ether.h>
+
+/*
+ * System initialization
+ */
+
+static int ifconf __P((int, caddr_t));
+static void ifinit __P((void *));
+static void if_qflush __P((struct ifqueue *));
+static void if_slowtimo __P((void *));
+static void link_rtrequest __P((int, struct rtentry *, struct sockaddr *));
+
+SYSINIT(interfaces, SI_SUB_PROTO_IF, SI_ORDER_FIRST, ifinit, NULL)
+
 
 int	ifqmaxlen = IFQ_MAXLEN;
 struct	ifnet *ifnet;
@@ -59,9 +74,13 @@ struct	ifnet *ifnet;
  *
  * Routines with ifa_ifwith* names take sockaddr *'s as
  * parameters.
+ *
+ * This routine assumes that it will be called at splimp() or higher.
  */
+/* ARGSUSED*/
 void
-ifinit()
+ifinit(dummy)
+	void *dummy;
 {
 	register struct ifnet *ifp;
 
@@ -71,25 +90,9 @@ ifinit()
 	if_slowtimo(0);
 }
 
-#ifdef vax
-/*
- * Call each interface on a Unibus reset.
- */
-void
-ifubareset(uban)
-	int uban;
-{
-	register struct ifnet *ifp;
-
-	for (ifp = ifnet; ifp; ifp = ifp->if_next)
-		if (ifp->if_reset)
-			(*ifp->if_reset)(ifp->if_unit, uban);
-}
-#endif
-
 int if_index = 0;
 struct ifaddr **ifnet_addrs;
-static char *sprint_d __P((u_int, char *, int));
+
 
 /*
  * Attach an interface to the
@@ -100,8 +103,8 @@ if_attach(ifp)
 	struct ifnet *ifp;
 {
 	unsigned socksize, ifasize;
-	int namelen, unitlen, masklen;
-	char workbuf[12], *unitname;
+	int namelen, masklen;
+	char workbuf[64];
 	register struct ifnet **p = &ifnet;
 	register struct sockaddr_dl *sdl;
 	register struct ifaddr *ifa;
@@ -127,12 +130,9 @@ if_attach(ifp)
 	/*
 	 * create a Link Level name for this device
 	 */
-	unitname = sprint_d((u_int)ifp->if_unit, workbuf, sizeof(workbuf));
-	namelen = strlen(ifp->if_name);
-	unitlen = strlen(unitname);
+	namelen = sprintf(workbuf, "%s%d", ifp->if_name, ifp->if_unit);
 #define _offsetof(t, m) ((int)((caddr_t)&((t *)0)->m))
-	masklen = _offsetof(struct sockaddr_dl, sdl_data[0]) +
-			       unitlen + namelen;
+	masklen = _offsetof(struct sockaddr_dl, sdl_data[0]) + namelen;
 	socksize = masklen + ifp->if_addrlen;
 #define ROUNDUP(a) (1 + (((a) - 1) | (sizeof(long) - 1)))
 	socksize = ROUNDUP(socksize);
@@ -145,9 +145,8 @@ if_attach(ifp)
 		sdl = (struct sockaddr_dl *)(ifa + 1);
 		sdl->sdl_len = socksize;
 		sdl->sdl_family = AF_LINK;
-		bcopy(ifp->if_name, sdl->sdl_data, namelen);
-		bcopy(unitname, namelen + (caddr_t)sdl->sdl_data, unitlen);
-		sdl->sdl_nlen = (namelen += unitlen);
+		bcopy(workbuf, sdl->sdl_data, namelen);
+		sdl->sdl_nlen = namelen;
 		sdl->sdl_index = ifp->if_index;
 		sdl->sdl_type = ifp->if_type;
 		ifnet_addrs[if_index - 1] = ifa;
@@ -156,17 +155,13 @@ if_attach(ifp)
 		ifa->ifa_rtrequest = link_rtrequest;
 		ifp->if_addrlist = ifa;
 		ifa->ifa_addr = (struct sockaddr *)sdl;
+
 		sdl = (struct sockaddr_dl *)(socksize + (caddr_t)sdl);
 		ifa->ifa_netmask = (struct sockaddr *)sdl;
 		sdl->sdl_len = masklen;
 		while (namelen != 0)
 			sdl->sdl_data[--namelen] = 0xff;
 	}
-	/* XXX -- Temporary fix before changing 10 ethernet drivers */
-#if NETHER > 0
-	if (ifp->if_output == ether_output)
-		ether_ifattach(ifp);
-#endif
 }
 /*
  * Locate an interface based on a complete address.
@@ -241,9 +236,29 @@ ifa_ifwithnet(addr)
 			if (ifa->ifa_addr->sa_family != af)
 				next: continue;
 			if (ifp->if_flags & IFF_POINTOPOINT) {
-				if (equal(addr, ifa->ifa_dstaddr))
+				if (ifa->ifa_dstaddr != 0
+				    && equal(addr, ifa->ifa_dstaddr))
  					return (ifa);
 			} else {
+				/*
+				 * if we have a special address handler,
+				 * then use it instead of the generic one.
+				 */
+	          		if (ifa->ifa_claim_addr) {
+					if ((*ifa->ifa_claim_addr)(ifa, addr)) {
+						return (ifa);
+					} else {
+						continue;
+					}
+				}
+
+				/*
+				 * Scan all the bits in the ifa's address.
+				 * If a bit dissagrees with what we are
+				 * looking for, mask it with the netmask
+				 * to see if it really matters.
+				 * (A byte at a time)
+				 */
 				if (ifa->ifa_netmask == 0)
 					continue;
 				cp = addr_data;
@@ -263,23 +278,6 @@ ifa_ifwithnet(addr)
 	return (ifa_maybe);
 }
 
-/*
- * Find an interface using a specific address family
- */
-struct ifaddr *
-ifa_ifwithaf(af)
-	register int af;
-{
-	register struct ifnet *ifp;
-	register struct ifaddr *ifa;
-
-	for (ifp = ifnet; ifp; ifp = ifp->if_next)
-	    for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next)
-		if (ifa->ifa_addr->sa_family == af)
-			return (ifa);
-	return ((struct ifaddr *)0);
-}
-
 /*
  * Find an interface address specific to an interface best matching
  * a given address.
@@ -300,7 +298,8 @@ ifaof_ifpforaddr(addr, ifp)
 	for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next) {
 		if (ifa->ifa_addr->sa_family != af)
 			continue;
-		ifa_maybe = ifa;
+		if (ifa_maybe == 0)
+			ifa_maybe = ifa;
 		if (ifa->ifa_netmask == 0) {
 			if (equal(addr, ifa->ifa_addr) ||
 			    (ifa->ifa_dstaddr && equal(addr, ifa->ifa_dstaddr)))
@@ -332,7 +331,7 @@ ifaof_ifpforaddr(addr, ifp)
  * Lookup an appropriate real ifa to point to.
  * This should be moved to /sys/net/link.c eventually.
  */
-void
+static void
 link_rtrequest(cmd, rt, sa)
 	int cmd;
 	register struct rtentry *rt;
@@ -398,7 +397,7 @@ if_up(ifp)
 /*
  * Flush an interface queue.
  */
-void
+static void
 if_qflush(ifq)
 	register struct ifqueue *ifq;
 {
@@ -419,7 +418,7 @@ if_qflush(ifq)
  * from softclock, we decrement timers (if set) and
  * call the appropriate interface routine on expiration.
  */
-void
+static void
 if_slowtimo(arg)
 	void *arg;
 {
@@ -430,7 +429,7 @@ if_slowtimo(arg)
 		if (ifp->if_timer == 0 || --ifp->if_timer)
 			continue;
 		if (ifp->if_watchdog)
-			(*ifp->if_watchdog)(ifp->if_unit);
+			(*ifp->if_watchdog)(ifp);
 	}
 	splx(s);
 	timeout(if_slowtimo, (void *)0, hz / IFNET_SLOWHZ);
@@ -444,35 +443,47 @@ struct ifnet *
 ifunit(name)
 	register char *name;
 {
-	register char *cp;
+	char namebuf[IFNAMSIZ + 1];
+	register char *cp, *cp2;
+	char *end;
 	register struct ifnet *ifp;
 	int unit;
 	unsigned len;
-	char *ep, c;
+	register char c = '\0';
 
-	for (cp = name; cp < name + IFNAMSIZ && *cp; cp++)
-		if (*cp >= '0' && *cp <= '9')
-			break;
-	if (*cp == '\0' || cp == name + IFNAMSIZ)
-		return ((struct ifnet *)0);
 	/*
-	 * Save first char of unit, and pointer to it,
-	 * so we can put a null there to avoid matching
-	 * initial substrings of interface names.
+	 * Look for a non numeric part
+	 */
+	end = name + IFNAMSIZ; 
+	cp2 = namebuf;
+	cp = name; 
+	while ((cp < end) && (c = *cp)) {
+		if (c >= '0' && c <= '9')
+			break;
+		*cp2++ = c;
+		cp++;
+	}
+	if ((cp == end) || (c == '\0') || (cp == name))
+		return ((struct ifnet *)0);
+	*cp2 = '\0';
+	/*
+	 * check we have a legal number (limit to 7 digits?)
 	 */
 	len = cp - name + 1;
-	c = *cp;
-	ep = cp;
-	for (unit = 0; *cp >= '0' && *cp <= '9'; )
-		unit = unit * 10 + *cp++ - '0';
-	*ep = 0;
+	for (unit = 0;
+	    ((c = *cp) >= '0') && (c <= '9') && (unit < 1000000); cp++ ) 
+		unit = (unit * 10) + (c - '0');
+	if (*cp != '\0')
+		return 0;	/* no trailing garbage allowed */
+	/*
+	 * Now search all the interfaces for this name/number
+	 */
 	for (ifp = ifnet; ifp; ifp = ifp->if_next) {
-		if (bcmp(ifp->if_name, name, len))
+		if (bcmp(ifp->if_name, namebuf, len))
 			continue;
 		if (unit == ifp->if_unit)
 			break;
 	}
-	*ep = c;
 	return (ifp);
 }
 
@@ -550,7 +561,7 @@ ifioctl(so, cmd, data, p)
 	case SIOCSIFPHYS:
 		error = suser(p->p_ucred, &p->p_acflag);
 		if (error)
-		        return error;
+			return error;
 		if (!ifp->if_ioctl)
 		        return EOPNOTSUPP;
 		error = (*ifp->if_ioctl)(ifp, cmd, data);
@@ -587,12 +598,29 @@ ifioctl(so, cmd, data, p)
 		    	microtime(&ifp->if_lastchange);
 		return(error);
 
+        case SIOCSIFMEDIA:
+		error = suser(p->p_ucred, &p->p_acflag);
+		if (error)
+			return (error);
+		if (ifp->if_ioctl == 0)
+			return (EOPNOTSUPP);
+		error = (*ifp->if_ioctl)(ifp, cmd, data);
+		if (error == 0)
+			microtime(&ifp->if_lastchange);
+		return error;
+
+	case SIOCGIFMEDIA:
+		if (ifp->if_ioctl == 0)
+			return (EOPNOTSUPP);
+		return ((*ifp->if_ioctl)(ifp, cmd, data));
+
 	default:
 		if (so->so_proto == 0)
 			return (EOPNOTSUPP);
 #ifndef COMPAT_43
-		return ((*so->so_proto->pr_usrreq)(so, PRU_CONTROL,
-			cmd, data, ifp));
+		return ((*so->so_proto->pr_usrreqs->pru_control)(so, cmd,
+								 data,
+								 ifp));
 #else
 	    {
 		int ocmd = cmd;
@@ -630,8 +658,10 @@ ifioctl(so, cmd, data, p)
 		case OSIOCGIFNETMASK:
 			cmd = SIOCGIFNETMASK;
 		}
-		error =  ((*so->so_proto->pr_usrreq)(so, PRU_CONTROL,
-							    cmd, data, ifp));
+		error =  ((*so->so_proto->pr_usrreqs->pru_control)(so,
+								   cmd,
+								   data,
+								   ifp));
 		switch (ocmd) {
 
 		case OSIOCGIFADDR:
@@ -648,6 +678,40 @@ ifioctl(so, cmd, data, p)
 	return (0);
 }
 
+/*
+ * Set/clear promiscuous mode on interface ifp based on the truth value
+ * of pswitch.  The calls are reference counted so that only the first
+ * "on" request actually has an effect, as does the final "off" request.
+ * Results are undefined if the "off" and "on" requests are not matched.
+ */
+int
+ifpromisc(ifp, pswitch)
+	struct ifnet *ifp;
+	int pswitch;
+{
+	struct ifreq ifr;
+
+	if (pswitch) {
+		/*
+		 * If the device is not configured up, we cannot put it in
+		 * promiscuous mode.
+		 */
+		if ((ifp->if_flags & IFF_UP) == 0)
+			return (ENETDOWN);
+		if (ifp->if_pcount++ != 0)
+			return (0);
+		ifp->if_flags |= IFF_PROMISC;
+		log(LOG_INFO, "%s%d: promiscuous mode enabled\n",
+		    ifp->if_name, ifp->if_unit);
+	} else {
+		if (--ifp->if_pcount > 0)
+			return (0);
+		ifp->if_flags &= ~IFF_PROMISC;
+	}
+	ifr.ifr_flags = ifp->if_flags;
+	return ((*ifp->if_ioctl)(ifp, SIOCSIFFLAGS, (caddr_t)&ifr));
+}
+
 /*
  * Return interface configuration
  * of system.  List may be used
@@ -655,7 +719,7 @@ ifioctl(so, cmd, data, p)
  * other information.
  */
 /*ARGSUSED*/
-int
+static int
 ifconf(cmd, data)
 	int cmd;
 	caddr_t data;
@@ -668,17 +732,14 @@ ifconf(cmd, data)
 
 	ifrp = ifc->ifc_req;
 	for (; space > sizeof (ifr) && ifp; ifp = ifp->if_next) {
-		char workbuf[12], *unitname;
-		int unitlen, ifnlen;
+		char workbuf[64];
+		int ifnlen;
 
-		unitname = sprint_d(ifp->if_unit, workbuf, sizeof workbuf);
-		unitlen = strlen(unitname);
-		ifnlen = strlen(ifp->if_name);
-		if(unitlen + ifnlen + 1 > sizeof ifr.ifr_name) {
+		ifnlen = sprintf(workbuf, "%s%d", ifp->if_name, ifp->if_unit);
+		if(ifnlen + 1 > sizeof ifr.ifr_name) {
 			error = ENAMETOOLONG;
 		} else {
-			strcpy(ifr.ifr_name, ifp->if_name);
-			strcpy(&ifr.ifr_name[ifnlen], unitname);
+			strcpy(ifr.ifr_name, workbuf);
 		}
 
 		if ((ifa = ifp->if_addrlist) == 0) {
@@ -728,19 +789,5 @@ ifconf(cmd, data)
 	return (error);
 }
 
-static char *
-sprint_d(n, buf, buflen)
-	u_int n;
-	char *buf;
-	int buflen;
-{
-	register char *cp = buf + buflen - 1;
-
-	*cp = 0;
-	do {
-		cp--;
-		*cp = "0123456789"[n % 10];
-		n /= 10;
-	} while (n != 0);
-	return (cp);
-}
+SYSCTL_NODE(_net, PF_LINK, link, CTLFLAG_RW, 0, "Link layers");
+SYSCTL_NODE(_net_link, 0, generic, CTLFLAG_RW, 0, "Generic link-management");
