@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)if.h	8.1 (Berkeley) 6/10/93
- * $FreeBSD: stable/2.1/sys/net/if.h 60866 2000-05-24 10:20:27Z peter $
+ * $FreeBSD: stable/2.2/sys/net/if.h 57027 2000-02-08 02:19:47Z peter $
  */
 
 #ifndef _NET_IF_H_
@@ -62,8 +62,6 @@
  * interfaces.  These routines live in the files if.c and route.c
  */
 
-#include <sys/socket.h>			/* for struct sockaddr */
-
 #ifndef _TIME_ /*  XXX fast fix for SNMP, going away soon */
 #include <sys/time.h>
 #endif
@@ -78,52 +76,71 @@ struct	rtentry;
 struct	socket;
 struct	ether_header;
 #endif
+
 /*
  * Structure describing information about an interface
  * which may be of interest to management entities.
  */
+struct if_data {
+	/* generic interface information */
+	u_char	ifi_type;		/* ethernet, tokenring, etc */
+	u_char	ifi_physical;		/* e.g., AUI, Thinnet, 10base-T, etc */
+	u_char	ifi_addrlen;		/* media address length */
+	u_char	ifi_hdrlen;		/* media header length */
+	u_char	ifi_recvquota;		/* polling quota for receive intrs */
+	u_char	ifi_xmitquota;		/* polling quota for xmit intrs */
+	u_long	ifi_mtu;		/* maximum transmission unit */
+	u_long	ifi_metric;		/* routing metric (external only) */
+	u_long	ifi_baudrate;		/* linespeed */
+	/* volatile statistics */
+	u_long	ifi_ipackets;		/* packets received on interface */
+	u_long	ifi_ierrors;		/* input errors on interface */
+	u_long	ifi_opackets;		/* packets sent on interface */
+	u_long	ifi_oerrors;		/* output errors on interface */
+	u_long	ifi_collisions;		/* collisions on csma interfaces */
+	u_long	ifi_ibytes;		/* total number of octets received */
+	u_long	ifi_obytes;		/* total number of octets sent */
+	u_long	ifi_imcasts;		/* packets received via multicast */
+	u_long	ifi_omcasts;		/* packets sent via multicast */
+	u_long	ifi_iqdrops;		/* dropped on input, this interface */
+	u_long	ifi_noproto;		/* destined for unsupported protocol */
+	u_long	ifi_recvtiming;		/* usec spent receiving when timing */
+	u_long	ifi_xmittiming;		/* usec spent xmitting when timing */
+	struct	timeval ifi_lastchange;	/* time of last administrative change */
+};
+
 /*
  * Structure defining a queue for a network interface.
+ */
+struct	ifqueue {
+	struct	mbuf *ifq_head;
+	struct	mbuf *ifq_tail;
+	int	ifq_len;
+	int	ifq_maxlen;
+	int	ifq_drops;
+};
+
+/*
+ * Structure defining a network interface.
  *
  * (Would like to call this struct ``if'', but C isn't PL/1.)
  */
-
 struct ifnet {
+	void	*if_softc;		/* pointer to driver state */
 	char	*if_name;		/* name, e.g. ``en'' or ``lo'' */
 	struct	ifnet *if_next;		/* all struct ifnets are chained */
 	struct	ifaddr *if_addrlist;	/* linked list of addresses per if */
         int	if_pcount;		/* number of promiscuous listeners */
-	caddr_t	if_bpf;			/* packet filter structure */
+	struct	bpf_if *if_bpf;		/* packet filter structure */
 	u_short	if_index;		/* numeric abbreviation for this if  */
 	short	if_unit;		/* sub-unit for lower level driver */
 	short	if_timer;		/* time 'til if_watchdog called */
 	short	if_flags;		/* up/down, broadcast, etc. */
-	struct	if_data {
-/* generic interface information */
-		u_char	ifi_type;	/* ethernet, tokenring, etc */
-		u_char	ifi_physical; 	/* e.g., AUI, Thinnet, 10base-T, etc */
-		u_char	ifi_addrlen;	/* media address length */
-		u_char	ifi_hdrlen;	/* media header length */
-		u_long	ifi_mtu;	/* maximum transmission unit */
-		u_long	ifi_metric;	/* routing metric (external only) */
-		u_long	ifi_baudrate;	/* linespeed */
-/* volatile statistics */
-		u_long	ifi_ipackets;	/* packets received on interface */
-		u_long	ifi_ierrors;	/* input errors on interface */
-		u_long	ifi_opackets;	/* packets sent on interface */
-		u_long	ifi_oerrors;	/* output errors on interface */
-		u_long	ifi_collisions;	/* collisions on csma interfaces */
-		u_long	ifi_ibytes;	/* total number of octets received */
-		u_long	ifi_obytes;	/* total number of octets sent */
-		u_long	ifi_imcasts;	/* packets received via multicast */
-		u_long	ifi_omcasts;	/* packets sent via multicast */
-		u_long	ifi_iqdrops;	/* dropped on input, this interface */
-		u_long	ifi_noproto;	/* destined for unsupported protocol */
-		struct	timeval ifi_lastchange;/* time of last administrative change */
-	}	if_data;
+	int	if_ipending;		/* interrupts pending */
+	void	*if_linkmib;		/* link-type-specific MIB data */
+	size_t	if_linkmiblen;		/* length of above data */
+	struct	if_data if_data;
 /* procedure handles */
-	void	(*if_init)		/* init routine */
-		__P((int));
 	int	(*if_output)		/* output routine (enqueue) */
 		__P((struct ifnet *, struct mbuf *, struct sockaddr *,
 		     struct rtentry *));
@@ -133,18 +150,23 @@ struct ifnet {
 		__P((struct ifnet *));	/* (XXX not used; fake prototype) */
 	int	(*if_ioctl)		/* ioctl routine */
 		__P((struct ifnet *, int, caddr_t));
-	void	(*if_reset)
-		__P((int));		/* new autoconfig will permit removal */
 	void	(*if_watchdog)		/* timer routine */
-		__P((int));
-	struct	ifqueue {
-		struct	mbuf *ifq_head;
-		struct	mbuf *ifq_tail;
-		int	ifq_len;
-		int	ifq_maxlen;
-		int	ifq_drops;
-	} if_snd;			/* output queue */
+		__P((struct ifnet *));
+	int	(*if_poll_recv)		/* polled receive routine */
+		__P((struct ifnet *, int *));
+	int	(*if_poll_xmit)		/* polled transmit routine */
+		__P((struct ifnet *, int *));
+	void	(*if_poll_intren)	/* polled interrupt reenable routine */
+		__P((struct ifnet *));
+	void	(*if_poll_slowinput)	/* input routine for slow devices */
+		__P((struct ifnet *, struct mbuf *));
+	void	(*if_init)		/* Init routine */
+		__P((void *));
+	struct	ifqueue if_snd;		/* output queue */
+	struct	ifqueue *if_poll_slowq;	/* input queue for slow devices */
 };
+typedef void if_init_f_t __P((void *));       
+
 #define	if_mtu		if_data.ifi_mtu
 #define	if_type		if_data.ifi_type
 #define if_physical	if_data.ifi_physical
@@ -164,6 +186,8 @@ struct ifnet {
 #define	if_iqdrops	if_data.ifi_iqdrops
 #define	if_noproto	if_data.ifi_noproto
 #define	if_lastchange	if_data.ifi_lastchange
+#define if_recvquota	if_data.ifi_recvquota
+#define	if_xmitquota	if_data.ifi_xmitquota
 #define if_rawoutput(if, m, sa) if_output(if, m, sa, (struct rtentry *)0)
 
 #define	IFF_UP		0x1		/* interface is up */
@@ -171,7 +195,7 @@ struct ifnet {
 #define	IFF_DEBUG	0x4		/* turn on debugging */
 #define	IFF_LOOPBACK	0x8		/* is a loopback net */
 #define	IFF_POINTOPOINT	0x10		/* interface is point-to-point link */
-#define	IFF_NOTRAILERS	0x20		/* avoid use of trailers */
+/*#define IFF_NOTRAILERS 0x20		 * obsolete: avoid use of trailers */
 #define	IFF_RUNNING	0x40		/* resources allocated */
 #define	IFF_NOARP	0x80		/* no address resolution protocol */
 #define	IFF_PROMISC	0x100		/* receive all packets */
@@ -200,8 +224,14 @@ struct ifnet {
 /* etc. */
 
 /*
- * Output queues (ifp->if_snd) and internetwork datagram level (pup level 1)
- * input routines have queues of messages stored on ifqueue structures
+ * Bit values in if_ipending
+ */
+#define	IFI_RECV	1	/* I want to receive */
+#define	IFI_XMIT	2	/* I want to transmit */
+
+/*
+ * Output queues (ifp->if_snd) and slow device input queues (*ifp->if_slowq)
+ * are queues of messages stored on ifqueue structures
  * (defined above).  Entries are added to and deleted from these structures
  * by these macros, which should be called with ipl raised to splimp().
  */
@@ -233,6 +263,35 @@ struct ifnet {
 	} \
 }
 
+#ifdef KERNEL
+#define	IF_ENQ_DROP(ifq, m)	if_enq_drop(ifq, m)
+
+#if defined(__GNUC__) && defined(MT_HEADER)
+static __inline int
+if_queue_drop(struct ifqueue *ifq, struct mbuf *m)
+{
+	IF_DROP(ifq);
+	return 0;
+}
+
+static __inline int
+if_enq_drop(struct ifqueue *ifq, struct mbuf *m)
+{
+	if (IF_QFULL(ifq) &&
+	    !if_queue_drop(ifq, m))
+		return 0;
+	IF_ENQUEUE(ifq, m);
+	return 1;
+}
+#else
+
+#ifdef MT_HEADER
+int	if_enq_drop __P((struct ifqueue *, struct mbuf *));
+#endif
+
+#endif
+#endif /* KERNEL */
+
 #define	IFQ_MAXLEN	50
 #define	IFNET_SLOWHZ	1		/* granularity is 1 second */
 
@@ -252,11 +311,14 @@ struct ifaddr {
 	void	(*ifa_rtrequest)	/* check or clean routes (+ or -)'d */
 		__P((int, struct rtentry *, struct sockaddr *));
 	u_short	ifa_flags;		/* mostly rt_flags for cloning */
-	u_int	ifa_refcnt;		/* extra to malloc for link info */
+	u_int	ifa_refcnt;		/* references to this structure */
 	int	ifa_metric;		/* cost of going out this interface */
 #ifdef notdef
 	struct	rtentry *ifa_rt;	/* XXXX for ROUTETOIF ????? */
 #endif
+	int (*ifa_claim_addr)		/* check if an addr goes to this if */
+		__P((struct ifaddr *, struct sockaddr *));
+
 };
 #define	IFA_ROUTE	RTF_UP		/* route installed */
 
@@ -305,6 +367,7 @@ struct	ifreq {
 		int	ifru_metric;
 		int	ifru_mtu;
 		int	ifru_phys;
+		int	ifru_media;
 		caddr_t	ifru_data;
 	} ifr_ifru;
 #define	ifr_addr	ifr_ifru.ifru_addr	/* address */
@@ -314,6 +377,7 @@ struct	ifreq {
 #define	ifr_metric	ifr_ifru.ifru_metric	/* metric */
 #define	ifr_mtu		ifr_ifru.ifru_mtu	/* mtu */
 #define ifr_phys	ifr_ifru.ifru_phys	/* physical wire */
+#define ifr_media	ifr_ifru.ifru_media	/* physical media */
 #define	ifr_data	ifr_ifru.ifru_data	/* for use by interface */
 };
 
@@ -324,6 +388,15 @@ struct ifaliasreq {
 	struct	sockaddr ifra_mask;
 };
 
+struct ifmediareq {
+	char	ifm_name[IFNAMSIZ];	/* if name, e.g. "en0" */
+	int	ifm_current;		/* current media options */
+	int	ifm_mask;		/* don't care mask */
+	int	ifm_status;		/* media status */
+	int	ifm_active;		/* active options */
+	int	ifm_count;		/* # entries in ifm_ulist array */
+	int	*ifm_ulist;		/* media words */
+};
 /*
  * Structure used in SIOCGIFCONF request.
  * Used to retrieve interface configuration
@@ -350,43 +423,45 @@ struct	ifconf {
 		(ifa)->ifa_refcnt--;
 
 extern struct	ifnet	*ifnet;
+extern int	ifqmaxlen;
 extern struct	ifnet	loif[];
+extern int	if_index;
+extern struct	ifaddr	**ifnet_addrs;
 
 void	ether_ifattach __P((struct ifnet *));
 void	ether_input __P((struct ifnet *, struct ether_header *, struct mbuf *));
 int	ether_output __P((struct ifnet *,
 	   struct mbuf *, struct sockaddr *, struct rtentry *));
-char	*ether_sprintf __P((u_char *));
+int	ether_ioctl __P((struct ifnet *, int , caddr_t ));
 
 void	if_attach __P((struct ifnet *));
 void	if_down __P((struct ifnet *));
-void	if_qflush __P((struct ifqueue *));
-void	if_slowtimo __P((void *));
 void	if_up __P((struct ifnet *));
 #ifdef vax
 void	ifubareset __P((int));
 #endif
-int	ifconf __P((int, caddr_t));
 /*void	ifinit __P((void));*/ /* declared in systm.h for main() */
 int	ifioctl __P((struct socket *, int, caddr_t, struct proc *));
 int	ifpromisc __P((struct ifnet *, int));
 struct	ifnet *ifunit __P((char *));
 
+int	if_poll_recv_slow __P((struct ifnet *ifp, int *quotap));
+void	if_poll_xmit_slow __P((struct ifnet *ifp, int *quotap));
+void	if_poll_throttle __P((void));
+void	if_poll_unthrottle __P((void *));
+void	if_poll_init __P((void));
+void	if_poll __P((void));
+
 struct	ifaddr *ifa_ifwithaddr __P((struct sockaddr *));
-struct	ifaddr *ifa_ifwithaf __P((int));
 struct	ifaddr *ifa_ifwithdstaddr __P((struct sockaddr *));
 struct	ifaddr *ifa_ifwithnet __P((struct sockaddr *));
 struct	ifaddr *ifa_ifwithroute __P((int, struct sockaddr *,
 					struct sockaddr *));
 struct	ifaddr *ifaof_ifpforaddr __P((struct sockaddr *, struct ifnet *));
 void	ifafree __P((struct ifaddr *));
-void	link_rtrequest __P((int, struct rtentry *, struct sockaddr *));
 
-int	loioctl __P((struct ifnet *, int, caddr_t));
-void	loopattach __P((void));
 int	looutput __P((struct ifnet *,
 	   struct mbuf *, struct sockaddr *, struct rtentry *));
-void	lortrequest __P((int, struct rtentry *, struct sockaddr *));
 #endif /* KERNEL */
 
 #endif /* !_NET_IF_H_ */
